// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: opentelemetry/proto/collector/logs/v1/logs_service.proto
// Protobuf C++ Version: 7.35.1

#include "opentelemetry/proto/collector/logs/v1/logs_service.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
#ifdef PROTOBUF_MESSAGE_GLOBALS
namespace {
PROTOBUF_CONSTINIT ::google::protobuf::internal::ReflectionData
    file_reflection_data[] = {
        // ::opentelemetry::proto::collector::logs::v1::ExportLogsServiceRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto, /* tracker*/ nullptr,},
        // ::opentelemetry::proto::collector::logs::v1::ExportLogsServiceResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto, /* tracker*/ nullptr,},
        // ::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess
        {&::_pbi::kDescriptorMethods, &::descriptor_table_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto, /* tracker*/ nullptr,},
};
}  // namespace
#endif
namespace opentelemetry {
namespace proto {
namespace collector {
namespace logs {
namespace v1 {
class ExportLogsPartialSuccess::_Internal {
 public:
  using HasBits = decltype(::std::declval<ExportLogsPartialSuccess>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ExportLogsPartialSuccess, _impl_._has_bits_);
};

constexpr ExportLogsPartialSuccess::ParseTableT_ ExportLogsPartialSuccess::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ExportLogsPartialSuccess, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string error_message = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 0, 0,
        PROTOBUF_FIELD_OFFSET(ExportLogsPartialSuccess, _impl_.error_message_)}},
      // int64 rejected_log_records = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(ExportLogsPartialSuccess, _impl_.rejected_log_records_), 1>(),
       {8, 1, 0,
        PROTOBUF_FIELD_OFFSET(ExportLogsPartialSuccess, _impl_.rejected_log_records_)}},
    }}, {{
      65535, 65535
    }}, {{
      // int64 rejected_log_records = 1;
      {PROTOBUF_FIELD_OFFSET(ExportLogsPartialSuccess, _impl_.rejected_log_records_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // string error_message = 2;
      {PROTOBUF_FIELD_OFFSET(ExportLogsPartialSuccess, _impl_.error_message_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\76\0\15\0\0\0\0\0"
      "opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess"
      "error_message"
    }},
  };
}


inline constexpr ExportLogsPartialSuccess::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        error_message_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        rejected_log_records_{::int64_t{0}} {}

template <typename>
constexpr ExportLogsPartialSuccess::ExportLogsPartialSuccess(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ExportLogsPartialSuccess::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ExportLogsPartialSuccess(arena);
}
constexpr auto ExportLogsPartialSuccess::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ExportLogsPartialSuccess), alignof(ExportLogsPartialSuccess));
}
constexpr auto ExportLogsPartialSuccess::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ExportLogsPartialSuccess::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ExportLogsPartialSuccess>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ExportLogsPartialSuccess::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ExportLogsPartialSuccess>(), &ExportLogsPartialSuccess::ByteSizeLong,
              &ExportLogsPartialSuccess::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ExportLogsPartialSuccess, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[2],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ExportLogsPartialSuccessGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ExportLogsPartialSuccessGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ExportLogsPartialSuccess_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ExportLogsPartialSuccess::InternalGenerateClassData_(
            _default, &ExportLogsPartialSuccess_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ExportLogsPartialSuccess>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ExportLogsPartialSuccessGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ExportLogsPartialSuccess _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ExportLogsPartialSuccess>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ExportLogsPartialSuccessGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ExportLogsPartialSuccessGlobalsTypeInternal ExportLogsPartialSuccess_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ExportLogsPartialSuccess_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ExportLogsPartialSuccess_globals_.GetClassData();
#else
  return ExportLogsPartialSuccess_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ExportLogsServiceResponse::_Internal {
 public:
  using HasBits = decltype(::std::declval<ExportLogsServiceResponse>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ExportLogsServiceResponse, _impl_._has_bits_);
};

constexpr ExportLogsServiceResponse::ParseTableT_ ExportLogsServiceResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ExportLogsServiceResponse, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::opentelemetry::proto::collector::logs::v1::ExportLogsServiceResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess partial_success = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ExportLogsServiceResponse, _impl_.partial_success_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess partial_success = 1;
      {PROTOBUF_FIELD_OFFSET(ExportLogsServiceResponse, _impl_.partial_success_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr ExportLogsServiceResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        partial_success_{nullptr} {}

template <typename>
constexpr ExportLogsServiceResponse::ExportLogsServiceResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ExportLogsServiceResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ExportLogsServiceResponse(arena);
}
constexpr auto ExportLogsServiceResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ExportLogsServiceResponse), alignof(ExportLogsServiceResponse));
}
constexpr auto ExportLogsServiceResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ExportLogsServiceResponse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ExportLogsServiceResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ExportLogsServiceResponse::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ExportLogsServiceResponse>(), &ExportLogsServiceResponse::ByteSizeLong,
              &ExportLogsServiceResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ExportLogsServiceResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[1],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ExportLogsServiceResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ExportLogsServiceResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ExportLogsServiceResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ExportLogsServiceResponse::InternalGenerateClassData_(
            _default, &ExportLogsServiceResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ExportLogsServiceResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ExportLogsServiceResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ExportLogsServiceResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ExportLogsServiceResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ExportLogsServiceResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ExportLogsServiceResponseGlobalsTypeInternal ExportLogsServiceResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ExportLogsServiceResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ExportLogsServiceResponse_globals_.GetClassData();
#else
  return ExportLogsServiceResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ExportLogsServiceRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<ExportLogsServiceRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ExportLogsServiceRequest, _impl_._has_bits_);
};

constexpr ExportLogsServiceRequest::ParseTableT_ ExportLogsServiceRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ExportLogsServiceRequest, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::opentelemetry::proto::collector::logs::v1::ExportLogsServiceRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated .opentelemetry.proto.logs.v1.ResourceLogs resource_logs = 1;
      {::_pbi::TcParser::FastMtR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ExportLogsServiceRequest, _impl_.resource_logs_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated .opentelemetry.proto.logs.v1.ResourceLogs resource_logs = 1;
      {PROTOBUF_FIELD_OFFSET(ExportLogsServiceRequest, _impl_.resource_logs_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::opentelemetry::proto::logs::v1::ResourceLogs>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::opentelemetry::proto::logs::v1::ResourceLogs_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr ExportLogsServiceRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        resource_logs_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::opentelemetry::proto::collector::logs::v1::ExportLogsServiceRequest,
            PROTOBUF_FIELD_OFFSET(::opentelemetry::proto::collector::logs::v1::ExportLogsServiceRequest, _impl_.resource_logs_)>()
         }
     {}

template <typename>
constexpr ExportLogsServiceRequest::ExportLogsServiceRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ExportLogsServiceRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ExportLogsServiceRequest(arena);
}
constexpr auto ExportLogsServiceRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ExportLogsServiceRequest), alignof(ExportLogsServiceRequest));
}
constexpr auto ExportLogsServiceRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ExportLogsServiceRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ExportLogsServiceRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ExportLogsServiceRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ExportLogsServiceRequest>(), &ExportLogsServiceRequest::ByteSizeLong,
              &ExportLogsServiceRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ExportLogsServiceRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[0],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ExportLogsServiceRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ExportLogsServiceRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ExportLogsServiceRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ExportLogsServiceRequest::InternalGenerateClassData_(
            _default, &ExportLogsServiceRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ExportLogsServiceRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ExportLogsServiceRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ExportLogsServiceRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ExportLogsServiceRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ExportLogsServiceRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ExportLogsServiceRequestGlobalsTypeInternal ExportLogsServiceRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ExportLogsServiceRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ExportLogsServiceRequest_globals_.GetClassData();
#else
  return ExportLogsServiceRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
}  // namespace v1
}  // namespace logs
}  // namespace collector
}  // namespace proto
}  // namespace opentelemetry
static constexpr const ::_pb::EnumDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_enum_descriptors_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto = nullptr;
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto = nullptr;
const ::uint32_t
    TableStruct_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::opentelemetry::proto::collector::logs::v1::ExportLogsServiceRequest, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::opentelemetry::proto::collector::logs::v1::ExportLogsServiceRequest, _impl_.resource_logs_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::opentelemetry::proto::collector::logs::v1::ExportLogsServiceResponse, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::opentelemetry::proto::collector::logs::v1::ExportLogsServiceResponse, _impl_.partial_success_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess, _impl_.rejected_log_records_),
        PROTOBUF_FIELD_OFFSET(::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess, _impl_.error_message_),
        1,
        0,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::opentelemetry::proto::collector::logs::v1::ExportLogsServiceRequest)},
        {5, sizeof(::opentelemetry::proto::collector::logs::v1::ExportLogsServiceResponse)},
        {10, sizeof(::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess)},
};
static const ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const
    file_message_globals[] = {
        &::opentelemetry::proto::collector::logs::v1::ExportLogsServiceRequest_globals_,
        &::opentelemetry::proto::collector::logs::v1::ExportLogsServiceResponse_globals_,
        &::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess_globals_,
};
const char descriptor_table_protodef_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n8opentelemetry/proto/collector/logs/v1/"
    "logs_service.proto\022%opentelemetry.proto."
    "collector.logs.v1\032&opentelemetry/proto/l"
    "ogs/v1/logs.proto\"\\\n\030ExportLogsServiceRe"
    "quest\022@\n\rresource_logs\030\001 \003(\0132).opentelem"
    "etry.proto.logs.v1.ResourceLogs\"u\n\031Expor"
    "tLogsServiceResponse\022X\n\017partial_success\030"
    "\001 \001(\0132\?.opentelemetry.proto.collector.lo"
    "gs.v1.ExportLogsPartialSuccess\"O\n\030Export"
    "LogsPartialSuccess\022\034\n\024rejected_log_recor"
    "ds\030\001 \001(\003\022\025\n\rerror_message\030\002 \001(\t2\235\001\n\013Logs"
    "Service\022\215\001\n\006Export\022\?.opentelemetry.proto"
    ".collector.logs.v1.ExportLogsServiceRequ"
    "est\032@.opentelemetry.proto.collector.logs"
    ".v1.ExportLogsServiceResponse\"\000B\230\001\n(io.o"
    "pentelemetry.proto.collector.logs.v1B\020Lo"
    "gsServiceProtoP\001Z0go.opentelemetry.io/pr"
    "oto/otlp/collector/logs/v1\252\002%OpenTelemet"
    "ry.Proto.Collector.Logs.V1b\006proto3"
};
static const ::_pbi::DescriptorTable* PROTOBUF_NONNULL const
    descriptor_table_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto_deps[1] = {
        &::descriptor_table_opentelemetry_2fproto_2flogs_2fv1_2flogs_2eproto,
};
static ::absl::once_flag descriptor_table_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto = {
    false,
    false,
    754,
    descriptor_table_protodef_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto,
    "opentelemetry/proto/collector/logs/v1/logs_service.proto",
    &descriptor_table_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto_once,
    descriptor_table_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto_deps,
    1,
    3,
    schemas,
    file_message_globals,
    TableStruct_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto::offsets,
    file_level_enum_descriptors_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto,
    file_level_service_descriptors_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto,
};
namespace opentelemetry {
namespace proto {
namespace collector {
namespace logs {
namespace v1 {
// ===================================================================

void ExportLogsServiceRequest::clear_resource_logs() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.resource_logs_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
ExportLogsServiceRequest::ExportLogsServiceRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ExportLogsServiceRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:opentelemetry.proto.collector.logs.v1.ExportLogsServiceRequest)
}
PROTOBUF_NDEBUG_INLINE ExportLogsServiceRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::opentelemetry::proto::collector::logs::v1::ExportLogsServiceRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        resource_logs_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::opentelemetry::proto::collector::logs::v1::ExportLogsServiceRequest,
              PROTOBUF_FIELD_OFFSET(::opentelemetry::proto::collector::logs::v1::ExportLogsServiceRequest, _impl_.resource_logs_)>()
          , from.resource_logs_
        }
     {}

ExportLogsServiceRequest::ExportLogsServiceRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ExportLogsServiceRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ExportLogsServiceRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ExportLogsServiceRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:opentelemetry.proto.collector.logs.v1.ExportLogsServiceRequest)
}
PROTOBUF_NDEBUG_INLINE ExportLogsServiceRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        resource_logs_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::opentelemetry::proto::collector::logs::v1::ExportLogsServiceRequest,
            PROTOBUF_FIELD_OFFSET(::opentelemetry::proto::collector::logs::v1::ExportLogsServiceRequest, _impl_.resource_logs_)>()
         }
     {}

inline void ExportLogsServiceRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
ExportLogsServiceRequest::~ExportLogsServiceRequest() {
  // @@protoc_insertion_point(destructor:opentelemetry.proto.collector.logs.v1.ExportLogsServiceRequest)
  SharedDtor(*this);
}
inline void ExportLogsServiceRequest::SharedDtor(MessageLite& self) {
  ExportLogsServiceRequest& this_ = static_cast<ExportLogsServiceRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ExportLogsServiceRequest_class_data_ =
        ExportLogsServiceRequest::InternalGenerateClassData_(ExportLogsServiceRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ExportLogsServiceRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ExportLogsServiceRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ExportLogsServiceRequest_class_data_.tc_table);
  return ExportLogsServiceRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ExportLogsServiceRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ExportLogsServiceRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ExportLogsServiceRequest_globals_));
  return ExportLogsServiceRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ExportLogsServiceRequest::ParseTableT_
    ExportLogsServiceRequest::_table_ =
        ExportLogsServiceRequest::InternalGenerateParseTable_(ExportLogsServiceRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ExportLogsServiceRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:opentelemetry.proto.collector.logs.v1.ExportLogsServiceRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.resource_logs_.Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ExportLogsServiceRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ExportLogsServiceRequest& this_ = static_cast<const ExportLogsServiceRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ExportLogsServiceRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ExportLogsServiceRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:opentelemetry.proto.collector.logs.v1.ExportLogsServiceRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated .opentelemetry.proto.logs.v1.ResourceLogs resource_logs = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_resource_logs_size());
         i < n; i++) {
      const auto& repfield = this_._internal_resource_logs().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              1, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:opentelemetry.proto.collector.logs.v1.ExportLogsServiceRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ExportLogsServiceRequest::ByteSizeLong(const MessageLite& base) {
  const ExportLogsServiceRequest& this_ = static_cast<const ExportLogsServiceRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ExportLogsServiceRequest::ByteSizeLong() const {
  const ExportLogsServiceRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:opentelemetry.proto.collector.logs.v1.ExportLogsServiceRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
   {
    // repeated .opentelemetry.proto.logs.v1.ResourceLogs resource_logs = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_resource_logs_size();
      for (const auto& msg : this_._internal_resource_logs()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ExportLogsServiceRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ExportLogsServiceRequest*>(&to_msg);
  auto& from = static_cast<const ExportLogsServiceRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:opentelemetry.proto.collector.logs.v1.ExportLogsServiceRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _this->_internal_mutable_resource_logs()->InternalMergeFromWithArena(
        ::google::protobuf::MessageLite::internal_visibility(), arena,
        from._internal_resource_logs());
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ExportLogsServiceRequest::CopyFrom(const ExportLogsServiceRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:opentelemetry.proto.collector.logs.v1.ExportLogsServiceRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ExportLogsServiceRequest::InternalSwap(ExportLogsServiceRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.resource_logs_.InternalSwap(&other->_impl_.resource_logs_);
}

::google::protobuf::Metadata ExportLogsServiceRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ExportLogsServiceResponse::ExportLogsServiceResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ExportLogsServiceResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:opentelemetry.proto.collector.logs.v1.ExportLogsServiceResponse)
}
PROTOBUF_NDEBUG_INLINE ExportLogsServiceResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::opentelemetry::proto::collector::logs::v1::ExportLogsServiceResponse& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

ExportLogsServiceResponse::ExportLogsServiceResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ExportLogsServiceResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ExportLogsServiceResponse_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ExportLogsServiceResponse* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.partial_success_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.partial_success_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:opentelemetry.proto.collector.logs.v1.ExportLogsServiceResponse)
}
PROTOBUF_NDEBUG_INLINE ExportLogsServiceResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void ExportLogsServiceResponse::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.partial_success_ = {};
}
ExportLogsServiceResponse::~ExportLogsServiceResponse() {
  // @@protoc_insertion_point(destructor:opentelemetry.proto.collector.logs.v1.ExportLogsServiceResponse)
  SharedDtor(*this);
}
inline void ExportLogsServiceResponse::SharedDtor(MessageLite& self) {
  ExportLogsServiceResponse& this_ = static_cast<ExportLogsServiceResponse&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.partial_success_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ExportLogsServiceResponse_class_data_ =
        ExportLogsServiceResponse::InternalGenerateClassData_(ExportLogsServiceResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ExportLogsServiceResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ExportLogsServiceResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ExportLogsServiceResponse_class_data_.tc_table);
  return ExportLogsServiceResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ExportLogsServiceResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ExportLogsServiceResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ExportLogsServiceResponse_globals_));
  return ExportLogsServiceResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ExportLogsServiceResponse::ParseTableT_
    ExportLogsServiceResponse::_table_ =
        ExportLogsServiceResponse::InternalGenerateParseTable_(ExportLogsServiceResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ExportLogsServiceResponse::Clear() {
// @@protoc_insertion_point(message_clear_start:opentelemetry.proto.collector.logs.v1.ExportLogsServiceResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(_impl_.partial_success_ != nullptr);
    _impl_.partial_success_->Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ExportLogsServiceResponse::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ExportLogsServiceResponse& this_ = static_cast<const ExportLogsServiceResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ExportLogsServiceResponse::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ExportLogsServiceResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:opentelemetry.proto.collector.logs.v1.ExportLogsServiceResponse)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess partial_success = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.partial_success_, this_._impl_.partial_success_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:opentelemetry.proto.collector.logs.v1.ExportLogsServiceResponse)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ExportLogsServiceResponse::ByteSizeLong(const MessageLite& base) {
  const ExportLogsServiceResponse& this_ = static_cast<const ExportLogsServiceResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ExportLogsServiceResponse::ByteSizeLong() const {
  const ExportLogsServiceResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:opentelemetry.proto.collector.logs.v1.ExportLogsServiceResponse)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // .opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess partial_success = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.partial_success_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ExportLogsServiceResponse::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ExportLogsServiceResponse*>(&to_msg);
  auto& from = static_cast<const ExportLogsServiceResponse&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:opentelemetry.proto.collector.logs.v1.ExportLogsServiceResponse)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(from._impl_.partial_success_ != nullptr);
    if (_this->_impl_.partial_success_ == nullptr) {
      _this->_impl_.partial_success_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.partial_success_);
    } else {
      _this->_impl_.partial_success_->MergeFrom(*from._impl_.partial_success_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ExportLogsServiceResponse::CopyFrom(const ExportLogsServiceResponse& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:opentelemetry.proto.collector.logs.v1.ExportLogsServiceResponse)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ExportLogsServiceResponse::InternalSwap(ExportLogsServiceResponse* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.partial_success_, other->_impl_.partial_success_);
}

::google::protobuf::Metadata ExportLogsServiceResponse::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ExportLogsPartialSuccess::ExportLogsPartialSuccess(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ExportLogsPartialSuccess_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess)
}
PROTOBUF_NDEBUG_INLINE ExportLogsPartialSuccess::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        error_message_(arena, from.error_message_) {}

ExportLogsPartialSuccess::ExportLogsPartialSuccess(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ExportLogsPartialSuccess& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ExportLogsPartialSuccess_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ExportLogsPartialSuccess* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.rejected_log_records_ = from._impl_.rejected_log_records_;

  // @@protoc_insertion_point(copy_constructor:opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess)
}
PROTOBUF_NDEBUG_INLINE ExportLogsPartialSuccess::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        error_message_(arena) {}

inline void ExportLogsPartialSuccess::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.rejected_log_records_ = {};
}
ExportLogsPartialSuccess::~ExportLogsPartialSuccess() {
  // @@protoc_insertion_point(destructor:opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess)
  SharedDtor(*this);
}
inline void ExportLogsPartialSuccess::SharedDtor(MessageLite& self) {
  ExportLogsPartialSuccess& this_ = static_cast<ExportLogsPartialSuccess&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.error_message_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ExportLogsPartialSuccess_class_data_ =
        ExportLogsPartialSuccess::InternalGenerateClassData_(ExportLogsPartialSuccess_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ExportLogsPartialSuccess::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ExportLogsPartialSuccess_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ExportLogsPartialSuccess_class_data_.tc_table);
  return ExportLogsPartialSuccess_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ExportLogsPartialSuccess::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ExportLogsPartialSuccess_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ExportLogsPartialSuccess_globals_));
  return ExportLogsPartialSuccess_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ExportLogsPartialSuccess::ParseTableT_
    ExportLogsPartialSuccess::_table_ =
        ExportLogsPartialSuccess::InternalGenerateParseTable_(ExportLogsPartialSuccess_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ExportLogsPartialSuccess::Clear() {
// @@protoc_insertion_point(message_clear_start:opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.error_message_.ClearNonDefaultToEmpty();
  }
  _impl_.rejected_log_records_ = ::int64_t{0};
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ExportLogsPartialSuccess::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ExportLogsPartialSuccess& this_ = static_cast<const ExportLogsPartialSuccess&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ExportLogsPartialSuccess::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ExportLogsPartialSuccess& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // int64 rejected_log_records = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_rejected_log_records() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<1>(
              stream, this_._internal_rejected_log_records(), target);
    }
  }

  // string error_message = 2;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_error_message().empty()) {
      const ::std::string& _s = this_._internal_error_message();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess.error_message");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ExportLogsPartialSuccess::ByteSizeLong(const MessageLite& base) {
  const ExportLogsPartialSuccess& this_ = static_cast<const ExportLogsPartialSuccess&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ExportLogsPartialSuccess::ByteSizeLong() const {
  const ExportLogsPartialSuccess& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string error_message = 2;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_error_message().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_error_message());
      }
    }
    // int64 rejected_log_records = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_rejected_log_records() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_rejected_log_records());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ExportLogsPartialSuccess::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ExportLogsPartialSuccess*>(&to_msg);
  auto& from = static_cast<const ExportLogsPartialSuccess&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_error_message().empty()) {
        _this->_internal_set_error_message(from._internal_error_message());
      } else {
        if (_this->_impl_.error_message_.IsDefault()) {
          _this->_internal_set_error_message("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_rejected_log_records() != 0) {
        _this->_impl_.rejected_log_records_ = from._impl_.rejected_log_records_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ExportLogsPartialSuccess::CopyFrom(const ExportLogsPartialSuccess& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ExportLogsPartialSuccess::InternalSwap(ExportLogsPartialSuccess* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.error_message_, &other->_impl_.error_message_, arena);
  swap(_impl_.rejected_log_records_, other->_impl_.rejected_log_records_);
}

::google::protobuf::Metadata ExportLogsPartialSuccess::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
}  // namespace v1
}  // namespace logs
}  // namespace collector
}  // namespace proto
}  // namespace opentelemetry
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
