// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: opentelemetry/proto/collector/logs/v1/logs_service.proto
// Protobuf C++ Version: 7.35.1

#ifndef opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto_2epb_2eh
#define opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7035001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/unknown_field_set.h"
#include "opentelemetry/proto/logs/v1/logs.pb.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto;
}  // extern "C"
namespace opentelemetry {
namespace proto {
namespace collector {
namespace logs {
namespace v1 {
class ExportLogsPartialSuccess;
struct ExportLogsPartialSuccessGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ExportLogsPartialSuccessGlobalsTypeInternal ExportLogsPartialSuccess_globals_;
extern const ::google::protobuf::internal::ClassDataFull ExportLogsPartialSuccess_class_data_;
#else
extern const ExportLogsPartialSuccessGlobalsTypeInternal ExportLogsPartialSuccess_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ExportLogsServiceRequest;
struct ExportLogsServiceRequestGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ExportLogsServiceRequestGlobalsTypeInternal ExportLogsServiceRequest_globals_;
extern const ::google::protobuf::internal::ClassDataFull ExportLogsServiceRequest_class_data_;
#else
extern const ExportLogsServiceRequestGlobalsTypeInternal ExportLogsServiceRequest_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ExportLogsServiceResponse;
struct ExportLogsServiceResponseGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ExportLogsServiceResponseGlobalsTypeInternal ExportLogsServiceResponse_globals_;
extern const ::google::protobuf::internal::ClassDataFull ExportLogsServiceResponse_class_data_;
#else
extern const ExportLogsServiceResponseGlobalsTypeInternal ExportLogsServiceResponse_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace v1
}  // namespace logs
}  // namespace collector
}  // namespace proto
}  // namespace opentelemetry
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace opentelemetry {
namespace proto {
namespace collector {
namespace logs {
namespace v1 {

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ExportLogsPartialSuccess final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess) */ {
 public:
  inline ExportLogsPartialSuccess() : ExportLogsPartialSuccess(nullptr) {}
  ~ExportLogsPartialSuccess() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ExportLogsPartialSuccess* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ExportLogsPartialSuccess));
  }
#endif

  template <typename = void>
  explicit constexpr ExportLogsPartialSuccess(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ExportLogsPartialSuccess(const ExportLogsPartialSuccess& from) : ExportLogsPartialSuccess(nullptr, from) {}
  inline ExportLogsPartialSuccess(ExportLogsPartialSuccess&& from) noexcept : ExportLogsPartialSuccess(nullptr, ::std::move(from)) {}
  inline ExportLogsPartialSuccess& operator=(const ExportLogsPartialSuccess& from) {
    CopyFrom(from);
    return *this;
  }
  inline ExportLogsPartialSuccess& operator=(ExportLogsPartialSuccess&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ExportLogsPartialSuccess& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ExportLogsPartialSuccess>(&ExportLogsPartialSuccess_globals_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(ExportLogsPartialSuccess& a, ExportLogsPartialSuccess& b) { a.Swap(&b); }
  inline void Swap(ExportLogsPartialSuccess* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ExportLogsPartialSuccess* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ExportLogsPartialSuccess* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ExportLogsPartialSuccess>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ExportLogsPartialSuccess& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ExportLogsPartialSuccess& from) { ExportLogsPartialSuccess::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ExportLogsPartialSuccess* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess"; }

  explicit ExportLogsPartialSuccess(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ExportLogsPartialSuccess(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ExportLogsPartialSuccess& from);
  ExportLogsPartialSuccess(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ExportLogsPartialSuccess&& from) noexcept
      : ExportLogsPartialSuccess(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kErrorMessageFieldNumber = 2,
    kRejectedLogRecordsFieldNumber = 1,
  };
  // string error_message = 2;
  void clear_error_message() ;
  [[nodiscard]] const ::std::string& error_message() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_error_message(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_error_message();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_error_message();
  void set_allocated_error_message(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_error_message() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_error_message(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_error_message();

  public:
  // int64 rejected_log_records = 1;
  void clear_rejected_log_records() ;
  [[nodiscard]] ::int64_t rejected_log_records() const;
  void set_rejected_log_records(::int64_t value);

  private:
  ::int64_t _internal_rejected_log_records() const;
  void _internal_set_rejected_log_records(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          0, 84,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ExportLogsPartialSuccess& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr error_message_;
    ::int64_t rejected_log_records_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ExportLogsServiceResponse final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.collector.logs.v1.ExportLogsServiceResponse) */ {
 public:
  inline ExportLogsServiceResponse() : ExportLogsServiceResponse(nullptr) {}
  ~ExportLogsServiceResponse() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ExportLogsServiceResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ExportLogsServiceResponse));
  }
#endif

  template <typename = void>
  explicit constexpr ExportLogsServiceResponse(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ExportLogsServiceResponse(const ExportLogsServiceResponse& from) : ExportLogsServiceResponse(nullptr, from) {}
  inline ExportLogsServiceResponse(ExportLogsServiceResponse&& from) noexcept : ExportLogsServiceResponse(nullptr, ::std::move(from)) {}
  inline ExportLogsServiceResponse& operator=(const ExportLogsServiceResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline ExportLogsServiceResponse& operator=(ExportLogsServiceResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ExportLogsServiceResponse& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ExportLogsServiceResponse>(&ExportLogsServiceResponse_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(ExportLogsServiceResponse& a, ExportLogsServiceResponse& b) { a.Swap(&b); }
  inline void Swap(ExportLogsServiceResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ExportLogsServiceResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ExportLogsServiceResponse* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ExportLogsServiceResponse>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ExportLogsServiceResponse& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ExportLogsServiceResponse& from) { ExportLogsServiceResponse::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ExportLogsServiceResponse* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.collector.logs.v1.ExportLogsServiceResponse"; }

  explicit ExportLogsServiceResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ExportLogsServiceResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ExportLogsServiceResponse& from);
  ExportLogsServiceResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ExportLogsServiceResponse&& from) noexcept
      : ExportLogsServiceResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kPartialSuccessFieldNumber = 1,
  };
  // .opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess partial_success = 1;
  [[nodiscard]] bool has_partial_success()
      const;
  void clear_partial_success() ;
  [[nodiscard]] const ::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess& partial_success() const;
  [[nodiscard]] ::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess* PROTOBUF_NULLABLE release_partial_success();
  ::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess* PROTOBUF_NONNULL mutable_partial_success();
  void set_allocated_partial_success(::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_partial_success(::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess* PROTOBUF_NULLABLE value);
  ::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess* PROTOBUF_NULLABLE unsafe_arena_release_partial_success();

  private:
  const ::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess& _internal_partial_success() const;
  ::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess* PROTOBUF_NONNULL _internal_mutable_partial_success();

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.collector.logs.v1.ExportLogsServiceResponse)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ExportLogsServiceResponse& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess* PROTOBUF_NULLABLE partial_success_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ExportLogsServiceRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.collector.logs.v1.ExportLogsServiceRequest) */ {
 public:
  inline ExportLogsServiceRequest() : ExportLogsServiceRequest(nullptr) {}
  ~ExportLogsServiceRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ExportLogsServiceRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ExportLogsServiceRequest));
  }
#endif

  template <typename = void>
  explicit constexpr ExportLogsServiceRequest(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ExportLogsServiceRequest(const ExportLogsServiceRequest& from) : ExportLogsServiceRequest(nullptr, from) {}
  inline ExportLogsServiceRequest(ExportLogsServiceRequest&& from) noexcept : ExportLogsServiceRequest(nullptr, ::std::move(from)) {}
  inline ExportLogsServiceRequest& operator=(const ExportLogsServiceRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline ExportLogsServiceRequest& operator=(ExportLogsServiceRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ExportLogsServiceRequest& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ExportLogsServiceRequest>(&ExportLogsServiceRequest_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(ExportLogsServiceRequest& a, ExportLogsServiceRequest& b) { a.Swap(&b); }
  inline void Swap(ExportLogsServiceRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ExportLogsServiceRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ExportLogsServiceRequest* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ExportLogsServiceRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ExportLogsServiceRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ExportLogsServiceRequest& from) { ExportLogsServiceRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ExportLogsServiceRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.collector.logs.v1.ExportLogsServiceRequest"; }

  explicit ExportLogsServiceRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ExportLogsServiceRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ExportLogsServiceRequest& from);
  ExportLogsServiceRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ExportLogsServiceRequest&& from) noexcept
      : ExportLogsServiceRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kResourceLogsFieldNumber = 1,
  };
  // repeated .opentelemetry.proto.logs.v1.ResourceLogs resource_logs = 1;
  [[nodiscard]] int resource_logs_size()
      const;
  private:
  int _internal_resource_logs_size() const;

  public:
  void clear_resource_logs() ;
  [[nodiscard]] const ::opentelemetry::proto::logs::v1::ResourceLogs& resource_logs(int index) const;
  [[nodiscard]] ::opentelemetry::proto::logs::v1::ResourceLogs* PROTOBUF_NONNULL mutable_resource_logs(int index);
  ::opentelemetry::proto::logs::v1::ResourceLogs* PROTOBUF_NONNULL add_resource_logs();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ResourceLogs>&
  resource_logs() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ResourceLogs>* PROTOBUF_NONNULL
  mutable_resource_logs();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ResourceLogs>& _internal_resource_logs() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ResourceLogs>* PROTOBUF_NONNULL _internal_mutable_resource_logs();

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.collector.logs.v1.ExportLogsServiceRequest)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ExportLogsServiceRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::logs::v1::ResourceLogs > resource_logs_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// ExportLogsServiceRequest

// repeated .opentelemetry.proto.logs.v1.ResourceLogs resource_logs = 1;
inline int ExportLogsServiceRequest::_internal_resource_logs_size() const {
  return _internal_resource_logs().size();
}
inline int ExportLogsServiceRequest::resource_logs_size() const {
  return _internal_resource_logs_size();
}
inline const ::opentelemetry::proto::logs::v1::ResourceLogs& ExportLogsServiceRequest::resource_logs(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.collector.logs.v1.ExportLogsServiceRequest.resource_logs)
  return _internal_resource_logs().Get(index);
}
inline ::opentelemetry::proto::logs::v1::ResourceLogs* PROTOBUF_NONNULL ExportLogsServiceRequest::mutable_resource_logs(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.collector.logs.v1.ExportLogsServiceRequest.resource_logs)
  return _internal_mutable_resource_logs()->Mutable(index);
}
inline ::opentelemetry::proto::logs::v1::ResourceLogs* PROTOBUF_NONNULL ExportLogsServiceRequest::add_resource_logs()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::logs::v1::ResourceLogs* _add =
      _internal_mutable_resource_logs()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.collector.logs.v1.ExportLogsServiceRequest.resource_logs)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ResourceLogs>& ExportLogsServiceRequest::resource_logs() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.collector.logs.v1.ExportLogsServiceRequest.resource_logs)
  return _internal_resource_logs();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ResourceLogs>* PROTOBUF_NONNULL
ExportLogsServiceRequest::mutable_resource_logs() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.collector.logs.v1.ExportLogsServiceRequest.resource_logs)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_resource_logs();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ResourceLogs>&
ExportLogsServiceRequest::_internal_resource_logs() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.resource_logs_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ResourceLogs>* PROTOBUF_NONNULL
ExportLogsServiceRequest::_internal_mutable_resource_logs() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.resource_logs_;
}

// -------------------------------------------------------------------

// ExportLogsServiceResponse

// .opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess partial_success = 1;
inline bool ExportLogsServiceResponse::has_partial_success() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.partial_success_ != nullptr);
  return value;
}
inline void ExportLogsServiceResponse::clear_partial_success() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.partial_success_ != nullptr) _impl_.partial_success_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess& ExportLogsServiceResponse::_internal_partial_success() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess* p = _impl_.partial_success_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess>(&::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess_globals_);
}
inline const ::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess& ExportLogsServiceResponse::partial_success() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.collector.logs.v1.ExportLogsServiceResponse.partial_success)
  return _internal_partial_success();
}
inline void ExportLogsServiceResponse::unsafe_arena_set_allocated_partial_success(
    ::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.partial_success_);
  }
  _impl_.partial_success_ = reinterpret_cast<::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:opentelemetry.proto.collector.logs.v1.ExportLogsServiceResponse.partial_success)
}
inline ::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess* PROTOBUF_NULLABLE ExportLogsServiceResponse::release_partial_success() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess* released = _impl_.partial_success_;
  _impl_.partial_success_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess* PROTOBUF_NULLABLE ExportLogsServiceResponse::unsafe_arena_release_partial_success() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.collector.logs.v1.ExportLogsServiceResponse.partial_success)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess* temp = _impl_.partial_success_;
  _impl_.partial_success_ = nullptr;
  return temp;
}
inline ::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess* PROTOBUF_NONNULL ExportLogsServiceResponse::_internal_mutable_partial_success() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.partial_success_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess>(GetArena());
    _impl_.partial_success_ = reinterpret_cast<::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess*>(p);
  }
  return _impl_.partial_success_;
}
inline ::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess* PROTOBUF_NONNULL ExportLogsServiceResponse::mutable_partial_success()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess* _msg = _internal_mutable_partial_success();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.collector.logs.v1.ExportLogsServiceResponse.partial_success)
  return _msg;
}
inline void ExportLogsServiceResponse::set_allocated_partial_success(::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.partial_success_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.partial_success_ = reinterpret_cast<::opentelemetry::proto::collector::logs::v1::ExportLogsPartialSuccess*>(value);
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.collector.logs.v1.ExportLogsServiceResponse.partial_success)
}

// -------------------------------------------------------------------

// ExportLogsPartialSuccess

// int64 rejected_log_records = 1;
inline void ExportLogsPartialSuccess::clear_rejected_log_records() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.rejected_log_records_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::int64_t ExportLogsPartialSuccess::rejected_log_records() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess.rejected_log_records)
  return _internal_rejected_log_records();
}
inline void ExportLogsPartialSuccess::set_rejected_log_records(::int64_t value) {
  _internal_set_rejected_log_records(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess.rejected_log_records)
}
inline ::int64_t ExportLogsPartialSuccess::_internal_rejected_log_records() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.rejected_log_records_;
}
inline void ExportLogsPartialSuccess::_internal_set_rejected_log_records(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.rejected_log_records_ = value;
}

// string error_message = 2;
inline void ExportLogsPartialSuccess::clear_error_message() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.error_message_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& ExportLogsPartialSuccess::error_message() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess.error_message)
  return _internal_error_message();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ExportLogsPartialSuccess::set_error_message(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.error_message_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess.error_message)
}
inline ::std::string* PROTOBUF_NONNULL ExportLogsPartialSuccess::mutable_error_message()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_error_message();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess.error_message)
  return _s;
}
inline const ::std::string& ExportLogsPartialSuccess::_internal_error_message() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.error_message_.Get();
}
inline void ExportLogsPartialSuccess::_internal_set_error_message(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.error_message_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ExportLogsPartialSuccess::_internal_mutable_error_message() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.error_message_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ExportLogsPartialSuccess::release_error_message() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess.error_message)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.error_message_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.error_message_.Set("", GetArena());
  }
  return released;
}
inline void ExportLogsPartialSuccess::set_allocated_error_message(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.error_message_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.error_message_.IsDefault()) {
    _impl_.error_message_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.collector.logs.v1.ExportLogsPartialSuccess.error_message)
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace v1
}  // namespace logs
}  // namespace collector
}  // namespace proto
}  // namespace opentelemetry


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // opentelemetry_2fproto_2fcollector_2flogs_2fv1_2flogs_5fservice_2eproto_2epb_2eh
