// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: google/cloud/bigquery/storage/v1/avro.proto
// Protobuf C++ Version: 7.35.1

#ifndef google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto_2epb_2eh
#define google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7035001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/unknown_field_set.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto;
}  // extern "C"
namespace google {
namespace cloud {
namespace bigquery {
namespace storage {
namespace v1 {
class AvroRows;
struct AvroRowsGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern AvroRowsGlobalsTypeInternal AvroRows_globals_;
extern const ::google::protobuf::internal::ClassDataFull AvroRows_class_data_;
#else
extern const AvroRowsGlobalsTypeInternal AvroRows_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class AvroSchema;
struct AvroSchemaGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern AvroSchemaGlobalsTypeInternal AvroSchema_globals_;
extern const ::google::protobuf::internal::ClassDataFull AvroSchema_class_data_;
#else
extern const AvroSchemaGlobalsTypeInternal AvroSchema_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class AvroSerializationOptions;
struct AvroSerializationOptionsGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern AvroSerializationOptionsGlobalsTypeInternal AvroSerializationOptions_globals_;
extern const ::google::protobuf::internal::ClassDataFull AvroSerializationOptions_class_data_;
#else
extern const AvroSerializationOptionsGlobalsTypeInternal AvroSerializationOptions_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace v1
}  // namespace storage
}  // namespace bigquery
}  // namespace cloud
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace google {
namespace cloud {
namespace bigquery {
namespace storage {
namespace v1 {

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED AvroSerializationOptions final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:google.cloud.bigquery.storage.v1.AvroSerializationOptions) */ {
 public:
  inline AvroSerializationOptions() : AvroSerializationOptions(nullptr) {}
  ~AvroSerializationOptions() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(AvroSerializationOptions* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(AvroSerializationOptions));
  }
#endif

  template <typename = void>
  explicit constexpr AvroSerializationOptions(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline AvroSerializationOptions(const AvroSerializationOptions& from) : AvroSerializationOptions(nullptr, from) {}
  inline AvroSerializationOptions(AvroSerializationOptions&& from) noexcept : AvroSerializationOptions(nullptr, ::std::move(from)) {}
  inline AvroSerializationOptions& operator=(const AvroSerializationOptions& from) {
    CopyFrom(from);
    return *this;
  }
  inline AvroSerializationOptions& operator=(AvroSerializationOptions&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const AvroSerializationOptions& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<AvroSerializationOptions>(&AvroSerializationOptions_globals_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(AvroSerializationOptions& a, AvroSerializationOptions& b) { a.Swap(&b); }
  inline void Swap(AvroSerializationOptions* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(AvroSerializationOptions* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] AvroSerializationOptions* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<AvroSerializationOptions>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const AvroSerializationOptions& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const AvroSerializationOptions& from) { AvroSerializationOptions::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(AvroSerializationOptions* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "google.cloud.bigquery.storage.v1.AvroSerializationOptions"; }

  explicit AvroSerializationOptions(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  AvroSerializationOptions(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const AvroSerializationOptions& from);
  AvroSerializationOptions(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, AvroSerializationOptions&& from) noexcept
      : AvroSerializationOptions(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kEnableDisplayNameAttributeFieldNumber = 1,
  };
  // bool enable_display_name_attribute = 1;
  void clear_enable_display_name_attribute() ;
  [[nodiscard]] bool enable_display_name_attribute() const;
  void set_enable_display_name_attribute(bool value);

  private:
  bool _internal_enable_display_name_attribute() const;
  void _internal_set_enable_display_name_attribute(bool value);

  public:
  // @@protoc_insertion_point(class_scope:google.cloud.bigquery.storage.v1.AvroSerializationOptions)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const AvroSerializationOptions& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    bool enable_display_name_attribute_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED AvroSchema final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:google.cloud.bigquery.storage.v1.AvroSchema) */ {
 public:
  inline AvroSchema() : AvroSchema(nullptr) {}
  ~AvroSchema() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(AvroSchema* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(AvroSchema));
  }
#endif

  template <typename = void>
  explicit constexpr AvroSchema(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline AvroSchema(const AvroSchema& from) : AvroSchema(nullptr, from) {}
  inline AvroSchema(AvroSchema&& from) noexcept : AvroSchema(nullptr, ::std::move(from)) {}
  inline AvroSchema& operator=(const AvroSchema& from) {
    CopyFrom(from);
    return *this;
  }
  inline AvroSchema& operator=(AvroSchema&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const AvroSchema& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<AvroSchema>(&AvroSchema_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(AvroSchema& a, AvroSchema& b) { a.Swap(&b); }
  inline void Swap(AvroSchema* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(AvroSchema* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] AvroSchema* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<AvroSchema>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const AvroSchema& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const AvroSchema& from) { AvroSchema::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(AvroSchema* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "google.cloud.bigquery.storage.v1.AvroSchema"; }

  explicit AvroSchema(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  AvroSchema(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const AvroSchema& from);
  AvroSchema(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, AvroSchema&& from) noexcept
      : AvroSchema(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kSchemaFieldNumber = 1,
  };
  // string schema = 1;
  void clear_schema() ;
  [[nodiscard]] const ::std::string& schema() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_schema(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_schema();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_schema();
  void set_allocated_schema(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_schema() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_schema(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_schema();

  public:
  // @@protoc_insertion_point(class_scope:google.cloud.bigquery.storage.v1.AvroSchema)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          0, 58,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const AvroSchema& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr schema_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED AvroRows final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:google.cloud.bigquery.storage.v1.AvroRows) */ {
 public:
  inline AvroRows() : AvroRows(nullptr) {}
  ~AvroRows() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(AvroRows* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(AvroRows));
  }
#endif

  template <typename = void>
  explicit constexpr AvroRows(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline AvroRows(const AvroRows& from) : AvroRows(nullptr, from) {}
  inline AvroRows(AvroRows&& from) noexcept : AvroRows(nullptr, ::std::move(from)) {}
  inline AvroRows& operator=(const AvroRows& from) {
    CopyFrom(from);
    return *this;
  }
  inline AvroRows& operator=(AvroRows&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const AvroRows& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<AvroRows>(&AvroRows_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(AvroRows& a, AvroRows& b) { a.Swap(&b); }
  inline void Swap(AvroRows* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(AvroRows* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] AvroRows* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<AvroRows>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const AvroRows& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const AvroRows& from) { AvroRows::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(AvroRows* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "google.cloud.bigquery.storage.v1.AvroRows"; }

  explicit AvroRows(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  AvroRows(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const AvroRows& from);
  AvroRows(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, AvroRows&& from) noexcept
      : AvroRows(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kSerializedBinaryRowsFieldNumber = 1,
    kRowCountFieldNumber = 2,
  };
  // bytes serialized_binary_rows = 1;
  void clear_serialized_binary_rows() ;
  [[nodiscard]] const ::std::string& serialized_binary_rows() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_serialized_binary_rows(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_serialized_binary_rows();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_serialized_binary_rows();
  void set_allocated_serialized_binary_rows(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_serialized_binary_rows() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_serialized_binary_rows(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_serialized_binary_rows();

  public:
  // int64 row_count = 2 [deprecated = true];
  [[deprecated]]  void clear_row_count() ;
  [[nodiscard]] [[deprecated]] ::int64_t row_count() const;
  [[deprecated]] void set_row_count(::int64_t value);

  private:
  ::int64_t _internal_row_count() const;
  void _internal_set_row_count(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:google.cloud.bigquery.storage.v1.AvroRows)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const AvroRows& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr serialized_binary_rows_;
    ::int64_t row_count_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// AvroSchema

// string schema = 1;
inline void AvroSchema::clear_schema() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.schema_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& AvroSchema::schema() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:google.cloud.bigquery.storage.v1.AvroSchema.schema)
  return _internal_schema();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void AvroSchema::set_schema(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.schema_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:google.cloud.bigquery.storage.v1.AvroSchema.schema)
}
inline ::std::string* PROTOBUF_NONNULL AvroSchema::mutable_schema()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_schema();
  // @@protoc_insertion_point(field_mutable:google.cloud.bigquery.storage.v1.AvroSchema.schema)
  return _s;
}
inline const ::std::string& AvroSchema::_internal_schema() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.schema_.Get();
}
inline void AvroSchema::_internal_set_schema(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.schema_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL AvroSchema::_internal_mutable_schema() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.schema_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE AvroSchema::release_schema() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:google.cloud.bigquery.storage.v1.AvroSchema.schema)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.schema_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.schema_.Set("", GetArena());
  }
  return released;
}
inline void AvroSchema::set_allocated_schema(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.schema_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.schema_.IsDefault()) {
    _impl_.schema_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:google.cloud.bigquery.storage.v1.AvroSchema.schema)
}

// -------------------------------------------------------------------

// AvroRows

// bytes serialized_binary_rows = 1;
inline void AvroRows::clear_serialized_binary_rows() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.serialized_binary_rows_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& AvroRows::serialized_binary_rows() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:google.cloud.bigquery.storage.v1.AvroRows.serialized_binary_rows)
  return _internal_serialized_binary_rows();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void AvroRows::set_serialized_binary_rows(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.serialized_binary_rows_.SetBytes(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:google.cloud.bigquery.storage.v1.AvroRows.serialized_binary_rows)
}
inline ::std::string* PROTOBUF_NONNULL AvroRows::mutable_serialized_binary_rows()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_serialized_binary_rows();
  // @@protoc_insertion_point(field_mutable:google.cloud.bigquery.storage.v1.AvroRows.serialized_binary_rows)
  return _s;
}
inline const ::std::string& AvroRows::_internal_serialized_binary_rows() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.serialized_binary_rows_.Get();
}
inline void AvroRows::_internal_set_serialized_binary_rows(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.serialized_binary_rows_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL AvroRows::_internal_mutable_serialized_binary_rows() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.serialized_binary_rows_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE AvroRows::release_serialized_binary_rows() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:google.cloud.bigquery.storage.v1.AvroRows.serialized_binary_rows)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.serialized_binary_rows_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.serialized_binary_rows_.Set("", GetArena());
  }
  return released;
}
inline void AvroRows::set_allocated_serialized_binary_rows(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.serialized_binary_rows_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.serialized_binary_rows_.IsDefault()) {
    _impl_.serialized_binary_rows_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:google.cloud.bigquery.storage.v1.AvroRows.serialized_binary_rows)
}

// int64 row_count = 2 [deprecated = true];
inline void AvroRows::clear_row_count() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.row_count_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::int64_t AvroRows::row_count() const {
  // @@protoc_insertion_point(field_get:google.cloud.bigquery.storage.v1.AvroRows.row_count)
  return _internal_row_count();
}
inline void AvroRows::set_row_count(::int64_t value) {
  _internal_set_row_count(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:google.cloud.bigquery.storage.v1.AvroRows.row_count)
}
inline ::int64_t AvroRows::_internal_row_count() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.row_count_;
}
inline void AvroRows::_internal_set_row_count(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.row_count_ = value;
}

// -------------------------------------------------------------------

// AvroSerializationOptions

// bool enable_display_name_attribute = 1;
inline void AvroSerializationOptions::clear_enable_display_name_attribute() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.enable_display_name_attribute_ = false;
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline bool AvroSerializationOptions::enable_display_name_attribute() const {
  // @@protoc_insertion_point(field_get:google.cloud.bigquery.storage.v1.AvroSerializationOptions.enable_display_name_attribute)
  return _internal_enable_display_name_attribute();
}
inline void AvroSerializationOptions::set_enable_display_name_attribute(bool value) {
  _internal_set_enable_display_name_attribute(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:google.cloud.bigquery.storage.v1.AvroSerializationOptions.enable_display_name_attribute)
}
inline bool AvroSerializationOptions::_internal_enable_display_name_attribute() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.enable_display_name_attribute_;
}
inline void AvroSerializationOptions::_internal_set_enable_display_name_attribute(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.enable_display_name_attribute_ = value;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace v1
}  // namespace storage
}  // namespace bigquery
}  // namespace cloud
}  // namespace google


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto_2epb_2eh
