// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: google/api/resource.proto
// Protobuf C++ Version: 7.35.1

#include "google/api/resource.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
#ifdef PROTOBUF_MESSAGE_GLOBALS
namespace {
PROTOBUF_CONSTINIT ::google::protobuf::internal::ReflectionData
    file_reflection_data[] = {
        // ::google::api::ResourceDescriptor
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fapi_2fresource_2eproto, /* tracker*/ nullptr,},
        // ::google::api::ResourceReference
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fapi_2fresource_2eproto, /* tracker*/ nullptr,},
};
}  // namespace
#endif
namespace google {
namespace api {
class ResourceReference::_Internal {
 public:
  using HasBits = decltype(::std::declval<ResourceReference>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ResourceReference, _impl_._has_bits_);
};

constexpr ResourceReference::ParseTableT_ ResourceReference::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ResourceReference, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::api::ResourceReference>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string child_type = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(ResourceReference, _impl_.child_type_)}},
      // string type = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ResourceReference, _impl_.type_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string type = 1;
      {PROTOBUF_FIELD_OFFSET(ResourceReference, _impl_.type_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string child_type = 2;
      {PROTOBUF_FIELD_OFFSET(ResourceReference, _impl_.child_type_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\34\4\12\0\0\0\0\0"
      "google.api.ResourceReference"
      "type"
      "child_type"
    }},
  };
}


inline constexpr ResourceReference::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        type_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        child_type_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr ResourceReference::ResourceReference(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ResourceReference::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ResourceReference(arena);
}
constexpr auto ResourceReference::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ResourceReference), alignof(ResourceReference));
}
constexpr auto ResourceReference::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ResourceReference::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ResourceReference>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ResourceReference::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ResourceReference>(), &ResourceReference::ByteSizeLong,
              &ResourceReference::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ResourceReference, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[1],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fapi_2fresource_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ResourceReferenceGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ResourceReferenceGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ResourceReference_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ResourceReference::InternalGenerateClassData_(
            _default, &ResourceReference_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ResourceReference>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ResourceReferenceGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ResourceReference _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ResourceReference>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ResourceReferenceGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ResourceReferenceGlobalsTypeInternal ResourceReference_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ResourceReference_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ResourceReference_globals_.GetClassData();
#else
  return ResourceReference_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ResourceDescriptor::_Internal {
 public:
  using HasBits = decltype(::std::declval<ResourceDescriptor>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ResourceDescriptor, _impl_._has_bits_);
};

constexpr ResourceDescriptor::ParseTableT_ ResourceDescriptor::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ResourceDescriptor, _impl_._has_bits_),
      0, // no _extensions_
      10, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294966720,  // skipmap
      offsetof(ParseTableT_, field_entries),
      7,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::api::ResourceDescriptor>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string type = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 1, 0,
        PROTOBUF_FIELD_OFFSET(ResourceDescriptor, _impl_.type_)}},
      // repeated string pattern = 2;
      {::_pbi::TcParser::FastUR1,
       {18, 0, 0,
        PROTOBUF_FIELD_OFFSET(ResourceDescriptor, _impl_.pattern_)}},
      // string name_field = 3;
      {::_pbi::TcParser::FastUS1,
       {26, 2, 0,
        PROTOBUF_FIELD_OFFSET(ResourceDescriptor, _impl_.name_field_)}},
      // .google.api.ResourceDescriptor.History history = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ResourceDescriptor, _impl_.history_), 5>(),
       {32, 5, 0,
        PROTOBUF_FIELD_OFFSET(ResourceDescriptor, _impl_.history_)}},
      // string plural = 5;
      {::_pbi::TcParser::FastUS1,
       {42, 3, 0,
        PROTOBUF_FIELD_OFFSET(ResourceDescriptor, _impl_.plural_)}},
      // string singular = 6;
      {::_pbi::TcParser::FastUS1,
       {50, 4, 0,
        PROTOBUF_FIELD_OFFSET(ResourceDescriptor, _impl_.singular_)}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // string type = 1;
      {PROTOBUF_FIELD_OFFSET(ResourceDescriptor, _impl_.type_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // repeated string pattern = 2;
      {PROTOBUF_FIELD_OFFSET(ResourceDescriptor, _impl_.pattern_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // string name_field = 3;
      {PROTOBUF_FIELD_OFFSET(ResourceDescriptor, _impl_.name_field_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.api.ResourceDescriptor.History history = 4;
      {PROTOBUF_FIELD_OFFSET(ResourceDescriptor, _impl_.history_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // string plural = 5;
      {PROTOBUF_FIELD_OFFSET(ResourceDescriptor, _impl_.plural_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string singular = 6;
      {PROTOBUF_FIELD_OFFSET(ResourceDescriptor, _impl_.singular_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // repeated .google.api.ResourceDescriptor.Style style = 10;
      {PROTOBUF_FIELD_OFFSET(ResourceDescriptor, _impl_.style_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedOpenEnum)},
    }},
    // no aux_entries
    {{
      "\35\4\7\12\0\6\10\0"
      "google.api.ResourceDescriptor"
      "type"
      "pattern"
      "name_field"
      "plural"
      "singular"
    }},
  };
}


inline constexpr ResourceDescriptor::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        pattern_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::ResourceDescriptor,
            PROTOBUF_FIELD_OFFSET(::google::api::ResourceDescriptor, _impl_.pattern_)>()
         }
        ,
        type_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        name_field_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        plural_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        singular_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        history_{static_cast< ::google::api::ResourceDescriptor_History >(0)},
        style_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::ResourceDescriptor,
            PROTOBUF_FIELD_OFFSET(::google::api::ResourceDescriptor, _impl_.style_)>()
         }
        ,
        _style_cached_byte_size_{0} {}

template <typename>
constexpr ResourceDescriptor::ResourceDescriptor(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ResourceDescriptor::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ResourceDescriptor(arena);
}
constexpr auto ResourceDescriptor::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ResourceDescriptor), alignof(ResourceDescriptor));
}
constexpr auto ResourceDescriptor::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ResourceDescriptor::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ResourceDescriptor>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ResourceDescriptor::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ResourceDescriptor>(), &ResourceDescriptor::ByteSizeLong,
              &ResourceDescriptor::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ResourceDescriptor, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[0],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fapi_2fresource_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ResourceDescriptorGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ResourceDescriptorGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ResourceDescriptor_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ResourceDescriptor::InternalGenerateClassData_(
            _default, &ResourceDescriptor_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ResourceDescriptor>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ResourceDescriptorGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ResourceDescriptor _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ResourceDescriptor>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ResourceDescriptorGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ResourceDescriptorGlobalsTypeInternal ResourceDescriptor_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ResourceDescriptor_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ResourceDescriptor_globals_.GetClassData();
#else
  return ResourceDescriptor_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
}  // namespace api
}  // namespace google
static const ::_pb::EnumDescriptor* PROTOBUF_NONNULL
    file_level_enum_descriptors_google_2fapi_2fresource_2eproto[2];
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_google_2fapi_2fresource_2eproto = nullptr;
const ::uint32_t
    TableStruct_google_2fapi_2fresource_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::api::ResourceDescriptor, _impl_._has_bits_),
        10, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::api::ResourceDescriptor, _impl_.type_),
        PROTOBUF_FIELD_OFFSET(::google::api::ResourceDescriptor, _impl_.pattern_),
        PROTOBUF_FIELD_OFFSET(::google::api::ResourceDescriptor, _impl_.name_field_),
        PROTOBUF_FIELD_OFFSET(::google::api::ResourceDescriptor, _impl_.history_),
        PROTOBUF_FIELD_OFFSET(::google::api::ResourceDescriptor, _impl_.plural_),
        PROTOBUF_FIELD_OFFSET(::google::api::ResourceDescriptor, _impl_.singular_),
        PROTOBUF_FIELD_OFFSET(::google::api::ResourceDescriptor, _impl_.style_),
        1,
        0,
        2,
        5,
        3,
        4,
        6,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::api::ResourceReference, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::api::ResourceReference, _impl_.type_),
        PROTOBUF_FIELD_OFFSET(::google::api::ResourceReference, _impl_.child_type_),
        0,
        1,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::google::api::ResourceDescriptor)},
        {17, sizeof(::google::api::ResourceReference)},
};
static const ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const
    file_message_globals[] = {
        &::google::api::ResourceDescriptor_globals_,
        &::google::api::ResourceReference_globals_,
};
const char descriptor_table_protodef_google_2fapi_2fresource_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n\031google/api/resource.proto\022\ngoogle.api\032"
    " google/protobuf/descriptor.proto\"\356\002\n\022Re"
    "sourceDescriptor\022\014\n\004type\030\001 \001(\t\022\017\n\007patter"
    "n\030\002 \003(\t\022\022\n\nname_field\030\003 \001(\t\0227\n\007history\030\004"
    " \001(\0162&.google.api.ResourceDescriptor.His"
    "tory\022\016\n\006plural\030\005 \001(\t\022\020\n\010singular\030\006 \001(\t\0223"
    "\n\005style\030\n \003(\0162$.google.api.ResourceDescr"
    "iptor.Style\"[\n\007History\022\027\n\023HISTORY_UNSPEC"
    "IFIED\020\000\022\035\n\031ORIGINALLY_SINGLE_PATTERN\020\001\022\030"
    "\n\024FUTURE_MULTI_PATTERN\020\002\"8\n\005Style\022\025\n\021STY"
    "LE_UNSPECIFIED\020\000\022\030\n\024DECLARATIVE_FRIENDLY"
    "\020\001\"5\n\021ResourceReference\022\014\n\004type\030\001 \001(\t\022\022\n"
    "\nchild_type\030\002 \001(\t:Y\n\022resource_reference\022"
    "\035.google.protobuf.FieldOptions\030\237\010 \001(\0132\035."
    "google.api.ResourceReference:Z\n\023resource"
    "_definition\022\034.google.protobuf.FileOption"
    "s\030\235\010 \003(\0132\036.google.api.ResourceDescriptor"
    ":R\n\010resource\022\037.google.protobuf.MessageOp"
    "tions\030\235\010 \001(\0132\036.google.api.ResourceDescri"
    "ptorBn\n\016com.google.apiB\rResourceProtoP\001Z"
    "Agoogle.golang.org/genproto/googleapis/a"
    "pi/annotations;annotations\370\001\001\242\002\004GAPIb\006pr"
    "oto3"
};
static const ::_pbi::DescriptorTable* PROTOBUF_NONNULL const
    descriptor_table_google_2fapi_2fresource_2eproto_deps[1] = {
        &::descriptor_table_google_2fprotobuf_2fdescriptor_2eproto,
};
static ::absl::once_flag descriptor_table_google_2fapi_2fresource_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_google_2fapi_2fresource_2eproto = {
    false,
    false,
    884,
    descriptor_table_protodef_google_2fapi_2fresource_2eproto,
    "google/api/resource.proto",
    &descriptor_table_google_2fapi_2fresource_2eproto_once,
    descriptor_table_google_2fapi_2fresource_2eproto_deps,
    1,
    2,
    schemas,
    file_message_globals,
    TableStruct_google_2fapi_2fresource_2eproto::offsets,
    file_level_enum_descriptors_google_2fapi_2fresource_2eproto,
    file_level_service_descriptors_google_2fapi_2fresource_2eproto,
};
namespace google {
namespace api {
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
ResourceDescriptor_History_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fapi_2fresource_2eproto);
  return file_level_enum_descriptors_google_2fapi_2fresource_2eproto[0];
}
PROTOBUF_CONSTINIT const uint32_t ResourceDescriptor_History_internal_data_[] = {
    196608u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
ResourceDescriptor_Style_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fapi_2fresource_2eproto);
  return file_level_enum_descriptors_google_2fapi_2fresource_2eproto[1];
}
PROTOBUF_CONSTINIT const uint32_t ResourceDescriptor_Style_internal_data_[] = {
    131072u, 0u, };
// ===================================================================

ResourceDescriptor::ResourceDescriptor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ResourceDescriptor_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.api.ResourceDescriptor)
}
PROTOBUF_NDEBUG_INLINE ResourceDescriptor::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::api::ResourceDescriptor& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        pattern_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::api::ResourceDescriptor,
              PROTOBUF_FIELD_OFFSET(::google::api::ResourceDescriptor, _impl_.pattern_)>()
          , from.pattern_
        }
        ,
        type_(arena, from.type_),
        name_field_(arena, from.name_field_),
        plural_(arena, from.plural_),
        singular_(arena, from.singular_),
        style_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::api::ResourceDescriptor,
              PROTOBUF_FIELD_OFFSET(::google::api::ResourceDescriptor, _impl_.style_)>()
          , from.style_
        }
        ,
        _style_cached_byte_size_{0} {}

ResourceDescriptor::ResourceDescriptor(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ResourceDescriptor& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ResourceDescriptor_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ResourceDescriptor* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.history_ = from._impl_.history_;

  // @@protoc_insertion_point(copy_constructor:google.api.ResourceDescriptor)
}
PROTOBUF_NDEBUG_INLINE ResourceDescriptor::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        pattern_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::ResourceDescriptor,
            PROTOBUF_FIELD_OFFSET(::google::api::ResourceDescriptor, _impl_.pattern_)>()
         }
        ,
        type_(arena),
        name_field_(arena),
        plural_(arena),
        singular_(arena),
        style_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::ResourceDescriptor,
            PROTOBUF_FIELD_OFFSET(::google::api::ResourceDescriptor, _impl_.style_)>()
         }
        ,
        _style_cached_byte_size_{0} {}

inline void ResourceDescriptor::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.history_ = {};
}
ResourceDescriptor::~ResourceDescriptor() {
  // @@protoc_insertion_point(destructor:google.api.ResourceDescriptor)
  SharedDtor(*this);
}
inline void ResourceDescriptor::SharedDtor(MessageLite& self) {
  ResourceDescriptor& this_ = static_cast<ResourceDescriptor&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.type_.Destroy();
  this_._impl_.name_field_.Destroy();
  this_._impl_.plural_.Destroy();
  this_._impl_.singular_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ResourceDescriptor_class_data_ =
        ResourceDescriptor::InternalGenerateClassData_(ResourceDescriptor_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ResourceDescriptor::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ResourceDescriptor_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ResourceDescriptor_class_data_.tc_table);
  return ResourceDescriptor_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ResourceDescriptor::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ResourceDescriptor_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ResourceDescriptor_globals_));
  return ResourceDescriptor_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ResourceDescriptor::ParseTableT_
    ResourceDescriptor::_table_ =
        ResourceDescriptor::InternalGenerateParseTable_(ResourceDescriptor_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ResourceDescriptor::Clear() {
// @@protoc_insertion_point(message_clear_start:google.api.ResourceDescriptor)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.pattern_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.type_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.name_field_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _impl_.plural_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      _impl_.singular_.ClearNonDefaultToEmpty();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000060U)) {
    _impl_.history_ = 0;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      _impl_.style_.Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ResourceDescriptor::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ResourceDescriptor& this_ = static_cast<const ResourceDescriptor&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ResourceDescriptor::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ResourceDescriptor& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.api.ResourceDescriptor)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string type = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_type().empty()) {
      const ::std::string& _s = this_._internal_type();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.ResourceDescriptor.type");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // repeated string pattern = 2;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (int i = 0, n = this_._internal_pattern_size(); i < n; ++i) {
      const auto& s = this_._internal_pattern().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.ResourceDescriptor.pattern");
      target = stream->WriteString(2, s, target);
    }
  }

  // string name_field = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_name_field().empty()) {
      const ::std::string& _s = this_._internal_name_field();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.ResourceDescriptor.name_field");
      target = stream->WriteStringMaybeAliased(3, _s, target);
    }
  }

  // .google.api.ResourceDescriptor.History history = 4;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_history() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          4, this_._internal_history(), target);
    }
  }

  // string plural = 5;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (!this_._internal_plural().empty()) {
      const ::std::string& _s = this_._internal_plural();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.ResourceDescriptor.plural");
      target = stream->WriteStringMaybeAliased(5, _s, target);
    }
  }

  // string singular = 6;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (!this_._internal_singular().empty()) {
      const ::std::string& _s = this_._internal_singular();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.ResourceDescriptor.singular");
      target = stream->WriteStringMaybeAliased(6, _s, target);
    }
  }

  // repeated .google.api.ResourceDescriptor.Style style = 10;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    {
      ::size_t byte_size = this_._impl_._style_cached_byte_size_.Get();
      if (byte_size > 0) {
        target = stream->WriteEnumPacked(
            10, this_._internal_style(), byte_size, target);
      }
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.api.ResourceDescriptor)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ResourceDescriptor::ByteSizeLong(const MessageLite& base) {
  const ResourceDescriptor& this_ = static_cast<const ResourceDescriptor&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ResourceDescriptor::ByteSizeLong() const {
  const ResourceDescriptor& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.api.ResourceDescriptor)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    // repeated string pattern = 2;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_pattern().size());
      for (int i = 0, n = this_._internal_pattern().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_pattern().Get(i));
      }
    }
    // string type = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_type().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_type());
      }
    }
    // string name_field = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!this_._internal_name_field().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_name_field());
      }
    }
    // string plural = 5;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!this_._internal_plural().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_plural());
      }
    }
    // string singular = 6;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (!this_._internal_singular().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_singular());
      }
    }
    // .google.api.ResourceDescriptor.History history = 4;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_history() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_history());
      }
    }
    // repeated .google.api.ResourceDescriptor.Style style = 10;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      total_size += ::_pbi::WireFormatLite::EnumSizeWithPackedTagSize(
          this_._internal_style(), 1, this_._impl_._style_cached_byte_size_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ResourceDescriptor::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ResourceDescriptor*>(&to_msg);
  auto& from = static_cast<const ResourceDescriptor&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.api.ResourceDescriptor)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_pattern()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_pattern());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_type().empty()) {
        _this->_internal_set_type(from._internal_type());
      } else {
        if (_this->_impl_.type_.IsDefault()) {
          _this->_internal_set_type("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!from._internal_name_field().empty()) {
        _this->_internal_set_name_field(from._internal_name_field());
      } else {
        if (_this->_impl_.name_field_.IsDefault()) {
          _this->_internal_set_name_field("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!from._internal_plural().empty()) {
        _this->_internal_set_plural(from._internal_plural());
      } else {
        if (_this->_impl_.plural_.IsDefault()) {
          _this->_internal_set_plural("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (!from._internal_singular().empty()) {
        _this->_internal_set_singular(from._internal_singular());
      } else {
        if (_this->_impl_.singular_.IsDefault()) {
          _this->_internal_set_singular("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_history() != 0) {
        _this->_impl_.history_ = from._impl_.history_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      _this->_internal_mutable_style()->MergeFrom(from._internal_style());
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ResourceDescriptor::CopyFrom(const ResourceDescriptor& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.api.ResourceDescriptor)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ResourceDescriptor::InternalSwap(ResourceDescriptor* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.pattern_.InternalSwap(&other->_impl_.pattern_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.type_, &other->_impl_.type_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.name_field_, &other->_impl_.name_field_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.plural_, &other->_impl_.plural_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.singular_, &other->_impl_.singular_, arena);
  swap(_impl_.history_, other->_impl_.history_);
  _impl_.style_.InternalSwap(&other->_impl_.style_);
}

::google::protobuf::Metadata ResourceDescriptor::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ResourceReference::ResourceReference(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ResourceReference_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.api.ResourceReference)
}
PROTOBUF_NDEBUG_INLINE ResourceReference::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::api::ResourceReference& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        type_(arena, from.type_),
        child_type_(arena, from.child_type_) {}

ResourceReference::ResourceReference(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ResourceReference& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ResourceReference_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ResourceReference* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.api.ResourceReference)
}
PROTOBUF_NDEBUG_INLINE ResourceReference::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        type_(arena),
        child_type_(arena) {}

inline void ResourceReference::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
ResourceReference::~ResourceReference() {
  // @@protoc_insertion_point(destructor:google.api.ResourceReference)
  SharedDtor(*this);
}
inline void ResourceReference::SharedDtor(MessageLite& self) {
  ResourceReference& this_ = static_cast<ResourceReference&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.type_.Destroy();
  this_._impl_.child_type_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ResourceReference_class_data_ =
        ResourceReference::InternalGenerateClassData_(ResourceReference_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ResourceReference::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ResourceReference_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ResourceReference_class_data_.tc_table);
  return ResourceReference_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ResourceReference::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ResourceReference_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ResourceReference_globals_));
  return ResourceReference_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ResourceReference::ParseTableT_
    ResourceReference::_table_ =
        ResourceReference::InternalGenerateParseTable_(ResourceReference_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ResourceReference::Clear() {
// @@protoc_insertion_point(message_clear_start:google.api.ResourceReference)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.type_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.child_type_.ClearNonDefaultToEmpty();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ResourceReference::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ResourceReference& this_ = static_cast<const ResourceReference&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ResourceReference::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ResourceReference& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.api.ResourceReference)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string type = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_type().empty()) {
      const ::std::string& _s = this_._internal_type();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.ResourceReference.type");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // string child_type = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_child_type().empty()) {
      const ::std::string& _s = this_._internal_child_type();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.ResourceReference.child_type");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.api.ResourceReference)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ResourceReference::ByteSizeLong(const MessageLite& base) {
  const ResourceReference& this_ = static_cast<const ResourceReference&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ResourceReference::ByteSizeLong() const {
  const ResourceReference& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.api.ResourceReference)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string type = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_type().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_type());
      }
    }
    // string child_type = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_child_type().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_child_type());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ResourceReference::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ResourceReference*>(&to_msg);
  auto& from = static_cast<const ResourceReference&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.api.ResourceReference)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_type().empty()) {
        _this->_internal_set_type(from._internal_type());
      } else {
        if (_this->_impl_.type_.IsDefault()) {
          _this->_internal_set_type("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_child_type().empty()) {
        _this->_internal_set_child_type(from._internal_child_type());
      } else {
        if (_this->_impl_.child_type_.IsDefault()) {
          _this->_internal_set_child_type("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ResourceReference::CopyFrom(const ResourceReference& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.api.ResourceReference)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ResourceReference::InternalSwap(ResourceReference* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.type_, &other->_impl_.type_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.child_type_, &other->_impl_.child_type_, arena);
}

::google::protobuf::Metadata ResourceReference::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 ::_pbi::ExtensionIdentifier<
        ::google::protobuf::FieldOptions, ::_pbi::MessageTypeTraits< ::google::api::ResourceReference >, 11, false>
        resource_reference(kResourceReferenceFieldNumber, &::google::api::ResourceReference_globals_);
PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 ::_pbi::ExtensionIdentifier<
        ::google::protobuf::FileOptions, ::_pbi::RepeatedMessageTypeTraits< ::google::api::ResourceDescriptor >, 11, false>
        resource_definition(kResourceDefinitionFieldNumber, &::google::api::ResourceDescriptor_globals_);
PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 ::_pbi::ExtensionIdentifier<
        ::google::protobuf::MessageOptions, ::_pbi::MessageTypeTraits< ::google::api::ResourceDescriptor >, 11, false>
        resource(kResourceFieldNumber, &::google::api::ResourceDescriptor_globals_);
// @@protoc_insertion_point(namespace_scope)
}  // namespace api
}  // namespace google
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_google_2fapi_2fresource_2eproto),
        ::_pbi::ExtensionSet::RegisterMessageExtension(
            &::google::protobuf::FieldOptions::default_instance(), 1055, 11,
            false, false, &::google::api::ResourceReference::default_instance(),
            nullptr, ::_pbi::LazyAnnotation::kUndefined),
        ::_pbi::ExtensionSet::RegisterMessageExtension(
            &::google::protobuf::FileOptions::default_instance(), 1053, 11,
            true, false, &::google::api::ResourceDescriptor::default_instance(),
            nullptr, ::_pbi::LazyAnnotation::kUndefined),
        ::_pbi::ExtensionSet::RegisterMessageExtension(
            &::google::protobuf::MessageOptions::default_instance(), 1053, 11,
            false, false, &::google::api::ResourceDescriptor::default_instance(),
            nullptr, ::_pbi::LazyAnnotation::kUndefined),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
