// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: google/api/field_behavior.proto
// Protobuf C++ Version: 7.35.1

#include "google/api/field_behavior.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
namespace google {
namespace api {
}  // namespace api
}  // namespace google
static const ::_pb::EnumDescriptor* PROTOBUF_NONNULL
    file_level_enum_descriptors_google_2fapi_2ffield_5fbehavior_2eproto[1];
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_google_2fapi_2ffield_5fbehavior_2eproto = nullptr;
const ::uint32_t TableStruct_google_2fapi_2ffield_5fbehavior_2eproto::offsets[1] = {};
static constexpr ::_pbi::MigrationSchema* PROTOBUF_NULLABLE schemas = nullptr;
static constexpr ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const* PROTOBUF_NULLABLE
    file_message_globals = nullptr;
const char descriptor_table_protodef_google_2fapi_2ffield_5fbehavior_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n\037google/api/field_behavior.proto\022\ngoogl"
    "e.api\032 google/protobuf/descriptor.proto*"
    "\246\001\n\rFieldBehavior\022\036\n\032FIELD_BEHAVIOR_UNSP"
    "ECIFIED\020\000\022\014\n\010OPTIONAL\020\001\022\014\n\010REQUIRED\020\002\022\017\n"
    "\013OUTPUT_ONLY\020\003\022\016\n\nINPUT_ONLY\020\004\022\r\n\tIMMUTA"
    "BLE\020\005\022\022\n\016UNORDERED_LIST\020\006\022\025\n\021NON_EMPTY_D"
    "EFAULT\020\007:Q\n\016field_behavior\022\035.google.prot"
    "obuf.FieldOptions\030\234\010 \003(\0162\031.google.api.Fi"
    "eldBehaviorBp\n\016com.google.apiB\022FieldBeha"
    "viorProtoP\001ZAgoogle.golang.org/genproto/"
    "googleapis/api/annotations;annotations\242\002"
    "\004GAPIb\006proto3"
};
static const ::_pbi::DescriptorTable* PROTOBUF_NONNULL const
    descriptor_table_google_2fapi_2ffield_5fbehavior_2eproto_deps[1] = {
        &::descriptor_table_google_2fprotobuf_2fdescriptor_2eproto,
};
static ::absl::once_flag descriptor_table_google_2fapi_2ffield_5fbehavior_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_google_2fapi_2ffield_5fbehavior_2eproto = {
    false,
    false,
    453,
    descriptor_table_protodef_google_2fapi_2ffield_5fbehavior_2eproto,
    "google/api/field_behavior.proto",
    &descriptor_table_google_2fapi_2ffield_5fbehavior_2eproto_once,
    descriptor_table_google_2fapi_2ffield_5fbehavior_2eproto_deps,
    1,
    0,
    schemas,
    file_message_globals,
    TableStruct_google_2fapi_2ffield_5fbehavior_2eproto::offsets,
    file_level_enum_descriptors_google_2fapi_2ffield_5fbehavior_2eproto,
    file_level_service_descriptors_google_2fapi_2ffield_5fbehavior_2eproto,
};
namespace google {
namespace api {
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
FieldBehavior_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fapi_2ffield_5fbehavior_2eproto);
  return file_level_enum_descriptors_google_2fapi_2ffield_5fbehavior_2eproto[0];
}
PROTOBUF_CONSTINIT const uint32_t FieldBehavior_internal_data_[] = {
    524288u, 0u, };
PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 ::_pbi::ExtensionIdentifier<
        ::google::protobuf::FieldOptions, ::_pbi::RepeatedEnumTypeTraits< ::google::api::FieldBehavior>, 14, true>
        field_behavior(kFieldBehaviorFieldNumber, static_cast< ::google::api::FieldBehavior >(0));
// @@protoc_insertion_point(namespace_scope)
}  // namespace api
}  // namespace google
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_google_2fapi_2ffield_5fbehavior_2eproto),
        ::_pbi::ExtensionSet::RegisterEnumExtension(
            &::google::protobuf::FieldOptions::default_instance(), 1052, 14,
            true, true, ::google::api::FieldBehavior_internal_data_),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
