// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: opentelemetry/proto/logs/v1/logs.proto
// Protobuf C++ Version: 7.35.1

#ifndef opentelemetry_2fproto_2flogs_2fv1_2flogs_2eproto_2epb_2eh
#define opentelemetry_2fproto_2flogs_2fv1_2flogs_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7035001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_reflection.h"
#include "google/protobuf/unknown_field_set.h"
#include "opentelemetry/proto/common/v1/common.pb.h"
#include "opentelemetry/proto/resource/v1/resource.pb.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_opentelemetry_2fproto_2flogs_2fv1_2flogs_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_opentelemetry_2fproto_2flogs_2fv1_2flogs_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_opentelemetry_2fproto_2flogs_2fv1_2flogs_2eproto;
}  // extern "C"
namespace opentelemetry {
namespace proto {
namespace logs {
namespace v1 {
enum LogRecordFlags : int;
extern const uint32_t LogRecordFlags_internal_data_[];
enum SeverityNumber : int;
extern const uint32_t SeverityNumber_internal_data_[];
class LogRecord;
struct LogRecordGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern LogRecordGlobalsTypeInternal LogRecord_globals_;
extern const ::google::protobuf::internal::ClassDataFull LogRecord_class_data_;
#else
extern const LogRecordGlobalsTypeInternal LogRecord_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class LogsData;
struct LogsDataGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern LogsDataGlobalsTypeInternal LogsData_globals_;
extern const ::google::protobuf::internal::ClassDataFull LogsData_class_data_;
#else
extern const LogsDataGlobalsTypeInternal LogsData_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ResourceLogs;
struct ResourceLogsGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ResourceLogsGlobalsTypeInternal ResourceLogs_globals_;
extern const ::google::protobuf::internal::ClassDataFull ResourceLogs_class_data_;
#else
extern const ResourceLogsGlobalsTypeInternal ResourceLogs_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ScopeLogs;
struct ScopeLogsGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ScopeLogsGlobalsTypeInternal ScopeLogs_globals_;
extern const ::google::protobuf::internal::ClassDataFull ScopeLogs_class_data_;
#else
extern const ScopeLogsGlobalsTypeInternal ScopeLogs_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace v1
}  // namespace logs
}  // namespace proto
}  // namespace opentelemetry
namespace google {
namespace protobuf {
template <>
internal::EnumTraitsT<::opentelemetry::proto::logs::v1::LogRecordFlags_internal_data_>
    internal::EnumTraitsImpl::value<::opentelemetry::proto::logs::v1::LogRecordFlags>;
template <>
internal::EnumTraitsT<::opentelemetry::proto::logs::v1::SeverityNumber_internal_data_>
    internal::EnumTraitsImpl::value<::opentelemetry::proto::logs::v1::SeverityNumber>;
}  // namespace protobuf
}  // namespace google

namespace opentelemetry {
namespace proto {
namespace logs {
namespace v1 {
enum SeverityNumber : int {
  SEVERITY_NUMBER_UNSPECIFIED = 0,
  SEVERITY_NUMBER_TRACE = 1,
  SEVERITY_NUMBER_TRACE2 = 2,
  SEVERITY_NUMBER_TRACE3 = 3,
  SEVERITY_NUMBER_TRACE4 = 4,
  SEVERITY_NUMBER_DEBUG = 5,
  SEVERITY_NUMBER_DEBUG2 = 6,
  SEVERITY_NUMBER_DEBUG3 = 7,
  SEVERITY_NUMBER_DEBUG4 = 8,
  SEVERITY_NUMBER_INFO = 9,
  SEVERITY_NUMBER_INFO2 = 10,
  SEVERITY_NUMBER_INFO3 = 11,
  SEVERITY_NUMBER_INFO4 = 12,
  SEVERITY_NUMBER_WARN = 13,
  SEVERITY_NUMBER_WARN2 = 14,
  SEVERITY_NUMBER_WARN3 = 15,
  SEVERITY_NUMBER_WARN4 = 16,
  SEVERITY_NUMBER_ERROR = 17,
  SEVERITY_NUMBER_ERROR2 = 18,
  SEVERITY_NUMBER_ERROR3 = 19,
  SEVERITY_NUMBER_ERROR4 = 20,
  SEVERITY_NUMBER_FATAL = 21,
  SEVERITY_NUMBER_FATAL2 = 22,
  SEVERITY_NUMBER_FATAL3 = 23,
  SEVERITY_NUMBER_FATAL4 = 24,
  SeverityNumber_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  SeverityNumber_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t SeverityNumber_internal_data_[];
inline constexpr SeverityNumber SeverityNumber_MIN =
    static_cast<SeverityNumber>(0);
inline constexpr SeverityNumber SeverityNumber_MAX =
    static_cast<SeverityNumber>(24);
[[nodiscard]] inline bool SeverityNumber_IsValid(int value) {
  return 0 <= value && value <= 24;
}
inline constexpr int SeverityNumber_ARRAYSIZE = 24 + 1;
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
SeverityNumber_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(SeverityNumber) {
  return SeverityNumber_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& SeverityNumber_Name(T value) {
  static_assert(::std::is_same<T, SeverityNumber>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to SeverityNumber_Name().");
  return SeverityNumber_Name(static_cast<SeverityNumber>(value));
}
template <>
[[nodiscard]] inline const ::std::string& SeverityNumber_Name(SeverityNumber value) {
  return ::google::protobuf::internal::NameOfDenseEnum<SeverityNumber_descriptor, 0, 24>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool SeverityNumber_Parse(
    ::absl::string_view name, SeverityNumber* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<SeverityNumber>(SeverityNumber_descriptor(), name,
                                           value);
}
enum LogRecordFlags : int {
  LOG_RECORD_FLAGS_DO_NOT_USE = 0,
  LOG_RECORD_FLAGS_TRACE_FLAGS_MASK = 255,
  LogRecordFlags_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  LogRecordFlags_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t LogRecordFlags_internal_data_[];
inline constexpr LogRecordFlags LogRecordFlags_MIN =
    static_cast<LogRecordFlags>(0);
inline constexpr LogRecordFlags LogRecordFlags_MAX =
    static_cast<LogRecordFlags>(255);
[[nodiscard]] inline bool LogRecordFlags_IsValid(int value) {
  return ::google::protobuf::internal::ValidateEnum(value, LogRecordFlags_internal_data_);
}
inline constexpr int LogRecordFlags_ARRAYSIZE = 255 + 1;
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
LogRecordFlags_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(LogRecordFlags) {
  return LogRecordFlags_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& LogRecordFlags_Name(T value) {
  static_assert(::std::is_same<T, LogRecordFlags>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to LogRecordFlags_Name().");
  return ::google::protobuf::internal::NameOfEnum(LogRecordFlags_descriptor(), value);
}
[[nodiscard]] inline bool LogRecordFlags_Parse(
    ::absl::string_view name, LogRecordFlags* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<LogRecordFlags>(LogRecordFlags_descriptor(), name,
                                           value);
}
using ::google::protobuf::internal::generated_enum::AbslParseFlag;
using ::google::protobuf::internal::generated_enum::AbslUnparseFlag;

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED LogRecord final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.logs.v1.LogRecord) */ {
 public:
  inline LogRecord() : LogRecord(nullptr) {}
  ~LogRecord() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(LogRecord* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(LogRecord));
  }
#endif

  template <typename = void>
  explicit constexpr LogRecord(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline LogRecord(const LogRecord& from) : LogRecord(nullptr, from) {}
  inline LogRecord(LogRecord&& from) noexcept : LogRecord(nullptr, ::std::move(from)) {}
  inline LogRecord& operator=(const LogRecord& from) {
    CopyFrom(from);
    return *this;
  }
  inline LogRecord& operator=(LogRecord&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const LogRecord& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<LogRecord>(&LogRecord_globals_);
  }
  static constexpr int kIndexInFileMessages = 3;
  friend void swap(LogRecord& a, LogRecord& b) { a.Swap(&b); }
  inline void Swap(LogRecord* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(LogRecord* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] LogRecord* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<LogRecord>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const LogRecord& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const LogRecord& from) { LogRecord::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(LogRecord* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.logs.v1.LogRecord"; }

  explicit LogRecord(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  LogRecord(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const LogRecord& from);
  LogRecord(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, LogRecord&& from) noexcept
      : LogRecord(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kAttributesFieldNumber = 6,
    kSeverityTextFieldNumber = 3,
    kTraceIdFieldNumber = 9,
    kSpanIdFieldNumber = 10,
    kBodyFieldNumber = 5,
    kTimeUnixNanoFieldNumber = 1,
    kSeverityNumberFieldNumber = 2,
    kDroppedAttributesCountFieldNumber = 7,
    kObservedTimeUnixNanoFieldNumber = 11,
    kFlagsFieldNumber = 8,
  };
  // repeated .opentelemetry.proto.common.v1.KeyValue attributes = 6;
  [[nodiscard]] int attributes_size()
      const;
  private:
  int _internal_attributes_size() const;

  public:
  void clear_attributes() ;
  [[nodiscard]] const ::opentelemetry::proto::common::v1::KeyValue& attributes(int index) const;
  [[nodiscard]] ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL mutable_attributes(int index);
  ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL add_attributes();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>&
  attributes() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL
  mutable_attributes();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>& _internal_attributes() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL _internal_mutable_attributes();

  public:
  // string severity_text = 3;
  void clear_severity_text() ;
  [[nodiscard]] const ::std::string& severity_text() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_severity_text(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_severity_text();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_severity_text();
  void set_allocated_severity_text(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_severity_text() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_severity_text(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_severity_text();

  public:
  // bytes trace_id = 9;
  void clear_trace_id() ;
  [[nodiscard]] const ::std::string& trace_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_trace_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_trace_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_trace_id();
  void set_allocated_trace_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_trace_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_trace_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_trace_id();

  public:
  // bytes span_id = 10;
  void clear_span_id() ;
  [[nodiscard]] const ::std::string& span_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_span_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_span_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_span_id();
  void set_allocated_span_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_span_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_span_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_span_id();

  public:
  // .opentelemetry.proto.common.v1.AnyValue body = 5;
  [[nodiscard]] bool has_body()
      const;
  void clear_body() ;
  [[nodiscard]] const ::opentelemetry::proto::common::v1::AnyValue& body() const;
  [[nodiscard]] ::opentelemetry::proto::common::v1::AnyValue* PROTOBUF_NULLABLE release_body();
  ::opentelemetry::proto::common::v1::AnyValue* PROTOBUF_NONNULL mutable_body();
  void set_allocated_body(::opentelemetry::proto::common::v1::AnyValue* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_body(::opentelemetry::proto::common::v1::AnyValue* PROTOBUF_NULLABLE value);
  ::opentelemetry::proto::common::v1::AnyValue* PROTOBUF_NULLABLE unsafe_arena_release_body();

  private:
  const ::opentelemetry::proto::common::v1::AnyValue& _internal_body() const;
  ::opentelemetry::proto::common::v1::AnyValue* PROTOBUF_NONNULL _internal_mutable_body();

  public:
  // fixed64 time_unix_nano = 1;
  void clear_time_unix_nano() ;
  [[nodiscard]] ::uint64_t time_unix_nano() const;
  void set_time_unix_nano(::uint64_t value);

  private:
  ::uint64_t _internal_time_unix_nano() const;
  void _internal_set_time_unix_nano(::uint64_t value);

  public:
  // .opentelemetry.proto.logs.v1.SeverityNumber severity_number = 2;
  void clear_severity_number() ;
  [[nodiscard]] ::opentelemetry::proto::logs::v1::SeverityNumber severity_number() const;
  void set_severity_number(::opentelemetry::proto::logs::v1::SeverityNumber value);

  private:
  ::opentelemetry::proto::logs::v1::SeverityNumber _internal_severity_number() const;
  void _internal_set_severity_number(::opentelemetry::proto::logs::v1::SeverityNumber value);

  public:
  // uint32 dropped_attributes_count = 7;
  void clear_dropped_attributes_count() ;
  [[nodiscard]] ::uint32_t dropped_attributes_count() const;
  void set_dropped_attributes_count(::uint32_t value);

  private:
  ::uint32_t _internal_dropped_attributes_count() const;
  void _internal_set_dropped_attributes_count(::uint32_t value);

  public:
  // fixed64 observed_time_unix_nano = 11;
  void clear_observed_time_unix_nano() ;
  [[nodiscard]] ::uint64_t observed_time_unix_nano() const;
  void set_observed_time_unix_nano(::uint64_t value);

  private:
  ::uint64_t _internal_observed_time_unix_nano() const;
  void _internal_set_observed_time_unix_nano(::uint64_t value);

  public:
  // fixed32 flags = 8;
  void clear_flags() ;
  [[nodiscard]] ::uint32_t flags() const;
  void set_flags(::uint32_t value);

  private:
  ::uint32_t _internal_flags() const;
  void _internal_set_flags(::uint32_t value);

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.logs.v1.LogRecord)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<4, 10,
                          2, 67,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const LogRecord& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::common::v1::KeyValue > attributes_;
    ::google::protobuf::internal::ArenaStringPtr severity_text_;
    ::google::protobuf::internal::ArenaStringPtr trace_id_;
    ::google::protobuf::internal::ArenaStringPtr span_id_;
    ::opentelemetry::proto::common::v1::AnyValue* PROTOBUF_NULLABLE body_;
    ::uint64_t time_unix_nano_;
    int severity_number_;
    ::uint32_t dropped_attributes_count_;
    ::uint64_t observed_time_unix_nano_;
    ::uint32_t flags_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2flogs_2fv1_2flogs_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ScopeLogs final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.logs.v1.ScopeLogs) */ {
 public:
  inline ScopeLogs() : ScopeLogs(nullptr) {}
  ~ScopeLogs() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ScopeLogs* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ScopeLogs));
  }
#endif

  template <typename = void>
  explicit constexpr ScopeLogs(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ScopeLogs(const ScopeLogs& from) : ScopeLogs(nullptr, from) {}
  inline ScopeLogs(ScopeLogs&& from) noexcept : ScopeLogs(nullptr, ::std::move(from)) {}
  inline ScopeLogs& operator=(const ScopeLogs& from) {
    CopyFrom(from);
    return *this;
  }
  inline ScopeLogs& operator=(ScopeLogs&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ScopeLogs& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ScopeLogs>(&ScopeLogs_globals_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(ScopeLogs& a, ScopeLogs& b) { a.Swap(&b); }
  inline void Swap(ScopeLogs* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ScopeLogs* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ScopeLogs* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ScopeLogs>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ScopeLogs& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ScopeLogs& from) { ScopeLogs::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ScopeLogs* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.logs.v1.ScopeLogs"; }

  explicit ScopeLogs(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ScopeLogs(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ScopeLogs& from);
  ScopeLogs(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ScopeLogs&& from) noexcept
      : ScopeLogs(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kLogRecordsFieldNumber = 2,
    kSchemaUrlFieldNumber = 3,
    kScopeFieldNumber = 1,
  };
  // repeated .opentelemetry.proto.logs.v1.LogRecord log_records = 2;
  [[nodiscard]] int log_records_size()
      const;
  private:
  int _internal_log_records_size() const;

  public:
  void clear_log_records() ;
  [[nodiscard]] const ::opentelemetry::proto::logs::v1::LogRecord& log_records(int index) const;
  [[nodiscard]] ::opentelemetry::proto::logs::v1::LogRecord* PROTOBUF_NONNULL mutable_log_records(int index);
  ::opentelemetry::proto::logs::v1::LogRecord* PROTOBUF_NONNULL add_log_records();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::LogRecord>&
  log_records() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::LogRecord>* PROTOBUF_NONNULL
  mutable_log_records();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::LogRecord>& _internal_log_records() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::LogRecord>* PROTOBUF_NONNULL _internal_mutable_log_records();

  public:
  // string schema_url = 3;
  void clear_schema_url() ;
  [[nodiscard]] const ::std::string& schema_url() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_schema_url(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_schema_url();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_schema_url();
  void set_allocated_schema_url(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_schema_url() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_schema_url(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_schema_url();

  public:
  // .opentelemetry.proto.common.v1.InstrumentationScope scope = 1;
  [[nodiscard]] bool has_scope()
      const;
  void clear_scope() ;
  [[nodiscard]] const ::opentelemetry::proto::common::v1::InstrumentationScope& scope() const;
  [[nodiscard]] ::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NULLABLE release_scope();
  ::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NONNULL mutable_scope();
  void set_allocated_scope(::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_scope(::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NULLABLE value);
  ::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NULLABLE unsafe_arena_release_scope();

  private:
  const ::opentelemetry::proto::common::v1::InstrumentationScope& _internal_scope() const;
  ::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NONNULL _internal_mutable_scope();

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.logs.v1.ScopeLogs)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 3,
                          2, 56,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ScopeLogs& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::logs::v1::LogRecord > log_records_;
    ::google::protobuf::internal::ArenaStringPtr schema_url_;
    ::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NULLABLE scope_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2flogs_2fv1_2flogs_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ResourceLogs final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.logs.v1.ResourceLogs) */ {
 public:
  inline ResourceLogs() : ResourceLogs(nullptr) {}
  ~ResourceLogs() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ResourceLogs* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ResourceLogs));
  }
#endif

  template <typename = void>
  explicit constexpr ResourceLogs(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ResourceLogs(const ResourceLogs& from) : ResourceLogs(nullptr, from) {}
  inline ResourceLogs(ResourceLogs&& from) noexcept : ResourceLogs(nullptr, ::std::move(from)) {}
  inline ResourceLogs& operator=(const ResourceLogs& from) {
    CopyFrom(from);
    return *this;
  }
  inline ResourceLogs& operator=(ResourceLogs&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ResourceLogs& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ResourceLogs>(&ResourceLogs_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(ResourceLogs& a, ResourceLogs& b) { a.Swap(&b); }
  inline void Swap(ResourceLogs* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ResourceLogs* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ResourceLogs* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ResourceLogs>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ResourceLogs& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ResourceLogs& from) { ResourceLogs::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ResourceLogs* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.logs.v1.ResourceLogs"; }

  explicit ResourceLogs(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ResourceLogs(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ResourceLogs& from);
  ResourceLogs(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ResourceLogs&& from) noexcept
      : ResourceLogs(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kScopeLogsFieldNumber = 2,
    kSchemaUrlFieldNumber = 3,
    kResourceFieldNumber = 1,
  };
  // repeated .opentelemetry.proto.logs.v1.ScopeLogs scope_logs = 2;
  [[nodiscard]] int scope_logs_size()
      const;
  private:
  int _internal_scope_logs_size() const;

  public:
  void clear_scope_logs() ;
  [[nodiscard]] const ::opentelemetry::proto::logs::v1::ScopeLogs& scope_logs(int index) const;
  [[nodiscard]] ::opentelemetry::proto::logs::v1::ScopeLogs* PROTOBUF_NONNULL mutable_scope_logs(int index);
  ::opentelemetry::proto::logs::v1::ScopeLogs* PROTOBUF_NONNULL add_scope_logs();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ScopeLogs>&
  scope_logs() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ScopeLogs>* PROTOBUF_NONNULL
  mutable_scope_logs();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ScopeLogs>& _internal_scope_logs() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ScopeLogs>* PROTOBUF_NONNULL _internal_mutable_scope_logs();

  public:
  // string schema_url = 3;
  void clear_schema_url() ;
  [[nodiscard]] const ::std::string& schema_url() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_schema_url(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_schema_url();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_schema_url();
  void set_allocated_schema_url(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_schema_url() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_schema_url(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_schema_url();

  public:
  // .opentelemetry.proto.resource.v1.Resource resource = 1;
  [[nodiscard]] bool has_resource()
      const;
  void clear_resource() ;
  [[nodiscard]] const ::opentelemetry::proto::resource::v1::Resource& resource() const;
  [[nodiscard]] ::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NULLABLE release_resource();
  ::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NONNULL mutable_resource();
  void set_allocated_resource(::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_resource(::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NULLABLE value);
  ::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NULLABLE unsafe_arena_release_resource();

  private:
  const ::opentelemetry::proto::resource::v1::Resource& _internal_resource() const;
  ::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NONNULL _internal_mutable_resource();

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.logs.v1.ResourceLogs)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 3,
                          2, 59,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ResourceLogs& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::logs::v1::ScopeLogs > scope_logs_;
    ::google::protobuf::internal::ArenaStringPtr schema_url_;
    ::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NULLABLE resource_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2flogs_2fv1_2flogs_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED LogsData final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.logs.v1.LogsData) */ {
 public:
  inline LogsData() : LogsData(nullptr) {}
  ~LogsData() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(LogsData* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(LogsData));
  }
#endif

  template <typename = void>
  explicit constexpr LogsData(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline LogsData(const LogsData& from) : LogsData(nullptr, from) {}
  inline LogsData(LogsData&& from) noexcept : LogsData(nullptr, ::std::move(from)) {}
  inline LogsData& operator=(const LogsData& from) {
    CopyFrom(from);
    return *this;
  }
  inline LogsData& operator=(LogsData&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const LogsData& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<LogsData>(&LogsData_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(LogsData& a, LogsData& b) { a.Swap(&b); }
  inline void Swap(LogsData* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(LogsData* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] LogsData* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<LogsData>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const LogsData& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const LogsData& from) { LogsData::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(LogsData* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.logs.v1.LogsData"; }

  explicit LogsData(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  LogsData(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const LogsData& from);
  LogsData(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, LogsData&& from) noexcept
      : LogsData(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kResourceLogsFieldNumber = 1,
  };
  // repeated .opentelemetry.proto.logs.v1.ResourceLogs resource_logs = 1;
  [[nodiscard]] int resource_logs_size()
      const;
  private:
  int _internal_resource_logs_size() const;

  public:
  void clear_resource_logs() ;
  [[nodiscard]] const ::opentelemetry::proto::logs::v1::ResourceLogs& resource_logs(int index) const;
  [[nodiscard]] ::opentelemetry::proto::logs::v1::ResourceLogs* PROTOBUF_NONNULL mutable_resource_logs(int index);
  ::opentelemetry::proto::logs::v1::ResourceLogs* PROTOBUF_NONNULL add_resource_logs();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ResourceLogs>&
  resource_logs() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ResourceLogs>* PROTOBUF_NONNULL
  mutable_resource_logs();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ResourceLogs>& _internal_resource_logs() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ResourceLogs>* PROTOBUF_NONNULL _internal_mutable_resource_logs();

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.logs.v1.LogsData)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const LogsData& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::logs::v1::ResourceLogs > resource_logs_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2flogs_2fv1_2flogs_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// LogsData

// repeated .opentelemetry.proto.logs.v1.ResourceLogs resource_logs = 1;
inline int LogsData::_internal_resource_logs_size() const {
  return _internal_resource_logs().size();
}
inline int LogsData::resource_logs_size() const {
  return _internal_resource_logs_size();
}
inline void LogsData::clear_resource_logs() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.resource_logs_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::opentelemetry::proto::logs::v1::ResourceLogs& LogsData::resource_logs(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.logs.v1.LogsData.resource_logs)
  return _internal_resource_logs().Get(index);
}
inline ::opentelemetry::proto::logs::v1::ResourceLogs* PROTOBUF_NONNULL LogsData::mutable_resource_logs(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.logs.v1.LogsData.resource_logs)
  return _internal_mutable_resource_logs()->Mutable(index);
}
inline ::opentelemetry::proto::logs::v1::ResourceLogs* PROTOBUF_NONNULL LogsData::add_resource_logs()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::logs::v1::ResourceLogs* _add =
      _internal_mutable_resource_logs()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.logs.v1.LogsData.resource_logs)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ResourceLogs>& LogsData::resource_logs() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.logs.v1.LogsData.resource_logs)
  return _internal_resource_logs();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ResourceLogs>* PROTOBUF_NONNULL
LogsData::mutable_resource_logs() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.logs.v1.LogsData.resource_logs)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_resource_logs();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ResourceLogs>&
LogsData::_internal_resource_logs() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.resource_logs_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ResourceLogs>* PROTOBUF_NONNULL
LogsData::_internal_mutable_resource_logs() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.resource_logs_;
}

// -------------------------------------------------------------------

// ResourceLogs

// .opentelemetry.proto.resource.v1.Resource resource = 1;
inline bool ResourceLogs::has_resource() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.resource_ != nullptr);
  return value;
}
inline const ::opentelemetry::proto::resource::v1::Resource& ResourceLogs::_internal_resource() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::opentelemetry::proto::resource::v1::Resource* p = _impl_.resource_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::opentelemetry::proto::resource::v1::Resource>(&::opentelemetry::proto::resource::v1::Resource_globals_);
}
inline const ::opentelemetry::proto::resource::v1::Resource& ResourceLogs::resource() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.logs.v1.ResourceLogs.resource)
  return _internal_resource();
}
inline void ResourceLogs::unsafe_arena_set_allocated_resource(
    ::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.resource_);
  }
  _impl_.resource_ = reinterpret_cast<::opentelemetry::proto::resource::v1::Resource*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:opentelemetry.proto.logs.v1.ResourceLogs.resource)
}
inline ::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NULLABLE ResourceLogs::release_resource() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::opentelemetry::proto::resource::v1::Resource* released = _impl_.resource_;
  _impl_.resource_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NULLABLE ResourceLogs::unsafe_arena_release_resource() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.logs.v1.ResourceLogs.resource)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::opentelemetry::proto::resource::v1::Resource* temp = _impl_.resource_;
  _impl_.resource_ = nullptr;
  return temp;
}
inline ::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NONNULL ResourceLogs::_internal_mutable_resource() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.resource_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::opentelemetry::proto::resource::v1::Resource>(GetArena());
    _impl_.resource_ = reinterpret_cast<::opentelemetry::proto::resource::v1::Resource*>(p);
  }
  return _impl_.resource_;
}
inline ::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NONNULL ResourceLogs::mutable_resource()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::opentelemetry::proto::resource::v1::Resource* _msg = _internal_mutable_resource();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.logs.v1.ResourceLogs.resource)
  return _msg;
}
inline void ResourceLogs::set_allocated_resource(::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.resource_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.resource_ = reinterpret_cast<::opentelemetry::proto::resource::v1::Resource*>(value);
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.logs.v1.ResourceLogs.resource)
}

// repeated .opentelemetry.proto.logs.v1.ScopeLogs scope_logs = 2;
inline int ResourceLogs::_internal_scope_logs_size() const {
  return _internal_scope_logs().size();
}
inline int ResourceLogs::scope_logs_size() const {
  return _internal_scope_logs_size();
}
inline void ResourceLogs::clear_scope_logs() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.scope_logs_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::opentelemetry::proto::logs::v1::ScopeLogs& ResourceLogs::scope_logs(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.logs.v1.ResourceLogs.scope_logs)
  return _internal_scope_logs().Get(index);
}
inline ::opentelemetry::proto::logs::v1::ScopeLogs* PROTOBUF_NONNULL ResourceLogs::mutable_scope_logs(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.logs.v1.ResourceLogs.scope_logs)
  return _internal_mutable_scope_logs()->Mutable(index);
}
inline ::opentelemetry::proto::logs::v1::ScopeLogs* PROTOBUF_NONNULL ResourceLogs::add_scope_logs()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::logs::v1::ScopeLogs* _add =
      _internal_mutable_scope_logs()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.logs.v1.ResourceLogs.scope_logs)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ScopeLogs>& ResourceLogs::scope_logs() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.logs.v1.ResourceLogs.scope_logs)
  return _internal_scope_logs();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ScopeLogs>* PROTOBUF_NONNULL
ResourceLogs::mutable_scope_logs() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.logs.v1.ResourceLogs.scope_logs)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_scope_logs();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ScopeLogs>&
ResourceLogs::_internal_scope_logs() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.scope_logs_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::ScopeLogs>* PROTOBUF_NONNULL
ResourceLogs::_internal_mutable_scope_logs() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.scope_logs_;
}

// string schema_url = 3;
inline void ResourceLogs::clear_schema_url() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.schema_url_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& ResourceLogs::schema_url() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.logs.v1.ResourceLogs.schema_url)
  return _internal_schema_url();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ResourceLogs::set_schema_url(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.schema_url_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:opentelemetry.proto.logs.v1.ResourceLogs.schema_url)
}
inline ::std::string* PROTOBUF_NONNULL ResourceLogs::mutable_schema_url()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_schema_url();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.logs.v1.ResourceLogs.schema_url)
  return _s;
}
inline const ::std::string& ResourceLogs::_internal_schema_url() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.schema_url_.Get();
}
inline void ResourceLogs::_internal_set_schema_url(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.schema_url_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ResourceLogs::_internal_mutable_schema_url() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.schema_url_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ResourceLogs::release_schema_url() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.logs.v1.ResourceLogs.schema_url)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.schema_url_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.schema_url_.Set("", GetArena());
  }
  return released;
}
inline void ResourceLogs::set_allocated_schema_url(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.schema_url_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.schema_url_.IsDefault()) {
    _impl_.schema_url_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.logs.v1.ResourceLogs.schema_url)
}

// -------------------------------------------------------------------

// ScopeLogs

// .opentelemetry.proto.common.v1.InstrumentationScope scope = 1;
inline bool ScopeLogs::has_scope() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.scope_ != nullptr);
  return value;
}
inline const ::opentelemetry::proto::common::v1::InstrumentationScope& ScopeLogs::_internal_scope() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::opentelemetry::proto::common::v1::InstrumentationScope* p = _impl_.scope_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::opentelemetry::proto::common::v1::InstrumentationScope>(&::opentelemetry::proto::common::v1::InstrumentationScope_globals_);
}
inline const ::opentelemetry::proto::common::v1::InstrumentationScope& ScopeLogs::scope() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.logs.v1.ScopeLogs.scope)
  return _internal_scope();
}
inline void ScopeLogs::unsafe_arena_set_allocated_scope(
    ::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.scope_);
  }
  _impl_.scope_ = reinterpret_cast<::opentelemetry::proto::common::v1::InstrumentationScope*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:opentelemetry.proto.logs.v1.ScopeLogs.scope)
}
inline ::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NULLABLE ScopeLogs::release_scope() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::opentelemetry::proto::common::v1::InstrumentationScope* released = _impl_.scope_;
  _impl_.scope_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NULLABLE ScopeLogs::unsafe_arena_release_scope() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.logs.v1.ScopeLogs.scope)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::opentelemetry::proto::common::v1::InstrumentationScope* temp = _impl_.scope_;
  _impl_.scope_ = nullptr;
  return temp;
}
inline ::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NONNULL ScopeLogs::_internal_mutable_scope() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.scope_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::opentelemetry::proto::common::v1::InstrumentationScope>(GetArena());
    _impl_.scope_ = reinterpret_cast<::opentelemetry::proto::common::v1::InstrumentationScope*>(p);
  }
  return _impl_.scope_;
}
inline ::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NONNULL ScopeLogs::mutable_scope()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::opentelemetry::proto::common::v1::InstrumentationScope* _msg = _internal_mutable_scope();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.logs.v1.ScopeLogs.scope)
  return _msg;
}
inline void ScopeLogs::set_allocated_scope(::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.scope_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.scope_ = reinterpret_cast<::opentelemetry::proto::common::v1::InstrumentationScope*>(value);
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.logs.v1.ScopeLogs.scope)
}

// repeated .opentelemetry.proto.logs.v1.LogRecord log_records = 2;
inline int ScopeLogs::_internal_log_records_size() const {
  return _internal_log_records().size();
}
inline int ScopeLogs::log_records_size() const {
  return _internal_log_records_size();
}
inline void ScopeLogs::clear_log_records() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.log_records_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::opentelemetry::proto::logs::v1::LogRecord& ScopeLogs::log_records(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.logs.v1.ScopeLogs.log_records)
  return _internal_log_records().Get(index);
}
inline ::opentelemetry::proto::logs::v1::LogRecord* PROTOBUF_NONNULL ScopeLogs::mutable_log_records(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.logs.v1.ScopeLogs.log_records)
  return _internal_mutable_log_records()->Mutable(index);
}
inline ::opentelemetry::proto::logs::v1::LogRecord* PROTOBUF_NONNULL ScopeLogs::add_log_records()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::logs::v1::LogRecord* _add =
      _internal_mutable_log_records()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.logs.v1.ScopeLogs.log_records)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::LogRecord>& ScopeLogs::log_records() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.logs.v1.ScopeLogs.log_records)
  return _internal_log_records();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::LogRecord>* PROTOBUF_NONNULL
ScopeLogs::mutable_log_records() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.logs.v1.ScopeLogs.log_records)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_log_records();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::LogRecord>&
ScopeLogs::_internal_log_records() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.log_records_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::logs::v1::LogRecord>* PROTOBUF_NONNULL
ScopeLogs::_internal_mutable_log_records() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.log_records_;
}

// string schema_url = 3;
inline void ScopeLogs::clear_schema_url() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.schema_url_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& ScopeLogs::schema_url() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.logs.v1.ScopeLogs.schema_url)
  return _internal_schema_url();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ScopeLogs::set_schema_url(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.schema_url_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:opentelemetry.proto.logs.v1.ScopeLogs.schema_url)
}
inline ::std::string* PROTOBUF_NONNULL ScopeLogs::mutable_schema_url()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_schema_url();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.logs.v1.ScopeLogs.schema_url)
  return _s;
}
inline const ::std::string& ScopeLogs::_internal_schema_url() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.schema_url_.Get();
}
inline void ScopeLogs::_internal_set_schema_url(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.schema_url_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ScopeLogs::_internal_mutable_schema_url() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.schema_url_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ScopeLogs::release_schema_url() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.logs.v1.ScopeLogs.schema_url)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.schema_url_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.schema_url_.Set("", GetArena());
  }
  return released;
}
inline void ScopeLogs::set_allocated_schema_url(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.schema_url_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.schema_url_.IsDefault()) {
    _impl_.schema_url_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.logs.v1.ScopeLogs.schema_url)
}

// -------------------------------------------------------------------

// LogRecord

// fixed64 time_unix_nano = 1;
inline void LogRecord::clear_time_unix_nano() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.time_unix_nano_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::uint64_t LogRecord::time_unix_nano() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.logs.v1.LogRecord.time_unix_nano)
  return _internal_time_unix_nano();
}
inline void LogRecord::set_time_unix_nano(::uint64_t value) {
  _internal_set_time_unix_nano(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.logs.v1.LogRecord.time_unix_nano)
}
inline ::uint64_t LogRecord::_internal_time_unix_nano() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.time_unix_nano_;
}
inline void LogRecord::_internal_set_time_unix_nano(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.time_unix_nano_ = value;
}

// fixed64 observed_time_unix_nano = 11;
inline void LogRecord::clear_observed_time_unix_nano() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.observed_time_unix_nano_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000100U);
}
inline ::uint64_t LogRecord::observed_time_unix_nano() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.logs.v1.LogRecord.observed_time_unix_nano)
  return _internal_observed_time_unix_nano();
}
inline void LogRecord::set_observed_time_unix_nano(::uint64_t value) {
  _internal_set_observed_time_unix_nano(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000100U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.logs.v1.LogRecord.observed_time_unix_nano)
}
inline ::uint64_t LogRecord::_internal_observed_time_unix_nano() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.observed_time_unix_nano_;
}
inline void LogRecord::_internal_set_observed_time_unix_nano(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.observed_time_unix_nano_ = value;
}

// .opentelemetry.proto.logs.v1.SeverityNumber severity_number = 2;
inline void LogRecord::clear_severity_number() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.severity_number_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::opentelemetry::proto::logs::v1::SeverityNumber LogRecord::severity_number() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.logs.v1.LogRecord.severity_number)
  return _internal_severity_number();
}
inline void LogRecord::set_severity_number(::opentelemetry::proto::logs::v1::SeverityNumber value) {
  _internal_set_severity_number(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.logs.v1.LogRecord.severity_number)
}
inline ::opentelemetry::proto::logs::v1::SeverityNumber LogRecord::_internal_severity_number() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::opentelemetry::proto::logs::v1::SeverityNumber>(_impl_.severity_number_);
}
inline void LogRecord::_internal_set_severity_number(::opentelemetry::proto::logs::v1::SeverityNumber value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.severity_number_ = value;
}

// string severity_text = 3;
inline void LogRecord::clear_severity_text() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.severity_text_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& LogRecord::severity_text() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.logs.v1.LogRecord.severity_text)
  return _internal_severity_text();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void LogRecord::set_severity_text(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.severity_text_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:opentelemetry.proto.logs.v1.LogRecord.severity_text)
}
inline ::std::string* PROTOBUF_NONNULL LogRecord::mutable_severity_text()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_severity_text();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.logs.v1.LogRecord.severity_text)
  return _s;
}
inline const ::std::string& LogRecord::_internal_severity_text() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.severity_text_.Get();
}
inline void LogRecord::_internal_set_severity_text(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.severity_text_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL LogRecord::_internal_mutable_severity_text() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.severity_text_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE LogRecord::release_severity_text() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.logs.v1.LogRecord.severity_text)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.severity_text_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.severity_text_.Set("", GetArena());
  }
  return released;
}
inline void LogRecord::set_allocated_severity_text(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.severity_text_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.severity_text_.IsDefault()) {
    _impl_.severity_text_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.logs.v1.LogRecord.severity_text)
}

// .opentelemetry.proto.common.v1.AnyValue body = 5;
inline bool LogRecord::has_body() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000010U);
  PROTOBUF_ASSUME(!value || _impl_.body_ != nullptr);
  return value;
}
inline const ::opentelemetry::proto::common::v1::AnyValue& LogRecord::_internal_body() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::opentelemetry::proto::common::v1::AnyValue* p = _impl_.body_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::opentelemetry::proto::common::v1::AnyValue>(&::opentelemetry::proto::common::v1::AnyValue_globals_);
}
inline const ::opentelemetry::proto::common::v1::AnyValue& LogRecord::body() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.logs.v1.LogRecord.body)
  return _internal_body();
}
inline void LogRecord::unsafe_arena_set_allocated_body(
    ::opentelemetry::proto::common::v1::AnyValue* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.body_);
  }
  _impl_.body_ = reinterpret_cast<::opentelemetry::proto::common::v1::AnyValue*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:opentelemetry.proto.logs.v1.LogRecord.body)
}
inline ::opentelemetry::proto::common::v1::AnyValue* PROTOBUF_NULLABLE LogRecord::release_body() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::opentelemetry::proto::common::v1::AnyValue* released = _impl_.body_;
  _impl_.body_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::opentelemetry::proto::common::v1::AnyValue* PROTOBUF_NULLABLE LogRecord::unsafe_arena_release_body() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.logs.v1.LogRecord.body)

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::opentelemetry::proto::common::v1::AnyValue* temp = _impl_.body_;
  _impl_.body_ = nullptr;
  return temp;
}
inline ::opentelemetry::proto::common::v1::AnyValue* PROTOBUF_NONNULL LogRecord::_internal_mutable_body() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.body_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::opentelemetry::proto::common::v1::AnyValue>(GetArena());
    _impl_.body_ = reinterpret_cast<::opentelemetry::proto::common::v1::AnyValue*>(p);
  }
  return _impl_.body_;
}
inline ::opentelemetry::proto::common::v1::AnyValue* PROTOBUF_NONNULL LogRecord::mutable_body()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::opentelemetry::proto::common::v1::AnyValue* _msg = _internal_mutable_body();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.logs.v1.LogRecord.body)
  return _msg;
}
inline void LogRecord::set_allocated_body(::opentelemetry::proto::common::v1::AnyValue* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.body_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }

  _impl_.body_ = reinterpret_cast<::opentelemetry::proto::common::v1::AnyValue*>(value);
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.logs.v1.LogRecord.body)
}

// repeated .opentelemetry.proto.common.v1.KeyValue attributes = 6;
inline int LogRecord::_internal_attributes_size() const {
  return _internal_attributes().size();
}
inline int LogRecord::attributes_size() const {
  return _internal_attributes_size();
}
inline const ::opentelemetry::proto::common::v1::KeyValue& LogRecord::attributes(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.logs.v1.LogRecord.attributes)
  return _internal_attributes().Get(index);
}
inline ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL LogRecord::mutable_attributes(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.logs.v1.LogRecord.attributes)
  return _internal_mutable_attributes()->Mutable(index);
}
inline ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL LogRecord::add_attributes()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::common::v1::KeyValue* _add =
      _internal_mutable_attributes()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.logs.v1.LogRecord.attributes)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>& LogRecord::attributes() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.logs.v1.LogRecord.attributes)
  return _internal_attributes();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL
LogRecord::mutable_attributes() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.logs.v1.LogRecord.attributes)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_attributes();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>&
LogRecord::_internal_attributes() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.attributes_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL
LogRecord::_internal_mutable_attributes() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.attributes_;
}

// uint32 dropped_attributes_count = 7;
inline void LogRecord::clear_dropped_attributes_count() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.dropped_attributes_count_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
}
inline ::uint32_t LogRecord::dropped_attributes_count() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.logs.v1.LogRecord.dropped_attributes_count)
  return _internal_dropped_attributes_count();
}
inline void LogRecord::set_dropped_attributes_count(::uint32_t value) {
  _internal_set_dropped_attributes_count(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.logs.v1.LogRecord.dropped_attributes_count)
}
inline ::uint32_t LogRecord::_internal_dropped_attributes_count() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.dropped_attributes_count_;
}
inline void LogRecord::_internal_set_dropped_attributes_count(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.dropped_attributes_count_ = value;
}

// fixed32 flags = 8;
inline void LogRecord::clear_flags() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000200U);
}
inline ::uint32_t LogRecord::flags() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.logs.v1.LogRecord.flags)
  return _internal_flags();
}
inline void LogRecord::set_flags(::uint32_t value) {
  _internal_set_flags(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000200U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.logs.v1.LogRecord.flags)
}
inline ::uint32_t LogRecord::_internal_flags() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.flags_;
}
inline void LogRecord::_internal_set_flags(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = value;
}

// bytes trace_id = 9;
inline void LogRecord::clear_trace_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.trace_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::std::string& LogRecord::trace_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.logs.v1.LogRecord.trace_id)
  return _internal_trace_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void LogRecord::set_trace_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.trace_id_.SetBytes(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:opentelemetry.proto.logs.v1.LogRecord.trace_id)
}
inline ::std::string* PROTOBUF_NONNULL LogRecord::mutable_trace_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_trace_id();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.logs.v1.LogRecord.trace_id)
  return _s;
}
inline const ::std::string& LogRecord::_internal_trace_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.trace_id_.Get();
}
inline void LogRecord::_internal_set_trace_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.trace_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL LogRecord::_internal_mutable_trace_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.trace_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE LogRecord::release_trace_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.logs.v1.LogRecord.trace_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.trace_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.trace_id_.Set("", GetArena());
  }
  return released;
}
inline void LogRecord::set_allocated_trace_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.trace_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.trace_id_.IsDefault()) {
    _impl_.trace_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.logs.v1.LogRecord.trace_id)
}

// bytes span_id = 10;
inline void LogRecord::clear_span_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.span_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::std::string& LogRecord::span_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.logs.v1.LogRecord.span_id)
  return _internal_span_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void LogRecord::set_span_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  _impl_.span_id_.SetBytes(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:opentelemetry.proto.logs.v1.LogRecord.span_id)
}
inline ::std::string* PROTOBUF_NONNULL LogRecord::mutable_span_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::std::string* _s = _internal_mutable_span_id();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.logs.v1.LogRecord.span_id)
  return _s;
}
inline const ::std::string& LogRecord::_internal_span_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.span_id_.Get();
}
inline void LogRecord::_internal_set_span_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.span_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL LogRecord::_internal_mutable_span_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.span_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE LogRecord::release_span_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.logs.v1.LogRecord.span_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000008U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  auto* released = _impl_.span_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.span_id_.Set("", GetArena());
  }
  return released;
}
inline void LogRecord::set_allocated_span_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  _impl_.span_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.span_id_.IsDefault()) {
    _impl_.span_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.logs.v1.LogRecord.span_id)
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace v1
}  // namespace logs
}  // namespace proto
}  // namespace opentelemetry


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::opentelemetry::proto::logs::v1::SeverityNumber> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::opentelemetry::proto::logs::v1::SeverityNumber>() {
  return ::opentelemetry::proto::logs::v1::SeverityNumber_descriptor();
}
template <>
struct is_proto_enum<::opentelemetry::proto::logs::v1::LogRecordFlags> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::opentelemetry::proto::logs::v1::LogRecordFlags>() {
  return ::opentelemetry::proto::logs::v1::LogRecordFlags_descriptor();
}

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // opentelemetry_2fproto_2flogs_2fv1_2flogs_2eproto_2epb_2eh
