// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: google/cloud/bigquery/storage/v1/avro.proto
// Protobuf C++ Version: 7.35.1

#include "google/cloud/bigquery/storage/v1/avro.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
#ifdef PROTOBUF_MESSAGE_GLOBALS
namespace {
PROTOBUF_CONSTINIT ::google::protobuf::internal::ReflectionData
    file_reflection_data[] = {
        // ::google::cloud::bigquery::storage::v1::AvroSchema
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::AvroRows
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::AvroSerializationOptions
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto, /* tracker*/ nullptr,},
};
}  // namespace
#endif
namespace google {
namespace cloud {
namespace bigquery {
namespace storage {
namespace v1 {
class AvroSerializationOptions::_Internal {
 public:
  using HasBits = decltype(::std::declval<AvroSerializationOptions>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(AvroSerializationOptions, _impl_._has_bits_);
};

constexpr AvroSerializationOptions::ParseTableT_ AvroSerializationOptions::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(AvroSerializationOptions, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::AvroSerializationOptions>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // bool enable_display_name_attribute = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(AvroSerializationOptions, _impl_.enable_display_name_attribute_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(AvroSerializationOptions, _impl_.enable_display_name_attribute_)}},
    }}, {{
      65535, 65535
    }}, {{
      // bool enable_display_name_attribute = 1;
      {PROTOBUF_FIELD_OFFSET(AvroSerializationOptions, _impl_.enable_display_name_attribute_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr AvroSerializationOptions::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        enable_display_name_attribute_{false} {}

template <typename>
constexpr AvroSerializationOptions::AvroSerializationOptions(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL AvroSerializationOptions::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) AvroSerializationOptions(arena);
}
constexpr auto AvroSerializationOptions::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(AvroSerializationOptions), alignof(AvroSerializationOptions));
}
constexpr auto AvroSerializationOptions::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &AvroSerializationOptions::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<AvroSerializationOptions>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &AvroSerializationOptions::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<AvroSerializationOptions>(), &AvroSerializationOptions::ByteSizeLong,
              &AvroSerializationOptions::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(AvroSerializationOptions, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[2],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct AvroSerializationOptionsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr AvroSerializationOptionsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 AvroSerializationOptions_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(AvroSerializationOptions::InternalGenerateClassData_(
            _default, &AvroSerializationOptions_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<AvroSerializationOptions>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~AvroSerializationOptionsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) AvroSerializationOptions _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<AvroSerializationOptions>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(AvroSerializationOptionsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST AvroSerializationOptionsGlobalsTypeInternal AvroSerializationOptions_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* AvroSerializationOptions_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return AvroSerializationOptions_globals_.GetClassData();
#else
  return AvroSerializationOptions_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class AvroSchema::_Internal {
 public:
  using HasBits = decltype(::std::declval<AvroSchema>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(AvroSchema, _impl_._has_bits_);
};

constexpr AvroSchema::ParseTableT_ AvroSchema::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(AvroSchema, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::AvroSchema>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string schema = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(AvroSchema, _impl_.schema_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string schema = 1;
      {PROTOBUF_FIELD_OFFSET(AvroSchema, _impl_.schema_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\53\6\0\0\0\0\0\0"
      "google.cloud.bigquery.storage.v1.AvroSchema"
      "schema"
    }},
  };
}


inline constexpr AvroSchema::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        schema_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr AvroSchema::AvroSchema(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL AvroSchema::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) AvroSchema(arena);
}
constexpr auto AvroSchema::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(AvroSchema), alignof(AvroSchema));
}
constexpr auto AvroSchema::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &AvroSchema::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<AvroSchema>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &AvroSchema::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<AvroSchema>(), &AvroSchema::ByteSizeLong,
              &AvroSchema::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(AvroSchema, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[0],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct AvroSchemaGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr AvroSchemaGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 AvroSchema_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(AvroSchema::InternalGenerateClassData_(
            _default, &AvroSchema_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<AvroSchema>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~AvroSchemaGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) AvroSchema _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<AvroSchema>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(AvroSchemaGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST AvroSchemaGlobalsTypeInternal AvroSchema_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* AvroSchema_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return AvroSchema_globals_.GetClassData();
#else
  return AvroSchema_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class AvroRows::_Internal {
 public:
  using HasBits = decltype(::std::declval<AvroRows>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(AvroRows, _impl_._has_bits_);
};

constexpr AvroRows::ParseTableT_ AvroRows::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(AvroRows, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::AvroRows>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // int64 row_count = 2 [deprecated = true];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(AvroRows, _impl_.row_count_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(AvroRows, _impl_.row_count_)}},
      // bytes serialized_binary_rows = 1;
      {::_pbi::TcParser::FastBS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(AvroRows, _impl_.serialized_binary_rows_)}},
    }}, {{
      65535, 65535
    }}, {{
      // bytes serialized_binary_rows = 1;
      {PROTOBUF_FIELD_OFFSET(AvroRows, _impl_.serialized_binary_rows_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
      // int64 row_count = 2 [deprecated = true];
      {PROTOBUF_FIELD_OFFSET(AvroRows, _impl_.row_count_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr AvroRows::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        serialized_binary_rows_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        row_count_{::int64_t{0}} {}

template <typename>
constexpr AvroRows::AvroRows(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL AvroRows::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) AvroRows(arena);
}
constexpr auto AvroRows::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(AvroRows), alignof(AvroRows));
}
constexpr auto AvroRows::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &AvroRows::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<AvroRows>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &AvroRows::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<AvroRows>(), &AvroRows::ByteSizeLong,
              &AvroRows::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(AvroRows, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[1],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct AvroRowsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr AvroRowsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 AvroRows_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(AvroRows::InternalGenerateClassData_(
            _default, &AvroRows_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<AvroRows>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~AvroRowsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) AvroRows _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<AvroRows>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(AvroRowsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST AvroRowsGlobalsTypeInternal AvroRows_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* AvroRows_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return AvroRows_globals_.GetClassData();
#else
  return AvroRows_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
}  // namespace v1
}  // namespace storage
}  // namespace bigquery
}  // namespace cloud
}  // namespace google
static constexpr const ::_pb::EnumDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_enum_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto = nullptr;
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto = nullptr;
const ::uint32_t
    TableStruct_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AvroSchema, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AvroSchema, _impl_.schema_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AvroRows, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AvroRows, _impl_.serialized_binary_rows_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AvroRows, _impl_.row_count_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AvroSerializationOptions, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AvroSerializationOptions, _impl_.enable_display_name_attribute_),
        0,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::google::cloud::bigquery::storage::v1::AvroSchema)},
        {5, sizeof(::google::cloud::bigquery::storage::v1::AvroRows)},
        {12, sizeof(::google::cloud::bigquery::storage::v1::AvroSerializationOptions)},
};
static const ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const
    file_message_globals[] = {
        &::google::cloud::bigquery::storage::v1::AvroSchema_globals_,
        &::google::cloud::bigquery::storage::v1::AvroRows_globals_,
        &::google::cloud::bigquery::storage::v1::AvroSerializationOptions_globals_,
};
const char descriptor_table_protodef_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n+google/cloud/bigquery/storage/v1/avro."
    "proto\022 google.cloud.bigquery.storage.v1\""
    "\034\n\nAvroSchema\022\016\n\006schema\030\001 \001(\t\"A\n\010AvroRow"
    "s\022\036\n\026serialized_binary_rows\030\001 \001(\014\022\025\n\trow"
    "_count\030\002 \001(\003B\002\030\001\"A\n\030AvroSerializationOpt"
    "ions\022%\n\035enable_display_name_attribute\030\001 "
    "\001(\010B\271\001\n$com.google.cloud.bigquery.storag"
    "e.v1B\tAvroProtoP\001Z>cloud.google.com/go/b"
    "igquery/storage/apiv1/storagepb;storagep"
    "b\252\002 Google.Cloud.BigQuery.Storage.V1\312\002 G"
    "oogle\\Cloud\\BigQuery\\Storage\\V1b\006proto3"
};
static ::absl::once_flag descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto = {
    false,
    false,
    439,
    descriptor_table_protodef_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto,
    "google/cloud/bigquery/storage/v1/avro.proto",
    &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto_once,
    nullptr,
    0,
    3,
    schemas,
    file_message_globals,
    TableStruct_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto::offsets,
    file_level_enum_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto,
    file_level_service_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto,
};
namespace google {
namespace cloud {
namespace bigquery {
namespace storage {
namespace v1 {
// ===================================================================

AvroSchema::AvroSchema(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AvroSchema_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.AvroSchema)
}
PROTOBUF_NDEBUG_INLINE AvroSchema::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::AvroSchema& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        schema_(arena, from.schema_) {}

AvroSchema::AvroSchema(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const AvroSchema& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AvroSchema_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  AvroSchema* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.AvroSchema)
}
PROTOBUF_NDEBUG_INLINE AvroSchema::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        schema_(arena) {}

inline void AvroSchema::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
AvroSchema::~AvroSchema() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.AvroSchema)
  SharedDtor(*this);
}
inline void AvroSchema::SharedDtor(MessageLite& self) {
  AvroSchema& this_ = static_cast<AvroSchema&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.schema_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull AvroSchema_class_data_ =
        AvroSchema::InternalGenerateClassData_(AvroSchema_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AvroSchema::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AvroSchema_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(AvroSchema_class_data_.tc_table);
  return AvroSchema_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AvroSchema::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AvroSchema_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&AvroSchema_globals_));
  return AvroSchema_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const AvroSchema::ParseTableT_
    AvroSchema::_table_ =
        AvroSchema::InternalGenerateParseTable_(AvroSchema_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void AvroSchema::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.AvroSchema)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.schema_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL AvroSchema::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const AvroSchema& this_ = static_cast<const AvroSchema&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL AvroSchema::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const AvroSchema& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.AvroSchema)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string schema = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_schema().empty()) {
      const ::std::string& _s = this_._internal_schema();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.cloud.bigquery.storage.v1.AvroSchema.schema");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.AvroSchema)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t AvroSchema::ByteSizeLong(const MessageLite& base) {
  const AvroSchema& this_ = static_cast<const AvroSchema&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t AvroSchema::ByteSizeLong() const {
  const AvroSchema& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.AvroSchema)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string schema = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_schema().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_schema());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void AvroSchema::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<AvroSchema*>(&to_msg);
  auto& from = static_cast<const AvroSchema&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.AvroSchema)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_schema().empty()) {
      _this->_internal_set_schema(from._internal_schema());
    } else {
      if (_this->_impl_.schema_.IsDefault()) {
        _this->_internal_set_schema("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void AvroSchema::CopyFrom(const AvroSchema& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.AvroSchema)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void AvroSchema::InternalSwap(AvroSchema* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.schema_, &other->_impl_.schema_, arena);
}

::google::protobuf::Metadata AvroSchema::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

AvroRows::AvroRows(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AvroRows_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.AvroRows)
}
PROTOBUF_NDEBUG_INLINE AvroRows::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::AvroRows& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        serialized_binary_rows_(arena, from.serialized_binary_rows_) {}

AvroRows::AvroRows(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const AvroRows& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AvroRows_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  AvroRows* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.row_count_ = from._impl_.row_count_;

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.AvroRows)
}
PROTOBUF_NDEBUG_INLINE AvroRows::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        serialized_binary_rows_(arena) {}

inline void AvroRows::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.row_count_ = {};
}
AvroRows::~AvroRows() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.AvroRows)
  SharedDtor(*this);
}
inline void AvroRows::SharedDtor(MessageLite& self) {
  AvroRows& this_ = static_cast<AvroRows&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.serialized_binary_rows_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull AvroRows_class_data_ =
        AvroRows::InternalGenerateClassData_(AvroRows_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AvroRows::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AvroRows_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(AvroRows_class_data_.tc_table);
  return AvroRows_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AvroRows::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AvroRows_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&AvroRows_globals_));
  return AvroRows_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const AvroRows::ParseTableT_
    AvroRows::_table_ =
        AvroRows::InternalGenerateParseTable_(AvroRows_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void AvroRows::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.AvroRows)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.serialized_binary_rows_.ClearNonDefaultToEmpty();
  }
  _impl_.row_count_ = ::int64_t{0};
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL AvroRows::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const AvroRows& this_ = static_cast<const AvroRows&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL AvroRows::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const AvroRows& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.AvroRows)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // bytes serialized_binary_rows = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_serialized_binary_rows().empty()) {
      const ::std::string& _s = this_._internal_serialized_binary_rows();
      target = stream->WriteBytesMaybeAliased(1, _s, target);
    }
  }

  // int64 row_count = 2 [deprecated = true];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_row_count() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<2>(
              stream, this_._internal_row_count(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.AvroRows)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t AvroRows::ByteSizeLong(const MessageLite& base) {
  const AvroRows& this_ = static_cast<const AvroRows&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t AvroRows::ByteSizeLong() const {
  const AvroRows& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.AvroRows)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // bytes serialized_binary_rows = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_serialized_binary_rows().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::BytesSize(
                                        this_._internal_serialized_binary_rows());
      }
    }
    // int64 row_count = 2 [deprecated = true];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_row_count() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_row_count());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void AvroRows::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<AvroRows*>(&to_msg);
  auto& from = static_cast<const AvroRows&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.AvroRows)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_serialized_binary_rows().empty()) {
        _this->_internal_set_serialized_binary_rows(from._internal_serialized_binary_rows());
      } else {
        if (_this->_impl_.serialized_binary_rows_.IsDefault()) {
          _this->_internal_set_serialized_binary_rows("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_row_count() != 0) {
        _this->_impl_.row_count_ = from._impl_.row_count_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void AvroRows::CopyFrom(const AvroRows& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.AvroRows)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void AvroRows::InternalSwap(AvroRows* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.serialized_binary_rows_, &other->_impl_.serialized_binary_rows_, arena);
  swap(_impl_.row_count_, other->_impl_.row_count_);
}

::google::protobuf::Metadata AvroRows::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

AvroSerializationOptions::AvroSerializationOptions(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AvroSerializationOptions_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.AvroSerializationOptions)
}
AvroSerializationOptions::AvroSerializationOptions(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const AvroSerializationOptions& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AvroSerializationOptions_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE AvroSerializationOptions::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void AvroSerializationOptions::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.enable_display_name_attribute_ = {};
}
AvroSerializationOptions::~AvroSerializationOptions() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.AvroSerializationOptions)
  SharedDtor(*this);
}
inline void AvroSerializationOptions::SharedDtor(MessageLite& self) {
  AvroSerializationOptions& this_ = static_cast<AvroSerializationOptions&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull AvroSerializationOptions_class_data_ =
        AvroSerializationOptions::InternalGenerateClassData_(AvroSerializationOptions_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AvroSerializationOptions::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AvroSerializationOptions_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(AvroSerializationOptions_class_data_.tc_table);
  return AvroSerializationOptions_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AvroSerializationOptions::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AvroSerializationOptions_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&AvroSerializationOptions_globals_));
  return AvroSerializationOptions_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const AvroSerializationOptions::ParseTableT_
    AvroSerializationOptions::_table_ =
        AvroSerializationOptions::InternalGenerateParseTable_(AvroSerializationOptions_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void AvroSerializationOptions::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.AvroSerializationOptions)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.enable_display_name_attribute_ = false;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL AvroSerializationOptions::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const AvroSerializationOptions& this_ = static_cast<const AvroSerializationOptions&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL AvroSerializationOptions::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const AvroSerializationOptions& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.AvroSerializationOptions)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // bool enable_display_name_attribute = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_enable_display_name_attribute() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          1, this_._internal_enable_display_name_attribute(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.AvroSerializationOptions)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t AvroSerializationOptions::ByteSizeLong(const MessageLite& base) {
  const AvroSerializationOptions& this_ = static_cast<const AvroSerializationOptions&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t AvroSerializationOptions::ByteSizeLong() const {
  const AvroSerializationOptions& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.AvroSerializationOptions)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // bool enable_display_name_attribute = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_enable_display_name_attribute() != 0) {
        total_size += 2;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void AvroSerializationOptions::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<AvroSerializationOptions*>(&to_msg);
  auto& from = static_cast<const AvroSerializationOptions&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.AvroSerializationOptions)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (from._internal_enable_display_name_attribute() != 0) {
      _this->_impl_.enable_display_name_attribute_ = from._impl_.enable_display_name_attribute_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void AvroSerializationOptions::CopyFrom(const AvroSerializationOptions& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.AvroSerializationOptions)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void AvroSerializationOptions::InternalSwap(AvroSerializationOptions* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.enable_display_name_attribute_, other->_impl_.enable_display_name_attribute_);
}

::google::protobuf::Metadata AvroSerializationOptions::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
}  // namespace v1
}  // namespace storage
}  // namespace bigquery
}  // namespace cloud
}  // namespace google
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
