// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: google/api/annotations.proto
// Protobuf C++ Version: 7.35.1

#include "google/api/annotations.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
namespace google {
namespace api {
}  // namespace api
}  // namespace google
static constexpr const ::_pb::EnumDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_enum_descriptors_google_2fapi_2fannotations_2eproto = nullptr;
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_google_2fapi_2fannotations_2eproto = nullptr;
const ::uint32_t TableStruct_google_2fapi_2fannotations_2eproto::offsets[1] = {};
static constexpr ::_pbi::MigrationSchema* PROTOBUF_NULLABLE schemas = nullptr;
static constexpr ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const* PROTOBUF_NULLABLE
    file_message_globals = nullptr;
const char descriptor_table_protodef_google_2fapi_2fannotations_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n\034google/api/annotations.proto\022\ngoogle.a"
    "pi\032\025google/api/http.proto\032 google/protob"
    "uf/descriptor.proto:E\n\004http\022\036.google.pro"
    "tobuf.MethodOptions\030\260\312\274\" \001(\0132\024.google.ap"
    "i.HttpRuleBn\n\016com.google.apiB\020Annotation"
    "sProtoP\001ZAgoogle.golang.org/genproto/goo"
    "gleapis/api/annotations;annotations\242\002\004GA"
    "PIb\006proto3"
};
static const ::_pbi::DescriptorTable* PROTOBUF_NONNULL const
    descriptor_table_google_2fapi_2fannotations_2eproto_deps[2] = {
        &::descriptor_table_google_2fapi_2fhttp_2eproto,
        &::descriptor_table_google_2fprotobuf_2fdescriptor_2eproto,
};
static ::absl::once_flag descriptor_table_google_2fapi_2fannotations_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_google_2fapi_2fannotations_2eproto = {
    false,
    false,
    290,
    descriptor_table_protodef_google_2fapi_2fannotations_2eproto,
    "google/api/annotations.proto",
    &descriptor_table_google_2fapi_2fannotations_2eproto_once,
    descriptor_table_google_2fapi_2fannotations_2eproto_deps,
    2,
    0,
    schemas,
    file_message_globals,
    TableStruct_google_2fapi_2fannotations_2eproto::offsets,
    file_level_enum_descriptors_google_2fapi_2fannotations_2eproto,
    file_level_service_descriptors_google_2fapi_2fannotations_2eproto,
};
namespace google {
namespace api {
PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 ::_pbi::ExtensionIdentifier<
        ::google::protobuf::MethodOptions, ::_pbi::MessageTypeTraits< ::google::api::HttpRule >, 11, false>
        http(kHttpFieldNumber, &::google::api::HttpRule_globals_);
// @@protoc_insertion_point(namespace_scope)
}  // namespace api
}  // namespace google
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_google_2fapi_2fannotations_2eproto),
        ::_pbi::ExtensionSet::RegisterMessageExtension(
            &::google::protobuf::MethodOptions::default_instance(), 72295728, 11,
            false, false, &::google::api::HttpRule::default_instance(),
            nullptr, ::_pbi::LazyAnnotation::kUndefined),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
