// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: google/cloud/bigquery/storage/v1/arrow.proto
// Protobuf C++ Version: 7.35.1

#ifndef google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto_2epb_2eh
#define google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7035001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_reflection.h"
#include "google/protobuf/unknown_field_set.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto;
}  // extern "C"
namespace google {
namespace cloud {
namespace bigquery {
namespace storage {
namespace v1 {
enum ArrowSerializationOptions_CompressionCodec : int;
extern const uint32_t ArrowSerializationOptions_CompressionCodec_internal_data_[];
class ArrowRecordBatch;
struct ArrowRecordBatchGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ArrowRecordBatchGlobalsTypeInternal ArrowRecordBatch_globals_;
extern const ::google::protobuf::internal::ClassDataFull ArrowRecordBatch_class_data_;
#else
extern const ArrowRecordBatchGlobalsTypeInternal ArrowRecordBatch_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ArrowSchema;
struct ArrowSchemaGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ArrowSchemaGlobalsTypeInternal ArrowSchema_globals_;
extern const ::google::protobuf::internal::ClassDataFull ArrowSchema_class_data_;
#else
extern const ArrowSchemaGlobalsTypeInternal ArrowSchema_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ArrowSerializationOptions;
struct ArrowSerializationOptionsGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ArrowSerializationOptionsGlobalsTypeInternal ArrowSerializationOptions_globals_;
extern const ::google::protobuf::internal::ClassDataFull ArrowSerializationOptions_class_data_;
#else
extern const ArrowSerializationOptionsGlobalsTypeInternal ArrowSerializationOptions_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace v1
}  // namespace storage
}  // namespace bigquery
}  // namespace cloud
namespace protobuf {
template <>
internal::EnumTraitsT<::google::cloud::bigquery::storage::v1::ArrowSerializationOptions_CompressionCodec_internal_data_>
    internal::EnumTraitsImpl::value<::google::cloud::bigquery::storage::v1::ArrowSerializationOptions_CompressionCodec>;
}  // namespace protobuf
}  // namespace google

namespace google {
namespace cloud {
namespace bigquery {
namespace storage {
namespace v1 {
enum ArrowSerializationOptions_CompressionCodec : int {
  ArrowSerializationOptions_CompressionCodec_COMPRESSION_UNSPECIFIED = 0,
  ArrowSerializationOptions_CompressionCodec_LZ4_FRAME = 1,
  ArrowSerializationOptions_CompressionCodec_ZSTD = 2,
  ArrowSerializationOptions_CompressionCodec_ArrowSerializationOptions_CompressionCodec_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  ArrowSerializationOptions_CompressionCodec_ArrowSerializationOptions_CompressionCodec_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t ArrowSerializationOptions_CompressionCodec_internal_data_[];
inline constexpr ArrowSerializationOptions_CompressionCodec ArrowSerializationOptions_CompressionCodec_CompressionCodec_MIN =
    static_cast<ArrowSerializationOptions_CompressionCodec>(0);
inline constexpr ArrowSerializationOptions_CompressionCodec ArrowSerializationOptions_CompressionCodec_CompressionCodec_MAX =
    static_cast<ArrowSerializationOptions_CompressionCodec>(2);
[[nodiscard]] inline bool ArrowSerializationOptions_CompressionCodec_IsValid(int value) {
  return 0 <= value && value <= 2;
}
inline constexpr int ArrowSerializationOptions_CompressionCodec_CompressionCodec_ARRAYSIZE = 2 + 1;
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
ArrowSerializationOptions_CompressionCodec_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(ArrowSerializationOptions_CompressionCodec) {
  return ArrowSerializationOptions_CompressionCodec_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& ArrowSerializationOptions_CompressionCodec_Name(T value) {
  static_assert(::std::is_same<T, ArrowSerializationOptions_CompressionCodec>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to CompressionCodec_Name().");
  return ArrowSerializationOptions_CompressionCodec_Name(static_cast<ArrowSerializationOptions_CompressionCodec>(value));
}
template <>
[[nodiscard]] inline const ::std::string& ArrowSerializationOptions_CompressionCodec_Name(ArrowSerializationOptions_CompressionCodec value) {
  return ::google::protobuf::internal::NameOfDenseEnum<ArrowSerializationOptions_CompressionCodec_descriptor, 0, 2>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool ArrowSerializationOptions_CompressionCodec_Parse(
    ::absl::string_view name, ArrowSerializationOptions_CompressionCodec* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<ArrowSerializationOptions_CompressionCodec>(ArrowSerializationOptions_CompressionCodec_descriptor(), name,
                                           value);
}
using ::google::protobuf::internal::generated_enum::AbslParseFlag;
using ::google::protobuf::internal::generated_enum::AbslUnparseFlag;

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ArrowSerializationOptions final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:google.cloud.bigquery.storage.v1.ArrowSerializationOptions) */ {
 public:
  inline ArrowSerializationOptions() : ArrowSerializationOptions(nullptr) {}
  ~ArrowSerializationOptions() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ArrowSerializationOptions* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ArrowSerializationOptions));
  }
#endif

  template <typename = void>
  explicit constexpr ArrowSerializationOptions(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ArrowSerializationOptions(const ArrowSerializationOptions& from) : ArrowSerializationOptions(nullptr, from) {}
  inline ArrowSerializationOptions(ArrowSerializationOptions&& from) noexcept : ArrowSerializationOptions(nullptr, ::std::move(from)) {}
  inline ArrowSerializationOptions& operator=(const ArrowSerializationOptions& from) {
    CopyFrom(from);
    return *this;
  }
  inline ArrowSerializationOptions& operator=(ArrowSerializationOptions&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ArrowSerializationOptions& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ArrowSerializationOptions>(&ArrowSerializationOptions_globals_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(ArrowSerializationOptions& a, ArrowSerializationOptions& b) { a.Swap(&b); }
  inline void Swap(ArrowSerializationOptions* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ArrowSerializationOptions* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ArrowSerializationOptions* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ArrowSerializationOptions>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ArrowSerializationOptions& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ArrowSerializationOptions& from) { ArrowSerializationOptions::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ArrowSerializationOptions* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "google.cloud.bigquery.storage.v1.ArrowSerializationOptions"; }

  explicit ArrowSerializationOptions(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ArrowSerializationOptions(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ArrowSerializationOptions& from);
  ArrowSerializationOptions(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ArrowSerializationOptions&& from) noexcept
      : ArrowSerializationOptions(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using CompressionCodec = ArrowSerializationOptions_CompressionCodec;
  static constexpr CompressionCodec COMPRESSION_UNSPECIFIED = ArrowSerializationOptions_CompressionCodec_COMPRESSION_UNSPECIFIED;
  static constexpr CompressionCodec LZ4_FRAME = ArrowSerializationOptions_CompressionCodec_LZ4_FRAME;
  static constexpr CompressionCodec ZSTD = ArrowSerializationOptions_CompressionCodec_ZSTD;
  [[nodiscard]] static inline bool CompressionCodec_IsValid(int value) {
    return ArrowSerializationOptions_CompressionCodec_IsValid(value);
  }
  static constexpr CompressionCodec CompressionCodec_MIN = ArrowSerializationOptions_CompressionCodec_CompressionCodec_MIN;
  static constexpr CompressionCodec CompressionCodec_MAX = ArrowSerializationOptions_CompressionCodec_CompressionCodec_MAX;
  static constexpr int CompressionCodec_ARRAYSIZE = ArrowSerializationOptions_CompressionCodec_CompressionCodec_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL CompressionCodec_descriptor() {
    return ArrowSerializationOptions_CompressionCodec_descriptor();
  }
  template <typename T>
  [[nodiscard]] static inline const ::std::string& CompressionCodec_Name(T value) {
    return ArrowSerializationOptions_CompressionCodec_Name(value);
  }
  [[nodiscard]] static inline bool CompressionCodec_Parse(
      ::absl::string_view name, CompressionCodec* PROTOBUF_NONNULL value) {
    return ArrowSerializationOptions_CompressionCodec_Parse(name, value);
  }

  // accessors -------------------------------------------------------
  enum : int {
    kBufferCompressionFieldNumber = 2,
  };
  // .google.cloud.bigquery.storage.v1.ArrowSerializationOptions.CompressionCodec buffer_compression = 2;
  void clear_buffer_compression() ;
  [[nodiscard]] ::google::cloud::bigquery::storage::v1::ArrowSerializationOptions_CompressionCodec buffer_compression() const;
  void set_buffer_compression(::google::cloud::bigquery::storage::v1::ArrowSerializationOptions_CompressionCodec value);

  private:
  ::google::cloud::bigquery::storage::v1::ArrowSerializationOptions_CompressionCodec _internal_buffer_compression() const;
  void _internal_set_buffer_compression(::google::cloud::bigquery::storage::v1::ArrowSerializationOptions_CompressionCodec value);

  public:
  // @@protoc_insertion_point(class_scope:google.cloud.bigquery.storage.v1.ArrowSerializationOptions)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ArrowSerializationOptions& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    int buffer_compression_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ArrowSchema final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:google.cloud.bigquery.storage.v1.ArrowSchema) */ {
 public:
  inline ArrowSchema() : ArrowSchema(nullptr) {}
  ~ArrowSchema() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ArrowSchema* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ArrowSchema));
  }
#endif

  template <typename = void>
  explicit constexpr ArrowSchema(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ArrowSchema(const ArrowSchema& from) : ArrowSchema(nullptr, from) {}
  inline ArrowSchema(ArrowSchema&& from) noexcept : ArrowSchema(nullptr, ::std::move(from)) {}
  inline ArrowSchema& operator=(const ArrowSchema& from) {
    CopyFrom(from);
    return *this;
  }
  inline ArrowSchema& operator=(ArrowSchema&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ArrowSchema& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ArrowSchema>(&ArrowSchema_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(ArrowSchema& a, ArrowSchema& b) { a.Swap(&b); }
  inline void Swap(ArrowSchema* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ArrowSchema* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ArrowSchema* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ArrowSchema>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ArrowSchema& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ArrowSchema& from) { ArrowSchema::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ArrowSchema* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "google.cloud.bigquery.storage.v1.ArrowSchema"; }

  explicit ArrowSchema(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ArrowSchema(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ArrowSchema& from);
  ArrowSchema(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ArrowSchema&& from) noexcept
      : ArrowSchema(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kSerializedSchemaFieldNumber = 1,
  };
  // bytes serialized_schema = 1;
  void clear_serialized_schema() ;
  [[nodiscard]] const ::std::string& serialized_schema() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_serialized_schema(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_serialized_schema();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_serialized_schema();
  void set_allocated_serialized_schema(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_serialized_schema() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_serialized_schema(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_serialized_schema();

  public:
  // @@protoc_insertion_point(class_scope:google.cloud.bigquery.storage.v1.ArrowSchema)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ArrowSchema& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr serialized_schema_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ArrowRecordBatch final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:google.cloud.bigquery.storage.v1.ArrowRecordBatch) */ {
 public:
  inline ArrowRecordBatch() : ArrowRecordBatch(nullptr) {}
  ~ArrowRecordBatch() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ArrowRecordBatch* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ArrowRecordBatch));
  }
#endif

  template <typename = void>
  explicit constexpr ArrowRecordBatch(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ArrowRecordBatch(const ArrowRecordBatch& from) : ArrowRecordBatch(nullptr, from) {}
  inline ArrowRecordBatch(ArrowRecordBatch&& from) noexcept : ArrowRecordBatch(nullptr, ::std::move(from)) {}
  inline ArrowRecordBatch& operator=(const ArrowRecordBatch& from) {
    CopyFrom(from);
    return *this;
  }
  inline ArrowRecordBatch& operator=(ArrowRecordBatch&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ArrowRecordBatch& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ArrowRecordBatch>(&ArrowRecordBatch_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(ArrowRecordBatch& a, ArrowRecordBatch& b) { a.Swap(&b); }
  inline void Swap(ArrowRecordBatch* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ArrowRecordBatch* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ArrowRecordBatch* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ArrowRecordBatch>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ArrowRecordBatch& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ArrowRecordBatch& from) { ArrowRecordBatch::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ArrowRecordBatch* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "google.cloud.bigquery.storage.v1.ArrowRecordBatch"; }

  explicit ArrowRecordBatch(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ArrowRecordBatch(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ArrowRecordBatch& from);
  ArrowRecordBatch(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ArrowRecordBatch&& from) noexcept
      : ArrowRecordBatch(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kSerializedRecordBatchFieldNumber = 1,
    kRowCountFieldNumber = 2,
  };
  // bytes serialized_record_batch = 1;
  void clear_serialized_record_batch() ;
  [[nodiscard]] const ::std::string& serialized_record_batch() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_serialized_record_batch(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_serialized_record_batch();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_serialized_record_batch();
  void set_allocated_serialized_record_batch(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_serialized_record_batch() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_serialized_record_batch(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_serialized_record_batch();

  public:
  // int64 row_count = 2 [deprecated = true];
  [[deprecated]]  void clear_row_count() ;
  [[nodiscard]] [[deprecated]] ::int64_t row_count() const;
  [[deprecated]] void set_row_count(::int64_t value);

  private:
  ::int64_t _internal_row_count() const;
  void _internal_set_row_count(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:google.cloud.bigquery.storage.v1.ArrowRecordBatch)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ArrowRecordBatch& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr serialized_record_batch_;
    ::int64_t row_count_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// ArrowSchema

// bytes serialized_schema = 1;
inline void ArrowSchema::clear_serialized_schema() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.serialized_schema_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& ArrowSchema::serialized_schema() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:google.cloud.bigquery.storage.v1.ArrowSchema.serialized_schema)
  return _internal_serialized_schema();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ArrowSchema::set_serialized_schema(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.serialized_schema_.SetBytes(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:google.cloud.bigquery.storage.v1.ArrowSchema.serialized_schema)
}
inline ::std::string* PROTOBUF_NONNULL ArrowSchema::mutable_serialized_schema()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_serialized_schema();
  // @@protoc_insertion_point(field_mutable:google.cloud.bigquery.storage.v1.ArrowSchema.serialized_schema)
  return _s;
}
inline const ::std::string& ArrowSchema::_internal_serialized_schema() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.serialized_schema_.Get();
}
inline void ArrowSchema::_internal_set_serialized_schema(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.serialized_schema_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ArrowSchema::_internal_mutable_serialized_schema() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.serialized_schema_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ArrowSchema::release_serialized_schema() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:google.cloud.bigquery.storage.v1.ArrowSchema.serialized_schema)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.serialized_schema_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.serialized_schema_.Set("", GetArena());
  }
  return released;
}
inline void ArrowSchema::set_allocated_serialized_schema(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.serialized_schema_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.serialized_schema_.IsDefault()) {
    _impl_.serialized_schema_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:google.cloud.bigquery.storage.v1.ArrowSchema.serialized_schema)
}

// -------------------------------------------------------------------

// ArrowRecordBatch

// bytes serialized_record_batch = 1;
inline void ArrowRecordBatch::clear_serialized_record_batch() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.serialized_record_batch_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& ArrowRecordBatch::serialized_record_batch() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:google.cloud.bigquery.storage.v1.ArrowRecordBatch.serialized_record_batch)
  return _internal_serialized_record_batch();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ArrowRecordBatch::set_serialized_record_batch(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.serialized_record_batch_.SetBytes(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:google.cloud.bigquery.storage.v1.ArrowRecordBatch.serialized_record_batch)
}
inline ::std::string* PROTOBUF_NONNULL ArrowRecordBatch::mutable_serialized_record_batch()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_serialized_record_batch();
  // @@protoc_insertion_point(field_mutable:google.cloud.bigquery.storage.v1.ArrowRecordBatch.serialized_record_batch)
  return _s;
}
inline const ::std::string& ArrowRecordBatch::_internal_serialized_record_batch() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.serialized_record_batch_.Get();
}
inline void ArrowRecordBatch::_internal_set_serialized_record_batch(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.serialized_record_batch_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ArrowRecordBatch::_internal_mutable_serialized_record_batch() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.serialized_record_batch_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ArrowRecordBatch::release_serialized_record_batch() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:google.cloud.bigquery.storage.v1.ArrowRecordBatch.serialized_record_batch)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.serialized_record_batch_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.serialized_record_batch_.Set("", GetArena());
  }
  return released;
}
inline void ArrowRecordBatch::set_allocated_serialized_record_batch(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.serialized_record_batch_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.serialized_record_batch_.IsDefault()) {
    _impl_.serialized_record_batch_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:google.cloud.bigquery.storage.v1.ArrowRecordBatch.serialized_record_batch)
}

// int64 row_count = 2 [deprecated = true];
inline void ArrowRecordBatch::clear_row_count() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.row_count_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::int64_t ArrowRecordBatch::row_count() const {
  // @@protoc_insertion_point(field_get:google.cloud.bigquery.storage.v1.ArrowRecordBatch.row_count)
  return _internal_row_count();
}
inline void ArrowRecordBatch::set_row_count(::int64_t value) {
  _internal_set_row_count(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:google.cloud.bigquery.storage.v1.ArrowRecordBatch.row_count)
}
inline ::int64_t ArrowRecordBatch::_internal_row_count() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.row_count_;
}
inline void ArrowRecordBatch::_internal_set_row_count(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.row_count_ = value;
}

// -------------------------------------------------------------------

// ArrowSerializationOptions

// .google.cloud.bigquery.storage.v1.ArrowSerializationOptions.CompressionCodec buffer_compression = 2;
inline void ArrowSerializationOptions::clear_buffer_compression() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.buffer_compression_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline ::google::cloud::bigquery::storage::v1::ArrowSerializationOptions_CompressionCodec ArrowSerializationOptions::buffer_compression() const {
  // @@protoc_insertion_point(field_get:google.cloud.bigquery.storage.v1.ArrowSerializationOptions.buffer_compression)
  return _internal_buffer_compression();
}
inline void ArrowSerializationOptions::set_buffer_compression(::google::cloud::bigquery::storage::v1::ArrowSerializationOptions_CompressionCodec value) {
  _internal_set_buffer_compression(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:google.cloud.bigquery.storage.v1.ArrowSerializationOptions.buffer_compression)
}
inline ::google::cloud::bigquery::storage::v1::ArrowSerializationOptions_CompressionCodec ArrowSerializationOptions::_internal_buffer_compression() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::google::cloud::bigquery::storage::v1::ArrowSerializationOptions_CompressionCodec>(_impl_.buffer_compression_);
}
inline void ArrowSerializationOptions::_internal_set_buffer_compression(::google::cloud::bigquery::storage::v1::ArrowSerializationOptions_CompressionCodec value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.buffer_compression_ = value;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace v1
}  // namespace storage
}  // namespace bigquery
}  // namespace cloud
}  // namespace google


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::google::cloud::bigquery::storage::v1::ArrowSerializationOptions_CompressionCodec> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::google::cloud::bigquery::storage::v1::ArrowSerializationOptions_CompressionCodec>() {
  return ::google::cloud::bigquery::storage::v1::ArrowSerializationOptions_CompressionCodec_descriptor();
}

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto_2epb_2eh
