// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: google/cloud/bigquery/storage/v1/table.proto
// Protobuf C++ Version: 7.35.1

#ifndef google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto_2epb_2eh
#define google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7035001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_reflection.h"
#include "google/protobuf/unknown_field_set.h"
#include "google/api/field_behavior.pb.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto;
}  // extern "C"
namespace google {
namespace cloud {
namespace bigquery {
namespace storage {
namespace v1 {
enum TableFieldSchema_Mode : int;
extern const uint32_t TableFieldSchema_Mode_internal_data_[];
enum TableFieldSchema_Type : int;
extern const uint32_t TableFieldSchema_Type_internal_data_[];
class TableFieldSchema;
struct TableFieldSchemaGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern TableFieldSchemaGlobalsTypeInternal TableFieldSchema_globals_;
extern const ::google::protobuf::internal::ClassDataFull TableFieldSchema_class_data_;
#else
extern const TableFieldSchemaGlobalsTypeInternal TableFieldSchema_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class TableSchema;
struct TableSchemaGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern TableSchemaGlobalsTypeInternal TableSchema_globals_;
extern const ::google::protobuf::internal::ClassDataFull TableSchema_class_data_;
#else
extern const TableSchemaGlobalsTypeInternal TableSchema_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace v1
}  // namespace storage
}  // namespace bigquery
}  // namespace cloud
namespace protobuf {
template <>
internal::EnumTraitsT<::google::cloud::bigquery::storage::v1::TableFieldSchema_Mode_internal_data_>
    internal::EnumTraitsImpl::value<::google::cloud::bigquery::storage::v1::TableFieldSchema_Mode>;
template <>
internal::EnumTraitsT<::google::cloud::bigquery::storage::v1::TableFieldSchema_Type_internal_data_>
    internal::EnumTraitsImpl::value<::google::cloud::bigquery::storage::v1::TableFieldSchema_Type>;
}  // namespace protobuf
}  // namespace google

namespace google {
namespace cloud {
namespace bigquery {
namespace storage {
namespace v1 {
enum TableFieldSchema_Type : int {
  TableFieldSchema_Type_TYPE_UNSPECIFIED = 0,
  TableFieldSchema_Type_STRING = 1,
  TableFieldSchema_Type_INT64 = 2,
  TableFieldSchema_Type_DOUBLE = 3,
  TableFieldSchema_Type_STRUCT = 4,
  TableFieldSchema_Type_BYTES = 5,
  TableFieldSchema_Type_BOOL = 6,
  TableFieldSchema_Type_TIMESTAMP = 7,
  TableFieldSchema_Type_DATE = 8,
  TableFieldSchema_Type_TIME = 9,
  TableFieldSchema_Type_DATETIME = 10,
  TableFieldSchema_Type_GEOGRAPHY = 11,
  TableFieldSchema_Type_NUMERIC = 12,
  TableFieldSchema_Type_BIGNUMERIC = 13,
  TableFieldSchema_Type_INTERVAL = 14,
  TableFieldSchema_Type_JSON = 15,
  TableFieldSchema_Type_TableFieldSchema_Type_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  TableFieldSchema_Type_TableFieldSchema_Type_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t TableFieldSchema_Type_internal_data_[];
inline constexpr TableFieldSchema_Type TableFieldSchema_Type_Type_MIN =
    static_cast<TableFieldSchema_Type>(0);
inline constexpr TableFieldSchema_Type TableFieldSchema_Type_Type_MAX =
    static_cast<TableFieldSchema_Type>(15);
[[nodiscard]] inline bool TableFieldSchema_Type_IsValid(int value) {
  return 0 <= value && value <= 15;
}
inline constexpr int TableFieldSchema_Type_Type_ARRAYSIZE = 15 + 1;
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
TableFieldSchema_Type_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(TableFieldSchema_Type) {
  return TableFieldSchema_Type_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& TableFieldSchema_Type_Name(T value) {
  static_assert(::std::is_same<T, TableFieldSchema_Type>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to Type_Name().");
  return TableFieldSchema_Type_Name(static_cast<TableFieldSchema_Type>(value));
}
template <>
[[nodiscard]] inline const ::std::string& TableFieldSchema_Type_Name(TableFieldSchema_Type value) {
  return ::google::protobuf::internal::NameOfDenseEnum<TableFieldSchema_Type_descriptor, 0, 15>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool TableFieldSchema_Type_Parse(
    ::absl::string_view name, TableFieldSchema_Type* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<TableFieldSchema_Type>(TableFieldSchema_Type_descriptor(), name,
                                           value);
}
enum TableFieldSchema_Mode : int {
  TableFieldSchema_Mode_MODE_UNSPECIFIED = 0,
  TableFieldSchema_Mode_NULLABLE = 1,
  TableFieldSchema_Mode_REQUIRED = 2,
  TableFieldSchema_Mode_REPEATED = 3,
  TableFieldSchema_Mode_TableFieldSchema_Mode_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  TableFieldSchema_Mode_TableFieldSchema_Mode_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t TableFieldSchema_Mode_internal_data_[];
inline constexpr TableFieldSchema_Mode TableFieldSchema_Mode_Mode_MIN =
    static_cast<TableFieldSchema_Mode>(0);
inline constexpr TableFieldSchema_Mode TableFieldSchema_Mode_Mode_MAX =
    static_cast<TableFieldSchema_Mode>(3);
[[nodiscard]] inline bool TableFieldSchema_Mode_IsValid(int value) {
  return 0 <= value && value <= 3;
}
inline constexpr int TableFieldSchema_Mode_Mode_ARRAYSIZE = 3 + 1;
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
TableFieldSchema_Mode_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(TableFieldSchema_Mode) {
  return TableFieldSchema_Mode_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& TableFieldSchema_Mode_Name(T value) {
  static_assert(::std::is_same<T, TableFieldSchema_Mode>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to Mode_Name().");
  return TableFieldSchema_Mode_Name(static_cast<TableFieldSchema_Mode>(value));
}
template <>
[[nodiscard]] inline const ::std::string& TableFieldSchema_Mode_Name(TableFieldSchema_Mode value) {
  return ::google::protobuf::internal::NameOfDenseEnum<TableFieldSchema_Mode_descriptor, 0, 3>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool TableFieldSchema_Mode_Parse(
    ::absl::string_view name, TableFieldSchema_Mode* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<TableFieldSchema_Mode>(TableFieldSchema_Mode_descriptor(), name,
                                           value);
}
using ::google::protobuf::internal::generated_enum::AbslParseFlag;
using ::google::protobuf::internal::generated_enum::AbslUnparseFlag;

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED TableFieldSchema final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:google.cloud.bigquery.storage.v1.TableFieldSchema) */ {
 public:
  inline TableFieldSchema() : TableFieldSchema(nullptr) {}
  ~TableFieldSchema() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(TableFieldSchema* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(TableFieldSchema));
  }
#endif

  template <typename = void>
  explicit constexpr TableFieldSchema(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline TableFieldSchema(const TableFieldSchema& from) : TableFieldSchema(nullptr, from) {}
  inline TableFieldSchema(TableFieldSchema&& from) noexcept : TableFieldSchema(nullptr, ::std::move(from)) {}
  inline TableFieldSchema& operator=(const TableFieldSchema& from) {
    CopyFrom(from);
    return *this;
  }
  inline TableFieldSchema& operator=(TableFieldSchema&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const TableFieldSchema& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<TableFieldSchema>(&TableFieldSchema_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(TableFieldSchema& a, TableFieldSchema& b) { a.Swap(&b); }
  inline void Swap(TableFieldSchema* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(TableFieldSchema* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] TableFieldSchema* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<TableFieldSchema>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const TableFieldSchema& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const TableFieldSchema& from) { TableFieldSchema::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(TableFieldSchema* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "google.cloud.bigquery.storage.v1.TableFieldSchema"; }

  explicit TableFieldSchema(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  TableFieldSchema(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const TableFieldSchema& from);
  TableFieldSchema(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, TableFieldSchema&& from) noexcept
      : TableFieldSchema(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using Type = TableFieldSchema_Type;
  static constexpr Type TYPE_UNSPECIFIED = TableFieldSchema_Type_TYPE_UNSPECIFIED;
  static constexpr Type STRING = TableFieldSchema_Type_STRING;
  static constexpr Type INT64 = TableFieldSchema_Type_INT64;
  static constexpr Type DOUBLE = TableFieldSchema_Type_DOUBLE;
  static constexpr Type STRUCT = TableFieldSchema_Type_STRUCT;
  static constexpr Type BYTES = TableFieldSchema_Type_BYTES;
  static constexpr Type BOOL = TableFieldSchema_Type_BOOL;
  static constexpr Type TIMESTAMP = TableFieldSchema_Type_TIMESTAMP;
  static constexpr Type DATE = TableFieldSchema_Type_DATE;
  static constexpr Type TIME = TableFieldSchema_Type_TIME;
  static constexpr Type DATETIME = TableFieldSchema_Type_DATETIME;
  static constexpr Type GEOGRAPHY = TableFieldSchema_Type_GEOGRAPHY;
  static constexpr Type NUMERIC = TableFieldSchema_Type_NUMERIC;
  static constexpr Type BIGNUMERIC = TableFieldSchema_Type_BIGNUMERIC;
  static constexpr Type INTERVAL = TableFieldSchema_Type_INTERVAL;
  static constexpr Type JSON = TableFieldSchema_Type_JSON;
  [[nodiscard]] static inline bool Type_IsValid(int value) {
    return TableFieldSchema_Type_IsValid(value);
  }
  static constexpr Type Type_MIN = TableFieldSchema_Type_Type_MIN;
  static constexpr Type Type_MAX = TableFieldSchema_Type_Type_MAX;
  static constexpr int Type_ARRAYSIZE = TableFieldSchema_Type_Type_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL Type_descriptor() {
    return TableFieldSchema_Type_descriptor();
  }
  template <typename T>
  [[nodiscard]] static inline const ::std::string& Type_Name(T value) {
    return TableFieldSchema_Type_Name(value);
  }
  [[nodiscard]] static inline bool Type_Parse(
      ::absl::string_view name, Type* PROTOBUF_NONNULL value) {
    return TableFieldSchema_Type_Parse(name, value);
  }
  using Mode = TableFieldSchema_Mode;
  static constexpr Mode MODE_UNSPECIFIED = TableFieldSchema_Mode_MODE_UNSPECIFIED;
  static constexpr Mode NULLABLE = TableFieldSchema_Mode_NULLABLE;
  static constexpr Mode REQUIRED = TableFieldSchema_Mode_REQUIRED;
  static constexpr Mode REPEATED = TableFieldSchema_Mode_REPEATED;
  [[nodiscard]] static inline bool Mode_IsValid(int value) {
    return TableFieldSchema_Mode_IsValid(value);
  }
  static constexpr Mode Mode_MIN = TableFieldSchema_Mode_Mode_MIN;
  static constexpr Mode Mode_MAX = TableFieldSchema_Mode_Mode_MAX;
  static constexpr int Mode_ARRAYSIZE = TableFieldSchema_Mode_Mode_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL Mode_descriptor() {
    return TableFieldSchema_Mode_descriptor();
  }
  template <typename T>
  [[nodiscard]] static inline const ::std::string& Mode_Name(T value) {
    return TableFieldSchema_Mode_Name(value);
  }
  [[nodiscard]] static inline bool Mode_Parse(
      ::absl::string_view name, Mode* PROTOBUF_NONNULL value) {
    return TableFieldSchema_Mode_Parse(name, value);
  }

  // accessors -------------------------------------------------------
  enum : int {
    kFieldsFieldNumber = 4,
    kNameFieldNumber = 1,
    kDescriptionFieldNumber = 6,
    kDefaultValueExpressionFieldNumber = 10,
    kTypeFieldNumber = 2,
    kModeFieldNumber = 3,
    kMaxLengthFieldNumber = 7,
    kPrecisionFieldNumber = 8,
    kScaleFieldNumber = 9,
  };
  // repeated .google.cloud.bigquery.storage.v1.TableFieldSchema fields = 4 [(.google.api.field_behavior) = OPTIONAL];
  [[nodiscard]] int fields_size()
      const;
  private:
  int _internal_fields_size() const;

  public:
  void clear_fields() ;
  [[nodiscard]] const ::google::cloud::bigquery::storage::v1::TableFieldSchema& fields(int index) const;
  [[nodiscard]] ::google::cloud::bigquery::storage::v1::TableFieldSchema* PROTOBUF_NONNULL mutable_fields(int index);
  ::google::cloud::bigquery::storage::v1::TableFieldSchema* PROTOBUF_NONNULL add_fields();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::google::cloud::bigquery::storage::v1::TableFieldSchema>&
  fields() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::google::cloud::bigquery::storage::v1::TableFieldSchema>* PROTOBUF_NONNULL
  mutable_fields();

  private:
  const ::google::protobuf::RepeatedPtrField<::google::cloud::bigquery::storage::v1::TableFieldSchema>& _internal_fields() const;
  ::google::protobuf::RepeatedPtrField<::google::cloud::bigquery::storage::v1::TableFieldSchema>* PROTOBUF_NONNULL _internal_mutable_fields();

  public:
  // string name = 1 [(.google.api.field_behavior) = REQUIRED];
  void clear_name() ;
  [[nodiscard]] const ::std::string& name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_name();
  void set_allocated_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_name();

  public:
  // string description = 6 [(.google.api.field_behavior) = OPTIONAL];
  void clear_description() ;
  [[nodiscard]] const ::std::string& description() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_description(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_description();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_description();
  void set_allocated_description(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_description() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_description(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_description();

  public:
  // string default_value_expression = 10 [(.google.api.field_behavior) = OPTIONAL];
  void clear_default_value_expression() ;
  [[nodiscard]] const ::std::string& default_value_expression() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_default_value_expression(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_default_value_expression();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_default_value_expression();
  void set_allocated_default_value_expression(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_default_value_expression() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_default_value_expression(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_default_value_expression();

  public:
  // .google.cloud.bigquery.storage.v1.TableFieldSchema.Type type = 2 [(.google.api.field_behavior) = REQUIRED];
  void clear_type() ;
  [[nodiscard]] ::google::cloud::bigquery::storage::v1::TableFieldSchema_Type type() const;
  void set_type(::google::cloud::bigquery::storage::v1::TableFieldSchema_Type value);

  private:
  ::google::cloud::bigquery::storage::v1::TableFieldSchema_Type _internal_type() const;
  void _internal_set_type(::google::cloud::bigquery::storage::v1::TableFieldSchema_Type value);

  public:
  // .google.cloud.bigquery.storage.v1.TableFieldSchema.Mode mode = 3 [(.google.api.field_behavior) = OPTIONAL];
  void clear_mode() ;
  [[nodiscard]] ::google::cloud::bigquery::storage::v1::TableFieldSchema_Mode mode() const;
  void set_mode(::google::cloud::bigquery::storage::v1::TableFieldSchema_Mode value);

  private:
  ::google::cloud::bigquery::storage::v1::TableFieldSchema_Mode _internal_mode() const;
  void _internal_set_mode(::google::cloud::bigquery::storage::v1::TableFieldSchema_Mode value);

  public:
  // int64 max_length = 7 [(.google.api.field_behavior) = OPTIONAL];
  void clear_max_length() ;
  [[nodiscard]] ::int64_t max_length() const;
  void set_max_length(::int64_t value);

  private:
  ::int64_t _internal_max_length() const;
  void _internal_set_max_length(::int64_t value);

  public:
  // int64 precision = 8 [(.google.api.field_behavior) = OPTIONAL];
  void clear_precision() ;
  [[nodiscard]] ::int64_t precision() const;
  void set_precision(::int64_t value);

  private:
  ::int64_t _internal_precision() const;
  void _internal_set_precision(::int64_t value);

  public:
  // int64 scale = 9 [(.google.api.field_behavior) = OPTIONAL];
  void clear_scale() ;
  [[nodiscard]] ::int64_t scale() const;
  void set_scale(::int64_t value);

  private:
  ::int64_t _internal_scale() const;
  void _internal_set_scale(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:google.cloud.bigquery.storage.v1.TableFieldSchema)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<4, 9,
                          1, 105,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const TableFieldSchema& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::google::cloud::bigquery::storage::v1::TableFieldSchema > fields_;
    ::google::protobuf::internal::ArenaStringPtr name_;
    ::google::protobuf::internal::ArenaStringPtr description_;
    ::google::protobuf::internal::ArenaStringPtr default_value_expression_;
    int type_;
    int mode_;
    ::int64_t max_length_;
    ::int64_t precision_;
    ::int64_t scale_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED TableSchema final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:google.cloud.bigquery.storage.v1.TableSchema) */ {
 public:
  inline TableSchema() : TableSchema(nullptr) {}
  ~TableSchema() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(TableSchema* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(TableSchema));
  }
#endif

  template <typename = void>
  explicit constexpr TableSchema(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline TableSchema(const TableSchema& from) : TableSchema(nullptr, from) {}
  inline TableSchema(TableSchema&& from) noexcept : TableSchema(nullptr, ::std::move(from)) {}
  inline TableSchema& operator=(const TableSchema& from) {
    CopyFrom(from);
    return *this;
  }
  inline TableSchema& operator=(TableSchema&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const TableSchema& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<TableSchema>(&TableSchema_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(TableSchema& a, TableSchema& b) { a.Swap(&b); }
  inline void Swap(TableSchema* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(TableSchema* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] TableSchema* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<TableSchema>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const TableSchema& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const TableSchema& from) { TableSchema::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(TableSchema* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "google.cloud.bigquery.storage.v1.TableSchema"; }

  explicit TableSchema(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  TableSchema(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const TableSchema& from);
  TableSchema(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, TableSchema&& from) noexcept
      : TableSchema(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFieldsFieldNumber = 1,
  };
  // repeated .google.cloud.bigquery.storage.v1.TableFieldSchema fields = 1;
  [[nodiscard]] int fields_size()
      const;
  private:
  int _internal_fields_size() const;

  public:
  void clear_fields() ;
  [[nodiscard]] const ::google::cloud::bigquery::storage::v1::TableFieldSchema& fields(int index) const;
  [[nodiscard]] ::google::cloud::bigquery::storage::v1::TableFieldSchema* PROTOBUF_NONNULL mutable_fields(int index);
  ::google::cloud::bigquery::storage::v1::TableFieldSchema* PROTOBUF_NONNULL add_fields();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::google::cloud::bigquery::storage::v1::TableFieldSchema>&
  fields() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::google::cloud::bigquery::storage::v1::TableFieldSchema>* PROTOBUF_NONNULL
  mutable_fields();

  private:
  const ::google::protobuf::RepeatedPtrField<::google::cloud::bigquery::storage::v1::TableFieldSchema>& _internal_fields() const;
  ::google::protobuf::RepeatedPtrField<::google::cloud::bigquery::storage::v1::TableFieldSchema>* PROTOBUF_NONNULL _internal_mutable_fields();

  public:
  // @@protoc_insertion_point(class_scope:google.cloud.bigquery.storage.v1.TableSchema)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const TableSchema& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::google::cloud::bigquery::storage::v1::TableFieldSchema > fields_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// TableSchema

// repeated .google.cloud.bigquery.storage.v1.TableFieldSchema fields = 1;
inline int TableSchema::_internal_fields_size() const {
  return _internal_fields().size();
}
inline int TableSchema::fields_size() const {
  return _internal_fields_size();
}
inline void TableSchema::clear_fields() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fields_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::google::cloud::bigquery::storage::v1::TableFieldSchema& TableSchema::fields(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:google.cloud.bigquery.storage.v1.TableSchema.fields)
  return _internal_fields().Get(index);
}
inline ::google::cloud::bigquery::storage::v1::TableFieldSchema* PROTOBUF_NONNULL TableSchema::mutable_fields(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:google.cloud.bigquery.storage.v1.TableSchema.fields)
  return _internal_mutable_fields()->Mutable(index);
}
inline ::google::cloud::bigquery::storage::v1::TableFieldSchema* PROTOBUF_NONNULL TableSchema::add_fields()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::cloud::bigquery::storage::v1::TableFieldSchema* _add =
      _internal_mutable_fields()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:google.cloud.bigquery.storage.v1.TableSchema.fields)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::google::cloud::bigquery::storage::v1::TableFieldSchema>& TableSchema::fields() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:google.cloud.bigquery.storage.v1.TableSchema.fields)
  return _internal_fields();
}
inline ::google::protobuf::RepeatedPtrField<::google::cloud::bigquery::storage::v1::TableFieldSchema>* PROTOBUF_NONNULL
TableSchema::mutable_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:google.cloud.bigquery.storage.v1.TableSchema.fields)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_fields();
}
inline const ::google::protobuf::RepeatedPtrField<::google::cloud::bigquery::storage::v1::TableFieldSchema>&
TableSchema::_internal_fields() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fields_;
}
inline ::google::protobuf::RepeatedPtrField<::google::cloud::bigquery::storage::v1::TableFieldSchema>* PROTOBUF_NONNULL
TableSchema::_internal_mutable_fields() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.fields_;
}

// -------------------------------------------------------------------

// TableFieldSchema

// string name = 1 [(.google.api.field_behavior) = REQUIRED];
inline void TableFieldSchema::clear_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& TableFieldSchema::name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:google.cloud.bigquery.storage.v1.TableFieldSchema.name)
  return _internal_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void TableFieldSchema::set_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:google.cloud.bigquery.storage.v1.TableFieldSchema.name)
}
inline ::std::string* PROTOBUF_NONNULL TableFieldSchema::mutable_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:google.cloud.bigquery.storage.v1.TableFieldSchema.name)
  return _s;
}
inline const ::std::string& TableFieldSchema::_internal_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.name_.Get();
}
inline void TableFieldSchema::_internal_set_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL TableFieldSchema::_internal_mutable_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE TableFieldSchema::release_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:google.cloud.bigquery.storage.v1.TableFieldSchema.name)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.name_.Set("", GetArena());
  }
  return released;
}
inline void TableFieldSchema::set_allocated_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.name_.IsDefault()) {
    _impl_.name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:google.cloud.bigquery.storage.v1.TableFieldSchema.name)
}

// .google.cloud.bigquery.storage.v1.TableFieldSchema.Type type = 2 [(.google.api.field_behavior) = REQUIRED];
inline void TableFieldSchema::clear_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::google::cloud::bigquery::storage::v1::TableFieldSchema_Type TableFieldSchema::type() const {
  // @@protoc_insertion_point(field_get:google.cloud.bigquery.storage.v1.TableFieldSchema.type)
  return _internal_type();
}
inline void TableFieldSchema::set_type(::google::cloud::bigquery::storage::v1::TableFieldSchema_Type value) {
  _internal_set_type(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:google.cloud.bigquery.storage.v1.TableFieldSchema.type)
}
inline ::google::cloud::bigquery::storage::v1::TableFieldSchema_Type TableFieldSchema::_internal_type() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::google::cloud::bigquery::storage::v1::TableFieldSchema_Type>(_impl_.type_);
}
inline void TableFieldSchema::_internal_set_type(::google::cloud::bigquery::storage::v1::TableFieldSchema_Type value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_ = value;
}

// .google.cloud.bigquery.storage.v1.TableFieldSchema.Mode mode = 3 [(.google.api.field_behavior) = OPTIONAL];
inline void TableFieldSchema::clear_mode() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mode_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::google::cloud::bigquery::storage::v1::TableFieldSchema_Mode TableFieldSchema::mode() const {
  // @@protoc_insertion_point(field_get:google.cloud.bigquery.storage.v1.TableFieldSchema.mode)
  return _internal_mode();
}
inline void TableFieldSchema::set_mode(::google::cloud::bigquery::storage::v1::TableFieldSchema_Mode value) {
  _internal_set_mode(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:google.cloud.bigquery.storage.v1.TableFieldSchema.mode)
}
inline ::google::cloud::bigquery::storage::v1::TableFieldSchema_Mode TableFieldSchema::_internal_mode() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::google::cloud::bigquery::storage::v1::TableFieldSchema_Mode>(_impl_.mode_);
}
inline void TableFieldSchema::_internal_set_mode(::google::cloud::bigquery::storage::v1::TableFieldSchema_Mode value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mode_ = value;
}

// repeated .google.cloud.bigquery.storage.v1.TableFieldSchema fields = 4 [(.google.api.field_behavior) = OPTIONAL];
inline int TableFieldSchema::_internal_fields_size() const {
  return _internal_fields().size();
}
inline int TableFieldSchema::fields_size() const {
  return _internal_fields_size();
}
inline void TableFieldSchema::clear_fields() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fields_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::google::cloud::bigquery::storage::v1::TableFieldSchema& TableFieldSchema::fields(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:google.cloud.bigquery.storage.v1.TableFieldSchema.fields)
  return _internal_fields().Get(index);
}
inline ::google::cloud::bigquery::storage::v1::TableFieldSchema* PROTOBUF_NONNULL TableFieldSchema::mutable_fields(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:google.cloud.bigquery.storage.v1.TableFieldSchema.fields)
  return _internal_mutable_fields()->Mutable(index);
}
inline ::google::cloud::bigquery::storage::v1::TableFieldSchema* PROTOBUF_NONNULL TableFieldSchema::add_fields()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::cloud::bigquery::storage::v1::TableFieldSchema* _add =
      _internal_mutable_fields()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:google.cloud.bigquery.storage.v1.TableFieldSchema.fields)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::google::cloud::bigquery::storage::v1::TableFieldSchema>& TableFieldSchema::fields() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:google.cloud.bigquery.storage.v1.TableFieldSchema.fields)
  return _internal_fields();
}
inline ::google::protobuf::RepeatedPtrField<::google::cloud::bigquery::storage::v1::TableFieldSchema>* PROTOBUF_NONNULL
TableFieldSchema::mutable_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:google.cloud.bigquery.storage.v1.TableFieldSchema.fields)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_fields();
}
inline const ::google::protobuf::RepeatedPtrField<::google::cloud::bigquery::storage::v1::TableFieldSchema>&
TableFieldSchema::_internal_fields() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fields_;
}
inline ::google::protobuf::RepeatedPtrField<::google::cloud::bigquery::storage::v1::TableFieldSchema>* PROTOBUF_NONNULL
TableFieldSchema::_internal_mutable_fields() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.fields_;
}

// string description = 6 [(.google.api.field_behavior) = OPTIONAL];
inline void TableFieldSchema::clear_description() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.description_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::std::string& TableFieldSchema::description() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:google.cloud.bigquery.storage.v1.TableFieldSchema.description)
  return _internal_description();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void TableFieldSchema::set_description(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.description_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:google.cloud.bigquery.storage.v1.TableFieldSchema.description)
}
inline ::std::string* PROTOBUF_NONNULL TableFieldSchema::mutable_description()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_description();
  // @@protoc_insertion_point(field_mutable:google.cloud.bigquery.storage.v1.TableFieldSchema.description)
  return _s;
}
inline const ::std::string& TableFieldSchema::_internal_description() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.description_.Get();
}
inline void TableFieldSchema::_internal_set_description(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.description_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL TableFieldSchema::_internal_mutable_description() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.description_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE TableFieldSchema::release_description() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:google.cloud.bigquery.storage.v1.TableFieldSchema.description)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.description_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.description_.Set("", GetArena());
  }
  return released;
}
inline void TableFieldSchema::set_allocated_description(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.description_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.description_.IsDefault()) {
    _impl_.description_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:google.cloud.bigquery.storage.v1.TableFieldSchema.description)
}

// int64 max_length = 7 [(.google.api.field_behavior) = OPTIONAL];
inline void TableFieldSchema::clear_max_length() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.max_length_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::int64_t TableFieldSchema::max_length() const {
  // @@protoc_insertion_point(field_get:google.cloud.bigquery.storage.v1.TableFieldSchema.max_length)
  return _internal_max_length();
}
inline void TableFieldSchema::set_max_length(::int64_t value) {
  _internal_set_max_length(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:google.cloud.bigquery.storage.v1.TableFieldSchema.max_length)
}
inline ::int64_t TableFieldSchema::_internal_max_length() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.max_length_;
}
inline void TableFieldSchema::_internal_set_max_length(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.max_length_ = value;
}

// int64 precision = 8 [(.google.api.field_behavior) = OPTIONAL];
inline void TableFieldSchema::clear_precision() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.precision_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
}
inline ::int64_t TableFieldSchema::precision() const {
  // @@protoc_insertion_point(field_get:google.cloud.bigquery.storage.v1.TableFieldSchema.precision)
  return _internal_precision();
}
inline void TableFieldSchema::set_precision(::int64_t value) {
  _internal_set_precision(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:google.cloud.bigquery.storage.v1.TableFieldSchema.precision)
}
inline ::int64_t TableFieldSchema::_internal_precision() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.precision_;
}
inline void TableFieldSchema::_internal_set_precision(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.precision_ = value;
}

// int64 scale = 9 [(.google.api.field_behavior) = OPTIONAL];
inline void TableFieldSchema::clear_scale() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.scale_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000100U);
}
inline ::int64_t TableFieldSchema::scale() const {
  // @@protoc_insertion_point(field_get:google.cloud.bigquery.storage.v1.TableFieldSchema.scale)
  return _internal_scale();
}
inline void TableFieldSchema::set_scale(::int64_t value) {
  _internal_set_scale(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000100U);
  // @@protoc_insertion_point(field_set:google.cloud.bigquery.storage.v1.TableFieldSchema.scale)
}
inline ::int64_t TableFieldSchema::_internal_scale() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.scale_;
}
inline void TableFieldSchema::_internal_set_scale(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.scale_ = value;
}

// string default_value_expression = 10 [(.google.api.field_behavior) = OPTIONAL];
inline void TableFieldSchema::clear_default_value_expression() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.default_value_expression_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::std::string& TableFieldSchema::default_value_expression() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:google.cloud.bigquery.storage.v1.TableFieldSchema.default_value_expression)
  return _internal_default_value_expression();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void TableFieldSchema::set_default_value_expression(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  _impl_.default_value_expression_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:google.cloud.bigquery.storage.v1.TableFieldSchema.default_value_expression)
}
inline ::std::string* PROTOBUF_NONNULL TableFieldSchema::mutable_default_value_expression()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::std::string* _s = _internal_mutable_default_value_expression();
  // @@protoc_insertion_point(field_mutable:google.cloud.bigquery.storage.v1.TableFieldSchema.default_value_expression)
  return _s;
}
inline const ::std::string& TableFieldSchema::_internal_default_value_expression() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.default_value_expression_.Get();
}
inline void TableFieldSchema::_internal_set_default_value_expression(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.default_value_expression_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL TableFieldSchema::_internal_mutable_default_value_expression() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.default_value_expression_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE TableFieldSchema::release_default_value_expression() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:google.cloud.bigquery.storage.v1.TableFieldSchema.default_value_expression)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000008U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  auto* released = _impl_.default_value_expression_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.default_value_expression_.Set("", GetArena());
  }
  return released;
}
inline void TableFieldSchema::set_allocated_default_value_expression(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  _impl_.default_value_expression_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.default_value_expression_.IsDefault()) {
    _impl_.default_value_expression_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:google.cloud.bigquery.storage.v1.TableFieldSchema.default_value_expression)
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace v1
}  // namespace storage
}  // namespace bigquery
}  // namespace cloud
}  // namespace google


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::google::cloud::bigquery::storage::v1::TableFieldSchema_Type> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::google::cloud::bigquery::storage::v1::TableFieldSchema_Type>() {
  return ::google::cloud::bigquery::storage::v1::TableFieldSchema_Type_descriptor();
}
template <>
struct is_proto_enum<::google::cloud::bigquery::storage::v1::TableFieldSchema_Mode> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::google::cloud::bigquery::storage::v1::TableFieldSchema_Mode>() {
  return ::google::cloud::bigquery::storage::v1::TableFieldSchema_Mode_descriptor();
}

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto_2epb_2eh
