// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: google/cloud/bigquery/storage/v1/arrow.proto
// Protobuf C++ Version: 7.35.1

#include "google/cloud/bigquery/storage/v1/arrow.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
#ifdef PROTOBUF_MESSAGE_GLOBALS
namespace {
PROTOBUF_CONSTINIT ::google::protobuf::internal::ReflectionData
    file_reflection_data[] = {
        // ::google::cloud::bigquery::storage::v1::ArrowSchema
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::ArrowRecordBatch
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::ArrowSerializationOptions
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto, /* tracker*/ nullptr,},
};
}  // namespace
#endif
namespace google {
namespace cloud {
namespace bigquery {
namespace storage {
namespace v1 {
class ArrowSerializationOptions::_Internal {
 public:
  using HasBits = decltype(::std::declval<ArrowSerializationOptions>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ArrowSerializationOptions, _impl_._has_bits_);
};

constexpr ArrowSerializationOptions::ParseTableT_ ArrowSerializationOptions::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ArrowSerializationOptions, _impl_._has_bits_),
      0, // no _extensions_
      2, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967293,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::ArrowSerializationOptions>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.cloud.bigquery.storage.v1.ArrowSerializationOptions.CompressionCodec buffer_compression = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ArrowSerializationOptions, _impl_.buffer_compression_), 0>(),
       {16, 0, 0,
        PROTOBUF_FIELD_OFFSET(ArrowSerializationOptions, _impl_.buffer_compression_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.cloud.bigquery.storage.v1.ArrowSerializationOptions.CompressionCodec buffer_compression = 2;
      {PROTOBUF_FIELD_OFFSET(ArrowSerializationOptions, _impl_.buffer_compression_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr ArrowSerializationOptions::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        buffer_compression_{static_cast< ::google::cloud::bigquery::storage::v1::ArrowSerializationOptions_CompressionCodec >(0)} {}

template <typename>
constexpr ArrowSerializationOptions::ArrowSerializationOptions(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ArrowSerializationOptions::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ArrowSerializationOptions(arena);
}
constexpr auto ArrowSerializationOptions::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ArrowSerializationOptions), alignof(ArrowSerializationOptions));
}
constexpr auto ArrowSerializationOptions::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ArrowSerializationOptions::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ArrowSerializationOptions>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ArrowSerializationOptions::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ArrowSerializationOptions>(), &ArrowSerializationOptions::ByteSizeLong,
              &ArrowSerializationOptions::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ArrowSerializationOptions, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[2],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ArrowSerializationOptionsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ArrowSerializationOptionsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ArrowSerializationOptions_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ArrowSerializationOptions::InternalGenerateClassData_(
            _default, &ArrowSerializationOptions_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ArrowSerializationOptions>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ArrowSerializationOptionsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ArrowSerializationOptions _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ArrowSerializationOptions>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ArrowSerializationOptionsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ArrowSerializationOptionsGlobalsTypeInternal ArrowSerializationOptions_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ArrowSerializationOptions_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ArrowSerializationOptions_globals_.GetClassData();
#else
  return ArrowSerializationOptions_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ArrowSchema::_Internal {
 public:
  using HasBits = decltype(::std::declval<ArrowSchema>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ArrowSchema, _impl_._has_bits_);
};

constexpr ArrowSchema::ParseTableT_ ArrowSchema::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ArrowSchema, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::ArrowSchema>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // bytes serialized_schema = 1;
      {::_pbi::TcParser::FastBS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ArrowSchema, _impl_.serialized_schema_)}},
    }}, {{
      65535, 65535
    }}, {{
      // bytes serialized_schema = 1;
      {PROTOBUF_FIELD_OFFSET(ArrowSchema, _impl_.serialized_schema_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr ArrowSchema::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        serialized_schema_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr ArrowSchema::ArrowSchema(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ArrowSchema::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ArrowSchema(arena);
}
constexpr auto ArrowSchema::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ArrowSchema), alignof(ArrowSchema));
}
constexpr auto ArrowSchema::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ArrowSchema::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ArrowSchema>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ArrowSchema::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ArrowSchema>(), &ArrowSchema::ByteSizeLong,
              &ArrowSchema::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ArrowSchema, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[0],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ArrowSchemaGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ArrowSchemaGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ArrowSchema_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ArrowSchema::InternalGenerateClassData_(
            _default, &ArrowSchema_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ArrowSchema>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ArrowSchemaGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ArrowSchema _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ArrowSchema>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ArrowSchemaGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ArrowSchemaGlobalsTypeInternal ArrowSchema_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ArrowSchema_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ArrowSchema_globals_.GetClassData();
#else
  return ArrowSchema_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ArrowRecordBatch::_Internal {
 public:
  using HasBits = decltype(::std::declval<ArrowRecordBatch>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ArrowRecordBatch, _impl_._has_bits_);
};

constexpr ArrowRecordBatch::ParseTableT_ ArrowRecordBatch::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ArrowRecordBatch, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::ArrowRecordBatch>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // int64 row_count = 2 [deprecated = true];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(ArrowRecordBatch, _impl_.row_count_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(ArrowRecordBatch, _impl_.row_count_)}},
      // bytes serialized_record_batch = 1;
      {::_pbi::TcParser::FastBS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ArrowRecordBatch, _impl_.serialized_record_batch_)}},
    }}, {{
      65535, 65535
    }}, {{
      // bytes serialized_record_batch = 1;
      {PROTOBUF_FIELD_OFFSET(ArrowRecordBatch, _impl_.serialized_record_batch_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
      // int64 row_count = 2 [deprecated = true];
      {PROTOBUF_FIELD_OFFSET(ArrowRecordBatch, _impl_.row_count_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr ArrowRecordBatch::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        serialized_record_batch_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        row_count_{::int64_t{0}} {}

template <typename>
constexpr ArrowRecordBatch::ArrowRecordBatch(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ArrowRecordBatch::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ArrowRecordBatch(arena);
}
constexpr auto ArrowRecordBatch::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ArrowRecordBatch), alignof(ArrowRecordBatch));
}
constexpr auto ArrowRecordBatch::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ArrowRecordBatch::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ArrowRecordBatch>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ArrowRecordBatch::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ArrowRecordBatch>(), &ArrowRecordBatch::ByteSizeLong,
              &ArrowRecordBatch::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ArrowRecordBatch, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[1],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ArrowRecordBatchGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ArrowRecordBatchGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ArrowRecordBatch_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ArrowRecordBatch::InternalGenerateClassData_(
            _default, &ArrowRecordBatch_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ArrowRecordBatch>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ArrowRecordBatchGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ArrowRecordBatch _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ArrowRecordBatch>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ArrowRecordBatchGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ArrowRecordBatchGlobalsTypeInternal ArrowRecordBatch_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ArrowRecordBatch_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ArrowRecordBatch_globals_.GetClassData();
#else
  return ArrowRecordBatch_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
}  // namespace v1
}  // namespace storage
}  // namespace bigquery
}  // namespace cloud
}  // namespace google
static const ::_pb::EnumDescriptor* PROTOBUF_NONNULL
    file_level_enum_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto[1];
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto = nullptr;
const ::uint32_t
    TableStruct_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ArrowSchema, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ArrowSchema, _impl_.serialized_schema_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ArrowRecordBatch, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ArrowRecordBatch, _impl_.serialized_record_batch_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ArrowRecordBatch, _impl_.row_count_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ArrowSerializationOptions, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ArrowSerializationOptions, _impl_.buffer_compression_),
        0,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::google::cloud::bigquery::storage::v1::ArrowSchema)},
        {5, sizeof(::google::cloud::bigquery::storage::v1::ArrowRecordBatch)},
        {12, sizeof(::google::cloud::bigquery::storage::v1::ArrowSerializationOptions)},
};
static const ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const
    file_message_globals[] = {
        &::google::cloud::bigquery::storage::v1::ArrowSchema_globals_,
        &::google::cloud::bigquery::storage::v1::ArrowRecordBatch_globals_,
        &::google::cloud::bigquery::storage::v1::ArrowSerializationOptions_globals_,
};
const char descriptor_table_protodef_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n,google/cloud/bigquery/storage/v1/arrow"
    ".proto\022 google.cloud.bigquery.storage.v1"
    "\"(\n\013ArrowSchema\022\031\n\021serialized_schema\030\001 \001"
    "(\014\"J\n\020ArrowRecordBatch\022\037\n\027serialized_rec"
    "ord_batch\030\001 \001(\014\022\025\n\trow_count\030\002 \001(\003B\002\030\001\"\317"
    "\001\n\031ArrowSerializationOptions\022h\n\022buffer_c"
    "ompression\030\002 \001(\0162L.google.cloud.bigquery"
    ".storage.v1.ArrowSerializationOptions.Co"
    "mpressionCodec\"H\n\020CompressionCodec\022\033\n\027CO"
    "MPRESSION_UNSPECIFIED\020\000\022\r\n\tLZ4_FRAME\020\001\022\010"
    "\n\004ZSTD\020\002B\272\001\n$com.google.cloud.bigquery.s"
    "torage.v1B\nArrowProtoP\001Z>cloud.google.co"
    "m/go/bigquery/storage/apiv1/storagepb;st"
    "oragepb\252\002 Google.Cloud.BigQuery.Storage."
    "V1\312\002 Google\\Cloud\\BigQuery\\Storage\\V1b\006p"
    "roto3"
};
static ::absl::once_flag descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto = {
    false,
    false,
    605,
    descriptor_table_protodef_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto,
    "google/cloud/bigquery/storage/v1/arrow.proto",
    &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto_once,
    nullptr,
    0,
    3,
    schemas,
    file_message_globals,
    TableStruct_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto::offsets,
    file_level_enum_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto,
    file_level_service_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto,
};
namespace google {
namespace cloud {
namespace bigquery {
namespace storage {
namespace v1 {
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
ArrowSerializationOptions_CompressionCodec_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto);
  return file_level_enum_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto[0];
}
PROTOBUF_CONSTINIT const uint32_t ArrowSerializationOptions_CompressionCodec_internal_data_[] = {
    196608u, 0u, };
// ===================================================================

ArrowSchema::ArrowSchema(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ArrowSchema_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.ArrowSchema)
}
PROTOBUF_NDEBUG_INLINE ArrowSchema::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::ArrowSchema& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        serialized_schema_(arena, from.serialized_schema_) {}

ArrowSchema::ArrowSchema(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ArrowSchema& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ArrowSchema_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ArrowSchema* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.ArrowSchema)
}
PROTOBUF_NDEBUG_INLINE ArrowSchema::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        serialized_schema_(arena) {}

inline void ArrowSchema::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
ArrowSchema::~ArrowSchema() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.ArrowSchema)
  SharedDtor(*this);
}
inline void ArrowSchema::SharedDtor(MessageLite& self) {
  ArrowSchema& this_ = static_cast<ArrowSchema&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.serialized_schema_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ArrowSchema_class_data_ =
        ArrowSchema::InternalGenerateClassData_(ArrowSchema_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ArrowSchema::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ArrowSchema_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ArrowSchema_class_data_.tc_table);
  return ArrowSchema_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ArrowSchema::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ArrowSchema_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ArrowSchema_globals_));
  return ArrowSchema_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ArrowSchema::ParseTableT_
    ArrowSchema::_table_ =
        ArrowSchema::InternalGenerateParseTable_(ArrowSchema_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ArrowSchema::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.ArrowSchema)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.serialized_schema_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ArrowSchema::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ArrowSchema& this_ = static_cast<const ArrowSchema&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ArrowSchema::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ArrowSchema& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.ArrowSchema)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // bytes serialized_schema = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_serialized_schema().empty()) {
      const ::std::string& _s = this_._internal_serialized_schema();
      target = stream->WriteBytesMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.ArrowSchema)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ArrowSchema::ByteSizeLong(const MessageLite& base) {
  const ArrowSchema& this_ = static_cast<const ArrowSchema&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ArrowSchema::ByteSizeLong() const {
  const ArrowSchema& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.ArrowSchema)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // bytes serialized_schema = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_serialized_schema().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::BytesSize(
                                        this_._internal_serialized_schema());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ArrowSchema::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ArrowSchema*>(&to_msg);
  auto& from = static_cast<const ArrowSchema&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.ArrowSchema)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_serialized_schema().empty()) {
      _this->_internal_set_serialized_schema(from._internal_serialized_schema());
    } else {
      if (_this->_impl_.serialized_schema_.IsDefault()) {
        _this->_internal_set_serialized_schema("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ArrowSchema::CopyFrom(const ArrowSchema& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.ArrowSchema)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ArrowSchema::InternalSwap(ArrowSchema* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.serialized_schema_, &other->_impl_.serialized_schema_, arena);
}

::google::protobuf::Metadata ArrowSchema::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ArrowRecordBatch::ArrowRecordBatch(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ArrowRecordBatch_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.ArrowRecordBatch)
}
PROTOBUF_NDEBUG_INLINE ArrowRecordBatch::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::ArrowRecordBatch& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        serialized_record_batch_(arena, from.serialized_record_batch_) {}

ArrowRecordBatch::ArrowRecordBatch(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ArrowRecordBatch& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ArrowRecordBatch_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ArrowRecordBatch* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.row_count_ = from._impl_.row_count_;

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.ArrowRecordBatch)
}
PROTOBUF_NDEBUG_INLINE ArrowRecordBatch::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        serialized_record_batch_(arena) {}

inline void ArrowRecordBatch::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.row_count_ = {};
}
ArrowRecordBatch::~ArrowRecordBatch() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.ArrowRecordBatch)
  SharedDtor(*this);
}
inline void ArrowRecordBatch::SharedDtor(MessageLite& self) {
  ArrowRecordBatch& this_ = static_cast<ArrowRecordBatch&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.serialized_record_batch_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ArrowRecordBatch_class_data_ =
        ArrowRecordBatch::InternalGenerateClassData_(ArrowRecordBatch_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ArrowRecordBatch::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ArrowRecordBatch_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ArrowRecordBatch_class_data_.tc_table);
  return ArrowRecordBatch_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ArrowRecordBatch::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ArrowRecordBatch_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ArrowRecordBatch_globals_));
  return ArrowRecordBatch_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ArrowRecordBatch::ParseTableT_
    ArrowRecordBatch::_table_ =
        ArrowRecordBatch::InternalGenerateParseTable_(ArrowRecordBatch_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ArrowRecordBatch::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.ArrowRecordBatch)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.serialized_record_batch_.ClearNonDefaultToEmpty();
  }
  _impl_.row_count_ = ::int64_t{0};
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ArrowRecordBatch::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ArrowRecordBatch& this_ = static_cast<const ArrowRecordBatch&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ArrowRecordBatch::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ArrowRecordBatch& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.ArrowRecordBatch)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // bytes serialized_record_batch = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_serialized_record_batch().empty()) {
      const ::std::string& _s = this_._internal_serialized_record_batch();
      target = stream->WriteBytesMaybeAliased(1, _s, target);
    }
  }

  // int64 row_count = 2 [deprecated = true];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_row_count() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<2>(
              stream, this_._internal_row_count(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.ArrowRecordBatch)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ArrowRecordBatch::ByteSizeLong(const MessageLite& base) {
  const ArrowRecordBatch& this_ = static_cast<const ArrowRecordBatch&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ArrowRecordBatch::ByteSizeLong() const {
  const ArrowRecordBatch& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.ArrowRecordBatch)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // bytes serialized_record_batch = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_serialized_record_batch().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::BytesSize(
                                        this_._internal_serialized_record_batch());
      }
    }
    // int64 row_count = 2 [deprecated = true];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_row_count() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_row_count());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ArrowRecordBatch::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ArrowRecordBatch*>(&to_msg);
  auto& from = static_cast<const ArrowRecordBatch&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.ArrowRecordBatch)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_serialized_record_batch().empty()) {
        _this->_internal_set_serialized_record_batch(from._internal_serialized_record_batch());
      } else {
        if (_this->_impl_.serialized_record_batch_.IsDefault()) {
          _this->_internal_set_serialized_record_batch("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_row_count() != 0) {
        _this->_impl_.row_count_ = from._impl_.row_count_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ArrowRecordBatch::CopyFrom(const ArrowRecordBatch& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.ArrowRecordBatch)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ArrowRecordBatch::InternalSwap(ArrowRecordBatch* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.serialized_record_batch_, &other->_impl_.serialized_record_batch_, arena);
  swap(_impl_.row_count_, other->_impl_.row_count_);
}

::google::protobuf::Metadata ArrowRecordBatch::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ArrowSerializationOptions::ArrowSerializationOptions(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ArrowSerializationOptions_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.ArrowSerializationOptions)
}
ArrowSerializationOptions::ArrowSerializationOptions(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ArrowSerializationOptions& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ArrowSerializationOptions_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE ArrowSerializationOptions::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void ArrowSerializationOptions::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.buffer_compression_ = {};
}
ArrowSerializationOptions::~ArrowSerializationOptions() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.ArrowSerializationOptions)
  SharedDtor(*this);
}
inline void ArrowSerializationOptions::SharedDtor(MessageLite& self) {
  ArrowSerializationOptions& this_ = static_cast<ArrowSerializationOptions&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ArrowSerializationOptions_class_data_ =
        ArrowSerializationOptions::InternalGenerateClassData_(ArrowSerializationOptions_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ArrowSerializationOptions::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ArrowSerializationOptions_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ArrowSerializationOptions_class_data_.tc_table);
  return ArrowSerializationOptions_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ArrowSerializationOptions::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ArrowSerializationOptions_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ArrowSerializationOptions_globals_));
  return ArrowSerializationOptions_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ArrowSerializationOptions::ParseTableT_
    ArrowSerializationOptions::_table_ =
        ArrowSerializationOptions::InternalGenerateParseTable_(ArrowSerializationOptions_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ArrowSerializationOptions::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.ArrowSerializationOptions)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.buffer_compression_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ArrowSerializationOptions::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ArrowSerializationOptions& this_ = static_cast<const ArrowSerializationOptions&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ArrowSerializationOptions::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ArrowSerializationOptions& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.ArrowSerializationOptions)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.cloud.bigquery.storage.v1.ArrowSerializationOptions.CompressionCodec buffer_compression = 2;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_buffer_compression() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          2, this_._internal_buffer_compression(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.ArrowSerializationOptions)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ArrowSerializationOptions::ByteSizeLong(const MessageLite& base) {
  const ArrowSerializationOptions& this_ = static_cast<const ArrowSerializationOptions&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ArrowSerializationOptions::ByteSizeLong() const {
  const ArrowSerializationOptions& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.ArrowSerializationOptions)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // .google.cloud.bigquery.storage.v1.ArrowSerializationOptions.CompressionCodec buffer_compression = 2;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_buffer_compression() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_buffer_compression());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ArrowSerializationOptions::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ArrowSerializationOptions*>(&to_msg);
  auto& from = static_cast<const ArrowSerializationOptions&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.ArrowSerializationOptions)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (from._internal_buffer_compression() != 0) {
      _this->_impl_.buffer_compression_ = from._impl_.buffer_compression_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ArrowSerializationOptions::CopyFrom(const ArrowSerializationOptions& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.ArrowSerializationOptions)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ArrowSerializationOptions::InternalSwap(ArrowSerializationOptions* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.buffer_compression_, other->_impl_.buffer_compression_);
}

::google::protobuf::Metadata ArrowSerializationOptions::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
}  // namespace v1
}  // namespace storage
}  // namespace bigquery
}  // namespace cloud
}  // namespace google
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
