// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: google/api/resource.proto
// Protobuf C++ Version: 7.35.1

#ifndef google_2fapi_2fresource_2eproto_2epb_2eh
#define google_2fapi_2fresource_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7035001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_reflection.h"
#include "google/protobuf/unknown_field_set.h"
#include "google/protobuf/descriptor.pb.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_google_2fapi_2fresource_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_google_2fapi_2fresource_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_google_2fapi_2fresource_2eproto;
}  // extern "C"
namespace google {
namespace api {
enum ResourceDescriptor_History : int;
extern const uint32_t ResourceDescriptor_History_internal_data_[];
enum ResourceDescriptor_Style : int;
extern const uint32_t ResourceDescriptor_Style_internal_data_[];
class ResourceDescriptor;
struct ResourceDescriptorGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ResourceDescriptorGlobalsTypeInternal ResourceDescriptor_globals_;
extern const ::google::protobuf::internal::ClassDataFull ResourceDescriptor_class_data_;
#else
extern const ResourceDescriptorGlobalsTypeInternal ResourceDescriptor_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ResourceReference;
struct ResourceReferenceGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ResourceReferenceGlobalsTypeInternal ResourceReference_globals_;
extern const ::google::protobuf::internal::ClassDataFull ResourceReference_class_data_;
#else
extern const ResourceReferenceGlobalsTypeInternal ResourceReference_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace api
namespace protobuf {
template <>
internal::EnumTraitsT<::google::api::ResourceDescriptor_History_internal_data_>
    internal::EnumTraitsImpl::value<::google::api::ResourceDescriptor_History>;
template <>
internal::EnumTraitsT<::google::api::ResourceDescriptor_Style_internal_data_>
    internal::EnumTraitsImpl::value<::google::api::ResourceDescriptor_Style>;
}  // namespace protobuf
}  // namespace google

namespace google {
namespace api {
enum ResourceDescriptor_History : int {
  ResourceDescriptor_History_HISTORY_UNSPECIFIED = 0,
  ResourceDescriptor_History_ORIGINALLY_SINGLE_PATTERN = 1,
  ResourceDescriptor_History_FUTURE_MULTI_PATTERN = 2,
  ResourceDescriptor_History_ResourceDescriptor_History_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  ResourceDescriptor_History_ResourceDescriptor_History_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t ResourceDescriptor_History_internal_data_[];
inline constexpr ResourceDescriptor_History ResourceDescriptor_History_History_MIN =
    static_cast<ResourceDescriptor_History>(0);
inline constexpr ResourceDescriptor_History ResourceDescriptor_History_History_MAX =
    static_cast<ResourceDescriptor_History>(2);
[[nodiscard]] inline bool ResourceDescriptor_History_IsValid(int value) {
  return 0 <= value && value <= 2;
}
inline constexpr int ResourceDescriptor_History_History_ARRAYSIZE = 2 + 1;
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
ResourceDescriptor_History_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(ResourceDescriptor_History) {
  return ResourceDescriptor_History_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& ResourceDescriptor_History_Name(T value) {
  static_assert(::std::is_same<T, ResourceDescriptor_History>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to History_Name().");
  return ResourceDescriptor_History_Name(static_cast<ResourceDescriptor_History>(value));
}
template <>
[[nodiscard]] inline const ::std::string& ResourceDescriptor_History_Name(ResourceDescriptor_History value) {
  return ::google::protobuf::internal::NameOfDenseEnum<ResourceDescriptor_History_descriptor, 0, 2>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool ResourceDescriptor_History_Parse(
    ::absl::string_view name, ResourceDescriptor_History* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<ResourceDescriptor_History>(ResourceDescriptor_History_descriptor(), name,
                                           value);
}
enum ResourceDescriptor_Style : int {
  ResourceDescriptor_Style_STYLE_UNSPECIFIED = 0,
  ResourceDescriptor_Style_DECLARATIVE_FRIENDLY = 1,
  ResourceDescriptor_Style_ResourceDescriptor_Style_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  ResourceDescriptor_Style_ResourceDescriptor_Style_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t ResourceDescriptor_Style_internal_data_[];
inline constexpr ResourceDescriptor_Style ResourceDescriptor_Style_Style_MIN =
    static_cast<ResourceDescriptor_Style>(0);
inline constexpr ResourceDescriptor_Style ResourceDescriptor_Style_Style_MAX =
    static_cast<ResourceDescriptor_Style>(1);
[[nodiscard]] inline bool ResourceDescriptor_Style_IsValid(int value) {
  return 0 <= value && value <= 1;
}
inline constexpr int ResourceDescriptor_Style_Style_ARRAYSIZE = 1 + 1;
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
ResourceDescriptor_Style_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(ResourceDescriptor_Style) {
  return ResourceDescriptor_Style_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& ResourceDescriptor_Style_Name(T value) {
  static_assert(::std::is_same<T, ResourceDescriptor_Style>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to Style_Name().");
  return ResourceDescriptor_Style_Name(static_cast<ResourceDescriptor_Style>(value));
}
template <>
[[nodiscard]] inline const ::std::string& ResourceDescriptor_Style_Name(ResourceDescriptor_Style value) {
  return ::google::protobuf::internal::NameOfDenseEnum<ResourceDescriptor_Style_descriptor, 0, 1>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool ResourceDescriptor_Style_Parse(
    ::absl::string_view name, ResourceDescriptor_Style* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<ResourceDescriptor_Style>(ResourceDescriptor_Style_descriptor(), name,
                                           value);
}
using ::google::protobuf::internal::generated_enum::AbslParseFlag;
using ::google::protobuf::internal::generated_enum::AbslUnparseFlag;

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ResourceReference final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:google.api.ResourceReference) */ {
 public:
  inline ResourceReference() : ResourceReference(nullptr) {}
  ~ResourceReference() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ResourceReference* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ResourceReference));
  }
#endif

  template <typename = void>
  explicit constexpr ResourceReference(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ResourceReference(const ResourceReference& from) : ResourceReference(nullptr, from) {}
  inline ResourceReference(ResourceReference&& from) noexcept : ResourceReference(nullptr, ::std::move(from)) {}
  inline ResourceReference& operator=(const ResourceReference& from) {
    CopyFrom(from);
    return *this;
  }
  inline ResourceReference& operator=(ResourceReference&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ResourceReference& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ResourceReference>(&ResourceReference_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(ResourceReference& a, ResourceReference& b) { a.Swap(&b); }
  inline void Swap(ResourceReference* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ResourceReference* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ResourceReference* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ResourceReference>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ResourceReference& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ResourceReference& from) { ResourceReference::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ResourceReference* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "google.api.ResourceReference"; }

  explicit ResourceReference(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ResourceReference(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ResourceReference& from);
  ResourceReference(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ResourceReference&& from) noexcept
      : ResourceReference(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kTypeFieldNumber = 1,
    kChildTypeFieldNumber = 2,
  };
  // string type = 1;
  void clear_type() ;
  [[nodiscard]] const ::std::string& type() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_type(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_type();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_type();
  void set_allocated_type(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_type() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_type(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_type();

  public:
  // string child_type = 2;
  void clear_child_type() ;
  [[nodiscard]] const ::std::string& child_type() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_child_type(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_child_type();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_child_type();
  void set_allocated_child_type(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_child_type() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_child_type(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_child_type();

  public:
  // @@protoc_insertion_point(class_scope:google.api.ResourceReference)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          0, 51,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ResourceReference& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr type_;
    ::google::protobuf::internal::ArenaStringPtr child_type_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_google_2fapi_2fresource_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ResourceDescriptor final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:google.api.ResourceDescriptor) */ {
 public:
  inline ResourceDescriptor() : ResourceDescriptor(nullptr) {}
  ~ResourceDescriptor() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ResourceDescriptor* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ResourceDescriptor));
  }
#endif

  template <typename = void>
  explicit constexpr ResourceDescriptor(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ResourceDescriptor(const ResourceDescriptor& from) : ResourceDescriptor(nullptr, from) {}
  inline ResourceDescriptor(ResourceDescriptor&& from) noexcept : ResourceDescriptor(nullptr, ::std::move(from)) {}
  inline ResourceDescriptor& operator=(const ResourceDescriptor& from) {
    CopyFrom(from);
    return *this;
  }
  inline ResourceDescriptor& operator=(ResourceDescriptor&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ResourceDescriptor& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ResourceDescriptor>(&ResourceDescriptor_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(ResourceDescriptor& a, ResourceDescriptor& b) { a.Swap(&b); }
  inline void Swap(ResourceDescriptor* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ResourceDescriptor* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ResourceDescriptor* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ResourceDescriptor>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ResourceDescriptor& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ResourceDescriptor& from) { ResourceDescriptor::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ResourceDescriptor* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "google.api.ResourceDescriptor"; }

  explicit ResourceDescriptor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ResourceDescriptor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ResourceDescriptor& from);
  ResourceDescriptor(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ResourceDescriptor&& from) noexcept
      : ResourceDescriptor(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using History = ResourceDescriptor_History;
  static constexpr History HISTORY_UNSPECIFIED = ResourceDescriptor_History_HISTORY_UNSPECIFIED;
  static constexpr History ORIGINALLY_SINGLE_PATTERN = ResourceDescriptor_History_ORIGINALLY_SINGLE_PATTERN;
  static constexpr History FUTURE_MULTI_PATTERN = ResourceDescriptor_History_FUTURE_MULTI_PATTERN;
  [[nodiscard]] static inline bool History_IsValid(int value) {
    return ResourceDescriptor_History_IsValid(value);
  }
  static constexpr History History_MIN = ResourceDescriptor_History_History_MIN;
  static constexpr History History_MAX = ResourceDescriptor_History_History_MAX;
  static constexpr int History_ARRAYSIZE = ResourceDescriptor_History_History_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL History_descriptor() {
    return ResourceDescriptor_History_descriptor();
  }
  template <typename T>
  [[nodiscard]] static inline const ::std::string& History_Name(T value) {
    return ResourceDescriptor_History_Name(value);
  }
  [[nodiscard]] static inline bool History_Parse(
      ::absl::string_view name, History* PROTOBUF_NONNULL value) {
    return ResourceDescriptor_History_Parse(name, value);
  }
  using Style = ResourceDescriptor_Style;
  static constexpr Style STYLE_UNSPECIFIED = ResourceDescriptor_Style_STYLE_UNSPECIFIED;
  static constexpr Style DECLARATIVE_FRIENDLY = ResourceDescriptor_Style_DECLARATIVE_FRIENDLY;
  [[nodiscard]] static inline bool Style_IsValid(int value) {
    return ResourceDescriptor_Style_IsValid(value);
  }
  static constexpr Style Style_MIN = ResourceDescriptor_Style_Style_MIN;
  static constexpr Style Style_MAX = ResourceDescriptor_Style_Style_MAX;
  static constexpr int Style_ARRAYSIZE = ResourceDescriptor_Style_Style_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL Style_descriptor() {
    return ResourceDescriptor_Style_descriptor();
  }
  template <typename T>
  [[nodiscard]] static inline const ::std::string& Style_Name(T value) {
    return ResourceDescriptor_Style_Name(value);
  }
  [[nodiscard]] static inline bool Style_Parse(
      ::absl::string_view name, Style* PROTOBUF_NONNULL value) {
    return ResourceDescriptor_Style_Parse(name, value);
  }

  // accessors -------------------------------------------------------
  enum : int {
    kPatternFieldNumber = 2,
    kTypeFieldNumber = 1,
    kNameFieldFieldNumber = 3,
    kPluralFieldNumber = 5,
    kSingularFieldNumber = 6,
    kHistoryFieldNumber = 4,
    kStyleFieldNumber = 10,
  };
  // repeated string pattern = 2;
  [[nodiscard]] int pattern_size()
      const;
  private:
  int _internal_pattern_size() const;

  public:
  void clear_pattern() ;
  [[nodiscard]] const ::std::string& pattern(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_pattern(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_pattern(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_pattern();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_pattern(Arg_&& value, Args_... args);
  [[nodiscard]] const
      ::google::protobuf::RepeatedPtrField<::std::string>&
      pattern() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::std::string>*
      PROTOBUF_NONNULL
      mutable_pattern();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_pattern() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_pattern();

  public:
  // string type = 1;
  void clear_type() ;
  [[nodiscard]] const ::std::string& type() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_type(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_type();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_type();
  void set_allocated_type(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_type() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_type(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_type();

  public:
  // string name_field = 3;
  void clear_name_field() ;
  [[nodiscard]] const ::std::string& name_field() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_name_field(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_name_field();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_name_field();
  void set_allocated_name_field(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_name_field() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_name_field(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_name_field();

  public:
  // string plural = 5;
  void clear_plural() ;
  [[nodiscard]] const ::std::string& plural() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_plural(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_plural();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_plural();
  void set_allocated_plural(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_plural() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_plural(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_plural();

  public:
  // string singular = 6;
  void clear_singular() ;
  [[nodiscard]] const ::std::string& singular() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_singular(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_singular();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_singular();
  void set_allocated_singular(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_singular() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_singular(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_singular();

  public:
  // .google.api.ResourceDescriptor.History history = 4;
  void clear_history() ;
  [[nodiscard]] ::google::api::ResourceDescriptor_History history() const;
  void set_history(::google::api::ResourceDescriptor_History value);

  private:
  ::google::api::ResourceDescriptor_History _internal_history() const;
  void _internal_set_history(::google::api::ResourceDescriptor_History value);

  public:
  // repeated .google.api.ResourceDescriptor.Style style = 10;
  [[nodiscard]] int style_size()
      const;
  private:
  int _internal_style_size() const;

  public:
  void clear_style() ;
  public:
  [[nodiscard]] ::google::api::ResourceDescriptor_Style style(int index) const;
  void set_style(int index, ::google::api::ResourceDescriptor_Style value);
  void add_style(::google::api::ResourceDescriptor_Style value);
  [[nodiscard]] const ::google::protobuf::RepeatedField<int>& style()
      const;
  [[nodiscard]] ::google::protobuf::RepeatedField<int>* PROTOBUF_NONNULL mutable_style();

  private:
  const ::google::protobuf::RepeatedField<int>& _internal_style() const;
  ::google::protobuf::RepeatedField<int>* PROTOBUF_NONNULL _internal_mutable_style();

  public:
  // @@protoc_insertion_point(class_scope:google.api.ResourceDescriptor)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 7,
                          0, 73,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ResourceDescriptor& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField<::std::string> pattern_;
    ::google::protobuf::internal::ArenaStringPtr type_;
    ::google::protobuf::internal::ArenaStringPtr name_field_;
    ::google::protobuf::internal::ArenaStringPtr plural_;
    ::google::protobuf::internal::ArenaStringPtr singular_;
    int history_;
    ::google::protobuf::RepeatedField<int> style_;
    ::google::protobuf::internal::CachedSize _style_cached_byte_size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_google_2fapi_2fresource_2eproto;
};

// ===================================================================



inline constexpr int kResourceReferenceFieldNumber = 1055;
extern ::google::protobuf::internal::ExtensionIdentifier<
    ::google::protobuf::FieldOptions, ::google::protobuf::internal::MessageTypeTraits< ::google::api::ResourceReference >, 11, false>(resource_reference);
inline constexpr int kResourceDefinitionFieldNumber = 1053;
extern ::google::protobuf::internal::ExtensionIdentifier<
    ::google::protobuf::FileOptions, ::google::protobuf::internal::RepeatedMessageTypeTraits< ::google::api::ResourceDescriptor >, 11, false>(resource_definition);
inline constexpr int kResourceFieldNumber = 1053;
extern ::google::protobuf::internal::ExtensionIdentifier<
    ::google::protobuf::MessageOptions, ::google::protobuf::internal::MessageTypeTraits< ::google::api::ResourceDescriptor >, 11, false>(resource);

// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// ResourceDescriptor

// string type = 1;
inline void ResourceDescriptor::clear_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& ResourceDescriptor::type() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:google.api.ResourceDescriptor.type)
  return _internal_type();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ResourceDescriptor::set_type(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.type_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:google.api.ResourceDescriptor.type)
}
inline ::std::string* PROTOBUF_NONNULL ResourceDescriptor::mutable_type()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_type();
  // @@protoc_insertion_point(field_mutable:google.api.ResourceDescriptor.type)
  return _s;
}
inline const ::std::string& ResourceDescriptor::_internal_type() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.type_.Get();
}
inline void ResourceDescriptor::_internal_set_type(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ResourceDescriptor::_internal_mutable_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.type_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ResourceDescriptor::release_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:google.api.ResourceDescriptor.type)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.type_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.type_.Set("", GetArena());
  }
  return released;
}
inline void ResourceDescriptor::set_allocated_type(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.type_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.type_.IsDefault()) {
    _impl_.type_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:google.api.ResourceDescriptor.type)
}

// repeated string pattern = 2;
inline int ResourceDescriptor::_internal_pattern_size() const {
  return _internal_pattern().size();
}
inline int ResourceDescriptor::pattern_size() const {
  return _internal_pattern_size();
}
inline void ResourceDescriptor::clear_pattern() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pattern_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline ::std::string* PROTOBUF_NONNULL ResourceDescriptor::add_pattern()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_pattern()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add_mutable:google.api.ResourceDescriptor.pattern)
  return _s;
}
inline const ::std::string& ResourceDescriptor::pattern(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:google.api.ResourceDescriptor.pattern)
  return _internal_pattern().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL ResourceDescriptor::mutable_pattern(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:google.api.ResourceDescriptor.pattern)
  return _internal_mutable_pattern()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void ResourceDescriptor::set_pattern(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(
      *_internal_mutable_pattern()->Mutable(index),
      ::std::forward<Arg_>(value), args... );
  // @@protoc_insertion_point(field_set:google.api.ResourceDescriptor.pattern)
}
template <typename Arg_, typename... Args_>
inline void ResourceDescriptor::add_pattern(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_pattern(), ::std::forward<Arg_>(value),
      args... );
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:google.api.ResourceDescriptor.pattern)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& ResourceDescriptor::pattern()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:google.api.ResourceDescriptor.pattern)
  return _internal_pattern();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
ResourceDescriptor::mutable_pattern() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:google.api.ResourceDescriptor.pattern)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_pattern();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
ResourceDescriptor::_internal_pattern() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.pattern_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
ResourceDescriptor::_internal_mutable_pattern() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.pattern_;
}

// string name_field = 3;
inline void ResourceDescriptor::clear_name_field() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_field_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::std::string& ResourceDescriptor::name_field() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:google.api.ResourceDescriptor.name_field)
  return _internal_name_field();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ResourceDescriptor::set_name_field(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.name_field_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:google.api.ResourceDescriptor.name_field)
}
inline ::std::string* PROTOBUF_NONNULL ResourceDescriptor::mutable_name_field()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_name_field();
  // @@protoc_insertion_point(field_mutable:google.api.ResourceDescriptor.name_field)
  return _s;
}
inline const ::std::string& ResourceDescriptor::_internal_name_field() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.name_field_.Get();
}
inline void ResourceDescriptor::_internal_set_name_field(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_field_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ResourceDescriptor::_internal_mutable_name_field() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.name_field_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ResourceDescriptor::release_name_field() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:google.api.ResourceDescriptor.name_field)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.name_field_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.name_field_.Set("", GetArena());
  }
  return released;
}
inline void ResourceDescriptor::set_allocated_name_field(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.name_field_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.name_field_.IsDefault()) {
    _impl_.name_field_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:google.api.ResourceDescriptor.name_field)
}

// .google.api.ResourceDescriptor.History history = 4;
inline void ResourceDescriptor::clear_history() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.history_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::google::api::ResourceDescriptor_History ResourceDescriptor::history() const {
  // @@protoc_insertion_point(field_get:google.api.ResourceDescriptor.history)
  return _internal_history();
}
inline void ResourceDescriptor::set_history(::google::api::ResourceDescriptor_History value) {
  _internal_set_history(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:google.api.ResourceDescriptor.history)
}
inline ::google::api::ResourceDescriptor_History ResourceDescriptor::_internal_history() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::google::api::ResourceDescriptor_History>(_impl_.history_);
}
inline void ResourceDescriptor::_internal_set_history(::google::api::ResourceDescriptor_History value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.history_ = value;
}

// string plural = 5;
inline void ResourceDescriptor::clear_plural() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.plural_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::std::string& ResourceDescriptor::plural() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:google.api.ResourceDescriptor.plural)
  return _internal_plural();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ResourceDescriptor::set_plural(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  _impl_.plural_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:google.api.ResourceDescriptor.plural)
}
inline ::std::string* PROTOBUF_NONNULL ResourceDescriptor::mutable_plural()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::std::string* _s = _internal_mutable_plural();
  // @@protoc_insertion_point(field_mutable:google.api.ResourceDescriptor.plural)
  return _s;
}
inline const ::std::string& ResourceDescriptor::_internal_plural() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.plural_.Get();
}
inline void ResourceDescriptor::_internal_set_plural(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.plural_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ResourceDescriptor::_internal_mutable_plural() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.plural_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ResourceDescriptor::release_plural() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:google.api.ResourceDescriptor.plural)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000008U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  auto* released = _impl_.plural_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.plural_.Set("", GetArena());
  }
  return released;
}
inline void ResourceDescriptor::set_allocated_plural(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  _impl_.plural_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.plural_.IsDefault()) {
    _impl_.plural_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:google.api.ResourceDescriptor.plural)
}

// string singular = 6;
inline void ResourceDescriptor::clear_singular() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.singular_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline const ::std::string& ResourceDescriptor::singular() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:google.api.ResourceDescriptor.singular)
  return _internal_singular();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ResourceDescriptor::set_singular(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  _impl_.singular_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:google.api.ResourceDescriptor.singular)
}
inline ::std::string* PROTOBUF_NONNULL ResourceDescriptor::mutable_singular()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::std::string* _s = _internal_mutable_singular();
  // @@protoc_insertion_point(field_mutable:google.api.ResourceDescriptor.singular)
  return _s;
}
inline const ::std::string& ResourceDescriptor::_internal_singular() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.singular_.Get();
}
inline void ResourceDescriptor::_internal_set_singular(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.singular_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ResourceDescriptor::_internal_mutable_singular() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.singular_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ResourceDescriptor::release_singular() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:google.api.ResourceDescriptor.singular)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000010U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  auto* released = _impl_.singular_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.singular_.Set("", GetArena());
  }
  return released;
}
inline void ResourceDescriptor::set_allocated_singular(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }
  _impl_.singular_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.singular_.IsDefault()) {
    _impl_.singular_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:google.api.ResourceDescriptor.singular)
}

// repeated .google.api.ResourceDescriptor.Style style = 10;
inline int ResourceDescriptor::_internal_style_size() const {
  return _internal_style().size();
}
inline int ResourceDescriptor::style_size() const {
  return _internal_style_size();
}
inline void ResourceDescriptor::clear_style() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.style_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::google::api::ResourceDescriptor_Style ResourceDescriptor::style(int index) const {
  // @@protoc_insertion_point(field_get:google.api.ResourceDescriptor.style)
  return static_cast<::google::api::ResourceDescriptor_Style>(_internal_style().Get(index));
}
inline void ResourceDescriptor::set_style(int index, ::google::api::ResourceDescriptor_Style value) {
  _internal_mutable_style()->Set(index, value);
  // @@protoc_insertion_point(field_set:google.api.ResourceDescriptor.style)
}
inline void ResourceDescriptor::add_style(::google::api::ResourceDescriptor_Style value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_style()
      ->InternalAddWithArena<const ::google::protobuf::MessageLite*>(
          internal_visibility(), this, value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_add:google.api.ResourceDescriptor.style)
}
inline const ::google::protobuf::RepeatedField<int>& ResourceDescriptor::style() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:google.api.ResourceDescriptor.style)
  return _internal_style();
}
inline ::google::protobuf::RepeatedField<int>* PROTOBUF_NONNULL ResourceDescriptor::mutable_style()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_mutable_list:google.api.ResourceDescriptor.style)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_style();
}
inline const ::google::protobuf::RepeatedField<int>& ResourceDescriptor::_internal_style()
    const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.style_;
}
inline ::google::protobuf::RepeatedField<int>* PROTOBUF_NONNULL
ResourceDescriptor::_internal_mutable_style() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.style_;
}

// -------------------------------------------------------------------

// ResourceReference

// string type = 1;
inline void ResourceReference::clear_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& ResourceReference::type() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:google.api.ResourceReference.type)
  return _internal_type();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ResourceReference::set_type(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.type_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:google.api.ResourceReference.type)
}
inline ::std::string* PROTOBUF_NONNULL ResourceReference::mutable_type()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_type();
  // @@protoc_insertion_point(field_mutable:google.api.ResourceReference.type)
  return _s;
}
inline const ::std::string& ResourceReference::_internal_type() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.type_.Get();
}
inline void ResourceReference::_internal_set_type(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ResourceReference::_internal_mutable_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.type_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ResourceReference::release_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:google.api.ResourceReference.type)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.type_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.type_.Set("", GetArena());
  }
  return released;
}
inline void ResourceReference::set_allocated_type(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.type_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.type_.IsDefault()) {
    _impl_.type_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:google.api.ResourceReference.type)
}

// string child_type = 2;
inline void ResourceReference::clear_child_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.child_type_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& ResourceReference::child_type() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:google.api.ResourceReference.child_type)
  return _internal_child_type();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ResourceReference::set_child_type(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.child_type_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:google.api.ResourceReference.child_type)
}
inline ::std::string* PROTOBUF_NONNULL ResourceReference::mutable_child_type()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_child_type();
  // @@protoc_insertion_point(field_mutable:google.api.ResourceReference.child_type)
  return _s;
}
inline const ::std::string& ResourceReference::_internal_child_type() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.child_type_.Get();
}
inline void ResourceReference::_internal_set_child_type(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.child_type_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ResourceReference::_internal_mutable_child_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.child_type_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ResourceReference::release_child_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:google.api.ResourceReference.child_type)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.child_type_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.child_type_.Set("", GetArena());
  }
  return released;
}
inline void ResourceReference::set_allocated_child_type(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.child_type_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.child_type_.IsDefault()) {
    _impl_.child_type_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:google.api.ResourceReference.child_type)
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace api
}  // namespace google


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::google::api::ResourceDescriptor_History> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::google::api::ResourceDescriptor_History>() {
  return ::google::api::ResourceDescriptor_History_descriptor();
}
template <>
struct is_proto_enum<::google::api::ResourceDescriptor_Style> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::google::api::ResourceDescriptor_Style>() {
  return ::google::api::ResourceDescriptor_Style_descriptor();
}

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // google_2fapi_2fresource_2eproto_2epb_2eh
