// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: google/cloud/bigquery/storage/v1/table.proto
// Protobuf C++ Version: 7.35.1

#include "google/cloud/bigquery/storage/v1/table.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
#ifdef PROTOBUF_MESSAGE_GLOBALS
namespace {
PROTOBUF_CONSTINIT ::google::protobuf::internal::ReflectionData
    file_reflection_data[] = {
        // ::google::cloud::bigquery::storage::v1::TableSchema
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::TableFieldSchema
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto, /* tracker*/ nullptr,},
};
}  // namespace
#endif
namespace google {
namespace cloud {
namespace bigquery {
namespace storage {
namespace v1 {
class TableFieldSchema::_Internal {
 public:
  using HasBits = decltype(::std::declval<TableFieldSchema>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_._has_bits_);
};

constexpr TableFieldSchema::ParseTableT_ TableFieldSchema::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_._has_bits_),
      0, // no _extensions_
      10, 120,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294966288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      9,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::TableFieldSchema>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string name = 1 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastUS1,
       {10, 1, 0,
        PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_.name_)}},
      // .google.cloud.bigquery.storage.v1.TableFieldSchema.Type type = 2 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(TableFieldSchema, _impl_.type_), 4>(),
       {16, 4, 0,
        PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_.type_)}},
      // .google.cloud.bigquery.storage.v1.TableFieldSchema.Mode mode = 3 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(TableFieldSchema, _impl_.mode_), 5>(),
       {24, 5, 0,
        PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_.mode_)}},
      // repeated .google.cloud.bigquery.storage.v1.TableFieldSchema fields = 4 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtR1,
       {34, 0, 0,
        PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_.fields_)}},
      {::_pbi::TcParser::MiniParse, {}},
      // string description = 6 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {50, 2, 0,
        PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_.description_)}},
      // int64 max_length = 7 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(TableFieldSchema, _impl_.max_length_), 6>(),
       {56, 6, 0,
        PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_.max_length_)}},
      // int64 precision = 8 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(TableFieldSchema, _impl_.precision_), 7>(),
       {64, 7, 0,
        PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_.precision_)}},
      // int64 scale = 9 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(TableFieldSchema, _impl_.scale_), 8>(),
       {72, 8, 0,
        PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_.scale_)}},
      // string default_value_expression = 10 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {82, 3, 0,
        PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_.default_value_expression_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // string name = 1 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_.name_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.cloud.bigquery.storage.v1.TableFieldSchema.Type type = 2 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_.type_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // .google.cloud.bigquery.storage.v1.TableFieldSchema.Mode mode = 3 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_.mode_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // repeated .google.cloud.bigquery.storage.v1.TableFieldSchema fields = 4 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_.fields_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
      // string description = 6 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_.description_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // int64 max_length = 7 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_.max_length_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // int64 precision = 8 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_.precision_), _Internal::kHasBitsOffset + 7, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // int64 scale = 9 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_.scale_), _Internal::kHasBitsOffset + 8, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // string default_value_expression = 10 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_.default_value_expression_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::TableFieldSchema>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::cloud::bigquery::storage::v1::TableFieldSchema_globals_},
        #endif
    }},
    {{
      "\61\4\0\0\0\13\0\0\0\30\0\0\0\0\0\0"
      "google.cloud.bigquery.storage.v1.TableFieldSchema"
      "name"
      "description"
      "default_value_expression"
    }},
  };
}


inline constexpr TableFieldSchema::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        fields_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::cloud::bigquery::storage::v1::TableFieldSchema,
            PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::TableFieldSchema, _impl_.fields_)>()
         }
        ,
        name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        description_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        default_value_expression_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        type_{static_cast< ::google::cloud::bigquery::storage::v1::TableFieldSchema_Type >(0)},
        mode_{static_cast< ::google::cloud::bigquery::storage::v1::TableFieldSchema_Mode >(0)},
        max_length_{::int64_t{0}},
        precision_{::int64_t{0}},
        scale_{::int64_t{0}} {}

template <typename>
constexpr TableFieldSchema::TableFieldSchema(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL TableFieldSchema::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) TableFieldSchema(arena);
}
constexpr auto TableFieldSchema::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(TableFieldSchema), alignof(TableFieldSchema));
}
constexpr auto TableFieldSchema::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &TableFieldSchema::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<TableFieldSchema>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &TableFieldSchema::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<TableFieldSchema>(), &TableFieldSchema::ByteSizeLong,
              &TableFieldSchema::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[1],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct TableFieldSchemaGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr TableFieldSchemaGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 TableFieldSchema_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(TableFieldSchema::InternalGenerateClassData_(
            _default, &TableFieldSchema_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<TableFieldSchema>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~TableFieldSchemaGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) TableFieldSchema _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<TableFieldSchema>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(TableFieldSchemaGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST TableFieldSchemaGlobalsTypeInternal TableFieldSchema_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* TableFieldSchema_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return TableFieldSchema_globals_.GetClassData();
#else
  return TableFieldSchema_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class TableSchema::_Internal {
 public:
  using HasBits = decltype(::std::declval<TableSchema>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(TableSchema, _impl_._has_bits_);
};

constexpr TableSchema::ParseTableT_ TableSchema::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(TableSchema, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::TableSchema>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated .google.cloud.bigquery.storage.v1.TableFieldSchema fields = 1;
      {::_pbi::TcParser::FastMtR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(TableSchema, _impl_.fields_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated .google.cloud.bigquery.storage.v1.TableFieldSchema fields = 1;
      {PROTOBUF_FIELD_OFFSET(TableSchema, _impl_.fields_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::TableFieldSchema>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::cloud::bigquery::storage::v1::TableFieldSchema_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr TableSchema::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        fields_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::cloud::bigquery::storage::v1::TableSchema,
            PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::TableSchema, _impl_.fields_)>()
         }
     {}

template <typename>
constexpr TableSchema::TableSchema(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL TableSchema::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) TableSchema(arena);
}
constexpr auto TableSchema::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(TableSchema), alignof(TableSchema));
}
constexpr auto TableSchema::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &TableSchema::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<TableSchema>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &TableSchema::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<TableSchema>(), &TableSchema::ByteSizeLong,
              &TableSchema::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(TableSchema, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[0],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct TableSchemaGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr TableSchemaGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 TableSchema_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(TableSchema::InternalGenerateClassData_(
            _default, &TableSchema_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<TableSchema>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~TableSchemaGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) TableSchema _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<TableSchema>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(TableSchemaGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST TableSchemaGlobalsTypeInternal TableSchema_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* TableSchema_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return TableSchema_globals_.GetClassData();
#else
  return TableSchema_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
}  // namespace v1
}  // namespace storage
}  // namespace bigquery
}  // namespace cloud
}  // namespace google
static const ::_pb::EnumDescriptor* PROTOBUF_NONNULL
    file_level_enum_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto[2];
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto = nullptr;
const ::uint32_t
    TableStruct_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::TableSchema, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::TableSchema, _impl_.fields_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::TableFieldSchema, _impl_._has_bits_),
        12, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::TableFieldSchema, _impl_.name_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::TableFieldSchema, _impl_.type_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::TableFieldSchema, _impl_.mode_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::TableFieldSchema, _impl_.fields_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::TableFieldSchema, _impl_.description_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::TableFieldSchema, _impl_.max_length_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::TableFieldSchema, _impl_.precision_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::TableFieldSchema, _impl_.scale_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::TableFieldSchema, _impl_.default_value_expression_),
        1,
        4,
        5,
        0,
        2,
        6,
        7,
        8,
        3,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::google::cloud::bigquery::storage::v1::TableSchema)},
        {5, sizeof(::google::cloud::bigquery::storage::v1::TableFieldSchema)},
};
static const ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const
    file_message_globals[] = {
        &::google::cloud::bigquery::storage::v1::TableSchema_globals_,
        &::google::cloud::bigquery::storage::v1::TableFieldSchema_globals_,
};
const char descriptor_table_protodef_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n,google/cloud/bigquery/storage/v1/table"
    ".proto\022 google.cloud.bigquery.storage.v1"
    "\032\037google/api/field_behavior.proto\"Q\n\013Tab"
    "leSchema\022B\n\006fields\030\001 \003(\01322.google.cloud."
    "bigquery.storage.v1.TableFieldSchema\"\265\005\n"
    "\020TableFieldSchema\022\022\n\004name\030\001 \001(\tB\004\342A\001\002\022K\n"
    "\004type\030\002 \001(\01627.google.cloud.bigquery.stor"
    "age.v1.TableFieldSchema.TypeB\004\342A\001\002\022K\n\004mo"
    "de\030\003 \001(\01627.google.cloud.bigquery.storage"
    ".v1.TableFieldSchema.ModeB\004\342A\001\001\022H\n\006field"
    "s\030\004 \003(\01322.google.cloud.bigquery.storage."
    "v1.TableFieldSchemaB\004\342A\001\001\022\031\n\013description"
    "\030\006 \001(\tB\004\342A\001\001\022\030\n\nmax_length\030\007 \001(\003B\004\342A\001\001\022\027"
    "\n\tprecision\030\010 \001(\003B\004\342A\001\001\022\023\n\005scale\030\t \001(\003B\004"
    "\342A\001\001\022&\n\030default_value_expression\030\n \001(\tB\004"
    "\342A\001\001\"\325\001\n\004Type\022\024\n\020TYPE_UNSPECIFIED\020\000\022\n\n\006S"
    "TRING\020\001\022\t\n\005INT64\020\002\022\n\n\006DOUBLE\020\003\022\n\n\006STRUCT"
    "\020\004\022\t\n\005BYTES\020\005\022\010\n\004BOOL\020\006\022\r\n\tTIMESTAMP\020\007\022\010"
    "\n\004DATE\020\010\022\010\n\004TIME\020\t\022\014\n\010DATETIME\020\n\022\r\n\tGEOG"
    "RAPHY\020\013\022\013\n\007NUMERIC\020\014\022\016\n\nBIGNUMERIC\020\r\022\014\n\010"
    "INTERVAL\020\016\022\010\n\004JSON\020\017\"F\n\004Mode\022\024\n\020MODE_UNS"
    "PECIFIED\020\000\022\014\n\010NULLABLE\020\001\022\014\n\010REQUIRED\020\002\022\014"
    "\n\010REPEATED\020\003B\272\001\n$com.google.cloud.bigque"
    "ry.storage.v1B\nTableProtoP\001Z>cloud.googl"
    "e.com/go/bigquery/storage/apiv1/storagep"
    "b;storagepb\252\002 Google.Cloud.BigQuery.Stor"
    "age.V1\312\002 Google\\Cloud\\BigQuery\\Storage\\V"
    "1b\006proto3"
};
static const ::_pbi::DescriptorTable* PROTOBUF_NONNULL const
    descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto_deps[1] = {
        &::descriptor_table_google_2fapi_2ffield_5fbehavior_2eproto,
};
static ::absl::once_flag descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto = {
    false,
    false,
    1089,
    descriptor_table_protodef_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto,
    "google/cloud/bigquery/storage/v1/table.proto",
    &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto_once,
    descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto_deps,
    1,
    2,
    schemas,
    file_message_globals,
    TableStruct_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto::offsets,
    file_level_enum_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto,
    file_level_service_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto,
};
namespace google {
namespace cloud {
namespace bigquery {
namespace storage {
namespace v1 {
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
TableFieldSchema_Type_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto);
  return file_level_enum_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto[0];
}
PROTOBUF_CONSTINIT const uint32_t TableFieldSchema_Type_internal_data_[] = {
    1048576u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
TableFieldSchema_Mode_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto);
  return file_level_enum_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto[1];
}
PROTOBUF_CONSTINIT const uint32_t TableFieldSchema_Mode_internal_data_[] = {
    262144u, 0u, };
// ===================================================================

TableSchema::TableSchema(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, TableSchema_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.TableSchema)
}
PROTOBUF_NDEBUG_INLINE TableSchema::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::TableSchema& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        fields_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::cloud::bigquery::storage::v1::TableSchema,
              PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::TableSchema, _impl_.fields_)>()
          , from.fields_
        }
     {}

TableSchema::TableSchema(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const TableSchema& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, TableSchema_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  TableSchema* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.TableSchema)
}
PROTOBUF_NDEBUG_INLINE TableSchema::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        fields_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::cloud::bigquery::storage::v1::TableSchema,
            PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::TableSchema, _impl_.fields_)>()
         }
     {}

inline void TableSchema::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
TableSchema::~TableSchema() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.TableSchema)
  SharedDtor(*this);
}
inline void TableSchema::SharedDtor(MessageLite& self) {
  TableSchema& this_ = static_cast<TableSchema&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull TableSchema_class_data_ =
        TableSchema::InternalGenerateClassData_(TableSchema_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
TableSchema::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&TableSchema_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(TableSchema_class_data_.tc_table);
  return TableSchema_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
TableSchema::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&TableSchema_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&TableSchema_globals_));
  return TableSchema_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const TableSchema::ParseTableT_
    TableSchema::_table_ =
        TableSchema::InternalGenerateParseTable_(TableSchema_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void TableSchema::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.TableSchema)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.fields_.Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL TableSchema::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const TableSchema& this_ = static_cast<const TableSchema&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL TableSchema::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const TableSchema& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.TableSchema)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated .google.cloud.bigquery.storage.v1.TableFieldSchema fields = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_fields_size());
         i < n; i++) {
      const auto& repfield = this_._internal_fields().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              1, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.TableSchema)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t TableSchema::ByteSizeLong(const MessageLite& base) {
  const TableSchema& this_ = static_cast<const TableSchema&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t TableSchema::ByteSizeLong() const {
  const TableSchema& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.TableSchema)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
   {
    // repeated .google.cloud.bigquery.storage.v1.TableFieldSchema fields = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_fields_size();
      for (const auto& msg : this_._internal_fields()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void TableSchema::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<TableSchema*>(&to_msg);
  auto& from = static_cast<const TableSchema&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.TableSchema)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _this->_internal_mutable_fields()->InternalMergeFromWithArena(
        ::google::protobuf::MessageLite::internal_visibility(), arena,
        from._internal_fields());
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void TableSchema::CopyFrom(const TableSchema& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.TableSchema)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void TableSchema::InternalSwap(TableSchema* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.fields_.InternalSwap(&other->_impl_.fields_);
}

::google::protobuf::Metadata TableSchema::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

TableFieldSchema::TableFieldSchema(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, TableFieldSchema_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.TableFieldSchema)
}
PROTOBUF_NDEBUG_INLINE TableFieldSchema::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::TableFieldSchema& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        fields_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::cloud::bigquery::storage::v1::TableFieldSchema,
              PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::TableFieldSchema, _impl_.fields_)>()
          , from.fields_
        }
        ,
        name_(arena, from.name_),
        description_(arena, from.description_),
        default_value_expression_(arena, from.default_value_expression_) {}

TableFieldSchema::TableFieldSchema(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const TableFieldSchema& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, TableFieldSchema_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  TableFieldSchema* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, type_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, type_),
           offsetof(Impl_, scale_) -
               offsetof(Impl_, type_) +
               sizeof(Impl_::scale_));

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.TableFieldSchema)
}
PROTOBUF_NDEBUG_INLINE TableFieldSchema::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        fields_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::cloud::bigquery::storage::v1::TableFieldSchema,
            PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::TableFieldSchema, _impl_.fields_)>()
         }
        ,
        name_(arena),
        description_(arena),
        default_value_expression_(arena) {}

inline void TableFieldSchema::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, type_),
           0,
           offsetof(Impl_, scale_) -
               offsetof(Impl_, type_) +
               sizeof(Impl_::scale_));
}
TableFieldSchema::~TableFieldSchema() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.TableFieldSchema)
  SharedDtor(*this);
}
inline void TableFieldSchema::SharedDtor(MessageLite& self) {
  TableFieldSchema& this_ = static_cast<TableFieldSchema&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.name_.Destroy();
  this_._impl_.description_.Destroy();
  this_._impl_.default_value_expression_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull TableFieldSchema_class_data_ =
        TableFieldSchema::InternalGenerateClassData_(TableFieldSchema_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
TableFieldSchema::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&TableFieldSchema_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(TableFieldSchema_class_data_.tc_table);
  return TableFieldSchema_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
TableFieldSchema::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&TableFieldSchema_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&TableFieldSchema_globals_));
  return TableFieldSchema_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const TableFieldSchema::ParseTableT_
    TableFieldSchema::_table_ =
        TableFieldSchema::InternalGenerateParseTable_(TableFieldSchema_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void TableFieldSchema::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.TableFieldSchema)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.fields_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.name_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.description_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _impl_.default_value_expression_.ClearNonDefaultToEmpty();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x000000f0U)) {
    ::memset(&_impl_.type_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.precision_) -
        reinterpret_cast<char*>(&_impl_.type_)) + sizeof(_impl_.precision_));
  }
  _impl_.scale_ = ::int64_t{0};
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL TableFieldSchema::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const TableFieldSchema& this_ = static_cast<const TableFieldSchema&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL TableFieldSchema::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const TableFieldSchema& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.TableFieldSchema)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string name = 1 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_name().empty()) {
      const ::std::string& _s = this_._internal_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.cloud.bigquery.storage.v1.TableFieldSchema.name");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // .google.cloud.bigquery.storage.v1.TableFieldSchema.Type type = 2 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_type() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          2, this_._internal_type(), target);
    }
  }

  // .google.cloud.bigquery.storage.v1.TableFieldSchema.Mode mode = 3 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_mode() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          3, this_._internal_mode(), target);
    }
  }

  // repeated .google.cloud.bigquery.storage.v1.TableFieldSchema fields = 4 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_fields_size());
         i < n; i++) {
      const auto& repfield = this_._internal_fields().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              4, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  // string description = 6 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_description().empty()) {
      const ::std::string& _s = this_._internal_description();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.cloud.bigquery.storage.v1.TableFieldSchema.description");
      target = stream->WriteStringMaybeAliased(6, _s, target);
    }
  }

  // int64 max_length = 7 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_max_length() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<7>(
              stream, this_._internal_max_length(), target);
    }
  }

  // int64 precision = 8 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    if (this_._internal_precision() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<8>(
              stream, this_._internal_precision(), target);
    }
  }

  // int64 scale = 9 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    if (this_._internal_scale() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<9>(
              stream, this_._internal_scale(), target);
    }
  }

  // string default_value_expression = 10 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (!this_._internal_default_value_expression().empty()) {
      const ::std::string& _s = this_._internal_default_value_expression();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.cloud.bigquery.storage.v1.TableFieldSchema.default_value_expression");
      target = stream->WriteStringMaybeAliased(10, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.TableFieldSchema)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t TableFieldSchema::ByteSizeLong(const MessageLite& base) {
  const TableFieldSchema& this_ = static_cast<const TableFieldSchema&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t TableFieldSchema::ByteSizeLong() const {
  const TableFieldSchema& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.TableFieldSchema)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // repeated .google.cloud.bigquery.storage.v1.TableFieldSchema fields = 4 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_fields_size();
      for (const auto& msg : this_._internal_fields()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
    // string name = 1 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_name());
      }
    }
    // string description = 6 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!this_._internal_description().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_description());
      }
    }
    // string default_value_expression = 10 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!this_._internal_default_value_expression().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_default_value_expression());
      }
    }
    // .google.cloud.bigquery.storage.v1.TableFieldSchema.Type type = 2 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_type() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_type());
      }
    }
    // .google.cloud.bigquery.storage.v1.TableFieldSchema.Mode mode = 3 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_mode() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_mode());
      }
    }
    // int64 max_length = 7 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_max_length() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_max_length());
      }
    }
    // int64 precision = 8 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (this_._internal_precision() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_precision());
      }
    }
  }
   {
    // int64 scale = 9 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      if (this_._internal_scale() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_scale());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void TableFieldSchema::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<TableFieldSchema*>(&to_msg);
  auto& from = static_cast<const TableFieldSchema&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.TableFieldSchema)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_fields()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_fields());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_name().empty()) {
        _this->_internal_set_name(from._internal_name());
      } else {
        if (_this->_impl_.name_.IsDefault()) {
          _this->_internal_set_name("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!from._internal_description().empty()) {
        _this->_internal_set_description(from._internal_description());
      } else {
        if (_this->_impl_.description_.IsDefault()) {
          _this->_internal_set_description("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!from._internal_default_value_expression().empty()) {
        _this->_internal_set_default_value_expression(from._internal_default_value_expression());
      } else {
        if (_this->_impl_.default_value_expression_.IsDefault()) {
          _this->_internal_set_default_value_expression("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_type() != 0) {
        _this->_impl_.type_ = from._impl_.type_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_mode() != 0) {
        _this->_impl_.mode_ = from._impl_.mode_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_max_length() != 0) {
        _this->_impl_.max_length_ = from._impl_.max_length_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (from._internal_precision() != 0) {
        _this->_impl_.precision_ = from._impl_.precision_;
      }
    }
  }
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    if (from._internal_scale() != 0) {
      _this->_impl_.scale_ = from._impl_.scale_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void TableFieldSchema::CopyFrom(const TableFieldSchema& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.TableFieldSchema)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void TableFieldSchema::InternalSwap(TableFieldSchema* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.fields_.InternalSwap(&other->_impl_.fields_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.name_, &other->_impl_.name_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.description_, &other->_impl_.description_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.default_value_expression_, &other->_impl_.default_value_expression_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_.scale_)
      + sizeof(TableFieldSchema::_impl_.scale_)
      - PROTOBUF_FIELD_OFFSET(TableFieldSchema, _impl_.type_)>(
          reinterpret_cast<char*>(&_impl_.type_),
          reinterpret_cast<char*>(&other->_impl_.type_));
}

::google::protobuf::Metadata TableFieldSchema::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
}  // namespace v1
}  // namespace storage
}  // namespace bigquery
}  // namespace cloud
}  // namespace google
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
