// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: opentelemetry/proto/collector/trace/v1/trace_service.proto
// Protobuf C++ Version: 7.35.1

#ifndef opentelemetry_2fproto_2fcollector_2ftrace_2fv1_2ftrace_5fservice_2eproto_2epb_2eh
#define opentelemetry_2fproto_2fcollector_2ftrace_2fv1_2ftrace_5fservice_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7035001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/unknown_field_set.h"
#include "opentelemetry/proto/trace/v1/trace.pb.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_opentelemetry_2fproto_2fcollector_2ftrace_2fv1_2ftrace_5fservice_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_opentelemetry_2fproto_2fcollector_2ftrace_2fv1_2ftrace_5fservice_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_opentelemetry_2fproto_2fcollector_2ftrace_2fv1_2ftrace_5fservice_2eproto;
}  // extern "C"
namespace opentelemetry {
namespace proto {
namespace collector {
namespace trace {
namespace v1 {
class ExportTracePartialSuccess;
struct ExportTracePartialSuccessGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ExportTracePartialSuccessGlobalsTypeInternal ExportTracePartialSuccess_globals_;
extern const ::google::protobuf::internal::ClassDataFull ExportTracePartialSuccess_class_data_;
#else
extern const ExportTracePartialSuccessGlobalsTypeInternal ExportTracePartialSuccess_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ExportTraceServiceRequest;
struct ExportTraceServiceRequestGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ExportTraceServiceRequestGlobalsTypeInternal ExportTraceServiceRequest_globals_;
extern const ::google::protobuf::internal::ClassDataFull ExportTraceServiceRequest_class_data_;
#else
extern const ExportTraceServiceRequestGlobalsTypeInternal ExportTraceServiceRequest_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ExportTraceServiceResponse;
struct ExportTraceServiceResponseGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ExportTraceServiceResponseGlobalsTypeInternal ExportTraceServiceResponse_globals_;
extern const ::google::protobuf::internal::ClassDataFull ExportTraceServiceResponse_class_data_;
#else
extern const ExportTraceServiceResponseGlobalsTypeInternal ExportTraceServiceResponse_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace v1
}  // namespace trace
}  // namespace collector
}  // namespace proto
}  // namespace opentelemetry
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace opentelemetry {
namespace proto {
namespace collector {
namespace trace {
namespace v1 {

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ExportTracePartialSuccess final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.collector.trace.v1.ExportTracePartialSuccess) */ {
 public:
  inline ExportTracePartialSuccess() : ExportTracePartialSuccess(nullptr) {}
  ~ExportTracePartialSuccess() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ExportTracePartialSuccess* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ExportTracePartialSuccess));
  }
#endif

  template <typename = void>
  explicit constexpr ExportTracePartialSuccess(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ExportTracePartialSuccess(const ExportTracePartialSuccess& from) : ExportTracePartialSuccess(nullptr, from) {}
  inline ExportTracePartialSuccess(ExportTracePartialSuccess&& from) noexcept : ExportTracePartialSuccess(nullptr, ::std::move(from)) {}
  inline ExportTracePartialSuccess& operator=(const ExportTracePartialSuccess& from) {
    CopyFrom(from);
    return *this;
  }
  inline ExportTracePartialSuccess& operator=(ExportTracePartialSuccess&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ExportTracePartialSuccess& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ExportTracePartialSuccess>(&ExportTracePartialSuccess_globals_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(ExportTracePartialSuccess& a, ExportTracePartialSuccess& b) { a.Swap(&b); }
  inline void Swap(ExportTracePartialSuccess* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ExportTracePartialSuccess* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ExportTracePartialSuccess* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ExportTracePartialSuccess>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ExportTracePartialSuccess& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ExportTracePartialSuccess& from) { ExportTracePartialSuccess::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ExportTracePartialSuccess* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.collector.trace.v1.ExportTracePartialSuccess"; }

  explicit ExportTracePartialSuccess(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ExportTracePartialSuccess(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ExportTracePartialSuccess& from);
  ExportTracePartialSuccess(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ExportTracePartialSuccess&& from) noexcept
      : ExportTracePartialSuccess(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kErrorMessageFieldNumber = 2,
    kRejectedSpansFieldNumber = 1,
  };
  // string error_message = 2;
  void clear_error_message() ;
  [[nodiscard]] const ::std::string& error_message() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_error_message(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_error_message();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_error_message();
  void set_allocated_error_message(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_error_message() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_error_message(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_error_message();

  public:
  // int64 rejected_spans = 1;
  void clear_rejected_spans() ;
  [[nodiscard]] ::int64_t rejected_spans() const;
  void set_rejected_spans(::int64_t value);

  private:
  ::int64_t _internal_rejected_spans() const;
  void _internal_set_rejected_spans(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.collector.trace.v1.ExportTracePartialSuccess)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          0, 86,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ExportTracePartialSuccess& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr error_message_;
    ::int64_t rejected_spans_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fcollector_2ftrace_2fv1_2ftrace_5fservice_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ExportTraceServiceResponse final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.collector.trace.v1.ExportTraceServiceResponse) */ {
 public:
  inline ExportTraceServiceResponse() : ExportTraceServiceResponse(nullptr) {}
  ~ExportTraceServiceResponse() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ExportTraceServiceResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ExportTraceServiceResponse));
  }
#endif

  template <typename = void>
  explicit constexpr ExportTraceServiceResponse(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ExportTraceServiceResponse(const ExportTraceServiceResponse& from) : ExportTraceServiceResponse(nullptr, from) {}
  inline ExportTraceServiceResponse(ExportTraceServiceResponse&& from) noexcept : ExportTraceServiceResponse(nullptr, ::std::move(from)) {}
  inline ExportTraceServiceResponse& operator=(const ExportTraceServiceResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline ExportTraceServiceResponse& operator=(ExportTraceServiceResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ExportTraceServiceResponse& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ExportTraceServiceResponse>(&ExportTraceServiceResponse_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(ExportTraceServiceResponse& a, ExportTraceServiceResponse& b) { a.Swap(&b); }
  inline void Swap(ExportTraceServiceResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ExportTraceServiceResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ExportTraceServiceResponse* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ExportTraceServiceResponse>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ExportTraceServiceResponse& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ExportTraceServiceResponse& from) { ExportTraceServiceResponse::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ExportTraceServiceResponse* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.collector.trace.v1.ExportTraceServiceResponse"; }

  explicit ExportTraceServiceResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ExportTraceServiceResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ExportTraceServiceResponse& from);
  ExportTraceServiceResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ExportTraceServiceResponse&& from) noexcept
      : ExportTraceServiceResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kPartialSuccessFieldNumber = 1,
  };
  // .opentelemetry.proto.collector.trace.v1.ExportTracePartialSuccess partial_success = 1;
  [[nodiscard]] bool has_partial_success()
      const;
  void clear_partial_success() ;
  [[nodiscard]] const ::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess& partial_success() const;
  [[nodiscard]] ::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess* PROTOBUF_NULLABLE release_partial_success();
  ::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess* PROTOBUF_NONNULL mutable_partial_success();
  void set_allocated_partial_success(::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_partial_success(::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess* PROTOBUF_NULLABLE value);
  ::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess* PROTOBUF_NULLABLE unsafe_arena_release_partial_success();

  private:
  const ::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess& _internal_partial_success() const;
  ::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess* PROTOBUF_NONNULL _internal_mutable_partial_success();

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.collector.trace.v1.ExportTraceServiceResponse)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ExportTraceServiceResponse& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess* PROTOBUF_NULLABLE partial_success_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fcollector_2ftrace_2fv1_2ftrace_5fservice_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ExportTraceServiceRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.collector.trace.v1.ExportTraceServiceRequest) */ {
 public:
  inline ExportTraceServiceRequest() : ExportTraceServiceRequest(nullptr) {}
  ~ExportTraceServiceRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ExportTraceServiceRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ExportTraceServiceRequest));
  }
#endif

  template <typename = void>
  explicit constexpr ExportTraceServiceRequest(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ExportTraceServiceRequest(const ExportTraceServiceRequest& from) : ExportTraceServiceRequest(nullptr, from) {}
  inline ExportTraceServiceRequest(ExportTraceServiceRequest&& from) noexcept : ExportTraceServiceRequest(nullptr, ::std::move(from)) {}
  inline ExportTraceServiceRequest& operator=(const ExportTraceServiceRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline ExportTraceServiceRequest& operator=(ExportTraceServiceRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ExportTraceServiceRequest& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ExportTraceServiceRequest>(&ExportTraceServiceRequest_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(ExportTraceServiceRequest& a, ExportTraceServiceRequest& b) { a.Swap(&b); }
  inline void Swap(ExportTraceServiceRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ExportTraceServiceRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ExportTraceServiceRequest* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ExportTraceServiceRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ExportTraceServiceRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ExportTraceServiceRequest& from) { ExportTraceServiceRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ExportTraceServiceRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.collector.trace.v1.ExportTraceServiceRequest"; }

  explicit ExportTraceServiceRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ExportTraceServiceRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ExportTraceServiceRequest& from);
  ExportTraceServiceRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ExportTraceServiceRequest&& from) noexcept
      : ExportTraceServiceRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kResourceSpansFieldNumber = 1,
  };
  // repeated .opentelemetry.proto.trace.v1.ResourceSpans resource_spans = 1;
  [[nodiscard]] int resource_spans_size()
      const;
  private:
  int _internal_resource_spans_size() const;

  public:
  void clear_resource_spans() ;
  [[nodiscard]] const ::opentelemetry::proto::trace::v1::ResourceSpans& resource_spans(int index) const;
  [[nodiscard]] ::opentelemetry::proto::trace::v1::ResourceSpans* PROTOBUF_NONNULL mutable_resource_spans(int index);
  ::opentelemetry::proto::trace::v1::ResourceSpans* PROTOBUF_NONNULL add_resource_spans();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::trace::v1::ResourceSpans>&
  resource_spans() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::trace::v1::ResourceSpans>* PROTOBUF_NONNULL
  mutable_resource_spans();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::trace::v1::ResourceSpans>& _internal_resource_spans() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::trace::v1::ResourceSpans>* PROTOBUF_NONNULL _internal_mutable_resource_spans();

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.collector.trace.v1.ExportTraceServiceRequest)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ExportTraceServiceRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::trace::v1::ResourceSpans > resource_spans_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fcollector_2ftrace_2fv1_2ftrace_5fservice_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// ExportTraceServiceRequest

// repeated .opentelemetry.proto.trace.v1.ResourceSpans resource_spans = 1;
inline int ExportTraceServiceRequest::_internal_resource_spans_size() const {
  return _internal_resource_spans().size();
}
inline int ExportTraceServiceRequest::resource_spans_size() const {
  return _internal_resource_spans_size();
}
inline const ::opentelemetry::proto::trace::v1::ResourceSpans& ExportTraceServiceRequest::resource_spans(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.collector.trace.v1.ExportTraceServiceRequest.resource_spans)
  return _internal_resource_spans().Get(index);
}
inline ::opentelemetry::proto::trace::v1::ResourceSpans* PROTOBUF_NONNULL ExportTraceServiceRequest::mutable_resource_spans(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.collector.trace.v1.ExportTraceServiceRequest.resource_spans)
  return _internal_mutable_resource_spans()->Mutable(index);
}
inline ::opentelemetry::proto::trace::v1::ResourceSpans* PROTOBUF_NONNULL ExportTraceServiceRequest::add_resource_spans()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::trace::v1::ResourceSpans* _add =
      _internal_mutable_resource_spans()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.collector.trace.v1.ExportTraceServiceRequest.resource_spans)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::trace::v1::ResourceSpans>& ExportTraceServiceRequest::resource_spans() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.collector.trace.v1.ExportTraceServiceRequest.resource_spans)
  return _internal_resource_spans();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::trace::v1::ResourceSpans>* PROTOBUF_NONNULL
ExportTraceServiceRequest::mutable_resource_spans() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.collector.trace.v1.ExportTraceServiceRequest.resource_spans)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_resource_spans();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::trace::v1::ResourceSpans>&
ExportTraceServiceRequest::_internal_resource_spans() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.resource_spans_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::trace::v1::ResourceSpans>* PROTOBUF_NONNULL
ExportTraceServiceRequest::_internal_mutable_resource_spans() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.resource_spans_;
}

// -------------------------------------------------------------------

// ExportTraceServiceResponse

// .opentelemetry.proto.collector.trace.v1.ExportTracePartialSuccess partial_success = 1;
inline bool ExportTraceServiceResponse::has_partial_success() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.partial_success_ != nullptr);
  return value;
}
inline void ExportTraceServiceResponse::clear_partial_success() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.partial_success_ != nullptr) _impl_.partial_success_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess& ExportTraceServiceResponse::_internal_partial_success() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess* p = _impl_.partial_success_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess>(&::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess_globals_);
}
inline const ::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess& ExportTraceServiceResponse::partial_success() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.collector.trace.v1.ExportTraceServiceResponse.partial_success)
  return _internal_partial_success();
}
inline void ExportTraceServiceResponse::unsafe_arena_set_allocated_partial_success(
    ::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.partial_success_);
  }
  _impl_.partial_success_ = reinterpret_cast<::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:opentelemetry.proto.collector.trace.v1.ExportTraceServiceResponse.partial_success)
}
inline ::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess* PROTOBUF_NULLABLE ExportTraceServiceResponse::release_partial_success() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess* released = _impl_.partial_success_;
  _impl_.partial_success_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess* PROTOBUF_NULLABLE ExportTraceServiceResponse::unsafe_arena_release_partial_success() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.collector.trace.v1.ExportTraceServiceResponse.partial_success)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess* temp = _impl_.partial_success_;
  _impl_.partial_success_ = nullptr;
  return temp;
}
inline ::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess* PROTOBUF_NONNULL ExportTraceServiceResponse::_internal_mutable_partial_success() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.partial_success_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess>(GetArena());
    _impl_.partial_success_ = reinterpret_cast<::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess*>(p);
  }
  return _impl_.partial_success_;
}
inline ::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess* PROTOBUF_NONNULL ExportTraceServiceResponse::mutable_partial_success()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess* _msg = _internal_mutable_partial_success();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.collector.trace.v1.ExportTraceServiceResponse.partial_success)
  return _msg;
}
inline void ExportTraceServiceResponse::set_allocated_partial_success(::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.partial_success_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.partial_success_ = reinterpret_cast<::opentelemetry::proto::collector::trace::v1::ExportTracePartialSuccess*>(value);
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.collector.trace.v1.ExportTraceServiceResponse.partial_success)
}

// -------------------------------------------------------------------

// ExportTracePartialSuccess

// int64 rejected_spans = 1;
inline void ExportTracePartialSuccess::clear_rejected_spans() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.rejected_spans_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::int64_t ExportTracePartialSuccess::rejected_spans() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.collector.trace.v1.ExportTracePartialSuccess.rejected_spans)
  return _internal_rejected_spans();
}
inline void ExportTracePartialSuccess::set_rejected_spans(::int64_t value) {
  _internal_set_rejected_spans(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.collector.trace.v1.ExportTracePartialSuccess.rejected_spans)
}
inline ::int64_t ExportTracePartialSuccess::_internal_rejected_spans() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.rejected_spans_;
}
inline void ExportTracePartialSuccess::_internal_set_rejected_spans(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.rejected_spans_ = value;
}

// string error_message = 2;
inline void ExportTracePartialSuccess::clear_error_message() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.error_message_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& ExportTracePartialSuccess::error_message() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.collector.trace.v1.ExportTracePartialSuccess.error_message)
  return _internal_error_message();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ExportTracePartialSuccess::set_error_message(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.error_message_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:opentelemetry.proto.collector.trace.v1.ExportTracePartialSuccess.error_message)
}
inline ::std::string* PROTOBUF_NONNULL ExportTracePartialSuccess::mutable_error_message()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_error_message();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.collector.trace.v1.ExportTracePartialSuccess.error_message)
  return _s;
}
inline const ::std::string& ExportTracePartialSuccess::_internal_error_message() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.error_message_.Get();
}
inline void ExportTracePartialSuccess::_internal_set_error_message(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.error_message_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ExportTracePartialSuccess::_internal_mutable_error_message() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.error_message_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ExportTracePartialSuccess::release_error_message() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.collector.trace.v1.ExportTracePartialSuccess.error_message)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.error_message_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.error_message_.Set("", GetArena());
  }
  return released;
}
inline void ExportTracePartialSuccess::set_allocated_error_message(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.error_message_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.error_message_.IsDefault()) {
    _impl_.error_message_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.collector.trace.v1.ExportTracePartialSuccess.error_message)
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace v1
}  // namespace trace
}  // namespace collector
}  // namespace proto
}  // namespace opentelemetry


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // opentelemetry_2fproto_2fcollector_2ftrace_2fv1_2ftrace_5fservice_2eproto_2epb_2eh
