// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: google/api/launch_stage.proto
// Protobuf C++ Version: 7.35.1

#include "google/api/launch_stage.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
namespace google {
namespace api {
}  // namespace api
}  // namespace google
static const ::_pb::EnumDescriptor* PROTOBUF_NONNULL
    file_level_enum_descriptors_google_2fapi_2flaunch_5fstage_2eproto[1];
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_google_2fapi_2flaunch_5fstage_2eproto = nullptr;
const ::uint32_t TableStruct_google_2fapi_2flaunch_5fstage_2eproto::offsets[1] = {};
static constexpr ::_pbi::MigrationSchema* PROTOBUF_NULLABLE schemas = nullptr;
static constexpr ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const* PROTOBUF_NULLABLE
    file_message_globals = nullptr;
const char descriptor_table_protodef_google_2fapi_2flaunch_5fstage_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n\035google/api/launch_stage.proto\022\ngoogle."
    "api*\214\001\n\013LaunchStage\022\034\n\030LAUNCH_STAGE_UNSP"
    "ECIFIED\020\000\022\021\n\rUNIMPLEMENTED\020\006\022\r\n\tPRELAUNC"
    "H\020\007\022\020\n\014EARLY_ACCESS\020\001\022\t\n\005ALPHA\020\002\022\010\n\004BETA"
    "\020\003\022\006\n\002GA\020\004\022\016\n\nDEPRECATED\020\005BZ\n\016com.google"
    ".apiB\020LaunchStageProtoP\001Z-google.golang."
    "org/genproto/googleapis/api;api\242\002\004GAPIb\006"
    "proto3"
};
static ::absl::once_flag descriptor_table_google_2fapi_2flaunch_5fstage_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_google_2fapi_2flaunch_5fstage_2eproto = {
    false,
    false,
    286,
    descriptor_table_protodef_google_2fapi_2flaunch_5fstage_2eproto,
    "google/api/launch_stage.proto",
    &descriptor_table_google_2fapi_2flaunch_5fstage_2eproto_once,
    nullptr,
    0,
    0,
    schemas,
    file_message_globals,
    TableStruct_google_2fapi_2flaunch_5fstage_2eproto::offsets,
    file_level_enum_descriptors_google_2fapi_2flaunch_5fstage_2eproto,
    file_level_service_descriptors_google_2fapi_2flaunch_5fstage_2eproto,
};
namespace google {
namespace api {
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
LaunchStage_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fapi_2flaunch_5fstage_2eproto);
  return file_level_enum_descriptors_google_2fapi_2flaunch_5fstage_2eproto[0];
}
PROTOBUF_CONSTINIT const uint32_t LaunchStage_internal_data_[] = {
    524288u, 0u, };
// @@protoc_insertion_point(namespace_scope)
}  // namespace api
}  // namespace google
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_google_2fapi_2flaunch_5fstage_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
