// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: push.proto
// Protobuf C++ Version: 7.35.1

#ifndef push_2eproto_2epb_2eh
#define push_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7035001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_bases.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/unknown_field_set.h"
#include "google/protobuf/timestamp.pb.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_push_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_push_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_push_2eproto;
}  // extern "C"
namespace logproto {
class EntryAdapter;
struct EntryAdapterGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern EntryAdapterGlobalsTypeInternal EntryAdapter_globals_;
extern const ::google::protobuf::internal::ClassDataFull EntryAdapter_class_data_;
#else
extern const EntryAdapterGlobalsTypeInternal EntryAdapter_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class LabelPairAdapter;
struct LabelPairAdapterGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern LabelPairAdapterGlobalsTypeInternal LabelPairAdapter_globals_;
extern const ::google::protobuf::internal::ClassDataFull LabelPairAdapter_class_data_;
#else
extern const LabelPairAdapterGlobalsTypeInternal LabelPairAdapter_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class PushRequest;
struct PushRequestGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern PushRequestGlobalsTypeInternal PushRequest_globals_;
extern const ::google::protobuf::internal::ClassDataFull PushRequest_class_data_;
#else
extern const PushRequestGlobalsTypeInternal PushRequest_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class PushResponse;
struct PushResponseGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern PushResponseGlobalsTypeInternal PushResponse_globals_;
extern const ::google::protobuf::internal::ClassDataFull PushResponse_class_data_;
#else
extern const PushResponseGlobalsTypeInternal PushResponse_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class StreamAdapter;
struct StreamAdapterGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern StreamAdapterGlobalsTypeInternal StreamAdapter_globals_;
extern const ::google::protobuf::internal::ClassDataFull StreamAdapter_class_data_;
#else
extern const StreamAdapterGlobalsTypeInternal StreamAdapter_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace logproto
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace logproto {

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED PushResponse final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:logproto.PushResponse) */ {
 public:
  inline PushResponse() : PushResponse(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PushResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PushResponse));
  }
#endif

  template <typename = void>
  explicit constexpr PushResponse(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline PushResponse(const PushResponse& from) : PushResponse(nullptr, from) {}
  inline PushResponse(PushResponse&& from) noexcept : PushResponse(nullptr, ::std::move(from)) {}
  inline PushResponse& operator=(const PushResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline PushResponse& operator=(PushResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const PushResponse& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<PushResponse>(&PushResponse_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(PushResponse& a, PushResponse& b) { a.Swap(&b); }
  inline void Swap(PushResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PushResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] PushResponse* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<PushResponse>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const PushResponse& from) { ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from); }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const PushResponse& from) { ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from); }

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "logproto.PushResponse"; }

  explicit PushResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PushResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PushResponse& from);
  PushResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PushResponse&& from) noexcept
      : PushResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  // @@protoc_insertion_point(class_scope:logproto.PushResponse)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 0,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_push_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED LabelPairAdapter final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:logproto.LabelPairAdapter) */ {
 public:
  inline LabelPairAdapter() : LabelPairAdapter(nullptr) {}
  ~LabelPairAdapter() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(LabelPairAdapter* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(LabelPairAdapter));
  }
#endif

  template <typename = void>
  explicit constexpr LabelPairAdapter(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline LabelPairAdapter(const LabelPairAdapter& from) : LabelPairAdapter(nullptr, from) {}
  inline LabelPairAdapter(LabelPairAdapter&& from) noexcept : LabelPairAdapter(nullptr, ::std::move(from)) {}
  inline LabelPairAdapter& operator=(const LabelPairAdapter& from) {
    CopyFrom(from);
    return *this;
  }
  inline LabelPairAdapter& operator=(LabelPairAdapter&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const LabelPairAdapter& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<LabelPairAdapter>(&LabelPairAdapter_globals_);
  }
  static constexpr int kIndexInFileMessages = 3;
  friend void swap(LabelPairAdapter& a, LabelPairAdapter& b) { a.Swap(&b); }
  inline void Swap(LabelPairAdapter* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(LabelPairAdapter* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] LabelPairAdapter* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<LabelPairAdapter>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const LabelPairAdapter& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const LabelPairAdapter& from) { LabelPairAdapter::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(LabelPairAdapter* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "logproto.LabelPairAdapter"; }

  explicit LabelPairAdapter(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  LabelPairAdapter(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const LabelPairAdapter& from);
  LabelPairAdapter(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, LabelPairAdapter&& from) noexcept
      : LabelPairAdapter(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kNameFieldNumber = 1,
    kValueFieldNumber = 2,
  };
  // string name = 1;
  void clear_name() ;
  [[nodiscard]] const ::std::string& name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_name();
  void set_allocated_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_name();

  public:
  // string value = 2;
  void clear_value() ;
  [[nodiscard]] const ::std::string& value() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_value(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_value();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_value();
  void set_allocated_value(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_value() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_value(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_value();

  public:
  // @@protoc_insertion_point(class_scope:logproto.LabelPairAdapter)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          0, 43,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const LabelPairAdapter& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr name_;
    ::google::protobuf::internal::ArenaStringPtr value_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_push_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED EntryAdapter final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:logproto.EntryAdapter) */ {
 public:
  inline EntryAdapter() : EntryAdapter(nullptr) {}
  ~EntryAdapter() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(EntryAdapter* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(EntryAdapter));
  }
#endif

  template <typename = void>
  explicit constexpr EntryAdapter(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline EntryAdapter(const EntryAdapter& from) : EntryAdapter(nullptr, from) {}
  inline EntryAdapter(EntryAdapter&& from) noexcept : EntryAdapter(nullptr, ::std::move(from)) {}
  inline EntryAdapter& operator=(const EntryAdapter& from) {
    CopyFrom(from);
    return *this;
  }
  inline EntryAdapter& operator=(EntryAdapter&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const EntryAdapter& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<EntryAdapter>(&EntryAdapter_globals_);
  }
  static constexpr int kIndexInFileMessages = 4;
  friend void swap(EntryAdapter& a, EntryAdapter& b) { a.Swap(&b); }
  inline void Swap(EntryAdapter* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(EntryAdapter* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] EntryAdapter* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<EntryAdapter>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const EntryAdapter& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const EntryAdapter& from) { EntryAdapter::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(EntryAdapter* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "logproto.EntryAdapter"; }

  explicit EntryAdapter(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  EntryAdapter(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const EntryAdapter& from);
  EntryAdapter(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, EntryAdapter&& from) noexcept
      : EntryAdapter(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kNonIndexedLabelsFieldNumber = 3,
    kLineFieldNumber = 2,
    kTimestampFieldNumber = 1,
  };
  // repeated .logproto.LabelPairAdapter nonIndexedLabels = 3;
  [[nodiscard]] int nonindexedlabels_size()
      const;
  private:
  int _internal_nonindexedlabels_size() const;

  public:
  void clear_nonindexedlabels() ;
  [[nodiscard]] const ::logproto::LabelPairAdapter& nonindexedlabels(int index) const;
  [[nodiscard]] ::logproto::LabelPairAdapter* PROTOBUF_NONNULL mutable_nonindexedlabels(int index);
  ::logproto::LabelPairAdapter* PROTOBUF_NONNULL add_nonindexedlabels();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::logproto::LabelPairAdapter>&
  nonindexedlabels() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::logproto::LabelPairAdapter>* PROTOBUF_NONNULL
  mutable_nonindexedlabels();

  private:
  const ::google::protobuf::RepeatedPtrField<::logproto::LabelPairAdapter>& _internal_nonindexedlabels() const;
  ::google::protobuf::RepeatedPtrField<::logproto::LabelPairAdapter>* PROTOBUF_NONNULL _internal_mutable_nonindexedlabels();

  public:
  // string line = 2;
  void clear_line() ;
  [[nodiscard]] const ::std::string& line() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_line(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_line();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_line();
  void set_allocated_line(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_line() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_line(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_line();

  public:
  // .google.protobuf.Timestamp timestamp = 1;
  [[nodiscard]] bool has_timestamp()
      const;
  void clear_timestamp() ;
  [[nodiscard]] const ::google::protobuf::Timestamp& timestamp() const;
  [[nodiscard]] ::google::protobuf::Timestamp* PROTOBUF_NULLABLE release_timestamp();
  ::google::protobuf::Timestamp* PROTOBUF_NONNULL mutable_timestamp();
  void set_allocated_timestamp(::google::protobuf::Timestamp* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_timestamp(::google::protobuf::Timestamp* PROTOBUF_NULLABLE value);
  ::google::protobuf::Timestamp* PROTOBUF_NULLABLE unsafe_arena_release_timestamp();

  private:
  const ::google::protobuf::Timestamp& _internal_timestamp() const;
  ::google::protobuf::Timestamp* PROTOBUF_NONNULL _internal_mutable_timestamp();

  public:
  // @@protoc_insertion_point(class_scope:logproto.EntryAdapter)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 3,
                          2, 34,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const EntryAdapter& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::logproto::LabelPairAdapter > nonindexedlabels_;
    ::google::protobuf::internal::ArenaStringPtr line_;
    ::google::protobuf::Timestamp* PROTOBUF_NULLABLE timestamp_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_push_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED StreamAdapter final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:logproto.StreamAdapter) */ {
 public:
  inline StreamAdapter() : StreamAdapter(nullptr) {}
  ~StreamAdapter() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(StreamAdapter* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(StreamAdapter));
  }
#endif

  template <typename = void>
  explicit constexpr StreamAdapter(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline StreamAdapter(const StreamAdapter& from) : StreamAdapter(nullptr, from) {}
  inline StreamAdapter(StreamAdapter&& from) noexcept : StreamAdapter(nullptr, ::std::move(from)) {}
  inline StreamAdapter& operator=(const StreamAdapter& from) {
    CopyFrom(from);
    return *this;
  }
  inline StreamAdapter& operator=(StreamAdapter&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const StreamAdapter& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<StreamAdapter>(&StreamAdapter_globals_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(StreamAdapter& a, StreamAdapter& b) { a.Swap(&b); }
  inline void Swap(StreamAdapter* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(StreamAdapter* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] StreamAdapter* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<StreamAdapter>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const StreamAdapter& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const StreamAdapter& from) { StreamAdapter::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(StreamAdapter* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "logproto.StreamAdapter"; }

  explicit StreamAdapter(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  StreamAdapter(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const StreamAdapter& from);
  StreamAdapter(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, StreamAdapter&& from) noexcept
      : StreamAdapter(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kEntriesFieldNumber = 2,
    kLabelsFieldNumber = 1,
    kHashFieldNumber = 3,
  };
  // repeated .logproto.EntryAdapter entries = 2;
  [[nodiscard]] int entries_size()
      const;
  private:
  int _internal_entries_size() const;

  public:
  void clear_entries() ;
  [[nodiscard]] const ::logproto::EntryAdapter& entries(int index) const;
  [[nodiscard]] ::logproto::EntryAdapter* PROTOBUF_NONNULL mutable_entries(int index);
  ::logproto::EntryAdapter* PROTOBUF_NONNULL add_entries();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::logproto::EntryAdapter>&
  entries() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::logproto::EntryAdapter>* PROTOBUF_NONNULL
  mutable_entries();

  private:
  const ::google::protobuf::RepeatedPtrField<::logproto::EntryAdapter>& _internal_entries() const;
  ::google::protobuf::RepeatedPtrField<::logproto::EntryAdapter>* PROTOBUF_NONNULL _internal_mutable_entries();

  public:
  // string labels = 1;
  void clear_labels() ;
  [[nodiscard]] const ::std::string& labels() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_labels(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_labels();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_labels();
  void set_allocated_labels(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_labels() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_labels(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_labels();

  public:
  // uint64 hash = 3;
  void clear_hash() ;
  [[nodiscard]] ::uint64_t hash() const;
  void set_hash(::uint64_t value);

  private:
  ::uint64_t _internal_hash() const;
  void _internal_set_hash(::uint64_t value);

  public:
  // @@protoc_insertion_point(class_scope:logproto.StreamAdapter)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 3,
                          1, 37,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const StreamAdapter& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::logproto::EntryAdapter > entries_;
    ::google::protobuf::internal::ArenaStringPtr labels_;
    ::uint64_t hash_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_push_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED PushRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:logproto.PushRequest) */ {
 public:
  inline PushRequest() : PushRequest(nullptr) {}
  ~PushRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PushRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PushRequest));
  }
#endif

  template <typename = void>
  explicit constexpr PushRequest(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline PushRequest(const PushRequest& from) : PushRequest(nullptr, from) {}
  inline PushRequest(PushRequest&& from) noexcept : PushRequest(nullptr, ::std::move(from)) {}
  inline PushRequest& operator=(const PushRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline PushRequest& operator=(PushRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const PushRequest& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<PushRequest>(&PushRequest_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(PushRequest& a, PushRequest& b) { a.Swap(&b); }
  inline void Swap(PushRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PushRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] PushRequest* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<PushRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const PushRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const PushRequest& from) { PushRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(PushRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "logproto.PushRequest"; }

  explicit PushRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PushRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PushRequest& from);
  PushRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PushRequest&& from) noexcept
      : PushRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kStreamsFieldNumber = 1,
  };
  // repeated .logproto.StreamAdapter streams = 1;
  [[nodiscard]] int streams_size()
      const;
  private:
  int _internal_streams_size() const;

  public:
  void clear_streams() ;
  [[nodiscard]] const ::logproto::StreamAdapter& streams(int index) const;
  [[nodiscard]] ::logproto::StreamAdapter* PROTOBUF_NONNULL mutable_streams(int index);
  ::logproto::StreamAdapter* PROTOBUF_NONNULL add_streams();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::logproto::StreamAdapter>&
  streams() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::logproto::StreamAdapter>* PROTOBUF_NONNULL
  mutable_streams();

  private:
  const ::google::protobuf::RepeatedPtrField<::logproto::StreamAdapter>& _internal_streams() const;
  ::google::protobuf::RepeatedPtrField<::logproto::StreamAdapter>* PROTOBUF_NONNULL _internal_mutable_streams();

  public:
  // @@protoc_insertion_point(class_scope:logproto.PushRequest)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const PushRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::logproto::StreamAdapter > streams_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_push_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// PushRequest

// repeated .logproto.StreamAdapter streams = 1;
inline int PushRequest::_internal_streams_size() const {
  return _internal_streams().size();
}
inline int PushRequest::streams_size() const {
  return _internal_streams_size();
}
inline void PushRequest::clear_streams() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.streams_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::logproto::StreamAdapter& PushRequest::streams(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:logproto.PushRequest.streams)
  return _internal_streams().Get(index);
}
inline ::logproto::StreamAdapter* PROTOBUF_NONNULL PushRequest::mutable_streams(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:logproto.PushRequest.streams)
  return _internal_mutable_streams()->Mutable(index);
}
inline ::logproto::StreamAdapter* PROTOBUF_NONNULL PushRequest::add_streams()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::logproto::StreamAdapter* _add =
      _internal_mutable_streams()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:logproto.PushRequest.streams)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::logproto::StreamAdapter>& PushRequest::streams() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:logproto.PushRequest.streams)
  return _internal_streams();
}
inline ::google::protobuf::RepeatedPtrField<::logproto::StreamAdapter>* PROTOBUF_NONNULL
PushRequest::mutable_streams() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:logproto.PushRequest.streams)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_streams();
}
inline const ::google::protobuf::RepeatedPtrField<::logproto::StreamAdapter>&
PushRequest::_internal_streams() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.streams_;
}
inline ::google::protobuf::RepeatedPtrField<::logproto::StreamAdapter>* PROTOBUF_NONNULL
PushRequest::_internal_mutable_streams() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.streams_;
}

// -------------------------------------------------------------------

// PushResponse

// -------------------------------------------------------------------

// StreamAdapter

// string labels = 1;
inline void StreamAdapter::clear_labels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.labels_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& StreamAdapter::labels() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:logproto.StreamAdapter.labels)
  return _internal_labels();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void StreamAdapter::set_labels(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.labels_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:logproto.StreamAdapter.labels)
}
inline ::std::string* PROTOBUF_NONNULL StreamAdapter::mutable_labels()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_labels();
  // @@protoc_insertion_point(field_mutable:logproto.StreamAdapter.labels)
  return _s;
}
inline const ::std::string& StreamAdapter::_internal_labels() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.labels_.Get();
}
inline void StreamAdapter::_internal_set_labels(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.labels_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL StreamAdapter::_internal_mutable_labels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.labels_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE StreamAdapter::release_labels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:logproto.StreamAdapter.labels)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.labels_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.labels_.Set("", GetArena());
  }
  return released;
}
inline void StreamAdapter::set_allocated_labels(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.labels_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.labels_.IsDefault()) {
    _impl_.labels_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:logproto.StreamAdapter.labels)
}

// repeated .logproto.EntryAdapter entries = 2;
inline int StreamAdapter::_internal_entries_size() const {
  return _internal_entries().size();
}
inline int StreamAdapter::entries_size() const {
  return _internal_entries_size();
}
inline void StreamAdapter::clear_entries() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.entries_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::logproto::EntryAdapter& StreamAdapter::entries(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:logproto.StreamAdapter.entries)
  return _internal_entries().Get(index);
}
inline ::logproto::EntryAdapter* PROTOBUF_NONNULL StreamAdapter::mutable_entries(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:logproto.StreamAdapter.entries)
  return _internal_mutable_entries()->Mutable(index);
}
inline ::logproto::EntryAdapter* PROTOBUF_NONNULL StreamAdapter::add_entries()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::logproto::EntryAdapter* _add =
      _internal_mutable_entries()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:logproto.StreamAdapter.entries)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::logproto::EntryAdapter>& StreamAdapter::entries() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:logproto.StreamAdapter.entries)
  return _internal_entries();
}
inline ::google::protobuf::RepeatedPtrField<::logproto::EntryAdapter>* PROTOBUF_NONNULL
StreamAdapter::mutable_entries() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:logproto.StreamAdapter.entries)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_entries();
}
inline const ::google::protobuf::RepeatedPtrField<::logproto::EntryAdapter>&
StreamAdapter::_internal_entries() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.entries_;
}
inline ::google::protobuf::RepeatedPtrField<::logproto::EntryAdapter>* PROTOBUF_NONNULL
StreamAdapter::_internal_mutable_entries() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.entries_;
}

// uint64 hash = 3;
inline void StreamAdapter::clear_hash() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.hash_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::uint64_t StreamAdapter::hash() const {
  // @@protoc_insertion_point(field_get:logproto.StreamAdapter.hash)
  return _internal_hash();
}
inline void StreamAdapter::set_hash(::uint64_t value) {
  _internal_set_hash(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:logproto.StreamAdapter.hash)
}
inline ::uint64_t StreamAdapter::_internal_hash() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.hash_;
}
inline void StreamAdapter::_internal_set_hash(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.hash_ = value;
}

// -------------------------------------------------------------------

// LabelPairAdapter

// string name = 1;
inline void LabelPairAdapter::clear_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& LabelPairAdapter::name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:logproto.LabelPairAdapter.name)
  return _internal_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void LabelPairAdapter::set_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:logproto.LabelPairAdapter.name)
}
inline ::std::string* PROTOBUF_NONNULL LabelPairAdapter::mutable_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:logproto.LabelPairAdapter.name)
  return _s;
}
inline const ::std::string& LabelPairAdapter::_internal_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.name_.Get();
}
inline void LabelPairAdapter::_internal_set_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL LabelPairAdapter::_internal_mutable_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE LabelPairAdapter::release_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:logproto.LabelPairAdapter.name)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.name_.Set("", GetArena());
  }
  return released;
}
inline void LabelPairAdapter::set_allocated_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.name_.IsDefault()) {
    _impl_.name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:logproto.LabelPairAdapter.name)
}

// string value = 2;
inline void LabelPairAdapter::clear_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.value_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& LabelPairAdapter::value() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:logproto.LabelPairAdapter.value)
  return _internal_value();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void LabelPairAdapter::set_value(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.value_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:logproto.LabelPairAdapter.value)
}
inline ::std::string* PROTOBUF_NONNULL LabelPairAdapter::mutable_value()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_value();
  // @@protoc_insertion_point(field_mutable:logproto.LabelPairAdapter.value)
  return _s;
}
inline const ::std::string& LabelPairAdapter::_internal_value() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.value_.Get();
}
inline void LabelPairAdapter::_internal_set_value(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.value_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL LabelPairAdapter::_internal_mutable_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.value_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE LabelPairAdapter::release_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:logproto.LabelPairAdapter.value)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.value_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.value_.Set("", GetArena());
  }
  return released;
}
inline void LabelPairAdapter::set_allocated_value(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.value_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.value_.IsDefault()) {
    _impl_.value_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:logproto.LabelPairAdapter.value)
}

// -------------------------------------------------------------------

// EntryAdapter

// .google.protobuf.Timestamp timestamp = 1;
inline bool EntryAdapter::has_timestamp() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.timestamp_ != nullptr);
  return value;
}
inline const ::google::protobuf::Timestamp& EntryAdapter::_internal_timestamp() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::google::protobuf::Timestamp* p = _impl_.timestamp_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::google::protobuf::Timestamp>(&::google::protobuf::Timestamp_globals_);
}
inline const ::google::protobuf::Timestamp& EntryAdapter::timestamp() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:logproto.EntryAdapter.timestamp)
  return _internal_timestamp();
}
inline void EntryAdapter::unsafe_arena_set_allocated_timestamp(
    ::google::protobuf::Timestamp* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.timestamp_);
  }
  _impl_.timestamp_ = reinterpret_cast<::google::protobuf::Timestamp*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:logproto.EntryAdapter.timestamp)
}
inline ::google::protobuf::Timestamp* PROTOBUF_NULLABLE EntryAdapter::release_timestamp() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::google::protobuf::Timestamp* released = _impl_.timestamp_;
  _impl_.timestamp_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::google::protobuf::Timestamp* PROTOBUF_NULLABLE EntryAdapter::unsafe_arena_release_timestamp() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:logproto.EntryAdapter.timestamp)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::google::protobuf::Timestamp* temp = _impl_.timestamp_;
  _impl_.timestamp_ = nullptr;
  return temp;
}
inline ::google::protobuf::Timestamp* PROTOBUF_NONNULL EntryAdapter::_internal_mutable_timestamp() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.timestamp_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::google::protobuf::Timestamp>(GetArena());
    _impl_.timestamp_ = reinterpret_cast<::google::protobuf::Timestamp*>(p);
  }
  return _impl_.timestamp_;
}
inline ::google::protobuf::Timestamp* PROTOBUF_NONNULL EntryAdapter::mutable_timestamp()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::google::protobuf::Timestamp* _msg = _internal_mutable_timestamp();
  // @@protoc_insertion_point(field_mutable:logproto.EntryAdapter.timestamp)
  return _msg;
}
inline void EntryAdapter::set_allocated_timestamp(::google::protobuf::Timestamp* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.timestamp_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.timestamp_ = reinterpret_cast<::google::protobuf::Timestamp*>(value);
  // @@protoc_insertion_point(field_set_allocated:logproto.EntryAdapter.timestamp)
}

// string line = 2;
inline void EntryAdapter::clear_line() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.line_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& EntryAdapter::line() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:logproto.EntryAdapter.line)
  return _internal_line();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void EntryAdapter::set_line(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.line_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:logproto.EntryAdapter.line)
}
inline ::std::string* PROTOBUF_NONNULL EntryAdapter::mutable_line()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_line();
  // @@protoc_insertion_point(field_mutable:logproto.EntryAdapter.line)
  return _s;
}
inline const ::std::string& EntryAdapter::_internal_line() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.line_.Get();
}
inline void EntryAdapter::_internal_set_line(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.line_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL EntryAdapter::_internal_mutable_line() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.line_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE EntryAdapter::release_line() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:logproto.EntryAdapter.line)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.line_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.line_.Set("", GetArena());
  }
  return released;
}
inline void EntryAdapter::set_allocated_line(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.line_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.line_.IsDefault()) {
    _impl_.line_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:logproto.EntryAdapter.line)
}

// repeated .logproto.LabelPairAdapter nonIndexedLabels = 3;
inline int EntryAdapter::_internal_nonindexedlabels_size() const {
  return _internal_nonindexedlabels().size();
}
inline int EntryAdapter::nonindexedlabels_size() const {
  return _internal_nonindexedlabels_size();
}
inline void EntryAdapter::clear_nonindexedlabels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.nonindexedlabels_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::logproto::LabelPairAdapter& EntryAdapter::nonindexedlabels(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:logproto.EntryAdapter.nonIndexedLabels)
  return _internal_nonindexedlabels().Get(index);
}
inline ::logproto::LabelPairAdapter* PROTOBUF_NONNULL EntryAdapter::mutable_nonindexedlabels(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:logproto.EntryAdapter.nonIndexedLabels)
  return _internal_mutable_nonindexedlabels()->Mutable(index);
}
inline ::logproto::LabelPairAdapter* PROTOBUF_NONNULL EntryAdapter::add_nonindexedlabels()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::logproto::LabelPairAdapter* _add =
      _internal_mutable_nonindexedlabels()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:logproto.EntryAdapter.nonIndexedLabels)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::logproto::LabelPairAdapter>& EntryAdapter::nonindexedlabels() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:logproto.EntryAdapter.nonIndexedLabels)
  return _internal_nonindexedlabels();
}
inline ::google::protobuf::RepeatedPtrField<::logproto::LabelPairAdapter>* PROTOBUF_NONNULL
EntryAdapter::mutable_nonindexedlabels() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:logproto.EntryAdapter.nonIndexedLabels)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_nonindexedlabels();
}
inline const ::google::protobuf::RepeatedPtrField<::logproto::LabelPairAdapter>&
EntryAdapter::_internal_nonindexedlabels() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.nonindexedlabels_;
}
inline ::google::protobuf::RepeatedPtrField<::logproto::LabelPairAdapter>* PROTOBUF_NONNULL
EntryAdapter::_internal_mutable_nonindexedlabels() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.nonindexedlabels_;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace logproto


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // push_2eproto_2epb_2eh
