// Generated by the gRPC C++ plugin.
// If you make any local change, they will be lost.
// source: clickhouse_grpc.proto
// Original file comments:
// This file describes gRPC protocol supported in ClickHouse.
//
// To use this protocol a client should send one or more messages of the QueryInfo type
// and then receive one or more messages of the Result type.
// According to that the service provides four methods for that:
// ExecuteQuery(QueryInfo) returns (Result)
// ExecuteQueryWithStreamInput(stream QueryInfo) returns (Result)
// ExecuteQueryWithStreamOutput(QueryInfo) returns (stream Result)
// ExecuteQueryWithStreamIO(stream QueryInfo) returns (stream Result)
// It's up to the client to choose which method to use.
// For example, ExecuteQueryWithStreamInput() allows the client to add data multiple times
// while executing a query, which is suitable for inserting many rows.
//
#ifndef GRPC_clickhouse_5fgrpc_2eproto__INCLUDED
#define GRPC_clickhouse_5fgrpc_2eproto__INCLUDED

#include "clickhouse_grpc.pb.h"

#include <functional>
#include <grpcpp/generic/async_generic_service.h>
#include <grpcpp/support/async_stream.h>
#include <grpcpp/support/async_unary_call.h>
#include <grpcpp/support/client_callback.h>
#include <grpcpp/client_context.h>
#include <grpcpp/completion_queue.h>
#include <grpcpp/support/message_allocator.h>
#include <grpcpp/support/method_handler.h>
#include <grpcpp/impl/proto_utils.h>
#include <grpcpp/impl/rpc_method.h>
#include <grpcpp/support/server_callback.h>
#include <grpcpp/impl/server_callback_handlers.h>
#include <grpcpp/server_context.h>
#include <grpcpp/impl/service_type.h>
#include <grpcpp/support/status.h>
#include <grpcpp/support/stub_options.h>
#include <grpcpp/support/sync_stream.h>
#include <grpcpp/ports_def.inc>

namespace clickhouse {
namespace grpc {

class ClickHouse final {
 public:
  static constexpr char const* service_full_name() {
    return "clickhouse.grpc.ClickHouse";
  }
  class StubInterface {
   public:
    virtual ~StubInterface() {}
    virtual ::grpc::Status ExecuteQuery(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request, ::clickhouse::grpc::Result* response) = 0;
    std::unique_ptr< ::grpc::ClientAsyncResponseReaderInterface< ::clickhouse::grpc::Result>> AsyncExecuteQuery(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request, ::grpc::CompletionQueue* cq) {
      return std::unique_ptr< ::grpc::ClientAsyncResponseReaderInterface< ::clickhouse::grpc::Result>>(AsyncExecuteQueryRaw(context, request, cq));
    }
    std::unique_ptr< ::grpc::ClientAsyncResponseReaderInterface< ::clickhouse::grpc::Result>> PrepareAsyncExecuteQuery(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request, ::grpc::CompletionQueue* cq) {
      return std::unique_ptr< ::grpc::ClientAsyncResponseReaderInterface< ::clickhouse::grpc::Result>>(PrepareAsyncExecuteQueryRaw(context, request, cq));
    }
    std::unique_ptr< ::grpc::ClientWriterInterface< ::clickhouse::grpc::QueryInfo>> ExecuteQueryWithStreamInput(::grpc::ClientContext* context, ::clickhouse::grpc::Result* response) {
      return std::unique_ptr< ::grpc::ClientWriterInterface< ::clickhouse::grpc::QueryInfo>>(ExecuteQueryWithStreamInputRaw(context, response));
    }
    std::unique_ptr< ::grpc::ClientAsyncWriterInterface< ::clickhouse::grpc::QueryInfo>> AsyncExecuteQueryWithStreamInput(::grpc::ClientContext* context, ::clickhouse::grpc::Result* response, ::grpc::CompletionQueue* cq, void* tag) {
      return std::unique_ptr< ::grpc::ClientAsyncWriterInterface< ::clickhouse::grpc::QueryInfo>>(AsyncExecuteQueryWithStreamInputRaw(context, response, cq, tag));
    }
    std::unique_ptr< ::grpc::ClientAsyncWriterInterface< ::clickhouse::grpc::QueryInfo>> PrepareAsyncExecuteQueryWithStreamInput(::grpc::ClientContext* context, ::clickhouse::grpc::Result* response, ::grpc::CompletionQueue* cq) {
      return std::unique_ptr< ::grpc::ClientAsyncWriterInterface< ::clickhouse::grpc::QueryInfo>>(PrepareAsyncExecuteQueryWithStreamInputRaw(context, response, cq));
    }
    std::unique_ptr< ::grpc::ClientReaderInterface< ::clickhouse::grpc::Result>> ExecuteQueryWithStreamOutput(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request) {
      return std::unique_ptr< ::grpc::ClientReaderInterface< ::clickhouse::grpc::Result>>(ExecuteQueryWithStreamOutputRaw(context, request));
    }
    std::unique_ptr< ::grpc::ClientAsyncReaderInterface< ::clickhouse::grpc::Result>> AsyncExecuteQueryWithStreamOutput(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request, ::grpc::CompletionQueue* cq, void* tag) {
      return std::unique_ptr< ::grpc::ClientAsyncReaderInterface< ::clickhouse::grpc::Result>>(AsyncExecuteQueryWithStreamOutputRaw(context, request, cq, tag));
    }
    std::unique_ptr< ::grpc::ClientAsyncReaderInterface< ::clickhouse::grpc::Result>> PrepareAsyncExecuteQueryWithStreamOutput(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request, ::grpc::CompletionQueue* cq) {
      return std::unique_ptr< ::grpc::ClientAsyncReaderInterface< ::clickhouse::grpc::Result>>(PrepareAsyncExecuteQueryWithStreamOutputRaw(context, request, cq));
    }
    std::unique_ptr< ::grpc::ClientReaderWriterInterface< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>> ExecuteQueryWithStreamIO(::grpc::ClientContext* context) {
      return std::unique_ptr< ::grpc::ClientReaderWriterInterface< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>>(ExecuteQueryWithStreamIORaw(context));
    }
    std::unique_ptr< ::grpc::ClientAsyncReaderWriterInterface< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>> AsyncExecuteQueryWithStreamIO(::grpc::ClientContext* context, ::grpc::CompletionQueue* cq, void* tag) {
      return std::unique_ptr< ::grpc::ClientAsyncReaderWriterInterface< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>>(AsyncExecuteQueryWithStreamIORaw(context, cq, tag));
    }
    std::unique_ptr< ::grpc::ClientAsyncReaderWriterInterface< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>> PrepareAsyncExecuteQueryWithStreamIO(::grpc::ClientContext* context, ::grpc::CompletionQueue* cq) {
      return std::unique_ptr< ::grpc::ClientAsyncReaderWriterInterface< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>>(PrepareAsyncExecuteQueryWithStreamIORaw(context, cq));
    }
    class async_interface {
     public:
      virtual ~async_interface() {}
      virtual void ExecuteQuery(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo* request, ::clickhouse::grpc::Result* response, std::function<void(::grpc::Status)>) = 0;
      virtual void ExecuteQuery(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo* request, ::clickhouse::grpc::Result* response, ::grpc::ClientUnaryReactor* reactor) = 0;
      virtual void ExecuteQueryWithStreamInput(::grpc::ClientContext* context, ::clickhouse::grpc::Result* response, ::grpc::ClientWriteReactor< ::clickhouse::grpc::QueryInfo>* reactor) = 0;
      virtual void ExecuteQueryWithStreamOutput(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo* request, ::grpc::ClientReadReactor< ::clickhouse::grpc::Result>* reactor) = 0;
      virtual void ExecuteQueryWithStreamIO(::grpc::ClientContext* context, ::grpc::ClientBidiReactor< ::clickhouse::grpc::QueryInfo,::clickhouse::grpc::Result>* reactor) = 0;
    };
    typedef class async_interface experimental_async_interface;
    virtual class async_interface* async() { return nullptr; }
    class async_interface* experimental_async() { return async(); }
   private:
    virtual ::grpc::ClientAsyncResponseReaderInterface< ::clickhouse::grpc::Result>* AsyncExecuteQueryRaw(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request, ::grpc::CompletionQueue* cq) = 0;
    virtual ::grpc::ClientAsyncResponseReaderInterface< ::clickhouse::grpc::Result>* PrepareAsyncExecuteQueryRaw(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request, ::grpc::CompletionQueue* cq) = 0;
    virtual ::grpc::ClientWriterInterface< ::clickhouse::grpc::QueryInfo>* ExecuteQueryWithStreamInputRaw(::grpc::ClientContext* context, ::clickhouse::grpc::Result* response) = 0;
    virtual ::grpc::ClientAsyncWriterInterface< ::clickhouse::grpc::QueryInfo>* AsyncExecuteQueryWithStreamInputRaw(::grpc::ClientContext* context, ::clickhouse::grpc::Result* response, ::grpc::CompletionQueue* cq, void* tag) = 0;
    virtual ::grpc::ClientAsyncWriterInterface< ::clickhouse::grpc::QueryInfo>* PrepareAsyncExecuteQueryWithStreamInputRaw(::grpc::ClientContext* context, ::clickhouse::grpc::Result* response, ::grpc::CompletionQueue* cq) = 0;
    virtual ::grpc::ClientReaderInterface< ::clickhouse::grpc::Result>* ExecuteQueryWithStreamOutputRaw(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request) = 0;
    virtual ::grpc::ClientAsyncReaderInterface< ::clickhouse::grpc::Result>* AsyncExecuteQueryWithStreamOutputRaw(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request, ::grpc::CompletionQueue* cq, void* tag) = 0;
    virtual ::grpc::ClientAsyncReaderInterface< ::clickhouse::grpc::Result>* PrepareAsyncExecuteQueryWithStreamOutputRaw(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request, ::grpc::CompletionQueue* cq) = 0;
    virtual ::grpc::ClientReaderWriterInterface< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>* ExecuteQueryWithStreamIORaw(::grpc::ClientContext* context) = 0;
    virtual ::grpc::ClientAsyncReaderWriterInterface< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>* AsyncExecuteQueryWithStreamIORaw(::grpc::ClientContext* context, ::grpc::CompletionQueue* cq, void* tag) = 0;
    virtual ::grpc::ClientAsyncReaderWriterInterface< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>* PrepareAsyncExecuteQueryWithStreamIORaw(::grpc::ClientContext* context, ::grpc::CompletionQueue* cq) = 0;
  };
  class Stub final : public StubInterface {
   public:
    Stub(const std::shared_ptr< ::grpc::ChannelInterface>& channel, const ::grpc::StubOptions& options = ::grpc::StubOptions());
    ::grpc::Status ExecuteQuery(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request, ::clickhouse::grpc::Result* response) override;
    std::unique_ptr< ::grpc::ClientAsyncResponseReader< ::clickhouse::grpc::Result>> AsyncExecuteQuery(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request, ::grpc::CompletionQueue* cq) {
      return std::unique_ptr< ::grpc::ClientAsyncResponseReader< ::clickhouse::grpc::Result>>(AsyncExecuteQueryRaw(context, request, cq));
    }
    std::unique_ptr< ::grpc::ClientAsyncResponseReader< ::clickhouse::grpc::Result>> PrepareAsyncExecuteQuery(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request, ::grpc::CompletionQueue* cq) {
      return std::unique_ptr< ::grpc::ClientAsyncResponseReader< ::clickhouse::grpc::Result>>(PrepareAsyncExecuteQueryRaw(context, request, cq));
    }
    std::unique_ptr< ::grpc::ClientWriter< ::clickhouse::grpc::QueryInfo>> ExecuteQueryWithStreamInput(::grpc::ClientContext* context, ::clickhouse::grpc::Result* response) {
      return std::unique_ptr< ::grpc::ClientWriter< ::clickhouse::grpc::QueryInfo>>(ExecuteQueryWithStreamInputRaw(context, response));
    }
    std::unique_ptr< ::grpc::ClientAsyncWriter< ::clickhouse::grpc::QueryInfo>> AsyncExecuteQueryWithStreamInput(::grpc::ClientContext* context, ::clickhouse::grpc::Result* response, ::grpc::CompletionQueue* cq, void* tag) {
      return std::unique_ptr< ::grpc::ClientAsyncWriter< ::clickhouse::grpc::QueryInfo>>(AsyncExecuteQueryWithStreamInputRaw(context, response, cq, tag));
    }
    std::unique_ptr< ::grpc::ClientAsyncWriter< ::clickhouse::grpc::QueryInfo>> PrepareAsyncExecuteQueryWithStreamInput(::grpc::ClientContext* context, ::clickhouse::grpc::Result* response, ::grpc::CompletionQueue* cq) {
      return std::unique_ptr< ::grpc::ClientAsyncWriter< ::clickhouse::grpc::QueryInfo>>(PrepareAsyncExecuteQueryWithStreamInputRaw(context, response, cq));
    }
    std::unique_ptr< ::grpc::ClientReader< ::clickhouse::grpc::Result>> ExecuteQueryWithStreamOutput(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request) {
      return std::unique_ptr< ::grpc::ClientReader< ::clickhouse::grpc::Result>>(ExecuteQueryWithStreamOutputRaw(context, request));
    }
    std::unique_ptr< ::grpc::ClientAsyncReader< ::clickhouse::grpc::Result>> AsyncExecuteQueryWithStreamOutput(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request, ::grpc::CompletionQueue* cq, void* tag) {
      return std::unique_ptr< ::grpc::ClientAsyncReader< ::clickhouse::grpc::Result>>(AsyncExecuteQueryWithStreamOutputRaw(context, request, cq, tag));
    }
    std::unique_ptr< ::grpc::ClientAsyncReader< ::clickhouse::grpc::Result>> PrepareAsyncExecuteQueryWithStreamOutput(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request, ::grpc::CompletionQueue* cq) {
      return std::unique_ptr< ::grpc::ClientAsyncReader< ::clickhouse::grpc::Result>>(PrepareAsyncExecuteQueryWithStreamOutputRaw(context, request, cq));
    }
    std::unique_ptr< ::grpc::ClientReaderWriter< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>> ExecuteQueryWithStreamIO(::grpc::ClientContext* context) {
      return std::unique_ptr< ::grpc::ClientReaderWriter< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>>(ExecuteQueryWithStreamIORaw(context));
    }
    std::unique_ptr<  ::grpc::ClientAsyncReaderWriter< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>> AsyncExecuteQueryWithStreamIO(::grpc::ClientContext* context, ::grpc::CompletionQueue* cq, void* tag) {
      return std::unique_ptr< ::grpc::ClientAsyncReaderWriter< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>>(AsyncExecuteQueryWithStreamIORaw(context, cq, tag));
    }
    std::unique_ptr<  ::grpc::ClientAsyncReaderWriter< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>> PrepareAsyncExecuteQueryWithStreamIO(::grpc::ClientContext* context, ::grpc::CompletionQueue* cq) {
      return std::unique_ptr< ::grpc::ClientAsyncReaderWriter< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>>(PrepareAsyncExecuteQueryWithStreamIORaw(context, cq));
    }
    class async final :
      public StubInterface::async_interface {
     public:
      void ExecuteQuery(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo* request, ::clickhouse::grpc::Result* response, std::function<void(::grpc::Status)>) override;
      void ExecuteQuery(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo* request, ::clickhouse::grpc::Result* response, ::grpc::ClientUnaryReactor* reactor) override;
      void ExecuteQueryWithStreamInput(::grpc::ClientContext* context, ::clickhouse::grpc::Result* response, ::grpc::ClientWriteReactor< ::clickhouse::grpc::QueryInfo>* reactor) override;
      void ExecuteQueryWithStreamOutput(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo* request, ::grpc::ClientReadReactor< ::clickhouse::grpc::Result>* reactor) override;
      void ExecuteQueryWithStreamIO(::grpc::ClientContext* context, ::grpc::ClientBidiReactor< ::clickhouse::grpc::QueryInfo,::clickhouse::grpc::Result>* reactor) override;
     private:
      friend class Stub;
      explicit async(Stub* stub): stub_(stub) { }
      Stub* stub() { return stub_; }
      Stub* stub_;
    };
    class async* async() override { return &async_stub_; }

   private:
    std::shared_ptr< ::grpc::ChannelInterface> channel_;
    class async async_stub_{this};
    ::grpc::ClientAsyncResponseReader< ::clickhouse::grpc::Result>* AsyncExecuteQueryRaw(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request, ::grpc::CompletionQueue* cq) override;
    ::grpc::ClientAsyncResponseReader< ::clickhouse::grpc::Result>* PrepareAsyncExecuteQueryRaw(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request, ::grpc::CompletionQueue* cq) override;
    ::grpc::ClientWriter< ::clickhouse::grpc::QueryInfo>* ExecuteQueryWithStreamInputRaw(::grpc::ClientContext* context, ::clickhouse::grpc::Result* response) override;
    ::grpc::ClientAsyncWriter< ::clickhouse::grpc::QueryInfo>* AsyncExecuteQueryWithStreamInputRaw(::grpc::ClientContext* context, ::clickhouse::grpc::Result* response, ::grpc::CompletionQueue* cq, void* tag) override;
    ::grpc::ClientAsyncWriter< ::clickhouse::grpc::QueryInfo>* PrepareAsyncExecuteQueryWithStreamInputRaw(::grpc::ClientContext* context, ::clickhouse::grpc::Result* response, ::grpc::CompletionQueue* cq) override;
    ::grpc::ClientReader< ::clickhouse::grpc::Result>* ExecuteQueryWithStreamOutputRaw(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request) override;
    ::grpc::ClientAsyncReader< ::clickhouse::grpc::Result>* AsyncExecuteQueryWithStreamOutputRaw(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request, ::grpc::CompletionQueue* cq, void* tag) override;
    ::grpc::ClientAsyncReader< ::clickhouse::grpc::Result>* PrepareAsyncExecuteQueryWithStreamOutputRaw(::grpc::ClientContext* context, const ::clickhouse::grpc::QueryInfo& request, ::grpc::CompletionQueue* cq) override;
    ::grpc::ClientReaderWriter< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>* ExecuteQueryWithStreamIORaw(::grpc::ClientContext* context) override;
    ::grpc::ClientAsyncReaderWriter< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>* AsyncExecuteQueryWithStreamIORaw(::grpc::ClientContext* context, ::grpc::CompletionQueue* cq, void* tag) override;
    ::grpc::ClientAsyncReaderWriter< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>* PrepareAsyncExecuteQueryWithStreamIORaw(::grpc::ClientContext* context, ::grpc::CompletionQueue* cq) override;
    const ::grpc::internal::RpcMethod rpcmethod_ExecuteQuery_;
    const ::grpc::internal::RpcMethod rpcmethod_ExecuteQueryWithStreamInput_;
    const ::grpc::internal::RpcMethod rpcmethod_ExecuteQueryWithStreamOutput_;
    const ::grpc::internal::RpcMethod rpcmethod_ExecuteQueryWithStreamIO_;
  };
  static std::unique_ptr<Stub> NewStub(const std::shared_ptr< ::grpc::ChannelInterface>& channel, const ::grpc::StubOptions& options = ::grpc::StubOptions());

  class Service : public ::grpc::Service {
   public:
    Service();
    virtual ~Service();
    virtual ::grpc::Status ExecuteQuery(::grpc::ServerContext* context, const ::clickhouse::grpc::QueryInfo* request, ::clickhouse::grpc::Result* response);
    virtual ::grpc::Status ExecuteQueryWithStreamInput(::grpc::ServerContext* context, ::grpc::ServerReader< ::clickhouse::grpc::QueryInfo>* reader, ::clickhouse::grpc::Result* response);
    virtual ::grpc::Status ExecuteQueryWithStreamOutput(::grpc::ServerContext* context, const ::clickhouse::grpc::QueryInfo* request, ::grpc::ServerWriter< ::clickhouse::grpc::Result>* writer);
    virtual ::grpc::Status ExecuteQueryWithStreamIO(::grpc::ServerContext* context, ::grpc::ServerReaderWriter< ::clickhouse::grpc::Result, ::clickhouse::grpc::QueryInfo>* stream);
  };
  template <class BaseClass>
  class WithAsyncMethod_ExecuteQuery : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithAsyncMethod_ExecuteQuery() {
      ::grpc::Service::MarkMethodAsync(0);
    }
    ~WithAsyncMethod_ExecuteQuery() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable synchronous version of this method
    ::grpc::Status ExecuteQuery(::grpc::ServerContext* /*context*/, const ::clickhouse::grpc::QueryInfo* /*request*/, ::clickhouse::grpc::Result* /*response*/) override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
    void RequestExecuteQuery(::grpc::ServerContext* context, ::clickhouse::grpc::QueryInfo* request, ::grpc::ServerAsyncResponseWriter< ::clickhouse::grpc::Result>* response, ::grpc::CompletionQueue* new_call_cq, ::grpc::ServerCompletionQueue* notification_cq, void *tag) {
      ::grpc::Service::RequestAsyncUnary(0, context, request, response, new_call_cq, notification_cq, tag);
    }
  };
  template <class BaseClass>
  class WithAsyncMethod_ExecuteQueryWithStreamInput : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithAsyncMethod_ExecuteQueryWithStreamInput() {
      ::grpc::Service::MarkMethodAsync(1);
    }
    ~WithAsyncMethod_ExecuteQueryWithStreamInput() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable synchronous version of this method
    ::grpc::Status ExecuteQueryWithStreamInput(::grpc::ServerContext* /*context*/, ::grpc::ServerReader< ::clickhouse::grpc::QueryInfo>* /*reader*/, ::clickhouse::grpc::Result* /*response*/) override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
    void RequestExecuteQueryWithStreamInput(::grpc::ServerContext* context, ::grpc::ServerAsyncReader< ::clickhouse::grpc::Result, ::clickhouse::grpc::QueryInfo>* reader, ::grpc::CompletionQueue* new_call_cq, ::grpc::ServerCompletionQueue* notification_cq, void *tag) {
      ::grpc::Service::RequestAsyncClientStreaming(1, context, reader, new_call_cq, notification_cq, tag);
    }
  };
  template <class BaseClass>
  class WithAsyncMethod_ExecuteQueryWithStreamOutput : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithAsyncMethod_ExecuteQueryWithStreamOutput() {
      ::grpc::Service::MarkMethodAsync(2);
    }
    ~WithAsyncMethod_ExecuteQueryWithStreamOutput() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable synchronous version of this method
    ::grpc::Status ExecuteQueryWithStreamOutput(::grpc::ServerContext* /*context*/, const ::clickhouse::grpc::QueryInfo* /*request*/, ::grpc::ServerWriter< ::clickhouse::grpc::Result>* /*writer*/) override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
    void RequestExecuteQueryWithStreamOutput(::grpc::ServerContext* context, ::clickhouse::grpc::QueryInfo* request, ::grpc::ServerAsyncWriter< ::clickhouse::grpc::Result>* writer, ::grpc::CompletionQueue* new_call_cq, ::grpc::ServerCompletionQueue* notification_cq, void *tag) {
      ::grpc::Service::RequestAsyncServerStreaming(2, context, request, writer, new_call_cq, notification_cq, tag);
    }
  };
  template <class BaseClass>
  class WithAsyncMethod_ExecuteQueryWithStreamIO : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithAsyncMethod_ExecuteQueryWithStreamIO() {
      ::grpc::Service::MarkMethodAsync(3);
    }
    ~WithAsyncMethod_ExecuteQueryWithStreamIO() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable synchronous version of this method
    ::grpc::Status ExecuteQueryWithStreamIO(::grpc::ServerContext* /*context*/, ::grpc::ServerReaderWriter< ::clickhouse::grpc::Result, ::clickhouse::grpc::QueryInfo>* /*stream*/)  override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
    void RequestExecuteQueryWithStreamIO(::grpc::ServerContext* context, ::grpc::ServerAsyncReaderWriter< ::clickhouse::grpc::Result, ::clickhouse::grpc::QueryInfo>* stream, ::grpc::CompletionQueue* new_call_cq, ::grpc::ServerCompletionQueue* notification_cq, void *tag) {
      ::grpc::Service::RequestAsyncBidiStreaming(3, context, stream, new_call_cq, notification_cq, tag);
    }
  };
  typedef WithAsyncMethod_ExecuteQuery<WithAsyncMethod_ExecuteQueryWithStreamInput<WithAsyncMethod_ExecuteQueryWithStreamOutput<WithAsyncMethod_ExecuteQueryWithStreamIO<Service > > > > AsyncService;
  template <class BaseClass>
  class WithCallbackMethod_ExecuteQuery : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithCallbackMethod_ExecuteQuery() {
      ::grpc::Service::MarkMethodCallback(0,
          new ::grpc::internal::CallbackUnaryHandler< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>(
            [this](
                   ::grpc::CallbackServerContext* context, const ::clickhouse::grpc::QueryInfo* request, ::clickhouse::grpc::Result* response) { return this->ExecuteQuery(context, request, response); }));}
    void SetMessageAllocatorFor_ExecuteQuery(
        ::grpc::MessageAllocator< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>* allocator) {
      ::grpc::internal::MethodHandler* const handler = ::grpc::Service::GetHandler(0);
      static_cast<::grpc::internal::CallbackUnaryHandler< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>*>(handler)
              ->SetMessageAllocator(allocator);
    }
    ~WithCallbackMethod_ExecuteQuery() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable synchronous version of this method
    ::grpc::Status ExecuteQuery(::grpc::ServerContext* /*context*/, const ::clickhouse::grpc::QueryInfo* /*request*/, ::clickhouse::grpc::Result* /*response*/) override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
    virtual ::grpc::ServerUnaryReactor* ExecuteQuery(
      ::grpc::CallbackServerContext* /*context*/, const ::clickhouse::grpc::QueryInfo* /*request*/, ::clickhouse::grpc::Result* /*response*/)  { return nullptr; }
  };
  template <class BaseClass>
  class WithCallbackMethod_ExecuteQueryWithStreamInput : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithCallbackMethod_ExecuteQueryWithStreamInput() {
      ::grpc::Service::MarkMethodCallback(1,
          new ::grpc::internal::CallbackClientStreamingHandler< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>(
            [this](
                   ::grpc::CallbackServerContext* context, ::clickhouse::grpc::Result* response) { return this->ExecuteQueryWithStreamInput(context, response); }));
    }
    ~WithCallbackMethod_ExecuteQueryWithStreamInput() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable synchronous version of this method
    ::grpc::Status ExecuteQueryWithStreamInput(::grpc::ServerContext* /*context*/, ::grpc::ServerReader< ::clickhouse::grpc::QueryInfo>* /*reader*/, ::clickhouse::grpc::Result* /*response*/) override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
    virtual ::grpc::ServerReadReactor< ::clickhouse::grpc::QueryInfo>* ExecuteQueryWithStreamInput(
      ::grpc::CallbackServerContext* /*context*/, ::clickhouse::grpc::Result* /*response*/)  { return nullptr; }
  };
  template <class BaseClass>
  class WithCallbackMethod_ExecuteQueryWithStreamOutput : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithCallbackMethod_ExecuteQueryWithStreamOutput() {
      ::grpc::Service::MarkMethodCallback(2,
          new ::grpc::internal::CallbackServerStreamingHandler< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>(
            [this](
                   ::grpc::CallbackServerContext* context, const ::clickhouse::grpc::QueryInfo* request) { return this->ExecuteQueryWithStreamOutput(context, request); }));
    }
    ~WithCallbackMethod_ExecuteQueryWithStreamOutput() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable synchronous version of this method
    ::grpc::Status ExecuteQueryWithStreamOutput(::grpc::ServerContext* /*context*/, const ::clickhouse::grpc::QueryInfo* /*request*/, ::grpc::ServerWriter< ::clickhouse::grpc::Result>* /*writer*/) override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
    virtual ::grpc::ServerWriteReactor< ::clickhouse::grpc::Result>* ExecuteQueryWithStreamOutput(
      ::grpc::CallbackServerContext* /*context*/, const ::clickhouse::grpc::QueryInfo* /*request*/)  { return nullptr; }
  };
  template <class BaseClass>
  class WithCallbackMethod_ExecuteQueryWithStreamIO : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithCallbackMethod_ExecuteQueryWithStreamIO() {
      ::grpc::Service::MarkMethodCallback(3,
          new ::grpc::internal::CallbackBidiHandler< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>(
            [this](
                   ::grpc::CallbackServerContext* context) { return this->ExecuteQueryWithStreamIO(context); }));
    }
    ~WithCallbackMethod_ExecuteQueryWithStreamIO() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable synchronous version of this method
    ::grpc::Status ExecuteQueryWithStreamIO(::grpc::ServerContext* /*context*/, ::grpc::ServerReaderWriter< ::clickhouse::grpc::Result, ::clickhouse::grpc::QueryInfo>* /*stream*/)  override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
    virtual ::grpc::ServerBidiReactor< ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>* ExecuteQueryWithStreamIO(
      ::grpc::CallbackServerContext* /*context*/)
      { return nullptr; }
  };
  typedef WithCallbackMethod_ExecuteQuery<WithCallbackMethod_ExecuteQueryWithStreamInput<WithCallbackMethod_ExecuteQueryWithStreamOutput<WithCallbackMethod_ExecuteQueryWithStreamIO<Service > > > > CallbackService;
  typedef CallbackService ExperimentalCallbackService;
  template <class BaseClass>
  class WithGenericMethod_ExecuteQuery : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithGenericMethod_ExecuteQuery() {
      ::grpc::Service::MarkMethodGeneric(0);
    }
    ~WithGenericMethod_ExecuteQuery() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable synchronous version of this method
    ::grpc::Status ExecuteQuery(::grpc::ServerContext* /*context*/, const ::clickhouse::grpc::QueryInfo* /*request*/, ::clickhouse::grpc::Result* /*response*/) override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
  };
  template <class BaseClass>
  class WithGenericMethod_ExecuteQueryWithStreamInput : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithGenericMethod_ExecuteQueryWithStreamInput() {
      ::grpc::Service::MarkMethodGeneric(1);
    }
    ~WithGenericMethod_ExecuteQueryWithStreamInput() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable synchronous version of this method
    ::grpc::Status ExecuteQueryWithStreamInput(::grpc::ServerContext* /*context*/, ::grpc::ServerReader< ::clickhouse::grpc::QueryInfo>* /*reader*/, ::clickhouse::grpc::Result* /*response*/) override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
  };
  template <class BaseClass>
  class WithGenericMethod_ExecuteQueryWithStreamOutput : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithGenericMethod_ExecuteQueryWithStreamOutput() {
      ::grpc::Service::MarkMethodGeneric(2);
    }
    ~WithGenericMethod_ExecuteQueryWithStreamOutput() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable synchronous version of this method
    ::grpc::Status ExecuteQueryWithStreamOutput(::grpc::ServerContext* /*context*/, const ::clickhouse::grpc::QueryInfo* /*request*/, ::grpc::ServerWriter< ::clickhouse::grpc::Result>* /*writer*/) override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
  };
  template <class BaseClass>
  class WithGenericMethod_ExecuteQueryWithStreamIO : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithGenericMethod_ExecuteQueryWithStreamIO() {
      ::grpc::Service::MarkMethodGeneric(3);
    }
    ~WithGenericMethod_ExecuteQueryWithStreamIO() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable synchronous version of this method
    ::grpc::Status ExecuteQueryWithStreamIO(::grpc::ServerContext* /*context*/, ::grpc::ServerReaderWriter< ::clickhouse::grpc::Result, ::clickhouse::grpc::QueryInfo>* /*stream*/)  override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
  };
  template <class BaseClass>
  class WithRawMethod_ExecuteQuery : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithRawMethod_ExecuteQuery() {
      ::grpc::Service::MarkMethodRaw(0);
    }
    ~WithRawMethod_ExecuteQuery() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable synchronous version of this method
    ::grpc::Status ExecuteQuery(::grpc::ServerContext* /*context*/, const ::clickhouse::grpc::QueryInfo* /*request*/, ::clickhouse::grpc::Result* /*response*/) override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
    void RequestExecuteQuery(::grpc::ServerContext* context, ::grpc::ByteBuffer* request, ::grpc::ServerAsyncResponseWriter< ::grpc::ByteBuffer>* response, ::grpc::CompletionQueue* new_call_cq, ::grpc::ServerCompletionQueue* notification_cq, void *tag) {
      ::grpc::Service::RequestAsyncUnary(0, context, request, response, new_call_cq, notification_cq, tag);
    }
  };
  template <class BaseClass>
  class WithRawMethod_ExecuteQueryWithStreamInput : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithRawMethod_ExecuteQueryWithStreamInput() {
      ::grpc::Service::MarkMethodRaw(1);
    }
    ~WithRawMethod_ExecuteQueryWithStreamInput() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable synchronous version of this method
    ::grpc::Status ExecuteQueryWithStreamInput(::grpc::ServerContext* /*context*/, ::grpc::ServerReader< ::clickhouse::grpc::QueryInfo>* /*reader*/, ::clickhouse::grpc::Result* /*response*/) override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
    void RequestExecuteQueryWithStreamInput(::grpc::ServerContext* context, ::grpc::ServerAsyncReader< ::grpc::ByteBuffer, ::grpc::ByteBuffer>* reader, ::grpc::CompletionQueue* new_call_cq, ::grpc::ServerCompletionQueue* notification_cq, void *tag) {
      ::grpc::Service::RequestAsyncClientStreaming(1, context, reader, new_call_cq, notification_cq, tag);
    }
  };
  template <class BaseClass>
  class WithRawMethod_ExecuteQueryWithStreamOutput : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithRawMethod_ExecuteQueryWithStreamOutput() {
      ::grpc::Service::MarkMethodRaw(2);
    }
    ~WithRawMethod_ExecuteQueryWithStreamOutput() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable synchronous version of this method
    ::grpc::Status ExecuteQueryWithStreamOutput(::grpc::ServerContext* /*context*/, const ::clickhouse::grpc::QueryInfo* /*request*/, ::grpc::ServerWriter< ::clickhouse::grpc::Result>* /*writer*/) override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
    void RequestExecuteQueryWithStreamOutput(::grpc::ServerContext* context, ::grpc::ByteBuffer* request, ::grpc::ServerAsyncWriter< ::grpc::ByteBuffer>* writer, ::grpc::CompletionQueue* new_call_cq, ::grpc::ServerCompletionQueue* notification_cq, void *tag) {
      ::grpc::Service::RequestAsyncServerStreaming(2, context, request, writer, new_call_cq, notification_cq, tag);
    }
  };
  template <class BaseClass>
  class WithRawMethod_ExecuteQueryWithStreamIO : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithRawMethod_ExecuteQueryWithStreamIO() {
      ::grpc::Service::MarkMethodRaw(3);
    }
    ~WithRawMethod_ExecuteQueryWithStreamIO() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable synchronous version of this method
    ::grpc::Status ExecuteQueryWithStreamIO(::grpc::ServerContext* /*context*/, ::grpc::ServerReaderWriter< ::clickhouse::grpc::Result, ::clickhouse::grpc::QueryInfo>* /*stream*/)  override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
    void RequestExecuteQueryWithStreamIO(::grpc::ServerContext* context, ::grpc::ServerAsyncReaderWriter< ::grpc::ByteBuffer, ::grpc::ByteBuffer>* stream, ::grpc::CompletionQueue* new_call_cq, ::grpc::ServerCompletionQueue* notification_cq, void *tag) {
      ::grpc::Service::RequestAsyncBidiStreaming(3, context, stream, new_call_cq, notification_cq, tag);
    }
  };
  template <class BaseClass>
  class WithRawCallbackMethod_ExecuteQuery : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithRawCallbackMethod_ExecuteQuery() {
      ::grpc::Service::MarkMethodRawCallback(0,
          new ::grpc::internal::CallbackUnaryHandler< ::grpc::ByteBuffer, ::grpc::ByteBuffer>(
            [this](
                   ::grpc::CallbackServerContext* context, const ::grpc::ByteBuffer* request, ::grpc::ByteBuffer* response) { return this->ExecuteQuery(context, request, response); }));
    }
    ~WithRawCallbackMethod_ExecuteQuery() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable synchronous version of this method
    ::grpc::Status ExecuteQuery(::grpc::ServerContext* /*context*/, const ::clickhouse::grpc::QueryInfo* /*request*/, ::clickhouse::grpc::Result* /*response*/) override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
    virtual ::grpc::ServerUnaryReactor* ExecuteQuery(
      ::grpc::CallbackServerContext* /*context*/, const ::grpc::ByteBuffer* /*request*/, ::grpc::ByteBuffer* /*response*/)  { return nullptr; }
  };
  template <class BaseClass>
  class WithRawCallbackMethod_ExecuteQueryWithStreamInput : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithRawCallbackMethod_ExecuteQueryWithStreamInput() {
      ::grpc::Service::MarkMethodRawCallback(1,
          new ::grpc::internal::CallbackClientStreamingHandler< ::grpc::ByteBuffer, ::grpc::ByteBuffer>(
            [this](
                   ::grpc::CallbackServerContext* context, ::grpc::ByteBuffer* response) { return this->ExecuteQueryWithStreamInput(context, response); }));
    }
    ~WithRawCallbackMethod_ExecuteQueryWithStreamInput() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable synchronous version of this method
    ::grpc::Status ExecuteQueryWithStreamInput(::grpc::ServerContext* /*context*/, ::grpc::ServerReader< ::clickhouse::grpc::QueryInfo>* /*reader*/, ::clickhouse::grpc::Result* /*response*/) override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
    virtual ::grpc::ServerReadReactor< ::grpc::ByteBuffer>* ExecuteQueryWithStreamInput(
      ::grpc::CallbackServerContext* /*context*/, ::grpc::ByteBuffer* /*response*/)  { return nullptr; }
  };
  template <class BaseClass>
  class WithRawCallbackMethod_ExecuteQueryWithStreamOutput : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithRawCallbackMethod_ExecuteQueryWithStreamOutput() {
      ::grpc::Service::MarkMethodRawCallback(2,
          new ::grpc::internal::CallbackServerStreamingHandler< ::grpc::ByteBuffer, ::grpc::ByteBuffer>(
            [this](
                   ::grpc::CallbackServerContext* context, const::grpc::ByteBuffer* request) { return this->ExecuteQueryWithStreamOutput(context, request); }));
    }
    ~WithRawCallbackMethod_ExecuteQueryWithStreamOutput() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable synchronous version of this method
    ::grpc::Status ExecuteQueryWithStreamOutput(::grpc::ServerContext* /*context*/, const ::clickhouse::grpc::QueryInfo* /*request*/, ::grpc::ServerWriter< ::clickhouse::grpc::Result>* /*writer*/) override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
    virtual ::grpc::ServerWriteReactor< ::grpc::ByteBuffer>* ExecuteQueryWithStreamOutput(
      ::grpc::CallbackServerContext* /*context*/, const ::grpc::ByteBuffer* /*request*/)  { return nullptr; }
  };
  template <class BaseClass>
  class WithRawCallbackMethod_ExecuteQueryWithStreamIO : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithRawCallbackMethod_ExecuteQueryWithStreamIO() {
      ::grpc::Service::MarkMethodRawCallback(3,
          new ::grpc::internal::CallbackBidiHandler< ::grpc::ByteBuffer, ::grpc::ByteBuffer>(
            [this](
                   ::grpc::CallbackServerContext* context) { return this->ExecuteQueryWithStreamIO(context); }));
    }
    ~WithRawCallbackMethod_ExecuteQueryWithStreamIO() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable synchronous version of this method
    ::grpc::Status ExecuteQueryWithStreamIO(::grpc::ServerContext* /*context*/, ::grpc::ServerReaderWriter< ::clickhouse::grpc::Result, ::clickhouse::grpc::QueryInfo>* /*stream*/)  override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
    virtual ::grpc::ServerBidiReactor< ::grpc::ByteBuffer, ::grpc::ByteBuffer>* ExecuteQueryWithStreamIO(
      ::grpc::CallbackServerContext* /*context*/)
      { return nullptr; }
  };
  template <class BaseClass>
  class WithStreamedUnaryMethod_ExecuteQuery : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithStreamedUnaryMethod_ExecuteQuery() {
      ::grpc::Service::MarkMethodStreamed(0,
        new ::grpc::internal::StreamedUnaryHandler<
          ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>(
            [this](::grpc::ServerContext* context,
                   ::grpc::ServerUnaryStreamer<
                     ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>* streamer) {
                       return this->StreamedExecuteQuery(context,
                         streamer);
                  }));
    }
    ~WithStreamedUnaryMethod_ExecuteQuery() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable regular version of this method
    ::grpc::Status ExecuteQuery(::grpc::ServerContext* /*context*/, const ::clickhouse::grpc::QueryInfo* /*request*/, ::clickhouse::grpc::Result* /*response*/) override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
    // replace default version of method with streamed unary
    virtual ::grpc::Status StreamedExecuteQuery(::grpc::ServerContext* context, ::grpc::ServerUnaryStreamer< ::clickhouse::grpc::QueryInfo,::clickhouse::grpc::Result>* server_unary_streamer) = 0;
  };
  typedef WithStreamedUnaryMethod_ExecuteQuery<Service > StreamedUnaryService;
  template <class BaseClass>
  class WithSplitStreamingMethod_ExecuteQueryWithStreamOutput : public BaseClass {
   private:
    void BaseClassMustBeDerivedFromService(const Service* /*service*/) {}
   public:
    WithSplitStreamingMethod_ExecuteQueryWithStreamOutput() {
      ::grpc::Service::MarkMethodStreamed(2,
        new ::grpc::internal::SplitServerStreamingHandler<
          ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>(
            [this](::grpc::ServerContext* context,
                   ::grpc::ServerSplitStreamer<
                     ::clickhouse::grpc::QueryInfo, ::clickhouse::grpc::Result>* streamer) {
                       return this->StreamedExecuteQueryWithStreamOutput(context,
                         streamer);
                  }));
    }
    ~WithSplitStreamingMethod_ExecuteQueryWithStreamOutput() override {
      BaseClassMustBeDerivedFromService(this);
    }
    // disable regular version of this method
    ::grpc::Status ExecuteQueryWithStreamOutput(::grpc::ServerContext* /*context*/, const ::clickhouse::grpc::QueryInfo* /*request*/, ::grpc::ServerWriter< ::clickhouse::grpc::Result>* /*writer*/) override {
      abort();
      return ::grpc::Status(::grpc::StatusCode::UNIMPLEMENTED, "");
    }
    // replace default version of method with split streamed
    virtual ::grpc::Status StreamedExecuteQueryWithStreamOutput(::grpc::ServerContext* context, ::grpc::ServerSplitStreamer< ::clickhouse::grpc::QueryInfo,::clickhouse::grpc::Result>* server_split_streamer) = 0;
  };
  typedef WithSplitStreamingMethod_ExecuteQueryWithStreamOutput<Service > SplitStreamedService;
  typedef WithStreamedUnaryMethod_ExecuteQuery<WithSplitStreamingMethod_ExecuteQueryWithStreamOutput<Service > > StreamedService;
};

}  // namespace grpc
}  // namespace clickhouse


#include <grpcpp/ports_undef.inc>
#endif  // GRPC_clickhouse_5fgrpc_2eproto__INCLUDED
