// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: google/cloud/bigquery/storage/v1/annotations.proto
// Protobuf C++ Version: 7.35.1

#include "google/cloud/bigquery/storage/v1/annotations.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
namespace google {
namespace cloud {
namespace bigquery {
namespace storage {
namespace v1 {
}  // namespace v1
}  // namespace storage
}  // namespace bigquery
}  // namespace cloud
}  // namespace google
static constexpr const ::_pb::EnumDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_enum_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2fannotations_2eproto = nullptr;
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2fannotations_2eproto = nullptr;
const ::uint32_t TableStruct_google_2fcloud_2fbigquery_2fstorage_2fv1_2fannotations_2eproto::offsets[1] = {};
static constexpr ::_pbi::MigrationSchema* PROTOBUF_NULLABLE schemas = nullptr;
static constexpr ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const* PROTOBUF_NULLABLE
    file_message_globals = nullptr;
const char descriptor_table_protodef_google_2fcloud_2fbigquery_2fstorage_2fv1_2fannotations_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n2google/cloud/bigquery/storage/v1/annot"
    "ations.proto\022 google.cloud.bigquery.stor"
    "age.v1\032 google/protobuf/descriptor.proto"
    ":9\n\013column_name\022\035.google.protobuf.FieldO"
    "ptions\030\265\303\367\330\001 \001(\t\210\001\001B\300\001\n$com.google.cloud"
    ".bigquery.storage.v1B\020AnnotationsProtoP\001"
    "Z>cloud.google.com/go/bigquery/storage/a"
    "piv1/storagepb;storagepb\252\002 Google.Cloud."
    "BigQuery.Storage.V1\312\002 Google\\Cloud\\BigQu"
    "ery\\Storage\\V1b\006proto3"
};
static const ::_pbi::DescriptorTable* PROTOBUF_NONNULL const
    descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fannotations_2eproto_deps[1] = {
        &::descriptor_table_google_2fprotobuf_2fdescriptor_2eproto,
};
static ::absl::once_flag descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fannotations_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fannotations_2eproto = {
    false,
    false,
    382,
    descriptor_table_protodef_google_2fcloud_2fbigquery_2fstorage_2fv1_2fannotations_2eproto,
    "google/cloud/bigquery/storage/v1/annotations.proto",
    &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fannotations_2eproto_once,
    descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fannotations_2eproto_deps,
    1,
    0,
    schemas,
    file_message_globals,
    TableStruct_google_2fcloud_2fbigquery_2fstorage_2fv1_2fannotations_2eproto::offsets,
    file_level_enum_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2fannotations_2eproto,
    file_level_service_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2fannotations_2eproto,
};
namespace google {
namespace cloud {
namespace bigquery {
namespace storage {
namespace v1 {
const std::string column_name_default("");
PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 ::_pbi::ExtensionIdentifier<
        ::google::protobuf::FieldOptions, ::_pbi::StringTypeTraits, 9, false>
        column_name(kColumnNameFieldNumber, column_name_default);
// @@protoc_insertion_point(namespace_scope)
}  // namespace v1
}  // namespace storage
}  // namespace bigquery
}  // namespace cloud
}  // namespace google
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fannotations_2eproto),
        ::_pbi::ExtensionSet::RegisterExtension(
            &::google::protobuf::FieldOptions::default_instance(), 454943157, 9,
            false, false, /*is_utf8=*/true),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
