// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: google/pubsub/v1/schema.proto
// Protobuf C++ Version: 7.35.1

#include "google/pubsub/v1/schema.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
#ifdef PROTOBUF_MESSAGE_GLOBALS
namespace {
PROTOBUF_CONSTINIT ::google::protobuf::internal::ReflectionData
    file_reflection_data[] = {
        // ::google::pubsub::v1::Schema
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::CreateSchemaRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::GetSchemaRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ListSchemasRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ListSchemasResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ListSchemaRevisionsRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ListSchemaRevisionsResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::CommitSchemaRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::RollbackSchemaRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::DeleteSchemaRevisionRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::DeleteSchemaRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ValidateSchemaRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ValidateSchemaResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ValidateMessageRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ValidateMessageResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto, /* tracker*/ nullptr,},
};
}  // namespace
#endif
namespace google {
namespace pubsub {
namespace v1 {
class ValidateSchemaResponse::_Internal {
 public:
};

constexpr ValidateSchemaResponse::ParseTableT_ ValidateSchemaResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ValidateSchemaResponse,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ValidateSchemaResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr ValidateSchemaResponse::ValidateSchemaResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL ValidateSchemaResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ValidateSchemaResponse(arena);
}
constexpr auto ValidateSchemaResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ValidateSchemaResponse), alignof(ValidateSchemaResponse));
}
constexpr auto ValidateSchemaResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ValidateSchemaResponse::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<ValidateSchemaResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ValidateSchemaResponse::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<ValidateSchemaResponse>(), &ValidateSchemaResponse::ByteSizeLong,
              &ValidateSchemaResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ValidateSchemaResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[12],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ValidateSchemaResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ValidateSchemaResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ValidateSchemaResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ValidateSchemaResponse::InternalGenerateClassData_(
            _default, &ValidateSchemaResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ValidateSchemaResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ValidateSchemaResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ValidateSchemaResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ValidateSchemaResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ValidateSchemaResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ValidateSchemaResponseGlobalsTypeInternal ValidateSchemaResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ValidateSchemaResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ValidateSchemaResponse_globals_.GetClassData();
#else
  return ValidateSchemaResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ValidateMessageResponse::_Internal {
 public:
};

constexpr ValidateMessageResponse::ParseTableT_ ValidateMessageResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ValidateMessageResponse,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ValidateMessageResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr ValidateMessageResponse::ValidateMessageResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL ValidateMessageResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ValidateMessageResponse(arena);
}
constexpr auto ValidateMessageResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ValidateMessageResponse), alignof(ValidateMessageResponse));
}
constexpr auto ValidateMessageResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ValidateMessageResponse::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<ValidateMessageResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ValidateMessageResponse::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<ValidateMessageResponse>(), &ValidateMessageResponse::ByteSizeLong,
              &ValidateMessageResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ValidateMessageResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[14],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ValidateMessageResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ValidateMessageResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ValidateMessageResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ValidateMessageResponse::InternalGenerateClassData_(
            _default, &ValidateMessageResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ValidateMessageResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ValidateMessageResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ValidateMessageResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ValidateMessageResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ValidateMessageResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ValidateMessageResponseGlobalsTypeInternal ValidateMessageResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ValidateMessageResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ValidateMessageResponse_globals_.GetClassData();
#else
  return ValidateMessageResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class RollbackSchemaRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<RollbackSchemaRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(RollbackSchemaRequest, _impl_._has_bits_);
};

constexpr RollbackSchemaRequest::ParseTableT_ RollbackSchemaRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(RollbackSchemaRequest, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::RollbackSchemaRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string revision_id = 2 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(RollbackSchemaRequest, _impl_.revision_id_)}},
      // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(RollbackSchemaRequest, _impl_.name_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(RollbackSchemaRequest, _impl_.name_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string revision_id = 2 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(RollbackSchemaRequest, _impl_.revision_id_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\46\4\13\0\0\0\0\0"
      "google.pubsub.v1.RollbackSchemaRequest"
      "name"
      "revision_id"
    }},
  };
}


inline constexpr RollbackSchemaRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        revision_id_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr RollbackSchemaRequest::RollbackSchemaRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL RollbackSchemaRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) RollbackSchemaRequest(arena);
}
constexpr auto RollbackSchemaRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(RollbackSchemaRequest), alignof(RollbackSchemaRequest));
}
constexpr auto RollbackSchemaRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &RollbackSchemaRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<RollbackSchemaRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &RollbackSchemaRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<RollbackSchemaRequest>(), &RollbackSchemaRequest::ByteSizeLong,
              &RollbackSchemaRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(RollbackSchemaRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[8],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct RollbackSchemaRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr RollbackSchemaRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 RollbackSchemaRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(RollbackSchemaRequest::InternalGenerateClassData_(
            _default, &RollbackSchemaRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<RollbackSchemaRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~RollbackSchemaRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) RollbackSchemaRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<RollbackSchemaRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(RollbackSchemaRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST RollbackSchemaRequestGlobalsTypeInternal RollbackSchemaRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* RollbackSchemaRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return RollbackSchemaRequest_globals_.GetClassData();
#else
  return RollbackSchemaRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ListSchemasRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<ListSchemasRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ListSchemasRequest, _impl_._has_bits_);
};

constexpr ListSchemasRequest::ParseTableT_ ListSchemasRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ListSchemasRequest, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ListSchemasRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string page_token = 4;
      {::_pbi::TcParser::FastUS1,
       {34, 1, 0,
        PROTOBUF_FIELD_OFFSET(ListSchemasRequest, _impl_.page_token_)}},
      // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ListSchemasRequest, _impl_.parent_)}},
      // .google.pubsub.v1.SchemaView view = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ListSchemasRequest, _impl_.view_), 2>(),
       {16, 2, 0,
        PROTOBUF_FIELD_OFFSET(ListSchemasRequest, _impl_.view_)}},
      // int32 page_size = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ListSchemasRequest, _impl_.page_size_), 3>(),
       {24, 3, 0,
        PROTOBUF_FIELD_OFFSET(ListSchemasRequest, _impl_.page_size_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(ListSchemasRequest, _impl_.parent_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.pubsub.v1.SchemaView view = 2;
      {PROTOBUF_FIELD_OFFSET(ListSchemasRequest, _impl_.view_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // int32 page_size = 3;
      {PROTOBUF_FIELD_OFFSET(ListSchemasRequest, _impl_.page_size_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // string page_token = 4;
      {PROTOBUF_FIELD_OFFSET(ListSchemasRequest, _impl_.page_token_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\43\6\0\0\12\0\0\0"
      "google.pubsub.v1.ListSchemasRequest"
      "parent"
      "page_token"
    }},
  };
}


inline constexpr ListSchemasRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        parent_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        page_token_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        view_{static_cast< ::google::pubsub::v1::SchemaView >(0)},
        page_size_{0} {}

template <typename>
constexpr ListSchemasRequest::ListSchemasRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ListSchemasRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ListSchemasRequest(arena);
}
constexpr auto ListSchemasRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ListSchemasRequest), alignof(ListSchemasRequest));
}
constexpr auto ListSchemasRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ListSchemasRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ListSchemasRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ListSchemasRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ListSchemasRequest>(), &ListSchemasRequest::ByteSizeLong,
              &ListSchemasRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ListSchemasRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[3],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ListSchemasRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ListSchemasRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ListSchemasRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ListSchemasRequest::InternalGenerateClassData_(
            _default, &ListSchemasRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ListSchemasRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ListSchemasRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ListSchemasRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ListSchemasRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ListSchemasRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ListSchemasRequestGlobalsTypeInternal ListSchemasRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ListSchemasRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ListSchemasRequest_globals_.GetClassData();
#else
  return ListSchemasRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ListSchemaRevisionsRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<ListSchemaRevisionsRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsRequest, _impl_._has_bits_);
};

constexpr ListSchemaRevisionsRequest::ParseTableT_ ListSchemaRevisionsRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsRequest, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ListSchemaRevisionsRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string page_token = 4;
      {::_pbi::TcParser::FastUS1,
       {34, 1, 0,
        PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsRequest, _impl_.page_token_)}},
      // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsRequest, _impl_.name_)}},
      // .google.pubsub.v1.SchemaView view = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ListSchemaRevisionsRequest, _impl_.view_), 2>(),
       {16, 2, 0,
        PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsRequest, _impl_.view_)}},
      // int32 page_size = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ListSchemaRevisionsRequest, _impl_.page_size_), 3>(),
       {24, 3, 0,
        PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsRequest, _impl_.page_size_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsRequest, _impl_.name_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.pubsub.v1.SchemaView view = 2;
      {PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsRequest, _impl_.view_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // int32 page_size = 3;
      {PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsRequest, _impl_.page_size_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // string page_token = 4;
      {PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsRequest, _impl_.page_token_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\53\4\0\0\12\0\0\0"
      "google.pubsub.v1.ListSchemaRevisionsRequest"
      "name"
      "page_token"
    }},
  };
}


inline constexpr ListSchemaRevisionsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        page_token_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        view_{static_cast< ::google::pubsub::v1::SchemaView >(0)},
        page_size_{0} {}

template <typename>
constexpr ListSchemaRevisionsRequest::ListSchemaRevisionsRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ListSchemaRevisionsRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ListSchemaRevisionsRequest(arena);
}
constexpr auto ListSchemaRevisionsRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ListSchemaRevisionsRequest), alignof(ListSchemaRevisionsRequest));
}
constexpr auto ListSchemaRevisionsRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ListSchemaRevisionsRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ListSchemaRevisionsRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ListSchemaRevisionsRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ListSchemaRevisionsRequest>(), &ListSchemaRevisionsRequest::ByteSizeLong,
              &ListSchemaRevisionsRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[5],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ListSchemaRevisionsRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ListSchemaRevisionsRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ListSchemaRevisionsRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ListSchemaRevisionsRequest::InternalGenerateClassData_(
            _default, &ListSchemaRevisionsRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ListSchemaRevisionsRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ListSchemaRevisionsRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ListSchemaRevisionsRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ListSchemaRevisionsRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ListSchemaRevisionsRequestGlobalsTypeInternal ListSchemaRevisionsRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ListSchemaRevisionsRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ListSchemaRevisionsRequest_globals_.GetClassData();
#else
  return ListSchemaRevisionsRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class GetSchemaRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<GetSchemaRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(GetSchemaRequest, _impl_._has_bits_);
};

constexpr GetSchemaRequest::ParseTableT_ GetSchemaRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(GetSchemaRequest, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::GetSchemaRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.pubsub.v1.SchemaView view = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(GetSchemaRequest, _impl_.view_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(GetSchemaRequest, _impl_.view_)}},
      // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(GetSchemaRequest, _impl_.name_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(GetSchemaRequest, _impl_.name_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.pubsub.v1.SchemaView view = 2;
      {PROTOBUF_FIELD_OFFSET(GetSchemaRequest, _impl_.view_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
    }},
    // no aux_entries
    {{
      "\41\4\0\0\0\0\0\0"
      "google.pubsub.v1.GetSchemaRequest"
      "name"
    }},
  };
}


inline constexpr GetSchemaRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        view_{static_cast< ::google::pubsub::v1::SchemaView >(0)} {}

template <typename>
constexpr GetSchemaRequest::GetSchemaRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL GetSchemaRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) GetSchemaRequest(arena);
}
constexpr auto GetSchemaRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(GetSchemaRequest), alignof(GetSchemaRequest));
}
constexpr auto GetSchemaRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &GetSchemaRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<GetSchemaRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &GetSchemaRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<GetSchemaRequest>(), &GetSchemaRequest::ByteSizeLong,
              &GetSchemaRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(GetSchemaRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[2],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct GetSchemaRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr GetSchemaRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 GetSchemaRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(GetSchemaRequest::InternalGenerateClassData_(
            _default, &GetSchemaRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<GetSchemaRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~GetSchemaRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) GetSchemaRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<GetSchemaRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(GetSchemaRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST GetSchemaRequestGlobalsTypeInternal GetSchemaRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* GetSchemaRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return GetSchemaRequest_globals_.GetClassData();
#else
  return GetSchemaRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class DeleteSchemaRevisionRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<DeleteSchemaRevisionRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(DeleteSchemaRevisionRequest, _impl_._has_bits_);
};

constexpr DeleteSchemaRevisionRequest::ParseTableT_ DeleteSchemaRevisionRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(DeleteSchemaRevisionRequest, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::DeleteSchemaRevisionRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string revision_id = 2 [deprecated = true, (.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(DeleteSchemaRevisionRequest, _impl_.revision_id_)}},
      // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(DeleteSchemaRevisionRequest, _impl_.name_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(DeleteSchemaRevisionRequest, _impl_.name_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string revision_id = 2 [deprecated = true, (.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(DeleteSchemaRevisionRequest, _impl_.revision_id_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\54\4\13\0\0\0\0\0"
      "google.pubsub.v1.DeleteSchemaRevisionRequest"
      "name"
      "revision_id"
    }},
  };
}


inline constexpr DeleteSchemaRevisionRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        revision_id_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr DeleteSchemaRevisionRequest::DeleteSchemaRevisionRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL DeleteSchemaRevisionRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) DeleteSchemaRevisionRequest(arena);
}
constexpr auto DeleteSchemaRevisionRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(DeleteSchemaRevisionRequest), alignof(DeleteSchemaRevisionRequest));
}
constexpr auto DeleteSchemaRevisionRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &DeleteSchemaRevisionRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<DeleteSchemaRevisionRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &DeleteSchemaRevisionRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<DeleteSchemaRevisionRequest>(), &DeleteSchemaRevisionRequest::ByteSizeLong,
              &DeleteSchemaRevisionRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(DeleteSchemaRevisionRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[9],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct DeleteSchemaRevisionRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr DeleteSchemaRevisionRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 DeleteSchemaRevisionRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(DeleteSchemaRevisionRequest::InternalGenerateClassData_(
            _default, &DeleteSchemaRevisionRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<DeleteSchemaRevisionRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~DeleteSchemaRevisionRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) DeleteSchemaRevisionRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<DeleteSchemaRevisionRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(DeleteSchemaRevisionRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST DeleteSchemaRevisionRequestGlobalsTypeInternal DeleteSchemaRevisionRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* DeleteSchemaRevisionRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return DeleteSchemaRevisionRequest_globals_.GetClassData();
#else
  return DeleteSchemaRevisionRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class DeleteSchemaRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<DeleteSchemaRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(DeleteSchemaRequest, _impl_._has_bits_);
};

constexpr DeleteSchemaRequest::ParseTableT_ DeleteSchemaRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(DeleteSchemaRequest, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::DeleteSchemaRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(DeleteSchemaRequest, _impl_.name_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(DeleteSchemaRequest, _impl_.name_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\44\4\0\0\0\0\0\0"
      "google.pubsub.v1.DeleteSchemaRequest"
      "name"
    }},
  };
}


inline constexpr DeleteSchemaRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr DeleteSchemaRequest::DeleteSchemaRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL DeleteSchemaRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) DeleteSchemaRequest(arena);
}
constexpr auto DeleteSchemaRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(DeleteSchemaRequest), alignof(DeleteSchemaRequest));
}
constexpr auto DeleteSchemaRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &DeleteSchemaRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<DeleteSchemaRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &DeleteSchemaRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<DeleteSchemaRequest>(), &DeleteSchemaRequest::ByteSizeLong,
              &DeleteSchemaRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(DeleteSchemaRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[10],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct DeleteSchemaRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr DeleteSchemaRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 DeleteSchemaRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(DeleteSchemaRequest::InternalGenerateClassData_(
            _default, &DeleteSchemaRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<DeleteSchemaRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~DeleteSchemaRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) DeleteSchemaRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<DeleteSchemaRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(DeleteSchemaRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST DeleteSchemaRequestGlobalsTypeInternal DeleteSchemaRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* DeleteSchemaRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return DeleteSchemaRequest_globals_.GetClassData();
#else
  return DeleteSchemaRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Schema::_Internal {
 public:
  using HasBits = decltype(::std::declval<Schema>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Schema, _impl_._has_bits_);
};

constexpr Schema::ParseTableT_ Schema::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Schema, _impl_._has_bits_),
      0, // no _extensions_
      6, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967248,  // skipmap
      offsetof(ParseTableT_, field_entries),
      5,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::Schema>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string name = 1 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(Schema, _impl_.name_)}},
      // .google.pubsub.v1.Schema.Type type = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Schema, _impl_.type_), 4>(),
       {16, 4, 0,
        PROTOBUF_FIELD_OFFSET(Schema, _impl_.type_)}},
      // string definition = 3;
      {::_pbi::TcParser::FastUS1,
       {26, 1, 0,
        PROTOBUF_FIELD_OFFSET(Schema, _impl_.definition_)}},
      // string revision_id = 4 [(.google.api.field_behavior) = IMMUTABLE, (.google.api.field_behavior) = OUTPUT_ONLY];
      {::_pbi::TcParser::FastUS1,
       {34, 2, 0,
        PROTOBUF_FIELD_OFFSET(Schema, _impl_.revision_id_)}},
      {::_pbi::TcParser::MiniParse, {}},
      // .google.protobuf.Timestamp revision_create_time = 6 [(.google.api.field_behavior) = OUTPUT_ONLY];
      {::_pbi::TcParser::FastMtS1,
       {50, 3, 0,
        PROTOBUF_FIELD_OFFSET(Schema, _impl_.revision_create_time_)}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // string name = 1 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(Schema, _impl_.name_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.pubsub.v1.Schema.Type type = 2;
      {PROTOBUF_FIELD_OFFSET(Schema, _impl_.type_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // string definition = 3;
      {PROTOBUF_FIELD_OFFSET(Schema, _impl_.definition_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string revision_id = 4 [(.google.api.field_behavior) = IMMUTABLE, (.google.api.field_behavior) = OUTPUT_ONLY];
      {PROTOBUF_FIELD_OFFSET(Schema, _impl_.revision_id_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.protobuf.Timestamp revision_create_time = 6 [(.google.api.field_behavior) = OUTPUT_ONLY];
      {PROTOBUF_FIELD_OFFSET(Schema, _impl_.revision_create_time_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::Timestamp>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::Timestamp_globals_},
        #endif
    }},
    {{
      "\27\4\0\12\13\0\0\0"
      "google.pubsub.v1.Schema"
      "name"
      "definition"
      "revision_id"
    }},
  };
}


inline constexpr Schema::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        definition_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        revision_id_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        revision_create_time_{nullptr},
        type_{static_cast< ::google::pubsub::v1::Schema_Type >(0)} {}

template <typename>
constexpr Schema::Schema(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Schema::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Schema(arena);
}
constexpr auto Schema::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Schema), alignof(Schema));
}
constexpr auto Schema::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Schema::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<Schema>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Schema::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<Schema>(), &Schema::ByteSizeLong,
              &Schema::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Schema, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[0],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct SchemaGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr SchemaGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Schema_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Schema::InternalGenerateClassData_(
            _default, &Schema_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Schema>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~SchemaGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Schema _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Schema>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(SchemaGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST SchemaGlobalsTypeInternal Schema_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Schema_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Schema_globals_.GetClassData();
#else
  return Schema_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ValidateSchemaRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<ValidateSchemaRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ValidateSchemaRequest, _impl_._has_bits_);
};

constexpr ValidateSchemaRequest::ParseTableT_ ValidateSchemaRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ValidateSchemaRequest, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ValidateSchemaRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.pubsub.v1.Schema schema = 2 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(ValidateSchemaRequest, _impl_.schema_)}},
      // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ValidateSchemaRequest, _impl_.parent_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(ValidateSchemaRequest, _impl_.parent_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.pubsub.v1.Schema schema = 2 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(ValidateSchemaRequest, _impl_.schema_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::Schema>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::Schema_globals_},
        #endif
    }},
    {{
      "\46\6\0\0\0\0\0\0"
      "google.pubsub.v1.ValidateSchemaRequest"
      "parent"
    }},
  };
}


inline constexpr ValidateSchemaRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        parent_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        schema_{nullptr} {}

template <typename>
constexpr ValidateSchemaRequest::ValidateSchemaRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ValidateSchemaRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ValidateSchemaRequest(arena);
}
constexpr auto ValidateSchemaRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ValidateSchemaRequest), alignof(ValidateSchemaRequest));
}
constexpr auto ValidateSchemaRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ValidateSchemaRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ValidateSchemaRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ValidateSchemaRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ValidateSchemaRequest>(), &ValidateSchemaRequest::ByteSizeLong,
              &ValidateSchemaRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ValidateSchemaRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[11],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ValidateSchemaRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ValidateSchemaRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ValidateSchemaRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ValidateSchemaRequest::InternalGenerateClassData_(
            _default, &ValidateSchemaRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ValidateSchemaRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ValidateSchemaRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ValidateSchemaRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ValidateSchemaRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ValidateSchemaRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ValidateSchemaRequestGlobalsTypeInternal ValidateSchemaRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ValidateSchemaRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ValidateSchemaRequest_globals_.GetClassData();
#else
  return ValidateSchemaRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ValidateMessageRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<ValidateMessageRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ValidateMessageRequest, _impl_._has_bits_);
  static constexpr ::int32_t kOneofCaseOffset =
      PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ValidateMessageRequest, _impl_._oneof_case_);
};

constexpr ValidateMessageRequest::ParseTableT_ ValidateMessageRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ValidateMessageRequest, _impl_._has_bits_),
      0, // no _extensions_
      5, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967264,  // skipmap
      offsetof(ParseTableT_, field_entries),
      5,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ValidateMessageRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ValidateMessageRequest, _impl_.parent_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      // bytes message = 4;
      {::_pbi::TcParser::FastBS1,
       {34, 1, 0,
        PROTOBUF_FIELD_OFFSET(ValidateMessageRequest, _impl_.message_)}},
      // .google.pubsub.v1.Encoding encoding = 5;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ValidateMessageRequest, _impl_.encoding_), 2>(),
       {40, 2, 0,
        PROTOBUF_FIELD_OFFSET(ValidateMessageRequest, _impl_.encoding_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(ValidateMessageRequest, _impl_.parent_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string name = 2 [(.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(ValidateMessageRequest, _impl_.schema_spec_.name_), _Internal::kOneofCaseOffset + 0, 0, (0 | ::_fl::kFcOneof | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.pubsub.v1.Schema schema = 3;
      {PROTOBUF_FIELD_OFFSET(ValidateMessageRequest, _impl_.schema_spec_.schema_), _Internal::kOneofCaseOffset + 0, 0, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // bytes message = 4;
      {PROTOBUF_FIELD_OFFSET(ValidateMessageRequest, _impl_.message_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
      // .google.pubsub.v1.Encoding encoding = 5;
      {PROTOBUF_FIELD_OFFSET(ValidateMessageRequest, _impl_.encoding_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::Schema>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::Schema_globals_},
        #endif
    }},
    {{
      "\47\6\4\0\0\0\0\0"
      "google.pubsub.v1.ValidateMessageRequest"
      "parent"
      "name"
    }},
  };
}


inline constexpr ValidateMessageRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        parent_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        message_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        encoding_{static_cast< ::google::pubsub::v1::Encoding >(0)},
        schema_spec_{},
        _oneof_case_{} {}

template <typename>
constexpr ValidateMessageRequest::ValidateMessageRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ValidateMessageRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ValidateMessageRequest(arena);
}
constexpr auto ValidateMessageRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ValidateMessageRequest), alignof(ValidateMessageRequest));
}
constexpr auto ValidateMessageRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ValidateMessageRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ValidateMessageRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ValidateMessageRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ValidateMessageRequest>(), &ValidateMessageRequest::ByteSizeLong,
              &ValidateMessageRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ValidateMessageRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[13],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ValidateMessageRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ValidateMessageRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ValidateMessageRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ValidateMessageRequest::InternalGenerateClassData_(
            _default, &ValidateMessageRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ValidateMessageRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ValidateMessageRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ValidateMessageRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ValidateMessageRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ValidateMessageRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ValidateMessageRequestGlobalsTypeInternal ValidateMessageRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ValidateMessageRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ValidateMessageRequest_globals_.GetClassData();
#else
  return ValidateMessageRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ListSchemasResponse::_Internal {
 public:
  using HasBits = decltype(::std::declval<ListSchemasResponse>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ListSchemasResponse, _impl_._has_bits_);
};

constexpr ListSchemasResponse::ParseTableT_ ListSchemasResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ListSchemasResponse, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ListSchemasResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string next_page_token = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(ListSchemasResponse, _impl_.next_page_token_)}},
      // repeated .google.pubsub.v1.Schema schemas = 1;
      {::_pbi::TcParser::FastMtR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ListSchemasResponse, _impl_.schemas_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated .google.pubsub.v1.Schema schemas = 1;
      {PROTOBUF_FIELD_OFFSET(ListSchemasResponse, _impl_.schemas_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
      // string next_page_token = 2;
      {PROTOBUF_FIELD_OFFSET(ListSchemasResponse, _impl_.next_page_token_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::Schema>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::Schema_globals_},
        #endif
    }},
    {{
      "\44\0\17\0\0\0\0\0"
      "google.pubsub.v1.ListSchemasResponse"
      "next_page_token"
    }},
  };
}


inline constexpr ListSchemasResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        schemas_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::ListSchemasResponse,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemasResponse, _impl_.schemas_)>()
         }
        ,
        next_page_token_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr ListSchemasResponse::ListSchemasResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ListSchemasResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ListSchemasResponse(arena);
}
constexpr auto ListSchemasResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ListSchemasResponse), alignof(ListSchemasResponse));
}
constexpr auto ListSchemasResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ListSchemasResponse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ListSchemasResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ListSchemasResponse::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ListSchemasResponse>(), &ListSchemasResponse::ByteSizeLong,
              &ListSchemasResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ListSchemasResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[4],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ListSchemasResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ListSchemasResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ListSchemasResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ListSchemasResponse::InternalGenerateClassData_(
            _default, &ListSchemasResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ListSchemasResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ListSchemasResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ListSchemasResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ListSchemasResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ListSchemasResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ListSchemasResponseGlobalsTypeInternal ListSchemasResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ListSchemasResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ListSchemasResponse_globals_.GetClassData();
#else
  return ListSchemasResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ListSchemaRevisionsResponse::_Internal {
 public:
  using HasBits = decltype(::std::declval<ListSchemaRevisionsResponse>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsResponse, _impl_._has_bits_);
};

constexpr ListSchemaRevisionsResponse::ParseTableT_ ListSchemaRevisionsResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsResponse, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ListSchemaRevisionsResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string next_page_token = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsResponse, _impl_.next_page_token_)}},
      // repeated .google.pubsub.v1.Schema schemas = 1;
      {::_pbi::TcParser::FastMtR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsResponse, _impl_.schemas_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated .google.pubsub.v1.Schema schemas = 1;
      {PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsResponse, _impl_.schemas_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
      // string next_page_token = 2;
      {PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsResponse, _impl_.next_page_token_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::Schema>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::Schema_globals_},
        #endif
    }},
    {{
      "\54\0\17\0\0\0\0\0"
      "google.pubsub.v1.ListSchemaRevisionsResponse"
      "next_page_token"
    }},
  };
}


inline constexpr ListSchemaRevisionsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        schemas_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::ListSchemaRevisionsResponse,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemaRevisionsResponse, _impl_.schemas_)>()
         }
        ,
        next_page_token_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr ListSchemaRevisionsResponse::ListSchemaRevisionsResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ListSchemaRevisionsResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ListSchemaRevisionsResponse(arena);
}
constexpr auto ListSchemaRevisionsResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ListSchemaRevisionsResponse), alignof(ListSchemaRevisionsResponse));
}
constexpr auto ListSchemaRevisionsResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ListSchemaRevisionsResponse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ListSchemaRevisionsResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ListSchemaRevisionsResponse::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ListSchemaRevisionsResponse>(), &ListSchemaRevisionsResponse::ByteSizeLong,
              &ListSchemaRevisionsResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[6],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ListSchemaRevisionsResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ListSchemaRevisionsResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ListSchemaRevisionsResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ListSchemaRevisionsResponse::InternalGenerateClassData_(
            _default, &ListSchemaRevisionsResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ListSchemaRevisionsResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ListSchemaRevisionsResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ListSchemaRevisionsResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ListSchemaRevisionsResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ListSchemaRevisionsResponseGlobalsTypeInternal ListSchemaRevisionsResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ListSchemaRevisionsResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ListSchemaRevisionsResponse_globals_.GetClassData();
#else
  return ListSchemaRevisionsResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class CreateSchemaRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<CreateSchemaRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(CreateSchemaRequest, _impl_._has_bits_);
};

constexpr CreateSchemaRequest::ParseTableT_ CreateSchemaRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(CreateSchemaRequest, _impl_._has_bits_),
      0, // no _extensions_
      3, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::CreateSchemaRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(CreateSchemaRequest, _impl_.parent_)}},
      // .google.pubsub.v1.Schema schema = 2 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastMtS1,
       {18, 2, 0,
        PROTOBUF_FIELD_OFFSET(CreateSchemaRequest, _impl_.schema_)}},
      // string schema_id = 3;
      {::_pbi::TcParser::FastUS1,
       {26, 1, 0,
        PROTOBUF_FIELD_OFFSET(CreateSchemaRequest, _impl_.schema_id_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(CreateSchemaRequest, _impl_.parent_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.pubsub.v1.Schema schema = 2 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(CreateSchemaRequest, _impl_.schema_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // string schema_id = 3;
      {PROTOBUF_FIELD_OFFSET(CreateSchemaRequest, _impl_.schema_id_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::Schema>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::Schema_globals_},
        #endif
    }},
    {{
      "\44\6\0\11\0\0\0\0"
      "google.pubsub.v1.CreateSchemaRequest"
      "parent"
      "schema_id"
    }},
  };
}


inline constexpr CreateSchemaRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        parent_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        schema_id_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        schema_{nullptr} {}

template <typename>
constexpr CreateSchemaRequest::CreateSchemaRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL CreateSchemaRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) CreateSchemaRequest(arena);
}
constexpr auto CreateSchemaRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(CreateSchemaRequest), alignof(CreateSchemaRequest));
}
constexpr auto CreateSchemaRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &CreateSchemaRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<CreateSchemaRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &CreateSchemaRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<CreateSchemaRequest>(), &CreateSchemaRequest::ByteSizeLong,
              &CreateSchemaRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(CreateSchemaRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[1],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct CreateSchemaRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr CreateSchemaRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 CreateSchemaRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(CreateSchemaRequest::InternalGenerateClassData_(
            _default, &CreateSchemaRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<CreateSchemaRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~CreateSchemaRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) CreateSchemaRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<CreateSchemaRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(CreateSchemaRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST CreateSchemaRequestGlobalsTypeInternal CreateSchemaRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* CreateSchemaRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return CreateSchemaRequest_globals_.GetClassData();
#else
  return CreateSchemaRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class CommitSchemaRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<CommitSchemaRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(CommitSchemaRequest, _impl_._has_bits_);
};

constexpr CommitSchemaRequest::ParseTableT_ CommitSchemaRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(CommitSchemaRequest, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::CommitSchemaRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.pubsub.v1.Schema schema = 2 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(CommitSchemaRequest, _impl_.schema_)}},
      // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(CommitSchemaRequest, _impl_.name_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(CommitSchemaRequest, _impl_.name_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.pubsub.v1.Schema schema = 2 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(CommitSchemaRequest, _impl_.schema_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::Schema>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::Schema_globals_},
        #endif
    }},
    {{
      "\44\4\0\0\0\0\0\0"
      "google.pubsub.v1.CommitSchemaRequest"
      "name"
    }},
  };
}


inline constexpr CommitSchemaRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        schema_{nullptr} {}

template <typename>
constexpr CommitSchemaRequest::CommitSchemaRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL CommitSchemaRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) CommitSchemaRequest(arena);
}
constexpr auto CommitSchemaRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(CommitSchemaRequest), alignof(CommitSchemaRequest));
}
constexpr auto CommitSchemaRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &CommitSchemaRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<CommitSchemaRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &CommitSchemaRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<CommitSchemaRequest>(), &CommitSchemaRequest::ByteSizeLong,
              &CommitSchemaRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(CommitSchemaRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[7],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct CommitSchemaRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr CommitSchemaRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 CommitSchemaRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(CommitSchemaRequest::InternalGenerateClassData_(
            _default, &CommitSchemaRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<CommitSchemaRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~CommitSchemaRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) CommitSchemaRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<CommitSchemaRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(CommitSchemaRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST CommitSchemaRequestGlobalsTypeInternal CommitSchemaRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* CommitSchemaRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return CommitSchemaRequest_globals_.GetClassData();
#else
  return CommitSchemaRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
}  // namespace v1
}  // namespace pubsub
}  // namespace google
static const ::_pb::EnumDescriptor* PROTOBUF_NONNULL
    file_level_enum_descriptors_google_2fpubsub_2fv1_2fschema_2eproto[3];
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_google_2fpubsub_2fv1_2fschema_2eproto = nullptr;
const ::uint32_t
    TableStruct_google_2fpubsub_2fv1_2fschema_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Schema, _impl_._has_bits_),
        8, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Schema, _impl_.name_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Schema, _impl_.type_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Schema, _impl_.definition_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Schema, _impl_.revision_id_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Schema, _impl_.revision_create_time_),
        0,
        4,
        1,
        2,
        3,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CreateSchemaRequest, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CreateSchemaRequest, _impl_.parent_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CreateSchemaRequest, _impl_.schema_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CreateSchemaRequest, _impl_.schema_id_),
        0,
        2,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::GetSchemaRequest, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::GetSchemaRequest, _impl_.name_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::GetSchemaRequest, _impl_.view_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemasRequest, _impl_._has_bits_),
        7, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemasRequest, _impl_.parent_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemasRequest, _impl_.view_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemasRequest, _impl_.page_size_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemasRequest, _impl_.page_token_),
        0,
        2,
        3,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemasResponse, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemasResponse, _impl_.schemas_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemasResponse, _impl_.next_page_token_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemaRevisionsRequest, _impl_._has_bits_),
        7, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemaRevisionsRequest, _impl_.name_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemaRevisionsRequest, _impl_.view_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemaRevisionsRequest, _impl_.page_size_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemaRevisionsRequest, _impl_.page_token_),
        0,
        2,
        3,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemaRevisionsResponse, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemaRevisionsResponse, _impl_.schemas_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemaRevisionsResponse, _impl_.next_page_token_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CommitSchemaRequest, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CommitSchemaRequest, _impl_.name_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CommitSchemaRequest, _impl_.schema_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::RollbackSchemaRequest, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::RollbackSchemaRequest, _impl_.name_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::RollbackSchemaRequest, _impl_.revision_id_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::DeleteSchemaRevisionRequest, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::DeleteSchemaRevisionRequest, _impl_.name_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::DeleteSchemaRevisionRequest, _impl_.revision_id_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::DeleteSchemaRequest, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::DeleteSchemaRequest, _impl_.name_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ValidateSchemaRequest, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ValidateSchemaRequest, _impl_.parent_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ValidateSchemaRequest, _impl_.schema_),
        0,
        1,
        0x000, // bitmap
        0x085, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ValidateMessageRequest, _impl_._has_bits_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ValidateMessageRequest, _impl_._oneof_case_[0]),
        10, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ValidateMessageRequest, _impl_.parent_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ValidateMessageRequest, _impl_.schema_spec_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ValidateMessageRequest, _impl_.schema_spec_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ValidateMessageRequest, _impl_.message_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ValidateMessageRequest, _impl_.encoding_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ValidateMessageRequest, _impl_.schema_spec_),
        0,
        ~0u,
        ~0u,
        1,
        2,
        0x000, // bitmap
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::google::pubsub::v1::Schema)},
        {13, sizeof(::google::pubsub::v1::CreateSchemaRequest)},
        {22, sizeof(::google::pubsub::v1::GetSchemaRequest)},
        {29, sizeof(::google::pubsub::v1::ListSchemasRequest)},
        {40, sizeof(::google::pubsub::v1::ListSchemasResponse)},
        {47, sizeof(::google::pubsub::v1::ListSchemaRevisionsRequest)},
        {58, sizeof(::google::pubsub::v1::ListSchemaRevisionsResponse)},
        {65, sizeof(::google::pubsub::v1::CommitSchemaRequest)},
        {72, sizeof(::google::pubsub::v1::RollbackSchemaRequest)},
        {79, sizeof(::google::pubsub::v1::DeleteSchemaRevisionRequest)},
        {86, sizeof(::google::pubsub::v1::DeleteSchemaRequest)},
        {91, sizeof(::google::pubsub::v1::ValidateSchemaRequest)},
        {98, sizeof(::google::pubsub::v1::ValidateSchemaResponse)},
        {99, sizeof(::google::pubsub::v1::ValidateMessageRequest)},
        {114, sizeof(::google::pubsub::v1::ValidateMessageResponse)},
};
static const ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const
    file_message_globals[] = {
        &::google::pubsub::v1::Schema_globals_,
        &::google::pubsub::v1::CreateSchemaRequest_globals_,
        &::google::pubsub::v1::GetSchemaRequest_globals_,
        &::google::pubsub::v1::ListSchemasRequest_globals_,
        &::google::pubsub::v1::ListSchemasResponse_globals_,
        &::google::pubsub::v1::ListSchemaRevisionsRequest_globals_,
        &::google::pubsub::v1::ListSchemaRevisionsResponse_globals_,
        &::google::pubsub::v1::CommitSchemaRequest_globals_,
        &::google::pubsub::v1::RollbackSchemaRequest_globals_,
        &::google::pubsub::v1::DeleteSchemaRevisionRequest_globals_,
        &::google::pubsub::v1::DeleteSchemaRequest_globals_,
        &::google::pubsub::v1::ValidateSchemaRequest_globals_,
        &::google::pubsub::v1::ValidateSchemaResponse_globals_,
        &::google::pubsub::v1::ValidateMessageRequest_globals_,
        &::google::pubsub::v1::ValidateMessageResponse_globals_,
};
const char descriptor_table_protodef_google_2fpubsub_2fv1_2fschema_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n\035google/pubsub/v1/schema.proto\022\020google."
    "pubsub.v1\032\034google/api/annotations.proto\032"
    "\027google/api/client.proto\032\037google/api/fie"
    "ld_behavior.proto\032\031google/api/resource.p"
    "roto\032\033google/protobuf/empty.proto\032\037googl"
    "e/protobuf/timestamp.proto\"\276\002\n\006Schema\022\022\n"
    "\004name\030\001 \001(\tB\004\342A\001\002\022+\n\004type\030\002 \001(\0162\035.google"
    ".pubsub.v1.Schema.Type\022\022\n\ndefinition\030\003 \001"
    "(\t\022\032\n\013revision_id\030\004 \001(\tB\005\342A\002\005\003\022>\n\024revisi"
    "on_create_time\030\006 \001(\0132\032.google.protobuf.T"
    "imestampB\004\342A\001\003\";\n\004Type\022\024\n\020TYPE_UNSPECIFI"
    "ED\020\000\022\023\n\017PROTOCOL_BUFFER\020\001\022\010\n\004AVRO\020\002:F\352AC"
    "\n\034pubsub.googleapis.com/Schema\022#projects"
    "/{project}/schemas/{schema}\"\217\001\n\023CreateSc"
    "hemaRequest\0225\n\006parent\030\001 \001(\tB%\342A\001\002\372A\036\022\034pu"
    "bsub.googleapis.com/Schema\022.\n\006schema\030\002 \001"
    "(\0132\030.google.pubsub.v1.SchemaB\004\342A\001\002\022\021\n\tsc"
    "hema_id\030\003 \001(\t\"s\n\020GetSchemaRequest\0223\n\004nam"
    "e\030\001 \001(\tB%\342A\001\002\372A\036\n\034pubsub.googleapis.com/"
    "Schema\022*\n\004view\030\002 \001(\0162\034.google.pubsub.v1."
    "SchemaView\"\255\001\n\022ListSchemasRequest\022D\n\006par"
    "ent\030\001 \001(\tB4\342A\001\002\372A-\n+cloudresourcemanager"
    ".googleapis.com/Project\022*\n\004view\030\002 \001(\0162\034."
    "google.pubsub.v1.SchemaView\022\021\n\tpage_size"
    "\030\003 \001(\005\022\022\n\npage_token\030\004 \001(\t\"Y\n\023ListSchema"
    "sResponse\022)\n\007schemas\030\001 \003(\0132\030.google.pubs"
    "ub.v1.Schema\022\027\n\017next_page_token\030\002 \001(\t\"\244\001"
    "\n\032ListSchemaRevisionsRequest\0223\n\004name\030\001 \001"
    "(\tB%\342A\001\002\372A\036\n\034pubsub.googleapis.com/Schem"
    "a\022*\n\004view\030\002 \001(\0162\034.google.pubsub.v1.Schem"
    "aView\022\021\n\tpage_size\030\003 \001(\005\022\022\n\npage_token\030\004"
    " \001(\t\"a\n\033ListSchemaRevisionsResponse\022)\n\007s"
    "chemas\030\001 \003(\0132\030.google.pubsub.v1.Schema\022\027"
    "\n\017next_page_token\030\002 \001(\t\"z\n\023CommitSchemaR"
    "equest\0223\n\004name\030\001 \001(\tB%\342A\001\002\372A\036\n\034pubsub.go"
    "ogleapis.com/Schema\022.\n\006schema\030\002 \001(\0132\030.go"
    "ogle.pubsub.v1.SchemaB\004\342A\001\002\"g\n\025RollbackS"
    "chemaRequest\0223\n\004name\030\001 \001(\tB%\342A\001\002\372A\036\n\034pub"
    "sub.googleapis.com/Schema\022\031\n\013revision_id"
    "\030\002 \001(\tB\004\342A\001\002\"o\n\033DeleteSchemaRevisionRequ"
    "est\0223\n\004name\030\001 \001(\tB%\342A\001\002\372A\036\n\034pubsub.googl"
    "eapis.com/Schema\022\033\n\013revision_id\030\002 \001(\tB\006\030"
    "\001\342A\001\001\"J\n\023DeleteSchemaRequest\0223\n\004name\030\001 \001"
    "(\tB%\342A\001\002\372A\036\n\034pubsub.googleapis.com/Schem"
    "a\"\215\001\n\025ValidateSchemaRequest\022D\n\006parent\030\001 "
    "\001(\tB4\342A\001\002\372A-\n+cloudresourcemanager.googl"
    "eapis.com/Project\022.\n\006schema\030\002 \001(\0132\030.goog"
    "le.pubsub.v1.SchemaB\004\342A\001\002\"\030\n\026ValidateSch"
    "emaResponse\"\213\002\n\026ValidateMessageRequest\022D"
    "\n\006parent\030\001 \001(\tB4\342A\001\002\372A-\n+cloudresourcema"
    "nager.googleapis.com/Project\0221\n\004name\030\002 \001"
    "(\tB!\372A\036\n\034pubsub.googleapis.com/SchemaH\000\022"
    "*\n\006schema\030\003 \001(\0132\030.google.pubsub.v1.Schem"
    "aH\000\022\017\n\007message\030\004 \001(\014\022,\n\010encoding\030\005 \001(\0162\032"
    ".google.pubsub.v1.EncodingB\r\n\013schema_spe"
    "c\"\031\n\027ValidateMessageResponse*>\n\nSchemaVi"
    "ew\022\033\n\027SCHEMA_VIEW_UNSPECIFIED\020\000\022\t\n\005BASIC"
    "\020\001\022\010\n\004FULL\020\002*:\n\010Encoding\022\030\n\024ENCODING_UNS"
    "PECIFIED\020\000\022\010\n\004JSON\020\001\022\n\n\006BINARY\020\0022\210\r\n\rSch"
    "emaService\022\232\001\n\014CreateSchema\022%.google.pub"
    "sub.v1.CreateSchemaRequest\032\030.google.pubs"
    "ub.v1.Schema\"I\332A\027parent,schema,schema_id"
    "\202\323\344\223\002)\"\037/v1/{parent=projects/*}/schemas:"
    "\006schema\022y\n\tGetSchema\022\".google.pubsub.v1."
    "GetSchemaRequest\032\030.google.pubsub.v1.Sche"
    "ma\".\332A\004name\202\323\344\223\002!\022\037/v1/{name=projects/*/"
    "schemas/*}\022\214\001\n\013ListSchemas\022$.google.pubs"
    "ub.v1.ListSchemasRequest\032%.google.pubsub"
    ".v1.ListSchemasResponse\"0\332A\006parent\202\323\344\223\002!"
    "\022\037/v1/{parent=projects/*}/schemas\022\260\001\n\023Li"
    "stSchemaRevisions\022,.google.pubsub.v1.Lis"
    "tSchemaRevisionsRequest\032-.google.pubsub."
    "v1.ListSchemaRevisionsResponse\"<\332A\004name\202"
    "\323\344\223\002/\022-/v1/{name=projects/*/schemas/*}:l"
    "istRevisions\022\220\001\n\014CommitSchema\022%.google.p"
    "ubsub.v1.CommitSchemaRequest\032\030.google.pu"
    "bsub.v1.Schema\"\?\332A\013name,schema\202\323\344\223\002+\"&/v"
    "1/{name=projects/*/schemas/*}:commit:\001*\022"
    "\233\001\n\016RollbackSchema\022\'.google.pubsub.v1.Ro"
    "llbackSchemaRequest\032\030.google.pubsub.v1.S"
    "chema\"F\332A\020name,revision_id\202\323\344\223\002-\"(/v1/{n"
    "ame=projects/*/schemas/*}:rollback:\001*\022\252\001"
    "\n\024DeleteSchemaRevision\022-.google.pubsub.v"
    "1.DeleteSchemaRevisionRequest\032\030.google.p"
    "ubsub.v1.Schema\"I\332A\020name,revision_id\202\323\344\223"
    "\0020*./v1/{name=projects/*/schemas/*}:dele"
    "teRevision\022}\n\014DeleteSchema\022%.google.pubs"
    "ub.v1.DeleteSchemaRequest\032\026.google.proto"
    "buf.Empty\".\332A\004name\202\323\344\223\002!*\037/v1/{name=proj"
    "ects/*/schemas/*}\022\250\001\n\016ValidateSchema\022\'.g"
    "oogle.pubsub.v1.ValidateSchemaRequest\032(."
    "google.pubsub.v1.ValidateSchemaResponse\""
    "C\332A\rparent,schema\202\323\344\223\002-\"(/v1/{parent=pro"
    "jects/*}/schemas:validate:\001*\022\242\001\n\017Validat"
    "eMessage\022(.google.pubsub.v1.ValidateMess"
    "ageRequest\032).google.pubsub.v1.ValidateMe"
    "ssageResponse\":\202\323\344\223\0024\"//v1/{parent=proje"
    "cts/*}/schemas:validateMessage:\001*\032p\312A\025pu"
    "bsub.googleapis.com\322AUhttps://www.google"
    "apis.com/auth/cloud-platform,https://www"
    ".googleapis.com/auth/pubsubB\252\001\n\024com.goog"
    "le.pubsub.v1B\013SchemaProtoP\001Z2cloud.googl"
    "e.com/go/pubsub/apiv1/pubsubpb;pubsubpb\370"
    "\001\001\252\002\026Google.Cloud.PubSub.V1\312\002\026Google\\Clo"
    "ud\\PubSub\\V1\352\002\031Google::Cloud::PubSub::V1"
    "b\006proto3"
};
static const ::_pbi::DescriptorTable* PROTOBUF_NONNULL const
    descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto_deps[6] = {
        &::descriptor_table_google_2fapi_2fannotations_2eproto,
        &::descriptor_table_google_2fapi_2fclient_2eproto,
        &::descriptor_table_google_2fapi_2ffield_5fbehavior_2eproto,
        &::descriptor_table_google_2fapi_2fresource_2eproto,
        &::descriptor_table_google_2fprotobuf_2fempty_2eproto,
        &::descriptor_table_google_2fprotobuf_2ftimestamp_2eproto,
};
static ::absl::once_flag descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto = {
    false,
    false,
    4208,
    descriptor_table_protodef_google_2fpubsub_2fv1_2fschema_2eproto,
    "google/pubsub/v1/schema.proto",
    &descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto_once,
    descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto_deps,
    6,
    15,
    schemas,
    file_message_globals,
    TableStruct_google_2fpubsub_2fv1_2fschema_2eproto::offsets,
    file_level_enum_descriptors_google_2fpubsub_2fv1_2fschema_2eproto,
    file_level_service_descriptors_google_2fpubsub_2fv1_2fschema_2eproto,
};
namespace google {
namespace pubsub {
namespace v1 {
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
Schema_Type_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto);
  return file_level_enum_descriptors_google_2fpubsub_2fv1_2fschema_2eproto[0];
}
PROTOBUF_CONSTINIT const uint32_t Schema_Type_internal_data_[] = {
    196608u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
SchemaView_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto);
  return file_level_enum_descriptors_google_2fpubsub_2fv1_2fschema_2eproto[1];
}
PROTOBUF_CONSTINIT const uint32_t SchemaView_internal_data_[] = {
    196608u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
Encoding_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto);
  return file_level_enum_descriptors_google_2fpubsub_2fv1_2fschema_2eproto[2];
}
PROTOBUF_CONSTINIT const uint32_t Encoding_internal_data_[] = {
    196608u, 0u, };
// ===================================================================

void Schema::clear_revision_create_time() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.revision_create_time_ != nullptr) _impl_.revision_create_time_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
Schema::Schema(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Schema_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.Schema)
}
PROTOBUF_NDEBUG_INLINE Schema::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::Schema& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        name_(arena, from.name_),
        definition_(arena, from.definition_),
        revision_id_(arena, from.revision_id_) {}

Schema::Schema(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Schema& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Schema_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Schema* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.revision_create_time_ = (CheckHasBit(cached_has_bits, 0x00000008U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.revision_create_time_)
                : nullptr;
  _impl_.type_ = from._impl_.type_;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.Schema)
}
PROTOBUF_NDEBUG_INLINE Schema::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        name_(arena),
        definition_(arena),
        revision_id_(arena) {}

inline void Schema::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, revision_create_time_),
           0,
           offsetof(Impl_, type_) -
               offsetof(Impl_, revision_create_time_) +
               sizeof(Impl_::type_));
}
Schema::~Schema() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.Schema)
  SharedDtor(*this);
}
inline void Schema::SharedDtor(MessageLite& self) {
  Schema& this_ = static_cast<Schema&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.name_.Destroy();
  this_._impl_.definition_.Destroy();
  this_._impl_.revision_id_.Destroy();
  delete this_._impl_.revision_create_time_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Schema_class_data_ =
        Schema::InternalGenerateClassData_(Schema_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Schema::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Schema_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Schema_class_data_.tc_table);
  return Schema_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Schema::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Schema_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Schema_globals_));
  return Schema_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Schema::ParseTableT_
    Schema::_table_ =
        Schema::InternalGenerateParseTable_(Schema_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Schema::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.Schema)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.name_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.definition_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.revision_id_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(_impl_.revision_create_time_ != nullptr);
      _impl_.revision_create_time_->Clear();
    }
  }
  _impl_.type_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Schema::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Schema& this_ = static_cast<const Schema&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Schema::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Schema& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.Schema)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string name = 1 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_name().empty()) {
      const ::std::string& _s = this_._internal_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Schema.name");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // .google.pubsub.v1.Schema.Type type = 2;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_type() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          2, this_._internal_type(), target);
    }
  }

  // string definition = 3;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_definition().empty()) {
      const ::std::string& _s = this_._internal_definition();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Schema.definition");
      target = stream->WriteStringMaybeAliased(3, _s, target);
    }
  }

  // string revision_id = 4 [(.google.api.field_behavior) = IMMUTABLE, (.google.api.field_behavior) = OUTPUT_ONLY];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_revision_id().empty()) {
      const ::std::string& _s = this_._internal_revision_id();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Schema.revision_id");
      target = stream->WriteStringMaybeAliased(4, _s, target);
    }
  }

  // .google.protobuf.Timestamp revision_create_time = 6 [(.google.api.field_behavior) = OUTPUT_ONLY];
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        6, *this_._impl_.revision_create_time_, this_._impl_.revision_create_time_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.Schema)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Schema::ByteSizeLong(const MessageLite& base) {
  const Schema& this_ = static_cast<const Schema&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Schema::ByteSizeLong() const {
  const Schema& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.Schema)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    // string name = 1 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_name());
      }
    }
    // string definition = 3;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_definition().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_definition());
      }
    }
    // string revision_id = 4 [(.google.api.field_behavior) = IMMUTABLE, (.google.api.field_behavior) = OUTPUT_ONLY];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!this_._internal_revision_id().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_revision_id());
      }
    }
    // .google.protobuf.Timestamp revision_create_time = 6 [(.google.api.field_behavior) = OUTPUT_ONLY];
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.revision_create_time_);
    }
    // .google.pubsub.v1.Schema.Type type = 2;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_type() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_type());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Schema::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Schema*>(&to_msg);
  auto& from = static_cast<const Schema&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.Schema)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_name().empty()) {
        _this->_internal_set_name(from._internal_name());
      } else {
        if (_this->_impl_.name_.IsDefault()) {
          _this->_internal_set_name("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_definition().empty()) {
        _this->_internal_set_definition(from._internal_definition());
      } else {
        if (_this->_impl_.definition_.IsDefault()) {
          _this->_internal_set_definition("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!from._internal_revision_id().empty()) {
        _this->_internal_set_revision_id(from._internal_revision_id());
      } else {
        if (_this->_impl_.revision_id_.IsDefault()) {
          _this->_internal_set_revision_id("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(from._impl_.revision_create_time_ != nullptr);
      if (_this->_impl_.revision_create_time_ == nullptr) {
        _this->_impl_.revision_create_time_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.revision_create_time_);
      } else {
        _this->_impl_.revision_create_time_->MergeFrom(*from._impl_.revision_create_time_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_type() != 0) {
        _this->_impl_.type_ = from._impl_.type_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Schema::CopyFrom(const Schema& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.Schema)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Schema::InternalSwap(Schema* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.name_, &other->_impl_.name_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.definition_, &other->_impl_.definition_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.revision_id_, &other->_impl_.revision_id_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Schema, _impl_.type_)
      + sizeof(Schema::_impl_.type_)
      - PROTOBUF_FIELD_OFFSET(Schema, _impl_.revision_create_time_)>(
          reinterpret_cast<char*>(&_impl_.revision_create_time_),
          reinterpret_cast<char*>(&other->_impl_.revision_create_time_));
}

::google::protobuf::Metadata Schema::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

CreateSchemaRequest::CreateSchemaRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, CreateSchemaRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.CreateSchemaRequest)
}
PROTOBUF_NDEBUG_INLINE CreateSchemaRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::CreateSchemaRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        parent_(arena, from.parent_),
        schema_id_(arena, from.schema_id_) {}

CreateSchemaRequest::CreateSchemaRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const CreateSchemaRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, CreateSchemaRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  CreateSchemaRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.schema_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.schema_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.CreateSchemaRequest)
}
PROTOBUF_NDEBUG_INLINE CreateSchemaRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        parent_(arena),
        schema_id_(arena) {}

inline void CreateSchemaRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.schema_ = {};
}
CreateSchemaRequest::~CreateSchemaRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.CreateSchemaRequest)
  SharedDtor(*this);
}
inline void CreateSchemaRequest::SharedDtor(MessageLite& self) {
  CreateSchemaRequest& this_ = static_cast<CreateSchemaRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.parent_.Destroy();
  this_._impl_.schema_id_.Destroy();
  delete this_._impl_.schema_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull CreateSchemaRequest_class_data_ =
        CreateSchemaRequest::InternalGenerateClassData_(CreateSchemaRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CreateSchemaRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CreateSchemaRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(CreateSchemaRequest_class_data_.tc_table);
  return CreateSchemaRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CreateSchemaRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CreateSchemaRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&CreateSchemaRequest_globals_));
  return CreateSchemaRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const CreateSchemaRequest::ParseTableT_
    CreateSchemaRequest::_table_ =
        CreateSchemaRequest::InternalGenerateParseTable_(CreateSchemaRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void CreateSchemaRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.CreateSchemaRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.parent_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.schema_id_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(_impl_.schema_ != nullptr);
      _impl_.schema_->Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL CreateSchemaRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const CreateSchemaRequest& this_ = static_cast<const CreateSchemaRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL CreateSchemaRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const CreateSchemaRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.CreateSchemaRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_parent().empty()) {
      const ::std::string& _s = this_._internal_parent();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.CreateSchemaRequest.parent");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // .google.pubsub.v1.Schema schema = 2 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.schema_, this_._impl_.schema_->GetCachedSize(), target,
        stream);
  }

  // string schema_id = 3;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_schema_id().empty()) {
      const ::std::string& _s = this_._internal_schema_id();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.CreateSchemaRequest.schema_id");
      target = stream->WriteStringMaybeAliased(3, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.CreateSchemaRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t CreateSchemaRequest::ByteSizeLong(const MessageLite& base) {
  const CreateSchemaRequest& this_ = static_cast<const CreateSchemaRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t CreateSchemaRequest::ByteSizeLong() const {
  const CreateSchemaRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.CreateSchemaRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_parent().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_parent());
      }
    }
    // string schema_id = 3;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_schema_id().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_schema_id());
      }
    }
    // .google.pubsub.v1.Schema schema = 2 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.schema_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void CreateSchemaRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<CreateSchemaRequest*>(&to_msg);
  auto& from = static_cast<const CreateSchemaRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.CreateSchemaRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_parent().empty()) {
        _this->_internal_set_parent(from._internal_parent());
      } else {
        if (_this->_impl_.parent_.IsDefault()) {
          _this->_internal_set_parent("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_schema_id().empty()) {
        _this->_internal_set_schema_id(from._internal_schema_id());
      } else {
        if (_this->_impl_.schema_id_.IsDefault()) {
          _this->_internal_set_schema_id("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.schema_ != nullptr);
      if (_this->_impl_.schema_ == nullptr) {
        _this->_impl_.schema_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.schema_);
      } else {
        _this->_impl_.schema_->MergeFrom(*from._impl_.schema_);
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void CreateSchemaRequest::CopyFrom(const CreateSchemaRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.CreateSchemaRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void CreateSchemaRequest::InternalSwap(CreateSchemaRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.parent_, &other->_impl_.parent_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.schema_id_, &other->_impl_.schema_id_, arena);
  swap(_impl_.schema_, other->_impl_.schema_);
}

::google::protobuf::Metadata CreateSchemaRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

GetSchemaRequest::GetSchemaRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, GetSchemaRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.GetSchemaRequest)
}
PROTOBUF_NDEBUG_INLINE GetSchemaRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::GetSchemaRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        name_(arena, from.name_) {}

GetSchemaRequest::GetSchemaRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const GetSchemaRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, GetSchemaRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  GetSchemaRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.view_ = from._impl_.view_;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.GetSchemaRequest)
}
PROTOBUF_NDEBUG_INLINE GetSchemaRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        name_(arena) {}

inline void GetSchemaRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.view_ = {};
}
GetSchemaRequest::~GetSchemaRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.GetSchemaRequest)
  SharedDtor(*this);
}
inline void GetSchemaRequest::SharedDtor(MessageLite& self) {
  GetSchemaRequest& this_ = static_cast<GetSchemaRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.name_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull GetSchemaRequest_class_data_ =
        GetSchemaRequest::InternalGenerateClassData_(GetSchemaRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
GetSchemaRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&GetSchemaRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(GetSchemaRequest_class_data_.tc_table);
  return GetSchemaRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
GetSchemaRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&GetSchemaRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&GetSchemaRequest_globals_));
  return GetSchemaRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const GetSchemaRequest::ParseTableT_
    GetSchemaRequest::_table_ =
        GetSchemaRequest::InternalGenerateParseTable_(GetSchemaRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void GetSchemaRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.GetSchemaRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.name_.ClearNonDefaultToEmpty();
  }
  _impl_.view_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL GetSchemaRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const GetSchemaRequest& this_ = static_cast<const GetSchemaRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL GetSchemaRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const GetSchemaRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.GetSchemaRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_name().empty()) {
      const ::std::string& _s = this_._internal_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.GetSchemaRequest.name");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // .google.pubsub.v1.SchemaView view = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_view() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          2, this_._internal_view(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.GetSchemaRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t GetSchemaRequest::ByteSizeLong(const MessageLite& base) {
  const GetSchemaRequest& this_ = static_cast<const GetSchemaRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t GetSchemaRequest::ByteSizeLong() const {
  const GetSchemaRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.GetSchemaRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_name());
      }
    }
    // .google.pubsub.v1.SchemaView view = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_view() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_view());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void GetSchemaRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<GetSchemaRequest*>(&to_msg);
  auto& from = static_cast<const GetSchemaRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.GetSchemaRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_name().empty()) {
        _this->_internal_set_name(from._internal_name());
      } else {
        if (_this->_impl_.name_.IsDefault()) {
          _this->_internal_set_name("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_view() != 0) {
        _this->_impl_.view_ = from._impl_.view_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void GetSchemaRequest::CopyFrom(const GetSchemaRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.GetSchemaRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void GetSchemaRequest::InternalSwap(GetSchemaRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.name_, &other->_impl_.name_, arena);
  swap(_impl_.view_, other->_impl_.view_);
}

::google::protobuf::Metadata GetSchemaRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ListSchemasRequest::ListSchemasRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListSchemasRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ListSchemasRequest)
}
PROTOBUF_NDEBUG_INLINE ListSchemasRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::ListSchemasRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        parent_(arena, from.parent_),
        page_token_(arena, from.page_token_) {}

ListSchemasRequest::ListSchemasRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ListSchemasRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListSchemasRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ListSchemasRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, view_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, view_),
           offsetof(Impl_, page_size_) -
               offsetof(Impl_, view_) +
               sizeof(Impl_::page_size_));

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ListSchemasRequest)
}
PROTOBUF_NDEBUG_INLINE ListSchemasRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        parent_(arena),
        page_token_(arena) {}

inline void ListSchemasRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, view_),
           0,
           offsetof(Impl_, page_size_) -
               offsetof(Impl_, view_) +
               sizeof(Impl_::page_size_));
}
ListSchemasRequest::~ListSchemasRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.ListSchemasRequest)
  SharedDtor(*this);
}
inline void ListSchemasRequest::SharedDtor(MessageLite& self) {
  ListSchemasRequest& this_ = static_cast<ListSchemasRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.parent_.Destroy();
  this_._impl_.page_token_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ListSchemasRequest_class_data_ =
        ListSchemasRequest::InternalGenerateClassData_(ListSchemasRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListSchemasRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListSchemasRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ListSchemasRequest_class_data_.tc_table);
  return ListSchemasRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListSchemasRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListSchemasRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ListSchemasRequest_globals_));
  return ListSchemasRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ListSchemasRequest::ParseTableT_
    ListSchemasRequest::_table_ =
        ListSchemasRequest::InternalGenerateParseTable_(ListSchemasRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ListSchemasRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.ListSchemasRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.parent_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.page_token_.ClearNonDefaultToEmpty();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000000cU)) {
    ::memset(&_impl_.view_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.page_size_) -
        reinterpret_cast<char*>(&_impl_.view_)) + sizeof(_impl_.page_size_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ListSchemasRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ListSchemasRequest& this_ = static_cast<const ListSchemasRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ListSchemasRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ListSchemasRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.ListSchemasRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_parent().empty()) {
      const ::std::string& _s = this_._internal_parent();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListSchemasRequest.parent");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // .google.pubsub.v1.SchemaView view = 2;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_view() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          2, this_._internal_view(), target);
    }
  }

  // int32 page_size = 3;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_page_size() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<3>(
              stream, this_._internal_page_size(), target);
    }
  }

  // string page_token = 4;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_page_token().empty()) {
      const ::std::string& _s = this_._internal_page_token();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListSchemasRequest.page_token");
      target = stream->WriteStringMaybeAliased(4, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.ListSchemasRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ListSchemasRequest::ByteSizeLong(const MessageLite& base) {
  const ListSchemasRequest& this_ = static_cast<const ListSchemasRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ListSchemasRequest::ByteSizeLong() const {
  const ListSchemasRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.ListSchemasRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_parent().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_parent());
      }
    }
    // string page_token = 4;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_page_token().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_page_token());
      }
    }
    // .google.pubsub.v1.SchemaView view = 2;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_view() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_view());
      }
    }
    // int32 page_size = 3;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_page_size() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_page_size());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ListSchemasRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ListSchemasRequest*>(&to_msg);
  auto& from = static_cast<const ListSchemasRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.ListSchemasRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_parent().empty()) {
        _this->_internal_set_parent(from._internal_parent());
      } else {
        if (_this->_impl_.parent_.IsDefault()) {
          _this->_internal_set_parent("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_page_token().empty()) {
        _this->_internal_set_page_token(from._internal_page_token());
      } else {
        if (_this->_impl_.page_token_.IsDefault()) {
          _this->_internal_set_page_token("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_view() != 0) {
        _this->_impl_.view_ = from._impl_.view_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_page_size() != 0) {
        _this->_impl_.page_size_ = from._impl_.page_size_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ListSchemasRequest::CopyFrom(const ListSchemasRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.ListSchemasRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ListSchemasRequest::InternalSwap(ListSchemasRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.parent_, &other->_impl_.parent_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.page_token_, &other->_impl_.page_token_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(ListSchemasRequest, _impl_.page_size_)
      + sizeof(ListSchemasRequest::_impl_.page_size_)
      - PROTOBUF_FIELD_OFFSET(ListSchemasRequest, _impl_.view_)>(
          reinterpret_cast<char*>(&_impl_.view_),
          reinterpret_cast<char*>(&other->_impl_.view_));
}

::google::protobuf::Metadata ListSchemasRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ListSchemasResponse::ListSchemasResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListSchemasResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ListSchemasResponse)
}
PROTOBUF_NDEBUG_INLINE ListSchemasResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::ListSchemasResponse& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        schemas_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::ListSchemasResponse,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemasResponse, _impl_.schemas_)>()
          , from.schemas_
        }
        ,
        next_page_token_(arena, from.next_page_token_) {}

ListSchemasResponse::ListSchemasResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ListSchemasResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListSchemasResponse_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ListSchemasResponse* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ListSchemasResponse)
}
PROTOBUF_NDEBUG_INLINE ListSchemasResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        schemas_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::ListSchemasResponse,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemasResponse, _impl_.schemas_)>()
         }
        ,
        next_page_token_(arena) {}

inline void ListSchemasResponse::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
ListSchemasResponse::~ListSchemasResponse() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.ListSchemasResponse)
  SharedDtor(*this);
}
inline void ListSchemasResponse::SharedDtor(MessageLite& self) {
  ListSchemasResponse& this_ = static_cast<ListSchemasResponse&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.next_page_token_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ListSchemasResponse_class_data_ =
        ListSchemasResponse::InternalGenerateClassData_(ListSchemasResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListSchemasResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListSchemasResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ListSchemasResponse_class_data_.tc_table);
  return ListSchemasResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListSchemasResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListSchemasResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ListSchemasResponse_globals_));
  return ListSchemasResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ListSchemasResponse::ParseTableT_
    ListSchemasResponse::_table_ =
        ListSchemasResponse::InternalGenerateParseTable_(ListSchemasResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ListSchemasResponse::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.ListSchemasResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.schemas_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.next_page_token_.ClearNonDefaultToEmpty();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ListSchemasResponse::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ListSchemasResponse& this_ = static_cast<const ListSchemasResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ListSchemasResponse::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ListSchemasResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.ListSchemasResponse)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated .google.pubsub.v1.Schema schemas = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_schemas_size());
         i < n; i++) {
      const auto& repfield = this_._internal_schemas().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              1, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  // string next_page_token = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_next_page_token().empty()) {
      const ::std::string& _s = this_._internal_next_page_token();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListSchemasResponse.next_page_token");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.ListSchemasResponse)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ListSchemasResponse::ByteSizeLong(const MessageLite& base) {
  const ListSchemasResponse& this_ = static_cast<const ListSchemasResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ListSchemasResponse::ByteSizeLong() const {
  const ListSchemasResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.ListSchemasResponse)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // repeated .google.pubsub.v1.Schema schemas = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_schemas_size();
      for (const auto& msg : this_._internal_schemas()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
    // string next_page_token = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_next_page_token().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_next_page_token());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ListSchemasResponse::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ListSchemasResponse*>(&to_msg);
  auto& from = static_cast<const ListSchemasResponse&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.ListSchemasResponse)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_schemas()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_schemas());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_next_page_token().empty()) {
        _this->_internal_set_next_page_token(from._internal_next_page_token());
      } else {
        if (_this->_impl_.next_page_token_.IsDefault()) {
          _this->_internal_set_next_page_token("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ListSchemasResponse::CopyFrom(const ListSchemasResponse& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.ListSchemasResponse)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ListSchemasResponse::InternalSwap(ListSchemasResponse* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.schemas_.InternalSwap(&other->_impl_.schemas_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.next_page_token_, &other->_impl_.next_page_token_, arena);
}

::google::protobuf::Metadata ListSchemasResponse::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ListSchemaRevisionsRequest::ListSchemaRevisionsRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListSchemaRevisionsRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ListSchemaRevisionsRequest)
}
PROTOBUF_NDEBUG_INLINE ListSchemaRevisionsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::ListSchemaRevisionsRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        name_(arena, from.name_),
        page_token_(arena, from.page_token_) {}

ListSchemaRevisionsRequest::ListSchemaRevisionsRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ListSchemaRevisionsRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListSchemaRevisionsRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ListSchemaRevisionsRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, view_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, view_),
           offsetof(Impl_, page_size_) -
               offsetof(Impl_, view_) +
               sizeof(Impl_::page_size_));

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ListSchemaRevisionsRequest)
}
PROTOBUF_NDEBUG_INLINE ListSchemaRevisionsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        name_(arena),
        page_token_(arena) {}

inline void ListSchemaRevisionsRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, view_),
           0,
           offsetof(Impl_, page_size_) -
               offsetof(Impl_, view_) +
               sizeof(Impl_::page_size_));
}
ListSchemaRevisionsRequest::~ListSchemaRevisionsRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.ListSchemaRevisionsRequest)
  SharedDtor(*this);
}
inline void ListSchemaRevisionsRequest::SharedDtor(MessageLite& self) {
  ListSchemaRevisionsRequest& this_ = static_cast<ListSchemaRevisionsRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.name_.Destroy();
  this_._impl_.page_token_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ListSchemaRevisionsRequest_class_data_ =
        ListSchemaRevisionsRequest::InternalGenerateClassData_(ListSchemaRevisionsRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListSchemaRevisionsRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListSchemaRevisionsRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ListSchemaRevisionsRequest_class_data_.tc_table);
  return ListSchemaRevisionsRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListSchemaRevisionsRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListSchemaRevisionsRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ListSchemaRevisionsRequest_globals_));
  return ListSchemaRevisionsRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ListSchemaRevisionsRequest::ParseTableT_
    ListSchemaRevisionsRequest::_table_ =
        ListSchemaRevisionsRequest::InternalGenerateParseTable_(ListSchemaRevisionsRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ListSchemaRevisionsRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.ListSchemaRevisionsRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.name_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.page_token_.ClearNonDefaultToEmpty();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000000cU)) {
    ::memset(&_impl_.view_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.page_size_) -
        reinterpret_cast<char*>(&_impl_.view_)) + sizeof(_impl_.page_size_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ListSchemaRevisionsRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ListSchemaRevisionsRequest& this_ = static_cast<const ListSchemaRevisionsRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ListSchemaRevisionsRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ListSchemaRevisionsRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.ListSchemaRevisionsRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_name().empty()) {
      const ::std::string& _s = this_._internal_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListSchemaRevisionsRequest.name");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // .google.pubsub.v1.SchemaView view = 2;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_view() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          2, this_._internal_view(), target);
    }
  }

  // int32 page_size = 3;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_page_size() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<3>(
              stream, this_._internal_page_size(), target);
    }
  }

  // string page_token = 4;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_page_token().empty()) {
      const ::std::string& _s = this_._internal_page_token();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListSchemaRevisionsRequest.page_token");
      target = stream->WriteStringMaybeAliased(4, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.ListSchemaRevisionsRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ListSchemaRevisionsRequest::ByteSizeLong(const MessageLite& base) {
  const ListSchemaRevisionsRequest& this_ = static_cast<const ListSchemaRevisionsRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ListSchemaRevisionsRequest::ByteSizeLong() const {
  const ListSchemaRevisionsRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.ListSchemaRevisionsRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_name());
      }
    }
    // string page_token = 4;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_page_token().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_page_token());
      }
    }
    // .google.pubsub.v1.SchemaView view = 2;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_view() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_view());
      }
    }
    // int32 page_size = 3;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_page_size() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_page_size());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ListSchemaRevisionsRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ListSchemaRevisionsRequest*>(&to_msg);
  auto& from = static_cast<const ListSchemaRevisionsRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.ListSchemaRevisionsRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_name().empty()) {
        _this->_internal_set_name(from._internal_name());
      } else {
        if (_this->_impl_.name_.IsDefault()) {
          _this->_internal_set_name("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_page_token().empty()) {
        _this->_internal_set_page_token(from._internal_page_token());
      } else {
        if (_this->_impl_.page_token_.IsDefault()) {
          _this->_internal_set_page_token("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_view() != 0) {
        _this->_impl_.view_ = from._impl_.view_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_page_size() != 0) {
        _this->_impl_.page_size_ = from._impl_.page_size_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ListSchemaRevisionsRequest::CopyFrom(const ListSchemaRevisionsRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.ListSchemaRevisionsRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ListSchemaRevisionsRequest::InternalSwap(ListSchemaRevisionsRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.name_, &other->_impl_.name_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.page_token_, &other->_impl_.page_token_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsRequest, _impl_.page_size_)
      + sizeof(ListSchemaRevisionsRequest::_impl_.page_size_)
      - PROTOBUF_FIELD_OFFSET(ListSchemaRevisionsRequest, _impl_.view_)>(
          reinterpret_cast<char*>(&_impl_.view_),
          reinterpret_cast<char*>(&other->_impl_.view_));
}

::google::protobuf::Metadata ListSchemaRevisionsRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ListSchemaRevisionsResponse::ListSchemaRevisionsResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListSchemaRevisionsResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ListSchemaRevisionsResponse)
}
PROTOBUF_NDEBUG_INLINE ListSchemaRevisionsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::ListSchemaRevisionsResponse& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        schemas_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::ListSchemaRevisionsResponse,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemaRevisionsResponse, _impl_.schemas_)>()
          , from.schemas_
        }
        ,
        next_page_token_(arena, from.next_page_token_) {}

ListSchemaRevisionsResponse::ListSchemaRevisionsResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ListSchemaRevisionsResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListSchemaRevisionsResponse_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ListSchemaRevisionsResponse* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ListSchemaRevisionsResponse)
}
PROTOBUF_NDEBUG_INLINE ListSchemaRevisionsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        schemas_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::ListSchemaRevisionsResponse,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSchemaRevisionsResponse, _impl_.schemas_)>()
         }
        ,
        next_page_token_(arena) {}

inline void ListSchemaRevisionsResponse::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
ListSchemaRevisionsResponse::~ListSchemaRevisionsResponse() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.ListSchemaRevisionsResponse)
  SharedDtor(*this);
}
inline void ListSchemaRevisionsResponse::SharedDtor(MessageLite& self) {
  ListSchemaRevisionsResponse& this_ = static_cast<ListSchemaRevisionsResponse&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.next_page_token_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ListSchemaRevisionsResponse_class_data_ =
        ListSchemaRevisionsResponse::InternalGenerateClassData_(ListSchemaRevisionsResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListSchemaRevisionsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListSchemaRevisionsResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ListSchemaRevisionsResponse_class_data_.tc_table);
  return ListSchemaRevisionsResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListSchemaRevisionsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListSchemaRevisionsResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ListSchemaRevisionsResponse_globals_));
  return ListSchemaRevisionsResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ListSchemaRevisionsResponse::ParseTableT_
    ListSchemaRevisionsResponse::_table_ =
        ListSchemaRevisionsResponse::InternalGenerateParseTable_(ListSchemaRevisionsResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ListSchemaRevisionsResponse::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.ListSchemaRevisionsResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.schemas_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.next_page_token_.ClearNonDefaultToEmpty();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ListSchemaRevisionsResponse::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ListSchemaRevisionsResponse& this_ = static_cast<const ListSchemaRevisionsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ListSchemaRevisionsResponse::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ListSchemaRevisionsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.ListSchemaRevisionsResponse)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated .google.pubsub.v1.Schema schemas = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_schemas_size());
         i < n; i++) {
      const auto& repfield = this_._internal_schemas().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              1, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  // string next_page_token = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_next_page_token().empty()) {
      const ::std::string& _s = this_._internal_next_page_token();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListSchemaRevisionsResponse.next_page_token");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.ListSchemaRevisionsResponse)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ListSchemaRevisionsResponse::ByteSizeLong(const MessageLite& base) {
  const ListSchemaRevisionsResponse& this_ = static_cast<const ListSchemaRevisionsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ListSchemaRevisionsResponse::ByteSizeLong() const {
  const ListSchemaRevisionsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.ListSchemaRevisionsResponse)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // repeated .google.pubsub.v1.Schema schemas = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_schemas_size();
      for (const auto& msg : this_._internal_schemas()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
    // string next_page_token = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_next_page_token().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_next_page_token());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ListSchemaRevisionsResponse::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ListSchemaRevisionsResponse*>(&to_msg);
  auto& from = static_cast<const ListSchemaRevisionsResponse&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.ListSchemaRevisionsResponse)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_schemas()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_schemas());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_next_page_token().empty()) {
        _this->_internal_set_next_page_token(from._internal_next_page_token());
      } else {
        if (_this->_impl_.next_page_token_.IsDefault()) {
          _this->_internal_set_next_page_token("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ListSchemaRevisionsResponse::CopyFrom(const ListSchemaRevisionsResponse& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.ListSchemaRevisionsResponse)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ListSchemaRevisionsResponse::InternalSwap(ListSchemaRevisionsResponse* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.schemas_.InternalSwap(&other->_impl_.schemas_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.next_page_token_, &other->_impl_.next_page_token_, arena);
}

::google::protobuf::Metadata ListSchemaRevisionsResponse::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

CommitSchemaRequest::CommitSchemaRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, CommitSchemaRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.CommitSchemaRequest)
}
PROTOBUF_NDEBUG_INLINE CommitSchemaRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::CommitSchemaRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        name_(arena, from.name_) {}

CommitSchemaRequest::CommitSchemaRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const CommitSchemaRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, CommitSchemaRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  CommitSchemaRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.schema_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.schema_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.CommitSchemaRequest)
}
PROTOBUF_NDEBUG_INLINE CommitSchemaRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        name_(arena) {}

inline void CommitSchemaRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.schema_ = {};
}
CommitSchemaRequest::~CommitSchemaRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.CommitSchemaRequest)
  SharedDtor(*this);
}
inline void CommitSchemaRequest::SharedDtor(MessageLite& self) {
  CommitSchemaRequest& this_ = static_cast<CommitSchemaRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.name_.Destroy();
  delete this_._impl_.schema_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull CommitSchemaRequest_class_data_ =
        CommitSchemaRequest::InternalGenerateClassData_(CommitSchemaRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CommitSchemaRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CommitSchemaRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(CommitSchemaRequest_class_data_.tc_table);
  return CommitSchemaRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CommitSchemaRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CommitSchemaRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&CommitSchemaRequest_globals_));
  return CommitSchemaRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const CommitSchemaRequest::ParseTableT_
    CommitSchemaRequest::_table_ =
        CommitSchemaRequest::InternalGenerateParseTable_(CommitSchemaRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void CommitSchemaRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.CommitSchemaRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.name_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.schema_ != nullptr);
      _impl_.schema_->Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL CommitSchemaRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const CommitSchemaRequest& this_ = static_cast<const CommitSchemaRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL CommitSchemaRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const CommitSchemaRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.CommitSchemaRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_name().empty()) {
      const ::std::string& _s = this_._internal_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.CommitSchemaRequest.name");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // .google.pubsub.v1.Schema schema = 2 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.schema_, this_._impl_.schema_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.CommitSchemaRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t CommitSchemaRequest::ByteSizeLong(const MessageLite& base) {
  const CommitSchemaRequest& this_ = static_cast<const CommitSchemaRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t CommitSchemaRequest::ByteSizeLong() const {
  const CommitSchemaRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.CommitSchemaRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_name());
      }
    }
    // .google.pubsub.v1.Schema schema = 2 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.schema_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void CommitSchemaRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<CommitSchemaRequest*>(&to_msg);
  auto& from = static_cast<const CommitSchemaRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.CommitSchemaRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_name().empty()) {
        _this->_internal_set_name(from._internal_name());
      } else {
        if (_this->_impl_.name_.IsDefault()) {
          _this->_internal_set_name("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.schema_ != nullptr);
      if (_this->_impl_.schema_ == nullptr) {
        _this->_impl_.schema_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.schema_);
      } else {
        _this->_impl_.schema_->MergeFrom(*from._impl_.schema_);
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void CommitSchemaRequest::CopyFrom(const CommitSchemaRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.CommitSchemaRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void CommitSchemaRequest::InternalSwap(CommitSchemaRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.name_, &other->_impl_.name_, arena);
  swap(_impl_.schema_, other->_impl_.schema_);
}

::google::protobuf::Metadata CommitSchemaRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

RollbackSchemaRequest::RollbackSchemaRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, RollbackSchemaRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.RollbackSchemaRequest)
}
PROTOBUF_NDEBUG_INLINE RollbackSchemaRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::RollbackSchemaRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        name_(arena, from.name_),
        revision_id_(arena, from.revision_id_) {}

RollbackSchemaRequest::RollbackSchemaRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const RollbackSchemaRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, RollbackSchemaRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  RollbackSchemaRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.RollbackSchemaRequest)
}
PROTOBUF_NDEBUG_INLINE RollbackSchemaRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        name_(arena),
        revision_id_(arena) {}

inline void RollbackSchemaRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
RollbackSchemaRequest::~RollbackSchemaRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.RollbackSchemaRequest)
  SharedDtor(*this);
}
inline void RollbackSchemaRequest::SharedDtor(MessageLite& self) {
  RollbackSchemaRequest& this_ = static_cast<RollbackSchemaRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.name_.Destroy();
  this_._impl_.revision_id_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull RollbackSchemaRequest_class_data_ =
        RollbackSchemaRequest::InternalGenerateClassData_(RollbackSchemaRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
RollbackSchemaRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&RollbackSchemaRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(RollbackSchemaRequest_class_data_.tc_table);
  return RollbackSchemaRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
RollbackSchemaRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&RollbackSchemaRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&RollbackSchemaRequest_globals_));
  return RollbackSchemaRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const RollbackSchemaRequest::ParseTableT_
    RollbackSchemaRequest::_table_ =
        RollbackSchemaRequest::InternalGenerateParseTable_(RollbackSchemaRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void RollbackSchemaRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.RollbackSchemaRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.name_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.revision_id_.ClearNonDefaultToEmpty();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL RollbackSchemaRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const RollbackSchemaRequest& this_ = static_cast<const RollbackSchemaRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL RollbackSchemaRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const RollbackSchemaRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.RollbackSchemaRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_name().empty()) {
      const ::std::string& _s = this_._internal_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.RollbackSchemaRequest.name");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // string revision_id = 2 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_revision_id().empty()) {
      const ::std::string& _s = this_._internal_revision_id();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.RollbackSchemaRequest.revision_id");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.RollbackSchemaRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t RollbackSchemaRequest::ByteSizeLong(const MessageLite& base) {
  const RollbackSchemaRequest& this_ = static_cast<const RollbackSchemaRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t RollbackSchemaRequest::ByteSizeLong() const {
  const RollbackSchemaRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.RollbackSchemaRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_name());
      }
    }
    // string revision_id = 2 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_revision_id().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_revision_id());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void RollbackSchemaRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<RollbackSchemaRequest*>(&to_msg);
  auto& from = static_cast<const RollbackSchemaRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.RollbackSchemaRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_name().empty()) {
        _this->_internal_set_name(from._internal_name());
      } else {
        if (_this->_impl_.name_.IsDefault()) {
          _this->_internal_set_name("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_revision_id().empty()) {
        _this->_internal_set_revision_id(from._internal_revision_id());
      } else {
        if (_this->_impl_.revision_id_.IsDefault()) {
          _this->_internal_set_revision_id("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void RollbackSchemaRequest::CopyFrom(const RollbackSchemaRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.RollbackSchemaRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void RollbackSchemaRequest::InternalSwap(RollbackSchemaRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.name_, &other->_impl_.name_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.revision_id_, &other->_impl_.revision_id_, arena);
}

::google::protobuf::Metadata RollbackSchemaRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

DeleteSchemaRevisionRequest::DeleteSchemaRevisionRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DeleteSchemaRevisionRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.DeleteSchemaRevisionRequest)
}
PROTOBUF_NDEBUG_INLINE DeleteSchemaRevisionRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::DeleteSchemaRevisionRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        name_(arena, from.name_),
        revision_id_(arena, from.revision_id_) {}

DeleteSchemaRevisionRequest::DeleteSchemaRevisionRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const DeleteSchemaRevisionRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DeleteSchemaRevisionRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  DeleteSchemaRevisionRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.DeleteSchemaRevisionRequest)
}
PROTOBUF_NDEBUG_INLINE DeleteSchemaRevisionRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        name_(arena),
        revision_id_(arena) {}

inline void DeleteSchemaRevisionRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
DeleteSchemaRevisionRequest::~DeleteSchemaRevisionRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.DeleteSchemaRevisionRequest)
  SharedDtor(*this);
}
inline void DeleteSchemaRevisionRequest::SharedDtor(MessageLite& self) {
  DeleteSchemaRevisionRequest& this_ = static_cast<DeleteSchemaRevisionRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.name_.Destroy();
  this_._impl_.revision_id_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull DeleteSchemaRevisionRequest_class_data_ =
        DeleteSchemaRevisionRequest::InternalGenerateClassData_(DeleteSchemaRevisionRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DeleteSchemaRevisionRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DeleteSchemaRevisionRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(DeleteSchemaRevisionRequest_class_data_.tc_table);
  return DeleteSchemaRevisionRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DeleteSchemaRevisionRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DeleteSchemaRevisionRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&DeleteSchemaRevisionRequest_globals_));
  return DeleteSchemaRevisionRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const DeleteSchemaRevisionRequest::ParseTableT_
    DeleteSchemaRevisionRequest::_table_ =
        DeleteSchemaRevisionRequest::InternalGenerateParseTable_(DeleteSchemaRevisionRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void DeleteSchemaRevisionRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.DeleteSchemaRevisionRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.name_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.revision_id_.ClearNonDefaultToEmpty();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL DeleteSchemaRevisionRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const DeleteSchemaRevisionRequest& this_ = static_cast<const DeleteSchemaRevisionRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL DeleteSchemaRevisionRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const DeleteSchemaRevisionRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.DeleteSchemaRevisionRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_name().empty()) {
      const ::std::string& _s = this_._internal_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.DeleteSchemaRevisionRequest.name");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // string revision_id = 2 [deprecated = true, (.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_revision_id().empty()) {
      const ::std::string& _s = this_._internal_revision_id();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.DeleteSchemaRevisionRequest.revision_id");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.DeleteSchemaRevisionRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t DeleteSchemaRevisionRequest::ByteSizeLong(const MessageLite& base) {
  const DeleteSchemaRevisionRequest& this_ = static_cast<const DeleteSchemaRevisionRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t DeleteSchemaRevisionRequest::ByteSizeLong() const {
  const DeleteSchemaRevisionRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.DeleteSchemaRevisionRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_name());
      }
    }
    // string revision_id = 2 [deprecated = true, (.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_revision_id().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_revision_id());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void DeleteSchemaRevisionRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<DeleteSchemaRevisionRequest*>(&to_msg);
  auto& from = static_cast<const DeleteSchemaRevisionRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.DeleteSchemaRevisionRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_name().empty()) {
        _this->_internal_set_name(from._internal_name());
      } else {
        if (_this->_impl_.name_.IsDefault()) {
          _this->_internal_set_name("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_revision_id().empty()) {
        _this->_internal_set_revision_id(from._internal_revision_id());
      } else {
        if (_this->_impl_.revision_id_.IsDefault()) {
          _this->_internal_set_revision_id("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void DeleteSchemaRevisionRequest::CopyFrom(const DeleteSchemaRevisionRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.DeleteSchemaRevisionRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void DeleteSchemaRevisionRequest::InternalSwap(DeleteSchemaRevisionRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.name_, &other->_impl_.name_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.revision_id_, &other->_impl_.revision_id_, arena);
}

::google::protobuf::Metadata DeleteSchemaRevisionRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

DeleteSchemaRequest::DeleteSchemaRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DeleteSchemaRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.DeleteSchemaRequest)
}
PROTOBUF_NDEBUG_INLINE DeleteSchemaRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::DeleteSchemaRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        name_(arena, from.name_) {}

DeleteSchemaRequest::DeleteSchemaRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const DeleteSchemaRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DeleteSchemaRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  DeleteSchemaRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.DeleteSchemaRequest)
}
PROTOBUF_NDEBUG_INLINE DeleteSchemaRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        name_(arena) {}

inline void DeleteSchemaRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
DeleteSchemaRequest::~DeleteSchemaRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.DeleteSchemaRequest)
  SharedDtor(*this);
}
inline void DeleteSchemaRequest::SharedDtor(MessageLite& self) {
  DeleteSchemaRequest& this_ = static_cast<DeleteSchemaRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.name_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull DeleteSchemaRequest_class_data_ =
        DeleteSchemaRequest::InternalGenerateClassData_(DeleteSchemaRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DeleteSchemaRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DeleteSchemaRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(DeleteSchemaRequest_class_data_.tc_table);
  return DeleteSchemaRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DeleteSchemaRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DeleteSchemaRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&DeleteSchemaRequest_globals_));
  return DeleteSchemaRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const DeleteSchemaRequest::ParseTableT_
    DeleteSchemaRequest::_table_ =
        DeleteSchemaRequest::InternalGenerateParseTable_(DeleteSchemaRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void DeleteSchemaRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.DeleteSchemaRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.name_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL DeleteSchemaRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const DeleteSchemaRequest& this_ = static_cast<const DeleteSchemaRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL DeleteSchemaRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const DeleteSchemaRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.DeleteSchemaRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_name().empty()) {
      const ::std::string& _s = this_._internal_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.DeleteSchemaRequest.name");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.DeleteSchemaRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t DeleteSchemaRequest::ByteSizeLong(const MessageLite& base) {
  const DeleteSchemaRequest& this_ = static_cast<const DeleteSchemaRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t DeleteSchemaRequest::ByteSizeLong() const {
  const DeleteSchemaRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.DeleteSchemaRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_name());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void DeleteSchemaRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<DeleteSchemaRequest*>(&to_msg);
  auto& from = static_cast<const DeleteSchemaRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.DeleteSchemaRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_name().empty()) {
      _this->_internal_set_name(from._internal_name());
    } else {
      if (_this->_impl_.name_.IsDefault()) {
        _this->_internal_set_name("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void DeleteSchemaRequest::CopyFrom(const DeleteSchemaRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.DeleteSchemaRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void DeleteSchemaRequest::InternalSwap(DeleteSchemaRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.name_, &other->_impl_.name_, arena);
}

::google::protobuf::Metadata DeleteSchemaRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ValidateSchemaRequest::ValidateSchemaRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ValidateSchemaRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ValidateSchemaRequest)
}
PROTOBUF_NDEBUG_INLINE ValidateSchemaRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::ValidateSchemaRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        parent_(arena, from.parent_) {}

ValidateSchemaRequest::ValidateSchemaRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ValidateSchemaRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ValidateSchemaRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ValidateSchemaRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.schema_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.schema_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ValidateSchemaRequest)
}
PROTOBUF_NDEBUG_INLINE ValidateSchemaRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        parent_(arena) {}

inline void ValidateSchemaRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.schema_ = {};
}
ValidateSchemaRequest::~ValidateSchemaRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.ValidateSchemaRequest)
  SharedDtor(*this);
}
inline void ValidateSchemaRequest::SharedDtor(MessageLite& self) {
  ValidateSchemaRequest& this_ = static_cast<ValidateSchemaRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.parent_.Destroy();
  delete this_._impl_.schema_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ValidateSchemaRequest_class_data_ =
        ValidateSchemaRequest::InternalGenerateClassData_(ValidateSchemaRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ValidateSchemaRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ValidateSchemaRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ValidateSchemaRequest_class_data_.tc_table);
  return ValidateSchemaRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ValidateSchemaRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ValidateSchemaRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ValidateSchemaRequest_globals_));
  return ValidateSchemaRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ValidateSchemaRequest::ParseTableT_
    ValidateSchemaRequest::_table_ =
        ValidateSchemaRequest::InternalGenerateParseTable_(ValidateSchemaRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ValidateSchemaRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.ValidateSchemaRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.parent_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.schema_ != nullptr);
      _impl_.schema_->Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ValidateSchemaRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ValidateSchemaRequest& this_ = static_cast<const ValidateSchemaRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ValidateSchemaRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ValidateSchemaRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.ValidateSchemaRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_parent().empty()) {
      const ::std::string& _s = this_._internal_parent();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ValidateSchemaRequest.parent");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // .google.pubsub.v1.Schema schema = 2 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.schema_, this_._impl_.schema_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.ValidateSchemaRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ValidateSchemaRequest::ByteSizeLong(const MessageLite& base) {
  const ValidateSchemaRequest& this_ = static_cast<const ValidateSchemaRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ValidateSchemaRequest::ByteSizeLong() const {
  const ValidateSchemaRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.ValidateSchemaRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_parent().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_parent());
      }
    }
    // .google.pubsub.v1.Schema schema = 2 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.schema_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ValidateSchemaRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ValidateSchemaRequest*>(&to_msg);
  auto& from = static_cast<const ValidateSchemaRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.ValidateSchemaRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_parent().empty()) {
        _this->_internal_set_parent(from._internal_parent());
      } else {
        if (_this->_impl_.parent_.IsDefault()) {
          _this->_internal_set_parent("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.schema_ != nullptr);
      if (_this->_impl_.schema_ == nullptr) {
        _this->_impl_.schema_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.schema_);
      } else {
        _this->_impl_.schema_->MergeFrom(*from._impl_.schema_);
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ValidateSchemaRequest::CopyFrom(const ValidateSchemaRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.ValidateSchemaRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ValidateSchemaRequest::InternalSwap(ValidateSchemaRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.parent_, &other->_impl_.parent_, arena);
  swap(_impl_.schema_, other->_impl_.schema_);
}

::google::protobuf::Metadata ValidateSchemaRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ValidateSchemaResponse::ValidateSchemaResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ValidateSchemaResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ValidateSchemaResponse)
}
ValidateSchemaResponse::ValidateSchemaResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ValidateSchemaResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ValidateSchemaResponse_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ValidateSchemaResponse* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ValidateSchemaResponse)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ValidateSchemaResponse_class_data_ =
        ValidateSchemaResponse::InternalGenerateClassData_(ValidateSchemaResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ValidateSchemaResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ValidateSchemaResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ValidateSchemaResponse_class_data_.tc_table);
  return ValidateSchemaResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ValidateSchemaResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ValidateSchemaResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ValidateSchemaResponse_globals_));
  return ValidateSchemaResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ValidateSchemaResponse::ParseTableT_
    ValidateSchemaResponse::_table_ =
        ValidateSchemaResponse::InternalGenerateParseTable_(ValidateSchemaResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata ValidateSchemaResponse::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void ValidateMessageRequest::set_allocated_schema(::google::pubsub::v1::Schema* PROTOBUF_NULLABLE schema) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_schema_spec();
  if (schema) {
    ::google::protobuf::Arena* submessage_arena = schema->GetArena();
    if (message_arena != submessage_arena) {
      schema = ::google::protobuf::internal::GetOwnedMessage(message_arena, schema, submessage_arena);
    }
    set_has_schema();
    _impl_.schema_spec_.schema_ = schema;
  }
  // @@protoc_insertion_point(field_set_allocated:google.pubsub.v1.ValidateMessageRequest.schema)
}
ValidateMessageRequest::ValidateMessageRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ValidateMessageRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ValidateMessageRequest)
}
PROTOBUF_NDEBUG_INLINE ValidateMessageRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::ValidateMessageRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        parent_(arena, from.parent_),
        message_(arena, from.message_),
        schema_spec_{},
        _oneof_case_{from._oneof_case_[0]} {}

ValidateMessageRequest::ValidateMessageRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ValidateMessageRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ValidateMessageRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ValidateMessageRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.encoding_ = from._impl_.encoding_;
  switch (schema_spec_case()) {
    case SCHEMA_SPEC_NOT_SET:
      break;
      case kName:
        new (&_impl_.schema_spec_.name_) decltype(_impl_.schema_spec_.name_){arena, from._impl_.schema_spec_.name_};
        break;
      case kSchema:
        _impl_.schema_spec_.schema_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.schema_spec_.schema_);
        break;
  }

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ValidateMessageRequest)
}
PROTOBUF_NDEBUG_INLINE ValidateMessageRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        parent_(arena),
        message_(arena),
        schema_spec_{},
        _oneof_case_{} {}

inline void ValidateMessageRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.encoding_ = {};
}
ValidateMessageRequest::~ValidateMessageRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.ValidateMessageRequest)
  SharedDtor(*this);
}
inline void ValidateMessageRequest::SharedDtor(MessageLite& self) {
  ValidateMessageRequest& this_ = static_cast<ValidateMessageRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.parent_.Destroy();
  this_._impl_.message_.Destroy();
  if (this_.has_schema_spec()) {
    this_.clear_schema_spec();
  }
  this_._impl_.~Impl_();
}

void ValidateMessageRequest::clear_schema_spec() {
// @@protoc_insertion_point(one_of_clear_start:google.pubsub.v1.ValidateMessageRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (schema_spec_case()) {
    case kName: {
      _impl_.schema_spec_.name_.Destroy();
      break;
    }
    case kSchema: {
      if (GetArena() == nullptr) {
        delete _impl_.schema_spec_.schema_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.schema_spec_.schema_);
      }
      break;
    }
    case SCHEMA_SPEC_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = SCHEMA_SPEC_NOT_SET;
}


#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ValidateMessageRequest_class_data_ =
        ValidateMessageRequest::InternalGenerateClassData_(ValidateMessageRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ValidateMessageRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ValidateMessageRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ValidateMessageRequest_class_data_.tc_table);
  return ValidateMessageRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ValidateMessageRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ValidateMessageRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ValidateMessageRequest_globals_));
  return ValidateMessageRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ValidateMessageRequest::ParseTableT_
    ValidateMessageRequest::_table_ =
        ValidateMessageRequest::InternalGenerateParseTable_(ValidateMessageRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ValidateMessageRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.ValidateMessageRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.parent_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.message_.ClearNonDefaultToEmpty();
    }
  }
  _impl_.encoding_ = 0;
  clear_schema_spec();
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ValidateMessageRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ValidateMessageRequest& this_ = static_cast<const ValidateMessageRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ValidateMessageRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ValidateMessageRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.ValidateMessageRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_parent().empty()) {
      const ::std::string& _s = this_._internal_parent();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ValidateMessageRequest.parent");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  switch (this_.schema_spec_case()) {
    case kName: {
      const ::std::string& _s = this_._internal_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ValidateMessageRequest.name");
      target = stream->WriteStringMaybeAliased(2, _s, target);
      break;
    }
    case kSchema: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          3, *this_._impl_.schema_spec_.schema_, this_._impl_.schema_spec_.schema_->GetCachedSize(), target,
          stream);
      break;
    }
    default:
      break;
  }
  // bytes message = 4;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_message().empty()) {
      const ::std::string& _s = this_._internal_message();
      target = stream->WriteBytesMaybeAliased(4, _s, target);
    }
  }

  // .google.pubsub.v1.Encoding encoding = 5;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_encoding() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          5, this_._internal_encoding(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.ValidateMessageRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ValidateMessageRequest::ByteSizeLong(const MessageLite& base) {
  const ValidateMessageRequest& this_ = static_cast<const ValidateMessageRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ValidateMessageRequest::ByteSizeLong() const {
  const ValidateMessageRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.ValidateMessageRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_parent().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_parent());
      }
    }
    // bytes message = 4;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_message().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::BytesSize(
                                        this_._internal_message());
      }
    }
    // .google.pubsub.v1.Encoding encoding = 5;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_encoding() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_encoding());
      }
    }
  }
  switch (this_.schema_spec_case()) {
    // string name = 2 [(.google.api.resource_reference) = {
    case kName: {
      total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                      this_._internal_name());
      break;
    }
    // .google.pubsub.v1.Schema schema = 3;
    case kSchema: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.schema_spec_.schema_);
      break;
    }
    case SCHEMA_SPEC_NOT_SET: {
      break;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ValidateMessageRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ValidateMessageRequest*>(&to_msg);
  auto& from = static_cast<const ValidateMessageRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.ValidateMessageRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_parent().empty()) {
        _this->_internal_set_parent(from._internal_parent());
      } else {
        if (_this->_impl_.parent_.IsDefault()) {
          _this->_internal_set_parent("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_message().empty()) {
        _this->_internal_set_message(from._internal_message());
      } else {
        if (_this->_impl_.message_.IsDefault()) {
          _this->_internal_set_message("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_encoding() != 0) {
        _this->_impl_.encoding_ = from._impl_.encoding_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[0]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[0];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_schema_spec();
      }
      _this->_impl_._oneof_case_[0] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kName: {
        if (oneof_needs_init) {
          _this->_impl_.schema_spec_.name_.InitDefault();
        }
        _this->_impl_.schema_spec_.name_.Set(from._internal_name(), arena);
        break;
      }
      case kSchema: {
        if (oneof_needs_init) {
          _this->_impl_.schema_spec_.schema_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.schema_spec_.schema_);
        } else {
          _this->_impl_.schema_spec_.schema_->MergeFrom(*from._impl_.schema_spec_.schema_);
        }
        break;
      }
      case SCHEMA_SPEC_NOT_SET:
        break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ValidateMessageRequest::CopyFrom(const ValidateMessageRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.ValidateMessageRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ValidateMessageRequest::InternalSwap(ValidateMessageRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.parent_, &other->_impl_.parent_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.message_, &other->_impl_.message_, arena);
  swap(_impl_.encoding_, other->_impl_.encoding_);
  swap(_impl_.schema_spec_, other->_impl_.schema_spec_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
}

::google::protobuf::Metadata ValidateMessageRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ValidateMessageResponse::ValidateMessageResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ValidateMessageResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ValidateMessageResponse)
}
ValidateMessageResponse::ValidateMessageResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ValidateMessageResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, ValidateMessageResponse_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ValidateMessageResponse* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ValidateMessageResponse)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ValidateMessageResponse_class_data_ =
        ValidateMessageResponse::InternalGenerateClassData_(ValidateMessageResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ValidateMessageResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ValidateMessageResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ValidateMessageResponse_class_data_.tc_table);
  return ValidateMessageResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ValidateMessageResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ValidateMessageResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ValidateMessageResponse_globals_));
  return ValidateMessageResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ValidateMessageResponse::ParseTableT_
    ValidateMessageResponse::_table_ =
        ValidateMessageResponse::InternalGenerateParseTable_(ValidateMessageResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata ValidateMessageResponse::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
}  // namespace v1
}  // namespace pubsub
}  // namespace google
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
