/* A Bison parser, made by GNU Bison 3.8.2.  */

/* Bison implementation for Yacc-like parsers in C

   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2021 Free Software Foundation,
   Inc.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */

/* As a special exception, you may create a larger work that contains
   part or all of the Bison parser skeleton and distribute that work
   under terms of your choice, so long as that work isn't itself a
   parser generator using the skeleton or a modified version thereof
   as a parser skeleton.  Alternatively, if you modify or redistribute
   the parser skeleton itself, you may (at your option) remove this
   special exception, which will cause the skeleton and the resulting
   Bison output files to be licensed under the GNU General Public
   License without this special exception.

   This special exception was added by the Free Software Foundation in
   version 2.2 of Bison.  */

/* C LALR(1) parser skeleton written by Richard Stallman, by
   simplifying the original so-called "semantic" parser.  */

/* DO NOT RELY ON FEATURES THAT ARE NOT DOCUMENTED in the manual,
   especially those whose name start with YY_ or yy_.  They are
   private implementation details that can be changed or removed.  */

/* All symbols defined below should begin with yy or YY, to avoid
   infringing on user name space.  This should be done even for local
   variables, as they might otherwise be expanded by user macros.
   There are some unavoidable exceptions within include files to
   define necessary library symbols; they are noted "INFRINGES ON
   USER NAME SPACE" below.  */

/* Identify Bison output, and Bison version.  */
#define YYBISON 30802

/* Bison version string.  */
#define YYBISON_VERSION "3.8.2"

/* Skeleton name.  */
#define YYSKELETON_NAME "yacc.c"

/* Pure parsers.  */
#define YYPURE 1

/* Push parsers.  */
#define YYPUSH 0

/* Pull parsers.  */
#define YYPULL 1

/* "%code top" blocks.  */
#line 24 "modules/afsocket/afsocket-grammar.y"

#include "afsocket-parser.h"


#line 73 "modules/afsocket/afsocket-grammar.c"
/* Substitute the type names.  */
#define YYSTYPE         AFSOCKET_STYPE
#define YYLTYPE         AFSOCKET_LTYPE
/* Substitute the variable and function names.  */
#define yyparse         afsocket_parse
#define yylex           afsocket_lex
#define yyerror         afsocket_error
#define yydebug         afsocket_debug
#define yynerrs         afsocket_nerrs


# ifndef YY_CAST
#  ifdef __cplusplus
#   define YY_CAST(Type, Val) static_cast<Type> (Val)
#   define YY_REINTERPRET_CAST(Type, Val) reinterpret_cast<Type> (Val)
#  else
#   define YY_CAST(Type, Val) ((Type) (Val))
#   define YY_REINTERPRET_CAST(Type, Val) ((Type) (Val))
#  endif
# endif
# ifndef YY_NULLPTR
#  if defined __cplusplus
#   if 201103L <= __cplusplus
#    define YY_NULLPTR nullptr
#   else
#    define YY_NULLPTR 0
#   endif
#  else
#   define YY_NULLPTR ((void*)0)
#  endif
# endif

/* Use api.header.include to #include this header
   instead of duplicating it here.  */
#ifndef YY_AFSOCKET_MODULES_AFSOCKET_AFSOCKET_GRAMMAR_H_INCLUDED
# define YY_AFSOCKET_MODULES_AFSOCKET_AFSOCKET_GRAMMAR_H_INCLUDED
/* Debug traces.  */
#ifndef AFSOCKET_DEBUG
# if defined YYDEBUG
#if YYDEBUG
#   define AFSOCKET_DEBUG 1
#  else
#   define AFSOCKET_DEBUG 0
#  endif
# else /* ! defined YYDEBUG */
#  define AFSOCKET_DEBUG 0
# endif /* ! defined YYDEBUG */
#endif  /* ! defined AFSOCKET_DEBUG */
#if AFSOCKET_DEBUG
extern int afsocket_debug;
#endif

/* Token kinds.  */
#ifndef AFSOCKET_TOKENTYPE
# define AFSOCKET_TOKENTYPE
  enum afsocket_tokentype
  {
    AFSOCKET_EMPTY = -2,
    AFSOCKET_EOF = 0,              /* "end of file"  */
    AFSOCKET_error = 256,          /* error  */
    AFSOCKET_UNDEF = 20001,        /* "invalid token"  */
    KW_UNIX_STREAM = 20000,        /* KW_UNIX_STREAM  */
    KW_UNIX_DGRAM = 20002,         /* KW_UNIX_DGRAM  */
    KW_TCP = 20003,                /* KW_TCP  */
    KW_UDP = 20004,                /* KW_UDP  */
    KW_TCP6 = 20005,               /* KW_TCP6  */
    KW_UDP6 = 20006,               /* KW_UDP6  */
    KW_NETWORK = 20007,            /* KW_NETWORK  */
    KW_HTTP = 10324,               /* KW_HTTP  */
    KW_TRANSPORT = 20008,          /* KW_TRANSPORT  */
    KW_IP_PROTOCOL = 20009,        /* KW_IP_PROTOCOL  */
    KW_SYSTEMD_SYSLOG = 20010,     /* KW_SYSTEMD_SYSLOG  */
    KW_HTTP_SCRAPER = 20011,       /* KW_HTTP_SCRAPER  */
    KW_SCRAPE_PATTERN = 20012,     /* KW_SCRAPE_PATTERN  */
    KW_SCRAPE_FREQ_LIMIT = 20013,  /* KW_SCRAPE_FREQ_LIMIT  */
    KW_SINGLE_INSTANCE = 20014,    /* KW_SINGLE_INSTANCE  */
    KW_STAT_TYPE = 20015,          /* KW_STAT_TYPE  */
    KW_STAT_QUERY = 20016,         /* KW_STAT_QUERY  */
    KW_STAT_FORMAT = 20017,        /* KW_STAT_FORMAT  */
    KW_STATS_WITHOUT_ORPHANED = 20018, /* KW_STATS_WITHOUT_ORPHANED  */
    KW_STATS_WITH_LEGACY = 20019,  /* KW_STATS_WITH_LEGACY  */
    KW_IP_TTL = 20020,             /* KW_IP_TTL  */
    KW_SO_BROADCAST = 20021,       /* KW_SO_BROADCAST  */
    KW_IP_TOS = 20022,             /* KW_IP_TOS  */
    KW_IP_FREEBIND = 20023,        /* KW_IP_FREEBIND  */
    KW_SO_SNDBUF = 20024,          /* KW_SO_SNDBUF  */
    KW_SO_RCVBUF = 20025,          /* KW_SO_RCVBUF  */
    KW_SO_KEEPALIVE = 20026,       /* KW_SO_KEEPALIVE  */
    KW_SO_REUSEPORT = 20027,       /* KW_SO_REUSEPORT  */
    KW_TCP_KEEPALIVE_TIME = 20028, /* KW_TCP_KEEPALIVE_TIME  */
    KW_TCP_KEEPALIVE_PROBES = 20029, /* KW_TCP_KEEPALIVE_PROBES  */
    KW_TCP_KEEPALIVE_INTVL = 20030, /* KW_TCP_KEEPALIVE_INTVL  */
    KW_SO_PASSCRED = 20031,        /* KW_SO_PASSCRED  */
    KW_LISTEN_BACKLOG = 20032,     /* KW_LISTEN_BACKLOG  */
    KW_SPOOF_SOURCE = 20033,       /* KW_SPOOF_SOURCE  */
    KW_SPOOF_SOURCE_MAX_MSGLEN = 20034, /* KW_SPOOF_SOURCE_MAX_MSGLEN  */
    KW_KEEP_ALIVE = 20035,         /* KW_KEEP_ALIVE  */
    KW_MAX_CONNECTIONS = 20036,    /* KW_MAX_CONNECTIONS  */
    KW_CLOSE_ON_INPUT = 20037,     /* KW_CLOSE_ON_INPUT  */
    KW_LOCALIP = 20038,            /* KW_LOCALIP  */
    KW_IP = 20039,                 /* KW_IP  */
    KW_INTERFACE = 20040,          /* KW_INTERFACE  */
    KW_LOCALPORT = 20041,          /* KW_LOCALPORT  */
    KW_DESTPORT = 20042,           /* KW_DESTPORT  */
    KW_FAILOVER_SERVERS = 20043,   /* KW_FAILOVER_SERVERS  */
    KW_FAILOVER = 20044,           /* KW_FAILOVER  */
    KW_SERVERS = 20045,            /* KW_SERVERS  */
    KW_FAILBACK = 20046,           /* KW_FAILBACK  */
    KW_TCP_PROBE_INTERVAL = 20047, /* KW_TCP_PROBE_INTERVAL  */
    KW_SUCCESSFUL_PROBES_REQUIRED = 20048, /* KW_SUCCESSFUL_PROBES_REQUIRED  */
    KW_DYNAMIC_WINDOW_SIZE = 20049, /* KW_DYNAMIC_WINDOW_SIZE  */
    KW_DYNAMIC_WINDOW_STATS_FREQ = 20050, /* KW_DYNAMIC_WINDOW_STATS_FREQ  */
    KW_DYNAMIC_WINDOW_REALLOC_TICKS = 20051, /* KW_DYNAMIC_WINDOW_REALLOC_TICKS  */
    KW_TLS = 20052,                /* KW_TLS  */
    KW_PEER_VERIFY = 20053,        /* KW_PEER_VERIFY  */
    KW_KEY_FILE = 20054,           /* KW_KEY_FILE  */
    KW_CERT_FILE = 20055,          /* KW_CERT_FILE  */
    KW_DHPARAM_FILE = 20056,       /* KW_DHPARAM_FILE  */
    KW_PKCS12_FILE = 20057,        /* KW_PKCS12_FILE  */
    KW_CA_DIR = 20058,             /* KW_CA_DIR  */
    KW_CRL_DIR = 20059,            /* KW_CRL_DIR  */
    KW_CA_FILE = 20060,            /* KW_CA_FILE  */
    KW_TRUSTED_KEYS = 20061,       /* KW_TRUSTED_KEYS  */
    KW_TRUSTED_DN = 20062,         /* KW_TRUSTED_DN  */
    KW_CIPHER_SUITE = 20063,       /* KW_CIPHER_SUITE  */
    KW_TLS12_AND_OLDER = 20064,    /* KW_TLS12_AND_OLDER  */
    KW_TLS13 = 20065,              /* KW_TLS13  */
    KW_SIGALGS = 20066,            /* KW_SIGALGS  */
    KW_CLIENT_SIGALGS = 20067,     /* KW_CLIENT_SIGALGS  */
    KW_ECDH_CURVE_LIST = 20068,    /* KW_ECDH_CURVE_LIST  */
    KW_SSL_OPTIONS = 20069,        /* KW_SSL_OPTIONS  */
    KW_SSL_VERSION = 20070,        /* KW_SSL_VERSION  */
    KW_SNI = 20071,                /* KW_SNI  */
    KW_ALLOW_COMPRESS = 20072,     /* KW_ALLOW_COMPRESS  */
    KW_KEYLOG_FILE = 20073,        /* KW_KEYLOG_FILE  */
    KW_OCSP_STAPLING_VERIFY = 20074, /* KW_OCSP_STAPLING_VERIFY  */
    KW_EXTENDED_KEY_USAGE_VERIFY = 20075, /* KW_EXTENDED_KEY_USAGE_VERIFY  */
    KW_CONF_CMDS = 20076,          /* KW_CONF_CMDS  */
    LL_CONTEXT_ROOT = 1,           /* LL_CONTEXT_ROOT  */
    LL_CONTEXT_DESTINATION = 2,    /* LL_CONTEXT_DESTINATION  */
    LL_CONTEXT_SOURCE = 3,         /* LL_CONTEXT_SOURCE  */
    LL_CONTEXT_PARSER = 4,         /* LL_CONTEXT_PARSER  */
    LL_CONTEXT_REWRITE = 5,        /* LL_CONTEXT_REWRITE  */
    LL_CONTEXT_FILTER = 6,         /* LL_CONTEXT_FILTER  */
    LL_CONTEXT_LOG = 7,            /* LL_CONTEXT_LOG  */
    LL_CONTEXT_BLOCK_DEF = 8,      /* LL_CONTEXT_BLOCK_DEF  */
    LL_CONTEXT_BLOCK_REF = 9,      /* LL_CONTEXT_BLOCK_REF  */
    LL_CONTEXT_BLOCK_CONTENT = 10, /* LL_CONTEXT_BLOCK_CONTENT  */
    LL_CONTEXT_BLOCK_ARG = 11,     /* LL_CONTEXT_BLOCK_ARG  */
    LL_CONTEXT_BLOCK_FUNCARG = 12, /* LL_CONTEXT_BLOCK_FUNCARG  */
    LL_CONTEXT_PRAGMA = 13,        /* LL_CONTEXT_PRAGMA  */
    LL_CONTEXT_FORMAT = 14,        /* LL_CONTEXT_FORMAT  */
    LL_CONTEXT_TEMPLATE_FUNC = 15, /* LL_CONTEXT_TEMPLATE_FUNC  */
    LL_CONTEXT_INNER_DEST = 16,    /* LL_CONTEXT_INNER_DEST  */
    LL_CONTEXT_INNER_SRC = 17,     /* LL_CONTEXT_INNER_SRC  */
    LL_CONTEXT_CLIENT_PROTO = 18,  /* LL_CONTEXT_CLIENT_PROTO  */
    LL_CONTEXT_SERVER_PROTO = 19,  /* LL_CONTEXT_SERVER_PROTO  */
    LL_CONTEXT_OPTIONS = 20,       /* LL_CONTEXT_OPTIONS  */
    LL_CONTEXT_CONFIG = 21,        /* LL_CONTEXT_CONFIG  */
    LL_CONTEXT_TEMPLATE_REF = 22,  /* LL_CONTEXT_TEMPLATE_REF  */
    LL_CONTEXT_MAX = 23,           /* LL_CONTEXT_MAX  */
    KW_ASSIGN = 9000,              /* KW_ASSIGN  */
    KW_OR = 9010,                  /* KW_OR  */
    KW_AND = 9020,                 /* KW_AND  */
    KW_STR_EQ = 9030,              /* KW_STR_EQ  */
    KW_STR_NE = 9031,              /* KW_STR_NE  */
    KW_TA_EQ = 9032,               /* KW_TA_EQ  */
    KW_TA_NE = 9033,               /* KW_TA_NE  */
    KW_TAV_EQ = 9034,              /* KW_TAV_EQ  */
    KW_TAV_NE = 9035,              /* KW_TAV_NE  */
    KW_STR_LT = 9040,              /* KW_STR_LT  */
    KW_STR_LE = 9041,              /* KW_STR_LE  */
    KW_STR_GE = 9042,              /* KW_STR_GE  */
    KW_STR_GT = 9043,              /* KW_STR_GT  */
    KW_TA_LT = 9044,               /* KW_TA_LT  */
    KW_TA_LE = 9045,               /* KW_TA_LE  */
    KW_TA_GE = 9046,               /* KW_TA_GE  */
    KW_TA_GT = 9047,               /* KW_TA_GT  */
    KW_PLUS_ASSIGN = 9050,         /* KW_PLUS_ASSIGN  */
    KW_REGEXP_MATCH = 9060,        /* KW_REGEXP_MATCH  */
    KW_NOT = 9049,                 /* KW_NOT  */
    KW_SOURCE = 10000,             /* KW_SOURCE  */
    KW_FILTER = 10001,             /* KW_FILTER  */
    KW_PARSER = 10002,             /* KW_PARSER  */
    KW_DESTINATION = 10003,        /* KW_DESTINATION  */
    KW_LOG = 10004,                /* KW_LOG  */
    KW_OPTIONS = 10005,            /* KW_OPTIONS  */
    KW_INCLUDE = 10006,            /* KW_INCLUDE  */
    KW_BLOCK = 10007,              /* KW_BLOCK  */
    KW_JUNCTION = 10008,           /* KW_JUNCTION  */
    KW_CHANNEL = 10009,            /* KW_CHANNEL  */
    KW_IF = 10010,                 /* KW_IF  */
    KW_ELSE = 10011,               /* KW_ELSE  */
    KW_ELIF = 10012,               /* KW_ELIF  */
    KW_INTERNAL = 10020,           /* KW_INTERNAL  */
    KW_SYSLOG = 10060,             /* KW_SYSLOG  */
    KW_MARK_FREQ = 10071,          /* KW_MARK_FREQ  */
    KW_STATS_FREQ = 10072,         /* KW_STATS_FREQ  */
    KW_STATS_LEVEL = 10073,        /* KW_STATS_LEVEL  */
    KW_STATS_LIFETIME = 10074,     /* KW_STATS_LIFETIME  */
    KW_FLUSH_LINES = 10075,        /* KW_FLUSH_LINES  */
    KW_SUPPRESS = 10076,           /* KW_SUPPRESS  */
    KW_FLUSH_TIMEOUT = 10077,      /* KW_FLUSH_TIMEOUT  */
    KW_LOG_MSG_SIZE = 10078,       /* KW_LOG_MSG_SIZE  */
    KW_FILE_TEMPLATE = 10079,      /* KW_FILE_TEMPLATE  */
    KW_PROTO_TEMPLATE = 10080,     /* KW_PROTO_TEMPLATE  */
    KW_MARK_MODE = 10081,          /* KW_MARK_MODE  */
    KW_ENCODING = 10082,           /* KW_ENCODING  */
    KW_TYPE = 10083,               /* KW_TYPE  */
    KW_STATS_MAX_DYNAMIC = 10084,  /* KW_STATS_MAX_DYNAMIC  */
    KW_MIN_IW_SIZE_PER_READER = 10085, /* KW_MIN_IW_SIZE_PER_READER  */
    KW_WORKERS = 10086,            /* KW_WORKERS  */
    KW_BATCH_LINES = 10087,        /* KW_BATCH_LINES  */
    KW_BATCH_TIMEOUT = 10088,      /* KW_BATCH_TIMEOUT  */
    KW_TRIM_LARGE_MESSAGES = 10089, /* KW_TRIM_LARGE_MESSAGES  */
    KW_STATS = 10400,              /* KW_STATS  */
    KW_FREQ = 10401,               /* KW_FREQ  */
    KW_LEVEL = 10402,              /* KW_LEVEL  */
    KW_LIFETIME = 10403,           /* KW_LIFETIME  */
    KW_MAX_DYNAMIC = 10404,        /* KW_MAX_DYNAMIC  */
    KW_SYSLOG_STATS = 10405,       /* KW_SYSLOG_STATS  */
    KW_HEALTHCHECK_FREQ = 10406,   /* KW_HEALTHCHECK_FREQ  */
    KW_WORKER_PARTITION_KEY = 10407, /* KW_WORKER_PARTITION_KEY  */
    KW_CHAIN_HOSTNAMES = 10090,    /* KW_CHAIN_HOSTNAMES  */
    KW_NORMALIZE_HOSTNAMES = 10091, /* KW_NORMALIZE_HOSTNAMES  */
    KW_KEEP_HOSTNAME = 10092,      /* KW_KEEP_HOSTNAME  */
    KW_CHECK_HOSTNAME = 10093,     /* KW_CHECK_HOSTNAME  */
    KW_BAD_HOSTNAME = 10094,       /* KW_BAD_HOSTNAME  */
    KW_LOG_LEVEL = 10095,          /* KW_LOG_LEVEL  */
    KW_IDLE_TIMEOUT = 10096,       /* KW_IDLE_TIMEOUT  */
    KW_CHECK_PROGRAM = 10097,      /* KW_CHECK_PROGRAM  */
    KW_KEEP_TIMESTAMP = 10100,     /* KW_KEEP_TIMESTAMP  */
    KW_USE_DNS = 10110,            /* KW_USE_DNS  */
    KW_USE_FQDN = 10111,           /* KW_USE_FQDN  */
    KW_CUSTOM_DOMAIN = 10112,      /* KW_CUSTOM_DOMAIN  */
    KW_DNS_CACHE = 10120,          /* KW_DNS_CACHE  */
    KW_DNS_CACHE_SIZE = 10121,     /* KW_DNS_CACHE_SIZE  */
    KW_DNS_CACHE_EXPIRE = 10130,   /* KW_DNS_CACHE_EXPIRE  */
    KW_DNS_CACHE_EXPIRE_FAILED = 10131, /* KW_DNS_CACHE_EXPIRE_FAILED  */
    KW_DNS_CACHE_HOSTS = 10132,    /* KW_DNS_CACHE_HOSTS  */
    KW_PERSIST_ONLY = 10140,       /* KW_PERSIST_ONLY  */
    KW_USE_RCPTID = 10141,         /* KW_USE_RCPTID  */
    KW_USE_UNIQID = 10142,         /* KW_USE_UNIQID  */
    KW_TZ_CONVERT = 10150,         /* KW_TZ_CONVERT  */
    KW_TS_FORMAT = 10151,          /* KW_TS_FORMAT  */
    KW_FRAC_DIGITS = 10152,        /* KW_FRAC_DIGITS  */
    KW_LOG_FIFO_SIZE = 10160,      /* KW_LOG_FIFO_SIZE  */
    KW_LOG_FETCH_LIMIT = 10162,    /* KW_LOG_FETCH_LIMIT  */
    KW_LOG_IW_SIZE = 10163,        /* KW_LOG_IW_SIZE  */
    KW_LOG_PREFIX = 10164,         /* KW_LOG_PREFIX  */
    KW_PROGRAM_OVERRIDE = 10165,   /* KW_PROGRAM_OVERRIDE  */
    KW_HOST_OVERRIDE = 10166,      /* KW_HOST_OVERRIDE  */
    KW_THROTTLE = 10170,           /* KW_THROTTLE  */
    KW_THREADED = 10171,           /* KW_THREADED  */
    KW_PASS_UNIX_CREDENTIALS = 10180, /* KW_PASS_UNIX_CREDENTIALS  */
    KW_PERSIST_NAME = 10181,       /* KW_PERSIST_NAME  */
    KW_USE_SYSLOGNG_PID = 10182,   /* KW_USE_SYSLOGNG_PID  */
    KW_READ_OLD_RECORDS = 10185,   /* KW_READ_OLD_RECORDS  */
    KW_IGNORE_SAVED_BOOKMARKS = 10186, /* KW_IGNORE_SAVED_BOOKMARKS  */
    KW_DISABLE_BOOKMARKS = 10187,  /* KW_DISABLE_BOOKMARKS  */
    KW_FLAGS = 10190,              /* KW_FLAGS  */
    KW_PAD_SIZE = 10200,           /* KW_PAD_SIZE  */
    KW_TIME_ZONE = 10201,          /* KW_TIME_ZONE  */
    KW_RECV_TIME_ZONE = 10202,     /* KW_RECV_TIME_ZONE  */
    KW_SEND_TIME_ZONE = 10203,     /* KW_SEND_TIME_ZONE  */
    KW_LOCAL_TIME_ZONE = 10204,    /* KW_LOCAL_TIME_ZONE  */
    KW_FORMAT = 10205,             /* KW_FORMAT  */
    KW_MULTI_LINE_MODE = 10206,    /* KW_MULTI_LINE_MODE  */
    KW_MULTI_LINE_PREFIX = 10207,  /* KW_MULTI_LINE_PREFIX  */
    KW_MULTI_LINE_GARBAGE = 10208, /* KW_MULTI_LINE_GARBAGE  */
    KW_TRUNCATE_SIZE = 10209,      /* KW_TRUNCATE_SIZE  */
    KW_TIME_REOPEN = 10210,        /* KW_TIME_REOPEN  */
    KW_TIME_REAP = 10211,          /* KW_TIME_REAP  */
    KW_TIME_SLEEP = 10212,         /* KW_TIME_SLEEP  */
    KW_PARALLELIZE = 10215,        /* KW_PARALLELIZE  */
    KW_PARTITIONS = 10216,         /* KW_PARTITIONS  */
    KW_PARTITION_KEY = 10217,      /* KW_PARTITION_KEY  */
    KW_TMPL_ESCAPE = 10220,        /* KW_TMPL_ESCAPE  */
    KW_LOG_FETCH_DELAY = 10230,    /* KW_LOG_FETCH_DELAY  */
    KW_LOG_FETCH_RETRY_DELAY = 10231, /* KW_LOG_FETCH_RETRY_DELAY  */
    KW_OPTIONAL = 10235,           /* KW_OPTIONAL  */
    KW_CREATE_DIRS = 10240,        /* KW_CREATE_DIRS  */
    KW_OWNER = 10250,              /* KW_OWNER  */
    KW_GROUP = 10251,              /* KW_GROUP  */
    KW_PERM = 10252,               /* KW_PERM  */
    KW_DIR_OWNER = 10260,          /* KW_DIR_OWNER  */
    KW_DIR_GROUP = 10261,          /* KW_DIR_GROUP  */
    KW_DIR_PERM = 10262,           /* KW_DIR_PERM  */
    KW_TEMPLATE = 10270,           /* KW_TEMPLATE  */
    KW_TEMPLATE_ESCAPE = 10271,    /* KW_TEMPLATE_ESCAPE  */
    KW_TEMPLATE_FUNCTION = 10272,  /* KW_TEMPLATE_FUNCTION  */
    KW_DEFAULT_FACILITY = 10300,   /* KW_DEFAULT_FACILITY  */
    KW_DEFAULT_SEVERITY = 10301,   /* KW_DEFAULT_SEVERITY  */
    KW_SDATA_PREFIX = 10302,       /* KW_SDATA_PREFIX  */
    KW_PORT = 10323,               /* KW_PORT  */
    KW_USE_TIME_RECVD = 10340,     /* KW_USE_TIME_RECVD  */
    KW_FACILITY = 10350,           /* KW_FACILITY  */
    KW_SEVERITY = 10351,           /* KW_SEVERITY  */
    KW_HOST = 10352,               /* KW_HOST  */
    KW_MATCH = 10353,              /* KW_MATCH  */
    KW_MESSAGE = 10354,            /* KW_MESSAGE  */
    KW_NETMASK = 10355,            /* KW_NETMASK  */
    KW_TAGS = 10356,               /* KW_TAGS  */
    KW_NETMASK6 = 10357,           /* KW_NETMASK6  */
    KW_REWRITE = 10370,            /* KW_REWRITE  */
    KW_CONDITION = 10371,          /* KW_CONDITION  */
    KW_VALUE = 10372,              /* KW_VALUE  */
    KW_YES = 10380,                /* KW_YES  */
    KW_NO = 10381,                 /* KW_NO  */
    KW_AUTO = 10382,               /* KW_AUTO  */
    KW_IFDEF = 10410,              /* KW_IFDEF  */
    KW_ENDIF = 10411,              /* KW_ENDIF  */
    LL_DOTDOT = 10420,             /* LL_DOTDOT  */
    LL_DOTDOTDOT = 10421,          /* LL_DOTDOTDOT  */
    LL_PRAGMA = 10422,             /* LL_PRAGMA  */
    LL_EOL = 10423,                /* LL_EOL  */
    LL_ERROR = 10424,              /* LL_ERROR  */
    LL_ARROW = 10425,              /* LL_ARROW  */
    LL_IDENTIFIER = 10430,         /* LL_IDENTIFIER  */
    LL_NUMBER = 10431,             /* LL_NUMBER  */
    LL_FLOAT = 10432,              /* LL_FLOAT  */
    LL_STRING = 10433,             /* LL_STRING  */
    LL_TOKEN = 10434,              /* LL_TOKEN  */
    LL_BLOCK = 10435,              /* LL_BLOCK  */
    LL_PLUGIN = 10436,             /* LL_PLUGIN  */
    LL_TEMPLATE_REF = 10437,       /* LL_TEMPLATE_REF  */
    LL_MESSAGE_REF = 10438,        /* LL_MESSAGE_REF  */
    KW_VALUE_PAIRS = 10500,        /* KW_VALUE_PAIRS  */
    KW_EXCLUDE = 10502,            /* KW_EXCLUDE  */
    KW_PAIR = 10503,               /* KW_PAIR  */
    KW_KEY = 10504,                /* KW_KEY  */
    KW_SCOPE = 10505,              /* KW_SCOPE  */
    KW_SHIFT = 10506,              /* KW_SHIFT  */
    KW_SHIFT_LEVELS = 10507,       /* KW_SHIFT_LEVELS  */
    KW_REKEY = 10508,              /* KW_REKEY  */
    KW_ADD_PREFIX = 10509,         /* KW_ADD_PREFIX  */
    KW_REPLACE_PREFIX = 10510,     /* KW_REPLACE_PREFIX  */
    KW_CAST = 10511,               /* KW_CAST  */
    KW_UPPER = 10512,              /* KW_UPPER  */
    KW_LOWER = 10513,              /* KW_LOWER  */
    KW_INCLUDE_BYTES = 10514,      /* KW_INCLUDE_BYTES  */
    KW_ON_ERROR = 10520,           /* KW_ON_ERROR  */
    KW_RETRIES = 10521,            /* KW_RETRIES  */
    KW_FETCH_NO_DATA_DELAY = 10522, /* KW_FETCH_NO_DATA_DELAY  */
    KW_LABELS = 10530              /* KW_LABELS  */
  };
  typedef enum afsocket_tokentype afsocket_token_kind_t;
#endif
/* Token kinds.  */
#define AFSOCKET_EMPTY -2
#define AFSOCKET_EOF 0
#define AFSOCKET_error 256
#define AFSOCKET_UNDEF 20001
#define KW_UNIX_STREAM 20000
#define KW_UNIX_DGRAM 20002
#define KW_TCP 20003
#define KW_UDP 20004
#define KW_TCP6 20005
#define KW_UDP6 20006
#define KW_NETWORK 20007
#define KW_HTTP 10324
#define KW_TRANSPORT 20008
#define KW_IP_PROTOCOL 20009
#define KW_SYSTEMD_SYSLOG 20010
#define KW_HTTP_SCRAPER 20011
#define KW_SCRAPE_PATTERN 20012
#define KW_SCRAPE_FREQ_LIMIT 20013
#define KW_SINGLE_INSTANCE 20014
#define KW_STAT_TYPE 20015
#define KW_STAT_QUERY 20016
#define KW_STAT_FORMAT 20017
#define KW_STATS_WITHOUT_ORPHANED 20018
#define KW_STATS_WITH_LEGACY 20019
#define KW_IP_TTL 20020
#define KW_SO_BROADCAST 20021
#define KW_IP_TOS 20022
#define KW_IP_FREEBIND 20023
#define KW_SO_SNDBUF 20024
#define KW_SO_RCVBUF 20025
#define KW_SO_KEEPALIVE 20026
#define KW_SO_REUSEPORT 20027
#define KW_TCP_KEEPALIVE_TIME 20028
#define KW_TCP_KEEPALIVE_PROBES 20029
#define KW_TCP_KEEPALIVE_INTVL 20030
#define KW_SO_PASSCRED 20031
#define KW_LISTEN_BACKLOG 20032
#define KW_SPOOF_SOURCE 20033
#define KW_SPOOF_SOURCE_MAX_MSGLEN 20034
#define KW_KEEP_ALIVE 20035
#define KW_MAX_CONNECTIONS 20036
#define KW_CLOSE_ON_INPUT 20037
#define KW_LOCALIP 20038
#define KW_IP 20039
#define KW_INTERFACE 20040
#define KW_LOCALPORT 20041
#define KW_DESTPORT 20042
#define KW_FAILOVER_SERVERS 20043
#define KW_FAILOVER 20044
#define KW_SERVERS 20045
#define KW_FAILBACK 20046
#define KW_TCP_PROBE_INTERVAL 20047
#define KW_SUCCESSFUL_PROBES_REQUIRED 20048
#define KW_DYNAMIC_WINDOW_SIZE 20049
#define KW_DYNAMIC_WINDOW_STATS_FREQ 20050
#define KW_DYNAMIC_WINDOW_REALLOC_TICKS 20051
#define KW_TLS 20052
#define KW_PEER_VERIFY 20053
#define KW_KEY_FILE 20054
#define KW_CERT_FILE 20055
#define KW_DHPARAM_FILE 20056
#define KW_PKCS12_FILE 20057
#define KW_CA_DIR 20058
#define KW_CRL_DIR 20059
#define KW_CA_FILE 20060
#define KW_TRUSTED_KEYS 20061
#define KW_TRUSTED_DN 20062
#define KW_CIPHER_SUITE 20063
#define KW_TLS12_AND_OLDER 20064
#define KW_TLS13 20065
#define KW_SIGALGS 20066
#define KW_CLIENT_SIGALGS 20067
#define KW_ECDH_CURVE_LIST 20068
#define KW_SSL_OPTIONS 20069
#define KW_SSL_VERSION 20070
#define KW_SNI 20071
#define KW_ALLOW_COMPRESS 20072
#define KW_KEYLOG_FILE 20073
#define KW_OCSP_STAPLING_VERIFY 20074
#define KW_EXTENDED_KEY_USAGE_VERIFY 20075
#define KW_CONF_CMDS 20076
#define LL_CONTEXT_ROOT 1
#define LL_CONTEXT_DESTINATION 2
#define LL_CONTEXT_SOURCE 3
#define LL_CONTEXT_PARSER 4
#define LL_CONTEXT_REWRITE 5
#define LL_CONTEXT_FILTER 6
#define LL_CONTEXT_LOG 7
#define LL_CONTEXT_BLOCK_DEF 8
#define LL_CONTEXT_BLOCK_REF 9
#define LL_CONTEXT_BLOCK_CONTENT 10
#define LL_CONTEXT_BLOCK_ARG 11
#define LL_CONTEXT_BLOCK_FUNCARG 12
#define LL_CONTEXT_PRAGMA 13
#define LL_CONTEXT_FORMAT 14
#define LL_CONTEXT_TEMPLATE_FUNC 15
#define LL_CONTEXT_INNER_DEST 16
#define LL_CONTEXT_INNER_SRC 17
#define LL_CONTEXT_CLIENT_PROTO 18
#define LL_CONTEXT_SERVER_PROTO 19
#define LL_CONTEXT_OPTIONS 20
#define LL_CONTEXT_CONFIG 21
#define LL_CONTEXT_TEMPLATE_REF 22
#define LL_CONTEXT_MAX 23
#define KW_ASSIGN 9000
#define KW_OR 9010
#define KW_AND 9020
#define KW_STR_EQ 9030
#define KW_STR_NE 9031
#define KW_TA_EQ 9032
#define KW_TA_NE 9033
#define KW_TAV_EQ 9034
#define KW_TAV_NE 9035
#define KW_STR_LT 9040
#define KW_STR_LE 9041
#define KW_STR_GE 9042
#define KW_STR_GT 9043
#define KW_TA_LT 9044
#define KW_TA_LE 9045
#define KW_TA_GE 9046
#define KW_TA_GT 9047
#define KW_PLUS_ASSIGN 9050
#define KW_REGEXP_MATCH 9060
#define KW_NOT 9049
#define KW_SOURCE 10000
#define KW_FILTER 10001
#define KW_PARSER 10002
#define KW_DESTINATION 10003
#define KW_LOG 10004
#define KW_OPTIONS 10005
#define KW_INCLUDE 10006
#define KW_BLOCK 10007
#define KW_JUNCTION 10008
#define KW_CHANNEL 10009
#define KW_IF 10010
#define KW_ELSE 10011
#define KW_ELIF 10012
#define KW_INTERNAL 10020
#define KW_SYSLOG 10060
#define KW_MARK_FREQ 10071
#define KW_STATS_FREQ 10072
#define KW_STATS_LEVEL 10073
#define KW_STATS_LIFETIME 10074
#define KW_FLUSH_LINES 10075
#define KW_SUPPRESS 10076
#define KW_FLUSH_TIMEOUT 10077
#define KW_LOG_MSG_SIZE 10078
#define KW_FILE_TEMPLATE 10079
#define KW_PROTO_TEMPLATE 10080
#define KW_MARK_MODE 10081
#define KW_ENCODING 10082
#define KW_TYPE 10083
#define KW_STATS_MAX_DYNAMIC 10084
#define KW_MIN_IW_SIZE_PER_READER 10085
#define KW_WORKERS 10086
#define KW_BATCH_LINES 10087
#define KW_BATCH_TIMEOUT 10088
#define KW_TRIM_LARGE_MESSAGES 10089
#define KW_STATS 10400
#define KW_FREQ 10401
#define KW_LEVEL 10402
#define KW_LIFETIME 10403
#define KW_MAX_DYNAMIC 10404
#define KW_SYSLOG_STATS 10405
#define KW_HEALTHCHECK_FREQ 10406
#define KW_WORKER_PARTITION_KEY 10407
#define KW_CHAIN_HOSTNAMES 10090
#define KW_NORMALIZE_HOSTNAMES 10091
#define KW_KEEP_HOSTNAME 10092
#define KW_CHECK_HOSTNAME 10093
#define KW_BAD_HOSTNAME 10094
#define KW_LOG_LEVEL 10095
#define KW_IDLE_TIMEOUT 10096
#define KW_CHECK_PROGRAM 10097
#define KW_KEEP_TIMESTAMP 10100
#define KW_USE_DNS 10110
#define KW_USE_FQDN 10111
#define KW_CUSTOM_DOMAIN 10112
#define KW_DNS_CACHE 10120
#define KW_DNS_CACHE_SIZE 10121
#define KW_DNS_CACHE_EXPIRE 10130
#define KW_DNS_CACHE_EXPIRE_FAILED 10131
#define KW_DNS_CACHE_HOSTS 10132
#define KW_PERSIST_ONLY 10140
#define KW_USE_RCPTID 10141
#define KW_USE_UNIQID 10142
#define KW_TZ_CONVERT 10150
#define KW_TS_FORMAT 10151
#define KW_FRAC_DIGITS 10152
#define KW_LOG_FIFO_SIZE 10160
#define KW_LOG_FETCH_LIMIT 10162
#define KW_LOG_IW_SIZE 10163
#define KW_LOG_PREFIX 10164
#define KW_PROGRAM_OVERRIDE 10165
#define KW_HOST_OVERRIDE 10166
#define KW_THROTTLE 10170
#define KW_THREADED 10171
#define KW_PASS_UNIX_CREDENTIALS 10180
#define KW_PERSIST_NAME 10181
#define KW_USE_SYSLOGNG_PID 10182
#define KW_READ_OLD_RECORDS 10185
#define KW_IGNORE_SAVED_BOOKMARKS 10186
#define KW_DISABLE_BOOKMARKS 10187
#define KW_FLAGS 10190
#define KW_PAD_SIZE 10200
#define KW_TIME_ZONE 10201
#define KW_RECV_TIME_ZONE 10202
#define KW_SEND_TIME_ZONE 10203
#define KW_LOCAL_TIME_ZONE 10204
#define KW_FORMAT 10205
#define KW_MULTI_LINE_MODE 10206
#define KW_MULTI_LINE_PREFIX 10207
#define KW_MULTI_LINE_GARBAGE 10208
#define KW_TRUNCATE_SIZE 10209
#define KW_TIME_REOPEN 10210
#define KW_TIME_REAP 10211
#define KW_TIME_SLEEP 10212
#define KW_PARALLELIZE 10215
#define KW_PARTITIONS 10216
#define KW_PARTITION_KEY 10217
#define KW_TMPL_ESCAPE 10220
#define KW_LOG_FETCH_DELAY 10230
#define KW_LOG_FETCH_RETRY_DELAY 10231
#define KW_OPTIONAL 10235
#define KW_CREATE_DIRS 10240
#define KW_OWNER 10250
#define KW_GROUP 10251
#define KW_PERM 10252
#define KW_DIR_OWNER 10260
#define KW_DIR_GROUP 10261
#define KW_DIR_PERM 10262
#define KW_TEMPLATE 10270
#define KW_TEMPLATE_ESCAPE 10271
#define KW_TEMPLATE_FUNCTION 10272
#define KW_DEFAULT_FACILITY 10300
#define KW_DEFAULT_SEVERITY 10301
#define KW_SDATA_PREFIX 10302
#define KW_PORT 10323
#define KW_USE_TIME_RECVD 10340
#define KW_FACILITY 10350
#define KW_SEVERITY 10351
#define KW_HOST 10352
#define KW_MATCH 10353
#define KW_MESSAGE 10354
#define KW_NETMASK 10355
#define KW_TAGS 10356
#define KW_NETMASK6 10357
#define KW_REWRITE 10370
#define KW_CONDITION 10371
#define KW_VALUE 10372
#define KW_YES 10380
#define KW_NO 10381
#define KW_AUTO 10382
#define KW_IFDEF 10410
#define KW_ENDIF 10411
#define LL_DOTDOT 10420
#define LL_DOTDOTDOT 10421
#define LL_PRAGMA 10422
#define LL_EOL 10423
#define LL_ERROR 10424
#define LL_ARROW 10425
#define LL_IDENTIFIER 10430
#define LL_NUMBER 10431
#define LL_FLOAT 10432
#define LL_STRING 10433
#define LL_TOKEN 10434
#define LL_BLOCK 10435
#define LL_PLUGIN 10436
#define LL_TEMPLATE_REF 10437
#define LL_MESSAGE_REF 10438
#define KW_VALUE_PAIRS 10500
#define KW_EXCLUDE 10502
#define KW_PAIR 10503
#define KW_KEY 10504
#define KW_SCOPE 10505
#define KW_SHIFT 10506
#define KW_SHIFT_LEVELS 10507
#define KW_REKEY 10508
#define KW_ADD_PREFIX 10509
#define KW_REPLACE_PREFIX 10510
#define KW_CAST 10511
#define KW_UPPER 10512
#define KW_LOWER 10513
#define KW_INCLUDE_BYTES 10514
#define KW_ON_ERROR 10520
#define KW_RETRIES 10521
#define KW_FETCH_NO_DATA_DELAY 10522
#define KW_LABELS 10530

/* Value type.  */
#if ! defined AFSOCKET_STYPE && ! defined AFSOCKET_STYPE_IS_DECLARED
typedef CFG_STYPE AFSOCKET_STYPE;
# define AFSOCKET_STYPE_IS_TRIVIAL 1
# define AFSOCKET_STYPE_IS_DECLARED 1
#endif

/* Location type.  */
typedef CFG_LTYPE AFSOCKET_LTYPE;




int afsocket_parse (CfgLexer *lexer, LogDriver **instance, gpointer arg);


#endif /* !YY_AFSOCKET_MODULES_AFSOCKET_AFSOCKET_GRAMMAR_H_INCLUDED  */
/* Symbol kind.  */
enum yysymbol_kind_t
{
  YYSYMBOL_YYEMPTY = -2,
  YYSYMBOL_YYEOF = 0,                      /* "end of file"  */
  YYSYMBOL_YYerror = 1,                    /* error  */
  YYSYMBOL_YYUNDEF = 2,                    /* "invalid token"  */
  YYSYMBOL_KW_UNIX_STREAM = 3,             /* KW_UNIX_STREAM  */
  YYSYMBOL_KW_UNIX_DGRAM = 4,              /* KW_UNIX_DGRAM  */
  YYSYMBOL_KW_TCP = 5,                     /* KW_TCP  */
  YYSYMBOL_KW_UDP = 6,                     /* KW_UDP  */
  YYSYMBOL_KW_TCP6 = 7,                    /* KW_TCP6  */
  YYSYMBOL_KW_UDP6 = 8,                    /* KW_UDP6  */
  YYSYMBOL_KW_NETWORK = 9,                 /* KW_NETWORK  */
  YYSYMBOL_KW_HTTP = 10,                   /* KW_HTTP  */
  YYSYMBOL_KW_TRANSPORT = 11,              /* KW_TRANSPORT  */
  YYSYMBOL_KW_IP_PROTOCOL = 12,            /* KW_IP_PROTOCOL  */
  YYSYMBOL_KW_SYSTEMD_SYSLOG = 13,         /* KW_SYSTEMD_SYSLOG  */
  YYSYMBOL_KW_HTTP_SCRAPER = 14,           /* KW_HTTP_SCRAPER  */
  YYSYMBOL_KW_SCRAPE_PATTERN = 15,         /* KW_SCRAPE_PATTERN  */
  YYSYMBOL_KW_SCRAPE_FREQ_LIMIT = 16,      /* KW_SCRAPE_FREQ_LIMIT  */
  YYSYMBOL_KW_SINGLE_INSTANCE = 17,        /* KW_SINGLE_INSTANCE  */
  YYSYMBOL_KW_STAT_TYPE = 18,              /* KW_STAT_TYPE  */
  YYSYMBOL_KW_STAT_QUERY = 19,             /* KW_STAT_QUERY  */
  YYSYMBOL_KW_STAT_FORMAT = 20,            /* KW_STAT_FORMAT  */
  YYSYMBOL_KW_STATS_WITHOUT_ORPHANED = 21, /* KW_STATS_WITHOUT_ORPHANED  */
  YYSYMBOL_KW_STATS_WITH_LEGACY = 22,      /* KW_STATS_WITH_LEGACY  */
  YYSYMBOL_KW_IP_TTL = 23,                 /* KW_IP_TTL  */
  YYSYMBOL_KW_SO_BROADCAST = 24,           /* KW_SO_BROADCAST  */
  YYSYMBOL_KW_IP_TOS = 25,                 /* KW_IP_TOS  */
  YYSYMBOL_KW_IP_FREEBIND = 26,            /* KW_IP_FREEBIND  */
  YYSYMBOL_KW_SO_SNDBUF = 27,              /* KW_SO_SNDBUF  */
  YYSYMBOL_KW_SO_RCVBUF = 28,              /* KW_SO_RCVBUF  */
  YYSYMBOL_KW_SO_KEEPALIVE = 29,           /* KW_SO_KEEPALIVE  */
  YYSYMBOL_KW_SO_REUSEPORT = 30,           /* KW_SO_REUSEPORT  */
  YYSYMBOL_KW_TCP_KEEPALIVE_TIME = 31,     /* KW_TCP_KEEPALIVE_TIME  */
  YYSYMBOL_KW_TCP_KEEPALIVE_PROBES = 32,   /* KW_TCP_KEEPALIVE_PROBES  */
  YYSYMBOL_KW_TCP_KEEPALIVE_INTVL = 33,    /* KW_TCP_KEEPALIVE_INTVL  */
  YYSYMBOL_KW_SO_PASSCRED = 34,            /* KW_SO_PASSCRED  */
  YYSYMBOL_KW_LISTEN_BACKLOG = 35,         /* KW_LISTEN_BACKLOG  */
  YYSYMBOL_KW_SPOOF_SOURCE = 36,           /* KW_SPOOF_SOURCE  */
  YYSYMBOL_KW_SPOOF_SOURCE_MAX_MSGLEN = 37, /* KW_SPOOF_SOURCE_MAX_MSGLEN  */
  YYSYMBOL_KW_KEEP_ALIVE = 38,             /* KW_KEEP_ALIVE  */
  YYSYMBOL_KW_MAX_CONNECTIONS = 39,        /* KW_MAX_CONNECTIONS  */
  YYSYMBOL_KW_CLOSE_ON_INPUT = 40,         /* KW_CLOSE_ON_INPUT  */
  YYSYMBOL_KW_LOCALIP = 41,                /* KW_LOCALIP  */
  YYSYMBOL_KW_IP = 42,                     /* KW_IP  */
  YYSYMBOL_KW_INTERFACE = 43,              /* KW_INTERFACE  */
  YYSYMBOL_KW_LOCALPORT = 44,              /* KW_LOCALPORT  */
  YYSYMBOL_KW_DESTPORT = 45,               /* KW_DESTPORT  */
  YYSYMBOL_KW_FAILOVER_SERVERS = 46,       /* KW_FAILOVER_SERVERS  */
  YYSYMBOL_KW_FAILOVER = 47,               /* KW_FAILOVER  */
  YYSYMBOL_KW_SERVERS = 48,                /* KW_SERVERS  */
  YYSYMBOL_KW_FAILBACK = 49,               /* KW_FAILBACK  */
  YYSYMBOL_KW_TCP_PROBE_INTERVAL = 50,     /* KW_TCP_PROBE_INTERVAL  */
  YYSYMBOL_KW_SUCCESSFUL_PROBES_REQUIRED = 51, /* KW_SUCCESSFUL_PROBES_REQUIRED  */
  YYSYMBOL_KW_DYNAMIC_WINDOW_SIZE = 52,    /* KW_DYNAMIC_WINDOW_SIZE  */
  YYSYMBOL_KW_DYNAMIC_WINDOW_STATS_FREQ = 53, /* KW_DYNAMIC_WINDOW_STATS_FREQ  */
  YYSYMBOL_KW_DYNAMIC_WINDOW_REALLOC_TICKS = 54, /* KW_DYNAMIC_WINDOW_REALLOC_TICKS  */
  YYSYMBOL_KW_TLS = 55,                    /* KW_TLS  */
  YYSYMBOL_KW_PEER_VERIFY = 56,            /* KW_PEER_VERIFY  */
  YYSYMBOL_KW_KEY_FILE = 57,               /* KW_KEY_FILE  */
  YYSYMBOL_KW_CERT_FILE = 58,              /* KW_CERT_FILE  */
  YYSYMBOL_KW_DHPARAM_FILE = 59,           /* KW_DHPARAM_FILE  */
  YYSYMBOL_KW_PKCS12_FILE = 60,            /* KW_PKCS12_FILE  */
  YYSYMBOL_KW_CA_DIR = 61,                 /* KW_CA_DIR  */
  YYSYMBOL_KW_CRL_DIR = 62,                /* KW_CRL_DIR  */
  YYSYMBOL_KW_CA_FILE = 63,                /* KW_CA_FILE  */
  YYSYMBOL_KW_TRUSTED_KEYS = 64,           /* KW_TRUSTED_KEYS  */
  YYSYMBOL_KW_TRUSTED_DN = 65,             /* KW_TRUSTED_DN  */
  YYSYMBOL_KW_CIPHER_SUITE = 66,           /* KW_CIPHER_SUITE  */
  YYSYMBOL_KW_TLS12_AND_OLDER = 67,        /* KW_TLS12_AND_OLDER  */
  YYSYMBOL_KW_TLS13 = 68,                  /* KW_TLS13  */
  YYSYMBOL_KW_SIGALGS = 69,                /* KW_SIGALGS  */
  YYSYMBOL_KW_CLIENT_SIGALGS = 70,         /* KW_CLIENT_SIGALGS  */
  YYSYMBOL_KW_ECDH_CURVE_LIST = 71,        /* KW_ECDH_CURVE_LIST  */
  YYSYMBOL_KW_SSL_OPTIONS = 72,            /* KW_SSL_OPTIONS  */
  YYSYMBOL_KW_SSL_VERSION = 73,            /* KW_SSL_VERSION  */
  YYSYMBOL_KW_SNI = 74,                    /* KW_SNI  */
  YYSYMBOL_KW_ALLOW_COMPRESS = 75,         /* KW_ALLOW_COMPRESS  */
  YYSYMBOL_KW_KEYLOG_FILE = 76,            /* KW_KEYLOG_FILE  */
  YYSYMBOL_KW_OCSP_STAPLING_VERIFY = 77,   /* KW_OCSP_STAPLING_VERIFY  */
  YYSYMBOL_KW_EXTENDED_KEY_USAGE_VERIFY = 78, /* KW_EXTENDED_KEY_USAGE_VERIFY  */
  YYSYMBOL_KW_CONF_CMDS = 79,              /* KW_CONF_CMDS  */
  YYSYMBOL_LL_CONTEXT_ROOT = 80,           /* LL_CONTEXT_ROOT  */
  YYSYMBOL_LL_CONTEXT_DESTINATION = 81,    /* LL_CONTEXT_DESTINATION  */
  YYSYMBOL_LL_CONTEXT_SOURCE = 82,         /* LL_CONTEXT_SOURCE  */
  YYSYMBOL_LL_CONTEXT_PARSER = 83,         /* LL_CONTEXT_PARSER  */
  YYSYMBOL_LL_CONTEXT_REWRITE = 84,        /* LL_CONTEXT_REWRITE  */
  YYSYMBOL_LL_CONTEXT_FILTER = 85,         /* LL_CONTEXT_FILTER  */
  YYSYMBOL_LL_CONTEXT_LOG = 86,            /* LL_CONTEXT_LOG  */
  YYSYMBOL_LL_CONTEXT_BLOCK_DEF = 87,      /* LL_CONTEXT_BLOCK_DEF  */
  YYSYMBOL_LL_CONTEXT_BLOCK_REF = 88,      /* LL_CONTEXT_BLOCK_REF  */
  YYSYMBOL_LL_CONTEXT_BLOCK_CONTENT = 89,  /* LL_CONTEXT_BLOCK_CONTENT  */
  YYSYMBOL_LL_CONTEXT_BLOCK_ARG = 90,      /* LL_CONTEXT_BLOCK_ARG  */
  YYSYMBOL_LL_CONTEXT_BLOCK_FUNCARG = 91,  /* LL_CONTEXT_BLOCK_FUNCARG  */
  YYSYMBOL_LL_CONTEXT_PRAGMA = 92,         /* LL_CONTEXT_PRAGMA  */
  YYSYMBOL_LL_CONTEXT_FORMAT = 93,         /* LL_CONTEXT_FORMAT  */
  YYSYMBOL_LL_CONTEXT_TEMPLATE_FUNC = 94,  /* LL_CONTEXT_TEMPLATE_FUNC  */
  YYSYMBOL_LL_CONTEXT_INNER_DEST = 95,     /* LL_CONTEXT_INNER_DEST  */
  YYSYMBOL_LL_CONTEXT_INNER_SRC = 96,      /* LL_CONTEXT_INNER_SRC  */
  YYSYMBOL_LL_CONTEXT_CLIENT_PROTO = 97,   /* LL_CONTEXT_CLIENT_PROTO  */
  YYSYMBOL_LL_CONTEXT_SERVER_PROTO = 98,   /* LL_CONTEXT_SERVER_PROTO  */
  YYSYMBOL_LL_CONTEXT_OPTIONS = 99,        /* LL_CONTEXT_OPTIONS  */
  YYSYMBOL_LL_CONTEXT_CONFIG = 100,        /* LL_CONTEXT_CONFIG  */
  YYSYMBOL_LL_CONTEXT_TEMPLATE_REF = 101,  /* LL_CONTEXT_TEMPLATE_REF  */
  YYSYMBOL_LL_CONTEXT_MAX = 102,           /* LL_CONTEXT_MAX  */
  YYSYMBOL_103_ = 103,                     /* ';'  */
  YYSYMBOL_KW_ASSIGN = 104,                /* KW_ASSIGN  */
  YYSYMBOL_105_ = 105,                     /* '?'  */
  YYSYMBOL_106_ = 106,                     /* ':'  */
  YYSYMBOL_KW_OR = 107,                    /* KW_OR  */
  YYSYMBOL_KW_AND = 108,                   /* KW_AND  */
  YYSYMBOL_KW_STR_EQ = 109,                /* KW_STR_EQ  */
  YYSYMBOL_KW_STR_NE = 110,                /* KW_STR_NE  */
  YYSYMBOL_KW_TA_EQ = 111,                 /* KW_TA_EQ  */
  YYSYMBOL_KW_TA_NE = 112,                 /* KW_TA_NE  */
  YYSYMBOL_KW_TAV_EQ = 113,                /* KW_TAV_EQ  */
  YYSYMBOL_KW_TAV_NE = 114,                /* KW_TAV_NE  */
  YYSYMBOL_KW_STR_LT = 115,                /* KW_STR_LT  */
  YYSYMBOL_KW_STR_LE = 116,                /* KW_STR_LE  */
  YYSYMBOL_KW_STR_GE = 117,                /* KW_STR_GE  */
  YYSYMBOL_KW_STR_GT = 118,                /* KW_STR_GT  */
  YYSYMBOL_KW_TA_LT = 119,                 /* KW_TA_LT  */
  YYSYMBOL_KW_TA_LE = 120,                 /* KW_TA_LE  */
  YYSYMBOL_KW_TA_GE = 121,                 /* KW_TA_GE  */
  YYSYMBOL_KW_TA_GT = 122,                 /* KW_TA_GT  */
  YYSYMBOL_KW_PLUS_ASSIGN = 123,           /* KW_PLUS_ASSIGN  */
  YYSYMBOL_KW_REGEXP_MATCH = 124,          /* KW_REGEXP_MATCH  */
  YYSYMBOL_125_ = 125,                     /* '+'  */
  YYSYMBOL_126_ = 126,                     /* '-'  */
  YYSYMBOL_127_ = 127,                     /* '*'  */
  YYSYMBOL_128_ = 128,                     /* '.'  */
  YYSYMBOL_129_ = 129,                     /* '['  */
  YYSYMBOL_130_ = 130,                     /* ']'  */
  YYSYMBOL_KW_NOT = 131,                   /* KW_NOT  */
  YYSYMBOL_KW_SOURCE = 132,                /* KW_SOURCE  */
  YYSYMBOL_KW_FILTER = 133,                /* KW_FILTER  */
  YYSYMBOL_KW_PARSER = 134,                /* KW_PARSER  */
  YYSYMBOL_KW_DESTINATION = 135,           /* KW_DESTINATION  */
  YYSYMBOL_KW_LOG = 136,                   /* KW_LOG  */
  YYSYMBOL_KW_OPTIONS = 137,               /* KW_OPTIONS  */
  YYSYMBOL_KW_INCLUDE = 138,               /* KW_INCLUDE  */
  YYSYMBOL_KW_BLOCK = 139,                 /* KW_BLOCK  */
  YYSYMBOL_KW_JUNCTION = 140,              /* KW_JUNCTION  */
  YYSYMBOL_KW_CHANNEL = 141,               /* KW_CHANNEL  */
  YYSYMBOL_KW_IF = 142,                    /* KW_IF  */
  YYSYMBOL_KW_ELSE = 143,                  /* KW_ELSE  */
  YYSYMBOL_KW_ELIF = 144,                  /* KW_ELIF  */
  YYSYMBOL_KW_INTERNAL = 145,              /* KW_INTERNAL  */
  YYSYMBOL_KW_SYSLOG = 146,                /* KW_SYSLOG  */
  YYSYMBOL_KW_MARK_FREQ = 147,             /* KW_MARK_FREQ  */
  YYSYMBOL_KW_STATS_FREQ = 148,            /* KW_STATS_FREQ  */
  YYSYMBOL_KW_STATS_LEVEL = 149,           /* KW_STATS_LEVEL  */
  YYSYMBOL_KW_STATS_LIFETIME = 150,        /* KW_STATS_LIFETIME  */
  YYSYMBOL_KW_FLUSH_LINES = 151,           /* KW_FLUSH_LINES  */
  YYSYMBOL_KW_SUPPRESS = 152,              /* KW_SUPPRESS  */
  YYSYMBOL_KW_FLUSH_TIMEOUT = 153,         /* KW_FLUSH_TIMEOUT  */
  YYSYMBOL_KW_LOG_MSG_SIZE = 154,          /* KW_LOG_MSG_SIZE  */
  YYSYMBOL_KW_FILE_TEMPLATE = 155,         /* KW_FILE_TEMPLATE  */
  YYSYMBOL_KW_PROTO_TEMPLATE = 156,        /* KW_PROTO_TEMPLATE  */
  YYSYMBOL_KW_MARK_MODE = 157,             /* KW_MARK_MODE  */
  YYSYMBOL_KW_ENCODING = 158,              /* KW_ENCODING  */
  YYSYMBOL_KW_TYPE = 159,                  /* KW_TYPE  */
  YYSYMBOL_KW_STATS_MAX_DYNAMIC = 160,     /* KW_STATS_MAX_DYNAMIC  */
  YYSYMBOL_KW_MIN_IW_SIZE_PER_READER = 161, /* KW_MIN_IW_SIZE_PER_READER  */
  YYSYMBOL_KW_WORKERS = 162,               /* KW_WORKERS  */
  YYSYMBOL_KW_BATCH_LINES = 163,           /* KW_BATCH_LINES  */
  YYSYMBOL_KW_BATCH_TIMEOUT = 164,         /* KW_BATCH_TIMEOUT  */
  YYSYMBOL_KW_TRIM_LARGE_MESSAGES = 165,   /* KW_TRIM_LARGE_MESSAGES  */
  YYSYMBOL_KW_STATS = 166,                 /* KW_STATS  */
  YYSYMBOL_KW_FREQ = 167,                  /* KW_FREQ  */
  YYSYMBOL_KW_LEVEL = 168,                 /* KW_LEVEL  */
  YYSYMBOL_KW_LIFETIME = 169,              /* KW_LIFETIME  */
  YYSYMBOL_KW_MAX_DYNAMIC = 170,           /* KW_MAX_DYNAMIC  */
  YYSYMBOL_KW_SYSLOG_STATS = 171,          /* KW_SYSLOG_STATS  */
  YYSYMBOL_KW_HEALTHCHECK_FREQ = 172,      /* KW_HEALTHCHECK_FREQ  */
  YYSYMBOL_KW_WORKER_PARTITION_KEY = 173,  /* KW_WORKER_PARTITION_KEY  */
  YYSYMBOL_KW_CHAIN_HOSTNAMES = 174,       /* KW_CHAIN_HOSTNAMES  */
  YYSYMBOL_KW_NORMALIZE_HOSTNAMES = 175,   /* KW_NORMALIZE_HOSTNAMES  */
  YYSYMBOL_KW_KEEP_HOSTNAME = 176,         /* KW_KEEP_HOSTNAME  */
  YYSYMBOL_KW_CHECK_HOSTNAME = 177,        /* KW_CHECK_HOSTNAME  */
  YYSYMBOL_KW_BAD_HOSTNAME = 178,          /* KW_BAD_HOSTNAME  */
  YYSYMBOL_KW_LOG_LEVEL = 179,             /* KW_LOG_LEVEL  */
  YYSYMBOL_KW_IDLE_TIMEOUT = 180,          /* KW_IDLE_TIMEOUT  */
  YYSYMBOL_KW_CHECK_PROGRAM = 181,         /* KW_CHECK_PROGRAM  */
  YYSYMBOL_KW_KEEP_TIMESTAMP = 182,        /* KW_KEEP_TIMESTAMP  */
  YYSYMBOL_KW_USE_DNS = 183,               /* KW_USE_DNS  */
  YYSYMBOL_KW_USE_FQDN = 184,              /* KW_USE_FQDN  */
  YYSYMBOL_KW_CUSTOM_DOMAIN = 185,         /* KW_CUSTOM_DOMAIN  */
  YYSYMBOL_KW_DNS_CACHE = 186,             /* KW_DNS_CACHE  */
  YYSYMBOL_KW_DNS_CACHE_SIZE = 187,        /* KW_DNS_CACHE_SIZE  */
  YYSYMBOL_KW_DNS_CACHE_EXPIRE = 188,      /* KW_DNS_CACHE_EXPIRE  */
  YYSYMBOL_KW_DNS_CACHE_EXPIRE_FAILED = 189, /* KW_DNS_CACHE_EXPIRE_FAILED  */
  YYSYMBOL_KW_DNS_CACHE_HOSTS = 190,       /* KW_DNS_CACHE_HOSTS  */
  YYSYMBOL_KW_PERSIST_ONLY = 191,          /* KW_PERSIST_ONLY  */
  YYSYMBOL_KW_USE_RCPTID = 192,            /* KW_USE_RCPTID  */
  YYSYMBOL_KW_USE_UNIQID = 193,            /* KW_USE_UNIQID  */
  YYSYMBOL_KW_TZ_CONVERT = 194,            /* KW_TZ_CONVERT  */
  YYSYMBOL_KW_TS_FORMAT = 195,             /* KW_TS_FORMAT  */
  YYSYMBOL_KW_FRAC_DIGITS = 196,           /* KW_FRAC_DIGITS  */
  YYSYMBOL_KW_LOG_FIFO_SIZE = 197,         /* KW_LOG_FIFO_SIZE  */
  YYSYMBOL_KW_LOG_FETCH_LIMIT = 198,       /* KW_LOG_FETCH_LIMIT  */
  YYSYMBOL_KW_LOG_IW_SIZE = 199,           /* KW_LOG_IW_SIZE  */
  YYSYMBOL_KW_LOG_PREFIX = 200,            /* KW_LOG_PREFIX  */
  YYSYMBOL_KW_PROGRAM_OVERRIDE = 201,      /* KW_PROGRAM_OVERRIDE  */
  YYSYMBOL_KW_HOST_OVERRIDE = 202,         /* KW_HOST_OVERRIDE  */
  YYSYMBOL_KW_THROTTLE = 203,              /* KW_THROTTLE  */
  YYSYMBOL_KW_THREADED = 204,              /* KW_THREADED  */
  YYSYMBOL_KW_PASS_UNIX_CREDENTIALS = 205, /* KW_PASS_UNIX_CREDENTIALS  */
  YYSYMBOL_KW_PERSIST_NAME = 206,          /* KW_PERSIST_NAME  */
  YYSYMBOL_KW_USE_SYSLOGNG_PID = 207,      /* KW_USE_SYSLOGNG_PID  */
  YYSYMBOL_KW_READ_OLD_RECORDS = 208,      /* KW_READ_OLD_RECORDS  */
  YYSYMBOL_KW_IGNORE_SAVED_BOOKMARKS = 209, /* KW_IGNORE_SAVED_BOOKMARKS  */
  YYSYMBOL_KW_DISABLE_BOOKMARKS = 210,     /* KW_DISABLE_BOOKMARKS  */
  YYSYMBOL_KW_FLAGS = 211,                 /* KW_FLAGS  */
  YYSYMBOL_KW_PAD_SIZE = 212,              /* KW_PAD_SIZE  */
  YYSYMBOL_KW_TIME_ZONE = 213,             /* KW_TIME_ZONE  */
  YYSYMBOL_KW_RECV_TIME_ZONE = 214,        /* KW_RECV_TIME_ZONE  */
  YYSYMBOL_KW_SEND_TIME_ZONE = 215,        /* KW_SEND_TIME_ZONE  */
  YYSYMBOL_KW_LOCAL_TIME_ZONE = 216,       /* KW_LOCAL_TIME_ZONE  */
  YYSYMBOL_KW_FORMAT = 217,                /* KW_FORMAT  */
  YYSYMBOL_KW_MULTI_LINE_MODE = 218,       /* KW_MULTI_LINE_MODE  */
  YYSYMBOL_KW_MULTI_LINE_PREFIX = 219,     /* KW_MULTI_LINE_PREFIX  */
  YYSYMBOL_KW_MULTI_LINE_GARBAGE = 220,    /* KW_MULTI_LINE_GARBAGE  */
  YYSYMBOL_KW_TRUNCATE_SIZE = 221,         /* KW_TRUNCATE_SIZE  */
  YYSYMBOL_KW_TIME_REOPEN = 222,           /* KW_TIME_REOPEN  */
  YYSYMBOL_KW_TIME_REAP = 223,             /* KW_TIME_REAP  */
  YYSYMBOL_KW_TIME_SLEEP = 224,            /* KW_TIME_SLEEP  */
  YYSYMBOL_KW_PARALLELIZE = 225,           /* KW_PARALLELIZE  */
  YYSYMBOL_KW_PARTITIONS = 226,            /* KW_PARTITIONS  */
  YYSYMBOL_KW_PARTITION_KEY = 227,         /* KW_PARTITION_KEY  */
  YYSYMBOL_KW_TMPL_ESCAPE = 228,           /* KW_TMPL_ESCAPE  */
  YYSYMBOL_KW_LOG_FETCH_DELAY = 229,       /* KW_LOG_FETCH_DELAY  */
  YYSYMBOL_KW_LOG_FETCH_RETRY_DELAY = 230, /* KW_LOG_FETCH_RETRY_DELAY  */
  YYSYMBOL_KW_OPTIONAL = 231,              /* KW_OPTIONAL  */
  YYSYMBOL_KW_CREATE_DIRS = 232,           /* KW_CREATE_DIRS  */
  YYSYMBOL_KW_OWNER = 233,                 /* KW_OWNER  */
  YYSYMBOL_KW_GROUP = 234,                 /* KW_GROUP  */
  YYSYMBOL_KW_PERM = 235,                  /* KW_PERM  */
  YYSYMBOL_KW_DIR_OWNER = 236,             /* KW_DIR_OWNER  */
  YYSYMBOL_KW_DIR_GROUP = 237,             /* KW_DIR_GROUP  */
  YYSYMBOL_KW_DIR_PERM = 238,              /* KW_DIR_PERM  */
  YYSYMBOL_KW_TEMPLATE = 239,              /* KW_TEMPLATE  */
  YYSYMBOL_KW_TEMPLATE_ESCAPE = 240,       /* KW_TEMPLATE_ESCAPE  */
  YYSYMBOL_KW_TEMPLATE_FUNCTION = 241,     /* KW_TEMPLATE_FUNCTION  */
  YYSYMBOL_KW_DEFAULT_FACILITY = 242,      /* KW_DEFAULT_FACILITY  */
  YYSYMBOL_KW_DEFAULT_SEVERITY = 243,      /* KW_DEFAULT_SEVERITY  */
  YYSYMBOL_KW_SDATA_PREFIX = 244,          /* KW_SDATA_PREFIX  */
  YYSYMBOL_KW_PORT = 245,                  /* KW_PORT  */
  YYSYMBOL_KW_USE_TIME_RECVD = 246,        /* KW_USE_TIME_RECVD  */
  YYSYMBOL_KW_FACILITY = 247,              /* KW_FACILITY  */
  YYSYMBOL_KW_SEVERITY = 248,              /* KW_SEVERITY  */
  YYSYMBOL_KW_HOST = 249,                  /* KW_HOST  */
  YYSYMBOL_KW_MATCH = 250,                 /* KW_MATCH  */
  YYSYMBOL_KW_MESSAGE = 251,               /* KW_MESSAGE  */
  YYSYMBOL_KW_NETMASK = 252,               /* KW_NETMASK  */
  YYSYMBOL_KW_TAGS = 253,                  /* KW_TAGS  */
  YYSYMBOL_KW_NETMASK6 = 254,              /* KW_NETMASK6  */
  YYSYMBOL_KW_REWRITE = 255,               /* KW_REWRITE  */
  YYSYMBOL_KW_CONDITION = 256,             /* KW_CONDITION  */
  YYSYMBOL_KW_VALUE = 257,                 /* KW_VALUE  */
  YYSYMBOL_KW_YES = 258,                   /* KW_YES  */
  YYSYMBOL_KW_NO = 259,                    /* KW_NO  */
  YYSYMBOL_KW_AUTO = 260,                  /* KW_AUTO  */
  YYSYMBOL_KW_IFDEF = 261,                 /* KW_IFDEF  */
  YYSYMBOL_KW_ENDIF = 262,                 /* KW_ENDIF  */
  YYSYMBOL_LL_DOTDOT = 263,                /* LL_DOTDOT  */
  YYSYMBOL_LL_DOTDOTDOT = 264,             /* LL_DOTDOTDOT  */
  YYSYMBOL_LL_PRAGMA = 265,                /* LL_PRAGMA  */
  YYSYMBOL_LL_EOL = 266,                   /* LL_EOL  */
  YYSYMBOL_LL_ERROR = 267,                 /* LL_ERROR  */
  YYSYMBOL_LL_ARROW = 268,                 /* LL_ARROW  */
  YYSYMBOL_LL_IDENTIFIER = 269,            /* LL_IDENTIFIER  */
  YYSYMBOL_LL_NUMBER = 270,                /* LL_NUMBER  */
  YYSYMBOL_LL_FLOAT = 271,                 /* LL_FLOAT  */
  YYSYMBOL_LL_STRING = 272,                /* LL_STRING  */
  YYSYMBOL_LL_TOKEN = 273,                 /* LL_TOKEN  */
  YYSYMBOL_LL_BLOCK = 274,                 /* LL_BLOCK  */
  YYSYMBOL_LL_PLUGIN = 275,                /* LL_PLUGIN  */
  YYSYMBOL_LL_TEMPLATE_REF = 276,          /* LL_TEMPLATE_REF  */
  YYSYMBOL_LL_MESSAGE_REF = 277,           /* LL_MESSAGE_REF  */
  YYSYMBOL_KW_VALUE_PAIRS = 278,           /* KW_VALUE_PAIRS  */
  YYSYMBOL_KW_EXCLUDE = 279,               /* KW_EXCLUDE  */
  YYSYMBOL_KW_PAIR = 280,                  /* KW_PAIR  */
  YYSYMBOL_KW_KEY = 281,                   /* KW_KEY  */
  YYSYMBOL_KW_SCOPE = 282,                 /* KW_SCOPE  */
  YYSYMBOL_KW_SHIFT = 283,                 /* KW_SHIFT  */
  YYSYMBOL_KW_SHIFT_LEVELS = 284,          /* KW_SHIFT_LEVELS  */
  YYSYMBOL_KW_REKEY = 285,                 /* KW_REKEY  */
  YYSYMBOL_KW_ADD_PREFIX = 286,            /* KW_ADD_PREFIX  */
  YYSYMBOL_KW_REPLACE_PREFIX = 287,        /* KW_REPLACE_PREFIX  */
  YYSYMBOL_KW_CAST = 288,                  /* KW_CAST  */
  YYSYMBOL_KW_UPPER = 289,                 /* KW_UPPER  */
  YYSYMBOL_KW_LOWER = 290,                 /* KW_LOWER  */
  YYSYMBOL_KW_INCLUDE_BYTES = 291,         /* KW_INCLUDE_BYTES  */
  YYSYMBOL_KW_ON_ERROR = 292,              /* KW_ON_ERROR  */
  YYSYMBOL_KW_RETRIES = 293,               /* KW_RETRIES  */
  YYSYMBOL_KW_FETCH_NO_DATA_DELAY = 294,   /* KW_FETCH_NO_DATA_DELAY  */
  YYSYMBOL_KW_LABELS = 295,                /* KW_LABELS  */
  YYSYMBOL_296_ = 296,                     /* '('  */
  YYSYMBOL_297_ = 297,                     /* ')'  */
  YYSYMBOL_298_ = 298,                     /* '{'  */
  YYSYMBOL_299_ = 299,                     /* '}'  */
  YYSYMBOL_YYACCEPT = 300,                 /* $accept  */
  YYSYMBOL_start = 301,                    /* start  */
  YYSYMBOL_driver = 302,                   /* driver  */
  YYSYMBOL_source_afunix = 303,            /* source_afunix  */
  YYSYMBOL_source_afunix_dgram_params = 304, /* source_afunix_dgram_params  */
  YYSYMBOL_305_1 = 305,                    /* $@1  */
  YYSYMBOL_source_afunix_stream_params = 306, /* source_afunix_stream_params  */
  YYSYMBOL_307_2 = 307,                    /* $@2  */
  YYSYMBOL_source_afunix_options = 308,    /* source_afunix_options  */
  YYSYMBOL_source_afunix_option = 309,     /* source_afunix_option  */
  YYSYMBOL_source_afinet = 310,            /* source_afinet  */
  YYSYMBOL_source_afinet_udp_params = 311, /* source_afinet_udp_params  */
  YYSYMBOL_312_3 = 312,                    /* $@3  */
  YYSYMBOL_source_afinet_udp6_params = 313, /* source_afinet_udp6_params  */
  YYSYMBOL_314_4 = 314,                    /* $@4  */
  YYSYMBOL_source_afinet_udp_options = 315, /* source_afinet_udp_options  */
  YYSYMBOL_source_afinet_udp_option = 316, /* source_afinet_udp_option  */
  YYSYMBOL_source_afinet_multi_line_option = 317, /* source_afinet_multi_line_option  */
  YYSYMBOL_source_afinet_option = 318,     /* source_afinet_option  */
  YYSYMBOL_source_afinet_tcp_params = 319, /* source_afinet_tcp_params  */
  YYSYMBOL_320_5 = 320,                    /* $@5  */
  YYSYMBOL_source_afinet_tcp6_params = 321, /* source_afinet_tcp6_params  */
  YYSYMBOL_322_6 = 322,                    /* $@6  */
  YYSYMBOL_source_afinet_tcp_options = 323, /* source_afinet_tcp_options  */
  YYSYMBOL_source_afinet_tcp_option = 324, /* source_afinet_tcp_option  */
  YYSYMBOL_325_7 = 325,                    /* $@7  */
  YYSYMBOL_source_afsocket_stream_params = 326, /* source_afsocket_stream_params  */
  YYSYMBOL_source_afsyslog = 327,          /* source_afsyslog  */
  YYSYMBOL_source_afsyslog_params = 328,   /* source_afsyslog_params  */
  YYSYMBOL_329_8 = 329,                    /* $@8  */
  YYSYMBOL_source_afsyslog_options = 330,  /* source_afsyslog_options  */
  YYSYMBOL_source_afsyslog_option = 331,   /* source_afsyslog_option  */
  YYSYMBOL_source_afnetwork = 332,         /* source_afnetwork  */
  YYSYMBOL_source_afnetwork_params = 333,  /* source_afnetwork_params  */
  YYSYMBOL_334_9 = 334,                    /* $@9  */
  YYSYMBOL_source_afnetwork_options = 335, /* source_afnetwork_options  */
  YYSYMBOL_source_afnetwork_option = 336,  /* source_afnetwork_option  */
  YYSYMBOL_stats_option = 337,             /* stats_option  */
  YYSYMBOL_stats_options = 338,            /* stats_options  */
  YYSYMBOL_proto_http_scraper_option = 339, /* proto_http_scraper_option  */
  YYSYMBOL_proto_http_scraper_options = 340, /* proto_http_scraper_options  */
  YYSYMBOL_proto_http_scraper_transport = 341, /* proto_http_scraper_transport  */
  YYSYMBOL_342_10 = 342,                   /* $@10  */
  YYSYMBOL_source_afsocket_transport = 343, /* source_afsocket_transport  */
  YYSYMBOL_344_11 = 344,                   /* $@11  */
  YYSYMBOL_345_12 = 345,                   /* $@12  */
  YYSYMBOL_source_systemd_syslog = 346,    /* source_systemd_syslog  */
  YYSYMBOL_source_systemd_syslog_params = 347, /* source_systemd_syslog_params  */
  YYSYMBOL_348_13 = 348,                   /* $@13  */
  YYSYMBOL_source_systemd_syslog_options = 349, /* source_systemd_syslog_options  */
  YYSYMBOL_source_systemd_syslog_option = 350, /* source_systemd_syslog_option  */
  YYSYMBOL_dest_afunix = 351,              /* dest_afunix  */
  YYSYMBOL_dest_afunix_dgram_params = 352, /* dest_afunix_dgram_params  */
  YYSYMBOL_353_14 = 353,                   /* $@14  */
  YYSYMBOL_dest_afunix_stream_params = 354, /* dest_afunix_stream_params  */
  YYSYMBOL_355_15 = 355,                   /* $@15  */
  YYSYMBOL_dest_afunix_options = 356,      /* dest_afunix_options  */
  YYSYMBOL_dest_afunix_option = 357,       /* dest_afunix_option  */
  YYSYMBOL_dest_afinet = 358,              /* dest_afinet  */
  YYSYMBOL_dest_afinet_udp_params = 359,   /* dest_afinet_udp_params  */
  YYSYMBOL_360_16 = 360,                   /* $@16  */
  YYSYMBOL_dest_afinet_udp6_params = 361,  /* dest_afinet_udp6_params  */
  YYSYMBOL_362_17 = 362,                   /* $@17  */
  YYSYMBOL_dest_afinet_udp_options = 363,  /* dest_afinet_udp_options  */
  YYSYMBOL_dest_afinet_option = 364,       /* dest_afinet_option  */
  YYSYMBOL_365_18 = 365,                   /* $@18  */
  YYSYMBOL_366_19 = 366,                   /* $@19  */
  YYSYMBOL_dest_failover_options = 367,    /* dest_failover_options  */
  YYSYMBOL_368_20 = 368,                   /* $@20  */
  YYSYMBOL_dest_failover_modes_options = 369, /* dest_failover_modes_options  */
  YYSYMBOL_dest_failback_options = 370,    /* dest_failback_options  */
  YYSYMBOL_371_21 = 371,                   /* $@21  */
  YYSYMBOL_dest_failback_probe_options = 372, /* dest_failback_probe_options  */
  YYSYMBOL_dest_failback_probe_option = 373, /* dest_failback_probe_option  */
  YYSYMBOL_dest_afinet_dgram_option = 374, /* dest_afinet_dgram_option  */
  YYSYMBOL_dest_afinet_udp_option = 375,   /* dest_afinet_udp_option  */
  YYSYMBOL_dest_afinet_tcp_params = 376,   /* dest_afinet_tcp_params  */
  YYSYMBOL_377_22 = 377,                   /* $@22  */
  YYSYMBOL_dest_afinet_tcp6_params = 378,  /* dest_afinet_tcp6_params  */
  YYSYMBOL_379_23 = 379,                   /* $@23  */
  YYSYMBOL_dest_afinet_tcp_options = 380,  /* dest_afinet_tcp_options  */
  YYSYMBOL_dest_afinet_tcp_option = 381,   /* dest_afinet_tcp_option  */
  YYSYMBOL_382_24 = 382,                   /* $@24  */
  YYSYMBOL_dest_afsocket_option = 383,     /* dest_afsocket_option  */
  YYSYMBOL_dest_afsyslog = 384,            /* dest_afsyslog  */
  YYSYMBOL_dest_afsyslog_params = 385,     /* dest_afsyslog_params  */
  YYSYMBOL_386_25 = 386,                   /* $@25  */
  YYSYMBOL_dest_afsyslog_options = 387,    /* dest_afsyslog_options  */
  YYSYMBOL_dest_afsyslog_option = 388,     /* dest_afsyslog_option  */
  YYSYMBOL_dest_afnetwork = 389,           /* dest_afnetwork  */
  YYSYMBOL_dest_afnetwork_params = 390,    /* dest_afnetwork_params  */
  YYSYMBOL_391_26 = 391,                   /* $@26  */
  YYSYMBOL_dest_afnetwork_options = 392,   /* dest_afnetwork_options  */
  YYSYMBOL_dest_afnetwork_option = 393,    /* dest_afnetwork_option  */
  YYSYMBOL_dest_afsocket_transport = 394,  /* dest_afsocket_transport  */
  YYSYMBOL_395_27 = 395,                   /* $@27  */
  YYSYMBOL_396_28 = 396,                   /* $@28  */
  YYSYMBOL_afsocket_transport = 397,       /* afsocket_transport  */
  YYSYMBOL_dest_tls_options = 398,         /* dest_tls_options  */
  YYSYMBOL_dest_tls_option = 399,          /* dest_tls_option  */
  YYSYMBOL_tls_options = 400,              /* tls_options  */
  YYSYMBOL_tls_option = 401,               /* tls_option  */
  YYSYMBOL_tls_conf_cmds = 402,            /* tls_conf_cmds  */
  YYSYMBOL_tls_conf_cmd_list_build = 403,  /* tls_conf_cmd_list_build  */
  YYSYMBOL_tls_cipher_suites = 404,        /* tls_cipher_suites  */
  YYSYMBOL_tls_cipher_suite = 405,         /* tls_cipher_suite  */
  YYSYMBOL_socket_option = 406,            /* socket_option  */
  YYSYMBOL_unix_socket_option = 407,       /* unix_socket_option  */
  YYSYMBOL_inet_socket_option = 408,       /* inet_socket_option  */
  YYSYMBOL_inet_ip_protocol_option = 409,  /* inet_ip_protocol_option  */
  YYSYMBOL_template_content_inner = 410,   /* template_content_inner  */
  YYSYMBOL_type_hint = 411,                /* type_hint  */
  YYSYMBOL_template_content = 412,         /* template_content  */
  YYSYMBOL_413_31 = 413,                   /* @31  */
  YYSYMBOL_template_name_or_content = 414, /* template_name_or_content  */
  YYSYMBOL_template_name_or_content_tail = 415, /* template_name_or_content_tail  */
  YYSYMBOL_string = 416,                   /* string  */
  YYSYMBOL_yesno_strict = 417,             /* yesno_strict  */
  YYSYMBOL_yesno = 418,                    /* yesno  */
  YYSYMBOL_dnsmode = 419,                  /* dnsmode  */
  YYSYMBOL_nonnegative_integer64 = 420,    /* nonnegative_integer64  */
  YYSYMBOL_nonnegative_integer = 421,      /* nonnegative_integer  */
  YYSYMBOL_positive_integer64 = 422,       /* positive_integer64  */
  YYSYMBOL_positive_integer = 423,         /* positive_integer  */
  YYSYMBOL_nonnegative_float = 424,        /* nonnegative_float  */
  YYSYMBOL_string_or_number = 425,         /* string_or_number  */
  YYSYMBOL_path = 426,                     /* path  */
  YYSYMBOL_path_check = 427,               /* path_check  */
  YYSYMBOL_path_secret = 428,              /* path_secret  */
  YYSYMBOL_string_list = 429,              /* string_list  */
  YYSYMBOL_string_list_build = 430,        /* string_list_build  */
  YYSYMBOL_severity_string = 431,          /* severity_string  */
  YYSYMBOL_facility_string = 432,          /* facility_string  */
  YYSYMBOL_driver_option = 433,            /* driver_option  */
  YYSYMBOL_inner_source = 434,             /* inner_source  */
  YYSYMBOL_source_driver_option = 435,     /* source_driver_option  */
  YYSYMBOL_inner_dest = 436,               /* inner_dest  */
  YYSYMBOL_dest_driver_option = 437,       /* dest_driver_option  */
  YYSYMBOL_source_option = 438,            /* source_option  */
  YYSYMBOL_439_34 = 439,                   /* $@34  */
  YYSYMBOL_source_reader_option = 440,     /* source_reader_option  */
  YYSYMBOL_441_35 = 441,                   /* $@35  */
  YYSYMBOL_442_36 = 442,                   /* $@36  */
  YYSYMBOL_443_37 = 443,                   /* $@37  */
  YYSYMBOL_source_reader_option_flags = 444, /* source_reader_option_flags  */
  YYSYMBOL_source_proto_option = 445,      /* source_proto_option  */
  YYSYMBOL_host_resolve_option = 446,      /* host_resolve_option  */
  YYSYMBOL_msg_format_option = 447,        /* msg_format_option  */
  YYSYMBOL_dest_writer_option = 448,       /* dest_writer_option  */
  YYSYMBOL_449_38 = 449,                   /* $@38  */
  YYSYMBOL_dest_writer_options_flags = 450, /* dest_writer_options_flags  */
  YYSYMBOL_file_perm_option = 451,         /* file_perm_option  */
  YYSYMBOL_template_option = 452,          /* template_option  */
  YYSYMBOL__template_ref_context_push = 453, /* _template_ref_context_push  */
  YYSYMBOL__inner_dest_context_push = 454, /* _inner_dest_context_push  */
  YYSYMBOL__inner_dest_context_pop = 455,  /* _inner_dest_context_pop  */
  YYSYMBOL__inner_src_context_push = 456,  /* _inner_src_context_push  */
  YYSYMBOL__inner_src_context_pop = 457    /* _inner_src_context_pop  */
};
typedef enum yysymbol_kind_t yysymbol_kind_t;



/* Unqualified %code blocks.  */
#line 30 "modules/afsocket/afsocket-grammar.y"


#include "afsocket.h"
#include "cfg-parser.h"
#include "afunix-source.h"
#include "afunix-dest.h"
#include "afinet-source.h"
#include "afinet-dest.h"
#include "messages.h"
#include "syslog-names.h"
#include "plugin.h"
#include "cfg-grammar-internal.h"
#include "cfg-helpers.h"
#include "socket-options-inet.h"
#include "socket-options-unix.h"
#include "transport-mapper-inet.h"
#include "service-management.h"
#include "string-list.h"
#include "multi-line/multi-line-factory.h"
#include "logproto/logproto-http-scraper-responder-server.h"

#include "systemd-syslog-source.h"
#include "afsocket-systemd-override.h"

#include "transport/tls-context.h"


static SocketOptions *last_sock_options;
static TransportMapper *last_transport_mapper;

TLSContext *last_tls_context;
MultiLineOptions *last_multi_line_options;


#if ! SYSLOG_NG_ENABLE_IPV6
#undef AF_INET6
#define AF_INET6 0; g_assert_not_reached()

#endif

static void
afsocket_grammar_set_source_driver(AFSocketSourceDriver *sd)
{
  last_driver = &sd->super.super;

  last_reader_options = &((AFSocketSourceDriver *) last_driver)->reader_options;
  last_proto_server_options = &last_reader_options->proto_options;
  last_multi_line_options = &last_proto_server_options->super.multi_line_options;
  last_sock_options = ((AFSocketSourceDriver *) last_driver)->socket_options;
  last_transport_mapper = ((AFSocketSourceDriver *) last_driver)->transport_mapper;
}

static void
afsocket_grammar_set_dest_driver(AFSocketDestDriver *dd)
{
  last_driver = &dd->super.super;

  last_writer_options = &((AFSocketDestDriver *) last_driver)->writer_options;
  last_sock_options = ((AFSocketDestDriver *) last_driver)->socket_options;
  last_transport_mapper = ((AFSocketDestDriver *) last_driver)->transport_mapper;
  last_proto_client_options = &last_writer_options->proto_options;
}

void
afunix_grammar_set_source_driver(AFUnixSourceDriver *sd)
{
  afsocket_grammar_set_source_driver(&sd->super);
  last_file_perm_options = &sd->file_perm_options;
}

static void
afinet_grammar_set_source_driver(AFInetSourceDriver *sd)
{
  afsocket_grammar_set_source_driver(&sd->super);
}

static void
afunix_grammar_set_dest_driver(AFUnixDestDriver *dd)
{
  afsocket_grammar_set_dest_driver(&dd->super);
}

static void
afinet_grammar_set_dest_driver(AFInetDestDriver *dd)
{
  afsocket_grammar_set_dest_driver(&dd->super);
}

void
systemd_syslog_grammar_set_source_driver(SystemDSyslogSourceDriver *sd)
{
  afsocket_grammar_set_source_driver(&sd->super);
}

#line 229 "modules/afsocket/afsocket-grammar.y"


#pragma GCC diagnostic ignored "-Wswitch-default"
#pragma GCC diagnostic ignored "-Wunused-but-set-variable"

#if (defined(__GNUC__) && __GNUC__ >= 6) || (defined(__clang__) && __clang_major__ >= 10)
#  pragma GCC diagnostic ignored "-Wmisleading-indentation"
#endif


# define YYLLOC_DEFAULT(Current, Rhs, N)                                \
  do {                                                                  \
    if (N)                                                              \
      {                                                                 \
        (Current).name         = YYRHSLOC(Rhs, 1).name;                 \
        (Current).first_line   = YYRHSLOC (Rhs, 1).first_line;          \
        (Current).first_column = YYRHSLOC (Rhs, 1).first_column;        \
        (Current).last_line    = YYRHSLOC (Rhs, N).last_line;           \
        (Current).last_column  = YYRHSLOC (Rhs, N).last_column;         \
      }                                                                 \
    else                                                                \
      {                                                                 \
        (Current).name         = YYRHSLOC(Rhs, 0).name;                 \
        (Current).first_line   = (Current).last_line   =                \
          YYRHSLOC (Rhs, 0).last_line;                                  \
        (Current).first_column = (Current).last_column =                \
          YYRHSLOC (Rhs, 0).last_column;                                \
      }                                                                 \
  } while (0)

#define CHECK_ERROR_WITHOUT_MESSAGE(val, token) do {                    \
    if (!(val))                                                         \
      {                                                                 \
        YYERROR;                                                        \
      }                                                                 \
  } while (0)

#define CHECK_ERROR(val, token, errorfmt, ...) do {                     \
    if (!(val))                                                         \
      {                                                                 \
        if (errorfmt)                                                   \
          {                                                             \
            gchar __buf[256];                                           \
            g_snprintf(__buf, sizeof(__buf), errorfmt, ## __VA_ARGS__); \
            yyerror(& (token), lexer, NULL, NULL, __buf);               \
          }                                                             \
        YYERROR;                                                        \
      }                                                                 \
  } while (0)

#define CHECK_ERROR_GERROR(val, token, error, errorfmt, ...) do {       \
    if (!(val))                                                         \
      {                                                                 \
        if (errorfmt)                                                   \
          {                                                             \
            gchar __buf[256];                                           \
            g_snprintf(__buf, sizeof(__buf), errorfmt ", error=%s", ## __VA_ARGS__, error->message); \
            yyerror(& (token), lexer, NULL, NULL, __buf);               \
          }                                                             \
        g_clear_error(&error);						\
        YYERROR;                                                        \
      }                                                                 \
  } while (0)

#define YYMAXDEPTH 20000



#line 1361 "modules/afsocket/afsocket-grammar.c"

#ifdef short
# undef short
#endif

/* On compilers that do not define __PTRDIFF_MAX__ etc., make sure
   <limits.h> and (if available) <stdint.h> are included
   so that the code can choose integer types of a good width.  */

#ifndef __PTRDIFF_MAX__
# include <limits.h> /* INFRINGES ON USER NAME SPACE */
# if defined __STDC_VERSION__ && 199901 <= __STDC_VERSION__
#  include <stdint.h> /* INFRINGES ON USER NAME SPACE */
#  define YY_STDINT_H
# endif
#endif

/* Narrow types that promote to a signed type and that can represent a
   signed or unsigned integer of at least N bits.  In tables they can
   save space and decrease cache pressure.  Promoting to a signed type
   helps avoid bugs in integer arithmetic.  */

#ifdef __INT_LEAST8_MAX__
typedef __INT_LEAST8_TYPE__ yytype_int8;
#elif defined YY_STDINT_H
typedef int_least8_t yytype_int8;
#else
typedef signed char yytype_int8;
#endif

#ifdef __INT_LEAST16_MAX__
typedef __INT_LEAST16_TYPE__ yytype_int16;
#elif defined YY_STDINT_H
typedef int_least16_t yytype_int16;
#else
typedef short yytype_int16;
#endif

/* Work around bug in HP-UX 11.23, which defines these macros
   incorrectly for preprocessor constants.  This workaround can likely
   be removed in 2023, as HPE has promised support for HP-UX 11.23
   (aka HP-UX 11i v2) only through the end of 2022; see Table 2 of
   <https://h20195.www2.hpe.com/V2/getpdf.aspx/4AA4-7673ENW.pdf>.  */
#ifdef __hpux
# undef UINT_LEAST8_MAX
# undef UINT_LEAST16_MAX
# define UINT_LEAST8_MAX 255
# define UINT_LEAST16_MAX 65535
#endif

#if defined __UINT_LEAST8_MAX__ && __UINT_LEAST8_MAX__ <= __INT_MAX__
typedef __UINT_LEAST8_TYPE__ yytype_uint8;
#elif (!defined __UINT_LEAST8_MAX__ && defined YY_STDINT_H \
       && UINT_LEAST8_MAX <= INT_MAX)
typedef uint_least8_t yytype_uint8;
#elif !defined __UINT_LEAST8_MAX__ && UCHAR_MAX <= INT_MAX
typedef unsigned char yytype_uint8;
#else
typedef short yytype_uint8;
#endif

#if defined __UINT_LEAST16_MAX__ && __UINT_LEAST16_MAX__ <= __INT_MAX__
typedef __UINT_LEAST16_TYPE__ yytype_uint16;
#elif (!defined __UINT_LEAST16_MAX__ && defined YY_STDINT_H \
       && UINT_LEAST16_MAX <= INT_MAX)
typedef uint_least16_t yytype_uint16;
#elif !defined __UINT_LEAST16_MAX__ && USHRT_MAX <= INT_MAX
typedef unsigned short yytype_uint16;
#else
typedef int yytype_uint16;
#endif

#ifndef YYPTRDIFF_T
# if defined __PTRDIFF_TYPE__ && defined __PTRDIFF_MAX__
#  define YYPTRDIFF_T __PTRDIFF_TYPE__
#  define YYPTRDIFF_MAXIMUM __PTRDIFF_MAX__
# elif defined PTRDIFF_MAX
#  ifndef ptrdiff_t
#   include <stddef.h> /* INFRINGES ON USER NAME SPACE */
#  endif
#  define YYPTRDIFF_T ptrdiff_t
#  define YYPTRDIFF_MAXIMUM PTRDIFF_MAX
# else
#  define YYPTRDIFF_T long
#  define YYPTRDIFF_MAXIMUM LONG_MAX
# endif
#endif

#ifndef YYSIZE_T
# ifdef __SIZE_TYPE__
#  define YYSIZE_T __SIZE_TYPE__
# elif defined size_t
#  define YYSIZE_T size_t
# elif defined __STDC_VERSION__ && 199901 <= __STDC_VERSION__
#  include <stddef.h> /* INFRINGES ON USER NAME SPACE */
#  define YYSIZE_T size_t
# else
#  define YYSIZE_T unsigned
# endif
#endif

#define YYSIZE_MAXIMUM                                  \
  YY_CAST (YYPTRDIFF_T,                                 \
           (YYPTRDIFF_MAXIMUM < YY_CAST (YYSIZE_T, -1)  \
            ? YYPTRDIFF_MAXIMUM                         \
            : YY_CAST (YYSIZE_T, -1)))

#define YYSIZEOF(X) YY_CAST (YYPTRDIFF_T, sizeof (X))


/* Stored state numbers (used for stacks). */
typedef yytype_int16 yy_state_t;

/* State numbers in computations.  */
typedef int yy_state_fast_t;

#ifndef YY_
# if defined YYENABLE_NLS && YYENABLE_NLS
#  if ENABLE_NLS
#   include <libintl.h> /* INFRINGES ON USER NAME SPACE */
#   define YY_(Msgid) dgettext ("bison-runtime", Msgid)
#  endif
# endif
# ifndef YY_
#  define YY_(Msgid) Msgid
# endif
#endif


#ifndef YY_ATTRIBUTE_PURE
# if defined __GNUC__ && 2 < __GNUC__ + (96 <= __GNUC_MINOR__)
#  define YY_ATTRIBUTE_PURE __attribute__ ((__pure__))
# else
#  define YY_ATTRIBUTE_PURE
# endif
#endif

#ifndef YY_ATTRIBUTE_UNUSED
# if defined __GNUC__ && 2 < __GNUC__ + (7 <= __GNUC_MINOR__)
#  define YY_ATTRIBUTE_UNUSED __attribute__ ((__unused__))
# else
#  define YY_ATTRIBUTE_UNUSED
# endif
#endif

/* Suppress unused-variable warnings by "using" E.  */
#if ! defined lint || defined __GNUC__
# define YY_USE(E) ((void) (E))
#else
# define YY_USE(E) /* empty */
#endif

/* Suppress an incorrect diagnostic about yylval being uninitialized.  */
#if defined __GNUC__ && ! defined __ICC && 406 <= __GNUC__ * 100 + __GNUC_MINOR__
# if __GNUC__ * 100 + __GNUC_MINOR__ < 407
#  define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN                           \
    _Pragma ("GCC diagnostic push")                                     \
    _Pragma ("GCC diagnostic ignored \"-Wuninitialized\"")
# else
#  define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN                           \
    _Pragma ("GCC diagnostic push")                                     \
    _Pragma ("GCC diagnostic ignored \"-Wuninitialized\"")              \
    _Pragma ("GCC diagnostic ignored \"-Wmaybe-uninitialized\"")
# endif
# define YY_IGNORE_MAYBE_UNINITIALIZED_END      \
    _Pragma ("GCC diagnostic pop")
#else
# define YY_INITIAL_VALUE(Value) Value
#endif
#ifndef YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
# define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
# define YY_IGNORE_MAYBE_UNINITIALIZED_END
#endif
#ifndef YY_INITIAL_VALUE
# define YY_INITIAL_VALUE(Value) /* Nothing. */
#endif

#if defined __cplusplus && defined __GNUC__ && ! defined __ICC && 6 <= __GNUC__
# define YY_IGNORE_USELESS_CAST_BEGIN                          \
    _Pragma ("GCC diagnostic push")                            \
    _Pragma ("GCC diagnostic ignored \"-Wuseless-cast\"")
# define YY_IGNORE_USELESS_CAST_END            \
    _Pragma ("GCC diagnostic pop")
#endif
#ifndef YY_IGNORE_USELESS_CAST_BEGIN
# define YY_IGNORE_USELESS_CAST_BEGIN
# define YY_IGNORE_USELESS_CAST_END
#endif


#define YY_ASSERT(E) ((void) (0 && (E)))

#if 1

/* The parser invokes alloca or malloc; define the necessary symbols.  */

# ifdef YYSTACK_USE_ALLOCA
#  if YYSTACK_USE_ALLOCA
#   ifdef __GNUC__
#    define YYSTACK_ALLOC __builtin_alloca
#   elif defined __BUILTIN_VA_ARG_INCR
#    include <alloca.h> /* INFRINGES ON USER NAME SPACE */
#   elif defined _AIX
#    define YYSTACK_ALLOC __alloca
#   elif defined _MSC_VER
#    include <malloc.h> /* INFRINGES ON USER NAME SPACE */
#    define alloca _alloca
#   else
#    define YYSTACK_ALLOC alloca
#    if ! defined _ALLOCA_H && ! defined EXIT_SUCCESS
#     include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
      /* Use EXIT_SUCCESS as a witness for stdlib.h.  */
#     ifndef EXIT_SUCCESS
#      define EXIT_SUCCESS 0
#     endif
#    endif
#   endif
#  endif
# endif

# ifdef YYSTACK_ALLOC
   /* Pacify GCC's 'empty if-body' warning.  */
#  define YYSTACK_FREE(Ptr) do { /* empty */; } while (0)
#  ifndef YYSTACK_ALLOC_MAXIMUM
    /* The OS might guarantee only one guard page at the bottom of the stack,
       and a page size can be as small as 4096 bytes.  So we cannot safely
       invoke alloca (N) if N exceeds 4096.  Use a slightly smaller number
       to allow for a few compiler-allocated temporary stack slots.  */
#   define YYSTACK_ALLOC_MAXIMUM 4032 /* reasonable circa 2006 */
#  endif
# else
#  define YYSTACK_ALLOC YYMALLOC
#  define YYSTACK_FREE YYFREE
#  ifndef YYSTACK_ALLOC_MAXIMUM
#   define YYSTACK_ALLOC_MAXIMUM YYSIZE_MAXIMUM
#  endif
#  if (defined __cplusplus && ! defined EXIT_SUCCESS \
       && ! ((defined YYMALLOC || defined malloc) \
             && (defined YYFREE || defined free)))
#   include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
#   ifndef EXIT_SUCCESS
#    define EXIT_SUCCESS 0
#   endif
#  endif
#  ifndef YYMALLOC
#   define YYMALLOC malloc
#   if ! defined malloc && ! defined EXIT_SUCCESS
void *malloc (YYSIZE_T); /* INFRINGES ON USER NAME SPACE */
#   endif
#  endif
#  ifndef YYFREE
#   define YYFREE free
#   if ! defined free && ! defined EXIT_SUCCESS
void free (void *); /* INFRINGES ON USER NAME SPACE */
#   endif
#  endif
# endif
#endif /* 1 */

#if (! defined yyoverflow \
     && (! defined __cplusplus \
         || (defined AFSOCKET_LTYPE_IS_TRIVIAL && AFSOCKET_LTYPE_IS_TRIVIAL \
             && defined AFSOCKET_STYPE_IS_TRIVIAL && AFSOCKET_STYPE_IS_TRIVIAL)))

/* A type that is properly aligned for any stack member.  */
union yyalloc
{
  yy_state_t yyss_alloc;
  YYSTYPE yyvs_alloc;
  YYLTYPE yyls_alloc;
};

/* The size of the maximum gap between one aligned stack and the next.  */
# define YYSTACK_GAP_MAXIMUM (YYSIZEOF (union yyalloc) - 1)

/* The size of an array large to enough to hold all stacks, each with
   N elements.  */
# define YYSTACK_BYTES(N) \
     ((N) * (YYSIZEOF (yy_state_t) + YYSIZEOF (YYSTYPE) \
             + YYSIZEOF (YYLTYPE)) \
      + 2 * YYSTACK_GAP_MAXIMUM)

# define YYCOPY_NEEDED 1

/* Relocate STACK from its old location to the new one.  The
   local variables YYSIZE and YYSTACKSIZE give the old and new number of
   elements in the stack, and YYPTR gives the new location of the
   stack.  Advance YYPTR to a properly aligned location for the next
   stack.  */
# define YYSTACK_RELOCATE(Stack_alloc, Stack)                           \
    do                                                                  \
      {                                                                 \
        YYPTRDIFF_T yynewbytes;                                         \
        YYCOPY (&yyptr->Stack_alloc, Stack, yysize);                    \
        Stack = &yyptr->Stack_alloc;                                    \
        yynewbytes = yystacksize * YYSIZEOF (*Stack) + YYSTACK_GAP_MAXIMUM; \
        yyptr += yynewbytes / YYSIZEOF (*yyptr);                        \
      }                                                                 \
    while (0)

#endif

#if defined YYCOPY_NEEDED && YYCOPY_NEEDED
/* Copy COUNT objects from SRC to DST.  The source and destination do
   not overlap.  */
# ifndef YYCOPY
#  if defined __GNUC__ && 1 < __GNUC__
#   define YYCOPY(Dst, Src, Count) \
      __builtin_memcpy (Dst, Src, YY_CAST (YYSIZE_T, (Count)) * sizeof (*(Src)))
#  else
#   define YYCOPY(Dst, Src, Count)              \
      do                                        \
        {                                       \
          YYPTRDIFF_T yyi;                      \
          for (yyi = 0; yyi < (Count); yyi++)   \
            (Dst)[yyi] = (Src)[yyi];            \
        }                                       \
      while (0)
#  endif
# endif
#endif /* !YYCOPY_NEEDED */

/* YYFINAL -- State number of the termination state.  */
#define YYFINAL  31
/* YYLAST -- Last index in YYTABLE.  */
#define YYLAST   1914

/* YYNTOKENS -- Number of terminals.  */
#define YYNTOKENS  300
/* YYNNTS -- Number of nonterminals.  */
#define YYNNTS  158
/* YYNRULES -- Number of rules.  */
#define YYNRULES  369
/* YYNSTATES -- Number of states.  */
#define YYNSTATES  881

/* YYMAXUTOK -- Last valid token kind.  */
#define YYMAXUTOK   20076


/* YYTRANSLATE(TOKEN-NUM) -- Symbol number corresponding to TOKEN-NUM
   as returned by yylex, with out-of-bounds checking.  */
#define YYTRANSLATE(YYX)                                \
  (0 <= (YYX) && (YYX) <= YYMAXUTOK                     \
   ? YY_CAST (yysymbol_kind_t, yytranslate[YYX])        \
   : YYSYMBOL_YYUNDEF)

/* YYTRANSLATE[TOKEN-NUM] -- Symbol number corresponding to TOKEN-NUM
   as returned by yylex.  */
static const yytype_int16 yytranslate[] =
{
       0,    80,    81,    82,    83,    84,    85,    86,    87,    88,
      89,    90,    91,    92,    93,    94,    95,    96,    97,    98,
      99,   100,   101,   102,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     296,   297,   127,   125,     2,   126,   128,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,   106,   103,
       2,     2,     2,   105,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,   129,     2,   130,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,   298,     2,   299,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     1,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     104,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     107,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     108,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     109,   110,   111,   112,   113,   114,     2,     2,     2,     2,
     115,   116,   117,   118,   119,   120,   121,   122,     2,   131,
     123,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     124,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     132,   133,   134,   135,   136,   137,   138,   139,   140,   141,
     142,   143,   144,     2,     2,     2,     2,     2,     2,     2,
     145,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     146,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,   147,   148,   149,   150,   151,   152,   153,   154,   155,
     156,   157,   158,   159,   160,   161,   162,   163,   164,   165,
     174,   175,   176,   177,   178,   179,   180,   181,     2,     2,
     182,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     183,   184,   185,     2,     2,     2,     2,     2,     2,     2,
     186,   187,     2,     2,     2,     2,     2,     2,     2,     2,
     188,   189,   190,     2,     2,     2,     2,     2,     2,     2,
     191,   192,   193,     2,     2,     2,     2,     2,     2,     2,
     194,   195,   196,     2,     2,     2,     2,     2,     2,     2,
     197,     2,   198,   199,   200,   201,   202,     2,     2,     2,
     203,   204,     2,     2,     2,     2,     2,     2,     2,     2,
     205,   206,   207,     2,     2,   208,   209,   210,     2,     2,
     211,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     212,   213,   214,   215,   216,   217,   218,   219,   220,   221,
     222,   223,   224,     2,     2,   225,   226,   227,     2,     2,
     228,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     229,   230,     2,     2,     2,   231,     2,     2,     2,     2,
     232,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     233,   234,   235,     2,     2,     2,     2,     2,     2,     2,
     236,   237,   238,     2,     2,     2,     2,     2,     2,     2,
     239,   240,   241,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     242,   243,   244,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,   245,    10,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     246,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     247,   248,   249,   250,   251,   252,   253,   254,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     255,   256,   257,     2,     2,     2,     2,     2,     2,     2,
     258,   259,   260,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     166,   167,   168,   169,   170,   171,   172,   173,     2,     2,
     261,   262,     2,     2,     2,     2,     2,     2,     2,     2,
     263,   264,   265,   266,   267,   268,     2,     2,     2,     2,
     269,   270,   271,   272,   273,   274,   275,   276,   277,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     278,     2,   279,   280,   281,   282,   283,   284,   285,   286,
     287,   288,   289,   290,   291,     2,     2,     2,     2,     2,
     292,   293,   294,     2,     2,     2,     2,     2,     2,     2,
     295,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       3,     2,     4,     5,     6,     7,     8,     9,    11,    12,
      13,    14,    15,    16,    17,    18,    19,    20,    21,    22,
      23,    24,    25,    26,    27,    28,    29,    30,    31,    32,
      33,    34,    35,    36,    37,    38,    39,    40,    41,    42,
      43,    44,    45,    46,    47,    48,    49,    50,    51,    52,
      53,    54,    55,    56,    57,    58,    59,    60,    61,    62,
      63,    64,    65,    66,    67,    68,    69,    70,    71,    72,
      73,    74,    75,    76,    77,    78,    79
};

#if AFSOCKET_DEBUG
/* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
static const yytype_int16 yyrline[] =
{
       0,   684,   684,   694,   695,   696,   697,   698,   699,   700,
     701,   702,   707,   708,   713,   712,   721,   720,   729,   730,
     734,   735,   736,   737,   738,   739,   740,   745,   753,   754,
     755,   756,   761,   761,   771,   771,   780,   781,   785,   789,
     798,   799,   800,   801,   802,   803,   804,   805,   810,   810,
     820,   820,   829,   830,   834,   836,   835,   845,   849,   850,
     851,   852,   853,   854,   858,   863,   863,   873,   874,   878,
     879,   880,   884,   889,   889,   899,   900,   904,   905,   906,
     907,   911,   912,   913,   914,   915,   919,   920,   924,   925,
     926,   930,   931,   937,   935,   947,   949,   948,   967,   966,
     979,   983,   983,   998,   999,  1003,  1004,  1005,  1009,  1010,
    1015,  1014,  1025,  1024,  1034,  1035,  1039,  1040,  1041,  1042,
    1046,  1047,  1048,  1049,  1054,  1053,  1064,  1063,  1074,  1075,
    1080,  1081,  1082,  1083,  1084,  1084,  1085,  1085,  1086,  1087,
    1088,  1089,  1093,  1093,  1097,  1098,  1102,  1102,  1106,  1107,
    1111,  1112,  1116,  1117,  1121,  1122,  1127,  1126,  1137,  1136,
    1146,  1147,  1151,  1153,  1152,  1164,  1165,  1173,  1177,  1176,
    1187,  1188,  1192,  1193,  1197,  1202,  1201,  1211,  1212,  1216,
    1217,  1221,  1223,  1222,  1242,  1244,  1243,  1256,  1257,  1258,
    1259,  1260,  1264,  1265,  1269,  1270,  1275,  1279,  1286,  1287,
    1291,  1295,  1300,  1305,  1310,  1316,  1321,  1326,  1331,  1336,
    1341,  1346,  1350,  1354,  1355,  1361,  1367,  1373,  1378,  1383,
    1388,  1392,  1397,  1403,  1410,  1420,  1426,  1427,  1431,  1436,
    1445,  1450,  1455,  1456,  1457,  1461,  1462,  1466,  1467,  1468,
    1469,  1470,  1471,  1476,  1481,  1489,  1779,  1786,  1795,  1803,
    1811,  1822,  1822,  1833,  1837,  1838,  1844,  1845,  1849,  1850,
    1854,  1855,  1865,  1866,  1870,  1877,  1884,  1891,  1898,  1902,
    1920,  1921,  1922,  1931,  1941,  1945,  1957,  1961,  1962,  1971,
    1982,  1990,  1999,  2000,  2004,  2031,  2032,  2036,  2064,  2065,
    2066,  2067,  2134,  2135,  2136,  2137,  2138,  2139,  2140,  2141,
    2142,  2143,  2144,  2144,  2151,  2152,  2153,  2154,  2155,  2156,
    2156,  2157,  2157,  2158,  2158,  2162,  2163,  2164,  2165,  2170,
    2177,  2178,  2179,  2183,  2184,  2185,  2186,  2190,  2191,  2197,
    2203,  2218,  2219,  2220,  2221,  2222,  2223,  2224,  2225,  2226,
    2227,  2233,  2234,  2234,  2238,  2239,  2243,  2244,  2245,  2246,
    2247,  2248,  2249,  2250,  2251,  2252,  2253,  2254,  2258,  2259,
    2260,  2261,  2262,  2263,  2264,  2449,  2450,  2451,  2452,  2453
};
#endif

/** Accessing symbol of state STATE.  */
#define YY_ACCESSING_SYMBOL(State) YY_CAST (yysymbol_kind_t, yystos[State])

#if 1
/* The user-facing name of the symbol whose (internal) number is
   YYSYMBOL.  No bounds checking.  */
static const char *yysymbol_name (yysymbol_kind_t yysymbol) YY_ATTRIBUTE_UNUSED;

/* YYTNAME[SYMBOL-NUM] -- String name of the symbol SYMBOL-NUM.
   First, the terminals, then, starting at YYNTOKENS, nonterminals.  */
static const char *const yytname[] =
{
  "\"end of file\"", "error", "\"invalid token\"", "KW_UNIX_STREAM",
  "KW_UNIX_DGRAM", "KW_TCP", "KW_UDP", "KW_TCP6", "KW_UDP6", "KW_NETWORK",
  "KW_HTTP", "KW_TRANSPORT", "KW_IP_PROTOCOL", "KW_SYSTEMD_SYSLOG",
  "KW_HTTP_SCRAPER", "KW_SCRAPE_PATTERN", "KW_SCRAPE_FREQ_LIMIT",
  "KW_SINGLE_INSTANCE", "KW_STAT_TYPE", "KW_STAT_QUERY", "KW_STAT_FORMAT",
  "KW_STATS_WITHOUT_ORPHANED", "KW_STATS_WITH_LEGACY", "KW_IP_TTL",
  "KW_SO_BROADCAST", "KW_IP_TOS", "KW_IP_FREEBIND", "KW_SO_SNDBUF",
  "KW_SO_RCVBUF", "KW_SO_KEEPALIVE", "KW_SO_REUSEPORT",
  "KW_TCP_KEEPALIVE_TIME", "KW_TCP_KEEPALIVE_PROBES",
  "KW_TCP_KEEPALIVE_INTVL", "KW_SO_PASSCRED", "KW_LISTEN_BACKLOG",
  "KW_SPOOF_SOURCE", "KW_SPOOF_SOURCE_MAX_MSGLEN", "KW_KEEP_ALIVE",
  "KW_MAX_CONNECTIONS", "KW_CLOSE_ON_INPUT", "KW_LOCALIP", "KW_IP",
  "KW_INTERFACE", "KW_LOCALPORT", "KW_DESTPORT", "KW_FAILOVER_SERVERS",
  "KW_FAILOVER", "KW_SERVERS", "KW_FAILBACK", "KW_TCP_PROBE_INTERVAL",
  "KW_SUCCESSFUL_PROBES_REQUIRED", "KW_DYNAMIC_WINDOW_SIZE",
  "KW_DYNAMIC_WINDOW_STATS_FREQ", "KW_DYNAMIC_WINDOW_REALLOC_TICKS",
  "KW_TLS", "KW_PEER_VERIFY", "KW_KEY_FILE", "KW_CERT_FILE",
  "KW_DHPARAM_FILE", "KW_PKCS12_FILE", "KW_CA_DIR", "KW_CRL_DIR",
  "KW_CA_FILE", "KW_TRUSTED_KEYS", "KW_TRUSTED_DN", "KW_CIPHER_SUITE",
  "KW_TLS12_AND_OLDER", "KW_TLS13", "KW_SIGALGS", "KW_CLIENT_SIGALGS",
  "KW_ECDH_CURVE_LIST", "KW_SSL_OPTIONS", "KW_SSL_VERSION", "KW_SNI",
  "KW_ALLOW_COMPRESS", "KW_KEYLOG_FILE", "KW_OCSP_STAPLING_VERIFY",
  "KW_EXTENDED_KEY_USAGE_VERIFY", "KW_CONF_CMDS", "LL_CONTEXT_ROOT",
  "LL_CONTEXT_DESTINATION", "LL_CONTEXT_SOURCE", "LL_CONTEXT_PARSER",
  "LL_CONTEXT_REWRITE", "LL_CONTEXT_FILTER", "LL_CONTEXT_LOG",
  "LL_CONTEXT_BLOCK_DEF", "LL_CONTEXT_BLOCK_REF",
  "LL_CONTEXT_BLOCK_CONTENT", "LL_CONTEXT_BLOCK_ARG",
  "LL_CONTEXT_BLOCK_FUNCARG", "LL_CONTEXT_PRAGMA", "LL_CONTEXT_FORMAT",
  "LL_CONTEXT_TEMPLATE_FUNC", "LL_CONTEXT_INNER_DEST",
  "LL_CONTEXT_INNER_SRC", "LL_CONTEXT_CLIENT_PROTO",
  "LL_CONTEXT_SERVER_PROTO", "LL_CONTEXT_OPTIONS", "LL_CONTEXT_CONFIG",
  "LL_CONTEXT_TEMPLATE_REF", "LL_CONTEXT_MAX", "';'", "KW_ASSIGN", "'?'",
  "':'", "KW_OR", "KW_AND", "KW_STR_EQ", "KW_STR_NE", "KW_TA_EQ",
  "KW_TA_NE", "KW_TAV_EQ", "KW_TAV_NE", "KW_STR_LT", "KW_STR_LE",
  "KW_STR_GE", "KW_STR_GT", "KW_TA_LT", "KW_TA_LE", "KW_TA_GE", "KW_TA_GT",
  "KW_PLUS_ASSIGN", "KW_REGEXP_MATCH", "'+'", "'-'", "'*'", "'.'", "'['",
  "']'", "KW_NOT", "KW_SOURCE", "KW_FILTER", "KW_PARSER", "KW_DESTINATION",
  "KW_LOG", "KW_OPTIONS", "KW_INCLUDE", "KW_BLOCK", "KW_JUNCTION",
  "KW_CHANNEL", "KW_IF", "KW_ELSE", "KW_ELIF", "KW_INTERNAL", "KW_SYSLOG",
  "KW_MARK_FREQ", "KW_STATS_FREQ", "KW_STATS_LEVEL", "KW_STATS_LIFETIME",
  "KW_FLUSH_LINES", "KW_SUPPRESS", "KW_FLUSH_TIMEOUT", "KW_LOG_MSG_SIZE",
  "KW_FILE_TEMPLATE", "KW_PROTO_TEMPLATE", "KW_MARK_MODE", "KW_ENCODING",
  "KW_TYPE", "KW_STATS_MAX_DYNAMIC", "KW_MIN_IW_SIZE_PER_READER",
  "KW_WORKERS", "KW_BATCH_LINES", "KW_BATCH_TIMEOUT",
  "KW_TRIM_LARGE_MESSAGES", "KW_STATS", "KW_FREQ", "KW_LEVEL",
  "KW_LIFETIME", "KW_MAX_DYNAMIC", "KW_SYSLOG_STATS",
  "KW_HEALTHCHECK_FREQ", "KW_WORKER_PARTITION_KEY", "KW_CHAIN_HOSTNAMES",
  "KW_NORMALIZE_HOSTNAMES", "KW_KEEP_HOSTNAME", "KW_CHECK_HOSTNAME",
  "KW_BAD_HOSTNAME", "KW_LOG_LEVEL", "KW_IDLE_TIMEOUT", "KW_CHECK_PROGRAM",
  "KW_KEEP_TIMESTAMP", "KW_USE_DNS", "KW_USE_FQDN", "KW_CUSTOM_DOMAIN",
  "KW_DNS_CACHE", "KW_DNS_CACHE_SIZE", "KW_DNS_CACHE_EXPIRE",
  "KW_DNS_CACHE_EXPIRE_FAILED", "KW_DNS_CACHE_HOSTS", "KW_PERSIST_ONLY",
  "KW_USE_RCPTID", "KW_USE_UNIQID", "KW_TZ_CONVERT", "KW_TS_FORMAT",
  "KW_FRAC_DIGITS", "KW_LOG_FIFO_SIZE", "KW_LOG_FETCH_LIMIT",
  "KW_LOG_IW_SIZE", "KW_LOG_PREFIX", "KW_PROGRAM_OVERRIDE",
  "KW_HOST_OVERRIDE", "KW_THROTTLE", "KW_THREADED",
  "KW_PASS_UNIX_CREDENTIALS", "KW_PERSIST_NAME", "KW_USE_SYSLOGNG_PID",
  "KW_READ_OLD_RECORDS", "KW_IGNORE_SAVED_BOOKMARKS",
  "KW_DISABLE_BOOKMARKS", "KW_FLAGS", "KW_PAD_SIZE", "KW_TIME_ZONE",
  "KW_RECV_TIME_ZONE", "KW_SEND_TIME_ZONE", "KW_LOCAL_TIME_ZONE",
  "KW_FORMAT", "KW_MULTI_LINE_MODE", "KW_MULTI_LINE_PREFIX",
  "KW_MULTI_LINE_GARBAGE", "KW_TRUNCATE_SIZE", "KW_TIME_REOPEN",
  "KW_TIME_REAP", "KW_TIME_SLEEP", "KW_PARALLELIZE", "KW_PARTITIONS",
  "KW_PARTITION_KEY", "KW_TMPL_ESCAPE", "KW_LOG_FETCH_DELAY",
  "KW_LOG_FETCH_RETRY_DELAY", "KW_OPTIONAL", "KW_CREATE_DIRS", "KW_OWNER",
  "KW_GROUP", "KW_PERM", "KW_DIR_OWNER", "KW_DIR_GROUP", "KW_DIR_PERM",
  "KW_TEMPLATE", "KW_TEMPLATE_ESCAPE", "KW_TEMPLATE_FUNCTION",
  "KW_DEFAULT_FACILITY", "KW_DEFAULT_SEVERITY", "KW_SDATA_PREFIX",
  "KW_PORT", "KW_USE_TIME_RECVD", "KW_FACILITY", "KW_SEVERITY", "KW_HOST",
  "KW_MATCH", "KW_MESSAGE", "KW_NETMASK", "KW_TAGS", "KW_NETMASK6",
  "KW_REWRITE", "KW_CONDITION", "KW_VALUE", "KW_YES", "KW_NO", "KW_AUTO",
  "KW_IFDEF", "KW_ENDIF", "LL_DOTDOT", "LL_DOTDOTDOT", "LL_PRAGMA",
  "LL_EOL", "LL_ERROR", "LL_ARROW", "LL_IDENTIFIER", "LL_NUMBER",
  "LL_FLOAT", "LL_STRING", "LL_TOKEN", "LL_BLOCK", "LL_PLUGIN",
  "LL_TEMPLATE_REF", "LL_MESSAGE_REF", "KW_VALUE_PAIRS", "KW_EXCLUDE",
  "KW_PAIR", "KW_KEY", "KW_SCOPE", "KW_SHIFT", "KW_SHIFT_LEVELS",
  "KW_REKEY", "KW_ADD_PREFIX", "KW_REPLACE_PREFIX", "KW_CAST", "KW_UPPER",
  "KW_LOWER", "KW_INCLUDE_BYTES", "KW_ON_ERROR", "KW_RETRIES",
  "KW_FETCH_NO_DATA_DELAY", "KW_LABELS", "'('", "')'", "'{'", "'}'",
  "$accept", "start", "driver", "source_afunix",
  "source_afunix_dgram_params", "$@1", "source_afunix_stream_params",
  "$@2", "source_afunix_options", "source_afunix_option", "source_afinet",
  "source_afinet_udp_params", "$@3", "source_afinet_udp6_params", "$@4",
  "source_afinet_udp_options", "source_afinet_udp_option",
  "source_afinet_multi_line_option", "source_afinet_option",
  "source_afinet_tcp_params", "$@5", "source_afinet_tcp6_params", "$@6",
  "source_afinet_tcp_options", "source_afinet_tcp_option", "$@7",
  "source_afsocket_stream_params", "source_afsyslog",
  "source_afsyslog_params", "$@8", "source_afsyslog_options",
  "source_afsyslog_option", "source_afnetwork", "source_afnetwork_params",
  "$@9", "source_afnetwork_options", "source_afnetwork_option",
  "stats_option", "stats_options", "proto_http_scraper_option",
  "proto_http_scraper_options", "proto_http_scraper_transport", "$@10",
  "source_afsocket_transport", "$@11", "$@12", "source_systemd_syslog",
  "source_systemd_syslog_params", "$@13", "source_systemd_syslog_options",
  "source_systemd_syslog_option", "dest_afunix",
  "dest_afunix_dgram_params", "$@14", "dest_afunix_stream_params", "$@15",
  "dest_afunix_options", "dest_afunix_option", "dest_afinet",
  "dest_afinet_udp_params", "$@16", "dest_afinet_udp6_params", "$@17",
  "dest_afinet_udp_options", "dest_afinet_option", "$@18", "$@19",
  "dest_failover_options", "$@20", "dest_failover_modes_options",
  "dest_failback_options", "$@21", "dest_failback_probe_options",
  "dest_failback_probe_option", "dest_afinet_dgram_option",
  "dest_afinet_udp_option", "dest_afinet_tcp_params", "$@22",
  "dest_afinet_tcp6_params", "$@23", "dest_afinet_tcp_options",
  "dest_afinet_tcp_option", "$@24", "dest_afsocket_option",
  "dest_afsyslog", "dest_afsyslog_params", "$@25", "dest_afsyslog_options",
  "dest_afsyslog_option", "dest_afnetwork", "dest_afnetwork_params",
  "$@26", "dest_afnetwork_options", "dest_afnetwork_option",
  "dest_afsocket_transport", "$@27", "$@28", "afsocket_transport",
  "dest_tls_options", "dest_tls_option", "tls_options", "tls_option",
  "tls_conf_cmds", "tls_conf_cmd_list_build", "tls_cipher_suites",
  "tls_cipher_suite", "socket_option", "unix_socket_option",
  "inet_socket_option", "inet_ip_protocol_option",
  "template_content_inner", "type_hint", "template_content", "@31",
  "template_name_or_content", "template_name_or_content_tail", "string",
  "yesno_strict", "yesno", "dnsmode", "nonnegative_integer64",
  "nonnegative_integer", "positive_integer64", "positive_integer",
  "nonnegative_float", "string_or_number", "path", "path_check",
  "path_secret", "string_list", "string_list_build", "severity_string",
  "facility_string", "driver_option", "inner_source",
  "source_driver_option", "inner_dest", "dest_driver_option",
  "source_option", "$@34", "source_reader_option", "$@35", "$@36", "$@37",
  "source_reader_option_flags", "source_proto_option",
  "host_resolve_option", "msg_format_option", "dest_writer_option", "$@38",
  "dest_writer_options_flags", "file_perm_option", "template_option",
  "_template_ref_context_push", "_inner_dest_context_push",
  "_inner_dest_context_pop", "_inner_src_context_push",
  "_inner_src_context_pop", YY_NULLPTR
};

static const char *
yysymbol_name (yysymbol_kind_t yysymbol)
{
  return yytname[yysymbol];
}
#endif

#define YYPACT_NINF (-603)

#define yypact_value_is_default(Yyn) \
  ((Yyn) == YYPACT_NINF)

#define YYTABLE_NINF (-314)

#define yytable_value_is_error(Yyn) \
  0

/* YYPACT[STATE-NUM] -- Index in YYTABLE of the portion describing
   STATE-NUM.  */
static const yytype_int16 yypact[] =
{
     -39,    22,    50,    37,  -603,  -219,  -211,  -198,  -193,  -175,
    -162,  -138,  -120,  -603,  -603,  -603,  -603,  -119,  -106,   -89,
     -44,   -38,   -22,   -17,   -12,   -11,  -603,  -603,  -603,  -603,
    -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,
    -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -188,
    -188,  -188,  -188,  -188,  -188,  -188,  -188,  -188,  -188,  -603,
    -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,
    -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,
    -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,   827,
    -603,  1379,  -603,   827,  -603,  1379,  -603,   282,  -603,  1498,
    -603,   479,   -73,  -603,   -34,  -603,    -8,  -603,    -2,  -603,
       0,  -603,     3,  -603,    19,  -603,    21,  -603,    25,  1574,
      35,  1574,    44,    47,    49,    57,    58,    67,    79,    88,
      89,    94,    95,   108,   111,   121,   123,   124,   133,   136,
     137,   138,   142,   146,  -603,   148,   149,   155,   159,   161,
     165,   168,   170,   172,  -603,  -603,  -603,  -603,   827,  -603,
    -603,  -603,  -603,  -603,  -603,  -603,   157,    43,   -14,    85,
    -603,  1379,  -603,   152,  -603,   156,  -603,   173,   176,   178,
    -603,  -603,  -603,  -603,   282,  -603,  -603,  -603,   181,  -603,
    1498,  -603,  -603,  -603,   192,   196,  -603,  -603,  -603,   479,
    -603,  -603,    80,  -603,  1617,  -603,  1308,  -603,  1053,  -603,
    1343,  -603,  1082,  -603,   754,  -603,   791,  -603,   198,   200,
     201,   202,   205,   219,   220,   223,   232,   233,  -603,  1574,
    -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,    70,
     -24,    70,   -24,    70,    70,   -24,   -24,    70,    70,    70,
      76,   -24,    76,  -188,  -188,  -188,    -4,    70,   -90,    70,
     234,   -24,   -24,   -24,    76,  -188,   -94,  -188,  -188,    -4,
    -603,   255,   256,   257,   258,   259,   265,   267,   268,   269,
     279,  -603,    28,   280,   281,   285,   286,  -603,   287,   304,
     315,   316,  -603,  -603,  -603,  -603,  -603,  -603,     9,   308,
     317,  -603,  -603,  -603,  -603,    27,  -603,   321,   322,   323,
     324,   325,   326,   329,   330,   331,   332,   333,   334,   335,
     336,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,   134,
     338,   342,   343,  -603,  -603,  -603,   344,  -603,  -603,  -603,
    -603,  -603,  -603,   345,   346,  -603,  -603,  -603,   347,  -603,
    -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,   -24,   -24,
     -24,   -24,     1,   109,  -222,   127,   131,  -221,  -603,  -603,
    -603,   349,  -603,  -603,  -603,  -603,   350,   352,   353,   357,
     358,   360,   361,   364,   365,   366,  -603,  -603,   367,   368,
     369,   370,   371,   372,  -603,  -603,  -603,   373,   374,  -603,
    -603,   375,   376,    90,   377,   378,   379,   381,   382,   -94,
     -94,   -94,   383,   384,   385,   386,   -24,   -24,   -24,    76,
    -188,  -188,  -188,   -24,   -24,  -188,   388,   390,   391,   392,
    -603,    76,  -188,   -24,    76,  -188,  -105,  -188,  -188,   394,
     396,   393,   397,   398,  -603,  -603,   401,    90,   -24,   -24,
      70,    70,    70,    76,   -80,    76,    70,  -188,    70,    70,
      76,  -603,   403,   404,   405,   406,   407,   408,   409,  -603,
    -188,    -4,    -4,   410,   411,   412,    -4,   -24,    76,    27,
     413,   414,   415,   416,   417,  -603,   418,  -603,   419,   420,
    -603,  -603,   421,  -603,   422,   423,  -603,  -603,  -603,  -603,
    -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,
    -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,   429,   430,
     431,   432,   433,   434,   435,   436,   437,   438,   439,   441,
     442,   443,   444,   445,   446,   447,   449,  -603,  -603,   451,
      90,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,
    -603,  -603,  -603,   452,   453,   454,   455,   456,   458,   461,
     462,   463,  -188,   464,  -603,   -24,  -153,   -24,   -24,   466,
     467,   470,   471,   474,  -603,  -603,   475,  -603,   492,   496,
    -603,  -603,  -603,  -603,  -603,   499,  -603,   507,   508,   509,
     510,   511,   513,   514,   515,   516,   528,   529,  -188,   533,
     536,   542,   543,   544,   468,  -188,    70,  -188,  -188,  -188,
     -24,  -188,   545,   546,   547,  -188,   597,    66,   548,   550,
     551,  -603,    66,  -603,  -603,  -603,  -603,  -603,  -603,  -603,
    -603,  -603,  -603,   153,  -188,  -188,  -188,  -188,  -188,  -188,
    -188,  -188,  -188,   -33,  -188,  -188,  -188,  -188,  -188,   -24,
    -188,  -188,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,
    -603,  -603,  -603,  -603,  -603,   564,  -603,  -603,   566,   567,
     570,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,   552,
    -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,
    -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,
     144,  -603,   575,   576,   577,   578,   579,   580,   581,  -603,
    -603,  -603,   586,   553,   587,   589,   590,   591,   592,    66,
    -603,  -603,  -603,  -603,   593,   594,   595,   596,  -603,  -603,
     598,  -603,   599,   600,   601,   603,   605,   606,   607,   611,
     613,   614,   615,   180,   616,   617,   618,   619,   620,   621,
     622,   623,   624,  -603,   478,  -603,  -603,  -603,  -603,   626,
     627,   628,   629,   630,   631,   632,   633,   348,  -603,   552,
     634,   635,  -603,  -603,  -603,   637,  -603,  -603,  -603,  -603,
    -603,  -603,  -603,  -603,  -603,  -188,  -603,   -24,   -24,   -24,
    -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,
    -603,  -603,  -603,  -603,  -603,  -188,  -188,  -603,  -603,  -603,
    -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -188,  -188,
      70,   -24,  -188,  -188,  -188,   -24,   -24,  -603,  -603,   638,
      -4,   640,   642,   643,   644,   648,   649,  -188,   650,   652,
     653,   655,   656,   657,   658,   659,  -603,   661,  -603,  -603,
    -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,
    -603,  -603,  -603,  -603,   698,  -603,  -603,  -603,   663,  -603,
     -40,   665,   666,  -603,  -603,    76,    76,   667,   670,  -603,
    -603
};

/* YYDEFACT[STATE-NUM] -- Default reduction number in state STATE-NUM.
   Performed when YYTABLE does not specify something else to do.  Zero
   means the default is an error.  */
static const yytype_int16 yydefact[] =
{
       0,     0,     0,     0,     2,     0,     0,     0,     0,     0,
       0,     0,     0,     8,     9,    10,    11,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     3,     4,     5,     6,
       7,     1,   366,   366,   366,   366,   366,   366,   366,   366,
     368,   368,   368,   368,   368,   368,   368,   368,   368,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,    48,
      32,    50,    34,    73,   101,    65,   256,   257,   367,   112,
     367,   110,   367,   156,   367,   124,   367,   158,   367,   126,
     367,   175,   367,   168,   369,    16,   369,    14,   369,   309,
     369,   309,   369,   309,   369,   309,   369,   309,   369,   309,
     369,   309,     0,   115,     0,   115,     0,   161,     0,   129,
       0,   161,     0,   129,     0,   178,     0,   171,     0,   309,
       0,   309,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,    55,     0,     0,     0,     0,     0,
       0,     0,     0,     0,   284,    44,    54,    49,   309,    57,
     237,    47,   286,   285,    46,    45,   302,     0,     0,     0,
      33,   309,    38,     0,    51,     0,    35,     0,     0,     0,
      98,    77,    80,    74,   309,    79,    78,    95,     0,   102,
     309,   106,   107,   105,     0,     0,    69,    71,    66,   309,
      70,   109,   342,   108,   342,   121,   342,   120,   342,   123,
     342,   122,   342,   174,   342,   167,   342,    13,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,    17,   309,
      21,   235,    24,    23,    22,    20,    12,    15,    29,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,   318,     0,     0,     0,
      52,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,   310,     0,     0,     0,     0,     0,   312,     0,     0,
       0,     0,   314,    28,    36,    31,    30,    72,     0,     0,
       0,    75,   100,   103,    64,     0,    67,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,   287,   114,   117,   118,   291,   290,   119,   116,     0,
       0,     0,     0,   134,   136,   163,     0,   162,   160,   140,
     138,   141,   139,     0,     0,   154,   155,   128,     0,   185,
     179,   184,   177,   180,   181,   172,   170,   173,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,    18,   264,
     265,     0,   258,   259,   261,   260,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,   266,   267,     0,     0,
       0,     0,     0,     0,   271,   272,   270,     0,     0,   268,
     269,     0,     0,   199,     0,     0,     0,     0,     0,   318,
     318,   318,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,   278,     0,     0,     0,     0,
     303,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,    96,   245,     0,   199,     0,     0,
       0,     0,     0,     0,     0,     0,     0,   345,     0,     0,
       0,   365,     0,     0,     0,     0,     0,     0,     0,   343,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,   347,     0,   349,     0,     0,
     351,   353,     0,   355,     0,     0,   357,   238,   232,   239,
     240,   230,   231,   233,   234,   242,   244,   243,    60,    58,
      59,    40,    41,   241,    42,    61,    62,    63,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,   200,   222,     0,
     199,   283,   304,   305,   307,   282,   316,   317,   315,   306,
     308,    39,    43,     0,     0,     0,     0,     0,     0,     0,
       0,     0,   278,     0,   276,     0,     0,     0,     0,     0,
       0,     0,     0,     0,   281,   280,     0,   279,     0,     0,
     187,   188,    93,   189,   190,     0,   191,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,   345,     0,
       0,     0,     0,     0,   251,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,   278,     0,   193,     0,     0,
       0,   182,   193,   236,    26,    25,    27,   346,   348,   350,
     352,   354,   356,     0,     0,     0,     0,     0,     0,     0,
       0,   278,   278,   227,     0,     0,     0,   278,     0,     0,
       0,   225,    56,   198,   293,   294,   298,   292,   297,   295,
     296,   300,   299,   277,   301,     0,   263,   262,     0,     0,
       0,   320,   319,   321,   322,   327,   329,   328,   330,    87,
      97,    99,   165,   166,   338,   332,   334,   333,   339,   340,
     288,   289,   344,   331,   336,   337,   341,   335,   254,   255,
       0,   253,     0,     0,     0,     0,     0,     0,     0,   130,
     131,   133,     0,     0,     0,     0,     0,     0,     0,   193,
     194,   132,   152,   153,     0,     0,     0,     0,   273,   275,
       0,   274,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,   227,     0,     0,     0,     0,     0,     0,
       0,     0,     0,   223,     0,   326,   324,   323,   325,     0,
       0,     0,     0,     0,     0,     0,     0,    87,    92,    87,
       0,   250,   248,   249,   252,     0,   246,   358,   359,   360,
     361,   362,   363,   364,   135,   278,   137,     0,     0,     0,
     164,   192,   183,   186,   202,   201,   203,   205,   206,   207,
     208,   209,   210,   211,   212,     0,     0,   213,   226,   214,
     215,   216,   217,   218,   219,   220,   204,   221,     0,     0,
       0,     0,     0,     0,     0,     0,     0,    86,    91,     0,
       0,     0,     0,     0,     0,     0,     0,   225,     0,     0,
       0,     0,     0,     0,     0,     0,    94,     0,   142,   195,
     196,   197,   228,   229,   224,    88,    89,    90,    81,    82,
      83,    84,    85,   247,   145,   146,   143,   144,     0,   149,
       0,     0,     0,   147,   148,     0,     0,     0,     0,   150,
     151
};

/* YYPGOTO[NTERM-NUM].  */
static const yytype_int16 yypgoto[] =
{
    -603,  -603,  -603,  -603,  -603,  -603,  -603,  -603,   -55,  -603,
    -603,  -603,  -603,  -603,  -603,   -75,  -603,  -603,   247,  -603,
    -603,  -603,  -603,   -72,  -603,  -603,   -27,  -603,  -603,  -603,
     563,  -603,  -603,  -603,  -603,   526,  -603,  -603,   163,  -603,
     119,  -603,  -603,   -85,  -603,  -603,  -603,  -603,  -603,   704,
    -603,  -603,  -603,  -603,  -603,  -603,   829,  -603,  -603,  -603,
    -603,  -603,  -603,   850,   227,  -603,  -603,  -603,  -603,  -603,
    -603,  -603,  -603,  -603,  -123,  -603,  -603,  -603,  -603,  -603,
     857,  -603,  -603,  -107,  -603,  -603,  -603,  -603,  -603,  -603,
    -603,  -603,  -603,  -603,   753,  -603,  -603,   -16,  -586,  -603,
    -429,  -600,  -603,   140,   228,  -603,   -20,  -603,   242,  -603,
    -603,  -603,  -603,  -603,  -603,  -603,   -49,  -603,  -173,  -603,
    -603,  -197,  -603,  -240,  -603,  -250,   339,  -569,  -603,  -602,
     424,  -603,  -603,   263,  -603,   -19,  -603,    78,  -603,  -603,
      74,  -603,  -603,  -603,  -169,  -603,  -603,  -603,    86,  -603,
     380,  -603,  -603,  -603,   448,   465,   562,   450
};

/* YYDEFGOTO[NTERM-NUM].  */
static const yytype_int16 yydefgoto[] =
{
       0,     3,     4,    26,    86,   121,    84,   119,   228,   229,
      27,    90,    91,    94,    95,   170,   171,   155,   156,    88,
      89,    92,    93,   157,   158,   260,   159,    28,   100,   101,
     198,   199,    29,    96,    97,   183,   184,   767,   768,   769,
     770,   185,   679,   186,   585,   300,    30,    98,    99,   189,
     190,    13,    70,   105,    68,   103,   202,   322,    14,    74,
     109,    78,   113,   208,   337,   473,   474,   714,   864,   866,
     867,   868,   870,   874,   346,   347,    72,   107,    76,   111,
     206,   338,   475,   339,    15,    82,   117,   216,   356,    16,
      80,   115,   214,   352,   353,   724,   480,   187,   718,   719,
     539,   540,   752,   753,   742,   743,   160,   232,   161,   446,
     774,   775,   699,   700,   603,   701,   396,   375,   376,   668,
     370,   371,   387,   388,   401,   397,   731,   732,   730,   563,
     564,   578,   576,   162,   163,   164,   326,   341,   281,   282,
     165,   166,   167,   168,   412,   287,   430,   292,   342,   329,
     599,   235,   469,   604,    49,   102,    57,   118
};

/* YYTABLE[YYPACT[STATE-NUM]] -- What to do in state STATE-NUM.  If
   positive, shift that token.  If negative, reduce the rule whose
   number is the opposite.  If YYTABLE_NINF, syntax error.  */
static const yytype_int16 yytable[] =
{
      69,    71,    73,    75,    77,    79,    81,    83,    85,    87,
     871,   872,   390,   712,   439,   440,   200,   720,   587,   415,
     176,   174,   720,   441,   407,     5,     6,     7,     8,     9,
      10,    11,   439,   440,   740,   741,   725,    31,   666,   738,
     739,   574,     1,     2,   377,   748,   379,   380,   489,   495,
     383,   384,   385,    17,    18,    19,    20,    21,    22,    23,
     398,   400,   402,    24,   442,   594,   237,   733,   734,   378,
     182,   737,   381,   382,   197,   490,   496,    32,   389,   191,
     192,    66,   442,   409,    67,    33,   270,   410,   404,   405,
     406,   351,   230,   351,   230,   323,   294,   323,    34,   231,
     233,   231,   233,    35,   124,   372,   373,   127,   128,   129,
     130,   653,   486,   488,   200,   492,   494,   374,   307,   720,
     308,    36,   518,   519,   520,   521,   522,   523,   524,   525,
     526,   527,   528,   791,    37,   529,   530,   531,   532,   533,
     715,   534,   535,   716,   717,   536,   518,   519,   520,   521,
     522,   523,   524,   525,   526,   527,   528,   182,    38,   529,
     530,   531,   532,   533,    66,   534,   535,    67,    12,   536,
     191,   192,   197,   193,   368,    66,    39,    40,    67,   556,
     369,   399,   324,   831,   324,   481,   482,   483,   484,    66,
      41,   569,    67,   234,   572,   234,    25,   283,   354,   288,
     354,   284,   230,   426,   391,   392,   393,    42,   285,   231,
     233,   427,   428,   593,   429,   596,   408,   411,   413,   414,
     602,   613,   614,   286,   201,   145,   618,   309,   289,   290,
     291,   310,   311,   312,   372,   373,    66,   313,   620,    67,
     546,   547,   548,   553,   554,   555,   374,   740,   741,   444,
     560,   561,    43,   590,   591,   592,   444,   873,    44,   597,
     571,   600,   601,   203,   193,    66,   394,   395,    67,   443,
      66,   394,   395,    67,    45,   588,   589,   314,    66,    46,
     327,    67,   327,   315,    47,    48,   149,   443,   328,   205,
     328,   316,   317,   178,   179,   207,    66,   209,   485,    67,
     211,   318,   319,   234,   619,   123,   124,   125,   126,   127,
     128,   129,   130,   131,   132,   133,   213,   134,   215,   320,
     135,   136,   217,   137,   138,   139,   140,   537,   538,   462,
     463,   271,   236,   272,   141,   142,   143,   180,   172,   273,
     369,   238,   172,   239,   181,   240,   386,   464,   196,   465,
     466,   537,   538,   241,   242,   321,   274,   275,   276,   277,
     411,   411,   411,   243,   278,   279,   762,   763,   764,   765,
     766,   557,   558,   559,   467,   244,   562,  -113,    66,   394,
     395,    67,   293,   570,   245,   246,   573,   575,   577,   579,
     247,   248,   665,   667,   669,   670,    66,   394,   395,    67,
      66,   394,   395,    67,   249,   595,   487,   250,   598,   703,
     280,   372,   373,   771,   772,   773,    67,   251,   172,   252,
     253,   612,    66,   374,   491,    67,   468,   145,   493,   254,
     621,   181,   255,   256,   257,   345,  -311,   707,   258,   345,
    -311,   350,   259,   355,   261,   262,   196,  -311,   340,   295,
     340,   263,   340,   296,   340,   264,   340,   265,   340,   146,
     727,   266,  -311,   147,   267,   325,   268,   325,   269,   325,
     297,   325,   298,   325,   299,   325,   750,   325,   302,   325,
     148,    50,    51,    52,    53,    54,    55,    56,   149,   304,
     195,   179,   305,   150,   358,  -313,   359,   360,   361,   151,
     152,   362,   123,   124,   125,   126,   127,   128,   129,   130,
     131,   132,   133,   562,   134,   363,   364,   135,   136,   365,
     137,   138,   139,   140,  -313,  -313,  -313,   153,   366,   367,
     403,   141,   142,   143,   180,   104,   120,   106,   122,   108,
     169,   110,   173,   112,   175,   114,   177,   116,   188,   598,
     194,   416,   417,   418,   419,   420,   702,   154,   704,   705,
     706,   421,   708,   422,   423,   424,   562,   759,   760,   761,
     762,   763,   764,   765,   766,   425,   431,   432,   445,   -76,
     847,   433,   434,   435,   726,   728,   728,   728,   728,   735,
     736,   728,   562,   562,   744,   745,   746,   747,   562,   749,
     436,   751,   754,    58,    59,    60,    61,    62,    63,    64,
      65,   437,   438,   447,   832,   833,   834,   448,   449,   450,
     451,   452,   453,   839,   145,   454,   455,   456,   457,   458,
     459,   460,   461,  -311,   470,   877,   878,  -311,   471,   472,
     476,   477,   478,   479,  -311,   713,   497,   498,   840,   499,
     500,   776,   844,   845,   501,   502,   146,   503,   504,  -311,
     147,   505,   506,   507,   508,   509,   510,   511,   512,   513,
     514,   515,   516,   517,   541,   542,   543,   148,   544,   545,
     549,   550,   551,   552,   565,   149,   566,   567,   568,   582,
     150,   580,  -313,   581,   583,   584,   151,   152,   586,   605,
     606,   607,   608,   609,   610,   611,   615,   616,   617,   622,
     301,   623,   624,   625,   626,   627,   628,   629,   630,   631,
     632,  -313,  -313,  -313,   153,   633,   634,   635,   636,   637,
     638,   639,   640,   641,   642,   643,   562,   644,   645,   646,
     647,   648,   649,   650,   698,   651,   818,   865,   652,   654,
     655,   656,   657,   658,   154,   659,   835,   836,   660,   661,
     662,   664,   306,   671,   672,   348,   179,   673,   674,   837,
     838,   675,   676,   841,   842,   843,   -68,   123,   124,   125,
     126,   127,   128,   129,   130,   131,   132,   133,   754,   677,
     343,   344,   307,   678,   308,   330,   680,   139,   331,   332,
     333,   334,   348,   179,   681,   682,   683,   684,   685,   349,
     686,   687,   688,   689,   123,   124,   125,   126,   127,   128,
     129,   130,   131,   132,   133,   690,   691,   343,   344,   307,
     693,   308,   330,   694,   139,   331,   332,   333,   334,   695,
     696,   697,   709,   710,   711,   721,   349,   722,   723,   785,
     123,   124,   125,   126,   127,   128,   129,   130,   131,   132,
     133,   755,   134,   756,   757,   135,   136,   758,   137,   138,
     139,   140,   777,   778,   779,   780,   781,   782,   783,   141,
     142,   143,   144,   784,   786,   787,   788,   789,   828,   790,
     792,   793,   794,   795,   303,   796,   797,   798,   799,   145,
     800,   309,   801,   802,   803,   310,   311,   312,   804,   805,
     806,   313,   807,   809,   810,   811,   812,   813,   814,   815,
     816,   817,   819,   820,   821,   822,   823,   824,   825,   826,
     827,   829,  -256,   830,   204,   846,   145,   848,   309,   849,
     850,   851,   310,   311,   312,   852,   853,   855,   313,   856,
     857,   314,   858,   859,   860,   861,   862,   315,   863,   869,
     149,   875,   876,   212,   879,   316,   317,   880,   210,   357,
       0,   808,   145,   729,     0,   318,   319,   854,   692,     0,
       0,  -311,     0,     0,     0,  -311,   663,     0,   314,     0,
       0,     0,  -311,   320,   315,     0,     0,   149,     0,   336,
       0,     0,   316,   317,   146,     0,     0,  -311,   147,     0,
       0,     0,   318,   319,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,   148,     0,     0,     0,   321,
     320,     0,     0,   149,     0,     0,   336,     0,   150,     0,
    -313,     0,     0,     0,   151,   152,     0,     0,     0,     0,
       0,  -176,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,   321,     0,     0,  -313,
    -313,  -313,   153,     0,     0,     0,   123,   124,   125,   126,
     127,   128,   129,   130,   131,   132,   133,     0,  -169,   343,
     344,   307,     0,   308,   330,     0,   139,   331,   332,   333,
     334,     0,   154,     0,     0,   123,   124,   125,   126,   127,
     128,   129,   130,   131,   132,   133,     0,     0,   343,   344,
     307,     0,   308,   330,   -53,   139,   331,   332,   333,   334,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,   145,     0,
     309,     0,     0,     0,   310,   311,   312,     0,     0,     0,
     313,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,   145,     0,   309,
       0,     0,     0,   310,   311,   312,     0,     0,     0,   313,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     314,     0,     0,     0,     0,     0,   315,     0,     0,   149,
       0,     0,     0,     0,   316,   317,     0,     0,     0,     0,
       0,     0,     0,     0,   318,   319,     0,     0,     0,   314,
       0,     0,     0,     0,     0,   315,     0,     0,   149,     0,
       0,     0,   320,   316,   317,     0,     0,     0,   336,     0,
       0,     0,     0,   318,   319,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,   320,     0,     0,     0,     0,     0,   336,   321,     0,
       0,   123,   124,   125,   126,   127,   128,   129,   130,   131,
     132,   133,     0,     0,     0,     0,   307,     0,   308,   330,
    -125,   139,   331,   332,   333,   334,     0,   321,     0,     0,
       0,     0,     0,   335,     0,     0,   123,   124,   125,   126,
     127,   128,   129,   130,   131,   132,   133,     0,     0,  -127,
       0,   307,     0,   308,   330,     0,   139,   331,   332,   333,
     334,     0,     0,     0,     0,     0,     0,     0,   335,     0,
       0,     0,   123,   124,   125,   126,   127,   128,   129,   130,
     131,   132,   133,     0,     0,     0,     0,     0,     0,     0,
     137,   138,   139,   140,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,   145,     0,   309,     0,     0,     0,   310,
     311,   312,     0,     0,     0,   313,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,   145,     0,
     309,     0,     0,     0,   310,   311,   312,     0,     0,     0,
     313,     0,     0,     0,     0,   314,     0,     0,     0,     0,
       0,   315,     0,     0,   149,     0,     0,     0,     0,   316,
     317,     0,   124,     0,   145,   127,   128,   129,   130,   318,
     319,     0,     0,  -311,     0,     0,     0,  -311,     0,     0,
     314,     0,     0,     0,  -311,     0,   315,   320,     0,   149,
       0,     0,     0,   336,   316,   317,   146,     0,     0,  -311,
     147,     0,     0,     0,   318,   319,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,   148,     0,     0,
       0,     0,   320,   321,     0,   149,     0,     0,   336,     0,
     150,     0,  -313,     0,     0,     0,   151,   152,   124,     0,
       0,   127,   128,   129,   130,  -157,     0,     0,   218,   134,
       0,     0,   135,   136,     0,     0,     0,     0,   321,     0,
       0,  -313,  -313,  -313,   153,     0,   141,   142,   143,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
    -159,   124,     0,   145,   127,   128,   129,   130,     0,     0,
       0,     0,  -311,     0,   154,   307,  -311,   308,     0,     0,
       0,     0,     0,  -311,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,   146,   -37,     0,  -311,   147,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,   148,     0,     0,     0,
       0,     0,     0,     0,   149,     0,     0,     0,     0,   150,
       0,  -313,     0,     0,     0,   151,     0,     0,     0,   145,
       0,     0,     0,     0,     0,     0,     0,     0,  -311,     0,
       0,     0,  -311,     0,     0,     0,     0,     0,     0,  -311,
    -313,  -313,  -313,     0,     0,     0,     0,     0,     0,     0,
       0,   146,     0,     0,  -311,   147,     0,     0,     0,     0,
       0,     0,   145,     0,   309,     0,     0,     0,   310,   311,
     312,     0,   148,   154,   313,     0,     0,     0,     0,   219,
     149,     0,     0,     0,     0,   150,     0,  -313,     0,     0,
       0,   151,     0,     0,     0,  -104,     0,     0,     0,     0,
       0,     0,     0,     0,     0,   220,   221,   222,   223,   224,
     225,   226,   227,     0,   314,     0,  -313,  -313,  -313,     0,
     315,     0,     0,   149,     0,     0,     0,     0,   316,   317,
       0,     0,     0,     0,     0,     0,     0,     0,   318,   319,
       0,     0,     0,     0,     0,     0,     0,     0,     0,   154,
       0,     0,     0,     0,     0,     0,   320,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,   -19,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,   321,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,  -111
};

static const yytype_int16 yycheck[] =
{
      49,    50,    51,    52,    53,    54,    55,    56,    57,    58,
      50,    51,   252,   615,     5,     6,   101,   617,   447,   269,
      95,    93,   622,    14,   264,     3,     4,     5,     6,     7,
       8,     9,     5,     6,    67,    68,   622,     0,   191,   641,
     642,   146,    81,    82,   241,   647,   243,   244,   270,   270,
     247,   248,   249,     3,     4,     5,     6,     7,     8,     9,
     257,   258,   259,    13,    55,   145,   121,   636,   637,   242,
      97,   640,   245,   246,   101,   297,   297,   296,   251,    99,
      99,   269,    55,   177,   272,   296,   158,   181,   261,   262,
     263,   214,   119,   216,   121,   202,   171,   204,   296,   119,
     119,   121,   121,   296,    24,   258,   259,    27,    28,    29,
      30,   540,   362,   363,   199,   365,   366,   270,    38,   719,
      40,   296,    56,    57,    58,    59,    60,    61,    62,    63,
      64,    65,    66,   719,   296,    69,    70,    71,    72,    73,
      74,    75,    76,    77,    78,    79,    56,    57,    58,    59,
      60,    61,    62,    63,    64,    65,    66,   184,   296,    69,
      70,    71,    72,    73,   269,    75,    76,   272,   146,    79,
     190,   190,   199,    99,   229,   269,   296,   296,   272,   419,
     270,   271,   202,   785,   204,   358,   359,   360,   361,   269,
     296,   431,   272,   119,   434,   121,   146,   154,   214,   213,
     216,   158,   229,   175,   253,   254,   255,   296,   165,   229,
     229,   183,   184,   453,   186,   455,   265,   266,   267,   268,
     460,   471,   472,   180,   297,   145,   476,   147,   242,   243,
     244,   151,   152,   153,   258,   259,   269,   157,   478,   272,
     409,   410,   411,   416,   417,   418,   270,    67,    68,   298,
     423,   424,   296,   450,   451,   452,   305,   297,   296,   456,
     433,   458,   459,   297,   190,   269,   270,   271,   272,   260,
     269,   270,   271,   272,   296,   448,   449,   197,   269,   296,
     202,   272,   204,   203,   296,   296,   206,   260,   202,   297,
     204,   211,   212,    11,    12,   297,   269,   297,   297,   272,
     297,   221,   222,   229,   477,    23,    24,    25,    26,    27,
      28,    29,    30,    31,    32,    33,   297,    35,   297,   239,
      38,    39,   297,    41,    42,    43,    44,   261,   262,   195,
     196,   174,   297,   176,    52,    53,    54,    55,    91,   182,
     270,   297,    95,   296,    97,   296,   270,   213,   101,   215,
     216,   261,   262,   296,   296,   275,   199,   200,   201,   202,
     409,   410,   411,   296,   207,   208,    18,    19,    20,    21,
      22,   420,   421,   422,   240,   296,   425,   297,   269,   270,
     271,   272,   297,   432,   296,   296,   435,   436,   437,   438,
     296,   296,   565,   566,   567,   568,   269,   270,   271,   272,
     269,   270,   271,   272,   296,   454,   297,   296,   457,   606,
     253,   258,   259,   269,   270,   271,   272,   296,   171,   296,
     296,   470,   269,   270,   297,   272,   292,   145,   297,   296,
     479,   184,   296,   296,   296,   208,   154,   610,   296,   212,
     158,   214,   296,   216,   296,   296,   199,   165,   206,   297,
     208,   296,   210,   297,   212,   296,   214,   296,   216,   177,
     633,   296,   180,   181,   296,   202,   296,   204,   296,   206,
     297,   208,   296,   210,   296,   212,   649,   214,   297,   216,
     198,    33,    34,    35,    36,    37,    38,    39,   206,   297,
      11,    12,   296,   211,   296,   213,   296,   296,   296,   217,
     218,   296,    23,    24,    25,    26,    27,    28,    29,    30,
      31,    32,    33,   562,    35,   296,   296,    38,    39,   296,
      41,    42,    43,    44,   242,   243,   244,   245,   296,   296,
     296,    52,    53,    54,    55,    70,    86,    72,    88,    74,
      90,    76,    92,    78,    94,    80,    96,    82,    98,   598,
     100,   296,   296,   296,   296,   296,   605,   275,   607,   608,
     609,   296,   611,   296,   296,   296,   615,    15,    16,    17,
      18,    19,    20,    21,    22,   296,   296,   296,   270,   297,
     830,   296,   296,   296,   633,   634,   635,   636,   637,   638,
     639,   640,   641,   642,   643,   644,   645,   646,   647,   648,
     296,   650,   651,    41,    42,    43,    44,    45,    46,    47,
      48,   296,   296,   296,   787,   788,   789,   296,   296,   296,
     296,   296,   296,   820,   145,   296,   296,   296,   296,   296,
     296,   296,   296,   154,   296,   875,   876,   158,   296,   296,
     296,   296,   296,   296,   165,    48,   297,   297,   821,   297,
     297,   700,   825,   826,   297,   297,   177,   297,   297,   180,
     181,   297,   297,   297,   297,   297,   297,   297,   297,   297,
     297,   297,   297,   297,   297,   297,   297,   198,   297,   297,
     297,   297,   297,   297,   296,   206,   296,   296,   296,   296,
     211,   297,   213,   297,   297,   297,   217,   218,   297,   296,
     296,   296,   296,   296,   296,   296,   296,   296,   296,   296,
     184,   297,   297,   297,   297,   297,   297,   297,   297,   297,
     297,   242,   243,   244,   245,   296,   296,   296,   296,   296,
     296,   296,   296,   296,   296,   296,   785,   296,   296,   296,
     296,   296,   296,   296,   276,   296,   268,    49,   297,   297,
     297,   297,   297,   297,   275,   297,   805,   806,   297,   297,
     297,   297,   199,   297,   297,    11,    12,   297,   297,   818,
     819,   297,   297,   822,   823,   824,   297,    23,    24,    25,
      26,    27,    28,    29,    30,    31,    32,    33,   837,   297,
      36,    37,    38,   297,    40,    41,   297,    43,    44,    45,
      46,    47,    11,    12,   297,   297,   297,   297,   297,    55,
     297,   297,   297,   297,    23,    24,    25,    26,    27,    28,
      29,    30,    31,    32,    33,   297,   297,    36,    37,    38,
     297,    40,    41,   297,    43,    44,    45,    46,    47,   297,
     297,   297,   297,   297,   297,   297,    55,   297,   297,   296,
      23,    24,    25,    26,    27,    28,    29,    30,    31,    32,
      33,   297,    35,   297,   297,    38,    39,   297,    41,    42,
      43,    44,   297,   297,   297,   297,   297,   297,   297,    52,
      53,    54,    55,   297,   297,   296,   296,   296,   769,   297,
     297,   297,   297,   297,   190,   297,   297,   297,   297,   145,
     297,   147,   297,   297,   297,   151,   152,   153,   297,   296,
     296,   157,   297,   297,   297,   297,   297,   297,   297,   297,
     297,   297,   296,   296,   296,   296,   296,   296,   296,   296,
     767,   297,   297,   296,   105,   297,   145,   297,   147,   297,
     297,   297,   151,   152,   153,   297,   297,   297,   157,   297,
     297,   197,   297,   297,   297,   297,   297,   203,   297,   296,
     206,   296,   296,   113,   297,   211,   212,   297,   111,   216,
      -1,   743,   145,   634,    -1,   221,   222,   837,   598,    -1,
      -1,   154,    -1,    -1,    -1,   158,   562,    -1,   197,    -1,
      -1,    -1,   165,   239,   203,    -1,    -1,   206,    -1,   245,
      -1,    -1,   211,   212,   177,    -1,    -1,   180,   181,    -1,
      -1,    -1,   221,   222,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,   198,    -1,    -1,    -1,   275,
     239,    -1,    -1,   206,    -1,    -1,   245,    -1,   211,    -1,
     213,    -1,    -1,    -1,   217,   218,    -1,    -1,    -1,    -1,
      -1,   297,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,   275,    -1,    -1,   242,
     243,   244,   245,    -1,    -1,    -1,    23,    24,    25,    26,
      27,    28,    29,    30,    31,    32,    33,    -1,   297,    36,
      37,    38,    -1,    40,    41,    -1,    43,    44,    45,    46,
      47,    -1,   275,    -1,    -1,    23,    24,    25,    26,    27,
      28,    29,    30,    31,    32,    33,    -1,    -1,    36,    37,
      38,    -1,    40,    41,   297,    43,    44,    45,    46,    47,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   145,    -1,
     147,    -1,    -1,    -1,   151,   152,   153,    -1,    -1,    -1,
     157,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,   145,    -1,   147,
      -1,    -1,    -1,   151,   152,   153,    -1,    -1,    -1,   157,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
     197,    -1,    -1,    -1,    -1,    -1,   203,    -1,    -1,   206,
      -1,    -1,    -1,    -1,   211,   212,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,   221,   222,    -1,    -1,    -1,   197,
      -1,    -1,    -1,    -1,    -1,   203,    -1,    -1,   206,    -1,
      -1,    -1,   239,   211,   212,    -1,    -1,    -1,   245,    -1,
      -1,    -1,    -1,   221,   222,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,   239,    -1,    -1,    -1,    -1,    -1,   245,   275,    -1,
      -1,    23,    24,    25,    26,    27,    28,    29,    30,    31,
      32,    33,    -1,    -1,    -1,    -1,    38,    -1,    40,    41,
     297,    43,    44,    45,    46,    47,    -1,   275,    -1,    -1,
      -1,    -1,    -1,    55,    -1,    -1,    23,    24,    25,    26,
      27,    28,    29,    30,    31,    32,    33,    -1,    -1,   297,
      -1,    38,    -1,    40,    41,    -1,    43,    44,    45,    46,
      47,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    55,    -1,
      -1,    -1,    23,    24,    25,    26,    27,    28,    29,    30,
      31,    32,    33,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      41,    42,    43,    44,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,   145,    -1,   147,    -1,    -1,    -1,   151,
     152,   153,    -1,    -1,    -1,   157,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   145,    -1,
     147,    -1,    -1,    -1,   151,   152,   153,    -1,    -1,    -1,
     157,    -1,    -1,    -1,    -1,   197,    -1,    -1,    -1,    -1,
      -1,   203,    -1,    -1,   206,    -1,    -1,    -1,    -1,   211,
     212,    -1,    24,    -1,   145,    27,    28,    29,    30,   221,
     222,    -1,    -1,   154,    -1,    -1,    -1,   158,    -1,    -1,
     197,    -1,    -1,    -1,   165,    -1,   203,   239,    -1,   206,
      -1,    -1,    -1,   245,   211,   212,   177,    -1,    -1,   180,
     181,    -1,    -1,    -1,   221,   222,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,   198,    -1,    -1,
      -1,    -1,   239,   275,    -1,   206,    -1,    -1,   245,    -1,
     211,    -1,   213,    -1,    -1,    -1,   217,   218,    24,    -1,
      -1,    27,    28,    29,    30,   297,    -1,    -1,    34,    35,
      -1,    -1,    38,    39,    -1,    -1,    -1,    -1,   275,    -1,
      -1,   242,   243,   244,   245,    -1,    52,    53,    54,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
     297,    24,    -1,   145,    27,    28,    29,    30,    -1,    -1,
      -1,    -1,   154,    -1,   275,    38,   158,    40,    -1,    -1,
      -1,    -1,    -1,   165,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,   177,   297,    -1,   180,   181,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,   198,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,   206,    -1,    -1,    -1,    -1,   211,
      -1,   213,    -1,    -1,    -1,   217,    -1,    -1,    -1,   145,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   154,    -1,
      -1,    -1,   158,    -1,    -1,    -1,    -1,    -1,    -1,   165,
     242,   243,   244,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,   177,    -1,    -1,   180,   181,    -1,    -1,    -1,    -1,
      -1,    -1,   145,    -1,   147,    -1,    -1,    -1,   151,   152,
     153,    -1,   198,   275,   157,    -1,    -1,    -1,    -1,   205,
     206,    -1,    -1,    -1,    -1,   211,    -1,   213,    -1,    -1,
      -1,   217,    -1,    -1,    -1,   297,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,   231,   232,   233,   234,   235,
     236,   237,   238,    -1,   197,    -1,   242,   243,   244,    -1,
     203,    -1,    -1,   206,    -1,    -1,    -1,    -1,   211,   212,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   221,   222,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   275,
      -1,    -1,    -1,    -1,    -1,    -1,   239,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,   297,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,   275,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,   297
};

/* YYSTOS[STATE-NUM] -- The symbol kind of the accessing symbol of
   state STATE-NUM.  */
static const yytype_int16 yystos[] =
{
       0,    81,    82,   301,   302,     3,     4,     5,     6,     7,
       8,     9,   146,   351,   358,   384,   389,     3,     4,     5,
       6,     7,     8,     9,    13,   146,   303,   310,   327,   332,
     346,     0,   296,   296,   296,   296,   296,   296,   296,   296,
     296,   296,   296,   296,   296,   296,   296,   296,   296,   454,
     454,   454,   454,   454,   454,   454,   454,   456,   456,   456,
     456,   456,   456,   456,   456,   456,   269,   272,   354,   416,
     352,   416,   376,   416,   359,   416,   378,   416,   361,   416,
     390,   416,   385,   416,   306,   416,   304,   416,   319,   320,
     311,   312,   321,   322,   313,   314,   333,   334,   347,   348,
     328,   329,   455,   355,   455,   353,   455,   377,   455,   360,
     455,   379,   455,   362,   455,   391,   455,   386,   457,   307,
     457,   305,   457,    23,    24,    25,    26,    27,    28,    29,
      30,    31,    32,    33,    35,    38,    39,    41,    42,    43,
      44,    52,    53,    54,    55,   145,   177,   181,   198,   206,
     211,   217,   218,   245,   275,   317,   318,   323,   324,   326,
     406,   408,   433,   434,   435,   440,   441,   442,   443,   457,
     315,   316,   318,   457,   323,   457,   315,   457,    11,    12,
      55,   318,   326,   335,   336,   341,   343,   397,   457,   349,
     350,   406,   435,   440,   457,    11,   318,   326,   330,   331,
     343,   297,   356,   297,   356,   297,   380,   297,   363,   297,
     380,   297,   363,   297,   392,   297,   387,   297,    34,   205,
     231,   232,   233,   234,   235,   236,   237,   238,   308,   309,
     326,   406,   407,   435,   440,   451,   297,   308,   297,   296,
     296,   296,   296,   296,   296,   296,   296,   296,   296,   296,
     296,   296,   296,   296,   296,   296,   296,   296,   296,   296,
     325,   296,   296,   296,   296,   296,   296,   296,   296,   296,
     323,   174,   176,   182,   199,   200,   201,   202,   207,   208,
     253,   438,   439,   154,   158,   165,   180,   445,   213,   242,
     243,   244,   447,   297,   315,   297,   297,   297,   296,   296,
     345,   335,   297,   349,   297,   296,   330,    38,    40,   147,
     151,   152,   153,   157,   197,   203,   211,   212,   221,   222,
     239,   275,   357,   383,   406,   433,   436,   437,   448,   449,
      41,    44,    45,    46,    47,    55,   245,   364,   381,   383,
     408,   437,   448,    36,    37,   364,   374,   375,    11,    55,
     364,   374,   393,   394,   397,   364,   388,   394,   296,   296,
     296,   296,   296,   296,   296,   296,   296,   296,   308,   270,
     420,   421,   258,   259,   270,   417,   418,   421,   418,   421,
     421,   418,   418,   421,   421,   421,   270,   422,   423,   418,
     423,   416,   416,   416,   270,   271,   416,   425,   421,   271,
     421,   424,   421,   296,   418,   418,   418,   423,   416,   177,
     181,   416,   444,   416,   416,   425,   296,   296,   296,   296,
     296,   296,   296,   296,   296,   296,   175,   183,   184,   186,
     446,   296,   296,   296,   296,   296,   296,   296,   296,     5,
       6,    14,    55,   260,   416,   270,   409,   296,   296,   296,
     296,   296,   296,   296,   296,   296,   296,   296,   296,   296,
     296,   296,   195,   196,   213,   215,   216,   240,   292,   452,
     296,   296,   296,   365,   366,   382,   296,   296,   296,   296,
     396,   418,   418,   418,   418,   297,   425,   297,   425,   270,
     297,   297,   425,   297,   425,   270,   297,   297,   297,   297,
     297,   297,   297,   297,   297,   297,   297,   297,   297,   297,
     297,   297,   297,   297,   297,   297,   297,   297,    56,    57,
      58,    59,    60,    61,    62,    63,    64,    65,    66,    69,
      70,    71,    72,    73,    75,    76,    79,   261,   262,   400,
     401,   297,   297,   297,   297,   297,   444,   444,   444,   297,
     297,   297,   297,   418,   418,   418,   423,   416,   416,   416,
     418,   418,   416,   429,   430,   296,   296,   296,   296,   423,
     416,   418,   423,   416,   146,   416,   432,   416,   431,   416,
     297,   297,   296,   297,   297,   344,   297,   400,   418,   418,
     421,   421,   421,   423,   145,   416,   423,   421,   416,   450,
     421,   421,   423,   414,   453,   296,   296,   296,   296,   296,
     296,   296,   416,   425,   425,   296,   296,   296,   425,   418,
     423,   416,   296,   297,   297,   297,   297,   297,   297,   297,
     297,   297,   297,   296,   296,   296,   296,   296,   296,   296,
     296,   296,   296,   296,   296,   296,   296,   296,   296,   296,
     296,   296,   297,   400,   297,   297,   297,   297,   297,   297,
     297,   297,   297,   430,   297,   418,   191,   418,   419,   418,
     418,   297,   297,   297,   297,   297,   297,   297,   297,   342,
     297,   297,   297,   297,   297,   297,   297,   297,   297,   297,
     297,   297,   450,   297,   297,   297,   297,   297,   276,   412,
     413,   415,   416,   421,   416,   416,   416,   418,   416,   297,
     297,   297,   429,    48,   367,    74,    77,    78,   398,   399,
     401,   297,   297,   297,   395,   398,   416,   418,   416,   426,
     428,   426,   427,   427,   427,   416,   416,   427,   429,   429,
      67,    68,   404,   405,   416,   416,   416,   416,   429,   416,
     418,   416,   402,   403,   416,   297,   297,   297,   297,    15,
      16,    17,    18,    19,    20,    21,    22,   337,   338,   339,
     340,   269,   270,   271,   410,   411,   416,   297,   297,   297,
     297,   297,   297,   297,   297,   296,   297,   296,   296,   296,
     297,   398,   297,   297,   297,   297,   297,   297,   297,   297,
     297,   297,   297,   297,   297,   296,   296,   297,   404,   297,
     297,   297,   297,   297,   297,   297,   297,   297,   268,   296,
     296,   296,   296,   296,   296,   296,   296,   338,   340,   297,
     296,   429,   418,   418,   418,   416,   416,   416,   416,   421,
     418,   416,   416,   416,   418,   418,   297,   425,   297,   297,
     297,   297,   297,   297,   403,   297,   297,   297,   297,   297,
     297,   297,   297,   297,   368,    49,   369,   370,   371,   296,
     372,    50,    51,   297,   373,   296,   296,   423,   423,   297,
     297
};

/* YYR1[RULE-NUM] -- Symbol kind of the left-hand side of rule RULE-NUM.  */
static const yytype_int16 yyr1[] =
{
       0,   300,   301,   302,   302,   302,   302,   302,   302,   302,
     302,   302,   303,   303,   305,   304,   307,   306,   308,   308,
     309,   309,   309,   309,   309,   309,   309,   309,   310,   310,
     310,   310,   312,   311,   314,   313,   315,   315,   316,   317,
     318,   318,   318,   318,   318,   318,   318,   318,   320,   319,
     322,   321,   323,   323,   324,   325,   324,   324,   326,   326,
     326,   326,   326,   326,   327,   329,   328,   330,   330,   331,
     331,   331,   332,   334,   333,   335,   335,   336,   336,   336,
     336,   337,   337,   337,   337,   337,   338,   338,   339,   339,
     339,   340,   340,   342,   341,   343,   344,   343,   345,   343,
     346,   348,   347,   349,   349,   350,   350,   350,   351,   351,
     353,   352,   355,   354,   356,   356,   357,   357,   357,   357,
     358,   358,   358,   358,   360,   359,   362,   361,   363,   363,
     364,   364,   364,   364,   365,   364,   366,   364,   364,   364,
     364,   364,   368,   367,   369,   369,   371,   370,   372,   372,
     373,   373,   374,   374,   375,   375,   377,   376,   379,   378,
     380,   380,   381,   382,   381,   383,   383,   384,   386,   385,
     387,   387,   388,   388,   389,   391,   390,   392,   392,   393,
     393,   394,   395,   394,   394,   396,   394,   397,   397,   397,
     397,   397,   398,   398,   399,   399,   399,   399,   400,   400,
     401,   401,   401,   401,   401,   401,   401,   401,   401,   401,
     401,   401,   401,   401,   401,   401,   401,   401,   401,   401,
     401,   401,   401,   402,   403,   403,   404,   404,   405,   405,
     406,   406,   406,   406,   406,   407,   407,   408,   408,   408,
     408,   408,   408,   408,   408,   409,   410,   410,   410,   410,
     411,   413,   412,   414,   415,   415,   416,   416,   417,   417,
     418,   418,   419,   419,   420,   421,   422,   423,   424,   424,
     425,   425,   425,   426,   427,   428,   429,   430,   430,   431,
     432,   432,   433,   433,   434,   435,   435,   436,   437,   437,
     437,   437,   438,   438,   438,   438,   438,   438,   438,   438,
     438,   438,   439,   438,   440,   440,   440,   440,   440,   441,
     440,   442,   440,   443,   440,   444,   444,   444,   444,   445,
     445,   445,   445,   446,   446,   446,   446,   447,   447,   447,
     447,   448,   448,   448,   448,   448,   448,   448,   448,   448,
     448,   448,   449,   448,   450,   450,   451,   451,   451,   451,
     451,   451,   451,   451,   451,   451,   451,   451,   452,   452,
     452,   452,   452,   452,   452,   453,   454,   455,   456,   457
};

/* YYR2[RULE-NUM] -- Number of symbols on the right-hand side of rule RULE-NUM.  */
static const yytype_int8 yyr2[] =
{
       0,     2,     1,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     6,     6,     0,     3,     0,     3,     2,     0,
       1,     1,     1,     1,     1,     4,     4,     4,     6,     6,
       6,     6,     0,     2,     0,     2,     2,     0,     1,     4,
       4,     4,     4,     4,     1,     1,     1,     1,     0,     2,
       0,     2,     2,     0,     1,     0,     5,     1,     4,     4,
       4,     4,     4,     4,     6,     0,     2,     2,     0,     1,
       1,     1,     6,     0,     2,     2,     0,     1,     1,     1,
       1,     4,     4,     4,     4,     4,     2,     0,     4,     4,
       4,     2,     1,     0,     8,     1,     0,     5,     0,     5,
       6,     0,     2,     2,     0,     1,     1,     1,     6,     6,
       0,     3,     0,     3,     2,     0,     1,     1,     1,     1,
       6,     6,     6,     6,     0,     3,     0,     3,     2,     0,
       4,     4,     4,     4,     0,     5,     0,     5,     1,     1,
       1,     1,     0,     6,     1,     0,     0,     5,     2,     0,
       4,     4,     4,     4,     1,     1,     0,     3,     0,     3,
       2,     0,     1,     0,     5,     4,     4,     6,     0,     3,
       2,     0,     1,     1,     6,     0,     3,     2,     0,     1,
       1,     1,     0,     5,     1,     0,     5,     4,     4,     4,
       4,     4,     2,     0,     1,     4,     4,     4,     2,     0,
       1,     4,     4,     4,     4,     4,     4,     4,     4,     4,
       4,     4,     4,     4,     4,     4,     4,     4,     4,     4,
       4,     4,     1,     1,     4,     0,     2,     0,     4,     4,
       4,     4,     4,     4,     4,     1,     4,     1,     4,     4,
       4,     4,     4,     4,     4,     1,     1,     4,     1,     1,
       1,     0,     2,     2,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     2,     0,     1,
       1,     1,     4,     4,     1,     1,     1,     1,     4,     4,
       1,     1,     4,     4,     4,     4,     4,     4,     4,     4,
       4,     4,     0,     2,     4,     4,     4,     4,     4,     0,
       2,     0,     2,     0,     2,     2,     2,     2,     0,     4,
       4,     4,     4,     4,     4,     4,     4,     4,     4,     4,
       4,     4,     4,     4,     4,     4,     4,     4,     4,     4,
       4,     4,     0,     2,     2,     0,     4,     3,     4,     3,
       4,     3,     4,     3,     4,     3,     4,     3,     4,     4,
       4,     4,     4,     4,     4,     0,     0,     0,     0,     0
};


enum { YYENOMEM = -2 };

#define yyerrok         (yyerrstatus = 0)
#define yyclearin       (yychar = AFSOCKET_EMPTY)

#define YYACCEPT        goto yyacceptlab
#define YYABORT         goto yyabortlab
#define YYERROR         goto yyerrorlab
#define YYNOMEM         goto yyexhaustedlab


#define YYRECOVERING()  (!!yyerrstatus)

#define YYBACKUP(Token, Value)                                    \
  do                                                              \
    if (yychar == AFSOCKET_EMPTY)                                        \
      {                                                           \
        yychar = (Token);                                         \
        yylval = (Value);                                         \
        YYPOPSTACK (yylen);                                       \
        yystate = *yyssp;                                         \
        goto yybackup;                                            \
      }                                                           \
    else                                                          \
      {                                                           \
        yyerror (&yylloc, lexer, instance, arg, YY_("syntax error: cannot back up")); \
        YYERROR;                                                  \
      }                                                           \
  while (0)

/* Backward compatibility with an undocumented macro.
   Use AFSOCKET_error or AFSOCKET_UNDEF. */
#define YYERRCODE AFSOCKET_UNDEF

/* YYLLOC_DEFAULT -- Set CURRENT to span from RHS[1] to RHS[N].
   If N is 0, then set CURRENT to the empty location which ends
   the previous symbol: RHS[0] (always defined).  */

#ifndef YYLLOC_DEFAULT
# define YYLLOC_DEFAULT(Current, Rhs, N)                                \
    do                                                                  \
      if (N)                                                            \
        {                                                               \
          (Current).first_line   = YYRHSLOC (Rhs, 1).first_line;        \
          (Current).first_column = YYRHSLOC (Rhs, 1).first_column;      \
          (Current).last_line    = YYRHSLOC (Rhs, N).last_line;         \
          (Current).last_column  = YYRHSLOC (Rhs, N).last_column;       \
        }                                                               \
      else                                                              \
        {                                                               \
          (Current).first_line   = (Current).last_line   =              \
            YYRHSLOC (Rhs, 0).last_line;                                \
          (Current).first_column = (Current).last_column =              \
            YYRHSLOC (Rhs, 0).last_column;                              \
        }                                                               \
    while (0)
#endif

#define YYRHSLOC(Rhs, K) ((Rhs)[K])


/* Enable debugging if requested.  */
#if AFSOCKET_DEBUG

# ifndef YYFPRINTF
#  include <stdio.h> /* INFRINGES ON USER NAME SPACE */
#  define YYFPRINTF fprintf
# endif

# define YYDPRINTF(Args)                        \
do {                                            \
  if (yydebug)                                  \
    YYFPRINTF Args;                             \
} while (0)


/* YYLOCATION_PRINT -- Print the location on the stream.
   This macro was not mandated originally: define only if we know
   we won't break user code: when these are the locations we know.  */

# ifndef YYLOCATION_PRINT

#  if defined YY_LOCATION_PRINT

   /* Temporary convenience wrapper in case some people defined the
      undocumented and private YY_LOCATION_PRINT macros.  */
#   define YYLOCATION_PRINT(File, Loc)  YY_LOCATION_PRINT(File, *(Loc))

#  elif defined AFSOCKET_LTYPE_IS_TRIVIAL && AFSOCKET_LTYPE_IS_TRIVIAL

/* Print *YYLOCP on YYO.  Private, do not rely on its existence. */

YY_ATTRIBUTE_UNUSED
static int
yy_location_print_ (FILE *yyo, YYLTYPE const * const yylocp)
{
  int res = 0;
  int end_col = 0 != yylocp->last_column ? yylocp->last_column - 1 : 0;
  if (0 <= yylocp->first_line)
    {
      res += YYFPRINTF (yyo, "%d", yylocp->first_line);
      if (0 <= yylocp->first_column)
        res += YYFPRINTF (yyo, ".%d", yylocp->first_column);
    }
  if (0 <= yylocp->last_line)
    {
      if (yylocp->first_line < yylocp->last_line)
        {
          res += YYFPRINTF (yyo, "-%d", yylocp->last_line);
          if (0 <= end_col)
            res += YYFPRINTF (yyo, ".%d", end_col);
        }
      else if (0 <= end_col && yylocp->first_column < end_col)
        res += YYFPRINTF (yyo, "-%d", end_col);
    }
  return res;
}

#   define YYLOCATION_PRINT  yy_location_print_

    /* Temporary convenience wrapper in case some people defined the
       undocumented and private YY_LOCATION_PRINT macros.  */
#   define YY_LOCATION_PRINT(File, Loc)  YYLOCATION_PRINT(File, &(Loc))

#  else

#   define YYLOCATION_PRINT(File, Loc) ((void) 0)
    /* Temporary convenience wrapper in case some people defined the
       undocumented and private YY_LOCATION_PRINT macros.  */
#   define YY_LOCATION_PRINT  YYLOCATION_PRINT

#  endif
# endif /* !defined YYLOCATION_PRINT */


# define YY_SYMBOL_PRINT(Title, Kind, Value, Location)                    \
do {                                                                      \
  if (yydebug)                                                            \
    {                                                                     \
      YYFPRINTF (stderr, "%s ", Title);                                   \
      yy_symbol_print (stderr,                                            \
                  Kind, Value, Location, lexer, instance, arg); \
      YYFPRINTF (stderr, "\n");                                           \
    }                                                                     \
} while (0)


/*-----------------------------------.
| Print this symbol's value on YYO.  |
`-----------------------------------*/

static void
yy_symbol_value_print (FILE *yyo,
                       yysymbol_kind_t yykind, YYSTYPE const * const yyvaluep, YYLTYPE const * const yylocationp, CfgLexer *lexer, LogDriver **instance, gpointer arg)
{
  FILE *yyoutput = yyo;
  YY_USE (yyoutput);
  YY_USE (yylocationp);
  YY_USE (lexer);
  YY_USE (instance);
  YY_USE (arg);
  if (!yyvaluep)
    return;
  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
  YY_USE (yykind);
  YY_IGNORE_MAYBE_UNINITIALIZED_END
}


/*---------------------------.
| Print this symbol on YYO.  |
`---------------------------*/

static void
yy_symbol_print (FILE *yyo,
                 yysymbol_kind_t yykind, YYSTYPE const * const yyvaluep, YYLTYPE const * const yylocationp, CfgLexer *lexer, LogDriver **instance, gpointer arg)
{
  YYFPRINTF (yyo, "%s %s (",
             yykind < YYNTOKENS ? "token" : "nterm", yysymbol_name (yykind));

  YYLOCATION_PRINT (yyo, yylocationp);
  YYFPRINTF (yyo, ": ");
  yy_symbol_value_print (yyo, yykind, yyvaluep, yylocationp, lexer, instance, arg);
  YYFPRINTF (yyo, ")");
}

/*------------------------------------------------------------------.
| yy_stack_print -- Print the state stack from its BOTTOM up to its |
| TOP (included).                                                   |
`------------------------------------------------------------------*/

static void
yy_stack_print (yy_state_t *yybottom, yy_state_t *yytop)
{
  YYFPRINTF (stderr, "Stack now");
  for (; yybottom <= yytop; yybottom++)
    {
      int yybot = *yybottom;
      YYFPRINTF (stderr, " %d", yybot);
    }
  YYFPRINTF (stderr, "\n");
}

# define YY_STACK_PRINT(Bottom, Top)                            \
do {                                                            \
  if (yydebug)                                                  \
    yy_stack_print ((Bottom), (Top));                           \
} while (0)


/*------------------------------------------------.
| Report that the YYRULE is going to be reduced.  |
`------------------------------------------------*/

static void
yy_reduce_print (yy_state_t *yyssp, YYSTYPE *yyvsp, YYLTYPE *yylsp,
                 int yyrule, CfgLexer *lexer, LogDriver **instance, gpointer arg)
{
  int yylno = yyrline[yyrule];
  int yynrhs = yyr2[yyrule];
  int yyi;
  YYFPRINTF (stderr, "Reducing stack by rule %d (line %d):\n",
             yyrule - 1, yylno);
  /* The symbols being reduced.  */
  for (yyi = 0; yyi < yynrhs; yyi++)
    {
      YYFPRINTF (stderr, "   $%d = ", yyi + 1);
      yy_symbol_print (stderr,
                       YY_ACCESSING_SYMBOL (+yyssp[yyi + 1 - yynrhs]),
                       &yyvsp[(yyi + 1) - (yynrhs)],
                       &(yylsp[(yyi + 1) - (yynrhs)]), lexer, instance, arg);
      YYFPRINTF (stderr, "\n");
    }
}

# define YY_REDUCE_PRINT(Rule)          \
do {                                    \
  if (yydebug)                          \
    yy_reduce_print (yyssp, yyvsp, yylsp, Rule, lexer, instance, arg); \
} while (0)

/* Nonzero means print parse trace.  It is left uninitialized so that
   multiple parsers can coexist.  */
int yydebug;
#else /* !AFSOCKET_DEBUG */
# define YYDPRINTF(Args) ((void) 0)
# define YY_SYMBOL_PRINT(Title, Kind, Value, Location)
# define YY_STACK_PRINT(Bottom, Top)
# define YY_REDUCE_PRINT(Rule)
#endif /* !AFSOCKET_DEBUG */


/* YYINITDEPTH -- initial size of the parser's stacks.  */
#ifndef YYINITDEPTH
# define YYINITDEPTH 200
#endif

/* YYMAXDEPTH -- maximum size the stacks can grow to (effective only
   if the built-in stack extension method is used).

   Do not make this value too large; the results are undefined if
   YYSTACK_ALLOC_MAXIMUM < YYSTACK_BYTES (YYMAXDEPTH)
   evaluated with infinite-precision integer arithmetic.  */

#ifndef YYMAXDEPTH
# define YYMAXDEPTH 10000
#endif


/* Context of a parse error.  */
typedef struct
{
  yy_state_t *yyssp;
  yysymbol_kind_t yytoken;
  YYLTYPE *yylloc;
} yypcontext_t;

/* Put in YYARG at most YYARGN of the expected tokens given the
   current YYCTX, and return the number of tokens stored in YYARG.  If
   YYARG is null, return the number of expected tokens (guaranteed to
   be less than YYNTOKENS).  Return YYENOMEM on memory exhaustion.
   Return 0 if there are more than YYARGN expected tokens, yet fill
   YYARG up to YYARGN. */
static int
yypcontext_expected_tokens (const yypcontext_t *yyctx,
                            yysymbol_kind_t yyarg[], int yyargn)
{
  /* Actual size of YYARG. */
  int yycount = 0;
  int yyn = yypact[+*yyctx->yyssp];
  if (!yypact_value_is_default (yyn))
    {
      /* Start YYX at -YYN if negative to avoid negative indexes in
         YYCHECK.  In other words, skip the first -YYN actions for
         this state because they are default actions.  */
      int yyxbegin = yyn < 0 ? -yyn : 0;
      /* Stay within bounds of both yycheck and yytname.  */
      int yychecklim = YYLAST - yyn + 1;
      int yyxend = yychecklim < YYNTOKENS ? yychecklim : YYNTOKENS;
      int yyx;
      for (yyx = yyxbegin; yyx < yyxend; ++yyx)
        if (yycheck[yyx + yyn] == yyx && yyx != YYSYMBOL_YYerror
            && !yytable_value_is_error (yytable[yyx + yyn]))
          {
            if (!yyarg)
              ++yycount;
            else if (yycount == yyargn)
              return 0;
            else
              yyarg[yycount++] = YY_CAST (yysymbol_kind_t, yyx);
          }
    }
  if (yyarg && yycount == 0 && 0 < yyargn)
    yyarg[0] = YYSYMBOL_YYEMPTY;
  return yycount;
}




#ifndef yystrlen
# if defined __GLIBC__ && defined _STRING_H
#  define yystrlen(S) (YY_CAST (YYPTRDIFF_T, strlen (S)))
# else
/* Return the length of YYSTR.  */
static YYPTRDIFF_T
yystrlen (const char *yystr)
{
  YYPTRDIFF_T yylen;
  for (yylen = 0; yystr[yylen]; yylen++)
    continue;
  return yylen;
}
# endif
#endif

#ifndef yystpcpy
# if defined __GLIBC__ && defined _STRING_H && defined _GNU_SOURCE
#  define yystpcpy stpcpy
# else
/* Copy YYSRC to YYDEST, returning the address of the terminating '\0' in
   YYDEST.  */
static char *
yystpcpy (char *yydest, const char *yysrc)
{
  char *yyd = yydest;
  const char *yys = yysrc;

  while ((*yyd++ = *yys++) != '\0')
    continue;

  return yyd - 1;
}
# endif
#endif

#ifndef yytnamerr
/* Copy to YYRES the contents of YYSTR after stripping away unnecessary
   quotes and backslashes, so that it's suitable for yyerror.  The
   heuristic is that double-quoting is unnecessary unless the string
   contains an apostrophe, a comma, or backslash (other than
   backslash-backslash).  YYSTR is taken from yytname.  If YYRES is
   null, do not copy; instead, return the length of what the result
   would have been.  */
static YYPTRDIFF_T
yytnamerr (char *yyres, const char *yystr)
{
  if (*yystr == '"')
    {
      YYPTRDIFF_T yyn = 0;
      char const *yyp = yystr;
      for (;;)
        switch (*++yyp)
          {
          case '\'':
          case ',':
            goto do_not_strip_quotes;

          case '\\':
            if (*++yyp != '\\')
              goto do_not_strip_quotes;
            else
              goto append;

          append:
          default:
            if (yyres)
              yyres[yyn] = *yyp;
            yyn++;
            break;

          case '"':
            if (yyres)
              yyres[yyn] = '\0';
            return yyn;
          }
    do_not_strip_quotes: ;
    }

  if (yyres)
    return yystpcpy (yyres, yystr) - yyres;
  else
    return yystrlen (yystr);
}
#endif


static int
yy_syntax_error_arguments (const yypcontext_t *yyctx,
                           yysymbol_kind_t yyarg[], int yyargn)
{
  /* Actual size of YYARG. */
  int yycount = 0;
  /* There are many possibilities here to consider:
     - If this state is a consistent state with a default action, then
       the only way this function was invoked is if the default action
       is an error action.  In that case, don't check for expected
       tokens because there are none.
     - The only way there can be no lookahead present (in yychar) is if
       this state is a consistent state with a default action.  Thus,
       detecting the absence of a lookahead is sufficient to determine
       that there is no unexpected or expected token to report.  In that
       case, just report a simple "syntax error".
     - Don't assume there isn't a lookahead just because this state is a
       consistent state with a default action.  There might have been a
       previous inconsistent state, consistent state with a non-default
       action, or user semantic action that manipulated yychar.
     - Of course, the expected token list depends on states to have
       correct lookahead information, and it depends on the parser not
       to perform extra reductions after fetching a lookahead from the
       scanner and before detecting a syntax error.  Thus, state merging
       (from LALR or IELR) and default reductions corrupt the expected
       token list.  However, the list is correct for canonical LR with
       one exception: it will still contain any token that will not be
       accepted due to an error action in a later state.
  */
  if (yyctx->yytoken != YYSYMBOL_YYEMPTY)
    {
      int yyn;
      if (yyarg)
        yyarg[yycount] = yyctx->yytoken;
      ++yycount;
      yyn = yypcontext_expected_tokens (yyctx,
                                        yyarg ? yyarg + 1 : yyarg, yyargn - 1);
      if (yyn == YYENOMEM)
        return YYENOMEM;
      else
        yycount += yyn;
    }
  return yycount;
}

/* Copy into *YYMSG, which is of size *YYMSG_ALLOC, an error message
   about the unexpected token YYTOKEN for the state stack whose top is
   YYSSP.

   Return 0 if *YYMSG was successfully written.  Return -1 if *YYMSG is
   not large enough to hold the message.  In that case, also set
   *YYMSG_ALLOC to the required number of bytes.  Return YYENOMEM if the
   required number of bytes is too large to store.  */
static int
yysyntax_error (YYPTRDIFF_T *yymsg_alloc, char **yymsg,
                const yypcontext_t *yyctx)
{
  enum { YYARGS_MAX = 5 };
  /* Internationalized format string. */
  const char *yyformat = YY_NULLPTR;
  /* Arguments of yyformat: reported tokens (one for the "unexpected",
     one per "expected"). */
  yysymbol_kind_t yyarg[YYARGS_MAX];
  /* Cumulated lengths of YYARG.  */
  YYPTRDIFF_T yysize = 0;

  /* Actual size of YYARG. */
  int yycount = yy_syntax_error_arguments (yyctx, yyarg, YYARGS_MAX);
  if (yycount == YYENOMEM)
    return YYENOMEM;

  switch (yycount)
    {
#define YYCASE_(N, S)                       \
      case N:                               \
        yyformat = S;                       \
        break
    default: /* Avoid compiler warnings. */
      YYCASE_(0, YY_("syntax error"));
      YYCASE_(1, YY_("syntax error, unexpected %s"));
      YYCASE_(2, YY_("syntax error, unexpected %s, expecting %s"));
      YYCASE_(3, YY_("syntax error, unexpected %s, expecting %s or %s"));
      YYCASE_(4, YY_("syntax error, unexpected %s, expecting %s or %s or %s"));
      YYCASE_(5, YY_("syntax error, unexpected %s, expecting %s or %s or %s or %s"));
#undef YYCASE_
    }

  /* Compute error message size.  Don't count the "%s"s, but reserve
     room for the terminator.  */
  yysize = yystrlen (yyformat) - 2 * yycount + 1;
  {
    int yyi;
    for (yyi = 0; yyi < yycount; ++yyi)
      {
        YYPTRDIFF_T yysize1
          = yysize + yytnamerr (YY_NULLPTR, yytname[yyarg[yyi]]);
        if (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM)
          yysize = yysize1;
        else
          return YYENOMEM;
      }
  }

  if (*yymsg_alloc < yysize)
    {
      *yymsg_alloc = 2 * yysize;
      if (! (yysize <= *yymsg_alloc
             && *yymsg_alloc <= YYSTACK_ALLOC_MAXIMUM))
        *yymsg_alloc = YYSTACK_ALLOC_MAXIMUM;
      return -1;
    }

  /* Avoid sprintf, as that infringes on the user's name space.
     Don't have undefined behavior even if the translation
     produced a string with the wrong number of "%s"s.  */
  {
    char *yyp = *yymsg;
    int yyi = 0;
    while ((*yyp = *yyformat) != '\0')
      if (*yyp == '%' && yyformat[1] == 's' && yyi < yycount)
        {
          yyp += yytnamerr (yyp, yytname[yyarg[yyi++]]);
          yyformat += 2;
        }
      else
        {
          ++yyp;
          ++yyformat;
        }
  }
  return 0;
}


/*-----------------------------------------------.
| Release the memory associated to this symbol.  |
`-----------------------------------------------*/

static void
yydestruct (const char *yymsg,
            yysymbol_kind_t yykind, YYSTYPE *yyvaluep, YYLTYPE *yylocationp, CfgLexer *lexer, LogDriver **instance, gpointer arg)
{
  YY_USE (yyvaluep);
  YY_USE (yylocationp);
  YY_USE (lexer);
  YY_USE (instance);
  YY_USE (arg);
  if (!yymsg)
    yymsg = "Deleting";
  YY_SYMBOL_PRINT (yymsg, yykind, yyvaluep, yylocationp);

  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
  switch (yykind)
    {
    case YYSYMBOL_LL_IDENTIFIER: /* LL_IDENTIFIER  */
#line 544 "modules/afsocket/afsocket-grammar.y"
            { free(((*yyvaluep).cptr)); }
#line 5299 "modules/afsocket/afsocket-grammar.c"
        break;

    case YYSYMBOL_LL_STRING: /* LL_STRING  */
#line 544 "modules/afsocket/afsocket-grammar.y"
            { free(((*yyvaluep).cptr)); }
#line 5305 "modules/afsocket/afsocket-grammar.c"
        break;

    case YYSYMBOL_LL_BLOCK: /* LL_BLOCK  */
#line 544 "modules/afsocket/afsocket-grammar.y"
            { free(((*yyvaluep).cptr)); }
#line 5311 "modules/afsocket/afsocket-grammar.c"
        break;

    case YYSYMBOL_LL_PLUGIN: /* LL_PLUGIN  */
#line 544 "modules/afsocket/afsocket-grammar.y"
            { free(((*yyvaluep).cptr)); }
#line 5317 "modules/afsocket/afsocket-grammar.c"
        break;

    case YYSYMBOL_LL_TEMPLATE_REF: /* LL_TEMPLATE_REF  */
#line 544 "modules/afsocket/afsocket-grammar.y"
            { free(((*yyvaluep).cptr)); }
#line 5323 "modules/afsocket/afsocket-grammar.c"
        break;

    case YYSYMBOL_LL_MESSAGE_REF: /* LL_MESSAGE_REF  */
#line 544 "modules/afsocket/afsocket-grammar.y"
            { free(((*yyvaluep).cptr)); }
#line 5329 "modules/afsocket/afsocket-grammar.c"
        break;

    case YYSYMBOL_string: /* string  */
#line 544 "modules/afsocket/afsocket-grammar.y"
            { free(((*yyvaluep).cptr)); }
#line 5335 "modules/afsocket/afsocket-grammar.c"
        break;

    case YYSYMBOL_string_or_number: /* string_or_number  */
#line 544 "modules/afsocket/afsocket-grammar.y"
            { free(((*yyvaluep).cptr)); }
#line 5341 "modules/afsocket/afsocket-grammar.c"
        break;

    case YYSYMBOL_path: /* path  */
#line 544 "modules/afsocket/afsocket-grammar.y"
            { free(((*yyvaluep).cptr)); }
#line 5347 "modules/afsocket/afsocket-grammar.c"
        break;

    case YYSYMBOL_path_check: /* path_check  */
#line 544 "modules/afsocket/afsocket-grammar.y"
            { free(((*yyvaluep).cptr)); }
#line 5353 "modules/afsocket/afsocket-grammar.c"
        break;

    case YYSYMBOL_path_secret: /* path_secret  */
#line 544 "modules/afsocket/afsocket-grammar.y"
            { free(((*yyvaluep).cptr)); }
#line 5359 "modules/afsocket/afsocket-grammar.c"
        break;

      default:
        break;
    }
  YY_IGNORE_MAYBE_UNINITIALIZED_END
}






/*----------.
| yyparse.  |
`----------*/

int
yyparse (CfgLexer *lexer, LogDriver **instance, gpointer arg)
{
/* Lookahead token kind.  */
int yychar;


/* The semantic value of the lookahead symbol.  */
/* Default value used for initialization, for pacifying older GCCs
   or non-GCC compilers.  */
YY_INITIAL_VALUE (static YYSTYPE yyval_default;)
YYSTYPE yylval YY_INITIAL_VALUE (= yyval_default);

/* Location data for the lookahead symbol.  */
static YYLTYPE yyloc_default
# if defined AFSOCKET_LTYPE_IS_TRIVIAL && AFSOCKET_LTYPE_IS_TRIVIAL
  = { 1, 1, 1, 1 }
# endif
;
YYLTYPE yylloc = yyloc_default;

    /* Number of syntax errors so far.  */
    int yynerrs = 0;

    yy_state_fast_t yystate = 0;
    /* Number of tokens to shift before error messages enabled.  */
    int yyerrstatus = 0;

    /* Refer to the stacks through separate pointers, to allow yyoverflow
       to reallocate them elsewhere.  */

    /* Their size.  */
    YYPTRDIFF_T yystacksize = YYINITDEPTH;

    /* The state stack: array, bottom, top.  */
    yy_state_t yyssa[YYINITDEPTH];
    yy_state_t *yyss = yyssa;
    yy_state_t *yyssp = yyss;

    /* The semantic value stack: array, bottom, top.  */
    YYSTYPE yyvsa[YYINITDEPTH];
    YYSTYPE *yyvs = yyvsa;
    YYSTYPE *yyvsp = yyvs;

    /* The location stack: array, bottom, top.  */
    YYLTYPE yylsa[YYINITDEPTH];
    YYLTYPE *yyls = yylsa;
    YYLTYPE *yylsp = yyls;

  int yyn;
  /* The return value of yyparse.  */
  int yyresult;
  /* Lookahead symbol kind.  */
  yysymbol_kind_t yytoken = YYSYMBOL_YYEMPTY;
  /* The variables used to return semantic value and location from the
     action routines.  */
  YYSTYPE yyval;
  YYLTYPE yyloc;

  /* The locations where the error started and ended.  */
  YYLTYPE yyerror_range[3];

  /* Buffer for error messages, and its allocated size.  */
  char yymsgbuf[128];
  char *yymsg = yymsgbuf;
  YYPTRDIFF_T yymsg_alloc = sizeof yymsgbuf;

#define YYPOPSTACK(N)   (yyvsp -= (N), yyssp -= (N), yylsp -= (N))

  /* The number of symbols on the RHS of the reduced rule.
     Keep to zero when no symbol should be popped.  */
  int yylen = 0;

  YYDPRINTF ((stderr, "Starting parse\n"));

  yychar = AFSOCKET_EMPTY; /* Cause a token to be read.  */

  yylsp[0] = yylloc;
  goto yysetstate;


/*------------------------------------------------------------.
| yynewstate -- push a new state, which is found in yystate.  |
`------------------------------------------------------------*/
yynewstate:
  /* In all cases, when you get here, the value and location stacks
     have just been pushed.  So pushing a state here evens the stacks.  */
  yyssp++;


/*--------------------------------------------------------------------.
| yysetstate -- set current state (the top of the stack) to yystate.  |
`--------------------------------------------------------------------*/
yysetstate:
  YYDPRINTF ((stderr, "Entering state %d\n", yystate));
  YY_ASSERT (0 <= yystate && yystate < YYNSTATES);
  YY_IGNORE_USELESS_CAST_BEGIN
  *yyssp = YY_CAST (yy_state_t, yystate);
  YY_IGNORE_USELESS_CAST_END
  YY_STACK_PRINT (yyss, yyssp);

  if (yyss + yystacksize - 1 <= yyssp)
#if !defined yyoverflow && !defined YYSTACK_RELOCATE
    YYNOMEM;
#else
    {
      /* Get the current used size of the three stacks, in elements.  */
      YYPTRDIFF_T yysize = yyssp - yyss + 1;

# if defined yyoverflow
      {
        /* Give user a chance to reallocate the stack.  Use copies of
           these so that the &'s don't force the real ones into
           memory.  */
        yy_state_t *yyss1 = yyss;
        YYSTYPE *yyvs1 = yyvs;
        YYLTYPE *yyls1 = yyls;

        /* Each stack pointer address is followed by the size of the
           data in use in that stack, in bytes.  This used to be a
           conditional around just the two extra args, but that might
           be undefined if yyoverflow is a macro.  */
        yyoverflow (YY_("memory exhausted"),
                    &yyss1, yysize * YYSIZEOF (*yyssp),
                    &yyvs1, yysize * YYSIZEOF (*yyvsp),
                    &yyls1, yysize * YYSIZEOF (*yylsp),
                    &yystacksize);
        yyss = yyss1;
        yyvs = yyvs1;
        yyls = yyls1;
      }
# else /* defined YYSTACK_RELOCATE */
      /* Extend the stack our own way.  */
      if (YYMAXDEPTH <= yystacksize)
        YYNOMEM;
      yystacksize *= 2;
      if (YYMAXDEPTH < yystacksize)
        yystacksize = YYMAXDEPTH;

      {
        yy_state_t *yyss1 = yyss;
        union yyalloc *yyptr =
          YY_CAST (union yyalloc *,
                   YYSTACK_ALLOC (YY_CAST (YYSIZE_T, YYSTACK_BYTES (yystacksize))));
        if (! yyptr)
          YYNOMEM;
        YYSTACK_RELOCATE (yyss_alloc, yyss);
        YYSTACK_RELOCATE (yyvs_alloc, yyvs);
        YYSTACK_RELOCATE (yyls_alloc, yyls);
#  undef YYSTACK_RELOCATE
        if (yyss1 != yyssa)
          YYSTACK_FREE (yyss1);
      }
# endif

      yyssp = yyss + yysize - 1;
      yyvsp = yyvs + yysize - 1;
      yylsp = yyls + yysize - 1;

      YY_IGNORE_USELESS_CAST_BEGIN
      YYDPRINTF ((stderr, "Stack size increased to %ld\n",
                  YY_CAST (long, yystacksize)));
      YY_IGNORE_USELESS_CAST_END

      if (yyss + yystacksize - 1 <= yyssp)
        YYABORT;
    }
#endif /* !defined yyoverflow && !defined YYSTACK_RELOCATE */


  if (yystate == YYFINAL)
    YYACCEPT;

  goto yybackup;


/*-----------.
| yybackup.  |
`-----------*/
yybackup:
  /* Do appropriate processing given the current state.  Read a
     lookahead token if we need one and don't already have one.  */

  /* First try to decide what to do without reference to lookahead token.  */
  yyn = yypact[yystate];
  if (yypact_value_is_default (yyn))
    goto yydefault;

  /* Not known => get a lookahead token if don't already have one.  */

  /* YYCHAR is either empty, or end-of-input, or a valid lookahead.  */
  if (yychar == AFSOCKET_EMPTY)
    {
      YYDPRINTF ((stderr, "Reading a token\n"));
      yychar = yylex (&yylval, &yylloc, lexer);
    }

  if (yychar <= AFSOCKET_EOF)
    {
      yychar = AFSOCKET_EOF;
      yytoken = YYSYMBOL_YYEOF;
      YYDPRINTF ((stderr, "Now at end of input.\n"));
    }
  else if (yychar == AFSOCKET_error)
    {
      /* The scanner already issued an error message, process directly
         to error recovery.  But do not keep the error token as
         lookahead, it is too special and may lead us to an endless
         loop in error recovery. */
      yychar = AFSOCKET_UNDEF;
      yytoken = YYSYMBOL_YYerror;
      yyerror_range[1] = yylloc;
      goto yyerrlab1;
    }
  else
    {
      yytoken = YYTRANSLATE (yychar);
      YY_SYMBOL_PRINT ("Next token is", yytoken, &yylval, &yylloc);
    }

  /* If the proper action on seeing token YYTOKEN is to reduce or to
     detect an error, take that action.  */
  yyn += yytoken;
  if (yyn < 0 || YYLAST < yyn || yycheck[yyn] != yytoken)
    goto yydefault;
  yyn = yytable[yyn];
  if (yyn <= 0)
    {
      if (yytable_value_is_error (yyn))
        goto yyerrlab;
      yyn = -yyn;
      goto yyreduce;
    }

  /* Count tokens shifted since error; after three, turn off error
     status.  */
  if (yyerrstatus)
    yyerrstatus--;

  /* Shift the lookahead token.  */
  YY_SYMBOL_PRINT ("Shifting", yytoken, &yylval, &yylloc);
  yystate = yyn;
  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
  *++yyvsp = yylval;
  YY_IGNORE_MAYBE_UNINITIALIZED_END
  *++yylsp = yylloc;

  /* Discard the shifted token.  */
  yychar = AFSOCKET_EMPTY;
  goto yynewstate;


/*-----------------------------------------------------------.
| yydefault -- do the default action for the current state.  |
`-----------------------------------------------------------*/
yydefault:
  yyn = yydefact[yystate];
  if (yyn == 0)
    goto yyerrlab;
  goto yyreduce;


/*-----------------------------.
| yyreduce -- do a reduction.  |
`-----------------------------*/
yyreduce:
  /* yyn is the number of a rule to reduce with.  */
  yylen = yyr2[yyn];

  /* If YYLEN is nonzero, implement the default value of the action:
     '$$ = $1'.

     Otherwise, the following line sets YYVAL to garbage.
     This behavior is undocumented and Bison
     users should not rely upon it.  Assigning to YYVAL
     unconditionally makes the parser a bit smaller, and it avoids a
     GCC warning that YYVAL may be used uninitialized.  */
  yyval = yyvsp[1-yylen];

  /* Default location. */
  YYLLOC_DEFAULT (yyloc, (yylsp - yylen), yylen);
  yyerror_range[1] = yyloc;
  YY_REDUCE_PRINT (yyn);
  switch (yyn)
    {
  case 2: /* start: driver  */
#line 685 "modules/afsocket/afsocket-grammar.y"
          {
            *instance = (yyvsp[0].ptr);
            if (yychar != AFSOCKET_EMPTY)
              cfg_lexer_unput_token(lexer, &yylval);
            YYACCEPT;
          }
#line 5670 "modules/afsocket/afsocket-grammar.c"
    break;

  case 3: /* driver: LL_CONTEXT_SOURCE source_afunix  */
#line 694 "modules/afsocket/afsocket-grammar.y"
                                                              { (yyval.ptr) = (yyvsp[0].ptr); }
#line 5676 "modules/afsocket/afsocket-grammar.c"
    break;

  case 4: /* driver: LL_CONTEXT_SOURCE source_afinet  */
#line 695 "modules/afsocket/afsocket-grammar.y"
                                                              { (yyval.ptr) = (yyvsp[0].ptr); }
#line 5682 "modules/afsocket/afsocket-grammar.c"
    break;

  case 5: /* driver: LL_CONTEXT_SOURCE source_afsyslog  */
#line 696 "modules/afsocket/afsocket-grammar.y"
                                                              { (yyval.ptr) = (yyvsp[0].ptr); }
#line 5688 "modules/afsocket/afsocket-grammar.c"
    break;

  case 6: /* driver: LL_CONTEXT_SOURCE source_afnetwork  */
#line 697 "modules/afsocket/afsocket-grammar.y"
                                                              { (yyval.ptr) = (yyvsp[0].ptr); }
#line 5694 "modules/afsocket/afsocket-grammar.c"
    break;

  case 7: /* driver: LL_CONTEXT_SOURCE source_systemd_syslog  */
#line 698 "modules/afsocket/afsocket-grammar.y"
                                                              { (yyval.ptr) = (yyvsp[0].ptr); }
#line 5700 "modules/afsocket/afsocket-grammar.c"
    break;

  case 8: /* driver: LL_CONTEXT_DESTINATION dest_afunix  */
#line 699 "modules/afsocket/afsocket-grammar.y"
                                                              { (yyval.ptr) = (yyvsp[0].ptr); }
#line 5706 "modules/afsocket/afsocket-grammar.c"
    break;

  case 9: /* driver: LL_CONTEXT_DESTINATION dest_afinet  */
#line 700 "modules/afsocket/afsocket-grammar.y"
                                                              { (yyval.ptr) = (yyvsp[0].ptr); }
#line 5712 "modules/afsocket/afsocket-grammar.c"
    break;

  case 10: /* driver: LL_CONTEXT_DESTINATION dest_afsyslog  */
#line 701 "modules/afsocket/afsocket-grammar.y"
                                                              { (yyval.ptr) = (yyvsp[0].ptr); }
#line 5718 "modules/afsocket/afsocket-grammar.c"
    break;

  case 11: /* driver: LL_CONTEXT_DESTINATION dest_afnetwork  */
#line 702 "modules/afsocket/afsocket-grammar.y"
                                                              { (yyval.ptr) = (yyvsp[0].ptr); }
#line 5724 "modules/afsocket/afsocket-grammar.c"
    break;

  case 12: /* source_afunix: KW_UNIX_DGRAM '(' _inner_src_context_push source_afunix_dgram_params _inner_src_context_pop ')'  */
#line 707 "modules/afsocket/afsocket-grammar.y"
                                                                                                                        { (yyval.ptr) = (yyvsp[-2].ptr); }
#line 5730 "modules/afsocket/afsocket-grammar.c"
    break;

  case 13: /* source_afunix: KW_UNIX_STREAM '(' _inner_src_context_push source_afunix_stream_params _inner_src_context_pop ')'  */
#line 708 "modules/afsocket/afsocket-grammar.y"
                                                                                                                        { (yyval.ptr) = (yyvsp[-2].ptr); }
#line 5736 "modules/afsocket/afsocket-grammar.c"
    break;

  case 14: /* $@1: %empty  */
#line 713 "modules/afsocket/afsocket-grammar.y"
          {
            create_and_set_unix_dgram_or_systemd_syslog_source((yyvsp[0].cptr), configuration);
	  }
#line 5744 "modules/afsocket/afsocket-grammar.c"
    break;

  case 15: /* source_afunix_dgram_params: string $@1 source_afunix_options  */
#line 716 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.ptr) = last_driver; free((yyvsp[-2].cptr)); }
#line 5750 "modules/afsocket/afsocket-grammar.c"
    break;

  case 16: /* $@2: %empty  */
#line 721 "modules/afsocket/afsocket-grammar.y"
          {
            create_and_set_unix_stream_or_systemd_syslog_source((yyvsp[0].cptr), configuration);
	  }
#line 5758 "modules/afsocket/afsocket-grammar.c"
    break;

  case 17: /* source_afunix_stream_params: string $@2 source_afunix_options  */
#line 724 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.ptr) = last_driver; free((yyvsp[-2].cptr)); }
#line 5764 "modules/afsocket/afsocket-grammar.c"
    break;

  case 21: /* source_afunix_option: source_afsocket_stream_params  */
#line 735 "modules/afsocket/afsocket-grammar.y"
                                                {}
#line 5770 "modules/afsocket/afsocket-grammar.c"
    break;

  case 22: /* source_afunix_option: source_reader_option  */
#line 736 "modules/afsocket/afsocket-grammar.y"
                                                {}
#line 5776 "modules/afsocket/afsocket-grammar.c"
    break;

  case 24: /* source_afunix_option: unix_socket_option  */
#line 738 "modules/afsocket/afsocket-grammar.y"
                                                {}
#line 5782 "modules/afsocket/afsocket-grammar.c"
    break;

  case 25: /* source_afunix_option: KW_OPTIONAL '(' yesno ')'  */
#line 739 "modules/afsocket/afsocket-grammar.y"
                                                { last_driver->optional = (yyvsp[-1].num); }
#line 5788 "modules/afsocket/afsocket-grammar.c"
    break;

  case 26: /* source_afunix_option: KW_PASS_UNIX_CREDENTIALS '(' yesno ')'  */
#line 741 "modules/afsocket/afsocket-grammar.y"
          {
	    AFUnixSourceDriver *self = (AFUnixSourceDriver*) last_driver;
	    afunix_sd_set_pass_unix_credentials(self, (yyvsp[-1].num));
	  }
#line 5797 "modules/afsocket/afsocket-grammar.c"
    break;

  case 27: /* source_afunix_option: KW_CREATE_DIRS '(' yesno ')'  */
#line 746 "modules/afsocket/afsocket-grammar.y"
          {
	    AFUnixSourceDriver *self = (AFUnixSourceDriver*) last_driver;
	    afunix_sd_set_create_dirs(self, (yyvsp[-1].num));
	  }
#line 5806 "modules/afsocket/afsocket-grammar.c"
    break;

  case 28: /* source_afinet: KW_UDP '(' _inner_src_context_push source_afinet_udp_params _inner_src_context_pop ')'  */
#line 753 "modules/afsocket/afsocket-grammar.y"
                                                                                                        { (yyval.ptr) = (yyvsp[-2].ptr); }
#line 5812 "modules/afsocket/afsocket-grammar.c"
    break;

  case 29: /* source_afinet: KW_TCP '(' _inner_src_context_push source_afinet_tcp_params _inner_src_context_pop ')'  */
#line 754 "modules/afsocket/afsocket-grammar.y"
                                                                                                        { (yyval.ptr) = (yyvsp[-2].ptr); }
#line 5818 "modules/afsocket/afsocket-grammar.c"
    break;

  case 30: /* source_afinet: KW_UDP6 '(' _inner_src_context_push source_afinet_udp6_params _inner_src_context_pop ')'  */
#line 755 "modules/afsocket/afsocket-grammar.y"
                                                                                                        { (yyval.ptr) = (yyvsp[-2].ptr); }
#line 5824 "modules/afsocket/afsocket-grammar.c"
    break;

  case 31: /* source_afinet: KW_TCP6 '(' _inner_src_context_push source_afinet_tcp6_params _inner_src_context_pop ')'  */
#line 756 "modules/afsocket/afsocket-grammar.y"
                                                                                                        { (yyval.ptr) = (yyvsp[-2].ptr); }
#line 5830 "modules/afsocket/afsocket-grammar.c"
    break;

  case 32: /* $@3: %empty  */
#line 761 "modules/afsocket/afsocket-grammar.y"
          {
            AFInetSourceDriver *d = afinet_sd_new_udp(configuration);

            afinet_grammar_set_source_driver(d);
	  }
#line 5840 "modules/afsocket/afsocket-grammar.c"
    break;

  case 33: /* source_afinet_udp_params: $@3 source_afinet_udp_options  */
#line 766 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.ptr) = last_driver; }
#line 5846 "modules/afsocket/afsocket-grammar.c"
    break;

  case 34: /* $@4: %empty  */
#line 771 "modules/afsocket/afsocket-grammar.y"
          {
            AFInetSourceDriver *d = afinet_sd_new_udp6(configuration);

            afinet_grammar_set_source_driver(d);
          }
#line 5856 "modules/afsocket/afsocket-grammar.c"
    break;

  case 35: /* source_afinet_udp6_params: $@4 source_afinet_udp_options  */
#line 776 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.ptr) = last_driver; }
#line 5862 "modules/afsocket/afsocket-grammar.c"
    break;

  case 39: /* source_afinet_multi_line_option: KW_MULTI_LINE_MODE '(' string ')'  */
#line 790 "modules/afsocket/afsocket-grammar.y"
                {
			CHECK_ERROR(multi_line_options_set_mode(last_multi_line_options, (yyvsp[-1].cptr)), (yylsp[-1]), "Invalid multi-line mode");
			CHECK_ERROR(last_multi_line_options->mode == MLM_EMPTY_LINE_SEPARATED, (yylsp[-1]), "Invalid multi-line mode, only 'empty-line-separated' supported for an AFInetSource yet");
			free((yyvsp[-1].cptr));
		}
#line 5872 "modules/afsocket/afsocket-grammar.c"
    break;

  case 40: /* source_afinet_option: KW_LOCALIP '(' string ')'  */
#line 798 "modules/afsocket/afsocket-grammar.y"
                                                { afinet_sd_set_localip(last_driver, (yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 5878 "modules/afsocket/afsocket-grammar.c"
    break;

  case 41: /* source_afinet_option: KW_IP '(' string ')'  */
#line 799 "modules/afsocket/afsocket-grammar.y"
                                                { afinet_sd_set_localip(last_driver, (yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 5884 "modules/afsocket/afsocket-grammar.c"
    break;

  case 42: /* source_afinet_option: KW_LOCALPORT '(' string_or_number ')'  */
#line 800 "modules/afsocket/afsocket-grammar.y"
                                                { CHECK_ERROR(cfg_check_port((yyvsp[-1].cptr)), (yylsp[-1]), "Illegal port number: %s", (yyvsp[-1].cptr)); afinet_sd_set_localport(last_driver, (yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 5890 "modules/afsocket/afsocket-grammar.c"
    break;

  case 43: /* source_afinet_option: KW_PORT '(' string_or_number ')'  */
#line 801 "modules/afsocket/afsocket-grammar.y"
                                           { CHECK_ERROR(cfg_check_port((yyvsp[-1].cptr)), (yylsp[-1]), "Illegal port number: %s", (yyvsp[-1].cptr)); afinet_sd_set_localport(last_driver, (yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 5896 "modules/afsocket/afsocket-grammar.c"
    break;

  case 48: /* $@5: %empty  */
#line 810 "modules/afsocket/afsocket-grammar.y"
          {
	    AFInetSourceDriver *d = afinet_sd_new_tcp(configuration);

	    afinet_grammar_set_source_driver(d);
	  }
#line 5906 "modules/afsocket/afsocket-grammar.c"
    break;

  case 49: /* source_afinet_tcp_params: $@5 source_afinet_tcp_options  */
#line 815 "modules/afsocket/afsocket-grammar.y"
                                        { (yyval.ptr) = last_driver; }
#line 5912 "modules/afsocket/afsocket-grammar.c"
    break;

  case 50: /* $@6: %empty  */
#line 820 "modules/afsocket/afsocket-grammar.y"
          {
	    AFInetSourceDriver *d = afinet_sd_new_tcp6(configuration);

	    afinet_grammar_set_source_driver(d);
	  }
#line 5922 "modules/afsocket/afsocket-grammar.c"
    break;

  case 51: /* source_afinet_tcp6_params: $@6 source_afinet_tcp_options  */
#line 825 "modules/afsocket/afsocket-grammar.y"
                                        { (yyval.ptr) = last_driver; }
#line 5928 "modules/afsocket/afsocket-grammar.c"
    break;

  case 55: /* $@7: %empty  */
#line 836 "modules/afsocket/afsocket-grammar.y"
          {
            gchar buf[256];

	    last_tls_context = tls_context_new(TM_SERVER, cfg_lexer_format_location(lexer, &(yylsp[0]), buf, sizeof(buf)));
	  }
#line 5938 "modules/afsocket/afsocket-grammar.c"
    break;

  case 56: /* source_afinet_tcp_option: KW_TLS $@7 '(' tls_options ')'  */
#line 842 "modules/afsocket/afsocket-grammar.y"
          {
	    afinet_sd_set_tls_context(last_driver, last_tls_context);
          }
#line 5946 "modules/afsocket/afsocket-grammar.c"
    break;

  case 57: /* source_afinet_tcp_option: source_afsocket_stream_params  */
#line 845 "modules/afsocket/afsocket-grammar.y"
                                                {}
#line 5952 "modules/afsocket/afsocket-grammar.c"
    break;

  case 58: /* source_afsocket_stream_params: KW_KEEP_ALIVE '(' yesno ')'  */
#line 849 "modules/afsocket/afsocket-grammar.y"
                                                { afsocket_sd_set_keep_alive(last_driver, (yyvsp[-1].num)); }
#line 5958 "modules/afsocket/afsocket-grammar.c"
    break;

  case 59: /* source_afsocket_stream_params: KW_MAX_CONNECTIONS '(' positive_integer ')'  */
#line 850 "modules/afsocket/afsocket-grammar.y"
                                                         { afsocket_sd_set_max_connections(last_driver, (yyvsp[-1].num)); }
#line 5964 "modules/afsocket/afsocket-grammar.c"
    break;

  case 60: /* source_afsocket_stream_params: KW_LISTEN_BACKLOG '(' positive_integer ')'  */
#line 851 "modules/afsocket/afsocket-grammar.y"
                                                        { afsocket_sd_set_listen_backlog(last_driver, (yyvsp[-1].num)); }
#line 5970 "modules/afsocket/afsocket-grammar.c"
    break;

  case 61: /* source_afsocket_stream_params: KW_DYNAMIC_WINDOW_SIZE '(' nonnegative_integer ')'  */
#line 852 "modules/afsocket/afsocket-grammar.y"
                                                             { afsocket_sd_set_dynamic_window_size(last_driver, (yyvsp[-1].num)); }
#line 5976 "modules/afsocket/afsocket-grammar.c"
    break;

  case 62: /* source_afsocket_stream_params: KW_DYNAMIC_WINDOW_STATS_FREQ '(' nonnegative_float ')'  */
#line 853 "modules/afsocket/afsocket-grammar.y"
                                                           { afsocket_sd_set_dynamic_window_stats_freq(last_driver, (yyvsp[-1].fnum)); }
#line 5982 "modules/afsocket/afsocket-grammar.c"
    break;

  case 63: /* source_afsocket_stream_params: KW_DYNAMIC_WINDOW_REALLOC_TICKS '(' nonnegative_integer ')'  */
#line 854 "modules/afsocket/afsocket-grammar.y"
                                                                { afsocket_sd_set_dynamic_window_realloc_ticks(last_driver, (yyvsp[-1].num)); }
#line 5988 "modules/afsocket/afsocket-grammar.c"
    break;

  case 64: /* source_afsyslog: KW_SYSLOG '(' _inner_src_context_push source_afsyslog_params _inner_src_context_pop ')'  */
#line 858 "modules/afsocket/afsocket-grammar.y"
                                                                                                        { (yyval.ptr) = (yyvsp[-2].ptr); }
#line 5994 "modules/afsocket/afsocket-grammar.c"
    break;

  case 65: /* $@8: %empty  */
#line 863 "modules/afsocket/afsocket-grammar.y"
          {
            /* we use transport(tcp) transport by default */
            AFInetSourceDriver *d = afinet_sd_new_syslog(configuration);

            afinet_grammar_set_source_driver(d);
	  }
#line 6005 "modules/afsocket/afsocket-grammar.c"
    break;

  case 66: /* source_afsyslog_params: $@8 source_afsyslog_options  */
#line 869 "modules/afsocket/afsocket-grammar.y"
                                        { (yyval.ptr) = last_driver; }
#line 6011 "modules/afsocket/afsocket-grammar.c"
    break;

  case 71: /* source_afsyslog_option: source_afsocket_stream_params  */
#line 880 "modules/afsocket/afsocket-grammar.y"
                                                {}
#line 6017 "modules/afsocket/afsocket-grammar.c"
    break;

  case 72: /* source_afnetwork: KW_NETWORK '(' _inner_src_context_push source_afnetwork_params _inner_src_context_pop ')'  */
#line 884 "modules/afsocket/afsocket-grammar.y"
                                                                                                       { (yyval.ptr) = (yyvsp[-2].ptr); }
#line 6023 "modules/afsocket/afsocket-grammar.c"
    break;

  case 73: /* $@9: %empty  */
#line 889 "modules/afsocket/afsocket-grammar.y"
          {
            /* we use transport(tcp) transport by default */
            AFInetSourceDriver *d = afinet_sd_new_network(configuration);

            afinet_grammar_set_source_driver(d);
	  }
#line 6034 "modules/afsocket/afsocket-grammar.c"
    break;

  case 74: /* source_afnetwork_params: $@9 source_afnetwork_options  */
#line 895 "modules/afsocket/afsocket-grammar.y"
                                        { (yyval.ptr) = last_driver; }
#line 6040 "modules/afsocket/afsocket-grammar.c"
    break;

  case 80: /* source_afnetwork_option: source_afsocket_stream_params  */
#line 907 "modules/afsocket/afsocket-grammar.y"
                                                {}
#line 6046 "modules/afsocket/afsocket-grammar.c"
    break;

  case 81: /* stats_option: KW_STAT_TYPE '(' string ')'  */
#line 911 "modules/afsocket/afsocket-grammar.y"
                                      { CHECK_ERROR(log_proto_http_scraper_responder_options_set_stat_type(last_proto_server_options, (yyvsp[-1].cptr)), (yylsp[-1]), "Invalid prometheus scraper stat type: %s", (yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 6052 "modules/afsocket/afsocket-grammar.c"
    break;

  case 82: /* stats_option: KW_STAT_QUERY '(' string ')'  */
#line 912 "modules/afsocket/afsocket-grammar.y"
                                       { log_proto_http_scraper_responder_options_set_stat_query(last_proto_server_options, (yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 6058 "modules/afsocket/afsocket-grammar.c"
    break;

  case 83: /* stats_option: KW_STAT_FORMAT '(' string ')'  */
#line 913 "modules/afsocket/afsocket-grammar.y"
                                        { CHECK_ERROR(log_proto_http_scraper_responder_options_set_stat_format(last_proto_server_options, (yyvsp[-1].cptr)), (yylsp[-1]), "Invalid prometheus scraper stat format: %s", (yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 6064 "modules/afsocket/afsocket-grammar.c"
    break;

  case 84: /* stats_option: KW_STATS_WITHOUT_ORPHANED '(' yesno ')'  */
#line 914 "modules/afsocket/afsocket-grammar.y"
                                                  { log_proto_http_scraper_responder_options_set_stats_without_orpaned(last_proto_server_options, (yyvsp[-1].num)); }
#line 6070 "modules/afsocket/afsocket-grammar.c"
    break;

  case 85: /* stats_option: KW_STATS_WITH_LEGACY '(' yesno ')'  */
#line 915 "modules/afsocket/afsocket-grammar.y"
                                             { log_proto_http_scraper_responder_options_set_stats_with_legacy(last_proto_server_options, (yyvsp[-1].num)); }
#line 6076 "modules/afsocket/afsocket-grammar.c"
    break;

  case 88: /* proto_http_scraper_option: KW_SCRAPE_PATTERN '(' string ')'  */
#line 924 "modules/afsocket/afsocket-grammar.y"
                                           { log_proto_http_scraper_responder_options_set_scrape_pattern(last_proto_server_options, (yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 6082 "modules/afsocket/afsocket-grammar.c"
    break;

  case 89: /* proto_http_scraper_option: KW_SCRAPE_FREQ_LIMIT '(' nonnegative_integer ')'  */
#line 925 "modules/afsocket/afsocket-grammar.y"
                                                           { log_proto_http_scraper_responder_options_set_scrape_freq_limit(last_proto_server_options, (yyvsp[-1].num)); }
#line 6088 "modules/afsocket/afsocket-grammar.c"
    break;

  case 90: /* proto_http_scraper_option: KW_SINGLE_INSTANCE '(' yesno ')'  */
#line 926 "modules/afsocket/afsocket-grammar.y"
                                           { log_proto_http_scraper_responder_options_set_single_instance(last_proto_server_options, (yyvsp[-1].num)); }
#line 6094 "modules/afsocket/afsocket-grammar.c"
    break;

  case 93: /* $@10: %empty  */
#line 937 "modules/afsocket/afsocket-grammar.y"
                                {
					transport_mapper_set_transport(last_transport_mapper, "http-scraper");
					log_proto_http_scraper_responder_options_defaults(last_proto_server_options);
				}
#line 6103 "modules/afsocket/afsocket-grammar.c"
    break;

  case 96: /* $@11: %empty  */
#line 949 "modules/afsocket/afsocket-grammar.y"
                {
			Plugin *p;
			gint context = LL_CONTEXT_SERVER_PROTO;

			p = cfg_find_plugin(configuration, context, (yyvsp[0].cptr));
			if (p)
				{
					/* for transports with grammar */
					if (p->parser)
						{
							LogProtoServerFactory *sf = cfg_parse_plugin(configuration, p, &(yylsp[0]), last_proto_server_options);
							((AFSocketSourceDriver *) last_driver)->proto_factory = sf;
						}
				}
			transport_mapper_set_transport(last_transport_mapper, (yyvsp[0].cptr));
		}
#line 6124 "modules/afsocket/afsocket-grammar.c"
    break;

  case 97: /* source_afsocket_transport: KW_TRANSPORT '(' string $@11 ')'  */
#line 965 "modules/afsocket/afsocket-grammar.y"
                { free((yyvsp[-2].cptr)); }
#line 6130 "modules/afsocket/afsocket-grammar.c"
    break;

  case 98: /* $@12: %empty  */
#line 967 "modules/afsocket/afsocket-grammar.y"
          {
			gchar buf[256];

	    last_tls_context = tls_context_new(TM_SERVER, cfg_lexer_format_location(lexer, &(yylsp[0]), buf, sizeof(buf)));
	  }
#line 6140 "modules/afsocket/afsocket-grammar.c"
    break;

  case 99: /* source_afsocket_transport: KW_TLS $@12 '(' tls_options ')'  */
#line 973 "modules/afsocket/afsocket-grammar.y"
          {
	    afinet_sd_set_tls_context(last_driver, last_tls_context);
    }
#line 6148 "modules/afsocket/afsocket-grammar.c"
    break;

  case 100: /* source_systemd_syslog: KW_SYSTEMD_SYSLOG '(' _inner_src_context_push source_systemd_syslog_params _inner_src_context_pop ')'  */
#line 979 "modules/afsocket/afsocket-grammar.y"
                                                                                                                 { (yyval.ptr) = (yyvsp[-2].ptr); }
#line 6154 "modules/afsocket/afsocket-grammar.c"
    break;

  case 101: /* $@13: %empty  */
#line 983 "modules/afsocket/afsocket-grammar.y"
          {
#if ! SYSLOG_NG_ENABLE_SYSTEMD
            msg_error("systemd-syslog() source cannot be enabled and it is not"
                      " functioning. Please compile your syslog-ng with --enable-systemd"
                      " flag", cfg_lexer_format_location_tag(lexer, &(yylsp[0])));
            YYERROR;
#else
            SystemDSyslogSourceDriver *d = systemd_syslog_sd_new(configuration, FALSE);
            systemd_syslog_grammar_set_source_driver(d);
#endif
          }
#line 6170 "modules/afsocket/afsocket-grammar.c"
    break;

  case 102: /* source_systemd_syslog_params: $@13 source_systemd_syslog_options  */
#line 994 "modules/afsocket/afsocket-grammar.y"
                                        { (yyval.ptr) = last_driver; }
#line 6176 "modules/afsocket/afsocket-grammar.c"
    break;

  case 108: /* dest_afunix: KW_UNIX_DGRAM '(' _inner_dest_context_push dest_afunix_dgram_params _inner_dest_context_pop ')'  */
#line 1009 "modules/afsocket/afsocket-grammar.y"
                                                                                                                { (yyval.ptr) = (yyvsp[-2].ptr); }
#line 6182 "modules/afsocket/afsocket-grammar.c"
    break;

  case 109: /* dest_afunix: KW_UNIX_STREAM '(' _inner_dest_context_push dest_afunix_stream_params _inner_dest_context_pop ')'  */
#line 1010 "modules/afsocket/afsocket-grammar.y"
                                                                                                                { (yyval.ptr) = (yyvsp[-2].ptr); }
#line 6188 "modules/afsocket/afsocket-grammar.c"
    break;

  case 110: /* $@14: %empty  */
#line 1015 "modules/afsocket/afsocket-grammar.y"
          {
	    AFUnixDestDriver *d = afunix_dd_new_dgram((yyvsp[0].cptr), configuration);

      afunix_grammar_set_dest_driver(d);
	  }
#line 6198 "modules/afsocket/afsocket-grammar.c"
    break;

  case 111: /* dest_afunix_dgram_params: string $@14 dest_afunix_options  */
#line 1020 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.ptr) = last_driver; free((yyvsp[-2].cptr)); }
#line 6204 "modules/afsocket/afsocket-grammar.c"
    break;

  case 112: /* $@15: %empty  */
#line 1025 "modules/afsocket/afsocket-grammar.y"
          {
	    AFUnixDestDriver *d = afunix_dd_new_stream((yyvsp[0].cptr), configuration);

      afunix_grammar_set_dest_driver(d);
	  }
#line 6214 "modules/afsocket/afsocket-grammar.c"
    break;

  case 113: /* dest_afunix_stream_params: string $@15 dest_afunix_options  */
#line 1030 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.ptr) = last_driver; free((yyvsp[-2].cptr)); }
#line 6220 "modules/afsocket/afsocket-grammar.c"
    break;

  case 120: /* dest_afinet: KW_UDP '(' _inner_dest_context_push dest_afinet_udp_params _inner_dest_context_pop ')'  */
#line 1046 "modules/afsocket/afsocket-grammar.y"
                                                                                                                        { (yyval.ptr) = (yyvsp[-2].ptr); }
#line 6226 "modules/afsocket/afsocket-grammar.c"
    break;

  case 121: /* dest_afinet: KW_TCP '(' _inner_dest_context_push dest_afinet_tcp_params _inner_dest_context_pop ')'  */
#line 1047 "modules/afsocket/afsocket-grammar.y"
                                                                                                                        { (yyval.ptr) = (yyvsp[-2].ptr); }
#line 6232 "modules/afsocket/afsocket-grammar.c"
    break;

  case 122: /* dest_afinet: KW_UDP6 '(' _inner_dest_context_push dest_afinet_udp6_params _inner_dest_context_pop ')'  */
#line 1048 "modules/afsocket/afsocket-grammar.y"
                                                                                                                { (yyval.ptr) = (yyvsp[-2].ptr); }
#line 6238 "modules/afsocket/afsocket-grammar.c"
    break;

  case 123: /* dest_afinet: KW_TCP6 '(' _inner_dest_context_push dest_afinet_tcp6_params _inner_dest_context_pop ')'  */
#line 1049 "modules/afsocket/afsocket-grammar.y"
                                                                                                                { (yyval.ptr) = (yyvsp[-2].ptr); }
#line 6244 "modules/afsocket/afsocket-grammar.c"
    break;

  case 124: /* $@16: %empty  */
#line 1054 "modules/afsocket/afsocket-grammar.y"
          {
	    AFInetDestDriver *d = afinet_dd_new_udp((yyvsp[0].cptr), configuration);

      afinet_grammar_set_dest_driver(d);
	  }
#line 6254 "modules/afsocket/afsocket-grammar.c"
    break;

  case 125: /* dest_afinet_udp_params: string $@16 dest_afinet_udp_options  */
#line 1059 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.ptr) = last_driver; free((yyvsp[-2].cptr)); }
#line 6260 "modules/afsocket/afsocket-grammar.c"
    break;

  case 126: /* $@17: %empty  */
#line 1064 "modules/afsocket/afsocket-grammar.y"
          {
	    AFInetDestDriver *d = afinet_dd_new_udp6((yyvsp[0].cptr), configuration);

	    afinet_grammar_set_dest_driver(d);
	  }
#line 6270 "modules/afsocket/afsocket-grammar.c"
    break;

  case 127: /* dest_afinet_udp6_params: string $@17 dest_afinet_udp_options  */
#line 1069 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.ptr) = last_driver; free((yyvsp[-2].cptr)); }
#line 6276 "modules/afsocket/afsocket-grammar.c"
    break;

  case 130: /* dest_afinet_option: KW_LOCALIP '(' string ')'  */
#line 1080 "modules/afsocket/afsocket-grammar.y"
                                                { afinet_dd_set_localip(last_driver, (yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 6282 "modules/afsocket/afsocket-grammar.c"
    break;

  case 131: /* dest_afinet_option: KW_LOCALPORT '(' string_or_number ')'  */
#line 1081 "modules/afsocket/afsocket-grammar.y"
                                                { CHECK_ERROR(cfg_check_port((yyvsp[-1].cptr)), (yylsp[-1]), "Illegal port number: %s", (yyvsp[-1].cptr)); afinet_dd_set_localport(last_driver, (yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 6288 "modules/afsocket/afsocket-grammar.c"
    break;

  case 132: /* dest_afinet_option: KW_PORT '(' string_or_number ')'  */
#line 1082 "modules/afsocket/afsocket-grammar.y"
                                           { CHECK_ERROR(cfg_check_port((yyvsp[-1].cptr)), (yylsp[-1]), "Illegal port number: %s", (yyvsp[-1].cptr)); afinet_dd_set_destport(last_driver, (yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 6294 "modules/afsocket/afsocket-grammar.c"
    break;

  case 133: /* dest_afinet_option: KW_DESTPORT '(' string_or_number ')'  */
#line 1083 "modules/afsocket/afsocket-grammar.y"
                                               { CHECK_ERROR(cfg_check_port((yyvsp[-1].cptr)), (yylsp[-1]), "Illegal port number: %s", (yyvsp[-1].cptr)); afinet_dd_set_destport(last_driver, (yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 6300 "modules/afsocket/afsocket-grammar.c"
    break;

  case 134: /* $@18: %empty  */
#line 1084 "modules/afsocket/afsocket-grammar.y"
                              { afinet_dd_enable_failover(last_driver); }
#line 6306 "modules/afsocket/afsocket-grammar.c"
    break;

  case 135: /* dest_afinet_option: KW_FAILOVER_SERVERS $@18 '(' string_list ')'  */
#line 1084 "modules/afsocket/afsocket-grammar.y"
                                                                                                { afinet_dd_add_failovers(last_driver, (yyvsp[-1].ptr)); }
#line 6312 "modules/afsocket/afsocket-grammar.c"
    break;

  case 136: /* $@19: %empty  */
#line 1085 "modules/afsocket/afsocket-grammar.y"
                      { afinet_dd_enable_failover(last_driver); }
#line 6318 "modules/afsocket/afsocket-grammar.c"
    break;

  case 137: /* dest_afinet_option: KW_FAILOVER $@19 '(' dest_failover_options ')'  */
#line 1085 "modules/afsocket/afsocket-grammar.y"
                                                                                                { (yyval.ptr) = (yyvsp[-1].ptr); }
#line 6324 "modules/afsocket/afsocket-grammar.c"
    break;

  case 142: /* $@20: %empty  */
#line 1093 "modules/afsocket/afsocket-grammar.y"
                                         { afinet_dd_add_failovers(last_driver, (yyvsp[-1].ptr)); }
#line 6330 "modules/afsocket/afsocket-grammar.c"
    break;

  case 146: /* $@21: %empty  */
#line 1102 "modules/afsocket/afsocket-grammar.y"
                { afinet_dd_enable_failback(last_driver); }
#line 6336 "modules/afsocket/afsocket-grammar.c"
    break;

  case 150: /* dest_failback_probe_option: KW_TCP_PROBE_INTERVAL '(' positive_integer ')'  */
#line 1111 "modules/afsocket/afsocket-grammar.y"
                                                         { afinet_dd_set_failback_tcp_probe_interval(last_driver, (yyvsp[-1].num)); }
#line 6342 "modules/afsocket/afsocket-grammar.c"
    break;

  case 151: /* dest_failback_probe_option: KW_SUCCESSFUL_PROBES_REQUIRED '(' positive_integer ')'  */
#line 1112 "modules/afsocket/afsocket-grammar.y"
                                                                 { afinet_dd_set_failback_successful_probes_required(last_driver, (yyvsp[-1].num)); }
#line 6348 "modules/afsocket/afsocket-grammar.c"
    break;

  case 152: /* dest_afinet_dgram_option: KW_SPOOF_SOURCE '(' yesno ')'  */
#line 1116 "modules/afsocket/afsocket-grammar.y"
                                                { afinet_dd_set_spoof_source(last_driver, (yyvsp[-1].num)); }
#line 6354 "modules/afsocket/afsocket-grammar.c"
    break;

  case 153: /* dest_afinet_dgram_option: KW_SPOOF_SOURCE_MAX_MSGLEN '(' positive_integer ')'  */
#line 1117 "modules/afsocket/afsocket-grammar.y"
                                                              { afinet_dd_set_spoof_source_max_msglen(last_driver, (yyvsp[-1].num)); }
#line 6360 "modules/afsocket/afsocket-grammar.c"
    break;

  case 156: /* $@22: %empty  */
#line 1127 "modules/afsocket/afsocket-grammar.y"
          {
	    AFInetDestDriver *d = afinet_dd_new_tcp((yyvsp[0].cptr), configuration);

      afinet_grammar_set_dest_driver(d);
	  }
#line 6370 "modules/afsocket/afsocket-grammar.c"
    break;

  case 157: /* dest_afinet_tcp_params: string $@22 dest_afinet_tcp_options  */
#line 1132 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.ptr) = last_driver; free((yyvsp[-2].cptr)); }
#line 6376 "modules/afsocket/afsocket-grammar.c"
    break;

  case 158: /* $@23: %empty  */
#line 1137 "modules/afsocket/afsocket-grammar.y"
          {
	    AFInetDestDriver *d = afinet_dd_new_tcp6((yyvsp[0].cptr), configuration);

	    afinet_grammar_set_dest_driver(d);
	  }
#line 6386 "modules/afsocket/afsocket-grammar.c"
    break;

  case 159: /* dest_afinet_tcp6_params: string $@23 dest_afinet_tcp_options  */
#line 1142 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.ptr) = last_driver; free((yyvsp[-2].cptr)); }
#line 6392 "modules/afsocket/afsocket-grammar.c"
    break;

  case 163: /* $@24: %empty  */
#line 1153 "modules/afsocket/afsocket-grammar.y"
          {
            gchar buf[256];
	    last_tls_context = tls_context_new(TM_CLIENT, cfg_lexer_format_location(lexer, &(yylsp[0]), buf, sizeof(buf)));
	  }
#line 6401 "modules/afsocket/afsocket-grammar.c"
    break;

  case 164: /* dest_afinet_tcp_option: KW_TLS $@24 '(' dest_tls_options ')'  */
#line 1158 "modules/afsocket/afsocket-grammar.y"
          {
	    afinet_dd_set_tls_context(last_driver, last_tls_context);
          }
#line 6409 "modules/afsocket/afsocket-grammar.c"
    break;

  case 165: /* dest_afsocket_option: KW_KEEP_ALIVE '(' yesno ')'  */
#line 1164 "modules/afsocket/afsocket-grammar.y"
                                             { afsocket_dd_set_keep_alive(last_driver, (yyvsp[-1].num)); }
#line 6415 "modules/afsocket/afsocket-grammar.c"
    break;

  case 166: /* dest_afsocket_option: KW_CLOSE_ON_INPUT '(' yesno ')'  */
#line 1166 "modules/afsocket/afsocket-grammar.y"
          {
            log_proto_client_options_set_drop_input(last_proto_client_options, !(yyvsp[-1].num));
          }
#line 6423 "modules/afsocket/afsocket-grammar.c"
    break;

  case 167: /* dest_afsyslog: KW_SYSLOG '(' _inner_dest_context_push dest_afsyslog_params _inner_dest_context_pop ')'  */
#line 1173 "modules/afsocket/afsocket-grammar.y"
                                                                                                    { (yyval.ptr) = (yyvsp[-2].ptr); }
#line 6429 "modules/afsocket/afsocket-grammar.c"
    break;

  case 168: /* $@25: %empty  */
#line 1177 "modules/afsocket/afsocket-grammar.y"
          {
            AFInetDestDriver *d = afinet_dd_new_syslog((yyvsp[0].cptr), configuration);

            afinet_grammar_set_dest_driver(d);
	  }
#line 6439 "modules/afsocket/afsocket-grammar.c"
    break;

  case 169: /* dest_afsyslog_params: string $@25 dest_afsyslog_options  */
#line 1182 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.ptr) = last_driver; free((yyvsp[-2].cptr)); }
#line 6445 "modules/afsocket/afsocket-grammar.c"
    break;

  case 174: /* dest_afnetwork: KW_NETWORK '(' _inner_dest_context_push dest_afnetwork_params _inner_dest_context_pop ')'  */
#line 1197 "modules/afsocket/afsocket-grammar.y"
                                                                                                        { (yyval.ptr) = (yyvsp[-2].ptr); }
#line 6451 "modules/afsocket/afsocket-grammar.c"
    break;

  case 175: /* $@26: %empty  */
#line 1202 "modules/afsocket/afsocket-grammar.y"
          {
            AFInetDestDriver *d = afinet_dd_new_network((yyvsp[0].cptr), configuration);

            afinet_grammar_set_dest_driver(d);
	  }
#line 6461 "modules/afsocket/afsocket-grammar.c"
    break;

  case 176: /* dest_afnetwork_params: string $@26 dest_afnetwork_options  */
#line 1207 "modules/afsocket/afsocket-grammar.y"
                                                        { (yyval.ptr) = last_driver; free((yyvsp[-2].cptr)); }
#line 6467 "modules/afsocket/afsocket-grammar.c"
    break;

  case 182: /* $@27: %empty  */
#line 1223 "modules/afsocket/afsocket-grammar.y"
                {
			CHECK_ERROR(strcmp((yyvsp[0].cptr), "http"), (yylsp[-2]), "This transport cannot be used with this destination driver, use the http() destination driver instead");

			Plugin *p;
			gint context = LL_CONTEXT_CLIENT_PROTO;

			p = cfg_find_plugin(configuration, context, (yyvsp[0].cptr));
			if (p)
				{
					/* for transports with grammar */
					if (p->parser)
						{
							LogProtoClientFactory *cf = cfg_parse_plugin(configuration, p, &(yylsp[0]), last_proto_client_options);
							((AFSocketDestDriver *) last_driver)->proto_factory = cf;
						}
				}
			transport_mapper_set_transport(last_transport_mapper, (yyvsp[0].cptr));
		}
#line 6490 "modules/afsocket/afsocket-grammar.c"
    break;

  case 183: /* dest_afsocket_transport: KW_TRANSPORT '(' string $@27 ')'  */
#line 1241 "modules/afsocket/afsocket-grammar.y"
        { free((yyvsp[-2].cptr)); }
#line 6496 "modules/afsocket/afsocket-grammar.c"
    break;

  case 185: /* $@28: %empty  */
#line 1244 "modules/afsocket/afsocket-grammar.y"
          {
            gchar buf[256];

            last_tls_context = tls_context_new(TM_CLIENT, cfg_lexer_format_location(lexer, &(yylsp[0]), buf, sizeof(buf)));
          }
#line 6506 "modules/afsocket/afsocket-grammar.c"
    break;

  case 186: /* dest_afsocket_transport: KW_TLS $@28 '(' dest_tls_options ')'  */
#line 1250 "modules/afsocket/afsocket-grammar.y"
          {
            afinet_dd_set_tls_context(last_driver, last_tls_context);
          }
#line 6514 "modules/afsocket/afsocket-grammar.c"
    break;

  case 187: /* afsocket_transport: KW_TRANSPORT '(' KW_TCP ')'  */
#line 1256 "modules/afsocket/afsocket-grammar.y"
                                                         { transport_mapper_set_transport(last_transport_mapper, "tcp"); }
#line 6520 "modules/afsocket/afsocket-grammar.c"
    break;

  case 188: /* afsocket_transport: KW_TRANSPORT '(' KW_UDP ')'  */
#line 1257 "modules/afsocket/afsocket-grammar.y"
                                                         { transport_mapper_set_transport(last_transport_mapper, "udp"); }
#line 6526 "modules/afsocket/afsocket-grammar.c"
    break;

  case 189: /* afsocket_transport: KW_TRANSPORT '(' KW_TLS ')'  */
#line 1258 "modules/afsocket/afsocket-grammar.y"
                                                         { transport_mapper_set_transport(last_transport_mapper, "tls"); }
#line 6532 "modules/afsocket/afsocket-grammar.c"
    break;

  case 190: /* afsocket_transport: KW_TRANSPORT '(' KW_AUTO ')'  */
#line 1259 "modules/afsocket/afsocket-grammar.y"
                                                         { transport_mapper_set_transport(last_transport_mapper, "auto"); }
#line 6538 "modules/afsocket/afsocket-grammar.c"
    break;

  case 191: /* afsocket_transport: KW_IP_PROTOCOL '(' inet_ip_protocol_option ')'  */
#line 1260 "modules/afsocket/afsocket-grammar.y"
                                                         { transport_mapper_set_address_family(last_transport_mapper, (yyvsp[-1].num)); }
#line 6544 "modules/afsocket/afsocket-grammar.c"
    break;

  case 195: /* dest_tls_option: KW_SNI '(' yesno ')'  */
#line 1271 "modules/afsocket/afsocket-grammar.y"
          {
            if ((yyvsp[-1].num))
              tls_context_set_sni(last_tls_context, ((AFInetDestDriver *)last_driver)->primary);
          }
#line 6553 "modules/afsocket/afsocket-grammar.c"
    break;

  case 196: /* dest_tls_option: KW_OCSP_STAPLING_VERIFY '(' yesno ')'  */
#line 1276 "modules/afsocket/afsocket-grammar.y"
          {
            tls_context_set_ocsp_stapling_verify(last_tls_context, (yyvsp[-1].num));
          }
#line 6561 "modules/afsocket/afsocket-grammar.c"
    break;

  case 197: /* dest_tls_option: KW_EXTENDED_KEY_USAGE_VERIFY '(' yesno ')'  */
#line 1280 "modules/afsocket/afsocket-grammar.y"
          {
            tls_context_set_extended_key_usage_verify(last_tls_context, (yyvsp[-1].num));
          }
#line 6569 "modules/afsocket/afsocket-grammar.c"
    break;

  case 200: /* tls_option: KW_IFDEF  */
#line 1292 "modules/afsocket/afsocket-grammar.y"
    {
    }
#line 6576 "modules/afsocket/afsocket-grammar.c"
    break;

  case 201: /* tls_option: KW_PEER_VERIFY '(' yesno ')'  */
#line 1296 "modules/afsocket/afsocket-grammar.y"
          {
	    gint verify_mode = (yyvsp[-1].num) ? (TVM_REQUIRED | TVM_TRUSTED) : (TVM_OPTIONAL | TVM_UNTRUSTED);
	    tls_context_set_verify_mode(last_tls_context, verify_mode);
    }
#line 6585 "modules/afsocket/afsocket-grammar.c"
    break;

  case 202: /* tls_option: KW_PEER_VERIFY '(' string ')'  */
#line 1301 "modules/afsocket/afsocket-grammar.y"
          {
	    CHECK_ERROR(tls_context_set_verify_mode_by_name(last_tls_context, (yyvsp[-1].cptr)), (yylsp[-1]), "unknown peer-verify() argument");
      free((yyvsp[-1].cptr));
    }
#line 6594 "modules/afsocket/afsocket-grammar.c"
    break;

  case 203: /* tls_option: KW_KEY_FILE '(' path_secret ')'  */
#line 1306 "modules/afsocket/afsocket-grammar.y"
          {
	    tls_context_set_key_file(last_tls_context, (yyvsp[-1].cptr));
      free((yyvsp[-1].cptr));
    }
#line 6603 "modules/afsocket/afsocket-grammar.c"
    break;

  case 204: /* tls_option: KW_KEYLOG_FILE '(' string ')'  */
#line 1311 "modules/afsocket/afsocket-grammar.y"
          {
	    GError *error = NULL;
	    CHECK_ERROR_GERROR(tls_context_set_keylog_file(last_tls_context, (yyvsp[-1].cptr), &error), (yylsp[-1]), error, "Error setting keylog-file()");
	    free((yyvsp[-1].cptr));
	  }
#line 6613 "modules/afsocket/afsocket-grammar.c"
    break;

  case 205: /* tls_option: KW_CERT_FILE '(' path_check ')'  */
#line 1317 "modules/afsocket/afsocket-grammar.y"
          {
	    tls_context_set_cert_file(last_tls_context, (yyvsp[-1].cptr));
      free((yyvsp[-1].cptr));
    }
#line 6622 "modules/afsocket/afsocket-grammar.c"
    break;

  case 206: /* tls_option: KW_DHPARAM_FILE '(' path_check ')'  */
#line 1322 "modules/afsocket/afsocket-grammar.y"
    {
      tls_context_set_dhparam_file(last_tls_context, (yyvsp[-1].cptr));
      free((yyvsp[-1].cptr));
    }
#line 6631 "modules/afsocket/afsocket-grammar.c"
    break;

  case 207: /* tls_option: KW_PKCS12_FILE '(' path_check ')'  */
#line 1327 "modules/afsocket/afsocket-grammar.y"
    {
      tls_context_set_pkcs12_file(last_tls_context, (yyvsp[-1].cptr));
      free((yyvsp[-1].cptr));
    }
#line 6640 "modules/afsocket/afsocket-grammar.c"
    break;

  case 208: /* tls_option: KW_CA_DIR '(' string ')'  */
#line 1332 "modules/afsocket/afsocket-grammar.y"
          {
	    tls_context_set_ca_dir(last_tls_context, (yyvsp[-1].cptr));
      free((yyvsp[-1].cptr));
    }
#line 6649 "modules/afsocket/afsocket-grammar.c"
    break;

  case 209: /* tls_option: KW_CRL_DIR '(' string ')'  */
#line 1337 "modules/afsocket/afsocket-grammar.y"
          {
	    tls_context_set_crl_dir(last_tls_context, (yyvsp[-1].cptr));
      free((yyvsp[-1].cptr));
    }
#line 6658 "modules/afsocket/afsocket-grammar.c"
    break;

  case 210: /* tls_option: KW_CA_FILE '(' path_check ')'  */
#line 1342 "modules/afsocket/afsocket-grammar.y"
    {
      tls_context_set_ca_file(last_tls_context, (yyvsp[-1].cptr));
      free((yyvsp[-1].cptr));
    }
#line 6667 "modules/afsocket/afsocket-grammar.c"
    break;

  case 211: /* tls_option: KW_TRUSTED_KEYS '(' string_list ')'  */
#line 1347 "modules/afsocket/afsocket-grammar.y"
    {
      tls_session_set_trusted_fingerprints(last_tls_context, (yyvsp[-1].ptr));
    }
#line 6675 "modules/afsocket/afsocket-grammar.c"
    break;

  case 212: /* tls_option: KW_TRUSTED_DN '(' string_list ')'  */
#line 1351 "modules/afsocket/afsocket-grammar.y"
    {
      tls_session_set_trusted_dn(last_tls_context, (yyvsp[-1].ptr));
    }
#line 6683 "modules/afsocket/afsocket-grammar.c"
    break;

  case 214: /* tls_option: KW_CIPHER_SUITE '(' string ')'  */
#line 1356 "modules/afsocket/afsocket-grammar.y"
    {
      /* compat for specifying TLS 1.2-or-older ciphers */
      tls_context_set_cipher_suite(last_tls_context, (yyvsp[-1].cptr));
      free((yyvsp[-1].cptr));
    }
#line 6693 "modules/afsocket/afsocket-grammar.c"
    break;

  case 215: /* tls_option: KW_SIGALGS '(' string ')'  */
#line 1362 "modules/afsocket/afsocket-grammar.y"
    {
      GError *error = NULL;
      CHECK_ERROR_GERROR(tls_context_set_sigalgs(last_tls_context, (yyvsp[-1].cptr), &error), (yylsp[-1]), error, "Error setting sigalgs()");
      free((yyvsp[-1].cptr));
    }
#line 6703 "modules/afsocket/afsocket-grammar.c"
    break;

  case 216: /* tls_option: KW_CLIENT_SIGALGS '(' string ')'  */
#line 1368 "modules/afsocket/afsocket-grammar.y"
    {
      GError *error = NULL;
      CHECK_ERROR_GERROR(tls_context_set_client_sigalgs(last_tls_context, (yyvsp[-1].cptr), &error), (yylsp[-1]), error, "Error setting client-sigalgs()");
      free((yyvsp[-1].cptr));
    }
#line 6713 "modules/afsocket/afsocket-grammar.c"
    break;

  case 217: /* tls_option: KW_ECDH_CURVE_LIST '(' string ')'  */
#line 1374 "modules/afsocket/afsocket-grammar.y"
    {
      tls_context_set_ecdh_curve_list(last_tls_context, (yyvsp[-1].cptr));
      free((yyvsp[-1].cptr));
    }
#line 6722 "modules/afsocket/afsocket-grammar.c"
    break;

  case 218: /* tls_option: KW_SSL_OPTIONS '(' string_list ')'  */
#line 1379 "modules/afsocket/afsocket-grammar.y"
          {
      CHECK_ERROR(tls_context_set_ssl_options_by_name(last_tls_context, (yyvsp[-1].ptr)), (yylsp[-1]), "unknown ssl-options() argument");
      string_list_free((yyvsp[-1].ptr));
	  }
#line 6731 "modules/afsocket/afsocket-grammar.c"
    break;

  case 219: /* tls_option: KW_SSL_VERSION '(' string ')'  */
#line 1384 "modules/afsocket/afsocket-grammar.y"
          {
      CHECK_ERROR(tls_context_set_ssl_version_by_name(last_tls_context, (yyvsp[-1].cptr)), (yylsp[-1]), "unknown ssl-version() argument");
      free((yyvsp[-1].cptr));
    }
#line 6740 "modules/afsocket/afsocket-grammar.c"
    break;

  case 220: /* tls_option: KW_ALLOW_COMPRESS '(' yesno ')'  */
#line 1389 "modules/afsocket/afsocket-grammar.y"
    {
      tls_context_set_allow_compress(last_tls_context, (yyvsp[-1].num));
    }
#line 6748 "modules/afsocket/afsocket-grammar.c"
    break;

  case 221: /* tls_option: KW_CONF_CMDS '(' tls_conf_cmds ')'  */
#line 1393 "modules/afsocket/afsocket-grammar.y"
          {
	    GError *error = NULL;
	    CHECK_ERROR_GERROR(tls_context_set_conf_cmds(last_tls_context, (yyvsp[-1].ptr), &error), (yylsp[-1]), error, "Error setting conf-cmds()");
	  }
#line 6757 "modules/afsocket/afsocket-grammar.c"
    break;

  case 222: /* tls_option: KW_ENDIF  */
#line 1398 "modules/afsocket/afsocket-grammar.y"
    {
    }
#line 6764 "modules/afsocket/afsocket-grammar.c"
    break;

  case 223: /* tls_conf_cmds: tls_conf_cmd_list_build  */
#line 1404 "modules/afsocket/afsocket-grammar.y"
                {
			(yyval.ptr) = (yyvsp[0].ptr);
		}
#line 6772 "modules/afsocket/afsocket-grammar.c"
    break;

  case 224: /* tls_conf_cmd_list_build: string LL_ARROW string tls_conf_cmd_list_build  */
#line 1411 "modules/afsocket/afsocket-grammar.y"
                {
			/* Revers order is important here as this is inside of a bison right recursion */
			(yyval.ptr) = g_list_prepend((yyvsp[0].ptr), g_strdup((yyvsp[-1].cptr)));
			(yyval.ptr) = g_list_prepend((yyval.ptr), g_strdup((yyvsp[-3].cptr)));

			free((yyvsp[-3].cptr));
			free((yyvsp[-1].cptr));
		}
#line 6785 "modules/afsocket/afsocket-grammar.c"
    break;

  case 225: /* tls_conf_cmd_list_build: %empty  */
#line 1420 "modules/afsocket/afsocket-grammar.y"
                {
			(yyval.ptr) = NULL;
		}
#line 6793 "modules/afsocket/afsocket-grammar.c"
    break;

  case 228: /* tls_cipher_suite: KW_TLS12_AND_OLDER '(' string ')'  */
#line 1432 "modules/afsocket/afsocket-grammar.y"
          {
            tls_context_set_cipher_suite(last_tls_context, (yyvsp[-1].cptr));
            free((yyvsp[-1].cptr));
          }
#line 6802 "modules/afsocket/afsocket-grammar.c"
    break;

  case 229: /* tls_cipher_suite: KW_TLS13 '(' string ')'  */
#line 1437 "modules/afsocket/afsocket-grammar.y"
          {
            GError *error = NULL;
            CHECK_ERROR_GERROR(tls_context_set_tls13_cipher_suite(last_tls_context, (yyvsp[-1].cptr), &error), (yylsp[-1]), error, "Error setting cipher-suite(tls13())");
            free((yyvsp[-1].cptr));
          }
#line 6812 "modules/afsocket/afsocket-grammar.c"
    break;

  case 230: /* socket_option: KW_SO_SNDBUF '(' nonnegative_integer ')'  */
#line 1446 "modules/afsocket/afsocket-grammar.y"
        {
		CHECK_ERROR((yyvsp[-1].num) <= G_MAXINT, (yylsp[-1]), "Invalid so_sndbuf, it has to be less than %d", G_MAXINT);
		last_sock_options->so_sndbuf = (yyvsp[-1].num);
	}
#line 6821 "modules/afsocket/afsocket-grammar.c"
    break;

  case 231: /* socket_option: KW_SO_RCVBUF '(' nonnegative_integer ')'  */
#line 1451 "modules/afsocket/afsocket-grammar.y"
        {
		CHECK_ERROR((yyvsp[-1].num) <= G_MAXINT, (yylsp[-1]), "Invalid so_rcvbuf, it has to be less than %d", G_MAXINT);
		last_sock_options->so_rcvbuf = (yyvsp[-1].num);
	}
#line 6830 "modules/afsocket/afsocket-grammar.c"
    break;

  case 232: /* socket_option: KW_SO_BROADCAST '(' yesno ')'  */
#line 1455 "modules/afsocket/afsocket-grammar.y"
                                                    { last_sock_options->so_broadcast = (yyvsp[-1].num); }
#line 6836 "modules/afsocket/afsocket-grammar.c"
    break;

  case 233: /* socket_option: KW_SO_KEEPALIVE '(' yesno ')'  */
#line 1456 "modules/afsocket/afsocket-grammar.y"
                                                    { last_sock_options->so_keepalive = (yyvsp[-1].num); }
#line 6842 "modules/afsocket/afsocket-grammar.c"
    break;

  case 234: /* socket_option: KW_SO_REUSEPORT '(' yesno ')'  */
#line 1457 "modules/afsocket/afsocket-grammar.y"
                                                    { last_sock_options->so_reuseport = (yyvsp[-1].num); }
#line 6848 "modules/afsocket/afsocket-grammar.c"
    break;

  case 236: /* unix_socket_option: KW_SO_PASSCRED '(' yesno ')'  */
#line 1462 "modules/afsocket/afsocket-grammar.y"
                                                    { socket_options_unix_set_so_passcred(last_sock_options, (yyvsp[-1].num)); }
#line 6854 "modules/afsocket/afsocket-grammar.c"
    break;

  case 238: /* inet_socket_option: KW_IP_TTL '(' nonnegative_integer ')'  */
#line 1467 "modules/afsocket/afsocket-grammar.y"
                                                              { ((SocketOptionsInet *) last_sock_options)->ip_ttl_val = (yyvsp[-1].num); }
#line 6860 "modules/afsocket/afsocket-grammar.c"
    break;

  case 239: /* inet_socket_option: KW_IP_TOS '(' nonnegative_integer ')'  */
#line 1468 "modules/afsocket/afsocket-grammar.y"
                                                              { ((SocketOptionsInet *) last_sock_options)->ip_tos_val = (yyvsp[-1].num); }
#line 6866 "modules/afsocket/afsocket-grammar.c"
    break;

  case 240: /* inet_socket_option: KW_IP_FREEBIND '(' yesno ')'  */
#line 1469 "modules/afsocket/afsocket-grammar.y"
                                                    { ((SocketOptionsInet *) last_sock_options)->ip_freebind = (yyvsp[-1].num); }
#line 6872 "modules/afsocket/afsocket-grammar.c"
    break;

  case 241: /* inet_socket_option: KW_INTERFACE '(' string ')'  */
#line 1470 "modules/afsocket/afsocket-grammar.y"
                                                    { socket_options_inet_set_interface_name((SocketOptionsInet *) last_sock_options, (yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 6878 "modules/afsocket/afsocket-grammar.c"
    break;

  case 242: /* inet_socket_option: KW_TCP_KEEPALIVE_TIME '(' nonnegative_integer ')'  */
#line 1472 "modules/afsocket/afsocket-grammar.y"
          {
	    CHECK_ERROR(socket_options_inet_set_tcp_keepalive_time((SocketOptionsInet *) last_sock_options, (yyvsp[-1].num)), (yylsp[-3]),
		            "The tcp-keepalive-time() option is not supported on this platform");
	  }
#line 6887 "modules/afsocket/afsocket-grammar.c"
    break;

  case 243: /* inet_socket_option: KW_TCP_KEEPALIVE_INTVL '(' nonnegative_integer ')'  */
#line 1477 "modules/afsocket/afsocket-grammar.y"
          {
	    CHECK_ERROR(socket_options_inet_set_tcp_keepalive_intvl((SocketOptionsInet *) last_sock_options, (yyvsp[-1].num)), (yylsp[-3]),
		            "The tcp-keepalive-intvl() option is not supported on this platform");
	  }
#line 6896 "modules/afsocket/afsocket-grammar.c"
    break;

  case 244: /* inet_socket_option: KW_TCP_KEEPALIVE_PROBES '(' nonnegative_integer ')'  */
#line 1482 "modules/afsocket/afsocket-grammar.y"
          {
	    CHECK_ERROR(socket_options_inet_set_tcp_keepalive_probes((SocketOptionsInet *) last_sock_options, (yyvsp[-1].num)), (yylsp[-3]),
		            "The tcp-keepalive-probes() option is not supported on this platform");
	  }
#line 6905 "modules/afsocket/afsocket-grammar.c"
    break;

  case 245: /* inet_ip_protocol_option: LL_NUMBER  */
#line 1490 "modules/afsocket/afsocket-grammar.y"
        {
          CHECK_ERROR((yyvsp[0].num) == 4 || (yyvsp[0].num) == 6, (yylsp[0]), "ip-protocol option can only be 4 or 6!");
          if ((yyvsp[0].num) == 4)
            {
              (yyval.num) = AF_INET;
            }
          else
            {
              (yyval.num) = AF_INET6;
            }
        }
#line 6921 "modules/afsocket/afsocket-grammar.c"
    break;

  case 246: /* template_content_inner: string  */
#line 1780 "modules/afsocket/afsocket-grammar.y"
        {
          GError *error = NULL;

          CHECK_ERROR_GERROR(log_template_compile((yyvsp[-1].ptr), (yyvsp[0].cptr), &error), (yylsp[0]), error, "Error compiling template");
          free((yyvsp[0].cptr));
        }
#line 6932 "modules/afsocket/afsocket-grammar.c"
    break;

  case 247: /* template_content_inner: type_hint '(' string_or_number ')'  */
#line 1787 "modules/afsocket/afsocket-grammar.y"
        {
          GError *error = NULL;

          CHECK_ERROR_GERROR(log_template_compile((yyvsp[-4].ptr), (yyvsp[-1].cptr), &error), (yylsp[-1]), error, "Error compiling template");
          free((yyvsp[-1].cptr));

          log_template_set_type_hint_value((yyvsp[-4].ptr), (yyvsp[-3].num));
        }
#line 6945 "modules/afsocket/afsocket-grammar.c"
    break;

  case 248: /* template_content_inner: LL_NUMBER  */
#line 1796 "modules/afsocket/afsocket-grammar.y"
        {
          gchar decimal[32];

          g_snprintf(decimal, sizeof(decimal), "%" G_GINT64_FORMAT, (yyvsp[0].num));
          log_template_compile_literal_string((yyvsp[-1].ptr), decimal);
          log_template_set_type_hint((yyvsp[-1].ptr), "int64", NULL);
        }
#line 6957 "modules/afsocket/afsocket-grammar.c"
    break;

  case 249: /* template_content_inner: LL_FLOAT  */
#line 1804 "modules/afsocket/afsocket-grammar.y"
        {
          log_template_compile_literal_string((yyvsp[-1].ptr), lexer->token_text->str);
          log_template_set_type_hint((yyvsp[-1].ptr), "float", NULL);
        }
#line 6966 "modules/afsocket/afsocket-grammar.c"
    break;

  case 250: /* type_hint: LL_IDENTIFIER  */
#line 1812 "modules/afsocket/afsocket-grammar.y"
          {
            LogMessageValueType type;
            GError *error = NULL;

            CHECK_ERROR_GERROR(type_hint_parse((yyvsp[0].cptr), &type, &error), (yylsp[0]), error, "Unknown type hint");
            free((yyvsp[0].cptr));
            (yyval.num) = type;
          }
#line 6979 "modules/afsocket/afsocket-grammar.c"
    break;

  case 251: /* @31: %empty  */
#line 1822 "modules/afsocket/afsocket-grammar.y"
               {
            (yyval.ptr) = log_template_new(configuration, NULL);
          }
#line 6987 "modules/afsocket/afsocket-grammar.c"
    break;

  case 252: /* template_content: @31 template_content_inner  */
#line 1824 "modules/afsocket/afsocket-grammar.y"
                                                                            { (yyval.ptr) = (yyvsp[-1].ptr); }
#line 6993 "modules/afsocket/afsocket-grammar.c"
    break;

  case 253: /* template_name_or_content: _template_ref_context_push template_name_or_content_tail  */
#line 1833 "modules/afsocket/afsocket-grammar.y"
                                                                            { (yyval.ptr) = (yyvsp[0].ptr); }
#line 6999 "modules/afsocket/afsocket-grammar.c"
    break;

  case 254: /* template_name_or_content_tail: LL_TEMPLATE_REF  */
#line 1837 "modules/afsocket/afsocket-grammar.y"
                                                                            { (yyval.ptr) = cfg_tree_lookup_template(&configuration->tree, (yyvsp[0].cptr)); free((yyvsp[0].cptr)); }
#line 7005 "modules/afsocket/afsocket-grammar.c"
    break;

  case 255: /* template_name_or_content_tail: template_content  */
#line 1838 "modules/afsocket/afsocket-grammar.y"
                                                                            { (yyval.ptr) = (yyvsp[0].ptr); }
#line 7011 "modules/afsocket/afsocket-grammar.c"
    break;

  case 258: /* yesno_strict: KW_YES  */
#line 1849 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.num) = 1; }
#line 7017 "modules/afsocket/afsocket-grammar.c"
    break;

  case 259: /* yesno_strict: KW_NO  */
#line 1850 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.num) = 0; }
#line 7023 "modules/afsocket/afsocket-grammar.c"
    break;

  case 261: /* yesno: LL_NUMBER  */
#line 1855 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.num) = (yyvsp[0].num); }
#line 7029 "modules/afsocket/afsocket-grammar.c"
    break;

  case 262: /* dnsmode: yesno  */
#line 1865 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.num) = (yyvsp[0].num); }
#line 7035 "modules/afsocket/afsocket-grammar.c"
    break;

  case 263: /* dnsmode: KW_PERSIST_ONLY  */
#line 1866 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.num) = 2; }
#line 7041 "modules/afsocket/afsocket-grammar.c"
    break;

  case 264: /* nonnegative_integer64: LL_NUMBER  */
#line 1871 "modules/afsocket/afsocket-grammar.y"
          {
            CHECK_ERROR(((yyvsp[0].num) >= 0), (yylsp[0]), "It cannot be negative");
          }
#line 7049 "modules/afsocket/afsocket-grammar.c"
    break;

  case 265: /* nonnegative_integer: nonnegative_integer64  */
#line 1878 "modules/afsocket/afsocket-grammar.y"
          {
            CHECK_ERROR(((yyvsp[0].num) <= G_MAXINT32), (yylsp[0]), "Must be smaller than 2^31");
          }
#line 7057 "modules/afsocket/afsocket-grammar.c"
    break;

  case 266: /* positive_integer64: LL_NUMBER  */
#line 1885 "modules/afsocket/afsocket-grammar.y"
          {
            CHECK_ERROR(((yyvsp[0].num) > 0), (yylsp[0]), "Must be positive");
          }
#line 7065 "modules/afsocket/afsocket-grammar.c"
    break;

  case 267: /* positive_integer: positive_integer64  */
#line 1892 "modules/afsocket/afsocket-grammar.y"
          {
            CHECK_ERROR(((yyvsp[0].num) <= G_MAXINT32), (yylsp[0]), "Must be smaller than 2^31");
          }
#line 7073 "modules/afsocket/afsocket-grammar.c"
    break;

  case 268: /* nonnegative_float: LL_FLOAT  */
#line 1899 "modules/afsocket/afsocket-grammar.y"
          {
            CHECK_ERROR(((yyvsp[0].fnum) >= 0), (yylsp[0]), "It cannot be negative");
          }
#line 7081 "modules/afsocket/afsocket-grammar.c"
    break;

  case 269: /* nonnegative_float: nonnegative_integer  */
#line 1903 "modules/afsocket/afsocket-grammar.y"
          {
            (yyval.fnum) = (double) (yyvsp[0].num);
          }
#line 7089 "modules/afsocket/afsocket-grammar.c"
    break;

  case 270: /* string_or_number: string  */
#line 1920 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.cptr) = (yyvsp[0].cptr); }
#line 7095 "modules/afsocket/afsocket-grammar.c"
    break;

  case 271: /* string_or_number: LL_NUMBER  */
#line 1921 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.cptr) = strdup(lexer->token_text->str); }
#line 7101 "modules/afsocket/afsocket-grammar.c"
    break;

  case 272: /* string_or_number: LL_FLOAT  */
#line 1922 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.cptr) = strdup(lexer->token_text->str); }
#line 7107 "modules/afsocket/afsocket-grammar.c"
    break;

  case 273: /* path: string  */
#line 1932 "modules/afsocket/afsocket-grammar.y"
          {
            struct stat buffer;
            int ret = stat((yyvsp[0].cptr), &buffer);
            CHECK_ERROR((ret == 0), (yylsp[0]), "File \"%s\" not found: %s", (yyvsp[0].cptr), strerror(errno));
            (yyval.cptr) = (yyvsp[0].cptr);
	  }
#line 7118 "modules/afsocket/afsocket-grammar.c"
    break;

  case 274: /* path_check: path  */
#line 1941 "modules/afsocket/afsocket-grammar.y"
           { cfg_path_track_file(configuration, (yyvsp[0].cptr), "path_check"); }
#line 7124 "modules/afsocket/afsocket-grammar.c"
    break;

  case 275: /* path_secret: path  */
#line 1945 "modules/afsocket/afsocket-grammar.y"
           { cfg_path_track_file(configuration, (yyvsp[0].cptr), "path_secret"); }
#line 7130 "modules/afsocket/afsocket-grammar.c"
    break;

  case 276: /* string_list: string_list_build  */
#line 1957 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.ptr) = (yyvsp[0].ptr); }
#line 7136 "modules/afsocket/afsocket-grammar.c"
    break;

  case 277: /* string_list_build: string string_list_build  */
#line 1961 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.ptr) = g_list_prepend((yyvsp[0].ptr), g_strdup((yyvsp[-1].cptr))); free((yyvsp[-1].cptr)); }
#line 7142 "modules/afsocket/afsocket-grammar.c"
    break;

  case 278: /* string_list_build: %empty  */
#line 1962 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.ptr) = NULL; }
#line 7148 "modules/afsocket/afsocket-grammar.c"
    break;

  case 279: /* severity_string: string  */
#line 1972 "modules/afsocket/afsocket-grammar.y"
          {
	    /* return the numeric value of the "level" */
	    int n = syslog_name_lookup_severity_by_name((yyvsp[0].cptr));
	    CHECK_ERROR((n != -1), (yylsp[0]), "Unknown priority level\"%s\"", (yyvsp[0].cptr));
	    free((yyvsp[0].cptr));
            (yyval.num) = n;
	  }
#line 7160 "modules/afsocket/afsocket-grammar.c"
    break;

  case 280: /* facility_string: string  */
#line 1983 "modules/afsocket/afsocket-grammar.y"
          {
            /* return the numeric value of facility */
	    int n = syslog_name_lookup_facility_by_name((yyvsp[0].cptr));
	    CHECK_ERROR((n != -1), (yylsp[0]), "Unknown facility \"%s\"", (yyvsp[0].cptr));
	    free((yyvsp[0].cptr));
	    (yyval.num) = n;
	  }
#line 7172 "modules/afsocket/afsocket-grammar.c"
    break;

  case 281: /* facility_string: KW_SYSLOG  */
#line 1990 "modules/afsocket/afsocket-grammar.y"
                                                { (yyval.num) = LOG_SYSLOG; }
#line 7178 "modules/afsocket/afsocket-grammar.c"
    break;

  case 282: /* driver_option: KW_PERSIST_NAME '(' string ')'  */
#line 1999 "modules/afsocket/afsocket-grammar.y"
                                         { log_pipe_set_persist_name(&last_driver->super, (yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 7184 "modules/afsocket/afsocket-grammar.c"
    break;

  case 283: /* driver_option: KW_INTERNAL '(' yesno ')'  */
#line 2000 "modules/afsocket/afsocket-grammar.y"
                                    { log_pipe_set_internal(&last_driver->super, (yyvsp[-1].num)); }
#line 7190 "modules/afsocket/afsocket-grammar.c"
    break;

  case 284: /* inner_source: LL_PLUGIN  */
#line 2005 "modules/afsocket/afsocket-grammar.y"
          {
            Plugin *p;
            gint context = LL_CONTEXT_INNER_SRC;
            gpointer value;

            p = cfg_find_plugin(configuration, context, (yyvsp[0].cptr));
            CHECK_ERROR(p, (yylsp[0]), "%s plugin %s not found", cfg_lexer_lookup_context_name_by_type(context), (yyvsp[0].cptr));

            value = cfg_parse_plugin(configuration, p, &(yylsp[0]), last_driver);

            free((yyvsp[0].cptr));
            if (!value)
              {
                YYERROR;
              }
            if (!log_driver_add_plugin(last_driver, (LogDriverPlugin *) value))
              {
                log_driver_plugin_free(value);
                CHECK_ERROR(TRUE, (yylsp[0]), "Error while registering the plugin %s in this destination", (yyvsp[0].cptr));
              }
          }
#line 7216 "modules/afsocket/afsocket-grammar.c"
    break;

  case 287: /* inner_dest: LL_PLUGIN  */
#line 2037 "modules/afsocket/afsocket-grammar.y"
          {
            Plugin *p;
            gint context = LL_CONTEXT_INNER_DEST;
            gpointer value;

            p = cfg_find_plugin(configuration, context, (yyvsp[0].cptr));
            CHECK_ERROR(p, (yylsp[0]), "%s plugin %s not found", cfg_lexer_lookup_context_name_by_type(context), (yyvsp[0].cptr));

            value = cfg_parse_plugin(configuration, p, &(yylsp[0]), last_driver);

            free((yyvsp[0].cptr));
            if (!value)
              {
                YYERROR;
              }
            if (!log_driver_add_plugin(last_driver, (LogDriverPlugin *) value))
              {
                log_driver_plugin_free(value);
                CHECK_ERROR(TRUE, (yylsp[0]), "Error while registering the plugin %s in this destination", (yyvsp[0].cptr));
              }
          }
#line 7242 "modules/afsocket/afsocket-grammar.c"
    break;

  case 288: /* dest_driver_option: KW_LOG_FIFO_SIZE '(' positive_integer ')'  */
#line 2064 "modules/afsocket/afsocket-grammar.y"
                                                        { ((LogDestDriver *) last_driver)->log_fifo_size = (yyvsp[-1].num); }
#line 7248 "modules/afsocket/afsocket-grammar.c"
    break;

  case 289: /* dest_driver_option: KW_THROTTLE '(' nonnegative_integer ')'  */
#line 2065 "modules/afsocket/afsocket-grammar.y"
                                                          { ((LogDestDriver *) last_driver)->throttle = (yyvsp[-1].num); }
#line 7254 "modules/afsocket/afsocket-grammar.c"
    break;

  case 292: /* source_option: KW_LOG_IW_SIZE '(' positive_integer ')'  */
#line 2134 "modules/afsocket/afsocket-grammar.y"
                                                        { last_source_options->init_window_size = (yyvsp[-1].num); }
#line 7260 "modules/afsocket/afsocket-grammar.c"
    break;

  case 293: /* source_option: KW_CHAIN_HOSTNAMES '(' yesno ')'  */
#line 2135 "modules/afsocket/afsocket-grammar.y"
                                                { last_source_options->chain_hostnames = (yyvsp[-1].num); }
#line 7266 "modules/afsocket/afsocket-grammar.c"
    break;

  case 294: /* source_option: KW_KEEP_HOSTNAME '(' yesno ')'  */
#line 2136 "modules/afsocket/afsocket-grammar.y"
                                                { last_source_options->keep_hostname = (yyvsp[-1].num); }
#line 7272 "modules/afsocket/afsocket-grammar.c"
    break;

  case 295: /* source_option: KW_PROGRAM_OVERRIDE '(' string ')'  */
#line 2137 "modules/afsocket/afsocket-grammar.y"
                                                { last_source_options->program_override = g_strdup((yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 7278 "modules/afsocket/afsocket-grammar.c"
    break;

  case 296: /* source_option: KW_HOST_OVERRIDE '(' string ')'  */
#line 2138 "modules/afsocket/afsocket-grammar.y"
                                                { last_source_options->host_override = g_strdup((yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 7284 "modules/afsocket/afsocket-grammar.c"
    break;

  case 297: /* source_option: KW_LOG_PREFIX '(' string ')'  */
#line 2139 "modules/afsocket/afsocket-grammar.y"
                                                { gchar *p = strrchr((yyvsp[-1].cptr), ':'); if (p) *p = 0; last_source_options->program_override = g_strdup((yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 7290 "modules/afsocket/afsocket-grammar.c"
    break;

  case 298: /* source_option: KW_KEEP_TIMESTAMP '(' yesno ')'  */
#line 2140 "modules/afsocket/afsocket-grammar.y"
                                                { last_source_options->keep_timestamp = (yyvsp[-1].num); }
#line 7296 "modules/afsocket/afsocket-grammar.c"
    break;

  case 299: /* source_option: KW_READ_OLD_RECORDS '(' yesno ')'  */
#line 2141 "modules/afsocket/afsocket-grammar.y"
                                                { last_source_options->read_old_records = (yyvsp[-1].num); }
#line 7302 "modules/afsocket/afsocket-grammar.c"
    break;

  case 300: /* source_option: KW_USE_SYSLOGNG_PID '(' yesno ')'  */
#line 2142 "modules/afsocket/afsocket-grammar.y"
                                                { last_source_options->use_syslogng_pid = (yyvsp[-1].num); }
#line 7308 "modules/afsocket/afsocket-grammar.c"
    break;

  case 301: /* source_option: KW_TAGS '(' string_list ')'  */
#line 2143 "modules/afsocket/afsocket-grammar.y"
                                                { log_source_options_set_tags(last_source_options, (yyvsp[-1].ptr)); }
#line 7314 "modules/afsocket/afsocket-grammar.c"
    break;

  case 302: /* $@34: %empty  */
#line 2144 "modules/afsocket/afsocket-grammar.y"
          { last_host_resolve_options = &last_source_options->host_resolve_options; }
#line 7320 "modules/afsocket/afsocket-grammar.c"
    break;

  case 304: /* source_reader_option: KW_CHECK_HOSTNAME '(' yesno ')'  */
#line 2151 "modules/afsocket/afsocket-grammar.y"
                                                { last_reader_options->check_hostname = (yyvsp[-1].num); }
#line 7326 "modules/afsocket/afsocket-grammar.c"
    break;

  case 305: /* source_reader_option: KW_CHECK_PROGRAM '(' yesno ')'  */
#line 2152 "modules/afsocket/afsocket-grammar.y"
                                         { last_reader_options->check_program = (yyvsp[-1].num); }
#line 7332 "modules/afsocket/afsocket-grammar.c"
    break;

  case 307: /* source_reader_option: KW_LOG_FETCH_LIMIT '(' positive_integer ')'  */
#line 2154 "modules/afsocket/afsocket-grammar.y"
                                                        { last_reader_options->fetch_limit = (yyvsp[-1].num); }
#line 7338 "modules/afsocket/afsocket-grammar.c"
    break;

  case 308: /* source_reader_option: KW_FORMAT '(' string ')'  */
#line 2155 "modules/afsocket/afsocket-grammar.y"
                                                { last_reader_options->parse_options.format = g_strdup((yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 7344 "modules/afsocket/afsocket-grammar.c"
    break;

  case 309: /* $@35: %empty  */
#line 2156 "modules/afsocket/afsocket-grammar.y"
          { last_source_options = &last_reader_options->super; }
#line 7350 "modules/afsocket/afsocket-grammar.c"
    break;

  case 311: /* $@36: %empty  */
#line 2157 "modules/afsocket/afsocket-grammar.y"
          { last_proto_server_options = &last_reader_options->proto_options; }
#line 7356 "modules/afsocket/afsocket-grammar.c"
    break;

  case 313: /* $@37: %empty  */
#line 2158 "modules/afsocket/afsocket-grammar.y"
          { last_msg_format_options = &last_reader_options->parse_options; }
#line 7362 "modules/afsocket/afsocket-grammar.c"
    break;

  case 315: /* source_reader_option_flags: string source_reader_option_flags  */
#line 2162 "modules/afsocket/afsocket-grammar.y"
                                                { CHECK_ERROR(log_reader_options_process_flag(last_reader_options, (yyvsp[-1].cptr)), (yylsp[-1]), "Unknown flag \"%s\"", (yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 7368 "modules/afsocket/afsocket-grammar.c"
    break;

  case 316: /* source_reader_option_flags: KW_CHECK_HOSTNAME source_reader_option_flags  */
#line 2163 "modules/afsocket/afsocket-grammar.y"
                                                           { log_reader_options_process_flag(last_reader_options, "check-hostname"); }
#line 7374 "modules/afsocket/afsocket-grammar.c"
    break;

  case 317: /* source_reader_option_flags: KW_CHECK_PROGRAM source_reader_option_flags  */
#line 2164 "modules/afsocket/afsocket-grammar.y"
                                                          { log_reader_options_process_flag(last_reader_options, "check-program"); }
#line 7380 "modules/afsocket/afsocket-grammar.c"
    break;

  case 319: /* source_proto_option: KW_ENCODING '(' string ')'  */
#line 2171 "modules/afsocket/afsocket-grammar.y"
          {
            CHECK_ERROR(log_proto_server_options_set_encoding(last_proto_server_options, (yyvsp[-1].cptr)),
                        (yylsp[-1]),
                        "unknown encoding \"%s\"", (yyvsp[-1].cptr));
            free((yyvsp[-1].cptr));
          }
#line 7391 "modules/afsocket/afsocket-grammar.c"
    break;

  case 320: /* source_proto_option: KW_LOG_MSG_SIZE '(' positive_integer ')'  */
#line 2177 "modules/afsocket/afsocket-grammar.y"
                                                        { last_proto_server_options->super.max_msg_size = (yyvsp[-1].num); }
#line 7397 "modules/afsocket/afsocket-grammar.c"
    break;

  case 321: /* source_proto_option: KW_TRIM_LARGE_MESSAGES '(' yesno ')'  */
#line 2178 "modules/afsocket/afsocket-grammar.y"
                                                        { last_proto_server_options->super.trim_large_messages = (yyvsp[-1].num); }
#line 7403 "modules/afsocket/afsocket-grammar.c"
    break;

  case 322: /* source_proto_option: KW_IDLE_TIMEOUT '(' positive_integer ')'  */
#line 2179 "modules/afsocket/afsocket-grammar.y"
                                                        { last_proto_server_options->super.idle_timeout = (yyvsp[-1].num); }
#line 7409 "modules/afsocket/afsocket-grammar.c"
    break;

  case 323: /* host_resolve_option: KW_USE_FQDN '(' yesno ')'  */
#line 2183 "modules/afsocket/afsocket-grammar.y"
                                                { last_host_resolve_options->use_fqdn = (yyvsp[-1].num); }
#line 7415 "modules/afsocket/afsocket-grammar.c"
    break;

  case 324: /* host_resolve_option: KW_USE_DNS '(' dnsmode ')'  */
#line 2184 "modules/afsocket/afsocket-grammar.y"
                                                { last_host_resolve_options->use_dns = (yyvsp[-1].num); }
#line 7421 "modules/afsocket/afsocket-grammar.c"
    break;

  case 325: /* host_resolve_option: KW_DNS_CACHE '(' yesno ')'  */
#line 2185 "modules/afsocket/afsocket-grammar.y"
                                                { last_host_resolve_options->use_dns_cache = (yyvsp[-1].num); }
#line 7427 "modules/afsocket/afsocket-grammar.c"
    break;

  case 326: /* host_resolve_option: KW_NORMALIZE_HOSTNAMES '(' yesno ')'  */
#line 2186 "modules/afsocket/afsocket-grammar.y"
                                                { last_host_resolve_options->normalize_hostnames = (yyvsp[-1].num); }
#line 7433 "modules/afsocket/afsocket-grammar.c"
    break;

  case 327: /* msg_format_option: KW_TIME_ZONE '(' string ')'  */
#line 2190 "modules/afsocket/afsocket-grammar.y"
                                                { last_msg_format_options->recv_time_zone = g_strdup((yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 7439 "modules/afsocket/afsocket-grammar.c"
    break;

  case 328: /* msg_format_option: KW_DEFAULT_SEVERITY '(' severity_string ')'  */
#line 2192 "modules/afsocket/afsocket-grammar.y"
          {
	    if (last_msg_format_options->default_pri == 0xFFFF)
	      last_msg_format_options->default_pri = LOG_USER;
	    last_msg_format_options->default_pri = (last_msg_format_options->default_pri & ~SYSLOG_PRIMASK) | (yyvsp[-1].num);
          }
#line 7449 "modules/afsocket/afsocket-grammar.c"
    break;

  case 329: /* msg_format_option: KW_DEFAULT_FACILITY '(' facility_string ')'  */
#line 2198 "modules/afsocket/afsocket-grammar.y"
          {
	    if (last_msg_format_options->default_pri == 0xFFFF)
	      last_msg_format_options->default_pri = LOG_NOTICE;
	    last_msg_format_options->default_pri = (last_msg_format_options->default_pri & SYSLOG_PRIMASK) | (yyvsp[-1].num);
          }
#line 7459 "modules/afsocket/afsocket-grammar.c"
    break;

  case 330: /* msg_format_option: KW_SDATA_PREFIX '(' string ')'  */
#line 2204 "modules/afsocket/afsocket-grammar.y"
          {
            CHECK_ERROR(msg_format_options_set_sdata_prefix(last_msg_format_options, (yyvsp[-1].cptr)), (yylsp[-1]), "Prefix is too long \"%s\"", (yyvsp[-1].cptr));
            free((yyvsp[-1].cptr));
          }
#line 7468 "modules/afsocket/afsocket-grammar.c"
    break;

  case 332: /* dest_writer_option: KW_FLUSH_LINES '(' nonnegative_integer ')'  */
#line 2219 "modules/afsocket/afsocket-grammar.y"
                                                                { last_writer_options->flush_lines = (yyvsp[-1].num); }
#line 7474 "modules/afsocket/afsocket-grammar.c"
    break;

  case 333: /* dest_writer_option: KW_FLUSH_TIMEOUT '(' positive_integer ')'  */
#line 2220 "modules/afsocket/afsocket-grammar.y"
                                                        { }
#line 7480 "modules/afsocket/afsocket-grammar.c"
    break;

  case 334: /* dest_writer_option: KW_SUPPRESS '(' nonnegative_integer ')'  */
#line 2221 "modules/afsocket/afsocket-grammar.y"
                                                             { last_writer_options->suppress = (yyvsp[-1].num); }
#line 7486 "modules/afsocket/afsocket-grammar.c"
    break;

  case 335: /* dest_writer_option: KW_TEMPLATE '(' template_name_or_content ')'  */
#line 2222 "modules/afsocket/afsocket-grammar.y"
                                                             { last_writer_options->template = (yyvsp[-1].ptr); }
#line 7492 "modules/afsocket/afsocket-grammar.c"
    break;

  case 336: /* dest_writer_option: KW_PAD_SIZE '(' nonnegative_integer ')'  */
#line 2223 "modules/afsocket/afsocket-grammar.y"
                                                          { last_writer_options->padding = (yyvsp[-1].num); }
#line 7498 "modules/afsocket/afsocket-grammar.c"
    break;

  case 337: /* dest_writer_option: KW_TRUNCATE_SIZE '(' nonnegative_integer ')'  */
#line 2224 "modules/afsocket/afsocket-grammar.y"
                                                               { last_writer_options->truncate_size = (yyvsp[-1].num); }
#line 7504 "modules/afsocket/afsocket-grammar.c"
    break;

  case 338: /* dest_writer_option: KW_MARK_FREQ '(' nonnegative_integer ')'  */
#line 2225 "modules/afsocket/afsocket-grammar.y"
                                                          { last_writer_options->mark_freq = (yyvsp[-1].num); }
#line 7510 "modules/afsocket/afsocket-grammar.c"
    break;

  case 339: /* dest_writer_option: KW_MARK_MODE '(' KW_INTERNAL ')'  */
#line 2226 "modules/afsocket/afsocket-grammar.y"
                                                { log_writer_options_set_mark_mode(last_writer_options, "internal"); }
#line 7516 "modules/afsocket/afsocket-grammar.c"
    break;

  case 340: /* dest_writer_option: KW_MARK_MODE '(' string ')'  */
#line 2228 "modules/afsocket/afsocket-grammar.y"
          {
	    CHECK_ERROR(cfg_lookup_mark_mode((yyvsp[-1].cptr)) != -1, (yylsp[-1]), "illegal mark mode \"%s\"", (yyvsp[-1].cptr));
            log_writer_options_set_mark_mode(last_writer_options, (yyvsp[-1].cptr));
            free((yyvsp[-1].cptr));
          }
#line 7526 "modules/afsocket/afsocket-grammar.c"
    break;

  case 341: /* dest_writer_option: KW_TIME_REOPEN '(' positive_integer ')'  */
#line 2233 "modules/afsocket/afsocket-grammar.y"
                                                  { last_writer_options->time_reopen = (yyvsp[-1].num); }
#line 7532 "modules/afsocket/afsocket-grammar.c"
    break;

  case 342: /* $@38: %empty  */
#line 2234 "modules/afsocket/afsocket-grammar.y"
          { last_template_options = &last_writer_options->template_options; }
#line 7538 "modules/afsocket/afsocket-grammar.c"
    break;

  case 344: /* dest_writer_options_flags: string dest_writer_options_flags  */
#line 2238 "modules/afsocket/afsocket-grammar.y"
                                               { CHECK_ERROR(log_writer_options_process_flag(last_writer_options, (yyvsp[-1].cptr)), (yylsp[-1]), "Unknown flag \"%s\"", (yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 7544 "modules/afsocket/afsocket-grammar.c"
    break;

  case 346: /* file_perm_option: KW_OWNER '(' string_or_number ')'  */
#line 2243 "modules/afsocket/afsocket-grammar.y"
                                                { file_perm_options_set_file_uid(last_file_perm_options, (yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 7550 "modules/afsocket/afsocket-grammar.c"
    break;

  case 347: /* file_perm_option: KW_OWNER '(' ')'  */
#line 2244 "modules/afsocket/afsocket-grammar.y"
                                                { file_perm_options_dont_change_file_uid(last_file_perm_options); }
#line 7556 "modules/afsocket/afsocket-grammar.c"
    break;

  case 348: /* file_perm_option: KW_GROUP '(' string_or_number ')'  */
#line 2245 "modules/afsocket/afsocket-grammar.y"
                                                { file_perm_options_set_file_gid(last_file_perm_options, (yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 7562 "modules/afsocket/afsocket-grammar.c"
    break;

  case 349: /* file_perm_option: KW_GROUP '(' ')'  */
#line 2246 "modules/afsocket/afsocket-grammar.y"
                                                { file_perm_options_dont_change_file_gid(last_file_perm_options); }
#line 7568 "modules/afsocket/afsocket-grammar.c"
    break;

  case 350: /* file_perm_option: KW_PERM '(' LL_NUMBER ')'  */
#line 2247 "modules/afsocket/afsocket-grammar.y"
                                                { file_perm_options_set_file_perm(last_file_perm_options, (yyvsp[-1].num)); }
#line 7574 "modules/afsocket/afsocket-grammar.c"
    break;

  case 351: /* file_perm_option: KW_PERM '(' ')'  */
#line 2248 "modules/afsocket/afsocket-grammar.y"
                                                { file_perm_options_dont_change_file_perm(last_file_perm_options); }
#line 7580 "modules/afsocket/afsocket-grammar.c"
    break;

  case 352: /* file_perm_option: KW_DIR_OWNER '(' string_or_number ')'  */
#line 2249 "modules/afsocket/afsocket-grammar.y"
                                                { file_perm_options_set_dir_uid(last_file_perm_options, (yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 7586 "modules/afsocket/afsocket-grammar.c"
    break;

  case 353: /* file_perm_option: KW_DIR_OWNER '(' ')'  */
#line 2250 "modules/afsocket/afsocket-grammar.y"
                                                { file_perm_options_dont_change_dir_uid(last_file_perm_options); }
#line 7592 "modules/afsocket/afsocket-grammar.c"
    break;

  case 354: /* file_perm_option: KW_DIR_GROUP '(' string_or_number ')'  */
#line 2251 "modules/afsocket/afsocket-grammar.y"
                                                { file_perm_options_set_dir_gid(last_file_perm_options, (yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 7598 "modules/afsocket/afsocket-grammar.c"
    break;

  case 355: /* file_perm_option: KW_DIR_GROUP '(' ')'  */
#line 2252 "modules/afsocket/afsocket-grammar.y"
                                                { file_perm_options_dont_change_dir_gid(last_file_perm_options); }
#line 7604 "modules/afsocket/afsocket-grammar.c"
    break;

  case 356: /* file_perm_option: KW_DIR_PERM '(' LL_NUMBER ')'  */
#line 2253 "modules/afsocket/afsocket-grammar.y"
                                                { file_perm_options_set_dir_perm(last_file_perm_options, (yyvsp[-1].num)); }
#line 7610 "modules/afsocket/afsocket-grammar.c"
    break;

  case 357: /* file_perm_option: KW_DIR_PERM '(' ')'  */
#line 2254 "modules/afsocket/afsocket-grammar.y"
                                                { file_perm_options_dont_change_dir_perm(last_file_perm_options); }
#line 7616 "modules/afsocket/afsocket-grammar.c"
    break;

  case 358: /* template_option: KW_TS_FORMAT '(' string ')'  */
#line 2258 "modules/afsocket/afsocket-grammar.y"
                                                { last_template_options->ts_format = cfg_ts_format_value((yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 7622 "modules/afsocket/afsocket-grammar.c"
    break;

  case 359: /* template_option: KW_FRAC_DIGITS '(' nonnegative_integer ')'  */
#line 2259 "modules/afsocket/afsocket-grammar.y"
                                                        { last_template_options->frac_digits = (yyvsp[-1].num); }
#line 7628 "modules/afsocket/afsocket-grammar.c"
    break;

  case 360: /* template_option: KW_TIME_ZONE '(' string ')'  */
#line 2260 "modules/afsocket/afsocket-grammar.y"
                                                { last_template_options->time_zone[LTZ_SEND] = g_strdup((yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 7634 "modules/afsocket/afsocket-grammar.c"
    break;

  case 361: /* template_option: KW_SEND_TIME_ZONE '(' string ')'  */
#line 2261 "modules/afsocket/afsocket-grammar.y"
                                                { last_template_options->time_zone[LTZ_SEND] = g_strdup((yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 7640 "modules/afsocket/afsocket-grammar.c"
    break;

  case 362: /* template_option: KW_LOCAL_TIME_ZONE '(' string ')'  */
#line 2262 "modules/afsocket/afsocket-grammar.y"
                                                { last_template_options->time_zone[LTZ_LOCAL] = g_strdup((yyvsp[-1].cptr)); free((yyvsp[-1].cptr)); }
#line 7646 "modules/afsocket/afsocket-grammar.c"
    break;

  case 363: /* template_option: KW_TEMPLATE_ESCAPE '(' yesno ')'  */
#line 2263 "modules/afsocket/afsocket-grammar.y"
                                                { last_template_options->escape = (yyvsp[-1].num); }
#line 7652 "modules/afsocket/afsocket-grammar.c"
    break;

  case 364: /* template_option: KW_ON_ERROR '(' string ')'  */
#line 2265 "modules/afsocket/afsocket-grammar.y"
        {
          gint on_error;

          CHECK_ERROR(log_template_on_error_parse((yyvsp[-1].cptr), &on_error), (yylsp[-1]), "Invalid on-error() setting \"%s\"", (yyvsp[-1].cptr));
          free((yyvsp[-1].cptr));

          log_template_options_set_on_error(last_template_options, on_error);
        }
#line 7665 "modules/afsocket/afsocket-grammar.c"
    break;

  case 365: /* _template_ref_context_push: %empty  */
#line 2449 "modules/afsocket/afsocket-grammar.y"
                            { cfg_lexer_push_context(lexer, LL_CONTEXT_TEMPLATE_REF, NULL, "template reference"); }
#line 7671 "modules/afsocket/afsocket-grammar.c"
    break;

  case 366: /* _inner_dest_context_push: %empty  */
#line 2450 "modules/afsocket/afsocket-grammar.y"
                          { cfg_lexer_push_context(lexer, LL_CONTEXT_INNER_DEST, NULL, "within destination"); }
#line 7677 "modules/afsocket/afsocket-grammar.c"
    break;

  case 367: /* _inner_dest_context_pop: %empty  */
#line 2451 "modules/afsocket/afsocket-grammar.y"
                         { cfg_lexer_pop_context(lexer); }
#line 7683 "modules/afsocket/afsocket-grammar.c"
    break;

  case 368: /* _inner_src_context_push: %empty  */
#line 2452 "modules/afsocket/afsocket-grammar.y"
                         { cfg_lexer_push_context(lexer, LL_CONTEXT_INNER_SRC, NULL, "within source"); }
#line 7689 "modules/afsocket/afsocket-grammar.c"
    break;

  case 369: /* _inner_src_context_pop: %empty  */
#line 2453 "modules/afsocket/afsocket-grammar.y"
                        { cfg_lexer_pop_context(lexer); }
#line 7695 "modules/afsocket/afsocket-grammar.c"
    break;


#line 7699 "modules/afsocket/afsocket-grammar.c"

      default: break;
    }
  /* User semantic actions sometimes alter yychar, and that requires
     that yytoken be updated with the new translation.  We take the
     approach of translating immediately before every use of yytoken.
     One alternative is translating here after every semantic action,
     but that translation would be missed if the semantic action invokes
     YYABORT, YYACCEPT, or YYERROR immediately after altering yychar or
     if it invokes YYBACKUP.  In the case of YYABORT or YYACCEPT, an
     incorrect destructor might then be invoked immediately.  In the
     case of YYERROR or YYBACKUP, subsequent parser actions might lead
     to an incorrect destructor call or verbose syntax error message
     before the lookahead is translated.  */
  YY_SYMBOL_PRINT ("-> $$ =", YY_CAST (yysymbol_kind_t, yyr1[yyn]), &yyval, &yyloc);

  YYPOPSTACK (yylen);
  yylen = 0;

  *++yyvsp = yyval;
  *++yylsp = yyloc;

  /* Now 'shift' the result of the reduction.  Determine what state
     that goes to, based on the state we popped back to and the rule
     number reduced by.  */
  {
    const int yylhs = yyr1[yyn] - YYNTOKENS;
    const int yyi = yypgoto[yylhs] + *yyssp;
    yystate = (0 <= yyi && yyi <= YYLAST && yycheck[yyi] == *yyssp
               ? yytable[yyi]
               : yydefgoto[yylhs]);
  }

  goto yynewstate;


/*--------------------------------------.
| yyerrlab -- here on detecting error.  |
`--------------------------------------*/
yyerrlab:
  /* Make sure we have latest lookahead translation.  See comments at
     user semantic actions for why this is necessary.  */
  yytoken = yychar == AFSOCKET_EMPTY ? YYSYMBOL_YYEMPTY : YYTRANSLATE (yychar);
  /* If not already recovering from an error, report this error.  */
  if (!yyerrstatus)
    {
      ++yynerrs;
      {
        yypcontext_t yyctx
          = {yyssp, yytoken, &yylloc};
        char const *yymsgp = YY_("syntax error");
        int yysyntax_error_status;
        yysyntax_error_status = yysyntax_error (&yymsg_alloc, &yymsg, &yyctx);
        if (yysyntax_error_status == 0)
          yymsgp = yymsg;
        else if (yysyntax_error_status == -1)
          {
            if (yymsg != yymsgbuf)
              YYSTACK_FREE (yymsg);
            yymsg = YY_CAST (char *,
                             YYSTACK_ALLOC (YY_CAST (YYSIZE_T, yymsg_alloc)));
            if (yymsg)
              {
                yysyntax_error_status
                  = yysyntax_error (&yymsg_alloc, &yymsg, &yyctx);
                yymsgp = yymsg;
              }
            else
              {
                yymsg = yymsgbuf;
                yymsg_alloc = sizeof yymsgbuf;
                yysyntax_error_status = YYENOMEM;
              }
          }
        yyerror (&yylloc, lexer, instance, arg, yymsgp);
        if (yysyntax_error_status == YYENOMEM)
          YYNOMEM;
      }
    }

  yyerror_range[1] = yylloc;
  if (yyerrstatus == 3)
    {
      /* If just tried and failed to reuse lookahead token after an
         error, discard it.  */

      if (yychar <= AFSOCKET_EOF)
        {
          /* Return failure if at end of input.  */
          if (yychar == AFSOCKET_EOF)
            YYABORT;
        }
      else
        {
          yydestruct ("Error: discarding",
                      yytoken, &yylval, &yylloc, lexer, instance, arg);
          yychar = AFSOCKET_EMPTY;
        }
    }

  /* Else will try to reuse lookahead token after shifting the error
     token.  */
  goto yyerrlab1;


/*---------------------------------------------------.
| yyerrorlab -- error raised explicitly by YYERROR.  |
`---------------------------------------------------*/
yyerrorlab:
  /* Pacify compilers when the user code never invokes YYERROR and the
     label yyerrorlab therefore never appears in user code.  */
  if (0)
    YYERROR;
  ++yynerrs;

  /* Do not reclaim the symbols of the rule whose action triggered
     this YYERROR.  */
  YYPOPSTACK (yylen);
  yylen = 0;
  YY_STACK_PRINT (yyss, yyssp);
  yystate = *yyssp;
  goto yyerrlab1;


/*-------------------------------------------------------------.
| yyerrlab1 -- common code for both syntax error and YYERROR.  |
`-------------------------------------------------------------*/
yyerrlab1:
  yyerrstatus = 3;      /* Each real token shifted decrements this.  */

  /* Pop stack until we find a state that shifts the error token.  */
  for (;;)
    {
      yyn = yypact[yystate];
      if (!yypact_value_is_default (yyn))
        {
          yyn += YYSYMBOL_YYerror;
          if (0 <= yyn && yyn <= YYLAST && yycheck[yyn] == YYSYMBOL_YYerror)
            {
              yyn = yytable[yyn];
              if (0 < yyn)
                break;
            }
        }

      /* Pop the current state because it cannot handle the error token.  */
      if (yyssp == yyss)
        YYABORT;

      yyerror_range[1] = *yylsp;
      yydestruct ("Error: popping",
                  YY_ACCESSING_SYMBOL (yystate), yyvsp, yylsp, lexer, instance, arg);
      YYPOPSTACK (1);
      yystate = *yyssp;
      YY_STACK_PRINT (yyss, yyssp);
    }

  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
  *++yyvsp = yylval;
  YY_IGNORE_MAYBE_UNINITIALIZED_END

  yyerror_range[2] = yylloc;
  ++yylsp;
  YYLLOC_DEFAULT (*yylsp, yyerror_range, 2);

  /* Shift the error token.  */
  YY_SYMBOL_PRINT ("Shifting", YY_ACCESSING_SYMBOL (yyn), yyvsp, yylsp);

  yystate = yyn;
  goto yynewstate;


/*-------------------------------------.
| yyacceptlab -- YYACCEPT comes here.  |
`-------------------------------------*/
yyacceptlab:
  yyresult = 0;
  goto yyreturnlab;


/*-----------------------------------.
| yyabortlab -- YYABORT comes here.  |
`-----------------------------------*/
yyabortlab:
  yyresult = 1;
  goto yyreturnlab;


/*-----------------------------------------------------------.
| yyexhaustedlab -- YYNOMEM (memory exhaustion) comes here.  |
`-----------------------------------------------------------*/
yyexhaustedlab:
  yyerror (&yylloc, lexer, instance, arg, YY_("memory exhausted"));
  yyresult = 2;
  goto yyreturnlab;


/*----------------------------------------------------------.
| yyreturnlab -- parsing is finished, clean up and return.  |
`----------------------------------------------------------*/
yyreturnlab:
  if (yychar != AFSOCKET_EMPTY)
    {
      /* Make sure we have latest lookahead translation.  See comments at
         user semantic actions for why this is necessary.  */
      yytoken = YYTRANSLATE (yychar);
      yydestruct ("Cleanup: discarding lookahead",
                  yytoken, &yylval, &yylloc, lexer, instance, arg);
    }
  /* Do not reclaim the symbols of the rule whose action triggered
     this YYABORT or YYACCEPT.  */
  YYPOPSTACK (yylen);
  YY_STACK_PRINT (yyss, yyssp);
  while (yyssp != yyss)
    {
      yydestruct ("Cleanup: popping",
                  YY_ACCESSING_SYMBOL (+*yyssp), yyvsp, yylsp, lexer, instance, arg);
      YYPOPSTACK (1);
    }
#ifndef yyoverflow
  if (yyss != yyssa)
    YYSTACK_FREE (yyss);
#endif
  if (yymsg != yymsgbuf)
    YYSTACK_FREE (yymsg);
  return yyresult;
}

#line 2458 "modules/afsocket/afsocket-grammar.y"

