// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: clickhouse_grpc.proto
// Protobuf C++ Version: 7.35.1

#include "clickhouse_grpc.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
#ifdef PROTOBUF_MESSAGE_GLOBALS
namespace {
PROTOBUF_CONSTINIT ::google::protobuf::internal::ReflectionData
    file_reflection_data[] = {
        // ::clickhouse::grpc::NameAndType
        {&::_pbi::kDescriptorMethods, &::descriptor_table_clickhouse_5fgrpc_2eproto, /* tracker*/ nullptr,},
        // ::clickhouse::grpc::ExternalTable_SettingsEntry_DoNotUse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_clickhouse_5fgrpc_2eproto, /* tracker*/ nullptr,},
        // ::clickhouse::grpc::ExternalTable
        {&::_pbi::kDescriptorMethods, &::descriptor_table_clickhouse_5fgrpc_2eproto, /* tracker*/ nullptr,},
        // ::clickhouse::grpc::ObsoleteTransportCompression
        {&::_pbi::kDescriptorMethods, &::descriptor_table_clickhouse_5fgrpc_2eproto, /* tracker*/ nullptr,},
        // ::clickhouse::grpc::QueryInfo_SettingsEntry_DoNotUse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_clickhouse_5fgrpc_2eproto, /* tracker*/ nullptr,},
        // ::clickhouse::grpc::QueryInfo
        {&::_pbi::kDescriptorMethods, &::descriptor_table_clickhouse_5fgrpc_2eproto, /* tracker*/ nullptr,},
        // ::clickhouse::grpc::LogEntry
        {&::_pbi::kDescriptorMethods, &::descriptor_table_clickhouse_5fgrpc_2eproto, /* tracker*/ nullptr,},
        // ::clickhouse::grpc::Progress
        {&::_pbi::kDescriptorMethods, &::descriptor_table_clickhouse_5fgrpc_2eproto, /* tracker*/ nullptr,},
        // ::clickhouse::grpc::Stats
        {&::_pbi::kDescriptorMethods, &::descriptor_table_clickhouse_5fgrpc_2eproto, /* tracker*/ nullptr,},
        // ::clickhouse::grpc::Exception
        {&::_pbi::kDescriptorMethods, &::descriptor_table_clickhouse_5fgrpc_2eproto, /* tracker*/ nullptr,},
        // ::clickhouse::grpc::Result
        {&::_pbi::kDescriptorMethods, &::descriptor_table_clickhouse_5fgrpc_2eproto, /* tracker*/ nullptr,},
};
}  // namespace
#endif
namespace clickhouse {
namespace grpc {
class Stats::_Internal {
 public:
  using HasBits = decltype(::std::declval<Stats>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Stats, _impl_._has_bits_);
};

constexpr Stats::ParseTableT_ Stats::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Stats, _impl_._has_bits_),
      0, // no _extensions_
      7, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967168,  // skipmap
      offsetof(ParseTableT_, field_entries),
      7,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::clickhouse::grpc::Stats>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // uint64 rows = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Stats, _impl_.rows_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(Stats, _impl_.rows_)}},
      // uint64 blocks = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Stats, _impl_.blocks_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(Stats, _impl_.blocks_)}},
      // uint64 allocated_bytes = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Stats, _impl_.allocated_bytes_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(Stats, _impl_.allocated_bytes_)}},
      // bool applied_limit = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(Stats, _impl_.applied_limit_), 5>(),
       {32, 5, 0,
        PROTOBUF_FIELD_OFFSET(Stats, _impl_.applied_limit_)}},
      // uint64 rows_before_limit = 5;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Stats, _impl_.rows_before_limit_), 3>(),
       {40, 3, 0,
        PROTOBUF_FIELD_OFFSET(Stats, _impl_.rows_before_limit_)}},
      // bool applied_aggregation = 6;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(Stats, _impl_.applied_aggregation_), 6>(),
       {48, 6, 0,
        PROTOBUF_FIELD_OFFSET(Stats, _impl_.applied_aggregation_)}},
      // uint64 rows_before_aggregation = 7;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Stats, _impl_.rows_before_aggregation_), 4>(),
       {56, 4, 0,
        PROTOBUF_FIELD_OFFSET(Stats, _impl_.rows_before_aggregation_)}},
    }}, {{
      65535, 65535
    }}, {{
      // uint64 rows = 1;
      {PROTOBUF_FIELD_OFFSET(Stats, _impl_.rows_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint64 blocks = 2;
      {PROTOBUF_FIELD_OFFSET(Stats, _impl_.blocks_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint64 allocated_bytes = 3;
      {PROTOBUF_FIELD_OFFSET(Stats, _impl_.allocated_bytes_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // bool applied_limit = 4;
      {PROTOBUF_FIELD_OFFSET(Stats, _impl_.applied_limit_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // uint64 rows_before_limit = 5;
      {PROTOBUF_FIELD_OFFSET(Stats, _impl_.rows_before_limit_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // bool applied_aggregation = 6;
      {PROTOBUF_FIELD_OFFSET(Stats, _impl_.applied_aggregation_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // uint64 rows_before_aggregation = 7;
      {PROTOBUF_FIELD_OFFSET(Stats, _impl_.rows_before_aggregation_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr Stats::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        rows_{::uint64_t{0u}},
        blocks_{::uint64_t{0u}},
        allocated_bytes_{::uint64_t{0u}},
        rows_before_limit_{::uint64_t{0u}},
        rows_before_aggregation_{::uint64_t{0u}},
        applied_limit_{false},
        applied_aggregation_{false} {}

template <typename>
constexpr Stats::Stats(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Stats::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Stats(arena);
}
constexpr auto Stats::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Stats), alignof(Stats));
}
constexpr auto Stats::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Stats::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<Stats>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Stats::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<Stats>(), &Stats::ByteSizeLong,
              &Stats::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Stats, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[8],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_clickhouse_5fgrpc_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct StatsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr StatsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Stats_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Stats::InternalGenerateClassData_(
            _default, &Stats_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Stats>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~StatsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Stats _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Stats>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(StatsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST StatsGlobalsTypeInternal Stats_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Stats_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Stats_globals_.GetClassData();
#else
  return Stats_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
constexpr QueryInfo_SettingsEntry_DoNotUse::ParseTableT_ QueryInfo_SettingsEntry_DoNotUse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(QueryInfo_SettingsEntry_DoNotUse, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::DiscardEverythingFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::clickhouse::grpc::QueryInfo_SettingsEntry_DoNotUse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string value = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo_SettingsEntry_DoNotUse, _impl_.value_)}},
      // string key = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo_SettingsEntry_DoNotUse, _impl_.key_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string key = 1;
      {PROTOBUF_FIELD_OFFSET(QueryInfo_SettingsEntry_DoNotUse, _impl_.key_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string value = 2;
      {PROTOBUF_FIELD_OFFSET(QueryInfo_SettingsEntry_DoNotUse, _impl_.value_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\47\3\5\0\0\0\0\0"
      "clickhouse.grpc.QueryInfo.SettingsEntry"
      "key"
      "value"
    }},
  };
}

template <typename>
constexpr QueryInfo_SettingsEntry_DoNotUse::QueryInfo_SettingsEntry_DoNotUse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : QueryInfo_SettingsEntry_DoNotUse::MapEntry(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL QueryInfo_SettingsEntry_DoNotUse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) QueryInfo_SettingsEntry_DoNotUse(arena);
}
constexpr auto QueryInfo_SettingsEntry_DoNotUse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(QueryInfo_SettingsEntry_DoNotUse), alignof(QueryInfo_SettingsEntry_DoNotUse));
}
constexpr auto QueryInfo_SettingsEntry_DoNotUse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &QueryInfo_SettingsEntry_DoNotUse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<QueryInfo_SettingsEntry_DoNotUse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &QueryInfo_SettingsEntry_DoNotUse::SharedDtor,
          static_cast<void (::google::protobuf::MessageLite::*)()>(&QueryInfo_SettingsEntry_DoNotUse::ClearImpl),
              ::google::protobuf::Message::ByteSizeLongImpl, ::google::protobuf::Message::_InternalSerializeImpl
              ,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(QueryInfo_SettingsEntry_DoNotUse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[4],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_clickhouse_5fgrpc_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct QueryInfo_SettingsEntry_DoNotUseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr QueryInfo_SettingsEntry_DoNotUseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 QueryInfo_SettingsEntry_DoNotUse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(QueryInfo_SettingsEntry_DoNotUse::InternalGenerateClassData_(
            _default, &QueryInfo_SettingsEntry_DoNotUse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<QueryInfo_SettingsEntry_DoNotUse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~QueryInfo_SettingsEntry_DoNotUseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) QueryInfo_SettingsEntry_DoNotUse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<QueryInfo_SettingsEntry_DoNotUse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(QueryInfo_SettingsEntry_DoNotUseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST QueryInfo_SettingsEntry_DoNotUseGlobalsTypeInternal QueryInfo_SettingsEntry_DoNotUse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* QueryInfo_SettingsEntry_DoNotUse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return QueryInfo_SettingsEntry_DoNotUse_globals_.GetClassData();
#else
  return QueryInfo_SettingsEntry_DoNotUse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Progress::_Internal {
 public:
  using HasBits = decltype(::std::declval<Progress>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Progress, _impl_._has_bits_);
};

constexpr Progress::ParseTableT_ Progress::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Progress, _impl_._has_bits_),
      0, // no _extensions_
      5, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967264,  // skipmap
      offsetof(ParseTableT_, field_entries),
      5,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::clickhouse::grpc::Progress>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // uint64 read_rows = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Progress, _impl_.read_rows_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(Progress, _impl_.read_rows_)}},
      // uint64 read_bytes = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Progress, _impl_.read_bytes_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(Progress, _impl_.read_bytes_)}},
      // uint64 total_rows_to_read = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Progress, _impl_.total_rows_to_read_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(Progress, _impl_.total_rows_to_read_)}},
      // uint64 written_rows = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Progress, _impl_.written_rows_), 3>(),
       {32, 3, 0,
        PROTOBUF_FIELD_OFFSET(Progress, _impl_.written_rows_)}},
      // uint64 written_bytes = 5;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Progress, _impl_.written_bytes_), 4>(),
       {40, 4, 0,
        PROTOBUF_FIELD_OFFSET(Progress, _impl_.written_bytes_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // uint64 read_rows = 1;
      {PROTOBUF_FIELD_OFFSET(Progress, _impl_.read_rows_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint64 read_bytes = 2;
      {PROTOBUF_FIELD_OFFSET(Progress, _impl_.read_bytes_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint64 total_rows_to_read = 3;
      {PROTOBUF_FIELD_OFFSET(Progress, _impl_.total_rows_to_read_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint64 written_rows = 4;
      {PROTOBUF_FIELD_OFFSET(Progress, _impl_.written_rows_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint64 written_bytes = 5;
      {PROTOBUF_FIELD_OFFSET(Progress, _impl_.written_bytes_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr Progress::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        read_rows_{::uint64_t{0u}},
        read_bytes_{::uint64_t{0u}},
        total_rows_to_read_{::uint64_t{0u}},
        written_rows_{::uint64_t{0u}},
        written_bytes_{::uint64_t{0u}} {}

template <typename>
constexpr Progress::Progress(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Progress::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Progress(arena);
}
constexpr auto Progress::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Progress), alignof(Progress));
}
constexpr auto Progress::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Progress::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<Progress>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Progress::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<Progress>(), &Progress::ByteSizeLong,
              &Progress::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Progress, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[7],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_clickhouse_5fgrpc_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ProgressGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ProgressGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Progress_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Progress::InternalGenerateClassData_(
            _default, &Progress_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Progress>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ProgressGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Progress _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Progress>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ProgressGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ProgressGlobalsTypeInternal Progress_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Progress_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Progress_globals_.GetClassData();
#else
  return Progress_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ObsoleteTransportCompression::_Internal {
 public:
  using HasBits = decltype(::std::declval<ObsoleteTransportCompression>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ObsoleteTransportCompression, _impl_._has_bits_);
};

constexpr ObsoleteTransportCompression::ParseTableT_ ObsoleteTransportCompression::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ObsoleteTransportCompression, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::clickhouse::grpc::ObsoleteTransportCompression>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .clickhouse.grpc.ObsoleteTransportCompression.CompressionLevel level = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ObsoleteTransportCompression, _impl_.level_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(ObsoleteTransportCompression, _impl_.level_)}},
      // .clickhouse.grpc.ObsoleteTransportCompression.CompressionAlgorithm algorithm = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ObsoleteTransportCompression, _impl_.algorithm_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(ObsoleteTransportCompression, _impl_.algorithm_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .clickhouse.grpc.ObsoleteTransportCompression.CompressionAlgorithm algorithm = 1;
      {PROTOBUF_FIELD_OFFSET(ObsoleteTransportCompression, _impl_.algorithm_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // .clickhouse.grpc.ObsoleteTransportCompression.CompressionLevel level = 2;
      {PROTOBUF_FIELD_OFFSET(ObsoleteTransportCompression, _impl_.level_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr ObsoleteTransportCompression::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        algorithm_{static_cast< ::clickhouse::grpc::ObsoleteTransportCompression_CompressionAlgorithm >(0)},
        level_{static_cast< ::clickhouse::grpc::ObsoleteTransportCompression_CompressionLevel >(0)} {}

template <typename>
constexpr ObsoleteTransportCompression::ObsoleteTransportCompression(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ObsoleteTransportCompression::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ObsoleteTransportCompression(arena);
}
constexpr auto ObsoleteTransportCompression::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ObsoleteTransportCompression), alignof(ObsoleteTransportCompression));
}
constexpr auto ObsoleteTransportCompression::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ObsoleteTransportCompression::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ObsoleteTransportCompression>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ObsoleteTransportCompression::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ObsoleteTransportCompression>(), &ObsoleteTransportCompression::ByteSizeLong,
              &ObsoleteTransportCompression::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ObsoleteTransportCompression, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[3],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_clickhouse_5fgrpc_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ObsoleteTransportCompressionGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ObsoleteTransportCompressionGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ObsoleteTransportCompression_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ObsoleteTransportCompression::InternalGenerateClassData_(
            _default, &ObsoleteTransportCompression_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ObsoleteTransportCompression>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ObsoleteTransportCompressionGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ObsoleteTransportCompression _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ObsoleteTransportCompression>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ObsoleteTransportCompressionGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ObsoleteTransportCompressionGlobalsTypeInternal ObsoleteTransportCompression_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ObsoleteTransportCompression_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ObsoleteTransportCompression_globals_.GetClassData();
#else
  return ObsoleteTransportCompression_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class NameAndType::_Internal {
 public:
  using HasBits = decltype(::std::declval<NameAndType>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(NameAndType, _impl_._has_bits_);
};

constexpr NameAndType::ParseTableT_ NameAndType::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(NameAndType, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::clickhouse::grpc::NameAndType>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string type = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(NameAndType, _impl_.type_)}},
      // string name = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(NameAndType, _impl_.name_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string name = 1;
      {PROTOBUF_FIELD_OFFSET(NameAndType, _impl_.name_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string type = 2;
      {PROTOBUF_FIELD_OFFSET(NameAndType, _impl_.type_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\33\4\4\0\0\0\0\0"
      "clickhouse.grpc.NameAndType"
      "name"
      "type"
    }},
  };
}


inline constexpr NameAndType::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        type_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr NameAndType::NameAndType(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL NameAndType::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) NameAndType(arena);
}
constexpr auto NameAndType::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(NameAndType), alignof(NameAndType));
}
constexpr auto NameAndType::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &NameAndType::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<NameAndType>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &NameAndType::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<NameAndType>(), &NameAndType::ByteSizeLong,
              &NameAndType::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(NameAndType, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[0],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_clickhouse_5fgrpc_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct NameAndTypeGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr NameAndTypeGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 NameAndType_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(NameAndType::InternalGenerateClassData_(
            _default, &NameAndType_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<NameAndType>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~NameAndTypeGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) NameAndType _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<NameAndType>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(NameAndTypeGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST NameAndTypeGlobalsTypeInternal NameAndType_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* NameAndType_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return NameAndType_globals_.GetClassData();
#else
  return NameAndType_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class LogEntry::_Internal {
 public:
  using HasBits = decltype(::std::declval<LogEntry>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(LogEntry, _impl_._has_bits_);
};

constexpr LogEntry::ParseTableT_ LogEntry::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(LogEntry, _impl_._has_bits_),
      0, // no _extensions_
      7, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967168,  // skipmap
      offsetof(ParseTableT_, field_entries),
      7,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::clickhouse::grpc::LogEntry>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // uint32 time = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(LogEntry, _impl_.time_), 3>(),
       {8, 3, 0,
        PROTOBUF_FIELD_OFFSET(LogEntry, _impl_.time_)}},
      // uint32 time_microseconds = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(LogEntry, _impl_.time_microseconds_), 4>(),
       {16, 4, 0,
        PROTOBUF_FIELD_OFFSET(LogEntry, _impl_.time_microseconds_)}},
      // uint64 thread_id = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(LogEntry, _impl_.thread_id_), 5>(),
       {24, 5, 0,
        PROTOBUF_FIELD_OFFSET(LogEntry, _impl_.thread_id_)}},
      // string query_id = 4;
      {::_pbi::TcParser::FastUS1,
       {34, 0, 0,
        PROTOBUF_FIELD_OFFSET(LogEntry, _impl_.query_id_)}},
      // .clickhouse.grpc.LogsLevel level = 5;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(LogEntry, _impl_.level_), 6>(),
       {40, 6, 0,
        PROTOBUF_FIELD_OFFSET(LogEntry, _impl_.level_)}},
      // string source = 6;
      {::_pbi::TcParser::FastUS1,
       {50, 1, 0,
        PROTOBUF_FIELD_OFFSET(LogEntry, _impl_.source_)}},
      // string text = 7;
      {::_pbi::TcParser::FastUS1,
       {58, 2, 0,
        PROTOBUF_FIELD_OFFSET(LogEntry, _impl_.text_)}},
    }}, {{
      65535, 65535
    }}, {{
      // uint32 time = 1;
      {PROTOBUF_FIELD_OFFSET(LogEntry, _impl_.time_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
      // uint32 time_microseconds = 2;
      {PROTOBUF_FIELD_OFFSET(LogEntry, _impl_.time_microseconds_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
      // uint64 thread_id = 3;
      {PROTOBUF_FIELD_OFFSET(LogEntry, _impl_.thread_id_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // string query_id = 4;
      {PROTOBUF_FIELD_OFFSET(LogEntry, _impl_.query_id_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .clickhouse.grpc.LogsLevel level = 5;
      {PROTOBUF_FIELD_OFFSET(LogEntry, _impl_.level_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // string source = 6;
      {PROTOBUF_FIELD_OFFSET(LogEntry, _impl_.source_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string text = 7;
      {PROTOBUF_FIELD_OFFSET(LogEntry, _impl_.text_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\30\0\0\0\10\0\6\4"
      "clickhouse.grpc.LogEntry"
      "query_id"
      "source"
      "text"
    }},
  };
}


inline constexpr LogEntry::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        query_id_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        source_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        text_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        time_{0u},
        time_microseconds_{0u},
        thread_id_{::uint64_t{0u}},
        level_{static_cast< ::clickhouse::grpc::LogsLevel >(0)} {}

template <typename>
constexpr LogEntry::LogEntry(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL LogEntry::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) LogEntry(arena);
}
constexpr auto LogEntry::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(LogEntry), alignof(LogEntry));
}
constexpr auto LogEntry::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &LogEntry::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<LogEntry>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &LogEntry::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<LogEntry>(), &LogEntry::ByteSizeLong,
              &LogEntry::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(LogEntry, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[6],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_clickhouse_5fgrpc_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct LogEntryGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr LogEntryGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 LogEntry_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(LogEntry::InternalGenerateClassData_(
            _default, &LogEntry_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<LogEntry>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~LogEntryGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) LogEntry _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<LogEntry>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(LogEntryGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST LogEntryGlobalsTypeInternal LogEntry_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* LogEntry_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return LogEntry_globals_.GetClassData();
#else
  return LogEntry_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
constexpr ExternalTable_SettingsEntry_DoNotUse::ParseTableT_ ExternalTable_SettingsEntry_DoNotUse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ExternalTable_SettingsEntry_DoNotUse, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::DiscardEverythingFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::clickhouse::grpc::ExternalTable_SettingsEntry_DoNotUse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string value = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(ExternalTable_SettingsEntry_DoNotUse, _impl_.value_)}},
      // string key = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ExternalTable_SettingsEntry_DoNotUse, _impl_.key_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string key = 1;
      {PROTOBUF_FIELD_OFFSET(ExternalTable_SettingsEntry_DoNotUse, _impl_.key_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string value = 2;
      {PROTOBUF_FIELD_OFFSET(ExternalTable_SettingsEntry_DoNotUse, _impl_.value_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\53\3\5\0\0\0\0\0"
      "clickhouse.grpc.ExternalTable.SettingsEntry"
      "key"
      "value"
    }},
  };
}

template <typename>
constexpr ExternalTable_SettingsEntry_DoNotUse::ExternalTable_SettingsEntry_DoNotUse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ExternalTable_SettingsEntry_DoNotUse::MapEntry(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL ExternalTable_SettingsEntry_DoNotUse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ExternalTable_SettingsEntry_DoNotUse(arena);
}
constexpr auto ExternalTable_SettingsEntry_DoNotUse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ExternalTable_SettingsEntry_DoNotUse), alignof(ExternalTable_SettingsEntry_DoNotUse));
}
constexpr auto ExternalTable_SettingsEntry_DoNotUse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ExternalTable_SettingsEntry_DoNotUse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ExternalTable_SettingsEntry_DoNotUse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ExternalTable_SettingsEntry_DoNotUse::SharedDtor,
          static_cast<void (::google::protobuf::MessageLite::*)()>(&ExternalTable_SettingsEntry_DoNotUse::ClearImpl),
              ::google::protobuf::Message::ByteSizeLongImpl, ::google::protobuf::Message::_InternalSerializeImpl
              ,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ExternalTable_SettingsEntry_DoNotUse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[1],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_clickhouse_5fgrpc_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ExternalTable_SettingsEntry_DoNotUseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ExternalTable_SettingsEntry_DoNotUseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ExternalTable_SettingsEntry_DoNotUse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ExternalTable_SettingsEntry_DoNotUse::InternalGenerateClassData_(
            _default, &ExternalTable_SettingsEntry_DoNotUse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ExternalTable_SettingsEntry_DoNotUse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ExternalTable_SettingsEntry_DoNotUseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ExternalTable_SettingsEntry_DoNotUse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ExternalTable_SettingsEntry_DoNotUse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ExternalTable_SettingsEntry_DoNotUseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ExternalTable_SettingsEntry_DoNotUseGlobalsTypeInternal ExternalTable_SettingsEntry_DoNotUse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ExternalTable_SettingsEntry_DoNotUse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ExternalTable_SettingsEntry_DoNotUse_globals_.GetClassData();
#else
  return ExternalTable_SettingsEntry_DoNotUse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Exception::_Internal {
 public:
  using HasBits = decltype(::std::declval<Exception>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Exception, _impl_._has_bits_);
};

constexpr Exception::ParseTableT_ Exception::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Exception, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::clickhouse::grpc::Exception>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string stack_trace = 4;
      {::_pbi::TcParser::FastUS1,
       {34, 2, 0,
        PROTOBUF_FIELD_OFFSET(Exception, _impl_.stack_trace_)}},
      // int32 code = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Exception, _impl_.code_), 3>(),
       {8, 3, 0,
        PROTOBUF_FIELD_OFFSET(Exception, _impl_.code_)}},
      // string name = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 0, 0,
        PROTOBUF_FIELD_OFFSET(Exception, _impl_.name_)}},
      // string display_text = 3;
      {::_pbi::TcParser::FastUS1,
       {26, 1, 0,
        PROTOBUF_FIELD_OFFSET(Exception, _impl_.display_text_)}},
    }}, {{
      65535, 65535
    }}, {{
      // int32 code = 1;
      {PROTOBUF_FIELD_OFFSET(Exception, _impl_.code_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // string name = 2;
      {PROTOBUF_FIELD_OFFSET(Exception, _impl_.name_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string display_text = 3;
      {PROTOBUF_FIELD_OFFSET(Exception, _impl_.display_text_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string stack_trace = 4;
      {PROTOBUF_FIELD_OFFSET(Exception, _impl_.stack_trace_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\31\0\4\14\13\0\0\0"
      "clickhouse.grpc.Exception"
      "name"
      "display_text"
      "stack_trace"
    }},
  };
}


inline constexpr Exception::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        display_text_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        stack_trace_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        code_{0} {}

template <typename>
constexpr Exception::Exception(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Exception::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Exception(arena);
}
constexpr auto Exception::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Exception), alignof(Exception));
}
constexpr auto Exception::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Exception::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<Exception>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Exception::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<Exception>(), &Exception::ByteSizeLong,
              &Exception::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Exception, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[9],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_clickhouse_5fgrpc_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ExceptionGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ExceptionGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Exception_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Exception::InternalGenerateClassData_(
            _default, &Exception_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Exception>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ExceptionGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Exception _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Exception>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ExceptionGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ExceptionGlobalsTypeInternal Exception_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Exception_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Exception_globals_.GetClassData();
#else
  return Exception_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Result::_Internal {
 public:
  using HasBits = decltype(::std::declval<Result>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Result, _impl_._has_bits_);
};

constexpr Result::ParseTableT_ Result::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Result, _impl_._has_bits_),
      0, // no _extensions_
      12, 120,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294963200,  // skipmap
      offsetof(ParseTableT_, field_entries),
      12,  // num_field_entries
      5,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::clickhouse::grpc::Result>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // bytes output = 1;
      {::_pbi::TcParser::FastBS1,
       {10, 2, 0,
        PROTOBUF_FIELD_OFFSET(Result, _impl_.output_)}},
      // bytes totals = 2;
      {::_pbi::TcParser::FastBS1,
       {18, 3, 0,
        PROTOBUF_FIELD_OFFSET(Result, _impl_.totals_)}},
      // bytes extremes = 3;
      {::_pbi::TcParser::FastBS1,
       {26, 4, 0,
        PROTOBUF_FIELD_OFFSET(Result, _impl_.extremes_)}},
      // repeated .clickhouse.grpc.LogEntry logs = 4;
      {::_pbi::TcParser::FastMtR1,
       {34, 0, 0,
        PROTOBUF_FIELD_OFFSET(Result, _impl_.logs_)}},
      // .clickhouse.grpc.Progress progress = 5;
      {::_pbi::TcParser::FastMtS1,
       {42, 8, 1,
        PROTOBUF_FIELD_OFFSET(Result, _impl_.progress_)}},
      // .clickhouse.grpc.Stats stats = 6;
      {::_pbi::TcParser::FastMtS1,
       {50, 9, 2,
        PROTOBUF_FIELD_OFFSET(Result, _impl_.stats_)}},
      // .clickhouse.grpc.Exception exception = 7;
      {::_pbi::TcParser::FastMtS1,
       {58, 10, 3,
        PROTOBUF_FIELD_OFFSET(Result, _impl_.exception_)}},
      // bool cancelled = 8;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(Result, _impl_.cancelled_), 11>(),
       {64, 11, 0,
        PROTOBUF_FIELD_OFFSET(Result, _impl_.cancelled_)}},
      // string query_id = 9;
      {::_pbi::TcParser::FastUS1,
       {74, 5, 0,
        PROTOBUF_FIELD_OFFSET(Result, _impl_.query_id_)}},
      // string time_zone = 10;
      {::_pbi::TcParser::FastUS1,
       {82, 6, 0,
        PROTOBUF_FIELD_OFFSET(Result, _impl_.time_zone_)}},
      // string output_format = 11;
      {::_pbi::TcParser::FastUS1,
       {90, 7, 0,
        PROTOBUF_FIELD_OFFSET(Result, _impl_.output_format_)}},
      // repeated .clickhouse.grpc.NameAndType output_columns = 12;
      {::_pbi::TcParser::FastMtR1,
       {98, 1, 4,
        PROTOBUF_FIELD_OFFSET(Result, _impl_.output_columns_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // bytes output = 1;
      {PROTOBUF_FIELD_OFFSET(Result, _impl_.output_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
      // bytes totals = 2;
      {PROTOBUF_FIELD_OFFSET(Result, _impl_.totals_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
      // bytes extremes = 3;
      {PROTOBUF_FIELD_OFFSET(Result, _impl_.extremes_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
      // repeated .clickhouse.grpc.LogEntry logs = 4;
      {PROTOBUF_FIELD_OFFSET(Result, _impl_.logs_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
      // .clickhouse.grpc.Progress progress = 5;
      {PROTOBUF_FIELD_OFFSET(Result, _impl_.progress_), _Internal::kHasBitsOffset + 8, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .clickhouse.grpc.Stats stats = 6;
      {PROTOBUF_FIELD_OFFSET(Result, _impl_.stats_), _Internal::kHasBitsOffset + 9, 2, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .clickhouse.grpc.Exception exception = 7;
      {PROTOBUF_FIELD_OFFSET(Result, _impl_.exception_), _Internal::kHasBitsOffset + 10, 3, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // bool cancelled = 8;
      {PROTOBUF_FIELD_OFFSET(Result, _impl_.cancelled_), _Internal::kHasBitsOffset + 11, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // string query_id = 9;
      {PROTOBUF_FIELD_OFFSET(Result, _impl_.query_id_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string time_zone = 10;
      {PROTOBUF_FIELD_OFFSET(Result, _impl_.time_zone_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string output_format = 11;
      {PROTOBUF_FIELD_OFFSET(Result, _impl_.output_format_), _Internal::kHasBitsOffset + 7, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // repeated .clickhouse.grpc.NameAndType output_columns = 12;
      {PROTOBUF_FIELD_OFFSET(Result, _impl_.output_columns_), _Internal::kHasBitsOffset + 1, 4, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::clickhouse::grpc::LogEntry>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::clickhouse::grpc::LogEntry_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::clickhouse::grpc::Progress>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::clickhouse::grpc::Progress_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::clickhouse::grpc::Stats>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::clickhouse::grpc::Stats_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::clickhouse::grpc::Exception>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::clickhouse::grpc::Exception_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::clickhouse::grpc::NameAndType>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::clickhouse::grpc::NameAndType_globals_},
        #endif
    }},
    {{
      "\26\0\0\0\0\0\0\0\0\10\11\15\0\0\0\0"
      "clickhouse.grpc.Result"
      "query_id"
      "time_zone"
      "output_format"
    }},
  };
}


inline constexpr Result::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        logs_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::clickhouse::grpc::Result,
            PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Result, _impl_.logs_)>()
         }
        ,
        output_columns_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::clickhouse::grpc::Result,
            PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Result, _impl_.output_columns_)>()
         }
        ,
        output_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        totals_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        extremes_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        query_id_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        time_zone_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        output_format_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        progress_{nullptr},
        stats_{nullptr},
        exception_{nullptr},
        cancelled_{false} {}

template <typename>
constexpr Result::Result(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Result::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Result(arena);
}
constexpr auto Result::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Result), alignof(Result));
}
constexpr auto Result::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Result::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<Result>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Result::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<Result>(), &Result::ByteSizeLong,
              &Result::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Result, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[10],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_clickhouse_5fgrpc_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ResultGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ResultGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Result_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Result::InternalGenerateClassData_(
            _default, &Result_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Result>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ResultGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Result _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Result>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ResultGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ResultGlobalsTypeInternal Result_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Result_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Result_globals_.GetClassData();
#else
  return Result_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ExternalTable::_Internal {
 public:
  using HasBits = decltype(::std::declval<ExternalTable>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ExternalTable, _impl_._has_bits_);
};

constexpr ExternalTable::ParseTableT_ ExternalTable::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ExternalTable, _impl_._has_bits_),
      0, // no _extensions_
      6, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967232,  // skipmap
      offsetof(ParseTableT_, field_entries),
      6,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::clickhouse::grpc::ExternalTable>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string name = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 1, 0,
        PROTOBUF_FIELD_OFFSET(ExternalTable, _impl_.name_)}},
      // repeated .clickhouse.grpc.NameAndType columns = 2;
      {::_pbi::TcParser::FastMtR1,
       {18, 0, 0,
        PROTOBUF_FIELD_OFFSET(ExternalTable, _impl_.columns_)}},
      // bytes data = 3;
      {::_pbi::TcParser::FastBS1,
       {26, 2, 0,
        PROTOBUF_FIELD_OFFSET(ExternalTable, _impl_.data_)}},
      // string format = 4;
      {::_pbi::TcParser::FastUS1,
       {34, 3, 0,
        PROTOBUF_FIELD_OFFSET(ExternalTable, _impl_.format_)}},
      {::_pbi::TcParser::MiniParse, {}},
      // string compression_type = 6;
      {::_pbi::TcParser::FastUS1,
       {50, 4, 0,
        PROTOBUF_FIELD_OFFSET(ExternalTable, _impl_.compression_type_)}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // string name = 1;
      {PROTOBUF_FIELD_OFFSET(ExternalTable, _impl_.name_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // repeated .clickhouse.grpc.NameAndType columns = 2;
      {PROTOBUF_FIELD_OFFSET(ExternalTable, _impl_.columns_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
      // bytes data = 3;
      {PROTOBUF_FIELD_OFFSET(ExternalTable, _impl_.data_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
      // string format = 4;
      {PROTOBUF_FIELD_OFFSET(ExternalTable, _impl_.format_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // map<string, string> settings = 5;
      {PROTOBUF_FIELD_OFFSET(ExternalTable, _impl_.settings_), _Internal::kHasBitsOffset + 5, 1, (0 | ::_fl::kFcRepeated | ::_fl::kMap)},
      // string compression_type = 6;
      {PROTOBUF_FIELD_OFFSET(ExternalTable, _impl_.compression_type_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::clickhouse::grpc::NameAndType>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::clickhouse::grpc::NameAndType_globals_},
        #endif
        {::_pbi::TcParser::GetMapAuxInfo(
            1, 0, 9, 9, 0)},
    }},
    {{
      "\35\4\0\0\6\10\20\0"
      "clickhouse.grpc.ExternalTable"
      "name"
      "format"
      "settings"
      "compression_type"
    }},
  };
}


inline constexpr ExternalTable::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        columns_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::clickhouse::grpc::ExternalTable,
            PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::ExternalTable, _impl_.columns_)>()
         }
        ,
        name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        data_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        format_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        compression_type_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        settings_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::clickhouse::grpc::ExternalTable,
            PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::ExternalTable, _impl_.settings_)>()
         }
     {}

template <typename>
constexpr ExternalTable::ExternalTable(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ExternalTable::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ExternalTable(arena);
}
constexpr auto ExternalTable::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ExternalTable), alignof(ExternalTable));
}
constexpr auto ExternalTable::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ExternalTable::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ExternalTable>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ExternalTable::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ExternalTable>(), &ExternalTable::ByteSizeLong,
              &ExternalTable::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ExternalTable, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[2],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_clickhouse_5fgrpc_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ExternalTableGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ExternalTableGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ExternalTable_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ExternalTable::InternalGenerateClassData_(
            _default, &ExternalTable_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ExternalTable>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ExternalTableGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ExternalTable _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ExternalTable>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ExternalTableGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ExternalTableGlobalsTypeInternal ExternalTable_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ExternalTable_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ExternalTable_globals_.GetClassData();
#else
  return ExternalTable_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class QueryInfo::_Internal {
 public:
  using HasBits = decltype(::std::declval<QueryInfo>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_._has_bits_);
};

constexpr QueryInfo::ParseTableT_ QueryInfo::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_._has_bits_),
      0, // no _extensions_
      25, 248,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4261412864,  // skipmap
      offsetof(ParseTableT_, field_entries),
      25,  // num_field_entries
      3,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::clickhouse::grpc::QueryInfo>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string query = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 1, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.query_)}},
      // string query_id = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 2, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.query_id_)}},
      {::_pbi::TcParser::MiniParse, {}},
      // string database = 4;
      {::_pbi::TcParser::FastUS1,
       {34, 3, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.database_)}},
      // bytes input_data = 5;
      {::_pbi::TcParser::FastBS1,
       {42, 4, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.input_data_)}},
      // bytes input_data_delimiter = 6;
      {::_pbi::TcParser::FastBS1,
       {50, 5, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.input_data_delimiter_)}},
      // string output_format = 7;
      {::_pbi::TcParser::FastUS1,
       {58, 6, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.output_format_)}},
      // repeated .clickhouse.grpc.ExternalTable external_tables = 8;
      {::_pbi::TcParser::FastMtR1,
       {66, 0, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.external_tables_)}},
      // string user_name = 9;
      {::_pbi::TcParser::FastUS1,
       {74, 7, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.user_name_)}},
      // string password = 10;
      {::_pbi::TcParser::FastUS1,
       {82, 8, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.password_)}},
      // string quota = 11;
      {::_pbi::TcParser::FastUS1,
       {90, 9, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.quota_)}},
      // string session_id = 12;
      {::_pbi::TcParser::FastUS1,
       {98, 10, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.session_id_)}},
      // bool session_check = 13;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(QueryInfo, _impl_.session_check_), 18>(),
       {104, 18, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.session_check_)}},
      // uint32 session_timeout = 14;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(QueryInfo, _impl_.session_timeout_), 17>(),
       {112, 17, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.session_timeout_)}},
      // bool cancel = 15;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(QueryInfo, _impl_.cancel_), 19>(),
       {120, 19, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.cancel_)}},
      // bool next_query_info = 16;
      {::_pbi::TcParser::FastV8S2,
       {384, 20, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.next_query_info_)}},
      // .clickhouse.grpc.ObsoleteTransportCompression obsolete_result_compression = 17;
      {::_pbi::TcParser::FastMtS2,
       {394, 16, 1,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.obsolete_result_compression_)}},
      // string obsolete_compression_type = 18;
      {::_pbi::TcParser::FastUS2,
       {402, 11, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.obsolete_compression_type_)}},
      // int32 output_compression_level = 19;
      {::_pbi::TcParser::FastV32S2,
       {408, 22, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.output_compression_level_)}},
      // string input_compression_type = 20;
      {::_pbi::TcParser::FastUS2,
       {418, 12, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.input_compression_type_)}},
      // string output_compression_type = 21;
      {::_pbi::TcParser::FastUS2,
       {426, 13, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.output_compression_type_)}},
      // string transport_compression_type = 22;
      {::_pbi::TcParser::FastUS2,
       {434, 14, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.transport_compression_type_)}},
      // int32 transport_compression_level = 23;
      {::_pbi::TcParser::FastV32S2,
       {440, 23, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.transport_compression_level_)}},
      // bool send_output_columns = 24;
      {::_pbi::TcParser::FastV8S2,
       {448, 21, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.send_output_columns_)}},
      // string jwt = 25;
      {::_pbi::TcParser::FastUS2,
       {458, 15, 0,
        PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.jwt_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // string query = 1;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.query_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string query_id = 2;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.query_id_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // map<string, string> settings = 3;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.settings_), _Internal::kHasBitsOffset + 24, 2, (0 | ::_fl::kFcRepeated | ::_fl::kMap)},
      // string database = 4;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.database_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // bytes input_data = 5;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.input_data_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
      // bytes input_data_delimiter = 6;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.input_data_delimiter_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
      // string output_format = 7;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.output_format_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // repeated .clickhouse.grpc.ExternalTable external_tables = 8;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.external_tables_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
      // string user_name = 9;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.user_name_), _Internal::kHasBitsOffset + 7, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string password = 10;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.password_), _Internal::kHasBitsOffset + 8, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string quota = 11;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.quota_), _Internal::kHasBitsOffset + 9, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string session_id = 12;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.session_id_), _Internal::kHasBitsOffset + 10, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // bool session_check = 13;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.session_check_), _Internal::kHasBitsOffset + 18, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // uint32 session_timeout = 14;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.session_timeout_), _Internal::kHasBitsOffset + 17, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
      // bool cancel = 15;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.cancel_), _Internal::kHasBitsOffset + 19, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // bool next_query_info = 16;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.next_query_info_), _Internal::kHasBitsOffset + 20, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // .clickhouse.grpc.ObsoleteTransportCompression obsolete_result_compression = 17;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.obsolete_result_compression_), _Internal::kHasBitsOffset + 16, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // string obsolete_compression_type = 18;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.obsolete_compression_type_), _Internal::kHasBitsOffset + 11, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // int32 output_compression_level = 19;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.output_compression_level_), _Internal::kHasBitsOffset + 22, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // string input_compression_type = 20;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.input_compression_type_), _Internal::kHasBitsOffset + 12, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string output_compression_type = 21;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.output_compression_type_), _Internal::kHasBitsOffset + 13, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string transport_compression_type = 22;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.transport_compression_type_), _Internal::kHasBitsOffset + 14, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // int32 transport_compression_level = 23;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.transport_compression_level_), _Internal::kHasBitsOffset + 23, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // bool send_output_columns = 24;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.send_output_columns_), _Internal::kHasBitsOffset + 21, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // string jwt = 25;
      {PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.jwt_), _Internal::kHasBitsOffset + 15, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::clickhouse::grpc::ExternalTable>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::clickhouse::grpc::ExternalTable_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::clickhouse::grpc::ObsoleteTransportCompression>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::clickhouse::grpc::ObsoleteTransportCompression_globals_},
        #endif
        {::_pbi::TcParser::GetMapAuxInfo(
            1, 0, 9, 9, 0)},
    }},
    {{
      "\31\5\10\10\10\0\0\15\0\11\10\5\12\0\0\0\0\0\31\0\26\27\32\0\0\3\0\0\0\0\0\0"
      "clickhouse.grpc.QueryInfo"
      "query"
      "query_id"
      "settings"
      "database"
      "output_format"
      "user_name"
      "password"
      "quota"
      "session_id"
      "obsolete_compression_type"
      "input_compression_type"
      "output_compression_type"
      "transport_compression_type"
      "jwt"
    }},
  };
}


inline constexpr QueryInfo::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        external_tables_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::clickhouse::grpc::QueryInfo,
            PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.external_tables_)>()
         }
        ,
        query_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        query_id_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        database_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        input_data_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        input_data_delimiter_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        output_format_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        user_name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        password_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        quota_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        session_id_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        obsolete_compression_type_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        input_compression_type_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        output_compression_type_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        transport_compression_type_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        jwt_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        obsolete_result_compression_{nullptr},
        session_timeout_{0u},
        session_check_{false},
        cancel_{false},
        next_query_info_{false},
        send_output_columns_{false},
        output_compression_level_{0},
        transport_compression_level_{0},
        settings_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::clickhouse::grpc::QueryInfo,
            PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.settings_)>()
         }
     {}

template <typename>
constexpr QueryInfo::QueryInfo(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL QueryInfo::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) QueryInfo(arena);
}
constexpr auto QueryInfo::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(QueryInfo), alignof(QueryInfo));
}
constexpr auto QueryInfo::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &QueryInfo::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<QueryInfo>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &QueryInfo::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<QueryInfo>(), &QueryInfo::ByteSizeLong,
              &QueryInfo::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[5],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_clickhouse_5fgrpc_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct QueryInfoGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr QueryInfoGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 QueryInfo_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(QueryInfo::InternalGenerateClassData_(
            _default, &QueryInfo_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<QueryInfo>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~QueryInfoGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) QueryInfo _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<QueryInfo>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(QueryInfoGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST QueryInfoGlobalsTypeInternal QueryInfo_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* QueryInfo_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return QueryInfo_globals_.GetClassData();
#else
  return QueryInfo_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
}  // namespace grpc
}  // namespace clickhouse
static const ::_pb::EnumDescriptor* PROTOBUF_NONNULL
    file_level_enum_descriptors_clickhouse_5fgrpc_2eproto[3];
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_clickhouse_5fgrpc_2eproto = nullptr;
const ::uint32_t
    TableStruct_clickhouse_5fgrpc_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::NameAndType, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::NameAndType, _impl_.name_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::NameAndType, _impl_.type_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::ExternalTable_SettingsEntry_DoNotUse, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::ExternalTable_SettingsEntry_DoNotUse, _impl_.key_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::ExternalTable_SettingsEntry_DoNotUse, _impl_.value_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::ExternalTable, _impl_._has_bits_),
        9, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::ExternalTable, _impl_.name_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::ExternalTable, _impl_.columns_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::ExternalTable, _impl_.data_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::ExternalTable, _impl_.format_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::ExternalTable, _impl_.compression_type_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::ExternalTable, _impl_.settings_),
        1,
        0,
        2,
        3,
        4,
        5,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::ObsoleteTransportCompression, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::ObsoleteTransportCompression, _impl_.algorithm_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::ObsoleteTransportCompression, _impl_.level_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo_SettingsEntry_DoNotUse, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo_SettingsEntry_DoNotUse, _impl_.key_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo_SettingsEntry_DoNotUse, _impl_.value_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_._has_bits_),
        28, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.query_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.query_id_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.settings_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.database_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.input_data_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.input_data_delimiter_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.output_format_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.send_output_columns_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.external_tables_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.user_name_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.password_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.quota_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.jwt_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.session_id_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.session_check_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.session_timeout_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.cancel_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.next_query_info_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.input_compression_type_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.output_compression_type_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.output_compression_level_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.transport_compression_type_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.transport_compression_level_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.obsolete_result_compression_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.obsolete_compression_type_),
        1,
        2,
        24,
        3,
        4,
        5,
        6,
        21,
        0,
        7,
        8,
        9,
        15,
        10,
        18,
        17,
        19,
        20,
        12,
        13,
        22,
        14,
        23,
        16,
        11,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::LogEntry, _impl_._has_bits_),
        10, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::LogEntry, _impl_.time_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::LogEntry, _impl_.time_microseconds_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::LogEntry, _impl_.thread_id_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::LogEntry, _impl_.query_id_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::LogEntry, _impl_.level_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::LogEntry, _impl_.source_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::LogEntry, _impl_.text_),
        3,
        4,
        5,
        0,
        6,
        1,
        2,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Progress, _impl_._has_bits_),
        8, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Progress, _impl_.read_rows_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Progress, _impl_.read_bytes_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Progress, _impl_.total_rows_to_read_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Progress, _impl_.written_rows_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Progress, _impl_.written_bytes_),
        0,
        1,
        2,
        3,
        4,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Stats, _impl_._has_bits_),
        10, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Stats, _impl_.rows_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Stats, _impl_.blocks_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Stats, _impl_.allocated_bytes_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Stats, _impl_.applied_limit_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Stats, _impl_.rows_before_limit_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Stats, _impl_.applied_aggregation_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Stats, _impl_.rows_before_aggregation_),
        0,
        1,
        2,
        5,
        3,
        6,
        4,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Exception, _impl_._has_bits_),
        7, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Exception, _impl_.code_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Exception, _impl_.name_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Exception, _impl_.display_text_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Exception, _impl_.stack_trace_),
        3,
        0,
        1,
        2,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Result, _impl_._has_bits_),
        15, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Result, _impl_.query_id_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Result, _impl_.time_zone_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Result, _impl_.output_format_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Result, _impl_.output_columns_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Result, _impl_.output_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Result, _impl_.totals_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Result, _impl_.extremes_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Result, _impl_.logs_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Result, _impl_.progress_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Result, _impl_.stats_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Result, _impl_.exception_),
        PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Result, _impl_.cancelled_),
        5,
        6,
        7,
        1,
        2,
        3,
        4,
        0,
        8,
        9,
        10,
        11,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::clickhouse::grpc::NameAndType)},
        {7, sizeof(::clickhouse::grpc::ExternalTable_SettingsEntry_DoNotUse)},
        {14, sizeof(::clickhouse::grpc::ExternalTable)},
        {29, sizeof(::clickhouse::grpc::ObsoleteTransportCompression)},
        {36, sizeof(::clickhouse::grpc::QueryInfo_SettingsEntry_DoNotUse)},
        {43, sizeof(::clickhouse::grpc::QueryInfo)},
        {96, sizeof(::clickhouse::grpc::LogEntry)},
        {113, sizeof(::clickhouse::grpc::Progress)},
        {126, sizeof(::clickhouse::grpc::Stats)},
        {143, sizeof(::clickhouse::grpc::Exception)},
        {154, sizeof(::clickhouse::grpc::Result)},
};
static const ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const
    file_message_globals[] = {
        &::clickhouse::grpc::NameAndType_globals_,
        &::clickhouse::grpc::ExternalTable_SettingsEntry_DoNotUse_globals_,
        &::clickhouse::grpc::ExternalTable_globals_,
        &::clickhouse::grpc::ObsoleteTransportCompression_globals_,
        &::clickhouse::grpc::QueryInfo_SettingsEntry_DoNotUse_globals_,
        &::clickhouse::grpc::QueryInfo_globals_,
        &::clickhouse::grpc::LogEntry_globals_,
        &::clickhouse::grpc::Progress_globals_,
        &::clickhouse::grpc::Stats_globals_,
        &::clickhouse::grpc::Exception_globals_,
        &::clickhouse::grpc::Result_globals_,
};
const char descriptor_table_protodef_clickhouse_5fgrpc_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n\025clickhouse_grpc.proto\022\017clickhouse.grpc"
    "\")\n\013NameAndType\022\014\n\004name\030\001 \001(\t\022\014\n\004type\030\002 "
    "\001(\t\"\365\001\n\rExternalTable\022\014\n\004name\030\001 \001(\t\022-\n\007c"
    "olumns\030\002 \003(\0132\034.clickhouse.grpc.NameAndTy"
    "pe\022\014\n\004data\030\003 \001(\014\022\016\n\006format\030\004 \001(\t\022\030\n\020comp"
    "ression_type\030\006 \001(\t\022>\n\010settings\030\005 \003(\0132,.c"
    "lickhouse.grpc.ExternalTable.SettingsEnt"
    "ry\032/\n\rSettingsEntry\022\013\n\003key\030\001 \001(\t\022\r\n\005valu"
    "e\030\002 \001(\t:\0028\001\"\205\003\n\034ObsoleteTransportCompres"
    "sion\022U\n\talgorithm\030\001 \001(\0162B.clickhouse.grp"
    "c.ObsoleteTransportCompression.Compressi"
    "onAlgorithm\022M\n\005level\030\002 \001(\0162>.clickhouse."
    "grpc.ObsoleteTransportCompression.Compre"
    "ssionLevel\"R\n\024CompressionAlgorithm\022\022\n\016NO"
    "_COMPRESSION\020\000\022\013\n\007DEFLATE\020\001\022\010\n\004GZIP\020\002\022\017\n"
    "\013STREAM_GZIP\020\003\"k\n\020CompressionLevel\022\024\n\020CO"
    "MPRESSION_NONE\020\000\022\023\n\017COMPRESSION_LOW\020\001\022\026\n"
    "\022COMPRESSION_MEDIUM\020\002\022\024\n\020COMPRESSION_HIG"
    "H\020\003\"\233\006\n\tQueryInfo\022\r\n\005query\030\001 \001(\t\022\020\n\010quer"
    "y_id\030\002 \001(\t\022:\n\010settings\030\003 \003(\0132(.clickhous"
    "e.grpc.QueryInfo.SettingsEntry\022\020\n\010databa"
    "se\030\004 \001(\t\022\022\n\ninput_data\030\005 \001(\014\022\034\n\024input_da"
    "ta_delimiter\030\006 \001(\014\022\025\n\routput_format\030\007 \001("
    "\t\022\033\n\023send_output_columns\030\030 \001(\010\0227\n\017extern"
    "al_tables\030\010 \003(\0132\036.clickhouse.grpc.Extern"
    "alTable\022\021\n\tuser_name\030\t \001(\t\022\020\n\010password\030\n"
    " \001(\t\022\r\n\005quota\030\013 \001(\t\022\013\n\003jwt\030\031 \001(\t\022\022\n\nsess"
    "ion_id\030\014 \001(\t\022\025\n\rsession_check\030\r \001(\010\022\027\n\017s"
    "ession_timeout\030\016 \001(\r\022\016\n\006cancel\030\017 \001(\010\022\027\n\017"
    "next_query_info\030\020 \001(\010\022\036\n\026input_compressi"
    "on_type\030\024 \001(\t\022\037\n\027output_compression_type"
    "\030\025 \001(\t\022 \n\030output_compression_level\030\023 \001(\005"
    "\022\"\n\032transport_compression_type\030\026 \001(\t\022#\n\033"
    "transport_compression_level\030\027 \001(\005\022R\n\033obs"
    "olete_result_compression\030\021 \001(\0132-.clickho"
    "use.grpc.ObsoleteTransportCompression\022!\n"
    "\031obsolete_compression_type\030\022 \001(\t\032/\n\rSett"
    "ingsEntry\022\013\n\003key\030\001 \001(\t\022\r\n\005value\030\002 \001(\t:\0028"
    "\001\"\241\001\n\010LogEntry\022\014\n\004time\030\001 \001(\r\022\031\n\021time_mic"
    "roseconds\030\002 \001(\r\022\021\n\tthread_id\030\003 \001(\004\022\020\n\010qu"
    "ery_id\030\004 \001(\t\022)\n\005level\030\005 \001(\0162\032.clickhouse"
    ".grpc.LogsLevel\022\016\n\006source\030\006 \001(\t\022\014\n\004text\030"
    "\007 \001(\t\"z\n\010Progress\022\021\n\tread_rows\030\001 \001(\004\022\022\n\n"
    "read_bytes\030\002 \001(\004\022\032\n\022total_rows_to_read\030\003"
    " \001(\004\022\024\n\014written_rows\030\004 \001(\004\022\025\n\rwritten_by"
    "tes\030\005 \001(\004\"\256\001\n\005Stats\022\014\n\004rows\030\001 \001(\004\022\016\n\006blo"
    "cks\030\002 \001(\004\022\027\n\017allocated_bytes\030\003 \001(\004\022\025\n\rap"
    "plied_limit\030\004 \001(\010\022\031\n\021rows_before_limit\030\005"
    " \001(\004\022\033\n\023applied_aggregation\030\006 \001(\010\022\037\n\027row"
    "s_before_aggregation\030\007 \001(\004\"R\n\tException\022"
    "\014\n\004code\030\001 \001(\005\022\014\n\004name\030\002 \001(\t\022\024\n\014display_t"
    "ext\030\003 \001(\t\022\023\n\013stack_trace\030\004 \001(\t\"\353\002\n\006Resul"
    "t\022\020\n\010query_id\030\t \001(\t\022\021\n\ttime_zone\030\n \001(\t\022\025"
    "\n\routput_format\030\013 \001(\t\0224\n\016output_columns\030"
    "\014 \003(\0132\034.clickhouse.grpc.NameAndType\022\016\n\006o"
    "utput\030\001 \001(\014\022\016\n\006totals\030\002 \001(\014\022\020\n\010extremes\030"
    "\003 \001(\014\022\'\n\004logs\030\004 \003(\0132\031.clickhouse.grpc.Lo"
    "gEntry\022+\n\010progress\030\005 \001(\0132\031.clickhouse.gr"
    "pc.Progress\022%\n\005stats\030\006 \001(\0132\026.clickhouse."
    "grpc.Stats\022-\n\texception\030\007 \001(\0132\032.clickhou"
    "se.grpc.Exception\022\021\n\tcancelled\030\010 \001(\010*\270\001\n"
    "\tLogsLevel\022\017\n\013CH_LOG_NONE\020\000\022\020\n\014CH_LOG_FA"
    "TAL\020\001\022\023\n\017CH_LOG_CRITICAL\020\002\022\020\n\014CH_LOG_ERR"
    "OR\020\003\022\022\n\016CH_LOG_WARNING\020\004\022\021\n\rCH_LOG_NOTIC"
    "E\020\005\022\026\n\022CH_LOG_INFORMATION\020\006\022\020\n\014CH_LOG_DE"
    "BUG\020\007\022\020\n\014CH_LOG_TRACE\020\0102\333\002\n\nClickHouse\022E"
    "\n\014ExecuteQuery\022\032.clickhouse.grpc.QueryIn"
    "fo\032\027.clickhouse.grpc.Result\"\000\022V\n\033Execute"
    "QueryWithStreamInput\022\032.clickhouse.grpc.Q"
    "ueryInfo\032\027.clickhouse.grpc.Result\"\000(\001\022W\n"
    "\034ExecuteQueryWithStreamOutput\022\032.clickhou"
    "se.grpc.QueryInfo\032\027.clickhouse.grpc.Resu"
    "lt\"\0000\001\022U\n\030ExecuteQueryWithStreamIO\022\032.cli"
    "ckhouse.grpc.QueryInfo\032\027.clickhouse.grpc"
    ".Result\"\000(\0010\001b\006proto3"
};
static ::absl::once_flag descriptor_table_clickhouse_5fgrpc_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_clickhouse_5fgrpc_2eproto = {
    false,
    false,
    2981,
    descriptor_table_protodef_clickhouse_5fgrpc_2eproto,
    "clickhouse_grpc.proto",
    &descriptor_table_clickhouse_5fgrpc_2eproto_once,
    nullptr,
    0,
    11,
    schemas,
    file_message_globals,
    TableStruct_clickhouse_5fgrpc_2eproto::offsets,
    file_level_enum_descriptors_clickhouse_5fgrpc_2eproto,
    file_level_service_descriptors_clickhouse_5fgrpc_2eproto,
};
namespace clickhouse {
namespace grpc {
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
ObsoleteTransportCompression_CompressionAlgorithm_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_clickhouse_5fgrpc_2eproto);
  return file_level_enum_descriptors_clickhouse_5fgrpc_2eproto[0];
}
PROTOBUF_CONSTINIT const uint32_t ObsoleteTransportCompression_CompressionAlgorithm_internal_data_[] = {
    262144u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
ObsoleteTransportCompression_CompressionLevel_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_clickhouse_5fgrpc_2eproto);
  return file_level_enum_descriptors_clickhouse_5fgrpc_2eproto[1];
}
PROTOBUF_CONSTINIT const uint32_t ObsoleteTransportCompression_CompressionLevel_internal_data_[] = {
    262144u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
LogsLevel_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_clickhouse_5fgrpc_2eproto);
  return file_level_enum_descriptors_clickhouse_5fgrpc_2eproto[2];
}
PROTOBUF_CONSTINIT const uint32_t LogsLevel_internal_data_[] = {
    589824u, 0u, };
// ===================================================================

NameAndType::NameAndType(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, NameAndType_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:clickhouse.grpc.NameAndType)
}
PROTOBUF_NDEBUG_INLINE NameAndType::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::clickhouse::grpc::NameAndType& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        name_(arena, from.name_),
        type_(arena, from.type_) {}

NameAndType::NameAndType(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const NameAndType& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, NameAndType_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  NameAndType* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:clickhouse.grpc.NameAndType)
}
PROTOBUF_NDEBUG_INLINE NameAndType::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        name_(arena),
        type_(arena) {}

inline void NameAndType::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
NameAndType::~NameAndType() {
  // @@protoc_insertion_point(destructor:clickhouse.grpc.NameAndType)
  SharedDtor(*this);
}
inline void NameAndType::SharedDtor(MessageLite& self) {
  NameAndType& this_ = static_cast<NameAndType&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.name_.Destroy();
  this_._impl_.type_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull NameAndType_class_data_ =
        NameAndType::InternalGenerateClassData_(NameAndType_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
NameAndType::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&NameAndType_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(NameAndType_class_data_.tc_table);
  return NameAndType_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
NameAndType::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&NameAndType_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&NameAndType_globals_));
  return NameAndType_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const NameAndType::ParseTableT_
    NameAndType::_table_ =
        NameAndType::InternalGenerateParseTable_(NameAndType_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void NameAndType::Clear() {
// @@protoc_insertion_point(message_clear_start:clickhouse.grpc.NameAndType)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.name_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.type_.ClearNonDefaultToEmpty();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL NameAndType::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const NameAndType& this_ = static_cast<const NameAndType&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL NameAndType::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const NameAndType& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:clickhouse.grpc.NameAndType)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string name = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_name().empty()) {
      const ::std::string& _s = this_._internal_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.NameAndType.name");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // string type = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_type().empty()) {
      const ::std::string& _s = this_._internal_type();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.NameAndType.type");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:clickhouse.grpc.NameAndType)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t NameAndType::ByteSizeLong(const MessageLite& base) {
  const NameAndType& this_ = static_cast<const NameAndType&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t NameAndType::ByteSizeLong() const {
  const NameAndType& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:clickhouse.grpc.NameAndType)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string name = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_name());
      }
    }
    // string type = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_type().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_type());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void NameAndType::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<NameAndType*>(&to_msg);
  auto& from = static_cast<const NameAndType&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:clickhouse.grpc.NameAndType)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_name().empty()) {
        _this->_internal_set_name(from._internal_name());
      } else {
        if (_this->_impl_.name_.IsDefault()) {
          _this->_internal_set_name("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_type().empty()) {
        _this->_internal_set_type(from._internal_type());
      } else {
        if (_this->_impl_.type_.IsDefault()) {
          _this->_internal_set_type("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void NameAndType::CopyFrom(const NameAndType& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:clickhouse.grpc.NameAndType)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void NameAndType::InternalSwap(NameAndType* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.name_, &other->_impl_.name_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.type_, &other->_impl_.type_, arena);
}

::google::protobuf::Metadata NameAndType::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

#if defined(PROTOBUF_CUSTOM_VTABLE)
ExternalTable_SettingsEntry_DoNotUse::ExternalTable_SettingsEntry_DoNotUse() : SuperType(ExternalTable_SettingsEntry_DoNotUse_get_class_data()) {}
ExternalTable_SettingsEntry_DoNotUse::ExternalTable_SettingsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
    : SuperType(arena, ExternalTable_SettingsEntry_DoNotUse_get_class_data()) {}
#else   // PROTOBUF_CUSTOM_VTABLE
ExternalTable_SettingsEntry_DoNotUse::ExternalTable_SettingsEntry_DoNotUse() : SuperType() {}
ExternalTable_SettingsEntry_DoNotUse::ExternalTable_SettingsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena) : SuperType(arena) {}
#endif  // PROTOBUF_CUSTOM_VTABLE
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ExternalTable_SettingsEntry_DoNotUse_class_data_ =
        ExternalTable_SettingsEntry_DoNotUse::InternalGenerateClassData_(ExternalTable_SettingsEntry_DoNotUse_globals_._default);

// including ~ all the vtables and everything they use become part
// of the same ~ SCC.
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ExternalTable_SettingsEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ExternalTable_SettingsEntry_DoNotUse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ExternalTable_SettingsEntry_DoNotUse_class_data_.tc_table);
  return ExternalTable_SettingsEntry_DoNotUse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ExternalTable_SettingsEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ExternalTable_SettingsEntry_DoNotUse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ExternalTable_SettingsEntry_DoNotUse_globals_));
  return ExternalTable_SettingsEntry_DoNotUse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ExternalTable_SettingsEntry_DoNotUse::ParseTableT_
    ExternalTable_SettingsEntry_DoNotUse::_table_ =
        ExternalTable_SettingsEntry_DoNotUse::InternalGenerateParseTable_(ExternalTable_SettingsEntry_DoNotUse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
// ===================================================================

ExternalTable::ExternalTable(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ExternalTable_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:clickhouse.grpc.ExternalTable)
}
PROTOBUF_NDEBUG_INLINE ExternalTable::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::clickhouse::grpc::ExternalTable& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        columns_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::clickhouse::grpc::ExternalTable,
              PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::ExternalTable, _impl_.columns_)>()
          , from.columns_
        }
        ,
        name_(arena, from.name_),
        data_(arena, from.data_),
        format_(arena, from.format_),
        compression_type_(arena, from.compression_type_),
        settings_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::clickhouse::grpc::ExternalTable,
              PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::ExternalTable, _impl_.settings_)>()
          , from.settings_
        }
     {}

ExternalTable::ExternalTable(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ExternalTable& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ExternalTable_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ExternalTable* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:clickhouse.grpc.ExternalTable)
}
PROTOBUF_NDEBUG_INLINE ExternalTable::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        columns_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::clickhouse::grpc::ExternalTable,
            PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::ExternalTable, _impl_.columns_)>()
         }
        ,
        name_(arena),
        data_(arena),
        format_(arena),
        compression_type_(arena),
        settings_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::clickhouse::grpc::ExternalTable,
            PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::ExternalTable, _impl_.settings_)>()
         }
     {}

inline void ExternalTable::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
ExternalTable::~ExternalTable() {
  // @@protoc_insertion_point(destructor:clickhouse.grpc.ExternalTable)
  SharedDtor(*this);
}
inline void ExternalTable::SharedDtor(MessageLite& self) {
  ExternalTable& this_ = static_cast<ExternalTable&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.name_.Destroy();
  this_._impl_.data_.Destroy();
  this_._impl_.format_.Destroy();
  this_._impl_.compression_type_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ExternalTable_class_data_ =
        ExternalTable::InternalGenerateClassData_(ExternalTable_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ExternalTable::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ExternalTable_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ExternalTable_class_data_.tc_table);
  return ExternalTable_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ExternalTable::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ExternalTable_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ExternalTable_globals_));
  return ExternalTable_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ExternalTable::ParseTableT_
    ExternalTable::_table_ =
        ExternalTable::InternalGenerateParseTable_(ExternalTable_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ExternalTable::Clear() {
// @@protoc_insertion_point(message_clear_start:clickhouse.grpc.ExternalTable)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.columns_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.name_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.data_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _impl_.format_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      _impl_.compression_type_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      _impl_.settings_.Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ExternalTable::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ExternalTable& this_ = static_cast<const ExternalTable&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ExternalTable::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ExternalTable& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:clickhouse.grpc.ExternalTable)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string name = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_name().empty()) {
      const ::std::string& _s = this_._internal_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.ExternalTable.name");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // repeated .clickhouse.grpc.NameAndType columns = 2;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_columns_size());
         i < n; i++) {
      const auto& repfield = this_._internal_columns().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              2, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  // bytes data = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_data().empty()) {
      const ::std::string& _s = this_._internal_data();
      target = stream->WriteBytesMaybeAliased(3, _s, target);
    }
  }

  // string format = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (!this_._internal_format().empty()) {
      const ::std::string& _s = this_._internal_format();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.ExternalTable.format");
      target = stream->WriteStringMaybeAliased(4, _s, target);
    }
  }

  // map<string, string> settings = 5;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (!this_._internal_settings().empty()) {
      using MapType = ::google::protobuf::Map<::std::string, ::std::string>;
      using WireHelper = _pbi::MapEntryFuncs<::std::string, ::std::string,
                                     _pbi::WireFormatLite::TYPE_STRING,
                                     _pbi::WireFormatLite::TYPE_STRING>;
      const auto& field = this_._internal_settings();

      if (stream->IsSerializationDeterministic() && field.size() > 1) {
        for (const auto& entry : ::google::protobuf::internal::MapSorterPtr<MapType>(field)) {
          target = WireHelper::InternalSerialize(
              5, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.ExternalTable.settings");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.ExternalTable.settings");
        }
      } else {
        for (const auto& entry : field) {
          target = WireHelper::InternalSerialize(
              5, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.ExternalTable.settings");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.ExternalTable.settings");
        }
      }
    }
  }

  // string compression_type = 6;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (!this_._internal_compression_type().empty()) {
      const ::std::string& _s = this_._internal_compression_type();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.ExternalTable.compression_type");
      target = stream->WriteStringMaybeAliased(6, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:clickhouse.grpc.ExternalTable)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ExternalTable::ByteSizeLong(const MessageLite& base) {
  const ExternalTable& this_ = static_cast<const ExternalTable&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ExternalTable::ByteSizeLong() const {
  const ExternalTable& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:clickhouse.grpc.ExternalTable)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    // repeated .clickhouse.grpc.NameAndType columns = 2;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_columns_size();
      for (const auto& msg : this_._internal_columns()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
    // string name = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_name());
      }
    }
    // bytes data = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!this_._internal_data().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::BytesSize(
                                        this_._internal_data());
      }
    }
    // string format = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!this_._internal_format().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_format());
      }
    }
    // string compression_type = 6;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (!this_._internal_compression_type().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_compression_type());
      }
    }
    // map<string, string> settings = 5;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_settings_size());
      for (const auto& entry : this_._internal_settings()) {
        total_size += _pbi::MapEntryFuncs<::std::string, ::std::string,
                                       _pbi::WireFormatLite::TYPE_STRING,
                                       _pbi::WireFormatLite::TYPE_STRING>::ByteSizeLong(entry.first, entry.second);
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ExternalTable::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ExternalTable*>(&to_msg);
  auto& from = static_cast<const ExternalTable&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:clickhouse.grpc.ExternalTable)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_columns()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_columns());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_name().empty()) {
        _this->_internal_set_name(from._internal_name());
      } else {
        if (_this->_impl_.name_.IsDefault()) {
          _this->_internal_set_name("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!from._internal_data().empty()) {
        _this->_internal_set_data(from._internal_data());
      } else {
        if (_this->_impl_.data_.IsDefault()) {
          _this->_internal_set_data("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!from._internal_format().empty()) {
        _this->_internal_set_format(from._internal_format());
      } else {
        if (_this->_impl_.format_.IsDefault()) {
          _this->_internal_set_format("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (!from._internal_compression_type().empty()) {
        _this->_internal_set_compression_type(from._internal_compression_type());
      } else {
        if (_this->_impl_.compression_type_.IsDefault()) {
          _this->_internal_set_compression_type("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      _this->_impl_.settings_.MergeFrom(from._impl_.settings_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ExternalTable::CopyFrom(const ExternalTable& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:clickhouse.grpc.ExternalTable)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ExternalTable::InternalSwap(ExternalTable* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.columns_.InternalSwap(&other->_impl_.columns_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.name_, &other->_impl_.name_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.data_, &other->_impl_.data_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.format_, &other->_impl_.format_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.compression_type_, &other->_impl_.compression_type_, arena);
  _impl_.settings_.InternalSwap(&other->_impl_.settings_);
}

::google::protobuf::Metadata ExternalTable::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ObsoleteTransportCompression::ObsoleteTransportCompression(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ObsoleteTransportCompression_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:clickhouse.grpc.ObsoleteTransportCompression)
}
ObsoleteTransportCompression::ObsoleteTransportCompression(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ObsoleteTransportCompression& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ObsoleteTransportCompression_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE ObsoleteTransportCompression::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void ObsoleteTransportCompression::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, algorithm_),
           0,
           offsetof(Impl_, level_) -
               offsetof(Impl_, algorithm_) +
               sizeof(Impl_::level_));
}
ObsoleteTransportCompression::~ObsoleteTransportCompression() {
  // @@protoc_insertion_point(destructor:clickhouse.grpc.ObsoleteTransportCompression)
  SharedDtor(*this);
}
inline void ObsoleteTransportCompression::SharedDtor(MessageLite& self) {
  ObsoleteTransportCompression& this_ = static_cast<ObsoleteTransportCompression&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ObsoleteTransportCompression_class_data_ =
        ObsoleteTransportCompression::InternalGenerateClassData_(ObsoleteTransportCompression_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ObsoleteTransportCompression::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ObsoleteTransportCompression_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ObsoleteTransportCompression_class_data_.tc_table);
  return ObsoleteTransportCompression_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ObsoleteTransportCompression::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ObsoleteTransportCompression_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ObsoleteTransportCompression_globals_));
  return ObsoleteTransportCompression_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ObsoleteTransportCompression::ParseTableT_
    ObsoleteTransportCompression::_table_ =
        ObsoleteTransportCompression::InternalGenerateParseTable_(ObsoleteTransportCompression_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ObsoleteTransportCompression::Clear() {
// @@protoc_insertion_point(message_clear_start:clickhouse.grpc.ObsoleteTransportCompression)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    ::memset(&_impl_.algorithm_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.level_) -
        reinterpret_cast<char*>(&_impl_.algorithm_)) + sizeof(_impl_.level_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ObsoleteTransportCompression::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ObsoleteTransportCompression& this_ = static_cast<const ObsoleteTransportCompression&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ObsoleteTransportCompression::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ObsoleteTransportCompression& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:clickhouse.grpc.ObsoleteTransportCompression)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .clickhouse.grpc.ObsoleteTransportCompression.CompressionAlgorithm algorithm = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_algorithm() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          1, this_._internal_algorithm(), target);
    }
  }

  // .clickhouse.grpc.ObsoleteTransportCompression.CompressionLevel level = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_level() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          2, this_._internal_level(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:clickhouse.grpc.ObsoleteTransportCompression)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ObsoleteTransportCompression::ByteSizeLong(const MessageLite& base) {
  const ObsoleteTransportCompression& this_ = static_cast<const ObsoleteTransportCompression&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ObsoleteTransportCompression::ByteSizeLong() const {
  const ObsoleteTransportCompression& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:clickhouse.grpc.ObsoleteTransportCompression)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // .clickhouse.grpc.ObsoleteTransportCompression.CompressionAlgorithm algorithm = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_algorithm() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_algorithm());
      }
    }
    // .clickhouse.grpc.ObsoleteTransportCompression.CompressionLevel level = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_level() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_level());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ObsoleteTransportCompression::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ObsoleteTransportCompression*>(&to_msg);
  auto& from = static_cast<const ObsoleteTransportCompression&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:clickhouse.grpc.ObsoleteTransportCompression)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (from._internal_algorithm() != 0) {
        _this->_impl_.algorithm_ = from._impl_.algorithm_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_level() != 0) {
        _this->_impl_.level_ = from._impl_.level_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ObsoleteTransportCompression::CopyFrom(const ObsoleteTransportCompression& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:clickhouse.grpc.ObsoleteTransportCompression)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ObsoleteTransportCompression::InternalSwap(ObsoleteTransportCompression* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(ObsoleteTransportCompression, _impl_.level_)
      + sizeof(ObsoleteTransportCompression::_impl_.level_)
      - PROTOBUF_FIELD_OFFSET(ObsoleteTransportCompression, _impl_.algorithm_)>(
          reinterpret_cast<char*>(&_impl_.algorithm_),
          reinterpret_cast<char*>(&other->_impl_.algorithm_));
}

::google::protobuf::Metadata ObsoleteTransportCompression::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

#if defined(PROTOBUF_CUSTOM_VTABLE)
QueryInfo_SettingsEntry_DoNotUse::QueryInfo_SettingsEntry_DoNotUse() : SuperType(QueryInfo_SettingsEntry_DoNotUse_get_class_data()) {}
QueryInfo_SettingsEntry_DoNotUse::QueryInfo_SettingsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
    : SuperType(arena, QueryInfo_SettingsEntry_DoNotUse_get_class_data()) {}
#else   // PROTOBUF_CUSTOM_VTABLE
QueryInfo_SettingsEntry_DoNotUse::QueryInfo_SettingsEntry_DoNotUse() : SuperType() {}
QueryInfo_SettingsEntry_DoNotUse::QueryInfo_SettingsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena) : SuperType(arena) {}
#endif  // PROTOBUF_CUSTOM_VTABLE
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull QueryInfo_SettingsEntry_DoNotUse_class_data_ =
        QueryInfo_SettingsEntry_DoNotUse::InternalGenerateClassData_(QueryInfo_SettingsEntry_DoNotUse_globals_._default);

// including ~ all the vtables and everything they use become part
// of the same ~ SCC.
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
QueryInfo_SettingsEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&QueryInfo_SettingsEntry_DoNotUse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(QueryInfo_SettingsEntry_DoNotUse_class_data_.tc_table);
  return QueryInfo_SettingsEntry_DoNotUse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
QueryInfo_SettingsEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&QueryInfo_SettingsEntry_DoNotUse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&QueryInfo_SettingsEntry_DoNotUse_globals_));
  return QueryInfo_SettingsEntry_DoNotUse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const QueryInfo_SettingsEntry_DoNotUse::ParseTableT_
    QueryInfo_SettingsEntry_DoNotUse::_table_ =
        QueryInfo_SettingsEntry_DoNotUse::InternalGenerateParseTable_(QueryInfo_SettingsEntry_DoNotUse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
// ===================================================================

QueryInfo::QueryInfo(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, QueryInfo_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:clickhouse.grpc.QueryInfo)
}
PROTOBUF_NDEBUG_INLINE QueryInfo::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::clickhouse::grpc::QueryInfo& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        external_tables_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::clickhouse::grpc::QueryInfo,
              PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.external_tables_)>()
          , from.external_tables_
        }
        ,
        query_(arena, from.query_),
        query_id_(arena, from.query_id_),
        database_(arena, from.database_),
        input_data_(arena, from.input_data_),
        input_data_delimiter_(arena, from.input_data_delimiter_),
        output_format_(arena, from.output_format_),
        user_name_(arena, from.user_name_),
        password_(arena, from.password_),
        quota_(arena, from.quota_),
        session_id_(arena, from.session_id_),
        obsolete_compression_type_(arena, from.obsolete_compression_type_),
        input_compression_type_(arena, from.input_compression_type_),
        output_compression_type_(arena, from.output_compression_type_),
        transport_compression_type_(arena, from.transport_compression_type_),
        jwt_(arena, from.jwt_),
        settings_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::clickhouse::grpc::QueryInfo,
              PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.settings_)>()
          , from.settings_
        }
     {}

QueryInfo::QueryInfo(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const QueryInfo& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, QueryInfo_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  QueryInfo* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.obsolete_result_compression_ = (CheckHasBit(cached_has_bits, 0x00010000U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.obsolete_result_compression_)
                : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, session_timeout_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, session_timeout_),
           offsetof(Impl_, transport_compression_level_) -
               offsetof(Impl_, session_timeout_) +
               sizeof(Impl_::transport_compression_level_));

  // @@protoc_insertion_point(copy_constructor:clickhouse.grpc.QueryInfo)
}
PROTOBUF_NDEBUG_INLINE QueryInfo::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        external_tables_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::clickhouse::grpc::QueryInfo,
            PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.external_tables_)>()
         }
        ,
        query_(arena),
        query_id_(arena),
        database_(arena),
        input_data_(arena),
        input_data_delimiter_(arena),
        output_format_(arena),
        user_name_(arena),
        password_(arena),
        quota_(arena),
        session_id_(arena),
        obsolete_compression_type_(arena),
        input_compression_type_(arena),
        output_compression_type_(arena),
        transport_compression_type_(arena),
        jwt_(arena),
        settings_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::clickhouse::grpc::QueryInfo,
            PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::QueryInfo, _impl_.settings_)>()
         }
     {}

inline void QueryInfo::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, obsolete_result_compression_),
           0,
           offsetof(Impl_, transport_compression_level_) -
               offsetof(Impl_, obsolete_result_compression_) +
               sizeof(Impl_::transport_compression_level_));
}
QueryInfo::~QueryInfo() {
  // @@protoc_insertion_point(destructor:clickhouse.grpc.QueryInfo)
  SharedDtor(*this);
}
inline void QueryInfo::SharedDtor(MessageLite& self) {
  QueryInfo& this_ = static_cast<QueryInfo&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.query_.Destroy();
  this_._impl_.query_id_.Destroy();
  this_._impl_.database_.Destroy();
  this_._impl_.input_data_.Destroy();
  this_._impl_.input_data_delimiter_.Destroy();
  this_._impl_.output_format_.Destroy();
  this_._impl_.user_name_.Destroy();
  this_._impl_.password_.Destroy();
  this_._impl_.quota_.Destroy();
  this_._impl_.session_id_.Destroy();
  this_._impl_.obsolete_compression_type_.Destroy();
  this_._impl_.input_compression_type_.Destroy();
  this_._impl_.output_compression_type_.Destroy();
  this_._impl_.transport_compression_type_.Destroy();
  this_._impl_.jwt_.Destroy();
  delete this_._impl_.obsolete_result_compression_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull QueryInfo_class_data_ =
        QueryInfo::InternalGenerateClassData_(QueryInfo_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
QueryInfo::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&QueryInfo_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(QueryInfo_class_data_.tc_table);
  return QueryInfo_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
QueryInfo::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&QueryInfo_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&QueryInfo_globals_));
  return QueryInfo_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const QueryInfo::ParseTableT_
    QueryInfo::_table_ =
        QueryInfo::InternalGenerateParseTable_(QueryInfo_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void QueryInfo::Clear() {
// @@protoc_insertion_point(message_clear_start:clickhouse.grpc.QueryInfo)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.external_tables_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.query_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.query_id_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _impl_.database_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      _impl_.input_data_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      _impl_.input_data_delimiter_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      _impl_.output_format_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      _impl_.user_name_.ClearNonDefaultToEmpty();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000ff00U)) {
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      _impl_.password_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000200U)) {
      _impl_.quota_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000400U)) {
      _impl_.session_id_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000800U)) {
      _impl_.obsolete_compression_type_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00001000U)) {
      _impl_.input_compression_type_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00002000U)) {
      _impl_.output_compression_type_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00004000U)) {
      _impl_.transport_compression_type_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00008000U)) {
      _impl_.jwt_.ClearNonDefaultToEmpty();
    }
  }
  if (CheckHasBit(cached_has_bits, 0x00010000U)) {
    ABSL_DCHECK(_impl_.obsolete_result_compression_ != nullptr);
    _impl_.obsolete_result_compression_->Clear();
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00fe0000U)) {
    ::memset(&_impl_.session_timeout_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.transport_compression_level_) -
        reinterpret_cast<char*>(&_impl_.session_timeout_)) + sizeof(_impl_.transport_compression_level_));
  }
  if (CheckHasBit(cached_has_bits, 0x01000000U)) {
    _impl_.settings_.Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL QueryInfo::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const QueryInfo& this_ = static_cast<const QueryInfo&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL QueryInfo::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const QueryInfo& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:clickhouse.grpc.QueryInfo)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string query = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_query().empty()) {
      const ::std::string& _s = this_._internal_query();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.QueryInfo.query");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // string query_id = 2;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_query_id().empty()) {
      const ::std::string& _s = this_._internal_query_id();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.QueryInfo.query_id");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  // map<string, string> settings = 3;
  if (CheckHasBit(cached_has_bits, 0x01000000U)) {
    if (!this_._internal_settings().empty()) {
      using MapType = ::google::protobuf::Map<::std::string, ::std::string>;
      using WireHelper = _pbi::MapEntryFuncs<::std::string, ::std::string,
                                     _pbi::WireFormatLite::TYPE_STRING,
                                     _pbi::WireFormatLite::TYPE_STRING>;
      const auto& field = this_._internal_settings();

      if (stream->IsSerializationDeterministic() && field.size() > 1) {
        for (const auto& entry : ::google::protobuf::internal::MapSorterPtr<MapType>(field)) {
          target = WireHelper::InternalSerialize(
              3, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.QueryInfo.settings");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.QueryInfo.settings");
        }
      } else {
        for (const auto& entry : field) {
          target = WireHelper::InternalSerialize(
              3, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.QueryInfo.settings");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.QueryInfo.settings");
        }
      }
    }
  }

  // string database = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (!this_._internal_database().empty()) {
      const ::std::string& _s = this_._internal_database();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.QueryInfo.database");
      target = stream->WriteStringMaybeAliased(4, _s, target);
    }
  }

  // bytes input_data = 5;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (!this_._internal_input_data().empty()) {
      const ::std::string& _s = this_._internal_input_data();
      target = stream->WriteBytesMaybeAliased(5, _s, target);
    }
  }

  // bytes input_data_delimiter = 6;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (!this_._internal_input_data_delimiter().empty()) {
      const ::std::string& _s = this_._internal_input_data_delimiter();
      target = stream->WriteBytesMaybeAliased(6, _s, target);
    }
  }

  // string output_format = 7;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (!this_._internal_output_format().empty()) {
      const ::std::string& _s = this_._internal_output_format();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.QueryInfo.output_format");
      target = stream->WriteStringMaybeAliased(7, _s, target);
    }
  }

  // repeated .clickhouse.grpc.ExternalTable external_tables = 8;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_external_tables_size());
         i < n; i++) {
      const auto& repfield = this_._internal_external_tables().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              8, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  // string user_name = 9;
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    if (!this_._internal_user_name().empty()) {
      const ::std::string& _s = this_._internal_user_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.QueryInfo.user_name");
      target = stream->WriteStringMaybeAliased(9, _s, target);
    }
  }

  // string password = 10;
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    if (!this_._internal_password().empty()) {
      const ::std::string& _s = this_._internal_password();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.QueryInfo.password");
      target = stream->WriteStringMaybeAliased(10, _s, target);
    }
  }

  // string quota = 11;
  if (CheckHasBit(cached_has_bits, 0x00000200U)) {
    if (!this_._internal_quota().empty()) {
      const ::std::string& _s = this_._internal_quota();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.QueryInfo.quota");
      target = stream->WriteStringMaybeAliased(11, _s, target);
    }
  }

  // string session_id = 12;
  if (CheckHasBit(cached_has_bits, 0x00000400U)) {
    if (!this_._internal_session_id().empty()) {
      const ::std::string& _s = this_._internal_session_id();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.QueryInfo.session_id");
      target = stream->WriteStringMaybeAliased(12, _s, target);
    }
  }

  // bool session_check = 13;
  if (CheckHasBit(cached_has_bits, 0x00040000U)) {
    if (this_._internal_session_check() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          13, this_._internal_session_check(), target);
    }
  }

  // uint32 session_timeout = 14;
  if (CheckHasBit(cached_has_bits, 0x00020000U)) {
    if (this_._internal_session_timeout() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          14, this_._internal_session_timeout(), target);
    }
  }

  // bool cancel = 15;
  if (CheckHasBit(cached_has_bits, 0x00080000U)) {
    if (this_._internal_cancel() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          15, this_._internal_cancel(), target);
    }
  }

  // bool next_query_info = 16;
  if (CheckHasBit(cached_has_bits, 0x00100000U)) {
    if (this_._internal_next_query_info() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          16, this_._internal_next_query_info(), target);
    }
  }

  // .clickhouse.grpc.ObsoleteTransportCompression obsolete_result_compression = 17;
  if (CheckHasBit(cached_has_bits, 0x00010000U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        17, *this_._impl_.obsolete_result_compression_, this_._impl_.obsolete_result_compression_->GetCachedSize(), target,
        stream);
  }

  // string obsolete_compression_type = 18;
  if (CheckHasBit(cached_has_bits, 0x00000800U)) {
    if (!this_._internal_obsolete_compression_type().empty()) {
      const ::std::string& _s = this_._internal_obsolete_compression_type();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.QueryInfo.obsolete_compression_type");
      target = stream->WriteStringMaybeAliased(18, _s, target);
    }
  }

  // int32 output_compression_level = 19;
  if (CheckHasBit(cached_has_bits, 0x00400000U)) {
    if (this_._internal_output_compression_level() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteInt32ToArray(
          19, this_._internal_output_compression_level(), target);
    }
  }

  // string input_compression_type = 20;
  if (CheckHasBit(cached_has_bits, 0x00001000U)) {
    if (!this_._internal_input_compression_type().empty()) {
      const ::std::string& _s = this_._internal_input_compression_type();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.QueryInfo.input_compression_type");
      target = stream->WriteStringMaybeAliased(20, _s, target);
    }
  }

  // string output_compression_type = 21;
  if (CheckHasBit(cached_has_bits, 0x00002000U)) {
    if (!this_._internal_output_compression_type().empty()) {
      const ::std::string& _s = this_._internal_output_compression_type();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.QueryInfo.output_compression_type");
      target = stream->WriteStringMaybeAliased(21, _s, target);
    }
  }

  // string transport_compression_type = 22;
  if (CheckHasBit(cached_has_bits, 0x00004000U)) {
    if (!this_._internal_transport_compression_type().empty()) {
      const ::std::string& _s = this_._internal_transport_compression_type();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.QueryInfo.transport_compression_type");
      target = stream->WriteStringMaybeAliased(22, _s, target);
    }
  }

  // int32 transport_compression_level = 23;
  if (CheckHasBit(cached_has_bits, 0x00800000U)) {
    if (this_._internal_transport_compression_level() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteInt32ToArray(
          23, this_._internal_transport_compression_level(), target);
    }
  }

  // bool send_output_columns = 24;
  if (CheckHasBit(cached_has_bits, 0x00200000U)) {
    if (this_._internal_send_output_columns() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          24, this_._internal_send_output_columns(), target);
    }
  }

  // string jwt = 25;
  if (CheckHasBit(cached_has_bits, 0x00008000U)) {
    if (!this_._internal_jwt().empty()) {
      const ::std::string& _s = this_._internal_jwt();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.QueryInfo.jwt");
      target = stream->WriteStringMaybeAliased(25, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:clickhouse.grpc.QueryInfo)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t QueryInfo::ByteSizeLong(const MessageLite& base) {
  const QueryInfo& this_ = static_cast<const QueryInfo&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t QueryInfo::ByteSizeLong() const {
  const QueryInfo& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:clickhouse.grpc.QueryInfo)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // repeated .clickhouse.grpc.ExternalTable external_tables = 8;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_external_tables_size();
      for (const auto& msg : this_._internal_external_tables()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
    // string query = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_query().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_query());
      }
    }
    // string query_id = 2;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!this_._internal_query_id().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_query_id());
      }
    }
    // string database = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!this_._internal_database().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_database());
      }
    }
    // bytes input_data = 5;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (!this_._internal_input_data().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::BytesSize(
                                        this_._internal_input_data());
      }
    }
    // bytes input_data_delimiter = 6;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (!this_._internal_input_data_delimiter().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::BytesSize(
                                        this_._internal_input_data_delimiter());
      }
    }
    // string output_format = 7;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (!this_._internal_output_format().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_output_format());
      }
    }
    // string user_name = 9;
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (!this_._internal_user_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_user_name());
      }
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000ff00U)) {
    // string password = 10;
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      if (!this_._internal_password().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_password());
      }
    }
    // string quota = 11;
    if (CheckHasBit(cached_has_bits, 0x00000200U)) {
      if (!this_._internal_quota().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_quota());
      }
    }
    // string session_id = 12;
    if (CheckHasBit(cached_has_bits, 0x00000400U)) {
      if (!this_._internal_session_id().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_session_id());
      }
    }
    // string obsolete_compression_type = 18;
    if (CheckHasBit(cached_has_bits, 0x00000800U)) {
      if (!this_._internal_obsolete_compression_type().empty()) {
        total_size += 2 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_obsolete_compression_type());
      }
    }
    // string input_compression_type = 20;
    if (CheckHasBit(cached_has_bits, 0x00001000U)) {
      if (!this_._internal_input_compression_type().empty()) {
        total_size += 2 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_input_compression_type());
      }
    }
    // string output_compression_type = 21;
    if (CheckHasBit(cached_has_bits, 0x00002000U)) {
      if (!this_._internal_output_compression_type().empty()) {
        total_size += 2 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_output_compression_type());
      }
    }
    // string transport_compression_type = 22;
    if (CheckHasBit(cached_has_bits, 0x00004000U)) {
      if (!this_._internal_transport_compression_type().empty()) {
        total_size += 2 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_transport_compression_type());
      }
    }
    // string jwt = 25;
    if (CheckHasBit(cached_has_bits, 0x00008000U)) {
      if (!this_._internal_jwt().empty()) {
        total_size += 2 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_jwt());
      }
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00ff0000U)) {
    // .clickhouse.grpc.ObsoleteTransportCompression obsolete_result_compression = 17;
    if (CheckHasBit(cached_has_bits, 0x00010000U)) {
      total_size += 2 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.obsolete_result_compression_);
    }
    // uint32 session_timeout = 14;
    if (CheckHasBit(cached_has_bits, 0x00020000U)) {
      if (this_._internal_session_timeout() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_session_timeout());
      }
    }
    // bool session_check = 13;
    if (CheckHasBit(cached_has_bits, 0x00040000U)) {
      if (this_._internal_session_check() != 0) {
        total_size += 2;
      }
    }
    // bool cancel = 15;
    if (CheckHasBit(cached_has_bits, 0x00080000U)) {
      if (this_._internal_cancel() != 0) {
        total_size += 2;
      }
    }
    // bool next_query_info = 16;
    if (CheckHasBit(cached_has_bits, 0x00100000U)) {
      if (this_._internal_next_query_info() != 0) {
        total_size += 3;
      }
    }
    // bool send_output_columns = 24;
    if (CheckHasBit(cached_has_bits, 0x00200000U)) {
      if (this_._internal_send_output_columns() != 0) {
        total_size += 3;
      }
    }
    // int32 output_compression_level = 19;
    if (CheckHasBit(cached_has_bits, 0x00400000U)) {
      if (this_._internal_output_compression_level() != 0) {
        total_size += 2 + ::_pbi::WireFormatLite::Int32Size(
                                        this_._internal_output_compression_level());
      }
    }
    // int32 transport_compression_level = 23;
    if (CheckHasBit(cached_has_bits, 0x00800000U)) {
      if (this_._internal_transport_compression_level() != 0) {
        total_size += 2 + ::_pbi::WireFormatLite::Int32Size(
                                        this_._internal_transport_compression_level());
      }
    }
  }
   {
    // map<string, string> settings = 3;
    if (CheckHasBit(cached_has_bits, 0x01000000U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_settings_size());
      for (const auto& entry : this_._internal_settings()) {
        total_size += _pbi::MapEntryFuncs<::std::string, ::std::string,
                                       _pbi::WireFormatLite::TYPE_STRING,
                                       _pbi::WireFormatLite::TYPE_STRING>::ByteSizeLong(entry.first, entry.second);
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void QueryInfo::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<QueryInfo*>(&to_msg);
  auto& from = static_cast<const QueryInfo&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:clickhouse.grpc.QueryInfo)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_external_tables()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_external_tables());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_query().empty()) {
        _this->_internal_set_query(from._internal_query());
      } else {
        if (_this->_impl_.query_.IsDefault()) {
          _this->_internal_set_query("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!from._internal_query_id().empty()) {
        _this->_internal_set_query_id(from._internal_query_id());
      } else {
        if (_this->_impl_.query_id_.IsDefault()) {
          _this->_internal_set_query_id("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!from._internal_database().empty()) {
        _this->_internal_set_database(from._internal_database());
      } else {
        if (_this->_impl_.database_.IsDefault()) {
          _this->_internal_set_database("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (!from._internal_input_data().empty()) {
        _this->_internal_set_input_data(from._internal_input_data());
      } else {
        if (_this->_impl_.input_data_.IsDefault()) {
          _this->_internal_set_input_data("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (!from._internal_input_data_delimiter().empty()) {
        _this->_internal_set_input_data_delimiter(from._internal_input_data_delimiter());
      } else {
        if (_this->_impl_.input_data_delimiter_.IsDefault()) {
          _this->_internal_set_input_data_delimiter("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (!from._internal_output_format().empty()) {
        _this->_internal_set_output_format(from._internal_output_format());
      } else {
        if (_this->_impl_.output_format_.IsDefault()) {
          _this->_internal_set_output_format("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (!from._internal_user_name().empty()) {
        _this->_internal_set_user_name(from._internal_user_name());
      } else {
        if (_this->_impl_.user_name_.IsDefault()) {
          _this->_internal_set_user_name("");
        }
      }
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000ff00U)) {
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      if (!from._internal_password().empty()) {
        _this->_internal_set_password(from._internal_password());
      } else {
        if (_this->_impl_.password_.IsDefault()) {
          _this->_internal_set_password("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000200U)) {
      if (!from._internal_quota().empty()) {
        _this->_internal_set_quota(from._internal_quota());
      } else {
        if (_this->_impl_.quota_.IsDefault()) {
          _this->_internal_set_quota("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000400U)) {
      if (!from._internal_session_id().empty()) {
        _this->_internal_set_session_id(from._internal_session_id());
      } else {
        if (_this->_impl_.session_id_.IsDefault()) {
          _this->_internal_set_session_id("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000800U)) {
      if (!from._internal_obsolete_compression_type().empty()) {
        _this->_internal_set_obsolete_compression_type(from._internal_obsolete_compression_type());
      } else {
        if (_this->_impl_.obsolete_compression_type_.IsDefault()) {
          _this->_internal_set_obsolete_compression_type("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00001000U)) {
      if (!from._internal_input_compression_type().empty()) {
        _this->_internal_set_input_compression_type(from._internal_input_compression_type());
      } else {
        if (_this->_impl_.input_compression_type_.IsDefault()) {
          _this->_internal_set_input_compression_type("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00002000U)) {
      if (!from._internal_output_compression_type().empty()) {
        _this->_internal_set_output_compression_type(from._internal_output_compression_type());
      } else {
        if (_this->_impl_.output_compression_type_.IsDefault()) {
          _this->_internal_set_output_compression_type("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00004000U)) {
      if (!from._internal_transport_compression_type().empty()) {
        _this->_internal_set_transport_compression_type(from._internal_transport_compression_type());
      } else {
        if (_this->_impl_.transport_compression_type_.IsDefault()) {
          _this->_internal_set_transport_compression_type("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00008000U)) {
      if (!from._internal_jwt().empty()) {
        _this->_internal_set_jwt(from._internal_jwt());
      } else {
        if (_this->_impl_.jwt_.IsDefault()) {
          _this->_internal_set_jwt("");
        }
      }
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00ff0000U)) {
    if (CheckHasBit(cached_has_bits, 0x00010000U)) {
      ABSL_DCHECK(from._impl_.obsolete_result_compression_ != nullptr);
      if (_this->_impl_.obsolete_result_compression_ == nullptr) {
        _this->_impl_.obsolete_result_compression_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.obsolete_result_compression_);
      } else {
        _this->_impl_.obsolete_result_compression_->MergeFrom(*from._impl_.obsolete_result_compression_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00020000U)) {
      if (from._internal_session_timeout() != 0) {
        _this->_impl_.session_timeout_ = from._impl_.session_timeout_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00040000U)) {
      if (from._internal_session_check() != 0) {
        _this->_impl_.session_check_ = from._impl_.session_check_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00080000U)) {
      if (from._internal_cancel() != 0) {
        _this->_impl_.cancel_ = from._impl_.cancel_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00100000U)) {
      if (from._internal_next_query_info() != 0) {
        _this->_impl_.next_query_info_ = from._impl_.next_query_info_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00200000U)) {
      if (from._internal_send_output_columns() != 0) {
        _this->_impl_.send_output_columns_ = from._impl_.send_output_columns_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00400000U)) {
      if (from._internal_output_compression_level() != 0) {
        _this->_impl_.output_compression_level_ = from._impl_.output_compression_level_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00800000U)) {
      if (from._internal_transport_compression_level() != 0) {
        _this->_impl_.transport_compression_level_ = from._impl_.transport_compression_level_;
      }
    }
  }
  if (CheckHasBit(cached_has_bits, 0x01000000U)) {
    _this->_impl_.settings_.MergeFrom(from._impl_.settings_);
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void QueryInfo::CopyFrom(const QueryInfo& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:clickhouse.grpc.QueryInfo)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void QueryInfo::InternalSwap(QueryInfo* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.external_tables_.InternalSwap(&other->_impl_.external_tables_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.query_, &other->_impl_.query_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.query_id_, &other->_impl_.query_id_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.database_, &other->_impl_.database_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.input_data_, &other->_impl_.input_data_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.input_data_delimiter_, &other->_impl_.input_data_delimiter_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.output_format_, &other->_impl_.output_format_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.user_name_, &other->_impl_.user_name_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.password_, &other->_impl_.password_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.quota_, &other->_impl_.quota_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.session_id_, &other->_impl_.session_id_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.obsolete_compression_type_, &other->_impl_.obsolete_compression_type_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.input_compression_type_, &other->_impl_.input_compression_type_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.output_compression_type_, &other->_impl_.output_compression_type_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.transport_compression_type_, &other->_impl_.transport_compression_type_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.jwt_, &other->_impl_.jwt_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.transport_compression_level_)
      + sizeof(QueryInfo::_impl_.transport_compression_level_)
      - PROTOBUF_FIELD_OFFSET(QueryInfo, _impl_.obsolete_result_compression_)>(
          reinterpret_cast<char*>(&_impl_.obsolete_result_compression_),
          reinterpret_cast<char*>(&other->_impl_.obsolete_result_compression_));
  _impl_.settings_.InternalSwap(&other->_impl_.settings_);
}

::google::protobuf::Metadata QueryInfo::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

LogEntry::LogEntry(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LogEntry_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:clickhouse.grpc.LogEntry)
}
PROTOBUF_NDEBUG_INLINE LogEntry::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::clickhouse::grpc::LogEntry& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        query_id_(arena, from.query_id_),
        source_(arena, from.source_),
        text_(arena, from.text_) {}

LogEntry::LogEntry(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const LogEntry& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, LogEntry_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  LogEntry* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, time_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, time_),
           offsetof(Impl_, level_) -
               offsetof(Impl_, time_) +
               sizeof(Impl_::level_));

  // @@protoc_insertion_point(copy_constructor:clickhouse.grpc.LogEntry)
}
PROTOBUF_NDEBUG_INLINE LogEntry::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        query_id_(arena),
        source_(arena),
        text_(arena) {}

inline void LogEntry::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, time_),
           0,
           offsetof(Impl_, level_) -
               offsetof(Impl_, time_) +
               sizeof(Impl_::level_));
}
LogEntry::~LogEntry() {
  // @@protoc_insertion_point(destructor:clickhouse.grpc.LogEntry)
  SharedDtor(*this);
}
inline void LogEntry::SharedDtor(MessageLite& self) {
  LogEntry& this_ = static_cast<LogEntry&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.query_id_.Destroy();
  this_._impl_.source_.Destroy();
  this_._impl_.text_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull LogEntry_class_data_ =
        LogEntry::InternalGenerateClassData_(LogEntry_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LogEntry::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LogEntry_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(LogEntry_class_data_.tc_table);
  return LogEntry_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
LogEntry::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&LogEntry_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&LogEntry_globals_));
  return LogEntry_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const LogEntry::ParseTableT_
    LogEntry::_table_ =
        LogEntry::InternalGenerateParseTable_(LogEntry_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void LogEntry::Clear() {
// @@protoc_insertion_point(message_clear_start:clickhouse.grpc.LogEntry)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.query_id_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.source_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.text_.ClearNonDefaultToEmpty();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000078U)) {
    ::memset(&_impl_.time_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.level_) -
        reinterpret_cast<char*>(&_impl_.time_)) + sizeof(_impl_.level_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL LogEntry::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const LogEntry& this_ = static_cast<const LogEntry&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL LogEntry::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const LogEntry& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:clickhouse.grpc.LogEntry)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // uint32 time = 1;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_time() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          1, this_._internal_time(), target);
    }
  }

  // uint32 time_microseconds = 2;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_time_microseconds() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          2, this_._internal_time_microseconds(), target);
    }
  }

  // uint64 thread_id = 3;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_thread_id() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_thread_id(), target);
    }
  }

  // string query_id = 4;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_query_id().empty()) {
      const ::std::string& _s = this_._internal_query_id();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.LogEntry.query_id");
      target = stream->WriteStringMaybeAliased(4, _s, target);
    }
  }

  // .clickhouse.grpc.LogsLevel level = 5;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_level() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          5, this_._internal_level(), target);
    }
  }

  // string source = 6;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_source().empty()) {
      const ::std::string& _s = this_._internal_source();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.LogEntry.source");
      target = stream->WriteStringMaybeAliased(6, _s, target);
    }
  }

  // string text = 7;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_text().empty()) {
      const ::std::string& _s = this_._internal_text();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.LogEntry.text");
      target = stream->WriteStringMaybeAliased(7, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:clickhouse.grpc.LogEntry)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t LogEntry::ByteSizeLong(const MessageLite& base) {
  const LogEntry& this_ = static_cast<const LogEntry&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t LogEntry::ByteSizeLong() const {
  const LogEntry& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:clickhouse.grpc.LogEntry)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    // string query_id = 4;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_query_id().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_query_id());
      }
    }
    // string source = 6;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_source().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_source());
      }
    }
    // string text = 7;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!this_._internal_text().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_text());
      }
    }
    // uint32 time = 1;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_time() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_time());
      }
    }
    // uint32 time_microseconds = 2;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_time_microseconds() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_time_microseconds());
      }
    }
    // uint64 thread_id = 3;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_thread_id() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_thread_id());
      }
    }
    // .clickhouse.grpc.LogsLevel level = 5;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_level() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_level());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void LogEntry::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<LogEntry*>(&to_msg);
  auto& from = static_cast<const LogEntry&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:clickhouse.grpc.LogEntry)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_query_id().empty()) {
        _this->_internal_set_query_id(from._internal_query_id());
      } else {
        if (_this->_impl_.query_id_.IsDefault()) {
          _this->_internal_set_query_id("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_source().empty()) {
        _this->_internal_set_source(from._internal_source());
      } else {
        if (_this->_impl_.source_.IsDefault()) {
          _this->_internal_set_source("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!from._internal_text().empty()) {
        _this->_internal_set_text(from._internal_text());
      } else {
        if (_this->_impl_.text_.IsDefault()) {
          _this->_internal_set_text("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_time() != 0) {
        _this->_impl_.time_ = from._impl_.time_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_time_microseconds() != 0) {
        _this->_impl_.time_microseconds_ = from._impl_.time_microseconds_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_thread_id() != 0) {
        _this->_impl_.thread_id_ = from._impl_.thread_id_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_level() != 0) {
        _this->_impl_.level_ = from._impl_.level_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void LogEntry::CopyFrom(const LogEntry& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:clickhouse.grpc.LogEntry)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void LogEntry::InternalSwap(LogEntry* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.query_id_, &other->_impl_.query_id_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.source_, &other->_impl_.source_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.text_, &other->_impl_.text_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(LogEntry, _impl_.level_)
      + sizeof(LogEntry::_impl_.level_)
      - PROTOBUF_FIELD_OFFSET(LogEntry, _impl_.time_)>(
          reinterpret_cast<char*>(&_impl_.time_),
          reinterpret_cast<char*>(&other->_impl_.time_));
}

::google::protobuf::Metadata LogEntry::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

Progress::Progress(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Progress_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:clickhouse.grpc.Progress)
}
Progress::Progress(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Progress& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Progress_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE Progress::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void Progress::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, read_rows_),
           0,
           offsetof(Impl_, written_bytes_) -
               offsetof(Impl_, read_rows_) +
               sizeof(Impl_::written_bytes_));
}
Progress::~Progress() {
  // @@protoc_insertion_point(destructor:clickhouse.grpc.Progress)
  SharedDtor(*this);
}
inline void Progress::SharedDtor(MessageLite& self) {
  Progress& this_ = static_cast<Progress&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Progress_class_data_ =
        Progress::InternalGenerateClassData_(Progress_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Progress::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Progress_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Progress_class_data_.tc_table);
  return Progress_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Progress::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Progress_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Progress_globals_));
  return Progress_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Progress::ParseTableT_
    Progress::_table_ =
        Progress::InternalGenerateParseTable_(Progress_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Progress::Clear() {
// @@protoc_insertion_point(message_clear_start:clickhouse.grpc.Progress)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    ::memset(&_impl_.read_rows_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.written_bytes_) -
        reinterpret_cast<char*>(&_impl_.read_rows_)) + sizeof(_impl_.written_bytes_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Progress::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Progress& this_ = static_cast<const Progress&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Progress::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Progress& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:clickhouse.grpc.Progress)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // uint64 read_rows = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_read_rows() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          1, this_._internal_read_rows(), target);
    }
  }

  // uint64 read_bytes = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_read_bytes() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          2, this_._internal_read_bytes(), target);
    }
  }

  // uint64 total_rows_to_read = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_total_rows_to_read() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_total_rows_to_read(), target);
    }
  }

  // uint64 written_rows = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_written_rows() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          4, this_._internal_written_rows(), target);
    }
  }

  // uint64 written_bytes = 5;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_written_bytes() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          5, this_._internal_written_bytes(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:clickhouse.grpc.Progress)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Progress::ByteSizeLong(const MessageLite& base) {
  const Progress& this_ = static_cast<const Progress&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Progress::ByteSizeLong() const {
  const Progress& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:clickhouse.grpc.Progress)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    // uint64 read_rows = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_read_rows() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_read_rows());
      }
    }
    // uint64 read_bytes = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_read_bytes() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_read_bytes());
      }
    }
    // uint64 total_rows_to_read = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_total_rows_to_read() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_total_rows_to_read());
      }
    }
    // uint64 written_rows = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_written_rows() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_written_rows());
      }
    }
    // uint64 written_bytes = 5;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_written_bytes() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_written_bytes());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Progress::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Progress*>(&to_msg);
  auto& from = static_cast<const Progress&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:clickhouse.grpc.Progress)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (from._internal_read_rows() != 0) {
        _this->_impl_.read_rows_ = from._impl_.read_rows_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_read_bytes() != 0) {
        _this->_impl_.read_bytes_ = from._impl_.read_bytes_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_total_rows_to_read() != 0) {
        _this->_impl_.total_rows_to_read_ = from._impl_.total_rows_to_read_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_written_rows() != 0) {
        _this->_impl_.written_rows_ = from._impl_.written_rows_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_written_bytes() != 0) {
        _this->_impl_.written_bytes_ = from._impl_.written_bytes_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Progress::CopyFrom(const Progress& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:clickhouse.grpc.Progress)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Progress::InternalSwap(Progress* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Progress, _impl_.written_bytes_)
      + sizeof(Progress::_impl_.written_bytes_)
      - PROTOBUF_FIELD_OFFSET(Progress, _impl_.read_rows_)>(
          reinterpret_cast<char*>(&_impl_.read_rows_),
          reinterpret_cast<char*>(&other->_impl_.read_rows_));
}

::google::protobuf::Metadata Progress::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

Stats::Stats(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Stats_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:clickhouse.grpc.Stats)
}
Stats::Stats(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Stats& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Stats_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE Stats::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void Stats::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, rows_),
           0,
           offsetof(Impl_, applied_aggregation_) -
               offsetof(Impl_, rows_) +
               sizeof(Impl_::applied_aggregation_));
}
Stats::~Stats() {
  // @@protoc_insertion_point(destructor:clickhouse.grpc.Stats)
  SharedDtor(*this);
}
inline void Stats::SharedDtor(MessageLite& self) {
  Stats& this_ = static_cast<Stats&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Stats_class_data_ =
        Stats::InternalGenerateClassData_(Stats_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Stats::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Stats_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Stats_class_data_.tc_table);
  return Stats_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Stats::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Stats_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Stats_globals_));
  return Stats_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Stats::ParseTableT_
    Stats::_table_ =
        Stats::InternalGenerateParseTable_(Stats_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Stats::Clear() {
// @@protoc_insertion_point(message_clear_start:clickhouse.grpc.Stats)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    ::memset(&_impl_.rows_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.applied_aggregation_) -
        reinterpret_cast<char*>(&_impl_.rows_)) + sizeof(_impl_.applied_aggregation_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Stats::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Stats& this_ = static_cast<const Stats&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Stats::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Stats& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:clickhouse.grpc.Stats)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // uint64 rows = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_rows() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          1, this_._internal_rows(), target);
    }
  }

  // uint64 blocks = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_blocks() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          2, this_._internal_blocks(), target);
    }
  }

  // uint64 allocated_bytes = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_allocated_bytes() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_allocated_bytes(), target);
    }
  }

  // bool applied_limit = 4;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_applied_limit() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          4, this_._internal_applied_limit(), target);
    }
  }

  // uint64 rows_before_limit = 5;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_rows_before_limit() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          5, this_._internal_rows_before_limit(), target);
    }
  }

  // bool applied_aggregation = 6;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_applied_aggregation() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          6, this_._internal_applied_aggregation(), target);
    }
  }

  // uint64 rows_before_aggregation = 7;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_rows_before_aggregation() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          7, this_._internal_rows_before_aggregation(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:clickhouse.grpc.Stats)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Stats::ByteSizeLong(const MessageLite& base) {
  const Stats& this_ = static_cast<const Stats&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Stats::ByteSizeLong() const {
  const Stats& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:clickhouse.grpc.Stats)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    // uint64 rows = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_rows() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_rows());
      }
    }
    // uint64 blocks = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_blocks() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_blocks());
      }
    }
    // uint64 allocated_bytes = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_allocated_bytes() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_allocated_bytes());
      }
    }
    // uint64 rows_before_limit = 5;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_rows_before_limit() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_rows_before_limit());
      }
    }
    // uint64 rows_before_aggregation = 7;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_rows_before_aggregation() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_rows_before_aggregation());
      }
    }
    // bool applied_limit = 4;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_applied_limit() != 0) {
        total_size += 2;
      }
    }
    // bool applied_aggregation = 6;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_applied_aggregation() != 0) {
        total_size += 2;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Stats::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Stats*>(&to_msg);
  auto& from = static_cast<const Stats&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:clickhouse.grpc.Stats)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (from._internal_rows() != 0) {
        _this->_impl_.rows_ = from._impl_.rows_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_blocks() != 0) {
        _this->_impl_.blocks_ = from._impl_.blocks_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_allocated_bytes() != 0) {
        _this->_impl_.allocated_bytes_ = from._impl_.allocated_bytes_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_rows_before_limit() != 0) {
        _this->_impl_.rows_before_limit_ = from._impl_.rows_before_limit_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_rows_before_aggregation() != 0) {
        _this->_impl_.rows_before_aggregation_ = from._impl_.rows_before_aggregation_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_applied_limit() != 0) {
        _this->_impl_.applied_limit_ = from._impl_.applied_limit_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_applied_aggregation() != 0) {
        _this->_impl_.applied_aggregation_ = from._impl_.applied_aggregation_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Stats::CopyFrom(const Stats& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:clickhouse.grpc.Stats)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Stats::InternalSwap(Stats* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Stats, _impl_.applied_aggregation_)
      + sizeof(Stats::_impl_.applied_aggregation_)
      - PROTOBUF_FIELD_OFFSET(Stats, _impl_.rows_)>(
          reinterpret_cast<char*>(&_impl_.rows_),
          reinterpret_cast<char*>(&other->_impl_.rows_));
}

::google::protobuf::Metadata Stats::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

Exception::Exception(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Exception_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:clickhouse.grpc.Exception)
}
PROTOBUF_NDEBUG_INLINE Exception::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::clickhouse::grpc::Exception& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        name_(arena, from.name_),
        display_text_(arena, from.display_text_),
        stack_trace_(arena, from.stack_trace_) {}

Exception::Exception(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Exception& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Exception_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Exception* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.code_ = from._impl_.code_;

  // @@protoc_insertion_point(copy_constructor:clickhouse.grpc.Exception)
}
PROTOBUF_NDEBUG_INLINE Exception::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        name_(arena),
        display_text_(arena),
        stack_trace_(arena) {}

inline void Exception::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.code_ = {};
}
Exception::~Exception() {
  // @@protoc_insertion_point(destructor:clickhouse.grpc.Exception)
  SharedDtor(*this);
}
inline void Exception::SharedDtor(MessageLite& self) {
  Exception& this_ = static_cast<Exception&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.name_.Destroy();
  this_._impl_.display_text_.Destroy();
  this_._impl_.stack_trace_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Exception_class_data_ =
        Exception::InternalGenerateClassData_(Exception_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Exception::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Exception_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Exception_class_data_.tc_table);
  return Exception_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Exception::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Exception_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Exception_globals_));
  return Exception_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Exception::ParseTableT_
    Exception::_table_ =
        Exception::InternalGenerateParseTable_(Exception_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Exception::Clear() {
// @@protoc_insertion_point(message_clear_start:clickhouse.grpc.Exception)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.name_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.display_text_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.stack_trace_.ClearNonDefaultToEmpty();
    }
  }
  _impl_.code_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Exception::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Exception& this_ = static_cast<const Exception&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Exception::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Exception& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:clickhouse.grpc.Exception)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // int32 code = 1;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_code() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<1>(
              stream, this_._internal_code(), target);
    }
  }

  // string name = 2;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_name().empty()) {
      const ::std::string& _s = this_._internal_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.Exception.name");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  // string display_text = 3;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_display_text().empty()) {
      const ::std::string& _s = this_._internal_display_text();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.Exception.display_text");
      target = stream->WriteStringMaybeAliased(3, _s, target);
    }
  }

  // string stack_trace = 4;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_stack_trace().empty()) {
      const ::std::string& _s = this_._internal_stack_trace();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.Exception.stack_trace");
      target = stream->WriteStringMaybeAliased(4, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:clickhouse.grpc.Exception)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Exception::ByteSizeLong(const MessageLite& base) {
  const Exception& this_ = static_cast<const Exception&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Exception::ByteSizeLong() const {
  const Exception& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:clickhouse.grpc.Exception)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // string name = 2;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_name());
      }
    }
    // string display_text = 3;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_display_text().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_display_text());
      }
    }
    // string stack_trace = 4;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!this_._internal_stack_trace().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_stack_trace());
      }
    }
    // int32 code = 1;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_code() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_code());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Exception::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Exception*>(&to_msg);
  auto& from = static_cast<const Exception&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:clickhouse.grpc.Exception)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_name().empty()) {
        _this->_internal_set_name(from._internal_name());
      } else {
        if (_this->_impl_.name_.IsDefault()) {
          _this->_internal_set_name("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_display_text().empty()) {
        _this->_internal_set_display_text(from._internal_display_text());
      } else {
        if (_this->_impl_.display_text_.IsDefault()) {
          _this->_internal_set_display_text("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!from._internal_stack_trace().empty()) {
        _this->_internal_set_stack_trace(from._internal_stack_trace());
      } else {
        if (_this->_impl_.stack_trace_.IsDefault()) {
          _this->_internal_set_stack_trace("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_code() != 0) {
        _this->_impl_.code_ = from._impl_.code_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Exception::CopyFrom(const Exception& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:clickhouse.grpc.Exception)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Exception::InternalSwap(Exception* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.name_, &other->_impl_.name_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.display_text_, &other->_impl_.display_text_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.stack_trace_, &other->_impl_.stack_trace_, arena);
  swap(_impl_.code_, other->_impl_.code_);
}

::google::protobuf::Metadata Exception::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

Result::Result(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Result_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:clickhouse.grpc.Result)
}
PROTOBUF_NDEBUG_INLINE Result::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::clickhouse::grpc::Result& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        logs_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::clickhouse::grpc::Result,
              PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Result, _impl_.logs_)>()
          , from.logs_
        }
        ,
        output_columns_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::clickhouse::grpc::Result,
              PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Result, _impl_.output_columns_)>()
          , from.output_columns_
        }
        ,
        output_(arena, from.output_),
        totals_(arena, from.totals_),
        extremes_(arena, from.extremes_),
        query_id_(arena, from.query_id_),
        time_zone_(arena, from.time_zone_),
        output_format_(arena, from.output_format_) {}

Result::Result(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Result& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Result_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Result* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.progress_ = (CheckHasBit(cached_has_bits, 0x00000100U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.progress_)
                : nullptr;
  _impl_.stats_ = (CheckHasBit(cached_has_bits, 0x00000200U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.stats_)
                : nullptr;
  _impl_.exception_ = (CheckHasBit(cached_has_bits, 0x00000400U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.exception_)
                : nullptr;
  _impl_.cancelled_ = from._impl_.cancelled_;

  // @@protoc_insertion_point(copy_constructor:clickhouse.grpc.Result)
}
PROTOBUF_NDEBUG_INLINE Result::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        logs_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::clickhouse::grpc::Result,
            PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Result, _impl_.logs_)>()
         }
        ,
        output_columns_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::clickhouse::grpc::Result,
            PROTOBUF_FIELD_OFFSET(::clickhouse::grpc::Result, _impl_.output_columns_)>()
         }
        ,
        output_(arena),
        totals_(arena),
        extremes_(arena),
        query_id_(arena),
        time_zone_(arena),
        output_format_(arena) {}

inline void Result::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, progress_),
           0,
           offsetof(Impl_, cancelled_) -
               offsetof(Impl_, progress_) +
               sizeof(Impl_::cancelled_));
}
Result::~Result() {
  // @@protoc_insertion_point(destructor:clickhouse.grpc.Result)
  SharedDtor(*this);
}
inline void Result::SharedDtor(MessageLite& self) {
  Result& this_ = static_cast<Result&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.output_.Destroy();
  this_._impl_.totals_.Destroy();
  this_._impl_.extremes_.Destroy();
  this_._impl_.query_id_.Destroy();
  this_._impl_.time_zone_.Destroy();
  this_._impl_.output_format_.Destroy();
  delete this_._impl_.progress_;
  delete this_._impl_.stats_;
  delete this_._impl_.exception_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Result_class_data_ =
        Result::InternalGenerateClassData_(Result_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Result::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Result_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Result_class_data_.tc_table);
  return Result_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Result::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Result_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Result_globals_));
  return Result_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Result::ParseTableT_
    Result::_table_ =
        Result::InternalGenerateParseTable_(Result_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Result::Clear() {
// @@protoc_insertion_point(message_clear_start:clickhouse.grpc.Result)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.logs_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.output_columns_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.output_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _impl_.totals_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      _impl_.extremes_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      _impl_.query_id_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      _impl_.time_zone_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      _impl_.output_format_.ClearNonDefaultToEmpty();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000700U)) {
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      ABSL_DCHECK(_impl_.progress_ != nullptr);
      _impl_.progress_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000200U)) {
      ABSL_DCHECK(_impl_.stats_ != nullptr);
      _impl_.stats_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000400U)) {
      ABSL_DCHECK(_impl_.exception_ != nullptr);
      _impl_.exception_->Clear();
    }
  }
  _impl_.cancelled_ = false;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Result::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Result& this_ = static_cast<const Result&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Result::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Result& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:clickhouse.grpc.Result)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // bytes output = 1;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_output().empty()) {
      const ::std::string& _s = this_._internal_output();
      target = stream->WriteBytesMaybeAliased(1, _s, target);
    }
  }

  // bytes totals = 2;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (!this_._internal_totals().empty()) {
      const ::std::string& _s = this_._internal_totals();
      target = stream->WriteBytesMaybeAliased(2, _s, target);
    }
  }

  // bytes extremes = 3;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (!this_._internal_extremes().empty()) {
      const ::std::string& _s = this_._internal_extremes();
      target = stream->WriteBytesMaybeAliased(3, _s, target);
    }
  }

  // repeated .clickhouse.grpc.LogEntry logs = 4;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_logs_size());
         i < n; i++) {
      const auto& repfield = this_._internal_logs().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              4, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  // .clickhouse.grpc.Progress progress = 5;
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        5, *this_._impl_.progress_, this_._impl_.progress_->GetCachedSize(), target,
        stream);
  }

  // .clickhouse.grpc.Stats stats = 6;
  if (CheckHasBit(cached_has_bits, 0x00000200U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        6, *this_._impl_.stats_, this_._impl_.stats_->GetCachedSize(), target,
        stream);
  }

  // .clickhouse.grpc.Exception exception = 7;
  if (CheckHasBit(cached_has_bits, 0x00000400U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        7, *this_._impl_.exception_, this_._impl_.exception_->GetCachedSize(), target,
        stream);
  }

  // bool cancelled = 8;
  if (CheckHasBit(cached_has_bits, 0x00000800U)) {
    if (this_._internal_cancelled() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          8, this_._internal_cancelled(), target);
    }
  }

  // string query_id = 9;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (!this_._internal_query_id().empty()) {
      const ::std::string& _s = this_._internal_query_id();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.Result.query_id");
      target = stream->WriteStringMaybeAliased(9, _s, target);
    }
  }

  // string time_zone = 10;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (!this_._internal_time_zone().empty()) {
      const ::std::string& _s = this_._internal_time_zone();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.Result.time_zone");
      target = stream->WriteStringMaybeAliased(10, _s, target);
    }
  }

  // string output_format = 11;
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    if (!this_._internal_output_format().empty()) {
      const ::std::string& _s = this_._internal_output_format();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "clickhouse.grpc.Result.output_format");
      target = stream->WriteStringMaybeAliased(11, _s, target);
    }
  }

  // repeated .clickhouse.grpc.NameAndType output_columns = 12;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_output_columns_size());
         i < n; i++) {
      const auto& repfield = this_._internal_output_columns().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              12, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:clickhouse.grpc.Result)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Result::ByteSizeLong(const MessageLite& base) {
  const Result& this_ = static_cast<const Result&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Result::ByteSizeLong() const {
  const Result& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:clickhouse.grpc.Result)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // repeated .clickhouse.grpc.LogEntry logs = 4;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_logs_size();
      for (const auto& msg : this_._internal_logs()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
    // repeated .clickhouse.grpc.NameAndType output_columns = 12;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1UL * this_._internal_output_columns_size();
      for (const auto& msg : this_._internal_output_columns()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
    // bytes output = 1;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!this_._internal_output().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::BytesSize(
                                        this_._internal_output());
      }
    }
    // bytes totals = 2;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!this_._internal_totals().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::BytesSize(
                                        this_._internal_totals());
      }
    }
    // bytes extremes = 3;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (!this_._internal_extremes().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::BytesSize(
                                        this_._internal_extremes());
      }
    }
    // string query_id = 9;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (!this_._internal_query_id().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_query_id());
      }
    }
    // string time_zone = 10;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (!this_._internal_time_zone().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_time_zone());
      }
    }
    // string output_format = 11;
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (!this_._internal_output_format().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_output_format());
      }
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000f00U)) {
    // .clickhouse.grpc.Progress progress = 5;
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.progress_);
    }
    // .clickhouse.grpc.Stats stats = 6;
    if (CheckHasBit(cached_has_bits, 0x00000200U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.stats_);
    }
    // .clickhouse.grpc.Exception exception = 7;
    if (CheckHasBit(cached_has_bits, 0x00000400U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exception_);
    }
    // bool cancelled = 8;
    if (CheckHasBit(cached_has_bits, 0x00000800U)) {
      if (this_._internal_cancelled() != 0) {
        total_size += 2;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Result::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Result*>(&to_msg);
  auto& from = static_cast<const Result&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:clickhouse.grpc.Result)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_logs()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_logs());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _this->_internal_mutable_output_columns()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_output_columns());
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!from._internal_output().empty()) {
        _this->_internal_set_output(from._internal_output());
      } else {
        if (_this->_impl_.output_.IsDefault()) {
          _this->_internal_set_output("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!from._internal_totals().empty()) {
        _this->_internal_set_totals(from._internal_totals());
      } else {
        if (_this->_impl_.totals_.IsDefault()) {
          _this->_internal_set_totals("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (!from._internal_extremes().empty()) {
        _this->_internal_set_extremes(from._internal_extremes());
      } else {
        if (_this->_impl_.extremes_.IsDefault()) {
          _this->_internal_set_extremes("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (!from._internal_query_id().empty()) {
        _this->_internal_set_query_id(from._internal_query_id());
      } else {
        if (_this->_impl_.query_id_.IsDefault()) {
          _this->_internal_set_query_id("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (!from._internal_time_zone().empty()) {
        _this->_internal_set_time_zone(from._internal_time_zone());
      } else {
        if (_this->_impl_.time_zone_.IsDefault()) {
          _this->_internal_set_time_zone("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (!from._internal_output_format().empty()) {
        _this->_internal_set_output_format(from._internal_output_format());
      } else {
        if (_this->_impl_.output_format_.IsDefault()) {
          _this->_internal_set_output_format("");
        }
      }
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000f00U)) {
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      ABSL_DCHECK(from._impl_.progress_ != nullptr);
      if (_this->_impl_.progress_ == nullptr) {
        _this->_impl_.progress_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.progress_);
      } else {
        _this->_impl_.progress_->MergeFrom(*from._impl_.progress_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000200U)) {
      ABSL_DCHECK(from._impl_.stats_ != nullptr);
      if (_this->_impl_.stats_ == nullptr) {
        _this->_impl_.stats_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.stats_);
      } else {
        _this->_impl_.stats_->MergeFrom(*from._impl_.stats_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000400U)) {
      ABSL_DCHECK(from._impl_.exception_ != nullptr);
      if (_this->_impl_.exception_ == nullptr) {
        _this->_impl_.exception_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.exception_);
      } else {
        _this->_impl_.exception_->MergeFrom(*from._impl_.exception_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000800U)) {
      if (from._internal_cancelled() != 0) {
        _this->_impl_.cancelled_ = from._impl_.cancelled_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Result::CopyFrom(const Result& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:clickhouse.grpc.Result)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Result::InternalSwap(Result* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.logs_.InternalSwap(&other->_impl_.logs_);
  _impl_.output_columns_.InternalSwap(&other->_impl_.output_columns_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.output_, &other->_impl_.output_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.totals_, &other->_impl_.totals_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.extremes_, &other->_impl_.extremes_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.query_id_, &other->_impl_.query_id_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.time_zone_, &other->_impl_.time_zone_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.output_format_, &other->_impl_.output_format_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Result, _impl_.cancelled_)
      + sizeof(Result::_impl_.cancelled_)
      - PROTOBUF_FIELD_OFFSET(Result, _impl_.progress_)>(
          reinterpret_cast<char*>(&_impl_.progress_),
          reinterpret_cast<char*>(&other->_impl_.progress_));
}

::google::protobuf::Metadata Result::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
}  // namespace grpc
}  // namespace clickhouse
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_clickhouse_5fgrpc_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
