// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: google/pubsub/v1/pubsub.proto
// Protobuf C++ Version: 7.35.1

#include "google/pubsub/v1/pubsub.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
#ifdef PROTOBUF_MESSAGE_GLOBALS
namespace {
PROTOBUF_CONSTINIT ::google::protobuf::internal::ReflectionData
    file_reflection_data[] = {
        // ::google::pubsub::v1::MessageStoragePolicy
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::SchemaSettings
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::IngestionDataSourceSettings_AwsKinesis
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_TextFormat
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_AvroFormat
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::IngestionDataSourceSettings
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::PlatformLogsSettings
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::IngestionFailureEvent_ApiViolationReason
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::IngestionFailureEvent_AvroFailureReason
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::IngestionFailureEvent_CloudStorageFailure
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::IngestionFailureEvent
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::Topic_LabelsEntry_DoNotUse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::Topic
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::PubsubMessage_AttributesEntry_DoNotUse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::PubsubMessage
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::GetTopicRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::UpdateTopicRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::PublishRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::PublishResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ListTopicsRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ListTopicsResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ListTopicSubscriptionsRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ListTopicSubscriptionsResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ListTopicSnapshotsRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ListTopicSnapshotsResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::DeleteTopicRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::DetachSubscriptionRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::DetachSubscriptionResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::Subscription_AnalyticsHubSubscriptionInfo
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::Subscription_LabelsEntry_DoNotUse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::Subscription
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::RetryPolicy
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::DeadLetterPolicy
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ExpirationPolicy
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::PushConfig_OidcToken
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::PushConfig_PubsubWrapper
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::PushConfig_NoWrapper
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::PushConfig_AttributesEntry_DoNotUse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::PushConfig
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::BigQueryConfig
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::CloudStorageConfig_TextConfig
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::CloudStorageConfig_AvroConfig
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::CloudStorageConfig
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ReceivedMessage
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::GetSubscriptionRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::UpdateSubscriptionRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ListSubscriptionsRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ListSubscriptionsResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::DeleteSubscriptionRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ModifyPushConfigRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::PullRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::PullResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ModifyAckDeadlineRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::AcknowledgeRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::StreamingPullRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::StreamingPullResponse_SubscriptionProperties
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::StreamingPullResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::CreateSnapshotRequest_LabelsEntry_DoNotUse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::CreateSnapshotRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::UpdateSnapshotRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::Snapshot_LabelsEntry_DoNotUse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::Snapshot
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::GetSnapshotRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ListSnapshotsRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::ListSnapshotsResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::DeleteSnapshotRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::SeekRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
        // ::google::pubsub::v1::SeekResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto, /* tracker*/ nullptr,},
};
}  // namespace
#endif
namespace google {
namespace pubsub {
namespace v1 {
constexpr Topic_LabelsEntry_DoNotUse::ParseTableT_ Topic_LabelsEntry_DoNotUse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Topic_LabelsEntry_DoNotUse, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::DiscardEverythingFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::Topic_LabelsEntry_DoNotUse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string value = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(Topic_LabelsEntry_DoNotUse, _impl_.value_)}},
      // string key = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(Topic_LabelsEntry_DoNotUse, _impl_.key_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string key = 1;
      {PROTOBUF_FIELD_OFFSET(Topic_LabelsEntry_DoNotUse, _impl_.key_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string value = 2;
      {PROTOBUF_FIELD_OFFSET(Topic_LabelsEntry_DoNotUse, _impl_.value_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\42\3\5\0\0\0\0\0"
      "google.pubsub.v1.Topic.LabelsEntry"
      "key"
      "value"
    }},
  };
}

template <typename>
constexpr Topic_LabelsEntry_DoNotUse::Topic_LabelsEntry_DoNotUse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Topic_LabelsEntry_DoNotUse::MapEntry(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL Topic_LabelsEntry_DoNotUse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Topic_LabelsEntry_DoNotUse(arena);
}
constexpr auto Topic_LabelsEntry_DoNotUse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Topic_LabelsEntry_DoNotUse), alignof(Topic_LabelsEntry_DoNotUse));
}
constexpr auto Topic_LabelsEntry_DoNotUse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Topic_LabelsEntry_DoNotUse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<Topic_LabelsEntry_DoNotUse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Topic_LabelsEntry_DoNotUse::SharedDtor,
          static_cast<void (::google::protobuf::MessageLite::*)()>(&Topic_LabelsEntry_DoNotUse::ClearImpl),
              ::google::protobuf::Message::ByteSizeLongImpl, ::google::protobuf::Message::_InternalSerializeImpl
              ,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Topic_LabelsEntry_DoNotUse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[13],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct Topic_LabelsEntry_DoNotUseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr Topic_LabelsEntry_DoNotUseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Topic_LabelsEntry_DoNotUse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Topic_LabelsEntry_DoNotUse::InternalGenerateClassData_(
            _default, &Topic_LabelsEntry_DoNotUse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Topic_LabelsEntry_DoNotUse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~Topic_LabelsEntry_DoNotUseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Topic_LabelsEntry_DoNotUse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Topic_LabelsEntry_DoNotUse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(Topic_LabelsEntry_DoNotUseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST Topic_LabelsEntry_DoNotUseGlobalsTypeInternal Topic_LabelsEntry_DoNotUse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Topic_LabelsEntry_DoNotUse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Topic_LabelsEntry_DoNotUse_globals_.GetClassData();
#else
  return Topic_LabelsEntry_DoNotUse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
constexpr Subscription_LabelsEntry_DoNotUse::ParseTableT_ Subscription_LabelsEntry_DoNotUse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Subscription_LabelsEntry_DoNotUse, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::DiscardEverythingFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::Subscription_LabelsEntry_DoNotUse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string value = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(Subscription_LabelsEntry_DoNotUse, _impl_.value_)}},
      // string key = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(Subscription_LabelsEntry_DoNotUse, _impl_.key_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string key = 1;
      {PROTOBUF_FIELD_OFFSET(Subscription_LabelsEntry_DoNotUse, _impl_.key_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string value = 2;
      {PROTOBUF_FIELD_OFFSET(Subscription_LabelsEntry_DoNotUse, _impl_.value_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\51\3\5\0\0\0\0\0"
      "google.pubsub.v1.Subscription.LabelsEntry"
      "key"
      "value"
    }},
  };
}

template <typename>
constexpr Subscription_LabelsEntry_DoNotUse::Subscription_LabelsEntry_DoNotUse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Subscription_LabelsEntry_DoNotUse::MapEntry(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL Subscription_LabelsEntry_DoNotUse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Subscription_LabelsEntry_DoNotUse(arena);
}
constexpr auto Subscription_LabelsEntry_DoNotUse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Subscription_LabelsEntry_DoNotUse), alignof(Subscription_LabelsEntry_DoNotUse));
}
constexpr auto Subscription_LabelsEntry_DoNotUse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Subscription_LabelsEntry_DoNotUse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<Subscription_LabelsEntry_DoNotUse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Subscription_LabelsEntry_DoNotUse::SharedDtor,
          static_cast<void (::google::protobuf::MessageLite::*)()>(&Subscription_LabelsEntry_DoNotUse::ClearImpl),
              ::google::protobuf::Message::ByteSizeLongImpl, ::google::protobuf::Message::_InternalSerializeImpl
              ,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Subscription_LabelsEntry_DoNotUse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[31],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct Subscription_LabelsEntry_DoNotUseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr Subscription_LabelsEntry_DoNotUseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Subscription_LabelsEntry_DoNotUse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Subscription_LabelsEntry_DoNotUse::InternalGenerateClassData_(
            _default, &Subscription_LabelsEntry_DoNotUse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Subscription_LabelsEntry_DoNotUse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~Subscription_LabelsEntry_DoNotUseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Subscription_LabelsEntry_DoNotUse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Subscription_LabelsEntry_DoNotUse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(Subscription_LabelsEntry_DoNotUseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST Subscription_LabelsEntry_DoNotUseGlobalsTypeInternal Subscription_LabelsEntry_DoNotUse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Subscription_LabelsEntry_DoNotUse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Subscription_LabelsEntry_DoNotUse_globals_.GetClassData();
#else
  return Subscription_LabelsEntry_DoNotUse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Subscription_AnalyticsHubSubscriptionInfo::_Internal {
 public:
  using HasBits = decltype(::std::declval<Subscription_AnalyticsHubSubscriptionInfo>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Subscription_AnalyticsHubSubscriptionInfo, _impl_._has_bits_);
};

constexpr Subscription_AnalyticsHubSubscriptionInfo::ParseTableT_ Subscription_AnalyticsHubSubscriptionInfo::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Subscription_AnalyticsHubSubscriptionInfo, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::Subscription_AnalyticsHubSubscriptionInfo>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string subscription = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(Subscription_AnalyticsHubSubscriptionInfo, _impl_.subscription_)}},
      // string listing = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(Subscription_AnalyticsHubSubscriptionInfo, _impl_.listing_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string listing = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Subscription_AnalyticsHubSubscriptionInfo, _impl_.listing_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string subscription = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Subscription_AnalyticsHubSubscriptionInfo, _impl_.subscription_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\72\7\14\0\0\0\0\0"
      "google.pubsub.v1.Subscription.AnalyticsHubSubscriptionInfo"
      "listing"
      "subscription"
    }},
  };
}


inline constexpr Subscription_AnalyticsHubSubscriptionInfo::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        listing_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        subscription_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr Subscription_AnalyticsHubSubscriptionInfo::Subscription_AnalyticsHubSubscriptionInfo(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Subscription_AnalyticsHubSubscriptionInfo::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Subscription_AnalyticsHubSubscriptionInfo(arena);
}
constexpr auto Subscription_AnalyticsHubSubscriptionInfo::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Subscription_AnalyticsHubSubscriptionInfo), alignof(Subscription_AnalyticsHubSubscriptionInfo));
}
constexpr auto Subscription_AnalyticsHubSubscriptionInfo::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Subscription_AnalyticsHubSubscriptionInfo::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<Subscription_AnalyticsHubSubscriptionInfo>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Subscription_AnalyticsHubSubscriptionInfo::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<Subscription_AnalyticsHubSubscriptionInfo>(), &Subscription_AnalyticsHubSubscriptionInfo::ByteSizeLong,
              &Subscription_AnalyticsHubSubscriptionInfo::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Subscription_AnalyticsHubSubscriptionInfo, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[30],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct Subscription_AnalyticsHubSubscriptionInfoGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr Subscription_AnalyticsHubSubscriptionInfoGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Subscription_AnalyticsHubSubscriptionInfo_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Subscription_AnalyticsHubSubscriptionInfo::InternalGenerateClassData_(
            _default, &Subscription_AnalyticsHubSubscriptionInfo_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Subscription_AnalyticsHubSubscriptionInfo>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~Subscription_AnalyticsHubSubscriptionInfoGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Subscription_AnalyticsHubSubscriptionInfo _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Subscription_AnalyticsHubSubscriptionInfo>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(Subscription_AnalyticsHubSubscriptionInfoGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST Subscription_AnalyticsHubSubscriptionInfoGlobalsTypeInternal Subscription_AnalyticsHubSubscriptionInfo_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Subscription_AnalyticsHubSubscriptionInfo_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Subscription_AnalyticsHubSubscriptionInfo_globals_.GetClassData();
#else
  return Subscription_AnalyticsHubSubscriptionInfo_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class StreamingPullResponse_SubscriptionProperties::_Internal {
 public:
  using HasBits = decltype(::std::declval<StreamingPullResponse_SubscriptionProperties>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(StreamingPullResponse_SubscriptionProperties, _impl_._has_bits_);
};

constexpr StreamingPullResponse_SubscriptionProperties::ParseTableT_ StreamingPullResponse_SubscriptionProperties::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(StreamingPullResponse_SubscriptionProperties, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::StreamingPullResponse_SubscriptionProperties>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // bool message_ordering_enabled = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(StreamingPullResponse_SubscriptionProperties, _impl_.message_ordering_enabled_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(StreamingPullResponse_SubscriptionProperties, _impl_.message_ordering_enabled_)}},
      // bool exactly_once_delivery_enabled = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(StreamingPullResponse_SubscriptionProperties, _impl_.exactly_once_delivery_enabled_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(StreamingPullResponse_SubscriptionProperties, _impl_.exactly_once_delivery_enabled_)}},
    }}, {{
      65535, 65535
    }}, {{
      // bool exactly_once_delivery_enabled = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(StreamingPullResponse_SubscriptionProperties, _impl_.exactly_once_delivery_enabled_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // bool message_ordering_enabled = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(StreamingPullResponse_SubscriptionProperties, _impl_.message_ordering_enabled_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr StreamingPullResponse_SubscriptionProperties::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        exactly_once_delivery_enabled_{false},
        message_ordering_enabled_{false} {}

template <typename>
constexpr StreamingPullResponse_SubscriptionProperties::StreamingPullResponse_SubscriptionProperties(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL StreamingPullResponse_SubscriptionProperties::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) StreamingPullResponse_SubscriptionProperties(arena);
}
constexpr auto StreamingPullResponse_SubscriptionProperties::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(StreamingPullResponse_SubscriptionProperties), alignof(StreamingPullResponse_SubscriptionProperties));
}
constexpr auto StreamingPullResponse_SubscriptionProperties::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &StreamingPullResponse_SubscriptionProperties::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<StreamingPullResponse_SubscriptionProperties>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &StreamingPullResponse_SubscriptionProperties::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<StreamingPullResponse_SubscriptionProperties>(), &StreamingPullResponse_SubscriptionProperties::ByteSizeLong,
              &StreamingPullResponse_SubscriptionProperties::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(StreamingPullResponse_SubscriptionProperties, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[59],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct StreamingPullResponse_SubscriptionPropertiesGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr StreamingPullResponse_SubscriptionPropertiesGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 StreamingPullResponse_SubscriptionProperties_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(StreamingPullResponse_SubscriptionProperties::InternalGenerateClassData_(
            _default, &StreamingPullResponse_SubscriptionProperties_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<StreamingPullResponse_SubscriptionProperties>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~StreamingPullResponse_SubscriptionPropertiesGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) StreamingPullResponse_SubscriptionProperties _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<StreamingPullResponse_SubscriptionProperties>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(StreamingPullResponse_SubscriptionPropertiesGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST StreamingPullResponse_SubscriptionPropertiesGlobalsTypeInternal StreamingPullResponse_SubscriptionProperties_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* StreamingPullResponse_SubscriptionProperties_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return StreamingPullResponse_SubscriptionProperties_globals_.GetClassData();
#else
  return StreamingPullResponse_SubscriptionProperties_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class StreamingPullResponse_ModifyAckDeadlineConfirmation::_Internal {
 public:
  using HasBits = decltype(::std::declval<StreamingPullResponse_ModifyAckDeadlineConfirmation>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_._has_bits_);
};

constexpr StreamingPullResponse_ModifyAckDeadlineConfirmation::ParseTableT_ StreamingPullResponse_ModifyAckDeadlineConfirmation::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_._has_bits_),
      0, // no _extensions_
      3, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // repeated string ack_ids = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_.ack_ids_)}},
      // repeated string invalid_ack_ids = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUR1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_.invalid_ack_ids_)}},
      // repeated string temporary_failed_ack_ids = 3 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUR1,
       {26, 2, 0,
        PROTOBUF_FIELD_OFFSET(StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_.temporary_failed_ack_ids_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated string ack_ids = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_.ack_ids_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // repeated string invalid_ack_ids = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_.invalid_ack_ids_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // repeated string temporary_failed_ack_ids = 3 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_.temporary_failed_ack_ids_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
    }},
    // no aux_entries
    {{
      "\104\7\17\30\0\0\0\0"
      "google.pubsub.v1.StreamingPullResponse.ModifyAckDeadlineConfirmation"
      "ack_ids"
      "invalid_ack_ids"
      "temporary_failed_ack_ids"
    }},
  };
}


inline constexpr StreamingPullResponse_ModifyAckDeadlineConfirmation::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_.ack_ids_)>()
         }
        ,
        invalid_ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_.invalid_ack_ids_)>()
         }
        ,
        temporary_failed_ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_.temporary_failed_ack_ids_)>()
         }
     {}

template <typename>
constexpr StreamingPullResponse_ModifyAckDeadlineConfirmation::StreamingPullResponse_ModifyAckDeadlineConfirmation(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL StreamingPullResponse_ModifyAckDeadlineConfirmation::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) StreamingPullResponse_ModifyAckDeadlineConfirmation(arena);
}
constexpr auto StreamingPullResponse_ModifyAckDeadlineConfirmation::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(StreamingPullResponse_ModifyAckDeadlineConfirmation), alignof(StreamingPullResponse_ModifyAckDeadlineConfirmation));
}
constexpr auto StreamingPullResponse_ModifyAckDeadlineConfirmation::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &StreamingPullResponse_ModifyAckDeadlineConfirmation::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<StreamingPullResponse_ModifyAckDeadlineConfirmation>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &StreamingPullResponse_ModifyAckDeadlineConfirmation::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<StreamingPullResponse_ModifyAckDeadlineConfirmation>(), &StreamingPullResponse_ModifyAckDeadlineConfirmation::ByteSizeLong,
              &StreamingPullResponse_ModifyAckDeadlineConfirmation::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[58],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct StreamingPullResponse_ModifyAckDeadlineConfirmationGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr StreamingPullResponse_ModifyAckDeadlineConfirmationGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 StreamingPullResponse_ModifyAckDeadlineConfirmation_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(StreamingPullResponse_ModifyAckDeadlineConfirmation::InternalGenerateClassData_(
            _default, &StreamingPullResponse_ModifyAckDeadlineConfirmation_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<StreamingPullResponse_ModifyAckDeadlineConfirmation>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~StreamingPullResponse_ModifyAckDeadlineConfirmationGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) StreamingPullResponse_ModifyAckDeadlineConfirmation _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<StreamingPullResponse_ModifyAckDeadlineConfirmation>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(StreamingPullResponse_ModifyAckDeadlineConfirmationGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST StreamingPullResponse_ModifyAckDeadlineConfirmationGlobalsTypeInternal StreamingPullResponse_ModifyAckDeadlineConfirmation_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* StreamingPullResponse_ModifyAckDeadlineConfirmation_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return StreamingPullResponse_ModifyAckDeadlineConfirmation_globals_.GetClassData();
#else
  return StreamingPullResponse_ModifyAckDeadlineConfirmation_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class StreamingPullResponse_AcknowledgeConfirmation::_Internal {
 public:
  using HasBits = decltype(::std::declval<StreamingPullResponse_AcknowledgeConfirmation>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(StreamingPullResponse_AcknowledgeConfirmation, _impl_._has_bits_);
};

constexpr StreamingPullResponse_AcknowledgeConfirmation::ParseTableT_ StreamingPullResponse_AcknowledgeConfirmation::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(StreamingPullResponse_AcknowledgeConfirmation, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated string temporary_failed_ack_ids = 4 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUR1,
       {34, 3, 0,
        PROTOBUF_FIELD_OFFSET(StreamingPullResponse_AcknowledgeConfirmation, _impl_.temporary_failed_ack_ids_)}},
      // repeated string ack_ids = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(StreamingPullResponse_AcknowledgeConfirmation, _impl_.ack_ids_)}},
      // repeated string invalid_ack_ids = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUR1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(StreamingPullResponse_AcknowledgeConfirmation, _impl_.invalid_ack_ids_)}},
      // repeated string unordered_ack_ids = 3 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUR1,
       {26, 2, 0,
        PROTOBUF_FIELD_OFFSET(StreamingPullResponse_AcknowledgeConfirmation, _impl_.unordered_ack_ids_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated string ack_ids = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(StreamingPullResponse_AcknowledgeConfirmation, _impl_.ack_ids_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // repeated string invalid_ack_ids = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(StreamingPullResponse_AcknowledgeConfirmation, _impl_.invalid_ack_ids_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // repeated string unordered_ack_ids = 3 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(StreamingPullResponse_AcknowledgeConfirmation, _impl_.unordered_ack_ids_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // repeated string temporary_failed_ack_ids = 4 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(StreamingPullResponse_AcknowledgeConfirmation, _impl_.temporary_failed_ack_ids_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
    }},
    // no aux_entries
    {{
      "\76\7\17\21\30\0\0\0"
      "google.pubsub.v1.StreamingPullResponse.AcknowledgeConfirmation"
      "ack_ids"
      "invalid_ack_ids"
      "unordered_ack_ids"
      "temporary_failed_ack_ids"
    }},
  };
}


inline constexpr StreamingPullResponse_AcknowledgeConfirmation::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation, _impl_.ack_ids_)>()
         }
        ,
        invalid_ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation, _impl_.invalid_ack_ids_)>()
         }
        ,
        unordered_ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation, _impl_.unordered_ack_ids_)>()
         }
        ,
        temporary_failed_ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation, _impl_.temporary_failed_ack_ids_)>()
         }
     {}

template <typename>
constexpr StreamingPullResponse_AcknowledgeConfirmation::StreamingPullResponse_AcknowledgeConfirmation(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL StreamingPullResponse_AcknowledgeConfirmation::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) StreamingPullResponse_AcknowledgeConfirmation(arena);
}
constexpr auto StreamingPullResponse_AcknowledgeConfirmation::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(StreamingPullResponse_AcknowledgeConfirmation), alignof(StreamingPullResponse_AcknowledgeConfirmation));
}
constexpr auto StreamingPullResponse_AcknowledgeConfirmation::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &StreamingPullResponse_AcknowledgeConfirmation::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<StreamingPullResponse_AcknowledgeConfirmation>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &StreamingPullResponse_AcknowledgeConfirmation::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<StreamingPullResponse_AcknowledgeConfirmation>(), &StreamingPullResponse_AcknowledgeConfirmation::ByteSizeLong,
              &StreamingPullResponse_AcknowledgeConfirmation::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(StreamingPullResponse_AcknowledgeConfirmation, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[57],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct StreamingPullResponse_AcknowledgeConfirmationGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr StreamingPullResponse_AcknowledgeConfirmationGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 StreamingPullResponse_AcknowledgeConfirmation_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(StreamingPullResponse_AcknowledgeConfirmation::InternalGenerateClassData_(
            _default, &StreamingPullResponse_AcknowledgeConfirmation_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<StreamingPullResponse_AcknowledgeConfirmation>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~StreamingPullResponse_AcknowledgeConfirmationGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) StreamingPullResponse_AcknowledgeConfirmation _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<StreamingPullResponse_AcknowledgeConfirmation>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(StreamingPullResponse_AcknowledgeConfirmationGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST StreamingPullResponse_AcknowledgeConfirmationGlobalsTypeInternal StreamingPullResponse_AcknowledgeConfirmation_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* StreamingPullResponse_AcknowledgeConfirmation_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return StreamingPullResponse_AcknowledgeConfirmation_globals_.GetClassData();
#else
  return StreamingPullResponse_AcknowledgeConfirmation_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class StreamingPullRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<StreamingPullRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(StreamingPullRequest, _impl_._has_bits_);
};

constexpr StreamingPullRequest::ParseTableT_ StreamingPullRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(StreamingPullRequest, _impl_._has_bits_),
      0, // no _extensions_
      8, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967040,  // skipmap
      offsetof(ParseTableT_, field_entries),
      8,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::StreamingPullRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // int64 max_outstanding_bytes = 8 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(StreamingPullRequest, _impl_.max_outstanding_bytes_), 6>(),
       {64, 6, 0,
        PROTOBUF_FIELD_OFFSET(StreamingPullRequest, _impl_.max_outstanding_bytes_)}},
      // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 3, 0,
        PROTOBUF_FIELD_OFFSET(StreamingPullRequest, _impl_.subscription_)}},
      // repeated string ack_ids = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUR1,
       {18, 0, 0,
        PROTOBUF_FIELD_OFFSET(StreamingPullRequest, _impl_.ack_ids_)}},
      // repeated int32 modify_deadline_seconds = 3 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastV32P1,
       {26, 1, 0,
        PROTOBUF_FIELD_OFFSET(StreamingPullRequest, _impl_.modify_deadline_seconds_)}},
      // repeated string modify_deadline_ack_ids = 4 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUR1,
       {34, 2, 0,
        PROTOBUF_FIELD_OFFSET(StreamingPullRequest, _impl_.modify_deadline_ack_ids_)}},
      // int32 stream_ack_deadline_seconds = 5 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(StreamingPullRequest, _impl_.stream_ack_deadline_seconds_), 7>(),
       {40, 7, 0,
        PROTOBUF_FIELD_OFFSET(StreamingPullRequest, _impl_.stream_ack_deadline_seconds_)}},
      // string client_id = 6 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {50, 4, 0,
        PROTOBUF_FIELD_OFFSET(StreamingPullRequest, _impl_.client_id_)}},
      // int64 max_outstanding_messages = 7 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(StreamingPullRequest, _impl_.max_outstanding_messages_), 5>(),
       {56, 5, 0,
        PROTOBUF_FIELD_OFFSET(StreamingPullRequest, _impl_.max_outstanding_messages_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(StreamingPullRequest, _impl_.subscription_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // repeated string ack_ids = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(StreamingPullRequest, _impl_.ack_ids_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // repeated int32 modify_deadline_seconds = 3 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(StreamingPullRequest, _impl_.modify_deadline_seconds_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedInt32)},
      // repeated string modify_deadline_ack_ids = 4 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(StreamingPullRequest, _impl_.modify_deadline_ack_ids_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // int32 stream_ack_deadline_seconds = 5 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(StreamingPullRequest, _impl_.stream_ack_deadline_seconds_), _Internal::kHasBitsOffset + 7, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // string client_id = 6 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(StreamingPullRequest, _impl_.client_id_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // int64 max_outstanding_messages = 7 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(StreamingPullRequest, _impl_.max_outstanding_messages_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // int64 max_outstanding_bytes = 8 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(StreamingPullRequest, _impl_.max_outstanding_bytes_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
    }},
    // no aux_entries
    {{
      "\45\14\7\0\27\0\11\0\0\0\0\0\0\0\0\0"
      "google.pubsub.v1.StreamingPullRequest"
      "subscription"
      "ack_ids"
      "modify_deadline_ack_ids"
      "client_id"
    }},
  };
}


inline constexpr StreamingPullRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullRequest,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullRequest, _impl_.ack_ids_)>()
         }
        ,
        modify_deadline_seconds_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullRequest,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullRequest, _impl_.modify_deadline_seconds_)>()
         }
        ,
        _modify_deadline_seconds_cached_byte_size_{0},
        modify_deadline_ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullRequest,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullRequest, _impl_.modify_deadline_ack_ids_)>()
         }
        ,
        subscription_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        client_id_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        max_outstanding_messages_{::int64_t{0}},
        max_outstanding_bytes_{::int64_t{0}},
        stream_ack_deadline_seconds_{0} {}

template <typename>
constexpr StreamingPullRequest::StreamingPullRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL StreamingPullRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) StreamingPullRequest(arena);
}
constexpr auto StreamingPullRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(StreamingPullRequest), alignof(StreamingPullRequest));
}
constexpr auto StreamingPullRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &StreamingPullRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<StreamingPullRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &StreamingPullRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<StreamingPullRequest>(), &StreamingPullRequest::ByteSizeLong,
              &StreamingPullRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(StreamingPullRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[56],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct StreamingPullRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr StreamingPullRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 StreamingPullRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(StreamingPullRequest::InternalGenerateClassData_(
            _default, &StreamingPullRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<StreamingPullRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~StreamingPullRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) StreamingPullRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<StreamingPullRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(StreamingPullRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST StreamingPullRequestGlobalsTypeInternal StreamingPullRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* StreamingPullRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return StreamingPullRequest_globals_.GetClassData();
#else
  return StreamingPullRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
constexpr Snapshot_LabelsEntry_DoNotUse::ParseTableT_ Snapshot_LabelsEntry_DoNotUse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Snapshot_LabelsEntry_DoNotUse, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::DiscardEverythingFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::Snapshot_LabelsEntry_DoNotUse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string value = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(Snapshot_LabelsEntry_DoNotUse, _impl_.value_)}},
      // string key = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(Snapshot_LabelsEntry_DoNotUse, _impl_.key_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string key = 1;
      {PROTOBUF_FIELD_OFFSET(Snapshot_LabelsEntry_DoNotUse, _impl_.key_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string value = 2;
      {PROTOBUF_FIELD_OFFSET(Snapshot_LabelsEntry_DoNotUse, _impl_.value_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\45\3\5\0\0\0\0\0"
      "google.pubsub.v1.Snapshot.LabelsEntry"
      "key"
      "value"
    }},
  };
}

template <typename>
constexpr Snapshot_LabelsEntry_DoNotUse::Snapshot_LabelsEntry_DoNotUse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Snapshot_LabelsEntry_DoNotUse::MapEntry(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL Snapshot_LabelsEntry_DoNotUse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Snapshot_LabelsEntry_DoNotUse(arena);
}
constexpr auto Snapshot_LabelsEntry_DoNotUse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Snapshot_LabelsEntry_DoNotUse), alignof(Snapshot_LabelsEntry_DoNotUse));
}
constexpr auto Snapshot_LabelsEntry_DoNotUse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Snapshot_LabelsEntry_DoNotUse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<Snapshot_LabelsEntry_DoNotUse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Snapshot_LabelsEntry_DoNotUse::SharedDtor,
          static_cast<void (::google::protobuf::MessageLite::*)()>(&Snapshot_LabelsEntry_DoNotUse::ClearImpl),
              ::google::protobuf::Message::ByteSizeLongImpl, ::google::protobuf::Message::_InternalSerializeImpl
              ,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Snapshot_LabelsEntry_DoNotUse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[64],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct Snapshot_LabelsEntry_DoNotUseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr Snapshot_LabelsEntry_DoNotUseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Snapshot_LabelsEntry_DoNotUse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Snapshot_LabelsEntry_DoNotUse::InternalGenerateClassData_(
            _default, &Snapshot_LabelsEntry_DoNotUse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Snapshot_LabelsEntry_DoNotUse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~Snapshot_LabelsEntry_DoNotUseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Snapshot_LabelsEntry_DoNotUse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Snapshot_LabelsEntry_DoNotUse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(Snapshot_LabelsEntry_DoNotUseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST Snapshot_LabelsEntry_DoNotUseGlobalsTypeInternal Snapshot_LabelsEntry_DoNotUse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Snapshot_LabelsEntry_DoNotUse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Snapshot_LabelsEntry_DoNotUse_globals_.GetClassData();
#else
  return Snapshot_LabelsEntry_DoNotUse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class SeekResponse::_Internal {
 public:
};

constexpr SeekResponse::ParseTableT_ SeekResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(SeekResponse,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::SeekResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr SeekResponse::SeekResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL SeekResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) SeekResponse(arena);
}
constexpr auto SeekResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(SeekResponse), alignof(SeekResponse));
}
constexpr auto SeekResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &SeekResponse::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<SeekResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &SeekResponse::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<SeekResponse>(), &SeekResponse::ByteSizeLong,
              &SeekResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(SeekResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[71],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct SeekResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr SeekResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 SeekResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(SeekResponse::InternalGenerateClassData_(
            _default, &SeekResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<SeekResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~SeekResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) SeekResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<SeekResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(SeekResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST SeekResponseGlobalsTypeInternal SeekResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* SeekResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return SeekResponse_globals_.GetClassData();
#else
  return SeekResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class SchemaSettings::_Internal {
 public:
  using HasBits = decltype(::std::declval<SchemaSettings>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(SchemaSettings, _impl_._has_bits_);
};

constexpr SchemaSettings::ParseTableT_ SchemaSettings::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(SchemaSettings, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::SchemaSettings>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string last_revision_id = 4 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {34, 2, 0,
        PROTOBUF_FIELD_OFFSET(SchemaSettings, _impl_.last_revision_id_)}},
      // string schema = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(SchemaSettings, _impl_.schema_)}},
      // .google.pubsub.v1.Encoding encoding = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(SchemaSettings, _impl_.encoding_), 3>(),
       {16, 3, 0,
        PROTOBUF_FIELD_OFFSET(SchemaSettings, _impl_.encoding_)}},
      // string first_revision_id = 3 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {26, 1, 0,
        PROTOBUF_FIELD_OFFSET(SchemaSettings, _impl_.first_revision_id_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string schema = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(SchemaSettings, _impl_.schema_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.pubsub.v1.Encoding encoding = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(SchemaSettings, _impl_.encoding_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // string first_revision_id = 3 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(SchemaSettings, _impl_.first_revision_id_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string last_revision_id = 4 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(SchemaSettings, _impl_.last_revision_id_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\37\6\0\21\20\0\0\0"
      "google.pubsub.v1.SchemaSettings"
      "schema"
      "first_revision_id"
      "last_revision_id"
    }},
  };
}


inline constexpr SchemaSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        schema_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        first_revision_id_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        last_revision_id_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        encoding_{static_cast< ::google::pubsub::v1::Encoding >(0)} {}

template <typename>
constexpr SchemaSettings::SchemaSettings(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL SchemaSettings::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) SchemaSettings(arena);
}
constexpr auto SchemaSettings::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(SchemaSettings), alignof(SchemaSettings));
}
constexpr auto SchemaSettings::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &SchemaSettings::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<SchemaSettings>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &SchemaSettings::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<SchemaSettings>(), &SchemaSettings::ByteSizeLong,
              &SchemaSettings::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(SchemaSettings, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[1],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct SchemaSettingsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr SchemaSettingsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 SchemaSettings_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(SchemaSettings::InternalGenerateClassData_(
            _default, &SchemaSettings_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<SchemaSettings>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~SchemaSettingsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) SchemaSettings _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<SchemaSettings>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(SchemaSettingsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST SchemaSettingsGlobalsTypeInternal SchemaSettings_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* SchemaSettings_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return SchemaSettings_globals_.GetClassData();
#else
  return SchemaSettings_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class PushConfig_PubsubWrapper::_Internal {
 public:
};

constexpr PushConfig_PubsubWrapper::ParseTableT_ PushConfig_PubsubWrapper::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(PushConfig_PubsubWrapper,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::PushConfig_PubsubWrapper>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr PushConfig_PubsubWrapper::PushConfig_PubsubWrapper(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL PushConfig_PubsubWrapper::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) PushConfig_PubsubWrapper(arena);
}
constexpr auto PushConfig_PubsubWrapper::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(PushConfig_PubsubWrapper), alignof(PushConfig_PubsubWrapper));
}
constexpr auto PushConfig_PubsubWrapper::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &PushConfig_PubsubWrapper::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<PushConfig_PubsubWrapper>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &PushConfig_PubsubWrapper::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<PushConfig_PubsubWrapper>(), &PushConfig_PubsubWrapper::ByteSizeLong,
              &PushConfig_PubsubWrapper::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(PushConfig_PubsubWrapper, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[37],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct PushConfig_PubsubWrapperGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr PushConfig_PubsubWrapperGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 PushConfig_PubsubWrapper_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(PushConfig_PubsubWrapper::InternalGenerateClassData_(
            _default, &PushConfig_PubsubWrapper_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<PushConfig_PubsubWrapper>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~PushConfig_PubsubWrapperGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) PushConfig_PubsubWrapper _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<PushConfig_PubsubWrapper>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(PushConfig_PubsubWrapperGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST PushConfig_PubsubWrapperGlobalsTypeInternal PushConfig_PubsubWrapper_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* PushConfig_PubsubWrapper_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return PushConfig_PubsubWrapper_globals_.GetClassData();
#else
  return PushConfig_PubsubWrapper_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class PushConfig_OidcToken::_Internal {
 public:
  using HasBits = decltype(::std::declval<PushConfig_OidcToken>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(PushConfig_OidcToken, _impl_._has_bits_);
};

constexpr PushConfig_OidcToken::ParseTableT_ PushConfig_OidcToken::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(PushConfig_OidcToken, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::PushConfig_OidcToken>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string audience = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(PushConfig_OidcToken, _impl_.audience_)}},
      // string service_account_email = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(PushConfig_OidcToken, _impl_.service_account_email_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string service_account_email = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(PushConfig_OidcToken, _impl_.service_account_email_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string audience = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(PushConfig_OidcToken, _impl_.audience_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\45\25\10\0\0\0\0\0"
      "google.pubsub.v1.PushConfig.OidcToken"
      "service_account_email"
      "audience"
    }},
  };
}


inline constexpr PushConfig_OidcToken::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        service_account_email_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        audience_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr PushConfig_OidcToken::PushConfig_OidcToken(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL PushConfig_OidcToken::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) PushConfig_OidcToken(arena);
}
constexpr auto PushConfig_OidcToken::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(PushConfig_OidcToken), alignof(PushConfig_OidcToken));
}
constexpr auto PushConfig_OidcToken::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &PushConfig_OidcToken::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<PushConfig_OidcToken>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &PushConfig_OidcToken::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<PushConfig_OidcToken>(), &PushConfig_OidcToken::ByteSizeLong,
              &PushConfig_OidcToken::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(PushConfig_OidcToken, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[36],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct PushConfig_OidcTokenGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr PushConfig_OidcTokenGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 PushConfig_OidcToken_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(PushConfig_OidcToken::InternalGenerateClassData_(
            _default, &PushConfig_OidcToken_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<PushConfig_OidcToken>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~PushConfig_OidcTokenGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) PushConfig_OidcToken _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<PushConfig_OidcToken>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(PushConfig_OidcTokenGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST PushConfig_OidcTokenGlobalsTypeInternal PushConfig_OidcToken_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* PushConfig_OidcToken_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return PushConfig_OidcToken_globals_.GetClassData();
#else
  return PushConfig_OidcToken_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class PushConfig_NoWrapper::_Internal {
 public:
  using HasBits = decltype(::std::declval<PushConfig_NoWrapper>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(PushConfig_NoWrapper, _impl_._has_bits_);
};

constexpr PushConfig_NoWrapper::ParseTableT_ PushConfig_NoWrapper::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(PushConfig_NoWrapper, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::PushConfig_NoWrapper>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // bool write_metadata = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(PushConfig_NoWrapper, _impl_.write_metadata_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(PushConfig_NoWrapper, _impl_.write_metadata_)}},
    }}, {{
      65535, 65535
    }}, {{
      // bool write_metadata = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(PushConfig_NoWrapper, _impl_.write_metadata_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr PushConfig_NoWrapper::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        write_metadata_{false} {}

template <typename>
constexpr PushConfig_NoWrapper::PushConfig_NoWrapper(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL PushConfig_NoWrapper::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) PushConfig_NoWrapper(arena);
}
constexpr auto PushConfig_NoWrapper::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(PushConfig_NoWrapper), alignof(PushConfig_NoWrapper));
}
constexpr auto PushConfig_NoWrapper::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &PushConfig_NoWrapper::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<PushConfig_NoWrapper>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &PushConfig_NoWrapper::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<PushConfig_NoWrapper>(), &PushConfig_NoWrapper::ByteSizeLong,
              &PushConfig_NoWrapper::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(PushConfig_NoWrapper, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[38],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct PushConfig_NoWrapperGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr PushConfig_NoWrapperGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 PushConfig_NoWrapper_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(PushConfig_NoWrapper::InternalGenerateClassData_(
            _default, &PushConfig_NoWrapper_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<PushConfig_NoWrapper>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~PushConfig_NoWrapperGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) PushConfig_NoWrapper _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<PushConfig_NoWrapper>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(PushConfig_NoWrapperGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST PushConfig_NoWrapperGlobalsTypeInternal PushConfig_NoWrapper_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* PushConfig_NoWrapper_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return PushConfig_NoWrapper_globals_.GetClassData();
#else
  return PushConfig_NoWrapper_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
constexpr PushConfig_AttributesEntry_DoNotUse::ParseTableT_ PushConfig_AttributesEntry_DoNotUse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(PushConfig_AttributesEntry_DoNotUse, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::DiscardEverythingFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::PushConfig_AttributesEntry_DoNotUse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string value = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(PushConfig_AttributesEntry_DoNotUse, _impl_.value_)}},
      // string key = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(PushConfig_AttributesEntry_DoNotUse, _impl_.key_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string key = 1;
      {PROTOBUF_FIELD_OFFSET(PushConfig_AttributesEntry_DoNotUse, _impl_.key_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string value = 2;
      {PROTOBUF_FIELD_OFFSET(PushConfig_AttributesEntry_DoNotUse, _impl_.value_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\53\3\5\0\0\0\0\0"
      "google.pubsub.v1.PushConfig.AttributesEntry"
      "key"
      "value"
    }},
  };
}

template <typename>
constexpr PushConfig_AttributesEntry_DoNotUse::PushConfig_AttributesEntry_DoNotUse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : PushConfig_AttributesEntry_DoNotUse::MapEntry(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL PushConfig_AttributesEntry_DoNotUse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) PushConfig_AttributesEntry_DoNotUse(arena);
}
constexpr auto PushConfig_AttributesEntry_DoNotUse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(PushConfig_AttributesEntry_DoNotUse), alignof(PushConfig_AttributesEntry_DoNotUse));
}
constexpr auto PushConfig_AttributesEntry_DoNotUse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &PushConfig_AttributesEntry_DoNotUse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<PushConfig_AttributesEntry_DoNotUse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &PushConfig_AttributesEntry_DoNotUse::SharedDtor,
          static_cast<void (::google::protobuf::MessageLite::*)()>(&PushConfig_AttributesEntry_DoNotUse::ClearImpl),
              ::google::protobuf::Message::ByteSizeLongImpl, ::google::protobuf::Message::_InternalSerializeImpl
              ,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(PushConfig_AttributesEntry_DoNotUse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[39],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct PushConfig_AttributesEntry_DoNotUseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr PushConfig_AttributesEntry_DoNotUseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 PushConfig_AttributesEntry_DoNotUse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(PushConfig_AttributesEntry_DoNotUse::InternalGenerateClassData_(
            _default, &PushConfig_AttributesEntry_DoNotUse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<PushConfig_AttributesEntry_DoNotUse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~PushConfig_AttributesEntry_DoNotUseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) PushConfig_AttributesEntry_DoNotUse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<PushConfig_AttributesEntry_DoNotUse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(PushConfig_AttributesEntry_DoNotUseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST PushConfig_AttributesEntry_DoNotUseGlobalsTypeInternal PushConfig_AttributesEntry_DoNotUse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* PushConfig_AttributesEntry_DoNotUse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return PushConfig_AttributesEntry_DoNotUse_globals_.GetClassData();
#else
  return PushConfig_AttributesEntry_DoNotUse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class PullRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<PullRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(PullRequest, _impl_._has_bits_);
};

constexpr PullRequest::ParseTableT_ PullRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(PullRequest, _impl_._has_bits_),
      0, // no _extensions_
      3, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::PullRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(PullRequest, _impl_.subscription_)}},
      // bool return_immediately = 2 [deprecated = true, (.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(PullRequest, _impl_.return_immediately_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(PullRequest, _impl_.return_immediately_)}},
      // int32 max_messages = 3 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(PullRequest, _impl_.max_messages_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(PullRequest, _impl_.max_messages_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(PullRequest, _impl_.subscription_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // bool return_immediately = 2 [deprecated = true, (.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(PullRequest, _impl_.return_immediately_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // int32 max_messages = 3 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(PullRequest, _impl_.max_messages_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    }},
    // no aux_entries
    {{
      "\34\14\0\0\0\0\0\0"
      "google.pubsub.v1.PullRequest"
      "subscription"
    }},
  };
}


inline constexpr PullRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        subscription_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        return_immediately_{false},
        max_messages_{0} {}

template <typename>
constexpr PullRequest::PullRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL PullRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) PullRequest(arena);
}
constexpr auto PullRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(PullRequest), alignof(PullRequest));
}
constexpr auto PullRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &PullRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<PullRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &PullRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<PullRequest>(), &PullRequest::ByteSizeLong,
              &PullRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(PullRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[52],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct PullRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr PullRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 PullRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(PullRequest::InternalGenerateClassData_(
            _default, &PullRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<PullRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~PullRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) PullRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<PullRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(PullRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST PullRequestGlobalsTypeInternal PullRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* PullRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return PullRequest_globals_.GetClassData();
#else
  return PullRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
constexpr PubsubMessage_AttributesEntry_DoNotUse::ParseTableT_ PubsubMessage_AttributesEntry_DoNotUse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(PubsubMessage_AttributesEntry_DoNotUse, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::DiscardEverythingFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::PubsubMessage_AttributesEntry_DoNotUse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string value = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(PubsubMessage_AttributesEntry_DoNotUse, _impl_.value_)}},
      // string key = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(PubsubMessage_AttributesEntry_DoNotUse, _impl_.key_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string key = 1;
      {PROTOBUF_FIELD_OFFSET(PubsubMessage_AttributesEntry_DoNotUse, _impl_.key_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string value = 2;
      {PROTOBUF_FIELD_OFFSET(PubsubMessage_AttributesEntry_DoNotUse, _impl_.value_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\56\3\5\0\0\0\0\0"
      "google.pubsub.v1.PubsubMessage.AttributesEntry"
      "key"
      "value"
    }},
  };
}

template <typename>
constexpr PubsubMessage_AttributesEntry_DoNotUse::PubsubMessage_AttributesEntry_DoNotUse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : PubsubMessage_AttributesEntry_DoNotUse::MapEntry(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL PubsubMessage_AttributesEntry_DoNotUse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) PubsubMessage_AttributesEntry_DoNotUse(arena);
}
constexpr auto PubsubMessage_AttributesEntry_DoNotUse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(PubsubMessage_AttributesEntry_DoNotUse), alignof(PubsubMessage_AttributesEntry_DoNotUse));
}
constexpr auto PubsubMessage_AttributesEntry_DoNotUse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &PubsubMessage_AttributesEntry_DoNotUse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<PubsubMessage_AttributesEntry_DoNotUse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &PubsubMessage_AttributesEntry_DoNotUse::SharedDtor,
          static_cast<void (::google::protobuf::MessageLite::*)()>(&PubsubMessage_AttributesEntry_DoNotUse::ClearImpl),
              ::google::protobuf::Message::ByteSizeLongImpl, ::google::protobuf::Message::_InternalSerializeImpl
              ,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(PubsubMessage_AttributesEntry_DoNotUse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[15],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct PubsubMessage_AttributesEntry_DoNotUseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr PubsubMessage_AttributesEntry_DoNotUseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 PubsubMessage_AttributesEntry_DoNotUse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(PubsubMessage_AttributesEntry_DoNotUse::InternalGenerateClassData_(
            _default, &PubsubMessage_AttributesEntry_DoNotUse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<PubsubMessage_AttributesEntry_DoNotUse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~PubsubMessage_AttributesEntry_DoNotUseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) PubsubMessage_AttributesEntry_DoNotUse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<PubsubMessage_AttributesEntry_DoNotUse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(PubsubMessage_AttributesEntry_DoNotUseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST PubsubMessage_AttributesEntry_DoNotUseGlobalsTypeInternal PubsubMessage_AttributesEntry_DoNotUse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* PubsubMessage_AttributesEntry_DoNotUse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return PubsubMessage_AttributesEntry_DoNotUse_globals_.GetClassData();
#else
  return PubsubMessage_AttributesEntry_DoNotUse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class PublishResponse::_Internal {
 public:
  using HasBits = decltype(::std::declval<PublishResponse>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(PublishResponse, _impl_._has_bits_);
};

constexpr PublishResponse::ParseTableT_ PublishResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(PublishResponse, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::PublishResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated string message_ids = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(PublishResponse, _impl_.message_ids_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated string message_ids = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(PublishResponse, _impl_.message_ids_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
    }},
    // no aux_entries
    {{
      "\40\13\0\0\0\0\0\0"
      "google.pubsub.v1.PublishResponse"
      "message_ids"
    }},
  };
}


inline constexpr PublishResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        message_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::PublishResponse,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PublishResponse, _impl_.message_ids_)>()
         }
     {}

template <typename>
constexpr PublishResponse::PublishResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL PublishResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) PublishResponse(arena);
}
constexpr auto PublishResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(PublishResponse), alignof(PublishResponse));
}
constexpr auto PublishResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &PublishResponse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<PublishResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &PublishResponse::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<PublishResponse>(), &PublishResponse::ByteSizeLong,
              &PublishResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(PublishResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[20],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct PublishResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr PublishResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 PublishResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(PublishResponse::InternalGenerateClassData_(
            _default, &PublishResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<PublishResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~PublishResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) PublishResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<PublishResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(PublishResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST PublishResponseGlobalsTypeInternal PublishResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* PublishResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return PublishResponse_globals_.GetClassData();
#else
  return PublishResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class PlatformLogsSettings::_Internal {
 public:
  using HasBits = decltype(::std::declval<PlatformLogsSettings>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(PlatformLogsSettings, _impl_._has_bits_);
};

constexpr PlatformLogsSettings::ParseTableT_ PlatformLogsSettings::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(PlatformLogsSettings, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::PlatformLogsSettings>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.pubsub.v1.PlatformLogsSettings.Severity severity = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(PlatformLogsSettings, _impl_.severity_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(PlatformLogsSettings, _impl_.severity_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.pubsub.v1.PlatformLogsSettings.Severity severity = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(PlatformLogsSettings, _impl_.severity_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr PlatformLogsSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        severity_{static_cast< ::google::pubsub::v1::PlatformLogsSettings_Severity >(0)} {}

template <typename>
constexpr PlatformLogsSettings::PlatformLogsSettings(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL PlatformLogsSettings::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) PlatformLogsSettings(arena);
}
constexpr auto PlatformLogsSettings::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(PlatformLogsSettings), alignof(PlatformLogsSettings));
}
constexpr auto PlatformLogsSettings::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &PlatformLogsSettings::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<PlatformLogsSettings>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &PlatformLogsSettings::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<PlatformLogsSettings>(), &PlatformLogsSettings::ByteSizeLong,
              &PlatformLogsSettings::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(PlatformLogsSettings, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[8],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct PlatformLogsSettingsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr PlatformLogsSettingsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 PlatformLogsSettings_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(PlatformLogsSettings::InternalGenerateClassData_(
            _default, &PlatformLogsSettings_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<PlatformLogsSettings>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~PlatformLogsSettingsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) PlatformLogsSettings _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<PlatformLogsSettings>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(PlatformLogsSettingsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST PlatformLogsSettingsGlobalsTypeInternal PlatformLogsSettings_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* PlatformLogsSettings_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return PlatformLogsSettings_globals_.GetClassData();
#else
  return PlatformLogsSettings_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ModifyAckDeadlineRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<ModifyAckDeadlineRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ModifyAckDeadlineRequest, _impl_._has_bits_);
};

constexpr ModifyAckDeadlineRequest::ParseTableT_ ModifyAckDeadlineRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ModifyAckDeadlineRequest, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967282,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ModifyAckDeadlineRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated string ack_ids = 4 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastUR1,
       {34, 0, 0,
        PROTOBUF_FIELD_OFFSET(ModifyAckDeadlineRequest, _impl_.ack_ids_)}},
      // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 1, 0,
        PROTOBUF_FIELD_OFFSET(ModifyAckDeadlineRequest, _impl_.subscription_)}},
      {::_pbi::TcParser::MiniParse, {}},
      // int32 ack_deadline_seconds = 3 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ModifyAckDeadlineRequest, _impl_.ack_deadline_seconds_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(ModifyAckDeadlineRequest, _impl_.ack_deadline_seconds_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(ModifyAckDeadlineRequest, _impl_.subscription_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // int32 ack_deadline_seconds = 3 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(ModifyAckDeadlineRequest, _impl_.ack_deadline_seconds_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // repeated string ack_ids = 4 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(ModifyAckDeadlineRequest, _impl_.ack_ids_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
    }},
    // no aux_entries
    {{
      "\51\14\0\7\0\0\0\0"
      "google.pubsub.v1.ModifyAckDeadlineRequest"
      "subscription"
      "ack_ids"
    }},
  };
}


inline constexpr ModifyAckDeadlineRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::ModifyAckDeadlineRequest,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ModifyAckDeadlineRequest, _impl_.ack_ids_)>()
         }
        ,
        subscription_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        ack_deadline_seconds_{0} {}

template <typename>
constexpr ModifyAckDeadlineRequest::ModifyAckDeadlineRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ModifyAckDeadlineRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ModifyAckDeadlineRequest(arena);
}
constexpr auto ModifyAckDeadlineRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ModifyAckDeadlineRequest), alignof(ModifyAckDeadlineRequest));
}
constexpr auto ModifyAckDeadlineRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ModifyAckDeadlineRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ModifyAckDeadlineRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ModifyAckDeadlineRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ModifyAckDeadlineRequest>(), &ModifyAckDeadlineRequest::ByteSizeLong,
              &ModifyAckDeadlineRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ModifyAckDeadlineRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[54],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ModifyAckDeadlineRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ModifyAckDeadlineRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ModifyAckDeadlineRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ModifyAckDeadlineRequest::InternalGenerateClassData_(
            _default, &ModifyAckDeadlineRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ModifyAckDeadlineRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ModifyAckDeadlineRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ModifyAckDeadlineRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ModifyAckDeadlineRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ModifyAckDeadlineRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ModifyAckDeadlineRequestGlobalsTypeInternal ModifyAckDeadlineRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ModifyAckDeadlineRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ModifyAckDeadlineRequest_globals_.GetClassData();
#else
  return ModifyAckDeadlineRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class MessageStoragePolicy::_Internal {
 public:
  using HasBits = decltype(::std::declval<MessageStoragePolicy>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(MessageStoragePolicy, _impl_._has_bits_);
};

constexpr MessageStoragePolicy::ParseTableT_ MessageStoragePolicy::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(MessageStoragePolicy, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::MessageStoragePolicy>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // bool enforce_in_transit = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(MessageStoragePolicy, _impl_.enforce_in_transit_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(MessageStoragePolicy, _impl_.enforce_in_transit_)}},
      // repeated string allowed_persistence_regions = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(MessageStoragePolicy, _impl_.allowed_persistence_regions_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated string allowed_persistence_regions = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(MessageStoragePolicy, _impl_.allowed_persistence_regions_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // bool enforce_in_transit = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(MessageStoragePolicy, _impl_.enforce_in_transit_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
    }},
    // no aux_entries
    {{
      "\45\33\0\0\0\0\0\0"
      "google.pubsub.v1.MessageStoragePolicy"
      "allowed_persistence_regions"
    }},
  };
}


inline constexpr MessageStoragePolicy::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        allowed_persistence_regions_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::MessageStoragePolicy,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::MessageStoragePolicy, _impl_.allowed_persistence_regions_)>()
         }
        ,
        enforce_in_transit_{false} {}

template <typename>
constexpr MessageStoragePolicy::MessageStoragePolicy(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL MessageStoragePolicy::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) MessageStoragePolicy(arena);
}
constexpr auto MessageStoragePolicy::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(MessageStoragePolicy), alignof(MessageStoragePolicy));
}
constexpr auto MessageStoragePolicy::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &MessageStoragePolicy::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<MessageStoragePolicy>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &MessageStoragePolicy::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<MessageStoragePolicy>(), &MessageStoragePolicy::ByteSizeLong,
              &MessageStoragePolicy::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(MessageStoragePolicy, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[0],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct MessageStoragePolicyGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr MessageStoragePolicyGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 MessageStoragePolicy_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(MessageStoragePolicy::InternalGenerateClassData_(
            _default, &MessageStoragePolicy_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<MessageStoragePolicy>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~MessageStoragePolicyGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) MessageStoragePolicy _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<MessageStoragePolicy>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(MessageStoragePolicyGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST MessageStoragePolicyGlobalsTypeInternal MessageStoragePolicy_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* MessageStoragePolicy_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return MessageStoragePolicy_globals_.GetClassData();
#else
  return MessageStoragePolicy_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ListTopicsRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<ListTopicsRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ListTopicsRequest, _impl_._has_bits_);
};

constexpr ListTopicsRequest::ParseTableT_ ListTopicsRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ListTopicsRequest, _impl_._has_bits_),
      0, // no _extensions_
      3, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ListTopicsRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string project = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ListTopicsRequest, _impl_.project_)}},
      // int32 page_size = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ListTopicsRequest, _impl_.page_size_), 2>(),
       {16, 2, 0,
        PROTOBUF_FIELD_OFFSET(ListTopicsRequest, _impl_.page_size_)}},
      // string page_token = 3 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {26, 1, 0,
        PROTOBUF_FIELD_OFFSET(ListTopicsRequest, _impl_.page_token_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string project = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(ListTopicsRequest, _impl_.project_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // int32 page_size = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ListTopicsRequest, _impl_.page_size_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // string page_token = 3 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ListTopicsRequest, _impl_.page_token_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\42\7\0\12\0\0\0\0"
      "google.pubsub.v1.ListTopicsRequest"
      "project"
      "page_token"
    }},
  };
}


inline constexpr ListTopicsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        project_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        page_token_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        page_size_{0} {}

template <typename>
constexpr ListTopicsRequest::ListTopicsRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ListTopicsRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ListTopicsRequest(arena);
}
constexpr auto ListTopicsRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ListTopicsRequest), alignof(ListTopicsRequest));
}
constexpr auto ListTopicsRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ListTopicsRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ListTopicsRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ListTopicsRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ListTopicsRequest>(), &ListTopicsRequest::ByteSizeLong,
              &ListTopicsRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ListTopicsRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[21],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ListTopicsRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ListTopicsRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ListTopicsRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ListTopicsRequest::InternalGenerateClassData_(
            _default, &ListTopicsRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ListTopicsRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ListTopicsRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ListTopicsRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ListTopicsRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ListTopicsRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ListTopicsRequestGlobalsTypeInternal ListTopicsRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ListTopicsRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ListTopicsRequest_globals_.GetClassData();
#else
  return ListTopicsRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ListTopicSubscriptionsResponse::_Internal {
 public:
  using HasBits = decltype(::std::declval<ListTopicSubscriptionsResponse>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ListTopicSubscriptionsResponse, _impl_._has_bits_);
};

constexpr ListTopicSubscriptionsResponse::ParseTableT_ ListTopicSubscriptionsResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ListTopicSubscriptionsResponse, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ListTopicSubscriptionsResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string next_page_token = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(ListTopicSubscriptionsResponse, _impl_.next_page_token_)}},
      // repeated string subscriptions = 1 [(.google.api.field_behavior) = OPTIONAL, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ListTopicSubscriptionsResponse, _impl_.subscriptions_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated string subscriptions = 1 [(.google.api.field_behavior) = OPTIONAL, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(ListTopicSubscriptionsResponse, _impl_.subscriptions_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // string next_page_token = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ListTopicSubscriptionsResponse, _impl_.next_page_token_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\57\15\17\0\0\0\0\0"
      "google.pubsub.v1.ListTopicSubscriptionsResponse"
      "subscriptions"
      "next_page_token"
    }},
  };
}


inline constexpr ListTopicSubscriptionsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        subscriptions_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::ListTopicSubscriptionsResponse,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicSubscriptionsResponse, _impl_.subscriptions_)>()
         }
        ,
        next_page_token_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr ListTopicSubscriptionsResponse::ListTopicSubscriptionsResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ListTopicSubscriptionsResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ListTopicSubscriptionsResponse(arena);
}
constexpr auto ListTopicSubscriptionsResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ListTopicSubscriptionsResponse), alignof(ListTopicSubscriptionsResponse));
}
constexpr auto ListTopicSubscriptionsResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ListTopicSubscriptionsResponse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ListTopicSubscriptionsResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ListTopicSubscriptionsResponse::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ListTopicSubscriptionsResponse>(), &ListTopicSubscriptionsResponse::ByteSizeLong,
              &ListTopicSubscriptionsResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ListTopicSubscriptionsResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[24],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ListTopicSubscriptionsResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ListTopicSubscriptionsResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ListTopicSubscriptionsResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ListTopicSubscriptionsResponse::InternalGenerateClassData_(
            _default, &ListTopicSubscriptionsResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ListTopicSubscriptionsResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ListTopicSubscriptionsResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ListTopicSubscriptionsResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ListTopicSubscriptionsResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ListTopicSubscriptionsResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ListTopicSubscriptionsResponseGlobalsTypeInternal ListTopicSubscriptionsResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ListTopicSubscriptionsResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ListTopicSubscriptionsResponse_globals_.GetClassData();
#else
  return ListTopicSubscriptionsResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ListTopicSubscriptionsRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<ListTopicSubscriptionsRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ListTopicSubscriptionsRequest, _impl_._has_bits_);
};

constexpr ListTopicSubscriptionsRequest::ParseTableT_ ListTopicSubscriptionsRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ListTopicSubscriptionsRequest, _impl_._has_bits_),
      0, // no _extensions_
      3, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ListTopicSubscriptionsRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string topic = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ListTopicSubscriptionsRequest, _impl_.topic_)}},
      // int32 page_size = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ListTopicSubscriptionsRequest, _impl_.page_size_), 2>(),
       {16, 2, 0,
        PROTOBUF_FIELD_OFFSET(ListTopicSubscriptionsRequest, _impl_.page_size_)}},
      // string page_token = 3 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {26, 1, 0,
        PROTOBUF_FIELD_OFFSET(ListTopicSubscriptionsRequest, _impl_.page_token_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string topic = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(ListTopicSubscriptionsRequest, _impl_.topic_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // int32 page_size = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ListTopicSubscriptionsRequest, _impl_.page_size_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // string page_token = 3 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ListTopicSubscriptionsRequest, _impl_.page_token_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\56\5\0\12\0\0\0\0"
      "google.pubsub.v1.ListTopicSubscriptionsRequest"
      "topic"
      "page_token"
    }},
  };
}


inline constexpr ListTopicSubscriptionsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        topic_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        page_token_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        page_size_{0} {}

template <typename>
constexpr ListTopicSubscriptionsRequest::ListTopicSubscriptionsRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ListTopicSubscriptionsRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ListTopicSubscriptionsRequest(arena);
}
constexpr auto ListTopicSubscriptionsRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ListTopicSubscriptionsRequest), alignof(ListTopicSubscriptionsRequest));
}
constexpr auto ListTopicSubscriptionsRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ListTopicSubscriptionsRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ListTopicSubscriptionsRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ListTopicSubscriptionsRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ListTopicSubscriptionsRequest>(), &ListTopicSubscriptionsRequest::ByteSizeLong,
              &ListTopicSubscriptionsRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ListTopicSubscriptionsRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[23],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ListTopicSubscriptionsRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ListTopicSubscriptionsRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ListTopicSubscriptionsRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ListTopicSubscriptionsRequest::InternalGenerateClassData_(
            _default, &ListTopicSubscriptionsRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ListTopicSubscriptionsRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ListTopicSubscriptionsRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ListTopicSubscriptionsRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ListTopicSubscriptionsRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ListTopicSubscriptionsRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ListTopicSubscriptionsRequestGlobalsTypeInternal ListTopicSubscriptionsRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ListTopicSubscriptionsRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ListTopicSubscriptionsRequest_globals_.GetClassData();
#else
  return ListTopicSubscriptionsRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ListTopicSnapshotsResponse::_Internal {
 public:
  using HasBits = decltype(::std::declval<ListTopicSnapshotsResponse>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ListTopicSnapshotsResponse, _impl_._has_bits_);
};

constexpr ListTopicSnapshotsResponse::ParseTableT_ ListTopicSnapshotsResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ListTopicSnapshotsResponse, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ListTopicSnapshotsResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string next_page_token = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(ListTopicSnapshotsResponse, _impl_.next_page_token_)}},
      // repeated string snapshots = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ListTopicSnapshotsResponse, _impl_.snapshots_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated string snapshots = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ListTopicSnapshotsResponse, _impl_.snapshots_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // string next_page_token = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ListTopicSnapshotsResponse, _impl_.next_page_token_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\53\11\17\0\0\0\0\0"
      "google.pubsub.v1.ListTopicSnapshotsResponse"
      "snapshots"
      "next_page_token"
    }},
  };
}


inline constexpr ListTopicSnapshotsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        snapshots_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::ListTopicSnapshotsResponse,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicSnapshotsResponse, _impl_.snapshots_)>()
         }
        ,
        next_page_token_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr ListTopicSnapshotsResponse::ListTopicSnapshotsResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ListTopicSnapshotsResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ListTopicSnapshotsResponse(arena);
}
constexpr auto ListTopicSnapshotsResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ListTopicSnapshotsResponse), alignof(ListTopicSnapshotsResponse));
}
constexpr auto ListTopicSnapshotsResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ListTopicSnapshotsResponse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ListTopicSnapshotsResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ListTopicSnapshotsResponse::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ListTopicSnapshotsResponse>(), &ListTopicSnapshotsResponse::ByteSizeLong,
              &ListTopicSnapshotsResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ListTopicSnapshotsResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[26],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ListTopicSnapshotsResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ListTopicSnapshotsResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ListTopicSnapshotsResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ListTopicSnapshotsResponse::InternalGenerateClassData_(
            _default, &ListTopicSnapshotsResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ListTopicSnapshotsResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ListTopicSnapshotsResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ListTopicSnapshotsResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ListTopicSnapshotsResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ListTopicSnapshotsResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ListTopicSnapshotsResponseGlobalsTypeInternal ListTopicSnapshotsResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ListTopicSnapshotsResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ListTopicSnapshotsResponse_globals_.GetClassData();
#else
  return ListTopicSnapshotsResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ListTopicSnapshotsRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<ListTopicSnapshotsRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ListTopicSnapshotsRequest, _impl_._has_bits_);
};

constexpr ListTopicSnapshotsRequest::ParseTableT_ ListTopicSnapshotsRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ListTopicSnapshotsRequest, _impl_._has_bits_),
      0, // no _extensions_
      3, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ListTopicSnapshotsRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string topic = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ListTopicSnapshotsRequest, _impl_.topic_)}},
      // int32 page_size = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ListTopicSnapshotsRequest, _impl_.page_size_), 2>(),
       {16, 2, 0,
        PROTOBUF_FIELD_OFFSET(ListTopicSnapshotsRequest, _impl_.page_size_)}},
      // string page_token = 3 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {26, 1, 0,
        PROTOBUF_FIELD_OFFSET(ListTopicSnapshotsRequest, _impl_.page_token_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string topic = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(ListTopicSnapshotsRequest, _impl_.topic_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // int32 page_size = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ListTopicSnapshotsRequest, _impl_.page_size_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // string page_token = 3 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ListTopicSnapshotsRequest, _impl_.page_token_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\52\5\0\12\0\0\0\0"
      "google.pubsub.v1.ListTopicSnapshotsRequest"
      "topic"
      "page_token"
    }},
  };
}


inline constexpr ListTopicSnapshotsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        topic_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        page_token_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        page_size_{0} {}

template <typename>
constexpr ListTopicSnapshotsRequest::ListTopicSnapshotsRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ListTopicSnapshotsRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ListTopicSnapshotsRequest(arena);
}
constexpr auto ListTopicSnapshotsRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ListTopicSnapshotsRequest), alignof(ListTopicSnapshotsRequest));
}
constexpr auto ListTopicSnapshotsRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ListTopicSnapshotsRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ListTopicSnapshotsRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ListTopicSnapshotsRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ListTopicSnapshotsRequest>(), &ListTopicSnapshotsRequest::ByteSizeLong,
              &ListTopicSnapshotsRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ListTopicSnapshotsRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[25],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ListTopicSnapshotsRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ListTopicSnapshotsRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ListTopicSnapshotsRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ListTopicSnapshotsRequest::InternalGenerateClassData_(
            _default, &ListTopicSnapshotsRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ListTopicSnapshotsRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ListTopicSnapshotsRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ListTopicSnapshotsRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ListTopicSnapshotsRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ListTopicSnapshotsRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ListTopicSnapshotsRequestGlobalsTypeInternal ListTopicSnapshotsRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ListTopicSnapshotsRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ListTopicSnapshotsRequest_globals_.GetClassData();
#else
  return ListTopicSnapshotsRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ListSubscriptionsRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<ListSubscriptionsRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ListSubscriptionsRequest, _impl_._has_bits_);
};

constexpr ListSubscriptionsRequest::ParseTableT_ ListSubscriptionsRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ListSubscriptionsRequest, _impl_._has_bits_),
      0, // no _extensions_
      3, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ListSubscriptionsRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string project = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ListSubscriptionsRequest, _impl_.project_)}},
      // int32 page_size = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ListSubscriptionsRequest, _impl_.page_size_), 2>(),
       {16, 2, 0,
        PROTOBUF_FIELD_OFFSET(ListSubscriptionsRequest, _impl_.page_size_)}},
      // string page_token = 3 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {26, 1, 0,
        PROTOBUF_FIELD_OFFSET(ListSubscriptionsRequest, _impl_.page_token_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string project = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(ListSubscriptionsRequest, _impl_.project_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // int32 page_size = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ListSubscriptionsRequest, _impl_.page_size_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // string page_token = 3 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ListSubscriptionsRequest, _impl_.page_token_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\51\7\0\12\0\0\0\0"
      "google.pubsub.v1.ListSubscriptionsRequest"
      "project"
      "page_token"
    }},
  };
}


inline constexpr ListSubscriptionsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        project_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        page_token_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        page_size_{0} {}

template <typename>
constexpr ListSubscriptionsRequest::ListSubscriptionsRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ListSubscriptionsRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ListSubscriptionsRequest(arena);
}
constexpr auto ListSubscriptionsRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ListSubscriptionsRequest), alignof(ListSubscriptionsRequest));
}
constexpr auto ListSubscriptionsRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ListSubscriptionsRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ListSubscriptionsRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ListSubscriptionsRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ListSubscriptionsRequest>(), &ListSubscriptionsRequest::ByteSizeLong,
              &ListSubscriptionsRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ListSubscriptionsRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[48],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ListSubscriptionsRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ListSubscriptionsRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ListSubscriptionsRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ListSubscriptionsRequest::InternalGenerateClassData_(
            _default, &ListSubscriptionsRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ListSubscriptionsRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ListSubscriptionsRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ListSubscriptionsRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ListSubscriptionsRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ListSubscriptionsRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ListSubscriptionsRequestGlobalsTypeInternal ListSubscriptionsRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ListSubscriptionsRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ListSubscriptionsRequest_globals_.GetClassData();
#else
  return ListSubscriptionsRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ListSnapshotsRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<ListSnapshotsRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ListSnapshotsRequest, _impl_._has_bits_);
};

constexpr ListSnapshotsRequest::ParseTableT_ ListSnapshotsRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ListSnapshotsRequest, _impl_._has_bits_),
      0, // no _extensions_
      3, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ListSnapshotsRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string project = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ListSnapshotsRequest, _impl_.project_)}},
      // int32 page_size = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ListSnapshotsRequest, _impl_.page_size_), 2>(),
       {16, 2, 0,
        PROTOBUF_FIELD_OFFSET(ListSnapshotsRequest, _impl_.page_size_)}},
      // string page_token = 3 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {26, 1, 0,
        PROTOBUF_FIELD_OFFSET(ListSnapshotsRequest, _impl_.page_token_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string project = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(ListSnapshotsRequest, _impl_.project_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // int32 page_size = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ListSnapshotsRequest, _impl_.page_size_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // string page_token = 3 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ListSnapshotsRequest, _impl_.page_token_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\45\7\0\12\0\0\0\0"
      "google.pubsub.v1.ListSnapshotsRequest"
      "project"
      "page_token"
    }},
  };
}


inline constexpr ListSnapshotsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        project_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        page_token_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        page_size_{0} {}

template <typename>
constexpr ListSnapshotsRequest::ListSnapshotsRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ListSnapshotsRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ListSnapshotsRequest(arena);
}
constexpr auto ListSnapshotsRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ListSnapshotsRequest), alignof(ListSnapshotsRequest));
}
constexpr auto ListSnapshotsRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ListSnapshotsRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ListSnapshotsRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ListSnapshotsRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ListSnapshotsRequest>(), &ListSnapshotsRequest::ByteSizeLong,
              &ListSnapshotsRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ListSnapshotsRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[67],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ListSnapshotsRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ListSnapshotsRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ListSnapshotsRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ListSnapshotsRequest::InternalGenerateClassData_(
            _default, &ListSnapshotsRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ListSnapshotsRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ListSnapshotsRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ListSnapshotsRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ListSnapshotsRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ListSnapshotsRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ListSnapshotsRequestGlobalsTypeInternal ListSnapshotsRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ListSnapshotsRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ListSnapshotsRequest_globals_.GetClassData();
#else
  return ListSnapshotsRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class IngestionFailureEvent_AvroFailureReason::_Internal {
 public:
};

constexpr IngestionFailureEvent_AvroFailureReason::ParseTableT_ IngestionFailureEvent_AvroFailureReason::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(IngestionFailureEvent_AvroFailureReason,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::IngestionFailureEvent_AvroFailureReason>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr IngestionFailureEvent_AvroFailureReason::IngestionFailureEvent_AvroFailureReason(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL IngestionFailureEvent_AvroFailureReason::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) IngestionFailureEvent_AvroFailureReason(arena);
}
constexpr auto IngestionFailureEvent_AvroFailureReason::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(IngestionFailureEvent_AvroFailureReason), alignof(IngestionFailureEvent_AvroFailureReason));
}
constexpr auto IngestionFailureEvent_AvroFailureReason::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &IngestionFailureEvent_AvroFailureReason::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<IngestionFailureEvent_AvroFailureReason>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &IngestionFailureEvent_AvroFailureReason::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<IngestionFailureEvent_AvroFailureReason>(), &IngestionFailureEvent_AvroFailureReason::ByteSizeLong,
              &IngestionFailureEvent_AvroFailureReason::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(IngestionFailureEvent_AvroFailureReason, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[10],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct IngestionFailureEvent_AvroFailureReasonGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr IngestionFailureEvent_AvroFailureReasonGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 IngestionFailureEvent_AvroFailureReason_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(IngestionFailureEvent_AvroFailureReason::InternalGenerateClassData_(
            _default, &IngestionFailureEvent_AvroFailureReason_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<IngestionFailureEvent_AvroFailureReason>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~IngestionFailureEvent_AvroFailureReasonGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) IngestionFailureEvent_AvroFailureReason _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<IngestionFailureEvent_AvroFailureReason>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(IngestionFailureEvent_AvroFailureReasonGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST IngestionFailureEvent_AvroFailureReasonGlobalsTypeInternal IngestionFailureEvent_AvroFailureReason_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* IngestionFailureEvent_AvroFailureReason_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return IngestionFailureEvent_AvroFailureReason_globals_.GetClassData();
#else
  return IngestionFailureEvent_AvroFailureReason_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class IngestionFailureEvent_ApiViolationReason::_Internal {
 public:
};

constexpr IngestionFailureEvent_ApiViolationReason::ParseTableT_ IngestionFailureEvent_ApiViolationReason::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(IngestionFailureEvent_ApiViolationReason,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::IngestionFailureEvent_ApiViolationReason>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr IngestionFailureEvent_ApiViolationReason::IngestionFailureEvent_ApiViolationReason(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL IngestionFailureEvent_ApiViolationReason::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) IngestionFailureEvent_ApiViolationReason(arena);
}
constexpr auto IngestionFailureEvent_ApiViolationReason::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(IngestionFailureEvent_ApiViolationReason), alignof(IngestionFailureEvent_ApiViolationReason));
}
constexpr auto IngestionFailureEvent_ApiViolationReason::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &IngestionFailureEvent_ApiViolationReason::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<IngestionFailureEvent_ApiViolationReason>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &IngestionFailureEvent_ApiViolationReason::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<IngestionFailureEvent_ApiViolationReason>(), &IngestionFailureEvent_ApiViolationReason::ByteSizeLong,
              &IngestionFailureEvent_ApiViolationReason::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(IngestionFailureEvent_ApiViolationReason, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[9],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct IngestionFailureEvent_ApiViolationReasonGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr IngestionFailureEvent_ApiViolationReasonGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 IngestionFailureEvent_ApiViolationReason_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(IngestionFailureEvent_ApiViolationReason::InternalGenerateClassData_(
            _default, &IngestionFailureEvent_ApiViolationReason_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<IngestionFailureEvent_ApiViolationReason>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~IngestionFailureEvent_ApiViolationReasonGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) IngestionFailureEvent_ApiViolationReason _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<IngestionFailureEvent_ApiViolationReason>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(IngestionFailureEvent_ApiViolationReasonGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST IngestionFailureEvent_ApiViolationReasonGlobalsTypeInternal IngestionFailureEvent_ApiViolationReason_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* IngestionFailureEvent_ApiViolationReason_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return IngestionFailureEvent_ApiViolationReason_globals_.GetClassData();
#else
  return IngestionFailureEvent_ApiViolationReason_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class IngestionDataSourceSettings_CloudStorage_TextFormat::_Internal {
 public:
  using HasBits = decltype(::std::declval<IngestionDataSourceSettings_CloudStorage_TextFormat>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage_TextFormat, _impl_._has_bits_);
};

constexpr IngestionDataSourceSettings_CloudStorage_TextFormat::ParseTableT_ IngestionDataSourceSettings_CloudStorage_TextFormat::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage_TextFormat, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_TextFormat>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // optional string delimiter = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage_TextFormat, _impl_.delimiter_)}},
    }}, {{
      65535, 65535
    }}, {{
      // optional string delimiter = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage_TextFormat, _impl_.delimiter_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\104\11\0\0\0\0\0\0"
      "google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.TextFormat"
      "delimiter"
    }},
  };
}


inline constexpr IngestionDataSourceSettings_CloudStorage_TextFormat::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        delimiter_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr IngestionDataSourceSettings_CloudStorage_TextFormat::IngestionDataSourceSettings_CloudStorage_TextFormat(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL IngestionDataSourceSettings_CloudStorage_TextFormat::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) IngestionDataSourceSettings_CloudStorage_TextFormat(arena);
}
constexpr auto IngestionDataSourceSettings_CloudStorage_TextFormat::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(IngestionDataSourceSettings_CloudStorage_TextFormat), alignof(IngestionDataSourceSettings_CloudStorage_TextFormat));
}
constexpr auto IngestionDataSourceSettings_CloudStorage_TextFormat::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &IngestionDataSourceSettings_CloudStorage_TextFormat::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<IngestionDataSourceSettings_CloudStorage_TextFormat>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &IngestionDataSourceSettings_CloudStorage_TextFormat::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<IngestionDataSourceSettings_CloudStorage_TextFormat>(), &IngestionDataSourceSettings_CloudStorage_TextFormat::ByteSizeLong,
              &IngestionDataSourceSettings_CloudStorage_TextFormat::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage_TextFormat, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[3],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct IngestionDataSourceSettings_CloudStorage_TextFormatGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr IngestionDataSourceSettings_CloudStorage_TextFormatGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 IngestionDataSourceSettings_CloudStorage_TextFormat_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(IngestionDataSourceSettings_CloudStorage_TextFormat::InternalGenerateClassData_(
            _default, &IngestionDataSourceSettings_CloudStorage_TextFormat_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<IngestionDataSourceSettings_CloudStorage_TextFormat>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~IngestionDataSourceSettings_CloudStorage_TextFormatGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) IngestionDataSourceSettings_CloudStorage_TextFormat _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<IngestionDataSourceSettings_CloudStorage_TextFormat>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage_TextFormatGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST IngestionDataSourceSettings_CloudStorage_TextFormatGlobalsTypeInternal IngestionDataSourceSettings_CloudStorage_TextFormat_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* IngestionDataSourceSettings_CloudStorage_TextFormat_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return IngestionDataSourceSettings_CloudStorage_TextFormat_globals_.GetClassData();
#else
  return IngestionDataSourceSettings_CloudStorage_TextFormat_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat::_Internal {
 public:
};

constexpr IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat::ParseTableT_ IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat::IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat(arena);
}
constexpr auto IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat), alignof(IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat));
}
constexpr auto IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat>(), &IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat::ByteSizeLong,
              &IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[5],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct IngestionDataSourceSettings_CloudStorage_PubSubAvroFormatGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr IngestionDataSourceSettings_CloudStorage_PubSubAvroFormatGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat::InternalGenerateClassData_(
            _default, &IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~IngestionDataSourceSettings_CloudStorage_PubSubAvroFormatGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage_PubSubAvroFormatGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST IngestionDataSourceSettings_CloudStorage_PubSubAvroFormatGlobalsTypeInternal IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat_globals_.GetClassData();
#else
  return IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class IngestionDataSourceSettings_CloudStorage_AvroFormat::_Internal {
 public:
};

constexpr IngestionDataSourceSettings_CloudStorage_AvroFormat::ParseTableT_ IngestionDataSourceSettings_CloudStorage_AvroFormat::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage_AvroFormat,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_AvroFormat>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr IngestionDataSourceSettings_CloudStorage_AvroFormat::IngestionDataSourceSettings_CloudStorage_AvroFormat(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL IngestionDataSourceSettings_CloudStorage_AvroFormat::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) IngestionDataSourceSettings_CloudStorage_AvroFormat(arena);
}
constexpr auto IngestionDataSourceSettings_CloudStorage_AvroFormat::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(IngestionDataSourceSettings_CloudStorage_AvroFormat), alignof(IngestionDataSourceSettings_CloudStorage_AvroFormat));
}
constexpr auto IngestionDataSourceSettings_CloudStorage_AvroFormat::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &IngestionDataSourceSettings_CloudStorage_AvroFormat::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<IngestionDataSourceSettings_CloudStorage_AvroFormat>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &IngestionDataSourceSettings_CloudStorage_AvroFormat::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<IngestionDataSourceSettings_CloudStorage_AvroFormat>(), &IngestionDataSourceSettings_CloudStorage_AvroFormat::ByteSizeLong,
              &IngestionDataSourceSettings_CloudStorage_AvroFormat::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage_AvroFormat, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[4],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct IngestionDataSourceSettings_CloudStorage_AvroFormatGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr IngestionDataSourceSettings_CloudStorage_AvroFormatGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 IngestionDataSourceSettings_CloudStorage_AvroFormat_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(IngestionDataSourceSettings_CloudStorage_AvroFormat::InternalGenerateClassData_(
            _default, &IngestionDataSourceSettings_CloudStorage_AvroFormat_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<IngestionDataSourceSettings_CloudStorage_AvroFormat>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~IngestionDataSourceSettings_CloudStorage_AvroFormatGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) IngestionDataSourceSettings_CloudStorage_AvroFormat _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<IngestionDataSourceSettings_CloudStorage_AvroFormat>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage_AvroFormatGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST IngestionDataSourceSettings_CloudStorage_AvroFormatGlobalsTypeInternal IngestionDataSourceSettings_CloudStorage_AvroFormat_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* IngestionDataSourceSettings_CloudStorage_AvroFormat_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return IngestionDataSourceSettings_CloudStorage_AvroFormat_globals_.GetClassData();
#else
  return IngestionDataSourceSettings_CloudStorage_AvroFormat_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class IngestionDataSourceSettings_AwsKinesis::_Internal {
 public:
  using HasBits = decltype(::std::declval<IngestionDataSourceSettings_AwsKinesis>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_AwsKinesis, _impl_._has_bits_);
};

constexpr IngestionDataSourceSettings_AwsKinesis::ParseTableT_ IngestionDataSourceSettings_AwsKinesis::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_AwsKinesis, _impl_._has_bits_),
      0, // no _extensions_
      5, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967264,  // skipmap
      offsetof(ParseTableT_, field_entries),
      5,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::IngestionDataSourceSettings_AwsKinesis>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .google.pubsub.v1.IngestionDataSourceSettings.AwsKinesis.State state = 1 [(.google.api.field_behavior) = OUTPUT_ONLY];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(IngestionDataSourceSettings_AwsKinesis, _impl_.state_), 4>(),
       {8, 4, 0,
        PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_AwsKinesis, _impl_.state_)}},
      // string stream_arn = 2 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastUS1,
       {18, 0, 0,
        PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_AwsKinesis, _impl_.stream_arn_)}},
      // string consumer_arn = 3 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastUS1,
       {26, 1, 0,
        PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_AwsKinesis, _impl_.consumer_arn_)}},
      // string aws_role_arn = 4 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastUS1,
       {34, 2, 0,
        PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_AwsKinesis, _impl_.aws_role_arn_)}},
      // string gcp_service_account = 5 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastUS1,
       {42, 3, 0,
        PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_AwsKinesis, _impl_.gcp_service_account_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.pubsub.v1.IngestionDataSourceSettings.AwsKinesis.State state = 1 [(.google.api.field_behavior) = OUTPUT_ONLY];
      {PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_AwsKinesis, _impl_.state_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // string stream_arn = 2 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_AwsKinesis, _impl_.stream_arn_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string consumer_arn = 3 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_AwsKinesis, _impl_.consumer_arn_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string aws_role_arn = 4 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_AwsKinesis, _impl_.aws_role_arn_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string gcp_service_account = 5 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_AwsKinesis, _impl_.gcp_service_account_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\67\0\12\14\14\23\0\0"
      "google.pubsub.v1.IngestionDataSourceSettings.AwsKinesis"
      "stream_arn"
      "consumer_arn"
      "aws_role_arn"
      "gcp_service_account"
    }},
  };
}


inline constexpr IngestionDataSourceSettings_AwsKinesis::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        stream_arn_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        consumer_arn_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        aws_role_arn_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        gcp_service_account_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        state_{static_cast< ::google::pubsub::v1::IngestionDataSourceSettings_AwsKinesis_State >(0)} {}

template <typename>
constexpr IngestionDataSourceSettings_AwsKinesis::IngestionDataSourceSettings_AwsKinesis(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL IngestionDataSourceSettings_AwsKinesis::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) IngestionDataSourceSettings_AwsKinesis(arena);
}
constexpr auto IngestionDataSourceSettings_AwsKinesis::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(IngestionDataSourceSettings_AwsKinesis), alignof(IngestionDataSourceSettings_AwsKinesis));
}
constexpr auto IngestionDataSourceSettings_AwsKinesis::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &IngestionDataSourceSettings_AwsKinesis::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<IngestionDataSourceSettings_AwsKinesis>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &IngestionDataSourceSettings_AwsKinesis::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<IngestionDataSourceSettings_AwsKinesis>(), &IngestionDataSourceSettings_AwsKinesis::ByteSizeLong,
              &IngestionDataSourceSettings_AwsKinesis::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_AwsKinesis, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[2],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct IngestionDataSourceSettings_AwsKinesisGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr IngestionDataSourceSettings_AwsKinesisGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 IngestionDataSourceSettings_AwsKinesis_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(IngestionDataSourceSettings_AwsKinesis::InternalGenerateClassData_(
            _default, &IngestionDataSourceSettings_AwsKinesis_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<IngestionDataSourceSettings_AwsKinesis>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~IngestionDataSourceSettings_AwsKinesisGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) IngestionDataSourceSettings_AwsKinesis _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<IngestionDataSourceSettings_AwsKinesis>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_AwsKinesisGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST IngestionDataSourceSettings_AwsKinesisGlobalsTypeInternal IngestionDataSourceSettings_AwsKinesis_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* IngestionDataSourceSettings_AwsKinesis_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return IngestionDataSourceSettings_AwsKinesis_globals_.GetClassData();
#else
  return IngestionDataSourceSettings_AwsKinesis_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class GetTopicRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<GetTopicRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(GetTopicRequest, _impl_._has_bits_);
};

constexpr GetTopicRequest::ParseTableT_ GetTopicRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(GetTopicRequest, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::GetTopicRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string topic = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(GetTopicRequest, _impl_.topic_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string topic = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(GetTopicRequest, _impl_.topic_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\40\5\0\0\0\0\0\0"
      "google.pubsub.v1.GetTopicRequest"
      "topic"
    }},
  };
}


inline constexpr GetTopicRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        topic_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr GetTopicRequest::GetTopicRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL GetTopicRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) GetTopicRequest(arena);
}
constexpr auto GetTopicRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(GetTopicRequest), alignof(GetTopicRequest));
}
constexpr auto GetTopicRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &GetTopicRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<GetTopicRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &GetTopicRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<GetTopicRequest>(), &GetTopicRequest::ByteSizeLong,
              &GetTopicRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(GetTopicRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[17],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct GetTopicRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr GetTopicRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 GetTopicRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(GetTopicRequest::InternalGenerateClassData_(
            _default, &GetTopicRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<GetTopicRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~GetTopicRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) GetTopicRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<GetTopicRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(GetTopicRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST GetTopicRequestGlobalsTypeInternal GetTopicRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* GetTopicRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return GetTopicRequest_globals_.GetClassData();
#else
  return GetTopicRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class GetSubscriptionRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<GetSubscriptionRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(GetSubscriptionRequest, _impl_._has_bits_);
};

constexpr GetSubscriptionRequest::ParseTableT_ GetSubscriptionRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(GetSubscriptionRequest, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::GetSubscriptionRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(GetSubscriptionRequest, _impl_.subscription_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(GetSubscriptionRequest, _impl_.subscription_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\47\14\0\0\0\0\0\0"
      "google.pubsub.v1.GetSubscriptionRequest"
      "subscription"
    }},
  };
}


inline constexpr GetSubscriptionRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        subscription_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr GetSubscriptionRequest::GetSubscriptionRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL GetSubscriptionRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) GetSubscriptionRequest(arena);
}
constexpr auto GetSubscriptionRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(GetSubscriptionRequest), alignof(GetSubscriptionRequest));
}
constexpr auto GetSubscriptionRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &GetSubscriptionRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<GetSubscriptionRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &GetSubscriptionRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<GetSubscriptionRequest>(), &GetSubscriptionRequest::ByteSizeLong,
              &GetSubscriptionRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(GetSubscriptionRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[46],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct GetSubscriptionRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr GetSubscriptionRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 GetSubscriptionRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(GetSubscriptionRequest::InternalGenerateClassData_(
            _default, &GetSubscriptionRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<GetSubscriptionRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~GetSubscriptionRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) GetSubscriptionRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<GetSubscriptionRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(GetSubscriptionRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST GetSubscriptionRequestGlobalsTypeInternal GetSubscriptionRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* GetSubscriptionRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return GetSubscriptionRequest_globals_.GetClassData();
#else
  return GetSubscriptionRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class GetSnapshotRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<GetSnapshotRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(GetSnapshotRequest, _impl_._has_bits_);
};

constexpr GetSnapshotRequest::ParseTableT_ GetSnapshotRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(GetSnapshotRequest, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::GetSnapshotRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string snapshot = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(GetSnapshotRequest, _impl_.snapshot_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string snapshot = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(GetSnapshotRequest, _impl_.snapshot_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\43\10\0\0\0\0\0\0"
      "google.pubsub.v1.GetSnapshotRequest"
      "snapshot"
    }},
  };
}


inline constexpr GetSnapshotRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        snapshot_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr GetSnapshotRequest::GetSnapshotRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL GetSnapshotRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) GetSnapshotRequest(arena);
}
constexpr auto GetSnapshotRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(GetSnapshotRequest), alignof(GetSnapshotRequest));
}
constexpr auto GetSnapshotRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &GetSnapshotRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<GetSnapshotRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &GetSnapshotRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<GetSnapshotRequest>(), &GetSnapshotRequest::ByteSizeLong,
              &GetSnapshotRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(GetSnapshotRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[66],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct GetSnapshotRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr GetSnapshotRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 GetSnapshotRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(GetSnapshotRequest::InternalGenerateClassData_(
            _default, &GetSnapshotRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<GetSnapshotRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~GetSnapshotRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) GetSnapshotRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<GetSnapshotRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(GetSnapshotRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST GetSnapshotRequestGlobalsTypeInternal GetSnapshotRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* GetSnapshotRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return GetSnapshotRequest_globals_.GetClassData();
#else
  return GetSnapshotRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class DetachSubscriptionResponse::_Internal {
 public:
};

constexpr DetachSubscriptionResponse::ParseTableT_ DetachSubscriptionResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(DetachSubscriptionResponse,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::DetachSubscriptionResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr DetachSubscriptionResponse::DetachSubscriptionResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL DetachSubscriptionResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) DetachSubscriptionResponse(arena);
}
constexpr auto DetachSubscriptionResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(DetachSubscriptionResponse), alignof(DetachSubscriptionResponse));
}
constexpr auto DetachSubscriptionResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &DetachSubscriptionResponse::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<DetachSubscriptionResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &DetachSubscriptionResponse::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<DetachSubscriptionResponse>(), &DetachSubscriptionResponse::ByteSizeLong,
              &DetachSubscriptionResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(DetachSubscriptionResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[29],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct DetachSubscriptionResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr DetachSubscriptionResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 DetachSubscriptionResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(DetachSubscriptionResponse::InternalGenerateClassData_(
            _default, &DetachSubscriptionResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<DetachSubscriptionResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~DetachSubscriptionResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) DetachSubscriptionResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<DetachSubscriptionResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(DetachSubscriptionResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST DetachSubscriptionResponseGlobalsTypeInternal DetachSubscriptionResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* DetachSubscriptionResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return DetachSubscriptionResponse_globals_.GetClassData();
#else
  return DetachSubscriptionResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class DetachSubscriptionRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<DetachSubscriptionRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(DetachSubscriptionRequest, _impl_._has_bits_);
};

constexpr DetachSubscriptionRequest::ParseTableT_ DetachSubscriptionRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(DetachSubscriptionRequest, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::DetachSubscriptionRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(DetachSubscriptionRequest, _impl_.subscription_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(DetachSubscriptionRequest, _impl_.subscription_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\52\14\0\0\0\0\0\0"
      "google.pubsub.v1.DetachSubscriptionRequest"
      "subscription"
    }},
  };
}


inline constexpr DetachSubscriptionRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        subscription_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr DetachSubscriptionRequest::DetachSubscriptionRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL DetachSubscriptionRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) DetachSubscriptionRequest(arena);
}
constexpr auto DetachSubscriptionRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(DetachSubscriptionRequest), alignof(DetachSubscriptionRequest));
}
constexpr auto DetachSubscriptionRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &DetachSubscriptionRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<DetachSubscriptionRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &DetachSubscriptionRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<DetachSubscriptionRequest>(), &DetachSubscriptionRequest::ByteSizeLong,
              &DetachSubscriptionRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(DetachSubscriptionRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[28],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct DetachSubscriptionRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr DetachSubscriptionRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 DetachSubscriptionRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(DetachSubscriptionRequest::InternalGenerateClassData_(
            _default, &DetachSubscriptionRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<DetachSubscriptionRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~DetachSubscriptionRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) DetachSubscriptionRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<DetachSubscriptionRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(DetachSubscriptionRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST DetachSubscriptionRequestGlobalsTypeInternal DetachSubscriptionRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* DetachSubscriptionRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return DetachSubscriptionRequest_globals_.GetClassData();
#else
  return DetachSubscriptionRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class DeleteTopicRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<DeleteTopicRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(DeleteTopicRequest, _impl_._has_bits_);
};

constexpr DeleteTopicRequest::ParseTableT_ DeleteTopicRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(DeleteTopicRequest, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::DeleteTopicRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string topic = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(DeleteTopicRequest, _impl_.topic_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string topic = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(DeleteTopicRequest, _impl_.topic_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\43\5\0\0\0\0\0\0"
      "google.pubsub.v1.DeleteTopicRequest"
      "topic"
    }},
  };
}


inline constexpr DeleteTopicRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        topic_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr DeleteTopicRequest::DeleteTopicRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL DeleteTopicRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) DeleteTopicRequest(arena);
}
constexpr auto DeleteTopicRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(DeleteTopicRequest), alignof(DeleteTopicRequest));
}
constexpr auto DeleteTopicRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &DeleteTopicRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<DeleteTopicRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &DeleteTopicRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<DeleteTopicRequest>(), &DeleteTopicRequest::ByteSizeLong,
              &DeleteTopicRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(DeleteTopicRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[27],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct DeleteTopicRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr DeleteTopicRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 DeleteTopicRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(DeleteTopicRequest::InternalGenerateClassData_(
            _default, &DeleteTopicRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<DeleteTopicRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~DeleteTopicRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) DeleteTopicRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<DeleteTopicRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(DeleteTopicRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST DeleteTopicRequestGlobalsTypeInternal DeleteTopicRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* DeleteTopicRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return DeleteTopicRequest_globals_.GetClassData();
#else
  return DeleteTopicRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class DeleteSubscriptionRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<DeleteSubscriptionRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(DeleteSubscriptionRequest, _impl_._has_bits_);
};

constexpr DeleteSubscriptionRequest::ParseTableT_ DeleteSubscriptionRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(DeleteSubscriptionRequest, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::DeleteSubscriptionRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(DeleteSubscriptionRequest, _impl_.subscription_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(DeleteSubscriptionRequest, _impl_.subscription_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\52\14\0\0\0\0\0\0"
      "google.pubsub.v1.DeleteSubscriptionRequest"
      "subscription"
    }},
  };
}


inline constexpr DeleteSubscriptionRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        subscription_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr DeleteSubscriptionRequest::DeleteSubscriptionRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL DeleteSubscriptionRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) DeleteSubscriptionRequest(arena);
}
constexpr auto DeleteSubscriptionRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(DeleteSubscriptionRequest), alignof(DeleteSubscriptionRequest));
}
constexpr auto DeleteSubscriptionRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &DeleteSubscriptionRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<DeleteSubscriptionRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &DeleteSubscriptionRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<DeleteSubscriptionRequest>(), &DeleteSubscriptionRequest::ByteSizeLong,
              &DeleteSubscriptionRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(DeleteSubscriptionRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[50],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct DeleteSubscriptionRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr DeleteSubscriptionRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 DeleteSubscriptionRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(DeleteSubscriptionRequest::InternalGenerateClassData_(
            _default, &DeleteSubscriptionRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<DeleteSubscriptionRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~DeleteSubscriptionRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) DeleteSubscriptionRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<DeleteSubscriptionRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(DeleteSubscriptionRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST DeleteSubscriptionRequestGlobalsTypeInternal DeleteSubscriptionRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* DeleteSubscriptionRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return DeleteSubscriptionRequest_globals_.GetClassData();
#else
  return DeleteSubscriptionRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class DeleteSnapshotRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<DeleteSnapshotRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(DeleteSnapshotRequest, _impl_._has_bits_);
};

constexpr DeleteSnapshotRequest::ParseTableT_ DeleteSnapshotRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(DeleteSnapshotRequest, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::DeleteSnapshotRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string snapshot = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(DeleteSnapshotRequest, _impl_.snapshot_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string snapshot = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(DeleteSnapshotRequest, _impl_.snapshot_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\46\10\0\0\0\0\0\0"
      "google.pubsub.v1.DeleteSnapshotRequest"
      "snapshot"
    }},
  };
}


inline constexpr DeleteSnapshotRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        snapshot_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr DeleteSnapshotRequest::DeleteSnapshotRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL DeleteSnapshotRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) DeleteSnapshotRequest(arena);
}
constexpr auto DeleteSnapshotRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(DeleteSnapshotRequest), alignof(DeleteSnapshotRequest));
}
constexpr auto DeleteSnapshotRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &DeleteSnapshotRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<DeleteSnapshotRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &DeleteSnapshotRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<DeleteSnapshotRequest>(), &DeleteSnapshotRequest::ByteSizeLong,
              &DeleteSnapshotRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(DeleteSnapshotRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[69],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct DeleteSnapshotRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr DeleteSnapshotRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 DeleteSnapshotRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(DeleteSnapshotRequest::InternalGenerateClassData_(
            _default, &DeleteSnapshotRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<DeleteSnapshotRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~DeleteSnapshotRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) DeleteSnapshotRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<DeleteSnapshotRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(DeleteSnapshotRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST DeleteSnapshotRequestGlobalsTypeInternal DeleteSnapshotRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* DeleteSnapshotRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return DeleteSnapshotRequest_globals_.GetClassData();
#else
  return DeleteSnapshotRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class DeadLetterPolicy::_Internal {
 public:
  using HasBits = decltype(::std::declval<DeadLetterPolicy>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(DeadLetterPolicy, _impl_._has_bits_);
};

constexpr DeadLetterPolicy::ParseTableT_ DeadLetterPolicy::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(DeadLetterPolicy, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::DeadLetterPolicy>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // int32 max_delivery_attempts = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(DeadLetterPolicy, _impl_.max_delivery_attempts_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(DeadLetterPolicy, _impl_.max_delivery_attempts_)}},
      // string dead_letter_topic = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(DeadLetterPolicy, _impl_.dead_letter_topic_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string dead_letter_topic = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(DeadLetterPolicy, _impl_.dead_letter_topic_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // int32 max_delivery_attempts = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(DeadLetterPolicy, _impl_.max_delivery_attempts_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    }},
    // no aux_entries
    {{
      "\41\21\0\0\0\0\0\0"
      "google.pubsub.v1.DeadLetterPolicy"
      "dead_letter_topic"
    }},
  };
}


inline constexpr DeadLetterPolicy::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        dead_letter_topic_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        max_delivery_attempts_{0} {}

template <typename>
constexpr DeadLetterPolicy::DeadLetterPolicy(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL DeadLetterPolicy::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) DeadLetterPolicy(arena);
}
constexpr auto DeadLetterPolicy::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(DeadLetterPolicy), alignof(DeadLetterPolicy));
}
constexpr auto DeadLetterPolicy::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &DeadLetterPolicy::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<DeadLetterPolicy>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &DeadLetterPolicy::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<DeadLetterPolicy>(), &DeadLetterPolicy::ByteSizeLong,
              &DeadLetterPolicy::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(DeadLetterPolicy, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[34],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct DeadLetterPolicyGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr DeadLetterPolicyGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 DeadLetterPolicy_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(DeadLetterPolicy::InternalGenerateClassData_(
            _default, &DeadLetterPolicy_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<DeadLetterPolicy>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~DeadLetterPolicyGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) DeadLetterPolicy _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<DeadLetterPolicy>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(DeadLetterPolicyGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST DeadLetterPolicyGlobalsTypeInternal DeadLetterPolicy_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* DeadLetterPolicy_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return DeadLetterPolicy_globals_.GetClassData();
#else
  return DeadLetterPolicy_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
constexpr CreateSnapshotRequest_LabelsEntry_DoNotUse::ParseTableT_ CreateSnapshotRequest_LabelsEntry_DoNotUse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(CreateSnapshotRequest_LabelsEntry_DoNotUse, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::DiscardEverythingFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::CreateSnapshotRequest_LabelsEntry_DoNotUse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string value = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(CreateSnapshotRequest_LabelsEntry_DoNotUse, _impl_.value_)}},
      // string key = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(CreateSnapshotRequest_LabelsEntry_DoNotUse, _impl_.key_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string key = 1;
      {PROTOBUF_FIELD_OFFSET(CreateSnapshotRequest_LabelsEntry_DoNotUse, _impl_.key_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string value = 2;
      {PROTOBUF_FIELD_OFFSET(CreateSnapshotRequest_LabelsEntry_DoNotUse, _impl_.value_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\62\3\5\0\0\0\0\0"
      "google.pubsub.v1.CreateSnapshotRequest.LabelsEntry"
      "key"
      "value"
    }},
  };
}

template <typename>
constexpr CreateSnapshotRequest_LabelsEntry_DoNotUse::CreateSnapshotRequest_LabelsEntry_DoNotUse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : CreateSnapshotRequest_LabelsEntry_DoNotUse::MapEntry(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL CreateSnapshotRequest_LabelsEntry_DoNotUse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) CreateSnapshotRequest_LabelsEntry_DoNotUse(arena);
}
constexpr auto CreateSnapshotRequest_LabelsEntry_DoNotUse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(CreateSnapshotRequest_LabelsEntry_DoNotUse), alignof(CreateSnapshotRequest_LabelsEntry_DoNotUse));
}
constexpr auto CreateSnapshotRequest_LabelsEntry_DoNotUse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &CreateSnapshotRequest_LabelsEntry_DoNotUse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<CreateSnapshotRequest_LabelsEntry_DoNotUse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &CreateSnapshotRequest_LabelsEntry_DoNotUse::SharedDtor,
          static_cast<void (::google::protobuf::MessageLite::*)()>(&CreateSnapshotRequest_LabelsEntry_DoNotUse::ClearImpl),
              ::google::protobuf::Message::ByteSizeLongImpl, ::google::protobuf::Message::_InternalSerializeImpl
              ,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(CreateSnapshotRequest_LabelsEntry_DoNotUse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[61],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct CreateSnapshotRequest_LabelsEntry_DoNotUseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr CreateSnapshotRequest_LabelsEntry_DoNotUseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 CreateSnapshotRequest_LabelsEntry_DoNotUse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(CreateSnapshotRequest_LabelsEntry_DoNotUse::InternalGenerateClassData_(
            _default, &CreateSnapshotRequest_LabelsEntry_DoNotUse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<CreateSnapshotRequest_LabelsEntry_DoNotUse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~CreateSnapshotRequest_LabelsEntry_DoNotUseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) CreateSnapshotRequest_LabelsEntry_DoNotUse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<CreateSnapshotRequest_LabelsEntry_DoNotUse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(CreateSnapshotRequest_LabelsEntry_DoNotUseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST CreateSnapshotRequest_LabelsEntry_DoNotUseGlobalsTypeInternal CreateSnapshotRequest_LabelsEntry_DoNotUse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* CreateSnapshotRequest_LabelsEntry_DoNotUse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return CreateSnapshotRequest_LabelsEntry_DoNotUse_globals_.GetClassData();
#else
  return CreateSnapshotRequest_LabelsEntry_DoNotUse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class CloudStorageConfig_TextConfig::_Internal {
 public:
};

constexpr CloudStorageConfig_TextConfig::ParseTableT_ CloudStorageConfig_TextConfig::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(CloudStorageConfig_TextConfig,
                            _impl_._cached_size_),  // no hasbits
      0, // no _extensions_
      0, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967295,  // skipmap
      offsetof(ParseTableT_, field_names),  // no field_entries
      0,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::CloudStorageConfig_TextConfig>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, // no field_entries, or aux_entries
    {{
    }},
  };
}

template <typename>
constexpr CloudStorageConfig_TextConfig::CloudStorageConfig_TextConfig(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::internal::ZeroFieldsBase(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL CloudStorageConfig_TextConfig::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) CloudStorageConfig_TextConfig(arena);
}
constexpr auto CloudStorageConfig_TextConfig::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(CloudStorageConfig_TextConfig), alignof(CloudStorageConfig_TextConfig));
}
constexpr auto CloudStorageConfig_TextConfig::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &CloudStorageConfig_TextConfig::MergeImpl,
          ::google::protobuf::internal::ZeroFieldsBase::GetNewImpl<CloudStorageConfig_TextConfig>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &CloudStorageConfig_TextConfig::SharedDtor,
          ::google::protobuf::internal::ZeroFieldsBase::GetClearImpl<CloudStorageConfig_TextConfig>(), &CloudStorageConfig_TextConfig::ByteSizeLong,
              &CloudStorageConfig_TextConfig::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(CloudStorageConfig_TextConfig, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[42],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct CloudStorageConfig_TextConfigGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr CloudStorageConfig_TextConfigGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 CloudStorageConfig_TextConfig_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(CloudStorageConfig_TextConfig::InternalGenerateClassData_(
            _default, &CloudStorageConfig_TextConfig_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<CloudStorageConfig_TextConfig>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~CloudStorageConfig_TextConfigGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) CloudStorageConfig_TextConfig _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<CloudStorageConfig_TextConfig>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(CloudStorageConfig_TextConfigGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST CloudStorageConfig_TextConfigGlobalsTypeInternal CloudStorageConfig_TextConfig_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* CloudStorageConfig_TextConfig_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return CloudStorageConfig_TextConfig_globals_.GetClassData();
#else
  return CloudStorageConfig_TextConfig_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class CloudStorageConfig_AvroConfig::_Internal {
 public:
  using HasBits = decltype(::std::declval<CloudStorageConfig_AvroConfig>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(CloudStorageConfig_AvroConfig, _impl_._has_bits_);
};

constexpr CloudStorageConfig_AvroConfig::ParseTableT_ CloudStorageConfig_AvroConfig::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(CloudStorageConfig_AvroConfig, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::CloudStorageConfig_AvroConfig>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // bool use_topic_schema = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(CloudStorageConfig_AvroConfig, _impl_.use_topic_schema_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(CloudStorageConfig_AvroConfig, _impl_.use_topic_schema_)}},
      // bool write_metadata = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(CloudStorageConfig_AvroConfig, _impl_.write_metadata_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(CloudStorageConfig_AvroConfig, _impl_.write_metadata_)}},
    }}, {{
      65535, 65535
    }}, {{
      // bool write_metadata = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(CloudStorageConfig_AvroConfig, _impl_.write_metadata_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // bool use_topic_schema = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(CloudStorageConfig_AvroConfig, _impl_.use_topic_schema_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr CloudStorageConfig_AvroConfig::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        write_metadata_{false},
        use_topic_schema_{false} {}

template <typename>
constexpr CloudStorageConfig_AvroConfig::CloudStorageConfig_AvroConfig(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL CloudStorageConfig_AvroConfig::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) CloudStorageConfig_AvroConfig(arena);
}
constexpr auto CloudStorageConfig_AvroConfig::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(CloudStorageConfig_AvroConfig), alignof(CloudStorageConfig_AvroConfig));
}
constexpr auto CloudStorageConfig_AvroConfig::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &CloudStorageConfig_AvroConfig::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<CloudStorageConfig_AvroConfig>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &CloudStorageConfig_AvroConfig::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<CloudStorageConfig_AvroConfig>(), &CloudStorageConfig_AvroConfig::ByteSizeLong,
              &CloudStorageConfig_AvroConfig::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(CloudStorageConfig_AvroConfig, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[43],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct CloudStorageConfig_AvroConfigGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr CloudStorageConfig_AvroConfigGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 CloudStorageConfig_AvroConfig_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(CloudStorageConfig_AvroConfig::InternalGenerateClassData_(
            _default, &CloudStorageConfig_AvroConfig_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<CloudStorageConfig_AvroConfig>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~CloudStorageConfig_AvroConfigGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) CloudStorageConfig_AvroConfig _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<CloudStorageConfig_AvroConfig>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(CloudStorageConfig_AvroConfigGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST CloudStorageConfig_AvroConfigGlobalsTypeInternal CloudStorageConfig_AvroConfig_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* CloudStorageConfig_AvroConfig_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return CloudStorageConfig_AvroConfig_globals_.GetClassData();
#else
  return CloudStorageConfig_AvroConfig_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class BigQueryConfig::_Internal {
 public:
  using HasBits = decltype(::std::declval<BigQueryConfig>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(BigQueryConfig, _impl_._has_bits_);
};

constexpr BigQueryConfig::ParseTableT_ BigQueryConfig::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(BigQueryConfig, _impl_._has_bits_),
      0, // no _extensions_
      7, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967168,  // skipmap
      offsetof(ParseTableT_, field_entries),
      7,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::BigQueryConfig>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string table = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(BigQueryConfig, _impl_.table_)}},
      // bool use_topic_schema = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(BigQueryConfig, _impl_.use_topic_schema_), 2>(),
       {16, 2, 0,
        PROTOBUF_FIELD_OFFSET(BigQueryConfig, _impl_.use_topic_schema_)}},
      // bool write_metadata = 3 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(BigQueryConfig, _impl_.write_metadata_), 3>(),
       {24, 3, 0,
        PROTOBUF_FIELD_OFFSET(BigQueryConfig, _impl_.write_metadata_)}},
      // bool drop_unknown_fields = 4 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(BigQueryConfig, _impl_.drop_unknown_fields_), 4>(),
       {32, 4, 0,
        PROTOBUF_FIELD_OFFSET(BigQueryConfig, _impl_.drop_unknown_fields_)}},
      // .google.pubsub.v1.BigQueryConfig.State state = 5 [(.google.api.field_behavior) = OUTPUT_ONLY];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(BigQueryConfig, _impl_.state_), 6>(),
       {40, 6, 0,
        PROTOBUF_FIELD_OFFSET(BigQueryConfig, _impl_.state_)}},
      // bool use_table_schema = 6 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(BigQueryConfig, _impl_.use_table_schema_), 5>(),
       {48, 5, 0,
        PROTOBUF_FIELD_OFFSET(BigQueryConfig, _impl_.use_table_schema_)}},
      // string service_account_email = 7 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {58, 1, 0,
        PROTOBUF_FIELD_OFFSET(BigQueryConfig, _impl_.service_account_email_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string table = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(BigQueryConfig, _impl_.table_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // bool use_topic_schema = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(BigQueryConfig, _impl_.use_topic_schema_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // bool write_metadata = 3 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(BigQueryConfig, _impl_.write_metadata_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // bool drop_unknown_fields = 4 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(BigQueryConfig, _impl_.drop_unknown_fields_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // .google.pubsub.v1.BigQueryConfig.State state = 5 [(.google.api.field_behavior) = OUTPUT_ONLY];
      {PROTOBUF_FIELD_OFFSET(BigQueryConfig, _impl_.state_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // bool use_table_schema = 6 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(BigQueryConfig, _impl_.use_table_schema_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // string service_account_email = 7 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(BigQueryConfig, _impl_.service_account_email_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\37\5\0\0\0\0\0\25"
      "google.pubsub.v1.BigQueryConfig"
      "table"
      "service_account_email"
    }},
  };
}


inline constexpr BigQueryConfig::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        table_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        service_account_email_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        use_topic_schema_{false},
        write_metadata_{false},
        drop_unknown_fields_{false},
        use_table_schema_{false},
        state_{static_cast< ::google::pubsub::v1::BigQueryConfig_State >(0)} {}

template <typename>
constexpr BigQueryConfig::BigQueryConfig(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL BigQueryConfig::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) BigQueryConfig(arena);
}
constexpr auto BigQueryConfig::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(BigQueryConfig), alignof(BigQueryConfig));
}
constexpr auto BigQueryConfig::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &BigQueryConfig::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<BigQueryConfig>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &BigQueryConfig::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<BigQueryConfig>(), &BigQueryConfig::ByteSizeLong,
              &BigQueryConfig::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(BigQueryConfig, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[41],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct BigQueryConfigGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr BigQueryConfigGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 BigQueryConfig_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(BigQueryConfig::InternalGenerateClassData_(
            _default, &BigQueryConfig_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<BigQueryConfig>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~BigQueryConfigGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) BigQueryConfig _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<BigQueryConfig>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(BigQueryConfigGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST BigQueryConfigGlobalsTypeInternal BigQueryConfig_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* BigQueryConfig_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return BigQueryConfig_globals_.GetClassData();
#else
  return BigQueryConfig_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class AcknowledgeRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<AcknowledgeRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(AcknowledgeRequest, _impl_._has_bits_);
};

constexpr AcknowledgeRequest::ParseTableT_ AcknowledgeRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(AcknowledgeRequest, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::AcknowledgeRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated string ack_ids = 2 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastUR1,
       {18, 0, 0,
        PROTOBUF_FIELD_OFFSET(AcknowledgeRequest, _impl_.ack_ids_)}},
      // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 1, 0,
        PROTOBUF_FIELD_OFFSET(AcknowledgeRequest, _impl_.subscription_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(AcknowledgeRequest, _impl_.subscription_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // repeated string ack_ids = 2 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(AcknowledgeRequest, _impl_.ack_ids_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
    }},
    // no aux_entries
    {{
      "\43\14\7\0\0\0\0\0"
      "google.pubsub.v1.AcknowledgeRequest"
      "subscription"
      "ack_ids"
    }},
  };
}


inline constexpr AcknowledgeRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::AcknowledgeRequest,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::AcknowledgeRequest, _impl_.ack_ids_)>()
         }
        ,
        subscription_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr AcknowledgeRequest::AcknowledgeRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL AcknowledgeRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) AcknowledgeRequest(arena);
}
constexpr auto AcknowledgeRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(AcknowledgeRequest), alignof(AcknowledgeRequest));
}
constexpr auto AcknowledgeRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &AcknowledgeRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<AcknowledgeRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &AcknowledgeRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<AcknowledgeRequest>(), &AcknowledgeRequest::ByteSizeLong,
              &AcknowledgeRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(AcknowledgeRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[55],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct AcknowledgeRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr AcknowledgeRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 AcknowledgeRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(AcknowledgeRequest::InternalGenerateClassData_(
            _default, &AcknowledgeRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<AcknowledgeRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~AcknowledgeRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) AcknowledgeRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<AcknowledgeRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(AcknowledgeRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST AcknowledgeRequestGlobalsTypeInternal AcknowledgeRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* AcknowledgeRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return AcknowledgeRequest_globals_.GetClassData();
#else
  return AcknowledgeRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Snapshot::_Internal {
 public:
  using HasBits = decltype(::std::declval<Snapshot>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Snapshot, _impl_._has_bits_);
};

constexpr Snapshot::ParseTableT_ Snapshot::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Snapshot, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::Snapshot>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string name = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(Snapshot, _impl_.name_)}},
      // string topic = 2 [(.google.api.field_behavior) = OPTIONAL, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(Snapshot, _impl_.topic_)}},
      // .google.protobuf.Timestamp expire_time = 3 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS1,
       {26, 2, 0,
        PROTOBUF_FIELD_OFFSET(Snapshot, _impl_.expire_time_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string name = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Snapshot, _impl_.name_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string topic = 2 [(.google.api.field_behavior) = OPTIONAL, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(Snapshot, _impl_.topic_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.protobuf.Timestamp expire_time = 3 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Snapshot, _impl_.expire_time_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // map<string, string> labels = 4 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Snapshot, _impl_.labels_), _Internal::kHasBitsOffset + 3, 1, (0 | ::_fl::kFcRepeated | ::_fl::kMap)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::Timestamp>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::Timestamp_globals_},
        #endif
        {::_pbi::TcParser::GetMapAuxInfo(
            1, 0, 9, 9, 0)},
    }},
    {{
      "\31\4\5\0\6\0\0\0"
      "google.pubsub.v1.Snapshot"
      "name"
      "topic"
      "labels"
    }},
  };
}


inline constexpr Snapshot::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        topic_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        expire_time_{nullptr},
        labels_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::Snapshot,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Snapshot, _impl_.labels_)>()
         }
     {}

template <typename>
constexpr Snapshot::Snapshot(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Snapshot::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Snapshot(arena);
}
constexpr auto Snapshot::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Snapshot), alignof(Snapshot));
}
constexpr auto Snapshot::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Snapshot::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<Snapshot>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Snapshot::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<Snapshot>(), &Snapshot::ByteSizeLong,
              &Snapshot::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Snapshot, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[65],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct SnapshotGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr SnapshotGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Snapshot_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Snapshot::InternalGenerateClassData_(
            _default, &Snapshot_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Snapshot>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~SnapshotGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Snapshot _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Snapshot>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(SnapshotGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST SnapshotGlobalsTypeInternal Snapshot_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Snapshot_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Snapshot_globals_.GetClassData();
#else
  return Snapshot_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class SeekRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<SeekRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(SeekRequest, _impl_._has_bits_);
  static constexpr ::int32_t kOneofCaseOffset =
      PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::SeekRequest, _impl_._oneof_case_);
};

constexpr SeekRequest::ParseTableT_ SeekRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(SeekRequest, _impl_._has_bits_),
      0, // no _extensions_
      3, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::SeekRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(SeekRequest, _impl_.subscription_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(SeekRequest, _impl_.subscription_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.protobuf.Timestamp time = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(SeekRequest, _impl_.target_.time_), _Internal::kOneofCaseOffset + 0, 0, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // string snapshot = 3 [(.google.api.field_behavior) = OPTIONAL, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(SeekRequest, _impl_.target_.snapshot_), _Internal::kOneofCaseOffset + 0, 0, (0 | ::_fl::kFcOneof | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::Timestamp>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::Timestamp_globals_},
        #endif
    }},
    {{
      "\34\14\0\10\0\0\0\0"
      "google.pubsub.v1.SeekRequest"
      "subscription"
      "snapshot"
    }},
  };
}


inline constexpr SeekRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        subscription_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        target_{},
        _oneof_case_{} {}

template <typename>
constexpr SeekRequest::SeekRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL SeekRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) SeekRequest(arena);
}
constexpr auto SeekRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(SeekRequest), alignof(SeekRequest));
}
constexpr auto SeekRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &SeekRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<SeekRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &SeekRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<SeekRequest>(), &SeekRequest::ByteSizeLong,
              &SeekRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(SeekRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[70],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct SeekRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr SeekRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 SeekRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(SeekRequest::InternalGenerateClassData_(
            _default, &SeekRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<SeekRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~SeekRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) SeekRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<SeekRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(SeekRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST SeekRequestGlobalsTypeInternal SeekRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* SeekRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return SeekRequest_globals_.GetClassData();
#else
  return SeekRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class RetryPolicy::_Internal {
 public:
  using HasBits = decltype(::std::declval<RetryPolicy>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(RetryPolicy, _impl_._has_bits_);
};

constexpr RetryPolicy::ParseTableT_ RetryPolicy::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(RetryPolicy, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::RetryPolicy>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.protobuf.Duration maximum_backoff = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(RetryPolicy, _impl_.maximum_backoff_)}},
      // .google.protobuf.Duration minimum_backoff = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(RetryPolicy, _impl_.minimum_backoff_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.protobuf.Duration minimum_backoff = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(RetryPolicy, _impl_.minimum_backoff_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.protobuf.Duration maximum_backoff = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(RetryPolicy, _impl_.maximum_backoff_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::Duration>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::Duration_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::Duration>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::Duration_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr RetryPolicy::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        minimum_backoff_{nullptr},
        maximum_backoff_{nullptr} {}

template <typename>
constexpr RetryPolicy::RetryPolicy(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL RetryPolicy::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) RetryPolicy(arena);
}
constexpr auto RetryPolicy::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(RetryPolicy), alignof(RetryPolicy));
}
constexpr auto RetryPolicy::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &RetryPolicy::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<RetryPolicy>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &RetryPolicy::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<RetryPolicy>(), &RetryPolicy::ByteSizeLong,
              &RetryPolicy::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(RetryPolicy, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[33],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct RetryPolicyGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr RetryPolicyGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 RetryPolicy_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(RetryPolicy::InternalGenerateClassData_(
            _default, &RetryPolicy_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<RetryPolicy>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~RetryPolicyGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) RetryPolicy _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<RetryPolicy>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(RetryPolicyGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST RetryPolicyGlobalsTypeInternal RetryPolicy_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* RetryPolicy_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return RetryPolicy_globals_.GetClassData();
#else
  return RetryPolicy_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class PushConfig::_Internal {
 public:
  using HasBits = decltype(::std::declval<PushConfig>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(PushConfig, _impl_._has_bits_);
  static constexpr ::int32_t kOneofCaseOffset =
      PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PushConfig, _impl_._oneof_case_);
};

constexpr PushConfig::ParseTableT_ PushConfig::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(PushConfig, _impl_._has_bits_),
      0, // no _extensions_
      5, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967264,  // skipmap
      offsetof(ParseTableT_, field_entries),
      5,  // num_field_entries
      4,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::PushConfig>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string push_endpoint = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(PushConfig, _impl_.push_endpoint_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string push_endpoint = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(PushConfig, _impl_.push_endpoint_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // map<string, string> attributes = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(PushConfig, _impl_.attributes_), _Internal::kHasBitsOffset + 1, 3, (0 | ::_fl::kFcRepeated | ::_fl::kMap)},
      // .google.pubsub.v1.PushConfig.OidcToken oidc_token = 3 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(PushConfig, _impl_.authentication_method_.oidc_token_), _Internal::kOneofCaseOffset + 0, 0, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.pubsub.v1.PushConfig.PubsubWrapper pubsub_wrapper = 4 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(PushConfig, _impl_.wrapper_.pubsub_wrapper_), _Internal::kOneofCaseOffset + 4, 1, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.pubsub.v1.PushConfig.NoWrapper no_wrapper = 5 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(PushConfig, _impl_.wrapper_.no_wrapper_), _Internal::kOneofCaseOffset + 4, 2, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::PushConfig_OidcToken>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::PushConfig_OidcToken_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::PushConfig_PubsubWrapper>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::PushConfig_PubsubWrapper_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::PushConfig_NoWrapper>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::PushConfig_NoWrapper_globals_},
        #endif
        {::_pbi::TcParser::GetMapAuxInfo(
            1, 0, 9, 9, 0)},
    }},
    {{
      "\33\15\12\0\0\0\0\0"
      "google.pubsub.v1.PushConfig"
      "push_endpoint"
      "attributes"
    }},
  };
}


inline constexpr PushConfig::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        push_endpoint_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        attributes_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::PushConfig,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PushConfig, _impl_.attributes_)>()
         }
        ,
        authentication_method_{},
        wrapper_{},
        _oneof_case_{} {}

template <typename>
constexpr PushConfig::PushConfig(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL PushConfig::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) PushConfig(arena);
}
constexpr auto PushConfig::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(PushConfig), alignof(PushConfig));
}
constexpr auto PushConfig::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &PushConfig::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<PushConfig>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &PushConfig::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<PushConfig>(), &PushConfig::ByteSizeLong,
              &PushConfig::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(PushConfig, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[40],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct PushConfigGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr PushConfigGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 PushConfig_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(PushConfig::InternalGenerateClassData_(
            _default, &PushConfig_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<PushConfig>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~PushConfigGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) PushConfig _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<PushConfig>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(PushConfigGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST PushConfigGlobalsTypeInternal PushConfig_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* PushConfig_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return PushConfig_globals_.GetClassData();
#else
  return PushConfig_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class PubsubMessage::_Internal {
 public:
  using HasBits = decltype(::std::declval<PubsubMessage>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(PubsubMessage, _impl_._has_bits_);
};

constexpr PubsubMessage::ParseTableT_ PubsubMessage::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(PubsubMessage, _impl_._has_bits_),
      0, // no _extensions_
      5, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967264,  // skipmap
      offsetof(ParseTableT_, field_entries),
      5,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::PubsubMessage>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // bytes data = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastBS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(PubsubMessage, _impl_.data_)}},
      {::_pbi::TcParser::MiniParse, {}},
      // string message_id = 3;
      {::_pbi::TcParser::FastUS1,
       {26, 1, 0,
        PROTOBUF_FIELD_OFFSET(PubsubMessage, _impl_.message_id_)}},
      // .google.protobuf.Timestamp publish_time = 4;
      {::_pbi::TcParser::FastMtS1,
       {34, 3, 0,
        PROTOBUF_FIELD_OFFSET(PubsubMessage, _impl_.publish_time_)}},
      // string ordering_key = 5 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {42, 2, 0,
        PROTOBUF_FIELD_OFFSET(PubsubMessage, _impl_.ordering_key_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // bytes data = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(PubsubMessage, _impl_.data_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
      // map<string, string> attributes = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(PubsubMessage, _impl_.attributes_), _Internal::kHasBitsOffset + 4, 1, (0 | ::_fl::kFcRepeated | ::_fl::kMap)},
      // string message_id = 3;
      {PROTOBUF_FIELD_OFFSET(PubsubMessage, _impl_.message_id_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.protobuf.Timestamp publish_time = 4;
      {PROTOBUF_FIELD_OFFSET(PubsubMessage, _impl_.publish_time_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // string ordering_key = 5 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(PubsubMessage, _impl_.ordering_key_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::Timestamp>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::Timestamp_globals_},
        #endif
        {::_pbi::TcParser::GetMapAuxInfo(
            1, 0, 9, 9, 0)},
    }},
    {{
      "\36\0\12\12\0\14\0\0"
      "google.pubsub.v1.PubsubMessage"
      "attributes"
      "message_id"
      "ordering_key"
    }},
  };
}


inline constexpr PubsubMessage::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        data_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        message_id_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        ordering_key_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        publish_time_{nullptr},
        attributes_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::PubsubMessage,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PubsubMessage, _impl_.attributes_)>()
         }
     {}

template <typename>
constexpr PubsubMessage::PubsubMessage(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL PubsubMessage::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) PubsubMessage(arena);
}
constexpr auto PubsubMessage::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(PubsubMessage), alignof(PubsubMessage));
}
constexpr auto PubsubMessage::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &PubsubMessage::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<PubsubMessage>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &PubsubMessage::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<PubsubMessage>(), &PubsubMessage::ByteSizeLong,
              &PubsubMessage::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(PubsubMessage, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[16],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct PubsubMessageGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr PubsubMessageGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 PubsubMessage_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(PubsubMessage::InternalGenerateClassData_(
            _default, &PubsubMessage_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<PubsubMessage>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~PubsubMessageGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) PubsubMessage _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<PubsubMessage>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(PubsubMessageGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST PubsubMessageGlobalsTypeInternal PubsubMessage_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* PubsubMessage_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return PubsubMessage_globals_.GetClassData();
#else
  return PubsubMessage_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class IngestionFailureEvent_CloudStorageFailure::_Internal {
 public:
  using HasBits = decltype(::std::declval<IngestionFailureEvent_CloudStorageFailure>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(IngestionFailureEvent_CloudStorageFailure, _impl_._has_bits_);
  static constexpr ::int32_t kOneofCaseOffset =
      PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionFailureEvent_CloudStorageFailure, _impl_._oneof_case_);
};

constexpr IngestionFailureEvent_CloudStorageFailure::ParseTableT_ IngestionFailureEvent_CloudStorageFailure::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(IngestionFailureEvent_CloudStorageFailure, _impl_._has_bits_),
      0, // no _extensions_
      6, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967240,  // skipmap
      offsetof(ParseTableT_, field_entries),
      5,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::IngestionFailureEvent_CloudStorageFailure>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string bucket = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(IngestionFailureEvent_CloudStorageFailure, _impl_.bucket_)}},
      // string object_name = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(IngestionFailureEvent_CloudStorageFailure, _impl_.object_name_)}},
      // int64 object_generation = 3 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(IngestionFailureEvent_CloudStorageFailure, _impl_.object_generation_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(IngestionFailureEvent_CloudStorageFailure, _impl_.object_generation_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string bucket = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(IngestionFailureEvent_CloudStorageFailure, _impl_.bucket_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string object_name = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(IngestionFailureEvent_CloudStorageFailure, _impl_.object_name_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // int64 object_generation = 3 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(IngestionFailureEvent_CloudStorageFailure, _impl_.object_generation_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // .google.pubsub.v1.IngestionFailureEvent.AvroFailureReason avro_failure_reason = 5 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(IngestionFailureEvent_CloudStorageFailure, _impl_.reason_.avro_failure_reason_), _Internal::kOneofCaseOffset + 0, 0, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.pubsub.v1.IngestionFailureEvent.ApiViolationReason api_violation_reason = 6 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(IngestionFailureEvent_CloudStorageFailure, _impl_.reason_.api_violation_reason_), _Internal::kOneofCaseOffset + 0, 1, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::IngestionFailureEvent_AvroFailureReason>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::IngestionFailureEvent_AvroFailureReason_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::IngestionFailureEvent_ApiViolationReason>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::IngestionFailureEvent_ApiViolationReason_globals_},
        #endif
    }},
    {{
      "\72\6\13\0\0\0\0\0"
      "google.pubsub.v1.IngestionFailureEvent.CloudStorageFailure"
      "bucket"
      "object_name"
    }},
  };
}


inline constexpr IngestionFailureEvent_CloudStorageFailure::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        bucket_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        object_name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        object_generation_{::int64_t{0}},
        reason_{},
        _oneof_case_{} {}

template <typename>
constexpr IngestionFailureEvent_CloudStorageFailure::IngestionFailureEvent_CloudStorageFailure(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL IngestionFailureEvent_CloudStorageFailure::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) IngestionFailureEvent_CloudStorageFailure(arena);
}
constexpr auto IngestionFailureEvent_CloudStorageFailure::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(IngestionFailureEvent_CloudStorageFailure), alignof(IngestionFailureEvent_CloudStorageFailure));
}
constexpr auto IngestionFailureEvent_CloudStorageFailure::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &IngestionFailureEvent_CloudStorageFailure::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<IngestionFailureEvent_CloudStorageFailure>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &IngestionFailureEvent_CloudStorageFailure::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<IngestionFailureEvent_CloudStorageFailure>(), &IngestionFailureEvent_CloudStorageFailure::ByteSizeLong,
              &IngestionFailureEvent_CloudStorageFailure::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(IngestionFailureEvent_CloudStorageFailure, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[11],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct IngestionFailureEvent_CloudStorageFailureGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr IngestionFailureEvent_CloudStorageFailureGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 IngestionFailureEvent_CloudStorageFailure_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(IngestionFailureEvent_CloudStorageFailure::InternalGenerateClassData_(
            _default, &IngestionFailureEvent_CloudStorageFailure_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<IngestionFailureEvent_CloudStorageFailure>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~IngestionFailureEvent_CloudStorageFailureGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) IngestionFailureEvent_CloudStorageFailure _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<IngestionFailureEvent_CloudStorageFailure>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(IngestionFailureEvent_CloudStorageFailureGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST IngestionFailureEvent_CloudStorageFailureGlobalsTypeInternal IngestionFailureEvent_CloudStorageFailure_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* IngestionFailureEvent_CloudStorageFailure_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return IngestionFailureEvent_CloudStorageFailure_globals_.GetClassData();
#else
  return IngestionFailureEvent_CloudStorageFailure_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class IngestionDataSourceSettings_CloudStorage::_Internal {
 public:
  using HasBits = decltype(::std::declval<IngestionDataSourceSettings_CloudStorage>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage, _impl_._has_bits_);
  static constexpr ::int32_t kOneofCaseOffset =
      PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage, _impl_._oneof_case_);
};

constexpr IngestionDataSourceSettings_CloudStorage::ParseTableT_ IngestionDataSourceSettings_CloudStorage::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage, _impl_._has_bits_),
      0, // no _extensions_
      9, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294966976,  // skipmap
      offsetof(ParseTableT_, field_entries),
      7,  // num_field_entries
      4,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.State state = 1 [(.google.api.field_behavior) = OUTPUT_ONLY];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(IngestionDataSourceSettings_CloudStorage, _impl_.state_), 3>(),
       {8, 3, 0,
        PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage, _impl_.state_)}},
      // string bucket = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {18, 0, 0,
        PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage, _impl_.bucket_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      // .google.protobuf.Timestamp minimum_object_create_time = 6 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS1,
       {50, 2, 3,
        PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage, _impl_.minimum_object_create_time_)}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.State state = 1 [(.google.api.field_behavior) = OUTPUT_ONLY];
      {PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage, _impl_.state_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // string bucket = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage, _impl_.bucket_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.TextFormat text_format = 3 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage, _impl_.input_format_.text_format_), _Internal::kOneofCaseOffset + 0, 0, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.AvroFormat avro_format = 4 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage, _impl_.input_format_.avro_format_), _Internal::kOneofCaseOffset + 0, 1, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.PubSubAvroFormat pubsub_avro_format = 5 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage, _impl_.input_format_.pubsub_avro_format_), _Internal::kOneofCaseOffset + 0, 2, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.protobuf.Timestamp minimum_object_create_time = 6 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage, _impl_.minimum_object_create_time_), _Internal::kHasBitsOffset + 2, 3, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // string match_glob = 9 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage, _impl_.match_glob_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_TextFormat>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_TextFormat_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_AvroFormat>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_AvroFormat_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::Timestamp>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::Timestamp_globals_},
        #endif
    }},
    {{
      "\71\0\6\0\0\0\0\12"
      "google.pubsub.v1.IngestionDataSourceSettings.CloudStorage"
      "bucket"
      "match_glob"
    }},
  };
}


inline constexpr IngestionDataSourceSettings_CloudStorage::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        bucket_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        match_glob_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        minimum_object_create_time_{nullptr},
        state_{static_cast< ::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_State >(0)},
        input_format_{},
        _oneof_case_{} {}

template <typename>
constexpr IngestionDataSourceSettings_CloudStorage::IngestionDataSourceSettings_CloudStorage(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL IngestionDataSourceSettings_CloudStorage::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) IngestionDataSourceSettings_CloudStorage(arena);
}
constexpr auto IngestionDataSourceSettings_CloudStorage::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(IngestionDataSourceSettings_CloudStorage), alignof(IngestionDataSourceSettings_CloudStorage));
}
constexpr auto IngestionDataSourceSettings_CloudStorage::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &IngestionDataSourceSettings_CloudStorage::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<IngestionDataSourceSettings_CloudStorage>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &IngestionDataSourceSettings_CloudStorage::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<IngestionDataSourceSettings_CloudStorage>(), &IngestionDataSourceSettings_CloudStorage::ByteSizeLong,
              &IngestionDataSourceSettings_CloudStorage::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[6],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct IngestionDataSourceSettings_CloudStorageGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr IngestionDataSourceSettings_CloudStorageGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 IngestionDataSourceSettings_CloudStorage_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(IngestionDataSourceSettings_CloudStorage::InternalGenerateClassData_(
            _default, &IngestionDataSourceSettings_CloudStorage_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<IngestionDataSourceSettings_CloudStorage>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~IngestionDataSourceSettings_CloudStorageGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) IngestionDataSourceSettings_CloudStorage _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<IngestionDataSourceSettings_CloudStorage>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorageGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST IngestionDataSourceSettings_CloudStorageGlobalsTypeInternal IngestionDataSourceSettings_CloudStorage_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* IngestionDataSourceSettings_CloudStorage_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return IngestionDataSourceSettings_CloudStorage_globals_.GetClassData();
#else
  return IngestionDataSourceSettings_CloudStorage_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ExpirationPolicy::_Internal {
 public:
  using HasBits = decltype(::std::declval<ExpirationPolicy>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ExpirationPolicy, _impl_._has_bits_);
};

constexpr ExpirationPolicy::ParseTableT_ ExpirationPolicy::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ExpirationPolicy, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ExpirationPolicy>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.protobuf.Duration ttl = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ExpirationPolicy, _impl_.ttl_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.protobuf.Duration ttl = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ExpirationPolicy, _impl_.ttl_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::Duration>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::Duration_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr ExpirationPolicy::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        ttl_{nullptr} {}

template <typename>
constexpr ExpirationPolicy::ExpirationPolicy(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ExpirationPolicy::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ExpirationPolicy(arena);
}
constexpr auto ExpirationPolicy::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ExpirationPolicy), alignof(ExpirationPolicy));
}
constexpr auto ExpirationPolicy::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ExpirationPolicy::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ExpirationPolicy>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ExpirationPolicy::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ExpirationPolicy>(), &ExpirationPolicy::ByteSizeLong,
              &ExpirationPolicy::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ExpirationPolicy, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[35],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ExpirationPolicyGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ExpirationPolicyGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ExpirationPolicy_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ExpirationPolicy::InternalGenerateClassData_(
            _default, &ExpirationPolicy_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ExpirationPolicy>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ExpirationPolicyGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ExpirationPolicy _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ExpirationPolicy>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ExpirationPolicyGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ExpirationPolicyGlobalsTypeInternal ExpirationPolicy_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ExpirationPolicy_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ExpirationPolicy_globals_.GetClassData();
#else
  return ExpirationPolicy_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class CreateSnapshotRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<CreateSnapshotRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(CreateSnapshotRequest, _impl_._has_bits_);
};

constexpr CreateSnapshotRequest::ParseTableT_ CreateSnapshotRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(CreateSnapshotRequest, _impl_._has_bits_),
      0, // no _extensions_
      3, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::CreateSnapshotRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string subscription = 2 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(CreateSnapshotRequest, _impl_.subscription_)}},
      // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(CreateSnapshotRequest, _impl_.name_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(CreateSnapshotRequest, _impl_.name_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string subscription = 2 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(CreateSnapshotRequest, _impl_.subscription_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // map<string, string> labels = 3 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(CreateSnapshotRequest, _impl_.labels_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMap)},
    }},
    {{
        {::_pbi::TcParser::GetMapAuxInfo(
            1, 0, 9, 9, 0)},
    }},
    {{
      "\46\4\14\6\0\0\0\0"
      "google.pubsub.v1.CreateSnapshotRequest"
      "name"
      "subscription"
      "labels"
    }},
  };
}


inline constexpr CreateSnapshotRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        subscription_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        labels_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::CreateSnapshotRequest,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CreateSnapshotRequest, _impl_.labels_)>()
         }
     {}

template <typename>
constexpr CreateSnapshotRequest::CreateSnapshotRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL CreateSnapshotRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) CreateSnapshotRequest(arena);
}
constexpr auto CreateSnapshotRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(CreateSnapshotRequest), alignof(CreateSnapshotRequest));
}
constexpr auto CreateSnapshotRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &CreateSnapshotRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<CreateSnapshotRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &CreateSnapshotRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<CreateSnapshotRequest>(), &CreateSnapshotRequest::ByteSizeLong,
              &CreateSnapshotRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(CreateSnapshotRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[62],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct CreateSnapshotRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr CreateSnapshotRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 CreateSnapshotRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(CreateSnapshotRequest::InternalGenerateClassData_(
            _default, &CreateSnapshotRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<CreateSnapshotRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~CreateSnapshotRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) CreateSnapshotRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<CreateSnapshotRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(CreateSnapshotRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST CreateSnapshotRequestGlobalsTypeInternal CreateSnapshotRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* CreateSnapshotRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return CreateSnapshotRequest_globals_.GetClassData();
#else
  return CreateSnapshotRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class CloudStorageConfig::_Internal {
 public:
  using HasBits = decltype(::std::declval<CloudStorageConfig>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_._has_bits_);
  static constexpr ::int32_t kOneofCaseOffset =
      PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CloudStorageConfig, _impl_._oneof_case_);
};

constexpr CloudStorageConfig::ParseTableT_ CloudStorageConfig::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_._has_bits_),
      0, // no _extensions_
      11, 120,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294965248,  // skipmap
      offsetof(ParseTableT_, field_entries),
      11,  // num_field_entries
      3,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::CloudStorageConfig>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string bucket = 1 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.bucket_)}},
      // string filename_prefix = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.filename_prefix_)}},
      // string filename_suffix = 3 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {26, 2, 0,
        PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.filename_suffix_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      // .google.protobuf.Duration max_duration = 6 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS1,
       {50, 5, 2,
        PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.max_duration_)}},
      // int64 max_bytes = 7 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(CloudStorageConfig, _impl_.max_bytes_), 6>(),
       {56, 6, 0,
        PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.max_bytes_)}},
      // int64 max_messages = 8 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(CloudStorageConfig, _impl_.max_messages_), 7>(),
       {64, 7, 0,
        PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.max_messages_)}},
      // .google.pubsub.v1.CloudStorageConfig.State state = 9 [(.google.api.field_behavior) = OUTPUT_ONLY];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(CloudStorageConfig, _impl_.state_), 8>(),
       {72, 8, 0,
        PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.state_)}},
      // string filename_datetime_format = 10 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {82, 3, 0,
        PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.filename_datetime_format_)}},
      // string service_account_email = 11 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {90, 4, 0,
        PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.service_account_email_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // string bucket = 1 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.bucket_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string filename_prefix = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.filename_prefix_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string filename_suffix = 3 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.filename_suffix_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.pubsub.v1.CloudStorageConfig.TextConfig text_config = 4 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.output_format_.text_config_), _Internal::kOneofCaseOffset + 0, 0, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.pubsub.v1.CloudStorageConfig.AvroConfig avro_config = 5 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.output_format_.avro_config_), _Internal::kOneofCaseOffset + 0, 1, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.protobuf.Duration max_duration = 6 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.max_duration_), _Internal::kHasBitsOffset + 5, 2, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // int64 max_bytes = 7 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.max_bytes_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // int64 max_messages = 8 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.max_messages_), _Internal::kHasBitsOffset + 7, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // .google.pubsub.v1.CloudStorageConfig.State state = 9 [(.google.api.field_behavior) = OUTPUT_ONLY];
      {PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.state_), _Internal::kHasBitsOffset + 8, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // string filename_datetime_format = 10 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.filename_datetime_format_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string service_account_email = 11 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.service_account_email_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::CloudStorageConfig_TextConfig>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::CloudStorageConfig_TextConfig_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::CloudStorageConfig_AvroConfig>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::CloudStorageConfig_AvroConfig_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::Duration>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::Duration_globals_},
        #endif
    }},
    {{
      "\43\6\17\17\0\0\0\0\0\0\30\25\0\0\0\0"
      "google.pubsub.v1.CloudStorageConfig"
      "bucket"
      "filename_prefix"
      "filename_suffix"
      "filename_datetime_format"
      "service_account_email"
    }},
  };
}


inline constexpr CloudStorageConfig::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        bucket_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        filename_prefix_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        filename_suffix_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        filename_datetime_format_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        service_account_email_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        max_duration_{nullptr},
        max_bytes_{::int64_t{0}},
        max_messages_{::int64_t{0}},
        state_{static_cast< ::google::pubsub::v1::CloudStorageConfig_State >(0)},
        output_format_{},
        _oneof_case_{} {}

template <typename>
constexpr CloudStorageConfig::CloudStorageConfig(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL CloudStorageConfig::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) CloudStorageConfig(arena);
}
constexpr auto CloudStorageConfig::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(CloudStorageConfig), alignof(CloudStorageConfig));
}
constexpr auto CloudStorageConfig::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &CloudStorageConfig::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<CloudStorageConfig>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &CloudStorageConfig::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<CloudStorageConfig>(), &CloudStorageConfig::ByteSizeLong,
              &CloudStorageConfig::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[44],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct CloudStorageConfigGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr CloudStorageConfigGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 CloudStorageConfig_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(CloudStorageConfig::InternalGenerateClassData_(
            _default, &CloudStorageConfig_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<CloudStorageConfig>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~CloudStorageConfigGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) CloudStorageConfig _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<CloudStorageConfig>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(CloudStorageConfigGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST CloudStorageConfigGlobalsTypeInternal CloudStorageConfig_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* CloudStorageConfig_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return CloudStorageConfig_globals_.GetClassData();
#else
  return CloudStorageConfig_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class UpdateSnapshotRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<UpdateSnapshotRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(UpdateSnapshotRequest, _impl_._has_bits_);
};

constexpr UpdateSnapshotRequest::ParseTableT_ UpdateSnapshotRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(UpdateSnapshotRequest, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::UpdateSnapshotRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.protobuf.FieldMask update_mask = 2 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(UpdateSnapshotRequest, _impl_.update_mask_)}},
      // .google.pubsub.v1.Snapshot snapshot = 1 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(UpdateSnapshotRequest, _impl_.snapshot_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.pubsub.v1.Snapshot snapshot = 1 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(UpdateSnapshotRequest, _impl_.snapshot_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.protobuf.FieldMask update_mask = 2 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(UpdateSnapshotRequest, _impl_.update_mask_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::Snapshot>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::Snapshot_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::FieldMask>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::FieldMask_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr UpdateSnapshotRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        snapshot_{nullptr},
        update_mask_{nullptr} {}

template <typename>
constexpr UpdateSnapshotRequest::UpdateSnapshotRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL UpdateSnapshotRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) UpdateSnapshotRequest(arena);
}
constexpr auto UpdateSnapshotRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(UpdateSnapshotRequest), alignof(UpdateSnapshotRequest));
}
constexpr auto UpdateSnapshotRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &UpdateSnapshotRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<UpdateSnapshotRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &UpdateSnapshotRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<UpdateSnapshotRequest>(), &UpdateSnapshotRequest::ByteSizeLong,
              &UpdateSnapshotRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(UpdateSnapshotRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[63],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UpdateSnapshotRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UpdateSnapshotRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 UpdateSnapshotRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(UpdateSnapshotRequest::InternalGenerateClassData_(
            _default, &UpdateSnapshotRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<UpdateSnapshotRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UpdateSnapshotRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) UpdateSnapshotRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<UpdateSnapshotRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UpdateSnapshotRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UpdateSnapshotRequestGlobalsTypeInternal UpdateSnapshotRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* UpdateSnapshotRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return UpdateSnapshotRequest_globals_.GetClassData();
#else
  return UpdateSnapshotRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Subscription::_Internal {
 public:
  using HasBits = decltype(::std::declval<Subscription>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Subscription, _impl_._has_bits_);
};

constexpr Subscription::ParseTableT_ Subscription::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Subscription, _impl_._has_bits_),
      0, // no _extensions_
      23, 248,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4288151588,  // skipmap
      offsetof(ParseTableT_, field_entries),
      19,  // num_field_entries
      10,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::Subscription>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string name = 1 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(Subscription, _impl_.name_)}},
      // string topic = 2 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(Subscription, _impl_.topic_)}},
      {::_pbi::TcParser::MiniParse, {}},
      // .google.pubsub.v1.PushConfig push_config = 4 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS1,
       {34, 3, 0,
        PROTOBUF_FIELD_OFFSET(Subscription, _impl_.push_config_)}},
      // int32 ack_deadline_seconds = 5 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Subscription, _impl_.ack_deadline_seconds_), 12>(),
       {40, 12, 0,
        PROTOBUF_FIELD_OFFSET(Subscription, _impl_.ack_deadline_seconds_)}},
      {::_pbi::TcParser::MiniParse, {}},
      // bool retain_acked_messages = 7 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(Subscription, _impl_.retain_acked_messages_), 13>(),
       {56, 13, 0,
        PROTOBUF_FIELD_OFFSET(Subscription, _impl_.retain_acked_messages_)}},
      // .google.protobuf.Duration message_retention_duration = 8 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS1,
       {66, 4, 1,
        PROTOBUF_FIELD_OFFSET(Subscription, _impl_.message_retention_duration_)}},
      {::_pbi::TcParser::MiniParse, {}},
      // bool enable_message_ordering = 10 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(Subscription, _impl_.enable_message_ordering_), 14>(),
       {80, 14, 0,
        PROTOBUF_FIELD_OFFSET(Subscription, _impl_.enable_message_ordering_)}},
      // .google.pubsub.v1.ExpirationPolicy expiration_policy = 11 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS1,
       {90, 5, 2,
        PROTOBUF_FIELD_OFFSET(Subscription, _impl_.expiration_policy_)}},
      // string filter = 12 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {98, 2, 0,
        PROTOBUF_FIELD_OFFSET(Subscription, _impl_.filter_)}},
      // .google.pubsub.v1.DeadLetterPolicy dead_letter_policy = 13 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS1,
       {106, 6, 3,
        PROTOBUF_FIELD_OFFSET(Subscription, _impl_.dead_letter_policy_)}},
      // .google.pubsub.v1.RetryPolicy retry_policy = 14 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS1,
       {114, 7, 4,
        PROTOBUF_FIELD_OFFSET(Subscription, _impl_.retry_policy_)}},
      // bool detached = 15 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(Subscription, _impl_.detached_), 15>(),
       {120, 15, 0,
        PROTOBUF_FIELD_OFFSET(Subscription, _impl_.detached_)}},
      // bool enable_exactly_once_delivery = 16 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastV8S2,
       {384, 16, 0,
        PROTOBUF_FIELD_OFFSET(Subscription, _impl_.enable_exactly_once_delivery_)}},
      // .google.protobuf.Duration topic_message_retention_duration = 17 [(.google.api.field_behavior) = OUTPUT_ONLY];
      {::_pbi::TcParser::FastMtS2,
       {394, 8, 5,
        PROTOBUF_FIELD_OFFSET(Subscription, _impl_.topic_message_retention_duration_)}},
      // .google.pubsub.v1.BigQueryConfig bigquery_config = 18 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS2,
       {402, 9, 6,
        PROTOBUF_FIELD_OFFSET(Subscription, _impl_.bigquery_config_)}},
      // .google.pubsub.v1.Subscription.State state = 19 [(.google.api.field_behavior) = OUTPUT_ONLY];
      {::_pbi::TcParser::FastV32S2,
       {408, 17, 0,
        PROTOBUF_FIELD_OFFSET(Subscription, _impl_.state_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      // .google.pubsub.v1.CloudStorageConfig cloud_storage_config = 22 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS2,
       {434, 10, 7,
        PROTOBUF_FIELD_OFFSET(Subscription, _impl_.cloud_storage_config_)}},
      // .google.pubsub.v1.Subscription.AnalyticsHubSubscriptionInfo analytics_hub_subscription_info = 23 [(.google.api.field_behavior) = OUTPUT_ONLY];
      {::_pbi::TcParser::FastMtS2,
       {442, 11, 8,
        PROTOBUF_FIELD_OFFSET(Subscription, _impl_.analytics_hub_subscription_info_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // string name = 1 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(Subscription, _impl_.name_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string topic = 2 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(Subscription, _impl_.topic_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.pubsub.v1.PushConfig push_config = 4 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Subscription, _impl_.push_config_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // int32 ack_deadline_seconds = 5 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Subscription, _impl_.ack_deadline_seconds_), _Internal::kHasBitsOffset + 12, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // bool retain_acked_messages = 7 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Subscription, _impl_.retain_acked_messages_), _Internal::kHasBitsOffset + 13, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // .google.protobuf.Duration message_retention_duration = 8 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Subscription, _impl_.message_retention_duration_), _Internal::kHasBitsOffset + 4, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // map<string, string> labels = 9 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Subscription, _impl_.labels_), _Internal::kHasBitsOffset + 18, 9, (0 | ::_fl::kFcRepeated | ::_fl::kMap)},
      // bool enable_message_ordering = 10 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Subscription, _impl_.enable_message_ordering_), _Internal::kHasBitsOffset + 14, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // .google.pubsub.v1.ExpirationPolicy expiration_policy = 11 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Subscription, _impl_.expiration_policy_), _Internal::kHasBitsOffset + 5, 2, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // string filter = 12 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Subscription, _impl_.filter_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.pubsub.v1.DeadLetterPolicy dead_letter_policy = 13 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Subscription, _impl_.dead_letter_policy_), _Internal::kHasBitsOffset + 6, 3, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.pubsub.v1.RetryPolicy retry_policy = 14 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Subscription, _impl_.retry_policy_), _Internal::kHasBitsOffset + 7, 4, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // bool detached = 15 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Subscription, _impl_.detached_), _Internal::kHasBitsOffset + 15, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // bool enable_exactly_once_delivery = 16 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Subscription, _impl_.enable_exactly_once_delivery_), _Internal::kHasBitsOffset + 16, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // .google.protobuf.Duration topic_message_retention_duration = 17 [(.google.api.field_behavior) = OUTPUT_ONLY];
      {PROTOBUF_FIELD_OFFSET(Subscription, _impl_.topic_message_retention_duration_), _Internal::kHasBitsOffset + 8, 5, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.pubsub.v1.BigQueryConfig bigquery_config = 18 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Subscription, _impl_.bigquery_config_), _Internal::kHasBitsOffset + 9, 6, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.pubsub.v1.Subscription.State state = 19 [(.google.api.field_behavior) = OUTPUT_ONLY];
      {PROTOBUF_FIELD_OFFSET(Subscription, _impl_.state_), _Internal::kHasBitsOffset + 17, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // .google.pubsub.v1.CloudStorageConfig cloud_storage_config = 22 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Subscription, _impl_.cloud_storage_config_), _Internal::kHasBitsOffset + 10, 7, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.pubsub.v1.Subscription.AnalyticsHubSubscriptionInfo analytics_hub_subscription_info = 23 [(.google.api.field_behavior) = OUTPUT_ONLY];
      {PROTOBUF_FIELD_OFFSET(Subscription, _impl_.analytics_hub_subscription_info_), _Internal::kHasBitsOffset + 11, 8, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::PushConfig>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::PushConfig_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::Duration>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::Duration_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::ExpirationPolicy>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::ExpirationPolicy_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::DeadLetterPolicy>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::DeadLetterPolicy_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::RetryPolicy>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::RetryPolicy_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::Duration>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::Duration_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::BigQueryConfig>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::BigQueryConfig_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::CloudStorageConfig>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::CloudStorageConfig_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::Subscription_AnalyticsHubSubscriptionInfo>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::Subscription_AnalyticsHubSubscriptionInfo_globals_},
        #endif
        {::_pbi::TcParser::GetMapAuxInfo(
            1, 0, 9, 9, 0)},
    }},
    {{
      "\35\4\5\0\0\0\0\6\0\0\6\0\0\0\0\0\0\0\0\0\0\0\0\0"
      "google.pubsub.v1.Subscription"
      "name"
      "topic"
      "labels"
      "filter"
    }},
  };
}


inline constexpr Subscription::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        topic_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        filter_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        push_config_{nullptr},
        message_retention_duration_{nullptr},
        expiration_policy_{nullptr},
        dead_letter_policy_{nullptr},
        retry_policy_{nullptr},
        topic_message_retention_duration_{nullptr},
        bigquery_config_{nullptr},
        cloud_storage_config_{nullptr},
        analytics_hub_subscription_info_{nullptr},
        ack_deadline_seconds_{0},
        retain_acked_messages_{false},
        enable_message_ordering_{false},
        detached_{false},
        enable_exactly_once_delivery_{false},
        state_{static_cast< ::google::pubsub::v1::Subscription_State >(0)},
        labels_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::Subscription,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.labels_)>()
         }
     {}

template <typename>
constexpr Subscription::Subscription(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Subscription::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Subscription(arena);
}
constexpr auto Subscription::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Subscription), alignof(Subscription));
}
constexpr auto Subscription::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Subscription::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<Subscription>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Subscription::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<Subscription>(), &Subscription::ByteSizeLong,
              &Subscription::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Subscription, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[32],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct SubscriptionGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr SubscriptionGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Subscription_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Subscription::InternalGenerateClassData_(
            _default, &Subscription_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Subscription>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~SubscriptionGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Subscription _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Subscription>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(SubscriptionGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST SubscriptionGlobalsTypeInternal Subscription_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Subscription_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Subscription_globals_.GetClassData();
#else
  return Subscription_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ReceivedMessage::_Internal {
 public:
  using HasBits = decltype(::std::declval<ReceivedMessage>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ReceivedMessage, _impl_._has_bits_);
};

constexpr ReceivedMessage::ParseTableT_ ReceivedMessage::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ReceivedMessage, _impl_._has_bits_),
      0, // no _extensions_
      3, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ReceivedMessage>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string ack_id = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ReceivedMessage, _impl_.ack_id_)}},
      // .google.pubsub.v1.PubsubMessage message = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(ReceivedMessage, _impl_.message_)}},
      // int32 delivery_attempt = 3 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ReceivedMessage, _impl_.delivery_attempt_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(ReceivedMessage, _impl_.delivery_attempt_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string ack_id = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ReceivedMessage, _impl_.ack_id_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.pubsub.v1.PubsubMessage message = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ReceivedMessage, _impl_.message_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // int32 delivery_attempt = 3 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ReceivedMessage, _impl_.delivery_attempt_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::PubsubMessage>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::PubsubMessage_globals_},
        #endif
    }},
    {{
      "\40\6\0\0\0\0\0\0"
      "google.pubsub.v1.ReceivedMessage"
      "ack_id"
    }},
  };
}


inline constexpr ReceivedMessage::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        ack_id_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        message_{nullptr},
        delivery_attempt_{0} {}

template <typename>
constexpr ReceivedMessage::ReceivedMessage(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ReceivedMessage::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ReceivedMessage(arena);
}
constexpr auto ReceivedMessage::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ReceivedMessage), alignof(ReceivedMessage));
}
constexpr auto ReceivedMessage::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ReceivedMessage::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ReceivedMessage>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ReceivedMessage::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ReceivedMessage>(), &ReceivedMessage::ByteSizeLong,
              &ReceivedMessage::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ReceivedMessage, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[45],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ReceivedMessageGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ReceivedMessageGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ReceivedMessage_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ReceivedMessage::InternalGenerateClassData_(
            _default, &ReceivedMessage_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ReceivedMessage>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ReceivedMessageGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ReceivedMessage _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ReceivedMessage>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ReceivedMessageGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ReceivedMessageGlobalsTypeInternal ReceivedMessage_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ReceivedMessage_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ReceivedMessage_globals_.GetClassData();
#else
  return ReceivedMessage_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class PublishRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<PublishRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(PublishRequest, _impl_._has_bits_);
};

constexpr PublishRequest::ParseTableT_ PublishRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(PublishRequest, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::PublishRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated .google.pubsub.v1.PubsubMessage messages = 2 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastMtR1,
       {18, 0, 0,
        PROTOBUF_FIELD_OFFSET(PublishRequest, _impl_.messages_)}},
      // string topic = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 1, 0,
        PROTOBUF_FIELD_OFFSET(PublishRequest, _impl_.topic_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string topic = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(PublishRequest, _impl_.topic_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // repeated .google.pubsub.v1.PubsubMessage messages = 2 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(PublishRequest, _impl_.messages_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::PubsubMessage>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::PubsubMessage_globals_},
        #endif
    }},
    {{
      "\37\5\0\0\0\0\0\0"
      "google.pubsub.v1.PublishRequest"
      "topic"
    }},
  };
}


inline constexpr PublishRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        messages_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::PublishRequest,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PublishRequest, _impl_.messages_)>()
         }
        ,
        topic_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr PublishRequest::PublishRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL PublishRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) PublishRequest(arena);
}
constexpr auto PublishRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(PublishRequest), alignof(PublishRequest));
}
constexpr auto PublishRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &PublishRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<PublishRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &PublishRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<PublishRequest>(), &PublishRequest::ByteSizeLong,
              &PublishRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(PublishRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[19],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct PublishRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr PublishRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 PublishRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(PublishRequest::InternalGenerateClassData_(
            _default, &PublishRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<PublishRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~PublishRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) PublishRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<PublishRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(PublishRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST PublishRequestGlobalsTypeInternal PublishRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* PublishRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return PublishRequest_globals_.GetClassData();
#else
  return PublishRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ModifyPushConfigRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<ModifyPushConfigRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ModifyPushConfigRequest, _impl_._has_bits_);
};

constexpr ModifyPushConfigRequest::ParseTableT_ ModifyPushConfigRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ModifyPushConfigRequest, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ModifyPushConfigRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.pubsub.v1.PushConfig push_config = 2 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(ModifyPushConfigRequest, _impl_.push_config_)}},
      // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ModifyPushConfigRequest, _impl_.subscription_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(ModifyPushConfigRequest, _impl_.subscription_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.pubsub.v1.PushConfig push_config = 2 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(ModifyPushConfigRequest, _impl_.push_config_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::PushConfig>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::PushConfig_globals_},
        #endif
    }},
    {{
      "\50\14\0\0\0\0\0\0"
      "google.pubsub.v1.ModifyPushConfigRequest"
      "subscription"
    }},
  };
}


inline constexpr ModifyPushConfigRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        subscription_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        push_config_{nullptr} {}

template <typename>
constexpr ModifyPushConfigRequest::ModifyPushConfigRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ModifyPushConfigRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ModifyPushConfigRequest(arena);
}
constexpr auto ModifyPushConfigRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ModifyPushConfigRequest), alignof(ModifyPushConfigRequest));
}
constexpr auto ModifyPushConfigRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ModifyPushConfigRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ModifyPushConfigRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ModifyPushConfigRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ModifyPushConfigRequest>(), &ModifyPushConfigRequest::ByteSizeLong,
              &ModifyPushConfigRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ModifyPushConfigRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[51],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ModifyPushConfigRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ModifyPushConfigRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ModifyPushConfigRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ModifyPushConfigRequest::InternalGenerateClassData_(
            _default, &ModifyPushConfigRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ModifyPushConfigRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ModifyPushConfigRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ModifyPushConfigRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ModifyPushConfigRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ModifyPushConfigRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ModifyPushConfigRequestGlobalsTypeInternal ModifyPushConfigRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ModifyPushConfigRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ModifyPushConfigRequest_globals_.GetClassData();
#else
  return ModifyPushConfigRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ListSnapshotsResponse::_Internal {
 public:
  using HasBits = decltype(::std::declval<ListSnapshotsResponse>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ListSnapshotsResponse, _impl_._has_bits_);
};

constexpr ListSnapshotsResponse::ParseTableT_ ListSnapshotsResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ListSnapshotsResponse, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ListSnapshotsResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string next_page_token = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(ListSnapshotsResponse, _impl_.next_page_token_)}},
      // repeated .google.pubsub.v1.Snapshot snapshots = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ListSnapshotsResponse, _impl_.snapshots_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated .google.pubsub.v1.Snapshot snapshots = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ListSnapshotsResponse, _impl_.snapshots_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
      // string next_page_token = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ListSnapshotsResponse, _impl_.next_page_token_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::Snapshot>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::Snapshot_globals_},
        #endif
    }},
    {{
      "\46\0\17\0\0\0\0\0"
      "google.pubsub.v1.ListSnapshotsResponse"
      "next_page_token"
    }},
  };
}


inline constexpr ListSnapshotsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        snapshots_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::ListSnapshotsResponse,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSnapshotsResponse, _impl_.snapshots_)>()
         }
        ,
        next_page_token_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr ListSnapshotsResponse::ListSnapshotsResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ListSnapshotsResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ListSnapshotsResponse(arena);
}
constexpr auto ListSnapshotsResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ListSnapshotsResponse), alignof(ListSnapshotsResponse));
}
constexpr auto ListSnapshotsResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ListSnapshotsResponse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ListSnapshotsResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ListSnapshotsResponse::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ListSnapshotsResponse>(), &ListSnapshotsResponse::ByteSizeLong,
              &ListSnapshotsResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ListSnapshotsResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[68],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ListSnapshotsResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ListSnapshotsResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ListSnapshotsResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ListSnapshotsResponse::InternalGenerateClassData_(
            _default, &ListSnapshotsResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ListSnapshotsResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ListSnapshotsResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ListSnapshotsResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ListSnapshotsResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ListSnapshotsResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ListSnapshotsResponseGlobalsTypeInternal ListSnapshotsResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ListSnapshotsResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ListSnapshotsResponse_globals_.GetClassData();
#else
  return ListSnapshotsResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class IngestionFailureEvent::_Internal {
 public:
  using HasBits = decltype(::std::declval<IngestionFailureEvent>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(IngestionFailureEvent, _impl_._has_bits_);
  static constexpr ::int32_t kOneofCaseOffset =
      PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionFailureEvent, _impl_._oneof_case_);
};

constexpr IngestionFailureEvent::ParseTableT_ IngestionFailureEvent::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(IngestionFailureEvent, _impl_._has_bits_),
      0, // no _extensions_
      3, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::IngestionFailureEvent>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string error_message = 2 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(IngestionFailureEvent, _impl_.error_message_)}},
      // string topic = 1 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(IngestionFailureEvent, _impl_.topic_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string topic = 1 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(IngestionFailureEvent, _impl_.topic_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string error_message = 2 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(IngestionFailureEvent, _impl_.error_message_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.pubsub.v1.IngestionFailureEvent.CloudStorageFailure cloud_storage_failure = 3 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(IngestionFailureEvent, _impl_.failure_.cloud_storage_failure_), _Internal::kOneofCaseOffset + 0, 0, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::IngestionFailureEvent_CloudStorageFailure>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::IngestionFailureEvent_CloudStorageFailure_globals_},
        #endif
    }},
    {{
      "\46\5\15\0\0\0\0\0"
      "google.pubsub.v1.IngestionFailureEvent"
      "topic"
      "error_message"
    }},
  };
}


inline constexpr IngestionFailureEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        topic_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        error_message_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        failure_{},
        _oneof_case_{} {}

template <typename>
constexpr IngestionFailureEvent::IngestionFailureEvent(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL IngestionFailureEvent::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) IngestionFailureEvent(arena);
}
constexpr auto IngestionFailureEvent::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(IngestionFailureEvent), alignof(IngestionFailureEvent));
}
constexpr auto IngestionFailureEvent::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &IngestionFailureEvent::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<IngestionFailureEvent>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &IngestionFailureEvent::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<IngestionFailureEvent>(), &IngestionFailureEvent::ByteSizeLong,
              &IngestionFailureEvent::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(IngestionFailureEvent, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[12],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct IngestionFailureEventGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr IngestionFailureEventGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 IngestionFailureEvent_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(IngestionFailureEvent::InternalGenerateClassData_(
            _default, &IngestionFailureEvent_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<IngestionFailureEvent>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~IngestionFailureEventGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) IngestionFailureEvent _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<IngestionFailureEvent>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(IngestionFailureEventGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST IngestionFailureEventGlobalsTypeInternal IngestionFailureEvent_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* IngestionFailureEvent_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return IngestionFailureEvent_globals_.GetClassData();
#else
  return IngestionFailureEvent_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class IngestionDataSourceSettings::_Internal {
 public:
  using HasBits = decltype(::std::declval<IngestionDataSourceSettings>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings, _impl_._has_bits_);
  static constexpr ::int32_t kOneofCaseOffset =
      PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings, _impl_._oneof_case_);
};

constexpr IngestionDataSourceSettings::ParseTableT_ IngestionDataSourceSettings::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings, _impl_._has_bits_),
      0, // no _extensions_
      4, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967284,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      3,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::IngestionDataSourceSettings>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.pubsub.v1.PlatformLogsSettings platform_logs_settings = 4 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS1,
       {34, 0, 2,
        PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings, _impl_.platform_logs_settings_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.pubsub.v1.IngestionDataSourceSettings.AwsKinesis aws_kinesis = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings, _impl_.source_.aws_kinesis_), _Internal::kOneofCaseOffset + 0, 0, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.pubsub.v1.IngestionDataSourceSettings.CloudStorage cloud_storage = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings, _impl_.source_.cloud_storage_), _Internal::kOneofCaseOffset + 0, 1, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.pubsub.v1.PlatformLogsSettings platform_logs_settings = 4 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings, _impl_.platform_logs_settings_), _Internal::kHasBitsOffset + 0, 2, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::IngestionDataSourceSettings_AwsKinesis>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::IngestionDataSourceSettings_AwsKinesis_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::PlatformLogsSettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::PlatformLogsSettings_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr IngestionDataSourceSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        platform_logs_settings_{nullptr},
        source_{},
        _oneof_case_{} {}

template <typename>
constexpr IngestionDataSourceSettings::IngestionDataSourceSettings(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL IngestionDataSourceSettings::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) IngestionDataSourceSettings(arena);
}
constexpr auto IngestionDataSourceSettings::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(IngestionDataSourceSettings), alignof(IngestionDataSourceSettings));
}
constexpr auto IngestionDataSourceSettings::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &IngestionDataSourceSettings::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<IngestionDataSourceSettings>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &IngestionDataSourceSettings::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<IngestionDataSourceSettings>(), &IngestionDataSourceSettings::ByteSizeLong,
              &IngestionDataSourceSettings::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[7],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct IngestionDataSourceSettingsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr IngestionDataSourceSettingsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 IngestionDataSourceSettings_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(IngestionDataSourceSettings::InternalGenerateClassData_(
            _default, &IngestionDataSourceSettings_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<IngestionDataSourceSettings>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~IngestionDataSourceSettingsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) IngestionDataSourceSettings _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<IngestionDataSourceSettings>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettingsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST IngestionDataSourceSettingsGlobalsTypeInternal IngestionDataSourceSettings_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* IngestionDataSourceSettings_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return IngestionDataSourceSettings_globals_.GetClassData();
#else
  return IngestionDataSourceSettings_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class UpdateSubscriptionRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<UpdateSubscriptionRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(UpdateSubscriptionRequest, _impl_._has_bits_);
};

constexpr UpdateSubscriptionRequest::ParseTableT_ UpdateSubscriptionRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(UpdateSubscriptionRequest, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::UpdateSubscriptionRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.protobuf.FieldMask update_mask = 2 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(UpdateSubscriptionRequest, _impl_.update_mask_)}},
      // .google.pubsub.v1.Subscription subscription = 1 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(UpdateSubscriptionRequest, _impl_.subscription_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.pubsub.v1.Subscription subscription = 1 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(UpdateSubscriptionRequest, _impl_.subscription_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.protobuf.FieldMask update_mask = 2 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(UpdateSubscriptionRequest, _impl_.update_mask_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::Subscription>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::Subscription_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::FieldMask>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::FieldMask_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr UpdateSubscriptionRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        subscription_{nullptr},
        update_mask_{nullptr} {}

template <typename>
constexpr UpdateSubscriptionRequest::UpdateSubscriptionRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL UpdateSubscriptionRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) UpdateSubscriptionRequest(arena);
}
constexpr auto UpdateSubscriptionRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(UpdateSubscriptionRequest), alignof(UpdateSubscriptionRequest));
}
constexpr auto UpdateSubscriptionRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &UpdateSubscriptionRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<UpdateSubscriptionRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &UpdateSubscriptionRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<UpdateSubscriptionRequest>(), &UpdateSubscriptionRequest::ByteSizeLong,
              &UpdateSubscriptionRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(UpdateSubscriptionRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[47],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UpdateSubscriptionRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UpdateSubscriptionRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 UpdateSubscriptionRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(UpdateSubscriptionRequest::InternalGenerateClassData_(
            _default, &UpdateSubscriptionRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<UpdateSubscriptionRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UpdateSubscriptionRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) UpdateSubscriptionRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<UpdateSubscriptionRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UpdateSubscriptionRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UpdateSubscriptionRequestGlobalsTypeInternal UpdateSubscriptionRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* UpdateSubscriptionRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return UpdateSubscriptionRequest_globals_.GetClassData();
#else
  return UpdateSubscriptionRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Topic::_Internal {
 public:
  using HasBits = decltype(::std::declval<Topic>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Topic, _impl_._has_bits_);
};

constexpr Topic::ParseTableT_ Topic::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Topic, _impl_._has_bits_),
      0, // no _extensions_
      10, 120,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294966280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      9,  // num_field_entries
      5,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::Topic>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string name = 1 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(Topic, _impl_.name_)}},
      {::_pbi::TcParser::MiniParse, {}},
      // .google.pubsub.v1.MessageStoragePolicy message_storage_policy = 3 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS1,
       {26, 2, 0,
        PROTOBUF_FIELD_OFFSET(Topic, _impl_.message_storage_policy_)}},
      {::_pbi::TcParser::MiniParse, {}},
      // string kms_key_name = 5 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {42, 1, 0,
        PROTOBUF_FIELD_OFFSET(Topic, _impl_.kms_key_name_)}},
      // .google.pubsub.v1.SchemaSettings schema_settings = 6 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS1,
       {50, 3, 1,
        PROTOBUF_FIELD_OFFSET(Topic, _impl_.schema_settings_)}},
      // bool satisfies_pzs = 7 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(Topic, _impl_.satisfies_pzs_), 6>(),
       {56, 6, 0,
        PROTOBUF_FIELD_OFFSET(Topic, _impl_.satisfies_pzs_)}},
      // .google.protobuf.Duration message_retention_duration = 8 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS1,
       {66, 4, 2,
        PROTOBUF_FIELD_OFFSET(Topic, _impl_.message_retention_duration_)}},
      // .google.pubsub.v1.Topic.State state = 9 [(.google.api.field_behavior) = OUTPUT_ONLY];
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Topic, _impl_.state_), 7>(),
       {72, 7, 0,
        PROTOBUF_FIELD_OFFSET(Topic, _impl_.state_)}},
      // .google.pubsub.v1.IngestionDataSourceSettings ingestion_data_source_settings = 10 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS1,
       {82, 5, 3,
        PROTOBUF_FIELD_OFFSET(Topic, _impl_.ingestion_data_source_settings_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // string name = 1 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(Topic, _impl_.name_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // map<string, string> labels = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Topic, _impl_.labels_), _Internal::kHasBitsOffset + 8, 4, (0 | ::_fl::kFcRepeated | ::_fl::kMap)},
      // .google.pubsub.v1.MessageStoragePolicy message_storage_policy = 3 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Topic, _impl_.message_storage_policy_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // string kms_key_name = 5 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Topic, _impl_.kms_key_name_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.pubsub.v1.SchemaSettings schema_settings = 6 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Topic, _impl_.schema_settings_), _Internal::kHasBitsOffset + 3, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // bool satisfies_pzs = 7 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Topic, _impl_.satisfies_pzs_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // .google.protobuf.Duration message_retention_duration = 8 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Topic, _impl_.message_retention_duration_), _Internal::kHasBitsOffset + 4, 2, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.pubsub.v1.Topic.State state = 9 [(.google.api.field_behavior) = OUTPUT_ONLY];
      {PROTOBUF_FIELD_OFFSET(Topic, _impl_.state_), _Internal::kHasBitsOffset + 7, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // .google.pubsub.v1.IngestionDataSourceSettings ingestion_data_source_settings = 10 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(Topic, _impl_.ingestion_data_source_settings_), _Internal::kHasBitsOffset + 5, 3, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::MessageStoragePolicy>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::MessageStoragePolicy_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::SchemaSettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::SchemaSettings_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::Duration>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::Duration_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::IngestionDataSourceSettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::IngestionDataSourceSettings_globals_},
        #endif
        {::_pbi::TcParser::GetMapAuxInfo(
            1, 0, 9, 9, 0)},
    }},
    {{
      "\26\4\6\0\14\0\0\0\0\0\0\0\0\0\0\0"
      "google.pubsub.v1.Topic"
      "name"
      "labels"
      "kms_key_name"
    }},
  };
}


inline constexpr Topic::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        kms_key_name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        message_storage_policy_{nullptr},
        schema_settings_{nullptr},
        message_retention_duration_{nullptr},
        ingestion_data_source_settings_{nullptr},
        satisfies_pzs_{false},
        state_{static_cast< ::google::pubsub::v1::Topic_State >(0)},
        labels_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::Topic,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Topic, _impl_.labels_)>()
         }
     {}

template <typename>
constexpr Topic::Topic(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Topic::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Topic(arena);
}
constexpr auto Topic::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Topic), alignof(Topic));
}
constexpr auto Topic::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Topic::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<Topic>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Topic::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<Topic>(), &Topic::ByteSizeLong,
              &Topic::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Topic, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[14],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct TopicGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr TopicGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Topic_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Topic::InternalGenerateClassData_(
            _default, &Topic_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Topic>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~TopicGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Topic _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Topic>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(TopicGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST TopicGlobalsTypeInternal Topic_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Topic_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Topic_globals_.GetClassData();
#else
  return Topic_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class StreamingPullResponse::_Internal {
 public:
  using HasBits = decltype(::std::declval<StreamingPullResponse>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(StreamingPullResponse, _impl_._has_bits_);
};

constexpr StreamingPullResponse::ParseTableT_ StreamingPullResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(StreamingPullResponse, _impl_._has_bits_),
      0, // no _extensions_
      5, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967266,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      4,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::StreamingPullResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // repeated .google.pubsub.v1.ReceivedMessage received_messages = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(StreamingPullResponse, _impl_.received_messages_)}},
      {::_pbi::TcParser::MiniParse, {}},
      // .google.pubsub.v1.StreamingPullResponse.ModifyAckDeadlineConfirmation modify_ack_deadline_confirmation = 3 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS1,
       {26, 1, 1,
        PROTOBUF_FIELD_OFFSET(StreamingPullResponse, _impl_.modify_ack_deadline_confirmation_)}},
      // .google.pubsub.v1.StreamingPullResponse.SubscriptionProperties subscription_properties = 4 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS1,
       {34, 2, 2,
        PROTOBUF_FIELD_OFFSET(StreamingPullResponse, _impl_.subscription_properties_)}},
      // .google.pubsub.v1.StreamingPullResponse.AcknowledgeConfirmation acknowledge_confirmation = 5 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtS1,
       {42, 3, 3,
        PROTOBUF_FIELD_OFFSET(StreamingPullResponse, _impl_.acknowledge_confirmation_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated .google.pubsub.v1.ReceivedMessage received_messages = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(StreamingPullResponse, _impl_.received_messages_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.pubsub.v1.StreamingPullResponse.ModifyAckDeadlineConfirmation modify_ack_deadline_confirmation = 3 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(StreamingPullResponse, _impl_.modify_ack_deadline_confirmation_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.pubsub.v1.StreamingPullResponse.SubscriptionProperties subscription_properties = 4 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(StreamingPullResponse, _impl_.subscription_properties_), _Internal::kHasBitsOffset + 2, 2, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.pubsub.v1.StreamingPullResponse.AcknowledgeConfirmation acknowledge_confirmation = 5 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(StreamingPullResponse, _impl_.acknowledge_confirmation_), _Internal::kHasBitsOffset + 3, 3, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::ReceivedMessage>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::ReceivedMessage_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::StreamingPullResponse_SubscriptionProperties>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::StreamingPullResponse_SubscriptionProperties_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr StreamingPullResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        received_messages_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullResponse,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse, _impl_.received_messages_)>()
         }
        ,
        modify_ack_deadline_confirmation_{nullptr},
        subscription_properties_{nullptr},
        acknowledge_confirmation_{nullptr} {}

template <typename>
constexpr StreamingPullResponse::StreamingPullResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL StreamingPullResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) StreamingPullResponse(arena);
}
constexpr auto StreamingPullResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(StreamingPullResponse), alignof(StreamingPullResponse));
}
constexpr auto StreamingPullResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &StreamingPullResponse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<StreamingPullResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &StreamingPullResponse::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<StreamingPullResponse>(), &StreamingPullResponse::ByteSizeLong,
              &StreamingPullResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(StreamingPullResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[60],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct StreamingPullResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr StreamingPullResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 StreamingPullResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(StreamingPullResponse::InternalGenerateClassData_(
            _default, &StreamingPullResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<StreamingPullResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~StreamingPullResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) StreamingPullResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<StreamingPullResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(StreamingPullResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST StreamingPullResponseGlobalsTypeInternal StreamingPullResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* StreamingPullResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return StreamingPullResponse_globals_.GetClassData();
#else
  return StreamingPullResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class PullResponse::_Internal {
 public:
  using HasBits = decltype(::std::declval<PullResponse>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(PullResponse, _impl_._has_bits_);
};

constexpr PullResponse::ParseTableT_ PullResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(PullResponse, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::PullResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated .google.pubsub.v1.ReceivedMessage received_messages = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(PullResponse, _impl_.received_messages_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated .google.pubsub.v1.ReceivedMessage received_messages = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(PullResponse, _impl_.received_messages_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::ReceivedMessage>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::ReceivedMessage_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr PullResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        received_messages_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::PullResponse,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PullResponse, _impl_.received_messages_)>()
         }
     {}

template <typename>
constexpr PullResponse::PullResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL PullResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) PullResponse(arena);
}
constexpr auto PullResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(PullResponse), alignof(PullResponse));
}
constexpr auto PullResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &PullResponse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<PullResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &PullResponse::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<PullResponse>(), &PullResponse::ByteSizeLong,
              &PullResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(PullResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[53],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct PullResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr PullResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 PullResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(PullResponse::InternalGenerateClassData_(
            _default, &PullResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<PullResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~PullResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) PullResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<PullResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(PullResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST PullResponseGlobalsTypeInternal PullResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* PullResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return PullResponse_globals_.GetClassData();
#else
  return PullResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ListSubscriptionsResponse::_Internal {
 public:
  using HasBits = decltype(::std::declval<ListSubscriptionsResponse>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ListSubscriptionsResponse, _impl_._has_bits_);
};

constexpr ListSubscriptionsResponse::ParseTableT_ ListSubscriptionsResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ListSubscriptionsResponse, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ListSubscriptionsResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string next_page_token = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(ListSubscriptionsResponse, _impl_.next_page_token_)}},
      // repeated .google.pubsub.v1.Subscription subscriptions = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ListSubscriptionsResponse, _impl_.subscriptions_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated .google.pubsub.v1.Subscription subscriptions = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ListSubscriptionsResponse, _impl_.subscriptions_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
      // string next_page_token = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ListSubscriptionsResponse, _impl_.next_page_token_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::Subscription>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::Subscription_globals_},
        #endif
    }},
    {{
      "\52\0\17\0\0\0\0\0"
      "google.pubsub.v1.ListSubscriptionsResponse"
      "next_page_token"
    }},
  };
}


inline constexpr ListSubscriptionsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        subscriptions_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::ListSubscriptionsResponse,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSubscriptionsResponse, _impl_.subscriptions_)>()
         }
        ,
        next_page_token_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr ListSubscriptionsResponse::ListSubscriptionsResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ListSubscriptionsResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ListSubscriptionsResponse(arena);
}
constexpr auto ListSubscriptionsResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ListSubscriptionsResponse), alignof(ListSubscriptionsResponse));
}
constexpr auto ListSubscriptionsResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ListSubscriptionsResponse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ListSubscriptionsResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ListSubscriptionsResponse::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ListSubscriptionsResponse>(), &ListSubscriptionsResponse::ByteSizeLong,
              &ListSubscriptionsResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ListSubscriptionsResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[49],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ListSubscriptionsResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ListSubscriptionsResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ListSubscriptionsResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ListSubscriptionsResponse::InternalGenerateClassData_(
            _default, &ListSubscriptionsResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ListSubscriptionsResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ListSubscriptionsResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ListSubscriptionsResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ListSubscriptionsResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ListSubscriptionsResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ListSubscriptionsResponseGlobalsTypeInternal ListSubscriptionsResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ListSubscriptionsResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ListSubscriptionsResponse_globals_.GetClassData();
#else
  return ListSubscriptionsResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class UpdateTopicRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<UpdateTopicRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(UpdateTopicRequest, _impl_._has_bits_);
};

constexpr UpdateTopicRequest::ParseTableT_ UpdateTopicRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(UpdateTopicRequest, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::UpdateTopicRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.protobuf.FieldMask update_mask = 2 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(UpdateTopicRequest, _impl_.update_mask_)}},
      // .google.pubsub.v1.Topic topic = 1 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(UpdateTopicRequest, _impl_.topic_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.pubsub.v1.Topic topic = 1 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(UpdateTopicRequest, _impl_.topic_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.protobuf.FieldMask update_mask = 2 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(UpdateTopicRequest, _impl_.update_mask_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::Topic>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::Topic_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::FieldMask>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::FieldMask_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr UpdateTopicRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        topic_{nullptr},
        update_mask_{nullptr} {}

template <typename>
constexpr UpdateTopicRequest::UpdateTopicRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL UpdateTopicRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) UpdateTopicRequest(arena);
}
constexpr auto UpdateTopicRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(UpdateTopicRequest), alignof(UpdateTopicRequest));
}
constexpr auto UpdateTopicRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &UpdateTopicRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<UpdateTopicRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &UpdateTopicRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<UpdateTopicRequest>(), &UpdateTopicRequest::ByteSizeLong,
              &UpdateTopicRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(UpdateTopicRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[18],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct UpdateTopicRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr UpdateTopicRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 UpdateTopicRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(UpdateTopicRequest::InternalGenerateClassData_(
            _default, &UpdateTopicRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<UpdateTopicRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~UpdateTopicRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) UpdateTopicRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<UpdateTopicRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(UpdateTopicRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST UpdateTopicRequestGlobalsTypeInternal UpdateTopicRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* UpdateTopicRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return UpdateTopicRequest_globals_.GetClassData();
#else
  return UpdateTopicRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ListTopicsResponse::_Internal {
 public:
  using HasBits = decltype(::std::declval<ListTopicsResponse>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ListTopicsResponse, _impl_._has_bits_);
};

constexpr ListTopicsResponse::ParseTableT_ ListTopicsResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ListTopicsResponse, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::pubsub::v1::ListTopicsResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string next_page_token = 2 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(ListTopicsResponse, _impl_.next_page_token_)}},
      // repeated .google.pubsub.v1.Topic topics = 1 [(.google.api.field_behavior) = OPTIONAL];
      {::_pbi::TcParser::FastMtR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ListTopicsResponse, _impl_.topics_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated .google.pubsub.v1.Topic topics = 1 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ListTopicsResponse, _impl_.topics_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
      // string next_page_token = 2 [(.google.api.field_behavior) = OPTIONAL];
      {PROTOBUF_FIELD_OFFSET(ListTopicsResponse, _impl_.next_page_token_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::pubsub::v1::Topic>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::pubsub::v1::Topic_globals_},
        #endif
    }},
    {{
      "\43\0\17\0\0\0\0\0"
      "google.pubsub.v1.ListTopicsResponse"
      "next_page_token"
    }},
  };
}


inline constexpr ListTopicsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        topics_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::ListTopicsResponse,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicsResponse, _impl_.topics_)>()
         }
        ,
        next_page_token_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr ListTopicsResponse::ListTopicsResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ListTopicsResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ListTopicsResponse(arena);
}
constexpr auto ListTopicsResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ListTopicsResponse), alignof(ListTopicsResponse));
}
constexpr auto ListTopicsResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ListTopicsResponse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ListTopicsResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ListTopicsResponse::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ListTopicsResponse>(), &ListTopicsResponse::ByteSizeLong,
              &ListTopicsResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ListTopicsResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[22],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ListTopicsResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ListTopicsResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ListTopicsResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ListTopicsResponse::InternalGenerateClassData_(
            _default, &ListTopicsResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ListTopicsResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ListTopicsResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ListTopicsResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ListTopicsResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ListTopicsResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ListTopicsResponseGlobalsTypeInternal ListTopicsResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ListTopicsResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ListTopicsResponse_globals_.GetClassData();
#else
  return ListTopicsResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
}  // namespace v1
}  // namespace pubsub
}  // namespace google
static const ::_pb::EnumDescriptor* PROTOBUF_NONNULL
    file_level_enum_descriptors_google_2fpubsub_2fv1_2fpubsub_2eproto[7];
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_google_2fpubsub_2fv1_2fpubsub_2eproto = nullptr;
const ::uint32_t
    TableStruct_google_2fpubsub_2fv1_2fpubsub_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::MessageStoragePolicy, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::MessageStoragePolicy, _impl_.allowed_persistence_regions_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::MessageStoragePolicy, _impl_.enforce_in_transit_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::SchemaSettings, _impl_._has_bits_),
        7, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::SchemaSettings, _impl_.schema_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::SchemaSettings, _impl_.encoding_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::SchemaSettings, _impl_.first_revision_id_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::SchemaSettings, _impl_.last_revision_id_),
        0,
        3,
        1,
        2,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings_AwsKinesis, _impl_._has_bits_),
        8, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings_AwsKinesis, _impl_.state_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings_AwsKinesis, _impl_.stream_arn_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings_AwsKinesis, _impl_.consumer_arn_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings_AwsKinesis, _impl_.aws_role_arn_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings_AwsKinesis, _impl_.gcp_service_account_),
        4,
        0,
        1,
        2,
        3,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_TextFormat, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_TextFormat, _impl_.delimiter_),
        0,
        0x000, // bitmap
        0x000, // bitmap
        0x085, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage, _impl_._has_bits_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage, _impl_._oneof_case_[0]),
        12, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage, _impl_.state_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage, _impl_.bucket_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage, _impl_.input_format_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage, _impl_.input_format_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage, _impl_.input_format_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage, _impl_.minimum_object_create_time_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage, _impl_.match_glob_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage, _impl_.input_format_),
        3,
        0,
        ~0u,
        ~0u,
        ~0u,
        2,
        1,
        0x085, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings, _impl_._has_bits_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings, _impl_._oneof_case_[0]),
        8, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings, _impl_.source_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings, _impl_.source_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings, _impl_.platform_logs_settings_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionDataSourceSettings, _impl_.source_),
        ~0u,
        ~0u,
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PlatformLogsSettings, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PlatformLogsSettings, _impl_.severity_),
        0,
        0x000, // bitmap
        0x000, // bitmap
        0x085, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionFailureEvent_CloudStorageFailure, _impl_._has_bits_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionFailureEvent_CloudStorageFailure, _impl_._oneof_case_[0]),
        10, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionFailureEvent_CloudStorageFailure, _impl_.bucket_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionFailureEvent_CloudStorageFailure, _impl_.object_name_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionFailureEvent_CloudStorageFailure, _impl_.object_generation_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionFailureEvent_CloudStorageFailure, _impl_.reason_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionFailureEvent_CloudStorageFailure, _impl_.reason_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionFailureEvent_CloudStorageFailure, _impl_.reason_),
        0,
        1,
        2,
        ~0u,
        ~0u,
        0x085, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionFailureEvent, _impl_._has_bits_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionFailureEvent, _impl_._oneof_case_[0]),
        8, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionFailureEvent, _impl_.topic_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionFailureEvent, _impl_.error_message_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionFailureEvent, _impl_.failure_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::IngestionFailureEvent, _impl_.failure_),
        0,
        1,
        ~0u,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Topic_LabelsEntry_DoNotUse, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Topic_LabelsEntry_DoNotUse, _impl_.key_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Topic_LabelsEntry_DoNotUse, _impl_.value_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Topic, _impl_._has_bits_),
        12, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Topic, _impl_.name_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Topic, _impl_.labels_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Topic, _impl_.message_storage_policy_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Topic, _impl_.kms_key_name_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Topic, _impl_.schema_settings_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Topic, _impl_.satisfies_pzs_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Topic, _impl_.message_retention_duration_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Topic, _impl_.state_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Topic, _impl_.ingestion_data_source_settings_),
        0,
        8,
        2,
        1,
        3,
        6,
        4,
        7,
        5,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PubsubMessage_AttributesEntry_DoNotUse, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PubsubMessage_AttributesEntry_DoNotUse, _impl_.key_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PubsubMessage_AttributesEntry_DoNotUse, _impl_.value_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PubsubMessage, _impl_._has_bits_),
        8, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PubsubMessage, _impl_.data_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PubsubMessage, _impl_.attributes_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PubsubMessage, _impl_.message_id_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PubsubMessage, _impl_.publish_time_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PubsubMessage, _impl_.ordering_key_),
        0,
        4,
        1,
        3,
        2,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::GetTopicRequest, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::GetTopicRequest, _impl_.topic_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::UpdateTopicRequest, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::UpdateTopicRequest, _impl_.topic_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::UpdateTopicRequest, _impl_.update_mask_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PublishRequest, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PublishRequest, _impl_.topic_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PublishRequest, _impl_.messages_),
        1,
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PublishResponse, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PublishResponse, _impl_.message_ids_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicsRequest, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicsRequest, _impl_.project_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicsRequest, _impl_.page_size_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicsRequest, _impl_.page_token_),
        0,
        2,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicsResponse, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicsResponse, _impl_.topics_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicsResponse, _impl_.next_page_token_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicSubscriptionsRequest, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicSubscriptionsRequest, _impl_.topic_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicSubscriptionsRequest, _impl_.page_size_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicSubscriptionsRequest, _impl_.page_token_),
        0,
        2,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicSubscriptionsResponse, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicSubscriptionsResponse, _impl_.subscriptions_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicSubscriptionsResponse, _impl_.next_page_token_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicSnapshotsRequest, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicSnapshotsRequest, _impl_.topic_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicSnapshotsRequest, _impl_.page_size_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicSnapshotsRequest, _impl_.page_token_),
        0,
        2,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicSnapshotsResponse, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicSnapshotsResponse, _impl_.snapshots_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicSnapshotsResponse, _impl_.next_page_token_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::DeleteTopicRequest, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::DeleteTopicRequest, _impl_.topic_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::DetachSubscriptionRequest, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::DetachSubscriptionRequest, _impl_.subscription_),
        0,
        0x000, // bitmap
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription_AnalyticsHubSubscriptionInfo, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription_AnalyticsHubSubscriptionInfo, _impl_.listing_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription_AnalyticsHubSubscriptionInfo, _impl_.subscription_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription_LabelsEntry_DoNotUse, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription_LabelsEntry_DoNotUse, _impl_.key_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription_LabelsEntry_DoNotUse, _impl_.value_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_._has_bits_),
        22, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.name_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.topic_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.push_config_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.bigquery_config_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.cloud_storage_config_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.ack_deadline_seconds_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.retain_acked_messages_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.message_retention_duration_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.labels_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.enable_message_ordering_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.expiration_policy_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.filter_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.dead_letter_policy_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.retry_policy_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.detached_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.enable_exactly_once_delivery_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.topic_message_retention_duration_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.state_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.analytics_hub_subscription_info_),
        0,
        1,
        3,
        9,
        10,
        12,
        13,
        4,
        18,
        14,
        5,
        2,
        6,
        7,
        15,
        16,
        8,
        17,
        11,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::RetryPolicy, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::RetryPolicy, _impl_.minimum_backoff_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::RetryPolicy, _impl_.maximum_backoff_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::DeadLetterPolicy, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::DeadLetterPolicy, _impl_.dead_letter_topic_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::DeadLetterPolicy, _impl_.max_delivery_attempts_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ExpirationPolicy, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ExpirationPolicy, _impl_.ttl_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PushConfig_OidcToken, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PushConfig_OidcToken, _impl_.service_account_email_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PushConfig_OidcToken, _impl_.audience_),
        0,
        1,
        0x000, // bitmap
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PushConfig_NoWrapper, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PushConfig_NoWrapper, _impl_.write_metadata_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PushConfig_AttributesEntry_DoNotUse, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PushConfig_AttributesEntry_DoNotUse, _impl_.key_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PushConfig_AttributesEntry_DoNotUse, _impl_.value_),
        0,
        1,
        0x085, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PushConfig, _impl_._has_bits_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PushConfig, _impl_._oneof_case_[0]),
        11, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PushConfig, _impl_.push_endpoint_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PushConfig, _impl_.attributes_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PushConfig, _impl_.authentication_method_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PushConfig, _impl_.wrapper_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PushConfig, _impl_.wrapper_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PushConfig, _impl_.authentication_method_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PushConfig, _impl_.wrapper_),
        0,
        1,
        ~0u,
        ~0u,
        ~0u,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::BigQueryConfig, _impl_._has_bits_),
        10, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::BigQueryConfig, _impl_.table_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::BigQueryConfig, _impl_.use_topic_schema_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::BigQueryConfig, _impl_.write_metadata_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::BigQueryConfig, _impl_.drop_unknown_fields_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::BigQueryConfig, _impl_.state_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::BigQueryConfig, _impl_.use_table_schema_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::BigQueryConfig, _impl_.service_account_email_),
        0,
        2,
        3,
        4,
        6,
        5,
        1,
        0x000, // bitmap
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CloudStorageConfig_AvroConfig, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CloudStorageConfig_AvroConfig, _impl_.write_metadata_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CloudStorageConfig_AvroConfig, _impl_.use_topic_schema_),
        0,
        1,
        0x085, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CloudStorageConfig, _impl_._has_bits_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CloudStorageConfig, _impl_._oneof_case_[0]),
        16, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CloudStorageConfig, _impl_.bucket_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CloudStorageConfig, _impl_.filename_prefix_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CloudStorageConfig, _impl_.filename_suffix_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CloudStorageConfig, _impl_.filename_datetime_format_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CloudStorageConfig, _impl_.output_format_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CloudStorageConfig, _impl_.output_format_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CloudStorageConfig, _impl_.max_duration_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CloudStorageConfig, _impl_.max_bytes_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CloudStorageConfig, _impl_.max_messages_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CloudStorageConfig, _impl_.state_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CloudStorageConfig, _impl_.service_account_email_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CloudStorageConfig, _impl_.output_format_),
        0,
        1,
        2,
        3,
        ~0u,
        ~0u,
        5,
        6,
        7,
        8,
        4,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ReceivedMessage, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ReceivedMessage, _impl_.ack_id_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ReceivedMessage, _impl_.message_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ReceivedMessage, _impl_.delivery_attempt_),
        0,
        1,
        2,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::GetSubscriptionRequest, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::GetSubscriptionRequest, _impl_.subscription_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::UpdateSubscriptionRequest, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::UpdateSubscriptionRequest, _impl_.subscription_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::UpdateSubscriptionRequest, _impl_.update_mask_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSubscriptionsRequest, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSubscriptionsRequest, _impl_.project_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSubscriptionsRequest, _impl_.page_size_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSubscriptionsRequest, _impl_.page_token_),
        0,
        2,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSubscriptionsResponse, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSubscriptionsResponse, _impl_.subscriptions_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSubscriptionsResponse, _impl_.next_page_token_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::DeleteSubscriptionRequest, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::DeleteSubscriptionRequest, _impl_.subscription_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ModifyPushConfigRequest, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ModifyPushConfigRequest, _impl_.subscription_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ModifyPushConfigRequest, _impl_.push_config_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PullRequest, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PullRequest, _impl_.subscription_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PullRequest, _impl_.return_immediately_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PullRequest, _impl_.max_messages_),
        0,
        1,
        2,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PullResponse, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PullResponse, _impl_.received_messages_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ModifyAckDeadlineRequest, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ModifyAckDeadlineRequest, _impl_.subscription_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ModifyAckDeadlineRequest, _impl_.ack_ids_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ModifyAckDeadlineRequest, _impl_.ack_deadline_seconds_),
        1,
        0,
        2,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::AcknowledgeRequest, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::AcknowledgeRequest, _impl_.subscription_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::AcknowledgeRequest, _impl_.ack_ids_),
        1,
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullRequest, _impl_._has_bits_),
        11, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullRequest, _impl_.subscription_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullRequest, _impl_.ack_ids_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullRequest, _impl_.modify_deadline_seconds_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullRequest, _impl_.modify_deadline_ack_ids_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullRequest, _impl_.stream_ack_deadline_seconds_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullRequest, _impl_.client_id_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullRequest, _impl_.max_outstanding_messages_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullRequest, _impl_.max_outstanding_bytes_),
        3,
        0,
        1,
        2,
        7,
        4,
        5,
        6,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation, _impl_._has_bits_),
        7, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation, _impl_.ack_ids_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation, _impl_.invalid_ack_ids_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation, _impl_.unordered_ack_ids_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation, _impl_.temporary_failed_ack_ids_),
        0,
        1,
        2,
        3,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_.ack_ids_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_.invalid_ack_ids_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_.temporary_failed_ack_ids_),
        0,
        1,
        2,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_SubscriptionProperties, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_SubscriptionProperties, _impl_.exactly_once_delivery_enabled_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_SubscriptionProperties, _impl_.message_ordering_enabled_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse, _impl_._has_bits_),
        7, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse, _impl_.received_messages_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse, _impl_.acknowledge_confirmation_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse, _impl_.modify_ack_deadline_confirmation_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse, _impl_.subscription_properties_),
        0,
        3,
        1,
        2,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CreateSnapshotRequest_LabelsEntry_DoNotUse, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CreateSnapshotRequest_LabelsEntry_DoNotUse, _impl_.key_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CreateSnapshotRequest_LabelsEntry_DoNotUse, _impl_.value_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CreateSnapshotRequest, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CreateSnapshotRequest, _impl_.name_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CreateSnapshotRequest, _impl_.subscription_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CreateSnapshotRequest, _impl_.labels_),
        0,
        1,
        2,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::UpdateSnapshotRequest, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::UpdateSnapshotRequest, _impl_.snapshot_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::UpdateSnapshotRequest, _impl_.update_mask_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Snapshot_LabelsEntry_DoNotUse, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Snapshot_LabelsEntry_DoNotUse, _impl_.key_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Snapshot_LabelsEntry_DoNotUse, _impl_.value_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Snapshot, _impl_._has_bits_),
        7, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Snapshot, _impl_.name_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Snapshot, _impl_.topic_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Snapshot, _impl_.expire_time_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Snapshot, _impl_.labels_),
        0,
        1,
        2,
        3,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::GetSnapshotRequest, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::GetSnapshotRequest, _impl_.snapshot_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSnapshotsRequest, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSnapshotsRequest, _impl_.project_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSnapshotsRequest, _impl_.page_size_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSnapshotsRequest, _impl_.page_token_),
        0,
        2,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSnapshotsResponse, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSnapshotsResponse, _impl_.snapshots_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSnapshotsResponse, _impl_.next_page_token_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::DeleteSnapshotRequest, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::DeleteSnapshotRequest, _impl_.snapshot_),
        0,
        0x085, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::SeekRequest, _impl_._has_bits_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::SeekRequest, _impl_._oneof_case_[0]),
        8, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::SeekRequest, _impl_.subscription_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::SeekRequest, _impl_.target_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::SeekRequest, _impl_.target_),
        PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::SeekRequest, _impl_.target_),
        0,
        ~0u,
        ~0u,
        0x000, // bitmap
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::google::pubsub::v1::MessageStoragePolicy)},
        {7, sizeof(::google::pubsub::v1::SchemaSettings)},
        {18, sizeof(::google::pubsub::v1::IngestionDataSourceSettings_AwsKinesis)},
        {31, sizeof(::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_TextFormat)},
        {36, sizeof(::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_AvroFormat)},
        {37, sizeof(::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat)},
        {38, sizeof(::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage)},
        {57, sizeof(::google::pubsub::v1::IngestionDataSourceSettings)},
        {68, sizeof(::google::pubsub::v1::PlatformLogsSettings)},
        {73, sizeof(::google::pubsub::v1::IngestionFailureEvent_ApiViolationReason)},
        {74, sizeof(::google::pubsub::v1::IngestionFailureEvent_AvroFailureReason)},
        {75, sizeof(::google::pubsub::v1::IngestionFailureEvent_CloudStorageFailure)},
        {90, sizeof(::google::pubsub::v1::IngestionFailureEvent)},
        {101, sizeof(::google::pubsub::v1::Topic_LabelsEntry_DoNotUse)},
        {108, sizeof(::google::pubsub::v1::Topic)},
        {129, sizeof(::google::pubsub::v1::PubsubMessage_AttributesEntry_DoNotUse)},
        {136, sizeof(::google::pubsub::v1::PubsubMessage)},
        {149, sizeof(::google::pubsub::v1::GetTopicRequest)},
        {154, sizeof(::google::pubsub::v1::UpdateTopicRequest)},
        {161, sizeof(::google::pubsub::v1::PublishRequest)},
        {168, sizeof(::google::pubsub::v1::PublishResponse)},
        {173, sizeof(::google::pubsub::v1::ListTopicsRequest)},
        {182, sizeof(::google::pubsub::v1::ListTopicsResponse)},
        {189, sizeof(::google::pubsub::v1::ListTopicSubscriptionsRequest)},
        {198, sizeof(::google::pubsub::v1::ListTopicSubscriptionsResponse)},
        {205, sizeof(::google::pubsub::v1::ListTopicSnapshotsRequest)},
        {214, sizeof(::google::pubsub::v1::ListTopicSnapshotsResponse)},
        {221, sizeof(::google::pubsub::v1::DeleteTopicRequest)},
        {226, sizeof(::google::pubsub::v1::DetachSubscriptionRequest)},
        {231, sizeof(::google::pubsub::v1::DetachSubscriptionResponse)},
        {232, sizeof(::google::pubsub::v1::Subscription_AnalyticsHubSubscriptionInfo)},
        {239, sizeof(::google::pubsub::v1::Subscription_LabelsEntry_DoNotUse)},
        {246, sizeof(::google::pubsub::v1::Subscription)},
        {287, sizeof(::google::pubsub::v1::RetryPolicy)},
        {294, sizeof(::google::pubsub::v1::DeadLetterPolicy)},
        {301, sizeof(::google::pubsub::v1::ExpirationPolicy)},
        {306, sizeof(::google::pubsub::v1::PushConfig_OidcToken)},
        {313, sizeof(::google::pubsub::v1::PushConfig_PubsubWrapper)},
        {314, sizeof(::google::pubsub::v1::PushConfig_NoWrapper)},
        {319, sizeof(::google::pubsub::v1::PushConfig_AttributesEntry_DoNotUse)},
        {326, sizeof(::google::pubsub::v1::PushConfig)},
        {342, sizeof(::google::pubsub::v1::BigQueryConfig)},
        {359, sizeof(::google::pubsub::v1::CloudStorageConfig_TextConfig)},
        {360, sizeof(::google::pubsub::v1::CloudStorageConfig_AvroConfig)},
        {367, sizeof(::google::pubsub::v1::CloudStorageConfig)},
        {394, sizeof(::google::pubsub::v1::ReceivedMessage)},
        {403, sizeof(::google::pubsub::v1::GetSubscriptionRequest)},
        {408, sizeof(::google::pubsub::v1::UpdateSubscriptionRequest)},
        {415, sizeof(::google::pubsub::v1::ListSubscriptionsRequest)},
        {424, sizeof(::google::pubsub::v1::ListSubscriptionsResponse)},
        {431, sizeof(::google::pubsub::v1::DeleteSubscriptionRequest)},
        {436, sizeof(::google::pubsub::v1::ModifyPushConfigRequest)},
        {443, sizeof(::google::pubsub::v1::PullRequest)},
        {452, sizeof(::google::pubsub::v1::PullResponse)},
        {457, sizeof(::google::pubsub::v1::ModifyAckDeadlineRequest)},
        {466, sizeof(::google::pubsub::v1::AcknowledgeRequest)},
        {473, sizeof(::google::pubsub::v1::StreamingPullRequest)},
        {492, sizeof(::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation)},
        {503, sizeof(::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation)},
        {512, sizeof(::google::pubsub::v1::StreamingPullResponse_SubscriptionProperties)},
        {519, sizeof(::google::pubsub::v1::StreamingPullResponse)},
        {530, sizeof(::google::pubsub::v1::CreateSnapshotRequest_LabelsEntry_DoNotUse)},
        {537, sizeof(::google::pubsub::v1::CreateSnapshotRequest)},
        {546, sizeof(::google::pubsub::v1::UpdateSnapshotRequest)},
        {553, sizeof(::google::pubsub::v1::Snapshot_LabelsEntry_DoNotUse)},
        {560, sizeof(::google::pubsub::v1::Snapshot)},
        {571, sizeof(::google::pubsub::v1::GetSnapshotRequest)},
        {576, sizeof(::google::pubsub::v1::ListSnapshotsRequest)},
        {585, sizeof(::google::pubsub::v1::ListSnapshotsResponse)},
        {592, sizeof(::google::pubsub::v1::DeleteSnapshotRequest)},
        {597, sizeof(::google::pubsub::v1::SeekRequest)},
        {608, sizeof(::google::pubsub::v1::SeekResponse)},
};
static const ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const
    file_message_globals[] = {
        &::google::pubsub::v1::MessageStoragePolicy_globals_,
        &::google::pubsub::v1::SchemaSettings_globals_,
        &::google::pubsub::v1::IngestionDataSourceSettings_AwsKinesis_globals_,
        &::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_TextFormat_globals_,
        &::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_AvroFormat_globals_,
        &::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat_globals_,
        &::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_globals_,
        &::google::pubsub::v1::IngestionDataSourceSettings_globals_,
        &::google::pubsub::v1::PlatformLogsSettings_globals_,
        &::google::pubsub::v1::IngestionFailureEvent_ApiViolationReason_globals_,
        &::google::pubsub::v1::IngestionFailureEvent_AvroFailureReason_globals_,
        &::google::pubsub::v1::IngestionFailureEvent_CloudStorageFailure_globals_,
        &::google::pubsub::v1::IngestionFailureEvent_globals_,
        &::google::pubsub::v1::Topic_LabelsEntry_DoNotUse_globals_,
        &::google::pubsub::v1::Topic_globals_,
        &::google::pubsub::v1::PubsubMessage_AttributesEntry_DoNotUse_globals_,
        &::google::pubsub::v1::PubsubMessage_globals_,
        &::google::pubsub::v1::GetTopicRequest_globals_,
        &::google::pubsub::v1::UpdateTopicRequest_globals_,
        &::google::pubsub::v1::PublishRequest_globals_,
        &::google::pubsub::v1::PublishResponse_globals_,
        &::google::pubsub::v1::ListTopicsRequest_globals_,
        &::google::pubsub::v1::ListTopicsResponse_globals_,
        &::google::pubsub::v1::ListTopicSubscriptionsRequest_globals_,
        &::google::pubsub::v1::ListTopicSubscriptionsResponse_globals_,
        &::google::pubsub::v1::ListTopicSnapshotsRequest_globals_,
        &::google::pubsub::v1::ListTopicSnapshotsResponse_globals_,
        &::google::pubsub::v1::DeleteTopicRequest_globals_,
        &::google::pubsub::v1::DetachSubscriptionRequest_globals_,
        &::google::pubsub::v1::DetachSubscriptionResponse_globals_,
        &::google::pubsub::v1::Subscription_AnalyticsHubSubscriptionInfo_globals_,
        &::google::pubsub::v1::Subscription_LabelsEntry_DoNotUse_globals_,
        &::google::pubsub::v1::Subscription_globals_,
        &::google::pubsub::v1::RetryPolicy_globals_,
        &::google::pubsub::v1::DeadLetterPolicy_globals_,
        &::google::pubsub::v1::ExpirationPolicy_globals_,
        &::google::pubsub::v1::PushConfig_OidcToken_globals_,
        &::google::pubsub::v1::PushConfig_PubsubWrapper_globals_,
        &::google::pubsub::v1::PushConfig_NoWrapper_globals_,
        &::google::pubsub::v1::PushConfig_AttributesEntry_DoNotUse_globals_,
        &::google::pubsub::v1::PushConfig_globals_,
        &::google::pubsub::v1::BigQueryConfig_globals_,
        &::google::pubsub::v1::CloudStorageConfig_TextConfig_globals_,
        &::google::pubsub::v1::CloudStorageConfig_AvroConfig_globals_,
        &::google::pubsub::v1::CloudStorageConfig_globals_,
        &::google::pubsub::v1::ReceivedMessage_globals_,
        &::google::pubsub::v1::GetSubscriptionRequest_globals_,
        &::google::pubsub::v1::UpdateSubscriptionRequest_globals_,
        &::google::pubsub::v1::ListSubscriptionsRequest_globals_,
        &::google::pubsub::v1::ListSubscriptionsResponse_globals_,
        &::google::pubsub::v1::DeleteSubscriptionRequest_globals_,
        &::google::pubsub::v1::ModifyPushConfigRequest_globals_,
        &::google::pubsub::v1::PullRequest_globals_,
        &::google::pubsub::v1::PullResponse_globals_,
        &::google::pubsub::v1::ModifyAckDeadlineRequest_globals_,
        &::google::pubsub::v1::AcknowledgeRequest_globals_,
        &::google::pubsub::v1::StreamingPullRequest_globals_,
        &::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation_globals_,
        &::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation_globals_,
        &::google::pubsub::v1::StreamingPullResponse_SubscriptionProperties_globals_,
        &::google::pubsub::v1::StreamingPullResponse_globals_,
        &::google::pubsub::v1::CreateSnapshotRequest_LabelsEntry_DoNotUse_globals_,
        &::google::pubsub::v1::CreateSnapshotRequest_globals_,
        &::google::pubsub::v1::UpdateSnapshotRequest_globals_,
        &::google::pubsub::v1::Snapshot_LabelsEntry_DoNotUse_globals_,
        &::google::pubsub::v1::Snapshot_globals_,
        &::google::pubsub::v1::GetSnapshotRequest_globals_,
        &::google::pubsub::v1::ListSnapshotsRequest_globals_,
        &::google::pubsub::v1::ListSnapshotsResponse_globals_,
        &::google::pubsub::v1::DeleteSnapshotRequest_globals_,
        &::google::pubsub::v1::SeekRequest_globals_,
        &::google::pubsub::v1::SeekResponse_globals_,
};
const char descriptor_table_protodef_google_2fpubsub_2fv1_2fpubsub_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n\035google/pubsub/v1/pubsub.proto\022\020google."
    "pubsub.v1\032\034google/api/annotations.proto\032"
    "\027google/api/client.proto\032\037google/api/fie"
    "ld_behavior.proto\032\031google/api/resource.p"
    "roto\032\036google/protobuf/duration.proto\032\033go"
    "ogle/protobuf/empty.proto\032 google/protob"
    "uf/field_mask.proto\032\037google/protobuf/tim"
    "estamp.proto\032\035google/pubsub/v1/schema.pr"
    "oto\"c\n\024MessageStoragePolicy\022)\n\033allowed_p"
    "ersistence_regions\030\001 \003(\tB\004\342A\001\001\022 \n\022enforc"
    "e_in_transit\030\002 \001(\010B\004\342A\001\001\"\274\001\n\016SchemaSetti"
    "ngs\0225\n\006schema\030\001 \001(\tB%\342A\001\002\372A\036\n\034pubsub.goo"
    "gleapis.com/Schema\0222\n\010encoding\030\002 \001(\0162\032.g"
    "oogle.pubsub.v1.EncodingB\004\342A\001\001\022\037\n\021first_"
    "revision_id\030\003 \001(\tB\004\342A\001\001\022\036\n\020last_revision"
    "_id\030\004 \001(\tB\004\342A\001\001\"\271\013\n\033IngestionDataSourceS"
    "ettings\022U\n\013aws_kinesis\030\001 \001(\01328.google.pu"
    "bsub.v1.IngestionDataSourceSettings.AwsK"
    "inesisB\004\342A\001\001H\000\022Y\n\rcloud_storage\030\002 \001(\0132:."
    "google.pubsub.v1.IngestionDataSourceSett"
    "ings.CloudStorageB\004\342A\001\001H\000\022L\n\026platform_lo"
    "gs_settings\030\004 \001(\0132&.google.pubsub.v1.Pla"
    "tformLogsSettingsB\004\342A\001\001\032\357\002\n\nAwsKinesis\022S"
    "\n\005state\030\001 \001(\0162>.google.pubsub.v1.Ingesti"
    "onDataSourceSettings.AwsKinesis.StateB\004\342"
    "A\001\003\022\030\n\nstream_arn\030\002 \001(\tB\004\342A\001\002\022\032\n\014consume"
    "r_arn\030\003 \001(\tB\004\342A\001\002\022\032\n\014aws_role_arn\030\004 \001(\tB"
    "\004\342A\001\002\022!\n\023gcp_service_account\030\005 \001(\tB\004\342A\001\002"
    "\"\226\001\n\005State\022\025\n\021STATE_UNSPECIFIED\020\000\022\n\n\006ACT"
    "IVE\020\001\022\035\n\031KINESIS_PERMISSION_DENIED\020\002\022\035\n\031"
    "PUBLISH_PERMISSION_DENIED\020\003\022\024\n\020STREAM_NO"
    "T_FOUND\020\004\022\026\n\022CONSUMER_NOT_FOUND\020\005\032\235\006\n\014Cl"
    "oudStorage\022U\n\005state\030\001 \001(\0162@.google.pubsu"
    "b.v1.IngestionDataSourceSettings.CloudSt"
    "orage.StateB\004\342A\001\003\022\024\n\006bucket\030\002 \001(\tB\004\342A\001\001\022"
    "b\n\013text_format\030\003 \001(\0132E.google.pubsub.v1."
    "IngestionDataSourceSettings.CloudStorage"
    ".TextFormatB\004\342A\001\001H\000\022b\n\013avro_format\030\004 \001(\013"
    "2E.google.pubsub.v1.IngestionDataSourceS"
    "ettings.CloudStorage.AvroFormatB\004\342A\001\001H\000\022"
    "o\n\022pubsub_avro_format\030\005 \001(\0132K.google.pub"
    "sub.v1.IngestionDataSourceSettings.Cloud"
    "Storage.PubSubAvroFormatB\004\342A\001\001H\000\022D\n\032mini"
    "mum_object_create_time\030\006 \001(\0132\032.google.pr"
    "otobuf.TimestampB\004\342A\001\001\022\030\n\nmatch_glob\030\t \001"
    "(\tB\004\342A\001\001\0328\n\nTextFormat\022\034\n\tdelimiter\030\001 \001("
    "\tB\004\342A\001\001H\000\210\001\001B\014\n\n_delimiter\032\014\n\nAvroFormat"
    "\032\022\n\020PubSubAvroFormat\"\232\001\n\005State\022\025\n\021STATE_"
    "UNSPECIFIED\020\000\022\n\n\006ACTIVE\020\001\022#\n\037CLOUD_STORA"
    "GE_PERMISSION_DENIED\020\002\022\035\n\031PUBLISH_PERMIS"
    "SION_DENIED\020\003\022\024\n\020BUCKET_NOT_FOUND\020\004\022\024\n\020T"
    "OO_MANY_OBJECTS\020\005B\016\n\014input_formatB\010\n\006sou"
    "rce\"\300\001\n\024PlatformLogsSettings\022G\n\010severity"
    "\030\001 \001(\0162/.google.pubsub.v1.PlatformLogsSe"
    "ttings.SeverityB\004\342A\001\001\"_\n\010Severity\022\030\n\024SEV"
    "ERITY_UNSPECIFIED\020\000\022\014\n\010DISABLED\020\001\022\t\n\005DEB"
    "UG\020\002\022\010\n\004INFO\020\003\022\013\n\007WARNING\020\004\022\t\n\005ERROR\020\005\"\231"
    "\004\n\025IngestionFailureEvent\022\023\n\005topic\030\001 \001(\tB"
    "\004\342A\001\002\022\033\n\rerror_message\030\002 \001(\tB\004\342A\001\002\022b\n\025cl"
    "oud_storage_failure\030\003 \001(\0132;.google.pubsu"
    "b.v1.IngestionFailureEvent.CloudStorageF"
    "ailureB\004\342A\001\001H\000\032\024\n\022ApiViolationReason\032\023\n\021"
    "AvroFailureReason\032\263\002\n\023CloudStorageFailur"
    "e\022\024\n\006bucket\030\001 \001(\tB\004\342A\001\001\022\031\n\013object_name\030\002"
    " \001(\tB\004\342A\001\001\022\037\n\021object_generation\030\003 \001(\003B\004\342"
    "A\001\001\022^\n\023avro_failure_reason\030\005 \001(\01329.googl"
    "e.pubsub.v1.IngestionFailureEvent.AvroFa"
    "ilureReasonB\004\342A\001\001H\000\022`\n\024api_violation_rea"
    "son\030\006 \001(\0132:.google.pubsub.v1.IngestionFa"
    "ilureEvent.ApiViolationReasonB\004\342A\001\001H\000B\010\n"
    "\006reasonB\t\n\007failure\"\303\005\n\005Topic\022\022\n\004name\030\001 \001"
    "(\tB\004\342A\001\002\0229\n\006labels\030\002 \003(\0132#.google.pubsub"
    ".v1.Topic.LabelsEntryB\004\342A\001\001\022L\n\026message_s"
    "torage_policy\030\003 \001(\0132&.google.pubsub.v1.M"
    "essageStoragePolicyB\004\342A\001\001\022\032\n\014kms_key_nam"
    "e\030\005 \001(\tB\004\342A\001\001\022\?\n\017schema_settings\030\006 \001(\0132 "
    ".google.pubsub.v1.SchemaSettingsB\004\342A\001\001\022\033"
    "\n\rsatisfies_pzs\030\007 \001(\010B\004\342A\001\001\022C\n\032message_r"
    "etention_duration\030\010 \001(\0132\031.google.protobu"
    "f.DurationB\004\342A\001\001\0222\n\005state\030\t \001(\0162\035.google"
    ".pubsub.v1.Topic.StateB\004\342A\001\003\022[\n\036ingestio"
    "n_data_source_settings\030\n \001(\0132-.google.pu"
    "bsub.v1.IngestionDataSourceSettingsB\004\342A\001"
    "\001\032-\n\013LabelsEntry\022\013\n\003key\030\001 \001(\t\022\r\n\005value\030\002"
    " \001(\t:\0028\001\"H\n\005State\022\025\n\021STATE_UNSPECIFIED\020\000"
    "\022\n\n\006ACTIVE\020\001\022\034\n\030INGESTION_RESOURCE_ERROR"
    "\020\002:T\352AQ\n\033pubsub.googleapis.com/Topic\022!pr"
    "ojects/{project}/topics/{topic}\022\017_delete"
    "d-topic_\"\203\002\n\rPubsubMessage\022\022\n\004data\030\001 \001(\014"
    "B\004\342A\001\001\022I\n\nattributes\030\002 \003(\0132/.google.pubs"
    "ub.v1.PubsubMessage.AttributesEntryB\004\342A\001"
    "\001\022\022\n\nmessage_id\030\003 \001(\t\0220\n\014publish_time\030\004 "
    "\001(\0132\032.google.protobuf.Timestamp\022\032\n\014order"
    "ing_key\030\005 \001(\tB\004\342A\001\001\0321\n\017AttributesEntry\022\013"
    "\n\003key\030\001 \001(\t\022\r\n\005value\030\002 \001(\t:\0028\001\"F\n\017GetTop"
    "icRequest\0223\n\005topic\030\001 \001(\tB$\342A\001\002\372A\035\n\033pubsu"
    "b.googleapis.com/Topic\"y\n\022UpdateTopicReq"
    "uest\022,\n\005topic\030\001 \001(\0132\027.google.pubsub.v1.T"
    "opicB\004\342A\001\002\0225\n\013update_mask\030\002 \001(\0132\032.google"
    ".protobuf.FieldMaskB\004\342A\001\002\"~\n\016PublishRequ"
    "est\0223\n\005topic\030\001 \001(\tB$\342A\001\002\372A\035\n\033pubsub.goog"
    "leapis.com/Topic\0227\n\010messages\030\002 \003(\0132\037.goo"
    "gle.pubsub.v1.PubsubMessageB\004\342A\001\002\",\n\017Pub"
    "lishResponse\022\031\n\013message_ids\030\001 \003(\tB\004\342A\001\001\""
    "\215\001\n\021ListTopicsRequest\022E\n\007project\030\001 \001(\tB4"
    "\342A\001\002\372A-\n+cloudresourcemanager.googleapis"
    ".com/Project\022\027\n\tpage_size\030\002 \001(\005B\004\342A\001\001\022\030\n"
    "\npage_token\030\003 \001(\tB\004\342A\001\001\"b\n\022ListTopicsRes"
    "ponse\022-\n\006topics\030\001 \003(\0132\027.google.pubsub.v1"
    ".TopicB\004\342A\001\001\022\035\n\017next_page_token\030\002 \001(\tB\004\342"
    "A\001\001\"\207\001\n\035ListTopicSubscriptionsRequest\0223\n"
    "\005topic\030\001 \001(\tB$\342A\001\002\372A\035\n\033pubsub.googleapis"
    ".com/Topic\022\027\n\tpage_size\030\002 \001(\005B\004\342A\001\001\022\030\n\np"
    "age_token\030\003 \001(\tB\004\342A\001\001\"\203\001\n\036ListTopicSubsc"
    "riptionsResponse\022B\n\rsubscriptions\030\001 \003(\tB"
    "+\342A\001\001\372A$\n\"pubsub.googleapis.com/Subscrip"
    "tion\022\035\n\017next_page_token\030\002 \001(\tB\004\342A\001\001\"\203\001\n\031"
    "ListTopicSnapshotsRequest\0223\n\005topic\030\001 \001(\t"
    "B$\342A\001\002\372A\035\n\033pubsub.googleapis.com/Topic\022\027"
    "\n\tpage_size\030\002 \001(\005B\004\342A\001\001\022\030\n\npage_token\030\003 "
    "\001(\tB\004\342A\001\001\"T\n\032ListTopicSnapshotsResponse\022"
    "\027\n\tsnapshots\030\001 \003(\tB\004\342A\001\001\022\035\n\017next_page_to"
    "ken\030\002 \001(\tB\004\342A\001\001\"I\n\022DeleteTopicRequest\0223\n"
    "\005topic\030\001 \001(\tB$\342A\001\002\372A\035\n\033pubsub.googleapis"
    ".com/Topic\"^\n\031DetachSubscriptionRequest\022"
    "A\n\014subscription\030\001 \001(\tB+\342A\001\002\372A$\n\"pubsub.g"
    "oogleapis.com/Subscription\"\034\n\032DetachSubs"
    "criptionResponse\"\300\n\n\014Subscription\022\022\n\004nam"
    "e\030\001 \001(\tB\004\342A\001\002\0223\n\005topic\030\002 \001(\tB$\342A\001\002\372A\035\n\033p"
    "ubsub.googleapis.com/Topic\0227\n\013push_confi"
    "g\030\004 \001(\0132\034.google.pubsub.v1.PushConfigB\004\342"
    "A\001\001\022\?\n\017bigquery_config\030\022 \001(\0132 .google.pu"
    "bsub.v1.BigQueryConfigB\004\342A\001\001\022H\n\024cloud_st"
    "orage_config\030\026 \001(\0132$.google.pubsub.v1.Cl"
    "oudStorageConfigB\004\342A\001\001\022\"\n\024ack_deadline_s"
    "econds\030\005 \001(\005B\004\342A\001\001\022#\n\025retain_acked_messa"
    "ges\030\007 \001(\010B\004\342A\001\001\022C\n\032message_retention_dur"
    "ation\030\010 \001(\0132\031.google.protobuf.DurationB\004"
    "\342A\001\001\022@\n\006labels\030\t \003(\0132*.google.pubsub.v1."
    "Subscription.LabelsEntryB\004\342A\001\001\022%\n\027enable"
    "_message_ordering\030\n \001(\010B\004\342A\001\001\022C\n\021expirat"
    "ion_policy\030\013 \001(\0132\".google.pubsub.v1.Expi"
    "rationPolicyB\004\342A\001\001\022\024\n\006filter\030\014 \001(\tB\004\342A\001\001"
    "\022D\n\022dead_letter_policy\030\r \001(\0132\".google.pu"
    "bsub.v1.DeadLetterPolicyB\004\342A\001\001\0229\n\014retry_"
    "policy\030\016 \001(\0132\035.google.pubsub.v1.RetryPol"
    "icyB\004\342A\001\001\022\026\n\010detached\030\017 \001(\010B\004\342A\001\001\022*\n\034ena"
    "ble_exactly_once_delivery\030\020 \001(\010B\004\342A\001\001\022I\n"
    " topic_message_retention_duration\030\021 \001(\0132"
    "\031.google.protobuf.DurationB\004\342A\001\003\0229\n\005stat"
    "e\030\023 \001(\0162$.google.pubsub.v1.Subscription."
    "StateB\004\342A\001\003\022j\n\037analytics_hub_subscriptio"
    "n_info\030\027 \001(\0132;.google.pubsub.v1.Subscrip"
    "tion.AnalyticsHubSubscriptionInfoB\004\342A\001\003\032"
    "Q\n\034AnalyticsHubSubscriptionInfo\022\025\n\007listi"
    "ng\030\001 \001(\tB\004\342A\001\001\022\032\n\014subscription\030\002 \001(\tB\004\342A"
    "\001\001\032-\n\013LabelsEntry\022\013\n\003key\030\001 \001(\t\022\r\n\005value\030"
    "\002 \001(\t:\0028\001\">\n\005State\022\025\n\021STATE_UNSPECIFIED\020"
    "\000\022\n\n\006ACTIVE\020\001\022\022\n\016RESOURCE_ERROR\020\002:X\352AU\n\""
    "pubsub.googleapis.com/Subscription\022/proj"
    "ects/{project}/subscriptions/{subscripti"
    "on}\"\201\001\n\013RetryPolicy\0228\n\017minimum_backoff\030\001"
    " \001(\0132\031.google.protobuf.DurationB\004\342A\001\001\0228\n"
    "\017maximum_backoff\030\002 \001(\0132\031.google.protobuf"
    ".DurationB\004\342A\001\001\"X\n\020DeadLetterPolicy\022\037\n\021d"
    "ead_letter_topic\030\001 \001(\tB\004\342A\001\001\022#\n\025max_deli"
    "very_attempts\030\002 \001(\005B\004\342A\001\001\"@\n\020ExpirationP"
    "olicy\022,\n\003ttl\030\001 \001(\0132\031.google.protobuf.Dur"
    "ationB\004\342A\001\001\"\242\004\n\nPushConfig\022\033\n\rpush_endpo"
    "int\030\001 \001(\tB\004\342A\001\001\022F\n\nattributes\030\002 \003(\0132,.go"
    "ogle.pubsub.v1.PushConfig.AttributesEntr"
    "yB\004\342A\001\001\022B\n\noidc_token\030\003 \001(\0132&.google.pub"
    "sub.v1.PushConfig.OidcTokenB\004\342A\001\001H\000\022J\n\016p"
    "ubsub_wrapper\030\004 \001(\0132*.google.pubsub.v1.P"
    "ushConfig.PubsubWrapperB\004\342A\001\001H\001\022B\n\nno_wr"
    "apper\030\005 \001(\0132&.google.pubsub.v1.PushConfi"
    "g.NoWrapperB\004\342A\001\001H\001\032H\n\tOidcToken\022#\n\025serv"
    "ice_account_email\030\001 \001(\tB\004\342A\001\001\022\026\n\010audienc"
    "e\030\002 \001(\tB\004\342A\001\001\032\017\n\rPubsubWrapper\032)\n\tNoWrap"
    "per\022\034\n\016write_metadata\030\001 \001(\010B\004\342A\001\001\0321\n\017Att"
    "ributesEntry\022\013\n\003key\030\001 \001(\t\022\r\n\005value\030\002 \001(\t"
    ":\0028\001B\027\n\025authentication_methodB\t\n\007wrapper"
    "\"\225\003\n\016BigQueryConfig\022\023\n\005table\030\001 \001(\tB\004\342A\001\001"
    "\022\036\n\020use_topic_schema\030\002 \001(\010B\004\342A\001\001\022\034\n\016writ"
    "e_metadata\030\003 \001(\010B\004\342A\001\001\022!\n\023drop_unknown_f"
    "ields\030\004 \001(\010B\004\342A\001\001\022;\n\005state\030\005 \001(\0162&.googl"
    "e.pubsub.v1.BigQueryConfig.StateB\004\342A\001\003\022\036"
    "\n\020use_table_schema\030\006 \001(\010B\004\342A\001\001\022#\n\025servic"
    "e_account_email\030\007 \001(\tB\004\342A\001\001\"\212\001\n\005State\022\025\n"
    "\021STATE_UNSPECIFIED\020\000\022\n\n\006ACTIVE\020\001\022\025\n\021PERM"
    "ISSION_DENIED\020\002\022\r\n\tNOT_FOUND\020\003\022\023\n\017SCHEMA"
    "_MISMATCH\020\004\022#\n\037IN_TRANSIT_LOCATION_RESTR"
    "ICTION\020\005\"\366\005\n\022CloudStorageConfig\022\024\n\006bucke"
    "t\030\001 \001(\tB\004\342A\001\002\022\035\n\017filename_prefix\030\002 \001(\tB\004"
    "\342A\001\001\022\035\n\017filename_suffix\030\003 \001(\tB\004\342A\001\001\022&\n\030f"
    "ilename_datetime_format\030\n \001(\tB\004\342A\001\001\022L\n\013t"
    "ext_config\030\004 \001(\0132/.google.pubsub.v1.Clou"
    "dStorageConfig.TextConfigB\004\342A\001\001H\000\022L\n\013avr"
    "o_config\030\005 \001(\0132/.google.pubsub.v1.CloudS"
    "torageConfig.AvroConfigB\004\342A\001\001H\000\0225\n\014max_d"
    "uration\030\006 \001(\0132\031.google.protobuf.Duration"
    "B\004\342A\001\001\022\027\n\tmax_bytes\030\007 \001(\003B\004\342A\001\001\022\032\n\014max_m"
    "essages\030\010 \001(\003B\004\342A\001\001\022\?\n\005state\030\t \001(\0162*.goo"
    "gle.pubsub.v1.CloudStorageConfig.StateB\004"
    "\342A\001\003\022#\n\025service_account_email\030\013 \001(\tB\004\342A\001"
    "\001\032\014\n\nTextConfig\032J\n\nAvroConfig\022\034\n\016write_m"
    "etadata\030\001 \001(\010B\004\342A\001\001\022\036\n\020use_topic_schema\030"
    "\002 \001(\010B\004\342A\001\001\"\212\001\n\005State\022\025\n\021STATE_UNSPECIFI"
    "ED\020\000\022\n\n\006ACTIVE\020\001\022\025\n\021PERMISSION_DENIED\020\002\022"
    "\r\n\tNOT_FOUND\020\003\022#\n\037IN_TRANSIT_LOCATION_RE"
    "STRICTION\020\004\022\023\n\017SCHEMA_MISMATCH\020\005B\017\n\routp"
    "ut_format\"\177\n\017ReceivedMessage\022\024\n\006ack_id\030\001"
    " \001(\tB\004\342A\001\001\0226\n\007message\030\002 \001(\0132\037.google.pub"
    "sub.v1.PubsubMessageB\004\342A\001\001\022\036\n\020delivery_a"
    "ttempt\030\003 \001(\005B\004\342A\001\001\"[\n\026GetSubscriptionReq"
    "uest\022A\n\014subscription\030\001 \001(\tB+\342A\001\002\372A$\n\"pub"
    "sub.googleapis.com/Subscription\"\216\001\n\031Upda"
    "teSubscriptionRequest\022:\n\014subscription\030\001 "
    "\001(\0132\036.google.pubsub.v1.SubscriptionB\004\342A\001"
    "\002\0225\n\013update_mask\030\002 \001(\0132\032.google.protobuf"
    ".FieldMaskB\004\342A\001\002\"\224\001\n\030ListSubscriptionsRe"
    "quest\022E\n\007project\030\001 \001(\tB4\342A\001\002\372A-\n+cloudre"
    "sourcemanager.googleapis.com/Project\022\027\n\t"
    "page_size\030\002 \001(\005B\004\342A\001\001\022\030\n\npage_token\030\003 \001("
    "\tB\004\342A\001\001\"w\n\031ListSubscriptionsResponse\022;\n\r"
    "subscriptions\030\001 \003(\0132\036.google.pubsub.v1.S"
    "ubscriptionB\004\342A\001\001\022\035\n\017next_page_token\030\002 \001"
    "(\tB\004\342A\001\001\"^\n\031DeleteSubscriptionRequest\022A\n"
    "\014subscription\030\001 \001(\tB+\342A\001\002\372A$\n\"pubsub.goo"
    "gleapis.com/Subscription\"\225\001\n\027ModifyPushC"
    "onfigRequest\022A\n\014subscription\030\001 \001(\tB+\342A\001\002"
    "\372A$\n\"pubsub.googleapis.com/Subscription\022"
    "7\n\013push_config\030\002 \001(\0132\034.google.pubsub.v1."
    "PushConfigB\004\342A\001\002\"\220\001\n\013PullRequest\022A\n\014subs"
    "cription\030\001 \001(\tB+\342A\001\002\372A$\n\"pubsub.googleap"
    "is.com/Subscription\022\"\n\022return_immediatel"
    "y\030\002 \001(\010B\006\030\001\342A\001\001\022\032\n\014max_messages\030\003 \001(\005B\004\342"
    "A\001\002\"R\n\014PullResponse\022B\n\021received_messages"
    "\030\001 \003(\0132!.google.pubsub.v1.ReceivedMessag"
    "eB\004\342A\001\001\"\230\001\n\030ModifyAckDeadlineRequest\022A\n\014"
    "subscription\030\001 \001(\tB+\342A\001\002\372A$\n\"pubsub.goog"
    "leapis.com/Subscription\022\025\n\007ack_ids\030\004 \003(\t"
    "B\004\342A\001\002\022\"\n\024ack_deadline_seconds\030\003 \001(\005B\004\342A"
    "\001\002\"n\n\022AcknowledgeRequest\022A\n\014subscription"
    "\030\001 \001(\tB+\342A\001\002\372A$\n\"pubsub.googleapis.com/S"
    "ubscription\022\025\n\007ack_ids\030\002 \003(\tB\004\342A\001\002\"\317\002\n\024S"
    "treamingPullRequest\022A\n\014subscription\030\001 \001("
    "\tB+\342A\001\002\372A$\n\"pubsub.googleapis.com/Subscr"
    "iption\022\025\n\007ack_ids\030\002 \003(\tB\004\342A\001\001\022%\n\027modify_"
    "deadline_seconds\030\003 \003(\005B\004\342A\001\001\022%\n\027modify_d"
    "eadline_ack_ids\030\004 \003(\tB\004\342A\001\001\022)\n\033stream_ac"
    "k_deadline_seconds\030\005 \001(\005B\004\342A\001\002\022\027\n\tclient"
    "_id\030\006 \001(\tB\004\342A\001\001\022&\n\030max_outstanding_messa"
    "ges\030\007 \001(\003B\004\342A\001\001\022#\n\025max_outstanding_bytes"
    "\030\010 \001(\003B\004\342A\001\001\"\253\006\n\025StreamingPullResponse\022B"
    "\n\021received_messages\030\001 \003(\0132!.google.pubsu"
    "b.v1.ReceivedMessageB\004\342A\001\001\022g\n\030acknowledg"
    "e_confirmation\030\005 \001(\0132\?.google.pubsub.v1."
    "StreamingPullResponse.AcknowledgeConfirm"
    "ationB\004\342A\001\001\022u\n modify_ack_deadline_confi"
    "rmation\030\003 \001(\0132E.google.pubsub.v1.Streami"
    "ngPullResponse.ModifyAckDeadlineConfirma"
    "tionB\004\342A\001\001\022e\n\027subscription_properties\030\004 "
    "\001(\0132>.google.pubsub.v1.StreamingPullResp"
    "onse.SubscriptionPropertiesB\004\342A\001\001\032\230\001\n\027Ac"
    "knowledgeConfirmation\022\025\n\007ack_ids\030\001 \003(\tB\004"
    "\342A\001\001\022\035\n\017invalid_ack_ids\030\002 \003(\tB\004\342A\001\001\022\037\n\021u"
    "nordered_ack_ids\030\003 \003(\tB\004\342A\001\001\022&\n\030temporar"
    "y_failed_ack_ids\030\004 \003(\tB\004\342A\001\001\032}\n\035ModifyAc"
    "kDeadlineConfirmation\022\025\n\007ack_ids\030\001 \003(\tB\004"
    "\342A\001\001\022\035\n\017invalid_ack_ids\030\002 \003(\tB\004\342A\001\001\022&\n\030t"
    "emporary_failed_ack_ids\030\003 \003(\tB\004\342A\001\001\032m\n\026S"
    "ubscriptionProperties\022+\n\035exactly_once_de"
    "livery_enabled\030\001 \001(\010B\004\342A\001\001\022&\n\030message_or"
    "dering_enabled\030\002 \001(\010B\004\342A\001\001\"\213\002\n\025CreateSna"
    "pshotRequest\0225\n\004name\030\001 \001(\tB\'\342A\001\002\372A \n\036pub"
    "sub.googleapis.com/Snapshot\022A\n\014subscript"
    "ion\030\002 \001(\tB+\342A\001\002\372A$\n\"pubsub.googleapis.co"
    "m/Subscription\022I\n\006labels\030\003 \003(\01323.google."
    "pubsub.v1.CreateSnapshotRequest.LabelsEn"
    "tryB\004\342A\001\001\032-\n\013LabelsEntry\022\013\n\003key\030\001 \001(\t\022\r\n"
    "\005value\030\002 \001(\t:\0028\001\"\202\001\n\025UpdateSnapshotReque"
    "st\0222\n\010snapshot\030\001 \001(\0132\032.google.pubsub.v1."
    "SnapshotB\004\342A\001\002\0225\n\013update_mask\030\002 \001(\0132\032.go"
    "ogle.protobuf.FieldMaskB\004\342A\001\002\"\305\002\n\010Snapsh"
    "ot\022\022\n\004name\030\001 \001(\tB\004\342A\001\001\0223\n\005topic\030\002 \001(\tB$\342"
    "A\001\001\372A\035\n\033pubsub.googleapis.com/Topic\0225\n\013e"
    "xpire_time\030\003 \001(\0132\032.google.protobuf.Times"
    "tampB\004\342A\001\001\022<\n\006labels\030\004 \003(\0132&.google.pubs"
    "ub.v1.Snapshot.LabelsEntryB\004\342A\001\001\032-\n\013Labe"
    "lsEntry\022\013\n\003key\030\001 \001(\t\022\r\n\005value\030\002 \001(\t:\0028\001:"
    "L\352AI\n\036pubsub.googleapis.com/Snapshot\022\'pr"
    "ojects/{project}/snapshots/{snapshot}\"O\n"
    "\022GetSnapshotRequest\0229\n\010snapshot\030\001 \001(\tB\'\342"
    "A\001\002\372A \n\036pubsub.googleapis.com/Snapshot\"\220"
    "\001\n\024ListSnapshotsRequest\022E\n\007project\030\001 \001(\t"
    "B4\342A\001\002\372A-\n+cloudresourcemanager.googleap"
    "is.com/Project\022\027\n\tpage_size\030\002 \001(\005B\004\342A\001\001\022"
    "\030\n\npage_token\030\003 \001(\tB\004\342A\001\001\"k\n\025ListSnapsho"
    "tsResponse\0223\n\tsnapshots\030\001 \003(\0132\032.google.p"
    "ubsub.v1.SnapshotB\004\342A\001\001\022\035\n\017next_page_tok"
    "en\030\002 \001(\tB\004\342A\001\001\"R\n\025DeleteSnapshotRequest\022"
    "9\n\010snapshot\030\001 \001(\tB\'\342A\001\002\372A \n\036pubsub.googl"
    "eapis.com/Snapshot\"\311\001\n\013SeekRequest\022A\n\014su"
    "bscription\030\001 \001(\tB+\342A\001\002\372A$\n\"pubsub.google"
    "apis.com/Subscription\0220\n\004time\030\002 \001(\0132\032.go"
    "ogle.protobuf.TimestampB\004\342A\001\001H\000\022;\n\010snaps"
    "hot\030\003 \001(\tB\'\342A\001\001\372A \n\036pubsub.googleapis.co"
    "m/SnapshotH\000B\010\n\006target\"\016\n\014SeekResponse2\270"
    "\013\n\tPublisher\022q\n\013CreateTopic\022\027.google.pub"
    "sub.v1.Topic\032\027.google.pubsub.v1.Topic\"0\332"
    "A\004name\202\323\344\223\002#\032\036/v1/{name=projects/*/topic"
    "s/*}:\001*\022\221\001\n\013UpdateTopic\022$.google.pubsub."
    "v1.UpdateTopicRequest\032\027.google.pubsub.v1"
    ".Topic\"C\332A\021topic,update_mask\202\323\344\223\002)2$/v1/"
    "{topic.name=projects/*/topics/*}:\001*\022\223\001\n\007"
    "Publish\022 .google.pubsub.v1.PublishReques"
    "t\032!.google.pubsub.v1.PublishResponse\"C\332A"
    "\016topic,messages\202\323\344\223\002,\"\'/v1/{topic=projec"
    "ts/*/topics/*}:publish:\001*\022w\n\010GetTopic\022!."
    "google.pubsub.v1.GetTopicRequest\032\027.googl"
    "e.pubsub.v1.Topic\"/\332A\005topic\202\323\344\223\002!\022\037/v1/{"
    "topic=projects/*/topics/*}\022\212\001\n\nListTopic"
    "s\022#.google.pubsub.v1.ListTopicsRequest\032$"
    ".google.pubsub.v1.ListTopicsResponse\"1\332A"
    "\007project\202\323\344\223\002!\022\037/v1/{project=projects/*}"
    "/topics\022\272\001\n\026ListTopicSubscriptions\022/.goo"
    "gle.pubsub.v1.ListTopicSubscriptionsRequ"
    "est\0320.google.pubsub.v1.ListTopicSubscrip"
    "tionsResponse\"=\332A\005topic\202\323\344\223\002/\022-/v1/{topi"
    "c=projects/*/topics/*}/subscriptions\022\252\001\n"
    "\022ListTopicSnapshots\022+.google.pubsub.v1.L"
    "istTopicSnapshotsRequest\032,.google.pubsub"
    ".v1.ListTopicSnapshotsResponse\"9\332A\005topic"
    "\202\323\344\223\002+\022)/v1/{topic=projects/*/topics/*}/"
    "snapshots\022|\n\013DeleteTopic\022$.google.pubsub"
    ".v1.DeleteTopicRequest\032\026.google.protobuf"
    ".Empty\"/\332A\005topic\202\323\344\223\002!*\037/v1/{topic=proje"
    "cts/*/topics/*}\022\255\001\n\022DetachSubscription\022+"
    ".google.pubsub.v1.DetachSubscriptionRequ"
    "est\032,.google.pubsub.v1.DetachSubscriptio"
    "nResponse\"<\202\323\344\223\0026\"4/v1/{subscription=pro"
    "jects/*/subscriptions/*}:detach\032p\312A\025pubs"
    "ub.googleapis.com\322AUhttps://www.googleap"
    "is.com/auth/cloud-platform,https://www.g"
    "oogleapis.com/auth/pubsub2\322\025\n\nSubscriber"
    "\022\264\001\n\022CreateSubscription\022\036.google.pubsub."
    "v1.Subscription\032\036.google.pubsub.v1.Subsc"
    "ription\"^\332A+name,topic,push_config,ack_d"
    "eadline_seconds\202\323\344\223\002*\032%/v1/{name=project"
    "s/*/subscriptions/*}:\001*\022\241\001\n\017GetSubscript"
    "ion\022(.google.pubsub.v1.GetSubscriptionRe"
    "quest\032\036.google.pubsub.v1.Subscription\"D\332"
    "A\014subscription\202\323\344\223\002/\022-/v1/{subscription="
    "projects/*/subscriptions/*}\022\273\001\n\022UpdateSu"
    "bscription\022+.google.pubsub.v1.UpdateSubs"
    "criptionRequest\032\036.google.pubsub.v1.Subsc"
    "ription\"X\332A\030subscription,update_mask\202\323\344\223"
    "\002722/v1/{subscription.name=projects/*/su"
    "bscriptions/*}:\001*\022\246\001\n\021ListSubscriptions\022"
    "*.google.pubsub.v1.ListSubscriptionsRequ"
    "est\032+.google.pubsub.v1.ListSubscriptions"
    "Response\"8\332A\007project\202\323\344\223\002(\022&/v1/{project"
    "=projects/*}/subscriptions\022\237\001\n\022DeleteSub"
    "scription\022+.google.pubsub.v1.DeleteSubsc"
    "riptionRequest\032\026.google.protobuf.Empty\"D"
    "\332A\014subscription\202\323\344\223\002/*-/v1/{subscription"
    "=projects/*/subscriptions/*}\022\317\001\n\021ModifyA"
    "ckDeadline\022*.google.pubsub.v1.ModifyAckD"
    "eadlineRequest\032\026.google.protobuf.Empty\"v"
    "\332A)subscription,ack_ids,ack_deadline_sec"
    "onds\202\323\344\223\002D\"\?/v1/{subscription=projects/*"
    "/subscriptions/*}:modifyAckDeadline:\001*\022\250"
    "\001\n\013Acknowledge\022$.google.pubsub.v1.Acknow"
    "ledgeRequest\032\026.google.protobuf.Empty\"[\332A"
    "\024subscription,ack_ids\202\323\344\223\002>\"9/v1/{subscr"
    "iption=projects/*/subscriptions/*}:ackno"
    "wledge:\001*\022\320\001\n\004Pull\022\035.google.pubsub.v1.Pu"
    "llRequest\032\036.google.pubsub.v1.PullRespons"
    "e\"\210\001\332A,subscription,return_immediately,m"
    "ax_messages\332A\031subscription,max_messages\202"
    "\323\344\223\0027\"2/v1/{subscription=projects/*/subs"
    "criptions/*}:pull:\001*\022f\n\rStreamingPull\022&."
    "google.pubsub.v1.StreamingPullRequest\032\'."
    "google.pubsub.v1.StreamingPullResponse\"\000"
    "(\0010\001\022\273\001\n\020ModifyPushConfig\022).google.pubsu"
    "b.v1.ModifyPushConfigRequest\032\026.google.pr"
    "otobuf.Empty\"d\332A\030subscription,push_confi"
    "g\202\323\344\223\002C\">/v1/{subscription=projects/*/su"
    "bscriptions/*}:modifyPushConfig:\001*\022\211\001\n\013G"
    "etSnapshot\022$.google.pubsub.v1.GetSnapsho"
    "tRequest\032\032.google.pubsub.v1.Snapshot\"8\332A"
    "\010snapshot\202\323\344\223\002\'\022%/v1/{snapshot=projects/"
    "*/snapshots/*}\022\226\001\n\rListSnapshots\022&.googl"
    "e.pubsub.v1.ListSnapshotsRequest\032\'.googl"
    "e.pubsub.v1.ListSnapshotsResponse\"4\332A\007pr"
    "oject\202\323\344\223\002$\022\"/v1/{project=projects/*}/sn"
    "apshots\022\227\001\n\016CreateSnapshot\022\'.google.pubs"
    "ub.v1.CreateSnapshotRequest\032\032.google.pub"
    "sub.v1.Snapshot\"@\332A\021name,subscription\202\323\344"
    "\223\002&\032!/v1/{name=projects/*/snapshots/*}:\001"
    "*\022\243\001\n\016UpdateSnapshot\022\'.google.pubsub.v1."
    "UpdateSnapshotRequest\032\032.google.pubsub.v1"
    ".Snapshot\"L\332A\024snapshot,update_mask\202\323\344\223\002/"
    "2*/v1/{snapshot.name=projects/*/snapshot"
    "s/*}:\001*\022\213\001\n\016DeleteSnapshot\022\'.google.pubs"
    "ub.v1.DeleteSnapshotRequest\032\026.google.pro"
    "tobuf.Empty\"8\332A\010snapshot\202\323\344\223\002\'*%/v1/{sna"
    "pshot=projects/*/snapshots/*}\022\204\001\n\004Seek\022\035"
    ".google.pubsub.v1.SeekRequest\032\036.google.p"
    "ubsub.v1.SeekResponse\"=\202\323\344\223\0027\"2/v1/{subs"
    "cription=projects/*/subscriptions/*}:see"
    "k:\001*\032p\312A\025pubsub.googleapis.com\322AUhttps:/"
    "/www.googleapis.com/auth/cloud-platform,"
    "https://www.googleapis.com/auth/pubsubB\252"
    "\001\n\024com.google.pubsub.v1B\013PubsubProtoP\001Z2"
    "cloud.google.com/go/pubsub/apiv1/pubsubp"
    "b;pubsubpb\370\001\001\252\002\026Google.Cloud.PubSub.V1\312\002"
    "\026Google\\Cloud\\PubSub\\V1\352\002\031Google::Cloud:"
    ":PubSub::V1b\006proto3"
};
static const ::_pbi::DescriptorTable* PROTOBUF_NONNULL const
    descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto_deps[9] = {
        &::descriptor_table_google_2fapi_2fannotations_2eproto,
        &::descriptor_table_google_2fapi_2fclient_2eproto,
        &::descriptor_table_google_2fapi_2ffield_5fbehavior_2eproto,
        &::descriptor_table_google_2fapi_2fresource_2eproto,
        &::descriptor_table_google_2fprotobuf_2fduration_2eproto,
        &::descriptor_table_google_2fprotobuf_2fempty_2eproto,
        &::descriptor_table_google_2fprotobuf_2ffield_5fmask_2eproto,
        &::descriptor_table_google_2fprotobuf_2ftimestamp_2eproto,
        &::descriptor_table_google_2fpubsub_2fv1_2fschema_2eproto,
};
static ::absl::once_flag descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto = {
    false,
    false,
    16779,
    descriptor_table_protodef_google_2fpubsub_2fv1_2fpubsub_2eproto,
    "google/pubsub/v1/pubsub.proto",
    &descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto_once,
    descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto_deps,
    9,
    72,
    schemas,
    file_message_globals,
    TableStruct_google_2fpubsub_2fv1_2fpubsub_2eproto::offsets,
    file_level_enum_descriptors_google_2fpubsub_2fv1_2fpubsub_2eproto,
    file_level_service_descriptors_google_2fpubsub_2fv1_2fpubsub_2eproto,
};
namespace google {
namespace pubsub {
namespace v1 {
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
IngestionDataSourceSettings_AwsKinesis_State_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto);
  return file_level_enum_descriptors_google_2fpubsub_2fv1_2fpubsub_2eproto[0];
}
PROTOBUF_CONSTINIT const uint32_t IngestionDataSourceSettings_AwsKinesis_State_internal_data_[] = {
    393216u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
IngestionDataSourceSettings_CloudStorage_State_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto);
  return file_level_enum_descriptors_google_2fpubsub_2fv1_2fpubsub_2eproto[1];
}
PROTOBUF_CONSTINIT const uint32_t IngestionDataSourceSettings_CloudStorage_State_internal_data_[] = {
    393216u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
PlatformLogsSettings_Severity_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto);
  return file_level_enum_descriptors_google_2fpubsub_2fv1_2fpubsub_2eproto[2];
}
PROTOBUF_CONSTINIT const uint32_t PlatformLogsSettings_Severity_internal_data_[] = {
    393216u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
Topic_State_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto);
  return file_level_enum_descriptors_google_2fpubsub_2fv1_2fpubsub_2eproto[3];
}
PROTOBUF_CONSTINIT const uint32_t Topic_State_internal_data_[] = {
    196608u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
Subscription_State_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto);
  return file_level_enum_descriptors_google_2fpubsub_2fv1_2fpubsub_2eproto[4];
}
PROTOBUF_CONSTINIT const uint32_t Subscription_State_internal_data_[] = {
    196608u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
BigQueryConfig_State_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto);
  return file_level_enum_descriptors_google_2fpubsub_2fv1_2fpubsub_2eproto[5];
}
PROTOBUF_CONSTINIT const uint32_t BigQueryConfig_State_internal_data_[] = {
    393216u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
CloudStorageConfig_State_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto);
  return file_level_enum_descriptors_google_2fpubsub_2fv1_2fpubsub_2eproto[6];
}
PROTOBUF_CONSTINIT const uint32_t CloudStorageConfig_State_internal_data_[] = {
    393216u, 0u, };
// ===================================================================

MessageStoragePolicy::MessageStoragePolicy(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, MessageStoragePolicy_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.MessageStoragePolicy)
}
PROTOBUF_NDEBUG_INLINE MessageStoragePolicy::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::MessageStoragePolicy& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        allowed_persistence_regions_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::MessageStoragePolicy,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::MessageStoragePolicy, _impl_.allowed_persistence_regions_)>()
          , from.allowed_persistence_regions_
        }
     {}

MessageStoragePolicy::MessageStoragePolicy(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const MessageStoragePolicy& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, MessageStoragePolicy_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  MessageStoragePolicy* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.enforce_in_transit_ = from._impl_.enforce_in_transit_;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.MessageStoragePolicy)
}
PROTOBUF_NDEBUG_INLINE MessageStoragePolicy::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        allowed_persistence_regions_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::MessageStoragePolicy,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::MessageStoragePolicy, _impl_.allowed_persistence_regions_)>()
         }
     {}

inline void MessageStoragePolicy::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.enforce_in_transit_ = {};
}
MessageStoragePolicy::~MessageStoragePolicy() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.MessageStoragePolicy)
  SharedDtor(*this);
}
inline void MessageStoragePolicy::SharedDtor(MessageLite& self) {
  MessageStoragePolicy& this_ = static_cast<MessageStoragePolicy&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull MessageStoragePolicy_class_data_ =
        MessageStoragePolicy::InternalGenerateClassData_(MessageStoragePolicy_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
MessageStoragePolicy::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&MessageStoragePolicy_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(MessageStoragePolicy_class_data_.tc_table);
  return MessageStoragePolicy_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
MessageStoragePolicy::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&MessageStoragePolicy_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&MessageStoragePolicy_globals_));
  return MessageStoragePolicy_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const MessageStoragePolicy::ParseTableT_
    MessageStoragePolicy::_table_ =
        MessageStoragePolicy::InternalGenerateParseTable_(MessageStoragePolicy_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void MessageStoragePolicy::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.MessageStoragePolicy)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.allowed_persistence_regions_.Clear();
  }
  _impl_.enforce_in_transit_ = false;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL MessageStoragePolicy::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const MessageStoragePolicy& this_ = static_cast<const MessageStoragePolicy&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL MessageStoragePolicy::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const MessageStoragePolicy& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.MessageStoragePolicy)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated string allowed_persistence_regions = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (int i = 0, n = this_._internal_allowed_persistence_regions_size(); i < n; ++i) {
      const auto& s = this_._internal_allowed_persistence_regions().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.MessageStoragePolicy.allowed_persistence_regions");
      target = stream->WriteString(1, s, target);
    }
  }

  // bool enforce_in_transit = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_enforce_in_transit() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          2, this_._internal_enforce_in_transit(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.MessageStoragePolicy)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t MessageStoragePolicy::ByteSizeLong(const MessageLite& base) {
  const MessageStoragePolicy& this_ = static_cast<const MessageStoragePolicy&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t MessageStoragePolicy::ByteSizeLong() const {
  const MessageStoragePolicy& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.MessageStoragePolicy)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // repeated string allowed_persistence_regions = 1 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_allowed_persistence_regions().size());
      for (int i = 0, n = this_._internal_allowed_persistence_regions().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_allowed_persistence_regions().Get(i));
      }
    }
    // bool enforce_in_transit = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_enforce_in_transit() != 0) {
        total_size += 2;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void MessageStoragePolicy::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<MessageStoragePolicy*>(&to_msg);
  auto& from = static_cast<const MessageStoragePolicy&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.MessageStoragePolicy)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_allowed_persistence_regions()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_allowed_persistence_regions());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_enforce_in_transit() != 0) {
        _this->_impl_.enforce_in_transit_ = from._impl_.enforce_in_transit_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void MessageStoragePolicy::CopyFrom(const MessageStoragePolicy& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.MessageStoragePolicy)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void MessageStoragePolicy::InternalSwap(MessageStoragePolicy* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.allowed_persistence_regions_.InternalSwap(&other->_impl_.allowed_persistence_regions_);
  swap(_impl_.enforce_in_transit_, other->_impl_.enforce_in_transit_);
}

::google::protobuf::Metadata MessageStoragePolicy::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

SchemaSettings::SchemaSettings(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, SchemaSettings_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.SchemaSettings)
}
PROTOBUF_NDEBUG_INLINE SchemaSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::SchemaSettings& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        schema_(arena, from.schema_),
        first_revision_id_(arena, from.first_revision_id_),
        last_revision_id_(arena, from.last_revision_id_) {}

SchemaSettings::SchemaSettings(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const SchemaSettings& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, SchemaSettings_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SchemaSettings* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.encoding_ = from._impl_.encoding_;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.SchemaSettings)
}
PROTOBUF_NDEBUG_INLINE SchemaSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        schema_(arena),
        first_revision_id_(arena),
        last_revision_id_(arena) {}

inline void SchemaSettings::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.encoding_ = {};
}
SchemaSettings::~SchemaSettings() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.SchemaSettings)
  SharedDtor(*this);
}
inline void SchemaSettings::SharedDtor(MessageLite& self) {
  SchemaSettings& this_ = static_cast<SchemaSettings&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.schema_.Destroy();
  this_._impl_.first_revision_id_.Destroy();
  this_._impl_.last_revision_id_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull SchemaSettings_class_data_ =
        SchemaSettings::InternalGenerateClassData_(SchemaSettings_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
SchemaSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&SchemaSettings_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(SchemaSettings_class_data_.tc_table);
  return SchemaSettings_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
SchemaSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&SchemaSettings_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&SchemaSettings_globals_));
  return SchemaSettings_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const SchemaSettings::ParseTableT_
    SchemaSettings::_table_ =
        SchemaSettings::InternalGenerateParseTable_(SchemaSettings_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void SchemaSettings::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.SchemaSettings)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.schema_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.first_revision_id_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.last_revision_id_.ClearNonDefaultToEmpty();
    }
  }
  _impl_.encoding_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL SchemaSettings::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const SchemaSettings& this_ = static_cast<const SchemaSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL SchemaSettings::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const SchemaSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.SchemaSettings)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string schema = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_schema().empty()) {
      const ::std::string& _s = this_._internal_schema();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.SchemaSettings.schema");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // .google.pubsub.v1.Encoding encoding = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_encoding() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          2, this_._internal_encoding(), target);
    }
  }

  // string first_revision_id = 3 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_first_revision_id().empty()) {
      const ::std::string& _s = this_._internal_first_revision_id();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.SchemaSettings.first_revision_id");
      target = stream->WriteStringMaybeAliased(3, _s, target);
    }
  }

  // string last_revision_id = 4 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_last_revision_id().empty()) {
      const ::std::string& _s = this_._internal_last_revision_id();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.SchemaSettings.last_revision_id");
      target = stream->WriteStringMaybeAliased(4, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.SchemaSettings)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t SchemaSettings::ByteSizeLong(const MessageLite& base) {
  const SchemaSettings& this_ = static_cast<const SchemaSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t SchemaSettings::ByteSizeLong() const {
  const SchemaSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.SchemaSettings)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // string schema = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_schema().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_schema());
      }
    }
    // string first_revision_id = 3 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_first_revision_id().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_first_revision_id());
      }
    }
    // string last_revision_id = 4 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!this_._internal_last_revision_id().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_last_revision_id());
      }
    }
    // .google.pubsub.v1.Encoding encoding = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_encoding() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_encoding());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void SchemaSettings::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<SchemaSettings*>(&to_msg);
  auto& from = static_cast<const SchemaSettings&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.SchemaSettings)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_schema().empty()) {
        _this->_internal_set_schema(from._internal_schema());
      } else {
        if (_this->_impl_.schema_.IsDefault()) {
          _this->_internal_set_schema("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_first_revision_id().empty()) {
        _this->_internal_set_first_revision_id(from._internal_first_revision_id());
      } else {
        if (_this->_impl_.first_revision_id_.IsDefault()) {
          _this->_internal_set_first_revision_id("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!from._internal_last_revision_id().empty()) {
        _this->_internal_set_last_revision_id(from._internal_last_revision_id());
      } else {
        if (_this->_impl_.last_revision_id_.IsDefault()) {
          _this->_internal_set_last_revision_id("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_encoding() != 0) {
        _this->_impl_.encoding_ = from._impl_.encoding_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void SchemaSettings::CopyFrom(const SchemaSettings& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.SchemaSettings)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void SchemaSettings::InternalSwap(SchemaSettings* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.schema_, &other->_impl_.schema_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.first_revision_id_, &other->_impl_.first_revision_id_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.last_revision_id_, &other->_impl_.last_revision_id_, arena);
  swap(_impl_.encoding_, other->_impl_.encoding_);
}

::google::protobuf::Metadata SchemaSettings::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

IngestionDataSourceSettings_AwsKinesis::IngestionDataSourceSettings_AwsKinesis(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, IngestionDataSourceSettings_AwsKinesis_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.IngestionDataSourceSettings.AwsKinesis)
}
PROTOBUF_NDEBUG_INLINE IngestionDataSourceSettings_AwsKinesis::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::IngestionDataSourceSettings_AwsKinesis& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        stream_arn_(arena, from.stream_arn_),
        consumer_arn_(arena, from.consumer_arn_),
        aws_role_arn_(arena, from.aws_role_arn_),
        gcp_service_account_(arena, from.gcp_service_account_) {}

IngestionDataSourceSettings_AwsKinesis::IngestionDataSourceSettings_AwsKinesis(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const IngestionDataSourceSettings_AwsKinesis& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, IngestionDataSourceSettings_AwsKinesis_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  IngestionDataSourceSettings_AwsKinesis* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.state_ = from._impl_.state_;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.IngestionDataSourceSettings.AwsKinesis)
}
PROTOBUF_NDEBUG_INLINE IngestionDataSourceSettings_AwsKinesis::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        stream_arn_(arena),
        consumer_arn_(arena),
        aws_role_arn_(arena),
        gcp_service_account_(arena) {}

inline void IngestionDataSourceSettings_AwsKinesis::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.state_ = {};
}
IngestionDataSourceSettings_AwsKinesis::~IngestionDataSourceSettings_AwsKinesis() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.IngestionDataSourceSettings.AwsKinesis)
  SharedDtor(*this);
}
inline void IngestionDataSourceSettings_AwsKinesis::SharedDtor(MessageLite& self) {
  IngestionDataSourceSettings_AwsKinesis& this_ = static_cast<IngestionDataSourceSettings_AwsKinesis&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.stream_arn_.Destroy();
  this_._impl_.consumer_arn_.Destroy();
  this_._impl_.aws_role_arn_.Destroy();
  this_._impl_.gcp_service_account_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull IngestionDataSourceSettings_AwsKinesis_class_data_ =
        IngestionDataSourceSettings_AwsKinesis::InternalGenerateClassData_(IngestionDataSourceSettings_AwsKinesis_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
IngestionDataSourceSettings_AwsKinesis::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&IngestionDataSourceSettings_AwsKinesis_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(IngestionDataSourceSettings_AwsKinesis_class_data_.tc_table);
  return IngestionDataSourceSettings_AwsKinesis_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
IngestionDataSourceSettings_AwsKinesis::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&IngestionDataSourceSettings_AwsKinesis_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&IngestionDataSourceSettings_AwsKinesis_globals_));
  return IngestionDataSourceSettings_AwsKinesis_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const IngestionDataSourceSettings_AwsKinesis::ParseTableT_
    IngestionDataSourceSettings_AwsKinesis::_table_ =
        IngestionDataSourceSettings_AwsKinesis::InternalGenerateParseTable_(IngestionDataSourceSettings_AwsKinesis_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void IngestionDataSourceSettings_AwsKinesis::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.IngestionDataSourceSettings.AwsKinesis)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.stream_arn_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.consumer_arn_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.aws_role_arn_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _impl_.gcp_service_account_.ClearNonDefaultToEmpty();
    }
  }
  _impl_.state_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL IngestionDataSourceSettings_AwsKinesis::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const IngestionDataSourceSettings_AwsKinesis& this_ = static_cast<const IngestionDataSourceSettings_AwsKinesis&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL IngestionDataSourceSettings_AwsKinesis::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const IngestionDataSourceSettings_AwsKinesis& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.IngestionDataSourceSettings.AwsKinesis)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.pubsub.v1.IngestionDataSourceSettings.AwsKinesis.State state = 1 [(.google.api.field_behavior) = OUTPUT_ONLY];
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_state() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          1, this_._internal_state(), target);
    }
  }

  // string stream_arn = 2 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_stream_arn().empty()) {
      const ::std::string& _s = this_._internal_stream_arn();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.IngestionDataSourceSettings.AwsKinesis.stream_arn");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  // string consumer_arn = 3 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_consumer_arn().empty()) {
      const ::std::string& _s = this_._internal_consumer_arn();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.IngestionDataSourceSettings.AwsKinesis.consumer_arn");
      target = stream->WriteStringMaybeAliased(3, _s, target);
    }
  }

  // string aws_role_arn = 4 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_aws_role_arn().empty()) {
      const ::std::string& _s = this_._internal_aws_role_arn();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.IngestionDataSourceSettings.AwsKinesis.aws_role_arn");
      target = stream->WriteStringMaybeAliased(4, _s, target);
    }
  }

  // string gcp_service_account = 5 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (!this_._internal_gcp_service_account().empty()) {
      const ::std::string& _s = this_._internal_gcp_service_account();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.IngestionDataSourceSettings.AwsKinesis.gcp_service_account");
      target = stream->WriteStringMaybeAliased(5, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.IngestionDataSourceSettings.AwsKinesis)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t IngestionDataSourceSettings_AwsKinesis::ByteSizeLong(const MessageLite& base) {
  const IngestionDataSourceSettings_AwsKinesis& this_ = static_cast<const IngestionDataSourceSettings_AwsKinesis&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t IngestionDataSourceSettings_AwsKinesis::ByteSizeLong() const {
  const IngestionDataSourceSettings_AwsKinesis& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.IngestionDataSourceSettings.AwsKinesis)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    // string stream_arn = 2 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_stream_arn().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_stream_arn());
      }
    }
    // string consumer_arn = 3 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_consumer_arn().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_consumer_arn());
      }
    }
    // string aws_role_arn = 4 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!this_._internal_aws_role_arn().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_aws_role_arn());
      }
    }
    // string gcp_service_account = 5 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!this_._internal_gcp_service_account().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_gcp_service_account());
      }
    }
    // .google.pubsub.v1.IngestionDataSourceSettings.AwsKinesis.State state = 1 [(.google.api.field_behavior) = OUTPUT_ONLY];
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_state() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_state());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void IngestionDataSourceSettings_AwsKinesis::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<IngestionDataSourceSettings_AwsKinesis*>(&to_msg);
  auto& from = static_cast<const IngestionDataSourceSettings_AwsKinesis&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.IngestionDataSourceSettings.AwsKinesis)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_stream_arn().empty()) {
        _this->_internal_set_stream_arn(from._internal_stream_arn());
      } else {
        if (_this->_impl_.stream_arn_.IsDefault()) {
          _this->_internal_set_stream_arn("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_consumer_arn().empty()) {
        _this->_internal_set_consumer_arn(from._internal_consumer_arn());
      } else {
        if (_this->_impl_.consumer_arn_.IsDefault()) {
          _this->_internal_set_consumer_arn("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!from._internal_aws_role_arn().empty()) {
        _this->_internal_set_aws_role_arn(from._internal_aws_role_arn());
      } else {
        if (_this->_impl_.aws_role_arn_.IsDefault()) {
          _this->_internal_set_aws_role_arn("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!from._internal_gcp_service_account().empty()) {
        _this->_internal_set_gcp_service_account(from._internal_gcp_service_account());
      } else {
        if (_this->_impl_.gcp_service_account_.IsDefault()) {
          _this->_internal_set_gcp_service_account("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_state() != 0) {
        _this->_impl_.state_ = from._impl_.state_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void IngestionDataSourceSettings_AwsKinesis::CopyFrom(const IngestionDataSourceSettings_AwsKinesis& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.IngestionDataSourceSettings.AwsKinesis)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void IngestionDataSourceSettings_AwsKinesis::InternalSwap(IngestionDataSourceSettings_AwsKinesis* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.stream_arn_, &other->_impl_.stream_arn_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.consumer_arn_, &other->_impl_.consumer_arn_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.aws_role_arn_, &other->_impl_.aws_role_arn_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.gcp_service_account_, &other->_impl_.gcp_service_account_, arena);
  swap(_impl_.state_, other->_impl_.state_);
}

::google::protobuf::Metadata IngestionDataSourceSettings_AwsKinesis::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

IngestionDataSourceSettings_CloudStorage_TextFormat::IngestionDataSourceSettings_CloudStorage_TextFormat(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, IngestionDataSourceSettings_CloudStorage_TextFormat_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.TextFormat)
}
PROTOBUF_NDEBUG_INLINE IngestionDataSourceSettings_CloudStorage_TextFormat::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_TextFormat& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        delimiter_(arena, from.delimiter_) {}

IngestionDataSourceSettings_CloudStorage_TextFormat::IngestionDataSourceSettings_CloudStorage_TextFormat(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const IngestionDataSourceSettings_CloudStorage_TextFormat& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, IngestionDataSourceSettings_CloudStorage_TextFormat_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  IngestionDataSourceSettings_CloudStorage_TextFormat* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.TextFormat)
}
PROTOBUF_NDEBUG_INLINE IngestionDataSourceSettings_CloudStorage_TextFormat::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        delimiter_(arena) {}

inline void IngestionDataSourceSettings_CloudStorage_TextFormat::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
IngestionDataSourceSettings_CloudStorage_TextFormat::~IngestionDataSourceSettings_CloudStorage_TextFormat() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.TextFormat)
  SharedDtor(*this);
}
inline void IngestionDataSourceSettings_CloudStorage_TextFormat::SharedDtor(MessageLite& self) {
  IngestionDataSourceSettings_CloudStorage_TextFormat& this_ = static_cast<IngestionDataSourceSettings_CloudStorage_TextFormat&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.delimiter_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull IngestionDataSourceSettings_CloudStorage_TextFormat_class_data_ =
        IngestionDataSourceSettings_CloudStorage_TextFormat::InternalGenerateClassData_(IngestionDataSourceSettings_CloudStorage_TextFormat_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
IngestionDataSourceSettings_CloudStorage_TextFormat::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&IngestionDataSourceSettings_CloudStorage_TextFormat_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(IngestionDataSourceSettings_CloudStorage_TextFormat_class_data_.tc_table);
  return IngestionDataSourceSettings_CloudStorage_TextFormat_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
IngestionDataSourceSettings_CloudStorage_TextFormat::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&IngestionDataSourceSettings_CloudStorage_TextFormat_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&IngestionDataSourceSettings_CloudStorage_TextFormat_globals_));
  return IngestionDataSourceSettings_CloudStorage_TextFormat_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const IngestionDataSourceSettings_CloudStorage_TextFormat::ParseTableT_
    IngestionDataSourceSettings_CloudStorage_TextFormat::_table_ =
        IngestionDataSourceSettings_CloudStorage_TextFormat::InternalGenerateParseTable_(IngestionDataSourceSettings_CloudStorage_TextFormat_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void IngestionDataSourceSettings_CloudStorage_TextFormat::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.TextFormat)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.delimiter_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL IngestionDataSourceSettings_CloudStorage_TextFormat::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const IngestionDataSourceSettings_CloudStorage_TextFormat& this_ = static_cast<const IngestionDataSourceSettings_CloudStorage_TextFormat&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL IngestionDataSourceSettings_CloudStorage_TextFormat::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const IngestionDataSourceSettings_CloudStorage_TextFormat& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.TextFormat)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // optional string delimiter = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    const ::std::string& _s = this_._internal_delimiter();
    ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
        _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.TextFormat.delimiter");
    target = stream->WriteStringMaybeAliased(1, _s, target);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.TextFormat)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t IngestionDataSourceSettings_CloudStorage_TextFormat::ByteSizeLong(const MessageLite& base) {
  const IngestionDataSourceSettings_CloudStorage_TextFormat& this_ = static_cast<const IngestionDataSourceSettings_CloudStorage_TextFormat&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t IngestionDataSourceSettings_CloudStorage_TextFormat::ByteSizeLong() const {
  const IngestionDataSourceSettings_CloudStorage_TextFormat& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.TextFormat)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // optional string delimiter = 1 [(.google.api.field_behavior) = OPTIONAL];
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                      this_._internal_delimiter());
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void IngestionDataSourceSettings_CloudStorage_TextFormat::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<IngestionDataSourceSettings_CloudStorage_TextFormat*>(&to_msg);
  auto& from = static_cast<const IngestionDataSourceSettings_CloudStorage_TextFormat&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.TextFormat)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _this->_internal_set_delimiter(from._internal_delimiter());
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void IngestionDataSourceSettings_CloudStorage_TextFormat::CopyFrom(const IngestionDataSourceSettings_CloudStorage_TextFormat& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.TextFormat)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void IngestionDataSourceSettings_CloudStorage_TextFormat::InternalSwap(IngestionDataSourceSettings_CloudStorage_TextFormat* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.delimiter_, &other->_impl_.delimiter_, arena);
}

::google::protobuf::Metadata IngestionDataSourceSettings_CloudStorage_TextFormat::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

IngestionDataSourceSettings_CloudStorage_AvroFormat::IngestionDataSourceSettings_CloudStorage_AvroFormat(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, IngestionDataSourceSettings_CloudStorage_AvroFormat_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.AvroFormat)
}
IngestionDataSourceSettings_CloudStorage_AvroFormat::IngestionDataSourceSettings_CloudStorage_AvroFormat(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const IngestionDataSourceSettings_CloudStorage_AvroFormat& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, IngestionDataSourceSettings_CloudStorage_AvroFormat_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  IngestionDataSourceSettings_CloudStorage_AvroFormat* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.AvroFormat)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull IngestionDataSourceSettings_CloudStorage_AvroFormat_class_data_ =
        IngestionDataSourceSettings_CloudStorage_AvroFormat::InternalGenerateClassData_(IngestionDataSourceSettings_CloudStorage_AvroFormat_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
IngestionDataSourceSettings_CloudStorage_AvroFormat::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&IngestionDataSourceSettings_CloudStorage_AvroFormat_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(IngestionDataSourceSettings_CloudStorage_AvroFormat_class_data_.tc_table);
  return IngestionDataSourceSettings_CloudStorage_AvroFormat_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
IngestionDataSourceSettings_CloudStorage_AvroFormat::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&IngestionDataSourceSettings_CloudStorage_AvroFormat_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&IngestionDataSourceSettings_CloudStorage_AvroFormat_globals_));
  return IngestionDataSourceSettings_CloudStorage_AvroFormat_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const IngestionDataSourceSettings_CloudStorage_AvroFormat::ParseTableT_
    IngestionDataSourceSettings_CloudStorage_AvroFormat::_table_ =
        IngestionDataSourceSettings_CloudStorage_AvroFormat::InternalGenerateParseTable_(IngestionDataSourceSettings_CloudStorage_AvroFormat_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata IngestionDataSourceSettings_CloudStorage_AvroFormat::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat::IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.PubSubAvroFormat)
}
IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat::IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.PubSubAvroFormat)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat_class_data_ =
        IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat::InternalGenerateClassData_(IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat_class_data_.tc_table);
  return IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat_globals_));
  return IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat::ParseTableT_
    IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat::_table_ =
        IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat::InternalGenerateParseTable_(IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void IngestionDataSourceSettings_CloudStorage::set_allocated_text_format(::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_TextFormat* PROTOBUF_NULLABLE text_format) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_input_format();
  if (text_format) {
    ::google::protobuf::Arena* submessage_arena = text_format->GetArena();
    if (message_arena != submessage_arena) {
      text_format = ::google::protobuf::internal::GetOwnedMessage(message_arena, text_format, submessage_arena);
    }
    set_has_text_format();
    _impl_.input_format_.text_format_ = text_format;
  }
  // @@protoc_insertion_point(field_set_allocated:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.text_format)
}
void IngestionDataSourceSettings_CloudStorage::set_allocated_avro_format(::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_AvroFormat* PROTOBUF_NULLABLE avro_format) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_input_format();
  if (avro_format) {
    ::google::protobuf::Arena* submessage_arena = avro_format->GetArena();
    if (message_arena != submessage_arena) {
      avro_format = ::google::protobuf::internal::GetOwnedMessage(message_arena, avro_format, submessage_arena);
    }
    set_has_avro_format();
    _impl_.input_format_.avro_format_ = avro_format;
  }
  // @@protoc_insertion_point(field_set_allocated:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.avro_format)
}
void IngestionDataSourceSettings_CloudStorage::set_allocated_pubsub_avro_format(::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage_PubSubAvroFormat* PROTOBUF_NULLABLE pubsub_avro_format) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_input_format();
  if (pubsub_avro_format) {
    ::google::protobuf::Arena* submessage_arena = pubsub_avro_format->GetArena();
    if (message_arena != submessage_arena) {
      pubsub_avro_format = ::google::protobuf::internal::GetOwnedMessage(message_arena, pubsub_avro_format, submessage_arena);
    }
    set_has_pubsub_avro_format();
    _impl_.input_format_.pubsub_avro_format_ = pubsub_avro_format;
  }
  // @@protoc_insertion_point(field_set_allocated:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.pubsub_avro_format)
}
void IngestionDataSourceSettings_CloudStorage::clear_minimum_object_create_time() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.minimum_object_create_time_ != nullptr) _impl_.minimum_object_create_time_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
IngestionDataSourceSettings_CloudStorage::IngestionDataSourceSettings_CloudStorage(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, IngestionDataSourceSettings_CloudStorage_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage)
}
PROTOBUF_NDEBUG_INLINE IngestionDataSourceSettings_CloudStorage::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        bucket_(arena, from.bucket_),
        match_glob_(arena, from.match_glob_),
        input_format_{},
        _oneof_case_{from._oneof_case_[0]} {}

IngestionDataSourceSettings_CloudStorage::IngestionDataSourceSettings_CloudStorage(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const IngestionDataSourceSettings_CloudStorage& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, IngestionDataSourceSettings_CloudStorage_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  IngestionDataSourceSettings_CloudStorage* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.minimum_object_create_time_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.minimum_object_create_time_)
                : nullptr;
  _impl_.state_ = from._impl_.state_;
  switch (input_format_case()) {
    case INPUT_FORMAT_NOT_SET:
      break;
      case kTextFormat:
        _impl_.input_format_.text_format_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.input_format_.text_format_);
        break;
      case kAvroFormat:
        _impl_.input_format_.avro_format_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.input_format_.avro_format_);
        break;
      case kPubsubAvroFormat:
        _impl_.input_format_.pubsub_avro_format_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.input_format_.pubsub_avro_format_);
        break;
  }

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage)
}
PROTOBUF_NDEBUG_INLINE IngestionDataSourceSettings_CloudStorage::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        bucket_(arena),
        match_glob_(arena),
        input_format_{},
        _oneof_case_{} {}

inline void IngestionDataSourceSettings_CloudStorage::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, minimum_object_create_time_),
           0,
           offsetof(Impl_, state_) -
               offsetof(Impl_, minimum_object_create_time_) +
               sizeof(Impl_::state_));
}
IngestionDataSourceSettings_CloudStorage::~IngestionDataSourceSettings_CloudStorage() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage)
  SharedDtor(*this);
}
inline void IngestionDataSourceSettings_CloudStorage::SharedDtor(MessageLite& self) {
  IngestionDataSourceSettings_CloudStorage& this_ = static_cast<IngestionDataSourceSettings_CloudStorage&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.bucket_.Destroy();
  this_._impl_.match_glob_.Destroy();
  delete this_._impl_.minimum_object_create_time_;
  if (this_.has_input_format()) {
    this_.clear_input_format();
  }
  this_._impl_.~Impl_();
}

void IngestionDataSourceSettings_CloudStorage::clear_input_format() {
// @@protoc_insertion_point(one_of_clear_start:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (input_format_case()) {
    case kTextFormat: {
      if (GetArena() == nullptr) {
        delete _impl_.input_format_.text_format_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.input_format_.text_format_);
      }
      break;
    }
    case kAvroFormat: {
      if (GetArena() == nullptr) {
        delete _impl_.input_format_.avro_format_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.input_format_.avro_format_);
      }
      break;
    }
    case kPubsubAvroFormat: {
      if (GetArena() == nullptr) {
        delete _impl_.input_format_.pubsub_avro_format_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.input_format_.pubsub_avro_format_);
      }
      break;
    }
    case INPUT_FORMAT_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = INPUT_FORMAT_NOT_SET;
}


#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull IngestionDataSourceSettings_CloudStorage_class_data_ =
        IngestionDataSourceSettings_CloudStorage::InternalGenerateClassData_(IngestionDataSourceSettings_CloudStorage_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
IngestionDataSourceSettings_CloudStorage::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&IngestionDataSourceSettings_CloudStorage_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(IngestionDataSourceSettings_CloudStorage_class_data_.tc_table);
  return IngestionDataSourceSettings_CloudStorage_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
IngestionDataSourceSettings_CloudStorage::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&IngestionDataSourceSettings_CloudStorage_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&IngestionDataSourceSettings_CloudStorage_globals_));
  return IngestionDataSourceSettings_CloudStorage_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const IngestionDataSourceSettings_CloudStorage::ParseTableT_
    IngestionDataSourceSettings_CloudStorage::_table_ =
        IngestionDataSourceSettings_CloudStorage::InternalGenerateParseTable_(IngestionDataSourceSettings_CloudStorage_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void IngestionDataSourceSettings_CloudStorage::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.bucket_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.match_glob_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(_impl_.minimum_object_create_time_ != nullptr);
      _impl_.minimum_object_create_time_->Clear();
    }
  }
  _impl_.state_ = 0;
  clear_input_format();
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL IngestionDataSourceSettings_CloudStorage::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const IngestionDataSourceSettings_CloudStorage& this_ = static_cast<const IngestionDataSourceSettings_CloudStorage&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL IngestionDataSourceSettings_CloudStorage::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const IngestionDataSourceSettings_CloudStorage& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.State state = 1 [(.google.api.field_behavior) = OUTPUT_ONLY];
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_state() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          1, this_._internal_state(), target);
    }
  }

  // string bucket = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_bucket().empty()) {
      const ::std::string& _s = this_._internal_bucket();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.bucket");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  switch (this_.input_format_case()) {
    case kTextFormat: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          3, *this_._impl_.input_format_.text_format_, this_._impl_.input_format_.text_format_->GetCachedSize(), target,
          stream);
      break;
    }
    case kAvroFormat: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          4, *this_._impl_.input_format_.avro_format_, this_._impl_.input_format_.avro_format_->GetCachedSize(), target,
          stream);
      break;
    }
    case kPubsubAvroFormat: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          5, *this_._impl_.input_format_.pubsub_avro_format_, this_._impl_.input_format_.pubsub_avro_format_->GetCachedSize(), target,
          stream);
      break;
    }
    default:
      break;
  }
  // .google.protobuf.Timestamp minimum_object_create_time = 6 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        6, *this_._impl_.minimum_object_create_time_, this_._impl_.minimum_object_create_time_->GetCachedSize(), target,
        stream);
  }

  // string match_glob = 9 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_match_glob().empty()) {
      const ::std::string& _s = this_._internal_match_glob();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.match_glob");
      target = stream->WriteStringMaybeAliased(9, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t IngestionDataSourceSettings_CloudStorage::ByteSizeLong(const MessageLite& base) {
  const IngestionDataSourceSettings_CloudStorage& this_ = static_cast<const IngestionDataSourceSettings_CloudStorage&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t IngestionDataSourceSettings_CloudStorage::ByteSizeLong() const {
  const IngestionDataSourceSettings_CloudStorage& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // string bucket = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_bucket().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_bucket());
      }
    }
    // string match_glob = 9 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_match_glob().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_match_glob());
      }
    }
    // .google.protobuf.Timestamp minimum_object_create_time = 6 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.minimum_object_create_time_);
    }
    // .google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.State state = 1 [(.google.api.field_behavior) = OUTPUT_ONLY];
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_state() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_state());
      }
    }
  }
  switch (this_.input_format_case()) {
    // .google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.TextFormat text_format = 3 [(.google.api.field_behavior) = OPTIONAL];
    case kTextFormat: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.input_format_.text_format_);
      break;
    }
    // .google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.AvroFormat avro_format = 4 [(.google.api.field_behavior) = OPTIONAL];
    case kAvroFormat: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.input_format_.avro_format_);
      break;
    }
    // .google.pubsub.v1.IngestionDataSourceSettings.CloudStorage.PubSubAvroFormat pubsub_avro_format = 5 [(.google.api.field_behavior) = OPTIONAL];
    case kPubsubAvroFormat: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.input_format_.pubsub_avro_format_);
      break;
    }
    case INPUT_FORMAT_NOT_SET: {
      break;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void IngestionDataSourceSettings_CloudStorage::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<IngestionDataSourceSettings_CloudStorage*>(&to_msg);
  auto& from = static_cast<const IngestionDataSourceSettings_CloudStorage&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_bucket().empty()) {
        _this->_internal_set_bucket(from._internal_bucket());
      } else {
        if (_this->_impl_.bucket_.IsDefault()) {
          _this->_internal_set_bucket("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_match_glob().empty()) {
        _this->_internal_set_match_glob(from._internal_match_glob());
      } else {
        if (_this->_impl_.match_glob_.IsDefault()) {
          _this->_internal_set_match_glob("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.minimum_object_create_time_ != nullptr);
      if (_this->_impl_.minimum_object_create_time_ == nullptr) {
        _this->_impl_.minimum_object_create_time_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.minimum_object_create_time_);
      } else {
        _this->_impl_.minimum_object_create_time_->MergeFrom(*from._impl_.minimum_object_create_time_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_state() != 0) {
        _this->_impl_.state_ = from._impl_.state_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[0]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[0];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_input_format();
      }
      _this->_impl_._oneof_case_[0] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kTextFormat: {
        if (oneof_needs_init) {
          _this->_impl_.input_format_.text_format_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.input_format_.text_format_);
        } else {
          _this->_impl_.input_format_.text_format_->MergeFrom(*from._impl_.input_format_.text_format_);
        }
        break;
      }
      case kAvroFormat: {
        if (oneof_needs_init) {
          _this->_impl_.input_format_.avro_format_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.input_format_.avro_format_);
        } else {
          _this->_impl_.input_format_.avro_format_->MergeFrom(*from._impl_.input_format_.avro_format_);
        }
        break;
      }
      case kPubsubAvroFormat: {
        if (oneof_needs_init) {
          _this->_impl_.input_format_.pubsub_avro_format_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.input_format_.pubsub_avro_format_);
        } else {
          _this->_impl_.input_format_.pubsub_avro_format_->MergeFrom(*from._impl_.input_format_.pubsub_avro_format_);
        }
        break;
      }
      case INPUT_FORMAT_NOT_SET:
        break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void IngestionDataSourceSettings_CloudStorage::CopyFrom(const IngestionDataSourceSettings_CloudStorage& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.IngestionDataSourceSettings.CloudStorage)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void IngestionDataSourceSettings_CloudStorage::InternalSwap(IngestionDataSourceSettings_CloudStorage* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.bucket_, &other->_impl_.bucket_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.match_glob_, &other->_impl_.match_glob_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage, _impl_.state_)
      + sizeof(IngestionDataSourceSettings_CloudStorage::_impl_.state_)
      - PROTOBUF_FIELD_OFFSET(IngestionDataSourceSettings_CloudStorage, _impl_.minimum_object_create_time_)>(
          reinterpret_cast<char*>(&_impl_.minimum_object_create_time_),
          reinterpret_cast<char*>(&other->_impl_.minimum_object_create_time_));
  swap(_impl_.input_format_, other->_impl_.input_format_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
}

::google::protobuf::Metadata IngestionDataSourceSettings_CloudStorage::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void IngestionDataSourceSettings::set_allocated_aws_kinesis(::google::pubsub::v1::IngestionDataSourceSettings_AwsKinesis* PROTOBUF_NULLABLE aws_kinesis) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_source();
  if (aws_kinesis) {
    ::google::protobuf::Arena* submessage_arena = aws_kinesis->GetArena();
    if (message_arena != submessage_arena) {
      aws_kinesis = ::google::protobuf::internal::GetOwnedMessage(message_arena, aws_kinesis, submessage_arena);
    }
    set_has_aws_kinesis();
    _impl_.source_.aws_kinesis_ = aws_kinesis;
  }
  // @@protoc_insertion_point(field_set_allocated:google.pubsub.v1.IngestionDataSourceSettings.aws_kinesis)
}
void IngestionDataSourceSettings::set_allocated_cloud_storage(::google::pubsub::v1::IngestionDataSourceSettings_CloudStorage* PROTOBUF_NULLABLE cloud_storage) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_source();
  if (cloud_storage) {
    ::google::protobuf::Arena* submessage_arena = cloud_storage->GetArena();
    if (message_arena != submessage_arena) {
      cloud_storage = ::google::protobuf::internal::GetOwnedMessage(message_arena, cloud_storage, submessage_arena);
    }
    set_has_cloud_storage();
    _impl_.source_.cloud_storage_ = cloud_storage;
  }
  // @@protoc_insertion_point(field_set_allocated:google.pubsub.v1.IngestionDataSourceSettings.cloud_storage)
}
IngestionDataSourceSettings::IngestionDataSourceSettings(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, IngestionDataSourceSettings_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.IngestionDataSourceSettings)
}
PROTOBUF_NDEBUG_INLINE IngestionDataSourceSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::IngestionDataSourceSettings& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        source_{},
        _oneof_case_{from._oneof_case_[0]} {}

IngestionDataSourceSettings::IngestionDataSourceSettings(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const IngestionDataSourceSettings& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, IngestionDataSourceSettings_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  IngestionDataSourceSettings* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.platform_logs_settings_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.platform_logs_settings_)
                : nullptr;
  switch (source_case()) {
    case SOURCE_NOT_SET:
      break;
      case kAwsKinesis:
        _impl_.source_.aws_kinesis_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.source_.aws_kinesis_);
        break;
      case kCloudStorage:
        _impl_.source_.cloud_storage_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.source_.cloud_storage_);
        break;
  }

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.IngestionDataSourceSettings)
}
PROTOBUF_NDEBUG_INLINE IngestionDataSourceSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        source_{},
        _oneof_case_{} {}

inline void IngestionDataSourceSettings::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.platform_logs_settings_ = {};
}
IngestionDataSourceSettings::~IngestionDataSourceSettings() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.IngestionDataSourceSettings)
  SharedDtor(*this);
}
inline void IngestionDataSourceSettings::SharedDtor(MessageLite& self) {
  IngestionDataSourceSettings& this_ = static_cast<IngestionDataSourceSettings&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.platform_logs_settings_;
  if (this_.has_source()) {
    this_.clear_source();
  }
  this_._impl_.~Impl_();
}

void IngestionDataSourceSettings::clear_source() {
// @@protoc_insertion_point(one_of_clear_start:google.pubsub.v1.IngestionDataSourceSettings)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (source_case()) {
    case kAwsKinesis: {
      if (GetArena() == nullptr) {
        delete _impl_.source_.aws_kinesis_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.source_.aws_kinesis_);
      }
      break;
    }
    case kCloudStorage: {
      if (GetArena() == nullptr) {
        delete _impl_.source_.cloud_storage_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.source_.cloud_storage_);
      }
      break;
    }
    case SOURCE_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = SOURCE_NOT_SET;
}


#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull IngestionDataSourceSettings_class_data_ =
        IngestionDataSourceSettings::InternalGenerateClassData_(IngestionDataSourceSettings_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
IngestionDataSourceSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&IngestionDataSourceSettings_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(IngestionDataSourceSettings_class_data_.tc_table);
  return IngestionDataSourceSettings_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
IngestionDataSourceSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&IngestionDataSourceSettings_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&IngestionDataSourceSettings_globals_));
  return IngestionDataSourceSettings_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const IngestionDataSourceSettings::ParseTableT_
    IngestionDataSourceSettings::_table_ =
        IngestionDataSourceSettings::InternalGenerateParseTable_(IngestionDataSourceSettings_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void IngestionDataSourceSettings::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.IngestionDataSourceSettings)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(_impl_.platform_logs_settings_ != nullptr);
    _impl_.platform_logs_settings_->Clear();
  }
  clear_source();
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL IngestionDataSourceSettings::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const IngestionDataSourceSettings& this_ = static_cast<const IngestionDataSourceSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL IngestionDataSourceSettings::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const IngestionDataSourceSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.IngestionDataSourceSettings)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  switch (this_.source_case()) {
    case kAwsKinesis: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          1, *this_._impl_.source_.aws_kinesis_, this_._impl_.source_.aws_kinesis_->GetCachedSize(), target,
          stream);
      break;
    }
    case kCloudStorage: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          2, *this_._impl_.source_.cloud_storage_, this_._impl_.source_.cloud_storage_->GetCachedSize(), target,
          stream);
      break;
    }
    default:
      break;
  }
  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.pubsub.v1.PlatformLogsSettings platform_logs_settings = 4 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        4, *this_._impl_.platform_logs_settings_, this_._impl_.platform_logs_settings_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.IngestionDataSourceSettings)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t IngestionDataSourceSettings::ByteSizeLong(const MessageLite& base) {
  const IngestionDataSourceSettings& this_ = static_cast<const IngestionDataSourceSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t IngestionDataSourceSettings::ByteSizeLong() const {
  const IngestionDataSourceSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.IngestionDataSourceSettings)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // .google.pubsub.v1.PlatformLogsSettings platform_logs_settings = 4 [(.google.api.field_behavior) = OPTIONAL];
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.platform_logs_settings_);
    }
  }
  switch (this_.source_case()) {
    // .google.pubsub.v1.IngestionDataSourceSettings.AwsKinesis aws_kinesis = 1 [(.google.api.field_behavior) = OPTIONAL];
    case kAwsKinesis: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.source_.aws_kinesis_);
      break;
    }
    // .google.pubsub.v1.IngestionDataSourceSettings.CloudStorage cloud_storage = 2 [(.google.api.field_behavior) = OPTIONAL];
    case kCloudStorage: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.source_.cloud_storage_);
      break;
    }
    case SOURCE_NOT_SET: {
      break;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void IngestionDataSourceSettings::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<IngestionDataSourceSettings*>(&to_msg);
  auto& from = static_cast<const IngestionDataSourceSettings&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.IngestionDataSourceSettings)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(from._impl_.platform_logs_settings_ != nullptr);
    if (_this->_impl_.platform_logs_settings_ == nullptr) {
      _this->_impl_.platform_logs_settings_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.platform_logs_settings_);
    } else {
      _this->_impl_.platform_logs_settings_->MergeFrom(*from._impl_.platform_logs_settings_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[0]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[0];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_source();
      }
      _this->_impl_._oneof_case_[0] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kAwsKinesis: {
        if (oneof_needs_init) {
          _this->_impl_.source_.aws_kinesis_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.source_.aws_kinesis_);
        } else {
          _this->_impl_.source_.aws_kinesis_->MergeFrom(*from._impl_.source_.aws_kinesis_);
        }
        break;
      }
      case kCloudStorage: {
        if (oneof_needs_init) {
          _this->_impl_.source_.cloud_storage_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.source_.cloud_storage_);
        } else {
          _this->_impl_.source_.cloud_storage_->MergeFrom(*from._impl_.source_.cloud_storage_);
        }
        break;
      }
      case SOURCE_NOT_SET:
        break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void IngestionDataSourceSettings::CopyFrom(const IngestionDataSourceSettings& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.IngestionDataSourceSettings)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void IngestionDataSourceSettings::InternalSwap(IngestionDataSourceSettings* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.platform_logs_settings_, other->_impl_.platform_logs_settings_);
  swap(_impl_.source_, other->_impl_.source_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
}

::google::protobuf::Metadata IngestionDataSourceSettings::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

PlatformLogsSettings::PlatformLogsSettings(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PlatformLogsSettings_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.PlatformLogsSettings)
}
PlatformLogsSettings::PlatformLogsSettings(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PlatformLogsSettings& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PlatformLogsSettings_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE PlatformLogsSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void PlatformLogsSettings::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.severity_ = {};
}
PlatformLogsSettings::~PlatformLogsSettings() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.PlatformLogsSettings)
  SharedDtor(*this);
}
inline void PlatformLogsSettings::SharedDtor(MessageLite& self) {
  PlatformLogsSettings& this_ = static_cast<PlatformLogsSettings&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull PlatformLogsSettings_class_data_ =
        PlatformLogsSettings::InternalGenerateClassData_(PlatformLogsSettings_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PlatformLogsSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PlatformLogsSettings_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(PlatformLogsSettings_class_data_.tc_table);
  return PlatformLogsSettings_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PlatformLogsSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PlatformLogsSettings_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&PlatformLogsSettings_globals_));
  return PlatformLogsSettings_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const PlatformLogsSettings::ParseTableT_
    PlatformLogsSettings::_table_ =
        PlatformLogsSettings::InternalGenerateParseTable_(PlatformLogsSettings_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void PlatformLogsSettings::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.PlatformLogsSettings)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.severity_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL PlatformLogsSettings::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const PlatformLogsSettings& this_ = static_cast<const PlatformLogsSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL PlatformLogsSettings::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const PlatformLogsSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.PlatformLogsSettings)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.pubsub.v1.PlatformLogsSettings.Severity severity = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_severity() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          1, this_._internal_severity(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.PlatformLogsSettings)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t PlatformLogsSettings::ByteSizeLong(const MessageLite& base) {
  const PlatformLogsSettings& this_ = static_cast<const PlatformLogsSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t PlatformLogsSettings::ByteSizeLong() const {
  const PlatformLogsSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.PlatformLogsSettings)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // .google.pubsub.v1.PlatformLogsSettings.Severity severity = 1 [(.google.api.field_behavior) = OPTIONAL];
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_severity() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_severity());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void PlatformLogsSettings::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<PlatformLogsSettings*>(&to_msg);
  auto& from = static_cast<const PlatformLogsSettings&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.PlatformLogsSettings)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (from._internal_severity() != 0) {
      _this->_impl_.severity_ = from._impl_.severity_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void PlatformLogsSettings::CopyFrom(const PlatformLogsSettings& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.PlatformLogsSettings)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void PlatformLogsSettings::InternalSwap(PlatformLogsSettings* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.severity_, other->_impl_.severity_);
}

::google::protobuf::Metadata PlatformLogsSettings::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

IngestionFailureEvent_ApiViolationReason::IngestionFailureEvent_ApiViolationReason(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, IngestionFailureEvent_ApiViolationReason_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.IngestionFailureEvent.ApiViolationReason)
}
IngestionFailureEvent_ApiViolationReason::IngestionFailureEvent_ApiViolationReason(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const IngestionFailureEvent_ApiViolationReason& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, IngestionFailureEvent_ApiViolationReason_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  IngestionFailureEvent_ApiViolationReason* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.IngestionFailureEvent.ApiViolationReason)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull IngestionFailureEvent_ApiViolationReason_class_data_ =
        IngestionFailureEvent_ApiViolationReason::InternalGenerateClassData_(IngestionFailureEvent_ApiViolationReason_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
IngestionFailureEvent_ApiViolationReason::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&IngestionFailureEvent_ApiViolationReason_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(IngestionFailureEvent_ApiViolationReason_class_data_.tc_table);
  return IngestionFailureEvent_ApiViolationReason_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
IngestionFailureEvent_ApiViolationReason::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&IngestionFailureEvent_ApiViolationReason_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&IngestionFailureEvent_ApiViolationReason_globals_));
  return IngestionFailureEvent_ApiViolationReason_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const IngestionFailureEvent_ApiViolationReason::ParseTableT_
    IngestionFailureEvent_ApiViolationReason::_table_ =
        IngestionFailureEvent_ApiViolationReason::InternalGenerateParseTable_(IngestionFailureEvent_ApiViolationReason_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata IngestionFailureEvent_ApiViolationReason::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

IngestionFailureEvent_AvroFailureReason::IngestionFailureEvent_AvroFailureReason(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, IngestionFailureEvent_AvroFailureReason_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.IngestionFailureEvent.AvroFailureReason)
}
IngestionFailureEvent_AvroFailureReason::IngestionFailureEvent_AvroFailureReason(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const IngestionFailureEvent_AvroFailureReason& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, IngestionFailureEvent_AvroFailureReason_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  IngestionFailureEvent_AvroFailureReason* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.IngestionFailureEvent.AvroFailureReason)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull IngestionFailureEvent_AvroFailureReason_class_data_ =
        IngestionFailureEvent_AvroFailureReason::InternalGenerateClassData_(IngestionFailureEvent_AvroFailureReason_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
IngestionFailureEvent_AvroFailureReason::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&IngestionFailureEvent_AvroFailureReason_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(IngestionFailureEvent_AvroFailureReason_class_data_.tc_table);
  return IngestionFailureEvent_AvroFailureReason_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
IngestionFailureEvent_AvroFailureReason::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&IngestionFailureEvent_AvroFailureReason_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&IngestionFailureEvent_AvroFailureReason_globals_));
  return IngestionFailureEvent_AvroFailureReason_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const IngestionFailureEvent_AvroFailureReason::ParseTableT_
    IngestionFailureEvent_AvroFailureReason::_table_ =
        IngestionFailureEvent_AvroFailureReason::InternalGenerateParseTable_(IngestionFailureEvent_AvroFailureReason_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata IngestionFailureEvent_AvroFailureReason::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void IngestionFailureEvent_CloudStorageFailure::set_allocated_avro_failure_reason(::google::pubsub::v1::IngestionFailureEvent_AvroFailureReason* PROTOBUF_NULLABLE avro_failure_reason) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_reason();
  if (avro_failure_reason) {
    ::google::protobuf::Arena* submessage_arena = avro_failure_reason->GetArena();
    if (message_arena != submessage_arena) {
      avro_failure_reason = ::google::protobuf::internal::GetOwnedMessage(message_arena, avro_failure_reason, submessage_arena);
    }
    set_has_avro_failure_reason();
    _impl_.reason_.avro_failure_reason_ = avro_failure_reason;
  }
  // @@protoc_insertion_point(field_set_allocated:google.pubsub.v1.IngestionFailureEvent.CloudStorageFailure.avro_failure_reason)
}
void IngestionFailureEvent_CloudStorageFailure::set_allocated_api_violation_reason(::google::pubsub::v1::IngestionFailureEvent_ApiViolationReason* PROTOBUF_NULLABLE api_violation_reason) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_reason();
  if (api_violation_reason) {
    ::google::protobuf::Arena* submessage_arena = api_violation_reason->GetArena();
    if (message_arena != submessage_arena) {
      api_violation_reason = ::google::protobuf::internal::GetOwnedMessage(message_arena, api_violation_reason, submessage_arena);
    }
    set_has_api_violation_reason();
    _impl_.reason_.api_violation_reason_ = api_violation_reason;
  }
  // @@protoc_insertion_point(field_set_allocated:google.pubsub.v1.IngestionFailureEvent.CloudStorageFailure.api_violation_reason)
}
IngestionFailureEvent_CloudStorageFailure::IngestionFailureEvent_CloudStorageFailure(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, IngestionFailureEvent_CloudStorageFailure_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.IngestionFailureEvent.CloudStorageFailure)
}
PROTOBUF_NDEBUG_INLINE IngestionFailureEvent_CloudStorageFailure::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::IngestionFailureEvent_CloudStorageFailure& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        bucket_(arena, from.bucket_),
        object_name_(arena, from.object_name_),
        reason_{},
        _oneof_case_{from._oneof_case_[0]} {}

IngestionFailureEvent_CloudStorageFailure::IngestionFailureEvent_CloudStorageFailure(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const IngestionFailureEvent_CloudStorageFailure& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, IngestionFailureEvent_CloudStorageFailure_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  IngestionFailureEvent_CloudStorageFailure* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.object_generation_ = from._impl_.object_generation_;
  switch (reason_case()) {
    case REASON_NOT_SET:
      break;
      case kAvroFailureReason:
        _impl_.reason_.avro_failure_reason_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.reason_.avro_failure_reason_);
        break;
      case kApiViolationReason:
        _impl_.reason_.api_violation_reason_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.reason_.api_violation_reason_);
        break;
  }

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.IngestionFailureEvent.CloudStorageFailure)
}
PROTOBUF_NDEBUG_INLINE IngestionFailureEvent_CloudStorageFailure::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        bucket_(arena),
        object_name_(arena),
        reason_{},
        _oneof_case_{} {}

inline void IngestionFailureEvent_CloudStorageFailure::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.object_generation_ = {};
}
IngestionFailureEvent_CloudStorageFailure::~IngestionFailureEvent_CloudStorageFailure() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.IngestionFailureEvent.CloudStorageFailure)
  SharedDtor(*this);
}
inline void IngestionFailureEvent_CloudStorageFailure::SharedDtor(MessageLite& self) {
  IngestionFailureEvent_CloudStorageFailure& this_ = static_cast<IngestionFailureEvent_CloudStorageFailure&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.bucket_.Destroy();
  this_._impl_.object_name_.Destroy();
  if (this_.has_reason()) {
    this_.clear_reason();
  }
  this_._impl_.~Impl_();
}

void IngestionFailureEvent_CloudStorageFailure::clear_reason() {
// @@protoc_insertion_point(one_of_clear_start:google.pubsub.v1.IngestionFailureEvent.CloudStorageFailure)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (reason_case()) {
    case kAvroFailureReason: {
      if (GetArena() == nullptr) {
        delete _impl_.reason_.avro_failure_reason_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.reason_.avro_failure_reason_);
      }
      break;
    }
    case kApiViolationReason: {
      if (GetArena() == nullptr) {
        delete _impl_.reason_.api_violation_reason_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.reason_.api_violation_reason_);
      }
      break;
    }
    case REASON_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = REASON_NOT_SET;
}


#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull IngestionFailureEvent_CloudStorageFailure_class_data_ =
        IngestionFailureEvent_CloudStorageFailure::InternalGenerateClassData_(IngestionFailureEvent_CloudStorageFailure_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
IngestionFailureEvent_CloudStorageFailure::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&IngestionFailureEvent_CloudStorageFailure_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(IngestionFailureEvent_CloudStorageFailure_class_data_.tc_table);
  return IngestionFailureEvent_CloudStorageFailure_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
IngestionFailureEvent_CloudStorageFailure::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&IngestionFailureEvent_CloudStorageFailure_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&IngestionFailureEvent_CloudStorageFailure_globals_));
  return IngestionFailureEvent_CloudStorageFailure_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const IngestionFailureEvent_CloudStorageFailure::ParseTableT_
    IngestionFailureEvent_CloudStorageFailure::_table_ =
        IngestionFailureEvent_CloudStorageFailure::InternalGenerateParseTable_(IngestionFailureEvent_CloudStorageFailure_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void IngestionFailureEvent_CloudStorageFailure::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.IngestionFailureEvent.CloudStorageFailure)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.bucket_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.object_name_.ClearNonDefaultToEmpty();
    }
  }
  _impl_.object_generation_ = ::int64_t{0};
  clear_reason();
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL IngestionFailureEvent_CloudStorageFailure::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const IngestionFailureEvent_CloudStorageFailure& this_ = static_cast<const IngestionFailureEvent_CloudStorageFailure&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL IngestionFailureEvent_CloudStorageFailure::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const IngestionFailureEvent_CloudStorageFailure& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.IngestionFailureEvent.CloudStorageFailure)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string bucket = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_bucket().empty()) {
      const ::std::string& _s = this_._internal_bucket();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.IngestionFailureEvent.CloudStorageFailure.bucket");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // string object_name = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_object_name().empty()) {
      const ::std::string& _s = this_._internal_object_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.IngestionFailureEvent.CloudStorageFailure.object_name");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  // int64 object_generation = 3 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_object_generation() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<3>(
              stream, this_._internal_object_generation(), target);
    }
  }

  switch (this_.reason_case()) {
    case kAvroFailureReason: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          5, *this_._impl_.reason_.avro_failure_reason_, this_._impl_.reason_.avro_failure_reason_->GetCachedSize(), target,
          stream);
      break;
    }
    case kApiViolationReason: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          6, *this_._impl_.reason_.api_violation_reason_, this_._impl_.reason_.api_violation_reason_->GetCachedSize(), target,
          stream);
      break;
    }
    default:
      break;
  }
  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.IngestionFailureEvent.CloudStorageFailure)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t IngestionFailureEvent_CloudStorageFailure::ByteSizeLong(const MessageLite& base) {
  const IngestionFailureEvent_CloudStorageFailure& this_ = static_cast<const IngestionFailureEvent_CloudStorageFailure&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t IngestionFailureEvent_CloudStorageFailure::ByteSizeLong() const {
  const IngestionFailureEvent_CloudStorageFailure& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.IngestionFailureEvent.CloudStorageFailure)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // string bucket = 1 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_bucket().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_bucket());
      }
    }
    // string object_name = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_object_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_object_name());
      }
    }
    // int64 object_generation = 3 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_object_generation() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_object_generation());
      }
    }
  }
  switch (this_.reason_case()) {
    // .google.pubsub.v1.IngestionFailureEvent.AvroFailureReason avro_failure_reason = 5 [(.google.api.field_behavior) = OPTIONAL];
    case kAvroFailureReason: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.reason_.avro_failure_reason_);
      break;
    }
    // .google.pubsub.v1.IngestionFailureEvent.ApiViolationReason api_violation_reason = 6 [(.google.api.field_behavior) = OPTIONAL];
    case kApiViolationReason: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.reason_.api_violation_reason_);
      break;
    }
    case REASON_NOT_SET: {
      break;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void IngestionFailureEvent_CloudStorageFailure::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<IngestionFailureEvent_CloudStorageFailure*>(&to_msg);
  auto& from = static_cast<const IngestionFailureEvent_CloudStorageFailure&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.IngestionFailureEvent.CloudStorageFailure)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_bucket().empty()) {
        _this->_internal_set_bucket(from._internal_bucket());
      } else {
        if (_this->_impl_.bucket_.IsDefault()) {
          _this->_internal_set_bucket("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_object_name().empty()) {
        _this->_internal_set_object_name(from._internal_object_name());
      } else {
        if (_this->_impl_.object_name_.IsDefault()) {
          _this->_internal_set_object_name("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_object_generation() != 0) {
        _this->_impl_.object_generation_ = from._impl_.object_generation_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[0]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[0];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_reason();
      }
      _this->_impl_._oneof_case_[0] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kAvroFailureReason: {
        if (oneof_needs_init) {
          _this->_impl_.reason_.avro_failure_reason_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.reason_.avro_failure_reason_);
        } else {
          _this->_impl_.reason_.avro_failure_reason_->MergeFrom(*from._impl_.reason_.avro_failure_reason_);
        }
        break;
      }
      case kApiViolationReason: {
        if (oneof_needs_init) {
          _this->_impl_.reason_.api_violation_reason_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.reason_.api_violation_reason_);
        } else {
          _this->_impl_.reason_.api_violation_reason_->MergeFrom(*from._impl_.reason_.api_violation_reason_);
        }
        break;
      }
      case REASON_NOT_SET:
        break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void IngestionFailureEvent_CloudStorageFailure::CopyFrom(const IngestionFailureEvent_CloudStorageFailure& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.IngestionFailureEvent.CloudStorageFailure)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void IngestionFailureEvent_CloudStorageFailure::InternalSwap(IngestionFailureEvent_CloudStorageFailure* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.bucket_, &other->_impl_.bucket_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.object_name_, &other->_impl_.object_name_, arena);
  swap(_impl_.object_generation_, other->_impl_.object_generation_);
  swap(_impl_.reason_, other->_impl_.reason_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
}

::google::protobuf::Metadata IngestionFailureEvent_CloudStorageFailure::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void IngestionFailureEvent::set_allocated_cloud_storage_failure(::google::pubsub::v1::IngestionFailureEvent_CloudStorageFailure* PROTOBUF_NULLABLE cloud_storage_failure) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_failure();
  if (cloud_storage_failure) {
    ::google::protobuf::Arena* submessage_arena = cloud_storage_failure->GetArena();
    if (message_arena != submessage_arena) {
      cloud_storage_failure = ::google::protobuf::internal::GetOwnedMessage(message_arena, cloud_storage_failure, submessage_arena);
    }
    set_has_cloud_storage_failure();
    _impl_.failure_.cloud_storage_failure_ = cloud_storage_failure;
  }
  // @@protoc_insertion_point(field_set_allocated:google.pubsub.v1.IngestionFailureEvent.cloud_storage_failure)
}
IngestionFailureEvent::IngestionFailureEvent(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, IngestionFailureEvent_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.IngestionFailureEvent)
}
PROTOBUF_NDEBUG_INLINE IngestionFailureEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::IngestionFailureEvent& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        topic_(arena, from.topic_),
        error_message_(arena, from.error_message_),
        failure_{},
        _oneof_case_{from._oneof_case_[0]} {}

IngestionFailureEvent::IngestionFailureEvent(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const IngestionFailureEvent& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, IngestionFailureEvent_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  IngestionFailureEvent* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  switch (failure_case()) {
    case FAILURE_NOT_SET:
      break;
      case kCloudStorageFailure:
        _impl_.failure_.cloud_storage_failure_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.failure_.cloud_storage_failure_);
        break;
  }

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.IngestionFailureEvent)
}
PROTOBUF_NDEBUG_INLINE IngestionFailureEvent::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        topic_(arena),
        error_message_(arena),
        failure_{},
        _oneof_case_{} {}

inline void IngestionFailureEvent::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
IngestionFailureEvent::~IngestionFailureEvent() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.IngestionFailureEvent)
  SharedDtor(*this);
}
inline void IngestionFailureEvent::SharedDtor(MessageLite& self) {
  IngestionFailureEvent& this_ = static_cast<IngestionFailureEvent&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.topic_.Destroy();
  this_._impl_.error_message_.Destroy();
  if (this_.has_failure()) {
    this_.clear_failure();
  }
  this_._impl_.~Impl_();
}

void IngestionFailureEvent::clear_failure() {
// @@protoc_insertion_point(one_of_clear_start:google.pubsub.v1.IngestionFailureEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (failure_case()) {
    case kCloudStorageFailure: {
      if (GetArena() == nullptr) {
        delete _impl_.failure_.cloud_storage_failure_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.failure_.cloud_storage_failure_);
      }
      break;
    }
    case FAILURE_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = FAILURE_NOT_SET;
}


#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull IngestionFailureEvent_class_data_ =
        IngestionFailureEvent::InternalGenerateClassData_(IngestionFailureEvent_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
IngestionFailureEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&IngestionFailureEvent_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(IngestionFailureEvent_class_data_.tc_table);
  return IngestionFailureEvent_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
IngestionFailureEvent::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&IngestionFailureEvent_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&IngestionFailureEvent_globals_));
  return IngestionFailureEvent_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const IngestionFailureEvent::ParseTableT_
    IngestionFailureEvent::_table_ =
        IngestionFailureEvent::InternalGenerateParseTable_(IngestionFailureEvent_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void IngestionFailureEvent::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.IngestionFailureEvent)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.topic_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.error_message_.ClearNonDefaultToEmpty();
    }
  }
  clear_failure();
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL IngestionFailureEvent::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const IngestionFailureEvent& this_ = static_cast<const IngestionFailureEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL IngestionFailureEvent::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const IngestionFailureEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.IngestionFailureEvent)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string topic = 1 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_topic().empty()) {
      const ::std::string& _s = this_._internal_topic();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.IngestionFailureEvent.topic");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // string error_message = 2 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_error_message().empty()) {
      const ::std::string& _s = this_._internal_error_message();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.IngestionFailureEvent.error_message");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  // .google.pubsub.v1.IngestionFailureEvent.CloudStorageFailure cloud_storage_failure = 3 [(.google.api.field_behavior) = OPTIONAL];
  if (this_.failure_case() == kCloudStorageFailure) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        3, *this_._impl_.failure_.cloud_storage_failure_, this_._impl_.failure_.cloud_storage_failure_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.IngestionFailureEvent)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t IngestionFailureEvent::ByteSizeLong(const MessageLite& base) {
  const IngestionFailureEvent& this_ = static_cast<const IngestionFailureEvent&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t IngestionFailureEvent::ByteSizeLong() const {
  const IngestionFailureEvent& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.IngestionFailureEvent)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string topic = 1 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_topic().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_topic());
      }
    }
    // string error_message = 2 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_error_message().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_error_message());
      }
    }
  }
  switch (this_.failure_case()) {
    // .google.pubsub.v1.IngestionFailureEvent.CloudStorageFailure cloud_storage_failure = 3 [(.google.api.field_behavior) = OPTIONAL];
    case kCloudStorageFailure: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.failure_.cloud_storage_failure_);
      break;
    }
    case FAILURE_NOT_SET: {
      break;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void IngestionFailureEvent::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<IngestionFailureEvent*>(&to_msg);
  auto& from = static_cast<const IngestionFailureEvent&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.IngestionFailureEvent)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_topic().empty()) {
        _this->_internal_set_topic(from._internal_topic());
      } else {
        if (_this->_impl_.topic_.IsDefault()) {
          _this->_internal_set_topic("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_error_message().empty()) {
        _this->_internal_set_error_message(from._internal_error_message());
      } else {
        if (_this->_impl_.error_message_.IsDefault()) {
          _this->_internal_set_error_message("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[0]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[0];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_failure();
      }
      _this->_impl_._oneof_case_[0] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kCloudStorageFailure: {
        if (oneof_needs_init) {
          _this->_impl_.failure_.cloud_storage_failure_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.failure_.cloud_storage_failure_);
        } else {
          _this->_impl_.failure_.cloud_storage_failure_->MergeFrom(*from._impl_.failure_.cloud_storage_failure_);
        }
        break;
      }
      case FAILURE_NOT_SET:
        break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void IngestionFailureEvent::CopyFrom(const IngestionFailureEvent& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.IngestionFailureEvent)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void IngestionFailureEvent::InternalSwap(IngestionFailureEvent* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.topic_, &other->_impl_.topic_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.error_message_, &other->_impl_.error_message_, arena);
  swap(_impl_.failure_, other->_impl_.failure_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
}

::google::protobuf::Metadata IngestionFailureEvent::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

#if defined(PROTOBUF_CUSTOM_VTABLE)
Topic_LabelsEntry_DoNotUse::Topic_LabelsEntry_DoNotUse() : SuperType(Topic_LabelsEntry_DoNotUse_get_class_data()) {}
Topic_LabelsEntry_DoNotUse::Topic_LabelsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
    : SuperType(arena, Topic_LabelsEntry_DoNotUse_get_class_data()) {}
#else   // PROTOBUF_CUSTOM_VTABLE
Topic_LabelsEntry_DoNotUse::Topic_LabelsEntry_DoNotUse() : SuperType() {}
Topic_LabelsEntry_DoNotUse::Topic_LabelsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena) : SuperType(arena) {}
#endif  // PROTOBUF_CUSTOM_VTABLE
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Topic_LabelsEntry_DoNotUse_class_data_ =
        Topic_LabelsEntry_DoNotUse::InternalGenerateClassData_(Topic_LabelsEntry_DoNotUse_globals_._default);

// including ~ all the vtables and everything they use become part
// of the same ~ SCC.
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Topic_LabelsEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Topic_LabelsEntry_DoNotUse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Topic_LabelsEntry_DoNotUse_class_data_.tc_table);
  return Topic_LabelsEntry_DoNotUse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Topic_LabelsEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Topic_LabelsEntry_DoNotUse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Topic_LabelsEntry_DoNotUse_globals_));
  return Topic_LabelsEntry_DoNotUse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Topic_LabelsEntry_DoNotUse::ParseTableT_
    Topic_LabelsEntry_DoNotUse::_table_ =
        Topic_LabelsEntry_DoNotUse::InternalGenerateParseTable_(Topic_LabelsEntry_DoNotUse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
// ===================================================================

void Topic::clear_message_retention_duration() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.message_retention_duration_ != nullptr) _impl_.message_retention_duration_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
Topic::Topic(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Topic_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.Topic)
}
PROTOBUF_NDEBUG_INLINE Topic::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::Topic& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        name_(arena, from.name_),
        kms_key_name_(arena, from.kms_key_name_),
        labels_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::Topic,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Topic, _impl_.labels_)>()
          , from.labels_
        }
     {}

Topic::Topic(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Topic& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Topic_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Topic* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.message_storage_policy_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.message_storage_policy_)
                : nullptr;
  _impl_.schema_settings_ = (CheckHasBit(cached_has_bits, 0x00000008U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.schema_settings_)
                : nullptr;
  _impl_.message_retention_duration_ = (CheckHasBit(cached_has_bits, 0x00000010U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.message_retention_duration_)
                : nullptr;
  _impl_.ingestion_data_source_settings_ = (CheckHasBit(cached_has_bits, 0x00000020U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.ingestion_data_source_settings_)
                : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, satisfies_pzs_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, satisfies_pzs_),
           offsetof(Impl_, state_) -
               offsetof(Impl_, satisfies_pzs_) +
               sizeof(Impl_::state_));

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.Topic)
}
PROTOBUF_NDEBUG_INLINE Topic::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        name_(arena),
        kms_key_name_(arena),
        labels_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::Topic,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Topic, _impl_.labels_)>()
         }
     {}

inline void Topic::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, message_storage_policy_),
           0,
           offsetof(Impl_, state_) -
               offsetof(Impl_, message_storage_policy_) +
               sizeof(Impl_::state_));
}
Topic::~Topic() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.Topic)
  SharedDtor(*this);
}
inline void Topic::SharedDtor(MessageLite& self) {
  Topic& this_ = static_cast<Topic&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.name_.Destroy();
  this_._impl_.kms_key_name_.Destroy();
  delete this_._impl_.message_storage_policy_;
  delete this_._impl_.schema_settings_;
  delete this_._impl_.message_retention_duration_;
  delete this_._impl_.ingestion_data_source_settings_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Topic_class_data_ =
        Topic::InternalGenerateClassData_(Topic_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Topic::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Topic_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Topic_class_data_.tc_table);
  return Topic_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Topic::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Topic_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Topic_globals_));
  return Topic_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Topic::ParseTableT_
    Topic::_table_ =
        Topic::InternalGenerateParseTable_(Topic_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Topic::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.Topic)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.name_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.kms_key_name_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(_impl_.message_storage_policy_ != nullptr);
      _impl_.message_storage_policy_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(_impl_.schema_settings_ != nullptr);
      _impl_.schema_settings_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      ABSL_DCHECK(_impl_.message_retention_duration_ != nullptr);
      _impl_.message_retention_duration_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      ABSL_DCHECK(_impl_.ingestion_data_source_settings_ != nullptr);
      _impl_.ingestion_data_source_settings_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x000000c0U)) {
    ::memset(&_impl_.satisfies_pzs_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.state_) -
        reinterpret_cast<char*>(&_impl_.satisfies_pzs_)) + sizeof(_impl_.state_));
  }
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    _impl_.labels_.Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Topic::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Topic& this_ = static_cast<const Topic&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Topic::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Topic& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.Topic)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string name = 1 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_name().empty()) {
      const ::std::string& _s = this_._internal_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Topic.name");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // map<string, string> labels = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    if (!this_._internal_labels().empty()) {
      using MapType = ::google::protobuf::Map<::std::string, ::std::string>;
      using WireHelper = _pbi::MapEntryFuncs<::std::string, ::std::string,
                                     _pbi::WireFormatLite::TYPE_STRING,
                                     _pbi::WireFormatLite::TYPE_STRING>;
      const auto& field = this_._internal_labels();

      if (stream->IsSerializationDeterministic() && field.size() > 1) {
        for (const auto& entry : ::google::protobuf::internal::MapSorterPtr<MapType>(field)) {
          target = WireHelper::InternalSerialize(
              2, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Topic.labels");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Topic.labels");
        }
      } else {
        for (const auto& entry : field) {
          target = WireHelper::InternalSerialize(
              2, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Topic.labels");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Topic.labels");
        }
      }
    }
  }

  // .google.pubsub.v1.MessageStoragePolicy message_storage_policy = 3 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        3, *this_._impl_.message_storage_policy_, this_._impl_.message_storage_policy_->GetCachedSize(), target,
        stream);
  }

  // string kms_key_name = 5 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_kms_key_name().empty()) {
      const ::std::string& _s = this_._internal_kms_key_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Topic.kms_key_name");
      target = stream->WriteStringMaybeAliased(5, _s, target);
    }
  }

  // .google.pubsub.v1.SchemaSettings schema_settings = 6 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        6, *this_._impl_.schema_settings_, this_._impl_.schema_settings_->GetCachedSize(), target,
        stream);
  }

  // bool satisfies_pzs = 7 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_satisfies_pzs() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          7, this_._internal_satisfies_pzs(), target);
    }
  }

  // .google.protobuf.Duration message_retention_duration = 8 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        8, *this_._impl_.message_retention_duration_, this_._impl_.message_retention_duration_->GetCachedSize(), target,
        stream);
  }

  // .google.pubsub.v1.Topic.State state = 9 [(.google.api.field_behavior) = OUTPUT_ONLY];
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    if (this_._internal_state() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          9, this_._internal_state(), target);
    }
  }

  // .google.pubsub.v1.IngestionDataSourceSettings ingestion_data_source_settings = 10 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        10, *this_._impl_.ingestion_data_source_settings_, this_._impl_.ingestion_data_source_settings_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.Topic)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Topic::ByteSizeLong(const MessageLite& base) {
  const Topic& this_ = static_cast<const Topic&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Topic::ByteSizeLong() const {
  const Topic& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.Topic)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // string name = 1 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_name());
      }
    }
    // string kms_key_name = 5 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_kms_key_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_kms_key_name());
      }
    }
    // .google.pubsub.v1.MessageStoragePolicy message_storage_policy = 3 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.message_storage_policy_);
    }
    // .google.pubsub.v1.SchemaSettings schema_settings = 6 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.schema_settings_);
    }
    // .google.protobuf.Duration message_retention_duration = 8 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.message_retention_duration_);
    }
    // .google.pubsub.v1.IngestionDataSourceSettings ingestion_data_source_settings = 10 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.ingestion_data_source_settings_);
    }
    // bool satisfies_pzs = 7 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_satisfies_pzs() != 0) {
        total_size += 2;
      }
    }
    // .google.pubsub.v1.Topic.State state = 9 [(.google.api.field_behavior) = OUTPUT_ONLY];
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (this_._internal_state() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_state());
      }
    }
  }
   {
    // map<string, string> labels = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_labels_size());
      for (const auto& entry : this_._internal_labels()) {
        total_size += _pbi::MapEntryFuncs<::std::string, ::std::string,
                                       _pbi::WireFormatLite::TYPE_STRING,
                                       _pbi::WireFormatLite::TYPE_STRING>::ByteSizeLong(entry.first, entry.second);
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Topic::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Topic*>(&to_msg);
  auto& from = static_cast<const Topic&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.Topic)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_name().empty()) {
        _this->_internal_set_name(from._internal_name());
      } else {
        if (_this->_impl_.name_.IsDefault()) {
          _this->_internal_set_name("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_kms_key_name().empty()) {
        _this->_internal_set_kms_key_name(from._internal_kms_key_name());
      } else {
        if (_this->_impl_.kms_key_name_.IsDefault()) {
          _this->_internal_set_kms_key_name("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.message_storage_policy_ != nullptr);
      if (_this->_impl_.message_storage_policy_ == nullptr) {
        _this->_impl_.message_storage_policy_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.message_storage_policy_);
      } else {
        _this->_impl_.message_storage_policy_->MergeFrom(*from._impl_.message_storage_policy_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(from._impl_.schema_settings_ != nullptr);
      if (_this->_impl_.schema_settings_ == nullptr) {
        _this->_impl_.schema_settings_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.schema_settings_);
      } else {
        _this->_impl_.schema_settings_->MergeFrom(*from._impl_.schema_settings_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      ABSL_DCHECK(from._impl_.message_retention_duration_ != nullptr);
      if (_this->_impl_.message_retention_duration_ == nullptr) {
        _this->_impl_.message_retention_duration_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.message_retention_duration_);
      } else {
        _this->_impl_.message_retention_duration_->MergeFrom(*from._impl_.message_retention_duration_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      ABSL_DCHECK(from._impl_.ingestion_data_source_settings_ != nullptr);
      if (_this->_impl_.ingestion_data_source_settings_ == nullptr) {
        _this->_impl_.ingestion_data_source_settings_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.ingestion_data_source_settings_);
      } else {
        _this->_impl_.ingestion_data_source_settings_->MergeFrom(*from._impl_.ingestion_data_source_settings_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_satisfies_pzs() != 0) {
        _this->_impl_.satisfies_pzs_ = from._impl_.satisfies_pzs_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (from._internal_state() != 0) {
        _this->_impl_.state_ = from._impl_.state_;
      }
    }
  }
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    _this->_impl_.labels_.MergeFrom(from._impl_.labels_);
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Topic::CopyFrom(const Topic& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.Topic)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Topic::InternalSwap(Topic* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.name_, &other->_impl_.name_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.kms_key_name_, &other->_impl_.kms_key_name_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Topic, _impl_.state_)
      + sizeof(Topic::_impl_.state_)
      - PROTOBUF_FIELD_OFFSET(Topic, _impl_.message_storage_policy_)>(
          reinterpret_cast<char*>(&_impl_.message_storage_policy_),
          reinterpret_cast<char*>(&other->_impl_.message_storage_policy_));
  _impl_.labels_.InternalSwap(&other->_impl_.labels_);
}

::google::protobuf::Metadata Topic::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

#if defined(PROTOBUF_CUSTOM_VTABLE)
PubsubMessage_AttributesEntry_DoNotUse::PubsubMessage_AttributesEntry_DoNotUse() : SuperType(PubsubMessage_AttributesEntry_DoNotUse_get_class_data()) {}
PubsubMessage_AttributesEntry_DoNotUse::PubsubMessage_AttributesEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
    : SuperType(arena, PubsubMessage_AttributesEntry_DoNotUse_get_class_data()) {}
#else   // PROTOBUF_CUSTOM_VTABLE
PubsubMessage_AttributesEntry_DoNotUse::PubsubMessage_AttributesEntry_DoNotUse() : SuperType() {}
PubsubMessage_AttributesEntry_DoNotUse::PubsubMessage_AttributesEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena) : SuperType(arena) {}
#endif  // PROTOBUF_CUSTOM_VTABLE
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull PubsubMessage_AttributesEntry_DoNotUse_class_data_ =
        PubsubMessage_AttributesEntry_DoNotUse::InternalGenerateClassData_(PubsubMessage_AttributesEntry_DoNotUse_globals_._default);

// including ~ all the vtables and everything they use become part
// of the same ~ SCC.
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PubsubMessage_AttributesEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PubsubMessage_AttributesEntry_DoNotUse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(PubsubMessage_AttributesEntry_DoNotUse_class_data_.tc_table);
  return PubsubMessage_AttributesEntry_DoNotUse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PubsubMessage_AttributesEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PubsubMessage_AttributesEntry_DoNotUse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&PubsubMessage_AttributesEntry_DoNotUse_globals_));
  return PubsubMessage_AttributesEntry_DoNotUse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const PubsubMessage_AttributesEntry_DoNotUse::ParseTableT_
    PubsubMessage_AttributesEntry_DoNotUse::_table_ =
        PubsubMessage_AttributesEntry_DoNotUse::InternalGenerateParseTable_(PubsubMessage_AttributesEntry_DoNotUse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
// ===================================================================

void PubsubMessage::clear_publish_time() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.publish_time_ != nullptr) _impl_.publish_time_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
PubsubMessage::PubsubMessage(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PubsubMessage_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.PubsubMessage)
}
PROTOBUF_NDEBUG_INLINE PubsubMessage::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::PubsubMessage& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        data_(arena, from.data_),
        message_id_(arena, from.message_id_),
        ordering_key_(arena, from.ordering_key_),
        attributes_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::PubsubMessage,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PubsubMessage, _impl_.attributes_)>()
          , from.attributes_
        }
     {}

PubsubMessage::PubsubMessage(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const PubsubMessage& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PubsubMessage_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  PubsubMessage* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.publish_time_ = (CheckHasBit(cached_has_bits, 0x00000008U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.publish_time_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.PubsubMessage)
}
PROTOBUF_NDEBUG_INLINE PubsubMessage::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        data_(arena),
        message_id_(arena),
        ordering_key_(arena),
        attributes_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::PubsubMessage,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PubsubMessage, _impl_.attributes_)>()
         }
     {}

inline void PubsubMessage::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.publish_time_ = {};
}
PubsubMessage::~PubsubMessage() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.PubsubMessage)
  SharedDtor(*this);
}
inline void PubsubMessage::SharedDtor(MessageLite& self) {
  PubsubMessage& this_ = static_cast<PubsubMessage&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.data_.Destroy();
  this_._impl_.message_id_.Destroy();
  this_._impl_.ordering_key_.Destroy();
  delete this_._impl_.publish_time_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull PubsubMessage_class_data_ =
        PubsubMessage::InternalGenerateClassData_(PubsubMessage_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PubsubMessage::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PubsubMessage_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(PubsubMessage_class_data_.tc_table);
  return PubsubMessage_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PubsubMessage::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PubsubMessage_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&PubsubMessage_globals_));
  return PubsubMessage_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const PubsubMessage::ParseTableT_
    PubsubMessage::_table_ =
        PubsubMessage::InternalGenerateParseTable_(PubsubMessage_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void PubsubMessage::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.PubsubMessage)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.data_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.message_id_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.ordering_key_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(_impl_.publish_time_ != nullptr);
      _impl_.publish_time_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      _impl_.attributes_.Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL PubsubMessage::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const PubsubMessage& this_ = static_cast<const PubsubMessage&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL PubsubMessage::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const PubsubMessage& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.PubsubMessage)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // bytes data = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_data().empty()) {
      const ::std::string& _s = this_._internal_data();
      target = stream->WriteBytesMaybeAliased(1, _s, target);
    }
  }

  // map<string, string> attributes = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (!this_._internal_attributes().empty()) {
      using MapType = ::google::protobuf::Map<::std::string, ::std::string>;
      using WireHelper = _pbi::MapEntryFuncs<::std::string, ::std::string,
                                     _pbi::WireFormatLite::TYPE_STRING,
                                     _pbi::WireFormatLite::TYPE_STRING>;
      const auto& field = this_._internal_attributes();

      if (stream->IsSerializationDeterministic() && field.size() > 1) {
        for (const auto& entry : ::google::protobuf::internal::MapSorterPtr<MapType>(field)) {
          target = WireHelper::InternalSerialize(
              2, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.PubsubMessage.attributes");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.PubsubMessage.attributes");
        }
      } else {
        for (const auto& entry : field) {
          target = WireHelper::InternalSerialize(
              2, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.PubsubMessage.attributes");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.PubsubMessage.attributes");
        }
      }
    }
  }

  // string message_id = 3;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_message_id().empty()) {
      const ::std::string& _s = this_._internal_message_id();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.PubsubMessage.message_id");
      target = stream->WriteStringMaybeAliased(3, _s, target);
    }
  }

  // .google.protobuf.Timestamp publish_time = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        4, *this_._impl_.publish_time_, this_._impl_.publish_time_->GetCachedSize(), target,
        stream);
  }

  // string ordering_key = 5 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_ordering_key().empty()) {
      const ::std::string& _s = this_._internal_ordering_key();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.PubsubMessage.ordering_key");
      target = stream->WriteStringMaybeAliased(5, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.PubsubMessage)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t PubsubMessage::ByteSizeLong(const MessageLite& base) {
  const PubsubMessage& this_ = static_cast<const PubsubMessage&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t PubsubMessage::ByteSizeLong() const {
  const PubsubMessage& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.PubsubMessage)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    // bytes data = 1 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_data().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::BytesSize(
                                        this_._internal_data());
      }
    }
    // string message_id = 3;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_message_id().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_message_id());
      }
    }
    // string ordering_key = 5 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!this_._internal_ordering_key().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_ordering_key());
      }
    }
    // .google.protobuf.Timestamp publish_time = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.publish_time_);
    }
    // map<string, string> attributes = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_attributes_size());
      for (const auto& entry : this_._internal_attributes()) {
        total_size += _pbi::MapEntryFuncs<::std::string, ::std::string,
                                       _pbi::WireFormatLite::TYPE_STRING,
                                       _pbi::WireFormatLite::TYPE_STRING>::ByteSizeLong(entry.first, entry.second);
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void PubsubMessage::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<PubsubMessage*>(&to_msg);
  auto& from = static_cast<const PubsubMessage&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.PubsubMessage)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_data().empty()) {
        _this->_internal_set_data(from._internal_data());
      } else {
        if (_this->_impl_.data_.IsDefault()) {
          _this->_internal_set_data("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_message_id().empty()) {
        _this->_internal_set_message_id(from._internal_message_id());
      } else {
        if (_this->_impl_.message_id_.IsDefault()) {
          _this->_internal_set_message_id("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!from._internal_ordering_key().empty()) {
        _this->_internal_set_ordering_key(from._internal_ordering_key());
      } else {
        if (_this->_impl_.ordering_key_.IsDefault()) {
          _this->_internal_set_ordering_key("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(from._impl_.publish_time_ != nullptr);
      if (_this->_impl_.publish_time_ == nullptr) {
        _this->_impl_.publish_time_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.publish_time_);
      } else {
        _this->_impl_.publish_time_->MergeFrom(*from._impl_.publish_time_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      _this->_impl_.attributes_.MergeFrom(from._impl_.attributes_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void PubsubMessage::CopyFrom(const PubsubMessage& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.PubsubMessage)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void PubsubMessage::InternalSwap(PubsubMessage* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.data_, &other->_impl_.data_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.message_id_, &other->_impl_.message_id_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.ordering_key_, &other->_impl_.ordering_key_, arena);
  swap(_impl_.publish_time_, other->_impl_.publish_time_);
  _impl_.attributes_.InternalSwap(&other->_impl_.attributes_);
}

::google::protobuf::Metadata PubsubMessage::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

GetTopicRequest::GetTopicRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, GetTopicRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.GetTopicRequest)
}
PROTOBUF_NDEBUG_INLINE GetTopicRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::GetTopicRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        topic_(arena, from.topic_) {}

GetTopicRequest::GetTopicRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const GetTopicRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, GetTopicRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  GetTopicRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.GetTopicRequest)
}
PROTOBUF_NDEBUG_INLINE GetTopicRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        topic_(arena) {}

inline void GetTopicRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
GetTopicRequest::~GetTopicRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.GetTopicRequest)
  SharedDtor(*this);
}
inline void GetTopicRequest::SharedDtor(MessageLite& self) {
  GetTopicRequest& this_ = static_cast<GetTopicRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.topic_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull GetTopicRequest_class_data_ =
        GetTopicRequest::InternalGenerateClassData_(GetTopicRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
GetTopicRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&GetTopicRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(GetTopicRequest_class_data_.tc_table);
  return GetTopicRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
GetTopicRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&GetTopicRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&GetTopicRequest_globals_));
  return GetTopicRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const GetTopicRequest::ParseTableT_
    GetTopicRequest::_table_ =
        GetTopicRequest::InternalGenerateParseTable_(GetTopicRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void GetTopicRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.GetTopicRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.topic_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL GetTopicRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const GetTopicRequest& this_ = static_cast<const GetTopicRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL GetTopicRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const GetTopicRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.GetTopicRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string topic = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_topic().empty()) {
      const ::std::string& _s = this_._internal_topic();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.GetTopicRequest.topic");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.GetTopicRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t GetTopicRequest::ByteSizeLong(const MessageLite& base) {
  const GetTopicRequest& this_ = static_cast<const GetTopicRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t GetTopicRequest::ByteSizeLong() const {
  const GetTopicRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.GetTopicRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string topic = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_topic().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_topic());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void GetTopicRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<GetTopicRequest*>(&to_msg);
  auto& from = static_cast<const GetTopicRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.GetTopicRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_topic().empty()) {
      _this->_internal_set_topic(from._internal_topic());
    } else {
      if (_this->_impl_.topic_.IsDefault()) {
        _this->_internal_set_topic("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void GetTopicRequest::CopyFrom(const GetTopicRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.GetTopicRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void GetTopicRequest::InternalSwap(GetTopicRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.topic_, &other->_impl_.topic_, arena);
}

::google::protobuf::Metadata GetTopicRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void UpdateTopicRequest::clear_update_mask() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.update_mask_ != nullptr) _impl_.update_mask_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
UpdateTopicRequest::UpdateTopicRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UpdateTopicRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.UpdateTopicRequest)
}
PROTOBUF_NDEBUG_INLINE UpdateTopicRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::UpdateTopicRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

UpdateTopicRequest::UpdateTopicRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const UpdateTopicRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UpdateTopicRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  UpdateTopicRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.topic_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.topic_)
                : nullptr;
  _impl_.update_mask_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.update_mask_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.UpdateTopicRequest)
}
PROTOBUF_NDEBUG_INLINE UpdateTopicRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void UpdateTopicRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, topic_),
           0,
           offsetof(Impl_, update_mask_) -
               offsetof(Impl_, topic_) +
               sizeof(Impl_::update_mask_));
}
UpdateTopicRequest::~UpdateTopicRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.UpdateTopicRequest)
  SharedDtor(*this);
}
inline void UpdateTopicRequest::SharedDtor(MessageLite& self) {
  UpdateTopicRequest& this_ = static_cast<UpdateTopicRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.topic_;
  delete this_._impl_.update_mask_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull UpdateTopicRequest_class_data_ =
        UpdateTopicRequest::InternalGenerateClassData_(UpdateTopicRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateTopicRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateTopicRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(UpdateTopicRequest_class_data_.tc_table);
  return UpdateTopicRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateTopicRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateTopicRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&UpdateTopicRequest_globals_));
  return UpdateTopicRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const UpdateTopicRequest::ParseTableT_
    UpdateTopicRequest::_table_ =
        UpdateTopicRequest::InternalGenerateParseTable_(UpdateTopicRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void UpdateTopicRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.UpdateTopicRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(_impl_.topic_ != nullptr);
      _impl_.topic_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.update_mask_ != nullptr);
      _impl_.update_mask_->Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL UpdateTopicRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const UpdateTopicRequest& this_ = static_cast<const UpdateTopicRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL UpdateTopicRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const UpdateTopicRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.UpdateTopicRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.pubsub.v1.Topic topic = 1 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.topic_, this_._impl_.topic_->GetCachedSize(), target,
        stream);
  }

  // .google.protobuf.FieldMask update_mask = 2 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.update_mask_, this_._impl_.update_mask_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.UpdateTopicRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t UpdateTopicRequest::ByteSizeLong(const MessageLite& base) {
  const UpdateTopicRequest& this_ = static_cast<const UpdateTopicRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t UpdateTopicRequest::ByteSizeLong() const {
  const UpdateTopicRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.UpdateTopicRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // .google.pubsub.v1.Topic topic = 1 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.topic_);
    }
    // .google.protobuf.FieldMask update_mask = 2 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.update_mask_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void UpdateTopicRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<UpdateTopicRequest*>(&to_msg);
  auto& from = static_cast<const UpdateTopicRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.UpdateTopicRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.topic_ != nullptr);
      if (_this->_impl_.topic_ == nullptr) {
        _this->_impl_.topic_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.topic_);
      } else {
        _this->_impl_.topic_->MergeFrom(*from._impl_.topic_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.update_mask_ != nullptr);
      if (_this->_impl_.update_mask_ == nullptr) {
        _this->_impl_.update_mask_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.update_mask_);
      } else {
        _this->_impl_.update_mask_->MergeFrom(*from._impl_.update_mask_);
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void UpdateTopicRequest::CopyFrom(const UpdateTopicRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.UpdateTopicRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void UpdateTopicRequest::InternalSwap(UpdateTopicRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(UpdateTopicRequest, _impl_.update_mask_)
      + sizeof(UpdateTopicRequest::_impl_.update_mask_)
      - PROTOBUF_FIELD_OFFSET(UpdateTopicRequest, _impl_.topic_)>(
          reinterpret_cast<char*>(&_impl_.topic_),
          reinterpret_cast<char*>(&other->_impl_.topic_));
}

::google::protobuf::Metadata UpdateTopicRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

PublishRequest::PublishRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PublishRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.PublishRequest)
}
PROTOBUF_NDEBUG_INLINE PublishRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::PublishRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        messages_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::PublishRequest,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PublishRequest, _impl_.messages_)>()
          , from.messages_
        }
        ,
        topic_(arena, from.topic_) {}

PublishRequest::PublishRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const PublishRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PublishRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  PublishRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.PublishRequest)
}
PROTOBUF_NDEBUG_INLINE PublishRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        messages_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::PublishRequest,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PublishRequest, _impl_.messages_)>()
         }
        ,
        topic_(arena) {}

inline void PublishRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
PublishRequest::~PublishRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.PublishRequest)
  SharedDtor(*this);
}
inline void PublishRequest::SharedDtor(MessageLite& self) {
  PublishRequest& this_ = static_cast<PublishRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.topic_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull PublishRequest_class_data_ =
        PublishRequest::InternalGenerateClassData_(PublishRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PublishRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PublishRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(PublishRequest_class_data_.tc_table);
  return PublishRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PublishRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PublishRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&PublishRequest_globals_));
  return PublishRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const PublishRequest::ParseTableT_
    PublishRequest::_table_ =
        PublishRequest::InternalGenerateParseTable_(PublishRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void PublishRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.PublishRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.messages_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.topic_.ClearNonDefaultToEmpty();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL PublishRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const PublishRequest& this_ = static_cast<const PublishRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL PublishRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const PublishRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.PublishRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string topic = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_topic().empty()) {
      const ::std::string& _s = this_._internal_topic();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.PublishRequest.topic");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // repeated .google.pubsub.v1.PubsubMessage messages = 2 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_messages_size());
         i < n; i++) {
      const auto& repfield = this_._internal_messages().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              2, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.PublishRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t PublishRequest::ByteSizeLong(const MessageLite& base) {
  const PublishRequest& this_ = static_cast<const PublishRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t PublishRequest::ByteSizeLong() const {
  const PublishRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.PublishRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // repeated .google.pubsub.v1.PubsubMessage messages = 2 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_messages_size();
      for (const auto& msg : this_._internal_messages()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
    // string topic = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_topic().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_topic());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void PublishRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<PublishRequest*>(&to_msg);
  auto& from = static_cast<const PublishRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.PublishRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_messages()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_messages());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_topic().empty()) {
        _this->_internal_set_topic(from._internal_topic());
      } else {
        if (_this->_impl_.topic_.IsDefault()) {
          _this->_internal_set_topic("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void PublishRequest::CopyFrom(const PublishRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.PublishRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void PublishRequest::InternalSwap(PublishRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.messages_.InternalSwap(&other->_impl_.messages_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.topic_, &other->_impl_.topic_, arena);
}

::google::protobuf::Metadata PublishRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

PublishResponse::PublishResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PublishResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.PublishResponse)
}
PROTOBUF_NDEBUG_INLINE PublishResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::PublishResponse& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        message_ids_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::PublishResponse,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PublishResponse, _impl_.message_ids_)>()
          , from.message_ids_
        }
     {}

PublishResponse::PublishResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const PublishResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PublishResponse_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  PublishResponse* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.PublishResponse)
}
PROTOBUF_NDEBUG_INLINE PublishResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        message_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::PublishResponse,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PublishResponse, _impl_.message_ids_)>()
         }
     {}

inline void PublishResponse::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
PublishResponse::~PublishResponse() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.PublishResponse)
  SharedDtor(*this);
}
inline void PublishResponse::SharedDtor(MessageLite& self) {
  PublishResponse& this_ = static_cast<PublishResponse&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull PublishResponse_class_data_ =
        PublishResponse::InternalGenerateClassData_(PublishResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PublishResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PublishResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(PublishResponse_class_data_.tc_table);
  return PublishResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PublishResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PublishResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&PublishResponse_globals_));
  return PublishResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const PublishResponse::ParseTableT_
    PublishResponse::_table_ =
        PublishResponse::InternalGenerateParseTable_(PublishResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void PublishResponse::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.PublishResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.message_ids_.Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL PublishResponse::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const PublishResponse& this_ = static_cast<const PublishResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL PublishResponse::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const PublishResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.PublishResponse)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated string message_ids = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (int i = 0, n = this_._internal_message_ids_size(); i < n; ++i) {
      const auto& s = this_._internal_message_ids().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.PublishResponse.message_ids");
      target = stream->WriteString(1, s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.PublishResponse)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t PublishResponse::ByteSizeLong(const MessageLite& base) {
  const PublishResponse& this_ = static_cast<const PublishResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t PublishResponse::ByteSizeLong() const {
  const PublishResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.PublishResponse)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
   {
    // repeated string message_ids = 1 [(.google.api.field_behavior) = OPTIONAL];
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_message_ids().size());
      for (int i = 0, n = this_._internal_message_ids().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_message_ids().Get(i));
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void PublishResponse::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<PublishResponse*>(&to_msg);
  auto& from = static_cast<const PublishResponse&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.PublishResponse)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _this->_internal_mutable_message_ids()->InternalMergeFromWithArena(
        ::google::protobuf::MessageLite::internal_visibility(), arena,
        from._internal_message_ids());
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void PublishResponse::CopyFrom(const PublishResponse& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.PublishResponse)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void PublishResponse::InternalSwap(PublishResponse* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.message_ids_.InternalSwap(&other->_impl_.message_ids_);
}

::google::protobuf::Metadata PublishResponse::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ListTopicsRequest::ListTopicsRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListTopicsRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ListTopicsRequest)
}
PROTOBUF_NDEBUG_INLINE ListTopicsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::ListTopicsRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        project_(arena, from.project_),
        page_token_(arena, from.page_token_) {}

ListTopicsRequest::ListTopicsRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ListTopicsRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListTopicsRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ListTopicsRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.page_size_ = from._impl_.page_size_;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ListTopicsRequest)
}
PROTOBUF_NDEBUG_INLINE ListTopicsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        project_(arena),
        page_token_(arena) {}

inline void ListTopicsRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.page_size_ = {};
}
ListTopicsRequest::~ListTopicsRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.ListTopicsRequest)
  SharedDtor(*this);
}
inline void ListTopicsRequest::SharedDtor(MessageLite& self) {
  ListTopicsRequest& this_ = static_cast<ListTopicsRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.project_.Destroy();
  this_._impl_.page_token_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ListTopicsRequest_class_data_ =
        ListTopicsRequest::InternalGenerateClassData_(ListTopicsRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListTopicsRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListTopicsRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ListTopicsRequest_class_data_.tc_table);
  return ListTopicsRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListTopicsRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListTopicsRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ListTopicsRequest_globals_));
  return ListTopicsRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ListTopicsRequest::ParseTableT_
    ListTopicsRequest::_table_ =
        ListTopicsRequest::InternalGenerateParseTable_(ListTopicsRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ListTopicsRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.ListTopicsRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.project_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.page_token_.ClearNonDefaultToEmpty();
    }
  }
  _impl_.page_size_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ListTopicsRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ListTopicsRequest& this_ = static_cast<const ListTopicsRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ListTopicsRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ListTopicsRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.ListTopicsRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string project = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_project().empty()) {
      const ::std::string& _s = this_._internal_project();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListTopicsRequest.project");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // int32 page_size = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_page_size() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<2>(
              stream, this_._internal_page_size(), target);
    }
  }

  // string page_token = 3 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_page_token().empty()) {
      const ::std::string& _s = this_._internal_page_token();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListTopicsRequest.page_token");
      target = stream->WriteStringMaybeAliased(3, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.ListTopicsRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ListTopicsRequest::ByteSizeLong(const MessageLite& base) {
  const ListTopicsRequest& this_ = static_cast<const ListTopicsRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ListTopicsRequest::ByteSizeLong() const {
  const ListTopicsRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.ListTopicsRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // string project = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_project().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_project());
      }
    }
    // string page_token = 3 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_page_token().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_page_token());
      }
    }
    // int32 page_size = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_page_size() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_page_size());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ListTopicsRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ListTopicsRequest*>(&to_msg);
  auto& from = static_cast<const ListTopicsRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.ListTopicsRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_project().empty()) {
        _this->_internal_set_project(from._internal_project());
      } else {
        if (_this->_impl_.project_.IsDefault()) {
          _this->_internal_set_project("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_page_token().empty()) {
        _this->_internal_set_page_token(from._internal_page_token());
      } else {
        if (_this->_impl_.page_token_.IsDefault()) {
          _this->_internal_set_page_token("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_page_size() != 0) {
        _this->_impl_.page_size_ = from._impl_.page_size_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ListTopicsRequest::CopyFrom(const ListTopicsRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.ListTopicsRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ListTopicsRequest::InternalSwap(ListTopicsRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.project_, &other->_impl_.project_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.page_token_, &other->_impl_.page_token_, arena);
  swap(_impl_.page_size_, other->_impl_.page_size_);
}

::google::protobuf::Metadata ListTopicsRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ListTopicsResponse::ListTopicsResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListTopicsResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ListTopicsResponse)
}
PROTOBUF_NDEBUG_INLINE ListTopicsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::ListTopicsResponse& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        topics_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::ListTopicsResponse,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicsResponse, _impl_.topics_)>()
          , from.topics_
        }
        ,
        next_page_token_(arena, from.next_page_token_) {}

ListTopicsResponse::ListTopicsResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ListTopicsResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListTopicsResponse_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ListTopicsResponse* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ListTopicsResponse)
}
PROTOBUF_NDEBUG_INLINE ListTopicsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        topics_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::ListTopicsResponse,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicsResponse, _impl_.topics_)>()
         }
        ,
        next_page_token_(arena) {}

inline void ListTopicsResponse::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
ListTopicsResponse::~ListTopicsResponse() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.ListTopicsResponse)
  SharedDtor(*this);
}
inline void ListTopicsResponse::SharedDtor(MessageLite& self) {
  ListTopicsResponse& this_ = static_cast<ListTopicsResponse&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.next_page_token_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ListTopicsResponse_class_data_ =
        ListTopicsResponse::InternalGenerateClassData_(ListTopicsResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListTopicsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListTopicsResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ListTopicsResponse_class_data_.tc_table);
  return ListTopicsResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListTopicsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListTopicsResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ListTopicsResponse_globals_));
  return ListTopicsResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ListTopicsResponse::ParseTableT_
    ListTopicsResponse::_table_ =
        ListTopicsResponse::InternalGenerateParseTable_(ListTopicsResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ListTopicsResponse::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.ListTopicsResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.topics_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.next_page_token_.ClearNonDefaultToEmpty();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ListTopicsResponse::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ListTopicsResponse& this_ = static_cast<const ListTopicsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ListTopicsResponse::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ListTopicsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.ListTopicsResponse)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated .google.pubsub.v1.Topic topics = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_topics_size());
         i < n; i++) {
      const auto& repfield = this_._internal_topics().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              1, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  // string next_page_token = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_next_page_token().empty()) {
      const ::std::string& _s = this_._internal_next_page_token();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListTopicsResponse.next_page_token");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.ListTopicsResponse)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ListTopicsResponse::ByteSizeLong(const MessageLite& base) {
  const ListTopicsResponse& this_ = static_cast<const ListTopicsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ListTopicsResponse::ByteSizeLong() const {
  const ListTopicsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.ListTopicsResponse)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // repeated .google.pubsub.v1.Topic topics = 1 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_topics_size();
      for (const auto& msg : this_._internal_topics()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
    // string next_page_token = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_next_page_token().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_next_page_token());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ListTopicsResponse::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ListTopicsResponse*>(&to_msg);
  auto& from = static_cast<const ListTopicsResponse&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.ListTopicsResponse)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_topics()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_topics());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_next_page_token().empty()) {
        _this->_internal_set_next_page_token(from._internal_next_page_token());
      } else {
        if (_this->_impl_.next_page_token_.IsDefault()) {
          _this->_internal_set_next_page_token("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ListTopicsResponse::CopyFrom(const ListTopicsResponse& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.ListTopicsResponse)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ListTopicsResponse::InternalSwap(ListTopicsResponse* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.topics_.InternalSwap(&other->_impl_.topics_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.next_page_token_, &other->_impl_.next_page_token_, arena);
}

::google::protobuf::Metadata ListTopicsResponse::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ListTopicSubscriptionsRequest::ListTopicSubscriptionsRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListTopicSubscriptionsRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ListTopicSubscriptionsRequest)
}
PROTOBUF_NDEBUG_INLINE ListTopicSubscriptionsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::ListTopicSubscriptionsRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        topic_(arena, from.topic_),
        page_token_(arena, from.page_token_) {}

ListTopicSubscriptionsRequest::ListTopicSubscriptionsRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ListTopicSubscriptionsRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListTopicSubscriptionsRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ListTopicSubscriptionsRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.page_size_ = from._impl_.page_size_;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ListTopicSubscriptionsRequest)
}
PROTOBUF_NDEBUG_INLINE ListTopicSubscriptionsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        topic_(arena),
        page_token_(arena) {}

inline void ListTopicSubscriptionsRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.page_size_ = {};
}
ListTopicSubscriptionsRequest::~ListTopicSubscriptionsRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.ListTopicSubscriptionsRequest)
  SharedDtor(*this);
}
inline void ListTopicSubscriptionsRequest::SharedDtor(MessageLite& self) {
  ListTopicSubscriptionsRequest& this_ = static_cast<ListTopicSubscriptionsRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.topic_.Destroy();
  this_._impl_.page_token_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ListTopicSubscriptionsRequest_class_data_ =
        ListTopicSubscriptionsRequest::InternalGenerateClassData_(ListTopicSubscriptionsRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListTopicSubscriptionsRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListTopicSubscriptionsRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ListTopicSubscriptionsRequest_class_data_.tc_table);
  return ListTopicSubscriptionsRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListTopicSubscriptionsRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListTopicSubscriptionsRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ListTopicSubscriptionsRequest_globals_));
  return ListTopicSubscriptionsRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ListTopicSubscriptionsRequest::ParseTableT_
    ListTopicSubscriptionsRequest::_table_ =
        ListTopicSubscriptionsRequest::InternalGenerateParseTable_(ListTopicSubscriptionsRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ListTopicSubscriptionsRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.ListTopicSubscriptionsRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.topic_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.page_token_.ClearNonDefaultToEmpty();
    }
  }
  _impl_.page_size_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ListTopicSubscriptionsRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ListTopicSubscriptionsRequest& this_ = static_cast<const ListTopicSubscriptionsRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ListTopicSubscriptionsRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ListTopicSubscriptionsRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.ListTopicSubscriptionsRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string topic = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_topic().empty()) {
      const ::std::string& _s = this_._internal_topic();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListTopicSubscriptionsRequest.topic");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // int32 page_size = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_page_size() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<2>(
              stream, this_._internal_page_size(), target);
    }
  }

  // string page_token = 3 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_page_token().empty()) {
      const ::std::string& _s = this_._internal_page_token();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListTopicSubscriptionsRequest.page_token");
      target = stream->WriteStringMaybeAliased(3, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.ListTopicSubscriptionsRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ListTopicSubscriptionsRequest::ByteSizeLong(const MessageLite& base) {
  const ListTopicSubscriptionsRequest& this_ = static_cast<const ListTopicSubscriptionsRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ListTopicSubscriptionsRequest::ByteSizeLong() const {
  const ListTopicSubscriptionsRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.ListTopicSubscriptionsRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // string topic = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_topic().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_topic());
      }
    }
    // string page_token = 3 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_page_token().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_page_token());
      }
    }
    // int32 page_size = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_page_size() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_page_size());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ListTopicSubscriptionsRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ListTopicSubscriptionsRequest*>(&to_msg);
  auto& from = static_cast<const ListTopicSubscriptionsRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.ListTopicSubscriptionsRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_topic().empty()) {
        _this->_internal_set_topic(from._internal_topic());
      } else {
        if (_this->_impl_.topic_.IsDefault()) {
          _this->_internal_set_topic("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_page_token().empty()) {
        _this->_internal_set_page_token(from._internal_page_token());
      } else {
        if (_this->_impl_.page_token_.IsDefault()) {
          _this->_internal_set_page_token("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_page_size() != 0) {
        _this->_impl_.page_size_ = from._impl_.page_size_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ListTopicSubscriptionsRequest::CopyFrom(const ListTopicSubscriptionsRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.ListTopicSubscriptionsRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ListTopicSubscriptionsRequest::InternalSwap(ListTopicSubscriptionsRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.topic_, &other->_impl_.topic_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.page_token_, &other->_impl_.page_token_, arena);
  swap(_impl_.page_size_, other->_impl_.page_size_);
}

::google::protobuf::Metadata ListTopicSubscriptionsRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ListTopicSubscriptionsResponse::ListTopicSubscriptionsResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListTopicSubscriptionsResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ListTopicSubscriptionsResponse)
}
PROTOBUF_NDEBUG_INLINE ListTopicSubscriptionsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::ListTopicSubscriptionsResponse& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        subscriptions_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::ListTopicSubscriptionsResponse,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicSubscriptionsResponse, _impl_.subscriptions_)>()
          , from.subscriptions_
        }
        ,
        next_page_token_(arena, from.next_page_token_) {}

ListTopicSubscriptionsResponse::ListTopicSubscriptionsResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ListTopicSubscriptionsResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListTopicSubscriptionsResponse_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ListTopicSubscriptionsResponse* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ListTopicSubscriptionsResponse)
}
PROTOBUF_NDEBUG_INLINE ListTopicSubscriptionsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        subscriptions_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::ListTopicSubscriptionsResponse,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicSubscriptionsResponse, _impl_.subscriptions_)>()
         }
        ,
        next_page_token_(arena) {}

inline void ListTopicSubscriptionsResponse::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
ListTopicSubscriptionsResponse::~ListTopicSubscriptionsResponse() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.ListTopicSubscriptionsResponse)
  SharedDtor(*this);
}
inline void ListTopicSubscriptionsResponse::SharedDtor(MessageLite& self) {
  ListTopicSubscriptionsResponse& this_ = static_cast<ListTopicSubscriptionsResponse&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.next_page_token_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ListTopicSubscriptionsResponse_class_data_ =
        ListTopicSubscriptionsResponse::InternalGenerateClassData_(ListTopicSubscriptionsResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListTopicSubscriptionsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListTopicSubscriptionsResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ListTopicSubscriptionsResponse_class_data_.tc_table);
  return ListTopicSubscriptionsResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListTopicSubscriptionsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListTopicSubscriptionsResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ListTopicSubscriptionsResponse_globals_));
  return ListTopicSubscriptionsResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ListTopicSubscriptionsResponse::ParseTableT_
    ListTopicSubscriptionsResponse::_table_ =
        ListTopicSubscriptionsResponse::InternalGenerateParseTable_(ListTopicSubscriptionsResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ListTopicSubscriptionsResponse::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.ListTopicSubscriptionsResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.subscriptions_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.next_page_token_.ClearNonDefaultToEmpty();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ListTopicSubscriptionsResponse::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ListTopicSubscriptionsResponse& this_ = static_cast<const ListTopicSubscriptionsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ListTopicSubscriptionsResponse::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ListTopicSubscriptionsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.ListTopicSubscriptionsResponse)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated string subscriptions = 1 [(.google.api.field_behavior) = OPTIONAL, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (int i = 0, n = this_._internal_subscriptions_size(); i < n; ++i) {
      const auto& s = this_._internal_subscriptions().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListTopicSubscriptionsResponse.subscriptions");
      target = stream->WriteString(1, s, target);
    }
  }

  // string next_page_token = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_next_page_token().empty()) {
      const ::std::string& _s = this_._internal_next_page_token();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListTopicSubscriptionsResponse.next_page_token");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.ListTopicSubscriptionsResponse)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ListTopicSubscriptionsResponse::ByteSizeLong(const MessageLite& base) {
  const ListTopicSubscriptionsResponse& this_ = static_cast<const ListTopicSubscriptionsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ListTopicSubscriptionsResponse::ByteSizeLong() const {
  const ListTopicSubscriptionsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.ListTopicSubscriptionsResponse)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // repeated string subscriptions = 1 [(.google.api.field_behavior) = OPTIONAL, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_subscriptions().size());
      for (int i = 0, n = this_._internal_subscriptions().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_subscriptions().Get(i));
      }
    }
    // string next_page_token = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_next_page_token().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_next_page_token());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ListTopicSubscriptionsResponse::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ListTopicSubscriptionsResponse*>(&to_msg);
  auto& from = static_cast<const ListTopicSubscriptionsResponse&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.ListTopicSubscriptionsResponse)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_subscriptions()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_subscriptions());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_next_page_token().empty()) {
        _this->_internal_set_next_page_token(from._internal_next_page_token());
      } else {
        if (_this->_impl_.next_page_token_.IsDefault()) {
          _this->_internal_set_next_page_token("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ListTopicSubscriptionsResponse::CopyFrom(const ListTopicSubscriptionsResponse& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.ListTopicSubscriptionsResponse)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ListTopicSubscriptionsResponse::InternalSwap(ListTopicSubscriptionsResponse* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.subscriptions_.InternalSwap(&other->_impl_.subscriptions_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.next_page_token_, &other->_impl_.next_page_token_, arena);
}

::google::protobuf::Metadata ListTopicSubscriptionsResponse::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ListTopicSnapshotsRequest::ListTopicSnapshotsRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListTopicSnapshotsRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ListTopicSnapshotsRequest)
}
PROTOBUF_NDEBUG_INLINE ListTopicSnapshotsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::ListTopicSnapshotsRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        topic_(arena, from.topic_),
        page_token_(arena, from.page_token_) {}

ListTopicSnapshotsRequest::ListTopicSnapshotsRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ListTopicSnapshotsRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListTopicSnapshotsRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ListTopicSnapshotsRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.page_size_ = from._impl_.page_size_;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ListTopicSnapshotsRequest)
}
PROTOBUF_NDEBUG_INLINE ListTopicSnapshotsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        topic_(arena),
        page_token_(arena) {}

inline void ListTopicSnapshotsRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.page_size_ = {};
}
ListTopicSnapshotsRequest::~ListTopicSnapshotsRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.ListTopicSnapshotsRequest)
  SharedDtor(*this);
}
inline void ListTopicSnapshotsRequest::SharedDtor(MessageLite& self) {
  ListTopicSnapshotsRequest& this_ = static_cast<ListTopicSnapshotsRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.topic_.Destroy();
  this_._impl_.page_token_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ListTopicSnapshotsRequest_class_data_ =
        ListTopicSnapshotsRequest::InternalGenerateClassData_(ListTopicSnapshotsRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListTopicSnapshotsRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListTopicSnapshotsRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ListTopicSnapshotsRequest_class_data_.tc_table);
  return ListTopicSnapshotsRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListTopicSnapshotsRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListTopicSnapshotsRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ListTopicSnapshotsRequest_globals_));
  return ListTopicSnapshotsRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ListTopicSnapshotsRequest::ParseTableT_
    ListTopicSnapshotsRequest::_table_ =
        ListTopicSnapshotsRequest::InternalGenerateParseTable_(ListTopicSnapshotsRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ListTopicSnapshotsRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.ListTopicSnapshotsRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.topic_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.page_token_.ClearNonDefaultToEmpty();
    }
  }
  _impl_.page_size_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ListTopicSnapshotsRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ListTopicSnapshotsRequest& this_ = static_cast<const ListTopicSnapshotsRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ListTopicSnapshotsRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ListTopicSnapshotsRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.ListTopicSnapshotsRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string topic = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_topic().empty()) {
      const ::std::string& _s = this_._internal_topic();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListTopicSnapshotsRequest.topic");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // int32 page_size = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_page_size() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<2>(
              stream, this_._internal_page_size(), target);
    }
  }

  // string page_token = 3 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_page_token().empty()) {
      const ::std::string& _s = this_._internal_page_token();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListTopicSnapshotsRequest.page_token");
      target = stream->WriteStringMaybeAliased(3, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.ListTopicSnapshotsRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ListTopicSnapshotsRequest::ByteSizeLong(const MessageLite& base) {
  const ListTopicSnapshotsRequest& this_ = static_cast<const ListTopicSnapshotsRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ListTopicSnapshotsRequest::ByteSizeLong() const {
  const ListTopicSnapshotsRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.ListTopicSnapshotsRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // string topic = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_topic().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_topic());
      }
    }
    // string page_token = 3 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_page_token().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_page_token());
      }
    }
    // int32 page_size = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_page_size() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_page_size());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ListTopicSnapshotsRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ListTopicSnapshotsRequest*>(&to_msg);
  auto& from = static_cast<const ListTopicSnapshotsRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.ListTopicSnapshotsRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_topic().empty()) {
        _this->_internal_set_topic(from._internal_topic());
      } else {
        if (_this->_impl_.topic_.IsDefault()) {
          _this->_internal_set_topic("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_page_token().empty()) {
        _this->_internal_set_page_token(from._internal_page_token());
      } else {
        if (_this->_impl_.page_token_.IsDefault()) {
          _this->_internal_set_page_token("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_page_size() != 0) {
        _this->_impl_.page_size_ = from._impl_.page_size_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ListTopicSnapshotsRequest::CopyFrom(const ListTopicSnapshotsRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.ListTopicSnapshotsRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ListTopicSnapshotsRequest::InternalSwap(ListTopicSnapshotsRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.topic_, &other->_impl_.topic_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.page_token_, &other->_impl_.page_token_, arena);
  swap(_impl_.page_size_, other->_impl_.page_size_);
}

::google::protobuf::Metadata ListTopicSnapshotsRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ListTopicSnapshotsResponse::ListTopicSnapshotsResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListTopicSnapshotsResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ListTopicSnapshotsResponse)
}
PROTOBUF_NDEBUG_INLINE ListTopicSnapshotsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::ListTopicSnapshotsResponse& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        snapshots_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::ListTopicSnapshotsResponse,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicSnapshotsResponse, _impl_.snapshots_)>()
          , from.snapshots_
        }
        ,
        next_page_token_(arena, from.next_page_token_) {}

ListTopicSnapshotsResponse::ListTopicSnapshotsResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ListTopicSnapshotsResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListTopicSnapshotsResponse_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ListTopicSnapshotsResponse* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ListTopicSnapshotsResponse)
}
PROTOBUF_NDEBUG_INLINE ListTopicSnapshotsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        snapshots_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::ListTopicSnapshotsResponse,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListTopicSnapshotsResponse, _impl_.snapshots_)>()
         }
        ,
        next_page_token_(arena) {}

inline void ListTopicSnapshotsResponse::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
ListTopicSnapshotsResponse::~ListTopicSnapshotsResponse() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.ListTopicSnapshotsResponse)
  SharedDtor(*this);
}
inline void ListTopicSnapshotsResponse::SharedDtor(MessageLite& self) {
  ListTopicSnapshotsResponse& this_ = static_cast<ListTopicSnapshotsResponse&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.next_page_token_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ListTopicSnapshotsResponse_class_data_ =
        ListTopicSnapshotsResponse::InternalGenerateClassData_(ListTopicSnapshotsResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListTopicSnapshotsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListTopicSnapshotsResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ListTopicSnapshotsResponse_class_data_.tc_table);
  return ListTopicSnapshotsResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListTopicSnapshotsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListTopicSnapshotsResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ListTopicSnapshotsResponse_globals_));
  return ListTopicSnapshotsResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ListTopicSnapshotsResponse::ParseTableT_
    ListTopicSnapshotsResponse::_table_ =
        ListTopicSnapshotsResponse::InternalGenerateParseTable_(ListTopicSnapshotsResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ListTopicSnapshotsResponse::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.ListTopicSnapshotsResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.snapshots_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.next_page_token_.ClearNonDefaultToEmpty();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ListTopicSnapshotsResponse::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ListTopicSnapshotsResponse& this_ = static_cast<const ListTopicSnapshotsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ListTopicSnapshotsResponse::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ListTopicSnapshotsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.ListTopicSnapshotsResponse)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated string snapshots = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (int i = 0, n = this_._internal_snapshots_size(); i < n; ++i) {
      const auto& s = this_._internal_snapshots().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListTopicSnapshotsResponse.snapshots");
      target = stream->WriteString(1, s, target);
    }
  }

  // string next_page_token = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_next_page_token().empty()) {
      const ::std::string& _s = this_._internal_next_page_token();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListTopicSnapshotsResponse.next_page_token");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.ListTopicSnapshotsResponse)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ListTopicSnapshotsResponse::ByteSizeLong(const MessageLite& base) {
  const ListTopicSnapshotsResponse& this_ = static_cast<const ListTopicSnapshotsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ListTopicSnapshotsResponse::ByteSizeLong() const {
  const ListTopicSnapshotsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.ListTopicSnapshotsResponse)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // repeated string snapshots = 1 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_snapshots().size());
      for (int i = 0, n = this_._internal_snapshots().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_snapshots().Get(i));
      }
    }
    // string next_page_token = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_next_page_token().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_next_page_token());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ListTopicSnapshotsResponse::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ListTopicSnapshotsResponse*>(&to_msg);
  auto& from = static_cast<const ListTopicSnapshotsResponse&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.ListTopicSnapshotsResponse)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_snapshots()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_snapshots());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_next_page_token().empty()) {
        _this->_internal_set_next_page_token(from._internal_next_page_token());
      } else {
        if (_this->_impl_.next_page_token_.IsDefault()) {
          _this->_internal_set_next_page_token("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ListTopicSnapshotsResponse::CopyFrom(const ListTopicSnapshotsResponse& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.ListTopicSnapshotsResponse)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ListTopicSnapshotsResponse::InternalSwap(ListTopicSnapshotsResponse* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.snapshots_.InternalSwap(&other->_impl_.snapshots_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.next_page_token_, &other->_impl_.next_page_token_, arena);
}

::google::protobuf::Metadata ListTopicSnapshotsResponse::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

DeleteTopicRequest::DeleteTopicRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DeleteTopicRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.DeleteTopicRequest)
}
PROTOBUF_NDEBUG_INLINE DeleteTopicRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::DeleteTopicRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        topic_(arena, from.topic_) {}

DeleteTopicRequest::DeleteTopicRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const DeleteTopicRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DeleteTopicRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  DeleteTopicRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.DeleteTopicRequest)
}
PROTOBUF_NDEBUG_INLINE DeleteTopicRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        topic_(arena) {}

inline void DeleteTopicRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
DeleteTopicRequest::~DeleteTopicRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.DeleteTopicRequest)
  SharedDtor(*this);
}
inline void DeleteTopicRequest::SharedDtor(MessageLite& self) {
  DeleteTopicRequest& this_ = static_cast<DeleteTopicRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.topic_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull DeleteTopicRequest_class_data_ =
        DeleteTopicRequest::InternalGenerateClassData_(DeleteTopicRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DeleteTopicRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DeleteTopicRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(DeleteTopicRequest_class_data_.tc_table);
  return DeleteTopicRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DeleteTopicRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DeleteTopicRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&DeleteTopicRequest_globals_));
  return DeleteTopicRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const DeleteTopicRequest::ParseTableT_
    DeleteTopicRequest::_table_ =
        DeleteTopicRequest::InternalGenerateParseTable_(DeleteTopicRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void DeleteTopicRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.DeleteTopicRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.topic_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL DeleteTopicRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const DeleteTopicRequest& this_ = static_cast<const DeleteTopicRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL DeleteTopicRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const DeleteTopicRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.DeleteTopicRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string topic = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_topic().empty()) {
      const ::std::string& _s = this_._internal_topic();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.DeleteTopicRequest.topic");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.DeleteTopicRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t DeleteTopicRequest::ByteSizeLong(const MessageLite& base) {
  const DeleteTopicRequest& this_ = static_cast<const DeleteTopicRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t DeleteTopicRequest::ByteSizeLong() const {
  const DeleteTopicRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.DeleteTopicRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string topic = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_topic().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_topic());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void DeleteTopicRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<DeleteTopicRequest*>(&to_msg);
  auto& from = static_cast<const DeleteTopicRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.DeleteTopicRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_topic().empty()) {
      _this->_internal_set_topic(from._internal_topic());
    } else {
      if (_this->_impl_.topic_.IsDefault()) {
        _this->_internal_set_topic("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void DeleteTopicRequest::CopyFrom(const DeleteTopicRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.DeleteTopicRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void DeleteTopicRequest::InternalSwap(DeleteTopicRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.topic_, &other->_impl_.topic_, arena);
}

::google::protobuf::Metadata DeleteTopicRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

DetachSubscriptionRequest::DetachSubscriptionRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DetachSubscriptionRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.DetachSubscriptionRequest)
}
PROTOBUF_NDEBUG_INLINE DetachSubscriptionRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::DetachSubscriptionRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        subscription_(arena, from.subscription_) {}

DetachSubscriptionRequest::DetachSubscriptionRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const DetachSubscriptionRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DetachSubscriptionRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  DetachSubscriptionRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.DetachSubscriptionRequest)
}
PROTOBUF_NDEBUG_INLINE DetachSubscriptionRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        subscription_(arena) {}

inline void DetachSubscriptionRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
DetachSubscriptionRequest::~DetachSubscriptionRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.DetachSubscriptionRequest)
  SharedDtor(*this);
}
inline void DetachSubscriptionRequest::SharedDtor(MessageLite& self) {
  DetachSubscriptionRequest& this_ = static_cast<DetachSubscriptionRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.subscription_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull DetachSubscriptionRequest_class_data_ =
        DetachSubscriptionRequest::InternalGenerateClassData_(DetachSubscriptionRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DetachSubscriptionRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DetachSubscriptionRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(DetachSubscriptionRequest_class_data_.tc_table);
  return DetachSubscriptionRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DetachSubscriptionRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DetachSubscriptionRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&DetachSubscriptionRequest_globals_));
  return DetachSubscriptionRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const DetachSubscriptionRequest::ParseTableT_
    DetachSubscriptionRequest::_table_ =
        DetachSubscriptionRequest::InternalGenerateParseTable_(DetachSubscriptionRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void DetachSubscriptionRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.DetachSubscriptionRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.subscription_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL DetachSubscriptionRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const DetachSubscriptionRequest& this_ = static_cast<const DetachSubscriptionRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL DetachSubscriptionRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const DetachSubscriptionRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.DetachSubscriptionRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_subscription().empty()) {
      const ::std::string& _s = this_._internal_subscription();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.DetachSubscriptionRequest.subscription");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.DetachSubscriptionRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t DetachSubscriptionRequest::ByteSizeLong(const MessageLite& base) {
  const DetachSubscriptionRequest& this_ = static_cast<const DetachSubscriptionRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t DetachSubscriptionRequest::ByteSizeLong() const {
  const DetachSubscriptionRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.DetachSubscriptionRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_subscription().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_subscription());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void DetachSubscriptionRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<DetachSubscriptionRequest*>(&to_msg);
  auto& from = static_cast<const DetachSubscriptionRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.DetachSubscriptionRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_subscription().empty()) {
      _this->_internal_set_subscription(from._internal_subscription());
    } else {
      if (_this->_impl_.subscription_.IsDefault()) {
        _this->_internal_set_subscription("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void DetachSubscriptionRequest::CopyFrom(const DetachSubscriptionRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.DetachSubscriptionRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void DetachSubscriptionRequest::InternalSwap(DetachSubscriptionRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.subscription_, &other->_impl_.subscription_, arena);
}

::google::protobuf::Metadata DetachSubscriptionRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

DetachSubscriptionResponse::DetachSubscriptionResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, DetachSubscriptionResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.DetachSubscriptionResponse)
}
DetachSubscriptionResponse::DetachSubscriptionResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const DetachSubscriptionResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, DetachSubscriptionResponse_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  DetachSubscriptionResponse* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.DetachSubscriptionResponse)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull DetachSubscriptionResponse_class_data_ =
        DetachSubscriptionResponse::InternalGenerateClassData_(DetachSubscriptionResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DetachSubscriptionResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DetachSubscriptionResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(DetachSubscriptionResponse_class_data_.tc_table);
  return DetachSubscriptionResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DetachSubscriptionResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DetachSubscriptionResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&DetachSubscriptionResponse_globals_));
  return DetachSubscriptionResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const DetachSubscriptionResponse::ParseTableT_
    DetachSubscriptionResponse::_table_ =
        DetachSubscriptionResponse::InternalGenerateParseTable_(DetachSubscriptionResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata DetachSubscriptionResponse::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

Subscription_AnalyticsHubSubscriptionInfo::Subscription_AnalyticsHubSubscriptionInfo(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Subscription_AnalyticsHubSubscriptionInfo_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.Subscription.AnalyticsHubSubscriptionInfo)
}
PROTOBUF_NDEBUG_INLINE Subscription_AnalyticsHubSubscriptionInfo::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::Subscription_AnalyticsHubSubscriptionInfo& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        listing_(arena, from.listing_),
        subscription_(arena, from.subscription_) {}

Subscription_AnalyticsHubSubscriptionInfo::Subscription_AnalyticsHubSubscriptionInfo(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Subscription_AnalyticsHubSubscriptionInfo& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Subscription_AnalyticsHubSubscriptionInfo_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Subscription_AnalyticsHubSubscriptionInfo* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.Subscription.AnalyticsHubSubscriptionInfo)
}
PROTOBUF_NDEBUG_INLINE Subscription_AnalyticsHubSubscriptionInfo::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        listing_(arena),
        subscription_(arena) {}

inline void Subscription_AnalyticsHubSubscriptionInfo::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
Subscription_AnalyticsHubSubscriptionInfo::~Subscription_AnalyticsHubSubscriptionInfo() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.Subscription.AnalyticsHubSubscriptionInfo)
  SharedDtor(*this);
}
inline void Subscription_AnalyticsHubSubscriptionInfo::SharedDtor(MessageLite& self) {
  Subscription_AnalyticsHubSubscriptionInfo& this_ = static_cast<Subscription_AnalyticsHubSubscriptionInfo&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.listing_.Destroy();
  this_._impl_.subscription_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Subscription_AnalyticsHubSubscriptionInfo_class_data_ =
        Subscription_AnalyticsHubSubscriptionInfo::InternalGenerateClassData_(Subscription_AnalyticsHubSubscriptionInfo_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Subscription_AnalyticsHubSubscriptionInfo::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Subscription_AnalyticsHubSubscriptionInfo_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Subscription_AnalyticsHubSubscriptionInfo_class_data_.tc_table);
  return Subscription_AnalyticsHubSubscriptionInfo_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Subscription_AnalyticsHubSubscriptionInfo::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Subscription_AnalyticsHubSubscriptionInfo_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Subscription_AnalyticsHubSubscriptionInfo_globals_));
  return Subscription_AnalyticsHubSubscriptionInfo_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Subscription_AnalyticsHubSubscriptionInfo::ParseTableT_
    Subscription_AnalyticsHubSubscriptionInfo::_table_ =
        Subscription_AnalyticsHubSubscriptionInfo::InternalGenerateParseTable_(Subscription_AnalyticsHubSubscriptionInfo_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Subscription_AnalyticsHubSubscriptionInfo::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.Subscription.AnalyticsHubSubscriptionInfo)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.listing_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.subscription_.ClearNonDefaultToEmpty();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Subscription_AnalyticsHubSubscriptionInfo::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Subscription_AnalyticsHubSubscriptionInfo& this_ = static_cast<const Subscription_AnalyticsHubSubscriptionInfo&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Subscription_AnalyticsHubSubscriptionInfo::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Subscription_AnalyticsHubSubscriptionInfo& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.Subscription.AnalyticsHubSubscriptionInfo)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string listing = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_listing().empty()) {
      const ::std::string& _s = this_._internal_listing();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Subscription.AnalyticsHubSubscriptionInfo.listing");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // string subscription = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_subscription().empty()) {
      const ::std::string& _s = this_._internal_subscription();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Subscription.AnalyticsHubSubscriptionInfo.subscription");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.Subscription.AnalyticsHubSubscriptionInfo)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Subscription_AnalyticsHubSubscriptionInfo::ByteSizeLong(const MessageLite& base) {
  const Subscription_AnalyticsHubSubscriptionInfo& this_ = static_cast<const Subscription_AnalyticsHubSubscriptionInfo&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Subscription_AnalyticsHubSubscriptionInfo::ByteSizeLong() const {
  const Subscription_AnalyticsHubSubscriptionInfo& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.Subscription.AnalyticsHubSubscriptionInfo)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string listing = 1 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_listing().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_listing());
      }
    }
    // string subscription = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_subscription().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_subscription());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Subscription_AnalyticsHubSubscriptionInfo::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Subscription_AnalyticsHubSubscriptionInfo*>(&to_msg);
  auto& from = static_cast<const Subscription_AnalyticsHubSubscriptionInfo&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.Subscription.AnalyticsHubSubscriptionInfo)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_listing().empty()) {
        _this->_internal_set_listing(from._internal_listing());
      } else {
        if (_this->_impl_.listing_.IsDefault()) {
          _this->_internal_set_listing("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_subscription().empty()) {
        _this->_internal_set_subscription(from._internal_subscription());
      } else {
        if (_this->_impl_.subscription_.IsDefault()) {
          _this->_internal_set_subscription("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Subscription_AnalyticsHubSubscriptionInfo::CopyFrom(const Subscription_AnalyticsHubSubscriptionInfo& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.Subscription.AnalyticsHubSubscriptionInfo)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Subscription_AnalyticsHubSubscriptionInfo::InternalSwap(Subscription_AnalyticsHubSubscriptionInfo* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.listing_, &other->_impl_.listing_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.subscription_, &other->_impl_.subscription_, arena);
}

::google::protobuf::Metadata Subscription_AnalyticsHubSubscriptionInfo::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

#if defined(PROTOBUF_CUSTOM_VTABLE)
Subscription_LabelsEntry_DoNotUse::Subscription_LabelsEntry_DoNotUse() : SuperType(Subscription_LabelsEntry_DoNotUse_get_class_data()) {}
Subscription_LabelsEntry_DoNotUse::Subscription_LabelsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
    : SuperType(arena, Subscription_LabelsEntry_DoNotUse_get_class_data()) {}
#else   // PROTOBUF_CUSTOM_VTABLE
Subscription_LabelsEntry_DoNotUse::Subscription_LabelsEntry_DoNotUse() : SuperType() {}
Subscription_LabelsEntry_DoNotUse::Subscription_LabelsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena) : SuperType(arena) {}
#endif  // PROTOBUF_CUSTOM_VTABLE
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Subscription_LabelsEntry_DoNotUse_class_data_ =
        Subscription_LabelsEntry_DoNotUse::InternalGenerateClassData_(Subscription_LabelsEntry_DoNotUse_globals_._default);

// including ~ all the vtables and everything they use become part
// of the same ~ SCC.
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Subscription_LabelsEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Subscription_LabelsEntry_DoNotUse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Subscription_LabelsEntry_DoNotUse_class_data_.tc_table);
  return Subscription_LabelsEntry_DoNotUse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Subscription_LabelsEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Subscription_LabelsEntry_DoNotUse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Subscription_LabelsEntry_DoNotUse_globals_));
  return Subscription_LabelsEntry_DoNotUse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Subscription_LabelsEntry_DoNotUse::ParseTableT_
    Subscription_LabelsEntry_DoNotUse::_table_ =
        Subscription_LabelsEntry_DoNotUse::InternalGenerateParseTable_(Subscription_LabelsEntry_DoNotUse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
// ===================================================================

void Subscription::clear_message_retention_duration() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.message_retention_duration_ != nullptr) _impl_.message_retention_duration_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
void Subscription::clear_topic_message_retention_duration() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.topic_message_retention_duration_ != nullptr) _impl_.topic_message_retention_duration_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000100U);
}
Subscription::Subscription(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Subscription_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.Subscription)
}
PROTOBUF_NDEBUG_INLINE Subscription::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::Subscription& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        name_(arena, from.name_),
        topic_(arena, from.topic_),
        filter_(arena, from.filter_),
        labels_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::Subscription,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.labels_)>()
          , from.labels_
        }
     {}

Subscription::Subscription(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Subscription& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Subscription_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Subscription* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.push_config_ = (CheckHasBit(cached_has_bits, 0x00000008U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.push_config_)
                : nullptr;
  _impl_.message_retention_duration_ = (CheckHasBit(cached_has_bits, 0x00000010U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.message_retention_duration_)
                : nullptr;
  _impl_.expiration_policy_ = (CheckHasBit(cached_has_bits, 0x00000020U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.expiration_policy_)
                : nullptr;
  _impl_.dead_letter_policy_ = (CheckHasBit(cached_has_bits, 0x00000040U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.dead_letter_policy_)
                : nullptr;
  _impl_.retry_policy_ = (CheckHasBit(cached_has_bits, 0x00000080U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.retry_policy_)
                : nullptr;
  _impl_.topic_message_retention_duration_ = (CheckHasBit(cached_has_bits, 0x00000100U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.topic_message_retention_duration_)
                : nullptr;
  _impl_.bigquery_config_ = (CheckHasBit(cached_has_bits, 0x00000200U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.bigquery_config_)
                : nullptr;
  _impl_.cloud_storage_config_ = (CheckHasBit(cached_has_bits, 0x00000400U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.cloud_storage_config_)
                : nullptr;
  _impl_.analytics_hub_subscription_info_ = (CheckHasBit(cached_has_bits, 0x00000800U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.analytics_hub_subscription_info_)
                : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, ack_deadline_seconds_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, ack_deadline_seconds_),
           offsetof(Impl_, state_) -
               offsetof(Impl_, ack_deadline_seconds_) +
               sizeof(Impl_::state_));

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.Subscription)
}
PROTOBUF_NDEBUG_INLINE Subscription::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        name_(arena),
        topic_(arena),
        filter_(arena),
        labels_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::Subscription,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Subscription, _impl_.labels_)>()
         }
     {}

inline void Subscription::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, push_config_),
           0,
           offsetof(Impl_, state_) -
               offsetof(Impl_, push_config_) +
               sizeof(Impl_::state_));
}
Subscription::~Subscription() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.Subscription)
  SharedDtor(*this);
}
inline void Subscription::SharedDtor(MessageLite& self) {
  Subscription& this_ = static_cast<Subscription&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.name_.Destroy();
  this_._impl_.topic_.Destroy();
  this_._impl_.filter_.Destroy();
  delete this_._impl_.push_config_;
  delete this_._impl_.message_retention_duration_;
  delete this_._impl_.expiration_policy_;
  delete this_._impl_.dead_letter_policy_;
  delete this_._impl_.retry_policy_;
  delete this_._impl_.topic_message_retention_duration_;
  delete this_._impl_.bigquery_config_;
  delete this_._impl_.cloud_storage_config_;
  delete this_._impl_.analytics_hub_subscription_info_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Subscription_class_data_ =
        Subscription::InternalGenerateClassData_(Subscription_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Subscription::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Subscription_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Subscription_class_data_.tc_table);
  return Subscription_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Subscription::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Subscription_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Subscription_globals_));
  return Subscription_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Subscription::ParseTableT_
    Subscription::_table_ =
        Subscription::InternalGenerateParseTable_(Subscription_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Subscription::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.Subscription)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.name_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.topic_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.filter_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(_impl_.push_config_ != nullptr);
      _impl_.push_config_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      ABSL_DCHECK(_impl_.message_retention_duration_ != nullptr);
      _impl_.message_retention_duration_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      ABSL_DCHECK(_impl_.expiration_policy_ != nullptr);
      _impl_.expiration_policy_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      ABSL_DCHECK(_impl_.dead_letter_policy_ != nullptr);
      _impl_.dead_letter_policy_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      ABSL_DCHECK(_impl_.retry_policy_ != nullptr);
      _impl_.retry_policy_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000f00U)) {
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      ABSL_DCHECK(_impl_.topic_message_retention_duration_ != nullptr);
      _impl_.topic_message_retention_duration_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000200U)) {
      ABSL_DCHECK(_impl_.bigquery_config_ != nullptr);
      _impl_.bigquery_config_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000400U)) {
      ABSL_DCHECK(_impl_.cloud_storage_config_ != nullptr);
      _impl_.cloud_storage_config_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000800U)) {
      ABSL_DCHECK(_impl_.analytics_hub_subscription_info_ != nullptr);
      _impl_.analytics_hub_subscription_info_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000f000U)) {
    ::memset(&_impl_.ack_deadline_seconds_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.detached_) -
        reinterpret_cast<char*>(&_impl_.ack_deadline_seconds_)) + sizeof(_impl_.detached_));
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00070000U)) {
    ::memset(&_impl_.enable_exactly_once_delivery_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.state_) -
        reinterpret_cast<char*>(&_impl_.enable_exactly_once_delivery_)) + sizeof(_impl_.state_));
    if (CheckHasBit(cached_has_bits, 0x00040000U)) {
      _impl_.labels_.Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Subscription::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Subscription& this_ = static_cast<const Subscription&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Subscription::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Subscription& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.Subscription)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string name = 1 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_name().empty()) {
      const ::std::string& _s = this_._internal_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Subscription.name");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // string topic = 2 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_topic().empty()) {
      const ::std::string& _s = this_._internal_topic();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Subscription.topic");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  // .google.pubsub.v1.PushConfig push_config = 4 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        4, *this_._impl_.push_config_, this_._impl_.push_config_->GetCachedSize(), target,
        stream);
  }

  // int32 ack_deadline_seconds = 5 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00001000U)) {
    if (this_._internal_ack_deadline_seconds() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<5>(
              stream, this_._internal_ack_deadline_seconds(), target);
    }
  }

  // bool retain_acked_messages = 7 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00002000U)) {
    if (this_._internal_retain_acked_messages() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          7, this_._internal_retain_acked_messages(), target);
    }
  }

  // .google.protobuf.Duration message_retention_duration = 8 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        8, *this_._impl_.message_retention_duration_, this_._impl_.message_retention_duration_->GetCachedSize(), target,
        stream);
  }

  // map<string, string> labels = 9 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00040000U)) {
    if (!this_._internal_labels().empty()) {
      using MapType = ::google::protobuf::Map<::std::string, ::std::string>;
      using WireHelper = _pbi::MapEntryFuncs<::std::string, ::std::string,
                                     _pbi::WireFormatLite::TYPE_STRING,
                                     _pbi::WireFormatLite::TYPE_STRING>;
      const auto& field = this_._internal_labels();

      if (stream->IsSerializationDeterministic() && field.size() > 1) {
        for (const auto& entry : ::google::protobuf::internal::MapSorterPtr<MapType>(field)) {
          target = WireHelper::InternalSerialize(
              9, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Subscription.labels");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Subscription.labels");
        }
      } else {
        for (const auto& entry : field) {
          target = WireHelper::InternalSerialize(
              9, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Subscription.labels");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Subscription.labels");
        }
      }
    }
  }

  // bool enable_message_ordering = 10 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00004000U)) {
    if (this_._internal_enable_message_ordering() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          10, this_._internal_enable_message_ordering(), target);
    }
  }

  // .google.pubsub.v1.ExpirationPolicy expiration_policy = 11 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        11, *this_._impl_.expiration_policy_, this_._impl_.expiration_policy_->GetCachedSize(), target,
        stream);
  }

  // string filter = 12 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_filter().empty()) {
      const ::std::string& _s = this_._internal_filter();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Subscription.filter");
      target = stream->WriteStringMaybeAliased(12, _s, target);
    }
  }

  // .google.pubsub.v1.DeadLetterPolicy dead_letter_policy = 13 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        13, *this_._impl_.dead_letter_policy_, this_._impl_.dead_letter_policy_->GetCachedSize(), target,
        stream);
  }

  // .google.pubsub.v1.RetryPolicy retry_policy = 14 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        14, *this_._impl_.retry_policy_, this_._impl_.retry_policy_->GetCachedSize(), target,
        stream);
  }

  // bool detached = 15 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00008000U)) {
    if (this_._internal_detached() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          15, this_._internal_detached(), target);
    }
  }

  // bool enable_exactly_once_delivery = 16 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00010000U)) {
    if (this_._internal_enable_exactly_once_delivery() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          16, this_._internal_enable_exactly_once_delivery(), target);
    }
  }

  // .google.protobuf.Duration topic_message_retention_duration = 17 [(.google.api.field_behavior) = OUTPUT_ONLY];
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        17, *this_._impl_.topic_message_retention_duration_, this_._impl_.topic_message_retention_duration_->GetCachedSize(), target,
        stream);
  }

  // .google.pubsub.v1.BigQueryConfig bigquery_config = 18 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000200U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        18, *this_._impl_.bigquery_config_, this_._impl_.bigquery_config_->GetCachedSize(), target,
        stream);
  }

  // .google.pubsub.v1.Subscription.State state = 19 [(.google.api.field_behavior) = OUTPUT_ONLY];
  if (CheckHasBit(cached_has_bits, 0x00020000U)) {
    if (this_._internal_state() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          19, this_._internal_state(), target);
    }
  }

  // .google.pubsub.v1.CloudStorageConfig cloud_storage_config = 22 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000400U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        22, *this_._impl_.cloud_storage_config_, this_._impl_.cloud_storage_config_->GetCachedSize(), target,
        stream);
  }

  // .google.pubsub.v1.Subscription.AnalyticsHubSubscriptionInfo analytics_hub_subscription_info = 23 [(.google.api.field_behavior) = OUTPUT_ONLY];
  if (CheckHasBit(cached_has_bits, 0x00000800U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        23, *this_._impl_.analytics_hub_subscription_info_, this_._impl_.analytics_hub_subscription_info_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.Subscription)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Subscription::ByteSizeLong(const MessageLite& base) {
  const Subscription& this_ = static_cast<const Subscription&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Subscription::ByteSizeLong() const {
  const Subscription& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.Subscription)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // string name = 1 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_name());
      }
    }
    // string topic = 2 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_topic().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_topic());
      }
    }
    // string filter = 12 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!this_._internal_filter().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_filter());
      }
    }
    // .google.pubsub.v1.PushConfig push_config = 4 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.push_config_);
    }
    // .google.protobuf.Duration message_retention_duration = 8 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.message_retention_duration_);
    }
    // .google.pubsub.v1.ExpirationPolicy expiration_policy = 11 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.expiration_policy_);
    }
    // .google.pubsub.v1.DeadLetterPolicy dead_letter_policy = 13 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.dead_letter_policy_);
    }
    // .google.pubsub.v1.RetryPolicy retry_policy = 14 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.retry_policy_);
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000ff00U)) {
    // .google.protobuf.Duration topic_message_retention_duration = 17 [(.google.api.field_behavior) = OUTPUT_ONLY];
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      total_size += 2 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.topic_message_retention_duration_);
    }
    // .google.pubsub.v1.BigQueryConfig bigquery_config = 18 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000200U)) {
      total_size += 2 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.bigquery_config_);
    }
    // .google.pubsub.v1.CloudStorageConfig cloud_storage_config = 22 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000400U)) {
      total_size += 2 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.cloud_storage_config_);
    }
    // .google.pubsub.v1.Subscription.AnalyticsHubSubscriptionInfo analytics_hub_subscription_info = 23 [(.google.api.field_behavior) = OUTPUT_ONLY];
    if (CheckHasBit(cached_has_bits, 0x00000800U)) {
      total_size += 2 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.analytics_hub_subscription_info_);
    }
    // int32 ack_deadline_seconds = 5 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00001000U)) {
      if (this_._internal_ack_deadline_seconds() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_ack_deadline_seconds());
      }
    }
    // bool retain_acked_messages = 7 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00002000U)) {
      if (this_._internal_retain_acked_messages() != 0) {
        total_size += 2;
      }
    }
    // bool enable_message_ordering = 10 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00004000U)) {
      if (this_._internal_enable_message_ordering() != 0) {
        total_size += 2;
      }
    }
    // bool detached = 15 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00008000U)) {
      if (this_._internal_detached() != 0) {
        total_size += 2;
      }
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00070000U)) {
    // bool enable_exactly_once_delivery = 16 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00010000U)) {
      if (this_._internal_enable_exactly_once_delivery() != 0) {
        total_size += 3;
      }
    }
    // .google.pubsub.v1.Subscription.State state = 19 [(.google.api.field_behavior) = OUTPUT_ONLY];
    if (CheckHasBit(cached_has_bits, 0x00020000U)) {
      if (this_._internal_state() != 0) {
        total_size += 2 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_state());
      }
    }
    // map<string, string> labels = 9 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00040000U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_labels_size());
      for (const auto& entry : this_._internal_labels()) {
        total_size += _pbi::MapEntryFuncs<::std::string, ::std::string,
                                       _pbi::WireFormatLite::TYPE_STRING,
                                       _pbi::WireFormatLite::TYPE_STRING>::ByteSizeLong(entry.first, entry.second);
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Subscription::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Subscription*>(&to_msg);
  auto& from = static_cast<const Subscription&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.Subscription)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_name().empty()) {
        _this->_internal_set_name(from._internal_name());
      } else {
        if (_this->_impl_.name_.IsDefault()) {
          _this->_internal_set_name("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_topic().empty()) {
        _this->_internal_set_topic(from._internal_topic());
      } else {
        if (_this->_impl_.topic_.IsDefault()) {
          _this->_internal_set_topic("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!from._internal_filter().empty()) {
        _this->_internal_set_filter(from._internal_filter());
      } else {
        if (_this->_impl_.filter_.IsDefault()) {
          _this->_internal_set_filter("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(from._impl_.push_config_ != nullptr);
      if (_this->_impl_.push_config_ == nullptr) {
        _this->_impl_.push_config_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.push_config_);
      } else {
        _this->_impl_.push_config_->MergeFrom(*from._impl_.push_config_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      ABSL_DCHECK(from._impl_.message_retention_duration_ != nullptr);
      if (_this->_impl_.message_retention_duration_ == nullptr) {
        _this->_impl_.message_retention_duration_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.message_retention_duration_);
      } else {
        _this->_impl_.message_retention_duration_->MergeFrom(*from._impl_.message_retention_duration_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      ABSL_DCHECK(from._impl_.expiration_policy_ != nullptr);
      if (_this->_impl_.expiration_policy_ == nullptr) {
        _this->_impl_.expiration_policy_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.expiration_policy_);
      } else {
        _this->_impl_.expiration_policy_->MergeFrom(*from._impl_.expiration_policy_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      ABSL_DCHECK(from._impl_.dead_letter_policy_ != nullptr);
      if (_this->_impl_.dead_letter_policy_ == nullptr) {
        _this->_impl_.dead_letter_policy_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.dead_letter_policy_);
      } else {
        _this->_impl_.dead_letter_policy_->MergeFrom(*from._impl_.dead_letter_policy_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      ABSL_DCHECK(from._impl_.retry_policy_ != nullptr);
      if (_this->_impl_.retry_policy_ == nullptr) {
        _this->_impl_.retry_policy_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.retry_policy_);
      } else {
        _this->_impl_.retry_policy_->MergeFrom(*from._impl_.retry_policy_);
      }
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000ff00U)) {
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      ABSL_DCHECK(from._impl_.topic_message_retention_duration_ != nullptr);
      if (_this->_impl_.topic_message_retention_duration_ == nullptr) {
        _this->_impl_.topic_message_retention_duration_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.topic_message_retention_duration_);
      } else {
        _this->_impl_.topic_message_retention_duration_->MergeFrom(*from._impl_.topic_message_retention_duration_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000200U)) {
      ABSL_DCHECK(from._impl_.bigquery_config_ != nullptr);
      if (_this->_impl_.bigquery_config_ == nullptr) {
        _this->_impl_.bigquery_config_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.bigquery_config_);
      } else {
        _this->_impl_.bigquery_config_->MergeFrom(*from._impl_.bigquery_config_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000400U)) {
      ABSL_DCHECK(from._impl_.cloud_storage_config_ != nullptr);
      if (_this->_impl_.cloud_storage_config_ == nullptr) {
        _this->_impl_.cloud_storage_config_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.cloud_storage_config_);
      } else {
        _this->_impl_.cloud_storage_config_->MergeFrom(*from._impl_.cloud_storage_config_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000800U)) {
      ABSL_DCHECK(from._impl_.analytics_hub_subscription_info_ != nullptr);
      if (_this->_impl_.analytics_hub_subscription_info_ == nullptr) {
        _this->_impl_.analytics_hub_subscription_info_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.analytics_hub_subscription_info_);
      } else {
        _this->_impl_.analytics_hub_subscription_info_->MergeFrom(*from._impl_.analytics_hub_subscription_info_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00001000U)) {
      if (from._internal_ack_deadline_seconds() != 0) {
        _this->_impl_.ack_deadline_seconds_ = from._impl_.ack_deadline_seconds_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00002000U)) {
      if (from._internal_retain_acked_messages() != 0) {
        _this->_impl_.retain_acked_messages_ = from._impl_.retain_acked_messages_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00004000U)) {
      if (from._internal_enable_message_ordering() != 0) {
        _this->_impl_.enable_message_ordering_ = from._impl_.enable_message_ordering_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00008000U)) {
      if (from._internal_detached() != 0) {
        _this->_impl_.detached_ = from._impl_.detached_;
      }
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00070000U)) {
    if (CheckHasBit(cached_has_bits, 0x00010000U)) {
      if (from._internal_enable_exactly_once_delivery() != 0) {
        _this->_impl_.enable_exactly_once_delivery_ = from._impl_.enable_exactly_once_delivery_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00020000U)) {
      if (from._internal_state() != 0) {
        _this->_impl_.state_ = from._impl_.state_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00040000U)) {
      _this->_impl_.labels_.MergeFrom(from._impl_.labels_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Subscription::CopyFrom(const Subscription& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.Subscription)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Subscription::InternalSwap(Subscription* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.name_, &other->_impl_.name_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.topic_, &other->_impl_.topic_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.filter_, &other->_impl_.filter_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Subscription, _impl_.state_)
      + sizeof(Subscription::_impl_.state_)
      - PROTOBUF_FIELD_OFFSET(Subscription, _impl_.push_config_)>(
          reinterpret_cast<char*>(&_impl_.push_config_),
          reinterpret_cast<char*>(&other->_impl_.push_config_));
  _impl_.labels_.InternalSwap(&other->_impl_.labels_);
}

::google::protobuf::Metadata Subscription::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void RetryPolicy::clear_minimum_backoff() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.minimum_backoff_ != nullptr) _impl_.minimum_backoff_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
void RetryPolicy::clear_maximum_backoff() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.maximum_backoff_ != nullptr) _impl_.maximum_backoff_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
RetryPolicy::RetryPolicy(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, RetryPolicy_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.RetryPolicy)
}
PROTOBUF_NDEBUG_INLINE RetryPolicy::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::RetryPolicy& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

RetryPolicy::RetryPolicy(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const RetryPolicy& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, RetryPolicy_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  RetryPolicy* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.minimum_backoff_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.minimum_backoff_)
                : nullptr;
  _impl_.maximum_backoff_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.maximum_backoff_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.RetryPolicy)
}
PROTOBUF_NDEBUG_INLINE RetryPolicy::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void RetryPolicy::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, minimum_backoff_),
           0,
           offsetof(Impl_, maximum_backoff_) -
               offsetof(Impl_, minimum_backoff_) +
               sizeof(Impl_::maximum_backoff_));
}
RetryPolicy::~RetryPolicy() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.RetryPolicy)
  SharedDtor(*this);
}
inline void RetryPolicy::SharedDtor(MessageLite& self) {
  RetryPolicy& this_ = static_cast<RetryPolicy&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.minimum_backoff_;
  delete this_._impl_.maximum_backoff_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull RetryPolicy_class_data_ =
        RetryPolicy::InternalGenerateClassData_(RetryPolicy_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
RetryPolicy::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&RetryPolicy_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(RetryPolicy_class_data_.tc_table);
  return RetryPolicy_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
RetryPolicy::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&RetryPolicy_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&RetryPolicy_globals_));
  return RetryPolicy_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const RetryPolicy::ParseTableT_
    RetryPolicy::_table_ =
        RetryPolicy::InternalGenerateParseTable_(RetryPolicy_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void RetryPolicy::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.RetryPolicy)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(_impl_.minimum_backoff_ != nullptr);
      _impl_.minimum_backoff_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.maximum_backoff_ != nullptr);
      _impl_.maximum_backoff_->Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL RetryPolicy::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const RetryPolicy& this_ = static_cast<const RetryPolicy&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL RetryPolicy::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const RetryPolicy& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.RetryPolicy)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.protobuf.Duration minimum_backoff = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.minimum_backoff_, this_._impl_.minimum_backoff_->GetCachedSize(), target,
        stream);
  }

  // .google.protobuf.Duration maximum_backoff = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.maximum_backoff_, this_._impl_.maximum_backoff_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.RetryPolicy)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t RetryPolicy::ByteSizeLong(const MessageLite& base) {
  const RetryPolicy& this_ = static_cast<const RetryPolicy&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t RetryPolicy::ByteSizeLong() const {
  const RetryPolicy& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.RetryPolicy)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // .google.protobuf.Duration minimum_backoff = 1 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.minimum_backoff_);
    }
    // .google.protobuf.Duration maximum_backoff = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.maximum_backoff_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void RetryPolicy::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<RetryPolicy*>(&to_msg);
  auto& from = static_cast<const RetryPolicy&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.RetryPolicy)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.minimum_backoff_ != nullptr);
      if (_this->_impl_.minimum_backoff_ == nullptr) {
        _this->_impl_.minimum_backoff_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.minimum_backoff_);
      } else {
        _this->_impl_.minimum_backoff_->MergeFrom(*from._impl_.minimum_backoff_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.maximum_backoff_ != nullptr);
      if (_this->_impl_.maximum_backoff_ == nullptr) {
        _this->_impl_.maximum_backoff_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.maximum_backoff_);
      } else {
        _this->_impl_.maximum_backoff_->MergeFrom(*from._impl_.maximum_backoff_);
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void RetryPolicy::CopyFrom(const RetryPolicy& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.RetryPolicy)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void RetryPolicy::InternalSwap(RetryPolicy* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(RetryPolicy, _impl_.maximum_backoff_)
      + sizeof(RetryPolicy::_impl_.maximum_backoff_)
      - PROTOBUF_FIELD_OFFSET(RetryPolicy, _impl_.minimum_backoff_)>(
          reinterpret_cast<char*>(&_impl_.minimum_backoff_),
          reinterpret_cast<char*>(&other->_impl_.minimum_backoff_));
}

::google::protobuf::Metadata RetryPolicy::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

DeadLetterPolicy::DeadLetterPolicy(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DeadLetterPolicy_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.DeadLetterPolicy)
}
PROTOBUF_NDEBUG_INLINE DeadLetterPolicy::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::DeadLetterPolicy& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        dead_letter_topic_(arena, from.dead_letter_topic_) {}

DeadLetterPolicy::DeadLetterPolicy(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const DeadLetterPolicy& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DeadLetterPolicy_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  DeadLetterPolicy* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.max_delivery_attempts_ = from._impl_.max_delivery_attempts_;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.DeadLetterPolicy)
}
PROTOBUF_NDEBUG_INLINE DeadLetterPolicy::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        dead_letter_topic_(arena) {}

inline void DeadLetterPolicy::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.max_delivery_attempts_ = {};
}
DeadLetterPolicy::~DeadLetterPolicy() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.DeadLetterPolicy)
  SharedDtor(*this);
}
inline void DeadLetterPolicy::SharedDtor(MessageLite& self) {
  DeadLetterPolicy& this_ = static_cast<DeadLetterPolicy&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.dead_letter_topic_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull DeadLetterPolicy_class_data_ =
        DeadLetterPolicy::InternalGenerateClassData_(DeadLetterPolicy_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DeadLetterPolicy::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DeadLetterPolicy_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(DeadLetterPolicy_class_data_.tc_table);
  return DeadLetterPolicy_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DeadLetterPolicy::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DeadLetterPolicy_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&DeadLetterPolicy_globals_));
  return DeadLetterPolicy_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const DeadLetterPolicy::ParseTableT_
    DeadLetterPolicy::_table_ =
        DeadLetterPolicy::InternalGenerateParseTable_(DeadLetterPolicy_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void DeadLetterPolicy::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.DeadLetterPolicy)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.dead_letter_topic_.ClearNonDefaultToEmpty();
  }
  _impl_.max_delivery_attempts_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL DeadLetterPolicy::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const DeadLetterPolicy& this_ = static_cast<const DeadLetterPolicy&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL DeadLetterPolicy::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const DeadLetterPolicy& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.DeadLetterPolicy)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string dead_letter_topic = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_dead_letter_topic().empty()) {
      const ::std::string& _s = this_._internal_dead_letter_topic();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.DeadLetterPolicy.dead_letter_topic");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // int32 max_delivery_attempts = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_max_delivery_attempts() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<2>(
              stream, this_._internal_max_delivery_attempts(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.DeadLetterPolicy)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t DeadLetterPolicy::ByteSizeLong(const MessageLite& base) {
  const DeadLetterPolicy& this_ = static_cast<const DeadLetterPolicy&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t DeadLetterPolicy::ByteSizeLong() const {
  const DeadLetterPolicy& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.DeadLetterPolicy)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string dead_letter_topic = 1 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_dead_letter_topic().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_dead_letter_topic());
      }
    }
    // int32 max_delivery_attempts = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_max_delivery_attempts() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_max_delivery_attempts());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void DeadLetterPolicy::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<DeadLetterPolicy*>(&to_msg);
  auto& from = static_cast<const DeadLetterPolicy&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.DeadLetterPolicy)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_dead_letter_topic().empty()) {
        _this->_internal_set_dead_letter_topic(from._internal_dead_letter_topic());
      } else {
        if (_this->_impl_.dead_letter_topic_.IsDefault()) {
          _this->_internal_set_dead_letter_topic("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_max_delivery_attempts() != 0) {
        _this->_impl_.max_delivery_attempts_ = from._impl_.max_delivery_attempts_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void DeadLetterPolicy::CopyFrom(const DeadLetterPolicy& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.DeadLetterPolicy)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void DeadLetterPolicy::InternalSwap(DeadLetterPolicy* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.dead_letter_topic_, &other->_impl_.dead_letter_topic_, arena);
  swap(_impl_.max_delivery_attempts_, other->_impl_.max_delivery_attempts_);
}

::google::protobuf::Metadata DeadLetterPolicy::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void ExpirationPolicy::clear_ttl() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.ttl_ != nullptr) _impl_.ttl_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
ExpirationPolicy::ExpirationPolicy(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ExpirationPolicy_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ExpirationPolicy)
}
PROTOBUF_NDEBUG_INLINE ExpirationPolicy::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::ExpirationPolicy& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

ExpirationPolicy::ExpirationPolicy(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ExpirationPolicy& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ExpirationPolicy_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ExpirationPolicy* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.ttl_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.ttl_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ExpirationPolicy)
}
PROTOBUF_NDEBUG_INLINE ExpirationPolicy::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void ExpirationPolicy::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.ttl_ = {};
}
ExpirationPolicy::~ExpirationPolicy() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.ExpirationPolicy)
  SharedDtor(*this);
}
inline void ExpirationPolicy::SharedDtor(MessageLite& self) {
  ExpirationPolicy& this_ = static_cast<ExpirationPolicy&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.ttl_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ExpirationPolicy_class_data_ =
        ExpirationPolicy::InternalGenerateClassData_(ExpirationPolicy_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ExpirationPolicy::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ExpirationPolicy_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ExpirationPolicy_class_data_.tc_table);
  return ExpirationPolicy_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ExpirationPolicy::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ExpirationPolicy_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ExpirationPolicy_globals_));
  return ExpirationPolicy_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ExpirationPolicy::ParseTableT_
    ExpirationPolicy::_table_ =
        ExpirationPolicy::InternalGenerateParseTable_(ExpirationPolicy_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ExpirationPolicy::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.ExpirationPolicy)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(_impl_.ttl_ != nullptr);
    _impl_.ttl_->Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ExpirationPolicy::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ExpirationPolicy& this_ = static_cast<const ExpirationPolicy&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ExpirationPolicy::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ExpirationPolicy& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.ExpirationPolicy)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.protobuf.Duration ttl = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.ttl_, this_._impl_.ttl_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.ExpirationPolicy)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ExpirationPolicy::ByteSizeLong(const MessageLite& base) {
  const ExpirationPolicy& this_ = static_cast<const ExpirationPolicy&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ExpirationPolicy::ByteSizeLong() const {
  const ExpirationPolicy& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.ExpirationPolicy)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // .google.protobuf.Duration ttl = 1 [(.google.api.field_behavior) = OPTIONAL];
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.ttl_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ExpirationPolicy::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ExpirationPolicy*>(&to_msg);
  auto& from = static_cast<const ExpirationPolicy&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.ExpirationPolicy)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(from._impl_.ttl_ != nullptr);
    if (_this->_impl_.ttl_ == nullptr) {
      _this->_impl_.ttl_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.ttl_);
    } else {
      _this->_impl_.ttl_->MergeFrom(*from._impl_.ttl_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ExpirationPolicy::CopyFrom(const ExpirationPolicy& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.ExpirationPolicy)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ExpirationPolicy::InternalSwap(ExpirationPolicy* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.ttl_, other->_impl_.ttl_);
}

::google::protobuf::Metadata ExpirationPolicy::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

PushConfig_OidcToken::PushConfig_OidcToken(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PushConfig_OidcToken_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.PushConfig.OidcToken)
}
PROTOBUF_NDEBUG_INLINE PushConfig_OidcToken::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::PushConfig_OidcToken& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        service_account_email_(arena, from.service_account_email_),
        audience_(arena, from.audience_) {}

PushConfig_OidcToken::PushConfig_OidcToken(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const PushConfig_OidcToken& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PushConfig_OidcToken_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  PushConfig_OidcToken* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.PushConfig.OidcToken)
}
PROTOBUF_NDEBUG_INLINE PushConfig_OidcToken::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        service_account_email_(arena),
        audience_(arena) {}

inline void PushConfig_OidcToken::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
PushConfig_OidcToken::~PushConfig_OidcToken() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.PushConfig.OidcToken)
  SharedDtor(*this);
}
inline void PushConfig_OidcToken::SharedDtor(MessageLite& self) {
  PushConfig_OidcToken& this_ = static_cast<PushConfig_OidcToken&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.service_account_email_.Destroy();
  this_._impl_.audience_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull PushConfig_OidcToken_class_data_ =
        PushConfig_OidcToken::InternalGenerateClassData_(PushConfig_OidcToken_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PushConfig_OidcToken::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PushConfig_OidcToken_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(PushConfig_OidcToken_class_data_.tc_table);
  return PushConfig_OidcToken_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PushConfig_OidcToken::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PushConfig_OidcToken_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&PushConfig_OidcToken_globals_));
  return PushConfig_OidcToken_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const PushConfig_OidcToken::ParseTableT_
    PushConfig_OidcToken::_table_ =
        PushConfig_OidcToken::InternalGenerateParseTable_(PushConfig_OidcToken_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void PushConfig_OidcToken::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.PushConfig.OidcToken)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.service_account_email_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.audience_.ClearNonDefaultToEmpty();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL PushConfig_OidcToken::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const PushConfig_OidcToken& this_ = static_cast<const PushConfig_OidcToken&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL PushConfig_OidcToken::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const PushConfig_OidcToken& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.PushConfig.OidcToken)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string service_account_email = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_service_account_email().empty()) {
      const ::std::string& _s = this_._internal_service_account_email();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.PushConfig.OidcToken.service_account_email");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // string audience = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_audience().empty()) {
      const ::std::string& _s = this_._internal_audience();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.PushConfig.OidcToken.audience");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.PushConfig.OidcToken)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t PushConfig_OidcToken::ByteSizeLong(const MessageLite& base) {
  const PushConfig_OidcToken& this_ = static_cast<const PushConfig_OidcToken&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t PushConfig_OidcToken::ByteSizeLong() const {
  const PushConfig_OidcToken& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.PushConfig.OidcToken)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string service_account_email = 1 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_service_account_email().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_service_account_email());
      }
    }
    // string audience = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_audience().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_audience());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void PushConfig_OidcToken::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<PushConfig_OidcToken*>(&to_msg);
  auto& from = static_cast<const PushConfig_OidcToken&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.PushConfig.OidcToken)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_service_account_email().empty()) {
        _this->_internal_set_service_account_email(from._internal_service_account_email());
      } else {
        if (_this->_impl_.service_account_email_.IsDefault()) {
          _this->_internal_set_service_account_email("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_audience().empty()) {
        _this->_internal_set_audience(from._internal_audience());
      } else {
        if (_this->_impl_.audience_.IsDefault()) {
          _this->_internal_set_audience("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void PushConfig_OidcToken::CopyFrom(const PushConfig_OidcToken& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.PushConfig.OidcToken)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void PushConfig_OidcToken::InternalSwap(PushConfig_OidcToken* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.service_account_email_, &other->_impl_.service_account_email_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.audience_, &other->_impl_.audience_, arena);
}

::google::protobuf::Metadata PushConfig_OidcToken::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

PushConfig_PubsubWrapper::PushConfig_PubsubWrapper(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, PushConfig_PubsubWrapper_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.PushConfig.PubsubWrapper)
}
PushConfig_PubsubWrapper::PushConfig_PubsubWrapper(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const PushConfig_PubsubWrapper& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, PushConfig_PubsubWrapper_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  PushConfig_PubsubWrapper* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.PushConfig.PubsubWrapper)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull PushConfig_PubsubWrapper_class_data_ =
        PushConfig_PubsubWrapper::InternalGenerateClassData_(PushConfig_PubsubWrapper_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PushConfig_PubsubWrapper::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PushConfig_PubsubWrapper_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(PushConfig_PubsubWrapper_class_data_.tc_table);
  return PushConfig_PubsubWrapper_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PushConfig_PubsubWrapper::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PushConfig_PubsubWrapper_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&PushConfig_PubsubWrapper_globals_));
  return PushConfig_PubsubWrapper_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const PushConfig_PubsubWrapper::ParseTableT_
    PushConfig_PubsubWrapper::_table_ =
        PushConfig_PubsubWrapper::InternalGenerateParseTable_(PushConfig_PubsubWrapper_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata PushConfig_PubsubWrapper::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

PushConfig_NoWrapper::PushConfig_NoWrapper(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PushConfig_NoWrapper_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.PushConfig.NoWrapper)
}
PushConfig_NoWrapper::PushConfig_NoWrapper(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PushConfig_NoWrapper& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PushConfig_NoWrapper_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE PushConfig_NoWrapper::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void PushConfig_NoWrapper::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.write_metadata_ = {};
}
PushConfig_NoWrapper::~PushConfig_NoWrapper() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.PushConfig.NoWrapper)
  SharedDtor(*this);
}
inline void PushConfig_NoWrapper::SharedDtor(MessageLite& self) {
  PushConfig_NoWrapper& this_ = static_cast<PushConfig_NoWrapper&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull PushConfig_NoWrapper_class_data_ =
        PushConfig_NoWrapper::InternalGenerateClassData_(PushConfig_NoWrapper_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PushConfig_NoWrapper::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PushConfig_NoWrapper_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(PushConfig_NoWrapper_class_data_.tc_table);
  return PushConfig_NoWrapper_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PushConfig_NoWrapper::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PushConfig_NoWrapper_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&PushConfig_NoWrapper_globals_));
  return PushConfig_NoWrapper_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const PushConfig_NoWrapper::ParseTableT_
    PushConfig_NoWrapper::_table_ =
        PushConfig_NoWrapper::InternalGenerateParseTable_(PushConfig_NoWrapper_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void PushConfig_NoWrapper::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.PushConfig.NoWrapper)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.write_metadata_ = false;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL PushConfig_NoWrapper::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const PushConfig_NoWrapper& this_ = static_cast<const PushConfig_NoWrapper&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL PushConfig_NoWrapper::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const PushConfig_NoWrapper& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.PushConfig.NoWrapper)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // bool write_metadata = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_write_metadata() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          1, this_._internal_write_metadata(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.PushConfig.NoWrapper)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t PushConfig_NoWrapper::ByteSizeLong(const MessageLite& base) {
  const PushConfig_NoWrapper& this_ = static_cast<const PushConfig_NoWrapper&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t PushConfig_NoWrapper::ByteSizeLong() const {
  const PushConfig_NoWrapper& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.PushConfig.NoWrapper)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // bool write_metadata = 1 [(.google.api.field_behavior) = OPTIONAL];
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_write_metadata() != 0) {
        total_size += 2;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void PushConfig_NoWrapper::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<PushConfig_NoWrapper*>(&to_msg);
  auto& from = static_cast<const PushConfig_NoWrapper&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.PushConfig.NoWrapper)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (from._internal_write_metadata() != 0) {
      _this->_impl_.write_metadata_ = from._impl_.write_metadata_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void PushConfig_NoWrapper::CopyFrom(const PushConfig_NoWrapper& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.PushConfig.NoWrapper)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void PushConfig_NoWrapper::InternalSwap(PushConfig_NoWrapper* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.write_metadata_, other->_impl_.write_metadata_);
}

::google::protobuf::Metadata PushConfig_NoWrapper::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

#if defined(PROTOBUF_CUSTOM_VTABLE)
PushConfig_AttributesEntry_DoNotUse::PushConfig_AttributesEntry_DoNotUse() : SuperType(PushConfig_AttributesEntry_DoNotUse_get_class_data()) {}
PushConfig_AttributesEntry_DoNotUse::PushConfig_AttributesEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
    : SuperType(arena, PushConfig_AttributesEntry_DoNotUse_get_class_data()) {}
#else   // PROTOBUF_CUSTOM_VTABLE
PushConfig_AttributesEntry_DoNotUse::PushConfig_AttributesEntry_DoNotUse() : SuperType() {}
PushConfig_AttributesEntry_DoNotUse::PushConfig_AttributesEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena) : SuperType(arena) {}
#endif  // PROTOBUF_CUSTOM_VTABLE
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull PushConfig_AttributesEntry_DoNotUse_class_data_ =
        PushConfig_AttributesEntry_DoNotUse::InternalGenerateClassData_(PushConfig_AttributesEntry_DoNotUse_globals_._default);

// including ~ all the vtables and everything they use become part
// of the same ~ SCC.
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PushConfig_AttributesEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PushConfig_AttributesEntry_DoNotUse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(PushConfig_AttributesEntry_DoNotUse_class_data_.tc_table);
  return PushConfig_AttributesEntry_DoNotUse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PushConfig_AttributesEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PushConfig_AttributesEntry_DoNotUse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&PushConfig_AttributesEntry_DoNotUse_globals_));
  return PushConfig_AttributesEntry_DoNotUse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const PushConfig_AttributesEntry_DoNotUse::ParseTableT_
    PushConfig_AttributesEntry_DoNotUse::_table_ =
        PushConfig_AttributesEntry_DoNotUse::InternalGenerateParseTable_(PushConfig_AttributesEntry_DoNotUse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
// ===================================================================

void PushConfig::set_allocated_oidc_token(::google::pubsub::v1::PushConfig_OidcToken* PROTOBUF_NULLABLE oidc_token) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_authentication_method();
  if (oidc_token) {
    ::google::protobuf::Arena* submessage_arena = oidc_token->GetArena();
    if (message_arena != submessage_arena) {
      oidc_token = ::google::protobuf::internal::GetOwnedMessage(message_arena, oidc_token, submessage_arena);
    }
    set_has_oidc_token();
    _impl_.authentication_method_.oidc_token_ = oidc_token;
  }
  // @@protoc_insertion_point(field_set_allocated:google.pubsub.v1.PushConfig.oidc_token)
}
void PushConfig::set_allocated_pubsub_wrapper(::google::pubsub::v1::PushConfig_PubsubWrapper* PROTOBUF_NULLABLE pubsub_wrapper) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_wrapper();
  if (pubsub_wrapper) {
    ::google::protobuf::Arena* submessage_arena = pubsub_wrapper->GetArena();
    if (message_arena != submessage_arena) {
      pubsub_wrapper = ::google::protobuf::internal::GetOwnedMessage(message_arena, pubsub_wrapper, submessage_arena);
    }
    set_has_pubsub_wrapper();
    _impl_.wrapper_.pubsub_wrapper_ = pubsub_wrapper;
  }
  // @@protoc_insertion_point(field_set_allocated:google.pubsub.v1.PushConfig.pubsub_wrapper)
}
void PushConfig::set_allocated_no_wrapper(::google::pubsub::v1::PushConfig_NoWrapper* PROTOBUF_NULLABLE no_wrapper) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_wrapper();
  if (no_wrapper) {
    ::google::protobuf::Arena* submessage_arena = no_wrapper->GetArena();
    if (message_arena != submessage_arena) {
      no_wrapper = ::google::protobuf::internal::GetOwnedMessage(message_arena, no_wrapper, submessage_arena);
    }
    set_has_no_wrapper();
    _impl_.wrapper_.no_wrapper_ = no_wrapper;
  }
  // @@protoc_insertion_point(field_set_allocated:google.pubsub.v1.PushConfig.no_wrapper)
}
PushConfig::PushConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PushConfig_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.PushConfig)
}
PROTOBUF_NDEBUG_INLINE PushConfig::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::PushConfig& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        push_endpoint_(arena, from.push_endpoint_),
        attributes_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::PushConfig,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PushConfig, _impl_.attributes_)>()
          , from.attributes_
        }
        ,
        authentication_method_{},
        wrapper_{},
        _oneof_case_{from._oneof_case_[0], from._oneof_case_[1]} {}

PushConfig::PushConfig(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const PushConfig& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PushConfig_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  PushConfig* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  switch (authentication_method_case()) {
    case AUTHENTICATION_METHOD_NOT_SET:
      break;
      case kOidcToken:
        _impl_.authentication_method_.oidc_token_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.authentication_method_.oidc_token_);
        break;
  }
  switch (wrapper_case()) {
    case WRAPPER_NOT_SET:
      break;
      case kPubsubWrapper:
        _impl_.wrapper_.pubsub_wrapper_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.wrapper_.pubsub_wrapper_);
        break;
      case kNoWrapper:
        _impl_.wrapper_.no_wrapper_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.wrapper_.no_wrapper_);
        break;
  }

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.PushConfig)
}
PROTOBUF_NDEBUG_INLINE PushConfig::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        push_endpoint_(arena),
        attributes_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::PushConfig,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PushConfig, _impl_.attributes_)>()
         }
        ,
        authentication_method_{},
        wrapper_{},
        _oneof_case_{} {}

inline void PushConfig::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
PushConfig::~PushConfig() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.PushConfig)
  SharedDtor(*this);
}
inline void PushConfig::SharedDtor(MessageLite& self) {
  PushConfig& this_ = static_cast<PushConfig&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.push_endpoint_.Destroy();
  if (this_.has_authentication_method()) {
    this_.clear_authentication_method();
  }
  if (this_.has_wrapper()) {
    this_.clear_wrapper();
  }
  this_._impl_.~Impl_();
}

void PushConfig::clear_authentication_method() {
// @@protoc_insertion_point(one_of_clear_start:google.pubsub.v1.PushConfig)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (authentication_method_case()) {
    case kOidcToken: {
      if (GetArena() == nullptr) {
        delete _impl_.authentication_method_.oidc_token_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.authentication_method_.oidc_token_);
      }
      break;
    }
    case AUTHENTICATION_METHOD_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = AUTHENTICATION_METHOD_NOT_SET;
}

void PushConfig::clear_wrapper() {
// @@protoc_insertion_point(one_of_clear_start:google.pubsub.v1.PushConfig)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (wrapper_case()) {
    case kPubsubWrapper: {
      if (GetArena() == nullptr) {
        delete _impl_.wrapper_.pubsub_wrapper_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.wrapper_.pubsub_wrapper_);
      }
      break;
    }
    case kNoWrapper: {
      if (GetArena() == nullptr) {
        delete _impl_.wrapper_.no_wrapper_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.wrapper_.no_wrapper_);
      }
      break;
    }
    case WRAPPER_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[1] = WRAPPER_NOT_SET;
}


#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull PushConfig_class_data_ =
        PushConfig::InternalGenerateClassData_(PushConfig_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PushConfig::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PushConfig_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(PushConfig_class_data_.tc_table);
  return PushConfig_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PushConfig::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PushConfig_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&PushConfig_globals_));
  return PushConfig_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const PushConfig::ParseTableT_
    PushConfig::_table_ =
        PushConfig::InternalGenerateParseTable_(PushConfig_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void PushConfig::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.PushConfig)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.push_endpoint_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.attributes_.Clear();
    }
  }
  clear_authentication_method();
  clear_wrapper();
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL PushConfig::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const PushConfig& this_ = static_cast<const PushConfig&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL PushConfig::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const PushConfig& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.PushConfig)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string push_endpoint = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_push_endpoint().empty()) {
      const ::std::string& _s = this_._internal_push_endpoint();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.PushConfig.push_endpoint");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // map<string, string> attributes = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_attributes().empty()) {
      using MapType = ::google::protobuf::Map<::std::string, ::std::string>;
      using WireHelper = _pbi::MapEntryFuncs<::std::string, ::std::string,
                                     _pbi::WireFormatLite::TYPE_STRING,
                                     _pbi::WireFormatLite::TYPE_STRING>;
      const auto& field = this_._internal_attributes();

      if (stream->IsSerializationDeterministic() && field.size() > 1) {
        for (const auto& entry : ::google::protobuf::internal::MapSorterPtr<MapType>(field)) {
          target = WireHelper::InternalSerialize(
              2, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.PushConfig.attributes");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.PushConfig.attributes");
        }
      } else {
        for (const auto& entry : field) {
          target = WireHelper::InternalSerialize(
              2, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.PushConfig.attributes");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.PushConfig.attributes");
        }
      }
    }
  }

  // .google.pubsub.v1.PushConfig.OidcToken oidc_token = 3 [(.google.api.field_behavior) = OPTIONAL];
  if (this_.authentication_method_case() == kOidcToken) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        3, *this_._impl_.authentication_method_.oidc_token_, this_._impl_.authentication_method_.oidc_token_->GetCachedSize(), target,
        stream);
  }

  switch (this_.wrapper_case()) {
    case kPubsubWrapper: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          4, *this_._impl_.wrapper_.pubsub_wrapper_, this_._impl_.wrapper_.pubsub_wrapper_->GetCachedSize(), target,
          stream);
      break;
    }
    case kNoWrapper: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          5, *this_._impl_.wrapper_.no_wrapper_, this_._impl_.wrapper_.no_wrapper_->GetCachedSize(), target,
          stream);
      break;
    }
    default:
      break;
  }
  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.PushConfig)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t PushConfig::ByteSizeLong(const MessageLite& base) {
  const PushConfig& this_ = static_cast<const PushConfig&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t PushConfig::ByteSizeLong() const {
  const PushConfig& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.PushConfig)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string push_endpoint = 1 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_push_endpoint().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_push_endpoint());
      }
    }
    // map<string, string> attributes = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_attributes_size());
      for (const auto& entry : this_._internal_attributes()) {
        total_size += _pbi::MapEntryFuncs<::std::string, ::std::string,
                                       _pbi::WireFormatLite::TYPE_STRING,
                                       _pbi::WireFormatLite::TYPE_STRING>::ByteSizeLong(entry.first, entry.second);
      }
    }
  }
  switch (this_.authentication_method_case()) {
    // .google.pubsub.v1.PushConfig.OidcToken oidc_token = 3 [(.google.api.field_behavior) = OPTIONAL];
    case kOidcToken: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.authentication_method_.oidc_token_);
      break;
    }
    case AUTHENTICATION_METHOD_NOT_SET: {
      break;
    }
  }
  switch (this_.wrapper_case()) {
    // .google.pubsub.v1.PushConfig.PubsubWrapper pubsub_wrapper = 4 [(.google.api.field_behavior) = OPTIONAL];
    case kPubsubWrapper: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.wrapper_.pubsub_wrapper_);
      break;
    }
    // .google.pubsub.v1.PushConfig.NoWrapper no_wrapper = 5 [(.google.api.field_behavior) = OPTIONAL];
    case kNoWrapper: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.wrapper_.no_wrapper_);
      break;
    }
    case WRAPPER_NOT_SET: {
      break;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void PushConfig::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<PushConfig*>(&to_msg);
  auto& from = static_cast<const PushConfig&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.PushConfig)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_push_endpoint().empty()) {
        _this->_internal_set_push_endpoint(from._internal_push_endpoint());
      } else {
        if (_this->_impl_.push_endpoint_.IsDefault()) {
          _this->_internal_set_push_endpoint("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _this->_impl_.attributes_.MergeFrom(from._impl_.attributes_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[0]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[0];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_authentication_method();
      }
      _this->_impl_._oneof_case_[0] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kOidcToken: {
        if (oneof_needs_init) {
          _this->_impl_.authentication_method_.oidc_token_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.authentication_method_.oidc_token_);
        } else {
          _this->_impl_.authentication_method_.oidc_token_->MergeFrom(*from._impl_.authentication_method_.oidc_token_);
        }
        break;
      }
      case AUTHENTICATION_METHOD_NOT_SET:
        break;
    }
  }
  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[1]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[1];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_wrapper();
      }
      _this->_impl_._oneof_case_[1] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kPubsubWrapper: {
        if (oneof_needs_init) {
          _this->_impl_.wrapper_.pubsub_wrapper_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.wrapper_.pubsub_wrapper_);
        } else {
          _this->_impl_.wrapper_.pubsub_wrapper_->MergeFrom(*from._impl_.wrapper_.pubsub_wrapper_);
        }
        break;
      }
      case kNoWrapper: {
        if (oneof_needs_init) {
          _this->_impl_.wrapper_.no_wrapper_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.wrapper_.no_wrapper_);
        } else {
          _this->_impl_.wrapper_.no_wrapper_->MergeFrom(*from._impl_.wrapper_.no_wrapper_);
        }
        break;
      }
      case WRAPPER_NOT_SET:
        break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void PushConfig::CopyFrom(const PushConfig& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.PushConfig)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void PushConfig::InternalSwap(PushConfig* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.push_endpoint_, &other->_impl_.push_endpoint_, arena);
  _impl_.attributes_.InternalSwap(&other->_impl_.attributes_);
  swap(_impl_.authentication_method_, other->_impl_.authentication_method_);
  swap(_impl_.wrapper_, other->_impl_.wrapper_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
  swap(_impl_._oneof_case_[1], other->_impl_._oneof_case_[1]);
}

::google::protobuf::Metadata PushConfig::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

BigQueryConfig::BigQueryConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, BigQueryConfig_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.BigQueryConfig)
}
PROTOBUF_NDEBUG_INLINE BigQueryConfig::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::BigQueryConfig& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        table_(arena, from.table_),
        service_account_email_(arena, from.service_account_email_) {}

BigQueryConfig::BigQueryConfig(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const BigQueryConfig& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, BigQueryConfig_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  BigQueryConfig* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, use_topic_schema_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, use_topic_schema_),
           offsetof(Impl_, state_) -
               offsetof(Impl_, use_topic_schema_) +
               sizeof(Impl_::state_));

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.BigQueryConfig)
}
PROTOBUF_NDEBUG_INLINE BigQueryConfig::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        table_(arena),
        service_account_email_(arena) {}

inline void BigQueryConfig::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, use_topic_schema_),
           0,
           offsetof(Impl_, state_) -
               offsetof(Impl_, use_topic_schema_) +
               sizeof(Impl_::state_));
}
BigQueryConfig::~BigQueryConfig() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.BigQueryConfig)
  SharedDtor(*this);
}
inline void BigQueryConfig::SharedDtor(MessageLite& self) {
  BigQueryConfig& this_ = static_cast<BigQueryConfig&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.table_.Destroy();
  this_._impl_.service_account_email_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull BigQueryConfig_class_data_ =
        BigQueryConfig::InternalGenerateClassData_(BigQueryConfig_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
BigQueryConfig::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&BigQueryConfig_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(BigQueryConfig_class_data_.tc_table);
  return BigQueryConfig_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
BigQueryConfig::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&BigQueryConfig_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&BigQueryConfig_globals_));
  return BigQueryConfig_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const BigQueryConfig::ParseTableT_
    BigQueryConfig::_table_ =
        BigQueryConfig::InternalGenerateParseTable_(BigQueryConfig_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void BigQueryConfig::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.BigQueryConfig)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.table_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.service_account_email_.ClearNonDefaultToEmpty();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000007cU)) {
    ::memset(&_impl_.use_topic_schema_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.state_) -
        reinterpret_cast<char*>(&_impl_.use_topic_schema_)) + sizeof(_impl_.state_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL BigQueryConfig::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const BigQueryConfig& this_ = static_cast<const BigQueryConfig&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL BigQueryConfig::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const BigQueryConfig& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.BigQueryConfig)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string table = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_table().empty()) {
      const ::std::string& _s = this_._internal_table();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.BigQueryConfig.table");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // bool use_topic_schema = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_use_topic_schema() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          2, this_._internal_use_topic_schema(), target);
    }
  }

  // bool write_metadata = 3 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_write_metadata() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          3, this_._internal_write_metadata(), target);
    }
  }

  // bool drop_unknown_fields = 4 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_drop_unknown_fields() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          4, this_._internal_drop_unknown_fields(), target);
    }
  }

  // .google.pubsub.v1.BigQueryConfig.State state = 5 [(.google.api.field_behavior) = OUTPUT_ONLY];
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_state() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          5, this_._internal_state(), target);
    }
  }

  // bool use_table_schema = 6 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_use_table_schema() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          6, this_._internal_use_table_schema(), target);
    }
  }

  // string service_account_email = 7 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_service_account_email().empty()) {
      const ::std::string& _s = this_._internal_service_account_email();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.BigQueryConfig.service_account_email");
      target = stream->WriteStringMaybeAliased(7, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.BigQueryConfig)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t BigQueryConfig::ByteSizeLong(const MessageLite& base) {
  const BigQueryConfig& this_ = static_cast<const BigQueryConfig&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t BigQueryConfig::ByteSizeLong() const {
  const BigQueryConfig& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.BigQueryConfig)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    // string table = 1 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_table().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_table());
      }
    }
    // string service_account_email = 7 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_service_account_email().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_service_account_email());
      }
    }
    // bool use_topic_schema = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_use_topic_schema() != 0) {
        total_size += 2;
      }
    }
    // bool write_metadata = 3 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_write_metadata() != 0) {
        total_size += 2;
      }
    }
    // bool drop_unknown_fields = 4 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_drop_unknown_fields() != 0) {
        total_size += 2;
      }
    }
    // bool use_table_schema = 6 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_use_table_schema() != 0) {
        total_size += 2;
      }
    }
    // .google.pubsub.v1.BigQueryConfig.State state = 5 [(.google.api.field_behavior) = OUTPUT_ONLY];
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_state() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_state());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void BigQueryConfig::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<BigQueryConfig*>(&to_msg);
  auto& from = static_cast<const BigQueryConfig&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.BigQueryConfig)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_table().empty()) {
        _this->_internal_set_table(from._internal_table());
      } else {
        if (_this->_impl_.table_.IsDefault()) {
          _this->_internal_set_table("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_service_account_email().empty()) {
        _this->_internal_set_service_account_email(from._internal_service_account_email());
      } else {
        if (_this->_impl_.service_account_email_.IsDefault()) {
          _this->_internal_set_service_account_email("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_use_topic_schema() != 0) {
        _this->_impl_.use_topic_schema_ = from._impl_.use_topic_schema_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_write_metadata() != 0) {
        _this->_impl_.write_metadata_ = from._impl_.write_metadata_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_drop_unknown_fields() != 0) {
        _this->_impl_.drop_unknown_fields_ = from._impl_.drop_unknown_fields_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_use_table_schema() != 0) {
        _this->_impl_.use_table_schema_ = from._impl_.use_table_schema_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_state() != 0) {
        _this->_impl_.state_ = from._impl_.state_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void BigQueryConfig::CopyFrom(const BigQueryConfig& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.BigQueryConfig)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void BigQueryConfig::InternalSwap(BigQueryConfig* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.table_, &other->_impl_.table_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.service_account_email_, &other->_impl_.service_account_email_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(BigQueryConfig, _impl_.state_)
      + sizeof(BigQueryConfig::_impl_.state_)
      - PROTOBUF_FIELD_OFFSET(BigQueryConfig, _impl_.use_topic_schema_)>(
          reinterpret_cast<char*>(&_impl_.use_topic_schema_),
          reinterpret_cast<char*>(&other->_impl_.use_topic_schema_));
}

::google::protobuf::Metadata BigQueryConfig::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

CloudStorageConfig_TextConfig::CloudStorageConfig_TextConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, CloudStorageConfig_TextConfig_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.CloudStorageConfig.TextConfig)
}
CloudStorageConfig_TextConfig::CloudStorageConfig_TextConfig(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const CloudStorageConfig_TextConfig& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, CloudStorageConfig_TextConfig_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  CloudStorageConfig_TextConfig* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.CloudStorageConfig.TextConfig)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull CloudStorageConfig_TextConfig_class_data_ =
        CloudStorageConfig_TextConfig::InternalGenerateClassData_(CloudStorageConfig_TextConfig_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CloudStorageConfig_TextConfig::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CloudStorageConfig_TextConfig_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(CloudStorageConfig_TextConfig_class_data_.tc_table);
  return CloudStorageConfig_TextConfig_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CloudStorageConfig_TextConfig::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CloudStorageConfig_TextConfig_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&CloudStorageConfig_TextConfig_globals_));
  return CloudStorageConfig_TextConfig_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const CloudStorageConfig_TextConfig::ParseTableT_
    CloudStorageConfig_TextConfig::_table_ =
        CloudStorageConfig_TextConfig::InternalGenerateParseTable_(CloudStorageConfig_TextConfig_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata CloudStorageConfig_TextConfig::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

CloudStorageConfig_AvroConfig::CloudStorageConfig_AvroConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, CloudStorageConfig_AvroConfig_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.CloudStorageConfig.AvroConfig)
}
CloudStorageConfig_AvroConfig::CloudStorageConfig_AvroConfig(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const CloudStorageConfig_AvroConfig& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, CloudStorageConfig_AvroConfig_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE CloudStorageConfig_AvroConfig::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void CloudStorageConfig_AvroConfig::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, write_metadata_),
           0,
           offsetof(Impl_, use_topic_schema_) -
               offsetof(Impl_, write_metadata_) +
               sizeof(Impl_::use_topic_schema_));
}
CloudStorageConfig_AvroConfig::~CloudStorageConfig_AvroConfig() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.CloudStorageConfig.AvroConfig)
  SharedDtor(*this);
}
inline void CloudStorageConfig_AvroConfig::SharedDtor(MessageLite& self) {
  CloudStorageConfig_AvroConfig& this_ = static_cast<CloudStorageConfig_AvroConfig&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull CloudStorageConfig_AvroConfig_class_data_ =
        CloudStorageConfig_AvroConfig::InternalGenerateClassData_(CloudStorageConfig_AvroConfig_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CloudStorageConfig_AvroConfig::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CloudStorageConfig_AvroConfig_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(CloudStorageConfig_AvroConfig_class_data_.tc_table);
  return CloudStorageConfig_AvroConfig_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CloudStorageConfig_AvroConfig::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CloudStorageConfig_AvroConfig_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&CloudStorageConfig_AvroConfig_globals_));
  return CloudStorageConfig_AvroConfig_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const CloudStorageConfig_AvroConfig::ParseTableT_
    CloudStorageConfig_AvroConfig::_table_ =
        CloudStorageConfig_AvroConfig::InternalGenerateParseTable_(CloudStorageConfig_AvroConfig_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void CloudStorageConfig_AvroConfig::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.CloudStorageConfig.AvroConfig)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  ::memset(&_impl_.write_metadata_, 0, static_cast<::size_t>(
      reinterpret_cast<char*>(&_impl_.use_topic_schema_) -
      reinterpret_cast<char*>(&_impl_.write_metadata_)) + sizeof(_impl_.use_topic_schema_));
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL CloudStorageConfig_AvroConfig::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const CloudStorageConfig_AvroConfig& this_ = static_cast<const CloudStorageConfig_AvroConfig&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL CloudStorageConfig_AvroConfig::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const CloudStorageConfig_AvroConfig& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.CloudStorageConfig.AvroConfig)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // bool write_metadata = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_write_metadata() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          1, this_._internal_write_metadata(), target);
    }
  }

  // bool use_topic_schema = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_use_topic_schema() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          2, this_._internal_use_topic_schema(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.CloudStorageConfig.AvroConfig)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t CloudStorageConfig_AvroConfig::ByteSizeLong(const MessageLite& base) {
  const CloudStorageConfig_AvroConfig& this_ = static_cast<const CloudStorageConfig_AvroConfig&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t CloudStorageConfig_AvroConfig::ByteSizeLong() const {
  const CloudStorageConfig_AvroConfig& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.CloudStorageConfig.AvroConfig)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // bool write_metadata = 1 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_write_metadata() != 0) {
        total_size += 2;
      }
    }
    // bool use_topic_schema = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_use_topic_schema() != 0) {
        total_size += 2;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void CloudStorageConfig_AvroConfig::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<CloudStorageConfig_AvroConfig*>(&to_msg);
  auto& from = static_cast<const CloudStorageConfig_AvroConfig&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.CloudStorageConfig.AvroConfig)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (from._internal_write_metadata() != 0) {
        _this->_impl_.write_metadata_ = from._impl_.write_metadata_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_use_topic_schema() != 0) {
        _this->_impl_.use_topic_schema_ = from._impl_.use_topic_schema_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void CloudStorageConfig_AvroConfig::CopyFrom(const CloudStorageConfig_AvroConfig& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.CloudStorageConfig.AvroConfig)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void CloudStorageConfig_AvroConfig::InternalSwap(CloudStorageConfig_AvroConfig* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(CloudStorageConfig_AvroConfig, _impl_.use_topic_schema_)
      + sizeof(CloudStorageConfig_AvroConfig::_impl_.use_topic_schema_)
      - PROTOBUF_FIELD_OFFSET(CloudStorageConfig_AvroConfig, _impl_.write_metadata_)>(
          reinterpret_cast<char*>(&_impl_.write_metadata_),
          reinterpret_cast<char*>(&other->_impl_.write_metadata_));
}

::google::protobuf::Metadata CloudStorageConfig_AvroConfig::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void CloudStorageConfig::set_allocated_text_config(::google::pubsub::v1::CloudStorageConfig_TextConfig* PROTOBUF_NULLABLE text_config) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_output_format();
  if (text_config) {
    ::google::protobuf::Arena* submessage_arena = text_config->GetArena();
    if (message_arena != submessage_arena) {
      text_config = ::google::protobuf::internal::GetOwnedMessage(message_arena, text_config, submessage_arena);
    }
    set_has_text_config();
    _impl_.output_format_.text_config_ = text_config;
  }
  // @@protoc_insertion_point(field_set_allocated:google.pubsub.v1.CloudStorageConfig.text_config)
}
void CloudStorageConfig::set_allocated_avro_config(::google::pubsub::v1::CloudStorageConfig_AvroConfig* PROTOBUF_NULLABLE avro_config) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_output_format();
  if (avro_config) {
    ::google::protobuf::Arena* submessage_arena = avro_config->GetArena();
    if (message_arena != submessage_arena) {
      avro_config = ::google::protobuf::internal::GetOwnedMessage(message_arena, avro_config, submessage_arena);
    }
    set_has_avro_config();
    _impl_.output_format_.avro_config_ = avro_config;
  }
  // @@protoc_insertion_point(field_set_allocated:google.pubsub.v1.CloudStorageConfig.avro_config)
}
void CloudStorageConfig::clear_max_duration() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.max_duration_ != nullptr) _impl_.max_duration_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
CloudStorageConfig::CloudStorageConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, CloudStorageConfig_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.CloudStorageConfig)
}
PROTOBUF_NDEBUG_INLINE CloudStorageConfig::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::CloudStorageConfig& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        bucket_(arena, from.bucket_),
        filename_prefix_(arena, from.filename_prefix_),
        filename_suffix_(arena, from.filename_suffix_),
        filename_datetime_format_(arena, from.filename_datetime_format_),
        service_account_email_(arena, from.service_account_email_),
        output_format_{},
        _oneof_case_{from._oneof_case_[0]} {}

CloudStorageConfig::CloudStorageConfig(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const CloudStorageConfig& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, CloudStorageConfig_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  CloudStorageConfig* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.max_duration_ = (CheckHasBit(cached_has_bits, 0x00000020U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.max_duration_)
                : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, max_bytes_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, max_bytes_),
           offsetof(Impl_, state_) -
               offsetof(Impl_, max_bytes_) +
               sizeof(Impl_::state_));
  switch (output_format_case()) {
    case OUTPUT_FORMAT_NOT_SET:
      break;
      case kTextConfig:
        _impl_.output_format_.text_config_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.output_format_.text_config_);
        break;
      case kAvroConfig:
        _impl_.output_format_.avro_config_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.output_format_.avro_config_);
        break;
  }

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.CloudStorageConfig)
}
PROTOBUF_NDEBUG_INLINE CloudStorageConfig::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        bucket_(arena),
        filename_prefix_(arena),
        filename_suffix_(arena),
        filename_datetime_format_(arena),
        service_account_email_(arena),
        output_format_{},
        _oneof_case_{} {}

inline void CloudStorageConfig::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, max_duration_),
           0,
           offsetof(Impl_, state_) -
               offsetof(Impl_, max_duration_) +
               sizeof(Impl_::state_));
}
CloudStorageConfig::~CloudStorageConfig() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.CloudStorageConfig)
  SharedDtor(*this);
}
inline void CloudStorageConfig::SharedDtor(MessageLite& self) {
  CloudStorageConfig& this_ = static_cast<CloudStorageConfig&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.bucket_.Destroy();
  this_._impl_.filename_prefix_.Destroy();
  this_._impl_.filename_suffix_.Destroy();
  this_._impl_.filename_datetime_format_.Destroy();
  this_._impl_.service_account_email_.Destroy();
  delete this_._impl_.max_duration_;
  if (this_.has_output_format()) {
    this_.clear_output_format();
  }
  this_._impl_.~Impl_();
}

void CloudStorageConfig::clear_output_format() {
// @@protoc_insertion_point(one_of_clear_start:google.pubsub.v1.CloudStorageConfig)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (output_format_case()) {
    case kTextConfig: {
      if (GetArena() == nullptr) {
        delete _impl_.output_format_.text_config_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.output_format_.text_config_);
      }
      break;
    }
    case kAvroConfig: {
      if (GetArena() == nullptr) {
        delete _impl_.output_format_.avro_config_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.output_format_.avro_config_);
      }
      break;
    }
    case OUTPUT_FORMAT_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = OUTPUT_FORMAT_NOT_SET;
}


#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull CloudStorageConfig_class_data_ =
        CloudStorageConfig::InternalGenerateClassData_(CloudStorageConfig_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CloudStorageConfig::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CloudStorageConfig_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(CloudStorageConfig_class_data_.tc_table);
  return CloudStorageConfig_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CloudStorageConfig::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CloudStorageConfig_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&CloudStorageConfig_globals_));
  return CloudStorageConfig_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const CloudStorageConfig::ParseTableT_
    CloudStorageConfig::_table_ =
        CloudStorageConfig::InternalGenerateParseTable_(CloudStorageConfig_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void CloudStorageConfig::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.CloudStorageConfig)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.bucket_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.filename_prefix_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.filename_suffix_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _impl_.filename_datetime_format_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      _impl_.service_account_email_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      ABSL_DCHECK(_impl_.max_duration_ != nullptr);
      _impl_.max_duration_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x000000c0U)) {
    ::memset(&_impl_.max_bytes_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.max_messages_) -
        reinterpret_cast<char*>(&_impl_.max_bytes_)) + sizeof(_impl_.max_messages_));
  }
  _impl_.state_ = 0;
  clear_output_format();
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL CloudStorageConfig::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const CloudStorageConfig& this_ = static_cast<const CloudStorageConfig&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL CloudStorageConfig::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const CloudStorageConfig& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.CloudStorageConfig)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string bucket = 1 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_bucket().empty()) {
      const ::std::string& _s = this_._internal_bucket();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.CloudStorageConfig.bucket");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // string filename_prefix = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_filename_prefix().empty()) {
      const ::std::string& _s = this_._internal_filename_prefix();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.CloudStorageConfig.filename_prefix");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  // string filename_suffix = 3 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_filename_suffix().empty()) {
      const ::std::string& _s = this_._internal_filename_suffix();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.CloudStorageConfig.filename_suffix");
      target = stream->WriteStringMaybeAliased(3, _s, target);
    }
  }

  switch (this_.output_format_case()) {
    case kTextConfig: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          4, *this_._impl_.output_format_.text_config_, this_._impl_.output_format_.text_config_->GetCachedSize(), target,
          stream);
      break;
    }
    case kAvroConfig: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          5, *this_._impl_.output_format_.avro_config_, this_._impl_.output_format_.avro_config_->GetCachedSize(), target,
          stream);
      break;
    }
    default:
      break;
  }
  // .google.protobuf.Duration max_duration = 6 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        6, *this_._impl_.max_duration_, this_._impl_.max_duration_->GetCachedSize(), target,
        stream);
  }

  // int64 max_bytes = 7 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_max_bytes() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<7>(
              stream, this_._internal_max_bytes(), target);
    }
  }

  // int64 max_messages = 8 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    if (this_._internal_max_messages() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<8>(
              stream, this_._internal_max_messages(), target);
    }
  }

  // .google.pubsub.v1.CloudStorageConfig.State state = 9 [(.google.api.field_behavior) = OUTPUT_ONLY];
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    if (this_._internal_state() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          9, this_._internal_state(), target);
    }
  }

  // string filename_datetime_format = 10 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (!this_._internal_filename_datetime_format().empty()) {
      const ::std::string& _s = this_._internal_filename_datetime_format();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.CloudStorageConfig.filename_datetime_format");
      target = stream->WriteStringMaybeAliased(10, _s, target);
    }
  }

  // string service_account_email = 11 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (!this_._internal_service_account_email().empty()) {
      const ::std::string& _s = this_._internal_service_account_email();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.CloudStorageConfig.service_account_email");
      target = stream->WriteStringMaybeAliased(11, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.CloudStorageConfig)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t CloudStorageConfig::ByteSizeLong(const MessageLite& base) {
  const CloudStorageConfig& this_ = static_cast<const CloudStorageConfig&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t CloudStorageConfig::ByteSizeLong() const {
  const CloudStorageConfig& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.CloudStorageConfig)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // string bucket = 1 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_bucket().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_bucket());
      }
    }
    // string filename_prefix = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_filename_prefix().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_filename_prefix());
      }
    }
    // string filename_suffix = 3 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!this_._internal_filename_suffix().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_filename_suffix());
      }
    }
    // string filename_datetime_format = 10 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!this_._internal_filename_datetime_format().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_filename_datetime_format());
      }
    }
    // string service_account_email = 11 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (!this_._internal_service_account_email().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_service_account_email());
      }
    }
    // .google.protobuf.Duration max_duration = 6 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.max_duration_);
    }
    // int64 max_bytes = 7 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_max_bytes() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_max_bytes());
      }
    }
    // int64 max_messages = 8 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (this_._internal_max_messages() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_max_messages());
      }
    }
  }
   {
    // .google.pubsub.v1.CloudStorageConfig.State state = 9 [(.google.api.field_behavior) = OUTPUT_ONLY];
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      if (this_._internal_state() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_state());
      }
    }
  }
  switch (this_.output_format_case()) {
    // .google.pubsub.v1.CloudStorageConfig.TextConfig text_config = 4 [(.google.api.field_behavior) = OPTIONAL];
    case kTextConfig: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.output_format_.text_config_);
      break;
    }
    // .google.pubsub.v1.CloudStorageConfig.AvroConfig avro_config = 5 [(.google.api.field_behavior) = OPTIONAL];
    case kAvroConfig: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.output_format_.avro_config_);
      break;
    }
    case OUTPUT_FORMAT_NOT_SET: {
      break;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void CloudStorageConfig::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<CloudStorageConfig*>(&to_msg);
  auto& from = static_cast<const CloudStorageConfig&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.CloudStorageConfig)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_bucket().empty()) {
        _this->_internal_set_bucket(from._internal_bucket());
      } else {
        if (_this->_impl_.bucket_.IsDefault()) {
          _this->_internal_set_bucket("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_filename_prefix().empty()) {
        _this->_internal_set_filename_prefix(from._internal_filename_prefix());
      } else {
        if (_this->_impl_.filename_prefix_.IsDefault()) {
          _this->_internal_set_filename_prefix("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!from._internal_filename_suffix().empty()) {
        _this->_internal_set_filename_suffix(from._internal_filename_suffix());
      } else {
        if (_this->_impl_.filename_suffix_.IsDefault()) {
          _this->_internal_set_filename_suffix("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!from._internal_filename_datetime_format().empty()) {
        _this->_internal_set_filename_datetime_format(from._internal_filename_datetime_format());
      } else {
        if (_this->_impl_.filename_datetime_format_.IsDefault()) {
          _this->_internal_set_filename_datetime_format("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (!from._internal_service_account_email().empty()) {
        _this->_internal_set_service_account_email(from._internal_service_account_email());
      } else {
        if (_this->_impl_.service_account_email_.IsDefault()) {
          _this->_internal_set_service_account_email("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      ABSL_DCHECK(from._impl_.max_duration_ != nullptr);
      if (_this->_impl_.max_duration_ == nullptr) {
        _this->_impl_.max_duration_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.max_duration_);
      } else {
        _this->_impl_.max_duration_->MergeFrom(*from._impl_.max_duration_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_max_bytes() != 0) {
        _this->_impl_.max_bytes_ = from._impl_.max_bytes_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (from._internal_max_messages() != 0) {
        _this->_impl_.max_messages_ = from._impl_.max_messages_;
      }
    }
  }
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    if (from._internal_state() != 0) {
      _this->_impl_.state_ = from._impl_.state_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[0]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[0];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_output_format();
      }
      _this->_impl_._oneof_case_[0] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kTextConfig: {
        if (oneof_needs_init) {
          _this->_impl_.output_format_.text_config_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.output_format_.text_config_);
        } else {
          _this->_impl_.output_format_.text_config_->MergeFrom(*from._impl_.output_format_.text_config_);
        }
        break;
      }
      case kAvroConfig: {
        if (oneof_needs_init) {
          _this->_impl_.output_format_.avro_config_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.output_format_.avro_config_);
        } else {
          _this->_impl_.output_format_.avro_config_->MergeFrom(*from._impl_.output_format_.avro_config_);
        }
        break;
      }
      case OUTPUT_FORMAT_NOT_SET:
        break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void CloudStorageConfig::CopyFrom(const CloudStorageConfig& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.CloudStorageConfig)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void CloudStorageConfig::InternalSwap(CloudStorageConfig* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.bucket_, &other->_impl_.bucket_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.filename_prefix_, &other->_impl_.filename_prefix_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.filename_suffix_, &other->_impl_.filename_suffix_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.filename_datetime_format_, &other->_impl_.filename_datetime_format_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.service_account_email_, &other->_impl_.service_account_email_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.state_)
      + sizeof(CloudStorageConfig::_impl_.state_)
      - PROTOBUF_FIELD_OFFSET(CloudStorageConfig, _impl_.max_duration_)>(
          reinterpret_cast<char*>(&_impl_.max_duration_),
          reinterpret_cast<char*>(&other->_impl_.max_duration_));
  swap(_impl_.output_format_, other->_impl_.output_format_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
}

::google::protobuf::Metadata CloudStorageConfig::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ReceivedMessage::ReceivedMessage(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ReceivedMessage_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ReceivedMessage)
}
PROTOBUF_NDEBUG_INLINE ReceivedMessage::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::ReceivedMessage& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        ack_id_(arena, from.ack_id_) {}

ReceivedMessage::ReceivedMessage(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ReceivedMessage& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ReceivedMessage_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ReceivedMessage* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.message_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.message_)
                : nullptr;
  _impl_.delivery_attempt_ = from._impl_.delivery_attempt_;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ReceivedMessage)
}
PROTOBUF_NDEBUG_INLINE ReceivedMessage::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        ack_id_(arena) {}

inline void ReceivedMessage::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, message_),
           0,
           offsetof(Impl_, delivery_attempt_) -
               offsetof(Impl_, message_) +
               sizeof(Impl_::delivery_attempt_));
}
ReceivedMessage::~ReceivedMessage() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.ReceivedMessage)
  SharedDtor(*this);
}
inline void ReceivedMessage::SharedDtor(MessageLite& self) {
  ReceivedMessage& this_ = static_cast<ReceivedMessage&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.ack_id_.Destroy();
  delete this_._impl_.message_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ReceivedMessage_class_data_ =
        ReceivedMessage::InternalGenerateClassData_(ReceivedMessage_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ReceivedMessage::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ReceivedMessage_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ReceivedMessage_class_data_.tc_table);
  return ReceivedMessage_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ReceivedMessage::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ReceivedMessage_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ReceivedMessage_globals_));
  return ReceivedMessage_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ReceivedMessage::ParseTableT_
    ReceivedMessage::_table_ =
        ReceivedMessage::InternalGenerateParseTable_(ReceivedMessage_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ReceivedMessage::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.ReceivedMessage)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.ack_id_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.message_ != nullptr);
      _impl_.message_->Clear();
    }
  }
  _impl_.delivery_attempt_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ReceivedMessage::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ReceivedMessage& this_ = static_cast<const ReceivedMessage&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ReceivedMessage::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ReceivedMessage& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.ReceivedMessage)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string ack_id = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_ack_id().empty()) {
      const ::std::string& _s = this_._internal_ack_id();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ReceivedMessage.ack_id");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // .google.pubsub.v1.PubsubMessage message = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.message_, this_._impl_.message_->GetCachedSize(), target,
        stream);
  }

  // int32 delivery_attempt = 3 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_delivery_attempt() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<3>(
              stream, this_._internal_delivery_attempt(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.ReceivedMessage)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ReceivedMessage::ByteSizeLong(const MessageLite& base) {
  const ReceivedMessage& this_ = static_cast<const ReceivedMessage&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ReceivedMessage::ByteSizeLong() const {
  const ReceivedMessage& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.ReceivedMessage)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // string ack_id = 1 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_ack_id().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_ack_id());
      }
    }
    // .google.pubsub.v1.PubsubMessage message = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.message_);
    }
    // int32 delivery_attempt = 3 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_delivery_attempt() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_delivery_attempt());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ReceivedMessage::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ReceivedMessage*>(&to_msg);
  auto& from = static_cast<const ReceivedMessage&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.ReceivedMessage)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_ack_id().empty()) {
        _this->_internal_set_ack_id(from._internal_ack_id());
      } else {
        if (_this->_impl_.ack_id_.IsDefault()) {
          _this->_internal_set_ack_id("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.message_ != nullptr);
      if (_this->_impl_.message_ == nullptr) {
        _this->_impl_.message_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.message_);
      } else {
        _this->_impl_.message_->MergeFrom(*from._impl_.message_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_delivery_attempt() != 0) {
        _this->_impl_.delivery_attempt_ = from._impl_.delivery_attempt_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ReceivedMessage::CopyFrom(const ReceivedMessage& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.ReceivedMessage)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ReceivedMessage::InternalSwap(ReceivedMessage* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.ack_id_, &other->_impl_.ack_id_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(ReceivedMessage, _impl_.delivery_attempt_)
      + sizeof(ReceivedMessage::_impl_.delivery_attempt_)
      - PROTOBUF_FIELD_OFFSET(ReceivedMessage, _impl_.message_)>(
          reinterpret_cast<char*>(&_impl_.message_),
          reinterpret_cast<char*>(&other->_impl_.message_));
}

::google::protobuf::Metadata ReceivedMessage::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

GetSubscriptionRequest::GetSubscriptionRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, GetSubscriptionRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.GetSubscriptionRequest)
}
PROTOBUF_NDEBUG_INLINE GetSubscriptionRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::GetSubscriptionRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        subscription_(arena, from.subscription_) {}

GetSubscriptionRequest::GetSubscriptionRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const GetSubscriptionRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, GetSubscriptionRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  GetSubscriptionRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.GetSubscriptionRequest)
}
PROTOBUF_NDEBUG_INLINE GetSubscriptionRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        subscription_(arena) {}

inline void GetSubscriptionRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
GetSubscriptionRequest::~GetSubscriptionRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.GetSubscriptionRequest)
  SharedDtor(*this);
}
inline void GetSubscriptionRequest::SharedDtor(MessageLite& self) {
  GetSubscriptionRequest& this_ = static_cast<GetSubscriptionRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.subscription_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull GetSubscriptionRequest_class_data_ =
        GetSubscriptionRequest::InternalGenerateClassData_(GetSubscriptionRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
GetSubscriptionRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&GetSubscriptionRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(GetSubscriptionRequest_class_data_.tc_table);
  return GetSubscriptionRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
GetSubscriptionRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&GetSubscriptionRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&GetSubscriptionRequest_globals_));
  return GetSubscriptionRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const GetSubscriptionRequest::ParseTableT_
    GetSubscriptionRequest::_table_ =
        GetSubscriptionRequest::InternalGenerateParseTable_(GetSubscriptionRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void GetSubscriptionRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.GetSubscriptionRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.subscription_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL GetSubscriptionRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const GetSubscriptionRequest& this_ = static_cast<const GetSubscriptionRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL GetSubscriptionRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const GetSubscriptionRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.GetSubscriptionRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_subscription().empty()) {
      const ::std::string& _s = this_._internal_subscription();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.GetSubscriptionRequest.subscription");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.GetSubscriptionRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t GetSubscriptionRequest::ByteSizeLong(const MessageLite& base) {
  const GetSubscriptionRequest& this_ = static_cast<const GetSubscriptionRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t GetSubscriptionRequest::ByteSizeLong() const {
  const GetSubscriptionRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.GetSubscriptionRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_subscription().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_subscription());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void GetSubscriptionRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<GetSubscriptionRequest*>(&to_msg);
  auto& from = static_cast<const GetSubscriptionRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.GetSubscriptionRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_subscription().empty()) {
      _this->_internal_set_subscription(from._internal_subscription());
    } else {
      if (_this->_impl_.subscription_.IsDefault()) {
        _this->_internal_set_subscription("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void GetSubscriptionRequest::CopyFrom(const GetSubscriptionRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.GetSubscriptionRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void GetSubscriptionRequest::InternalSwap(GetSubscriptionRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.subscription_, &other->_impl_.subscription_, arena);
}

::google::protobuf::Metadata GetSubscriptionRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void UpdateSubscriptionRequest::clear_update_mask() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.update_mask_ != nullptr) _impl_.update_mask_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
UpdateSubscriptionRequest::UpdateSubscriptionRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UpdateSubscriptionRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.UpdateSubscriptionRequest)
}
PROTOBUF_NDEBUG_INLINE UpdateSubscriptionRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::UpdateSubscriptionRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

UpdateSubscriptionRequest::UpdateSubscriptionRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const UpdateSubscriptionRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UpdateSubscriptionRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  UpdateSubscriptionRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.subscription_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.subscription_)
                : nullptr;
  _impl_.update_mask_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.update_mask_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.UpdateSubscriptionRequest)
}
PROTOBUF_NDEBUG_INLINE UpdateSubscriptionRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void UpdateSubscriptionRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, subscription_),
           0,
           offsetof(Impl_, update_mask_) -
               offsetof(Impl_, subscription_) +
               sizeof(Impl_::update_mask_));
}
UpdateSubscriptionRequest::~UpdateSubscriptionRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.UpdateSubscriptionRequest)
  SharedDtor(*this);
}
inline void UpdateSubscriptionRequest::SharedDtor(MessageLite& self) {
  UpdateSubscriptionRequest& this_ = static_cast<UpdateSubscriptionRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.subscription_;
  delete this_._impl_.update_mask_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull UpdateSubscriptionRequest_class_data_ =
        UpdateSubscriptionRequest::InternalGenerateClassData_(UpdateSubscriptionRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateSubscriptionRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateSubscriptionRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(UpdateSubscriptionRequest_class_data_.tc_table);
  return UpdateSubscriptionRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateSubscriptionRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateSubscriptionRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&UpdateSubscriptionRequest_globals_));
  return UpdateSubscriptionRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const UpdateSubscriptionRequest::ParseTableT_
    UpdateSubscriptionRequest::_table_ =
        UpdateSubscriptionRequest::InternalGenerateParseTable_(UpdateSubscriptionRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void UpdateSubscriptionRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.UpdateSubscriptionRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(_impl_.subscription_ != nullptr);
      _impl_.subscription_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.update_mask_ != nullptr);
      _impl_.update_mask_->Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL UpdateSubscriptionRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const UpdateSubscriptionRequest& this_ = static_cast<const UpdateSubscriptionRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL UpdateSubscriptionRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const UpdateSubscriptionRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.UpdateSubscriptionRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.pubsub.v1.Subscription subscription = 1 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.subscription_, this_._impl_.subscription_->GetCachedSize(), target,
        stream);
  }

  // .google.protobuf.FieldMask update_mask = 2 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.update_mask_, this_._impl_.update_mask_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.UpdateSubscriptionRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t UpdateSubscriptionRequest::ByteSizeLong(const MessageLite& base) {
  const UpdateSubscriptionRequest& this_ = static_cast<const UpdateSubscriptionRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t UpdateSubscriptionRequest::ByteSizeLong() const {
  const UpdateSubscriptionRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.UpdateSubscriptionRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // .google.pubsub.v1.Subscription subscription = 1 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.subscription_);
    }
    // .google.protobuf.FieldMask update_mask = 2 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.update_mask_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void UpdateSubscriptionRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<UpdateSubscriptionRequest*>(&to_msg);
  auto& from = static_cast<const UpdateSubscriptionRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.UpdateSubscriptionRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.subscription_ != nullptr);
      if (_this->_impl_.subscription_ == nullptr) {
        _this->_impl_.subscription_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.subscription_);
      } else {
        _this->_impl_.subscription_->MergeFrom(*from._impl_.subscription_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.update_mask_ != nullptr);
      if (_this->_impl_.update_mask_ == nullptr) {
        _this->_impl_.update_mask_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.update_mask_);
      } else {
        _this->_impl_.update_mask_->MergeFrom(*from._impl_.update_mask_);
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void UpdateSubscriptionRequest::CopyFrom(const UpdateSubscriptionRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.UpdateSubscriptionRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void UpdateSubscriptionRequest::InternalSwap(UpdateSubscriptionRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(UpdateSubscriptionRequest, _impl_.update_mask_)
      + sizeof(UpdateSubscriptionRequest::_impl_.update_mask_)
      - PROTOBUF_FIELD_OFFSET(UpdateSubscriptionRequest, _impl_.subscription_)>(
          reinterpret_cast<char*>(&_impl_.subscription_),
          reinterpret_cast<char*>(&other->_impl_.subscription_));
}

::google::protobuf::Metadata UpdateSubscriptionRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ListSubscriptionsRequest::ListSubscriptionsRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListSubscriptionsRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ListSubscriptionsRequest)
}
PROTOBUF_NDEBUG_INLINE ListSubscriptionsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::ListSubscriptionsRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        project_(arena, from.project_),
        page_token_(arena, from.page_token_) {}

ListSubscriptionsRequest::ListSubscriptionsRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ListSubscriptionsRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListSubscriptionsRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ListSubscriptionsRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.page_size_ = from._impl_.page_size_;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ListSubscriptionsRequest)
}
PROTOBUF_NDEBUG_INLINE ListSubscriptionsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        project_(arena),
        page_token_(arena) {}

inline void ListSubscriptionsRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.page_size_ = {};
}
ListSubscriptionsRequest::~ListSubscriptionsRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.ListSubscriptionsRequest)
  SharedDtor(*this);
}
inline void ListSubscriptionsRequest::SharedDtor(MessageLite& self) {
  ListSubscriptionsRequest& this_ = static_cast<ListSubscriptionsRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.project_.Destroy();
  this_._impl_.page_token_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ListSubscriptionsRequest_class_data_ =
        ListSubscriptionsRequest::InternalGenerateClassData_(ListSubscriptionsRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListSubscriptionsRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListSubscriptionsRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ListSubscriptionsRequest_class_data_.tc_table);
  return ListSubscriptionsRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListSubscriptionsRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListSubscriptionsRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ListSubscriptionsRequest_globals_));
  return ListSubscriptionsRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ListSubscriptionsRequest::ParseTableT_
    ListSubscriptionsRequest::_table_ =
        ListSubscriptionsRequest::InternalGenerateParseTable_(ListSubscriptionsRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ListSubscriptionsRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.ListSubscriptionsRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.project_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.page_token_.ClearNonDefaultToEmpty();
    }
  }
  _impl_.page_size_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ListSubscriptionsRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ListSubscriptionsRequest& this_ = static_cast<const ListSubscriptionsRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ListSubscriptionsRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ListSubscriptionsRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.ListSubscriptionsRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string project = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_project().empty()) {
      const ::std::string& _s = this_._internal_project();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListSubscriptionsRequest.project");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // int32 page_size = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_page_size() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<2>(
              stream, this_._internal_page_size(), target);
    }
  }

  // string page_token = 3 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_page_token().empty()) {
      const ::std::string& _s = this_._internal_page_token();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListSubscriptionsRequest.page_token");
      target = stream->WriteStringMaybeAliased(3, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.ListSubscriptionsRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ListSubscriptionsRequest::ByteSizeLong(const MessageLite& base) {
  const ListSubscriptionsRequest& this_ = static_cast<const ListSubscriptionsRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ListSubscriptionsRequest::ByteSizeLong() const {
  const ListSubscriptionsRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.ListSubscriptionsRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // string project = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_project().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_project());
      }
    }
    // string page_token = 3 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_page_token().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_page_token());
      }
    }
    // int32 page_size = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_page_size() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_page_size());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ListSubscriptionsRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ListSubscriptionsRequest*>(&to_msg);
  auto& from = static_cast<const ListSubscriptionsRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.ListSubscriptionsRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_project().empty()) {
        _this->_internal_set_project(from._internal_project());
      } else {
        if (_this->_impl_.project_.IsDefault()) {
          _this->_internal_set_project("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_page_token().empty()) {
        _this->_internal_set_page_token(from._internal_page_token());
      } else {
        if (_this->_impl_.page_token_.IsDefault()) {
          _this->_internal_set_page_token("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_page_size() != 0) {
        _this->_impl_.page_size_ = from._impl_.page_size_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ListSubscriptionsRequest::CopyFrom(const ListSubscriptionsRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.ListSubscriptionsRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ListSubscriptionsRequest::InternalSwap(ListSubscriptionsRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.project_, &other->_impl_.project_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.page_token_, &other->_impl_.page_token_, arena);
  swap(_impl_.page_size_, other->_impl_.page_size_);
}

::google::protobuf::Metadata ListSubscriptionsRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ListSubscriptionsResponse::ListSubscriptionsResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListSubscriptionsResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ListSubscriptionsResponse)
}
PROTOBUF_NDEBUG_INLINE ListSubscriptionsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::ListSubscriptionsResponse& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        subscriptions_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::ListSubscriptionsResponse,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSubscriptionsResponse, _impl_.subscriptions_)>()
          , from.subscriptions_
        }
        ,
        next_page_token_(arena, from.next_page_token_) {}

ListSubscriptionsResponse::ListSubscriptionsResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ListSubscriptionsResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListSubscriptionsResponse_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ListSubscriptionsResponse* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ListSubscriptionsResponse)
}
PROTOBUF_NDEBUG_INLINE ListSubscriptionsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        subscriptions_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::ListSubscriptionsResponse,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSubscriptionsResponse, _impl_.subscriptions_)>()
         }
        ,
        next_page_token_(arena) {}

inline void ListSubscriptionsResponse::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
ListSubscriptionsResponse::~ListSubscriptionsResponse() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.ListSubscriptionsResponse)
  SharedDtor(*this);
}
inline void ListSubscriptionsResponse::SharedDtor(MessageLite& self) {
  ListSubscriptionsResponse& this_ = static_cast<ListSubscriptionsResponse&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.next_page_token_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ListSubscriptionsResponse_class_data_ =
        ListSubscriptionsResponse::InternalGenerateClassData_(ListSubscriptionsResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListSubscriptionsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListSubscriptionsResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ListSubscriptionsResponse_class_data_.tc_table);
  return ListSubscriptionsResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListSubscriptionsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListSubscriptionsResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ListSubscriptionsResponse_globals_));
  return ListSubscriptionsResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ListSubscriptionsResponse::ParseTableT_
    ListSubscriptionsResponse::_table_ =
        ListSubscriptionsResponse::InternalGenerateParseTable_(ListSubscriptionsResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ListSubscriptionsResponse::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.ListSubscriptionsResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.subscriptions_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.next_page_token_.ClearNonDefaultToEmpty();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ListSubscriptionsResponse::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ListSubscriptionsResponse& this_ = static_cast<const ListSubscriptionsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ListSubscriptionsResponse::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ListSubscriptionsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.ListSubscriptionsResponse)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated .google.pubsub.v1.Subscription subscriptions = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_subscriptions_size());
         i < n; i++) {
      const auto& repfield = this_._internal_subscriptions().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              1, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  // string next_page_token = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_next_page_token().empty()) {
      const ::std::string& _s = this_._internal_next_page_token();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListSubscriptionsResponse.next_page_token");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.ListSubscriptionsResponse)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ListSubscriptionsResponse::ByteSizeLong(const MessageLite& base) {
  const ListSubscriptionsResponse& this_ = static_cast<const ListSubscriptionsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ListSubscriptionsResponse::ByteSizeLong() const {
  const ListSubscriptionsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.ListSubscriptionsResponse)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // repeated .google.pubsub.v1.Subscription subscriptions = 1 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_subscriptions_size();
      for (const auto& msg : this_._internal_subscriptions()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
    // string next_page_token = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_next_page_token().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_next_page_token());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ListSubscriptionsResponse::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ListSubscriptionsResponse*>(&to_msg);
  auto& from = static_cast<const ListSubscriptionsResponse&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.ListSubscriptionsResponse)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_subscriptions()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_subscriptions());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_next_page_token().empty()) {
        _this->_internal_set_next_page_token(from._internal_next_page_token());
      } else {
        if (_this->_impl_.next_page_token_.IsDefault()) {
          _this->_internal_set_next_page_token("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ListSubscriptionsResponse::CopyFrom(const ListSubscriptionsResponse& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.ListSubscriptionsResponse)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ListSubscriptionsResponse::InternalSwap(ListSubscriptionsResponse* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.subscriptions_.InternalSwap(&other->_impl_.subscriptions_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.next_page_token_, &other->_impl_.next_page_token_, arena);
}

::google::protobuf::Metadata ListSubscriptionsResponse::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

DeleteSubscriptionRequest::DeleteSubscriptionRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DeleteSubscriptionRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.DeleteSubscriptionRequest)
}
PROTOBUF_NDEBUG_INLINE DeleteSubscriptionRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::DeleteSubscriptionRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        subscription_(arena, from.subscription_) {}

DeleteSubscriptionRequest::DeleteSubscriptionRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const DeleteSubscriptionRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DeleteSubscriptionRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  DeleteSubscriptionRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.DeleteSubscriptionRequest)
}
PROTOBUF_NDEBUG_INLINE DeleteSubscriptionRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        subscription_(arena) {}

inline void DeleteSubscriptionRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
DeleteSubscriptionRequest::~DeleteSubscriptionRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.DeleteSubscriptionRequest)
  SharedDtor(*this);
}
inline void DeleteSubscriptionRequest::SharedDtor(MessageLite& self) {
  DeleteSubscriptionRequest& this_ = static_cast<DeleteSubscriptionRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.subscription_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull DeleteSubscriptionRequest_class_data_ =
        DeleteSubscriptionRequest::InternalGenerateClassData_(DeleteSubscriptionRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DeleteSubscriptionRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DeleteSubscriptionRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(DeleteSubscriptionRequest_class_data_.tc_table);
  return DeleteSubscriptionRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DeleteSubscriptionRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DeleteSubscriptionRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&DeleteSubscriptionRequest_globals_));
  return DeleteSubscriptionRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const DeleteSubscriptionRequest::ParseTableT_
    DeleteSubscriptionRequest::_table_ =
        DeleteSubscriptionRequest::InternalGenerateParseTable_(DeleteSubscriptionRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void DeleteSubscriptionRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.DeleteSubscriptionRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.subscription_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL DeleteSubscriptionRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const DeleteSubscriptionRequest& this_ = static_cast<const DeleteSubscriptionRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL DeleteSubscriptionRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const DeleteSubscriptionRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.DeleteSubscriptionRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_subscription().empty()) {
      const ::std::string& _s = this_._internal_subscription();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.DeleteSubscriptionRequest.subscription");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.DeleteSubscriptionRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t DeleteSubscriptionRequest::ByteSizeLong(const MessageLite& base) {
  const DeleteSubscriptionRequest& this_ = static_cast<const DeleteSubscriptionRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t DeleteSubscriptionRequest::ByteSizeLong() const {
  const DeleteSubscriptionRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.DeleteSubscriptionRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_subscription().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_subscription());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void DeleteSubscriptionRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<DeleteSubscriptionRequest*>(&to_msg);
  auto& from = static_cast<const DeleteSubscriptionRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.DeleteSubscriptionRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_subscription().empty()) {
      _this->_internal_set_subscription(from._internal_subscription());
    } else {
      if (_this->_impl_.subscription_.IsDefault()) {
        _this->_internal_set_subscription("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void DeleteSubscriptionRequest::CopyFrom(const DeleteSubscriptionRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.DeleteSubscriptionRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void DeleteSubscriptionRequest::InternalSwap(DeleteSubscriptionRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.subscription_, &other->_impl_.subscription_, arena);
}

::google::protobuf::Metadata DeleteSubscriptionRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ModifyPushConfigRequest::ModifyPushConfigRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ModifyPushConfigRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ModifyPushConfigRequest)
}
PROTOBUF_NDEBUG_INLINE ModifyPushConfigRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::ModifyPushConfigRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        subscription_(arena, from.subscription_) {}

ModifyPushConfigRequest::ModifyPushConfigRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ModifyPushConfigRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ModifyPushConfigRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ModifyPushConfigRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.push_config_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.push_config_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ModifyPushConfigRequest)
}
PROTOBUF_NDEBUG_INLINE ModifyPushConfigRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        subscription_(arena) {}

inline void ModifyPushConfigRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.push_config_ = {};
}
ModifyPushConfigRequest::~ModifyPushConfigRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.ModifyPushConfigRequest)
  SharedDtor(*this);
}
inline void ModifyPushConfigRequest::SharedDtor(MessageLite& self) {
  ModifyPushConfigRequest& this_ = static_cast<ModifyPushConfigRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.subscription_.Destroy();
  delete this_._impl_.push_config_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ModifyPushConfigRequest_class_data_ =
        ModifyPushConfigRequest::InternalGenerateClassData_(ModifyPushConfigRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ModifyPushConfigRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ModifyPushConfigRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ModifyPushConfigRequest_class_data_.tc_table);
  return ModifyPushConfigRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ModifyPushConfigRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ModifyPushConfigRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ModifyPushConfigRequest_globals_));
  return ModifyPushConfigRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ModifyPushConfigRequest::ParseTableT_
    ModifyPushConfigRequest::_table_ =
        ModifyPushConfigRequest::InternalGenerateParseTable_(ModifyPushConfigRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ModifyPushConfigRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.ModifyPushConfigRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.subscription_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.push_config_ != nullptr);
      _impl_.push_config_->Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ModifyPushConfigRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ModifyPushConfigRequest& this_ = static_cast<const ModifyPushConfigRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ModifyPushConfigRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ModifyPushConfigRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.ModifyPushConfigRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_subscription().empty()) {
      const ::std::string& _s = this_._internal_subscription();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ModifyPushConfigRequest.subscription");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // .google.pubsub.v1.PushConfig push_config = 2 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.push_config_, this_._impl_.push_config_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.ModifyPushConfigRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ModifyPushConfigRequest::ByteSizeLong(const MessageLite& base) {
  const ModifyPushConfigRequest& this_ = static_cast<const ModifyPushConfigRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ModifyPushConfigRequest::ByteSizeLong() const {
  const ModifyPushConfigRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.ModifyPushConfigRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_subscription().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_subscription());
      }
    }
    // .google.pubsub.v1.PushConfig push_config = 2 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.push_config_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ModifyPushConfigRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ModifyPushConfigRequest*>(&to_msg);
  auto& from = static_cast<const ModifyPushConfigRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.ModifyPushConfigRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_subscription().empty()) {
        _this->_internal_set_subscription(from._internal_subscription());
      } else {
        if (_this->_impl_.subscription_.IsDefault()) {
          _this->_internal_set_subscription("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.push_config_ != nullptr);
      if (_this->_impl_.push_config_ == nullptr) {
        _this->_impl_.push_config_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.push_config_);
      } else {
        _this->_impl_.push_config_->MergeFrom(*from._impl_.push_config_);
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ModifyPushConfigRequest::CopyFrom(const ModifyPushConfigRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.ModifyPushConfigRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ModifyPushConfigRequest::InternalSwap(ModifyPushConfigRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.subscription_, &other->_impl_.subscription_, arena);
  swap(_impl_.push_config_, other->_impl_.push_config_);
}

::google::protobuf::Metadata ModifyPushConfigRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

PullRequest::PullRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PullRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.PullRequest)
}
PROTOBUF_NDEBUG_INLINE PullRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::PullRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        subscription_(arena, from.subscription_) {}

PullRequest::PullRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const PullRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PullRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  PullRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, return_immediately_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, return_immediately_),
           offsetof(Impl_, max_messages_) -
               offsetof(Impl_, return_immediately_) +
               sizeof(Impl_::max_messages_));

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.PullRequest)
}
PROTOBUF_NDEBUG_INLINE PullRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        subscription_(arena) {}

inline void PullRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, return_immediately_),
           0,
           offsetof(Impl_, max_messages_) -
               offsetof(Impl_, return_immediately_) +
               sizeof(Impl_::max_messages_));
}
PullRequest::~PullRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.PullRequest)
  SharedDtor(*this);
}
inline void PullRequest::SharedDtor(MessageLite& self) {
  PullRequest& this_ = static_cast<PullRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.subscription_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull PullRequest_class_data_ =
        PullRequest::InternalGenerateClassData_(PullRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PullRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PullRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(PullRequest_class_data_.tc_table);
  return PullRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PullRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PullRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&PullRequest_globals_));
  return PullRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const PullRequest::ParseTableT_
    PullRequest::_table_ =
        PullRequest::InternalGenerateParseTable_(PullRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void PullRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.PullRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.subscription_.ClearNonDefaultToEmpty();
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000006U)) {
    ::memset(&_impl_.return_immediately_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.max_messages_) -
        reinterpret_cast<char*>(&_impl_.return_immediately_)) + sizeof(_impl_.max_messages_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL PullRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const PullRequest& this_ = static_cast<const PullRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL PullRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const PullRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.PullRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_subscription().empty()) {
      const ::std::string& _s = this_._internal_subscription();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.PullRequest.subscription");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // bool return_immediately = 2 [deprecated = true, (.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_return_immediately() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          2, this_._internal_return_immediately(), target);
    }
  }

  // int32 max_messages = 3 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_max_messages() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<3>(
              stream, this_._internal_max_messages(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.PullRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t PullRequest::ByteSizeLong(const MessageLite& base) {
  const PullRequest& this_ = static_cast<const PullRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t PullRequest::ByteSizeLong() const {
  const PullRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.PullRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_subscription().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_subscription());
      }
    }
    // bool return_immediately = 2 [deprecated = true, (.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_return_immediately() != 0) {
        total_size += 2;
      }
    }
    // int32 max_messages = 3 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_max_messages() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_max_messages());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void PullRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<PullRequest*>(&to_msg);
  auto& from = static_cast<const PullRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.PullRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_subscription().empty()) {
        _this->_internal_set_subscription(from._internal_subscription());
      } else {
        if (_this->_impl_.subscription_.IsDefault()) {
          _this->_internal_set_subscription("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_return_immediately() != 0) {
        _this->_impl_.return_immediately_ = from._impl_.return_immediately_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_max_messages() != 0) {
        _this->_impl_.max_messages_ = from._impl_.max_messages_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void PullRequest::CopyFrom(const PullRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.PullRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void PullRequest::InternalSwap(PullRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.subscription_, &other->_impl_.subscription_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(PullRequest, _impl_.max_messages_)
      + sizeof(PullRequest::_impl_.max_messages_)
      - PROTOBUF_FIELD_OFFSET(PullRequest, _impl_.return_immediately_)>(
          reinterpret_cast<char*>(&_impl_.return_immediately_),
          reinterpret_cast<char*>(&other->_impl_.return_immediately_));
}

::google::protobuf::Metadata PullRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

PullResponse::PullResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PullResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.PullResponse)
}
PROTOBUF_NDEBUG_INLINE PullResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::PullResponse& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        received_messages_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::PullResponse,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PullResponse, _impl_.received_messages_)>()
          , from.received_messages_
        }
     {}

PullResponse::PullResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const PullResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PullResponse_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  PullResponse* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.PullResponse)
}
PROTOBUF_NDEBUG_INLINE PullResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        received_messages_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::PullResponse,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::PullResponse, _impl_.received_messages_)>()
         }
     {}

inline void PullResponse::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
PullResponse::~PullResponse() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.PullResponse)
  SharedDtor(*this);
}
inline void PullResponse::SharedDtor(MessageLite& self) {
  PullResponse& this_ = static_cast<PullResponse&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull PullResponse_class_data_ =
        PullResponse::InternalGenerateClassData_(PullResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PullResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PullResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(PullResponse_class_data_.tc_table);
  return PullResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PullResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PullResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&PullResponse_globals_));
  return PullResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const PullResponse::ParseTableT_
    PullResponse::_table_ =
        PullResponse::InternalGenerateParseTable_(PullResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void PullResponse::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.PullResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.received_messages_.Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL PullResponse::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const PullResponse& this_ = static_cast<const PullResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL PullResponse::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const PullResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.PullResponse)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated .google.pubsub.v1.ReceivedMessage received_messages = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_received_messages_size());
         i < n; i++) {
      const auto& repfield = this_._internal_received_messages().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              1, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.PullResponse)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t PullResponse::ByteSizeLong(const MessageLite& base) {
  const PullResponse& this_ = static_cast<const PullResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t PullResponse::ByteSizeLong() const {
  const PullResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.PullResponse)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
   {
    // repeated .google.pubsub.v1.ReceivedMessage received_messages = 1 [(.google.api.field_behavior) = OPTIONAL];
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_received_messages_size();
      for (const auto& msg : this_._internal_received_messages()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void PullResponse::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<PullResponse*>(&to_msg);
  auto& from = static_cast<const PullResponse&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.PullResponse)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _this->_internal_mutable_received_messages()->InternalMergeFromWithArena(
        ::google::protobuf::MessageLite::internal_visibility(), arena,
        from._internal_received_messages());
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void PullResponse::CopyFrom(const PullResponse& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.PullResponse)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void PullResponse::InternalSwap(PullResponse* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.received_messages_.InternalSwap(&other->_impl_.received_messages_);
}

::google::protobuf::Metadata PullResponse::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ModifyAckDeadlineRequest::ModifyAckDeadlineRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ModifyAckDeadlineRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ModifyAckDeadlineRequest)
}
PROTOBUF_NDEBUG_INLINE ModifyAckDeadlineRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::ModifyAckDeadlineRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        ack_ids_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::ModifyAckDeadlineRequest,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ModifyAckDeadlineRequest, _impl_.ack_ids_)>()
          , from.ack_ids_
        }
        ,
        subscription_(arena, from.subscription_) {}

ModifyAckDeadlineRequest::ModifyAckDeadlineRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ModifyAckDeadlineRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ModifyAckDeadlineRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ModifyAckDeadlineRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.ack_deadline_seconds_ = from._impl_.ack_deadline_seconds_;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ModifyAckDeadlineRequest)
}
PROTOBUF_NDEBUG_INLINE ModifyAckDeadlineRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::ModifyAckDeadlineRequest,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ModifyAckDeadlineRequest, _impl_.ack_ids_)>()
         }
        ,
        subscription_(arena) {}

inline void ModifyAckDeadlineRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.ack_deadline_seconds_ = {};
}
ModifyAckDeadlineRequest::~ModifyAckDeadlineRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.ModifyAckDeadlineRequest)
  SharedDtor(*this);
}
inline void ModifyAckDeadlineRequest::SharedDtor(MessageLite& self) {
  ModifyAckDeadlineRequest& this_ = static_cast<ModifyAckDeadlineRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.subscription_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ModifyAckDeadlineRequest_class_data_ =
        ModifyAckDeadlineRequest::InternalGenerateClassData_(ModifyAckDeadlineRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ModifyAckDeadlineRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ModifyAckDeadlineRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ModifyAckDeadlineRequest_class_data_.tc_table);
  return ModifyAckDeadlineRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ModifyAckDeadlineRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ModifyAckDeadlineRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ModifyAckDeadlineRequest_globals_));
  return ModifyAckDeadlineRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ModifyAckDeadlineRequest::ParseTableT_
    ModifyAckDeadlineRequest::_table_ =
        ModifyAckDeadlineRequest::InternalGenerateParseTable_(ModifyAckDeadlineRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ModifyAckDeadlineRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.ModifyAckDeadlineRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.ack_ids_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.subscription_.ClearNonDefaultToEmpty();
    }
  }
  _impl_.ack_deadline_seconds_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ModifyAckDeadlineRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ModifyAckDeadlineRequest& this_ = static_cast<const ModifyAckDeadlineRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ModifyAckDeadlineRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ModifyAckDeadlineRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.ModifyAckDeadlineRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_subscription().empty()) {
      const ::std::string& _s = this_._internal_subscription();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ModifyAckDeadlineRequest.subscription");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // int32 ack_deadline_seconds = 3 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_ack_deadline_seconds() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<3>(
              stream, this_._internal_ack_deadline_seconds(), target);
    }
  }

  // repeated string ack_ids = 4 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (int i = 0, n = this_._internal_ack_ids_size(); i < n; ++i) {
      const auto& s = this_._internal_ack_ids().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ModifyAckDeadlineRequest.ack_ids");
      target = stream->WriteString(4, s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.ModifyAckDeadlineRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ModifyAckDeadlineRequest::ByteSizeLong(const MessageLite& base) {
  const ModifyAckDeadlineRequest& this_ = static_cast<const ModifyAckDeadlineRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ModifyAckDeadlineRequest::ByteSizeLong() const {
  const ModifyAckDeadlineRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.ModifyAckDeadlineRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // repeated string ack_ids = 4 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_ack_ids().size());
      for (int i = 0, n = this_._internal_ack_ids().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_ack_ids().Get(i));
      }
    }
    // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_subscription().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_subscription());
      }
    }
    // int32 ack_deadline_seconds = 3 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_ack_deadline_seconds() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_ack_deadline_seconds());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ModifyAckDeadlineRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ModifyAckDeadlineRequest*>(&to_msg);
  auto& from = static_cast<const ModifyAckDeadlineRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.ModifyAckDeadlineRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_ack_ids()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_ack_ids());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_subscription().empty()) {
        _this->_internal_set_subscription(from._internal_subscription());
      } else {
        if (_this->_impl_.subscription_.IsDefault()) {
          _this->_internal_set_subscription("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_ack_deadline_seconds() != 0) {
        _this->_impl_.ack_deadline_seconds_ = from._impl_.ack_deadline_seconds_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ModifyAckDeadlineRequest::CopyFrom(const ModifyAckDeadlineRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.ModifyAckDeadlineRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ModifyAckDeadlineRequest::InternalSwap(ModifyAckDeadlineRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.ack_ids_.InternalSwap(&other->_impl_.ack_ids_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.subscription_, &other->_impl_.subscription_, arena);
  swap(_impl_.ack_deadline_seconds_, other->_impl_.ack_deadline_seconds_);
}

::google::protobuf::Metadata ModifyAckDeadlineRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

AcknowledgeRequest::AcknowledgeRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AcknowledgeRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.AcknowledgeRequest)
}
PROTOBUF_NDEBUG_INLINE AcknowledgeRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::AcknowledgeRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        ack_ids_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::AcknowledgeRequest,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::AcknowledgeRequest, _impl_.ack_ids_)>()
          , from.ack_ids_
        }
        ,
        subscription_(arena, from.subscription_) {}

AcknowledgeRequest::AcknowledgeRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const AcknowledgeRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AcknowledgeRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  AcknowledgeRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.AcknowledgeRequest)
}
PROTOBUF_NDEBUG_INLINE AcknowledgeRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::AcknowledgeRequest,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::AcknowledgeRequest, _impl_.ack_ids_)>()
         }
        ,
        subscription_(arena) {}

inline void AcknowledgeRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
AcknowledgeRequest::~AcknowledgeRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.AcknowledgeRequest)
  SharedDtor(*this);
}
inline void AcknowledgeRequest::SharedDtor(MessageLite& self) {
  AcknowledgeRequest& this_ = static_cast<AcknowledgeRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.subscription_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull AcknowledgeRequest_class_data_ =
        AcknowledgeRequest::InternalGenerateClassData_(AcknowledgeRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AcknowledgeRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AcknowledgeRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(AcknowledgeRequest_class_data_.tc_table);
  return AcknowledgeRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AcknowledgeRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AcknowledgeRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&AcknowledgeRequest_globals_));
  return AcknowledgeRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const AcknowledgeRequest::ParseTableT_
    AcknowledgeRequest::_table_ =
        AcknowledgeRequest::InternalGenerateParseTable_(AcknowledgeRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void AcknowledgeRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.AcknowledgeRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.ack_ids_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.subscription_.ClearNonDefaultToEmpty();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL AcknowledgeRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const AcknowledgeRequest& this_ = static_cast<const AcknowledgeRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL AcknowledgeRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const AcknowledgeRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.AcknowledgeRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_subscription().empty()) {
      const ::std::string& _s = this_._internal_subscription();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.AcknowledgeRequest.subscription");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // repeated string ack_ids = 2 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (int i = 0, n = this_._internal_ack_ids_size(); i < n; ++i) {
      const auto& s = this_._internal_ack_ids().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.AcknowledgeRequest.ack_ids");
      target = stream->WriteString(2, s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.AcknowledgeRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t AcknowledgeRequest::ByteSizeLong(const MessageLite& base) {
  const AcknowledgeRequest& this_ = static_cast<const AcknowledgeRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t AcknowledgeRequest::ByteSizeLong() const {
  const AcknowledgeRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.AcknowledgeRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // repeated string ack_ids = 2 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_ack_ids().size());
      for (int i = 0, n = this_._internal_ack_ids().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_ack_ids().Get(i));
      }
    }
    // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_subscription().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_subscription());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void AcknowledgeRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<AcknowledgeRequest*>(&to_msg);
  auto& from = static_cast<const AcknowledgeRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.AcknowledgeRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_ack_ids()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_ack_ids());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_subscription().empty()) {
        _this->_internal_set_subscription(from._internal_subscription());
      } else {
        if (_this->_impl_.subscription_.IsDefault()) {
          _this->_internal_set_subscription("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void AcknowledgeRequest::CopyFrom(const AcknowledgeRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.AcknowledgeRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void AcknowledgeRequest::InternalSwap(AcknowledgeRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.ack_ids_.InternalSwap(&other->_impl_.ack_ids_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.subscription_, &other->_impl_.subscription_, arena);
}

::google::protobuf::Metadata AcknowledgeRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

StreamingPullRequest::StreamingPullRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, StreamingPullRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.StreamingPullRequest)
}
PROTOBUF_NDEBUG_INLINE StreamingPullRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::StreamingPullRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        ack_ids_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::StreamingPullRequest,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullRequest, _impl_.ack_ids_)>()
          , from.ack_ids_
        }
        ,
        modify_deadline_seconds_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::StreamingPullRequest,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullRequest, _impl_.modify_deadline_seconds_)>()
          , from.modify_deadline_seconds_
        }
        ,
        _modify_deadline_seconds_cached_byte_size_{0},
        modify_deadline_ack_ids_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::StreamingPullRequest,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullRequest, _impl_.modify_deadline_ack_ids_)>()
          , from.modify_deadline_ack_ids_
        }
        ,
        subscription_(arena, from.subscription_),
        client_id_(arena, from.client_id_) {}

StreamingPullRequest::StreamingPullRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const StreamingPullRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, StreamingPullRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  StreamingPullRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, max_outstanding_messages_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, max_outstanding_messages_),
           offsetof(Impl_, stream_ack_deadline_seconds_) -
               offsetof(Impl_, max_outstanding_messages_) +
               sizeof(Impl_::stream_ack_deadline_seconds_));

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.StreamingPullRequest)
}
PROTOBUF_NDEBUG_INLINE StreamingPullRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullRequest,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullRequest, _impl_.ack_ids_)>()
         }
        ,
        modify_deadline_seconds_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullRequest,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullRequest, _impl_.modify_deadline_seconds_)>()
         }
        ,
        _modify_deadline_seconds_cached_byte_size_{0},
        modify_deadline_ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullRequest,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullRequest, _impl_.modify_deadline_ack_ids_)>()
         }
        ,
        subscription_(arena),
        client_id_(arena) {}

inline void StreamingPullRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, max_outstanding_messages_),
           0,
           offsetof(Impl_, stream_ack_deadline_seconds_) -
               offsetof(Impl_, max_outstanding_messages_) +
               sizeof(Impl_::stream_ack_deadline_seconds_));
}
StreamingPullRequest::~StreamingPullRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.StreamingPullRequest)
  SharedDtor(*this);
}
inline void StreamingPullRequest::SharedDtor(MessageLite& self) {
  StreamingPullRequest& this_ = static_cast<StreamingPullRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.subscription_.Destroy();
  this_._impl_.client_id_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull StreamingPullRequest_class_data_ =
        StreamingPullRequest::InternalGenerateClassData_(StreamingPullRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StreamingPullRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StreamingPullRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(StreamingPullRequest_class_data_.tc_table);
  return StreamingPullRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StreamingPullRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StreamingPullRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&StreamingPullRequest_globals_));
  return StreamingPullRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const StreamingPullRequest::ParseTableT_
    StreamingPullRequest::_table_ =
        StreamingPullRequest::InternalGenerateParseTable_(StreamingPullRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void StreamingPullRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.StreamingPullRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.ack_ids_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.modify_deadline_seconds_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.modify_deadline_ack_ids_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _impl_.subscription_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      _impl_.client_id_.ClearNonDefaultToEmpty();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x000000e0U)) {
    ::memset(&_impl_.max_outstanding_messages_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.stream_ack_deadline_seconds_) -
        reinterpret_cast<char*>(&_impl_.max_outstanding_messages_)) + sizeof(_impl_.stream_ack_deadline_seconds_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL StreamingPullRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const StreamingPullRequest& this_ = static_cast<const StreamingPullRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL StreamingPullRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const StreamingPullRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.StreamingPullRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (!this_._internal_subscription().empty()) {
      const ::std::string& _s = this_._internal_subscription();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.StreamingPullRequest.subscription");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // repeated string ack_ids = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (int i = 0, n = this_._internal_ack_ids_size(); i < n; ++i) {
      const auto& s = this_._internal_ack_ids().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.StreamingPullRequest.ack_ids");
      target = stream->WriteString(2, s, target);
    }
  }

  // repeated int32 modify_deadline_seconds = 3 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    {
      int byte_size = this_._impl_._modify_deadline_seconds_cached_byte_size_.Get();
      if (byte_size > 0) {
        target = stream->WriteInt32Packed(
            3, this_._internal_modify_deadline_seconds(), byte_size, target);
      }
    }
  }

  // repeated string modify_deadline_ack_ids = 4 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    for (int i = 0, n = this_._internal_modify_deadline_ack_ids_size(); i < n; ++i) {
      const auto& s = this_._internal_modify_deadline_ack_ids().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.StreamingPullRequest.modify_deadline_ack_ids");
      target = stream->WriteString(4, s, target);
    }
  }

  // int32 stream_ack_deadline_seconds = 5 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    if (this_._internal_stream_ack_deadline_seconds() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<5>(
              stream, this_._internal_stream_ack_deadline_seconds(), target);
    }
  }

  // string client_id = 6 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (!this_._internal_client_id().empty()) {
      const ::std::string& _s = this_._internal_client_id();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.StreamingPullRequest.client_id");
      target = stream->WriteStringMaybeAliased(6, _s, target);
    }
  }

  // int64 max_outstanding_messages = 7 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_max_outstanding_messages() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<7>(
              stream, this_._internal_max_outstanding_messages(), target);
    }
  }

  // int64 max_outstanding_bytes = 8 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_max_outstanding_bytes() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<8>(
              stream, this_._internal_max_outstanding_bytes(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.StreamingPullRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t StreamingPullRequest::ByteSizeLong(const MessageLite& base) {
  const StreamingPullRequest& this_ = static_cast<const StreamingPullRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t StreamingPullRequest::ByteSizeLong() const {
  const StreamingPullRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.StreamingPullRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // repeated string ack_ids = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_ack_ids().size());
      for (int i = 0, n = this_._internal_ack_ids().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_ack_ids().Get(i));
      }
    }
    // repeated int32 modify_deadline_seconds = 3 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size +=
          ::_pbi::WireFormatLite::Int32SizeWithPackedTagSize(
              this_._internal_modify_deadline_seconds(), 1,
              this_._impl_._modify_deadline_seconds_cached_byte_size_);
    }
    // repeated string modify_deadline_ack_ids = 4 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_modify_deadline_ack_ids().size());
      for (int i = 0, n = this_._internal_modify_deadline_ack_ids().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_modify_deadline_ack_ids().Get(i));
      }
    }
    // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!this_._internal_subscription().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_subscription());
      }
    }
    // string client_id = 6 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (!this_._internal_client_id().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_client_id());
      }
    }
    // int64 max_outstanding_messages = 7 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_max_outstanding_messages() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_max_outstanding_messages());
      }
    }
    // int64 max_outstanding_bytes = 8 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_max_outstanding_bytes() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_max_outstanding_bytes());
      }
    }
    // int32 stream_ack_deadline_seconds = 5 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (this_._internal_stream_ack_deadline_seconds() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_stream_ack_deadline_seconds());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void StreamingPullRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<StreamingPullRequest*>(&to_msg);
  auto& from = static_cast<const StreamingPullRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.StreamingPullRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_ack_ids()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_ack_ids());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _this->_internal_mutable_modify_deadline_seconds()->MergeFrom(from._internal_modify_deadline_seconds());
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _this->_internal_mutable_modify_deadline_ack_ids()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_modify_deadline_ack_ids());
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!from._internal_subscription().empty()) {
        _this->_internal_set_subscription(from._internal_subscription());
      } else {
        if (_this->_impl_.subscription_.IsDefault()) {
          _this->_internal_set_subscription("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (!from._internal_client_id().empty()) {
        _this->_internal_set_client_id(from._internal_client_id());
      } else {
        if (_this->_impl_.client_id_.IsDefault()) {
          _this->_internal_set_client_id("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_max_outstanding_messages() != 0) {
        _this->_impl_.max_outstanding_messages_ = from._impl_.max_outstanding_messages_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_max_outstanding_bytes() != 0) {
        _this->_impl_.max_outstanding_bytes_ = from._impl_.max_outstanding_bytes_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (from._internal_stream_ack_deadline_seconds() != 0) {
        _this->_impl_.stream_ack_deadline_seconds_ = from._impl_.stream_ack_deadline_seconds_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void StreamingPullRequest::CopyFrom(const StreamingPullRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.StreamingPullRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void StreamingPullRequest::InternalSwap(StreamingPullRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.ack_ids_.InternalSwap(&other->_impl_.ack_ids_);
  _impl_.modify_deadline_seconds_.InternalSwap(&other->_impl_.modify_deadline_seconds_);
  _impl_.modify_deadline_ack_ids_.InternalSwap(&other->_impl_.modify_deadline_ack_ids_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.subscription_, &other->_impl_.subscription_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.client_id_, &other->_impl_.client_id_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(StreamingPullRequest, _impl_.stream_ack_deadline_seconds_)
      + sizeof(StreamingPullRequest::_impl_.stream_ack_deadline_seconds_)
      - PROTOBUF_FIELD_OFFSET(StreamingPullRequest, _impl_.max_outstanding_messages_)>(
          reinterpret_cast<char*>(&_impl_.max_outstanding_messages_),
          reinterpret_cast<char*>(&other->_impl_.max_outstanding_messages_));
}

::google::protobuf::Metadata StreamingPullRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

StreamingPullResponse_AcknowledgeConfirmation::StreamingPullResponse_AcknowledgeConfirmation(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, StreamingPullResponse_AcknowledgeConfirmation_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.StreamingPullResponse.AcknowledgeConfirmation)
}
PROTOBUF_NDEBUG_INLINE StreamingPullResponse_AcknowledgeConfirmation::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        ack_ids_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation, _impl_.ack_ids_)>()
          , from.ack_ids_
        }
        ,
        invalid_ack_ids_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation, _impl_.invalid_ack_ids_)>()
          , from.invalid_ack_ids_
        }
        ,
        unordered_ack_ids_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation, _impl_.unordered_ack_ids_)>()
          , from.unordered_ack_ids_
        }
        ,
        temporary_failed_ack_ids_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation, _impl_.temporary_failed_ack_ids_)>()
          , from.temporary_failed_ack_ids_
        }
     {}

StreamingPullResponse_AcknowledgeConfirmation::StreamingPullResponse_AcknowledgeConfirmation(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const StreamingPullResponse_AcknowledgeConfirmation& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, StreamingPullResponse_AcknowledgeConfirmation_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  StreamingPullResponse_AcknowledgeConfirmation* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.StreamingPullResponse.AcknowledgeConfirmation)
}
PROTOBUF_NDEBUG_INLINE StreamingPullResponse_AcknowledgeConfirmation::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation, _impl_.ack_ids_)>()
         }
        ,
        invalid_ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation, _impl_.invalid_ack_ids_)>()
         }
        ,
        unordered_ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation, _impl_.unordered_ack_ids_)>()
         }
        ,
        temporary_failed_ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_AcknowledgeConfirmation, _impl_.temporary_failed_ack_ids_)>()
         }
     {}

inline void StreamingPullResponse_AcknowledgeConfirmation::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
StreamingPullResponse_AcknowledgeConfirmation::~StreamingPullResponse_AcknowledgeConfirmation() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.StreamingPullResponse.AcknowledgeConfirmation)
  SharedDtor(*this);
}
inline void StreamingPullResponse_AcknowledgeConfirmation::SharedDtor(MessageLite& self) {
  StreamingPullResponse_AcknowledgeConfirmation& this_ = static_cast<StreamingPullResponse_AcknowledgeConfirmation&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull StreamingPullResponse_AcknowledgeConfirmation_class_data_ =
        StreamingPullResponse_AcknowledgeConfirmation::InternalGenerateClassData_(StreamingPullResponse_AcknowledgeConfirmation_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StreamingPullResponse_AcknowledgeConfirmation::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StreamingPullResponse_AcknowledgeConfirmation_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(StreamingPullResponse_AcknowledgeConfirmation_class_data_.tc_table);
  return StreamingPullResponse_AcknowledgeConfirmation_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StreamingPullResponse_AcknowledgeConfirmation::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StreamingPullResponse_AcknowledgeConfirmation_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&StreamingPullResponse_AcknowledgeConfirmation_globals_));
  return StreamingPullResponse_AcknowledgeConfirmation_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const StreamingPullResponse_AcknowledgeConfirmation::ParseTableT_
    StreamingPullResponse_AcknowledgeConfirmation::_table_ =
        StreamingPullResponse_AcknowledgeConfirmation::InternalGenerateParseTable_(StreamingPullResponse_AcknowledgeConfirmation_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void StreamingPullResponse_AcknowledgeConfirmation::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.StreamingPullResponse.AcknowledgeConfirmation)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.ack_ids_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.invalid_ack_ids_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.unordered_ack_ids_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _impl_.temporary_failed_ack_ids_.Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL StreamingPullResponse_AcknowledgeConfirmation::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const StreamingPullResponse_AcknowledgeConfirmation& this_ = static_cast<const StreamingPullResponse_AcknowledgeConfirmation&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL StreamingPullResponse_AcknowledgeConfirmation::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const StreamingPullResponse_AcknowledgeConfirmation& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.StreamingPullResponse.AcknowledgeConfirmation)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated string ack_ids = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (int i = 0, n = this_._internal_ack_ids_size(); i < n; ++i) {
      const auto& s = this_._internal_ack_ids().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.StreamingPullResponse.AcknowledgeConfirmation.ack_ids");
      target = stream->WriteString(1, s, target);
    }
  }

  // repeated string invalid_ack_ids = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    for (int i = 0, n = this_._internal_invalid_ack_ids_size(); i < n; ++i) {
      const auto& s = this_._internal_invalid_ack_ids().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.StreamingPullResponse.AcknowledgeConfirmation.invalid_ack_ids");
      target = stream->WriteString(2, s, target);
    }
  }

  // repeated string unordered_ack_ids = 3 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    for (int i = 0, n = this_._internal_unordered_ack_ids_size(); i < n; ++i) {
      const auto& s = this_._internal_unordered_ack_ids().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.StreamingPullResponse.AcknowledgeConfirmation.unordered_ack_ids");
      target = stream->WriteString(3, s, target);
    }
  }

  // repeated string temporary_failed_ack_ids = 4 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    for (int i = 0, n = this_._internal_temporary_failed_ack_ids_size(); i < n; ++i) {
      const auto& s = this_._internal_temporary_failed_ack_ids().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.StreamingPullResponse.AcknowledgeConfirmation.temporary_failed_ack_ids");
      target = stream->WriteString(4, s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.StreamingPullResponse.AcknowledgeConfirmation)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t StreamingPullResponse_AcknowledgeConfirmation::ByteSizeLong(const MessageLite& base) {
  const StreamingPullResponse_AcknowledgeConfirmation& this_ = static_cast<const StreamingPullResponse_AcknowledgeConfirmation&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t StreamingPullResponse_AcknowledgeConfirmation::ByteSizeLong() const {
  const StreamingPullResponse_AcknowledgeConfirmation& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.StreamingPullResponse.AcknowledgeConfirmation)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // repeated string ack_ids = 1 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_ack_ids().size());
      for (int i = 0, n = this_._internal_ack_ids().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_ack_ids().Get(i));
      }
    }
    // repeated string invalid_ack_ids = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_invalid_ack_ids().size());
      for (int i = 0, n = this_._internal_invalid_ack_ids().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_invalid_ack_ids().Get(i));
      }
    }
    // repeated string unordered_ack_ids = 3 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_unordered_ack_ids().size());
      for (int i = 0, n = this_._internal_unordered_ack_ids().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_unordered_ack_ids().Get(i));
      }
    }
    // repeated string temporary_failed_ack_ids = 4 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_temporary_failed_ack_ids().size());
      for (int i = 0, n = this_._internal_temporary_failed_ack_ids().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_temporary_failed_ack_ids().Get(i));
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void StreamingPullResponse_AcknowledgeConfirmation::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<StreamingPullResponse_AcknowledgeConfirmation*>(&to_msg);
  auto& from = static_cast<const StreamingPullResponse_AcknowledgeConfirmation&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.StreamingPullResponse.AcknowledgeConfirmation)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_ack_ids()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_ack_ids());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _this->_internal_mutable_invalid_ack_ids()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_invalid_ack_ids());
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _this->_internal_mutable_unordered_ack_ids()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_unordered_ack_ids());
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _this->_internal_mutable_temporary_failed_ack_ids()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_temporary_failed_ack_ids());
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void StreamingPullResponse_AcknowledgeConfirmation::CopyFrom(const StreamingPullResponse_AcknowledgeConfirmation& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.StreamingPullResponse.AcknowledgeConfirmation)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void StreamingPullResponse_AcknowledgeConfirmation::InternalSwap(StreamingPullResponse_AcknowledgeConfirmation* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.ack_ids_.InternalSwap(&other->_impl_.ack_ids_);
  _impl_.invalid_ack_ids_.InternalSwap(&other->_impl_.invalid_ack_ids_);
  _impl_.unordered_ack_ids_.InternalSwap(&other->_impl_.unordered_ack_ids_);
  _impl_.temporary_failed_ack_ids_.InternalSwap(&other->_impl_.temporary_failed_ack_ids_);
}

::google::protobuf::Metadata StreamingPullResponse_AcknowledgeConfirmation::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

StreamingPullResponse_ModifyAckDeadlineConfirmation::StreamingPullResponse_ModifyAckDeadlineConfirmation(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, StreamingPullResponse_ModifyAckDeadlineConfirmation_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.StreamingPullResponse.ModifyAckDeadlineConfirmation)
}
PROTOBUF_NDEBUG_INLINE StreamingPullResponse_ModifyAckDeadlineConfirmation::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        ack_ids_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_.ack_ids_)>()
          , from.ack_ids_
        }
        ,
        invalid_ack_ids_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_.invalid_ack_ids_)>()
          , from.invalid_ack_ids_
        }
        ,
        temporary_failed_ack_ids_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_.temporary_failed_ack_ids_)>()
          , from.temporary_failed_ack_ids_
        }
     {}

StreamingPullResponse_ModifyAckDeadlineConfirmation::StreamingPullResponse_ModifyAckDeadlineConfirmation(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const StreamingPullResponse_ModifyAckDeadlineConfirmation& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, StreamingPullResponse_ModifyAckDeadlineConfirmation_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  StreamingPullResponse_ModifyAckDeadlineConfirmation* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.StreamingPullResponse.ModifyAckDeadlineConfirmation)
}
PROTOBUF_NDEBUG_INLINE StreamingPullResponse_ModifyAckDeadlineConfirmation::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_.ack_ids_)>()
         }
        ,
        invalid_ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_.invalid_ack_ids_)>()
         }
        ,
        temporary_failed_ack_ids_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse_ModifyAckDeadlineConfirmation, _impl_.temporary_failed_ack_ids_)>()
         }
     {}

inline void StreamingPullResponse_ModifyAckDeadlineConfirmation::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
StreamingPullResponse_ModifyAckDeadlineConfirmation::~StreamingPullResponse_ModifyAckDeadlineConfirmation() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.StreamingPullResponse.ModifyAckDeadlineConfirmation)
  SharedDtor(*this);
}
inline void StreamingPullResponse_ModifyAckDeadlineConfirmation::SharedDtor(MessageLite& self) {
  StreamingPullResponse_ModifyAckDeadlineConfirmation& this_ = static_cast<StreamingPullResponse_ModifyAckDeadlineConfirmation&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull StreamingPullResponse_ModifyAckDeadlineConfirmation_class_data_ =
        StreamingPullResponse_ModifyAckDeadlineConfirmation::InternalGenerateClassData_(StreamingPullResponse_ModifyAckDeadlineConfirmation_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StreamingPullResponse_ModifyAckDeadlineConfirmation::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StreamingPullResponse_ModifyAckDeadlineConfirmation_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(StreamingPullResponse_ModifyAckDeadlineConfirmation_class_data_.tc_table);
  return StreamingPullResponse_ModifyAckDeadlineConfirmation_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StreamingPullResponse_ModifyAckDeadlineConfirmation::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StreamingPullResponse_ModifyAckDeadlineConfirmation_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&StreamingPullResponse_ModifyAckDeadlineConfirmation_globals_));
  return StreamingPullResponse_ModifyAckDeadlineConfirmation_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const StreamingPullResponse_ModifyAckDeadlineConfirmation::ParseTableT_
    StreamingPullResponse_ModifyAckDeadlineConfirmation::_table_ =
        StreamingPullResponse_ModifyAckDeadlineConfirmation::InternalGenerateParseTable_(StreamingPullResponse_ModifyAckDeadlineConfirmation_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void StreamingPullResponse_ModifyAckDeadlineConfirmation::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.StreamingPullResponse.ModifyAckDeadlineConfirmation)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.ack_ids_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.invalid_ack_ids_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.temporary_failed_ack_ids_.Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL StreamingPullResponse_ModifyAckDeadlineConfirmation::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const StreamingPullResponse_ModifyAckDeadlineConfirmation& this_ = static_cast<const StreamingPullResponse_ModifyAckDeadlineConfirmation&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL StreamingPullResponse_ModifyAckDeadlineConfirmation::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const StreamingPullResponse_ModifyAckDeadlineConfirmation& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.StreamingPullResponse.ModifyAckDeadlineConfirmation)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated string ack_ids = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (int i = 0, n = this_._internal_ack_ids_size(); i < n; ++i) {
      const auto& s = this_._internal_ack_ids().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.StreamingPullResponse.ModifyAckDeadlineConfirmation.ack_ids");
      target = stream->WriteString(1, s, target);
    }
  }

  // repeated string invalid_ack_ids = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    for (int i = 0, n = this_._internal_invalid_ack_ids_size(); i < n; ++i) {
      const auto& s = this_._internal_invalid_ack_ids().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.StreamingPullResponse.ModifyAckDeadlineConfirmation.invalid_ack_ids");
      target = stream->WriteString(2, s, target);
    }
  }

  // repeated string temporary_failed_ack_ids = 3 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    for (int i = 0, n = this_._internal_temporary_failed_ack_ids_size(); i < n; ++i) {
      const auto& s = this_._internal_temporary_failed_ack_ids().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.StreamingPullResponse.ModifyAckDeadlineConfirmation.temporary_failed_ack_ids");
      target = stream->WriteString(3, s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.StreamingPullResponse.ModifyAckDeadlineConfirmation)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t StreamingPullResponse_ModifyAckDeadlineConfirmation::ByteSizeLong(const MessageLite& base) {
  const StreamingPullResponse_ModifyAckDeadlineConfirmation& this_ = static_cast<const StreamingPullResponse_ModifyAckDeadlineConfirmation&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t StreamingPullResponse_ModifyAckDeadlineConfirmation::ByteSizeLong() const {
  const StreamingPullResponse_ModifyAckDeadlineConfirmation& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.StreamingPullResponse.ModifyAckDeadlineConfirmation)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // repeated string ack_ids = 1 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_ack_ids().size());
      for (int i = 0, n = this_._internal_ack_ids().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_ack_ids().Get(i));
      }
    }
    // repeated string invalid_ack_ids = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_invalid_ack_ids().size());
      for (int i = 0, n = this_._internal_invalid_ack_ids().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_invalid_ack_ids().Get(i));
      }
    }
    // repeated string temporary_failed_ack_ids = 3 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_temporary_failed_ack_ids().size());
      for (int i = 0, n = this_._internal_temporary_failed_ack_ids().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_temporary_failed_ack_ids().Get(i));
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void StreamingPullResponse_ModifyAckDeadlineConfirmation::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<StreamingPullResponse_ModifyAckDeadlineConfirmation*>(&to_msg);
  auto& from = static_cast<const StreamingPullResponse_ModifyAckDeadlineConfirmation&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.StreamingPullResponse.ModifyAckDeadlineConfirmation)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_ack_ids()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_ack_ids());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _this->_internal_mutable_invalid_ack_ids()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_invalid_ack_ids());
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _this->_internal_mutable_temporary_failed_ack_ids()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_temporary_failed_ack_ids());
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void StreamingPullResponse_ModifyAckDeadlineConfirmation::CopyFrom(const StreamingPullResponse_ModifyAckDeadlineConfirmation& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.StreamingPullResponse.ModifyAckDeadlineConfirmation)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void StreamingPullResponse_ModifyAckDeadlineConfirmation::InternalSwap(StreamingPullResponse_ModifyAckDeadlineConfirmation* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.ack_ids_.InternalSwap(&other->_impl_.ack_ids_);
  _impl_.invalid_ack_ids_.InternalSwap(&other->_impl_.invalid_ack_ids_);
  _impl_.temporary_failed_ack_ids_.InternalSwap(&other->_impl_.temporary_failed_ack_ids_);
}

::google::protobuf::Metadata StreamingPullResponse_ModifyAckDeadlineConfirmation::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

StreamingPullResponse_SubscriptionProperties::StreamingPullResponse_SubscriptionProperties(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, StreamingPullResponse_SubscriptionProperties_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.StreamingPullResponse.SubscriptionProperties)
}
StreamingPullResponse_SubscriptionProperties::StreamingPullResponse_SubscriptionProperties(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const StreamingPullResponse_SubscriptionProperties& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, StreamingPullResponse_SubscriptionProperties_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE StreamingPullResponse_SubscriptionProperties::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void StreamingPullResponse_SubscriptionProperties::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, exactly_once_delivery_enabled_),
           0,
           offsetof(Impl_, message_ordering_enabled_) -
               offsetof(Impl_, exactly_once_delivery_enabled_) +
               sizeof(Impl_::message_ordering_enabled_));
}
StreamingPullResponse_SubscriptionProperties::~StreamingPullResponse_SubscriptionProperties() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.StreamingPullResponse.SubscriptionProperties)
  SharedDtor(*this);
}
inline void StreamingPullResponse_SubscriptionProperties::SharedDtor(MessageLite& self) {
  StreamingPullResponse_SubscriptionProperties& this_ = static_cast<StreamingPullResponse_SubscriptionProperties&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull StreamingPullResponse_SubscriptionProperties_class_data_ =
        StreamingPullResponse_SubscriptionProperties::InternalGenerateClassData_(StreamingPullResponse_SubscriptionProperties_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StreamingPullResponse_SubscriptionProperties::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StreamingPullResponse_SubscriptionProperties_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(StreamingPullResponse_SubscriptionProperties_class_data_.tc_table);
  return StreamingPullResponse_SubscriptionProperties_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StreamingPullResponse_SubscriptionProperties::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StreamingPullResponse_SubscriptionProperties_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&StreamingPullResponse_SubscriptionProperties_globals_));
  return StreamingPullResponse_SubscriptionProperties_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const StreamingPullResponse_SubscriptionProperties::ParseTableT_
    StreamingPullResponse_SubscriptionProperties::_table_ =
        StreamingPullResponse_SubscriptionProperties::InternalGenerateParseTable_(StreamingPullResponse_SubscriptionProperties_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void StreamingPullResponse_SubscriptionProperties::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.StreamingPullResponse.SubscriptionProperties)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  ::memset(&_impl_.exactly_once_delivery_enabled_, 0, static_cast<::size_t>(
      reinterpret_cast<char*>(&_impl_.message_ordering_enabled_) -
      reinterpret_cast<char*>(&_impl_.exactly_once_delivery_enabled_)) + sizeof(_impl_.message_ordering_enabled_));
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL StreamingPullResponse_SubscriptionProperties::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const StreamingPullResponse_SubscriptionProperties& this_ = static_cast<const StreamingPullResponse_SubscriptionProperties&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL StreamingPullResponse_SubscriptionProperties::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const StreamingPullResponse_SubscriptionProperties& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.StreamingPullResponse.SubscriptionProperties)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // bool exactly_once_delivery_enabled = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_exactly_once_delivery_enabled() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          1, this_._internal_exactly_once_delivery_enabled(), target);
    }
  }

  // bool message_ordering_enabled = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_message_ordering_enabled() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          2, this_._internal_message_ordering_enabled(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.StreamingPullResponse.SubscriptionProperties)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t StreamingPullResponse_SubscriptionProperties::ByteSizeLong(const MessageLite& base) {
  const StreamingPullResponse_SubscriptionProperties& this_ = static_cast<const StreamingPullResponse_SubscriptionProperties&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t StreamingPullResponse_SubscriptionProperties::ByteSizeLong() const {
  const StreamingPullResponse_SubscriptionProperties& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.StreamingPullResponse.SubscriptionProperties)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // bool exactly_once_delivery_enabled = 1 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_exactly_once_delivery_enabled() != 0) {
        total_size += 2;
      }
    }
    // bool message_ordering_enabled = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_message_ordering_enabled() != 0) {
        total_size += 2;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void StreamingPullResponse_SubscriptionProperties::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<StreamingPullResponse_SubscriptionProperties*>(&to_msg);
  auto& from = static_cast<const StreamingPullResponse_SubscriptionProperties&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.StreamingPullResponse.SubscriptionProperties)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (from._internal_exactly_once_delivery_enabled() != 0) {
        _this->_impl_.exactly_once_delivery_enabled_ = from._impl_.exactly_once_delivery_enabled_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_message_ordering_enabled() != 0) {
        _this->_impl_.message_ordering_enabled_ = from._impl_.message_ordering_enabled_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void StreamingPullResponse_SubscriptionProperties::CopyFrom(const StreamingPullResponse_SubscriptionProperties& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.StreamingPullResponse.SubscriptionProperties)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void StreamingPullResponse_SubscriptionProperties::InternalSwap(StreamingPullResponse_SubscriptionProperties* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(StreamingPullResponse_SubscriptionProperties, _impl_.message_ordering_enabled_)
      + sizeof(StreamingPullResponse_SubscriptionProperties::_impl_.message_ordering_enabled_)
      - PROTOBUF_FIELD_OFFSET(StreamingPullResponse_SubscriptionProperties, _impl_.exactly_once_delivery_enabled_)>(
          reinterpret_cast<char*>(&_impl_.exactly_once_delivery_enabled_),
          reinterpret_cast<char*>(&other->_impl_.exactly_once_delivery_enabled_));
}

::google::protobuf::Metadata StreamingPullResponse_SubscriptionProperties::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

StreamingPullResponse::StreamingPullResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, StreamingPullResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.StreamingPullResponse)
}
PROTOBUF_NDEBUG_INLINE StreamingPullResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::StreamingPullResponse& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        received_messages_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::StreamingPullResponse,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse, _impl_.received_messages_)>()
          , from.received_messages_
        }
     {}

StreamingPullResponse::StreamingPullResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const StreamingPullResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, StreamingPullResponse_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  StreamingPullResponse* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.modify_ack_deadline_confirmation_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.modify_ack_deadline_confirmation_)
                : nullptr;
  _impl_.subscription_properties_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.subscription_properties_)
                : nullptr;
  _impl_.acknowledge_confirmation_ = (CheckHasBit(cached_has_bits, 0x00000008U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.acknowledge_confirmation_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.StreamingPullResponse)
}
PROTOBUF_NDEBUG_INLINE StreamingPullResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        received_messages_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::StreamingPullResponse,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::StreamingPullResponse, _impl_.received_messages_)>()
         }
     {}

inline void StreamingPullResponse::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, modify_ack_deadline_confirmation_),
           0,
           offsetof(Impl_, acknowledge_confirmation_) -
               offsetof(Impl_, modify_ack_deadline_confirmation_) +
               sizeof(Impl_::acknowledge_confirmation_));
}
StreamingPullResponse::~StreamingPullResponse() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.StreamingPullResponse)
  SharedDtor(*this);
}
inline void StreamingPullResponse::SharedDtor(MessageLite& self) {
  StreamingPullResponse& this_ = static_cast<StreamingPullResponse&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.modify_ack_deadline_confirmation_;
  delete this_._impl_.subscription_properties_;
  delete this_._impl_.acknowledge_confirmation_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull StreamingPullResponse_class_data_ =
        StreamingPullResponse::InternalGenerateClassData_(StreamingPullResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StreamingPullResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StreamingPullResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(StreamingPullResponse_class_data_.tc_table);
  return StreamingPullResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StreamingPullResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StreamingPullResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&StreamingPullResponse_globals_));
  return StreamingPullResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const StreamingPullResponse::ParseTableT_
    StreamingPullResponse::_table_ =
        StreamingPullResponse::InternalGenerateParseTable_(StreamingPullResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void StreamingPullResponse::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.StreamingPullResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.received_messages_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.modify_ack_deadline_confirmation_ != nullptr);
      _impl_.modify_ack_deadline_confirmation_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(_impl_.subscription_properties_ != nullptr);
      _impl_.subscription_properties_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(_impl_.acknowledge_confirmation_ != nullptr);
      _impl_.acknowledge_confirmation_->Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL StreamingPullResponse::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const StreamingPullResponse& this_ = static_cast<const StreamingPullResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL StreamingPullResponse::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const StreamingPullResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.StreamingPullResponse)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated .google.pubsub.v1.ReceivedMessage received_messages = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_received_messages_size());
         i < n; i++) {
      const auto& repfield = this_._internal_received_messages().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              1, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  // .google.pubsub.v1.StreamingPullResponse.ModifyAckDeadlineConfirmation modify_ack_deadline_confirmation = 3 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        3, *this_._impl_.modify_ack_deadline_confirmation_, this_._impl_.modify_ack_deadline_confirmation_->GetCachedSize(), target,
        stream);
  }

  // .google.pubsub.v1.StreamingPullResponse.SubscriptionProperties subscription_properties = 4 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        4, *this_._impl_.subscription_properties_, this_._impl_.subscription_properties_->GetCachedSize(), target,
        stream);
  }

  // .google.pubsub.v1.StreamingPullResponse.AcknowledgeConfirmation acknowledge_confirmation = 5 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        5, *this_._impl_.acknowledge_confirmation_, this_._impl_.acknowledge_confirmation_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.StreamingPullResponse)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t StreamingPullResponse::ByteSizeLong(const MessageLite& base) {
  const StreamingPullResponse& this_ = static_cast<const StreamingPullResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t StreamingPullResponse::ByteSizeLong() const {
  const StreamingPullResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.StreamingPullResponse)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // repeated .google.pubsub.v1.ReceivedMessage received_messages = 1 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_received_messages_size();
      for (const auto& msg : this_._internal_received_messages()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
    // .google.pubsub.v1.StreamingPullResponse.ModifyAckDeadlineConfirmation modify_ack_deadline_confirmation = 3 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.modify_ack_deadline_confirmation_);
    }
    // .google.pubsub.v1.StreamingPullResponse.SubscriptionProperties subscription_properties = 4 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.subscription_properties_);
    }
    // .google.pubsub.v1.StreamingPullResponse.AcknowledgeConfirmation acknowledge_confirmation = 5 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.acknowledge_confirmation_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void StreamingPullResponse::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<StreamingPullResponse*>(&to_msg);
  auto& from = static_cast<const StreamingPullResponse&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.StreamingPullResponse)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_received_messages()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_received_messages());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.modify_ack_deadline_confirmation_ != nullptr);
      if (_this->_impl_.modify_ack_deadline_confirmation_ == nullptr) {
        _this->_impl_.modify_ack_deadline_confirmation_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.modify_ack_deadline_confirmation_);
      } else {
        _this->_impl_.modify_ack_deadline_confirmation_->MergeFrom(*from._impl_.modify_ack_deadline_confirmation_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.subscription_properties_ != nullptr);
      if (_this->_impl_.subscription_properties_ == nullptr) {
        _this->_impl_.subscription_properties_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.subscription_properties_);
      } else {
        _this->_impl_.subscription_properties_->MergeFrom(*from._impl_.subscription_properties_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(from._impl_.acknowledge_confirmation_ != nullptr);
      if (_this->_impl_.acknowledge_confirmation_ == nullptr) {
        _this->_impl_.acknowledge_confirmation_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.acknowledge_confirmation_);
      } else {
        _this->_impl_.acknowledge_confirmation_->MergeFrom(*from._impl_.acknowledge_confirmation_);
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void StreamingPullResponse::CopyFrom(const StreamingPullResponse& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.StreamingPullResponse)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void StreamingPullResponse::InternalSwap(StreamingPullResponse* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.received_messages_.InternalSwap(&other->_impl_.received_messages_);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(StreamingPullResponse, _impl_.acknowledge_confirmation_)
      + sizeof(StreamingPullResponse::_impl_.acknowledge_confirmation_)
      - PROTOBUF_FIELD_OFFSET(StreamingPullResponse, _impl_.modify_ack_deadline_confirmation_)>(
          reinterpret_cast<char*>(&_impl_.modify_ack_deadline_confirmation_),
          reinterpret_cast<char*>(&other->_impl_.modify_ack_deadline_confirmation_));
}

::google::protobuf::Metadata StreamingPullResponse::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

#if defined(PROTOBUF_CUSTOM_VTABLE)
CreateSnapshotRequest_LabelsEntry_DoNotUse::CreateSnapshotRequest_LabelsEntry_DoNotUse() : SuperType(CreateSnapshotRequest_LabelsEntry_DoNotUse_get_class_data()) {}
CreateSnapshotRequest_LabelsEntry_DoNotUse::CreateSnapshotRequest_LabelsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
    : SuperType(arena, CreateSnapshotRequest_LabelsEntry_DoNotUse_get_class_data()) {}
#else   // PROTOBUF_CUSTOM_VTABLE
CreateSnapshotRequest_LabelsEntry_DoNotUse::CreateSnapshotRequest_LabelsEntry_DoNotUse() : SuperType() {}
CreateSnapshotRequest_LabelsEntry_DoNotUse::CreateSnapshotRequest_LabelsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena) : SuperType(arena) {}
#endif  // PROTOBUF_CUSTOM_VTABLE
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull CreateSnapshotRequest_LabelsEntry_DoNotUse_class_data_ =
        CreateSnapshotRequest_LabelsEntry_DoNotUse::InternalGenerateClassData_(CreateSnapshotRequest_LabelsEntry_DoNotUse_globals_._default);

// including ~ all the vtables and everything they use become part
// of the same ~ SCC.
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CreateSnapshotRequest_LabelsEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CreateSnapshotRequest_LabelsEntry_DoNotUse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(CreateSnapshotRequest_LabelsEntry_DoNotUse_class_data_.tc_table);
  return CreateSnapshotRequest_LabelsEntry_DoNotUse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CreateSnapshotRequest_LabelsEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CreateSnapshotRequest_LabelsEntry_DoNotUse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&CreateSnapshotRequest_LabelsEntry_DoNotUse_globals_));
  return CreateSnapshotRequest_LabelsEntry_DoNotUse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const CreateSnapshotRequest_LabelsEntry_DoNotUse::ParseTableT_
    CreateSnapshotRequest_LabelsEntry_DoNotUse::_table_ =
        CreateSnapshotRequest_LabelsEntry_DoNotUse::InternalGenerateParseTable_(CreateSnapshotRequest_LabelsEntry_DoNotUse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
// ===================================================================

CreateSnapshotRequest::CreateSnapshotRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, CreateSnapshotRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.CreateSnapshotRequest)
}
PROTOBUF_NDEBUG_INLINE CreateSnapshotRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::CreateSnapshotRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        name_(arena, from.name_),
        subscription_(arena, from.subscription_),
        labels_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::CreateSnapshotRequest,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CreateSnapshotRequest, _impl_.labels_)>()
          , from.labels_
        }
     {}

CreateSnapshotRequest::CreateSnapshotRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const CreateSnapshotRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, CreateSnapshotRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  CreateSnapshotRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.CreateSnapshotRequest)
}
PROTOBUF_NDEBUG_INLINE CreateSnapshotRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        name_(arena),
        subscription_(arena),
        labels_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::CreateSnapshotRequest,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::CreateSnapshotRequest, _impl_.labels_)>()
         }
     {}

inline void CreateSnapshotRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
CreateSnapshotRequest::~CreateSnapshotRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.CreateSnapshotRequest)
  SharedDtor(*this);
}
inline void CreateSnapshotRequest::SharedDtor(MessageLite& self) {
  CreateSnapshotRequest& this_ = static_cast<CreateSnapshotRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.name_.Destroy();
  this_._impl_.subscription_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull CreateSnapshotRequest_class_data_ =
        CreateSnapshotRequest::InternalGenerateClassData_(CreateSnapshotRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CreateSnapshotRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CreateSnapshotRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(CreateSnapshotRequest_class_data_.tc_table);
  return CreateSnapshotRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CreateSnapshotRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CreateSnapshotRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&CreateSnapshotRequest_globals_));
  return CreateSnapshotRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const CreateSnapshotRequest::ParseTableT_
    CreateSnapshotRequest::_table_ =
        CreateSnapshotRequest::InternalGenerateParseTable_(CreateSnapshotRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void CreateSnapshotRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.CreateSnapshotRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.name_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.subscription_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.labels_.Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL CreateSnapshotRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const CreateSnapshotRequest& this_ = static_cast<const CreateSnapshotRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL CreateSnapshotRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const CreateSnapshotRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.CreateSnapshotRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_name().empty()) {
      const ::std::string& _s = this_._internal_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.CreateSnapshotRequest.name");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // string subscription = 2 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_subscription().empty()) {
      const ::std::string& _s = this_._internal_subscription();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.CreateSnapshotRequest.subscription");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  // map<string, string> labels = 3 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_labels().empty()) {
      using MapType = ::google::protobuf::Map<::std::string, ::std::string>;
      using WireHelper = _pbi::MapEntryFuncs<::std::string, ::std::string,
                                     _pbi::WireFormatLite::TYPE_STRING,
                                     _pbi::WireFormatLite::TYPE_STRING>;
      const auto& field = this_._internal_labels();

      if (stream->IsSerializationDeterministic() && field.size() > 1) {
        for (const auto& entry : ::google::protobuf::internal::MapSorterPtr<MapType>(field)) {
          target = WireHelper::InternalSerialize(
              3, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.CreateSnapshotRequest.labels");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.CreateSnapshotRequest.labels");
        }
      } else {
        for (const auto& entry : field) {
          target = WireHelper::InternalSerialize(
              3, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.CreateSnapshotRequest.labels");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.CreateSnapshotRequest.labels");
        }
      }
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.CreateSnapshotRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t CreateSnapshotRequest::ByteSizeLong(const MessageLite& base) {
  const CreateSnapshotRequest& this_ = static_cast<const CreateSnapshotRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t CreateSnapshotRequest::ByteSizeLong() const {
  const CreateSnapshotRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.CreateSnapshotRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_name());
      }
    }
    // string subscription = 2 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_subscription().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_subscription());
      }
    }
    // map<string, string> labels = 3 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_labels_size());
      for (const auto& entry : this_._internal_labels()) {
        total_size += _pbi::MapEntryFuncs<::std::string, ::std::string,
                                       _pbi::WireFormatLite::TYPE_STRING,
                                       _pbi::WireFormatLite::TYPE_STRING>::ByteSizeLong(entry.first, entry.second);
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void CreateSnapshotRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<CreateSnapshotRequest*>(&to_msg);
  auto& from = static_cast<const CreateSnapshotRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.CreateSnapshotRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_name().empty()) {
        _this->_internal_set_name(from._internal_name());
      } else {
        if (_this->_impl_.name_.IsDefault()) {
          _this->_internal_set_name("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_subscription().empty()) {
        _this->_internal_set_subscription(from._internal_subscription());
      } else {
        if (_this->_impl_.subscription_.IsDefault()) {
          _this->_internal_set_subscription("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _this->_impl_.labels_.MergeFrom(from._impl_.labels_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void CreateSnapshotRequest::CopyFrom(const CreateSnapshotRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.CreateSnapshotRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void CreateSnapshotRequest::InternalSwap(CreateSnapshotRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.name_, &other->_impl_.name_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.subscription_, &other->_impl_.subscription_, arena);
  _impl_.labels_.InternalSwap(&other->_impl_.labels_);
}

::google::protobuf::Metadata CreateSnapshotRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void UpdateSnapshotRequest::clear_update_mask() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.update_mask_ != nullptr) _impl_.update_mask_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
UpdateSnapshotRequest::UpdateSnapshotRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UpdateSnapshotRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.UpdateSnapshotRequest)
}
PROTOBUF_NDEBUG_INLINE UpdateSnapshotRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::UpdateSnapshotRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

UpdateSnapshotRequest::UpdateSnapshotRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const UpdateSnapshotRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, UpdateSnapshotRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  UpdateSnapshotRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.snapshot_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.snapshot_)
                : nullptr;
  _impl_.update_mask_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.update_mask_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.UpdateSnapshotRequest)
}
PROTOBUF_NDEBUG_INLINE UpdateSnapshotRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void UpdateSnapshotRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, snapshot_),
           0,
           offsetof(Impl_, update_mask_) -
               offsetof(Impl_, snapshot_) +
               sizeof(Impl_::update_mask_));
}
UpdateSnapshotRequest::~UpdateSnapshotRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.UpdateSnapshotRequest)
  SharedDtor(*this);
}
inline void UpdateSnapshotRequest::SharedDtor(MessageLite& self) {
  UpdateSnapshotRequest& this_ = static_cast<UpdateSnapshotRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.snapshot_;
  delete this_._impl_.update_mask_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull UpdateSnapshotRequest_class_data_ =
        UpdateSnapshotRequest::InternalGenerateClassData_(UpdateSnapshotRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateSnapshotRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateSnapshotRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(UpdateSnapshotRequest_class_data_.tc_table);
  return UpdateSnapshotRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
UpdateSnapshotRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&UpdateSnapshotRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&UpdateSnapshotRequest_globals_));
  return UpdateSnapshotRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const UpdateSnapshotRequest::ParseTableT_
    UpdateSnapshotRequest::_table_ =
        UpdateSnapshotRequest::InternalGenerateParseTable_(UpdateSnapshotRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void UpdateSnapshotRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.UpdateSnapshotRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(_impl_.snapshot_ != nullptr);
      _impl_.snapshot_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.update_mask_ != nullptr);
      _impl_.update_mask_->Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL UpdateSnapshotRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const UpdateSnapshotRequest& this_ = static_cast<const UpdateSnapshotRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL UpdateSnapshotRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const UpdateSnapshotRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.UpdateSnapshotRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.pubsub.v1.Snapshot snapshot = 1 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.snapshot_, this_._impl_.snapshot_->GetCachedSize(), target,
        stream);
  }

  // .google.protobuf.FieldMask update_mask = 2 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.update_mask_, this_._impl_.update_mask_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.UpdateSnapshotRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t UpdateSnapshotRequest::ByteSizeLong(const MessageLite& base) {
  const UpdateSnapshotRequest& this_ = static_cast<const UpdateSnapshotRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t UpdateSnapshotRequest::ByteSizeLong() const {
  const UpdateSnapshotRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.UpdateSnapshotRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // .google.pubsub.v1.Snapshot snapshot = 1 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.snapshot_);
    }
    // .google.protobuf.FieldMask update_mask = 2 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.update_mask_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void UpdateSnapshotRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<UpdateSnapshotRequest*>(&to_msg);
  auto& from = static_cast<const UpdateSnapshotRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.UpdateSnapshotRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.snapshot_ != nullptr);
      if (_this->_impl_.snapshot_ == nullptr) {
        _this->_impl_.snapshot_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.snapshot_);
      } else {
        _this->_impl_.snapshot_->MergeFrom(*from._impl_.snapshot_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.update_mask_ != nullptr);
      if (_this->_impl_.update_mask_ == nullptr) {
        _this->_impl_.update_mask_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.update_mask_);
      } else {
        _this->_impl_.update_mask_->MergeFrom(*from._impl_.update_mask_);
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void UpdateSnapshotRequest::CopyFrom(const UpdateSnapshotRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.UpdateSnapshotRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void UpdateSnapshotRequest::InternalSwap(UpdateSnapshotRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(UpdateSnapshotRequest, _impl_.update_mask_)
      + sizeof(UpdateSnapshotRequest::_impl_.update_mask_)
      - PROTOBUF_FIELD_OFFSET(UpdateSnapshotRequest, _impl_.snapshot_)>(
          reinterpret_cast<char*>(&_impl_.snapshot_),
          reinterpret_cast<char*>(&other->_impl_.snapshot_));
}

::google::protobuf::Metadata UpdateSnapshotRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

#if defined(PROTOBUF_CUSTOM_VTABLE)
Snapshot_LabelsEntry_DoNotUse::Snapshot_LabelsEntry_DoNotUse() : SuperType(Snapshot_LabelsEntry_DoNotUse_get_class_data()) {}
Snapshot_LabelsEntry_DoNotUse::Snapshot_LabelsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
    : SuperType(arena, Snapshot_LabelsEntry_DoNotUse_get_class_data()) {}
#else   // PROTOBUF_CUSTOM_VTABLE
Snapshot_LabelsEntry_DoNotUse::Snapshot_LabelsEntry_DoNotUse() : SuperType() {}
Snapshot_LabelsEntry_DoNotUse::Snapshot_LabelsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena) : SuperType(arena) {}
#endif  // PROTOBUF_CUSTOM_VTABLE
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Snapshot_LabelsEntry_DoNotUse_class_data_ =
        Snapshot_LabelsEntry_DoNotUse::InternalGenerateClassData_(Snapshot_LabelsEntry_DoNotUse_globals_._default);

// including ~ all the vtables and everything they use become part
// of the same ~ SCC.
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Snapshot_LabelsEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Snapshot_LabelsEntry_DoNotUse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Snapshot_LabelsEntry_DoNotUse_class_data_.tc_table);
  return Snapshot_LabelsEntry_DoNotUse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Snapshot_LabelsEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Snapshot_LabelsEntry_DoNotUse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Snapshot_LabelsEntry_DoNotUse_globals_));
  return Snapshot_LabelsEntry_DoNotUse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Snapshot_LabelsEntry_DoNotUse::ParseTableT_
    Snapshot_LabelsEntry_DoNotUse::_table_ =
        Snapshot_LabelsEntry_DoNotUse::InternalGenerateParseTable_(Snapshot_LabelsEntry_DoNotUse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
// ===================================================================

void Snapshot::clear_expire_time() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.expire_time_ != nullptr) _impl_.expire_time_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
Snapshot::Snapshot(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Snapshot_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.Snapshot)
}
PROTOBUF_NDEBUG_INLINE Snapshot::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::Snapshot& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        name_(arena, from.name_),
        topic_(arena, from.topic_),
        labels_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::Snapshot,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Snapshot, _impl_.labels_)>()
          , from.labels_
        }
     {}

Snapshot::Snapshot(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Snapshot& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Snapshot_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Snapshot* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.expire_time_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.expire_time_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.Snapshot)
}
PROTOBUF_NDEBUG_INLINE Snapshot::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        name_(arena),
        topic_(arena),
        labels_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::Snapshot,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::Snapshot, _impl_.labels_)>()
         }
     {}

inline void Snapshot::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.expire_time_ = {};
}
Snapshot::~Snapshot() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.Snapshot)
  SharedDtor(*this);
}
inline void Snapshot::SharedDtor(MessageLite& self) {
  Snapshot& this_ = static_cast<Snapshot&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.name_.Destroy();
  this_._impl_.topic_.Destroy();
  delete this_._impl_.expire_time_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Snapshot_class_data_ =
        Snapshot::InternalGenerateClassData_(Snapshot_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Snapshot::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Snapshot_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Snapshot_class_data_.tc_table);
  return Snapshot_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Snapshot::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Snapshot_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Snapshot_globals_));
  return Snapshot_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Snapshot::ParseTableT_
    Snapshot::_table_ =
        Snapshot::InternalGenerateParseTable_(Snapshot_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Snapshot::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.Snapshot)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.name_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.topic_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(_impl_.expire_time_ != nullptr);
      _impl_.expire_time_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _impl_.labels_.Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Snapshot::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Snapshot& this_ = static_cast<const Snapshot&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Snapshot::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Snapshot& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.Snapshot)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string name = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_name().empty()) {
      const ::std::string& _s = this_._internal_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Snapshot.name");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // string topic = 2 [(.google.api.field_behavior) = OPTIONAL, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_topic().empty()) {
      const ::std::string& _s = this_._internal_topic();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Snapshot.topic");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  // .google.protobuf.Timestamp expire_time = 3 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        3, *this_._impl_.expire_time_, this_._impl_.expire_time_->GetCachedSize(), target,
        stream);
  }

  // map<string, string> labels = 4 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (!this_._internal_labels().empty()) {
      using MapType = ::google::protobuf::Map<::std::string, ::std::string>;
      using WireHelper = _pbi::MapEntryFuncs<::std::string, ::std::string,
                                     _pbi::WireFormatLite::TYPE_STRING,
                                     _pbi::WireFormatLite::TYPE_STRING>;
      const auto& field = this_._internal_labels();

      if (stream->IsSerializationDeterministic() && field.size() > 1) {
        for (const auto& entry : ::google::protobuf::internal::MapSorterPtr<MapType>(field)) {
          target = WireHelper::InternalSerialize(
              4, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Snapshot.labels");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Snapshot.labels");
        }
      } else {
        for (const auto& entry : field) {
          target = WireHelper::InternalSerialize(
              4, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Snapshot.labels");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.Snapshot.labels");
        }
      }
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.Snapshot)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Snapshot::ByteSizeLong(const MessageLite& base) {
  const Snapshot& this_ = static_cast<const Snapshot&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Snapshot::ByteSizeLong() const {
  const Snapshot& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.Snapshot)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // string name = 1 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_name());
      }
    }
    // string topic = 2 [(.google.api.field_behavior) = OPTIONAL, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_topic().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_topic());
      }
    }
    // .google.protobuf.Timestamp expire_time = 3 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.expire_time_);
    }
    // map<string, string> labels = 4 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_labels_size());
      for (const auto& entry : this_._internal_labels()) {
        total_size += _pbi::MapEntryFuncs<::std::string, ::std::string,
                                       _pbi::WireFormatLite::TYPE_STRING,
                                       _pbi::WireFormatLite::TYPE_STRING>::ByteSizeLong(entry.first, entry.second);
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Snapshot::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Snapshot*>(&to_msg);
  auto& from = static_cast<const Snapshot&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.Snapshot)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_name().empty()) {
        _this->_internal_set_name(from._internal_name());
      } else {
        if (_this->_impl_.name_.IsDefault()) {
          _this->_internal_set_name("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_topic().empty()) {
        _this->_internal_set_topic(from._internal_topic());
      } else {
        if (_this->_impl_.topic_.IsDefault()) {
          _this->_internal_set_topic("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.expire_time_ != nullptr);
      if (_this->_impl_.expire_time_ == nullptr) {
        _this->_impl_.expire_time_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.expire_time_);
      } else {
        _this->_impl_.expire_time_->MergeFrom(*from._impl_.expire_time_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _this->_impl_.labels_.MergeFrom(from._impl_.labels_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Snapshot::CopyFrom(const Snapshot& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.Snapshot)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Snapshot::InternalSwap(Snapshot* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.name_, &other->_impl_.name_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.topic_, &other->_impl_.topic_, arena);
  swap(_impl_.expire_time_, other->_impl_.expire_time_);
  _impl_.labels_.InternalSwap(&other->_impl_.labels_);
}

::google::protobuf::Metadata Snapshot::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

GetSnapshotRequest::GetSnapshotRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, GetSnapshotRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.GetSnapshotRequest)
}
PROTOBUF_NDEBUG_INLINE GetSnapshotRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::GetSnapshotRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        snapshot_(arena, from.snapshot_) {}

GetSnapshotRequest::GetSnapshotRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const GetSnapshotRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, GetSnapshotRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  GetSnapshotRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.GetSnapshotRequest)
}
PROTOBUF_NDEBUG_INLINE GetSnapshotRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        snapshot_(arena) {}

inline void GetSnapshotRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
GetSnapshotRequest::~GetSnapshotRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.GetSnapshotRequest)
  SharedDtor(*this);
}
inline void GetSnapshotRequest::SharedDtor(MessageLite& self) {
  GetSnapshotRequest& this_ = static_cast<GetSnapshotRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.snapshot_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull GetSnapshotRequest_class_data_ =
        GetSnapshotRequest::InternalGenerateClassData_(GetSnapshotRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
GetSnapshotRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&GetSnapshotRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(GetSnapshotRequest_class_data_.tc_table);
  return GetSnapshotRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
GetSnapshotRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&GetSnapshotRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&GetSnapshotRequest_globals_));
  return GetSnapshotRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const GetSnapshotRequest::ParseTableT_
    GetSnapshotRequest::_table_ =
        GetSnapshotRequest::InternalGenerateParseTable_(GetSnapshotRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void GetSnapshotRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.GetSnapshotRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.snapshot_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL GetSnapshotRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const GetSnapshotRequest& this_ = static_cast<const GetSnapshotRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL GetSnapshotRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const GetSnapshotRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.GetSnapshotRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string snapshot = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_snapshot().empty()) {
      const ::std::string& _s = this_._internal_snapshot();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.GetSnapshotRequest.snapshot");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.GetSnapshotRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t GetSnapshotRequest::ByteSizeLong(const MessageLite& base) {
  const GetSnapshotRequest& this_ = static_cast<const GetSnapshotRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t GetSnapshotRequest::ByteSizeLong() const {
  const GetSnapshotRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.GetSnapshotRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string snapshot = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_snapshot().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_snapshot());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void GetSnapshotRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<GetSnapshotRequest*>(&to_msg);
  auto& from = static_cast<const GetSnapshotRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.GetSnapshotRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_snapshot().empty()) {
      _this->_internal_set_snapshot(from._internal_snapshot());
    } else {
      if (_this->_impl_.snapshot_.IsDefault()) {
        _this->_internal_set_snapshot("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void GetSnapshotRequest::CopyFrom(const GetSnapshotRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.GetSnapshotRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void GetSnapshotRequest::InternalSwap(GetSnapshotRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.snapshot_, &other->_impl_.snapshot_, arena);
}

::google::protobuf::Metadata GetSnapshotRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ListSnapshotsRequest::ListSnapshotsRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListSnapshotsRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ListSnapshotsRequest)
}
PROTOBUF_NDEBUG_INLINE ListSnapshotsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::ListSnapshotsRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        project_(arena, from.project_),
        page_token_(arena, from.page_token_) {}

ListSnapshotsRequest::ListSnapshotsRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ListSnapshotsRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListSnapshotsRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ListSnapshotsRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.page_size_ = from._impl_.page_size_;

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ListSnapshotsRequest)
}
PROTOBUF_NDEBUG_INLINE ListSnapshotsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        project_(arena),
        page_token_(arena) {}

inline void ListSnapshotsRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.page_size_ = {};
}
ListSnapshotsRequest::~ListSnapshotsRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.ListSnapshotsRequest)
  SharedDtor(*this);
}
inline void ListSnapshotsRequest::SharedDtor(MessageLite& self) {
  ListSnapshotsRequest& this_ = static_cast<ListSnapshotsRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.project_.Destroy();
  this_._impl_.page_token_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ListSnapshotsRequest_class_data_ =
        ListSnapshotsRequest::InternalGenerateClassData_(ListSnapshotsRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListSnapshotsRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListSnapshotsRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ListSnapshotsRequest_class_data_.tc_table);
  return ListSnapshotsRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListSnapshotsRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListSnapshotsRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ListSnapshotsRequest_globals_));
  return ListSnapshotsRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ListSnapshotsRequest::ParseTableT_
    ListSnapshotsRequest::_table_ =
        ListSnapshotsRequest::InternalGenerateParseTable_(ListSnapshotsRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ListSnapshotsRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.ListSnapshotsRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.project_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.page_token_.ClearNonDefaultToEmpty();
    }
  }
  _impl_.page_size_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ListSnapshotsRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ListSnapshotsRequest& this_ = static_cast<const ListSnapshotsRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ListSnapshotsRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ListSnapshotsRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.ListSnapshotsRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string project = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_project().empty()) {
      const ::std::string& _s = this_._internal_project();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListSnapshotsRequest.project");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // int32 page_size = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_page_size() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<2>(
              stream, this_._internal_page_size(), target);
    }
  }

  // string page_token = 3 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_page_token().empty()) {
      const ::std::string& _s = this_._internal_page_token();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListSnapshotsRequest.page_token");
      target = stream->WriteStringMaybeAliased(3, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.ListSnapshotsRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ListSnapshotsRequest::ByteSizeLong(const MessageLite& base) {
  const ListSnapshotsRequest& this_ = static_cast<const ListSnapshotsRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ListSnapshotsRequest::ByteSizeLong() const {
  const ListSnapshotsRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.ListSnapshotsRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // string project = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_project().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_project());
      }
    }
    // string page_token = 3 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_page_token().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_page_token());
      }
    }
    // int32 page_size = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_page_size() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_page_size());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ListSnapshotsRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ListSnapshotsRequest*>(&to_msg);
  auto& from = static_cast<const ListSnapshotsRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.ListSnapshotsRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_project().empty()) {
        _this->_internal_set_project(from._internal_project());
      } else {
        if (_this->_impl_.project_.IsDefault()) {
          _this->_internal_set_project("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_page_token().empty()) {
        _this->_internal_set_page_token(from._internal_page_token());
      } else {
        if (_this->_impl_.page_token_.IsDefault()) {
          _this->_internal_set_page_token("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_page_size() != 0) {
        _this->_impl_.page_size_ = from._impl_.page_size_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ListSnapshotsRequest::CopyFrom(const ListSnapshotsRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.ListSnapshotsRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ListSnapshotsRequest::InternalSwap(ListSnapshotsRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.project_, &other->_impl_.project_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.page_token_, &other->_impl_.page_token_, arena);
  swap(_impl_.page_size_, other->_impl_.page_size_);
}

::google::protobuf::Metadata ListSnapshotsRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ListSnapshotsResponse::ListSnapshotsResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListSnapshotsResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.ListSnapshotsResponse)
}
PROTOBUF_NDEBUG_INLINE ListSnapshotsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::ListSnapshotsResponse& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        snapshots_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::pubsub::v1::ListSnapshotsResponse,
              PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSnapshotsResponse, _impl_.snapshots_)>()
          , from.snapshots_
        }
        ,
        next_page_token_(arena, from.next_page_token_) {}

ListSnapshotsResponse::ListSnapshotsResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ListSnapshotsResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ListSnapshotsResponse_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ListSnapshotsResponse* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.ListSnapshotsResponse)
}
PROTOBUF_NDEBUG_INLINE ListSnapshotsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        snapshots_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::pubsub::v1::ListSnapshotsResponse,
            PROTOBUF_FIELD_OFFSET(::google::pubsub::v1::ListSnapshotsResponse, _impl_.snapshots_)>()
         }
        ,
        next_page_token_(arena) {}

inline void ListSnapshotsResponse::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
ListSnapshotsResponse::~ListSnapshotsResponse() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.ListSnapshotsResponse)
  SharedDtor(*this);
}
inline void ListSnapshotsResponse::SharedDtor(MessageLite& self) {
  ListSnapshotsResponse& this_ = static_cast<ListSnapshotsResponse&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.next_page_token_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ListSnapshotsResponse_class_data_ =
        ListSnapshotsResponse::InternalGenerateClassData_(ListSnapshotsResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListSnapshotsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListSnapshotsResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ListSnapshotsResponse_class_data_.tc_table);
  return ListSnapshotsResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ListSnapshotsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ListSnapshotsResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ListSnapshotsResponse_globals_));
  return ListSnapshotsResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ListSnapshotsResponse::ParseTableT_
    ListSnapshotsResponse::_table_ =
        ListSnapshotsResponse::InternalGenerateParseTable_(ListSnapshotsResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ListSnapshotsResponse::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.ListSnapshotsResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.snapshots_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.next_page_token_.ClearNonDefaultToEmpty();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ListSnapshotsResponse::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ListSnapshotsResponse& this_ = static_cast<const ListSnapshotsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ListSnapshotsResponse::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ListSnapshotsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.ListSnapshotsResponse)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated .google.pubsub.v1.Snapshot snapshots = 1 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_snapshots_size());
         i < n; i++) {
      const auto& repfield = this_._internal_snapshots().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              1, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  // string next_page_token = 2 [(.google.api.field_behavior) = OPTIONAL];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_next_page_token().empty()) {
      const ::std::string& _s = this_._internal_next_page_token();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.ListSnapshotsResponse.next_page_token");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.ListSnapshotsResponse)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ListSnapshotsResponse::ByteSizeLong(const MessageLite& base) {
  const ListSnapshotsResponse& this_ = static_cast<const ListSnapshotsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ListSnapshotsResponse::ByteSizeLong() const {
  const ListSnapshotsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.ListSnapshotsResponse)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // repeated .google.pubsub.v1.Snapshot snapshots = 1 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_snapshots_size();
      for (const auto& msg : this_._internal_snapshots()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
    // string next_page_token = 2 [(.google.api.field_behavior) = OPTIONAL];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_next_page_token().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_next_page_token());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ListSnapshotsResponse::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ListSnapshotsResponse*>(&to_msg);
  auto& from = static_cast<const ListSnapshotsResponse&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.ListSnapshotsResponse)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_snapshots()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_snapshots());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_next_page_token().empty()) {
        _this->_internal_set_next_page_token(from._internal_next_page_token());
      } else {
        if (_this->_impl_.next_page_token_.IsDefault()) {
          _this->_internal_set_next_page_token("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ListSnapshotsResponse::CopyFrom(const ListSnapshotsResponse& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.ListSnapshotsResponse)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ListSnapshotsResponse::InternalSwap(ListSnapshotsResponse* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.snapshots_.InternalSwap(&other->_impl_.snapshots_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.next_page_token_, &other->_impl_.next_page_token_, arena);
}

::google::protobuf::Metadata ListSnapshotsResponse::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

DeleteSnapshotRequest::DeleteSnapshotRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DeleteSnapshotRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.DeleteSnapshotRequest)
}
PROTOBUF_NDEBUG_INLINE DeleteSnapshotRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::DeleteSnapshotRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        snapshot_(arena, from.snapshot_) {}

DeleteSnapshotRequest::DeleteSnapshotRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const DeleteSnapshotRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DeleteSnapshotRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  DeleteSnapshotRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.DeleteSnapshotRequest)
}
PROTOBUF_NDEBUG_INLINE DeleteSnapshotRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        snapshot_(arena) {}

inline void DeleteSnapshotRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
DeleteSnapshotRequest::~DeleteSnapshotRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.DeleteSnapshotRequest)
  SharedDtor(*this);
}
inline void DeleteSnapshotRequest::SharedDtor(MessageLite& self) {
  DeleteSnapshotRequest& this_ = static_cast<DeleteSnapshotRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.snapshot_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull DeleteSnapshotRequest_class_data_ =
        DeleteSnapshotRequest::InternalGenerateClassData_(DeleteSnapshotRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DeleteSnapshotRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DeleteSnapshotRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(DeleteSnapshotRequest_class_data_.tc_table);
  return DeleteSnapshotRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DeleteSnapshotRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DeleteSnapshotRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&DeleteSnapshotRequest_globals_));
  return DeleteSnapshotRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const DeleteSnapshotRequest::ParseTableT_
    DeleteSnapshotRequest::_table_ =
        DeleteSnapshotRequest::InternalGenerateParseTable_(DeleteSnapshotRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void DeleteSnapshotRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.DeleteSnapshotRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.snapshot_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL DeleteSnapshotRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const DeleteSnapshotRequest& this_ = static_cast<const DeleteSnapshotRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL DeleteSnapshotRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const DeleteSnapshotRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.DeleteSnapshotRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string snapshot = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_snapshot().empty()) {
      const ::std::string& _s = this_._internal_snapshot();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.DeleteSnapshotRequest.snapshot");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.DeleteSnapshotRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t DeleteSnapshotRequest::ByteSizeLong(const MessageLite& base) {
  const DeleteSnapshotRequest& this_ = static_cast<const DeleteSnapshotRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t DeleteSnapshotRequest::ByteSizeLong() const {
  const DeleteSnapshotRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.DeleteSnapshotRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string snapshot = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_snapshot().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_snapshot());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void DeleteSnapshotRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<DeleteSnapshotRequest*>(&to_msg);
  auto& from = static_cast<const DeleteSnapshotRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.DeleteSnapshotRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_snapshot().empty()) {
      _this->_internal_set_snapshot(from._internal_snapshot());
    } else {
      if (_this->_impl_.snapshot_.IsDefault()) {
        _this->_internal_set_snapshot("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void DeleteSnapshotRequest::CopyFrom(const DeleteSnapshotRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.DeleteSnapshotRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void DeleteSnapshotRequest::InternalSwap(DeleteSnapshotRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.snapshot_, &other->_impl_.snapshot_, arena);
}

::google::protobuf::Metadata DeleteSnapshotRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void SeekRequest::set_allocated_time(::google::protobuf::Timestamp* PROTOBUF_NULLABLE time) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_target();
  if (time) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(time)->GetArena();
    if (message_arena != submessage_arena) {
      time = ::google::protobuf::internal::GetOwnedMessage(message_arena, time, submessage_arena);
    }
    set_has_time();
    _impl_.target_.time_ = time;
  }
  // @@protoc_insertion_point(field_set_allocated:google.pubsub.v1.SeekRequest.time)
}
void SeekRequest::clear_time() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (target_case() == kTime) {
    if (GetArena() == nullptr) {
      delete _impl_.target_.time_;
    } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
      ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.target_.time_);
    }
    clear_has_target();
  }
}
SeekRequest::SeekRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, SeekRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.SeekRequest)
}
PROTOBUF_NDEBUG_INLINE SeekRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::pubsub::v1::SeekRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        subscription_(arena, from.subscription_),
        target_{},
        _oneof_case_{from._oneof_case_[0]} {}

SeekRequest::SeekRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const SeekRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, SeekRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SeekRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  switch (target_case()) {
    case TARGET_NOT_SET:
      break;
      case kTime:
        _impl_.target_.time_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.target_.time_);
        break;
      case kSnapshot:
        new (&_impl_.target_.snapshot_) decltype(_impl_.target_.snapshot_){arena, from._impl_.target_.snapshot_};
        break;
  }

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.SeekRequest)
}
PROTOBUF_NDEBUG_INLINE SeekRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        subscription_(arena),
        target_{},
        _oneof_case_{} {}

inline void SeekRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
SeekRequest::~SeekRequest() {
  // @@protoc_insertion_point(destructor:google.pubsub.v1.SeekRequest)
  SharedDtor(*this);
}
inline void SeekRequest::SharedDtor(MessageLite& self) {
  SeekRequest& this_ = static_cast<SeekRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.subscription_.Destroy();
  if (this_.has_target()) {
    this_.clear_target();
  }
  this_._impl_.~Impl_();
}

void SeekRequest::clear_target() {
// @@protoc_insertion_point(one_of_clear_start:google.pubsub.v1.SeekRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (target_case()) {
    case kTime: {
      if (GetArena() == nullptr) {
        delete _impl_.target_.time_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.target_.time_);
      }
      break;
    }
    case kSnapshot: {
      _impl_.target_.snapshot_.Destroy();
      break;
    }
    case TARGET_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = TARGET_NOT_SET;
}


#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull SeekRequest_class_data_ =
        SeekRequest::InternalGenerateClassData_(SeekRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
SeekRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&SeekRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(SeekRequest_class_data_.tc_table);
  return SeekRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
SeekRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&SeekRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&SeekRequest_globals_));
  return SeekRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const SeekRequest::ParseTableT_
    SeekRequest::_table_ =
        SeekRequest::InternalGenerateParseTable_(SeekRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void SeekRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.pubsub.v1.SeekRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.subscription_.ClearNonDefaultToEmpty();
  }
  clear_target();
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL SeekRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const SeekRequest& this_ = static_cast<const SeekRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL SeekRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const SeekRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.pubsub.v1.SeekRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_subscription().empty()) {
      const ::std::string& _s = this_._internal_subscription();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.SeekRequest.subscription");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  switch (this_.target_case()) {
    case kTime: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          2, *this_._impl_.target_.time_, this_._impl_.target_.time_->GetCachedSize(), target,
          stream);
      break;
    }
    case kSnapshot: {
      const ::std::string& _s = this_._internal_snapshot();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.pubsub.v1.SeekRequest.snapshot");
      target = stream->WriteStringMaybeAliased(3, _s, target);
      break;
    }
    default:
      break;
  }
  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.pubsub.v1.SeekRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t SeekRequest::ByteSizeLong(const MessageLite& base) {
  const SeekRequest& this_ = static_cast<const SeekRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t SeekRequest::ByteSizeLong() const {
  const SeekRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.pubsub.v1.SeekRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string subscription = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_subscription().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_subscription());
      }
    }
  }
  switch (this_.target_case()) {
    // .google.protobuf.Timestamp time = 2 [(.google.api.field_behavior) = OPTIONAL];
    case kTime: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.target_.time_);
      break;
    }
    // string snapshot = 3 [(.google.api.field_behavior) = OPTIONAL, (.google.api.resource_reference) = {
    case kSnapshot: {
      total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                      this_._internal_snapshot());
      break;
    }
    case TARGET_NOT_SET: {
      break;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void SeekRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<SeekRequest*>(&to_msg);
  auto& from = static_cast<const SeekRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.pubsub.v1.SeekRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_subscription().empty()) {
      _this->_internal_set_subscription(from._internal_subscription());
    } else {
      if (_this->_impl_.subscription_.IsDefault()) {
        _this->_internal_set_subscription("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[0]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[0];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_target();
      }
      _this->_impl_._oneof_case_[0] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kTime: {
        if (oneof_needs_init) {
          _this->_impl_.target_.time_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.target_.time_);
        } else {
          _this->_impl_.target_.time_->MergeFrom(*from._impl_.target_.time_);
        }
        break;
      }
      case kSnapshot: {
        if (oneof_needs_init) {
          _this->_impl_.target_.snapshot_.InitDefault();
        }
        _this->_impl_.target_.snapshot_.Set(from._internal_snapshot(), arena);
        break;
      }
      case TARGET_NOT_SET:
        break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void SeekRequest::CopyFrom(const SeekRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.pubsub.v1.SeekRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void SeekRequest::InternalSwap(SeekRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.subscription_, &other->_impl_.subscription_, arena);
  swap(_impl_.target_, other->_impl_.target_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
}

::google::protobuf::Metadata SeekRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

SeekResponse::SeekResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, SeekResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(arena_constructor:google.pubsub.v1.SeekResponse)
}
SeekResponse::SeekResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const SeekResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::internal::ZeroFieldsBase(arena, SeekResponse_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::internal::ZeroFieldsBase(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SeekResponse* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);

  // @@protoc_insertion_point(copy_constructor:google.pubsub.v1.SeekResponse)
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull SeekResponse_class_data_ =
        SeekResponse::InternalGenerateClassData_(SeekResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
SeekResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&SeekResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(SeekResponse_class_data_.tc_table);
  return SeekResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
SeekResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&SeekResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&SeekResponse_globals_));
  return SeekResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const SeekResponse::ParseTableT_
    SeekResponse::_table_ =
        SeekResponse::InternalGenerateParseTable_(SeekResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS







::google::protobuf::Metadata SeekResponse::GetMetadata() const {
  return ::google::protobuf::internal::ZeroFieldsBase::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
}  // namespace v1
}  // namespace pubsub
}  // namespace google
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_google_2fpubsub_2fv1_2fpubsub_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
