// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: google/api/client.proto
// Protobuf C++ Version: 7.35.1

#include "google/api/client.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
#ifdef PROTOBUF_MESSAGE_GLOBALS
namespace {
PROTOBUF_CONSTINIT ::google::protobuf::internal::ReflectionData
    file_reflection_data[] = {
        // ::google::api::CommonLanguageSettings
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fapi_2fclient_2eproto, /* tracker*/ nullptr,},
        // ::google::api::ClientLibrarySettings
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fapi_2fclient_2eproto, /* tracker*/ nullptr,},
        // ::google::api::Publishing
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fapi_2fclient_2eproto, /* tracker*/ nullptr,},
        // ::google::api::JavaSettings_ServiceClassNamesEntry_DoNotUse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fapi_2fclient_2eproto, /* tracker*/ nullptr,},
        // ::google::api::JavaSettings
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fapi_2fclient_2eproto, /* tracker*/ nullptr,},
        // ::google::api::CppSettings
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fapi_2fclient_2eproto, /* tracker*/ nullptr,},
        // ::google::api::PhpSettings
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fapi_2fclient_2eproto, /* tracker*/ nullptr,},
        // ::google::api::PythonSettings
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fapi_2fclient_2eproto, /* tracker*/ nullptr,},
        // ::google::api::NodeSettings
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fapi_2fclient_2eproto, /* tracker*/ nullptr,},
        // ::google::api::DotnetSettings_RenamedServicesEntry_DoNotUse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fapi_2fclient_2eproto, /* tracker*/ nullptr,},
        // ::google::api::DotnetSettings_RenamedResourcesEntry_DoNotUse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fapi_2fclient_2eproto, /* tracker*/ nullptr,},
        // ::google::api::DotnetSettings
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fapi_2fclient_2eproto, /* tracker*/ nullptr,},
        // ::google::api::RubySettings
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fapi_2fclient_2eproto, /* tracker*/ nullptr,},
        // ::google::api::GoSettings
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fapi_2fclient_2eproto, /* tracker*/ nullptr,},
        // ::google::api::MethodSettings_LongRunning
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fapi_2fclient_2eproto, /* tracker*/ nullptr,},
        // ::google::api::MethodSettings
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fapi_2fclient_2eproto, /* tracker*/ nullptr,},
};
}  // namespace
#endif
namespace google {
namespace api {
constexpr JavaSettings_ServiceClassNamesEntry_DoNotUse::ParseTableT_ JavaSettings_ServiceClassNamesEntry_DoNotUse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(JavaSettings_ServiceClassNamesEntry_DoNotUse, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::DiscardEverythingFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::api::JavaSettings_ServiceClassNamesEntry_DoNotUse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string value = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(JavaSettings_ServiceClassNamesEntry_DoNotUse, _impl_.value_)}},
      // string key = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(JavaSettings_ServiceClassNamesEntry_DoNotUse, _impl_.key_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string key = 1;
      {PROTOBUF_FIELD_OFFSET(JavaSettings_ServiceClassNamesEntry_DoNotUse, _impl_.key_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string value = 2;
      {PROTOBUF_FIELD_OFFSET(JavaSettings_ServiceClassNamesEntry_DoNotUse, _impl_.value_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\56\3\5\0\0\0\0\0"
      "google.api.JavaSettings.ServiceClassNamesEntry"
      "key"
      "value"
    }},
  };
}

template <typename>
constexpr JavaSettings_ServiceClassNamesEntry_DoNotUse::JavaSettings_ServiceClassNamesEntry_DoNotUse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : JavaSettings_ServiceClassNamesEntry_DoNotUse::MapEntry(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL JavaSettings_ServiceClassNamesEntry_DoNotUse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) JavaSettings_ServiceClassNamesEntry_DoNotUse(arena);
}
constexpr auto JavaSettings_ServiceClassNamesEntry_DoNotUse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(JavaSettings_ServiceClassNamesEntry_DoNotUse), alignof(JavaSettings_ServiceClassNamesEntry_DoNotUse));
}
constexpr auto JavaSettings_ServiceClassNamesEntry_DoNotUse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &JavaSettings_ServiceClassNamesEntry_DoNotUse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<JavaSettings_ServiceClassNamesEntry_DoNotUse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &JavaSettings_ServiceClassNamesEntry_DoNotUse::SharedDtor,
          static_cast<void (::google::protobuf::MessageLite::*)()>(&JavaSettings_ServiceClassNamesEntry_DoNotUse::ClearImpl),
              ::google::protobuf::Message::ByteSizeLongImpl, ::google::protobuf::Message::_InternalSerializeImpl
              ,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(JavaSettings_ServiceClassNamesEntry_DoNotUse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[3],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fapi_2fclient_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct JavaSettings_ServiceClassNamesEntry_DoNotUseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr JavaSettings_ServiceClassNamesEntry_DoNotUseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 JavaSettings_ServiceClassNamesEntry_DoNotUse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(JavaSettings_ServiceClassNamesEntry_DoNotUse::InternalGenerateClassData_(
            _default, &JavaSettings_ServiceClassNamesEntry_DoNotUse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<JavaSettings_ServiceClassNamesEntry_DoNotUse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~JavaSettings_ServiceClassNamesEntry_DoNotUseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) JavaSettings_ServiceClassNamesEntry_DoNotUse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<JavaSettings_ServiceClassNamesEntry_DoNotUse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(JavaSettings_ServiceClassNamesEntry_DoNotUseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST JavaSettings_ServiceClassNamesEntry_DoNotUseGlobalsTypeInternal JavaSettings_ServiceClassNamesEntry_DoNotUse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* JavaSettings_ServiceClassNamesEntry_DoNotUse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return JavaSettings_ServiceClassNamesEntry_DoNotUse_globals_.GetClassData();
#else
  return JavaSettings_ServiceClassNamesEntry_DoNotUse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
constexpr DotnetSettings_RenamedServicesEntry_DoNotUse::ParseTableT_ DotnetSettings_RenamedServicesEntry_DoNotUse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(DotnetSettings_RenamedServicesEntry_DoNotUse, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::DiscardEverythingFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::api::DotnetSettings_RenamedServicesEntry_DoNotUse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string value = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(DotnetSettings_RenamedServicesEntry_DoNotUse, _impl_.value_)}},
      // string key = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(DotnetSettings_RenamedServicesEntry_DoNotUse, _impl_.key_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string key = 1;
      {PROTOBUF_FIELD_OFFSET(DotnetSettings_RenamedServicesEntry_DoNotUse, _impl_.key_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string value = 2;
      {PROTOBUF_FIELD_OFFSET(DotnetSettings_RenamedServicesEntry_DoNotUse, _impl_.value_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\56\3\5\0\0\0\0\0"
      "google.api.DotnetSettings.RenamedServicesEntry"
      "key"
      "value"
    }},
  };
}

template <typename>
constexpr DotnetSettings_RenamedServicesEntry_DoNotUse::DotnetSettings_RenamedServicesEntry_DoNotUse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : DotnetSettings_RenamedServicesEntry_DoNotUse::MapEntry(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL DotnetSettings_RenamedServicesEntry_DoNotUse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) DotnetSettings_RenamedServicesEntry_DoNotUse(arena);
}
constexpr auto DotnetSettings_RenamedServicesEntry_DoNotUse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(DotnetSettings_RenamedServicesEntry_DoNotUse), alignof(DotnetSettings_RenamedServicesEntry_DoNotUse));
}
constexpr auto DotnetSettings_RenamedServicesEntry_DoNotUse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &DotnetSettings_RenamedServicesEntry_DoNotUse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<DotnetSettings_RenamedServicesEntry_DoNotUse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &DotnetSettings_RenamedServicesEntry_DoNotUse::SharedDtor,
          static_cast<void (::google::protobuf::MessageLite::*)()>(&DotnetSettings_RenamedServicesEntry_DoNotUse::ClearImpl),
              ::google::protobuf::Message::ByteSizeLongImpl, ::google::protobuf::Message::_InternalSerializeImpl
              ,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(DotnetSettings_RenamedServicesEntry_DoNotUse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[9],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fapi_2fclient_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct DotnetSettings_RenamedServicesEntry_DoNotUseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr DotnetSettings_RenamedServicesEntry_DoNotUseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 DotnetSettings_RenamedServicesEntry_DoNotUse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(DotnetSettings_RenamedServicesEntry_DoNotUse::InternalGenerateClassData_(
            _default, &DotnetSettings_RenamedServicesEntry_DoNotUse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<DotnetSettings_RenamedServicesEntry_DoNotUse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~DotnetSettings_RenamedServicesEntry_DoNotUseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) DotnetSettings_RenamedServicesEntry_DoNotUse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<DotnetSettings_RenamedServicesEntry_DoNotUse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(DotnetSettings_RenamedServicesEntry_DoNotUseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST DotnetSettings_RenamedServicesEntry_DoNotUseGlobalsTypeInternal DotnetSettings_RenamedServicesEntry_DoNotUse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* DotnetSettings_RenamedServicesEntry_DoNotUse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return DotnetSettings_RenamedServicesEntry_DoNotUse_globals_.GetClassData();
#else
  return DotnetSettings_RenamedServicesEntry_DoNotUse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
constexpr DotnetSettings_RenamedResourcesEntry_DoNotUse::ParseTableT_ DotnetSettings_RenamedResourcesEntry_DoNotUse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(DotnetSettings_RenamedResourcesEntry_DoNotUse, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::DiscardEverythingFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::api::DotnetSettings_RenamedResourcesEntry_DoNotUse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string value = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(DotnetSettings_RenamedResourcesEntry_DoNotUse, _impl_.value_)}},
      // string key = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(DotnetSettings_RenamedResourcesEntry_DoNotUse, _impl_.key_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string key = 1;
      {PROTOBUF_FIELD_OFFSET(DotnetSettings_RenamedResourcesEntry_DoNotUse, _impl_.key_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string value = 2;
      {PROTOBUF_FIELD_OFFSET(DotnetSettings_RenamedResourcesEntry_DoNotUse, _impl_.value_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\57\3\5\0\0\0\0\0"
      "google.api.DotnetSettings.RenamedResourcesEntry"
      "key"
      "value"
    }},
  };
}

template <typename>
constexpr DotnetSettings_RenamedResourcesEntry_DoNotUse::DotnetSettings_RenamedResourcesEntry_DoNotUse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : DotnetSettings_RenamedResourcesEntry_DoNotUse::MapEntry(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL DotnetSettings_RenamedResourcesEntry_DoNotUse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) DotnetSettings_RenamedResourcesEntry_DoNotUse(arena);
}
constexpr auto DotnetSettings_RenamedResourcesEntry_DoNotUse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(DotnetSettings_RenamedResourcesEntry_DoNotUse), alignof(DotnetSettings_RenamedResourcesEntry_DoNotUse));
}
constexpr auto DotnetSettings_RenamedResourcesEntry_DoNotUse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &DotnetSettings_RenamedResourcesEntry_DoNotUse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<DotnetSettings_RenamedResourcesEntry_DoNotUse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &DotnetSettings_RenamedResourcesEntry_DoNotUse::SharedDtor,
          static_cast<void (::google::protobuf::MessageLite::*)()>(&DotnetSettings_RenamedResourcesEntry_DoNotUse::ClearImpl),
              ::google::protobuf::Message::ByteSizeLongImpl, ::google::protobuf::Message::_InternalSerializeImpl
              ,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(DotnetSettings_RenamedResourcesEntry_DoNotUse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[10],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fapi_2fclient_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct DotnetSettings_RenamedResourcesEntry_DoNotUseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr DotnetSettings_RenamedResourcesEntry_DoNotUseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 DotnetSettings_RenamedResourcesEntry_DoNotUse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(DotnetSettings_RenamedResourcesEntry_DoNotUse::InternalGenerateClassData_(
            _default, &DotnetSettings_RenamedResourcesEntry_DoNotUse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<DotnetSettings_RenamedResourcesEntry_DoNotUse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~DotnetSettings_RenamedResourcesEntry_DoNotUseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) DotnetSettings_RenamedResourcesEntry_DoNotUse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<DotnetSettings_RenamedResourcesEntry_DoNotUse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(DotnetSettings_RenamedResourcesEntry_DoNotUseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST DotnetSettings_RenamedResourcesEntry_DoNotUseGlobalsTypeInternal DotnetSettings_RenamedResourcesEntry_DoNotUse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* DotnetSettings_RenamedResourcesEntry_DoNotUse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return DotnetSettings_RenamedResourcesEntry_DoNotUse_globals_.GetClassData();
#else
  return DotnetSettings_RenamedResourcesEntry_DoNotUse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class CommonLanguageSettings::_Internal {
 public:
  using HasBits = decltype(::std::declval<CommonLanguageSettings>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(CommonLanguageSettings, _impl_._has_bits_);
};

constexpr CommonLanguageSettings::ParseTableT_ CommonLanguageSettings::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(CommonLanguageSettings, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::api::CommonLanguageSettings>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated .google.api.ClientLibraryDestination destinations = 2;
      {::_pbi::TcParser::FastV32P1,
       {18, 0, 0,
        PROTOBUF_FIELD_OFFSET(CommonLanguageSettings, _impl_.destinations_)}},
      // string reference_docs_uri = 1 [deprecated = true];
      {::_pbi::TcParser::FastUS1,
       {10, 1, 0,
        PROTOBUF_FIELD_OFFSET(CommonLanguageSettings, _impl_.reference_docs_uri_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string reference_docs_uri = 1 [deprecated = true];
      {PROTOBUF_FIELD_OFFSET(CommonLanguageSettings, _impl_.reference_docs_uri_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // repeated .google.api.ClientLibraryDestination destinations = 2;
      {PROTOBUF_FIELD_OFFSET(CommonLanguageSettings, _impl_.destinations_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedOpenEnum)},
    }},
    // no aux_entries
    {{
      "\41\22\0\0\0\0\0\0"
      "google.api.CommonLanguageSettings"
      "reference_docs_uri"
    }},
  };
}


inline constexpr CommonLanguageSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        destinations_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::CommonLanguageSettings,
            PROTOBUF_FIELD_OFFSET(::google::api::CommonLanguageSettings, _impl_.destinations_)>()
         }
        ,
        _destinations_cached_byte_size_{0},
        reference_docs_uri_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr CommonLanguageSettings::CommonLanguageSettings(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL CommonLanguageSettings::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) CommonLanguageSettings(arena);
}
constexpr auto CommonLanguageSettings::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(CommonLanguageSettings), alignof(CommonLanguageSettings));
}
constexpr auto CommonLanguageSettings::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &CommonLanguageSettings::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<CommonLanguageSettings>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &CommonLanguageSettings::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<CommonLanguageSettings>(), &CommonLanguageSettings::ByteSizeLong,
              &CommonLanguageSettings::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(CommonLanguageSettings, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[0],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fapi_2fclient_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct CommonLanguageSettingsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr CommonLanguageSettingsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 CommonLanguageSettings_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(CommonLanguageSettings::InternalGenerateClassData_(
            _default, &CommonLanguageSettings_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<CommonLanguageSettings>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~CommonLanguageSettingsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) CommonLanguageSettings _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<CommonLanguageSettings>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(CommonLanguageSettingsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST CommonLanguageSettingsGlobalsTypeInternal CommonLanguageSettings_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* CommonLanguageSettings_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return CommonLanguageSettings_globals_.GetClassData();
#else
  return CommonLanguageSettings_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class RubySettings::_Internal {
 public:
  using HasBits = decltype(::std::declval<RubySettings>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(RubySettings, _impl_._has_bits_);
};

constexpr RubySettings::ParseTableT_ RubySettings::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(RubySettings, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::api::RubySettings>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.api.CommonLanguageSettings common = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(RubySettings, _impl_.common_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.api.CommonLanguageSettings common = 1;
      {PROTOBUF_FIELD_OFFSET(RubySettings, _impl_.common_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::api::CommonLanguageSettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::api::CommonLanguageSettings_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr RubySettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        common_{nullptr} {}

template <typename>
constexpr RubySettings::RubySettings(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL RubySettings::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) RubySettings(arena);
}
constexpr auto RubySettings::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(RubySettings), alignof(RubySettings));
}
constexpr auto RubySettings::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &RubySettings::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<RubySettings>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &RubySettings::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<RubySettings>(), &RubySettings::ByteSizeLong,
              &RubySettings::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(RubySettings, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[12],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fapi_2fclient_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct RubySettingsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr RubySettingsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 RubySettings_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(RubySettings::InternalGenerateClassData_(
            _default, &RubySettings_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<RubySettings>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~RubySettingsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) RubySettings _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<RubySettings>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(RubySettingsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST RubySettingsGlobalsTypeInternal RubySettings_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* RubySettings_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return RubySettings_globals_.GetClassData();
#else
  return RubySettings_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class PythonSettings::_Internal {
 public:
  using HasBits = decltype(::std::declval<PythonSettings>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(PythonSettings, _impl_._has_bits_);
};

constexpr PythonSettings::ParseTableT_ PythonSettings::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(PythonSettings, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::api::PythonSettings>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.api.CommonLanguageSettings common = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(PythonSettings, _impl_.common_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.api.CommonLanguageSettings common = 1;
      {PROTOBUF_FIELD_OFFSET(PythonSettings, _impl_.common_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::api::CommonLanguageSettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::api::CommonLanguageSettings_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr PythonSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        common_{nullptr} {}

template <typename>
constexpr PythonSettings::PythonSettings(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL PythonSettings::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) PythonSettings(arena);
}
constexpr auto PythonSettings::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(PythonSettings), alignof(PythonSettings));
}
constexpr auto PythonSettings::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &PythonSettings::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<PythonSettings>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &PythonSettings::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<PythonSettings>(), &PythonSettings::ByteSizeLong,
              &PythonSettings::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(PythonSettings, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[7],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fapi_2fclient_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct PythonSettingsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr PythonSettingsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 PythonSettings_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(PythonSettings::InternalGenerateClassData_(
            _default, &PythonSettings_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<PythonSettings>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~PythonSettingsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) PythonSettings _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<PythonSettings>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(PythonSettingsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST PythonSettingsGlobalsTypeInternal PythonSettings_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* PythonSettings_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return PythonSettings_globals_.GetClassData();
#else
  return PythonSettings_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class PhpSettings::_Internal {
 public:
  using HasBits = decltype(::std::declval<PhpSettings>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(PhpSettings, _impl_._has_bits_);
};

constexpr PhpSettings::ParseTableT_ PhpSettings::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(PhpSettings, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::api::PhpSettings>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.api.CommonLanguageSettings common = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(PhpSettings, _impl_.common_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.api.CommonLanguageSettings common = 1;
      {PROTOBUF_FIELD_OFFSET(PhpSettings, _impl_.common_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::api::CommonLanguageSettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::api::CommonLanguageSettings_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr PhpSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        common_{nullptr} {}

template <typename>
constexpr PhpSettings::PhpSettings(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL PhpSettings::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) PhpSettings(arena);
}
constexpr auto PhpSettings::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(PhpSettings), alignof(PhpSettings));
}
constexpr auto PhpSettings::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &PhpSettings::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<PhpSettings>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &PhpSettings::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<PhpSettings>(), &PhpSettings::ByteSizeLong,
              &PhpSettings::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(PhpSettings, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[6],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fapi_2fclient_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct PhpSettingsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr PhpSettingsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 PhpSettings_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(PhpSettings::InternalGenerateClassData_(
            _default, &PhpSettings_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<PhpSettings>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~PhpSettingsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) PhpSettings _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<PhpSettings>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(PhpSettingsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST PhpSettingsGlobalsTypeInternal PhpSettings_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* PhpSettings_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return PhpSettings_globals_.GetClassData();
#else
  return PhpSettings_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class NodeSettings::_Internal {
 public:
  using HasBits = decltype(::std::declval<NodeSettings>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(NodeSettings, _impl_._has_bits_);
};

constexpr NodeSettings::ParseTableT_ NodeSettings::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(NodeSettings, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::api::NodeSettings>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.api.CommonLanguageSettings common = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(NodeSettings, _impl_.common_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.api.CommonLanguageSettings common = 1;
      {PROTOBUF_FIELD_OFFSET(NodeSettings, _impl_.common_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::api::CommonLanguageSettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::api::CommonLanguageSettings_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr NodeSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        common_{nullptr} {}

template <typename>
constexpr NodeSettings::NodeSettings(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL NodeSettings::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) NodeSettings(arena);
}
constexpr auto NodeSettings::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(NodeSettings), alignof(NodeSettings));
}
constexpr auto NodeSettings::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &NodeSettings::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<NodeSettings>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &NodeSettings::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<NodeSettings>(), &NodeSettings::ByteSizeLong,
              &NodeSettings::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(NodeSettings, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[8],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fapi_2fclient_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct NodeSettingsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr NodeSettingsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 NodeSettings_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(NodeSettings::InternalGenerateClassData_(
            _default, &NodeSettings_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<NodeSettings>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~NodeSettingsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) NodeSettings _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<NodeSettings>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(NodeSettingsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST NodeSettingsGlobalsTypeInternal NodeSettings_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* NodeSettings_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return NodeSettings_globals_.GetClassData();
#else
  return NodeSettings_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class MethodSettings_LongRunning::_Internal {
 public:
  using HasBits = decltype(::std::declval<MethodSettings_LongRunning>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(MethodSettings_LongRunning, _impl_._has_bits_);
};

constexpr MethodSettings_LongRunning::ParseTableT_ MethodSettings_LongRunning::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(MethodSettings_LongRunning, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      3,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::api::MethodSettings_LongRunning>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.protobuf.Duration total_poll_timeout = 4;
      {::_pbi::TcParser::FastMtS1,
       {34, 2, 2,
        PROTOBUF_FIELD_OFFSET(MethodSettings_LongRunning, _impl_.total_poll_timeout_)}},
      // .google.protobuf.Duration initial_poll_delay = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(MethodSettings_LongRunning, _impl_.initial_poll_delay_)}},
      // float poll_delay_multiplier = 2;
      {::_pbi::TcParser::FastF32S1,
       {21, 3, 0,
        PROTOBUF_FIELD_OFFSET(MethodSettings_LongRunning, _impl_.poll_delay_multiplier_)}},
      // .google.protobuf.Duration max_poll_delay = 3;
      {::_pbi::TcParser::FastMtS1,
       {26, 1, 1,
        PROTOBUF_FIELD_OFFSET(MethodSettings_LongRunning, _impl_.max_poll_delay_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.protobuf.Duration initial_poll_delay = 1;
      {PROTOBUF_FIELD_OFFSET(MethodSettings_LongRunning, _impl_.initial_poll_delay_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // float poll_delay_multiplier = 2;
      {PROTOBUF_FIELD_OFFSET(MethodSettings_LongRunning, _impl_.poll_delay_multiplier_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kFloat)},
      // .google.protobuf.Duration max_poll_delay = 3;
      {PROTOBUF_FIELD_OFFSET(MethodSettings_LongRunning, _impl_.max_poll_delay_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.protobuf.Duration total_poll_timeout = 4;
      {PROTOBUF_FIELD_OFFSET(MethodSettings_LongRunning, _impl_.total_poll_timeout_), _Internal::kHasBitsOffset + 2, 2, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::Duration>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::Duration_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::Duration>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::Duration_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::Duration>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::Duration_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr MethodSettings_LongRunning::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        initial_poll_delay_{nullptr},
        max_poll_delay_{nullptr},
        total_poll_timeout_{nullptr},
        poll_delay_multiplier_{0} {}

template <typename>
constexpr MethodSettings_LongRunning::MethodSettings_LongRunning(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL MethodSettings_LongRunning::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) MethodSettings_LongRunning(arena);
}
constexpr auto MethodSettings_LongRunning::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(MethodSettings_LongRunning), alignof(MethodSettings_LongRunning));
}
constexpr auto MethodSettings_LongRunning::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &MethodSettings_LongRunning::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<MethodSettings_LongRunning>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &MethodSettings_LongRunning::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<MethodSettings_LongRunning>(), &MethodSettings_LongRunning::ByteSizeLong,
              &MethodSettings_LongRunning::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(MethodSettings_LongRunning, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[14],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fapi_2fclient_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct MethodSettings_LongRunningGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr MethodSettings_LongRunningGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 MethodSettings_LongRunning_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(MethodSettings_LongRunning::InternalGenerateClassData_(
            _default, &MethodSettings_LongRunning_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<MethodSettings_LongRunning>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~MethodSettings_LongRunningGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) MethodSettings_LongRunning _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<MethodSettings_LongRunning>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(MethodSettings_LongRunningGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST MethodSettings_LongRunningGlobalsTypeInternal MethodSettings_LongRunning_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* MethodSettings_LongRunning_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return MethodSettings_LongRunning_globals_.GetClassData();
#else
  return MethodSettings_LongRunning_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class JavaSettings::_Internal {
 public:
  using HasBits = decltype(::std::declval<JavaSettings>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(JavaSettings, _impl_._has_bits_);
};

constexpr JavaSettings::ParseTableT_ JavaSettings::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(JavaSettings, _impl_._has_bits_),
      0, // no _extensions_
      3, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::api::JavaSettings>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string library_package = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(JavaSettings, _impl_.library_package_)}},
      {::_pbi::TcParser::MiniParse, {}},
      // .google.api.CommonLanguageSettings common = 3;
      {::_pbi::TcParser::FastMtS1,
       {26, 1, 0,
        PROTOBUF_FIELD_OFFSET(JavaSettings, _impl_.common_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string library_package = 1;
      {PROTOBUF_FIELD_OFFSET(JavaSettings, _impl_.library_package_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // map<string, string> service_class_names = 2;
      {PROTOBUF_FIELD_OFFSET(JavaSettings, _impl_.service_class_names_), _Internal::kHasBitsOffset + 2, 1, (0 | ::_fl::kFcRepeated | ::_fl::kMap)},
      // .google.api.CommonLanguageSettings common = 3;
      {PROTOBUF_FIELD_OFFSET(JavaSettings, _impl_.common_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::api::CommonLanguageSettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::api::CommonLanguageSettings_globals_},
        #endif
        {::_pbi::TcParser::GetMapAuxInfo(
            1, 0, 9, 9, 0)},
    }},
    {{
      "\27\17\23\0\0\0\0\0"
      "google.api.JavaSettings"
      "library_package"
      "service_class_names"
    }},
  };
}


inline constexpr JavaSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        library_package_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        common_{nullptr},
        service_class_names_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::JavaSettings,
            PROTOBUF_FIELD_OFFSET(::google::api::JavaSettings, _impl_.service_class_names_)>()
         }
     {}

template <typename>
constexpr JavaSettings::JavaSettings(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL JavaSettings::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) JavaSettings(arena);
}
constexpr auto JavaSettings::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(JavaSettings), alignof(JavaSettings));
}
constexpr auto JavaSettings::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &JavaSettings::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<JavaSettings>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &JavaSettings::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<JavaSettings>(), &JavaSettings::ByteSizeLong,
              &JavaSettings::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(JavaSettings, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[4],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fapi_2fclient_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct JavaSettingsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr JavaSettingsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 JavaSettings_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(JavaSettings::InternalGenerateClassData_(
            _default, &JavaSettings_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<JavaSettings>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~JavaSettingsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) JavaSettings _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<JavaSettings>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(JavaSettingsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST JavaSettingsGlobalsTypeInternal JavaSettings_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* JavaSettings_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return JavaSettings_globals_.GetClassData();
#else
  return JavaSettings_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class GoSettings::_Internal {
 public:
  using HasBits = decltype(::std::declval<GoSettings>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(GoSettings, _impl_._has_bits_);
};

constexpr GoSettings::ParseTableT_ GoSettings::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(GoSettings, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::api::GoSettings>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.api.CommonLanguageSettings common = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(GoSettings, _impl_.common_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.api.CommonLanguageSettings common = 1;
      {PROTOBUF_FIELD_OFFSET(GoSettings, _impl_.common_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::api::CommonLanguageSettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::api::CommonLanguageSettings_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr GoSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        common_{nullptr} {}

template <typename>
constexpr GoSettings::GoSettings(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL GoSettings::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) GoSettings(arena);
}
constexpr auto GoSettings::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(GoSettings), alignof(GoSettings));
}
constexpr auto GoSettings::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &GoSettings::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<GoSettings>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &GoSettings::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<GoSettings>(), &GoSettings::ByteSizeLong,
              &GoSettings::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(GoSettings, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[13],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fapi_2fclient_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct GoSettingsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr GoSettingsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 GoSettings_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(GoSettings::InternalGenerateClassData_(
            _default, &GoSettings_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<GoSettings>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~GoSettingsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) GoSettings _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<GoSettings>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(GoSettingsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST GoSettingsGlobalsTypeInternal GoSettings_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* GoSettings_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return GoSettings_globals_.GetClassData();
#else
  return GoSettings_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class DotnetSettings::_Internal {
 public:
  using HasBits = decltype(::std::declval<DotnetSettings>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(DotnetSettings, _impl_._has_bits_);
};

constexpr DotnetSettings::ParseTableT_ DotnetSettings::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(DotnetSettings, _impl_._has_bits_),
      0, // no _extensions_
      6, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967232,  // skipmap
      offsetof(ParseTableT_, field_entries),
      6,  // num_field_entries
      3,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::api::DotnetSettings>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .google.api.CommonLanguageSettings common = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 3, 0,
        PROTOBUF_FIELD_OFFSET(DotnetSettings, _impl_.common_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      // repeated string ignored_resources = 4;
      {::_pbi::TcParser::FastUR1,
       {34, 0, 0,
        PROTOBUF_FIELD_OFFSET(DotnetSettings, _impl_.ignored_resources_)}},
      // repeated string forced_namespace_aliases = 5;
      {::_pbi::TcParser::FastUR1,
       {42, 1, 0,
        PROTOBUF_FIELD_OFFSET(DotnetSettings, _impl_.forced_namespace_aliases_)}},
      // repeated string handwritten_signatures = 6;
      {::_pbi::TcParser::FastUR1,
       {50, 2, 0,
        PROTOBUF_FIELD_OFFSET(DotnetSettings, _impl_.handwritten_signatures_)}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.api.CommonLanguageSettings common = 1;
      {PROTOBUF_FIELD_OFFSET(DotnetSettings, _impl_.common_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // map<string, string> renamed_services = 2;
      {PROTOBUF_FIELD_OFFSET(DotnetSettings, _impl_.renamed_services_), _Internal::kHasBitsOffset + 4, 1, (0 | ::_fl::kFcRepeated | ::_fl::kMap)},
      // map<string, string> renamed_resources = 3;
      {PROTOBUF_FIELD_OFFSET(DotnetSettings, _impl_.renamed_resources_), _Internal::kHasBitsOffset + 5, 2, (0 | ::_fl::kFcRepeated | ::_fl::kMap)},
      // repeated string ignored_resources = 4;
      {PROTOBUF_FIELD_OFFSET(DotnetSettings, _impl_.ignored_resources_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // repeated string forced_namespace_aliases = 5;
      {PROTOBUF_FIELD_OFFSET(DotnetSettings, _impl_.forced_namespace_aliases_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // repeated string handwritten_signatures = 6;
      {PROTOBUF_FIELD_OFFSET(DotnetSettings, _impl_.handwritten_signatures_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::api::CommonLanguageSettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::api::CommonLanguageSettings_globals_},
        #endif
        {::_pbi::TcParser::GetMapAuxInfo(
            1, 0, 9, 9, 0)},
        {::_pbi::TcParser::GetMapAuxInfo(
            1, 0, 9, 9, 0)},
    }},
    {{
      "\31\0\20\21\21\30\26\0"
      "google.api.DotnetSettings"
      "renamed_services"
      "renamed_resources"
      "ignored_resources"
      "forced_namespace_aliases"
      "handwritten_signatures"
    }},
  };
}


inline constexpr DotnetSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        ignored_resources_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::DotnetSettings,
            PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_.ignored_resources_)>()
         }
        ,
        forced_namespace_aliases_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::DotnetSettings,
            PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_.forced_namespace_aliases_)>()
         }
        ,
        handwritten_signatures_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::DotnetSettings,
            PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_.handwritten_signatures_)>()
         }
        ,
        common_{nullptr},
        renamed_services_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::DotnetSettings,
            PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_.renamed_services_)>()
         }
        ,
        renamed_resources_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::DotnetSettings,
            PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_.renamed_resources_)>()
         }
     {}

template <typename>
constexpr DotnetSettings::DotnetSettings(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL DotnetSettings::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) DotnetSettings(arena);
}
constexpr auto DotnetSettings::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(DotnetSettings), alignof(DotnetSettings));
}
constexpr auto DotnetSettings::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &DotnetSettings::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<DotnetSettings>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &DotnetSettings::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<DotnetSettings>(), &DotnetSettings::ByteSizeLong,
              &DotnetSettings::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(DotnetSettings, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[11],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fapi_2fclient_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct DotnetSettingsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr DotnetSettingsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 DotnetSettings_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(DotnetSettings::InternalGenerateClassData_(
            _default, &DotnetSettings_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<DotnetSettings>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~DotnetSettingsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) DotnetSettings _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<DotnetSettings>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(DotnetSettingsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST DotnetSettingsGlobalsTypeInternal DotnetSettings_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* DotnetSettings_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return DotnetSettings_globals_.GetClassData();
#else
  return DotnetSettings_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class CppSettings::_Internal {
 public:
  using HasBits = decltype(::std::declval<CppSettings>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(CppSettings, _impl_._has_bits_);
};

constexpr CppSettings::ParseTableT_ CppSettings::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(CppSettings, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::api::CppSettings>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.api.CommonLanguageSettings common = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(CppSettings, _impl_.common_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.api.CommonLanguageSettings common = 1;
      {PROTOBUF_FIELD_OFFSET(CppSettings, _impl_.common_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::api::CommonLanguageSettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::api::CommonLanguageSettings_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr CppSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        common_{nullptr} {}

template <typename>
constexpr CppSettings::CppSettings(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL CppSettings::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) CppSettings(arena);
}
constexpr auto CppSettings::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(CppSettings), alignof(CppSettings));
}
constexpr auto CppSettings::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &CppSettings::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<CppSettings>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &CppSettings::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<CppSettings>(), &CppSettings::ByteSizeLong,
              &CppSettings::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(CppSettings, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[5],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fapi_2fclient_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct CppSettingsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr CppSettingsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 CppSettings_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(CppSettings::InternalGenerateClassData_(
            _default, &CppSettings_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<CppSettings>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~CppSettingsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) CppSettings _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<CppSettings>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(CppSettingsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST CppSettingsGlobalsTypeInternal CppSettings_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* CppSettings_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return CppSettings_globals_.GetClassData();
#else
  return CppSettings_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class MethodSettings::_Internal {
 public:
  using HasBits = decltype(::std::declval<MethodSettings>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(MethodSettings, _impl_._has_bits_);
};

constexpr MethodSettings::ParseTableT_ MethodSettings::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(MethodSettings, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::api::MethodSettings>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.api.MethodSettings.LongRunning long_running = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(MethodSettings, _impl_.long_running_)}},
      // string selector = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(MethodSettings, _impl_.selector_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string selector = 1;
      {PROTOBUF_FIELD_OFFSET(MethodSettings, _impl_.selector_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.api.MethodSettings.LongRunning long_running = 2;
      {PROTOBUF_FIELD_OFFSET(MethodSettings, _impl_.long_running_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::api::MethodSettings_LongRunning>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::api::MethodSettings_LongRunning_globals_},
        #endif
    }},
    {{
      "\31\10\0\0\0\0\0\0"
      "google.api.MethodSettings"
      "selector"
    }},
  };
}


inline constexpr MethodSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        selector_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        long_running_{nullptr} {}

template <typename>
constexpr MethodSettings::MethodSettings(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL MethodSettings::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) MethodSettings(arena);
}
constexpr auto MethodSettings::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(MethodSettings), alignof(MethodSettings));
}
constexpr auto MethodSettings::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &MethodSettings::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<MethodSettings>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &MethodSettings::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<MethodSettings>(), &MethodSettings::ByteSizeLong,
              &MethodSettings::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(MethodSettings, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[15],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fapi_2fclient_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct MethodSettingsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr MethodSettingsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 MethodSettings_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(MethodSettings::InternalGenerateClassData_(
            _default, &MethodSettings_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<MethodSettings>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~MethodSettingsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) MethodSettings _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<MethodSettings>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(MethodSettingsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST MethodSettingsGlobalsTypeInternal MethodSettings_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* MethodSettings_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return MethodSettings_globals_.GetClassData();
#else
  return MethodSettings_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ClientLibrarySettings::_Internal {
 public:
  using HasBits = decltype(::std::declval<ClientLibrarySettings>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_._has_bits_);
};

constexpr ClientLibrarySettings::ParseTableT_ ClientLibrarySettings::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_._has_bits_),
      0, // no _extensions_
      28, 120,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4027580408,  // skipmap
      offsetof(ParseTableT_, field_entries),
      11,  // num_field_entries
      8,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::api::ClientLibrarySettings>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string version = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.version_)}},
      // .google.api.LaunchStage launch_stage = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ClientLibrarySettings, _impl_.launch_stage_), 9>(),
       {16, 9, 0,
        PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.launch_stage_)}},
      // bool rest_numeric_enums = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(ClientLibrarySettings, _impl_.rest_numeric_enums_), 10>(),
       {24, 10, 0,
        PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.rest_numeric_enums_)}},
      {::_pbi::TcParser::MiniParse, {}},
      // .google.api.JavaSettings java_settings = 21;
      {::_pbi::TcParser::FastMtS2,
       {426, 1, 0,
        PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.java_settings_)}},
      // .google.api.CppSettings cpp_settings = 22;
      {::_pbi::TcParser::FastMtS2,
       {434, 2, 1,
        PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.cpp_settings_)}},
      // .google.api.PhpSettings php_settings = 23;
      {::_pbi::TcParser::FastMtS2,
       {442, 3, 2,
        PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.php_settings_)}},
      // .google.api.PythonSettings python_settings = 24;
      {::_pbi::TcParser::FastMtS2,
       {450, 4, 3,
        PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.python_settings_)}},
      // .google.api.NodeSettings node_settings = 25;
      {::_pbi::TcParser::FastMtS2,
       {458, 5, 4,
        PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.node_settings_)}},
      // .google.api.DotnetSettings dotnet_settings = 26;
      {::_pbi::TcParser::FastMtS2,
       {466, 6, 5,
        PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.dotnet_settings_)}},
      // .google.api.RubySettings ruby_settings = 27;
      {::_pbi::TcParser::FastMtS2,
       {474, 7, 6,
        PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.ruby_settings_)}},
      // .google.api.GoSettings go_settings = 28;
      {::_pbi::TcParser::FastMtS2,
       {482, 8, 7,
        PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.go_settings_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // string version = 1;
      {PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.version_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.api.LaunchStage launch_stage = 2;
      {PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.launch_stage_), _Internal::kHasBitsOffset + 9, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // bool rest_numeric_enums = 3;
      {PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.rest_numeric_enums_), _Internal::kHasBitsOffset + 10, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // .google.api.JavaSettings java_settings = 21;
      {PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.java_settings_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.api.CppSettings cpp_settings = 22;
      {PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.cpp_settings_), _Internal::kHasBitsOffset + 2, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.api.PhpSettings php_settings = 23;
      {PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.php_settings_), _Internal::kHasBitsOffset + 3, 2, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.api.PythonSettings python_settings = 24;
      {PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.python_settings_), _Internal::kHasBitsOffset + 4, 3, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.api.NodeSettings node_settings = 25;
      {PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.node_settings_), _Internal::kHasBitsOffset + 5, 4, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.api.DotnetSettings dotnet_settings = 26;
      {PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.dotnet_settings_), _Internal::kHasBitsOffset + 6, 5, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.api.RubySettings ruby_settings = 27;
      {PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.ruby_settings_), _Internal::kHasBitsOffset + 7, 6, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.api.GoSettings go_settings = 28;
      {PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.go_settings_), _Internal::kHasBitsOffset + 8, 7, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::api::JavaSettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::api::JavaSettings_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::api::CppSettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::api::CppSettings_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::api::PhpSettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::api::PhpSettings_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::api::PythonSettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::api::PythonSettings_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::api::NodeSettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::api::NodeSettings_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::api::DotnetSettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::api::DotnetSettings_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::api::RubySettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::api::RubySettings_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::api::GoSettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::api::GoSettings_globals_},
        #endif
    }},
    {{
      "\40\7\0\0\0\0\0\0\0\0\0\0\0\0\0\0"
      "google.api.ClientLibrarySettings"
      "version"
    }},
  };
}


inline constexpr ClientLibrarySettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        version_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        java_settings_{nullptr},
        cpp_settings_{nullptr},
        php_settings_{nullptr},
        python_settings_{nullptr},
        node_settings_{nullptr},
        dotnet_settings_{nullptr},
        ruby_settings_{nullptr},
        go_settings_{nullptr},
        launch_stage_{static_cast< ::google::api::LaunchStage >(0)},
        rest_numeric_enums_{false} {}

template <typename>
constexpr ClientLibrarySettings::ClientLibrarySettings(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ClientLibrarySettings::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ClientLibrarySettings(arena);
}
constexpr auto ClientLibrarySettings::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ClientLibrarySettings), alignof(ClientLibrarySettings));
}
constexpr auto ClientLibrarySettings::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ClientLibrarySettings::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ClientLibrarySettings>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ClientLibrarySettings::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ClientLibrarySettings>(), &ClientLibrarySettings::ByteSizeLong,
              &ClientLibrarySettings::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[1],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fapi_2fclient_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ClientLibrarySettingsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ClientLibrarySettingsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ClientLibrarySettings_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ClientLibrarySettings::InternalGenerateClassData_(
            _default, &ClientLibrarySettings_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ClientLibrarySettings>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ClientLibrarySettingsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ClientLibrarySettings _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ClientLibrarySettings>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ClientLibrarySettingsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ClientLibrarySettingsGlobalsTypeInternal ClientLibrarySettings_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ClientLibrarySettings_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ClientLibrarySettings_globals_.GetClassData();
#else
  return ClientLibrarySettings_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Publishing::_Internal {
 public:
  using HasBits = decltype(::std::declval<Publishing>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Publishing, _impl_._has_bits_);
};

constexpr Publishing::ParseTableT_ Publishing::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Publishing, _impl_._has_bits_),
      0, // no _extensions_
      110, 120,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967293,  // skipmap
      offsetof(ParseTableT_, field_entries),
      10,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::api::Publishing>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      // repeated .google.api.MethodSettings method_settings = 2;
      {::_pbi::TcParser::FastMtR1,
       {18, 0, 0,
        PROTOBUF_FIELD_OFFSET(Publishing, _impl_.method_settings_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      // string new_issue_uri = 101;
      {::_pbi::TcParser::FastUS2,
       {1706, 3, 0,
        PROTOBUF_FIELD_OFFSET(Publishing, _impl_.new_issue_uri_)}},
      // string documentation_uri = 102;
      {::_pbi::TcParser::FastUS2,
       {1714, 4, 0,
        PROTOBUF_FIELD_OFFSET(Publishing, _impl_.documentation_uri_)}},
      // string api_short_name = 103;
      {::_pbi::TcParser::FastUS2,
       {1722, 5, 0,
        PROTOBUF_FIELD_OFFSET(Publishing, _impl_.api_short_name_)}},
      // string github_label = 104;
      {::_pbi::TcParser::FastUS2,
       {1730, 6, 0,
        PROTOBUF_FIELD_OFFSET(Publishing, _impl_.github_label_)}},
      // repeated string codeowner_github_teams = 105;
      {::_pbi::TcParser::FastUR2,
       {1738, 1, 0,
        PROTOBUF_FIELD_OFFSET(Publishing, _impl_.codeowner_github_teams_)}},
      // string doc_tag_prefix = 106;
      {::_pbi::TcParser::FastUS2,
       {1746, 7, 0,
        PROTOBUF_FIELD_OFFSET(Publishing, _impl_.doc_tag_prefix_)}},
      // .google.api.ClientLibraryOrganization organization = 107;
      {::_pbi::TcParser::FastV32S2,
       {1752, 9, 0,
        PROTOBUF_FIELD_OFFSET(Publishing, _impl_.organization_)}},
      {::_pbi::TcParser::MiniParse, {}},
      // repeated .google.api.ClientLibrarySettings library_settings = 109;
      {::_pbi::TcParser::FastMtR2,
       {1770, 2, 1,
        PROTOBUF_FIELD_OFFSET(Publishing, _impl_.library_settings_)}},
      // string proto_reference_documentation_uri = 110;
      {::_pbi::TcParser::FastUS2,
       {1778, 8, 0,
        PROTOBUF_FIELD_OFFSET(Publishing, _impl_.proto_reference_documentation_uri_)}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      101, 0, 1,
      64640, 1,
      65535, 65535
    }}, {{
      // repeated .google.api.MethodSettings method_settings = 2;
      {PROTOBUF_FIELD_OFFSET(Publishing, _impl_.method_settings_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
      // string new_issue_uri = 101;
      {PROTOBUF_FIELD_OFFSET(Publishing, _impl_.new_issue_uri_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string documentation_uri = 102;
      {PROTOBUF_FIELD_OFFSET(Publishing, _impl_.documentation_uri_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string api_short_name = 103;
      {PROTOBUF_FIELD_OFFSET(Publishing, _impl_.api_short_name_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string github_label = 104;
      {PROTOBUF_FIELD_OFFSET(Publishing, _impl_.github_label_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // repeated string codeowner_github_teams = 105;
      {PROTOBUF_FIELD_OFFSET(Publishing, _impl_.codeowner_github_teams_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // string doc_tag_prefix = 106;
      {PROTOBUF_FIELD_OFFSET(Publishing, _impl_.doc_tag_prefix_), _Internal::kHasBitsOffset + 7, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.api.ClientLibraryOrganization organization = 107;
      {PROTOBUF_FIELD_OFFSET(Publishing, _impl_.organization_), _Internal::kHasBitsOffset + 9, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // repeated .google.api.ClientLibrarySettings library_settings = 109;
      {PROTOBUF_FIELD_OFFSET(Publishing, _impl_.library_settings_), _Internal::kHasBitsOffset + 2, 1, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
      // string proto_reference_documentation_uri = 110;
      {PROTOBUF_FIELD_OFFSET(Publishing, _impl_.proto_reference_documentation_uri_), _Internal::kHasBitsOffset + 8, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::api::MethodSettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::api::MethodSettings_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::api::ClientLibrarySettings>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::api::ClientLibrarySettings_globals_},
        #endif
    }},
    {{
      "\25\0\15\21\16\14\26\16\0\0\41\0\0\0\0\0"
      "google.api.Publishing"
      "new_issue_uri"
      "documentation_uri"
      "api_short_name"
      "github_label"
      "codeowner_github_teams"
      "doc_tag_prefix"
      "proto_reference_documentation_uri"
    }},
  };
}


inline constexpr Publishing::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        method_settings_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::Publishing,
            PROTOBUF_FIELD_OFFSET(::google::api::Publishing, _impl_.method_settings_)>()
         }
        ,
        codeowner_github_teams_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::Publishing,
            PROTOBUF_FIELD_OFFSET(::google::api::Publishing, _impl_.codeowner_github_teams_)>()
         }
        ,
        library_settings_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::Publishing,
            PROTOBUF_FIELD_OFFSET(::google::api::Publishing, _impl_.library_settings_)>()
         }
        ,
        new_issue_uri_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        documentation_uri_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        api_short_name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        github_label_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        doc_tag_prefix_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        proto_reference_documentation_uri_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        organization_{static_cast< ::google::api::ClientLibraryOrganization >(0)} {}

template <typename>
constexpr Publishing::Publishing(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Publishing::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Publishing(arena);
}
constexpr auto Publishing::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Publishing), alignof(Publishing));
}
constexpr auto Publishing::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Publishing::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<Publishing>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Publishing::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<Publishing>(), &Publishing::ByteSizeLong,
              &Publishing::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Publishing, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[2],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fapi_2fclient_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct PublishingGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr PublishingGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Publishing_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Publishing::InternalGenerateClassData_(
            _default, &Publishing_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Publishing>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~PublishingGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Publishing _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Publishing>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(PublishingGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST PublishingGlobalsTypeInternal Publishing_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Publishing_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Publishing_globals_.GetClassData();
#else
  return Publishing_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
}  // namespace api
}  // namespace google
static const ::_pb::EnumDescriptor* PROTOBUF_NONNULL
    file_level_enum_descriptors_google_2fapi_2fclient_2eproto[2];
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_google_2fapi_2fclient_2eproto = nullptr;
const ::uint32_t
    TableStruct_google_2fapi_2fclient_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::api::CommonLanguageSettings, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::api::CommonLanguageSettings, _impl_.reference_docs_uri_),
        PROTOBUF_FIELD_OFFSET(::google::api::CommonLanguageSettings, _impl_.destinations_),
        1,
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::api::ClientLibrarySettings, _impl_._has_bits_),
        14, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::api::ClientLibrarySettings, _impl_.version_),
        PROTOBUF_FIELD_OFFSET(::google::api::ClientLibrarySettings, _impl_.launch_stage_),
        PROTOBUF_FIELD_OFFSET(::google::api::ClientLibrarySettings, _impl_.rest_numeric_enums_),
        PROTOBUF_FIELD_OFFSET(::google::api::ClientLibrarySettings, _impl_.java_settings_),
        PROTOBUF_FIELD_OFFSET(::google::api::ClientLibrarySettings, _impl_.cpp_settings_),
        PROTOBUF_FIELD_OFFSET(::google::api::ClientLibrarySettings, _impl_.php_settings_),
        PROTOBUF_FIELD_OFFSET(::google::api::ClientLibrarySettings, _impl_.python_settings_),
        PROTOBUF_FIELD_OFFSET(::google::api::ClientLibrarySettings, _impl_.node_settings_),
        PROTOBUF_FIELD_OFFSET(::google::api::ClientLibrarySettings, _impl_.dotnet_settings_),
        PROTOBUF_FIELD_OFFSET(::google::api::ClientLibrarySettings, _impl_.ruby_settings_),
        PROTOBUF_FIELD_OFFSET(::google::api::ClientLibrarySettings, _impl_.go_settings_),
        0,
        9,
        10,
        1,
        2,
        3,
        4,
        5,
        6,
        7,
        8,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::api::Publishing, _impl_._has_bits_),
        13, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::api::Publishing, _impl_.method_settings_),
        PROTOBUF_FIELD_OFFSET(::google::api::Publishing, _impl_.new_issue_uri_),
        PROTOBUF_FIELD_OFFSET(::google::api::Publishing, _impl_.documentation_uri_),
        PROTOBUF_FIELD_OFFSET(::google::api::Publishing, _impl_.api_short_name_),
        PROTOBUF_FIELD_OFFSET(::google::api::Publishing, _impl_.github_label_),
        PROTOBUF_FIELD_OFFSET(::google::api::Publishing, _impl_.codeowner_github_teams_),
        PROTOBUF_FIELD_OFFSET(::google::api::Publishing, _impl_.doc_tag_prefix_),
        PROTOBUF_FIELD_OFFSET(::google::api::Publishing, _impl_.organization_),
        PROTOBUF_FIELD_OFFSET(::google::api::Publishing, _impl_.library_settings_),
        PROTOBUF_FIELD_OFFSET(::google::api::Publishing, _impl_.proto_reference_documentation_uri_),
        0,
        3,
        4,
        5,
        6,
        1,
        7,
        9,
        2,
        8,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::api::JavaSettings_ServiceClassNamesEntry_DoNotUse, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::api::JavaSettings_ServiceClassNamesEntry_DoNotUse, _impl_.key_),
        PROTOBUF_FIELD_OFFSET(::google::api::JavaSettings_ServiceClassNamesEntry_DoNotUse, _impl_.value_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::api::JavaSettings, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::api::JavaSettings, _impl_.library_package_),
        PROTOBUF_FIELD_OFFSET(::google::api::JavaSettings, _impl_.service_class_names_),
        PROTOBUF_FIELD_OFFSET(::google::api::JavaSettings, _impl_.common_),
        0,
        2,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::api::CppSettings, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::api::CppSettings, _impl_.common_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::api::PhpSettings, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::api::PhpSettings, _impl_.common_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::api::PythonSettings, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::api::PythonSettings, _impl_.common_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::api::NodeSettings, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::api::NodeSettings, _impl_.common_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings_RenamedServicesEntry_DoNotUse, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings_RenamedServicesEntry_DoNotUse, _impl_.key_),
        PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings_RenamedServicesEntry_DoNotUse, _impl_.value_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings_RenamedResourcesEntry_DoNotUse, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings_RenamedResourcesEntry_DoNotUse, _impl_.key_),
        PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings_RenamedResourcesEntry_DoNotUse, _impl_.value_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_._has_bits_),
        9, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_.common_),
        PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_.renamed_services_),
        PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_.renamed_resources_),
        PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_.ignored_resources_),
        PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_.forced_namespace_aliases_),
        PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_.handwritten_signatures_),
        3,
        4,
        5,
        0,
        1,
        2,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::api::RubySettings, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::api::RubySettings, _impl_.common_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::api::GoSettings, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::api::GoSettings, _impl_.common_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::api::MethodSettings_LongRunning, _impl_._has_bits_),
        7, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::api::MethodSettings_LongRunning, _impl_.initial_poll_delay_),
        PROTOBUF_FIELD_OFFSET(::google::api::MethodSettings_LongRunning, _impl_.poll_delay_multiplier_),
        PROTOBUF_FIELD_OFFSET(::google::api::MethodSettings_LongRunning, _impl_.max_poll_delay_),
        PROTOBUF_FIELD_OFFSET(::google::api::MethodSettings_LongRunning, _impl_.total_poll_timeout_),
        0,
        3,
        1,
        2,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::api::MethodSettings, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::api::MethodSettings, _impl_.selector_),
        PROTOBUF_FIELD_OFFSET(::google::api::MethodSettings, _impl_.long_running_),
        0,
        1,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::google::api::CommonLanguageSettings)},
        {7, sizeof(::google::api::ClientLibrarySettings)},
        {32, sizeof(::google::api::Publishing)},
        {55, sizeof(::google::api::JavaSettings_ServiceClassNamesEntry_DoNotUse)},
        {62, sizeof(::google::api::JavaSettings)},
        {71, sizeof(::google::api::CppSettings)},
        {76, sizeof(::google::api::PhpSettings)},
        {81, sizeof(::google::api::PythonSettings)},
        {86, sizeof(::google::api::NodeSettings)},
        {91, sizeof(::google::api::DotnetSettings_RenamedServicesEntry_DoNotUse)},
        {98, sizeof(::google::api::DotnetSettings_RenamedResourcesEntry_DoNotUse)},
        {105, sizeof(::google::api::DotnetSettings)},
        {120, sizeof(::google::api::RubySettings)},
        {125, sizeof(::google::api::GoSettings)},
        {130, sizeof(::google::api::MethodSettings_LongRunning)},
        {141, sizeof(::google::api::MethodSettings)},
};
static const ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const
    file_message_globals[] = {
        &::google::api::CommonLanguageSettings_globals_,
        &::google::api::ClientLibrarySettings_globals_,
        &::google::api::Publishing_globals_,
        &::google::api::JavaSettings_ServiceClassNamesEntry_DoNotUse_globals_,
        &::google::api::JavaSettings_globals_,
        &::google::api::CppSettings_globals_,
        &::google::api::PhpSettings_globals_,
        &::google::api::PythonSettings_globals_,
        &::google::api::NodeSettings_globals_,
        &::google::api::DotnetSettings_RenamedServicesEntry_DoNotUse_globals_,
        &::google::api::DotnetSettings_RenamedResourcesEntry_DoNotUse_globals_,
        &::google::api::DotnetSettings_globals_,
        &::google::api::RubySettings_globals_,
        &::google::api::GoSettings_globals_,
        &::google::api::MethodSettings_LongRunning_globals_,
        &::google::api::MethodSettings_globals_,
};
const char descriptor_table_protodef_google_2fapi_2fclient_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n\027google/api/client.proto\022\ngoogle.api\032\035g"
    "oogle/api/launch_stage.proto\032 google/pro"
    "tobuf/descriptor.proto\032\036google/protobuf/"
    "duration.proto\"t\n\026CommonLanguageSettings"
    "\022\036\n\022reference_docs_uri\030\001 \001(\tB\002\030\001\022:\n\014dest"
    "inations\030\002 \003(\0162$.google.api.ClientLibrar"
    "yDestination\"\373\003\n\025ClientLibrarySettings\022\017"
    "\n\007version\030\001 \001(\t\022-\n\014launch_stage\030\002 \001(\0162\027."
    "google.api.LaunchStage\022\032\n\022rest_numeric_e"
    "nums\030\003 \001(\010\022/\n\rjava_settings\030\025 \001(\0132\030.goog"
    "le.api.JavaSettings\022-\n\014cpp_settings\030\026 \001("
    "\0132\027.google.api.CppSettings\022-\n\014php_settin"
    "gs\030\027 \001(\0132\027.google.api.PhpSettings\0223\n\017pyt"
    "hon_settings\030\030 \001(\0132\032.google.api.PythonSe"
    "ttings\022/\n\rnode_settings\030\031 \001(\0132\030.google.a"
    "pi.NodeSettings\0223\n\017dotnet_settings\030\032 \001(\013"
    "2\032.google.api.DotnetSettings\022/\n\rruby_set"
    "tings\030\033 \001(\0132\030.google.api.RubySettings\022+\n"
    "\013go_settings\030\034 \001(\0132\026.google.api.GoSettin"
    "gs\"\376\002\n\nPublishing\0223\n\017method_settings\030\002 \003"
    "(\0132\032.google.api.MethodSettings\022\025\n\rnew_is"
    "sue_uri\030e \001(\t\022\031\n\021documentation_uri\030f \001(\t"
    "\022\026\n\016api_short_name\030g \001(\t\022\024\n\014github_label"
    "\030h \001(\t\022\036\n\026codeowner_github_teams\030i \003(\t\022\026"
    "\n\016doc_tag_prefix\030j \001(\t\022;\n\014organization\030k"
    " \001(\0162%.google.api.ClientLibraryOrganizat"
    "ion\022;\n\020library_settings\030m \003(\0132!.google.a"
    "pi.ClientLibrarySettings\022)\n!proto_refere"
    "nce_documentation_uri\030n \001(\t\"\343\001\n\014JavaSett"
    "ings\022\027\n\017library_package\030\001 \001(\t\022L\n\023service"
    "_class_names\030\002 \003(\0132/.google.api.JavaSett"
    "ings.ServiceClassNamesEntry\0222\n\006common\030\003 "
    "\001(\0132\".google.api.CommonLanguageSettings\032"
    "8\n\026ServiceClassNamesEntry\022\013\n\003key\030\001 \001(\t\022\r"
    "\n\005value\030\002 \001(\t:\0028\001\"A\n\013CppSettings\0222\n\006comm"
    "on\030\001 \001(\0132\".google.api.CommonLanguageSett"
    "ings\"A\n\013PhpSettings\0222\n\006common\030\001 \001(\0132\".go"
    "ogle.api.CommonLanguageSettings\"D\n\016Pytho"
    "nSettings\0222\n\006common\030\001 \001(\0132\".google.api.C"
    "ommonLanguageSettings\"B\n\014NodeSettings\0222\n"
    "\006common\030\001 \001(\0132\".google.api.CommonLanguag"
    "eSettings\"\252\003\n\016DotnetSettings\0222\n\006common\030\001"
    " \001(\0132\".google.api.CommonLanguageSettings"
    "\022I\n\020renamed_services\030\002 \003(\0132/.google.api."
    "DotnetSettings.RenamedServicesEntry\022K\n\021r"
    "enamed_resources\030\003 \003(\01320.google.api.Dotn"
    "etSettings.RenamedResourcesEntry\022\031\n\021igno"
    "red_resources\030\004 \003(\t\022 \n\030forced_namespace_"
    "aliases\030\005 \003(\t\022\036\n\026handwritten_signatures\030"
    "\006 \003(\t\0326\n\024RenamedServicesEntry\022\013\n\003key\030\001 \001"
    "(\t\022\r\n\005value\030\002 \001(\t:\0028\001\0327\n\025RenamedResource"
    "sEntry\022\013\n\003key\030\001 \001(\t\022\r\n\005value\030\002 \001(\t:\0028\001\"B"
    "\n\014RubySettings\0222\n\006common\030\001 \001(\0132\".google."
    "api.CommonLanguageSettings\"@\n\nGoSettings"
    "\0222\n\006common\030\001 \001(\0132\".google.api.CommonLang"
    "uageSettings\"\260\002\n\016MethodSettings\022\020\n\010selec"
    "tor\030\001 \001(\t\022<\n\014long_running\030\002 \001(\0132&.google"
    ".api.MethodSettings.LongRunning\032\315\001\n\013Long"
    "Running\0225\n\022initial_poll_delay\030\001 \001(\0132\031.go"
    "ogle.protobuf.Duration\022\035\n\025poll_delay_mul"
    "tiplier\030\002 \001(\002\0221\n\016max_poll_delay\030\003 \001(\0132\031."
    "google.protobuf.Duration\0225\n\022total_poll_t"
    "imeout\030\004 \001(\0132\031.google.protobuf.Duration*"
    "\243\001\n\031ClientLibraryOrganization\022+\n\'CLIENT_"
    "LIBRARY_ORGANIZATION_UNSPECIFIED\020\000\022\t\n\005CL"
    "OUD\020\001\022\007\n\003ADS\020\002\022\n\n\006PHOTOS\020\003\022\017\n\013STREET_VIE"
    "W\020\004\022\014\n\010SHOPPING\020\005\022\007\n\003GEO\020\006\022\021\n\rGENERATIVE"
    "_AI\020\007*g\n\030ClientLibraryDestination\022*\n&CLI"
    "ENT_LIBRARY_DESTINATION_UNSPECIFIED\020\000\022\n\n"
    "\006GITHUB\020\n\022\023\n\017PACKAGE_MANAGER\020\024:9\n\020method"
    "_signature\022\036.google.protobuf.MethodOptio"
    "ns\030\233\010 \003(\t:6\n\014default_host\022\037.google.proto"
    "buf.ServiceOptions\030\231\010 \001(\t:6\n\014oauth_scope"
    "s\022\037.google.protobuf.ServiceOptions\030\232\010 \001("
    "\tBi\n\016com.google.apiB\013ClientProtoP\001ZAgoog"
    "le.golang.org/genproto/googleapis/api/an"
    "notations;annotations\242\002\004GAPIb\006proto3"
};
static const ::_pbi::DescriptorTable* PROTOBUF_NONNULL const
    descriptor_table_google_2fapi_2fclient_2eproto_deps[3] = {
        &::descriptor_table_google_2fapi_2flaunch_5fstage_2eproto,
        &::descriptor_table_google_2fprotobuf_2fdescriptor_2eproto,
        &::descriptor_table_google_2fprotobuf_2fduration_2eproto,
};
static ::absl::once_flag descriptor_table_google_2fapi_2fclient_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_google_2fapi_2fclient_2eproto = {
    false,
    false,
    3076,
    descriptor_table_protodef_google_2fapi_2fclient_2eproto,
    "google/api/client.proto",
    &descriptor_table_google_2fapi_2fclient_2eproto_once,
    descriptor_table_google_2fapi_2fclient_2eproto_deps,
    3,
    16,
    schemas,
    file_message_globals,
    TableStruct_google_2fapi_2fclient_2eproto::offsets,
    file_level_enum_descriptors_google_2fapi_2fclient_2eproto,
    file_level_service_descriptors_google_2fapi_2fclient_2eproto,
};
namespace google {
namespace api {
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
ClientLibraryOrganization_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fapi_2fclient_2eproto);
  return file_level_enum_descriptors_google_2fapi_2fclient_2eproto[0];
}
PROTOBUF_CONSTINIT const uint32_t ClientLibraryOrganization_internal_data_[] = {
    524288u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
ClientLibraryDestination_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fapi_2fclient_2eproto);
  return file_level_enum_descriptors_google_2fapi_2fclient_2eproto[1];
}
PROTOBUF_CONSTINIT const uint32_t ClientLibraryDestination_internal_data_[] = {
    65536u, 32u, 524800u, };
// ===================================================================

CommonLanguageSettings::CommonLanguageSettings(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, CommonLanguageSettings_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.api.CommonLanguageSettings)
}
PROTOBUF_NDEBUG_INLINE CommonLanguageSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::api::CommonLanguageSettings& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        destinations_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::api::CommonLanguageSettings,
              PROTOBUF_FIELD_OFFSET(::google::api::CommonLanguageSettings, _impl_.destinations_)>()
          , from.destinations_
        }
        ,
        _destinations_cached_byte_size_{0},
        reference_docs_uri_(arena, from.reference_docs_uri_) {}

CommonLanguageSettings::CommonLanguageSettings(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const CommonLanguageSettings& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, CommonLanguageSettings_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  CommonLanguageSettings* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.api.CommonLanguageSettings)
}
PROTOBUF_NDEBUG_INLINE CommonLanguageSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        destinations_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::CommonLanguageSettings,
            PROTOBUF_FIELD_OFFSET(::google::api::CommonLanguageSettings, _impl_.destinations_)>()
         }
        ,
        _destinations_cached_byte_size_{0},
        reference_docs_uri_(arena) {}

inline void CommonLanguageSettings::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
CommonLanguageSettings::~CommonLanguageSettings() {
  // @@protoc_insertion_point(destructor:google.api.CommonLanguageSettings)
  SharedDtor(*this);
}
inline void CommonLanguageSettings::SharedDtor(MessageLite& self) {
  CommonLanguageSettings& this_ = static_cast<CommonLanguageSettings&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.reference_docs_uri_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull CommonLanguageSettings_class_data_ =
        CommonLanguageSettings::InternalGenerateClassData_(CommonLanguageSettings_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CommonLanguageSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CommonLanguageSettings_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(CommonLanguageSettings_class_data_.tc_table);
  return CommonLanguageSettings_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CommonLanguageSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CommonLanguageSettings_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&CommonLanguageSettings_globals_));
  return CommonLanguageSettings_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const CommonLanguageSettings::ParseTableT_
    CommonLanguageSettings::_table_ =
        CommonLanguageSettings::InternalGenerateParseTable_(CommonLanguageSettings_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void CommonLanguageSettings::Clear() {
// @@protoc_insertion_point(message_clear_start:google.api.CommonLanguageSettings)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.destinations_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.reference_docs_uri_.ClearNonDefaultToEmpty();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL CommonLanguageSettings::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const CommonLanguageSettings& this_ = static_cast<const CommonLanguageSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL CommonLanguageSettings::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const CommonLanguageSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.api.CommonLanguageSettings)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string reference_docs_uri = 1 [deprecated = true];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_reference_docs_uri().empty()) {
      const ::std::string& _s = this_._internal_reference_docs_uri();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.CommonLanguageSettings.reference_docs_uri");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // repeated .google.api.ClientLibraryDestination destinations = 2;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    {
      ::size_t byte_size = this_._impl_._destinations_cached_byte_size_.Get();
      if (byte_size > 0) {
        target = stream->WriteEnumPacked(
            2, this_._internal_destinations(), byte_size, target);
      }
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.api.CommonLanguageSettings)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t CommonLanguageSettings::ByteSizeLong(const MessageLite& base) {
  const CommonLanguageSettings& this_ = static_cast<const CommonLanguageSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t CommonLanguageSettings::ByteSizeLong() const {
  const CommonLanguageSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.api.CommonLanguageSettings)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // repeated .google.api.ClientLibraryDestination destinations = 2;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += ::_pbi::WireFormatLite::EnumSizeWithPackedTagSize(
          this_._internal_destinations(), 1, this_._impl_._destinations_cached_byte_size_);
    }
    // string reference_docs_uri = 1 [deprecated = true];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_reference_docs_uri().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_reference_docs_uri());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void CommonLanguageSettings::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<CommonLanguageSettings*>(&to_msg);
  auto& from = static_cast<const CommonLanguageSettings&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.api.CommonLanguageSettings)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_destinations()->MergeFrom(from._internal_destinations());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_reference_docs_uri().empty()) {
        _this->_internal_set_reference_docs_uri(from._internal_reference_docs_uri());
      } else {
        if (_this->_impl_.reference_docs_uri_.IsDefault()) {
          _this->_internal_set_reference_docs_uri("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void CommonLanguageSettings::CopyFrom(const CommonLanguageSettings& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.api.CommonLanguageSettings)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void CommonLanguageSettings::InternalSwap(CommonLanguageSettings* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.destinations_.InternalSwap(&other->_impl_.destinations_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.reference_docs_uri_, &other->_impl_.reference_docs_uri_, arena);
}

::google::protobuf::Metadata CommonLanguageSettings::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ClientLibrarySettings::ClientLibrarySettings(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ClientLibrarySettings_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.api.ClientLibrarySettings)
}
PROTOBUF_NDEBUG_INLINE ClientLibrarySettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::api::ClientLibrarySettings& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        version_(arena, from.version_) {}

ClientLibrarySettings::ClientLibrarySettings(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ClientLibrarySettings& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ClientLibrarySettings_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ClientLibrarySettings* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.java_settings_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.java_settings_)
                : nullptr;
  _impl_.cpp_settings_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.cpp_settings_)
                : nullptr;
  _impl_.php_settings_ = (CheckHasBit(cached_has_bits, 0x00000008U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.php_settings_)
                : nullptr;
  _impl_.python_settings_ = (CheckHasBit(cached_has_bits, 0x00000010U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.python_settings_)
                : nullptr;
  _impl_.node_settings_ = (CheckHasBit(cached_has_bits, 0x00000020U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.node_settings_)
                : nullptr;
  _impl_.dotnet_settings_ = (CheckHasBit(cached_has_bits, 0x00000040U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.dotnet_settings_)
                : nullptr;
  _impl_.ruby_settings_ = (CheckHasBit(cached_has_bits, 0x00000080U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.ruby_settings_)
                : nullptr;
  _impl_.go_settings_ = (CheckHasBit(cached_has_bits, 0x00000100U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.go_settings_)
                : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, launch_stage_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, launch_stage_),
           offsetof(Impl_, rest_numeric_enums_) -
               offsetof(Impl_, launch_stage_) +
               sizeof(Impl_::rest_numeric_enums_));

  // @@protoc_insertion_point(copy_constructor:google.api.ClientLibrarySettings)
}
PROTOBUF_NDEBUG_INLINE ClientLibrarySettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        version_(arena) {}

inline void ClientLibrarySettings::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, java_settings_),
           0,
           offsetof(Impl_, rest_numeric_enums_) -
               offsetof(Impl_, java_settings_) +
               sizeof(Impl_::rest_numeric_enums_));
}
ClientLibrarySettings::~ClientLibrarySettings() {
  // @@protoc_insertion_point(destructor:google.api.ClientLibrarySettings)
  SharedDtor(*this);
}
inline void ClientLibrarySettings::SharedDtor(MessageLite& self) {
  ClientLibrarySettings& this_ = static_cast<ClientLibrarySettings&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.version_.Destroy();
  delete this_._impl_.java_settings_;
  delete this_._impl_.cpp_settings_;
  delete this_._impl_.php_settings_;
  delete this_._impl_.python_settings_;
  delete this_._impl_.node_settings_;
  delete this_._impl_.dotnet_settings_;
  delete this_._impl_.ruby_settings_;
  delete this_._impl_.go_settings_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ClientLibrarySettings_class_data_ =
        ClientLibrarySettings::InternalGenerateClassData_(ClientLibrarySettings_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ClientLibrarySettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ClientLibrarySettings_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ClientLibrarySettings_class_data_.tc_table);
  return ClientLibrarySettings_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ClientLibrarySettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ClientLibrarySettings_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ClientLibrarySettings_globals_));
  return ClientLibrarySettings_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ClientLibrarySettings::ParseTableT_
    ClientLibrarySettings::_table_ =
        ClientLibrarySettings::InternalGenerateParseTable_(ClientLibrarySettings_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ClientLibrarySettings::Clear() {
// @@protoc_insertion_point(message_clear_start:google.api.ClientLibrarySettings)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.version_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.java_settings_ != nullptr);
      _impl_.java_settings_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(_impl_.cpp_settings_ != nullptr);
      _impl_.cpp_settings_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(_impl_.php_settings_ != nullptr);
      _impl_.php_settings_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      ABSL_DCHECK(_impl_.python_settings_ != nullptr);
      _impl_.python_settings_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      ABSL_DCHECK(_impl_.node_settings_ != nullptr);
      _impl_.node_settings_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      ABSL_DCHECK(_impl_.dotnet_settings_ != nullptr);
      _impl_.dotnet_settings_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      ABSL_DCHECK(_impl_.ruby_settings_ != nullptr);
      _impl_.ruby_settings_->Clear();
    }
  }
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    ABSL_DCHECK(_impl_.go_settings_ != nullptr);
    _impl_.go_settings_->Clear();
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000600U)) {
    ::memset(&_impl_.launch_stage_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.rest_numeric_enums_) -
        reinterpret_cast<char*>(&_impl_.launch_stage_)) + sizeof(_impl_.rest_numeric_enums_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ClientLibrarySettings::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ClientLibrarySettings& this_ = static_cast<const ClientLibrarySettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ClientLibrarySettings::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ClientLibrarySettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.api.ClientLibrarySettings)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string version = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_version().empty()) {
      const ::std::string& _s = this_._internal_version();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.ClientLibrarySettings.version");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // .google.api.LaunchStage launch_stage = 2;
  if (CheckHasBit(cached_has_bits, 0x00000200U)) {
    if (this_._internal_launch_stage() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          2, this_._internal_launch_stage(), target);
    }
  }

  // bool rest_numeric_enums = 3;
  if (CheckHasBit(cached_has_bits, 0x00000400U)) {
    if (this_._internal_rest_numeric_enums() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          3, this_._internal_rest_numeric_enums(), target);
    }
  }

  // .google.api.JavaSettings java_settings = 21;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        21, *this_._impl_.java_settings_, this_._impl_.java_settings_->GetCachedSize(), target,
        stream);
  }

  // .google.api.CppSettings cpp_settings = 22;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        22, *this_._impl_.cpp_settings_, this_._impl_.cpp_settings_->GetCachedSize(), target,
        stream);
  }

  // .google.api.PhpSettings php_settings = 23;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        23, *this_._impl_.php_settings_, this_._impl_.php_settings_->GetCachedSize(), target,
        stream);
  }

  // .google.api.PythonSettings python_settings = 24;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        24, *this_._impl_.python_settings_, this_._impl_.python_settings_->GetCachedSize(), target,
        stream);
  }

  // .google.api.NodeSettings node_settings = 25;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        25, *this_._impl_.node_settings_, this_._impl_.node_settings_->GetCachedSize(), target,
        stream);
  }

  // .google.api.DotnetSettings dotnet_settings = 26;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        26, *this_._impl_.dotnet_settings_, this_._impl_.dotnet_settings_->GetCachedSize(), target,
        stream);
  }

  // .google.api.RubySettings ruby_settings = 27;
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        27, *this_._impl_.ruby_settings_, this_._impl_.ruby_settings_->GetCachedSize(), target,
        stream);
  }

  // .google.api.GoSettings go_settings = 28;
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        28, *this_._impl_.go_settings_, this_._impl_.go_settings_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.api.ClientLibrarySettings)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ClientLibrarySettings::ByteSizeLong(const MessageLite& base) {
  const ClientLibrarySettings& this_ = static_cast<const ClientLibrarySettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ClientLibrarySettings::ByteSizeLong() const {
  const ClientLibrarySettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.api.ClientLibrarySettings)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // string version = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_version().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_version());
      }
    }
    // .google.api.JavaSettings java_settings = 21;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 2 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.java_settings_);
    }
    // .google.api.CppSettings cpp_settings = 22;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 2 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.cpp_settings_);
    }
    // .google.api.PhpSettings php_settings = 23;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += 2 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.php_settings_);
    }
    // .google.api.PythonSettings python_settings = 24;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      total_size += 2 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.python_settings_);
    }
    // .google.api.NodeSettings node_settings = 25;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      total_size += 2 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.node_settings_);
    }
    // .google.api.DotnetSettings dotnet_settings = 26;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      total_size += 2 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.dotnet_settings_);
    }
    // .google.api.RubySettings ruby_settings = 27;
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      total_size += 2 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.ruby_settings_);
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000700U)) {
    // .google.api.GoSettings go_settings = 28;
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      total_size += 2 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.go_settings_);
    }
    // .google.api.LaunchStage launch_stage = 2;
    if (CheckHasBit(cached_has_bits, 0x00000200U)) {
      if (this_._internal_launch_stage() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_launch_stage());
      }
    }
    // bool rest_numeric_enums = 3;
    if (CheckHasBit(cached_has_bits, 0x00000400U)) {
      if (this_._internal_rest_numeric_enums() != 0) {
        total_size += 2;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ClientLibrarySettings::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ClientLibrarySettings*>(&to_msg);
  auto& from = static_cast<const ClientLibrarySettings&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.api.ClientLibrarySettings)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_version().empty()) {
        _this->_internal_set_version(from._internal_version());
      } else {
        if (_this->_impl_.version_.IsDefault()) {
          _this->_internal_set_version("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.java_settings_ != nullptr);
      if (_this->_impl_.java_settings_ == nullptr) {
        _this->_impl_.java_settings_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.java_settings_);
      } else {
        _this->_impl_.java_settings_->MergeFrom(*from._impl_.java_settings_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.cpp_settings_ != nullptr);
      if (_this->_impl_.cpp_settings_ == nullptr) {
        _this->_impl_.cpp_settings_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.cpp_settings_);
      } else {
        _this->_impl_.cpp_settings_->MergeFrom(*from._impl_.cpp_settings_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(from._impl_.php_settings_ != nullptr);
      if (_this->_impl_.php_settings_ == nullptr) {
        _this->_impl_.php_settings_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.php_settings_);
      } else {
        _this->_impl_.php_settings_->MergeFrom(*from._impl_.php_settings_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      ABSL_DCHECK(from._impl_.python_settings_ != nullptr);
      if (_this->_impl_.python_settings_ == nullptr) {
        _this->_impl_.python_settings_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.python_settings_);
      } else {
        _this->_impl_.python_settings_->MergeFrom(*from._impl_.python_settings_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      ABSL_DCHECK(from._impl_.node_settings_ != nullptr);
      if (_this->_impl_.node_settings_ == nullptr) {
        _this->_impl_.node_settings_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.node_settings_);
      } else {
        _this->_impl_.node_settings_->MergeFrom(*from._impl_.node_settings_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      ABSL_DCHECK(from._impl_.dotnet_settings_ != nullptr);
      if (_this->_impl_.dotnet_settings_ == nullptr) {
        _this->_impl_.dotnet_settings_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.dotnet_settings_);
      } else {
        _this->_impl_.dotnet_settings_->MergeFrom(*from._impl_.dotnet_settings_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      ABSL_DCHECK(from._impl_.ruby_settings_ != nullptr);
      if (_this->_impl_.ruby_settings_ == nullptr) {
        _this->_impl_.ruby_settings_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.ruby_settings_);
      } else {
        _this->_impl_.ruby_settings_->MergeFrom(*from._impl_.ruby_settings_);
      }
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000700U)) {
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      ABSL_DCHECK(from._impl_.go_settings_ != nullptr);
      if (_this->_impl_.go_settings_ == nullptr) {
        _this->_impl_.go_settings_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.go_settings_);
      } else {
        _this->_impl_.go_settings_->MergeFrom(*from._impl_.go_settings_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000200U)) {
      if (from._internal_launch_stage() != 0) {
        _this->_impl_.launch_stage_ = from._impl_.launch_stage_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000400U)) {
      if (from._internal_rest_numeric_enums() != 0) {
        _this->_impl_.rest_numeric_enums_ = from._impl_.rest_numeric_enums_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ClientLibrarySettings::CopyFrom(const ClientLibrarySettings& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.api.ClientLibrarySettings)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ClientLibrarySettings::InternalSwap(ClientLibrarySettings* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.version_, &other->_impl_.version_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.rest_numeric_enums_)
      + sizeof(ClientLibrarySettings::_impl_.rest_numeric_enums_)
      - PROTOBUF_FIELD_OFFSET(ClientLibrarySettings, _impl_.java_settings_)>(
          reinterpret_cast<char*>(&_impl_.java_settings_),
          reinterpret_cast<char*>(&other->_impl_.java_settings_));
}

::google::protobuf::Metadata ClientLibrarySettings::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

Publishing::Publishing(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Publishing_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.api.Publishing)
}
PROTOBUF_NDEBUG_INLINE Publishing::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::api::Publishing& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        method_settings_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::api::Publishing,
              PROTOBUF_FIELD_OFFSET(::google::api::Publishing, _impl_.method_settings_)>()
          , from.method_settings_
        }
        ,
        codeowner_github_teams_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::api::Publishing,
              PROTOBUF_FIELD_OFFSET(::google::api::Publishing, _impl_.codeowner_github_teams_)>()
          , from.codeowner_github_teams_
        }
        ,
        library_settings_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::api::Publishing,
              PROTOBUF_FIELD_OFFSET(::google::api::Publishing, _impl_.library_settings_)>()
          , from.library_settings_
        }
        ,
        new_issue_uri_(arena, from.new_issue_uri_),
        documentation_uri_(arena, from.documentation_uri_),
        api_short_name_(arena, from.api_short_name_),
        github_label_(arena, from.github_label_),
        doc_tag_prefix_(arena, from.doc_tag_prefix_),
        proto_reference_documentation_uri_(arena, from.proto_reference_documentation_uri_) {}

Publishing::Publishing(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Publishing& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Publishing_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Publishing* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.organization_ = from._impl_.organization_;

  // @@protoc_insertion_point(copy_constructor:google.api.Publishing)
}
PROTOBUF_NDEBUG_INLINE Publishing::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        method_settings_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::Publishing,
            PROTOBUF_FIELD_OFFSET(::google::api::Publishing, _impl_.method_settings_)>()
         }
        ,
        codeowner_github_teams_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::Publishing,
            PROTOBUF_FIELD_OFFSET(::google::api::Publishing, _impl_.codeowner_github_teams_)>()
         }
        ,
        library_settings_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::Publishing,
            PROTOBUF_FIELD_OFFSET(::google::api::Publishing, _impl_.library_settings_)>()
         }
        ,
        new_issue_uri_(arena),
        documentation_uri_(arena),
        api_short_name_(arena),
        github_label_(arena),
        doc_tag_prefix_(arena),
        proto_reference_documentation_uri_(arena) {}

inline void Publishing::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.organization_ = {};
}
Publishing::~Publishing() {
  // @@protoc_insertion_point(destructor:google.api.Publishing)
  SharedDtor(*this);
}
inline void Publishing::SharedDtor(MessageLite& self) {
  Publishing& this_ = static_cast<Publishing&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.new_issue_uri_.Destroy();
  this_._impl_.documentation_uri_.Destroy();
  this_._impl_.api_short_name_.Destroy();
  this_._impl_.github_label_.Destroy();
  this_._impl_.doc_tag_prefix_.Destroy();
  this_._impl_.proto_reference_documentation_uri_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Publishing_class_data_ =
        Publishing::InternalGenerateClassData_(Publishing_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Publishing::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Publishing_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Publishing_class_data_.tc_table);
  return Publishing_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Publishing::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Publishing_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Publishing_globals_));
  return Publishing_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Publishing::ParseTableT_
    Publishing::_table_ =
        Publishing::InternalGenerateParseTable_(Publishing_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Publishing::Clear() {
// @@protoc_insertion_point(message_clear_start:google.api.Publishing)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.method_settings_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.codeowner_github_teams_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.library_settings_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _impl_.new_issue_uri_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      _impl_.documentation_uri_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      _impl_.api_short_name_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      _impl_.github_label_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      _impl_.doc_tag_prefix_.ClearNonDefaultToEmpty();
    }
  }
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    _impl_.proto_reference_documentation_uri_.ClearNonDefaultToEmpty();
  }
  _impl_.organization_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Publishing::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Publishing& this_ = static_cast<const Publishing&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Publishing::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Publishing& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.api.Publishing)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated .google.api.MethodSettings method_settings = 2;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_method_settings_size());
         i < n; i++) {
      const auto& repfield = this_._internal_method_settings().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              2, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  // string new_issue_uri = 101;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (!this_._internal_new_issue_uri().empty()) {
      const ::std::string& _s = this_._internal_new_issue_uri();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.Publishing.new_issue_uri");
      target = stream->WriteStringMaybeAliased(101, _s, target);
    }
  }

  // string documentation_uri = 102;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (!this_._internal_documentation_uri().empty()) {
      const ::std::string& _s = this_._internal_documentation_uri();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.Publishing.documentation_uri");
      target = stream->WriteStringMaybeAliased(102, _s, target);
    }
  }

  // string api_short_name = 103;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (!this_._internal_api_short_name().empty()) {
      const ::std::string& _s = this_._internal_api_short_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.Publishing.api_short_name");
      target = stream->WriteStringMaybeAliased(103, _s, target);
    }
  }

  // string github_label = 104;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (!this_._internal_github_label().empty()) {
      const ::std::string& _s = this_._internal_github_label();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.Publishing.github_label");
      target = stream->WriteStringMaybeAliased(104, _s, target);
    }
  }

  // repeated string codeowner_github_teams = 105;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    for (int i = 0, n = this_._internal_codeowner_github_teams_size(); i < n; ++i) {
      const auto& s = this_._internal_codeowner_github_teams().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.Publishing.codeowner_github_teams");
      target = stream->WriteString(105, s, target);
    }
  }

  // string doc_tag_prefix = 106;
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    if (!this_._internal_doc_tag_prefix().empty()) {
      const ::std::string& _s = this_._internal_doc_tag_prefix();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.Publishing.doc_tag_prefix");
      target = stream->WriteStringMaybeAliased(106, _s, target);
    }
  }

  // .google.api.ClientLibraryOrganization organization = 107;
  if (CheckHasBit(cached_has_bits, 0x00000200U)) {
    if (this_._internal_organization() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          107, this_._internal_organization(), target);
    }
  }

  // repeated .google.api.ClientLibrarySettings library_settings = 109;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_library_settings_size());
         i < n; i++) {
      const auto& repfield = this_._internal_library_settings().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              109, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  // string proto_reference_documentation_uri = 110;
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    if (!this_._internal_proto_reference_documentation_uri().empty()) {
      const ::std::string& _s = this_._internal_proto_reference_documentation_uri();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.Publishing.proto_reference_documentation_uri");
      target = stream->WriteStringMaybeAliased(110, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.api.Publishing)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Publishing::ByteSizeLong(const MessageLite& base) {
  const Publishing& this_ = static_cast<const Publishing&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Publishing::ByteSizeLong() const {
  const Publishing& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.api.Publishing)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // repeated .google.api.MethodSettings method_settings = 2;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_method_settings_size();
      for (const auto& msg : this_._internal_method_settings()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
    // repeated string codeowner_github_teams = 105;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size +=
          2 * ::google::protobuf::internal::FromIntSize(this_._internal_codeowner_github_teams().size());
      for (int i = 0, n = this_._internal_codeowner_github_teams().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_codeowner_github_teams().Get(i));
      }
    }
    // repeated .google.api.ClientLibrarySettings library_settings = 109;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 2UL * this_._internal_library_settings_size();
      for (const auto& msg : this_._internal_library_settings()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
    // string new_issue_uri = 101;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!this_._internal_new_issue_uri().empty()) {
        total_size += 2 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_new_issue_uri());
      }
    }
    // string documentation_uri = 102;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (!this_._internal_documentation_uri().empty()) {
        total_size += 2 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_documentation_uri());
      }
    }
    // string api_short_name = 103;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (!this_._internal_api_short_name().empty()) {
        total_size += 2 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_api_short_name());
      }
    }
    // string github_label = 104;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (!this_._internal_github_label().empty()) {
        total_size += 2 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_github_label());
      }
    }
    // string doc_tag_prefix = 106;
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (!this_._internal_doc_tag_prefix().empty()) {
        total_size += 2 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_doc_tag_prefix());
      }
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000300U)) {
    // string proto_reference_documentation_uri = 110;
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      if (!this_._internal_proto_reference_documentation_uri().empty()) {
        total_size += 2 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_proto_reference_documentation_uri());
      }
    }
    // .google.api.ClientLibraryOrganization organization = 107;
    if (CheckHasBit(cached_has_bits, 0x00000200U)) {
      if (this_._internal_organization() != 0) {
        total_size += 2 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_organization());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Publishing::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Publishing*>(&to_msg);
  auto& from = static_cast<const Publishing&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.api.Publishing)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_method_settings()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_method_settings());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _this->_internal_mutable_codeowner_github_teams()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_codeowner_github_teams());
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _this->_internal_mutable_library_settings()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_library_settings());
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!from._internal_new_issue_uri().empty()) {
        _this->_internal_set_new_issue_uri(from._internal_new_issue_uri());
      } else {
        if (_this->_impl_.new_issue_uri_.IsDefault()) {
          _this->_internal_set_new_issue_uri("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (!from._internal_documentation_uri().empty()) {
        _this->_internal_set_documentation_uri(from._internal_documentation_uri());
      } else {
        if (_this->_impl_.documentation_uri_.IsDefault()) {
          _this->_internal_set_documentation_uri("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (!from._internal_api_short_name().empty()) {
        _this->_internal_set_api_short_name(from._internal_api_short_name());
      } else {
        if (_this->_impl_.api_short_name_.IsDefault()) {
          _this->_internal_set_api_short_name("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (!from._internal_github_label().empty()) {
        _this->_internal_set_github_label(from._internal_github_label());
      } else {
        if (_this->_impl_.github_label_.IsDefault()) {
          _this->_internal_set_github_label("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (!from._internal_doc_tag_prefix().empty()) {
        _this->_internal_set_doc_tag_prefix(from._internal_doc_tag_prefix());
      } else {
        if (_this->_impl_.doc_tag_prefix_.IsDefault()) {
          _this->_internal_set_doc_tag_prefix("");
        }
      }
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000300U)) {
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      if (!from._internal_proto_reference_documentation_uri().empty()) {
        _this->_internal_set_proto_reference_documentation_uri(from._internal_proto_reference_documentation_uri());
      } else {
        if (_this->_impl_.proto_reference_documentation_uri_.IsDefault()) {
          _this->_internal_set_proto_reference_documentation_uri("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000200U)) {
      if (from._internal_organization() != 0) {
        _this->_impl_.organization_ = from._impl_.organization_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Publishing::CopyFrom(const Publishing& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.api.Publishing)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Publishing::InternalSwap(Publishing* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.method_settings_.InternalSwap(&other->_impl_.method_settings_);
  _impl_.codeowner_github_teams_.InternalSwap(&other->_impl_.codeowner_github_teams_);
  _impl_.library_settings_.InternalSwap(&other->_impl_.library_settings_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.new_issue_uri_, &other->_impl_.new_issue_uri_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.documentation_uri_, &other->_impl_.documentation_uri_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.api_short_name_, &other->_impl_.api_short_name_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.github_label_, &other->_impl_.github_label_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.doc_tag_prefix_, &other->_impl_.doc_tag_prefix_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.proto_reference_documentation_uri_, &other->_impl_.proto_reference_documentation_uri_, arena);
  swap(_impl_.organization_, other->_impl_.organization_);
}

::google::protobuf::Metadata Publishing::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

#if defined(PROTOBUF_CUSTOM_VTABLE)
JavaSettings_ServiceClassNamesEntry_DoNotUse::JavaSettings_ServiceClassNamesEntry_DoNotUse() : SuperType(JavaSettings_ServiceClassNamesEntry_DoNotUse_get_class_data()) {}
JavaSettings_ServiceClassNamesEntry_DoNotUse::JavaSettings_ServiceClassNamesEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
    : SuperType(arena, JavaSettings_ServiceClassNamesEntry_DoNotUse_get_class_data()) {}
#else   // PROTOBUF_CUSTOM_VTABLE
JavaSettings_ServiceClassNamesEntry_DoNotUse::JavaSettings_ServiceClassNamesEntry_DoNotUse() : SuperType() {}
JavaSettings_ServiceClassNamesEntry_DoNotUse::JavaSettings_ServiceClassNamesEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena) : SuperType(arena) {}
#endif  // PROTOBUF_CUSTOM_VTABLE
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull JavaSettings_ServiceClassNamesEntry_DoNotUse_class_data_ =
        JavaSettings_ServiceClassNamesEntry_DoNotUse::InternalGenerateClassData_(JavaSettings_ServiceClassNamesEntry_DoNotUse_globals_._default);

// including ~ all the vtables and everything they use become part
// of the same ~ SCC.
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
JavaSettings_ServiceClassNamesEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&JavaSettings_ServiceClassNamesEntry_DoNotUse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(JavaSettings_ServiceClassNamesEntry_DoNotUse_class_data_.tc_table);
  return JavaSettings_ServiceClassNamesEntry_DoNotUse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
JavaSettings_ServiceClassNamesEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&JavaSettings_ServiceClassNamesEntry_DoNotUse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&JavaSettings_ServiceClassNamesEntry_DoNotUse_globals_));
  return JavaSettings_ServiceClassNamesEntry_DoNotUse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const JavaSettings_ServiceClassNamesEntry_DoNotUse::ParseTableT_
    JavaSettings_ServiceClassNamesEntry_DoNotUse::_table_ =
        JavaSettings_ServiceClassNamesEntry_DoNotUse::InternalGenerateParseTable_(JavaSettings_ServiceClassNamesEntry_DoNotUse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
// ===================================================================

JavaSettings::JavaSettings(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, JavaSettings_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.api.JavaSettings)
}
PROTOBUF_NDEBUG_INLINE JavaSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::api::JavaSettings& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        library_package_(arena, from.library_package_),
        service_class_names_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::api::JavaSettings,
              PROTOBUF_FIELD_OFFSET(::google::api::JavaSettings, _impl_.service_class_names_)>()
          , from.service_class_names_
        }
     {}

JavaSettings::JavaSettings(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const JavaSettings& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, JavaSettings_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  JavaSettings* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.common_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.common_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.api.JavaSettings)
}
PROTOBUF_NDEBUG_INLINE JavaSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        library_package_(arena),
        service_class_names_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::JavaSettings,
            PROTOBUF_FIELD_OFFSET(::google::api::JavaSettings, _impl_.service_class_names_)>()
         }
     {}

inline void JavaSettings::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.common_ = {};
}
JavaSettings::~JavaSettings() {
  // @@protoc_insertion_point(destructor:google.api.JavaSettings)
  SharedDtor(*this);
}
inline void JavaSettings::SharedDtor(MessageLite& self) {
  JavaSettings& this_ = static_cast<JavaSettings&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.library_package_.Destroy();
  delete this_._impl_.common_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull JavaSettings_class_data_ =
        JavaSettings::InternalGenerateClassData_(JavaSettings_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
JavaSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&JavaSettings_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(JavaSettings_class_data_.tc_table);
  return JavaSettings_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
JavaSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&JavaSettings_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&JavaSettings_globals_));
  return JavaSettings_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const JavaSettings::ParseTableT_
    JavaSettings::_table_ =
        JavaSettings::InternalGenerateParseTable_(JavaSettings_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void JavaSettings::Clear() {
// @@protoc_insertion_point(message_clear_start:google.api.JavaSettings)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.library_package_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.common_ != nullptr);
      _impl_.common_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.service_class_names_.Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL JavaSettings::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const JavaSettings& this_ = static_cast<const JavaSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL JavaSettings::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const JavaSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.api.JavaSettings)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string library_package = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_library_package().empty()) {
      const ::std::string& _s = this_._internal_library_package();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.JavaSettings.library_package");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // map<string, string> service_class_names = 2;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_service_class_names().empty()) {
      using MapType = ::google::protobuf::Map<::std::string, ::std::string>;
      using WireHelper = _pbi::MapEntryFuncs<::std::string, ::std::string,
                                     _pbi::WireFormatLite::TYPE_STRING,
                                     _pbi::WireFormatLite::TYPE_STRING>;
      const auto& field = this_._internal_service_class_names();

      if (stream->IsSerializationDeterministic() && field.size() > 1) {
        for (const auto& entry : ::google::protobuf::internal::MapSorterPtr<MapType>(field)) {
          target = WireHelper::InternalSerialize(
              2, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.JavaSettings.service_class_names");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.JavaSettings.service_class_names");
        }
      } else {
        for (const auto& entry : field) {
          target = WireHelper::InternalSerialize(
              2, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.JavaSettings.service_class_names");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.JavaSettings.service_class_names");
        }
      }
    }
  }

  // .google.api.CommonLanguageSettings common = 3;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        3, *this_._impl_.common_, this_._impl_.common_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.api.JavaSettings)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t JavaSettings::ByteSizeLong(const MessageLite& base) {
  const JavaSettings& this_ = static_cast<const JavaSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t JavaSettings::ByteSizeLong() const {
  const JavaSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.api.JavaSettings)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // string library_package = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_library_package().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_library_package());
      }
    }
    // .google.api.CommonLanguageSettings common = 3;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.common_);
    }
    // map<string, string> service_class_names = 2;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_service_class_names_size());
      for (const auto& entry : this_._internal_service_class_names()) {
        total_size += _pbi::MapEntryFuncs<::std::string, ::std::string,
                                       _pbi::WireFormatLite::TYPE_STRING,
                                       _pbi::WireFormatLite::TYPE_STRING>::ByteSizeLong(entry.first, entry.second);
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void JavaSettings::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<JavaSettings*>(&to_msg);
  auto& from = static_cast<const JavaSettings&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.api.JavaSettings)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_library_package().empty()) {
        _this->_internal_set_library_package(from._internal_library_package());
      } else {
        if (_this->_impl_.library_package_.IsDefault()) {
          _this->_internal_set_library_package("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.common_ != nullptr);
      if (_this->_impl_.common_ == nullptr) {
        _this->_impl_.common_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.common_);
      } else {
        _this->_impl_.common_->MergeFrom(*from._impl_.common_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _this->_impl_.service_class_names_.MergeFrom(from._impl_.service_class_names_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void JavaSettings::CopyFrom(const JavaSettings& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.api.JavaSettings)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void JavaSettings::InternalSwap(JavaSettings* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.library_package_, &other->_impl_.library_package_, arena);
  swap(_impl_.common_, other->_impl_.common_);
  _impl_.service_class_names_.InternalSwap(&other->_impl_.service_class_names_);
}

::google::protobuf::Metadata JavaSettings::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

CppSettings::CppSettings(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, CppSettings_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.api.CppSettings)
}
PROTOBUF_NDEBUG_INLINE CppSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::api::CppSettings& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

CppSettings::CppSettings(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const CppSettings& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, CppSettings_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  CppSettings* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.common_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.common_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.api.CppSettings)
}
PROTOBUF_NDEBUG_INLINE CppSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void CppSettings::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.common_ = {};
}
CppSettings::~CppSettings() {
  // @@protoc_insertion_point(destructor:google.api.CppSettings)
  SharedDtor(*this);
}
inline void CppSettings::SharedDtor(MessageLite& self) {
  CppSettings& this_ = static_cast<CppSettings&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.common_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull CppSettings_class_data_ =
        CppSettings::InternalGenerateClassData_(CppSettings_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CppSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CppSettings_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(CppSettings_class_data_.tc_table);
  return CppSettings_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CppSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CppSettings_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&CppSettings_globals_));
  return CppSettings_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const CppSettings::ParseTableT_
    CppSettings::_table_ =
        CppSettings::InternalGenerateParseTable_(CppSettings_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void CppSettings::Clear() {
// @@protoc_insertion_point(message_clear_start:google.api.CppSettings)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(_impl_.common_ != nullptr);
    _impl_.common_->Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL CppSettings::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const CppSettings& this_ = static_cast<const CppSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL CppSettings::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const CppSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.api.CppSettings)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.api.CommonLanguageSettings common = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.common_, this_._impl_.common_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.api.CppSettings)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t CppSettings::ByteSizeLong(const MessageLite& base) {
  const CppSettings& this_ = static_cast<const CppSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t CppSettings::ByteSizeLong() const {
  const CppSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.api.CppSettings)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // .google.api.CommonLanguageSettings common = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.common_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void CppSettings::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<CppSettings*>(&to_msg);
  auto& from = static_cast<const CppSettings&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.api.CppSettings)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(from._impl_.common_ != nullptr);
    if (_this->_impl_.common_ == nullptr) {
      _this->_impl_.common_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.common_);
    } else {
      _this->_impl_.common_->MergeFrom(*from._impl_.common_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void CppSettings::CopyFrom(const CppSettings& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.api.CppSettings)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void CppSettings::InternalSwap(CppSettings* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.common_, other->_impl_.common_);
}

::google::protobuf::Metadata CppSettings::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

PhpSettings::PhpSettings(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PhpSettings_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.api.PhpSettings)
}
PROTOBUF_NDEBUG_INLINE PhpSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::api::PhpSettings& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

PhpSettings::PhpSettings(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const PhpSettings& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PhpSettings_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  PhpSettings* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.common_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.common_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.api.PhpSettings)
}
PROTOBUF_NDEBUG_INLINE PhpSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void PhpSettings::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.common_ = {};
}
PhpSettings::~PhpSettings() {
  // @@protoc_insertion_point(destructor:google.api.PhpSettings)
  SharedDtor(*this);
}
inline void PhpSettings::SharedDtor(MessageLite& self) {
  PhpSettings& this_ = static_cast<PhpSettings&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.common_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull PhpSettings_class_data_ =
        PhpSettings::InternalGenerateClassData_(PhpSettings_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PhpSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PhpSettings_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(PhpSettings_class_data_.tc_table);
  return PhpSettings_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PhpSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PhpSettings_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&PhpSettings_globals_));
  return PhpSettings_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const PhpSettings::ParseTableT_
    PhpSettings::_table_ =
        PhpSettings::InternalGenerateParseTable_(PhpSettings_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void PhpSettings::Clear() {
// @@protoc_insertion_point(message_clear_start:google.api.PhpSettings)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(_impl_.common_ != nullptr);
    _impl_.common_->Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL PhpSettings::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const PhpSettings& this_ = static_cast<const PhpSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL PhpSettings::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const PhpSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.api.PhpSettings)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.api.CommonLanguageSettings common = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.common_, this_._impl_.common_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.api.PhpSettings)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t PhpSettings::ByteSizeLong(const MessageLite& base) {
  const PhpSettings& this_ = static_cast<const PhpSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t PhpSettings::ByteSizeLong() const {
  const PhpSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.api.PhpSettings)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // .google.api.CommonLanguageSettings common = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.common_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void PhpSettings::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<PhpSettings*>(&to_msg);
  auto& from = static_cast<const PhpSettings&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.api.PhpSettings)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(from._impl_.common_ != nullptr);
    if (_this->_impl_.common_ == nullptr) {
      _this->_impl_.common_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.common_);
    } else {
      _this->_impl_.common_->MergeFrom(*from._impl_.common_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void PhpSettings::CopyFrom(const PhpSettings& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.api.PhpSettings)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void PhpSettings::InternalSwap(PhpSettings* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.common_, other->_impl_.common_);
}

::google::protobuf::Metadata PhpSettings::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

PythonSettings::PythonSettings(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PythonSettings_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.api.PythonSettings)
}
PROTOBUF_NDEBUG_INLINE PythonSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::api::PythonSettings& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

PythonSettings::PythonSettings(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const PythonSettings& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, PythonSettings_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  PythonSettings* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.common_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.common_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.api.PythonSettings)
}
PROTOBUF_NDEBUG_INLINE PythonSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void PythonSettings::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.common_ = {};
}
PythonSettings::~PythonSettings() {
  // @@protoc_insertion_point(destructor:google.api.PythonSettings)
  SharedDtor(*this);
}
inline void PythonSettings::SharedDtor(MessageLite& self) {
  PythonSettings& this_ = static_cast<PythonSettings&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.common_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull PythonSettings_class_data_ =
        PythonSettings::InternalGenerateClassData_(PythonSettings_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PythonSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PythonSettings_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(PythonSettings_class_data_.tc_table);
  return PythonSettings_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
PythonSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&PythonSettings_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&PythonSettings_globals_));
  return PythonSettings_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const PythonSettings::ParseTableT_
    PythonSettings::_table_ =
        PythonSettings::InternalGenerateParseTable_(PythonSettings_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void PythonSettings::Clear() {
// @@protoc_insertion_point(message_clear_start:google.api.PythonSettings)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(_impl_.common_ != nullptr);
    _impl_.common_->Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL PythonSettings::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const PythonSettings& this_ = static_cast<const PythonSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL PythonSettings::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const PythonSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.api.PythonSettings)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.api.CommonLanguageSettings common = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.common_, this_._impl_.common_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.api.PythonSettings)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t PythonSettings::ByteSizeLong(const MessageLite& base) {
  const PythonSettings& this_ = static_cast<const PythonSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t PythonSettings::ByteSizeLong() const {
  const PythonSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.api.PythonSettings)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // .google.api.CommonLanguageSettings common = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.common_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void PythonSettings::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<PythonSettings*>(&to_msg);
  auto& from = static_cast<const PythonSettings&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.api.PythonSettings)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(from._impl_.common_ != nullptr);
    if (_this->_impl_.common_ == nullptr) {
      _this->_impl_.common_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.common_);
    } else {
      _this->_impl_.common_->MergeFrom(*from._impl_.common_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void PythonSettings::CopyFrom(const PythonSettings& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.api.PythonSettings)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void PythonSettings::InternalSwap(PythonSettings* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.common_, other->_impl_.common_);
}

::google::protobuf::Metadata PythonSettings::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

NodeSettings::NodeSettings(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, NodeSettings_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.api.NodeSettings)
}
PROTOBUF_NDEBUG_INLINE NodeSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::api::NodeSettings& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

NodeSettings::NodeSettings(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const NodeSettings& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, NodeSettings_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  NodeSettings* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.common_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.common_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.api.NodeSettings)
}
PROTOBUF_NDEBUG_INLINE NodeSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void NodeSettings::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.common_ = {};
}
NodeSettings::~NodeSettings() {
  // @@protoc_insertion_point(destructor:google.api.NodeSettings)
  SharedDtor(*this);
}
inline void NodeSettings::SharedDtor(MessageLite& self) {
  NodeSettings& this_ = static_cast<NodeSettings&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.common_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull NodeSettings_class_data_ =
        NodeSettings::InternalGenerateClassData_(NodeSettings_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
NodeSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&NodeSettings_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(NodeSettings_class_data_.tc_table);
  return NodeSettings_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
NodeSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&NodeSettings_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&NodeSettings_globals_));
  return NodeSettings_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const NodeSettings::ParseTableT_
    NodeSettings::_table_ =
        NodeSettings::InternalGenerateParseTable_(NodeSettings_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void NodeSettings::Clear() {
// @@protoc_insertion_point(message_clear_start:google.api.NodeSettings)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(_impl_.common_ != nullptr);
    _impl_.common_->Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL NodeSettings::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const NodeSettings& this_ = static_cast<const NodeSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL NodeSettings::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const NodeSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.api.NodeSettings)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.api.CommonLanguageSettings common = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.common_, this_._impl_.common_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.api.NodeSettings)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t NodeSettings::ByteSizeLong(const MessageLite& base) {
  const NodeSettings& this_ = static_cast<const NodeSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t NodeSettings::ByteSizeLong() const {
  const NodeSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.api.NodeSettings)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // .google.api.CommonLanguageSettings common = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.common_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void NodeSettings::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<NodeSettings*>(&to_msg);
  auto& from = static_cast<const NodeSettings&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.api.NodeSettings)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(from._impl_.common_ != nullptr);
    if (_this->_impl_.common_ == nullptr) {
      _this->_impl_.common_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.common_);
    } else {
      _this->_impl_.common_->MergeFrom(*from._impl_.common_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void NodeSettings::CopyFrom(const NodeSettings& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.api.NodeSettings)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void NodeSettings::InternalSwap(NodeSettings* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.common_, other->_impl_.common_);
}

::google::protobuf::Metadata NodeSettings::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

#if defined(PROTOBUF_CUSTOM_VTABLE)
DotnetSettings_RenamedServicesEntry_DoNotUse::DotnetSettings_RenamedServicesEntry_DoNotUse() : SuperType(DotnetSettings_RenamedServicesEntry_DoNotUse_get_class_data()) {}
DotnetSettings_RenamedServicesEntry_DoNotUse::DotnetSettings_RenamedServicesEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
    : SuperType(arena, DotnetSettings_RenamedServicesEntry_DoNotUse_get_class_data()) {}
#else   // PROTOBUF_CUSTOM_VTABLE
DotnetSettings_RenamedServicesEntry_DoNotUse::DotnetSettings_RenamedServicesEntry_DoNotUse() : SuperType() {}
DotnetSettings_RenamedServicesEntry_DoNotUse::DotnetSettings_RenamedServicesEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena) : SuperType(arena) {}
#endif  // PROTOBUF_CUSTOM_VTABLE
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull DotnetSettings_RenamedServicesEntry_DoNotUse_class_data_ =
        DotnetSettings_RenamedServicesEntry_DoNotUse::InternalGenerateClassData_(DotnetSettings_RenamedServicesEntry_DoNotUse_globals_._default);

// including ~ all the vtables and everything they use become part
// of the same ~ SCC.
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DotnetSettings_RenamedServicesEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DotnetSettings_RenamedServicesEntry_DoNotUse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(DotnetSettings_RenamedServicesEntry_DoNotUse_class_data_.tc_table);
  return DotnetSettings_RenamedServicesEntry_DoNotUse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DotnetSettings_RenamedServicesEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DotnetSettings_RenamedServicesEntry_DoNotUse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&DotnetSettings_RenamedServicesEntry_DoNotUse_globals_));
  return DotnetSettings_RenamedServicesEntry_DoNotUse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const DotnetSettings_RenamedServicesEntry_DoNotUse::ParseTableT_
    DotnetSettings_RenamedServicesEntry_DoNotUse::_table_ =
        DotnetSettings_RenamedServicesEntry_DoNotUse::InternalGenerateParseTable_(DotnetSettings_RenamedServicesEntry_DoNotUse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
// ===================================================================

#if defined(PROTOBUF_CUSTOM_VTABLE)
DotnetSettings_RenamedResourcesEntry_DoNotUse::DotnetSettings_RenamedResourcesEntry_DoNotUse() : SuperType(DotnetSettings_RenamedResourcesEntry_DoNotUse_get_class_data()) {}
DotnetSettings_RenamedResourcesEntry_DoNotUse::DotnetSettings_RenamedResourcesEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
    : SuperType(arena, DotnetSettings_RenamedResourcesEntry_DoNotUse_get_class_data()) {}
#else   // PROTOBUF_CUSTOM_VTABLE
DotnetSettings_RenamedResourcesEntry_DoNotUse::DotnetSettings_RenamedResourcesEntry_DoNotUse() : SuperType() {}
DotnetSettings_RenamedResourcesEntry_DoNotUse::DotnetSettings_RenamedResourcesEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena) : SuperType(arena) {}
#endif  // PROTOBUF_CUSTOM_VTABLE
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull DotnetSettings_RenamedResourcesEntry_DoNotUse_class_data_ =
        DotnetSettings_RenamedResourcesEntry_DoNotUse::InternalGenerateClassData_(DotnetSettings_RenamedResourcesEntry_DoNotUse_globals_._default);

// including ~ all the vtables and everything they use become part
// of the same ~ SCC.
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DotnetSettings_RenamedResourcesEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DotnetSettings_RenamedResourcesEntry_DoNotUse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(DotnetSettings_RenamedResourcesEntry_DoNotUse_class_data_.tc_table);
  return DotnetSettings_RenamedResourcesEntry_DoNotUse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DotnetSettings_RenamedResourcesEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DotnetSettings_RenamedResourcesEntry_DoNotUse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&DotnetSettings_RenamedResourcesEntry_DoNotUse_globals_));
  return DotnetSettings_RenamedResourcesEntry_DoNotUse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const DotnetSettings_RenamedResourcesEntry_DoNotUse::ParseTableT_
    DotnetSettings_RenamedResourcesEntry_DoNotUse::_table_ =
        DotnetSettings_RenamedResourcesEntry_DoNotUse::InternalGenerateParseTable_(DotnetSettings_RenamedResourcesEntry_DoNotUse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
// ===================================================================

DotnetSettings::DotnetSettings(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DotnetSettings_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.api.DotnetSettings)
}
PROTOBUF_NDEBUG_INLINE DotnetSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::api::DotnetSettings& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        ignored_resources_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::api::DotnetSettings,
              PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_.ignored_resources_)>()
          , from.ignored_resources_
        }
        ,
        forced_namespace_aliases_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::api::DotnetSettings,
              PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_.forced_namespace_aliases_)>()
          , from.forced_namespace_aliases_
        }
        ,
        handwritten_signatures_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::api::DotnetSettings,
              PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_.handwritten_signatures_)>()
          , from.handwritten_signatures_
        }
        ,
        renamed_services_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::api::DotnetSettings,
              PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_.renamed_services_)>()
          , from.renamed_services_
        }
        ,
        renamed_resources_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::api::DotnetSettings,
              PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_.renamed_resources_)>()
          , from.renamed_resources_
        }
     {}

DotnetSettings::DotnetSettings(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const DotnetSettings& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, DotnetSettings_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  DotnetSettings* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.common_ = (CheckHasBit(cached_has_bits, 0x00000008U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.common_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.api.DotnetSettings)
}
PROTOBUF_NDEBUG_INLINE DotnetSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        ignored_resources_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::DotnetSettings,
            PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_.ignored_resources_)>()
         }
        ,
        forced_namespace_aliases_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::DotnetSettings,
            PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_.forced_namespace_aliases_)>()
         }
        ,
        handwritten_signatures_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::DotnetSettings,
            PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_.handwritten_signatures_)>()
         }
        ,
        renamed_services_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::DotnetSettings,
            PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_.renamed_services_)>()
         }
        ,
        renamed_resources_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::api::DotnetSettings,
            PROTOBUF_FIELD_OFFSET(::google::api::DotnetSettings, _impl_.renamed_resources_)>()
         }
     {}

inline void DotnetSettings::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.common_ = {};
}
DotnetSettings::~DotnetSettings() {
  // @@protoc_insertion_point(destructor:google.api.DotnetSettings)
  SharedDtor(*this);
}
inline void DotnetSettings::SharedDtor(MessageLite& self) {
  DotnetSettings& this_ = static_cast<DotnetSettings&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.common_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull DotnetSettings_class_data_ =
        DotnetSettings::InternalGenerateClassData_(DotnetSettings_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DotnetSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DotnetSettings_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(DotnetSettings_class_data_.tc_table);
  return DotnetSettings_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DotnetSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DotnetSettings_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&DotnetSettings_globals_));
  return DotnetSettings_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const DotnetSettings::ParseTableT_
    DotnetSettings::_table_ =
        DotnetSettings::InternalGenerateParseTable_(DotnetSettings_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void DotnetSettings::Clear() {
// @@protoc_insertion_point(message_clear_start:google.api.DotnetSettings)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.ignored_resources_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.forced_namespace_aliases_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.handwritten_signatures_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(_impl_.common_ != nullptr);
      _impl_.common_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      _impl_.renamed_services_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      _impl_.renamed_resources_.Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL DotnetSettings::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const DotnetSettings& this_ = static_cast<const DotnetSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL DotnetSettings::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const DotnetSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.api.DotnetSettings)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.api.CommonLanguageSettings common = 1;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.common_, this_._impl_.common_->GetCachedSize(), target,
        stream);
  }

  // map<string, string> renamed_services = 2;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (!this_._internal_renamed_services().empty()) {
      using MapType = ::google::protobuf::Map<::std::string, ::std::string>;
      using WireHelper = _pbi::MapEntryFuncs<::std::string, ::std::string,
                                     _pbi::WireFormatLite::TYPE_STRING,
                                     _pbi::WireFormatLite::TYPE_STRING>;
      const auto& field = this_._internal_renamed_services();

      if (stream->IsSerializationDeterministic() && field.size() > 1) {
        for (const auto& entry : ::google::protobuf::internal::MapSorterPtr<MapType>(field)) {
          target = WireHelper::InternalSerialize(
              2, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.DotnetSettings.renamed_services");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.DotnetSettings.renamed_services");
        }
      } else {
        for (const auto& entry : field) {
          target = WireHelper::InternalSerialize(
              2, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.DotnetSettings.renamed_services");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.DotnetSettings.renamed_services");
        }
      }
    }
  }

  // map<string, string> renamed_resources = 3;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (!this_._internal_renamed_resources().empty()) {
      using MapType = ::google::protobuf::Map<::std::string, ::std::string>;
      using WireHelper = _pbi::MapEntryFuncs<::std::string, ::std::string,
                                     _pbi::WireFormatLite::TYPE_STRING,
                                     _pbi::WireFormatLite::TYPE_STRING>;
      const auto& field = this_._internal_renamed_resources();

      if (stream->IsSerializationDeterministic() && field.size() > 1) {
        for (const auto& entry : ::google::protobuf::internal::MapSorterPtr<MapType>(field)) {
          target = WireHelper::InternalSerialize(
              3, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.DotnetSettings.renamed_resources");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.DotnetSettings.renamed_resources");
        }
      } else {
        for (const auto& entry : field) {
          target = WireHelper::InternalSerialize(
              3, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.DotnetSettings.renamed_resources");
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.second.data(), static_cast<int>(entry.second.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.DotnetSettings.renamed_resources");
        }
      }
    }
  }

  // repeated string ignored_resources = 4;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (int i = 0, n = this_._internal_ignored_resources_size(); i < n; ++i) {
      const auto& s = this_._internal_ignored_resources().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.DotnetSettings.ignored_resources");
      target = stream->WriteString(4, s, target);
    }
  }

  // repeated string forced_namespace_aliases = 5;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    for (int i = 0, n = this_._internal_forced_namespace_aliases_size(); i < n; ++i) {
      const auto& s = this_._internal_forced_namespace_aliases().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.DotnetSettings.forced_namespace_aliases");
      target = stream->WriteString(5, s, target);
    }
  }

  // repeated string handwritten_signatures = 6;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    for (int i = 0, n = this_._internal_handwritten_signatures_size(); i < n; ++i) {
      const auto& s = this_._internal_handwritten_signatures().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.DotnetSettings.handwritten_signatures");
      target = stream->WriteString(6, s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.api.DotnetSettings)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t DotnetSettings::ByteSizeLong(const MessageLite& base) {
  const DotnetSettings& this_ = static_cast<const DotnetSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t DotnetSettings::ByteSizeLong() const {
  const DotnetSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.api.DotnetSettings)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    // repeated string ignored_resources = 4;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_ignored_resources().size());
      for (int i = 0, n = this_._internal_ignored_resources().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_ignored_resources().Get(i));
      }
    }
    // repeated string forced_namespace_aliases = 5;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_forced_namespace_aliases().size());
      for (int i = 0, n = this_._internal_forced_namespace_aliases().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_forced_namespace_aliases().Get(i));
      }
    }
    // repeated string handwritten_signatures = 6;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_handwritten_signatures().size());
      for (int i = 0, n = this_._internal_handwritten_signatures().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_handwritten_signatures().Get(i));
      }
    }
    // .google.api.CommonLanguageSettings common = 1;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.common_);
    }
    // map<string, string> renamed_services = 2;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_renamed_services_size());
      for (const auto& entry : this_._internal_renamed_services()) {
        total_size += _pbi::MapEntryFuncs<::std::string, ::std::string,
                                       _pbi::WireFormatLite::TYPE_STRING,
                                       _pbi::WireFormatLite::TYPE_STRING>::ByteSizeLong(entry.first, entry.second);
      }
    }
    // map<string, string> renamed_resources = 3;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_renamed_resources_size());
      for (const auto& entry : this_._internal_renamed_resources()) {
        total_size += _pbi::MapEntryFuncs<::std::string, ::std::string,
                                       _pbi::WireFormatLite::TYPE_STRING,
                                       _pbi::WireFormatLite::TYPE_STRING>::ByteSizeLong(entry.first, entry.second);
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void DotnetSettings::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<DotnetSettings*>(&to_msg);
  auto& from = static_cast<const DotnetSettings&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.api.DotnetSettings)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_ignored_resources()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_ignored_resources());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _this->_internal_mutable_forced_namespace_aliases()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_forced_namespace_aliases());
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _this->_internal_mutable_handwritten_signatures()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_handwritten_signatures());
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(from._impl_.common_ != nullptr);
      if (_this->_impl_.common_ == nullptr) {
        _this->_impl_.common_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.common_);
      } else {
        _this->_impl_.common_->MergeFrom(*from._impl_.common_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      _this->_impl_.renamed_services_.MergeFrom(from._impl_.renamed_services_);
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      _this->_impl_.renamed_resources_.MergeFrom(from._impl_.renamed_resources_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void DotnetSettings::CopyFrom(const DotnetSettings& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.api.DotnetSettings)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void DotnetSettings::InternalSwap(DotnetSettings* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.ignored_resources_.InternalSwap(&other->_impl_.ignored_resources_);
  _impl_.forced_namespace_aliases_.InternalSwap(&other->_impl_.forced_namespace_aliases_);
  _impl_.handwritten_signatures_.InternalSwap(&other->_impl_.handwritten_signatures_);
  swap(_impl_.common_, other->_impl_.common_);
  _impl_.renamed_services_.InternalSwap(&other->_impl_.renamed_services_);
  _impl_.renamed_resources_.InternalSwap(&other->_impl_.renamed_resources_);
}

::google::protobuf::Metadata DotnetSettings::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

RubySettings::RubySettings(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, RubySettings_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.api.RubySettings)
}
PROTOBUF_NDEBUG_INLINE RubySettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::api::RubySettings& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

RubySettings::RubySettings(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const RubySettings& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, RubySettings_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  RubySettings* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.common_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.common_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.api.RubySettings)
}
PROTOBUF_NDEBUG_INLINE RubySettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void RubySettings::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.common_ = {};
}
RubySettings::~RubySettings() {
  // @@protoc_insertion_point(destructor:google.api.RubySettings)
  SharedDtor(*this);
}
inline void RubySettings::SharedDtor(MessageLite& self) {
  RubySettings& this_ = static_cast<RubySettings&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.common_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull RubySettings_class_data_ =
        RubySettings::InternalGenerateClassData_(RubySettings_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
RubySettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&RubySettings_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(RubySettings_class_data_.tc_table);
  return RubySettings_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
RubySettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&RubySettings_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&RubySettings_globals_));
  return RubySettings_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const RubySettings::ParseTableT_
    RubySettings::_table_ =
        RubySettings::InternalGenerateParseTable_(RubySettings_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void RubySettings::Clear() {
// @@protoc_insertion_point(message_clear_start:google.api.RubySettings)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(_impl_.common_ != nullptr);
    _impl_.common_->Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL RubySettings::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const RubySettings& this_ = static_cast<const RubySettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL RubySettings::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const RubySettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.api.RubySettings)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.api.CommonLanguageSettings common = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.common_, this_._impl_.common_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.api.RubySettings)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t RubySettings::ByteSizeLong(const MessageLite& base) {
  const RubySettings& this_ = static_cast<const RubySettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t RubySettings::ByteSizeLong() const {
  const RubySettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.api.RubySettings)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // .google.api.CommonLanguageSettings common = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.common_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void RubySettings::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<RubySettings*>(&to_msg);
  auto& from = static_cast<const RubySettings&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.api.RubySettings)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(from._impl_.common_ != nullptr);
    if (_this->_impl_.common_ == nullptr) {
      _this->_impl_.common_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.common_);
    } else {
      _this->_impl_.common_->MergeFrom(*from._impl_.common_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void RubySettings::CopyFrom(const RubySettings& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.api.RubySettings)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void RubySettings::InternalSwap(RubySettings* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.common_, other->_impl_.common_);
}

::google::protobuf::Metadata RubySettings::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

GoSettings::GoSettings(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, GoSettings_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.api.GoSettings)
}
PROTOBUF_NDEBUG_INLINE GoSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::api::GoSettings& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

GoSettings::GoSettings(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const GoSettings& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, GoSettings_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  GoSettings* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.common_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.common_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.api.GoSettings)
}
PROTOBUF_NDEBUG_INLINE GoSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void GoSettings::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.common_ = {};
}
GoSettings::~GoSettings() {
  // @@protoc_insertion_point(destructor:google.api.GoSettings)
  SharedDtor(*this);
}
inline void GoSettings::SharedDtor(MessageLite& self) {
  GoSettings& this_ = static_cast<GoSettings&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.common_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull GoSettings_class_data_ =
        GoSettings::InternalGenerateClassData_(GoSettings_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
GoSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&GoSettings_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(GoSettings_class_data_.tc_table);
  return GoSettings_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
GoSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&GoSettings_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&GoSettings_globals_));
  return GoSettings_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const GoSettings::ParseTableT_
    GoSettings::_table_ =
        GoSettings::InternalGenerateParseTable_(GoSettings_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void GoSettings::Clear() {
// @@protoc_insertion_point(message_clear_start:google.api.GoSettings)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(_impl_.common_ != nullptr);
    _impl_.common_->Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL GoSettings::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const GoSettings& this_ = static_cast<const GoSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL GoSettings::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const GoSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.api.GoSettings)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.api.CommonLanguageSettings common = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.common_, this_._impl_.common_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.api.GoSettings)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t GoSettings::ByteSizeLong(const MessageLite& base) {
  const GoSettings& this_ = static_cast<const GoSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t GoSettings::ByteSizeLong() const {
  const GoSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.api.GoSettings)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // .google.api.CommonLanguageSettings common = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.common_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void GoSettings::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<GoSettings*>(&to_msg);
  auto& from = static_cast<const GoSettings&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.api.GoSettings)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(from._impl_.common_ != nullptr);
    if (_this->_impl_.common_ == nullptr) {
      _this->_impl_.common_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.common_);
    } else {
      _this->_impl_.common_->MergeFrom(*from._impl_.common_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void GoSettings::CopyFrom(const GoSettings& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.api.GoSettings)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void GoSettings::InternalSwap(GoSettings* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.common_, other->_impl_.common_);
}

::google::protobuf::Metadata GoSettings::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void MethodSettings_LongRunning::clear_initial_poll_delay() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.initial_poll_delay_ != nullptr) _impl_.initial_poll_delay_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
void MethodSettings_LongRunning::clear_max_poll_delay() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.max_poll_delay_ != nullptr) _impl_.max_poll_delay_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
void MethodSettings_LongRunning::clear_total_poll_timeout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.total_poll_timeout_ != nullptr) _impl_.total_poll_timeout_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
MethodSettings_LongRunning::MethodSettings_LongRunning(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, MethodSettings_LongRunning_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.api.MethodSettings.LongRunning)
}
PROTOBUF_NDEBUG_INLINE MethodSettings_LongRunning::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::api::MethodSettings_LongRunning& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

MethodSettings_LongRunning::MethodSettings_LongRunning(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const MethodSettings_LongRunning& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, MethodSettings_LongRunning_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  MethodSettings_LongRunning* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.initial_poll_delay_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.initial_poll_delay_)
                : nullptr;
  _impl_.max_poll_delay_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.max_poll_delay_)
                : nullptr;
  _impl_.total_poll_timeout_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.total_poll_timeout_)
                : nullptr;
  _impl_.poll_delay_multiplier_ = from._impl_.poll_delay_multiplier_;

  // @@protoc_insertion_point(copy_constructor:google.api.MethodSettings.LongRunning)
}
PROTOBUF_NDEBUG_INLINE MethodSettings_LongRunning::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void MethodSettings_LongRunning::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, initial_poll_delay_),
           0,
           offsetof(Impl_, poll_delay_multiplier_) -
               offsetof(Impl_, initial_poll_delay_) +
               sizeof(Impl_::poll_delay_multiplier_));
}
MethodSettings_LongRunning::~MethodSettings_LongRunning() {
  // @@protoc_insertion_point(destructor:google.api.MethodSettings.LongRunning)
  SharedDtor(*this);
}
inline void MethodSettings_LongRunning::SharedDtor(MessageLite& self) {
  MethodSettings_LongRunning& this_ = static_cast<MethodSettings_LongRunning&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.initial_poll_delay_;
  delete this_._impl_.max_poll_delay_;
  delete this_._impl_.total_poll_timeout_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull MethodSettings_LongRunning_class_data_ =
        MethodSettings_LongRunning::InternalGenerateClassData_(MethodSettings_LongRunning_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
MethodSettings_LongRunning::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&MethodSettings_LongRunning_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(MethodSettings_LongRunning_class_data_.tc_table);
  return MethodSettings_LongRunning_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
MethodSettings_LongRunning::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&MethodSettings_LongRunning_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&MethodSettings_LongRunning_globals_));
  return MethodSettings_LongRunning_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const MethodSettings_LongRunning::ParseTableT_
    MethodSettings_LongRunning::_table_ =
        MethodSettings_LongRunning::InternalGenerateParseTable_(MethodSettings_LongRunning_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void MethodSettings_LongRunning::Clear() {
// @@protoc_insertion_point(message_clear_start:google.api.MethodSettings.LongRunning)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(_impl_.initial_poll_delay_ != nullptr);
      _impl_.initial_poll_delay_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.max_poll_delay_ != nullptr);
      _impl_.max_poll_delay_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(_impl_.total_poll_timeout_ != nullptr);
      _impl_.total_poll_timeout_->Clear();
    }
  }
  _impl_.poll_delay_multiplier_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL MethodSettings_LongRunning::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const MethodSettings_LongRunning& this_ = static_cast<const MethodSettings_LongRunning&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL MethodSettings_LongRunning::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const MethodSettings_LongRunning& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.api.MethodSettings.LongRunning)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.protobuf.Duration initial_poll_delay = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.initial_poll_delay_, this_._impl_.initial_poll_delay_->GetCachedSize(), target,
        stream);
  }

  // float poll_delay_multiplier = 2;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (::absl::bit_cast<::uint32_t>(this_._internal_poll_delay_multiplier()) != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteFloatToArray(
          2, this_._internal_poll_delay_multiplier(), target);
    }
  }

  // .google.protobuf.Duration max_poll_delay = 3;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        3, *this_._impl_.max_poll_delay_, this_._impl_.max_poll_delay_->GetCachedSize(), target,
        stream);
  }

  // .google.protobuf.Duration total_poll_timeout = 4;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        4, *this_._impl_.total_poll_timeout_, this_._impl_.total_poll_timeout_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.api.MethodSettings.LongRunning)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t MethodSettings_LongRunning::ByteSizeLong(const MessageLite& base) {
  const MethodSettings_LongRunning& this_ = static_cast<const MethodSettings_LongRunning&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t MethodSettings_LongRunning::ByteSizeLong() const {
  const MethodSettings_LongRunning& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.api.MethodSettings.LongRunning)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // .google.protobuf.Duration initial_poll_delay = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.initial_poll_delay_);
    }
    // .google.protobuf.Duration max_poll_delay = 3;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.max_poll_delay_);
    }
    // .google.protobuf.Duration total_poll_timeout = 4;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.total_poll_timeout_);
    }
    // float poll_delay_multiplier = 2;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (::absl::bit_cast<::uint32_t>(this_._internal_poll_delay_multiplier()) != 0) {
        total_size += 5;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void MethodSettings_LongRunning::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<MethodSettings_LongRunning*>(&to_msg);
  auto& from = static_cast<const MethodSettings_LongRunning&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.api.MethodSettings.LongRunning)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.initial_poll_delay_ != nullptr);
      if (_this->_impl_.initial_poll_delay_ == nullptr) {
        _this->_impl_.initial_poll_delay_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.initial_poll_delay_);
      } else {
        _this->_impl_.initial_poll_delay_->MergeFrom(*from._impl_.initial_poll_delay_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.max_poll_delay_ != nullptr);
      if (_this->_impl_.max_poll_delay_ == nullptr) {
        _this->_impl_.max_poll_delay_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.max_poll_delay_);
      } else {
        _this->_impl_.max_poll_delay_->MergeFrom(*from._impl_.max_poll_delay_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.total_poll_timeout_ != nullptr);
      if (_this->_impl_.total_poll_timeout_ == nullptr) {
        _this->_impl_.total_poll_timeout_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.total_poll_timeout_);
      } else {
        _this->_impl_.total_poll_timeout_->MergeFrom(*from._impl_.total_poll_timeout_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (::absl::bit_cast<::uint32_t>(from._internal_poll_delay_multiplier()) != 0) {
        _this->_impl_.poll_delay_multiplier_ = from._impl_.poll_delay_multiplier_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void MethodSettings_LongRunning::CopyFrom(const MethodSettings_LongRunning& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.api.MethodSettings.LongRunning)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void MethodSettings_LongRunning::InternalSwap(MethodSettings_LongRunning* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(MethodSettings_LongRunning, _impl_.poll_delay_multiplier_)
      + sizeof(MethodSettings_LongRunning::_impl_.poll_delay_multiplier_)
      - PROTOBUF_FIELD_OFFSET(MethodSettings_LongRunning, _impl_.initial_poll_delay_)>(
          reinterpret_cast<char*>(&_impl_.initial_poll_delay_),
          reinterpret_cast<char*>(&other->_impl_.initial_poll_delay_));
}

::google::protobuf::Metadata MethodSettings_LongRunning::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

MethodSettings::MethodSettings(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, MethodSettings_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.api.MethodSettings)
}
PROTOBUF_NDEBUG_INLINE MethodSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::api::MethodSettings& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        selector_(arena, from.selector_) {}

MethodSettings::MethodSettings(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const MethodSettings& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, MethodSettings_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  MethodSettings* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.long_running_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.long_running_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.api.MethodSettings)
}
PROTOBUF_NDEBUG_INLINE MethodSettings::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        selector_(arena) {}

inline void MethodSettings::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.long_running_ = {};
}
MethodSettings::~MethodSettings() {
  // @@protoc_insertion_point(destructor:google.api.MethodSettings)
  SharedDtor(*this);
}
inline void MethodSettings::SharedDtor(MessageLite& self) {
  MethodSettings& this_ = static_cast<MethodSettings&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.selector_.Destroy();
  delete this_._impl_.long_running_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull MethodSettings_class_data_ =
        MethodSettings::InternalGenerateClassData_(MethodSettings_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
MethodSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&MethodSettings_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(MethodSettings_class_data_.tc_table);
  return MethodSettings_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
MethodSettings::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&MethodSettings_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&MethodSettings_globals_));
  return MethodSettings_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const MethodSettings::ParseTableT_
    MethodSettings::_table_ =
        MethodSettings::InternalGenerateParseTable_(MethodSettings_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void MethodSettings::Clear() {
// @@protoc_insertion_point(message_clear_start:google.api.MethodSettings)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.selector_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.long_running_ != nullptr);
      _impl_.long_running_->Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL MethodSettings::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const MethodSettings& this_ = static_cast<const MethodSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL MethodSettings::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const MethodSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.api.MethodSettings)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string selector = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_selector().empty()) {
      const ::std::string& _s = this_._internal_selector();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.api.MethodSettings.selector");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // .google.api.MethodSettings.LongRunning long_running = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.long_running_, this_._impl_.long_running_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.api.MethodSettings)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t MethodSettings::ByteSizeLong(const MessageLite& base) {
  const MethodSettings& this_ = static_cast<const MethodSettings&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t MethodSettings::ByteSizeLong() const {
  const MethodSettings& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.api.MethodSettings)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string selector = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_selector().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_selector());
      }
    }
    // .google.api.MethodSettings.LongRunning long_running = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.long_running_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void MethodSettings::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<MethodSettings*>(&to_msg);
  auto& from = static_cast<const MethodSettings&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.api.MethodSettings)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_selector().empty()) {
        _this->_internal_set_selector(from._internal_selector());
      } else {
        if (_this->_impl_.selector_.IsDefault()) {
          _this->_internal_set_selector("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.long_running_ != nullptr);
      if (_this->_impl_.long_running_ == nullptr) {
        _this->_impl_.long_running_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.long_running_);
      } else {
        _this->_impl_.long_running_->MergeFrom(*from._impl_.long_running_);
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void MethodSettings::CopyFrom(const MethodSettings& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.api.MethodSettings)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void MethodSettings::InternalSwap(MethodSettings* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.selector_, &other->_impl_.selector_, arena);
  swap(_impl_.long_running_, other->_impl_.long_running_);
}

::google::protobuf::Metadata MethodSettings::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
const std::string method_signature_default("");
PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 ::_pbi::ExtensionIdentifier<
        ::google::protobuf::MethodOptions, ::_pbi::RepeatedStringTypeTraits, 9, false>
        method_signature(kMethodSignatureFieldNumber, method_signature_default);
const std::string default_host_default("");
PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 ::_pbi::ExtensionIdentifier<
        ::google::protobuf::ServiceOptions, ::_pbi::StringTypeTraits, 9, false>
        default_host(kDefaultHostFieldNumber, default_host_default);
const std::string oauth_scopes_default("");
PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 ::_pbi::ExtensionIdentifier<
        ::google::protobuf::ServiceOptions, ::_pbi::StringTypeTraits, 9, false>
        oauth_scopes(kOauthScopesFieldNumber, oauth_scopes_default);
// @@protoc_insertion_point(namespace_scope)
}  // namespace api
}  // namespace google
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_google_2fapi_2fclient_2eproto),
        ::_pbi::ExtensionSet::RegisterExtension(
            &::google::protobuf::MethodOptions::default_instance(), 1051, 9,
            true, false, /*is_utf8=*/true),
        ::_pbi::ExtensionSet::RegisterExtension(
            &::google::protobuf::ServiceOptions::default_instance(), 1049, 9,
            false, false, /*is_utf8=*/true),
        ::_pbi::ExtensionSet::RegisterExtension(
            &::google::protobuf::ServiceOptions::default_instance(), 1050, 9,
            false, false, /*is_utf8=*/true),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
