// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: google/cloud/bigquery/storage/v1/storage.proto
// Protobuf C++ Version: 7.35.1

#include "google/cloud/bigquery/storage/v1/storage.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
#ifdef PROTOBUF_MESSAGE_GLOBALS
namespace {
PROTOBUF_CONSTINIT ::google::protobuf::internal::ReflectionData
    file_reflection_data[] = {
        // ::google::cloud::bigquery::storage::v1::CreateReadSessionRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::ReadRowsRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::ThrottleState
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::StreamStats_Progress
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::StreamStats
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::ReadRowsResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::SplitReadStreamRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::SplitReadStreamResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::CreateWriteStreamRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::AppendRowsRequest_ProtoData
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::AppendRowsRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::AppendRowsResponse_AppendResult
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::AppendRowsResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::GetWriteStreamRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::FinalizeWriteStreamRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::FinalizeWriteStreamResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::FlushRowsRequest
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::FlushRowsResponse
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::StorageError
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
        // ::google::cloud::bigquery::storage::v1::RowError
        {&::_pbi::kDescriptorMethods, &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto, /* tracker*/ nullptr,},
};
}  // namespace
#endif
namespace google {
namespace cloud {
namespace bigquery {
namespace storage {
namespace v1 {
class ThrottleState::_Internal {
 public:
  using HasBits = decltype(::std::declval<ThrottleState>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ThrottleState, _impl_._has_bits_);
};

constexpr ThrottleState::ParseTableT_ ThrottleState::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ThrottleState, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::ThrottleState>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // int32 throttle_percent = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ThrottleState, _impl_.throttle_percent_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(ThrottleState, _impl_.throttle_percent_)}},
    }}, {{
      65535, 65535
    }}, {{
      // int32 throttle_percent = 1;
      {PROTOBUF_FIELD_OFFSET(ThrottleState, _impl_.throttle_percent_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr ThrottleState::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        throttle_percent_{0} {}

template <typename>
constexpr ThrottleState::ThrottleState(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ThrottleState::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ThrottleState(arena);
}
constexpr auto ThrottleState::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ThrottleState), alignof(ThrottleState));
}
constexpr auto ThrottleState::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ThrottleState::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ThrottleState>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ThrottleState::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ThrottleState>(), &ThrottleState::ByteSizeLong,
              &ThrottleState::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ThrottleState, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[2],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ThrottleStateGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ThrottleStateGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ThrottleState_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ThrottleState::InternalGenerateClassData_(
            _default, &ThrottleState_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ThrottleState>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ThrottleStateGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ThrottleState _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ThrottleState>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ThrottleStateGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ThrottleStateGlobalsTypeInternal ThrottleState_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ThrottleState_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ThrottleState_globals_.GetClassData();
#else
  return ThrottleState_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class StreamStats_Progress::_Internal {
 public:
  using HasBits = decltype(::std::declval<StreamStats_Progress>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(StreamStats_Progress, _impl_._has_bits_);
};

constexpr StreamStats_Progress::ParseTableT_ StreamStats_Progress::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(StreamStats_Progress, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::StreamStats_Progress>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // double at_response_end = 2;
      {::_pbi::TcParser::FastF64S1,
       {17, 1, 0,
        PROTOBUF_FIELD_OFFSET(StreamStats_Progress, _impl_.at_response_end_)}},
      // double at_response_start = 1;
      {::_pbi::TcParser::FastF64S1,
       {9, 0, 0,
        PROTOBUF_FIELD_OFFSET(StreamStats_Progress, _impl_.at_response_start_)}},
    }}, {{
      65535, 65535
    }}, {{
      // double at_response_start = 1;
      {PROTOBUF_FIELD_OFFSET(StreamStats_Progress, _impl_.at_response_start_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kDouble)},
      // double at_response_end = 2;
      {PROTOBUF_FIELD_OFFSET(StreamStats_Progress, _impl_.at_response_end_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kDouble)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr StreamStats_Progress::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        at_response_start_{0},
        at_response_end_{0} {}

template <typename>
constexpr StreamStats_Progress::StreamStats_Progress(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL StreamStats_Progress::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) StreamStats_Progress(arena);
}
constexpr auto StreamStats_Progress::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(StreamStats_Progress), alignof(StreamStats_Progress));
}
constexpr auto StreamStats_Progress::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &StreamStats_Progress::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<StreamStats_Progress>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &StreamStats_Progress::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<StreamStats_Progress>(), &StreamStats_Progress::ByteSizeLong,
              &StreamStats_Progress::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(StreamStats_Progress, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[3],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct StreamStats_ProgressGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr StreamStats_ProgressGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 StreamStats_Progress_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(StreamStats_Progress::InternalGenerateClassData_(
            _default, &StreamStats_Progress_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<StreamStats_Progress>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~StreamStats_ProgressGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) StreamStats_Progress _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<StreamStats_Progress>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(StreamStats_ProgressGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST StreamStats_ProgressGlobalsTypeInternal StreamStats_Progress_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* StreamStats_Progress_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return StreamStats_Progress_globals_.GetClassData();
#else
  return StreamStats_Progress_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class StorageError::_Internal {
 public:
  using HasBits = decltype(::std::declval<StorageError>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(StorageError, _impl_._has_bits_);
};

constexpr StorageError::ParseTableT_ StorageError::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(StorageError, _impl_._has_bits_),
      0, // no _extensions_
      3, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::StorageError>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .google.cloud.bigquery.storage.v1.StorageError.StorageErrorCode code = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(StorageError, _impl_.code_), 2>(),
       {8, 2, 0,
        PROTOBUF_FIELD_OFFSET(StorageError, _impl_.code_)}},
      // string entity = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 0, 0,
        PROTOBUF_FIELD_OFFSET(StorageError, _impl_.entity_)}},
      // string error_message = 3;
      {::_pbi::TcParser::FastUS1,
       {26, 1, 0,
        PROTOBUF_FIELD_OFFSET(StorageError, _impl_.error_message_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.cloud.bigquery.storage.v1.StorageError.StorageErrorCode code = 1;
      {PROTOBUF_FIELD_OFFSET(StorageError, _impl_.code_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // string entity = 2;
      {PROTOBUF_FIELD_OFFSET(StorageError, _impl_.entity_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string error_message = 3;
      {PROTOBUF_FIELD_OFFSET(StorageError, _impl_.error_message_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\55\0\6\15\0\0\0\0"
      "google.cloud.bigquery.storage.v1.StorageError"
      "entity"
      "error_message"
    }},
  };
}


inline constexpr StorageError::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        entity_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        error_message_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        code_{static_cast< ::google::cloud::bigquery::storage::v1::StorageError_StorageErrorCode >(0)} {}

template <typename>
constexpr StorageError::StorageError(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL StorageError::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) StorageError(arena);
}
constexpr auto StorageError::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(StorageError), alignof(StorageError));
}
constexpr auto StorageError::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &StorageError::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<StorageError>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &StorageError::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<StorageError>(), &StorageError::ByteSizeLong,
              &StorageError::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(StorageError, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[21],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct StorageErrorGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr StorageErrorGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 StorageError_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(StorageError::InternalGenerateClassData_(
            _default, &StorageError_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<StorageError>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~StorageErrorGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) StorageError _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<StorageError>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(StorageErrorGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST StorageErrorGlobalsTypeInternal StorageError_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* StorageError_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return StorageError_globals_.GetClassData();
#else
  return StorageError_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class SplitReadStreamRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<SplitReadStreamRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(SplitReadStreamRequest, _impl_._has_bits_);
};

constexpr SplitReadStreamRequest::ParseTableT_ SplitReadStreamRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(SplitReadStreamRequest, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::SplitReadStreamRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // double fraction = 2;
      {::_pbi::TcParser::FastF64S1,
       {17, 1, 0,
        PROTOBUF_FIELD_OFFSET(SplitReadStreamRequest, _impl_.fraction_)}},
      // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(SplitReadStreamRequest, _impl_.name_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(SplitReadStreamRequest, _impl_.name_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // double fraction = 2;
      {PROTOBUF_FIELD_OFFSET(SplitReadStreamRequest, _impl_.fraction_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kDouble)},
    }},
    // no aux_entries
    {{
      "\67\4\0\0\0\0\0\0"
      "google.cloud.bigquery.storage.v1.SplitReadStreamRequest"
      "name"
    }},
  };
}


inline constexpr SplitReadStreamRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        fraction_{0} {}

template <typename>
constexpr SplitReadStreamRequest::SplitReadStreamRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL SplitReadStreamRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) SplitReadStreamRequest(arena);
}
constexpr auto SplitReadStreamRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(SplitReadStreamRequest), alignof(SplitReadStreamRequest));
}
constexpr auto SplitReadStreamRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &SplitReadStreamRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<SplitReadStreamRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &SplitReadStreamRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<SplitReadStreamRequest>(), &SplitReadStreamRequest::ByteSizeLong,
              &SplitReadStreamRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(SplitReadStreamRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[6],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct SplitReadStreamRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr SplitReadStreamRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 SplitReadStreamRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(SplitReadStreamRequest::InternalGenerateClassData_(
            _default, &SplitReadStreamRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<SplitReadStreamRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~SplitReadStreamRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) SplitReadStreamRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<SplitReadStreamRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(SplitReadStreamRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST SplitReadStreamRequestGlobalsTypeInternal SplitReadStreamRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* SplitReadStreamRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return SplitReadStreamRequest_globals_.GetClassData();
#else
  return SplitReadStreamRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class RowError::_Internal {
 public:
  using HasBits = decltype(::std::declval<RowError>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(RowError, _impl_._has_bits_);
};

constexpr RowError::ParseTableT_ RowError::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(RowError, _impl_._has_bits_),
      0, // no _extensions_
      3, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::RowError>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // int64 index = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(RowError, _impl_.index_), 1>(),
       {8, 1, 0,
        PROTOBUF_FIELD_OFFSET(RowError, _impl_.index_)}},
      // .google.cloud.bigquery.storage.v1.RowError.RowErrorCode code = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(RowError, _impl_.code_), 2>(),
       {16, 2, 0,
        PROTOBUF_FIELD_OFFSET(RowError, _impl_.code_)}},
      // string message = 3;
      {::_pbi::TcParser::FastUS1,
       {26, 0, 0,
        PROTOBUF_FIELD_OFFSET(RowError, _impl_.message_)}},
    }}, {{
      65535, 65535
    }}, {{
      // int64 index = 1;
      {PROTOBUF_FIELD_OFFSET(RowError, _impl_.index_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // .google.cloud.bigquery.storage.v1.RowError.RowErrorCode code = 2;
      {PROTOBUF_FIELD_OFFSET(RowError, _impl_.code_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // string message = 3;
      {PROTOBUF_FIELD_OFFSET(RowError, _impl_.message_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\51\0\0\7\0\0\0\0"
      "google.cloud.bigquery.storage.v1.RowError"
      "message"
    }},
  };
}


inline constexpr RowError::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        message_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        index_{::int64_t{0}},
        code_{static_cast< ::google::cloud::bigquery::storage::v1::RowError_RowErrorCode >(0)} {}

template <typename>
constexpr RowError::RowError(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL RowError::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) RowError(arena);
}
constexpr auto RowError::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(RowError), alignof(RowError));
}
constexpr auto RowError::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &RowError::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<RowError>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &RowError::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<RowError>(), &RowError::ByteSizeLong,
              &RowError::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(RowError, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[22],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct RowErrorGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr RowErrorGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 RowError_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(RowError::InternalGenerateClassData_(
            _default, &RowError_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<RowError>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~RowErrorGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) RowError _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<RowError>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(RowErrorGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST RowErrorGlobalsTypeInternal RowError_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* RowError_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return RowError_globals_.GetClassData();
#else
  return RowError_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ReadRowsRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<ReadRowsRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ReadRowsRequest, _impl_._has_bits_);
};

constexpr ReadRowsRequest::ParseTableT_ ReadRowsRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ReadRowsRequest, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::ReadRowsRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // int64 offset = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(ReadRowsRequest, _impl_.offset_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(ReadRowsRequest, _impl_.offset_)}},
      // string read_stream = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ReadRowsRequest, _impl_.read_stream_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string read_stream = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(ReadRowsRequest, _impl_.read_stream_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // int64 offset = 2;
      {PROTOBUF_FIELD_OFFSET(ReadRowsRequest, _impl_.offset_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
    }},
    // no aux_entries
    {{
      "\60\13\0\0\0\0\0\0"
      "google.cloud.bigquery.storage.v1.ReadRowsRequest"
      "read_stream"
    }},
  };
}


inline constexpr ReadRowsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        read_stream_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        offset_{::int64_t{0}} {}

template <typename>
constexpr ReadRowsRequest::ReadRowsRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ReadRowsRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ReadRowsRequest(arena);
}
constexpr auto ReadRowsRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ReadRowsRequest), alignof(ReadRowsRequest));
}
constexpr auto ReadRowsRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ReadRowsRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ReadRowsRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ReadRowsRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ReadRowsRequest>(), &ReadRowsRequest::ByteSizeLong,
              &ReadRowsRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ReadRowsRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[1],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ReadRowsRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ReadRowsRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ReadRowsRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ReadRowsRequest::InternalGenerateClassData_(
            _default, &ReadRowsRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ReadRowsRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ReadRowsRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ReadRowsRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ReadRowsRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ReadRowsRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ReadRowsRequestGlobalsTypeInternal ReadRowsRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ReadRowsRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ReadRowsRequest_globals_.GetClassData();
#else
  return ReadRowsRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class GetWriteStreamRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<GetWriteStreamRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(GetWriteStreamRequest, _impl_._has_bits_);
};

constexpr GetWriteStreamRequest::ParseTableT_ GetWriteStreamRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(GetWriteStreamRequest, _impl_._has_bits_),
      0, // no _extensions_
      3, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967290,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::GetWriteStreamRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(GetWriteStreamRequest, _impl_.name_)}},
      {::_pbi::TcParser::MiniParse, {}},
      // .google.cloud.bigquery.storage.v1.WriteStreamView view = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(GetWriteStreamRequest, _impl_.view_), 1>(),
       {24, 1, 0,
        PROTOBUF_FIELD_OFFSET(GetWriteStreamRequest, _impl_.view_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(GetWriteStreamRequest, _impl_.name_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.cloud.bigquery.storage.v1.WriteStreamView view = 3;
      {PROTOBUF_FIELD_OFFSET(GetWriteStreamRequest, _impl_.view_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
    }},
    // no aux_entries
    {{
      "\66\4\0\0\0\0\0\0"
      "google.cloud.bigquery.storage.v1.GetWriteStreamRequest"
      "name"
    }},
  };
}


inline constexpr GetWriteStreamRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        view_{static_cast< ::google::cloud::bigquery::storage::v1::WriteStreamView >(0)} {}

template <typename>
constexpr GetWriteStreamRequest::GetWriteStreamRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL GetWriteStreamRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) GetWriteStreamRequest(arena);
}
constexpr auto GetWriteStreamRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(GetWriteStreamRequest), alignof(GetWriteStreamRequest));
}
constexpr auto GetWriteStreamRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &GetWriteStreamRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<GetWriteStreamRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &GetWriteStreamRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<GetWriteStreamRequest>(), &GetWriteStreamRequest::ByteSizeLong,
              &GetWriteStreamRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(GetWriteStreamRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[14],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct GetWriteStreamRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr GetWriteStreamRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 GetWriteStreamRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(GetWriteStreamRequest::InternalGenerateClassData_(
            _default, &GetWriteStreamRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<GetWriteStreamRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~GetWriteStreamRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) GetWriteStreamRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<GetWriteStreamRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(GetWriteStreamRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST GetWriteStreamRequestGlobalsTypeInternal GetWriteStreamRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* GetWriteStreamRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return GetWriteStreamRequest_globals_.GetClassData();
#else
  return GetWriteStreamRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class FlushRowsResponse::_Internal {
 public:
  using HasBits = decltype(::std::declval<FlushRowsResponse>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(FlushRowsResponse, _impl_._has_bits_);
};

constexpr FlushRowsResponse::ParseTableT_ FlushRowsResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(FlushRowsResponse, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::FlushRowsResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // int64 offset = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(FlushRowsResponse, _impl_.offset_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(FlushRowsResponse, _impl_.offset_)}},
    }}, {{
      65535, 65535
    }}, {{
      // int64 offset = 1;
      {PROTOBUF_FIELD_OFFSET(FlushRowsResponse, _impl_.offset_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr FlushRowsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        offset_{::int64_t{0}} {}

template <typename>
constexpr FlushRowsResponse::FlushRowsResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL FlushRowsResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) FlushRowsResponse(arena);
}
constexpr auto FlushRowsResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(FlushRowsResponse), alignof(FlushRowsResponse));
}
constexpr auto FlushRowsResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &FlushRowsResponse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<FlushRowsResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &FlushRowsResponse::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<FlushRowsResponse>(), &FlushRowsResponse::ByteSizeLong,
              &FlushRowsResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(FlushRowsResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[20],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct FlushRowsResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr FlushRowsResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 FlushRowsResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(FlushRowsResponse::InternalGenerateClassData_(
            _default, &FlushRowsResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<FlushRowsResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~FlushRowsResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) FlushRowsResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<FlushRowsResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(FlushRowsResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST FlushRowsResponseGlobalsTypeInternal FlushRowsResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* FlushRowsResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return FlushRowsResponse_globals_.GetClassData();
#else
  return FlushRowsResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class FinalizeWriteStreamResponse::_Internal {
 public:
  using HasBits = decltype(::std::declval<FinalizeWriteStreamResponse>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(FinalizeWriteStreamResponse, _impl_._has_bits_);
};

constexpr FinalizeWriteStreamResponse::ParseTableT_ FinalizeWriteStreamResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(FinalizeWriteStreamResponse, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::FinalizeWriteStreamResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // int64 row_count = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(FinalizeWriteStreamResponse, _impl_.row_count_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(FinalizeWriteStreamResponse, _impl_.row_count_)}},
    }}, {{
      65535, 65535
    }}, {{
      // int64 row_count = 1;
      {PROTOBUF_FIELD_OFFSET(FinalizeWriteStreamResponse, _impl_.row_count_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr FinalizeWriteStreamResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        row_count_{::int64_t{0}} {}

template <typename>
constexpr FinalizeWriteStreamResponse::FinalizeWriteStreamResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL FinalizeWriteStreamResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) FinalizeWriteStreamResponse(arena);
}
constexpr auto FinalizeWriteStreamResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(FinalizeWriteStreamResponse), alignof(FinalizeWriteStreamResponse));
}
constexpr auto FinalizeWriteStreamResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &FinalizeWriteStreamResponse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<FinalizeWriteStreamResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &FinalizeWriteStreamResponse::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<FinalizeWriteStreamResponse>(), &FinalizeWriteStreamResponse::ByteSizeLong,
              &FinalizeWriteStreamResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(FinalizeWriteStreamResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[18],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct FinalizeWriteStreamResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr FinalizeWriteStreamResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 FinalizeWriteStreamResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(FinalizeWriteStreamResponse::InternalGenerateClassData_(
            _default, &FinalizeWriteStreamResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<FinalizeWriteStreamResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~FinalizeWriteStreamResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) FinalizeWriteStreamResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<FinalizeWriteStreamResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(FinalizeWriteStreamResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST FinalizeWriteStreamResponseGlobalsTypeInternal FinalizeWriteStreamResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* FinalizeWriteStreamResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return FinalizeWriteStreamResponse_globals_.GetClassData();
#else
  return FinalizeWriteStreamResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class FinalizeWriteStreamRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<FinalizeWriteStreamRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(FinalizeWriteStreamRequest, _impl_._has_bits_);
};

constexpr FinalizeWriteStreamRequest::ParseTableT_ FinalizeWriteStreamRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(FinalizeWriteStreamRequest, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::FinalizeWriteStreamRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(FinalizeWriteStreamRequest, _impl_.name_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(FinalizeWriteStreamRequest, _impl_.name_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\73\4\0\0\0\0\0\0"
      "google.cloud.bigquery.storage.v1.FinalizeWriteStreamRequest"
      "name"
    }},
  };
}


inline constexpr FinalizeWriteStreamRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr FinalizeWriteStreamRequest::FinalizeWriteStreamRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL FinalizeWriteStreamRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) FinalizeWriteStreamRequest(arena);
}
constexpr auto FinalizeWriteStreamRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(FinalizeWriteStreamRequest), alignof(FinalizeWriteStreamRequest));
}
constexpr auto FinalizeWriteStreamRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &FinalizeWriteStreamRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<FinalizeWriteStreamRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &FinalizeWriteStreamRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<FinalizeWriteStreamRequest>(), &FinalizeWriteStreamRequest::ByteSizeLong,
              &FinalizeWriteStreamRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(FinalizeWriteStreamRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[17],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct FinalizeWriteStreamRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr FinalizeWriteStreamRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 FinalizeWriteStreamRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(FinalizeWriteStreamRequest::InternalGenerateClassData_(
            _default, &FinalizeWriteStreamRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<FinalizeWriteStreamRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~FinalizeWriteStreamRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) FinalizeWriteStreamRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<FinalizeWriteStreamRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(FinalizeWriteStreamRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST FinalizeWriteStreamRequestGlobalsTypeInternal FinalizeWriteStreamRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* FinalizeWriteStreamRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return FinalizeWriteStreamRequest_globals_.GetClassData();
#else
  return FinalizeWriteStreamRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class BatchCommitWriteStreamsRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<BatchCommitWriteStreamsRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(BatchCommitWriteStreamsRequest, _impl_._has_bits_);
};

constexpr BatchCommitWriteStreamsRequest::ParseTableT_ BatchCommitWriteStreamsRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(BatchCommitWriteStreamsRequest, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated string write_streams = 2 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastUR1,
       {18, 0, 0,
        PROTOBUF_FIELD_OFFSET(BatchCommitWriteStreamsRequest, _impl_.write_streams_)}},
      // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 1, 0,
        PROTOBUF_FIELD_OFFSET(BatchCommitWriteStreamsRequest, _impl_.parent_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(BatchCommitWriteStreamsRequest, _impl_.parent_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // repeated string write_streams = 2 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(BatchCommitWriteStreamsRequest, _impl_.write_streams_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
    }},
    // no aux_entries
    {{
      "\77\6\15\0\0\0\0\0"
      "google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsRequest"
      "parent"
      "write_streams"
    }},
  };
}


inline constexpr BatchCommitWriteStreamsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        write_streams_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsRequest,
            PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsRequest, _impl_.write_streams_)>()
         }
        ,
        parent_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr BatchCommitWriteStreamsRequest::BatchCommitWriteStreamsRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL BatchCommitWriteStreamsRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) BatchCommitWriteStreamsRequest(arena);
}
constexpr auto BatchCommitWriteStreamsRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(BatchCommitWriteStreamsRequest), alignof(BatchCommitWriteStreamsRequest));
}
constexpr auto BatchCommitWriteStreamsRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &BatchCommitWriteStreamsRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<BatchCommitWriteStreamsRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &BatchCommitWriteStreamsRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<BatchCommitWriteStreamsRequest>(), &BatchCommitWriteStreamsRequest::ByteSizeLong,
              &BatchCommitWriteStreamsRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(BatchCommitWriteStreamsRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[15],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct BatchCommitWriteStreamsRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr BatchCommitWriteStreamsRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 BatchCommitWriteStreamsRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(BatchCommitWriteStreamsRequest::InternalGenerateClassData_(
            _default, &BatchCommitWriteStreamsRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<BatchCommitWriteStreamsRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~BatchCommitWriteStreamsRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) BatchCommitWriteStreamsRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<BatchCommitWriteStreamsRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(BatchCommitWriteStreamsRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST BatchCommitWriteStreamsRequestGlobalsTypeInternal BatchCommitWriteStreamsRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* BatchCommitWriteStreamsRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return BatchCommitWriteStreamsRequest_globals_.GetClassData();
#else
  return BatchCommitWriteStreamsRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
constexpr AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse::ParseTableT_ AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::DiscardEverythingFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.cloud.bigquery.storage.v1.AppendRowsRequest.MissingValueInterpretation value = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse, _impl_.value_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse, _impl_.value_)}},
      // string key = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse, _impl_.key_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string key = 1;
      {PROTOBUF_FIELD_OFFSET(AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse, _impl_.key_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.cloud.bigquery.storage.v1.AppendRowsRequest.MissingValueInterpretation value = 2;
      {PROTOBUF_FIELD_OFFSET(AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse, _impl_.value_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
    }},
    // no aux_entries
    {{
      "\123\3\0\0\0\0\0\0"
      "google.cloud.bigquery.storage.v1.AppendRowsRequest.MissingValueInterpretationsEntry"
      "key"
    }},
  };
}

template <typename>
constexpr AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse::AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse::MapEntry(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
      ) {
}
inline void* PROTOBUF_NONNULL AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse(arena);
}
constexpr auto AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse), alignof(AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse));
}
constexpr auto AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse::SharedDtor,
          static_cast<void (::google::protobuf::MessageLite::*)()>(&AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse::ClearImpl),
              ::google::protobuf::Message::ByteSizeLongImpl, ::google::protobuf::Message::_InternalSerializeImpl
              ,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[10],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse::InternalGenerateClassData_(
            _default, &AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUseGlobalsTypeInternal AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse_globals_.GetClassData();
#else
  return AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class StreamStats::_Internal {
 public:
  using HasBits = decltype(::std::declval<StreamStats>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(StreamStats, _impl_._has_bits_);
};

constexpr StreamStats::ParseTableT_ StreamStats::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(StreamStats, _impl_._has_bits_),
      0, // no _extensions_
      2, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967293,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::StreamStats>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.cloud.bigquery.storage.v1.StreamStats.Progress progress = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 0, 0,
        PROTOBUF_FIELD_OFFSET(StreamStats, _impl_.progress_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.cloud.bigquery.storage.v1.StreamStats.Progress progress = 2;
      {PROTOBUF_FIELD_OFFSET(StreamStats, _impl_.progress_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::StreamStats_Progress>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::cloud::bigquery::storage::v1::StreamStats_Progress_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr StreamStats::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        progress_{nullptr} {}

template <typename>
constexpr StreamStats::StreamStats(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL StreamStats::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) StreamStats(arena);
}
constexpr auto StreamStats::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(StreamStats), alignof(StreamStats));
}
constexpr auto StreamStats::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &StreamStats::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<StreamStats>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &StreamStats::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<StreamStats>(), &StreamStats::ByteSizeLong,
              &StreamStats::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(StreamStats, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[4],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct StreamStatsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr StreamStatsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 StreamStats_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(StreamStats::InternalGenerateClassData_(
            _default, &StreamStats_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<StreamStats>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~StreamStatsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) StreamStats _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<StreamStats>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(StreamStatsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST StreamStatsGlobalsTypeInternal StreamStats_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* StreamStats_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return StreamStats_globals_.GetClassData();
#else
  return StreamStats_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class SplitReadStreamResponse::_Internal {
 public:
  using HasBits = decltype(::std::declval<SplitReadStreamResponse>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(SplitReadStreamResponse, _impl_._has_bits_);
};

constexpr SplitReadStreamResponse::ParseTableT_ SplitReadStreamResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(SplitReadStreamResponse, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::SplitReadStreamResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.cloud.bigquery.storage.v1.ReadStream remainder_stream = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(SplitReadStreamResponse, _impl_.remainder_stream_)}},
      // .google.cloud.bigquery.storage.v1.ReadStream primary_stream = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(SplitReadStreamResponse, _impl_.primary_stream_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.cloud.bigquery.storage.v1.ReadStream primary_stream = 1;
      {PROTOBUF_FIELD_OFFSET(SplitReadStreamResponse, _impl_.primary_stream_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.cloud.bigquery.storage.v1.ReadStream remainder_stream = 2;
      {PROTOBUF_FIELD_OFFSET(SplitReadStreamResponse, _impl_.remainder_stream_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::ReadStream>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::cloud::bigquery::storage::v1::ReadStream_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::ReadStream>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::cloud::bigquery::storage::v1::ReadStream_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr SplitReadStreamResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        primary_stream_{nullptr},
        remainder_stream_{nullptr} {}

template <typename>
constexpr SplitReadStreamResponse::SplitReadStreamResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL SplitReadStreamResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) SplitReadStreamResponse(arena);
}
constexpr auto SplitReadStreamResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(SplitReadStreamResponse), alignof(SplitReadStreamResponse));
}
constexpr auto SplitReadStreamResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &SplitReadStreamResponse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<SplitReadStreamResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &SplitReadStreamResponse::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<SplitReadStreamResponse>(), &SplitReadStreamResponse::ByteSizeLong,
              &SplitReadStreamResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(SplitReadStreamResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[7],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct SplitReadStreamResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr SplitReadStreamResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 SplitReadStreamResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(SplitReadStreamResponse::InternalGenerateClassData_(
            _default, &SplitReadStreamResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<SplitReadStreamResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~SplitReadStreamResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) SplitReadStreamResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<SplitReadStreamResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(SplitReadStreamResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST SplitReadStreamResponseGlobalsTypeInternal SplitReadStreamResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* SplitReadStreamResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return SplitReadStreamResponse_globals_.GetClassData();
#else
  return SplitReadStreamResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class FlushRowsRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<FlushRowsRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(FlushRowsRequest, _impl_._has_bits_);
};

constexpr FlushRowsRequest::ParseTableT_ FlushRowsRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(FlushRowsRequest, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::FlushRowsRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.protobuf.Int64Value offset = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(FlushRowsRequest, _impl_.offset_)}},
      // string write_stream = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(FlushRowsRequest, _impl_.write_stream_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string write_stream = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(FlushRowsRequest, _impl_.write_stream_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.protobuf.Int64Value offset = 2;
      {PROTOBUF_FIELD_OFFSET(FlushRowsRequest, _impl_.offset_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::Int64Value>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::Int64Value_globals_},
        #endif
    }},
    {{
      "\61\14\0\0\0\0\0\0"
      "google.cloud.bigquery.storage.v1.FlushRowsRequest"
      "write_stream"
    }},
  };
}


inline constexpr FlushRowsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        write_stream_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        offset_{nullptr} {}

template <typename>
constexpr FlushRowsRequest::FlushRowsRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL FlushRowsRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) FlushRowsRequest(arena);
}
constexpr auto FlushRowsRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(FlushRowsRequest), alignof(FlushRowsRequest));
}
constexpr auto FlushRowsRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &FlushRowsRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<FlushRowsRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &FlushRowsRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<FlushRowsRequest>(), &FlushRowsRequest::ByteSizeLong,
              &FlushRowsRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(FlushRowsRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[19],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct FlushRowsRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr FlushRowsRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 FlushRowsRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(FlushRowsRequest::InternalGenerateClassData_(
            _default, &FlushRowsRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<FlushRowsRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~FlushRowsRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) FlushRowsRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<FlushRowsRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(FlushRowsRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST FlushRowsRequestGlobalsTypeInternal FlushRowsRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* FlushRowsRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return FlushRowsRequest_globals_.GetClassData();
#else
  return FlushRowsRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class BatchCommitWriteStreamsResponse::_Internal {
 public:
  using HasBits = decltype(::std::declval<BatchCommitWriteStreamsResponse>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(BatchCommitWriteStreamsResponse, _impl_._has_bits_);
};

constexpr BatchCommitWriteStreamsResponse::ParseTableT_ BatchCommitWriteStreamsResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(BatchCommitWriteStreamsResponse, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated .google.cloud.bigquery.storage.v1.StorageError stream_errors = 2;
      {::_pbi::TcParser::FastMtR1,
       {18, 0, 1,
        PROTOBUF_FIELD_OFFSET(BatchCommitWriteStreamsResponse, _impl_.stream_errors_)}},
      // .google.protobuf.Timestamp commit_time = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 1, 0,
        PROTOBUF_FIELD_OFFSET(BatchCommitWriteStreamsResponse, _impl_.commit_time_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.protobuf.Timestamp commit_time = 1;
      {PROTOBUF_FIELD_OFFSET(BatchCommitWriteStreamsResponse, _impl_.commit_time_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // repeated .google.cloud.bigquery.storage.v1.StorageError stream_errors = 2;
      {PROTOBUF_FIELD_OFFSET(BatchCommitWriteStreamsResponse, _impl_.stream_errors_), _Internal::kHasBitsOffset + 0, 1, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::Timestamp>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::Timestamp_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::StorageError>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::cloud::bigquery::storage::v1::StorageError_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr BatchCommitWriteStreamsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        stream_errors_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsResponse,
            PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsResponse, _impl_.stream_errors_)>()
         }
        ,
        commit_time_{nullptr} {}

template <typename>
constexpr BatchCommitWriteStreamsResponse::BatchCommitWriteStreamsResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL BatchCommitWriteStreamsResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) BatchCommitWriteStreamsResponse(arena);
}
constexpr auto BatchCommitWriteStreamsResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(BatchCommitWriteStreamsResponse), alignof(BatchCommitWriteStreamsResponse));
}
constexpr auto BatchCommitWriteStreamsResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &BatchCommitWriteStreamsResponse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<BatchCommitWriteStreamsResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &BatchCommitWriteStreamsResponse::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<BatchCommitWriteStreamsResponse>(), &BatchCommitWriteStreamsResponse::ByteSizeLong,
              &BatchCommitWriteStreamsResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(BatchCommitWriteStreamsResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[16],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct BatchCommitWriteStreamsResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr BatchCommitWriteStreamsResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 BatchCommitWriteStreamsResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(BatchCommitWriteStreamsResponse::InternalGenerateClassData_(
            _default, &BatchCommitWriteStreamsResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<BatchCommitWriteStreamsResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~BatchCommitWriteStreamsResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) BatchCommitWriteStreamsResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<BatchCommitWriteStreamsResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(BatchCommitWriteStreamsResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST BatchCommitWriteStreamsResponseGlobalsTypeInternal BatchCommitWriteStreamsResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* BatchCommitWriteStreamsResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return BatchCommitWriteStreamsResponse_globals_.GetClassData();
#else
  return BatchCommitWriteStreamsResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class AppendRowsResponse_AppendResult::_Internal {
 public:
  using HasBits = decltype(::std::declval<AppendRowsResponse_AppendResult>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(AppendRowsResponse_AppendResult, _impl_._has_bits_);
};

constexpr AppendRowsResponse_AppendResult::ParseTableT_ AppendRowsResponse_AppendResult::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(AppendRowsResponse_AppendResult, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::AppendRowsResponse_AppendResult>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.protobuf.Int64Value offset = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(AppendRowsResponse_AppendResult, _impl_.offset_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.protobuf.Int64Value offset = 1;
      {PROTOBUF_FIELD_OFFSET(AppendRowsResponse_AppendResult, _impl_.offset_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::Int64Value>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::Int64Value_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr AppendRowsResponse_AppendResult::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        offset_{nullptr} {}

template <typename>
constexpr AppendRowsResponse_AppendResult::AppendRowsResponse_AppendResult(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL AppendRowsResponse_AppendResult::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) AppendRowsResponse_AppendResult(arena);
}
constexpr auto AppendRowsResponse_AppendResult::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(AppendRowsResponse_AppendResult), alignof(AppendRowsResponse_AppendResult));
}
constexpr auto AppendRowsResponse_AppendResult::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &AppendRowsResponse_AppendResult::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<AppendRowsResponse_AppendResult>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &AppendRowsResponse_AppendResult::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<AppendRowsResponse_AppendResult>(), &AppendRowsResponse_AppendResult::ByteSizeLong,
              &AppendRowsResponse_AppendResult::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(AppendRowsResponse_AppendResult, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[12],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct AppendRowsResponse_AppendResultGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr AppendRowsResponse_AppendResultGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 AppendRowsResponse_AppendResult_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(AppendRowsResponse_AppendResult::InternalGenerateClassData_(
            _default, &AppendRowsResponse_AppendResult_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<AppendRowsResponse_AppendResult>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~AppendRowsResponse_AppendResultGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) AppendRowsResponse_AppendResult _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<AppendRowsResponse_AppendResult>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(AppendRowsResponse_AppendResultGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST AppendRowsResponse_AppendResultGlobalsTypeInternal AppendRowsResponse_AppendResult_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* AppendRowsResponse_AppendResult_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return AppendRowsResponse_AppendResult_globals_.GetClassData();
#else
  return AppendRowsResponse_AppendResult_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ReadRowsResponse::_Internal {
 public:
  using HasBits = decltype(::std::declval<ReadRowsResponse>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ReadRowsResponse, _impl_._has_bits_);
  static constexpr ::int32_t kOneofCaseOffset =
      PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ReadRowsResponse, _impl_._oneof_case_);
};

constexpr ReadRowsResponse::ParseTableT_ ReadRowsResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ReadRowsResponse, _impl_._has_bits_),
      0, // no _extensions_
      8, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967041,  // skipmap
      offsetof(ParseTableT_, field_entries),
      7,  // num_field_entries
      6,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::ReadRowsResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      // .google.cloud.bigquery.storage.v1.StreamStats stats = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 0, 0,
        PROTOBUF_FIELD_OFFSET(ReadRowsResponse, _impl_.stats_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      // .google.cloud.bigquery.storage.v1.ThrottleState throttle_state = 5;
      {::_pbi::TcParser::FastMtS1,
       {42, 1, 3,
        PROTOBUF_FIELD_OFFSET(ReadRowsResponse, _impl_.throttle_state_)}},
      // int64 row_count = 6;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(ReadRowsResponse, _impl_.row_count_), 2>(),
       {48, 2, 0,
        PROTOBUF_FIELD_OFFSET(ReadRowsResponse, _impl_.row_count_)}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.cloud.bigquery.storage.v1.StreamStats stats = 2;
      {PROTOBUF_FIELD_OFFSET(ReadRowsResponse, _impl_.stats_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.cloud.bigquery.storage.v1.AvroRows avro_rows = 3;
      {PROTOBUF_FIELD_OFFSET(ReadRowsResponse, _impl_.rows_.avro_rows_), _Internal::kOneofCaseOffset + 0, 1, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.cloud.bigquery.storage.v1.ArrowRecordBatch arrow_record_batch = 4;
      {PROTOBUF_FIELD_OFFSET(ReadRowsResponse, _impl_.rows_.arrow_record_batch_), _Internal::kOneofCaseOffset + 0, 2, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.cloud.bigquery.storage.v1.ThrottleState throttle_state = 5;
      {PROTOBUF_FIELD_OFFSET(ReadRowsResponse, _impl_.throttle_state_), _Internal::kHasBitsOffset + 1, 3, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // int64 row_count = 6;
      {PROTOBUF_FIELD_OFFSET(ReadRowsResponse, _impl_.row_count_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // .google.cloud.bigquery.storage.v1.AvroSchema avro_schema = 7 [(.google.api.field_behavior) = OUTPUT_ONLY];
      {PROTOBUF_FIELD_OFFSET(ReadRowsResponse, _impl_.schema_.avro_schema_), _Internal::kOneofCaseOffset + 4, 4, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.cloud.bigquery.storage.v1.ArrowSchema arrow_schema = 8 [(.google.api.field_behavior) = OUTPUT_ONLY];
      {PROTOBUF_FIELD_OFFSET(ReadRowsResponse, _impl_.schema_.arrow_schema_), _Internal::kOneofCaseOffset + 4, 5, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::StreamStats>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::cloud::bigquery::storage::v1::StreamStats_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::AvroRows>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::cloud::bigquery::storage::v1::AvroRows_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::ArrowRecordBatch>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::cloud::bigquery::storage::v1::ArrowRecordBatch_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::ThrottleState>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::cloud::bigquery::storage::v1::ThrottleState_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::AvroSchema>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::cloud::bigquery::storage::v1::AvroSchema_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::ArrowSchema>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::cloud::bigquery::storage::v1::ArrowSchema_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr ReadRowsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        stats_{nullptr},
        throttle_state_{nullptr},
        row_count_{::int64_t{0}},
        rows_{},
        schema_{},
        _oneof_case_{} {}

template <typename>
constexpr ReadRowsResponse::ReadRowsResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ReadRowsResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ReadRowsResponse(arena);
}
constexpr auto ReadRowsResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ReadRowsResponse), alignof(ReadRowsResponse));
}
constexpr auto ReadRowsResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ReadRowsResponse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ReadRowsResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ReadRowsResponse::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ReadRowsResponse>(), &ReadRowsResponse::ByteSizeLong,
              &ReadRowsResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ReadRowsResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[5],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ReadRowsResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ReadRowsResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ReadRowsResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ReadRowsResponse::InternalGenerateClassData_(
            _default, &ReadRowsResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ReadRowsResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ReadRowsResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ReadRowsResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ReadRowsResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ReadRowsResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ReadRowsResponseGlobalsTypeInternal ReadRowsResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ReadRowsResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ReadRowsResponse_globals_.GetClassData();
#else
  return ReadRowsResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class AppendRowsResponse::_Internal {
 public:
  using HasBits = decltype(::std::declval<AppendRowsResponse>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(AppendRowsResponse, _impl_._has_bits_);
  static constexpr ::int32_t kOneofCaseOffset =
      PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsResponse, _impl_._oneof_case_);
};

constexpr AppendRowsResponse::ParseTableT_ AppendRowsResponse::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(AppendRowsResponse, _impl_._has_bits_),
      0, // no _extensions_
      5, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967264,  // skipmap
      offsetof(ParseTableT_, field_entries),
      5,  // num_field_entries
      4,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::AppendRowsResponse>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated .google.cloud.bigquery.storage.v1.RowError row_errors = 4;
      {::_pbi::TcParser::FastMtR1,
       {34, 0, 3,
        PROTOBUF_FIELD_OFFSET(AppendRowsResponse, _impl_.row_errors_)}},
      // string write_stream = 5;
      {::_pbi::TcParser::FastUS1,
       {42, 1, 0,
        PROTOBUF_FIELD_OFFSET(AppendRowsResponse, _impl_.write_stream_)}},
      {::_pbi::TcParser::MiniParse, {}},
      // .google.cloud.bigquery.storage.v1.TableSchema updated_schema = 3;
      {::_pbi::TcParser::FastMtS1,
       {26, 2, 2,
        PROTOBUF_FIELD_OFFSET(AppendRowsResponse, _impl_.updated_schema_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.cloud.bigquery.storage.v1.AppendRowsResponse.AppendResult append_result = 1;
      {PROTOBUF_FIELD_OFFSET(AppendRowsResponse, _impl_.response_.append_result_), _Internal::kOneofCaseOffset + 0, 0, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.rpc.Status error = 2;
      {PROTOBUF_FIELD_OFFSET(AppendRowsResponse, _impl_.response_.error_), _Internal::kOneofCaseOffset + 0, 1, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.cloud.bigquery.storage.v1.TableSchema updated_schema = 3;
      {PROTOBUF_FIELD_OFFSET(AppendRowsResponse, _impl_.updated_schema_), _Internal::kHasBitsOffset + 2, 2, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // repeated .google.cloud.bigquery.storage.v1.RowError row_errors = 4;
      {PROTOBUF_FIELD_OFFSET(AppendRowsResponse, _impl_.row_errors_), _Internal::kHasBitsOffset + 0, 3, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
      // string write_stream = 5;
      {PROTOBUF_FIELD_OFFSET(AppendRowsResponse, _impl_.write_stream_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::AppendRowsResponse_AppendResult>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::cloud::bigquery::storage::v1::AppendRowsResponse_AppendResult_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::rpc::Status>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::rpc::Status_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::TableSchema>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::cloud::bigquery::storage::v1::TableSchema_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::RowError>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::cloud::bigquery::storage::v1::RowError_globals_},
        #endif
    }},
    {{
      "\63\0\0\0\0\14\0\0"
      "google.cloud.bigquery.storage.v1.AppendRowsResponse"
      "write_stream"
    }},
  };
}


inline constexpr AppendRowsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        row_errors_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::cloud::bigquery::storage::v1::AppendRowsResponse,
            PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsResponse, _impl_.row_errors_)>()
         }
        ,
        write_stream_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        updated_schema_{nullptr},
        response_{},
        _oneof_case_{} {}

template <typename>
constexpr AppendRowsResponse::AppendRowsResponse(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL AppendRowsResponse::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) AppendRowsResponse(arena);
}
constexpr auto AppendRowsResponse::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(AppendRowsResponse), alignof(AppendRowsResponse));
}
constexpr auto AppendRowsResponse::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &AppendRowsResponse::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<AppendRowsResponse>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &AppendRowsResponse::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<AppendRowsResponse>(), &AppendRowsResponse::ByteSizeLong,
              &AppendRowsResponse::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(AppendRowsResponse, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[13],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct AppendRowsResponseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr AppendRowsResponseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 AppendRowsResponse_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(AppendRowsResponse::InternalGenerateClassData_(
            _default, &AppendRowsResponse_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<AppendRowsResponse>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~AppendRowsResponseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) AppendRowsResponse _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<AppendRowsResponse>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(AppendRowsResponseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST AppendRowsResponseGlobalsTypeInternal AppendRowsResponse_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* AppendRowsResponse_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return AppendRowsResponse_globals_.GetClassData();
#else
  return AppendRowsResponse_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class CreateWriteStreamRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<CreateWriteStreamRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(CreateWriteStreamRequest, _impl_._has_bits_);
};

constexpr CreateWriteStreamRequest::ParseTableT_ CreateWriteStreamRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(CreateWriteStreamRequest, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::CreateWriteStreamRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.cloud.bigquery.storage.v1.WriteStream write_stream = 2 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(CreateWriteStreamRequest, _impl_.write_stream_)}},
      // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(CreateWriteStreamRequest, _impl_.parent_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(CreateWriteStreamRequest, _impl_.parent_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.cloud.bigquery.storage.v1.WriteStream write_stream = 2 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(CreateWriteStreamRequest, _impl_.write_stream_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::WriteStream>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::cloud::bigquery::storage::v1::WriteStream_globals_},
        #endif
    }},
    {{
      "\71\6\0\0\0\0\0\0"
      "google.cloud.bigquery.storage.v1.CreateWriteStreamRequest"
      "parent"
    }},
  };
}


inline constexpr CreateWriteStreamRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        parent_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        write_stream_{nullptr} {}

template <typename>
constexpr CreateWriteStreamRequest::CreateWriteStreamRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL CreateWriteStreamRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) CreateWriteStreamRequest(arena);
}
constexpr auto CreateWriteStreamRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(CreateWriteStreamRequest), alignof(CreateWriteStreamRequest));
}
constexpr auto CreateWriteStreamRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &CreateWriteStreamRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<CreateWriteStreamRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &CreateWriteStreamRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<CreateWriteStreamRequest>(), &CreateWriteStreamRequest::ByteSizeLong,
              &CreateWriteStreamRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(CreateWriteStreamRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[8],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct CreateWriteStreamRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr CreateWriteStreamRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 CreateWriteStreamRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(CreateWriteStreamRequest::InternalGenerateClassData_(
            _default, &CreateWriteStreamRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<CreateWriteStreamRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~CreateWriteStreamRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) CreateWriteStreamRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<CreateWriteStreamRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(CreateWriteStreamRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST CreateWriteStreamRequestGlobalsTypeInternal CreateWriteStreamRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* CreateWriteStreamRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return CreateWriteStreamRequest_globals_.GetClassData();
#else
  return CreateWriteStreamRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class CreateReadSessionRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<CreateReadSessionRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(CreateReadSessionRequest, _impl_._has_bits_);
};

constexpr CreateReadSessionRequest::ParseTableT_ CreateReadSessionRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(CreateReadSessionRequest, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::CreateReadSessionRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // int32 preferred_min_stream_count = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(CreateReadSessionRequest, _impl_.preferred_min_stream_count_), 3>(),
       {32, 3, 0,
        PROTOBUF_FIELD_OFFSET(CreateReadSessionRequest, _impl_.preferred_min_stream_count_)}},
      // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(CreateReadSessionRequest, _impl_.parent_)}},
      // .google.cloud.bigquery.storage.v1.ReadSession read_session = 2 [(.google.api.field_behavior) = REQUIRED];
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(CreateReadSessionRequest, _impl_.read_session_)}},
      // int32 max_stream_count = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(CreateReadSessionRequest, _impl_.max_stream_count_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(CreateReadSessionRequest, _impl_.max_stream_count_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(CreateReadSessionRequest, _impl_.parent_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.cloud.bigquery.storage.v1.ReadSession read_session = 2 [(.google.api.field_behavior) = REQUIRED];
      {PROTOBUF_FIELD_OFFSET(CreateReadSessionRequest, _impl_.read_session_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // int32 max_stream_count = 3;
      {PROTOBUF_FIELD_OFFSET(CreateReadSessionRequest, _impl_.max_stream_count_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // int32 preferred_min_stream_count = 4;
      {PROTOBUF_FIELD_OFFSET(CreateReadSessionRequest, _impl_.preferred_min_stream_count_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::ReadSession>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::cloud::bigquery::storage::v1::ReadSession_globals_},
        #endif
    }},
    {{
      "\71\6\0\0\0\0\0\0"
      "google.cloud.bigquery.storage.v1.CreateReadSessionRequest"
      "parent"
    }},
  };
}


inline constexpr CreateReadSessionRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        parent_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        read_session_{nullptr},
        max_stream_count_{0},
        preferred_min_stream_count_{0} {}

template <typename>
constexpr CreateReadSessionRequest::CreateReadSessionRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL CreateReadSessionRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) CreateReadSessionRequest(arena);
}
constexpr auto CreateReadSessionRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(CreateReadSessionRequest), alignof(CreateReadSessionRequest));
}
constexpr auto CreateReadSessionRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &CreateReadSessionRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<CreateReadSessionRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &CreateReadSessionRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<CreateReadSessionRequest>(), &CreateReadSessionRequest::ByteSizeLong,
              &CreateReadSessionRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(CreateReadSessionRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[0],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct CreateReadSessionRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr CreateReadSessionRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 CreateReadSessionRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(CreateReadSessionRequest::InternalGenerateClassData_(
            _default, &CreateReadSessionRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<CreateReadSessionRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~CreateReadSessionRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) CreateReadSessionRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<CreateReadSessionRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(CreateReadSessionRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST CreateReadSessionRequestGlobalsTypeInternal CreateReadSessionRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* CreateReadSessionRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return CreateReadSessionRequest_globals_.GetClassData();
#else
  return CreateReadSessionRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class AppendRowsRequest_ProtoData::_Internal {
 public:
  using HasBits = decltype(::std::declval<AppendRowsRequest_ProtoData>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(AppendRowsRequest_ProtoData, _impl_._has_bits_);
};

constexpr AppendRowsRequest_ProtoData::ParseTableT_ AppendRowsRequest_ProtoData::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(AppendRowsRequest_ProtoData, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::AppendRowsRequest_ProtoData>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .google.cloud.bigquery.storage.v1.ProtoRows rows = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(AppendRowsRequest_ProtoData, _impl_.rows_)}},
      // .google.cloud.bigquery.storage.v1.ProtoSchema writer_schema = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(AppendRowsRequest_ProtoData, _impl_.writer_schema_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .google.cloud.bigquery.storage.v1.ProtoSchema writer_schema = 1;
      {PROTOBUF_FIELD_OFFSET(AppendRowsRequest_ProtoData, _impl_.writer_schema_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.cloud.bigquery.storage.v1.ProtoRows rows = 2;
      {PROTOBUF_FIELD_OFFSET(AppendRowsRequest_ProtoData, _impl_.rows_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::ProtoSchema>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::cloud::bigquery::storage::v1::ProtoSchema_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::ProtoRows>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::cloud::bigquery::storage::v1::ProtoRows_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr AppendRowsRequest_ProtoData::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        writer_schema_{nullptr},
        rows_{nullptr} {}

template <typename>
constexpr AppendRowsRequest_ProtoData::AppendRowsRequest_ProtoData(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL AppendRowsRequest_ProtoData::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) AppendRowsRequest_ProtoData(arena);
}
constexpr auto AppendRowsRequest_ProtoData::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(AppendRowsRequest_ProtoData), alignof(AppendRowsRequest_ProtoData));
}
constexpr auto AppendRowsRequest_ProtoData::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          AppendRowsRequest_ProtoData::IsInitializedImpl,
          &AppendRowsRequest_ProtoData::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<AppendRowsRequest_ProtoData>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &AppendRowsRequest_ProtoData::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<AppendRowsRequest_ProtoData>(), &AppendRowsRequest_ProtoData::ByteSizeLong,
              &AppendRowsRequest_ProtoData::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(AppendRowsRequest_ProtoData, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[9],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct AppendRowsRequest_ProtoDataGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr AppendRowsRequest_ProtoDataGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 AppendRowsRequest_ProtoData_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(AppendRowsRequest_ProtoData::InternalGenerateClassData_(
            _default, &AppendRowsRequest_ProtoData_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<AppendRowsRequest_ProtoData>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~AppendRowsRequest_ProtoDataGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) AppendRowsRequest_ProtoData _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<AppendRowsRequest_ProtoData>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(AppendRowsRequest_ProtoDataGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST AppendRowsRequest_ProtoDataGlobalsTypeInternal AppendRowsRequest_ProtoData_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* AppendRowsRequest_ProtoData_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return AppendRowsRequest_ProtoData_globals_.GetClassData();
#else
  return AppendRowsRequest_ProtoData_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class AppendRowsRequest::_Internal {
 public:
  using HasBits = decltype(::std::declval<AppendRowsRequest>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(AppendRowsRequest, _impl_._has_bits_);
  static constexpr ::int32_t kOneofCaseOffset =
      PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsRequest, _impl_._oneof_case_);
};

constexpr AppendRowsRequest::ParseTableT_ AppendRowsRequest::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(AppendRowsRequest, _impl_._has_bits_),
      0, // no _extensions_
      7, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967188,  // skipmap
      offsetof(ParseTableT_, field_entries),
      5,  // num_field_entries
      3,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::AppendRowsRequest>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // string write_stream = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(AppendRowsRequest, _impl_.write_stream_)}},
      // .google.protobuf.Int64Value offset = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 2, 0,
        PROTOBUF_FIELD_OFFSET(AppendRowsRequest, _impl_.offset_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      // string trace_id = 6;
      {::_pbi::TcParser::FastUS1,
       {50, 1, 0,
        PROTOBUF_FIELD_OFFSET(AppendRowsRequest, _impl_.trace_id_)}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // string write_stream = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
      {PROTOBUF_FIELD_OFFSET(AppendRowsRequest, _impl_.write_stream_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .google.protobuf.Int64Value offset = 2;
      {PROTOBUF_FIELD_OFFSET(AppendRowsRequest, _impl_.offset_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .google.cloud.bigquery.storage.v1.AppendRowsRequest.ProtoData proto_rows = 4;
      {PROTOBUF_FIELD_OFFSET(AppendRowsRequest, _impl_.rows_.proto_rows_), _Internal::kOneofCaseOffset + 0, 1, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // string trace_id = 6;
      {PROTOBUF_FIELD_OFFSET(AppendRowsRequest, _impl_.trace_id_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // map<string, .google.cloud.bigquery.storage.v1.AppendRowsRequest.MissingValueInterpretation> missing_value_interpretations = 7;
      {PROTOBUF_FIELD_OFFSET(AppendRowsRequest, _impl_.missing_value_interpretations_), _Internal::kHasBitsOffset + 3, 2, (0 | ::_fl::kFcRepeated | ::_fl::kMap)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::protobuf::Int64Value>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::protobuf::Int64Value_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::google::cloud::bigquery::storage::v1::AppendRowsRequest_ProtoData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::google::cloud::bigquery::storage::v1::AppendRowsRequest_ProtoData_globals_},
        #endif
        {::_pbi::TcParser::GetMapAuxInfo(
            1, 0, 9, 14, 0)},
    }},
    {{
      "\62\14\0\0\10\35\0\0"
      "google.cloud.bigquery.storage.v1.AppendRowsRequest"
      "write_stream"
      "trace_id"
      "missing_value_interpretations"
    }},
  };
}


inline constexpr AppendRowsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        write_stream_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        trace_id_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        offset_{nullptr},
        missing_value_interpretations_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::cloud::bigquery::storage::v1::AppendRowsRequest,
            PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsRequest, _impl_.missing_value_interpretations_)>()
         }
        ,
        rows_{},
        _oneof_case_{} {}

template <typename>
constexpr AppendRowsRequest::AppendRowsRequest(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL AppendRowsRequest::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) AppendRowsRequest(arena);
}
constexpr auto AppendRowsRequest::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(AppendRowsRequest), alignof(AppendRowsRequest));
}
constexpr auto AppendRowsRequest::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          AppendRowsRequest::IsInitializedImpl,
          &AppendRowsRequest::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<AppendRowsRequest>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &AppendRowsRequest::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<AppendRowsRequest>(), &AppendRowsRequest::ByteSizeLong,
              &AppendRowsRequest::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(AppendRowsRequest, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[11],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct AppendRowsRequestGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr AppendRowsRequestGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 AppendRowsRequest_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(AppendRowsRequest::InternalGenerateClassData_(
            _default, &AppendRowsRequest_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<AppendRowsRequest>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~AppendRowsRequestGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) AppendRowsRequest _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<AppendRowsRequest>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(AppendRowsRequestGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST AppendRowsRequestGlobalsTypeInternal AppendRowsRequest_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* AppendRowsRequest_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return AppendRowsRequest_globals_.GetClassData();
#else
  return AppendRowsRequest_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
}  // namespace v1
}  // namespace storage
}  // namespace bigquery
}  // namespace cloud
}  // namespace google
static const ::_pb::EnumDescriptor* PROTOBUF_NONNULL
    file_level_enum_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto[3];
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto = nullptr;
const ::uint32_t
    TableStruct_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::CreateReadSessionRequest, _impl_._has_bits_),
        7, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::CreateReadSessionRequest, _impl_.parent_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::CreateReadSessionRequest, _impl_.read_session_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::CreateReadSessionRequest, _impl_.max_stream_count_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::CreateReadSessionRequest, _impl_.preferred_min_stream_count_),
        0,
        1,
        2,
        3,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ReadRowsRequest, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ReadRowsRequest, _impl_.read_stream_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ReadRowsRequest, _impl_.offset_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ThrottleState, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ThrottleState, _impl_.throttle_percent_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::StreamStats_Progress, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::StreamStats_Progress, _impl_.at_response_start_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::StreamStats_Progress, _impl_.at_response_end_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::StreamStats, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::StreamStats, _impl_.progress_),
        0,
        0x085, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ReadRowsResponse, _impl_._has_bits_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ReadRowsResponse, _impl_._oneof_case_[0]),
        13, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ReadRowsResponse, _impl_.rows_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ReadRowsResponse, _impl_.rows_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ReadRowsResponse, _impl_.row_count_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ReadRowsResponse, _impl_.stats_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ReadRowsResponse, _impl_.throttle_state_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ReadRowsResponse, _impl_.schema_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ReadRowsResponse, _impl_.schema_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ReadRowsResponse, _impl_.rows_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::ReadRowsResponse, _impl_.schema_),
        ~0u,
        ~0u,
        2,
        0,
        1,
        ~0u,
        ~0u,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::SplitReadStreamRequest, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::SplitReadStreamRequest, _impl_.name_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::SplitReadStreamRequest, _impl_.fraction_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::SplitReadStreamResponse, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::SplitReadStreamResponse, _impl_.primary_stream_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::SplitReadStreamResponse, _impl_.remainder_stream_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::CreateWriteStreamRequest, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::CreateWriteStreamRequest, _impl_.parent_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::CreateWriteStreamRequest, _impl_.write_stream_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsRequest_ProtoData, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsRequest_ProtoData, _impl_.writer_schema_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsRequest_ProtoData, _impl_.rows_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse, _impl_.key_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse, _impl_.value_),
        0,
        1,
        0x085, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsRequest, _impl_._has_bits_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsRequest, _impl_._oneof_case_[0]),
        10, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsRequest, _impl_.write_stream_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsRequest, _impl_.offset_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsRequest, _impl_.rows_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsRequest, _impl_.trace_id_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsRequest, _impl_.missing_value_interpretations_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsRequest, _impl_.rows_),
        0,
        2,
        ~0u,
        1,
        3,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsResponse_AppendResult, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsResponse_AppendResult, _impl_.offset_),
        0,
        0x085, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsResponse, _impl_._has_bits_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsResponse, _impl_._oneof_case_[0]),
        10, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsResponse, _impl_.response_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsResponse, _impl_.response_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsResponse, _impl_.updated_schema_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsResponse, _impl_.row_errors_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsResponse, _impl_.write_stream_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsResponse, _impl_.response_),
        ~0u,
        ~0u,
        2,
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::GetWriteStreamRequest, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::GetWriteStreamRequest, _impl_.name_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::GetWriteStreamRequest, _impl_.view_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsRequest, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsRequest, _impl_.parent_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsRequest, _impl_.write_streams_),
        1,
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsResponse, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsResponse, _impl_.commit_time_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsResponse, _impl_.stream_errors_),
        1,
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::FinalizeWriteStreamRequest, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::FinalizeWriteStreamRequest, _impl_.name_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::FinalizeWriteStreamResponse, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::FinalizeWriteStreamResponse, _impl_.row_count_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::FlushRowsRequest, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::FlushRowsRequest, _impl_.write_stream_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::FlushRowsRequest, _impl_.offset_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::FlushRowsResponse, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::FlushRowsResponse, _impl_.offset_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::StorageError, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::StorageError, _impl_.code_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::StorageError, _impl_.entity_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::StorageError, _impl_.error_message_),
        2,
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::RowError, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::RowError, _impl_.index_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::RowError, _impl_.code_),
        PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::RowError, _impl_.message_),
        1,
        2,
        0,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::google::cloud::bigquery::storage::v1::CreateReadSessionRequest)},
        {11, sizeof(::google::cloud::bigquery::storage::v1::ReadRowsRequest)},
        {18, sizeof(::google::cloud::bigquery::storage::v1::ThrottleState)},
        {23, sizeof(::google::cloud::bigquery::storage::v1::StreamStats_Progress)},
        {30, sizeof(::google::cloud::bigquery::storage::v1::StreamStats)},
        {35, sizeof(::google::cloud::bigquery::storage::v1::ReadRowsResponse)},
        {55, sizeof(::google::cloud::bigquery::storage::v1::SplitReadStreamRequest)},
        {62, sizeof(::google::cloud::bigquery::storage::v1::SplitReadStreamResponse)},
        {69, sizeof(::google::cloud::bigquery::storage::v1::CreateWriteStreamRequest)},
        {76, sizeof(::google::cloud::bigquery::storage::v1::AppendRowsRequest_ProtoData)},
        {83, sizeof(::google::cloud::bigquery::storage::v1::AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse)},
        {90, sizeof(::google::cloud::bigquery::storage::v1::AppendRowsRequest)},
        {105, sizeof(::google::cloud::bigquery::storage::v1::AppendRowsResponse_AppendResult)},
        {110, sizeof(::google::cloud::bigquery::storage::v1::AppendRowsResponse)},
        {125, sizeof(::google::cloud::bigquery::storage::v1::GetWriteStreamRequest)},
        {132, sizeof(::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsRequest)},
        {139, sizeof(::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsResponse)},
        {146, sizeof(::google::cloud::bigquery::storage::v1::FinalizeWriteStreamRequest)},
        {151, sizeof(::google::cloud::bigquery::storage::v1::FinalizeWriteStreamResponse)},
        {156, sizeof(::google::cloud::bigquery::storage::v1::FlushRowsRequest)},
        {163, sizeof(::google::cloud::bigquery::storage::v1::FlushRowsResponse)},
        {168, sizeof(::google::cloud::bigquery::storage::v1::StorageError)},
        {177, sizeof(::google::cloud::bigquery::storage::v1::RowError)},
};
static const ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const
    file_message_globals[] = {
        &::google::cloud::bigquery::storage::v1::CreateReadSessionRequest_globals_,
        &::google::cloud::bigquery::storage::v1::ReadRowsRequest_globals_,
        &::google::cloud::bigquery::storage::v1::ThrottleState_globals_,
        &::google::cloud::bigquery::storage::v1::StreamStats_Progress_globals_,
        &::google::cloud::bigquery::storage::v1::StreamStats_globals_,
        &::google::cloud::bigquery::storage::v1::ReadRowsResponse_globals_,
        &::google::cloud::bigquery::storage::v1::SplitReadStreamRequest_globals_,
        &::google::cloud::bigquery::storage::v1::SplitReadStreamResponse_globals_,
        &::google::cloud::bigquery::storage::v1::CreateWriteStreamRequest_globals_,
        &::google::cloud::bigquery::storage::v1::AppendRowsRequest_ProtoData_globals_,
        &::google::cloud::bigquery::storage::v1::AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse_globals_,
        &::google::cloud::bigquery::storage::v1::AppendRowsRequest_globals_,
        &::google::cloud::bigquery::storage::v1::AppendRowsResponse_AppendResult_globals_,
        &::google::cloud::bigquery::storage::v1::AppendRowsResponse_globals_,
        &::google::cloud::bigquery::storage::v1::GetWriteStreamRequest_globals_,
        &::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsRequest_globals_,
        &::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsResponse_globals_,
        &::google::cloud::bigquery::storage::v1::FinalizeWriteStreamRequest_globals_,
        &::google::cloud::bigquery::storage::v1::FinalizeWriteStreamResponse_globals_,
        &::google::cloud::bigquery::storage::v1::FlushRowsRequest_globals_,
        &::google::cloud::bigquery::storage::v1::FlushRowsResponse_globals_,
        &::google::cloud::bigquery::storage::v1::StorageError_globals_,
        &::google::cloud::bigquery::storage::v1::RowError_globals_,
};
const char descriptor_table_protodef_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n.google/cloud/bigquery/storage/v1/stora"
    "ge.proto\022 google.cloud.bigquery.storage."
    "v1\032\034google/api/annotations.proto\032\027google"
    "/api/client.proto\032\037google/api/field_beha"
    "vior.proto\032\031google/api/resource.proto\032,g"
    "oogle/cloud/bigquery/storage/v1/arrow.pr"
    "oto\032+google/cloud/bigquery/storage/v1/av"
    "ro.proto\032/google/cloud/bigquery/storage/"
    "v1/protobuf.proto\032-google/cloud/bigquery"
    "/storage/v1/stream.proto\032,google/cloud/b"
    "igquery/storage/v1/table.proto\032\037google/p"
    "rotobuf/timestamp.proto\032\036google/protobuf"
    "/wrappers.proto\032\027google/rpc/status.proto"
    "\"\351\001\n\030CreateReadSessionRequest\022D\n\006parent\030"
    "\001 \001(\tB4\342A\001\002\372A-\n+cloudresourcemanager.goo"
    "gleapis.com/Project\022I\n\014read_session\030\002 \001("
    "\0132-.google.cloud.bigquery.storage.v1.Rea"
    "dSessionB\004\342A\001\002\022\030\n\020max_stream_count\030\003 \001(\005"
    "\022\"\n\032preferred_min_stream_count\030\004 \001(\005\"j\n\017"
    "ReadRowsRequest\022G\n\013read_stream\030\001 \001(\tB2\342A"
    "\001\002\372A+\n)bigquerystorage.googleapis.com/Re"
    "adStream\022\016\n\006offset\030\002 \001(\003\")\n\rThrottleStat"
    "e\022\030\n\020throttle_percent\030\001 \001(\005\"\227\001\n\013StreamSt"
    "ats\022H\n\010progress\030\002 \001(\01326.google.cloud.big"
    "query.storage.v1.StreamStats.Progress\032>\n"
    "\010Progress\022\031\n\021at_response_start\030\001 \001(\001\022\027\n\017"
    "at_response_end\030\002 \001(\001\"\351\003\n\020ReadRowsRespon"
    "se\022\?\n\tavro_rows\030\003 \001(\0132*.google.cloud.big"
    "query.storage.v1.AvroRowsH\000\022P\n\022arrow_rec"
    "ord_batch\030\004 \001(\01322.google.cloud.bigquery."
    "storage.v1.ArrowRecordBatchH\000\022\021\n\trow_cou"
    "nt\030\006 \001(\003\022<\n\005stats\030\002 \001(\0132-.google.cloud.b"
    "igquery.storage.v1.StreamStats\022G\n\016thrott"
    "le_state\030\005 \001(\0132/.google.cloud.bigquery.s"
    "torage.v1.ThrottleState\022I\n\013avro_schema\030\007"
    " \001(\0132,.google.cloud.bigquery.storage.v1."
    "AvroSchemaB\004\342A\001\003H\001\022K\n\014arrow_schema\030\010 \001(\013"
    "2-.google.cloud.bigquery.storage.v1.Arro"
    "wSchemaB\004\342A\001\003H\001B\006\n\004rowsB\010\n\006schema\"l\n\026Spl"
    "itReadStreamRequest\022@\n\004name\030\001 \001(\tB2\342A\001\002\372"
    "A+\n)bigquerystorage.googleapis.com/ReadS"
    "tream\022\020\n\010fraction\030\002 \001(\001\"\247\001\n\027SplitReadStr"
    "eamResponse\022D\n\016primary_stream\030\001 \001(\0132,.go"
    "ogle.cloud.bigquery.storage.v1.ReadStrea"
    "m\022F\n\020remainder_stream\030\002 \001(\0132,.google.clo"
    "ud.bigquery.storage.v1.ReadStream\"\235\001\n\030Cr"
    "eateWriteStreamRequest\0226\n\006parent\030\001 \001(\tB&"
    "\342A\001\002\372A\037\n\035bigquery.googleapis.com/Table\022I"
    "\n\014write_stream\030\002 \001(\0132-.google.cloud.bigq"
    "uery.storage.v1.WriteStreamB\004\342A\001\002\"\212\006\n\021Ap"
    "pendRowsRequest\022I\n\014write_stream\030\001 \001(\tB3\342"
    "A\001\002\372A,\n*bigquerystorage.googleapis.com/W"
    "riteStream\022+\n\006offset\030\002 \001(\0132\033.google.prot"
    "obuf.Int64Value\022S\n\nproto_rows\030\004 \001(\0132=.go"
    "ogle.cloud.bigquery.storage.v1.AppendRow"
    "sRequest.ProtoDataH\000\022\020\n\010trace_id\030\006 \001(\t\022{"
    "\n\035missing_value_interpretations\030\007 \003(\0132T."
    "google.cloud.bigquery.storage.v1.AppendR"
    "owsRequest.MissingValueInterpretationsEn"
    "try\032\214\001\n\tProtoData\022D\n\rwriter_schema\030\001 \001(\013"
    "2-.google.cloud.bigquery.storage.v1.Prot"
    "oSchema\0229\n\004rows\030\002 \001(\0132+.google.cloud.big"
    "query.storage.v1.ProtoRows\032\222\001\n MissingVa"
    "lueInterpretationsEntry\022\013\n\003key\030\001 \001(\t\022]\n\005"
    "value\030\002 \001(\0162N.google.cloud.bigquery.stor"
    "age.v1.AppendRowsRequest.MissingValueInt"
    "erpretation:\0028\001\"m\n\032MissingValueInterpret"
    "ation\022,\n(MISSING_VALUE_INTERPRETATION_UN"
    "SPECIFIED\020\000\022\016\n\nNULL_VALUE\020\001\022\021\n\rDEFAULT_V"
    "ALUE\020\002B\006\n\004rows\"\373\002\n\022AppendRowsResponse\022Z\n"
    "\rappend_result\030\001 \001(\0132A.google.cloud.bigq"
    "uery.storage.v1.AppendRowsResponse.Appen"
    "dResultH\000\022#\n\005error\030\002 \001(\0132\022.google.rpc.St"
    "atusH\000\022E\n\016updated_schema\030\003 \001(\0132-.google."
    "cloud.bigquery.storage.v1.TableSchema\022>\n"
    "\nrow_errors\030\004 \003(\0132*.google.cloud.bigquer"
    "y.storage.v1.RowError\022\024\n\014write_stream\030\005 "
    "\001(\t\032;\n\014AppendResult\022+\n\006offset\030\001 \001(\0132\033.go"
    "ogle.protobuf.Int64ValueB\n\n\010response\"\233\001\n"
    "\025GetWriteStreamRequest\022A\n\004name\030\001 \001(\tB3\342A"
    "\001\002\372A,\n*bigquerystorage.googleapis.com/Wr"
    "iteStream\022\?\n\004view\030\003 \001(\01621.google.cloud.b"
    "igquery.storage.v1.WriteStreamView\"u\n\036Ba"
    "tchCommitWriteStreamsRequest\0226\n\006parent\030\001"
    " \001(\tB&\342A\001\002\372A\037\n\035bigquery.googleapis.com/T"
    "able\022\033\n\rwrite_streams\030\002 \003(\tB\004\342A\001\002\"\231\001\n\037Ba"
    "tchCommitWriteStreamsResponse\022/\n\013commit_"
    "time\030\001 \001(\0132\032.google.protobuf.Timestamp\022E"
    "\n\rstream_errors\030\002 \003(\0132..google.cloud.big"
    "query.storage.v1.StorageError\"_\n\032Finaliz"
    "eWriteStreamRequest\022A\n\004name\030\001 \001(\tB3\342A\001\002\372"
    "A,\n*bigquerystorage.googleapis.com/Write"
    "Stream\"0\n\033FinalizeWriteStreamResponse\022\021\n"
    "\trow_count\030\001 \001(\003\"\212\001\n\020FlushRowsRequest\022I\n"
    "\014write_stream\030\001 \001(\tB3\342A\001\002\372A,\n*bigqueryst"
    "orage.googleapis.com/WriteStream\022+\n\006offs"
    "et\030\002 \001(\0132\033.google.protobuf.Int64Value\"#\n"
    "\021FlushRowsResponse\022\016\n\006offset\030\001 \001(\003\"\244\004\n\014S"
    "torageError\022M\n\004code\030\001 \001(\0162\?.google.cloud"
    ".bigquery.storage.v1.StorageError.Storag"
    "eErrorCode\022\016\n\006entity\030\002 \001(\t\022\025\n\rerror_mess"
    "age\030\003 \001(\t\"\235\003\n\020StorageErrorCode\022\"\n\036STORAG"
    "E_ERROR_CODE_UNSPECIFIED\020\000\022\023\n\017TABLE_NOT_"
    "FOUND\020\001\022\034\n\030STREAM_ALREADY_COMMITTED\020\002\022\024\n"
    "\020STREAM_NOT_FOUND\020\003\022\027\n\023INVALID_STREAM_TY"
    "PE\020\004\022\030\n\024INVALID_STREAM_STATE\020\005\022\024\n\020STREAM"
    "_FINALIZED\020\006\022 \n\034SCHEMA_MISMATCH_EXTRA_FI"
    "ELDS\020\007\022\031\n\025OFFSET_ALREADY_EXISTS\020\010\022\027\n\023OFF"
    "SET_OUT_OF_RANGE\020\t\022\025\n\021CMEK_NOT_PROVIDED\020"
    "\n\022\031\n\025INVALID_CMEK_PROVIDED\020\013\022\031\n\025CMEK_ENC"
    "RYPTION_ERROR\020\014\022\025\n\021KMS_SERVICE_ERROR\020\r\022\031"
    "\n\025KMS_PERMISSION_DENIED\020\016\"\263\001\n\010RowError\022\r"
    "\n\005index\030\001 \001(\003\022E\n\004code\030\002 \001(\01627.google.clo"
    "ud.bigquery.storage.v1.RowError.RowError"
    "Code\022\017\n\007message\030\003 \001(\t\"@\n\014RowErrorCode\022\036\n"
    "\032ROW_ERROR_CODE_UNSPECIFIED\020\000\022\020\n\014FIELDS_"
    "ERROR\020\0012\222\006\n\014BigQueryRead\022\351\001\n\021CreateReadS"
    "ession\022:.google.cloud.bigquery.storage.v"
    "1.CreateReadSessionRequest\032-.google.clou"
    "d.bigquery.storage.v1.ReadSession\"i\332A$pa"
    "rent,read_session,max_stream_count\202\323\344\223\002<"
    "\"7/v1/{read_session.table=projects/*/dat"
    "asets/*/tables/*}:\001*\022\317\001\n\010ReadRows\0221.goog"
    "le.cloud.bigquery.storage.v1.ReadRowsReq"
    "uest\0322.google.cloud.bigquery.storage.v1."
    "ReadRowsResponse\"Z\332A\022read_stream,offset\202"
    "\323\344\223\002\?\022=/v1/{read_stream=projects/*/locat"
    "ions/*/sessions/*/streams/*}0\001\022\306\001\n\017Split"
    "ReadStream\0228.google.cloud.bigquery.stora"
    "ge.v1.SplitReadStreamRequest\0329.google.cl"
    "oud.bigquery.storage.v1.SplitReadStreamR"
    "esponse\">\202\323\344\223\0028\0226/v1/{name=projects/*/lo"
    "cations/*/sessions/*/streams/*}\032{\312A\036bigq"
    "uerystorage.googleapis.com\322AWhttps://www"
    ".googleapis.com/auth/bigquery,https://ww"
    "w.googleapis.com/auth/cloud-platform2\274\013\n"
    "\rBigQueryWrite\022\327\001\n\021CreateWriteStream\022:.g"
    "oogle.cloud.bigquery.storage.v1.CreateWr"
    "iteStreamRequest\032-.google.cloud.bigquery"
    ".storage.v1.WriteStream\"W\332A\023parent,write"
    "_stream\202\323\344\223\002;\"+/v1/{parent=projects/*/da"
    "tasets/*/tables/*}:\014write_stream\022\322\001\n\nApp"
    "endRows\0223.google.cloud.bigquery.storage."
    "v1.AppendRowsRequest\0324.google.cloud.bigq"
    "uery.storage.v1.AppendRowsResponse\"U\332A\014w"
    "rite_stream\202\323\344\223\002@\";/v1/{write_stream=pro"
    "jects/*/datasets/*/tables/*/streams/*}:\001"
    "*(\0010\001\022\277\001\n\016GetWriteStream\0227.google.cloud."
    "bigquery.storage.v1.GetWriteStreamReques"
    "t\032-.google.cloud.bigquery.storage.v1.Wri"
    "teStream\"E\332A\004name\202\323\344\223\0028\"3/v1/{name=proje"
    "cts/*/datasets/*/tables/*/streams/*}:\001*\022"
    "\331\001\n\023FinalizeWriteStream\022<.google.cloud.b"
    "igquery.storage.v1.FinalizeWriteStreamRe"
    "quest\032=.google.cloud.bigquery.storage.v1"
    ".FinalizeWriteStreamResponse\"E\332A\004name\202\323\344"
    "\223\0028\"3/v1/{name=projects/*/datasets/*/tab"
    "les/*/streams/*}:\001*\022\334\001\n\027BatchCommitWrite"
    "Streams\022@.google.cloud.bigquery.storage."
    "v1.BatchCommitWriteStreamsRequest\032A.goog"
    "le.cloud.bigquery.storage.v1.BatchCommit"
    "WriteStreamsResponse\"<\332A\006parent\202\323\344\223\002-\022+/"
    "v1/{parent=projects/*/datasets/*/tables/"
    "*}\022\313\001\n\tFlushRows\0222.google.cloud.bigquery"
    ".storage.v1.FlushRowsRequest\0323.google.cl"
    "oud.bigquery.storage.v1.FlushRowsRespons"
    "e\"U\332A\014write_stream\202\323\344\223\002@\";/v1/{write_str"
    "eam=projects/*/datasets/*/tables/*/strea"
    "ms/*}:\001*\032\260\001\312A\036bigquerystorage.googleapis"
    ".com\322A\213\001https://www.googleapis.com/auth/"
    "bigquery,https://www.googleapis.com/auth"
    "/bigquery.insertdata,https://www.googlea"
    "pis.com/auth/cloud-platformB\224\002\n$com.goog"
    "le.cloud.bigquery.storage.v1B\014StoragePro"
    "toP\001Z>cloud.google.com/go/bigquery/stora"
    "ge/apiv1/storagepb;storagepb\252\002 Google.Cl"
    "oud.BigQuery.Storage.V1\312\002 Google\\Cloud\\B"
    "igQuery\\Storage\\V1\352AU\n\035bigquery.googleap"
    "is.com/Table\0224projects/{project}/dataset"
    "s/{dataset}/tables/{table}b\006proto3"
};
static const ::_pbi::DescriptorTable* PROTOBUF_NONNULL const
    descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto_deps[12] = {
        &::descriptor_table_google_2fapi_2fannotations_2eproto,
        &::descriptor_table_google_2fapi_2fclient_2eproto,
        &::descriptor_table_google_2fapi_2ffield_5fbehavior_2eproto,
        &::descriptor_table_google_2fapi_2fresource_2eproto,
        &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2farrow_2eproto,
        &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2favro_2eproto,
        &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fprotobuf_2eproto,
        &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstream_2eproto,
        &::descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2ftable_2eproto,
        &::descriptor_table_google_2fprotobuf_2ftimestamp_2eproto,
        &::descriptor_table_google_2fprotobuf_2fwrappers_2eproto,
        &::descriptor_table_google_2frpc_2fstatus_2eproto,
};
static ::absl::once_flag descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto = {
    false,
    false,
    7194,
    descriptor_table_protodef_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
    "google/cloud/bigquery/storage/v1/storage.proto",
    &descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto_once,
    descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto_deps,
    12,
    23,
    schemas,
    file_message_globals,
    TableStruct_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto::offsets,
    file_level_enum_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
    file_level_service_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto,
};
namespace google {
namespace cloud {
namespace bigquery {
namespace storage {
namespace v1 {
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
AppendRowsRequest_MissingValueInterpretation_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto);
  return file_level_enum_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto[0];
}
PROTOBUF_CONSTINIT const uint32_t AppendRowsRequest_MissingValueInterpretation_internal_data_[] = {
    196608u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
StorageError_StorageErrorCode_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto);
  return file_level_enum_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto[1];
}
PROTOBUF_CONSTINIT const uint32_t StorageError_StorageErrorCode_internal_data_[] = {
    983040u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
RowError_RowErrorCode_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto);
  return file_level_enum_descriptors_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto[2];
}
PROTOBUF_CONSTINIT const uint32_t RowError_RowErrorCode_internal_data_[] = {
    131072u, 0u, };
// ===================================================================

void CreateReadSessionRequest::clear_read_session() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.read_session_ != nullptr) _impl_.read_session_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
CreateReadSessionRequest::CreateReadSessionRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, CreateReadSessionRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.CreateReadSessionRequest)
}
PROTOBUF_NDEBUG_INLINE CreateReadSessionRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::CreateReadSessionRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        parent_(arena, from.parent_) {}

CreateReadSessionRequest::CreateReadSessionRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const CreateReadSessionRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, CreateReadSessionRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  CreateReadSessionRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.read_session_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.read_session_)
                : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, max_stream_count_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, max_stream_count_),
           offsetof(Impl_, preferred_min_stream_count_) -
               offsetof(Impl_, max_stream_count_) +
               sizeof(Impl_::preferred_min_stream_count_));

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.CreateReadSessionRequest)
}
PROTOBUF_NDEBUG_INLINE CreateReadSessionRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        parent_(arena) {}

inline void CreateReadSessionRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, read_session_),
           0,
           offsetof(Impl_, preferred_min_stream_count_) -
               offsetof(Impl_, read_session_) +
               sizeof(Impl_::preferred_min_stream_count_));
}
CreateReadSessionRequest::~CreateReadSessionRequest() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.CreateReadSessionRequest)
  SharedDtor(*this);
}
inline void CreateReadSessionRequest::SharedDtor(MessageLite& self) {
  CreateReadSessionRequest& this_ = static_cast<CreateReadSessionRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.parent_.Destroy();
  delete this_._impl_.read_session_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull CreateReadSessionRequest_class_data_ =
        CreateReadSessionRequest::InternalGenerateClassData_(CreateReadSessionRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CreateReadSessionRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CreateReadSessionRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(CreateReadSessionRequest_class_data_.tc_table);
  return CreateReadSessionRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CreateReadSessionRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CreateReadSessionRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&CreateReadSessionRequest_globals_));
  return CreateReadSessionRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const CreateReadSessionRequest::ParseTableT_
    CreateReadSessionRequest::_table_ =
        CreateReadSessionRequest::InternalGenerateParseTable_(CreateReadSessionRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void CreateReadSessionRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.CreateReadSessionRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.parent_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.read_session_ != nullptr);
      _impl_.read_session_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000000cU)) {
    ::memset(&_impl_.max_stream_count_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.preferred_min_stream_count_) -
        reinterpret_cast<char*>(&_impl_.max_stream_count_)) + sizeof(_impl_.preferred_min_stream_count_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL CreateReadSessionRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const CreateReadSessionRequest& this_ = static_cast<const CreateReadSessionRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL CreateReadSessionRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const CreateReadSessionRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.CreateReadSessionRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_parent().empty()) {
      const ::std::string& _s = this_._internal_parent();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.cloud.bigquery.storage.v1.CreateReadSessionRequest.parent");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // .google.cloud.bigquery.storage.v1.ReadSession read_session = 2 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.read_session_, this_._impl_.read_session_->GetCachedSize(), target,
        stream);
  }

  // int32 max_stream_count = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_max_stream_count() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<3>(
              stream, this_._internal_max_stream_count(), target);
    }
  }

  // int32 preferred_min_stream_count = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_preferred_min_stream_count() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
              stream, this_._internal_preferred_min_stream_count(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.CreateReadSessionRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t CreateReadSessionRequest::ByteSizeLong(const MessageLite& base) {
  const CreateReadSessionRequest& this_ = static_cast<const CreateReadSessionRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t CreateReadSessionRequest::ByteSizeLong() const {
  const CreateReadSessionRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.CreateReadSessionRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_parent().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_parent());
      }
    }
    // .google.cloud.bigquery.storage.v1.ReadSession read_session = 2 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.read_session_);
    }
    // int32 max_stream_count = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_max_stream_count() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_max_stream_count());
      }
    }
    // int32 preferred_min_stream_count = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_preferred_min_stream_count() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_preferred_min_stream_count());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void CreateReadSessionRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<CreateReadSessionRequest*>(&to_msg);
  auto& from = static_cast<const CreateReadSessionRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.CreateReadSessionRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_parent().empty()) {
        _this->_internal_set_parent(from._internal_parent());
      } else {
        if (_this->_impl_.parent_.IsDefault()) {
          _this->_internal_set_parent("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.read_session_ != nullptr);
      if (_this->_impl_.read_session_ == nullptr) {
        _this->_impl_.read_session_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.read_session_);
      } else {
        _this->_impl_.read_session_->MergeFrom(*from._impl_.read_session_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_max_stream_count() != 0) {
        _this->_impl_.max_stream_count_ = from._impl_.max_stream_count_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_preferred_min_stream_count() != 0) {
        _this->_impl_.preferred_min_stream_count_ = from._impl_.preferred_min_stream_count_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void CreateReadSessionRequest::CopyFrom(const CreateReadSessionRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.CreateReadSessionRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void CreateReadSessionRequest::InternalSwap(CreateReadSessionRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.parent_, &other->_impl_.parent_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(CreateReadSessionRequest, _impl_.preferred_min_stream_count_)
      + sizeof(CreateReadSessionRequest::_impl_.preferred_min_stream_count_)
      - PROTOBUF_FIELD_OFFSET(CreateReadSessionRequest, _impl_.read_session_)>(
          reinterpret_cast<char*>(&_impl_.read_session_),
          reinterpret_cast<char*>(&other->_impl_.read_session_));
}

::google::protobuf::Metadata CreateReadSessionRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ReadRowsRequest::ReadRowsRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ReadRowsRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.ReadRowsRequest)
}
PROTOBUF_NDEBUG_INLINE ReadRowsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::ReadRowsRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        read_stream_(arena, from.read_stream_) {}

ReadRowsRequest::ReadRowsRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ReadRowsRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ReadRowsRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ReadRowsRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.offset_ = from._impl_.offset_;

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.ReadRowsRequest)
}
PROTOBUF_NDEBUG_INLINE ReadRowsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        read_stream_(arena) {}

inline void ReadRowsRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.offset_ = {};
}
ReadRowsRequest::~ReadRowsRequest() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.ReadRowsRequest)
  SharedDtor(*this);
}
inline void ReadRowsRequest::SharedDtor(MessageLite& self) {
  ReadRowsRequest& this_ = static_cast<ReadRowsRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.read_stream_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ReadRowsRequest_class_data_ =
        ReadRowsRequest::InternalGenerateClassData_(ReadRowsRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ReadRowsRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ReadRowsRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ReadRowsRequest_class_data_.tc_table);
  return ReadRowsRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ReadRowsRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ReadRowsRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ReadRowsRequest_globals_));
  return ReadRowsRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ReadRowsRequest::ParseTableT_
    ReadRowsRequest::_table_ =
        ReadRowsRequest::InternalGenerateParseTable_(ReadRowsRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ReadRowsRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.ReadRowsRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.read_stream_.ClearNonDefaultToEmpty();
  }
  _impl_.offset_ = ::int64_t{0};
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ReadRowsRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ReadRowsRequest& this_ = static_cast<const ReadRowsRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ReadRowsRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ReadRowsRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.ReadRowsRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string read_stream = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_read_stream().empty()) {
      const ::std::string& _s = this_._internal_read_stream();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.cloud.bigquery.storage.v1.ReadRowsRequest.read_stream");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // int64 offset = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_offset() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<2>(
              stream, this_._internal_offset(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.ReadRowsRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ReadRowsRequest::ByteSizeLong(const MessageLite& base) {
  const ReadRowsRequest& this_ = static_cast<const ReadRowsRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ReadRowsRequest::ByteSizeLong() const {
  const ReadRowsRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.ReadRowsRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string read_stream = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_read_stream().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_read_stream());
      }
    }
    // int64 offset = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_offset() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_offset());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ReadRowsRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ReadRowsRequest*>(&to_msg);
  auto& from = static_cast<const ReadRowsRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.ReadRowsRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_read_stream().empty()) {
        _this->_internal_set_read_stream(from._internal_read_stream());
      } else {
        if (_this->_impl_.read_stream_.IsDefault()) {
          _this->_internal_set_read_stream("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_offset() != 0) {
        _this->_impl_.offset_ = from._impl_.offset_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ReadRowsRequest::CopyFrom(const ReadRowsRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.ReadRowsRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ReadRowsRequest::InternalSwap(ReadRowsRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.read_stream_, &other->_impl_.read_stream_, arena);
  swap(_impl_.offset_, other->_impl_.offset_);
}

::google::protobuf::Metadata ReadRowsRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ThrottleState::ThrottleState(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ThrottleState_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.ThrottleState)
}
ThrottleState::ThrottleState(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ThrottleState& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ThrottleState_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE ThrottleState::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void ThrottleState::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.throttle_percent_ = {};
}
ThrottleState::~ThrottleState() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.ThrottleState)
  SharedDtor(*this);
}
inline void ThrottleState::SharedDtor(MessageLite& self) {
  ThrottleState& this_ = static_cast<ThrottleState&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ThrottleState_class_data_ =
        ThrottleState::InternalGenerateClassData_(ThrottleState_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ThrottleState::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ThrottleState_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ThrottleState_class_data_.tc_table);
  return ThrottleState_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ThrottleState::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ThrottleState_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ThrottleState_globals_));
  return ThrottleState_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ThrottleState::ParseTableT_
    ThrottleState::_table_ =
        ThrottleState::InternalGenerateParseTable_(ThrottleState_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ThrottleState::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.ThrottleState)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.throttle_percent_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ThrottleState::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ThrottleState& this_ = static_cast<const ThrottleState&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ThrottleState::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ThrottleState& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.ThrottleState)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // int32 throttle_percent = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_throttle_percent() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<1>(
              stream, this_._internal_throttle_percent(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.ThrottleState)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ThrottleState::ByteSizeLong(const MessageLite& base) {
  const ThrottleState& this_ = static_cast<const ThrottleState&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ThrottleState::ByteSizeLong() const {
  const ThrottleState& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.ThrottleState)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // int32 throttle_percent = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_throttle_percent() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_throttle_percent());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ThrottleState::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ThrottleState*>(&to_msg);
  auto& from = static_cast<const ThrottleState&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.ThrottleState)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (from._internal_throttle_percent() != 0) {
      _this->_impl_.throttle_percent_ = from._impl_.throttle_percent_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ThrottleState::CopyFrom(const ThrottleState& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.ThrottleState)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ThrottleState::InternalSwap(ThrottleState* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.throttle_percent_, other->_impl_.throttle_percent_);
}

::google::protobuf::Metadata ThrottleState::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

StreamStats_Progress::StreamStats_Progress(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, StreamStats_Progress_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.StreamStats.Progress)
}
StreamStats_Progress::StreamStats_Progress(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const StreamStats_Progress& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, StreamStats_Progress_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE StreamStats_Progress::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void StreamStats_Progress::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, at_response_start_),
           0,
           offsetof(Impl_, at_response_end_) -
               offsetof(Impl_, at_response_start_) +
               sizeof(Impl_::at_response_end_));
}
StreamStats_Progress::~StreamStats_Progress() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.StreamStats.Progress)
  SharedDtor(*this);
}
inline void StreamStats_Progress::SharedDtor(MessageLite& self) {
  StreamStats_Progress& this_ = static_cast<StreamStats_Progress&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull StreamStats_Progress_class_data_ =
        StreamStats_Progress::InternalGenerateClassData_(StreamStats_Progress_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StreamStats_Progress::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StreamStats_Progress_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(StreamStats_Progress_class_data_.tc_table);
  return StreamStats_Progress_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StreamStats_Progress::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StreamStats_Progress_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&StreamStats_Progress_globals_));
  return StreamStats_Progress_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const StreamStats_Progress::ParseTableT_
    StreamStats_Progress::_table_ =
        StreamStats_Progress::InternalGenerateParseTable_(StreamStats_Progress_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void StreamStats_Progress::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.StreamStats.Progress)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    ::memset(&_impl_.at_response_start_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.at_response_end_) -
        reinterpret_cast<char*>(&_impl_.at_response_start_)) + sizeof(_impl_.at_response_end_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL StreamStats_Progress::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const StreamStats_Progress& this_ = static_cast<const StreamStats_Progress&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL StreamStats_Progress::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const StreamStats_Progress& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.StreamStats.Progress)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // double at_response_start = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (::absl::bit_cast<::uint64_t>(this_._internal_at_response_start()) != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteDoubleToArray(
          1, this_._internal_at_response_start(), target);
    }
  }

  // double at_response_end = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (::absl::bit_cast<::uint64_t>(this_._internal_at_response_end()) != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteDoubleToArray(
          2, this_._internal_at_response_end(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.StreamStats.Progress)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t StreamStats_Progress::ByteSizeLong(const MessageLite& base) {
  const StreamStats_Progress& this_ = static_cast<const StreamStats_Progress&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t StreamStats_Progress::ByteSizeLong() const {
  const StreamStats_Progress& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.StreamStats.Progress)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // double at_response_start = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (::absl::bit_cast<::uint64_t>(this_._internal_at_response_start()) != 0) {
        total_size += 9;
      }
    }
    // double at_response_end = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (::absl::bit_cast<::uint64_t>(this_._internal_at_response_end()) != 0) {
        total_size += 9;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void StreamStats_Progress::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<StreamStats_Progress*>(&to_msg);
  auto& from = static_cast<const StreamStats_Progress&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.StreamStats.Progress)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (::absl::bit_cast<::uint64_t>(from._internal_at_response_start()) != 0) {
        _this->_impl_.at_response_start_ = from._impl_.at_response_start_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (::absl::bit_cast<::uint64_t>(from._internal_at_response_end()) != 0) {
        _this->_impl_.at_response_end_ = from._impl_.at_response_end_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void StreamStats_Progress::CopyFrom(const StreamStats_Progress& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.StreamStats.Progress)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void StreamStats_Progress::InternalSwap(StreamStats_Progress* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(StreamStats_Progress, _impl_.at_response_end_)
      + sizeof(StreamStats_Progress::_impl_.at_response_end_)
      - PROTOBUF_FIELD_OFFSET(StreamStats_Progress, _impl_.at_response_start_)>(
          reinterpret_cast<char*>(&_impl_.at_response_start_),
          reinterpret_cast<char*>(&other->_impl_.at_response_start_));
}

::google::protobuf::Metadata StreamStats_Progress::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

StreamStats::StreamStats(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, StreamStats_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.StreamStats)
}
PROTOBUF_NDEBUG_INLINE StreamStats::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::StreamStats& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

StreamStats::StreamStats(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const StreamStats& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, StreamStats_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  StreamStats* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.progress_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.progress_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.StreamStats)
}
PROTOBUF_NDEBUG_INLINE StreamStats::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void StreamStats::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.progress_ = {};
}
StreamStats::~StreamStats() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.StreamStats)
  SharedDtor(*this);
}
inline void StreamStats::SharedDtor(MessageLite& self) {
  StreamStats& this_ = static_cast<StreamStats&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.progress_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull StreamStats_class_data_ =
        StreamStats::InternalGenerateClassData_(StreamStats_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StreamStats::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StreamStats_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(StreamStats_class_data_.tc_table);
  return StreamStats_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StreamStats::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StreamStats_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&StreamStats_globals_));
  return StreamStats_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const StreamStats::ParseTableT_
    StreamStats::_table_ =
        StreamStats::InternalGenerateParseTable_(StreamStats_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void StreamStats::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.StreamStats)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(_impl_.progress_ != nullptr);
    _impl_.progress_->Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL StreamStats::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const StreamStats& this_ = static_cast<const StreamStats&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL StreamStats::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const StreamStats& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.StreamStats)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.cloud.bigquery.storage.v1.StreamStats.Progress progress = 2;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.progress_, this_._impl_.progress_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.StreamStats)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t StreamStats::ByteSizeLong(const MessageLite& base) {
  const StreamStats& this_ = static_cast<const StreamStats&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t StreamStats::ByteSizeLong() const {
  const StreamStats& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.StreamStats)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // .google.cloud.bigquery.storage.v1.StreamStats.Progress progress = 2;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.progress_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void StreamStats::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<StreamStats*>(&to_msg);
  auto& from = static_cast<const StreamStats&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.StreamStats)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(from._impl_.progress_ != nullptr);
    if (_this->_impl_.progress_ == nullptr) {
      _this->_impl_.progress_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.progress_);
    } else {
      _this->_impl_.progress_->MergeFrom(*from._impl_.progress_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void StreamStats::CopyFrom(const StreamStats& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.StreamStats)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void StreamStats::InternalSwap(StreamStats* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.progress_, other->_impl_.progress_);
}

::google::protobuf::Metadata StreamStats::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void ReadRowsResponse::set_allocated_avro_rows(::google::cloud::bigquery::storage::v1::AvroRows* PROTOBUF_NULLABLE avro_rows) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_rows();
  if (avro_rows) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(avro_rows)->GetArena();
    if (message_arena != submessage_arena) {
      avro_rows = ::google::protobuf::internal::GetOwnedMessage(message_arena, avro_rows, submessage_arena);
    }
    set_has_avro_rows();
    _impl_.rows_.avro_rows_ = avro_rows;
  }
  // @@protoc_insertion_point(field_set_allocated:google.cloud.bigquery.storage.v1.ReadRowsResponse.avro_rows)
}
void ReadRowsResponse::clear_avro_rows() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (rows_case() == kAvroRows) {
    if (GetArena() == nullptr) {
      delete _impl_.rows_.avro_rows_;
    } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
      ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.rows_.avro_rows_);
    }
    clear_has_rows();
  }
}
void ReadRowsResponse::set_allocated_arrow_record_batch(::google::cloud::bigquery::storage::v1::ArrowRecordBatch* PROTOBUF_NULLABLE arrow_record_batch) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_rows();
  if (arrow_record_batch) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(arrow_record_batch)->GetArena();
    if (message_arena != submessage_arena) {
      arrow_record_batch = ::google::protobuf::internal::GetOwnedMessage(message_arena, arrow_record_batch, submessage_arena);
    }
    set_has_arrow_record_batch();
    _impl_.rows_.arrow_record_batch_ = arrow_record_batch;
  }
  // @@protoc_insertion_point(field_set_allocated:google.cloud.bigquery.storage.v1.ReadRowsResponse.arrow_record_batch)
}
void ReadRowsResponse::clear_arrow_record_batch() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (rows_case() == kArrowRecordBatch) {
    if (GetArena() == nullptr) {
      delete _impl_.rows_.arrow_record_batch_;
    } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
      ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.rows_.arrow_record_batch_);
    }
    clear_has_rows();
  }
}
void ReadRowsResponse::set_allocated_avro_schema(::google::cloud::bigquery::storage::v1::AvroSchema* PROTOBUF_NULLABLE avro_schema) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_schema();
  if (avro_schema) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(avro_schema)->GetArena();
    if (message_arena != submessage_arena) {
      avro_schema = ::google::protobuf::internal::GetOwnedMessage(message_arena, avro_schema, submessage_arena);
    }
    set_has_avro_schema();
    _impl_.schema_.avro_schema_ = avro_schema;
  }
  // @@protoc_insertion_point(field_set_allocated:google.cloud.bigquery.storage.v1.ReadRowsResponse.avro_schema)
}
void ReadRowsResponse::clear_avro_schema() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (schema_case() == kAvroSchema) {
    if (GetArena() == nullptr) {
      delete _impl_.schema_.avro_schema_;
    } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
      ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.schema_.avro_schema_);
    }
    clear_has_schema();
  }
}
void ReadRowsResponse::set_allocated_arrow_schema(::google::cloud::bigquery::storage::v1::ArrowSchema* PROTOBUF_NULLABLE arrow_schema) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_schema();
  if (arrow_schema) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(arrow_schema)->GetArena();
    if (message_arena != submessage_arena) {
      arrow_schema = ::google::protobuf::internal::GetOwnedMessage(message_arena, arrow_schema, submessage_arena);
    }
    set_has_arrow_schema();
    _impl_.schema_.arrow_schema_ = arrow_schema;
  }
  // @@protoc_insertion_point(field_set_allocated:google.cloud.bigquery.storage.v1.ReadRowsResponse.arrow_schema)
}
void ReadRowsResponse::clear_arrow_schema() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (schema_case() == kArrowSchema) {
    if (GetArena() == nullptr) {
      delete _impl_.schema_.arrow_schema_;
    } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
      ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.schema_.arrow_schema_);
    }
    clear_has_schema();
  }
}
ReadRowsResponse::ReadRowsResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ReadRowsResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.ReadRowsResponse)
}
PROTOBUF_NDEBUG_INLINE ReadRowsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::ReadRowsResponse& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        rows_{},
        schema_{},
        _oneof_case_{from._oneof_case_[0], from._oneof_case_[1]} {}

ReadRowsResponse::ReadRowsResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ReadRowsResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ReadRowsResponse_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ReadRowsResponse* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.stats_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.stats_)
                : nullptr;
  _impl_.throttle_state_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.throttle_state_)
                : nullptr;
  _impl_.row_count_ = from._impl_.row_count_;
  switch (rows_case()) {
    case ROWS_NOT_SET:
      break;
      case kAvroRows:
        _impl_.rows_.avro_rows_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.rows_.avro_rows_);
        break;
      case kArrowRecordBatch:
        _impl_.rows_.arrow_record_batch_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.rows_.arrow_record_batch_);
        break;
  }
  switch (schema_case()) {
    case SCHEMA_NOT_SET:
      break;
      case kAvroSchema:
        _impl_.schema_.avro_schema_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.schema_.avro_schema_);
        break;
      case kArrowSchema:
        _impl_.schema_.arrow_schema_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.schema_.arrow_schema_);
        break;
  }

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.ReadRowsResponse)
}
PROTOBUF_NDEBUG_INLINE ReadRowsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        rows_{},
        schema_{},
        _oneof_case_{} {}

inline void ReadRowsResponse::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, stats_),
           0,
           offsetof(Impl_, row_count_) -
               offsetof(Impl_, stats_) +
               sizeof(Impl_::row_count_));
}
ReadRowsResponse::~ReadRowsResponse() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.ReadRowsResponse)
  SharedDtor(*this);
}
inline void ReadRowsResponse::SharedDtor(MessageLite& self) {
  ReadRowsResponse& this_ = static_cast<ReadRowsResponse&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.stats_;
  delete this_._impl_.throttle_state_;
  if (this_.has_rows()) {
    this_.clear_rows();
  }
  if (this_.has_schema()) {
    this_.clear_schema();
  }
  this_._impl_.~Impl_();
}

void ReadRowsResponse::clear_rows() {
// @@protoc_insertion_point(one_of_clear_start:google.cloud.bigquery.storage.v1.ReadRowsResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (rows_case()) {
    case kAvroRows: {
      if (GetArena() == nullptr) {
        delete _impl_.rows_.avro_rows_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.rows_.avro_rows_);
      }
      break;
    }
    case kArrowRecordBatch: {
      if (GetArena() == nullptr) {
        delete _impl_.rows_.arrow_record_batch_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.rows_.arrow_record_batch_);
      }
      break;
    }
    case ROWS_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = ROWS_NOT_SET;
}

void ReadRowsResponse::clear_schema() {
// @@protoc_insertion_point(one_of_clear_start:google.cloud.bigquery.storage.v1.ReadRowsResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (schema_case()) {
    case kAvroSchema: {
      if (GetArena() == nullptr) {
        delete _impl_.schema_.avro_schema_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.schema_.avro_schema_);
      }
      break;
    }
    case kArrowSchema: {
      if (GetArena() == nullptr) {
        delete _impl_.schema_.arrow_schema_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.schema_.arrow_schema_);
      }
      break;
    }
    case SCHEMA_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[1] = SCHEMA_NOT_SET;
}


#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ReadRowsResponse_class_data_ =
        ReadRowsResponse::InternalGenerateClassData_(ReadRowsResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ReadRowsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ReadRowsResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ReadRowsResponse_class_data_.tc_table);
  return ReadRowsResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ReadRowsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ReadRowsResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ReadRowsResponse_globals_));
  return ReadRowsResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ReadRowsResponse::ParseTableT_
    ReadRowsResponse::_table_ =
        ReadRowsResponse::InternalGenerateParseTable_(ReadRowsResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ReadRowsResponse::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.ReadRowsResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(_impl_.stats_ != nullptr);
      _impl_.stats_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.throttle_state_ != nullptr);
      _impl_.throttle_state_->Clear();
    }
  }
  _impl_.row_count_ = ::int64_t{0};
  clear_rows();
  clear_schema();
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ReadRowsResponse::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ReadRowsResponse& this_ = static_cast<const ReadRowsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ReadRowsResponse::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ReadRowsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.ReadRowsResponse)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.cloud.bigquery.storage.v1.StreamStats stats = 2;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.stats_, this_._impl_.stats_->GetCachedSize(), target,
        stream);
  }

  switch (this_.rows_case()) {
    case kAvroRows: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          3, *this_._impl_.rows_.avro_rows_, this_._impl_.rows_.avro_rows_->GetCachedSize(), target,
          stream);
      break;
    }
    case kArrowRecordBatch: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          4, *this_._impl_.rows_.arrow_record_batch_, this_._impl_.rows_.arrow_record_batch_->GetCachedSize(), target,
          stream);
      break;
    }
    default:
      break;
  }
  // .google.cloud.bigquery.storage.v1.ThrottleState throttle_state = 5;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        5, *this_._impl_.throttle_state_, this_._impl_.throttle_state_->GetCachedSize(), target,
        stream);
  }

  // int64 row_count = 6;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_row_count() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<6>(
              stream, this_._internal_row_count(), target);
    }
  }

  switch (this_.schema_case()) {
    case kAvroSchema: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          7, *this_._impl_.schema_.avro_schema_, this_._impl_.schema_.avro_schema_->GetCachedSize(), target,
          stream);
      break;
    }
    case kArrowSchema: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          8, *this_._impl_.schema_.arrow_schema_, this_._impl_.schema_.arrow_schema_->GetCachedSize(), target,
          stream);
      break;
    }
    default:
      break;
  }
  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.ReadRowsResponse)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ReadRowsResponse::ByteSizeLong(const MessageLite& base) {
  const ReadRowsResponse& this_ = static_cast<const ReadRowsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ReadRowsResponse::ByteSizeLong() const {
  const ReadRowsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.ReadRowsResponse)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // .google.cloud.bigquery.storage.v1.StreamStats stats = 2;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.stats_);
    }
    // .google.cloud.bigquery.storage.v1.ThrottleState throttle_state = 5;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.throttle_state_);
    }
    // int64 row_count = 6;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_row_count() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_row_count());
      }
    }
  }
  switch (this_.rows_case()) {
    // .google.cloud.bigquery.storage.v1.AvroRows avro_rows = 3;
    case kAvroRows: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.rows_.avro_rows_);
      break;
    }
    // .google.cloud.bigquery.storage.v1.ArrowRecordBatch arrow_record_batch = 4;
    case kArrowRecordBatch: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.rows_.arrow_record_batch_);
      break;
    }
    case ROWS_NOT_SET: {
      break;
    }
  }
  switch (this_.schema_case()) {
    // .google.cloud.bigquery.storage.v1.AvroSchema avro_schema = 7 [(.google.api.field_behavior) = OUTPUT_ONLY];
    case kAvroSchema: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.schema_.avro_schema_);
      break;
    }
    // .google.cloud.bigquery.storage.v1.ArrowSchema arrow_schema = 8 [(.google.api.field_behavior) = OUTPUT_ONLY];
    case kArrowSchema: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.schema_.arrow_schema_);
      break;
    }
    case SCHEMA_NOT_SET: {
      break;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ReadRowsResponse::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ReadRowsResponse*>(&to_msg);
  auto& from = static_cast<const ReadRowsResponse&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.ReadRowsResponse)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.stats_ != nullptr);
      if (_this->_impl_.stats_ == nullptr) {
        _this->_impl_.stats_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.stats_);
      } else {
        _this->_impl_.stats_->MergeFrom(*from._impl_.stats_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.throttle_state_ != nullptr);
      if (_this->_impl_.throttle_state_ == nullptr) {
        _this->_impl_.throttle_state_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.throttle_state_);
      } else {
        _this->_impl_.throttle_state_->MergeFrom(*from._impl_.throttle_state_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_row_count() != 0) {
        _this->_impl_.row_count_ = from._impl_.row_count_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[0]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[0];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_rows();
      }
      _this->_impl_._oneof_case_[0] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kAvroRows: {
        if (oneof_needs_init) {
          _this->_impl_.rows_.avro_rows_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.rows_.avro_rows_);
        } else {
          _this->_impl_.rows_.avro_rows_->MergeFrom(*from._impl_.rows_.avro_rows_);
        }
        break;
      }
      case kArrowRecordBatch: {
        if (oneof_needs_init) {
          _this->_impl_.rows_.arrow_record_batch_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.rows_.arrow_record_batch_);
        } else {
          _this->_impl_.rows_.arrow_record_batch_->MergeFrom(*from._impl_.rows_.arrow_record_batch_);
        }
        break;
      }
      case ROWS_NOT_SET:
        break;
    }
  }
  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[1]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[1];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_schema();
      }
      _this->_impl_._oneof_case_[1] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kAvroSchema: {
        if (oneof_needs_init) {
          _this->_impl_.schema_.avro_schema_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.schema_.avro_schema_);
        } else {
          _this->_impl_.schema_.avro_schema_->MergeFrom(*from._impl_.schema_.avro_schema_);
        }
        break;
      }
      case kArrowSchema: {
        if (oneof_needs_init) {
          _this->_impl_.schema_.arrow_schema_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.schema_.arrow_schema_);
        } else {
          _this->_impl_.schema_.arrow_schema_->MergeFrom(*from._impl_.schema_.arrow_schema_);
        }
        break;
      }
      case SCHEMA_NOT_SET:
        break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ReadRowsResponse::CopyFrom(const ReadRowsResponse& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.ReadRowsResponse)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ReadRowsResponse::InternalSwap(ReadRowsResponse* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(ReadRowsResponse, _impl_.row_count_)
      + sizeof(ReadRowsResponse::_impl_.row_count_)
      - PROTOBUF_FIELD_OFFSET(ReadRowsResponse, _impl_.stats_)>(
          reinterpret_cast<char*>(&_impl_.stats_),
          reinterpret_cast<char*>(&other->_impl_.stats_));
  swap(_impl_.rows_, other->_impl_.rows_);
  swap(_impl_.schema_, other->_impl_.schema_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
  swap(_impl_._oneof_case_[1], other->_impl_._oneof_case_[1]);
}

::google::protobuf::Metadata ReadRowsResponse::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

SplitReadStreamRequest::SplitReadStreamRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, SplitReadStreamRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.SplitReadStreamRequest)
}
PROTOBUF_NDEBUG_INLINE SplitReadStreamRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::SplitReadStreamRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        name_(arena, from.name_) {}

SplitReadStreamRequest::SplitReadStreamRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const SplitReadStreamRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, SplitReadStreamRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SplitReadStreamRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.fraction_ = from._impl_.fraction_;

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.SplitReadStreamRequest)
}
PROTOBUF_NDEBUG_INLINE SplitReadStreamRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        name_(arena) {}

inline void SplitReadStreamRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.fraction_ = {};
}
SplitReadStreamRequest::~SplitReadStreamRequest() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.SplitReadStreamRequest)
  SharedDtor(*this);
}
inline void SplitReadStreamRequest::SharedDtor(MessageLite& self) {
  SplitReadStreamRequest& this_ = static_cast<SplitReadStreamRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.name_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull SplitReadStreamRequest_class_data_ =
        SplitReadStreamRequest::InternalGenerateClassData_(SplitReadStreamRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
SplitReadStreamRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&SplitReadStreamRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(SplitReadStreamRequest_class_data_.tc_table);
  return SplitReadStreamRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
SplitReadStreamRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&SplitReadStreamRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&SplitReadStreamRequest_globals_));
  return SplitReadStreamRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const SplitReadStreamRequest::ParseTableT_
    SplitReadStreamRequest::_table_ =
        SplitReadStreamRequest::InternalGenerateParseTable_(SplitReadStreamRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void SplitReadStreamRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.SplitReadStreamRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.name_.ClearNonDefaultToEmpty();
  }
  _impl_.fraction_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL SplitReadStreamRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const SplitReadStreamRequest& this_ = static_cast<const SplitReadStreamRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL SplitReadStreamRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const SplitReadStreamRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.SplitReadStreamRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_name().empty()) {
      const ::std::string& _s = this_._internal_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.cloud.bigquery.storage.v1.SplitReadStreamRequest.name");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // double fraction = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (::absl::bit_cast<::uint64_t>(this_._internal_fraction()) != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteDoubleToArray(
          2, this_._internal_fraction(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.SplitReadStreamRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t SplitReadStreamRequest::ByteSizeLong(const MessageLite& base) {
  const SplitReadStreamRequest& this_ = static_cast<const SplitReadStreamRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t SplitReadStreamRequest::ByteSizeLong() const {
  const SplitReadStreamRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.SplitReadStreamRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_name());
      }
    }
    // double fraction = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (::absl::bit_cast<::uint64_t>(this_._internal_fraction()) != 0) {
        total_size += 9;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void SplitReadStreamRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<SplitReadStreamRequest*>(&to_msg);
  auto& from = static_cast<const SplitReadStreamRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.SplitReadStreamRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_name().empty()) {
        _this->_internal_set_name(from._internal_name());
      } else {
        if (_this->_impl_.name_.IsDefault()) {
          _this->_internal_set_name("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (::absl::bit_cast<::uint64_t>(from._internal_fraction()) != 0) {
        _this->_impl_.fraction_ = from._impl_.fraction_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void SplitReadStreamRequest::CopyFrom(const SplitReadStreamRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.SplitReadStreamRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void SplitReadStreamRequest::InternalSwap(SplitReadStreamRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.name_, &other->_impl_.name_, arena);
  swap(_impl_.fraction_, other->_impl_.fraction_);
}

::google::protobuf::Metadata SplitReadStreamRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void SplitReadStreamResponse::clear_primary_stream() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.primary_stream_ != nullptr) _impl_.primary_stream_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
void SplitReadStreamResponse::clear_remainder_stream() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.remainder_stream_ != nullptr) _impl_.remainder_stream_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
SplitReadStreamResponse::SplitReadStreamResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, SplitReadStreamResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.SplitReadStreamResponse)
}
PROTOBUF_NDEBUG_INLINE SplitReadStreamResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::SplitReadStreamResponse& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

SplitReadStreamResponse::SplitReadStreamResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const SplitReadStreamResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, SplitReadStreamResponse_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SplitReadStreamResponse* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.primary_stream_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.primary_stream_)
                : nullptr;
  _impl_.remainder_stream_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.remainder_stream_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.SplitReadStreamResponse)
}
PROTOBUF_NDEBUG_INLINE SplitReadStreamResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void SplitReadStreamResponse::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, primary_stream_),
           0,
           offsetof(Impl_, remainder_stream_) -
               offsetof(Impl_, primary_stream_) +
               sizeof(Impl_::remainder_stream_));
}
SplitReadStreamResponse::~SplitReadStreamResponse() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.SplitReadStreamResponse)
  SharedDtor(*this);
}
inline void SplitReadStreamResponse::SharedDtor(MessageLite& self) {
  SplitReadStreamResponse& this_ = static_cast<SplitReadStreamResponse&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.primary_stream_;
  delete this_._impl_.remainder_stream_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull SplitReadStreamResponse_class_data_ =
        SplitReadStreamResponse::InternalGenerateClassData_(SplitReadStreamResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
SplitReadStreamResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&SplitReadStreamResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(SplitReadStreamResponse_class_data_.tc_table);
  return SplitReadStreamResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
SplitReadStreamResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&SplitReadStreamResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&SplitReadStreamResponse_globals_));
  return SplitReadStreamResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const SplitReadStreamResponse::ParseTableT_
    SplitReadStreamResponse::_table_ =
        SplitReadStreamResponse::InternalGenerateParseTable_(SplitReadStreamResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void SplitReadStreamResponse::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.SplitReadStreamResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(_impl_.primary_stream_ != nullptr);
      _impl_.primary_stream_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.remainder_stream_ != nullptr);
      _impl_.remainder_stream_->Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL SplitReadStreamResponse::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const SplitReadStreamResponse& this_ = static_cast<const SplitReadStreamResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL SplitReadStreamResponse::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const SplitReadStreamResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.SplitReadStreamResponse)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.cloud.bigquery.storage.v1.ReadStream primary_stream = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.primary_stream_, this_._impl_.primary_stream_->GetCachedSize(), target,
        stream);
  }

  // .google.cloud.bigquery.storage.v1.ReadStream remainder_stream = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.remainder_stream_, this_._impl_.remainder_stream_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.SplitReadStreamResponse)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t SplitReadStreamResponse::ByteSizeLong(const MessageLite& base) {
  const SplitReadStreamResponse& this_ = static_cast<const SplitReadStreamResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t SplitReadStreamResponse::ByteSizeLong() const {
  const SplitReadStreamResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.SplitReadStreamResponse)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // .google.cloud.bigquery.storage.v1.ReadStream primary_stream = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.primary_stream_);
    }
    // .google.cloud.bigquery.storage.v1.ReadStream remainder_stream = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.remainder_stream_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void SplitReadStreamResponse::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<SplitReadStreamResponse*>(&to_msg);
  auto& from = static_cast<const SplitReadStreamResponse&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.SplitReadStreamResponse)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.primary_stream_ != nullptr);
      if (_this->_impl_.primary_stream_ == nullptr) {
        _this->_impl_.primary_stream_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.primary_stream_);
      } else {
        _this->_impl_.primary_stream_->MergeFrom(*from._impl_.primary_stream_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.remainder_stream_ != nullptr);
      if (_this->_impl_.remainder_stream_ == nullptr) {
        _this->_impl_.remainder_stream_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.remainder_stream_);
      } else {
        _this->_impl_.remainder_stream_->MergeFrom(*from._impl_.remainder_stream_);
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void SplitReadStreamResponse::CopyFrom(const SplitReadStreamResponse& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.SplitReadStreamResponse)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void SplitReadStreamResponse::InternalSwap(SplitReadStreamResponse* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(SplitReadStreamResponse, _impl_.remainder_stream_)
      + sizeof(SplitReadStreamResponse::_impl_.remainder_stream_)
      - PROTOBUF_FIELD_OFFSET(SplitReadStreamResponse, _impl_.primary_stream_)>(
          reinterpret_cast<char*>(&_impl_.primary_stream_),
          reinterpret_cast<char*>(&other->_impl_.primary_stream_));
}

::google::protobuf::Metadata SplitReadStreamResponse::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void CreateWriteStreamRequest::clear_write_stream() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.write_stream_ != nullptr) _impl_.write_stream_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
CreateWriteStreamRequest::CreateWriteStreamRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, CreateWriteStreamRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.CreateWriteStreamRequest)
}
PROTOBUF_NDEBUG_INLINE CreateWriteStreamRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::CreateWriteStreamRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        parent_(arena, from.parent_) {}

CreateWriteStreamRequest::CreateWriteStreamRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const CreateWriteStreamRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, CreateWriteStreamRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  CreateWriteStreamRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.write_stream_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.write_stream_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.CreateWriteStreamRequest)
}
PROTOBUF_NDEBUG_INLINE CreateWriteStreamRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        parent_(arena) {}

inline void CreateWriteStreamRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.write_stream_ = {};
}
CreateWriteStreamRequest::~CreateWriteStreamRequest() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.CreateWriteStreamRequest)
  SharedDtor(*this);
}
inline void CreateWriteStreamRequest::SharedDtor(MessageLite& self) {
  CreateWriteStreamRequest& this_ = static_cast<CreateWriteStreamRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.parent_.Destroy();
  delete this_._impl_.write_stream_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull CreateWriteStreamRequest_class_data_ =
        CreateWriteStreamRequest::InternalGenerateClassData_(CreateWriteStreamRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CreateWriteStreamRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CreateWriteStreamRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(CreateWriteStreamRequest_class_data_.tc_table);
  return CreateWriteStreamRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CreateWriteStreamRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CreateWriteStreamRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&CreateWriteStreamRequest_globals_));
  return CreateWriteStreamRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const CreateWriteStreamRequest::ParseTableT_
    CreateWriteStreamRequest::_table_ =
        CreateWriteStreamRequest::InternalGenerateParseTable_(CreateWriteStreamRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void CreateWriteStreamRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.CreateWriteStreamRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.parent_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.write_stream_ != nullptr);
      _impl_.write_stream_->Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL CreateWriteStreamRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const CreateWriteStreamRequest& this_ = static_cast<const CreateWriteStreamRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL CreateWriteStreamRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const CreateWriteStreamRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.CreateWriteStreamRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_parent().empty()) {
      const ::std::string& _s = this_._internal_parent();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.cloud.bigquery.storage.v1.CreateWriteStreamRequest.parent");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // .google.cloud.bigquery.storage.v1.WriteStream write_stream = 2 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.write_stream_, this_._impl_.write_stream_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.CreateWriteStreamRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t CreateWriteStreamRequest::ByteSizeLong(const MessageLite& base) {
  const CreateWriteStreamRequest& this_ = static_cast<const CreateWriteStreamRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t CreateWriteStreamRequest::ByteSizeLong() const {
  const CreateWriteStreamRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.CreateWriteStreamRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_parent().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_parent());
      }
    }
    // .google.cloud.bigquery.storage.v1.WriteStream write_stream = 2 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.write_stream_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void CreateWriteStreamRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<CreateWriteStreamRequest*>(&to_msg);
  auto& from = static_cast<const CreateWriteStreamRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.CreateWriteStreamRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_parent().empty()) {
        _this->_internal_set_parent(from._internal_parent());
      } else {
        if (_this->_impl_.parent_.IsDefault()) {
          _this->_internal_set_parent("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.write_stream_ != nullptr);
      if (_this->_impl_.write_stream_ == nullptr) {
        _this->_impl_.write_stream_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.write_stream_);
      } else {
        _this->_impl_.write_stream_->MergeFrom(*from._impl_.write_stream_);
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void CreateWriteStreamRequest::CopyFrom(const CreateWriteStreamRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.CreateWriteStreamRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void CreateWriteStreamRequest::InternalSwap(CreateWriteStreamRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.parent_, &other->_impl_.parent_, arena);
  swap(_impl_.write_stream_, other->_impl_.write_stream_);
}

::google::protobuf::Metadata CreateWriteStreamRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void AppendRowsRequest_ProtoData::clear_writer_schema() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.writer_schema_ != nullptr) _impl_.writer_schema_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
void AppendRowsRequest_ProtoData::clear_rows() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.rows_ != nullptr) _impl_.rows_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
AppendRowsRequest_ProtoData::AppendRowsRequest_ProtoData(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AppendRowsRequest_ProtoData_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.AppendRowsRequest.ProtoData)
}
PROTOBUF_NDEBUG_INLINE AppendRowsRequest_ProtoData::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::AppendRowsRequest_ProtoData& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

AppendRowsRequest_ProtoData::AppendRowsRequest_ProtoData(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const AppendRowsRequest_ProtoData& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AppendRowsRequest_ProtoData_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  AppendRowsRequest_ProtoData* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.writer_schema_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.writer_schema_)
                : nullptr;
  _impl_.rows_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.rows_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.AppendRowsRequest.ProtoData)
}
PROTOBUF_NDEBUG_INLINE AppendRowsRequest_ProtoData::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void AppendRowsRequest_ProtoData::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, writer_schema_),
           0,
           offsetof(Impl_, rows_) -
               offsetof(Impl_, writer_schema_) +
               sizeof(Impl_::rows_));
}
AppendRowsRequest_ProtoData::~AppendRowsRequest_ProtoData() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.AppendRowsRequest.ProtoData)
  SharedDtor(*this);
}
inline void AppendRowsRequest_ProtoData::SharedDtor(MessageLite& self) {
  AppendRowsRequest_ProtoData& this_ = static_cast<AppendRowsRequest_ProtoData&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.writer_schema_;
  delete this_._impl_.rows_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull AppendRowsRequest_ProtoData_class_data_ =
        AppendRowsRequest_ProtoData::InternalGenerateClassData_(AppendRowsRequest_ProtoData_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AppendRowsRequest_ProtoData::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AppendRowsRequest_ProtoData_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(AppendRowsRequest_ProtoData_class_data_.tc_table);
  return AppendRowsRequest_ProtoData_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AppendRowsRequest_ProtoData::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AppendRowsRequest_ProtoData_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&AppendRowsRequest_ProtoData_globals_));
  return AppendRowsRequest_ProtoData_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const AppendRowsRequest_ProtoData::ParseTableT_
    AppendRowsRequest_ProtoData::_table_ =
        AppendRowsRequest_ProtoData::InternalGenerateParseTable_(AppendRowsRequest_ProtoData_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void AppendRowsRequest_ProtoData::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.AppendRowsRequest.ProtoData)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(_impl_.writer_schema_ != nullptr);
      _impl_.writer_schema_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.rows_ != nullptr);
      _impl_.rows_->Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL AppendRowsRequest_ProtoData::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const AppendRowsRequest_ProtoData& this_ = static_cast<const AppendRowsRequest_ProtoData&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL AppendRowsRequest_ProtoData::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const AppendRowsRequest_ProtoData& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.AppendRowsRequest.ProtoData)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.cloud.bigquery.storage.v1.ProtoSchema writer_schema = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.writer_schema_, this_._impl_.writer_schema_->GetCachedSize(), target,
        stream);
  }

  // .google.cloud.bigquery.storage.v1.ProtoRows rows = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.rows_, this_._impl_.rows_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.AppendRowsRequest.ProtoData)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t AppendRowsRequest_ProtoData::ByteSizeLong(const MessageLite& base) {
  const AppendRowsRequest_ProtoData& this_ = static_cast<const AppendRowsRequest_ProtoData&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t AppendRowsRequest_ProtoData::ByteSizeLong() const {
  const AppendRowsRequest_ProtoData& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.AppendRowsRequest.ProtoData)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // .google.cloud.bigquery.storage.v1.ProtoSchema writer_schema = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.writer_schema_);
    }
    // .google.cloud.bigquery.storage.v1.ProtoRows rows = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.rows_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void AppendRowsRequest_ProtoData::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<AppendRowsRequest_ProtoData*>(&to_msg);
  auto& from = static_cast<const AppendRowsRequest_ProtoData&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.AppendRowsRequest.ProtoData)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.writer_schema_ != nullptr);
      if (_this->_impl_.writer_schema_ == nullptr) {
        _this->_impl_.writer_schema_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.writer_schema_);
      } else {
        _this->_impl_.writer_schema_->MergeFrom(*from._impl_.writer_schema_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.rows_ != nullptr);
      if (_this->_impl_.rows_ == nullptr) {
        _this->_impl_.rows_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.rows_);
      } else {
        _this->_impl_.rows_->MergeFrom(*from._impl_.rows_);
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void AppendRowsRequest_ProtoData::CopyFrom(const AppendRowsRequest_ProtoData& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.AppendRowsRequest.ProtoData)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

PROTOBUF_NOINLINE bool AppendRowsRequest_ProtoData::IsInitializedImpl(
    const MessageLite& msg) {
  auto& this_ = static_cast<const AppendRowsRequest_ProtoData&>(msg);
  if (CheckHasBit(this_._impl_._has_bits_[0], 0x00000001U)) {
    if (!this_._impl_.writer_schema_->IsInitialized()) return false;
  }
  return true;
}

void AppendRowsRequest_ProtoData::InternalSwap(AppendRowsRequest_ProtoData* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(AppendRowsRequest_ProtoData, _impl_.rows_)
      + sizeof(AppendRowsRequest_ProtoData::_impl_.rows_)
      - PROTOBUF_FIELD_OFFSET(AppendRowsRequest_ProtoData, _impl_.writer_schema_)>(
          reinterpret_cast<char*>(&_impl_.writer_schema_),
          reinterpret_cast<char*>(&other->_impl_.writer_schema_));
}

::google::protobuf::Metadata AppendRowsRequest_ProtoData::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

#if defined(PROTOBUF_CUSTOM_VTABLE)
AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse::AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse() : SuperType(AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse_get_class_data()) {}
AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse::AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
    : SuperType(arena, AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse_get_class_data()) {}
#else   // PROTOBUF_CUSTOM_VTABLE
AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse::AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse() : SuperType() {}
AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse::AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena) : SuperType(arena) {}
#endif  // PROTOBUF_CUSTOM_VTABLE
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse_class_data_ =
        AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse::InternalGenerateClassData_(AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse_globals_._default);

// including ~ all the vtables and everything they use become part
// of the same ~ SCC.
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse_class_data_.tc_table);
  return AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse_globals_));
  return AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse::ParseTableT_
    AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse::_table_ =
        AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse::InternalGenerateParseTable_(AppendRowsRequest_MissingValueInterpretationsEntry_DoNotUse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
// ===================================================================

void AppendRowsRequest::clear_offset() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.offset_ != nullptr) _impl_.offset_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
void AppendRowsRequest::set_allocated_proto_rows(::google::cloud::bigquery::storage::v1::AppendRowsRequest_ProtoData* PROTOBUF_NULLABLE proto_rows) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_rows();
  if (proto_rows) {
    ::google::protobuf::Arena* submessage_arena = proto_rows->GetArena();
    if (message_arena != submessage_arena) {
      proto_rows = ::google::protobuf::internal::GetOwnedMessage(message_arena, proto_rows, submessage_arena);
    }
    set_has_proto_rows();
    _impl_.rows_.proto_rows_ = proto_rows;
  }
  // @@protoc_insertion_point(field_set_allocated:google.cloud.bigquery.storage.v1.AppendRowsRequest.proto_rows)
}
AppendRowsRequest::AppendRowsRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AppendRowsRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.AppendRowsRequest)
}
PROTOBUF_NDEBUG_INLINE AppendRowsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::AppendRowsRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        write_stream_(arena, from.write_stream_),
        trace_id_(arena, from.trace_id_),
        missing_value_interpretations_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::cloud::bigquery::storage::v1::AppendRowsRequest,
              PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsRequest, _impl_.missing_value_interpretations_)>()
          , from.missing_value_interpretations_
        }
        ,
        rows_{},
        _oneof_case_{from._oneof_case_[0]} {}

AppendRowsRequest::AppendRowsRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const AppendRowsRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AppendRowsRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  AppendRowsRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.offset_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.offset_)
                : nullptr;
  switch (rows_case()) {
    case ROWS_NOT_SET:
      break;
      case kProtoRows:
        _impl_.rows_.proto_rows_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.rows_.proto_rows_);
        break;
  }

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.AppendRowsRequest)
}
PROTOBUF_NDEBUG_INLINE AppendRowsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        write_stream_(arena),
        trace_id_(arena),
        missing_value_interpretations_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::cloud::bigquery::storage::v1::AppendRowsRequest,
            PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsRequest, _impl_.missing_value_interpretations_)>()
         }
        ,
        rows_{},
        _oneof_case_{} {}

inline void AppendRowsRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.offset_ = {};
}
AppendRowsRequest::~AppendRowsRequest() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.AppendRowsRequest)
  SharedDtor(*this);
}
inline void AppendRowsRequest::SharedDtor(MessageLite& self) {
  AppendRowsRequest& this_ = static_cast<AppendRowsRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.write_stream_.Destroy();
  this_._impl_.trace_id_.Destroy();
  delete this_._impl_.offset_;
  if (this_.has_rows()) {
    this_.clear_rows();
  }
  this_._impl_.~Impl_();
}

void AppendRowsRequest::clear_rows() {
// @@protoc_insertion_point(one_of_clear_start:google.cloud.bigquery.storage.v1.AppendRowsRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (rows_case()) {
    case kProtoRows: {
      if (GetArena() == nullptr) {
        delete _impl_.rows_.proto_rows_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.rows_.proto_rows_);
      }
      break;
    }
    case ROWS_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = ROWS_NOT_SET;
}


#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull AppendRowsRequest_class_data_ =
        AppendRowsRequest::InternalGenerateClassData_(AppendRowsRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AppendRowsRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AppendRowsRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(AppendRowsRequest_class_data_.tc_table);
  return AppendRowsRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AppendRowsRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AppendRowsRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&AppendRowsRequest_globals_));
  return AppendRowsRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const AppendRowsRequest::ParseTableT_
    AppendRowsRequest::_table_ =
        AppendRowsRequest::InternalGenerateParseTable_(AppendRowsRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void AppendRowsRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.AppendRowsRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.write_stream_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.trace_id_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(_impl_.offset_ != nullptr);
      _impl_.offset_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _impl_.missing_value_interpretations_.Clear();
    }
  }
  clear_rows();
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL AppendRowsRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const AppendRowsRequest& this_ = static_cast<const AppendRowsRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL AppendRowsRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const AppendRowsRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.AppendRowsRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string write_stream = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_write_stream().empty()) {
      const ::std::string& _s = this_._internal_write_stream();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.cloud.bigquery.storage.v1.AppendRowsRequest.write_stream");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // .google.protobuf.Int64Value offset = 2;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.offset_, this_._impl_.offset_->GetCachedSize(), target,
        stream);
  }

  // .google.cloud.bigquery.storage.v1.AppendRowsRequest.ProtoData proto_rows = 4;
  if (this_.rows_case() == kProtoRows) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        4, *this_._impl_.rows_.proto_rows_, this_._impl_.rows_.proto_rows_->GetCachedSize(), target,
        stream);
  }

  // string trace_id = 6;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_trace_id().empty()) {
      const ::std::string& _s = this_._internal_trace_id();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.cloud.bigquery.storage.v1.AppendRowsRequest.trace_id");
      target = stream->WriteStringMaybeAliased(6, _s, target);
    }
  }

  // map<string, .google.cloud.bigquery.storage.v1.AppendRowsRequest.MissingValueInterpretation> missing_value_interpretations = 7;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (!this_._internal_missing_value_interpretations().empty()) {
      using MapType = ::google::protobuf::Map<::std::string, ::google::cloud::bigquery::storage::v1::AppendRowsRequest_MissingValueInterpretation>;
      using WireHelper = _pbi::MapEntryFuncs<::std::string, ::google::cloud::bigquery::storage::v1::AppendRowsRequest_MissingValueInterpretation,
                                     _pbi::WireFormatLite::TYPE_STRING,
                                     _pbi::WireFormatLite::TYPE_ENUM>;
      const auto& field = this_._internal_missing_value_interpretations();

      if (stream->IsSerializationDeterministic() && field.size() > 1) {
        for (const auto& entry : ::google::protobuf::internal::MapSorterPtr<MapType>(field)) {
          target = WireHelper::InternalSerialize(
              7, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.cloud.bigquery.storage.v1.AppendRowsRequest.missing_value_interpretations");
        }
      } else {
        for (const auto& entry : field) {
          target = WireHelper::InternalSerialize(
              7, entry.first, entry.second, target, stream);
          ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
              entry.first.data(), static_cast<int>(entry.first.length()),
 ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.cloud.bigquery.storage.v1.AppendRowsRequest.missing_value_interpretations");
        }
      }
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.AppendRowsRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t AppendRowsRequest::ByteSizeLong(const MessageLite& base) {
  const AppendRowsRequest& this_ = static_cast<const AppendRowsRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t AppendRowsRequest::ByteSizeLong() const {
  const AppendRowsRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.AppendRowsRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // string write_stream = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_write_stream().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_write_stream());
      }
    }
    // string trace_id = 6;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_trace_id().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_trace_id());
      }
    }
    // .google.protobuf.Int64Value offset = 2;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.offset_);
    }
    // map<string, .google.cloud.bigquery.storage.v1.AppendRowsRequest.MissingValueInterpretation> missing_value_interpretations = 7;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_missing_value_interpretations_size());
      for (const auto& entry : this_._internal_missing_value_interpretations()) {
        total_size += _pbi::MapEntryFuncs<::std::string, ::google::cloud::bigquery::storage::v1::AppendRowsRequest_MissingValueInterpretation,
                                       _pbi::WireFormatLite::TYPE_STRING,
                                       _pbi::WireFormatLite::TYPE_ENUM>::ByteSizeLong(entry.first, entry.second);
      }
    }
  }
  switch (this_.rows_case()) {
    // .google.cloud.bigquery.storage.v1.AppendRowsRequest.ProtoData proto_rows = 4;
    case kProtoRows: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.rows_.proto_rows_);
      break;
    }
    case ROWS_NOT_SET: {
      break;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void AppendRowsRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<AppendRowsRequest*>(&to_msg);
  auto& from = static_cast<const AppendRowsRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.AppendRowsRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_write_stream().empty()) {
        _this->_internal_set_write_stream(from._internal_write_stream());
      } else {
        if (_this->_impl_.write_stream_.IsDefault()) {
          _this->_internal_set_write_stream("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_trace_id().empty()) {
        _this->_internal_set_trace_id(from._internal_trace_id());
      } else {
        if (_this->_impl_.trace_id_.IsDefault()) {
          _this->_internal_set_trace_id("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.offset_ != nullptr);
      if (_this->_impl_.offset_ == nullptr) {
        _this->_impl_.offset_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.offset_);
      } else {
        _this->_impl_.offset_->MergeFrom(*from._impl_.offset_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _this->_impl_.missing_value_interpretations_.MergeFrom(from._impl_.missing_value_interpretations_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[0]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[0];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_rows();
      }
      _this->_impl_._oneof_case_[0] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kProtoRows: {
        if (oneof_needs_init) {
          _this->_impl_.rows_.proto_rows_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.rows_.proto_rows_);
        } else {
          _this->_impl_.rows_.proto_rows_->MergeFrom(*from._impl_.rows_.proto_rows_);
        }
        break;
      }
      case ROWS_NOT_SET:
        break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void AppendRowsRequest::CopyFrom(const AppendRowsRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.AppendRowsRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

PROTOBUF_NOINLINE bool AppendRowsRequest::IsInitializedImpl(
    const MessageLite& msg) {
  auto& this_ = static_cast<const AppendRowsRequest&>(msg);
  switch (this_.rows_case()) {
    case kProtoRows: {
      if (this_.rows_case() == kProtoRows && !this_._impl_.rows_.proto_rows_->IsInitialized())
        return false;
      break;
    }
    case ROWS_NOT_SET: {
      break;
    }
  }
  return true;
}

void AppendRowsRequest::InternalSwap(AppendRowsRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.write_stream_, &other->_impl_.write_stream_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.trace_id_, &other->_impl_.trace_id_, arena);
  swap(_impl_.offset_, other->_impl_.offset_);
  _impl_.missing_value_interpretations_.InternalSwap(&other->_impl_.missing_value_interpretations_);
  swap(_impl_.rows_, other->_impl_.rows_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
}

::google::protobuf::Metadata AppendRowsRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void AppendRowsResponse_AppendResult::clear_offset() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.offset_ != nullptr) _impl_.offset_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
AppendRowsResponse_AppendResult::AppendRowsResponse_AppendResult(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AppendRowsResponse_AppendResult_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.AppendRowsResponse.AppendResult)
}
PROTOBUF_NDEBUG_INLINE AppendRowsResponse_AppendResult::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::AppendRowsResponse_AppendResult& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

AppendRowsResponse_AppendResult::AppendRowsResponse_AppendResult(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const AppendRowsResponse_AppendResult& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AppendRowsResponse_AppendResult_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  AppendRowsResponse_AppendResult* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.offset_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.offset_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.AppendRowsResponse.AppendResult)
}
PROTOBUF_NDEBUG_INLINE AppendRowsResponse_AppendResult::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void AppendRowsResponse_AppendResult::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.offset_ = {};
}
AppendRowsResponse_AppendResult::~AppendRowsResponse_AppendResult() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.AppendRowsResponse.AppendResult)
  SharedDtor(*this);
}
inline void AppendRowsResponse_AppendResult::SharedDtor(MessageLite& self) {
  AppendRowsResponse_AppendResult& this_ = static_cast<AppendRowsResponse_AppendResult&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.offset_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull AppendRowsResponse_AppendResult_class_data_ =
        AppendRowsResponse_AppendResult::InternalGenerateClassData_(AppendRowsResponse_AppendResult_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AppendRowsResponse_AppendResult::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AppendRowsResponse_AppendResult_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(AppendRowsResponse_AppendResult_class_data_.tc_table);
  return AppendRowsResponse_AppendResult_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AppendRowsResponse_AppendResult::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AppendRowsResponse_AppendResult_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&AppendRowsResponse_AppendResult_globals_));
  return AppendRowsResponse_AppendResult_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const AppendRowsResponse_AppendResult::ParseTableT_
    AppendRowsResponse_AppendResult::_table_ =
        AppendRowsResponse_AppendResult::InternalGenerateParseTable_(AppendRowsResponse_AppendResult_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void AppendRowsResponse_AppendResult::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.AppendRowsResponse.AppendResult)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(_impl_.offset_ != nullptr);
    _impl_.offset_->Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL AppendRowsResponse_AppendResult::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const AppendRowsResponse_AppendResult& this_ = static_cast<const AppendRowsResponse_AppendResult&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL AppendRowsResponse_AppendResult::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const AppendRowsResponse_AppendResult& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.AppendRowsResponse.AppendResult)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.protobuf.Int64Value offset = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.offset_, this_._impl_.offset_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.AppendRowsResponse.AppendResult)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t AppendRowsResponse_AppendResult::ByteSizeLong(const MessageLite& base) {
  const AppendRowsResponse_AppendResult& this_ = static_cast<const AppendRowsResponse_AppendResult&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t AppendRowsResponse_AppendResult::ByteSizeLong() const {
  const AppendRowsResponse_AppendResult& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.AppendRowsResponse.AppendResult)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // .google.protobuf.Int64Value offset = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.offset_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void AppendRowsResponse_AppendResult::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<AppendRowsResponse_AppendResult*>(&to_msg);
  auto& from = static_cast<const AppendRowsResponse_AppendResult&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.AppendRowsResponse.AppendResult)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(from._impl_.offset_ != nullptr);
    if (_this->_impl_.offset_ == nullptr) {
      _this->_impl_.offset_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.offset_);
    } else {
      _this->_impl_.offset_->MergeFrom(*from._impl_.offset_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void AppendRowsResponse_AppendResult::CopyFrom(const AppendRowsResponse_AppendResult& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.AppendRowsResponse.AppendResult)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void AppendRowsResponse_AppendResult::InternalSwap(AppendRowsResponse_AppendResult* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.offset_, other->_impl_.offset_);
}

::google::protobuf::Metadata AppendRowsResponse_AppendResult::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void AppendRowsResponse::set_allocated_append_result(::google::cloud::bigquery::storage::v1::AppendRowsResponse_AppendResult* PROTOBUF_NULLABLE append_result) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_response();
  if (append_result) {
    ::google::protobuf::Arena* submessage_arena = append_result->GetArena();
    if (message_arena != submessage_arena) {
      append_result = ::google::protobuf::internal::GetOwnedMessage(message_arena, append_result, submessage_arena);
    }
    set_has_append_result();
    _impl_.response_.append_result_ = append_result;
  }
  // @@protoc_insertion_point(field_set_allocated:google.cloud.bigquery.storage.v1.AppendRowsResponse.append_result)
}
void AppendRowsResponse::set_allocated_error(::google::rpc::Status* PROTOBUF_NULLABLE error) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_response();
  if (error) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(error)->GetArena();
    if (message_arena != submessage_arena) {
      error = ::google::protobuf::internal::GetOwnedMessage(message_arena, error, submessage_arena);
    }
    set_has_error();
    _impl_.response_.error_ = error;
  }
  // @@protoc_insertion_point(field_set_allocated:google.cloud.bigquery.storage.v1.AppendRowsResponse.error)
}
void AppendRowsResponse::clear_error() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (response_case() == kError) {
    if (GetArena() == nullptr) {
      delete _impl_.response_.error_;
    } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
      ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.response_.error_);
    }
    clear_has_response();
  }
}
void AppendRowsResponse::clear_updated_schema() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.updated_schema_ != nullptr) _impl_.updated_schema_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
AppendRowsResponse::AppendRowsResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AppendRowsResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.AppendRowsResponse)
}
PROTOBUF_NDEBUG_INLINE AppendRowsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::AppendRowsResponse& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        row_errors_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::cloud::bigquery::storage::v1::AppendRowsResponse,
              PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsResponse, _impl_.row_errors_)>()
          , from.row_errors_
        }
        ,
        write_stream_(arena, from.write_stream_),
        response_{},
        _oneof_case_{from._oneof_case_[0]} {}

AppendRowsResponse::AppendRowsResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const AppendRowsResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AppendRowsResponse_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  AppendRowsResponse* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.updated_schema_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.updated_schema_)
                : nullptr;
  switch (response_case()) {
    case RESPONSE_NOT_SET:
      break;
      case kAppendResult:
        _impl_.response_.append_result_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.response_.append_result_);
        break;
      case kError:
        _impl_.response_.error_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.response_.error_);
        break;
  }

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.AppendRowsResponse)
}
PROTOBUF_NDEBUG_INLINE AppendRowsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        row_errors_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::cloud::bigquery::storage::v1::AppendRowsResponse,
            PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::AppendRowsResponse, _impl_.row_errors_)>()
         }
        ,
        write_stream_(arena),
        response_{},
        _oneof_case_{} {}

inline void AppendRowsResponse::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.updated_schema_ = {};
}
AppendRowsResponse::~AppendRowsResponse() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.AppendRowsResponse)
  SharedDtor(*this);
}
inline void AppendRowsResponse::SharedDtor(MessageLite& self) {
  AppendRowsResponse& this_ = static_cast<AppendRowsResponse&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.write_stream_.Destroy();
  delete this_._impl_.updated_schema_;
  if (this_.has_response()) {
    this_.clear_response();
  }
  this_._impl_.~Impl_();
}

void AppendRowsResponse::clear_response() {
// @@protoc_insertion_point(one_of_clear_start:google.cloud.bigquery.storage.v1.AppendRowsResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (response_case()) {
    case kAppendResult: {
      if (GetArena() == nullptr) {
        delete _impl_.response_.append_result_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.response_.append_result_);
      }
      break;
    }
    case kError: {
      if (GetArena() == nullptr) {
        delete _impl_.response_.error_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.response_.error_);
      }
      break;
    }
    case RESPONSE_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = RESPONSE_NOT_SET;
}


#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull AppendRowsResponse_class_data_ =
        AppendRowsResponse::InternalGenerateClassData_(AppendRowsResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AppendRowsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AppendRowsResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(AppendRowsResponse_class_data_.tc_table);
  return AppendRowsResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AppendRowsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AppendRowsResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&AppendRowsResponse_globals_));
  return AppendRowsResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const AppendRowsResponse::ParseTableT_
    AppendRowsResponse::_table_ =
        AppendRowsResponse::InternalGenerateParseTable_(AppendRowsResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void AppendRowsResponse::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.AppendRowsResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.row_errors_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.write_stream_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(_impl_.updated_schema_ != nullptr);
      _impl_.updated_schema_->Clear();
    }
  }
  clear_response();
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL AppendRowsResponse::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const AppendRowsResponse& this_ = static_cast<const AppendRowsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL AppendRowsResponse::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const AppendRowsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.AppendRowsResponse)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  switch (this_.response_case()) {
    case kAppendResult: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          1, *this_._impl_.response_.append_result_, this_._impl_.response_.append_result_->GetCachedSize(), target,
          stream);
      break;
    }
    case kError: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          2, *this_._impl_.response_.error_, this_._impl_.response_.error_->GetCachedSize(), target,
          stream);
      break;
    }
    default:
      break;
  }
  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.cloud.bigquery.storage.v1.TableSchema updated_schema = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        3, *this_._impl_.updated_schema_, this_._impl_.updated_schema_->GetCachedSize(), target,
        stream);
  }

  // repeated .google.cloud.bigquery.storage.v1.RowError row_errors = 4;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_row_errors_size());
         i < n; i++) {
      const auto& repfield = this_._internal_row_errors().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              4, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  // string write_stream = 5;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_write_stream().empty()) {
      const ::std::string& _s = this_._internal_write_stream();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.cloud.bigquery.storage.v1.AppendRowsResponse.write_stream");
      target = stream->WriteStringMaybeAliased(5, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.AppendRowsResponse)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t AppendRowsResponse::ByteSizeLong(const MessageLite& base) {
  const AppendRowsResponse& this_ = static_cast<const AppendRowsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t AppendRowsResponse::ByteSizeLong() const {
  const AppendRowsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.AppendRowsResponse)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // repeated .google.cloud.bigquery.storage.v1.RowError row_errors = 4;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_row_errors_size();
      for (const auto& msg : this_._internal_row_errors()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
    // string write_stream = 5;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_write_stream().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_write_stream());
      }
    }
    // .google.cloud.bigquery.storage.v1.TableSchema updated_schema = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.updated_schema_);
    }
  }
  switch (this_.response_case()) {
    // .google.cloud.bigquery.storage.v1.AppendRowsResponse.AppendResult append_result = 1;
    case kAppendResult: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.response_.append_result_);
      break;
    }
    // .google.rpc.Status error = 2;
    case kError: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.response_.error_);
      break;
    }
    case RESPONSE_NOT_SET: {
      break;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void AppendRowsResponse::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<AppendRowsResponse*>(&to_msg);
  auto& from = static_cast<const AppendRowsResponse&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.AppendRowsResponse)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_row_errors()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_row_errors());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_write_stream().empty()) {
        _this->_internal_set_write_stream(from._internal_write_stream());
      } else {
        if (_this->_impl_.write_stream_.IsDefault()) {
          _this->_internal_set_write_stream("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.updated_schema_ != nullptr);
      if (_this->_impl_.updated_schema_ == nullptr) {
        _this->_impl_.updated_schema_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.updated_schema_);
      } else {
        _this->_impl_.updated_schema_->MergeFrom(*from._impl_.updated_schema_);
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[0]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[0];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_response();
      }
      _this->_impl_._oneof_case_[0] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kAppendResult: {
        if (oneof_needs_init) {
          _this->_impl_.response_.append_result_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.response_.append_result_);
        } else {
          _this->_impl_.response_.append_result_->MergeFrom(*from._impl_.response_.append_result_);
        }
        break;
      }
      case kError: {
        if (oneof_needs_init) {
          _this->_impl_.response_.error_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.response_.error_);
        } else {
          _this->_impl_.response_.error_->MergeFrom(*from._impl_.response_.error_);
        }
        break;
      }
      case RESPONSE_NOT_SET:
        break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void AppendRowsResponse::CopyFrom(const AppendRowsResponse& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.AppendRowsResponse)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void AppendRowsResponse::InternalSwap(AppendRowsResponse* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.row_errors_.InternalSwap(&other->_impl_.row_errors_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.write_stream_, &other->_impl_.write_stream_, arena);
  swap(_impl_.updated_schema_, other->_impl_.updated_schema_);
  swap(_impl_.response_, other->_impl_.response_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
}

::google::protobuf::Metadata AppendRowsResponse::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

GetWriteStreamRequest::GetWriteStreamRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, GetWriteStreamRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.GetWriteStreamRequest)
}
PROTOBUF_NDEBUG_INLINE GetWriteStreamRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::GetWriteStreamRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        name_(arena, from.name_) {}

GetWriteStreamRequest::GetWriteStreamRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const GetWriteStreamRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, GetWriteStreamRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  GetWriteStreamRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.view_ = from._impl_.view_;

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.GetWriteStreamRequest)
}
PROTOBUF_NDEBUG_INLINE GetWriteStreamRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        name_(arena) {}

inline void GetWriteStreamRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.view_ = {};
}
GetWriteStreamRequest::~GetWriteStreamRequest() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.GetWriteStreamRequest)
  SharedDtor(*this);
}
inline void GetWriteStreamRequest::SharedDtor(MessageLite& self) {
  GetWriteStreamRequest& this_ = static_cast<GetWriteStreamRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.name_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull GetWriteStreamRequest_class_data_ =
        GetWriteStreamRequest::InternalGenerateClassData_(GetWriteStreamRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
GetWriteStreamRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&GetWriteStreamRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(GetWriteStreamRequest_class_data_.tc_table);
  return GetWriteStreamRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
GetWriteStreamRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&GetWriteStreamRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&GetWriteStreamRequest_globals_));
  return GetWriteStreamRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const GetWriteStreamRequest::ParseTableT_
    GetWriteStreamRequest::_table_ =
        GetWriteStreamRequest::InternalGenerateParseTable_(GetWriteStreamRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void GetWriteStreamRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.GetWriteStreamRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.name_.ClearNonDefaultToEmpty();
  }
  _impl_.view_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL GetWriteStreamRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const GetWriteStreamRequest& this_ = static_cast<const GetWriteStreamRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL GetWriteStreamRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const GetWriteStreamRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.GetWriteStreamRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_name().empty()) {
      const ::std::string& _s = this_._internal_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.cloud.bigquery.storage.v1.GetWriteStreamRequest.name");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // .google.cloud.bigquery.storage.v1.WriteStreamView view = 3;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_view() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          3, this_._internal_view(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.GetWriteStreamRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t GetWriteStreamRequest::ByteSizeLong(const MessageLite& base) {
  const GetWriteStreamRequest& this_ = static_cast<const GetWriteStreamRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t GetWriteStreamRequest::ByteSizeLong() const {
  const GetWriteStreamRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.GetWriteStreamRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_name());
      }
    }
    // .google.cloud.bigquery.storage.v1.WriteStreamView view = 3;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_view() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_view());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void GetWriteStreamRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<GetWriteStreamRequest*>(&to_msg);
  auto& from = static_cast<const GetWriteStreamRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.GetWriteStreamRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_name().empty()) {
        _this->_internal_set_name(from._internal_name());
      } else {
        if (_this->_impl_.name_.IsDefault()) {
          _this->_internal_set_name("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_view() != 0) {
        _this->_impl_.view_ = from._impl_.view_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void GetWriteStreamRequest::CopyFrom(const GetWriteStreamRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.GetWriteStreamRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void GetWriteStreamRequest::InternalSwap(GetWriteStreamRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.name_, &other->_impl_.name_, arena);
  swap(_impl_.view_, other->_impl_.view_);
}

::google::protobuf::Metadata GetWriteStreamRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

BatchCommitWriteStreamsRequest::BatchCommitWriteStreamsRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, BatchCommitWriteStreamsRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsRequest)
}
PROTOBUF_NDEBUG_INLINE BatchCommitWriteStreamsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        write_streams_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsRequest,
              PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsRequest, _impl_.write_streams_)>()
          , from.write_streams_
        }
        ,
        parent_(arena, from.parent_) {}

BatchCommitWriteStreamsRequest::BatchCommitWriteStreamsRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const BatchCommitWriteStreamsRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, BatchCommitWriteStreamsRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  BatchCommitWriteStreamsRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsRequest)
}
PROTOBUF_NDEBUG_INLINE BatchCommitWriteStreamsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        write_streams_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsRequest,
            PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsRequest, _impl_.write_streams_)>()
         }
        ,
        parent_(arena) {}

inline void BatchCommitWriteStreamsRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
BatchCommitWriteStreamsRequest::~BatchCommitWriteStreamsRequest() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsRequest)
  SharedDtor(*this);
}
inline void BatchCommitWriteStreamsRequest::SharedDtor(MessageLite& self) {
  BatchCommitWriteStreamsRequest& this_ = static_cast<BatchCommitWriteStreamsRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.parent_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull BatchCommitWriteStreamsRequest_class_data_ =
        BatchCommitWriteStreamsRequest::InternalGenerateClassData_(BatchCommitWriteStreamsRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
BatchCommitWriteStreamsRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&BatchCommitWriteStreamsRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(BatchCommitWriteStreamsRequest_class_data_.tc_table);
  return BatchCommitWriteStreamsRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
BatchCommitWriteStreamsRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&BatchCommitWriteStreamsRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&BatchCommitWriteStreamsRequest_globals_));
  return BatchCommitWriteStreamsRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const BatchCommitWriteStreamsRequest::ParseTableT_
    BatchCommitWriteStreamsRequest::_table_ =
        BatchCommitWriteStreamsRequest::InternalGenerateParseTable_(BatchCommitWriteStreamsRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void BatchCommitWriteStreamsRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.write_streams_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.parent_.ClearNonDefaultToEmpty();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL BatchCommitWriteStreamsRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const BatchCommitWriteStreamsRequest& this_ = static_cast<const BatchCommitWriteStreamsRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL BatchCommitWriteStreamsRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const BatchCommitWriteStreamsRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_parent().empty()) {
      const ::std::string& _s = this_._internal_parent();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsRequest.parent");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // repeated string write_streams = 2 [(.google.api.field_behavior) = REQUIRED];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (int i = 0, n = this_._internal_write_streams_size(); i < n; ++i) {
      const auto& s = this_._internal_write_streams().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsRequest.write_streams");
      target = stream->WriteString(2, s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t BatchCommitWriteStreamsRequest::ByteSizeLong(const MessageLite& base) {
  const BatchCommitWriteStreamsRequest& this_ = static_cast<const BatchCommitWriteStreamsRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t BatchCommitWriteStreamsRequest::ByteSizeLong() const {
  const BatchCommitWriteStreamsRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // repeated string write_streams = 2 [(.google.api.field_behavior) = REQUIRED];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_write_streams().size());
      for (int i = 0, n = this_._internal_write_streams().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_write_streams().Get(i));
      }
    }
    // string parent = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_parent().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_parent());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void BatchCommitWriteStreamsRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<BatchCommitWriteStreamsRequest*>(&to_msg);
  auto& from = static_cast<const BatchCommitWriteStreamsRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_write_streams()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_write_streams());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_parent().empty()) {
        _this->_internal_set_parent(from._internal_parent());
      } else {
        if (_this->_impl_.parent_.IsDefault()) {
          _this->_internal_set_parent("");
        }
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void BatchCommitWriteStreamsRequest::CopyFrom(const BatchCommitWriteStreamsRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void BatchCommitWriteStreamsRequest::InternalSwap(BatchCommitWriteStreamsRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.write_streams_.InternalSwap(&other->_impl_.write_streams_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.parent_, &other->_impl_.parent_, arena);
}

::google::protobuf::Metadata BatchCommitWriteStreamsRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void BatchCommitWriteStreamsResponse::clear_commit_time() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.commit_time_ != nullptr) _impl_.commit_time_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
BatchCommitWriteStreamsResponse::BatchCommitWriteStreamsResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, BatchCommitWriteStreamsResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsResponse)
}
PROTOBUF_NDEBUG_INLINE BatchCommitWriteStreamsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsResponse& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        stream_errors_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsResponse,
              PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsResponse, _impl_.stream_errors_)>()
          , from.stream_errors_
        }
     {}

BatchCommitWriteStreamsResponse::BatchCommitWriteStreamsResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const BatchCommitWriteStreamsResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, BatchCommitWriteStreamsResponse_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  BatchCommitWriteStreamsResponse* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.commit_time_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.commit_time_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsResponse)
}
PROTOBUF_NDEBUG_INLINE BatchCommitWriteStreamsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        stream_errors_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsResponse,
            PROTOBUF_FIELD_OFFSET(::google::cloud::bigquery::storage::v1::BatchCommitWriteStreamsResponse, _impl_.stream_errors_)>()
         }
     {}

inline void BatchCommitWriteStreamsResponse::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.commit_time_ = {};
}
BatchCommitWriteStreamsResponse::~BatchCommitWriteStreamsResponse() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsResponse)
  SharedDtor(*this);
}
inline void BatchCommitWriteStreamsResponse::SharedDtor(MessageLite& self) {
  BatchCommitWriteStreamsResponse& this_ = static_cast<BatchCommitWriteStreamsResponse&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.commit_time_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull BatchCommitWriteStreamsResponse_class_data_ =
        BatchCommitWriteStreamsResponse::InternalGenerateClassData_(BatchCommitWriteStreamsResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
BatchCommitWriteStreamsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&BatchCommitWriteStreamsResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(BatchCommitWriteStreamsResponse_class_data_.tc_table);
  return BatchCommitWriteStreamsResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
BatchCommitWriteStreamsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&BatchCommitWriteStreamsResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&BatchCommitWriteStreamsResponse_globals_));
  return BatchCommitWriteStreamsResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const BatchCommitWriteStreamsResponse::ParseTableT_
    BatchCommitWriteStreamsResponse::_table_ =
        BatchCommitWriteStreamsResponse::InternalGenerateParseTable_(BatchCommitWriteStreamsResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void BatchCommitWriteStreamsResponse::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.stream_errors_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.commit_time_ != nullptr);
      _impl_.commit_time_->Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL BatchCommitWriteStreamsResponse::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const BatchCommitWriteStreamsResponse& this_ = static_cast<const BatchCommitWriteStreamsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL BatchCommitWriteStreamsResponse::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const BatchCommitWriteStreamsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsResponse)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.protobuf.Timestamp commit_time = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.commit_time_, this_._impl_.commit_time_->GetCachedSize(), target,
        stream);
  }

  // repeated .google.cloud.bigquery.storage.v1.StorageError stream_errors = 2;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_stream_errors_size());
         i < n; i++) {
      const auto& repfield = this_._internal_stream_errors().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              2, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsResponse)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t BatchCommitWriteStreamsResponse::ByteSizeLong(const MessageLite& base) {
  const BatchCommitWriteStreamsResponse& this_ = static_cast<const BatchCommitWriteStreamsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t BatchCommitWriteStreamsResponse::ByteSizeLong() const {
  const BatchCommitWriteStreamsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsResponse)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // repeated .google.cloud.bigquery.storage.v1.StorageError stream_errors = 2;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_stream_errors_size();
      for (const auto& msg : this_._internal_stream_errors()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
    // .google.protobuf.Timestamp commit_time = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.commit_time_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void BatchCommitWriteStreamsResponse::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<BatchCommitWriteStreamsResponse*>(&to_msg);
  auto& from = static_cast<const BatchCommitWriteStreamsResponse&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsResponse)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_stream_errors()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_stream_errors());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.commit_time_ != nullptr);
      if (_this->_impl_.commit_time_ == nullptr) {
        _this->_impl_.commit_time_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.commit_time_);
      } else {
        _this->_impl_.commit_time_->MergeFrom(*from._impl_.commit_time_);
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void BatchCommitWriteStreamsResponse::CopyFrom(const BatchCommitWriteStreamsResponse& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.BatchCommitWriteStreamsResponse)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void BatchCommitWriteStreamsResponse::InternalSwap(BatchCommitWriteStreamsResponse* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.stream_errors_.InternalSwap(&other->_impl_.stream_errors_);
  swap(_impl_.commit_time_, other->_impl_.commit_time_);
}

::google::protobuf::Metadata BatchCommitWriteStreamsResponse::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

FinalizeWriteStreamRequest::FinalizeWriteStreamRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, FinalizeWriteStreamRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.FinalizeWriteStreamRequest)
}
PROTOBUF_NDEBUG_INLINE FinalizeWriteStreamRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::FinalizeWriteStreamRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        name_(arena, from.name_) {}

FinalizeWriteStreamRequest::FinalizeWriteStreamRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const FinalizeWriteStreamRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, FinalizeWriteStreamRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  FinalizeWriteStreamRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.FinalizeWriteStreamRequest)
}
PROTOBUF_NDEBUG_INLINE FinalizeWriteStreamRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        name_(arena) {}

inline void FinalizeWriteStreamRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
FinalizeWriteStreamRequest::~FinalizeWriteStreamRequest() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.FinalizeWriteStreamRequest)
  SharedDtor(*this);
}
inline void FinalizeWriteStreamRequest::SharedDtor(MessageLite& self) {
  FinalizeWriteStreamRequest& this_ = static_cast<FinalizeWriteStreamRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.name_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull FinalizeWriteStreamRequest_class_data_ =
        FinalizeWriteStreamRequest::InternalGenerateClassData_(FinalizeWriteStreamRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
FinalizeWriteStreamRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&FinalizeWriteStreamRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(FinalizeWriteStreamRequest_class_data_.tc_table);
  return FinalizeWriteStreamRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
FinalizeWriteStreamRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&FinalizeWriteStreamRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&FinalizeWriteStreamRequest_globals_));
  return FinalizeWriteStreamRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const FinalizeWriteStreamRequest::ParseTableT_
    FinalizeWriteStreamRequest::_table_ =
        FinalizeWriteStreamRequest::InternalGenerateParseTable_(FinalizeWriteStreamRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void FinalizeWriteStreamRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.FinalizeWriteStreamRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.name_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL FinalizeWriteStreamRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const FinalizeWriteStreamRequest& this_ = static_cast<const FinalizeWriteStreamRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL FinalizeWriteStreamRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const FinalizeWriteStreamRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.FinalizeWriteStreamRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_name().empty()) {
      const ::std::string& _s = this_._internal_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.cloud.bigquery.storage.v1.FinalizeWriteStreamRequest.name");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.FinalizeWriteStreamRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t FinalizeWriteStreamRequest::ByteSizeLong(const MessageLite& base) {
  const FinalizeWriteStreamRequest& this_ = static_cast<const FinalizeWriteStreamRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t FinalizeWriteStreamRequest::ByteSizeLong() const {
  const FinalizeWriteStreamRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.FinalizeWriteStreamRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string name = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_name());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void FinalizeWriteStreamRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<FinalizeWriteStreamRequest*>(&to_msg);
  auto& from = static_cast<const FinalizeWriteStreamRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.FinalizeWriteStreamRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_name().empty()) {
      _this->_internal_set_name(from._internal_name());
    } else {
      if (_this->_impl_.name_.IsDefault()) {
        _this->_internal_set_name("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void FinalizeWriteStreamRequest::CopyFrom(const FinalizeWriteStreamRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.FinalizeWriteStreamRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void FinalizeWriteStreamRequest::InternalSwap(FinalizeWriteStreamRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.name_, &other->_impl_.name_, arena);
}

::google::protobuf::Metadata FinalizeWriteStreamRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

FinalizeWriteStreamResponse::FinalizeWriteStreamResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, FinalizeWriteStreamResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.FinalizeWriteStreamResponse)
}
FinalizeWriteStreamResponse::FinalizeWriteStreamResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const FinalizeWriteStreamResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, FinalizeWriteStreamResponse_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE FinalizeWriteStreamResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void FinalizeWriteStreamResponse::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.row_count_ = {};
}
FinalizeWriteStreamResponse::~FinalizeWriteStreamResponse() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.FinalizeWriteStreamResponse)
  SharedDtor(*this);
}
inline void FinalizeWriteStreamResponse::SharedDtor(MessageLite& self) {
  FinalizeWriteStreamResponse& this_ = static_cast<FinalizeWriteStreamResponse&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull FinalizeWriteStreamResponse_class_data_ =
        FinalizeWriteStreamResponse::InternalGenerateClassData_(FinalizeWriteStreamResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
FinalizeWriteStreamResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&FinalizeWriteStreamResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(FinalizeWriteStreamResponse_class_data_.tc_table);
  return FinalizeWriteStreamResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
FinalizeWriteStreamResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&FinalizeWriteStreamResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&FinalizeWriteStreamResponse_globals_));
  return FinalizeWriteStreamResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const FinalizeWriteStreamResponse::ParseTableT_
    FinalizeWriteStreamResponse::_table_ =
        FinalizeWriteStreamResponse::InternalGenerateParseTable_(FinalizeWriteStreamResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void FinalizeWriteStreamResponse::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.FinalizeWriteStreamResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.row_count_ = ::int64_t{0};
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL FinalizeWriteStreamResponse::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const FinalizeWriteStreamResponse& this_ = static_cast<const FinalizeWriteStreamResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL FinalizeWriteStreamResponse::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const FinalizeWriteStreamResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.FinalizeWriteStreamResponse)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // int64 row_count = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_row_count() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<1>(
              stream, this_._internal_row_count(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.FinalizeWriteStreamResponse)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t FinalizeWriteStreamResponse::ByteSizeLong(const MessageLite& base) {
  const FinalizeWriteStreamResponse& this_ = static_cast<const FinalizeWriteStreamResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t FinalizeWriteStreamResponse::ByteSizeLong() const {
  const FinalizeWriteStreamResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.FinalizeWriteStreamResponse)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // int64 row_count = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_row_count() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_row_count());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void FinalizeWriteStreamResponse::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<FinalizeWriteStreamResponse*>(&to_msg);
  auto& from = static_cast<const FinalizeWriteStreamResponse&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.FinalizeWriteStreamResponse)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (from._internal_row_count() != 0) {
      _this->_impl_.row_count_ = from._impl_.row_count_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void FinalizeWriteStreamResponse::CopyFrom(const FinalizeWriteStreamResponse& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.FinalizeWriteStreamResponse)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void FinalizeWriteStreamResponse::InternalSwap(FinalizeWriteStreamResponse* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.row_count_, other->_impl_.row_count_);
}

::google::protobuf::Metadata FinalizeWriteStreamResponse::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void FlushRowsRequest::clear_offset() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.offset_ != nullptr) _impl_.offset_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
FlushRowsRequest::FlushRowsRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, FlushRowsRequest_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.FlushRowsRequest)
}
PROTOBUF_NDEBUG_INLINE FlushRowsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::FlushRowsRequest& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        write_stream_(arena, from.write_stream_) {}

FlushRowsRequest::FlushRowsRequest(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const FlushRowsRequest& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, FlushRowsRequest_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  FlushRowsRequest* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.offset_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.offset_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.FlushRowsRequest)
}
PROTOBUF_NDEBUG_INLINE FlushRowsRequest::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        write_stream_(arena) {}

inline void FlushRowsRequest::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.offset_ = {};
}
FlushRowsRequest::~FlushRowsRequest() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.FlushRowsRequest)
  SharedDtor(*this);
}
inline void FlushRowsRequest::SharedDtor(MessageLite& self) {
  FlushRowsRequest& this_ = static_cast<FlushRowsRequest&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.write_stream_.Destroy();
  delete this_._impl_.offset_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull FlushRowsRequest_class_data_ =
        FlushRowsRequest::InternalGenerateClassData_(FlushRowsRequest_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
FlushRowsRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&FlushRowsRequest_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(FlushRowsRequest_class_data_.tc_table);
  return FlushRowsRequest_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
FlushRowsRequest::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&FlushRowsRequest_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&FlushRowsRequest_globals_));
  return FlushRowsRequest_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const FlushRowsRequest::ParseTableT_
    FlushRowsRequest::_table_ =
        FlushRowsRequest::InternalGenerateParseTable_(FlushRowsRequest_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void FlushRowsRequest::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.FlushRowsRequest)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.write_stream_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.offset_ != nullptr);
      _impl_.offset_->Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL FlushRowsRequest::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const FlushRowsRequest& this_ = static_cast<const FlushRowsRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL FlushRowsRequest::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const FlushRowsRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.FlushRowsRequest)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string write_stream = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_write_stream().empty()) {
      const ::std::string& _s = this_._internal_write_stream();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.cloud.bigquery.storage.v1.FlushRowsRequest.write_stream");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  // .google.protobuf.Int64Value offset = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.offset_, this_._impl_.offset_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.FlushRowsRequest)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t FlushRowsRequest::ByteSizeLong(const MessageLite& base) {
  const FlushRowsRequest& this_ = static_cast<const FlushRowsRequest&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t FlushRowsRequest::ByteSizeLong() const {
  const FlushRowsRequest& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.FlushRowsRequest)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // string write_stream = 1 [(.google.api.field_behavior) = REQUIRED, (.google.api.resource_reference) = {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_write_stream().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_write_stream());
      }
    }
    // .google.protobuf.Int64Value offset = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.offset_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void FlushRowsRequest::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<FlushRowsRequest*>(&to_msg);
  auto& from = static_cast<const FlushRowsRequest&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.FlushRowsRequest)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_write_stream().empty()) {
        _this->_internal_set_write_stream(from._internal_write_stream());
      } else {
        if (_this->_impl_.write_stream_.IsDefault()) {
          _this->_internal_set_write_stream("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.offset_ != nullptr);
      if (_this->_impl_.offset_ == nullptr) {
        _this->_impl_.offset_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.offset_);
      } else {
        _this->_impl_.offset_->MergeFrom(*from._impl_.offset_);
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void FlushRowsRequest::CopyFrom(const FlushRowsRequest& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.FlushRowsRequest)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void FlushRowsRequest::InternalSwap(FlushRowsRequest* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.write_stream_, &other->_impl_.write_stream_, arena);
  swap(_impl_.offset_, other->_impl_.offset_);
}

::google::protobuf::Metadata FlushRowsRequest::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

FlushRowsResponse::FlushRowsResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, FlushRowsResponse_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.FlushRowsResponse)
}
FlushRowsResponse::FlushRowsResponse(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const FlushRowsResponse& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, FlushRowsResponse_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE FlushRowsResponse::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void FlushRowsResponse::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.offset_ = {};
}
FlushRowsResponse::~FlushRowsResponse() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.FlushRowsResponse)
  SharedDtor(*this);
}
inline void FlushRowsResponse::SharedDtor(MessageLite& self) {
  FlushRowsResponse& this_ = static_cast<FlushRowsResponse&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull FlushRowsResponse_class_data_ =
        FlushRowsResponse::InternalGenerateClassData_(FlushRowsResponse_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
FlushRowsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&FlushRowsResponse_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(FlushRowsResponse_class_data_.tc_table);
  return FlushRowsResponse_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
FlushRowsResponse::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&FlushRowsResponse_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&FlushRowsResponse_globals_));
  return FlushRowsResponse_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const FlushRowsResponse::ParseTableT_
    FlushRowsResponse::_table_ =
        FlushRowsResponse::InternalGenerateParseTable_(FlushRowsResponse_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void FlushRowsResponse::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.FlushRowsResponse)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.offset_ = ::int64_t{0};
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL FlushRowsResponse::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const FlushRowsResponse& this_ = static_cast<const FlushRowsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL FlushRowsResponse::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const FlushRowsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.FlushRowsResponse)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // int64 offset = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_offset() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<1>(
              stream, this_._internal_offset(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.FlushRowsResponse)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t FlushRowsResponse::ByteSizeLong(const MessageLite& base) {
  const FlushRowsResponse& this_ = static_cast<const FlushRowsResponse&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t FlushRowsResponse::ByteSizeLong() const {
  const FlushRowsResponse& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.FlushRowsResponse)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // int64 offset = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_offset() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_offset());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void FlushRowsResponse::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<FlushRowsResponse*>(&to_msg);
  auto& from = static_cast<const FlushRowsResponse&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.FlushRowsResponse)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (from._internal_offset() != 0) {
      _this->_impl_.offset_ = from._impl_.offset_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void FlushRowsResponse::CopyFrom(const FlushRowsResponse& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.FlushRowsResponse)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void FlushRowsResponse::InternalSwap(FlushRowsResponse* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.offset_, other->_impl_.offset_);
}

::google::protobuf::Metadata FlushRowsResponse::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

StorageError::StorageError(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, StorageError_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.StorageError)
}
PROTOBUF_NDEBUG_INLINE StorageError::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::StorageError& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        entity_(arena, from.entity_),
        error_message_(arena, from.error_message_) {}

StorageError::StorageError(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const StorageError& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, StorageError_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  StorageError* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.code_ = from._impl_.code_;

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.StorageError)
}
PROTOBUF_NDEBUG_INLINE StorageError::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        entity_(arena),
        error_message_(arena) {}

inline void StorageError::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.code_ = {};
}
StorageError::~StorageError() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.StorageError)
  SharedDtor(*this);
}
inline void StorageError::SharedDtor(MessageLite& self) {
  StorageError& this_ = static_cast<StorageError&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.entity_.Destroy();
  this_._impl_.error_message_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull StorageError_class_data_ =
        StorageError::InternalGenerateClassData_(StorageError_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StorageError::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StorageError_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(StorageError_class_data_.tc_table);
  return StorageError_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StorageError::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StorageError_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&StorageError_globals_));
  return StorageError_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const StorageError::ParseTableT_
    StorageError::_table_ =
        StorageError::InternalGenerateParseTable_(StorageError_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void StorageError::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.StorageError)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.entity_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.error_message_.ClearNonDefaultToEmpty();
    }
  }
  _impl_.code_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL StorageError::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const StorageError& this_ = static_cast<const StorageError&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL StorageError::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const StorageError& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.StorageError)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .google.cloud.bigquery.storage.v1.StorageError.StorageErrorCode code = 1;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_code() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          1, this_._internal_code(), target);
    }
  }

  // string entity = 2;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_entity().empty()) {
      const ::std::string& _s = this_._internal_entity();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.cloud.bigquery.storage.v1.StorageError.entity");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  // string error_message = 3;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_error_message().empty()) {
      const ::std::string& _s = this_._internal_error_message();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.cloud.bigquery.storage.v1.StorageError.error_message");
      target = stream->WriteStringMaybeAliased(3, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.StorageError)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t StorageError::ByteSizeLong(const MessageLite& base) {
  const StorageError& this_ = static_cast<const StorageError&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t StorageError::ByteSizeLong() const {
  const StorageError& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.StorageError)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // string entity = 2;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_entity().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_entity());
      }
    }
    // string error_message = 3;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_error_message().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_error_message());
      }
    }
    // .google.cloud.bigquery.storage.v1.StorageError.StorageErrorCode code = 1;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_code() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_code());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void StorageError::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<StorageError*>(&to_msg);
  auto& from = static_cast<const StorageError&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.StorageError)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_entity().empty()) {
        _this->_internal_set_entity(from._internal_entity());
      } else {
        if (_this->_impl_.entity_.IsDefault()) {
          _this->_internal_set_entity("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_error_message().empty()) {
        _this->_internal_set_error_message(from._internal_error_message());
      } else {
        if (_this->_impl_.error_message_.IsDefault()) {
          _this->_internal_set_error_message("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_code() != 0) {
        _this->_impl_.code_ = from._impl_.code_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void StorageError::CopyFrom(const StorageError& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.StorageError)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void StorageError::InternalSwap(StorageError* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.entity_, &other->_impl_.entity_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.error_message_, &other->_impl_.error_message_, arena);
  swap(_impl_.code_, other->_impl_.code_);
}

::google::protobuf::Metadata StorageError::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

RowError::RowError(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, RowError_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:google.cloud.bigquery.storage.v1.RowError)
}
PROTOBUF_NDEBUG_INLINE RowError::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::google::cloud::bigquery::storage::v1::RowError& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        message_(arena, from.message_) {}

RowError::RowError(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const RowError& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, RowError_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  RowError* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, index_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, index_),
           offsetof(Impl_, code_) -
               offsetof(Impl_, index_) +
               sizeof(Impl_::code_));

  // @@protoc_insertion_point(copy_constructor:google.cloud.bigquery.storage.v1.RowError)
}
PROTOBUF_NDEBUG_INLINE RowError::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        message_(arena) {}

inline void RowError::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, index_),
           0,
           offsetof(Impl_, code_) -
               offsetof(Impl_, index_) +
               sizeof(Impl_::code_));
}
RowError::~RowError() {
  // @@protoc_insertion_point(destructor:google.cloud.bigquery.storage.v1.RowError)
  SharedDtor(*this);
}
inline void RowError::SharedDtor(MessageLite& self) {
  RowError& this_ = static_cast<RowError&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.message_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull RowError_class_data_ =
        RowError::InternalGenerateClassData_(RowError_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
RowError::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&RowError_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(RowError_class_data_.tc_table);
  return RowError_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
RowError::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&RowError_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&RowError_globals_));
  return RowError_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const RowError::ParseTableT_
    RowError::_table_ =
        RowError::InternalGenerateParseTable_(RowError_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void RowError::Clear() {
// @@protoc_insertion_point(message_clear_start:google.cloud.bigquery.storage.v1.RowError)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.message_.ClearNonDefaultToEmpty();
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000006U)) {
    ::memset(&_impl_.index_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.code_) -
        reinterpret_cast<char*>(&_impl_.index_)) + sizeof(_impl_.code_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL RowError::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const RowError& this_ = static_cast<const RowError&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL RowError::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const RowError& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:google.cloud.bigquery.storage.v1.RowError)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // int64 index = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_index() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<1>(
              stream, this_._internal_index(), target);
    }
  }

  // .google.cloud.bigquery.storage.v1.RowError.RowErrorCode code = 2;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_code() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          2, this_._internal_code(), target);
    }
  }

  // string message = 3;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_message().empty()) {
      const ::std::string& _s = this_._internal_message();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "google.cloud.bigquery.storage.v1.RowError.message");
      target = stream->WriteStringMaybeAliased(3, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:google.cloud.bigquery.storage.v1.RowError)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t RowError::ByteSizeLong(const MessageLite& base) {
  const RowError& this_ = static_cast<const RowError&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t RowError::ByteSizeLong() const {
  const RowError& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:google.cloud.bigquery.storage.v1.RowError)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // string message = 3;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_message().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_message());
      }
    }
    // int64 index = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_index() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_index());
      }
    }
    // .google.cloud.bigquery.storage.v1.RowError.RowErrorCode code = 2;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_code() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_code());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void RowError::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<RowError*>(&to_msg);
  auto& from = static_cast<const RowError&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:google.cloud.bigquery.storage.v1.RowError)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_message().empty()) {
        _this->_internal_set_message(from._internal_message());
      } else {
        if (_this->_impl_.message_.IsDefault()) {
          _this->_internal_set_message("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_index() != 0) {
        _this->_impl_.index_ = from._impl_.index_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_code() != 0) {
        _this->_impl_.code_ = from._impl_.code_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void RowError::CopyFrom(const RowError& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:google.cloud.bigquery.storage.v1.RowError)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void RowError::InternalSwap(RowError* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.message_, &other->_impl_.message_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(RowError, _impl_.code_)
      + sizeof(RowError::_impl_.code_)
      - PROTOBUF_FIELD_OFFSET(RowError, _impl_.index_)>(
          reinterpret_cast<char*>(&_impl_.index_),
          reinterpret_cast<char*>(&other->_impl_.index_));
}

::google::protobuf::Metadata RowError::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
}  // namespace v1
}  // namespace storage
}  // namespace bigquery
}  // namespace cloud
}  // namespace google
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_google_2fcloud_2fbigquery_2fstorage_2fv1_2fstorage_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
