// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: opentelemetry/proto/metrics/v1/metrics.proto
// Protobuf C++ Version: 7.35.1

#ifndef opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto_2epb_2eh
#define opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7035001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_reflection.h"
#include "google/protobuf/unknown_field_set.h"
#include "opentelemetry/proto/common/v1/common.pb.h"
#include "opentelemetry/proto/resource/v1/resource.pb.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto;
}  // extern "C"
namespace opentelemetry {
namespace proto {
namespace metrics {
namespace v1 {
enum AggregationTemporality : int;
extern const uint32_t AggregationTemporality_internal_data_[];
enum DataPointFlags : int;
extern const uint32_t DataPointFlags_internal_data_[];
class Exemplar;
struct ExemplarGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ExemplarGlobalsTypeInternal Exemplar_globals_;
extern const ::google::protobuf::internal::ClassDataFull Exemplar_class_data_;
#else
extern const ExemplarGlobalsTypeInternal Exemplar_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ExponentialHistogram;
struct ExponentialHistogramGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ExponentialHistogramGlobalsTypeInternal ExponentialHistogram_globals_;
extern const ::google::protobuf::internal::ClassDataFull ExponentialHistogram_class_data_;
#else
extern const ExponentialHistogramGlobalsTypeInternal ExponentialHistogram_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ExponentialHistogramDataPoint;
struct ExponentialHistogramDataPointGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ExponentialHistogramDataPointGlobalsTypeInternal ExponentialHistogramDataPoint_globals_;
extern const ::google::protobuf::internal::ClassDataFull ExponentialHistogramDataPoint_class_data_;
#else
extern const ExponentialHistogramDataPointGlobalsTypeInternal ExponentialHistogramDataPoint_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ExponentialHistogramDataPoint_Buckets;
struct ExponentialHistogramDataPoint_BucketsGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ExponentialHistogramDataPoint_BucketsGlobalsTypeInternal ExponentialHistogramDataPoint_Buckets_globals_;
extern const ::google::protobuf::internal::ClassDataFull ExponentialHistogramDataPoint_Buckets_class_data_;
#else
extern const ExponentialHistogramDataPoint_BucketsGlobalsTypeInternal ExponentialHistogramDataPoint_Buckets_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Gauge;
struct GaugeGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern GaugeGlobalsTypeInternal Gauge_globals_;
extern const ::google::protobuf::internal::ClassDataFull Gauge_class_data_;
#else
extern const GaugeGlobalsTypeInternal Gauge_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Histogram;
struct HistogramGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern HistogramGlobalsTypeInternal Histogram_globals_;
extern const ::google::protobuf::internal::ClassDataFull Histogram_class_data_;
#else
extern const HistogramGlobalsTypeInternal Histogram_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class HistogramDataPoint;
struct HistogramDataPointGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern HistogramDataPointGlobalsTypeInternal HistogramDataPoint_globals_;
extern const ::google::protobuf::internal::ClassDataFull HistogramDataPoint_class_data_;
#else
extern const HistogramDataPointGlobalsTypeInternal HistogramDataPoint_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Metric;
struct MetricGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern MetricGlobalsTypeInternal Metric_globals_;
extern const ::google::protobuf::internal::ClassDataFull Metric_class_data_;
#else
extern const MetricGlobalsTypeInternal Metric_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class MetricsData;
struct MetricsDataGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern MetricsDataGlobalsTypeInternal MetricsData_globals_;
extern const ::google::protobuf::internal::ClassDataFull MetricsData_class_data_;
#else
extern const MetricsDataGlobalsTypeInternal MetricsData_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class NumberDataPoint;
struct NumberDataPointGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern NumberDataPointGlobalsTypeInternal NumberDataPoint_globals_;
extern const ::google::protobuf::internal::ClassDataFull NumberDataPoint_class_data_;
#else
extern const NumberDataPointGlobalsTypeInternal NumberDataPoint_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ResourceMetrics;
struct ResourceMetricsGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ResourceMetricsGlobalsTypeInternal ResourceMetrics_globals_;
extern const ::google::protobuf::internal::ClassDataFull ResourceMetrics_class_data_;
#else
extern const ResourceMetricsGlobalsTypeInternal ResourceMetrics_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ScopeMetrics;
struct ScopeMetricsGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ScopeMetricsGlobalsTypeInternal ScopeMetrics_globals_;
extern const ::google::protobuf::internal::ClassDataFull ScopeMetrics_class_data_;
#else
extern const ScopeMetricsGlobalsTypeInternal ScopeMetrics_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Sum;
struct SumGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern SumGlobalsTypeInternal Sum_globals_;
extern const ::google::protobuf::internal::ClassDataFull Sum_class_data_;
#else
extern const SumGlobalsTypeInternal Sum_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Summary;
struct SummaryGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern SummaryGlobalsTypeInternal Summary_globals_;
extern const ::google::protobuf::internal::ClassDataFull Summary_class_data_;
#else
extern const SummaryGlobalsTypeInternal Summary_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class SummaryDataPoint;
struct SummaryDataPointGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern SummaryDataPointGlobalsTypeInternal SummaryDataPoint_globals_;
extern const ::google::protobuf::internal::ClassDataFull SummaryDataPoint_class_data_;
#else
extern const SummaryDataPointGlobalsTypeInternal SummaryDataPoint_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class SummaryDataPoint_ValueAtQuantile;
struct SummaryDataPoint_ValueAtQuantileGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern SummaryDataPoint_ValueAtQuantileGlobalsTypeInternal SummaryDataPoint_ValueAtQuantile_globals_;
extern const ::google::protobuf::internal::ClassDataFull SummaryDataPoint_ValueAtQuantile_class_data_;
#else
extern const SummaryDataPoint_ValueAtQuantileGlobalsTypeInternal SummaryDataPoint_ValueAtQuantile_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace v1
}  // namespace metrics
}  // namespace proto
}  // namespace opentelemetry
namespace google {
namespace protobuf {
template <>
internal::EnumTraitsT<::opentelemetry::proto::metrics::v1::AggregationTemporality_internal_data_>
    internal::EnumTraitsImpl::value<::opentelemetry::proto::metrics::v1::AggregationTemporality>;
template <>
internal::EnumTraitsT<::opentelemetry::proto::metrics::v1::DataPointFlags_internal_data_>
    internal::EnumTraitsImpl::value<::opentelemetry::proto::metrics::v1::DataPointFlags>;
}  // namespace protobuf
}  // namespace google

namespace opentelemetry {
namespace proto {
namespace metrics {
namespace v1 {
enum AggregationTemporality : int {
  AGGREGATION_TEMPORALITY_UNSPECIFIED = 0,
  AGGREGATION_TEMPORALITY_DELTA = 1,
  AGGREGATION_TEMPORALITY_CUMULATIVE = 2,
  AggregationTemporality_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  AggregationTemporality_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t AggregationTemporality_internal_data_[];
inline constexpr AggregationTemporality AggregationTemporality_MIN =
    static_cast<AggregationTemporality>(0);
inline constexpr AggregationTemporality AggregationTemporality_MAX =
    static_cast<AggregationTemporality>(2);
[[nodiscard]] inline bool AggregationTemporality_IsValid(int value) {
  return 0 <= value && value <= 2;
}
inline constexpr int AggregationTemporality_ARRAYSIZE = 2 + 1;
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
AggregationTemporality_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(AggregationTemporality) {
  return AggregationTemporality_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& AggregationTemporality_Name(T value) {
  static_assert(::std::is_same<T, AggregationTemporality>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to AggregationTemporality_Name().");
  return AggregationTemporality_Name(static_cast<AggregationTemporality>(value));
}
template <>
[[nodiscard]] inline const ::std::string& AggregationTemporality_Name(AggregationTemporality value) {
  return ::google::protobuf::internal::NameOfDenseEnum<AggregationTemporality_descriptor, 0, 2>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool AggregationTemporality_Parse(
    ::absl::string_view name, AggregationTemporality* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<AggregationTemporality>(AggregationTemporality_descriptor(), name,
                                           value);
}
enum DataPointFlags : int {
  DATA_POINT_FLAGS_DO_NOT_USE = 0,
  DATA_POINT_FLAGS_NO_RECORDED_VALUE_MASK = 1,
  DataPointFlags_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  DataPointFlags_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t DataPointFlags_internal_data_[];
inline constexpr DataPointFlags DataPointFlags_MIN =
    static_cast<DataPointFlags>(0);
inline constexpr DataPointFlags DataPointFlags_MAX =
    static_cast<DataPointFlags>(1);
[[nodiscard]] inline bool DataPointFlags_IsValid(int value) {
  return 0 <= value && value <= 1;
}
inline constexpr int DataPointFlags_ARRAYSIZE = 1 + 1;
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
DataPointFlags_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(DataPointFlags) {
  return DataPointFlags_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& DataPointFlags_Name(T value) {
  static_assert(::std::is_same<T, DataPointFlags>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to DataPointFlags_Name().");
  return DataPointFlags_Name(static_cast<DataPointFlags>(value));
}
template <>
[[nodiscard]] inline const ::std::string& DataPointFlags_Name(DataPointFlags value) {
  return ::google::protobuf::internal::NameOfDenseEnum<DataPointFlags_descriptor, 0, 1>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool DataPointFlags_Parse(
    ::absl::string_view name, DataPointFlags* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<DataPointFlags>(DataPointFlags_descriptor(), name,
                                           value);
}
using ::google::protobuf::internal::generated_enum::AbslParseFlag;
using ::google::protobuf::internal::generated_enum::AbslUnparseFlag;

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED SummaryDataPoint_ValueAtQuantile final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.metrics.v1.SummaryDataPoint.ValueAtQuantile) */ {
 public:
  inline SummaryDataPoint_ValueAtQuantile() : SummaryDataPoint_ValueAtQuantile(nullptr) {}
  ~SummaryDataPoint_ValueAtQuantile() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(SummaryDataPoint_ValueAtQuantile* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(SummaryDataPoint_ValueAtQuantile));
  }
#endif

  template <typename = void>
  explicit constexpr SummaryDataPoint_ValueAtQuantile(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline SummaryDataPoint_ValueAtQuantile(const SummaryDataPoint_ValueAtQuantile& from) : SummaryDataPoint_ValueAtQuantile(nullptr, from) {}
  inline SummaryDataPoint_ValueAtQuantile(SummaryDataPoint_ValueAtQuantile&& from) noexcept : SummaryDataPoint_ValueAtQuantile(nullptr, ::std::move(from)) {}
  inline SummaryDataPoint_ValueAtQuantile& operator=(const SummaryDataPoint_ValueAtQuantile& from) {
    CopyFrom(from);
    return *this;
  }
  inline SummaryDataPoint_ValueAtQuantile& operator=(SummaryDataPoint_ValueAtQuantile&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const SummaryDataPoint_ValueAtQuantile& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<SummaryDataPoint_ValueAtQuantile>(&SummaryDataPoint_ValueAtQuantile_globals_);
  }
  static constexpr int kIndexInFileMessages = 13;
  friend void swap(SummaryDataPoint_ValueAtQuantile& a, SummaryDataPoint_ValueAtQuantile& b) { a.Swap(&b); }
  inline void Swap(SummaryDataPoint_ValueAtQuantile* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(SummaryDataPoint_ValueAtQuantile* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] SummaryDataPoint_ValueAtQuantile* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<SummaryDataPoint_ValueAtQuantile>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const SummaryDataPoint_ValueAtQuantile& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const SummaryDataPoint_ValueAtQuantile& from) { SummaryDataPoint_ValueAtQuantile::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(SummaryDataPoint_ValueAtQuantile* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.metrics.v1.SummaryDataPoint.ValueAtQuantile"; }

  explicit SummaryDataPoint_ValueAtQuantile(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  SummaryDataPoint_ValueAtQuantile(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const SummaryDataPoint_ValueAtQuantile& from);
  SummaryDataPoint_ValueAtQuantile(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, SummaryDataPoint_ValueAtQuantile&& from) noexcept
      : SummaryDataPoint_ValueAtQuantile(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kQuantileFieldNumber = 1,
    kValueFieldNumber = 2,
  };
  // double quantile = 1;
  void clear_quantile() ;
  [[nodiscard]] double quantile() const;
  void set_quantile(double value);

  private:
  double _internal_quantile() const;
  void _internal_set_quantile(double value);

  public:
  // double value = 2;
  void clear_value() ;
  [[nodiscard]] double value() const;
  void set_value(double value);

  private:
  double _internal_value() const;
  void _internal_set_value(double value);

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.metrics.v1.SummaryDataPoint.ValueAtQuantile)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const SummaryDataPoint_ValueAtQuantile& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    double quantile_;
    double value_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ExponentialHistogramDataPoint_Buckets final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.Buckets) */ {
 public:
  inline ExponentialHistogramDataPoint_Buckets() : ExponentialHistogramDataPoint_Buckets(nullptr) {}
  ~ExponentialHistogramDataPoint_Buckets() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ExponentialHistogramDataPoint_Buckets* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ExponentialHistogramDataPoint_Buckets));
  }
#endif

  template <typename = void>
  explicit constexpr ExponentialHistogramDataPoint_Buckets(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ExponentialHistogramDataPoint_Buckets(const ExponentialHistogramDataPoint_Buckets& from) : ExponentialHistogramDataPoint_Buckets(nullptr, from) {}
  inline ExponentialHistogramDataPoint_Buckets(ExponentialHistogramDataPoint_Buckets&& from) noexcept : ExponentialHistogramDataPoint_Buckets(nullptr, ::std::move(from)) {}
  inline ExponentialHistogramDataPoint_Buckets& operator=(const ExponentialHistogramDataPoint_Buckets& from) {
    CopyFrom(from);
    return *this;
  }
  inline ExponentialHistogramDataPoint_Buckets& operator=(ExponentialHistogramDataPoint_Buckets&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ExponentialHistogramDataPoint_Buckets& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ExponentialHistogramDataPoint_Buckets>(&ExponentialHistogramDataPoint_Buckets_globals_);
  }
  static constexpr int kIndexInFileMessages = 11;
  friend void swap(ExponentialHistogramDataPoint_Buckets& a, ExponentialHistogramDataPoint_Buckets& b) { a.Swap(&b); }
  inline void Swap(ExponentialHistogramDataPoint_Buckets* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ExponentialHistogramDataPoint_Buckets* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ExponentialHistogramDataPoint_Buckets* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ExponentialHistogramDataPoint_Buckets>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ExponentialHistogramDataPoint_Buckets& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ExponentialHistogramDataPoint_Buckets& from) { ExponentialHistogramDataPoint_Buckets::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ExponentialHistogramDataPoint_Buckets* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.Buckets"; }

  explicit ExponentialHistogramDataPoint_Buckets(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ExponentialHistogramDataPoint_Buckets(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ExponentialHistogramDataPoint_Buckets& from);
  ExponentialHistogramDataPoint_Buckets(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ExponentialHistogramDataPoint_Buckets&& from) noexcept
      : ExponentialHistogramDataPoint_Buckets(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kBucketCountsFieldNumber = 2,
    kOffsetFieldNumber = 1,
  };
  // repeated uint64 bucket_counts = 2;
  [[nodiscard]] int bucket_counts_size()
      const;
  private:
  int _internal_bucket_counts_size() const;

  public:
  void clear_bucket_counts() ;
  [[nodiscard]] ::uint64_t bucket_counts(int index) const;
  void set_bucket_counts(int index, ::uint64_t value);
  void add_bucket_counts(::uint64_t value);
  [[nodiscard]] const ::google::protobuf::RepeatedField<::uint64_t>& bucket_counts()
      const;
  ::google::protobuf::RepeatedField<::uint64_t>* PROTOBUF_NONNULL mutable_bucket_counts();

  private:
  const ::google::protobuf::RepeatedField<::uint64_t>& _internal_bucket_counts() const;
  ::google::protobuf::RepeatedField<::uint64_t>* PROTOBUF_NONNULL _internal_mutable_bucket_counts();

  public:
  // sint32 offset = 1;
  void clear_offset() ;
  [[nodiscard]] ::int32_t offset() const;
  void set_offset(::int32_t value);

  private:
  ::int32_t _internal_offset() const;
  void _internal_set_offset(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.Buckets)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ExponentialHistogramDataPoint_Buckets& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedField<::uint64_t> bucket_counts_;
    ::google::protobuf::internal::CachedSize _bucket_counts_cached_byte_size_;
    ::int32_t offset_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED SummaryDataPoint final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.metrics.v1.SummaryDataPoint) */ {
 public:
  inline SummaryDataPoint() : SummaryDataPoint(nullptr) {}
  ~SummaryDataPoint() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(SummaryDataPoint* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(SummaryDataPoint));
  }
#endif

  template <typename = void>
  explicit constexpr SummaryDataPoint(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline SummaryDataPoint(const SummaryDataPoint& from) : SummaryDataPoint(nullptr, from) {}
  inline SummaryDataPoint(SummaryDataPoint&& from) noexcept : SummaryDataPoint(nullptr, ::std::move(from)) {}
  inline SummaryDataPoint& operator=(const SummaryDataPoint& from) {
    CopyFrom(from);
    return *this;
  }
  inline SummaryDataPoint& operator=(SummaryDataPoint&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const SummaryDataPoint& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<SummaryDataPoint>(&SummaryDataPoint_globals_);
  }
  static constexpr int kIndexInFileMessages = 14;
  friend void swap(SummaryDataPoint& a, SummaryDataPoint& b) { a.Swap(&b); }
  inline void Swap(SummaryDataPoint* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(SummaryDataPoint* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] SummaryDataPoint* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<SummaryDataPoint>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const SummaryDataPoint& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const SummaryDataPoint& from) { SummaryDataPoint::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(SummaryDataPoint* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.metrics.v1.SummaryDataPoint"; }

  explicit SummaryDataPoint(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  SummaryDataPoint(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const SummaryDataPoint& from);
  SummaryDataPoint(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, SummaryDataPoint&& from) noexcept
      : SummaryDataPoint(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using ValueAtQuantile = SummaryDataPoint_ValueAtQuantile;

  // accessors -------------------------------------------------------
  enum : int {
    kQuantileValuesFieldNumber = 6,
    kAttributesFieldNumber = 7,
    kStartTimeUnixNanoFieldNumber = 2,
    kTimeUnixNanoFieldNumber = 3,
    kCountFieldNumber = 4,
    kSumFieldNumber = 5,
    kFlagsFieldNumber = 8,
  };
  // repeated .opentelemetry.proto.metrics.v1.SummaryDataPoint.ValueAtQuantile quantile_values = 6;
  [[nodiscard]] int quantile_values_size()
      const;
  private:
  int _internal_quantile_values_size() const;

  public:
  void clear_quantile_values() ;
  [[nodiscard]] const ::opentelemetry::proto::metrics::v1::SummaryDataPoint_ValueAtQuantile& quantile_values(int index) const;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::SummaryDataPoint_ValueAtQuantile* PROTOBUF_NONNULL mutable_quantile_values(int index);
  ::opentelemetry::proto::metrics::v1::SummaryDataPoint_ValueAtQuantile* PROTOBUF_NONNULL add_quantile_values();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::SummaryDataPoint_ValueAtQuantile>&
  quantile_values() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::SummaryDataPoint_ValueAtQuantile>* PROTOBUF_NONNULL
  mutable_quantile_values();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::SummaryDataPoint_ValueAtQuantile>& _internal_quantile_values() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::SummaryDataPoint_ValueAtQuantile>* PROTOBUF_NONNULL _internal_mutable_quantile_values();

  public:
  // repeated .opentelemetry.proto.common.v1.KeyValue attributes = 7;
  [[nodiscard]] int attributes_size()
      const;
  private:
  int _internal_attributes_size() const;

  public:
  void clear_attributes() ;
  [[nodiscard]] const ::opentelemetry::proto::common::v1::KeyValue& attributes(int index) const;
  [[nodiscard]] ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL mutable_attributes(int index);
  ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL add_attributes();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>&
  attributes() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL
  mutable_attributes();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>& _internal_attributes() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL _internal_mutable_attributes();

  public:
  // fixed64 start_time_unix_nano = 2;
  void clear_start_time_unix_nano() ;
  [[nodiscard]] ::uint64_t start_time_unix_nano() const;
  void set_start_time_unix_nano(::uint64_t value);

  private:
  ::uint64_t _internal_start_time_unix_nano() const;
  void _internal_set_start_time_unix_nano(::uint64_t value);

  public:
  // fixed64 time_unix_nano = 3;
  void clear_time_unix_nano() ;
  [[nodiscard]] ::uint64_t time_unix_nano() const;
  void set_time_unix_nano(::uint64_t value);

  private:
  ::uint64_t _internal_time_unix_nano() const;
  void _internal_set_time_unix_nano(::uint64_t value);

  public:
  // fixed64 count = 4;
  void clear_count() ;
  [[nodiscard]] ::uint64_t count() const;
  void set_count(::uint64_t value);

  private:
  ::uint64_t _internal_count() const;
  void _internal_set_count(::uint64_t value);

  public:
  // double sum = 5;
  void clear_sum() ;
  [[nodiscard]] double sum() const;
  void set_sum(double value);

  private:
  double _internal_sum() const;
  void _internal_set_sum(double value);

  public:
  // uint32 flags = 8;
  void clear_flags() ;
  [[nodiscard]] ::uint32_t flags() const;
  void set_flags(::uint32_t value);

  private:
  ::uint32_t _internal_flags() const;
  void _internal_set_flags(::uint32_t value);

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.metrics.v1.SummaryDataPoint)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 7,
                          2, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const SummaryDataPoint& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::metrics::v1::SummaryDataPoint_ValueAtQuantile > quantile_values_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::common::v1::KeyValue > attributes_;
    ::uint64_t start_time_unix_nano_;
    ::uint64_t time_unix_nano_;
    ::uint64_t count_;
    double sum_;
    ::uint32_t flags_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Exemplar final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.metrics.v1.Exemplar) */ {
 public:
  inline Exemplar() : Exemplar(nullptr) {}
  ~Exemplar() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Exemplar* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Exemplar));
  }
#endif

  template <typename = void>
  explicit constexpr Exemplar(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Exemplar(const Exemplar& from) : Exemplar(nullptr, from) {}
  inline Exemplar(Exemplar&& from) noexcept : Exemplar(nullptr, ::std::move(from)) {}
  inline Exemplar& operator=(const Exemplar& from) {
    CopyFrom(from);
    return *this;
  }
  inline Exemplar& operator=(Exemplar&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Exemplar& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Exemplar>(&Exemplar_globals_);
  }
  enum ValueCase {
    kAsDouble = 3,
    kAsInt = 6,
    VALUE_NOT_SET = 0,
  };
  static constexpr int kIndexInFileMessages = 15;
  friend void swap(Exemplar& a, Exemplar& b) { a.Swap(&b); }
  inline void Swap(Exemplar* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Exemplar* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Exemplar* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Exemplar>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Exemplar& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Exemplar& from) { Exemplar::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Exemplar* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.metrics.v1.Exemplar"; }

  explicit Exemplar(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Exemplar(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Exemplar& from);
  Exemplar(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Exemplar&& from) noexcept
      : Exemplar(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFilteredAttributesFieldNumber = 7,
    kSpanIdFieldNumber = 4,
    kTraceIdFieldNumber = 5,
    kTimeUnixNanoFieldNumber = 2,
    kAsDoubleFieldNumber = 3,
    kAsIntFieldNumber = 6,
  };
  // repeated .opentelemetry.proto.common.v1.KeyValue filtered_attributes = 7;
  [[nodiscard]] int filtered_attributes_size()
      const;
  private:
  int _internal_filtered_attributes_size() const;

  public:
  void clear_filtered_attributes() ;
  [[nodiscard]] const ::opentelemetry::proto::common::v1::KeyValue& filtered_attributes(int index) const;
  [[nodiscard]] ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL mutable_filtered_attributes(int index);
  ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL add_filtered_attributes();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>&
  filtered_attributes() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL
  mutable_filtered_attributes();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>& _internal_filtered_attributes() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL _internal_mutable_filtered_attributes();

  public:
  // bytes span_id = 4;
  void clear_span_id() ;
  [[nodiscard]] const ::std::string& span_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_span_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_span_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_span_id();
  void set_allocated_span_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_span_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_span_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_span_id();

  public:
  // bytes trace_id = 5;
  void clear_trace_id() ;
  [[nodiscard]] const ::std::string& trace_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_trace_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_trace_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_trace_id();
  void set_allocated_trace_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_trace_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_trace_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_trace_id();

  public:
  // fixed64 time_unix_nano = 2;
  void clear_time_unix_nano() ;
  [[nodiscard]] ::uint64_t time_unix_nano() const;
  void set_time_unix_nano(::uint64_t value);

  private:
  ::uint64_t _internal_time_unix_nano() const;
  void _internal_set_time_unix_nano(::uint64_t value);

  public:
  // double as_double = 3;
  [[nodiscard]] bool has_as_double()
      const;
  void clear_as_double() ;
  [[nodiscard]] double as_double() const;
  void set_as_double(double value);

  private:
  double _internal_as_double() const;
  void _internal_set_as_double(double value);

  public:
  // sfixed64 as_int = 6;
  [[nodiscard]] bool has_as_int()
      const;
  void clear_as_int() ;
  [[nodiscard]] ::int64_t as_int() const;
  void set_as_int(::int64_t value);

  private:
  ::int64_t _internal_as_int() const;
  void _internal_set_as_int(::int64_t value);

  public:
  void clear_value();
  ValueCase value_case() const;
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.metrics.v1.Exemplar)
 private:
  class _Internal;
  void set_has_as_double();
  void set_has_as_int();
  [[nodiscard]] inline bool has_value() const;
  inline void clear_has_value();
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 6,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Exemplar& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::common::v1::KeyValue > filtered_attributes_;
    ::google::protobuf::internal::ArenaStringPtr span_id_;
    ::google::protobuf::internal::ArenaStringPtr trace_id_;
    ::uint64_t time_unix_nano_;
    union ValueUnion {
      constexpr ValueUnion() : _constinit_{} {}
      ::google::protobuf::internal::ConstantInitialized _constinit_;
      double as_double_;
      ::int64_t as_int_;
    } value_;
    ::uint32_t _oneof_case_[1];
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Summary final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.metrics.v1.Summary) */ {
 public:
  inline Summary() : Summary(nullptr) {}
  ~Summary() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Summary* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Summary));
  }
#endif

  template <typename = void>
  explicit constexpr Summary(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Summary(const Summary& from) : Summary(nullptr, from) {}
  inline Summary(Summary&& from) noexcept : Summary(nullptr, ::std::move(from)) {}
  inline Summary& operator=(const Summary& from) {
    CopyFrom(from);
    return *this;
  }
  inline Summary& operator=(Summary&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Summary& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Summary>(&Summary_globals_);
  }
  static constexpr int kIndexInFileMessages = 8;
  friend void swap(Summary& a, Summary& b) { a.Swap(&b); }
  inline void Swap(Summary* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Summary* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Summary* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Summary>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Summary& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Summary& from) { Summary::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Summary* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.metrics.v1.Summary"; }

  explicit Summary(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Summary(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Summary& from);
  Summary(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Summary&& from) noexcept
      : Summary(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kDataPointsFieldNumber = 1,
  };
  // repeated .opentelemetry.proto.metrics.v1.SummaryDataPoint data_points = 1;
  [[nodiscard]] int data_points_size()
      const;
  private:
  int _internal_data_points_size() const;

  public:
  void clear_data_points() ;
  [[nodiscard]] const ::opentelemetry::proto::metrics::v1::SummaryDataPoint& data_points(int index) const;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::SummaryDataPoint* PROTOBUF_NONNULL mutable_data_points(int index);
  ::opentelemetry::proto::metrics::v1::SummaryDataPoint* PROTOBUF_NONNULL add_data_points();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::SummaryDataPoint>&
  data_points() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::SummaryDataPoint>* PROTOBUF_NONNULL
  mutable_data_points();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::SummaryDataPoint>& _internal_data_points() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::SummaryDataPoint>* PROTOBUF_NONNULL _internal_mutable_data_points();

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.metrics.v1.Summary)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Summary& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::metrics::v1::SummaryDataPoint > data_points_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED NumberDataPoint final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.metrics.v1.NumberDataPoint) */ {
 public:
  inline NumberDataPoint() : NumberDataPoint(nullptr) {}
  ~NumberDataPoint() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(NumberDataPoint* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(NumberDataPoint));
  }
#endif

  template <typename = void>
  explicit constexpr NumberDataPoint(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline NumberDataPoint(const NumberDataPoint& from) : NumberDataPoint(nullptr, from) {}
  inline NumberDataPoint(NumberDataPoint&& from) noexcept : NumberDataPoint(nullptr, ::std::move(from)) {}
  inline NumberDataPoint& operator=(const NumberDataPoint& from) {
    CopyFrom(from);
    return *this;
  }
  inline NumberDataPoint& operator=(NumberDataPoint&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const NumberDataPoint& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<NumberDataPoint>(&NumberDataPoint_globals_);
  }
  enum ValueCase {
    kAsDouble = 4,
    kAsInt = 6,
    VALUE_NOT_SET = 0,
  };
  static constexpr int kIndexInFileMessages = 9;
  friend void swap(NumberDataPoint& a, NumberDataPoint& b) { a.Swap(&b); }
  inline void Swap(NumberDataPoint* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(NumberDataPoint* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] NumberDataPoint* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<NumberDataPoint>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const NumberDataPoint& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const NumberDataPoint& from) { NumberDataPoint::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(NumberDataPoint* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.metrics.v1.NumberDataPoint"; }

  explicit NumberDataPoint(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  NumberDataPoint(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const NumberDataPoint& from);
  NumberDataPoint(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, NumberDataPoint&& from) noexcept
      : NumberDataPoint(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kExemplarsFieldNumber = 5,
    kAttributesFieldNumber = 7,
    kStartTimeUnixNanoFieldNumber = 2,
    kTimeUnixNanoFieldNumber = 3,
    kFlagsFieldNumber = 8,
    kAsDoubleFieldNumber = 4,
    kAsIntFieldNumber = 6,
  };
  // repeated .opentelemetry.proto.metrics.v1.Exemplar exemplars = 5;
  [[nodiscard]] int exemplars_size()
      const;
  private:
  int _internal_exemplars_size() const;

  public:
  void clear_exemplars() ;
  [[nodiscard]] const ::opentelemetry::proto::metrics::v1::Exemplar& exemplars(int index) const;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::Exemplar* PROTOBUF_NONNULL mutable_exemplars(int index);
  ::opentelemetry::proto::metrics::v1::Exemplar* PROTOBUF_NONNULL add_exemplars();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>&
  exemplars() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>* PROTOBUF_NONNULL
  mutable_exemplars();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>& _internal_exemplars() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>* PROTOBUF_NONNULL _internal_mutable_exemplars();

  public:
  // repeated .opentelemetry.proto.common.v1.KeyValue attributes = 7;
  [[nodiscard]] int attributes_size()
      const;
  private:
  int _internal_attributes_size() const;

  public:
  void clear_attributes() ;
  [[nodiscard]] const ::opentelemetry::proto::common::v1::KeyValue& attributes(int index) const;
  [[nodiscard]] ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL mutable_attributes(int index);
  ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL add_attributes();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>&
  attributes() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL
  mutable_attributes();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>& _internal_attributes() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL _internal_mutable_attributes();

  public:
  // fixed64 start_time_unix_nano = 2;
  void clear_start_time_unix_nano() ;
  [[nodiscard]] ::uint64_t start_time_unix_nano() const;
  void set_start_time_unix_nano(::uint64_t value);

  private:
  ::uint64_t _internal_start_time_unix_nano() const;
  void _internal_set_start_time_unix_nano(::uint64_t value);

  public:
  // fixed64 time_unix_nano = 3;
  void clear_time_unix_nano() ;
  [[nodiscard]] ::uint64_t time_unix_nano() const;
  void set_time_unix_nano(::uint64_t value);

  private:
  ::uint64_t _internal_time_unix_nano() const;
  void _internal_set_time_unix_nano(::uint64_t value);

  public:
  // uint32 flags = 8;
  void clear_flags() ;
  [[nodiscard]] ::uint32_t flags() const;
  void set_flags(::uint32_t value);

  private:
  ::uint32_t _internal_flags() const;
  void _internal_set_flags(::uint32_t value);

  public:
  // double as_double = 4;
  [[nodiscard]] bool has_as_double()
      const;
  void clear_as_double() ;
  [[nodiscard]] double as_double() const;
  void set_as_double(double value);

  private:
  double _internal_as_double() const;
  void _internal_set_as_double(double value);

  public:
  // sfixed64 as_int = 6;
  [[nodiscard]] bool has_as_int()
      const;
  void clear_as_int() ;
  [[nodiscard]] ::int64_t as_int() const;
  void set_as_int(::int64_t value);

  private:
  ::int64_t _internal_as_int() const;
  void _internal_set_as_int(::int64_t value);

  public:
  void clear_value();
  ValueCase value_case() const;
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.metrics.v1.NumberDataPoint)
 private:
  class _Internal;
  void set_has_as_double();
  void set_has_as_int();
  [[nodiscard]] inline bool has_value() const;
  inline void clear_has_value();
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 7,
                          2, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const NumberDataPoint& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::metrics::v1::Exemplar > exemplars_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::common::v1::KeyValue > attributes_;
    ::uint64_t start_time_unix_nano_;
    ::uint64_t time_unix_nano_;
    ::uint32_t flags_;
    union ValueUnion {
      constexpr ValueUnion() : _constinit_{} {}
      ::google::protobuf::internal::ConstantInitialized _constinit_;
      double as_double_;
      ::int64_t as_int_;
    } value_;
    ::uint32_t _oneof_case_[1];
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED HistogramDataPoint final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.metrics.v1.HistogramDataPoint) */ {
 public:
  inline HistogramDataPoint() : HistogramDataPoint(nullptr) {}
  ~HistogramDataPoint() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(HistogramDataPoint* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(HistogramDataPoint));
  }
#endif

  template <typename = void>
  explicit constexpr HistogramDataPoint(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline HistogramDataPoint(const HistogramDataPoint& from) : HistogramDataPoint(nullptr, from) {}
  inline HistogramDataPoint(HistogramDataPoint&& from) noexcept : HistogramDataPoint(nullptr, ::std::move(from)) {}
  inline HistogramDataPoint& operator=(const HistogramDataPoint& from) {
    CopyFrom(from);
    return *this;
  }
  inline HistogramDataPoint& operator=(HistogramDataPoint&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const HistogramDataPoint& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<HistogramDataPoint>(&HistogramDataPoint_globals_);
  }
  static constexpr int kIndexInFileMessages = 10;
  friend void swap(HistogramDataPoint& a, HistogramDataPoint& b) { a.Swap(&b); }
  inline void Swap(HistogramDataPoint* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(HistogramDataPoint* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] HistogramDataPoint* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<HistogramDataPoint>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const HistogramDataPoint& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const HistogramDataPoint& from) { HistogramDataPoint::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(HistogramDataPoint* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.metrics.v1.HistogramDataPoint"; }

  explicit HistogramDataPoint(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  HistogramDataPoint(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const HistogramDataPoint& from);
  HistogramDataPoint(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, HistogramDataPoint&& from) noexcept
      : HistogramDataPoint(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kBucketCountsFieldNumber = 6,
    kExplicitBoundsFieldNumber = 7,
    kExemplarsFieldNumber = 8,
    kAttributesFieldNumber = 9,
    kStartTimeUnixNanoFieldNumber = 2,
    kTimeUnixNanoFieldNumber = 3,
    kCountFieldNumber = 4,
    kSumFieldNumber = 5,
    kMinFieldNumber = 11,
    kMaxFieldNumber = 12,
    kFlagsFieldNumber = 10,
  };
  // repeated fixed64 bucket_counts = 6;
  [[nodiscard]] int bucket_counts_size()
      const;
  private:
  int _internal_bucket_counts_size() const;

  public:
  void clear_bucket_counts() ;
  [[nodiscard]] ::uint64_t bucket_counts(int index) const;
  void set_bucket_counts(int index, ::uint64_t value);
  void add_bucket_counts(::uint64_t value);
  [[nodiscard]] const ::google::protobuf::RepeatedField<::uint64_t>& bucket_counts()
      const;
  ::google::protobuf::RepeatedField<::uint64_t>* PROTOBUF_NONNULL mutable_bucket_counts();

  private:
  const ::google::protobuf::RepeatedField<::uint64_t>& _internal_bucket_counts() const;
  ::google::protobuf::RepeatedField<::uint64_t>* PROTOBUF_NONNULL _internal_mutable_bucket_counts();

  public:
  // repeated double explicit_bounds = 7;
  [[nodiscard]] int explicit_bounds_size()
      const;
  private:
  int _internal_explicit_bounds_size() const;

  public:
  void clear_explicit_bounds() ;
  [[nodiscard]] double explicit_bounds(int index) const;
  void set_explicit_bounds(int index, double value);
  void add_explicit_bounds(double value);
  [[nodiscard]] const ::google::protobuf::RepeatedField<double>& explicit_bounds()
      const;
  ::google::protobuf::RepeatedField<double>* PROTOBUF_NONNULL mutable_explicit_bounds();

  private:
  const ::google::protobuf::RepeatedField<double>& _internal_explicit_bounds() const;
  ::google::protobuf::RepeatedField<double>* PROTOBUF_NONNULL _internal_mutable_explicit_bounds();

  public:
  // repeated .opentelemetry.proto.metrics.v1.Exemplar exemplars = 8;
  [[nodiscard]] int exemplars_size()
      const;
  private:
  int _internal_exemplars_size() const;

  public:
  void clear_exemplars() ;
  [[nodiscard]] const ::opentelemetry::proto::metrics::v1::Exemplar& exemplars(int index) const;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::Exemplar* PROTOBUF_NONNULL mutable_exemplars(int index);
  ::opentelemetry::proto::metrics::v1::Exemplar* PROTOBUF_NONNULL add_exemplars();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>&
  exemplars() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>* PROTOBUF_NONNULL
  mutable_exemplars();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>& _internal_exemplars() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>* PROTOBUF_NONNULL _internal_mutable_exemplars();

  public:
  // repeated .opentelemetry.proto.common.v1.KeyValue attributes = 9;
  [[nodiscard]] int attributes_size()
      const;
  private:
  int _internal_attributes_size() const;

  public:
  void clear_attributes() ;
  [[nodiscard]] const ::opentelemetry::proto::common::v1::KeyValue& attributes(int index) const;
  [[nodiscard]] ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL mutable_attributes(int index);
  ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL add_attributes();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>&
  attributes() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL
  mutable_attributes();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>& _internal_attributes() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL _internal_mutable_attributes();

  public:
  // fixed64 start_time_unix_nano = 2;
  void clear_start_time_unix_nano() ;
  [[nodiscard]] ::uint64_t start_time_unix_nano() const;
  void set_start_time_unix_nano(::uint64_t value);

  private:
  ::uint64_t _internal_start_time_unix_nano() const;
  void _internal_set_start_time_unix_nano(::uint64_t value);

  public:
  // fixed64 time_unix_nano = 3;
  void clear_time_unix_nano() ;
  [[nodiscard]] ::uint64_t time_unix_nano() const;
  void set_time_unix_nano(::uint64_t value);

  private:
  ::uint64_t _internal_time_unix_nano() const;
  void _internal_set_time_unix_nano(::uint64_t value);

  public:
  // fixed64 count = 4;
  void clear_count() ;
  [[nodiscard]] ::uint64_t count() const;
  void set_count(::uint64_t value);

  private:
  ::uint64_t _internal_count() const;
  void _internal_set_count(::uint64_t value);

  public:
  // optional double sum = 5;
  [[nodiscard]] bool has_sum()
      const;
  void clear_sum() ;
  [[nodiscard]] double sum() const;
  void set_sum(double value);

  private:
  double _internal_sum() const;
  void _internal_set_sum(double value);

  public:
  // optional double min = 11;
  [[nodiscard]] bool has_min()
      const;
  void clear_min() ;
  [[nodiscard]] double min() const;
  void set_min(double value);

  private:
  double _internal_min() const;
  void _internal_set_min(double value);

  public:
  // optional double max = 12;
  [[nodiscard]] bool has_max()
      const;
  void clear_max() ;
  [[nodiscard]] double max() const;
  void set_max(double value);

  private:
  double _internal_max() const;
  void _internal_set_max(double value);

  public:
  // uint32 flags = 10;
  void clear_flags() ;
  [[nodiscard]] ::uint32_t flags() const;
  void set_flags(::uint32_t value);

  private:
  ::uint32_t _internal_flags() const;
  void _internal_set_flags(::uint32_t value);

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.metrics.v1.HistogramDataPoint)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<4, 11,
                          2, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const HistogramDataPoint& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedField<::uint64_t> bucket_counts_;
    ::google::protobuf::RepeatedField<double> explicit_bounds_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::metrics::v1::Exemplar > exemplars_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::common::v1::KeyValue > attributes_;
    ::uint64_t start_time_unix_nano_;
    ::uint64_t time_unix_nano_;
    ::uint64_t count_;
    double sum_;
    double min_;
    double max_;
    ::uint32_t flags_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ExponentialHistogramDataPoint final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint) */ {
 public:
  inline ExponentialHistogramDataPoint() : ExponentialHistogramDataPoint(nullptr) {}
  ~ExponentialHistogramDataPoint() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ExponentialHistogramDataPoint* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ExponentialHistogramDataPoint));
  }
#endif

  template <typename = void>
  explicit constexpr ExponentialHistogramDataPoint(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ExponentialHistogramDataPoint(const ExponentialHistogramDataPoint& from) : ExponentialHistogramDataPoint(nullptr, from) {}
  inline ExponentialHistogramDataPoint(ExponentialHistogramDataPoint&& from) noexcept : ExponentialHistogramDataPoint(nullptr, ::std::move(from)) {}
  inline ExponentialHistogramDataPoint& operator=(const ExponentialHistogramDataPoint& from) {
    CopyFrom(from);
    return *this;
  }
  inline ExponentialHistogramDataPoint& operator=(ExponentialHistogramDataPoint&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ExponentialHistogramDataPoint& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ExponentialHistogramDataPoint>(&ExponentialHistogramDataPoint_globals_);
  }
  static constexpr int kIndexInFileMessages = 12;
  friend void swap(ExponentialHistogramDataPoint& a, ExponentialHistogramDataPoint& b) { a.Swap(&b); }
  inline void Swap(ExponentialHistogramDataPoint* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ExponentialHistogramDataPoint* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ExponentialHistogramDataPoint* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ExponentialHistogramDataPoint>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ExponentialHistogramDataPoint& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ExponentialHistogramDataPoint& from) { ExponentialHistogramDataPoint::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ExponentialHistogramDataPoint* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint"; }

  explicit ExponentialHistogramDataPoint(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ExponentialHistogramDataPoint(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ExponentialHistogramDataPoint& from);
  ExponentialHistogramDataPoint(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ExponentialHistogramDataPoint&& from) noexcept
      : ExponentialHistogramDataPoint(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using Buckets = ExponentialHistogramDataPoint_Buckets;

  // accessors -------------------------------------------------------
  enum : int {
    kAttributesFieldNumber = 1,
    kExemplarsFieldNumber = 11,
    kPositiveFieldNumber = 8,
    kNegativeFieldNumber = 9,
    kStartTimeUnixNanoFieldNumber = 2,
    kTimeUnixNanoFieldNumber = 3,
    kCountFieldNumber = 4,
    kSumFieldNumber = 5,
    kZeroCountFieldNumber = 7,
    kScaleFieldNumber = 6,
    kFlagsFieldNumber = 10,
    kMinFieldNumber = 12,
    kMaxFieldNumber = 13,
    kZeroThresholdFieldNumber = 14,
  };
  // repeated .opentelemetry.proto.common.v1.KeyValue attributes = 1;
  [[nodiscard]] int attributes_size()
      const;
  private:
  int _internal_attributes_size() const;

  public:
  void clear_attributes() ;
  [[nodiscard]] const ::opentelemetry::proto::common::v1::KeyValue& attributes(int index) const;
  [[nodiscard]] ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL mutable_attributes(int index);
  ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL add_attributes();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>&
  attributes() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL
  mutable_attributes();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>& _internal_attributes() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL _internal_mutable_attributes();

  public:
  // repeated .opentelemetry.proto.metrics.v1.Exemplar exemplars = 11;
  [[nodiscard]] int exemplars_size()
      const;
  private:
  int _internal_exemplars_size() const;

  public:
  void clear_exemplars() ;
  [[nodiscard]] const ::opentelemetry::proto::metrics::v1::Exemplar& exemplars(int index) const;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::Exemplar* PROTOBUF_NONNULL mutable_exemplars(int index);
  ::opentelemetry::proto::metrics::v1::Exemplar* PROTOBUF_NONNULL add_exemplars();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>&
  exemplars() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>* PROTOBUF_NONNULL
  mutable_exemplars();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>& _internal_exemplars() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>* PROTOBUF_NONNULL _internal_mutable_exemplars();

  public:
  // .opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.Buckets positive = 8;
  [[nodiscard]] bool has_positive()
      const;
  void clear_positive() ;
  [[nodiscard]] const ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets& positive() const;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NULLABLE release_positive();
  ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NONNULL mutable_positive();
  void set_allocated_positive(::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_positive(::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NULLABLE value);
  ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NULLABLE unsafe_arena_release_positive();

  private:
  const ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets& _internal_positive() const;
  ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NONNULL _internal_mutable_positive();

  public:
  // .opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.Buckets negative = 9;
  [[nodiscard]] bool has_negative()
      const;
  void clear_negative() ;
  [[nodiscard]] const ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets& negative() const;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NULLABLE release_negative();
  ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NONNULL mutable_negative();
  void set_allocated_negative(::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_negative(::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NULLABLE value);
  ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NULLABLE unsafe_arena_release_negative();

  private:
  const ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets& _internal_negative() const;
  ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NONNULL _internal_mutable_negative();

  public:
  // fixed64 start_time_unix_nano = 2;
  void clear_start_time_unix_nano() ;
  [[nodiscard]] ::uint64_t start_time_unix_nano() const;
  void set_start_time_unix_nano(::uint64_t value);

  private:
  ::uint64_t _internal_start_time_unix_nano() const;
  void _internal_set_start_time_unix_nano(::uint64_t value);

  public:
  // fixed64 time_unix_nano = 3;
  void clear_time_unix_nano() ;
  [[nodiscard]] ::uint64_t time_unix_nano() const;
  void set_time_unix_nano(::uint64_t value);

  private:
  ::uint64_t _internal_time_unix_nano() const;
  void _internal_set_time_unix_nano(::uint64_t value);

  public:
  // fixed64 count = 4;
  void clear_count() ;
  [[nodiscard]] ::uint64_t count() const;
  void set_count(::uint64_t value);

  private:
  ::uint64_t _internal_count() const;
  void _internal_set_count(::uint64_t value);

  public:
  // optional double sum = 5;
  [[nodiscard]] bool has_sum()
      const;
  void clear_sum() ;
  [[nodiscard]] double sum() const;
  void set_sum(double value);

  private:
  double _internal_sum() const;
  void _internal_set_sum(double value);

  public:
  // fixed64 zero_count = 7;
  void clear_zero_count() ;
  [[nodiscard]] ::uint64_t zero_count() const;
  void set_zero_count(::uint64_t value);

  private:
  ::uint64_t _internal_zero_count() const;
  void _internal_set_zero_count(::uint64_t value);

  public:
  // sint32 scale = 6;
  void clear_scale() ;
  [[nodiscard]] ::int32_t scale() const;
  void set_scale(::int32_t value);

  private:
  ::int32_t _internal_scale() const;
  void _internal_set_scale(::int32_t value);

  public:
  // uint32 flags = 10;
  void clear_flags() ;
  [[nodiscard]] ::uint32_t flags() const;
  void set_flags(::uint32_t value);

  private:
  ::uint32_t _internal_flags() const;
  void _internal_set_flags(::uint32_t value);

  public:
  // optional double min = 12;
  [[nodiscard]] bool has_min()
      const;
  void clear_min() ;
  [[nodiscard]] double min() const;
  void set_min(double value);

  private:
  double _internal_min() const;
  void _internal_set_min(double value);

  public:
  // optional double max = 13;
  [[nodiscard]] bool has_max()
      const;
  void clear_max() ;
  [[nodiscard]] double max() const;
  void set_max(double value);

  private:
  double _internal_max() const;
  void _internal_set_max(double value);

  public:
  // double zero_threshold = 14;
  void clear_zero_threshold() ;
  [[nodiscard]] double zero_threshold() const;
  void set_zero_threshold(double value);

  private:
  double _internal_zero_threshold() const;
  void _internal_set_zero_threshold(double value);

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<4, 14,
                          4, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ExponentialHistogramDataPoint& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::common::v1::KeyValue > attributes_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::metrics::v1::Exemplar > exemplars_;
    ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NULLABLE positive_;
    ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NULLABLE negative_;
    ::uint64_t start_time_unix_nano_;
    ::uint64_t time_unix_nano_;
    ::uint64_t count_;
    double sum_;
    ::uint64_t zero_count_;
    ::int32_t scale_;
    ::uint32_t flags_;
    double min_;
    double max_;
    double zero_threshold_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Sum final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.metrics.v1.Sum) */ {
 public:
  inline Sum() : Sum(nullptr) {}
  ~Sum() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Sum* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Sum));
  }
#endif

  template <typename = void>
  explicit constexpr Sum(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Sum(const Sum& from) : Sum(nullptr, from) {}
  inline Sum(Sum&& from) noexcept : Sum(nullptr, ::std::move(from)) {}
  inline Sum& operator=(const Sum& from) {
    CopyFrom(from);
    return *this;
  }
  inline Sum& operator=(Sum&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Sum& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Sum>(&Sum_globals_);
  }
  static constexpr int kIndexInFileMessages = 5;
  friend void swap(Sum& a, Sum& b) { a.Swap(&b); }
  inline void Swap(Sum* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Sum* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Sum* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Sum>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Sum& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Sum& from) { Sum::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Sum* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.metrics.v1.Sum"; }

  explicit Sum(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Sum(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Sum& from);
  Sum(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Sum&& from) noexcept
      : Sum(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kDataPointsFieldNumber = 1,
    kAggregationTemporalityFieldNumber = 2,
    kIsMonotonicFieldNumber = 3,
  };
  // repeated .opentelemetry.proto.metrics.v1.NumberDataPoint data_points = 1;
  [[nodiscard]] int data_points_size()
      const;
  private:
  int _internal_data_points_size() const;

  public:
  void clear_data_points() ;
  [[nodiscard]] const ::opentelemetry::proto::metrics::v1::NumberDataPoint& data_points(int index) const;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::NumberDataPoint* PROTOBUF_NONNULL mutable_data_points(int index);
  ::opentelemetry::proto::metrics::v1::NumberDataPoint* PROTOBUF_NONNULL add_data_points();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::NumberDataPoint>&
  data_points() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::NumberDataPoint>* PROTOBUF_NONNULL
  mutable_data_points();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::NumberDataPoint>& _internal_data_points() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::NumberDataPoint>* PROTOBUF_NONNULL _internal_mutable_data_points();

  public:
  // .opentelemetry.proto.metrics.v1.AggregationTemporality aggregation_temporality = 2;
  void clear_aggregation_temporality() ;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::AggregationTemporality aggregation_temporality() const;
  void set_aggregation_temporality(::opentelemetry::proto::metrics::v1::AggregationTemporality value);

  private:
  ::opentelemetry::proto::metrics::v1::AggregationTemporality _internal_aggregation_temporality() const;
  void _internal_set_aggregation_temporality(::opentelemetry::proto::metrics::v1::AggregationTemporality value);

  public:
  // bool is_monotonic = 3;
  void clear_is_monotonic() ;
  [[nodiscard]] bool is_monotonic() const;
  void set_is_monotonic(bool value);

  private:
  bool _internal_is_monotonic() const;
  void _internal_set_is_monotonic(bool value);

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.metrics.v1.Sum)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 3,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Sum& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::metrics::v1::NumberDataPoint > data_points_;
    int aggregation_temporality_;
    bool is_monotonic_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Histogram final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.metrics.v1.Histogram) */ {
 public:
  inline Histogram() : Histogram(nullptr) {}
  ~Histogram() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Histogram* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Histogram));
  }
#endif

  template <typename = void>
  explicit constexpr Histogram(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Histogram(const Histogram& from) : Histogram(nullptr, from) {}
  inline Histogram(Histogram&& from) noexcept : Histogram(nullptr, ::std::move(from)) {}
  inline Histogram& operator=(const Histogram& from) {
    CopyFrom(from);
    return *this;
  }
  inline Histogram& operator=(Histogram&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Histogram& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Histogram>(&Histogram_globals_);
  }
  static constexpr int kIndexInFileMessages = 6;
  friend void swap(Histogram& a, Histogram& b) { a.Swap(&b); }
  inline void Swap(Histogram* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Histogram* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Histogram* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Histogram>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Histogram& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Histogram& from) { Histogram::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Histogram* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.metrics.v1.Histogram"; }

  explicit Histogram(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Histogram(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Histogram& from);
  Histogram(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Histogram&& from) noexcept
      : Histogram(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kDataPointsFieldNumber = 1,
    kAggregationTemporalityFieldNumber = 2,
  };
  // repeated .opentelemetry.proto.metrics.v1.HistogramDataPoint data_points = 1;
  [[nodiscard]] int data_points_size()
      const;
  private:
  int _internal_data_points_size() const;

  public:
  void clear_data_points() ;
  [[nodiscard]] const ::opentelemetry::proto::metrics::v1::HistogramDataPoint& data_points(int index) const;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::HistogramDataPoint* PROTOBUF_NONNULL mutable_data_points(int index);
  ::opentelemetry::proto::metrics::v1::HistogramDataPoint* PROTOBUF_NONNULL add_data_points();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::HistogramDataPoint>&
  data_points() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::HistogramDataPoint>* PROTOBUF_NONNULL
  mutable_data_points();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::HistogramDataPoint>& _internal_data_points() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::HistogramDataPoint>* PROTOBUF_NONNULL _internal_mutable_data_points();

  public:
  // .opentelemetry.proto.metrics.v1.AggregationTemporality aggregation_temporality = 2;
  void clear_aggregation_temporality() ;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::AggregationTemporality aggregation_temporality() const;
  void set_aggregation_temporality(::opentelemetry::proto::metrics::v1::AggregationTemporality value);

  private:
  ::opentelemetry::proto::metrics::v1::AggregationTemporality _internal_aggregation_temporality() const;
  void _internal_set_aggregation_temporality(::opentelemetry::proto::metrics::v1::AggregationTemporality value);

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.metrics.v1.Histogram)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Histogram& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::metrics::v1::HistogramDataPoint > data_points_;
    int aggregation_temporality_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Gauge final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.metrics.v1.Gauge) */ {
 public:
  inline Gauge() : Gauge(nullptr) {}
  ~Gauge() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Gauge* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Gauge));
  }
#endif

  template <typename = void>
  explicit constexpr Gauge(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Gauge(const Gauge& from) : Gauge(nullptr, from) {}
  inline Gauge(Gauge&& from) noexcept : Gauge(nullptr, ::std::move(from)) {}
  inline Gauge& operator=(const Gauge& from) {
    CopyFrom(from);
    return *this;
  }
  inline Gauge& operator=(Gauge&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Gauge& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Gauge>(&Gauge_globals_);
  }
  static constexpr int kIndexInFileMessages = 4;
  friend void swap(Gauge& a, Gauge& b) { a.Swap(&b); }
  inline void Swap(Gauge* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Gauge* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Gauge* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Gauge>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Gauge& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Gauge& from) { Gauge::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Gauge* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.metrics.v1.Gauge"; }

  explicit Gauge(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Gauge(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Gauge& from);
  Gauge(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Gauge&& from) noexcept
      : Gauge(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kDataPointsFieldNumber = 1,
  };
  // repeated .opentelemetry.proto.metrics.v1.NumberDataPoint data_points = 1;
  [[nodiscard]] int data_points_size()
      const;
  private:
  int _internal_data_points_size() const;

  public:
  void clear_data_points() ;
  [[nodiscard]] const ::opentelemetry::proto::metrics::v1::NumberDataPoint& data_points(int index) const;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::NumberDataPoint* PROTOBUF_NONNULL mutable_data_points(int index);
  ::opentelemetry::proto::metrics::v1::NumberDataPoint* PROTOBUF_NONNULL add_data_points();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::NumberDataPoint>&
  data_points() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::NumberDataPoint>* PROTOBUF_NONNULL
  mutable_data_points();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::NumberDataPoint>& _internal_data_points() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::NumberDataPoint>* PROTOBUF_NONNULL _internal_mutable_data_points();

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.metrics.v1.Gauge)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Gauge& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::metrics::v1::NumberDataPoint > data_points_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ExponentialHistogram final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.metrics.v1.ExponentialHistogram) */ {
 public:
  inline ExponentialHistogram() : ExponentialHistogram(nullptr) {}
  ~ExponentialHistogram() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ExponentialHistogram* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ExponentialHistogram));
  }
#endif

  template <typename = void>
  explicit constexpr ExponentialHistogram(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ExponentialHistogram(const ExponentialHistogram& from) : ExponentialHistogram(nullptr, from) {}
  inline ExponentialHistogram(ExponentialHistogram&& from) noexcept : ExponentialHistogram(nullptr, ::std::move(from)) {}
  inline ExponentialHistogram& operator=(const ExponentialHistogram& from) {
    CopyFrom(from);
    return *this;
  }
  inline ExponentialHistogram& operator=(ExponentialHistogram&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ExponentialHistogram& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ExponentialHistogram>(&ExponentialHistogram_globals_);
  }
  static constexpr int kIndexInFileMessages = 7;
  friend void swap(ExponentialHistogram& a, ExponentialHistogram& b) { a.Swap(&b); }
  inline void Swap(ExponentialHistogram* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ExponentialHistogram* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ExponentialHistogram* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ExponentialHistogram>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ExponentialHistogram& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ExponentialHistogram& from) { ExponentialHistogram::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ExponentialHistogram* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.metrics.v1.ExponentialHistogram"; }

  explicit ExponentialHistogram(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ExponentialHistogram(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ExponentialHistogram& from);
  ExponentialHistogram(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ExponentialHistogram&& from) noexcept
      : ExponentialHistogram(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kDataPointsFieldNumber = 1,
    kAggregationTemporalityFieldNumber = 2,
  };
  // repeated .opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint data_points = 1;
  [[nodiscard]] int data_points_size()
      const;
  private:
  int _internal_data_points_size() const;

  public:
  void clear_data_points() ;
  [[nodiscard]] const ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint& data_points(int index) const;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint* PROTOBUF_NONNULL mutable_data_points(int index);
  ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint* PROTOBUF_NONNULL add_data_points();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint>&
  data_points() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint>* PROTOBUF_NONNULL
  mutable_data_points();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint>& _internal_data_points() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint>* PROTOBUF_NONNULL _internal_mutable_data_points();

  public:
  // .opentelemetry.proto.metrics.v1.AggregationTemporality aggregation_temporality = 2;
  void clear_aggregation_temporality() ;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::AggregationTemporality aggregation_temporality() const;
  void set_aggregation_temporality(::opentelemetry::proto::metrics::v1::AggregationTemporality value);

  private:
  ::opentelemetry::proto::metrics::v1::AggregationTemporality _internal_aggregation_temporality() const;
  void _internal_set_aggregation_temporality(::opentelemetry::proto::metrics::v1::AggregationTemporality value);

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.metrics.v1.ExponentialHistogram)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ExponentialHistogram& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint > data_points_;
    int aggregation_temporality_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Metric final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.metrics.v1.Metric) */ {
 public:
  inline Metric() : Metric(nullptr) {}
  ~Metric() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Metric* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Metric));
  }
#endif

  template <typename = void>
  explicit constexpr Metric(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Metric(const Metric& from) : Metric(nullptr, from) {}
  inline Metric(Metric&& from) noexcept : Metric(nullptr, ::std::move(from)) {}
  inline Metric& operator=(const Metric& from) {
    CopyFrom(from);
    return *this;
  }
  inline Metric& operator=(Metric&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Metric& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Metric>(&Metric_globals_);
  }
  enum DataCase {
    kGauge = 5,
    kSum = 7,
    kHistogram = 9,
    kExponentialHistogram = 10,
    kSummary = 11,
    DATA_NOT_SET = 0,
  };
  static constexpr int kIndexInFileMessages = 3;
  friend void swap(Metric& a, Metric& b) { a.Swap(&b); }
  inline void Swap(Metric* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Metric* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Metric* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Metric>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Metric& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Metric& from) { Metric::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Metric* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.metrics.v1.Metric"; }

  explicit Metric(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Metric(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Metric& from);
  Metric(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Metric&& from) noexcept
      : Metric(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kNameFieldNumber = 1,
    kDescriptionFieldNumber = 2,
    kUnitFieldNumber = 3,
    kGaugeFieldNumber = 5,
    kSumFieldNumber = 7,
    kHistogramFieldNumber = 9,
    kExponentialHistogramFieldNumber = 10,
    kSummaryFieldNumber = 11,
  };
  // string name = 1;
  void clear_name() ;
  [[nodiscard]] const ::std::string& name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_name();
  void set_allocated_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_name();

  public:
  // string description = 2;
  void clear_description() ;
  [[nodiscard]] const ::std::string& description() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_description(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_description();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_description();
  void set_allocated_description(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_description() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_description(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_description();

  public:
  // string unit = 3;
  void clear_unit() ;
  [[nodiscard]] const ::std::string& unit() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_unit(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_unit();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_unit();
  void set_allocated_unit(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_unit() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_unit(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_unit();

  public:
  // .opentelemetry.proto.metrics.v1.Gauge gauge = 5;
  [[nodiscard]] bool has_gauge()
      const;
  private:
  bool _internal_has_gauge() const;

  public:
  void clear_gauge() ;
  [[nodiscard]] const ::opentelemetry::proto::metrics::v1::Gauge& gauge() const;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::Gauge* PROTOBUF_NULLABLE release_gauge();
  ::opentelemetry::proto::metrics::v1::Gauge* PROTOBUF_NONNULL mutable_gauge();
  void set_allocated_gauge(::opentelemetry::proto::metrics::v1::Gauge* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_gauge(::opentelemetry::proto::metrics::v1::Gauge* PROTOBUF_NULLABLE value);
  ::opentelemetry::proto::metrics::v1::Gauge* PROTOBUF_NULLABLE unsafe_arena_release_gauge();

  private:
  const ::opentelemetry::proto::metrics::v1::Gauge& _internal_gauge() const;
  ::opentelemetry::proto::metrics::v1::Gauge* PROTOBUF_NONNULL _internal_mutable_gauge();

  public:
  // .opentelemetry.proto.metrics.v1.Sum sum = 7;
  [[nodiscard]] bool has_sum()
      const;
  private:
  bool _internal_has_sum() const;

  public:
  void clear_sum() ;
  [[nodiscard]] const ::opentelemetry::proto::metrics::v1::Sum& sum() const;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::Sum* PROTOBUF_NULLABLE release_sum();
  ::opentelemetry::proto::metrics::v1::Sum* PROTOBUF_NONNULL mutable_sum();
  void set_allocated_sum(::opentelemetry::proto::metrics::v1::Sum* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_sum(::opentelemetry::proto::metrics::v1::Sum* PROTOBUF_NULLABLE value);
  ::opentelemetry::proto::metrics::v1::Sum* PROTOBUF_NULLABLE unsafe_arena_release_sum();

  private:
  const ::opentelemetry::proto::metrics::v1::Sum& _internal_sum() const;
  ::opentelemetry::proto::metrics::v1::Sum* PROTOBUF_NONNULL _internal_mutable_sum();

  public:
  // .opentelemetry.proto.metrics.v1.Histogram histogram = 9;
  [[nodiscard]] bool has_histogram()
      const;
  private:
  bool _internal_has_histogram() const;

  public:
  void clear_histogram() ;
  [[nodiscard]] const ::opentelemetry::proto::metrics::v1::Histogram& histogram() const;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::Histogram* PROTOBUF_NULLABLE release_histogram();
  ::opentelemetry::proto::metrics::v1::Histogram* PROTOBUF_NONNULL mutable_histogram();
  void set_allocated_histogram(::opentelemetry::proto::metrics::v1::Histogram* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_histogram(::opentelemetry::proto::metrics::v1::Histogram* PROTOBUF_NULLABLE value);
  ::opentelemetry::proto::metrics::v1::Histogram* PROTOBUF_NULLABLE unsafe_arena_release_histogram();

  private:
  const ::opentelemetry::proto::metrics::v1::Histogram& _internal_histogram() const;
  ::opentelemetry::proto::metrics::v1::Histogram* PROTOBUF_NONNULL _internal_mutable_histogram();

  public:
  // .opentelemetry.proto.metrics.v1.ExponentialHistogram exponential_histogram = 10;
  [[nodiscard]] bool has_exponential_histogram()
      const;
  private:
  bool _internal_has_exponential_histogram() const;

  public:
  void clear_exponential_histogram() ;
  [[nodiscard]] const ::opentelemetry::proto::metrics::v1::ExponentialHistogram& exponential_histogram() const;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::ExponentialHistogram* PROTOBUF_NULLABLE release_exponential_histogram();
  ::opentelemetry::proto::metrics::v1::ExponentialHistogram* PROTOBUF_NONNULL mutable_exponential_histogram();
  void set_allocated_exponential_histogram(::opentelemetry::proto::metrics::v1::ExponentialHistogram* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exponential_histogram(::opentelemetry::proto::metrics::v1::ExponentialHistogram* PROTOBUF_NULLABLE value);
  ::opentelemetry::proto::metrics::v1::ExponentialHistogram* PROTOBUF_NULLABLE unsafe_arena_release_exponential_histogram();

  private:
  const ::opentelemetry::proto::metrics::v1::ExponentialHistogram& _internal_exponential_histogram() const;
  ::opentelemetry::proto::metrics::v1::ExponentialHistogram* PROTOBUF_NONNULL _internal_mutable_exponential_histogram();

  public:
  // .opentelemetry.proto.metrics.v1.Summary summary = 11;
  [[nodiscard]] bool has_summary()
      const;
  private:
  bool _internal_has_summary() const;

  public:
  void clear_summary() ;
  [[nodiscard]] const ::opentelemetry::proto::metrics::v1::Summary& summary() const;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::Summary* PROTOBUF_NULLABLE release_summary();
  ::opentelemetry::proto::metrics::v1::Summary* PROTOBUF_NONNULL mutable_summary();
  void set_allocated_summary(::opentelemetry::proto::metrics::v1::Summary* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_summary(::opentelemetry::proto::metrics::v1::Summary* PROTOBUF_NULLABLE value);
  ::opentelemetry::proto::metrics::v1::Summary* PROTOBUF_NULLABLE unsafe_arena_release_summary();

  private:
  const ::opentelemetry::proto::metrics::v1::Summary& _internal_summary() const;
  ::opentelemetry::proto::metrics::v1::Summary* PROTOBUF_NONNULL _internal_mutable_summary();

  public:
  void clear_data();
  DataCase data_case() const;
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.metrics.v1.Metric)
 private:
  class _Internal;
  void set_has_gauge();
  void set_has_sum();
  void set_has_histogram();
  void set_has_exponential_histogram();
  void set_has_summary();
  [[nodiscard]] inline bool has_data() const;
  inline void clear_has_data();
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 8,
                          5, 73,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Metric& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr name_;
    ::google::protobuf::internal::ArenaStringPtr description_;
    ::google::protobuf::internal::ArenaStringPtr unit_;
    union DataUnion {
      constexpr DataUnion() : _constinit_{} {}
      ::google::protobuf::internal::ConstantInitialized _constinit_;
      ::google::protobuf::Message* PROTOBUF_NULLABLE gauge_;
      ::google::protobuf::Message* PROTOBUF_NULLABLE sum_;
      ::google::protobuf::Message* PROTOBUF_NULLABLE histogram_;
      ::google::protobuf::Message* PROTOBUF_NULLABLE exponential_histogram_;
      ::google::protobuf::Message* PROTOBUF_NULLABLE summary_;
    } data_;
    ::uint32_t _oneof_case_[1];
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ScopeMetrics final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.metrics.v1.ScopeMetrics) */ {
 public:
  inline ScopeMetrics() : ScopeMetrics(nullptr) {}
  ~ScopeMetrics() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ScopeMetrics* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ScopeMetrics));
  }
#endif

  template <typename = void>
  explicit constexpr ScopeMetrics(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ScopeMetrics(const ScopeMetrics& from) : ScopeMetrics(nullptr, from) {}
  inline ScopeMetrics(ScopeMetrics&& from) noexcept : ScopeMetrics(nullptr, ::std::move(from)) {}
  inline ScopeMetrics& operator=(const ScopeMetrics& from) {
    CopyFrom(from);
    return *this;
  }
  inline ScopeMetrics& operator=(ScopeMetrics&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ScopeMetrics& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ScopeMetrics>(&ScopeMetrics_globals_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(ScopeMetrics& a, ScopeMetrics& b) { a.Swap(&b); }
  inline void Swap(ScopeMetrics* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ScopeMetrics* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ScopeMetrics* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ScopeMetrics>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ScopeMetrics& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ScopeMetrics& from) { ScopeMetrics::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ScopeMetrics* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.metrics.v1.ScopeMetrics"; }

  explicit ScopeMetrics(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ScopeMetrics(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ScopeMetrics& from);
  ScopeMetrics(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ScopeMetrics&& from) noexcept
      : ScopeMetrics(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kMetricsFieldNumber = 2,
    kSchemaUrlFieldNumber = 3,
    kScopeFieldNumber = 1,
  };
  // repeated .opentelemetry.proto.metrics.v1.Metric metrics = 2;
  [[nodiscard]] int metrics_size()
      const;
  private:
  int _internal_metrics_size() const;

  public:
  void clear_metrics() ;
  [[nodiscard]] const ::opentelemetry::proto::metrics::v1::Metric& metrics(int index) const;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::Metric* PROTOBUF_NONNULL mutable_metrics(int index);
  ::opentelemetry::proto::metrics::v1::Metric* PROTOBUF_NONNULL add_metrics();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Metric>&
  metrics() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Metric>* PROTOBUF_NONNULL
  mutable_metrics();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Metric>& _internal_metrics() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Metric>* PROTOBUF_NONNULL _internal_mutable_metrics();

  public:
  // string schema_url = 3;
  void clear_schema_url() ;
  [[nodiscard]] const ::std::string& schema_url() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_schema_url(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_schema_url();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_schema_url();
  void set_allocated_schema_url(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_schema_url() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_schema_url(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_schema_url();

  public:
  // .opentelemetry.proto.common.v1.InstrumentationScope scope = 1;
  [[nodiscard]] bool has_scope()
      const;
  void clear_scope() ;
  [[nodiscard]] const ::opentelemetry::proto::common::v1::InstrumentationScope& scope() const;
  [[nodiscard]] ::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NULLABLE release_scope();
  ::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NONNULL mutable_scope();
  void set_allocated_scope(::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_scope(::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NULLABLE value);
  ::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NULLABLE unsafe_arena_release_scope();

  private:
  const ::opentelemetry::proto::common::v1::InstrumentationScope& _internal_scope() const;
  ::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NONNULL _internal_mutable_scope();

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.metrics.v1.ScopeMetrics)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 3,
                          2, 62,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ScopeMetrics& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::metrics::v1::Metric > metrics_;
    ::google::protobuf::internal::ArenaStringPtr schema_url_;
    ::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NULLABLE scope_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ResourceMetrics final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.metrics.v1.ResourceMetrics) */ {
 public:
  inline ResourceMetrics() : ResourceMetrics(nullptr) {}
  ~ResourceMetrics() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ResourceMetrics* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ResourceMetrics));
  }
#endif

  template <typename = void>
  explicit constexpr ResourceMetrics(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ResourceMetrics(const ResourceMetrics& from) : ResourceMetrics(nullptr, from) {}
  inline ResourceMetrics(ResourceMetrics&& from) noexcept : ResourceMetrics(nullptr, ::std::move(from)) {}
  inline ResourceMetrics& operator=(const ResourceMetrics& from) {
    CopyFrom(from);
    return *this;
  }
  inline ResourceMetrics& operator=(ResourceMetrics&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ResourceMetrics& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ResourceMetrics>(&ResourceMetrics_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(ResourceMetrics& a, ResourceMetrics& b) { a.Swap(&b); }
  inline void Swap(ResourceMetrics* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ResourceMetrics* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ResourceMetrics* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ResourceMetrics>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ResourceMetrics& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ResourceMetrics& from) { ResourceMetrics::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ResourceMetrics* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.metrics.v1.ResourceMetrics"; }

  explicit ResourceMetrics(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ResourceMetrics(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ResourceMetrics& from);
  ResourceMetrics(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ResourceMetrics&& from) noexcept
      : ResourceMetrics(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kScopeMetricsFieldNumber = 2,
    kSchemaUrlFieldNumber = 3,
    kResourceFieldNumber = 1,
  };
  // repeated .opentelemetry.proto.metrics.v1.ScopeMetrics scope_metrics = 2;
  [[nodiscard]] int scope_metrics_size()
      const;
  private:
  int _internal_scope_metrics_size() const;

  public:
  void clear_scope_metrics() ;
  [[nodiscard]] const ::opentelemetry::proto::metrics::v1::ScopeMetrics& scope_metrics(int index) const;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::ScopeMetrics* PROTOBUF_NONNULL mutable_scope_metrics(int index);
  ::opentelemetry::proto::metrics::v1::ScopeMetrics* PROTOBUF_NONNULL add_scope_metrics();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ScopeMetrics>&
  scope_metrics() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ScopeMetrics>* PROTOBUF_NONNULL
  mutable_scope_metrics();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ScopeMetrics>& _internal_scope_metrics() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ScopeMetrics>* PROTOBUF_NONNULL _internal_mutable_scope_metrics();

  public:
  // string schema_url = 3;
  void clear_schema_url() ;
  [[nodiscard]] const ::std::string& schema_url() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_schema_url(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_schema_url();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_schema_url();
  void set_allocated_schema_url(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_schema_url() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_schema_url(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_schema_url();

  public:
  // .opentelemetry.proto.resource.v1.Resource resource = 1;
  [[nodiscard]] bool has_resource()
      const;
  void clear_resource() ;
  [[nodiscard]] const ::opentelemetry::proto::resource::v1::Resource& resource() const;
  [[nodiscard]] ::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NULLABLE release_resource();
  ::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NONNULL mutable_resource();
  void set_allocated_resource(::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_resource(::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NULLABLE value);
  ::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NULLABLE unsafe_arena_release_resource();

  private:
  const ::opentelemetry::proto::resource::v1::Resource& _internal_resource() const;
  ::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NONNULL _internal_mutable_resource();

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.metrics.v1.ResourceMetrics)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 3,
                          2, 65,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ResourceMetrics& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::metrics::v1::ScopeMetrics > scope_metrics_;
    ::google::protobuf::internal::ArenaStringPtr schema_url_;
    ::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NULLABLE resource_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED MetricsData final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:opentelemetry.proto.metrics.v1.MetricsData) */ {
 public:
  inline MetricsData() : MetricsData(nullptr) {}
  ~MetricsData() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(MetricsData* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(MetricsData));
  }
#endif

  template <typename = void>
  explicit constexpr MetricsData(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline MetricsData(const MetricsData& from) : MetricsData(nullptr, from) {}
  inline MetricsData(MetricsData&& from) noexcept : MetricsData(nullptr, ::std::move(from)) {}
  inline MetricsData& operator=(const MetricsData& from) {
    CopyFrom(from);
    return *this;
  }
  inline MetricsData& operator=(MetricsData&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const MetricsData& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<MetricsData>(&MetricsData_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(MetricsData& a, MetricsData& b) { a.Swap(&b); }
  inline void Swap(MetricsData* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(MetricsData* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] MetricsData* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<MetricsData>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const MetricsData& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const MetricsData& from) { MetricsData::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(MetricsData* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "opentelemetry.proto.metrics.v1.MetricsData"; }

  explicit MetricsData(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  MetricsData(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const MetricsData& from);
  MetricsData(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, MetricsData&& from) noexcept
      : MetricsData(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kResourceMetricsFieldNumber = 1,
  };
  // repeated .opentelemetry.proto.metrics.v1.ResourceMetrics resource_metrics = 1;
  [[nodiscard]] int resource_metrics_size()
      const;
  private:
  int _internal_resource_metrics_size() const;

  public:
  void clear_resource_metrics() ;
  [[nodiscard]] const ::opentelemetry::proto::metrics::v1::ResourceMetrics& resource_metrics(int index) const;
  [[nodiscard]] ::opentelemetry::proto::metrics::v1::ResourceMetrics* PROTOBUF_NONNULL mutable_resource_metrics(int index);
  ::opentelemetry::proto::metrics::v1::ResourceMetrics* PROTOBUF_NONNULL add_resource_metrics();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ResourceMetrics>&
  resource_metrics() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ResourceMetrics>* PROTOBUF_NONNULL
  mutable_resource_metrics();

  private:
  const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ResourceMetrics>& _internal_resource_metrics() const;
  ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ResourceMetrics>* PROTOBUF_NONNULL _internal_mutable_resource_metrics();

  public:
  // @@protoc_insertion_point(class_scope:opentelemetry.proto.metrics.v1.MetricsData)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const MetricsData& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::opentelemetry::proto::metrics::v1::ResourceMetrics > resource_metrics_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// MetricsData

// repeated .opentelemetry.proto.metrics.v1.ResourceMetrics resource_metrics = 1;
inline int MetricsData::_internal_resource_metrics_size() const {
  return _internal_resource_metrics().size();
}
inline int MetricsData::resource_metrics_size() const {
  return _internal_resource_metrics_size();
}
inline void MetricsData::clear_resource_metrics() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.resource_metrics_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::opentelemetry::proto::metrics::v1::ResourceMetrics& MetricsData::resource_metrics(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.MetricsData.resource_metrics)
  return _internal_resource_metrics().Get(index);
}
inline ::opentelemetry::proto::metrics::v1::ResourceMetrics* PROTOBUF_NONNULL MetricsData::mutable_resource_metrics(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.MetricsData.resource_metrics)
  return _internal_mutable_resource_metrics()->Mutable(index);
}
inline ::opentelemetry::proto::metrics::v1::ResourceMetrics* PROTOBUF_NONNULL MetricsData::add_resource_metrics()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::metrics::v1::ResourceMetrics* _add =
      _internal_mutable_resource_metrics()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.metrics.v1.MetricsData.resource_metrics)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ResourceMetrics>& MetricsData::resource_metrics() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.metrics.v1.MetricsData.resource_metrics)
  return _internal_resource_metrics();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ResourceMetrics>* PROTOBUF_NONNULL
MetricsData::mutable_resource_metrics() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.metrics.v1.MetricsData.resource_metrics)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_resource_metrics();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ResourceMetrics>&
MetricsData::_internal_resource_metrics() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.resource_metrics_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ResourceMetrics>* PROTOBUF_NONNULL
MetricsData::_internal_mutable_resource_metrics() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.resource_metrics_;
}

// -------------------------------------------------------------------

// ResourceMetrics

// .opentelemetry.proto.resource.v1.Resource resource = 1;
inline bool ResourceMetrics::has_resource() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.resource_ != nullptr);
  return value;
}
inline const ::opentelemetry::proto::resource::v1::Resource& ResourceMetrics::_internal_resource() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::opentelemetry::proto::resource::v1::Resource* p = _impl_.resource_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::opentelemetry::proto::resource::v1::Resource>(&::opentelemetry::proto::resource::v1::Resource_globals_);
}
inline const ::opentelemetry::proto::resource::v1::Resource& ResourceMetrics::resource() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ResourceMetrics.resource)
  return _internal_resource();
}
inline void ResourceMetrics::unsafe_arena_set_allocated_resource(
    ::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.resource_);
  }
  _impl_.resource_ = reinterpret_cast<::opentelemetry::proto::resource::v1::Resource*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:opentelemetry.proto.metrics.v1.ResourceMetrics.resource)
}
inline ::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NULLABLE ResourceMetrics::release_resource() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::opentelemetry::proto::resource::v1::Resource* released = _impl_.resource_;
  _impl_.resource_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NULLABLE ResourceMetrics::unsafe_arena_release_resource() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.metrics.v1.ResourceMetrics.resource)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::opentelemetry::proto::resource::v1::Resource* temp = _impl_.resource_;
  _impl_.resource_ = nullptr;
  return temp;
}
inline ::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NONNULL ResourceMetrics::_internal_mutable_resource() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.resource_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::opentelemetry::proto::resource::v1::Resource>(GetArena());
    _impl_.resource_ = reinterpret_cast<::opentelemetry::proto::resource::v1::Resource*>(p);
  }
  return _impl_.resource_;
}
inline ::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NONNULL ResourceMetrics::mutable_resource()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::opentelemetry::proto::resource::v1::Resource* _msg = _internal_mutable_resource();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.ResourceMetrics.resource)
  return _msg;
}
inline void ResourceMetrics::set_allocated_resource(::opentelemetry::proto::resource::v1::Resource* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.resource_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.resource_ = reinterpret_cast<::opentelemetry::proto::resource::v1::Resource*>(value);
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.metrics.v1.ResourceMetrics.resource)
}

// repeated .opentelemetry.proto.metrics.v1.ScopeMetrics scope_metrics = 2;
inline int ResourceMetrics::_internal_scope_metrics_size() const {
  return _internal_scope_metrics().size();
}
inline int ResourceMetrics::scope_metrics_size() const {
  return _internal_scope_metrics_size();
}
inline void ResourceMetrics::clear_scope_metrics() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.scope_metrics_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::opentelemetry::proto::metrics::v1::ScopeMetrics& ResourceMetrics::scope_metrics(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ResourceMetrics.scope_metrics)
  return _internal_scope_metrics().Get(index);
}
inline ::opentelemetry::proto::metrics::v1::ScopeMetrics* PROTOBUF_NONNULL ResourceMetrics::mutable_scope_metrics(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.ResourceMetrics.scope_metrics)
  return _internal_mutable_scope_metrics()->Mutable(index);
}
inline ::opentelemetry::proto::metrics::v1::ScopeMetrics* PROTOBUF_NONNULL ResourceMetrics::add_scope_metrics()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::metrics::v1::ScopeMetrics* _add =
      _internal_mutable_scope_metrics()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.metrics.v1.ResourceMetrics.scope_metrics)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ScopeMetrics>& ResourceMetrics::scope_metrics() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.metrics.v1.ResourceMetrics.scope_metrics)
  return _internal_scope_metrics();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ScopeMetrics>* PROTOBUF_NONNULL
ResourceMetrics::mutable_scope_metrics() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.metrics.v1.ResourceMetrics.scope_metrics)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_scope_metrics();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ScopeMetrics>&
ResourceMetrics::_internal_scope_metrics() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.scope_metrics_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ScopeMetrics>* PROTOBUF_NONNULL
ResourceMetrics::_internal_mutable_scope_metrics() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.scope_metrics_;
}

// string schema_url = 3;
inline void ResourceMetrics::clear_schema_url() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.schema_url_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& ResourceMetrics::schema_url() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ResourceMetrics.schema_url)
  return _internal_schema_url();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ResourceMetrics::set_schema_url(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.schema_url_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.ResourceMetrics.schema_url)
}
inline ::std::string* PROTOBUF_NONNULL ResourceMetrics::mutable_schema_url()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_schema_url();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.ResourceMetrics.schema_url)
  return _s;
}
inline const ::std::string& ResourceMetrics::_internal_schema_url() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.schema_url_.Get();
}
inline void ResourceMetrics::_internal_set_schema_url(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.schema_url_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ResourceMetrics::_internal_mutable_schema_url() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.schema_url_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ResourceMetrics::release_schema_url() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.metrics.v1.ResourceMetrics.schema_url)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.schema_url_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.schema_url_.Set("", GetArena());
  }
  return released;
}
inline void ResourceMetrics::set_allocated_schema_url(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.schema_url_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.schema_url_.IsDefault()) {
    _impl_.schema_url_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.metrics.v1.ResourceMetrics.schema_url)
}

// -------------------------------------------------------------------

// ScopeMetrics

// .opentelemetry.proto.common.v1.InstrumentationScope scope = 1;
inline bool ScopeMetrics::has_scope() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.scope_ != nullptr);
  return value;
}
inline const ::opentelemetry::proto::common::v1::InstrumentationScope& ScopeMetrics::_internal_scope() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::opentelemetry::proto::common::v1::InstrumentationScope* p = _impl_.scope_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::opentelemetry::proto::common::v1::InstrumentationScope>(&::opentelemetry::proto::common::v1::InstrumentationScope_globals_);
}
inline const ::opentelemetry::proto::common::v1::InstrumentationScope& ScopeMetrics::scope() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ScopeMetrics.scope)
  return _internal_scope();
}
inline void ScopeMetrics::unsafe_arena_set_allocated_scope(
    ::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.scope_);
  }
  _impl_.scope_ = reinterpret_cast<::opentelemetry::proto::common::v1::InstrumentationScope*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:opentelemetry.proto.metrics.v1.ScopeMetrics.scope)
}
inline ::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NULLABLE ScopeMetrics::release_scope() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::opentelemetry::proto::common::v1::InstrumentationScope* released = _impl_.scope_;
  _impl_.scope_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NULLABLE ScopeMetrics::unsafe_arena_release_scope() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.metrics.v1.ScopeMetrics.scope)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::opentelemetry::proto::common::v1::InstrumentationScope* temp = _impl_.scope_;
  _impl_.scope_ = nullptr;
  return temp;
}
inline ::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NONNULL ScopeMetrics::_internal_mutable_scope() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.scope_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::opentelemetry::proto::common::v1::InstrumentationScope>(GetArena());
    _impl_.scope_ = reinterpret_cast<::opentelemetry::proto::common::v1::InstrumentationScope*>(p);
  }
  return _impl_.scope_;
}
inline ::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NONNULL ScopeMetrics::mutable_scope()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::opentelemetry::proto::common::v1::InstrumentationScope* _msg = _internal_mutable_scope();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.ScopeMetrics.scope)
  return _msg;
}
inline void ScopeMetrics::set_allocated_scope(::opentelemetry::proto::common::v1::InstrumentationScope* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.scope_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.scope_ = reinterpret_cast<::opentelemetry::proto::common::v1::InstrumentationScope*>(value);
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.metrics.v1.ScopeMetrics.scope)
}

// repeated .opentelemetry.proto.metrics.v1.Metric metrics = 2;
inline int ScopeMetrics::_internal_metrics_size() const {
  return _internal_metrics().size();
}
inline int ScopeMetrics::metrics_size() const {
  return _internal_metrics_size();
}
inline void ScopeMetrics::clear_metrics() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.metrics_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::opentelemetry::proto::metrics::v1::Metric& ScopeMetrics::metrics(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ScopeMetrics.metrics)
  return _internal_metrics().Get(index);
}
inline ::opentelemetry::proto::metrics::v1::Metric* PROTOBUF_NONNULL ScopeMetrics::mutable_metrics(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.ScopeMetrics.metrics)
  return _internal_mutable_metrics()->Mutable(index);
}
inline ::opentelemetry::proto::metrics::v1::Metric* PROTOBUF_NONNULL ScopeMetrics::add_metrics()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::metrics::v1::Metric* _add =
      _internal_mutable_metrics()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.metrics.v1.ScopeMetrics.metrics)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Metric>& ScopeMetrics::metrics() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.metrics.v1.ScopeMetrics.metrics)
  return _internal_metrics();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Metric>* PROTOBUF_NONNULL
ScopeMetrics::mutable_metrics() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.metrics.v1.ScopeMetrics.metrics)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_metrics();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Metric>&
ScopeMetrics::_internal_metrics() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.metrics_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Metric>* PROTOBUF_NONNULL
ScopeMetrics::_internal_mutable_metrics() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.metrics_;
}

// string schema_url = 3;
inline void ScopeMetrics::clear_schema_url() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.schema_url_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& ScopeMetrics::schema_url() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ScopeMetrics.schema_url)
  return _internal_schema_url();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ScopeMetrics::set_schema_url(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.schema_url_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.ScopeMetrics.schema_url)
}
inline ::std::string* PROTOBUF_NONNULL ScopeMetrics::mutable_schema_url()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_schema_url();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.ScopeMetrics.schema_url)
  return _s;
}
inline const ::std::string& ScopeMetrics::_internal_schema_url() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.schema_url_.Get();
}
inline void ScopeMetrics::_internal_set_schema_url(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.schema_url_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ScopeMetrics::_internal_mutable_schema_url() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.schema_url_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ScopeMetrics::release_schema_url() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.metrics.v1.ScopeMetrics.schema_url)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.schema_url_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.schema_url_.Set("", GetArena());
  }
  return released;
}
inline void ScopeMetrics::set_allocated_schema_url(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.schema_url_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.schema_url_.IsDefault()) {
    _impl_.schema_url_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.metrics.v1.ScopeMetrics.schema_url)
}

// -------------------------------------------------------------------

// Metric

// string name = 1;
inline void Metric::clear_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& Metric::name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.Metric.name)
  return _internal_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Metric::set_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.Metric.name)
}
inline ::std::string* PROTOBUF_NONNULL Metric::mutable_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.Metric.name)
  return _s;
}
inline const ::std::string& Metric::_internal_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.name_.Get();
}
inline void Metric::_internal_set_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Metric::_internal_mutable_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Metric::release_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.metrics.v1.Metric.name)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.name_.Set("", GetArena());
  }
  return released;
}
inline void Metric::set_allocated_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.name_.IsDefault()) {
    _impl_.name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.metrics.v1.Metric.name)
}

// string description = 2;
inline void Metric::clear_description() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.description_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& Metric::description() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.Metric.description)
  return _internal_description();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Metric::set_description(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.description_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.Metric.description)
}
inline ::std::string* PROTOBUF_NONNULL Metric::mutable_description()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_description();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.Metric.description)
  return _s;
}
inline const ::std::string& Metric::_internal_description() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.description_.Get();
}
inline void Metric::_internal_set_description(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.description_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Metric::_internal_mutable_description() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.description_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Metric::release_description() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.metrics.v1.Metric.description)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.description_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.description_.Set("", GetArena());
  }
  return released;
}
inline void Metric::set_allocated_description(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.description_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.description_.IsDefault()) {
    _impl_.description_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.metrics.v1.Metric.description)
}

// string unit = 3;
inline void Metric::clear_unit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.unit_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::std::string& Metric::unit() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.Metric.unit)
  return _internal_unit();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Metric::set_unit(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.unit_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.Metric.unit)
}
inline ::std::string* PROTOBUF_NONNULL Metric::mutable_unit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_unit();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.Metric.unit)
  return _s;
}
inline const ::std::string& Metric::_internal_unit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.unit_.Get();
}
inline void Metric::_internal_set_unit(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.unit_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Metric::_internal_mutable_unit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.unit_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Metric::release_unit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.metrics.v1.Metric.unit)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.unit_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.unit_.Set("", GetArena());
  }
  return released;
}
inline void Metric::set_allocated_unit(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.unit_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.unit_.IsDefault()) {
    _impl_.unit_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.metrics.v1.Metric.unit)
}

// .opentelemetry.proto.metrics.v1.Gauge gauge = 5;
inline bool Metric::has_gauge() const {
  return data_case() == kGauge;
}
inline bool Metric::_internal_has_gauge() const {
  return data_case() == kGauge;
}
inline void Metric::set_has_gauge() {
  _impl_._oneof_case_[0] = kGauge;
}
inline void Metric::clear_gauge() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (data_case() == kGauge) {
    if (GetArena() == nullptr) {
      delete _impl_.data_.gauge_;
    } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
      ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.data_.gauge_);
    }
    clear_has_data();
  }
}
inline ::opentelemetry::proto::metrics::v1::Gauge* PROTOBUF_NULLABLE Metric::release_gauge() {
  // @@protoc_insertion_point(field_release:opentelemetry.proto.metrics.v1.Metric.gauge)
  if (data_case() == kGauge) {
    clear_has_data();
    auto* temp = reinterpret_cast<::opentelemetry::proto::metrics::v1::Gauge*>(_impl_.data_.gauge_);
    if (GetArena() != nullptr) {
      temp = ::google::protobuf::internal::DuplicateIfNonNull(temp);
    }
    _impl_.data_.gauge_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::opentelemetry::proto::metrics::v1::Gauge& Metric::_internal_gauge() const {
  return data_case() == kGauge ? static_cast<const ::opentelemetry::proto::metrics::v1::Gauge&>(*reinterpret_cast<::opentelemetry::proto::metrics::v1::Gauge*>(_impl_.data_.gauge_))
                     : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::opentelemetry::proto::metrics::v1::Gauge>(&::opentelemetry::proto::metrics::v1::Gauge_globals_);
}
inline const ::opentelemetry::proto::metrics::v1::Gauge& Metric::gauge() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.Metric.gauge)
  return _internal_gauge();
}
inline ::opentelemetry::proto::metrics::v1::Gauge* PROTOBUF_NULLABLE Metric::unsafe_arena_release_gauge() {
  // @@protoc_insertion_point(field_unsafe_arena_release:opentelemetry.proto.metrics.v1.Metric.gauge)
  if (data_case() == kGauge) {
    clear_has_data();
    auto* temp = reinterpret_cast<::opentelemetry::proto::metrics::v1::Gauge*>(_impl_.data_.gauge_);
    _impl_.data_.gauge_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void Metric::unsafe_arena_set_allocated_gauge(
    ::opentelemetry::proto::metrics::v1::Gauge* PROTOBUF_NULLABLE value) {
  // We rely on the oneof clear method to free the earlier contents
  // of this oneof. We can directly use the pointer we're given to
  // set the new value.
  clear_data();
  if (value) {
    set_has_gauge();
    _impl_.data_.gauge_ = reinterpret_cast<::google::protobuf::Message*>(value);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:opentelemetry.proto.metrics.v1.Metric.gauge)
}
inline ::opentelemetry::proto::metrics::v1::Gauge* PROTOBUF_NONNULL Metric::_internal_mutable_gauge() {
  if (data_case() != kGauge) {
    clear_data();
    set_has_gauge();
    _impl_.data_.gauge_ = reinterpret_cast<::google::protobuf::Message*>(
        ::google::protobuf::Message::DefaultConstruct<::opentelemetry::proto::metrics::v1::Gauge>(GetArena()));
  }
  return reinterpret_cast<::opentelemetry::proto::metrics::v1::Gauge*>(_impl_.data_.gauge_);
}
inline ::opentelemetry::proto::metrics::v1::Gauge* PROTOBUF_NONNULL Metric::mutable_gauge()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::opentelemetry::proto::metrics::v1::Gauge* _msg = _internal_mutable_gauge();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.Metric.gauge)
  return _msg;
}

// .opentelemetry.proto.metrics.v1.Sum sum = 7;
inline bool Metric::has_sum() const {
  return data_case() == kSum;
}
inline bool Metric::_internal_has_sum() const {
  return data_case() == kSum;
}
inline void Metric::set_has_sum() {
  _impl_._oneof_case_[0] = kSum;
}
inline void Metric::clear_sum() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (data_case() == kSum) {
    if (GetArena() == nullptr) {
      delete _impl_.data_.sum_;
    } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
      ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.data_.sum_);
    }
    clear_has_data();
  }
}
inline ::opentelemetry::proto::metrics::v1::Sum* PROTOBUF_NULLABLE Metric::release_sum() {
  // @@protoc_insertion_point(field_release:opentelemetry.proto.metrics.v1.Metric.sum)
  if (data_case() == kSum) {
    clear_has_data();
    auto* temp = reinterpret_cast<::opentelemetry::proto::metrics::v1::Sum*>(_impl_.data_.sum_);
    if (GetArena() != nullptr) {
      temp = ::google::protobuf::internal::DuplicateIfNonNull(temp);
    }
    _impl_.data_.sum_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::opentelemetry::proto::metrics::v1::Sum& Metric::_internal_sum() const {
  return data_case() == kSum ? static_cast<const ::opentelemetry::proto::metrics::v1::Sum&>(*reinterpret_cast<::opentelemetry::proto::metrics::v1::Sum*>(_impl_.data_.sum_))
                     : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::opentelemetry::proto::metrics::v1::Sum>(&::opentelemetry::proto::metrics::v1::Sum_globals_);
}
inline const ::opentelemetry::proto::metrics::v1::Sum& Metric::sum() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.Metric.sum)
  return _internal_sum();
}
inline ::opentelemetry::proto::metrics::v1::Sum* PROTOBUF_NULLABLE Metric::unsafe_arena_release_sum() {
  // @@protoc_insertion_point(field_unsafe_arena_release:opentelemetry.proto.metrics.v1.Metric.sum)
  if (data_case() == kSum) {
    clear_has_data();
    auto* temp = reinterpret_cast<::opentelemetry::proto::metrics::v1::Sum*>(_impl_.data_.sum_);
    _impl_.data_.sum_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void Metric::unsafe_arena_set_allocated_sum(
    ::opentelemetry::proto::metrics::v1::Sum* PROTOBUF_NULLABLE value) {
  // We rely on the oneof clear method to free the earlier contents
  // of this oneof. We can directly use the pointer we're given to
  // set the new value.
  clear_data();
  if (value) {
    set_has_sum();
    _impl_.data_.sum_ = reinterpret_cast<::google::protobuf::Message*>(value);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:opentelemetry.proto.metrics.v1.Metric.sum)
}
inline ::opentelemetry::proto::metrics::v1::Sum* PROTOBUF_NONNULL Metric::_internal_mutable_sum() {
  if (data_case() != kSum) {
    clear_data();
    set_has_sum();
    _impl_.data_.sum_ = reinterpret_cast<::google::protobuf::Message*>(
        ::google::protobuf::Message::DefaultConstruct<::opentelemetry::proto::metrics::v1::Sum>(GetArena()));
  }
  return reinterpret_cast<::opentelemetry::proto::metrics::v1::Sum*>(_impl_.data_.sum_);
}
inline ::opentelemetry::proto::metrics::v1::Sum* PROTOBUF_NONNULL Metric::mutable_sum()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::opentelemetry::proto::metrics::v1::Sum* _msg = _internal_mutable_sum();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.Metric.sum)
  return _msg;
}

// .opentelemetry.proto.metrics.v1.Histogram histogram = 9;
inline bool Metric::has_histogram() const {
  return data_case() == kHistogram;
}
inline bool Metric::_internal_has_histogram() const {
  return data_case() == kHistogram;
}
inline void Metric::set_has_histogram() {
  _impl_._oneof_case_[0] = kHistogram;
}
inline void Metric::clear_histogram() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (data_case() == kHistogram) {
    if (GetArena() == nullptr) {
      delete _impl_.data_.histogram_;
    } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
      ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.data_.histogram_);
    }
    clear_has_data();
  }
}
inline ::opentelemetry::proto::metrics::v1::Histogram* PROTOBUF_NULLABLE Metric::release_histogram() {
  // @@protoc_insertion_point(field_release:opentelemetry.proto.metrics.v1.Metric.histogram)
  if (data_case() == kHistogram) {
    clear_has_data();
    auto* temp = reinterpret_cast<::opentelemetry::proto::metrics::v1::Histogram*>(_impl_.data_.histogram_);
    if (GetArena() != nullptr) {
      temp = ::google::protobuf::internal::DuplicateIfNonNull(temp);
    }
    _impl_.data_.histogram_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::opentelemetry::proto::metrics::v1::Histogram& Metric::_internal_histogram() const {
  return data_case() == kHistogram ? static_cast<const ::opentelemetry::proto::metrics::v1::Histogram&>(*reinterpret_cast<::opentelemetry::proto::metrics::v1::Histogram*>(_impl_.data_.histogram_))
                     : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::opentelemetry::proto::metrics::v1::Histogram>(&::opentelemetry::proto::metrics::v1::Histogram_globals_);
}
inline const ::opentelemetry::proto::metrics::v1::Histogram& Metric::histogram() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.Metric.histogram)
  return _internal_histogram();
}
inline ::opentelemetry::proto::metrics::v1::Histogram* PROTOBUF_NULLABLE Metric::unsafe_arena_release_histogram() {
  // @@protoc_insertion_point(field_unsafe_arena_release:opentelemetry.proto.metrics.v1.Metric.histogram)
  if (data_case() == kHistogram) {
    clear_has_data();
    auto* temp = reinterpret_cast<::opentelemetry::proto::metrics::v1::Histogram*>(_impl_.data_.histogram_);
    _impl_.data_.histogram_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void Metric::unsafe_arena_set_allocated_histogram(
    ::opentelemetry::proto::metrics::v1::Histogram* PROTOBUF_NULLABLE value) {
  // We rely on the oneof clear method to free the earlier contents
  // of this oneof. We can directly use the pointer we're given to
  // set the new value.
  clear_data();
  if (value) {
    set_has_histogram();
    _impl_.data_.histogram_ = reinterpret_cast<::google::protobuf::Message*>(value);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:opentelemetry.proto.metrics.v1.Metric.histogram)
}
inline ::opentelemetry::proto::metrics::v1::Histogram* PROTOBUF_NONNULL Metric::_internal_mutable_histogram() {
  if (data_case() != kHistogram) {
    clear_data();
    set_has_histogram();
    _impl_.data_.histogram_ = reinterpret_cast<::google::protobuf::Message*>(
        ::google::protobuf::Message::DefaultConstruct<::opentelemetry::proto::metrics::v1::Histogram>(GetArena()));
  }
  return reinterpret_cast<::opentelemetry::proto::metrics::v1::Histogram*>(_impl_.data_.histogram_);
}
inline ::opentelemetry::proto::metrics::v1::Histogram* PROTOBUF_NONNULL Metric::mutable_histogram()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::opentelemetry::proto::metrics::v1::Histogram* _msg = _internal_mutable_histogram();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.Metric.histogram)
  return _msg;
}

// .opentelemetry.proto.metrics.v1.ExponentialHistogram exponential_histogram = 10;
inline bool Metric::has_exponential_histogram() const {
  return data_case() == kExponentialHistogram;
}
inline bool Metric::_internal_has_exponential_histogram() const {
  return data_case() == kExponentialHistogram;
}
inline void Metric::set_has_exponential_histogram() {
  _impl_._oneof_case_[0] = kExponentialHistogram;
}
inline void Metric::clear_exponential_histogram() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (data_case() == kExponentialHistogram) {
    if (GetArena() == nullptr) {
      delete _impl_.data_.exponential_histogram_;
    } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
      ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.data_.exponential_histogram_);
    }
    clear_has_data();
  }
}
inline ::opentelemetry::proto::metrics::v1::ExponentialHistogram* PROTOBUF_NULLABLE Metric::release_exponential_histogram() {
  // @@protoc_insertion_point(field_release:opentelemetry.proto.metrics.v1.Metric.exponential_histogram)
  if (data_case() == kExponentialHistogram) {
    clear_has_data();
    auto* temp = reinterpret_cast<::opentelemetry::proto::metrics::v1::ExponentialHistogram*>(_impl_.data_.exponential_histogram_);
    if (GetArena() != nullptr) {
      temp = ::google::protobuf::internal::DuplicateIfNonNull(temp);
    }
    _impl_.data_.exponential_histogram_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::opentelemetry::proto::metrics::v1::ExponentialHistogram& Metric::_internal_exponential_histogram() const {
  return data_case() == kExponentialHistogram ? static_cast<const ::opentelemetry::proto::metrics::v1::ExponentialHistogram&>(*reinterpret_cast<::opentelemetry::proto::metrics::v1::ExponentialHistogram*>(_impl_.data_.exponential_histogram_))
                     : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::opentelemetry::proto::metrics::v1::ExponentialHistogram>(&::opentelemetry::proto::metrics::v1::ExponentialHistogram_globals_);
}
inline const ::opentelemetry::proto::metrics::v1::ExponentialHistogram& Metric::exponential_histogram() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.Metric.exponential_histogram)
  return _internal_exponential_histogram();
}
inline ::opentelemetry::proto::metrics::v1::ExponentialHistogram* PROTOBUF_NULLABLE Metric::unsafe_arena_release_exponential_histogram() {
  // @@protoc_insertion_point(field_unsafe_arena_release:opentelemetry.proto.metrics.v1.Metric.exponential_histogram)
  if (data_case() == kExponentialHistogram) {
    clear_has_data();
    auto* temp = reinterpret_cast<::opentelemetry::proto::metrics::v1::ExponentialHistogram*>(_impl_.data_.exponential_histogram_);
    _impl_.data_.exponential_histogram_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void Metric::unsafe_arena_set_allocated_exponential_histogram(
    ::opentelemetry::proto::metrics::v1::ExponentialHistogram* PROTOBUF_NULLABLE value) {
  // We rely on the oneof clear method to free the earlier contents
  // of this oneof. We can directly use the pointer we're given to
  // set the new value.
  clear_data();
  if (value) {
    set_has_exponential_histogram();
    _impl_.data_.exponential_histogram_ = reinterpret_cast<::google::protobuf::Message*>(value);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:opentelemetry.proto.metrics.v1.Metric.exponential_histogram)
}
inline ::opentelemetry::proto::metrics::v1::ExponentialHistogram* PROTOBUF_NONNULL Metric::_internal_mutable_exponential_histogram() {
  if (data_case() != kExponentialHistogram) {
    clear_data();
    set_has_exponential_histogram();
    _impl_.data_.exponential_histogram_ = reinterpret_cast<::google::protobuf::Message*>(
        ::google::protobuf::Message::DefaultConstruct<::opentelemetry::proto::metrics::v1::ExponentialHistogram>(GetArena()));
  }
  return reinterpret_cast<::opentelemetry::proto::metrics::v1::ExponentialHistogram*>(_impl_.data_.exponential_histogram_);
}
inline ::opentelemetry::proto::metrics::v1::ExponentialHistogram* PROTOBUF_NONNULL Metric::mutable_exponential_histogram()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::opentelemetry::proto::metrics::v1::ExponentialHistogram* _msg = _internal_mutable_exponential_histogram();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.Metric.exponential_histogram)
  return _msg;
}

// .opentelemetry.proto.metrics.v1.Summary summary = 11;
inline bool Metric::has_summary() const {
  return data_case() == kSummary;
}
inline bool Metric::_internal_has_summary() const {
  return data_case() == kSummary;
}
inline void Metric::set_has_summary() {
  _impl_._oneof_case_[0] = kSummary;
}
inline void Metric::clear_summary() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (data_case() == kSummary) {
    if (GetArena() == nullptr) {
      delete _impl_.data_.summary_;
    } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
      ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.data_.summary_);
    }
    clear_has_data();
  }
}
inline ::opentelemetry::proto::metrics::v1::Summary* PROTOBUF_NULLABLE Metric::release_summary() {
  // @@protoc_insertion_point(field_release:opentelemetry.proto.metrics.v1.Metric.summary)
  if (data_case() == kSummary) {
    clear_has_data();
    auto* temp = reinterpret_cast<::opentelemetry::proto::metrics::v1::Summary*>(_impl_.data_.summary_);
    if (GetArena() != nullptr) {
      temp = ::google::protobuf::internal::DuplicateIfNonNull(temp);
    }
    _impl_.data_.summary_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::opentelemetry::proto::metrics::v1::Summary& Metric::_internal_summary() const {
  return data_case() == kSummary ? static_cast<const ::opentelemetry::proto::metrics::v1::Summary&>(*reinterpret_cast<::opentelemetry::proto::metrics::v1::Summary*>(_impl_.data_.summary_))
                     : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::opentelemetry::proto::metrics::v1::Summary>(&::opentelemetry::proto::metrics::v1::Summary_globals_);
}
inline const ::opentelemetry::proto::metrics::v1::Summary& Metric::summary() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.Metric.summary)
  return _internal_summary();
}
inline ::opentelemetry::proto::metrics::v1::Summary* PROTOBUF_NULLABLE Metric::unsafe_arena_release_summary() {
  // @@protoc_insertion_point(field_unsafe_arena_release:opentelemetry.proto.metrics.v1.Metric.summary)
  if (data_case() == kSummary) {
    clear_has_data();
    auto* temp = reinterpret_cast<::opentelemetry::proto::metrics::v1::Summary*>(_impl_.data_.summary_);
    _impl_.data_.summary_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void Metric::unsafe_arena_set_allocated_summary(
    ::opentelemetry::proto::metrics::v1::Summary* PROTOBUF_NULLABLE value) {
  // We rely on the oneof clear method to free the earlier contents
  // of this oneof. We can directly use the pointer we're given to
  // set the new value.
  clear_data();
  if (value) {
    set_has_summary();
    _impl_.data_.summary_ = reinterpret_cast<::google::protobuf::Message*>(value);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:opentelemetry.proto.metrics.v1.Metric.summary)
}
inline ::opentelemetry::proto::metrics::v1::Summary* PROTOBUF_NONNULL Metric::_internal_mutable_summary() {
  if (data_case() != kSummary) {
    clear_data();
    set_has_summary();
    _impl_.data_.summary_ = reinterpret_cast<::google::protobuf::Message*>(
        ::google::protobuf::Message::DefaultConstruct<::opentelemetry::proto::metrics::v1::Summary>(GetArena()));
  }
  return reinterpret_cast<::opentelemetry::proto::metrics::v1::Summary*>(_impl_.data_.summary_);
}
inline ::opentelemetry::proto::metrics::v1::Summary* PROTOBUF_NONNULL Metric::mutable_summary()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::opentelemetry::proto::metrics::v1::Summary* _msg = _internal_mutable_summary();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.Metric.summary)
  return _msg;
}

inline bool Metric::has_data() const {
  return data_case() != DATA_NOT_SET;
}
inline void Metric::clear_has_data() {
  _impl_._oneof_case_[0] = DATA_NOT_SET;
}
inline Metric::DataCase Metric::data_case() const {
  return Metric::DataCase(_impl_._oneof_case_[0]);
}
// -------------------------------------------------------------------

// Gauge

// repeated .opentelemetry.proto.metrics.v1.NumberDataPoint data_points = 1;
inline int Gauge::_internal_data_points_size() const {
  return _internal_data_points().size();
}
inline int Gauge::data_points_size() const {
  return _internal_data_points_size();
}
inline void Gauge::clear_data_points() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.data_points_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::opentelemetry::proto::metrics::v1::NumberDataPoint& Gauge::data_points(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.Gauge.data_points)
  return _internal_data_points().Get(index);
}
inline ::opentelemetry::proto::metrics::v1::NumberDataPoint* PROTOBUF_NONNULL Gauge::mutable_data_points(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.Gauge.data_points)
  return _internal_mutable_data_points()->Mutable(index);
}
inline ::opentelemetry::proto::metrics::v1::NumberDataPoint* PROTOBUF_NONNULL Gauge::add_data_points()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::metrics::v1::NumberDataPoint* _add =
      _internal_mutable_data_points()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.metrics.v1.Gauge.data_points)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::NumberDataPoint>& Gauge::data_points() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.metrics.v1.Gauge.data_points)
  return _internal_data_points();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::NumberDataPoint>* PROTOBUF_NONNULL
Gauge::mutable_data_points() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.metrics.v1.Gauge.data_points)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_data_points();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::NumberDataPoint>&
Gauge::_internal_data_points() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.data_points_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::NumberDataPoint>* PROTOBUF_NONNULL
Gauge::_internal_mutable_data_points() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.data_points_;
}

// -------------------------------------------------------------------

// Sum

// repeated .opentelemetry.proto.metrics.v1.NumberDataPoint data_points = 1;
inline int Sum::_internal_data_points_size() const {
  return _internal_data_points().size();
}
inline int Sum::data_points_size() const {
  return _internal_data_points_size();
}
inline void Sum::clear_data_points() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.data_points_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::opentelemetry::proto::metrics::v1::NumberDataPoint& Sum::data_points(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.Sum.data_points)
  return _internal_data_points().Get(index);
}
inline ::opentelemetry::proto::metrics::v1::NumberDataPoint* PROTOBUF_NONNULL Sum::mutable_data_points(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.Sum.data_points)
  return _internal_mutable_data_points()->Mutable(index);
}
inline ::opentelemetry::proto::metrics::v1::NumberDataPoint* PROTOBUF_NONNULL Sum::add_data_points()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::metrics::v1::NumberDataPoint* _add =
      _internal_mutable_data_points()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.metrics.v1.Sum.data_points)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::NumberDataPoint>& Sum::data_points() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.metrics.v1.Sum.data_points)
  return _internal_data_points();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::NumberDataPoint>* PROTOBUF_NONNULL
Sum::mutable_data_points() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.metrics.v1.Sum.data_points)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_data_points();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::NumberDataPoint>&
Sum::_internal_data_points() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.data_points_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::NumberDataPoint>* PROTOBUF_NONNULL
Sum::_internal_mutable_data_points() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.data_points_;
}

// .opentelemetry.proto.metrics.v1.AggregationTemporality aggregation_temporality = 2;
inline void Sum::clear_aggregation_temporality() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.aggregation_temporality_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::opentelemetry::proto::metrics::v1::AggregationTemporality Sum::aggregation_temporality() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.Sum.aggregation_temporality)
  return _internal_aggregation_temporality();
}
inline void Sum::set_aggregation_temporality(::opentelemetry::proto::metrics::v1::AggregationTemporality value) {
  _internal_set_aggregation_temporality(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.Sum.aggregation_temporality)
}
inline ::opentelemetry::proto::metrics::v1::AggregationTemporality Sum::_internal_aggregation_temporality() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::opentelemetry::proto::metrics::v1::AggregationTemporality>(_impl_.aggregation_temporality_);
}
inline void Sum::_internal_set_aggregation_temporality(::opentelemetry::proto::metrics::v1::AggregationTemporality value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.aggregation_temporality_ = value;
}

// bool is_monotonic = 3;
inline void Sum::clear_is_monotonic() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.is_monotonic_ = false;
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline bool Sum::is_monotonic() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.Sum.is_monotonic)
  return _internal_is_monotonic();
}
inline void Sum::set_is_monotonic(bool value) {
  _internal_set_is_monotonic(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.Sum.is_monotonic)
}
inline bool Sum::_internal_is_monotonic() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.is_monotonic_;
}
inline void Sum::_internal_set_is_monotonic(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.is_monotonic_ = value;
}

// -------------------------------------------------------------------

// Histogram

// repeated .opentelemetry.proto.metrics.v1.HistogramDataPoint data_points = 1;
inline int Histogram::_internal_data_points_size() const {
  return _internal_data_points().size();
}
inline int Histogram::data_points_size() const {
  return _internal_data_points_size();
}
inline void Histogram::clear_data_points() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.data_points_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::opentelemetry::proto::metrics::v1::HistogramDataPoint& Histogram::data_points(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.Histogram.data_points)
  return _internal_data_points().Get(index);
}
inline ::opentelemetry::proto::metrics::v1::HistogramDataPoint* PROTOBUF_NONNULL Histogram::mutable_data_points(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.Histogram.data_points)
  return _internal_mutable_data_points()->Mutable(index);
}
inline ::opentelemetry::proto::metrics::v1::HistogramDataPoint* PROTOBUF_NONNULL Histogram::add_data_points()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::metrics::v1::HistogramDataPoint* _add =
      _internal_mutable_data_points()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.metrics.v1.Histogram.data_points)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::HistogramDataPoint>& Histogram::data_points() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.metrics.v1.Histogram.data_points)
  return _internal_data_points();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::HistogramDataPoint>* PROTOBUF_NONNULL
Histogram::mutable_data_points() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.metrics.v1.Histogram.data_points)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_data_points();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::HistogramDataPoint>&
Histogram::_internal_data_points() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.data_points_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::HistogramDataPoint>* PROTOBUF_NONNULL
Histogram::_internal_mutable_data_points() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.data_points_;
}

// .opentelemetry.proto.metrics.v1.AggregationTemporality aggregation_temporality = 2;
inline void Histogram::clear_aggregation_temporality() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.aggregation_temporality_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::opentelemetry::proto::metrics::v1::AggregationTemporality Histogram::aggregation_temporality() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.Histogram.aggregation_temporality)
  return _internal_aggregation_temporality();
}
inline void Histogram::set_aggregation_temporality(::opentelemetry::proto::metrics::v1::AggregationTemporality value) {
  _internal_set_aggregation_temporality(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.Histogram.aggregation_temporality)
}
inline ::opentelemetry::proto::metrics::v1::AggregationTemporality Histogram::_internal_aggregation_temporality() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::opentelemetry::proto::metrics::v1::AggregationTemporality>(_impl_.aggregation_temporality_);
}
inline void Histogram::_internal_set_aggregation_temporality(::opentelemetry::proto::metrics::v1::AggregationTemporality value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.aggregation_temporality_ = value;
}

// -------------------------------------------------------------------

// ExponentialHistogram

// repeated .opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint data_points = 1;
inline int ExponentialHistogram::_internal_data_points_size() const {
  return _internal_data_points().size();
}
inline int ExponentialHistogram::data_points_size() const {
  return _internal_data_points_size();
}
inline void ExponentialHistogram::clear_data_points() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.data_points_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint& ExponentialHistogram::data_points(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ExponentialHistogram.data_points)
  return _internal_data_points().Get(index);
}
inline ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint* PROTOBUF_NONNULL ExponentialHistogram::mutable_data_points(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.ExponentialHistogram.data_points)
  return _internal_mutable_data_points()->Mutable(index);
}
inline ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint* PROTOBUF_NONNULL ExponentialHistogram::add_data_points()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint* _add =
      _internal_mutable_data_points()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.metrics.v1.ExponentialHistogram.data_points)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint>& ExponentialHistogram::data_points() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.metrics.v1.ExponentialHistogram.data_points)
  return _internal_data_points();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint>* PROTOBUF_NONNULL
ExponentialHistogram::mutable_data_points() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.metrics.v1.ExponentialHistogram.data_points)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_data_points();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint>&
ExponentialHistogram::_internal_data_points() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.data_points_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint>* PROTOBUF_NONNULL
ExponentialHistogram::_internal_mutable_data_points() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.data_points_;
}

// .opentelemetry.proto.metrics.v1.AggregationTemporality aggregation_temporality = 2;
inline void ExponentialHistogram::clear_aggregation_temporality() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.aggregation_temporality_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::opentelemetry::proto::metrics::v1::AggregationTemporality ExponentialHistogram::aggregation_temporality() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ExponentialHistogram.aggregation_temporality)
  return _internal_aggregation_temporality();
}
inline void ExponentialHistogram::set_aggregation_temporality(::opentelemetry::proto::metrics::v1::AggregationTemporality value) {
  _internal_set_aggregation_temporality(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.ExponentialHistogram.aggregation_temporality)
}
inline ::opentelemetry::proto::metrics::v1::AggregationTemporality ExponentialHistogram::_internal_aggregation_temporality() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::opentelemetry::proto::metrics::v1::AggregationTemporality>(_impl_.aggregation_temporality_);
}
inline void ExponentialHistogram::_internal_set_aggregation_temporality(::opentelemetry::proto::metrics::v1::AggregationTemporality value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.aggregation_temporality_ = value;
}

// -------------------------------------------------------------------

// Summary

// repeated .opentelemetry.proto.metrics.v1.SummaryDataPoint data_points = 1;
inline int Summary::_internal_data_points_size() const {
  return _internal_data_points().size();
}
inline int Summary::data_points_size() const {
  return _internal_data_points_size();
}
inline void Summary::clear_data_points() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.data_points_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::opentelemetry::proto::metrics::v1::SummaryDataPoint& Summary::data_points(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.Summary.data_points)
  return _internal_data_points().Get(index);
}
inline ::opentelemetry::proto::metrics::v1::SummaryDataPoint* PROTOBUF_NONNULL Summary::mutable_data_points(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.Summary.data_points)
  return _internal_mutable_data_points()->Mutable(index);
}
inline ::opentelemetry::proto::metrics::v1::SummaryDataPoint* PROTOBUF_NONNULL Summary::add_data_points()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::metrics::v1::SummaryDataPoint* _add =
      _internal_mutable_data_points()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.metrics.v1.Summary.data_points)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::SummaryDataPoint>& Summary::data_points() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.metrics.v1.Summary.data_points)
  return _internal_data_points();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::SummaryDataPoint>* PROTOBUF_NONNULL
Summary::mutable_data_points() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.metrics.v1.Summary.data_points)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_data_points();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::SummaryDataPoint>&
Summary::_internal_data_points() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.data_points_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::SummaryDataPoint>* PROTOBUF_NONNULL
Summary::_internal_mutable_data_points() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.data_points_;
}

// -------------------------------------------------------------------

// NumberDataPoint

// repeated .opentelemetry.proto.common.v1.KeyValue attributes = 7;
inline int NumberDataPoint::_internal_attributes_size() const {
  return _internal_attributes().size();
}
inline int NumberDataPoint::attributes_size() const {
  return _internal_attributes_size();
}
inline const ::opentelemetry::proto::common::v1::KeyValue& NumberDataPoint::attributes(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.NumberDataPoint.attributes)
  return _internal_attributes().Get(index);
}
inline ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL NumberDataPoint::mutable_attributes(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.NumberDataPoint.attributes)
  return _internal_mutable_attributes()->Mutable(index);
}
inline ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL NumberDataPoint::add_attributes()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::common::v1::KeyValue* _add =
      _internal_mutable_attributes()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.metrics.v1.NumberDataPoint.attributes)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>& NumberDataPoint::attributes() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.metrics.v1.NumberDataPoint.attributes)
  return _internal_attributes();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL
NumberDataPoint::mutable_attributes() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.metrics.v1.NumberDataPoint.attributes)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_attributes();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>&
NumberDataPoint::_internal_attributes() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.attributes_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL
NumberDataPoint::_internal_mutable_attributes() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.attributes_;
}

// fixed64 start_time_unix_nano = 2;
inline void NumberDataPoint::clear_start_time_unix_nano() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.start_time_unix_nano_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::uint64_t NumberDataPoint::start_time_unix_nano() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.NumberDataPoint.start_time_unix_nano)
  return _internal_start_time_unix_nano();
}
inline void NumberDataPoint::set_start_time_unix_nano(::uint64_t value) {
  _internal_set_start_time_unix_nano(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.NumberDataPoint.start_time_unix_nano)
}
inline ::uint64_t NumberDataPoint::_internal_start_time_unix_nano() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.start_time_unix_nano_;
}
inline void NumberDataPoint::_internal_set_start_time_unix_nano(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.start_time_unix_nano_ = value;
}

// fixed64 time_unix_nano = 3;
inline void NumberDataPoint::clear_time_unix_nano() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.time_unix_nano_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::uint64_t NumberDataPoint::time_unix_nano() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.NumberDataPoint.time_unix_nano)
  return _internal_time_unix_nano();
}
inline void NumberDataPoint::set_time_unix_nano(::uint64_t value) {
  _internal_set_time_unix_nano(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.NumberDataPoint.time_unix_nano)
}
inline ::uint64_t NumberDataPoint::_internal_time_unix_nano() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.time_unix_nano_;
}
inline void NumberDataPoint::_internal_set_time_unix_nano(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.time_unix_nano_ = value;
}

// double as_double = 4;
inline bool NumberDataPoint::has_as_double() const {
  return value_case() == kAsDouble;
}
inline void NumberDataPoint::set_has_as_double() {
  _impl_._oneof_case_[0] = kAsDouble;
}
inline void NumberDataPoint::clear_as_double() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value_case() == kAsDouble) {
    _impl_.value_.as_double_ = 0;
    clear_has_value();
  }
}
inline double NumberDataPoint::as_double() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.NumberDataPoint.as_double)
  return _internal_as_double();
}
inline void NumberDataPoint::set_as_double(double value) {
  if (value_case() != kAsDouble) {
    clear_value();
    set_has_as_double();
  }
  _impl_.value_.as_double_ = value;
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.NumberDataPoint.as_double)
}
inline double NumberDataPoint::_internal_as_double() const {
  if (value_case() == kAsDouble) {
    return _impl_.value_.as_double_;
  }
  return 0;
}

// sfixed64 as_int = 6;
inline bool NumberDataPoint::has_as_int() const {
  return value_case() == kAsInt;
}
inline void NumberDataPoint::set_has_as_int() {
  _impl_._oneof_case_[0] = kAsInt;
}
inline void NumberDataPoint::clear_as_int() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value_case() == kAsInt) {
    _impl_.value_.as_int_ = ::int64_t{0};
    clear_has_value();
  }
}
inline ::int64_t NumberDataPoint::as_int() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.NumberDataPoint.as_int)
  return _internal_as_int();
}
inline void NumberDataPoint::set_as_int(::int64_t value) {
  if (value_case() != kAsInt) {
    clear_value();
    set_has_as_int();
  }
  _impl_.value_.as_int_ = value;
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.NumberDataPoint.as_int)
}
inline ::int64_t NumberDataPoint::_internal_as_int() const {
  if (value_case() == kAsInt) {
    return _impl_.value_.as_int_;
  }
  return ::int64_t{0};
}

// repeated .opentelemetry.proto.metrics.v1.Exemplar exemplars = 5;
inline int NumberDataPoint::_internal_exemplars_size() const {
  return _internal_exemplars().size();
}
inline int NumberDataPoint::exemplars_size() const {
  return _internal_exemplars_size();
}
inline void NumberDataPoint::clear_exemplars() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.exemplars_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::opentelemetry::proto::metrics::v1::Exemplar& NumberDataPoint::exemplars(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.NumberDataPoint.exemplars)
  return _internal_exemplars().Get(index);
}
inline ::opentelemetry::proto::metrics::v1::Exemplar* PROTOBUF_NONNULL NumberDataPoint::mutable_exemplars(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.NumberDataPoint.exemplars)
  return _internal_mutable_exemplars()->Mutable(index);
}
inline ::opentelemetry::proto::metrics::v1::Exemplar* PROTOBUF_NONNULL NumberDataPoint::add_exemplars()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::metrics::v1::Exemplar* _add =
      _internal_mutable_exemplars()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.metrics.v1.NumberDataPoint.exemplars)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>& NumberDataPoint::exemplars() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.metrics.v1.NumberDataPoint.exemplars)
  return _internal_exemplars();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>* PROTOBUF_NONNULL
NumberDataPoint::mutable_exemplars() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.metrics.v1.NumberDataPoint.exemplars)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_exemplars();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>&
NumberDataPoint::_internal_exemplars() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.exemplars_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>* PROTOBUF_NONNULL
NumberDataPoint::_internal_mutable_exemplars() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.exemplars_;
}

// uint32 flags = 8;
inline void NumberDataPoint::clear_flags() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::uint32_t NumberDataPoint::flags() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.NumberDataPoint.flags)
  return _internal_flags();
}
inline void NumberDataPoint::set_flags(::uint32_t value) {
  _internal_set_flags(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.NumberDataPoint.flags)
}
inline ::uint32_t NumberDataPoint::_internal_flags() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.flags_;
}
inline void NumberDataPoint::_internal_set_flags(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = value;
}

inline bool NumberDataPoint::has_value() const {
  return value_case() != VALUE_NOT_SET;
}
inline void NumberDataPoint::clear_has_value() {
  _impl_._oneof_case_[0] = VALUE_NOT_SET;
}
inline NumberDataPoint::ValueCase NumberDataPoint::value_case() const {
  return NumberDataPoint::ValueCase(_impl_._oneof_case_[0]);
}
// -------------------------------------------------------------------

// HistogramDataPoint

// repeated .opentelemetry.proto.common.v1.KeyValue attributes = 9;
inline int HistogramDataPoint::_internal_attributes_size() const {
  return _internal_attributes().size();
}
inline int HistogramDataPoint::attributes_size() const {
  return _internal_attributes_size();
}
inline const ::opentelemetry::proto::common::v1::KeyValue& HistogramDataPoint::attributes(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.HistogramDataPoint.attributes)
  return _internal_attributes().Get(index);
}
inline ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL HistogramDataPoint::mutable_attributes(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.HistogramDataPoint.attributes)
  return _internal_mutable_attributes()->Mutable(index);
}
inline ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL HistogramDataPoint::add_attributes()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::common::v1::KeyValue* _add =
      _internal_mutable_attributes()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.metrics.v1.HistogramDataPoint.attributes)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>& HistogramDataPoint::attributes() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.metrics.v1.HistogramDataPoint.attributes)
  return _internal_attributes();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL
HistogramDataPoint::mutable_attributes() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.metrics.v1.HistogramDataPoint.attributes)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_attributes();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>&
HistogramDataPoint::_internal_attributes() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.attributes_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL
HistogramDataPoint::_internal_mutable_attributes() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.attributes_;
}

// fixed64 start_time_unix_nano = 2;
inline void HistogramDataPoint::clear_start_time_unix_nano() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.start_time_unix_nano_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::uint64_t HistogramDataPoint::start_time_unix_nano() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.HistogramDataPoint.start_time_unix_nano)
  return _internal_start_time_unix_nano();
}
inline void HistogramDataPoint::set_start_time_unix_nano(::uint64_t value) {
  _internal_set_start_time_unix_nano(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.HistogramDataPoint.start_time_unix_nano)
}
inline ::uint64_t HistogramDataPoint::_internal_start_time_unix_nano() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.start_time_unix_nano_;
}
inline void HistogramDataPoint::_internal_set_start_time_unix_nano(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.start_time_unix_nano_ = value;
}

// fixed64 time_unix_nano = 3;
inline void HistogramDataPoint::clear_time_unix_nano() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.time_unix_nano_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::uint64_t HistogramDataPoint::time_unix_nano() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.HistogramDataPoint.time_unix_nano)
  return _internal_time_unix_nano();
}
inline void HistogramDataPoint::set_time_unix_nano(::uint64_t value) {
  _internal_set_time_unix_nano(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.HistogramDataPoint.time_unix_nano)
}
inline ::uint64_t HistogramDataPoint::_internal_time_unix_nano() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.time_unix_nano_;
}
inline void HistogramDataPoint::_internal_set_time_unix_nano(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.time_unix_nano_ = value;
}

// fixed64 count = 4;
inline void HistogramDataPoint::clear_count() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.count_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::uint64_t HistogramDataPoint::count() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.HistogramDataPoint.count)
  return _internal_count();
}
inline void HistogramDataPoint::set_count(::uint64_t value) {
  _internal_set_count(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.HistogramDataPoint.count)
}
inline ::uint64_t HistogramDataPoint::_internal_count() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.count_;
}
inline void HistogramDataPoint::_internal_set_count(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.count_ = value;
}

// optional double sum = 5;
inline bool HistogramDataPoint::has_sum() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000080U);
  return value;
}
inline void HistogramDataPoint::clear_sum() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sum_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
}
inline double HistogramDataPoint::sum() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.HistogramDataPoint.sum)
  return _internal_sum();
}
inline void HistogramDataPoint::set_sum(double value) {
  _internal_set_sum(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.HistogramDataPoint.sum)
}
inline double HistogramDataPoint::_internal_sum() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sum_;
}
inline void HistogramDataPoint::_internal_set_sum(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sum_ = value;
}

// repeated fixed64 bucket_counts = 6;
inline int HistogramDataPoint::_internal_bucket_counts_size() const {
  return _internal_bucket_counts().size();
}
inline int HistogramDataPoint::bucket_counts_size() const {
  return _internal_bucket_counts_size();
}
inline void HistogramDataPoint::clear_bucket_counts() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.bucket_counts_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline ::uint64_t HistogramDataPoint::bucket_counts(int index) const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.HistogramDataPoint.bucket_counts)
  return _internal_bucket_counts().Get(index);
}
inline void HistogramDataPoint::set_bucket_counts(int index, ::uint64_t value) {
  _internal_mutable_bucket_counts()->Set(index, value);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.HistogramDataPoint.bucket_counts)
}
inline void HistogramDataPoint::add_bucket_counts(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_bucket_counts()
      ->InternalAddWithArena<const ::google::protobuf::MessageLite*>(
          internal_visibility(), this, value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.metrics.v1.HistogramDataPoint.bucket_counts)
}
inline const ::google::protobuf::RepeatedField<::uint64_t>& HistogramDataPoint::bucket_counts() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.metrics.v1.HistogramDataPoint.bucket_counts)
  return _internal_bucket_counts();
}
inline ::google::protobuf::RepeatedField<::uint64_t>* PROTOBUF_NONNULL HistogramDataPoint::mutable_bucket_counts()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.metrics.v1.HistogramDataPoint.bucket_counts)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_bucket_counts();
}
inline const ::google::protobuf::RepeatedField<::uint64_t>&
HistogramDataPoint::_internal_bucket_counts() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.bucket_counts_;
}
inline ::google::protobuf::RepeatedField<::uint64_t>* PROTOBUF_NONNULL
HistogramDataPoint::_internal_mutable_bucket_counts() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.bucket_counts_;
}

// repeated double explicit_bounds = 7;
inline int HistogramDataPoint::_internal_explicit_bounds_size() const {
  return _internal_explicit_bounds().size();
}
inline int HistogramDataPoint::explicit_bounds_size() const {
  return _internal_explicit_bounds_size();
}
inline void HistogramDataPoint::clear_explicit_bounds() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.explicit_bounds_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline double HistogramDataPoint::explicit_bounds(int index) const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.HistogramDataPoint.explicit_bounds)
  return _internal_explicit_bounds().Get(index);
}
inline void HistogramDataPoint::set_explicit_bounds(int index, double value) {
  _internal_mutable_explicit_bounds()->Set(index, value);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.HistogramDataPoint.explicit_bounds)
}
inline void HistogramDataPoint::add_explicit_bounds(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_explicit_bounds()
      ->InternalAddWithArena<const ::google::protobuf::MessageLite*>(
          internal_visibility(), this, value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.metrics.v1.HistogramDataPoint.explicit_bounds)
}
inline const ::google::protobuf::RepeatedField<double>& HistogramDataPoint::explicit_bounds() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.metrics.v1.HistogramDataPoint.explicit_bounds)
  return _internal_explicit_bounds();
}
inline ::google::protobuf::RepeatedField<double>* PROTOBUF_NONNULL HistogramDataPoint::mutable_explicit_bounds()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.metrics.v1.HistogramDataPoint.explicit_bounds)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_explicit_bounds();
}
inline const ::google::protobuf::RepeatedField<double>&
HistogramDataPoint::_internal_explicit_bounds() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.explicit_bounds_;
}
inline ::google::protobuf::RepeatedField<double>* PROTOBUF_NONNULL
HistogramDataPoint::_internal_mutable_explicit_bounds() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.explicit_bounds_;
}

// repeated .opentelemetry.proto.metrics.v1.Exemplar exemplars = 8;
inline int HistogramDataPoint::_internal_exemplars_size() const {
  return _internal_exemplars().size();
}
inline int HistogramDataPoint::exemplars_size() const {
  return _internal_exemplars_size();
}
inline void HistogramDataPoint::clear_exemplars() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.exemplars_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::opentelemetry::proto::metrics::v1::Exemplar& HistogramDataPoint::exemplars(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.HistogramDataPoint.exemplars)
  return _internal_exemplars().Get(index);
}
inline ::opentelemetry::proto::metrics::v1::Exemplar* PROTOBUF_NONNULL HistogramDataPoint::mutable_exemplars(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.HistogramDataPoint.exemplars)
  return _internal_mutable_exemplars()->Mutable(index);
}
inline ::opentelemetry::proto::metrics::v1::Exemplar* PROTOBUF_NONNULL HistogramDataPoint::add_exemplars()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::metrics::v1::Exemplar* _add =
      _internal_mutable_exemplars()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.metrics.v1.HistogramDataPoint.exemplars)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>& HistogramDataPoint::exemplars() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.metrics.v1.HistogramDataPoint.exemplars)
  return _internal_exemplars();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>* PROTOBUF_NONNULL
HistogramDataPoint::mutable_exemplars() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.metrics.v1.HistogramDataPoint.exemplars)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_exemplars();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>&
HistogramDataPoint::_internal_exemplars() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.exemplars_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>* PROTOBUF_NONNULL
HistogramDataPoint::_internal_mutable_exemplars() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.exemplars_;
}

// uint32 flags = 10;
inline void HistogramDataPoint::clear_flags() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000400U);
}
inline ::uint32_t HistogramDataPoint::flags() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.HistogramDataPoint.flags)
  return _internal_flags();
}
inline void HistogramDataPoint::set_flags(::uint32_t value) {
  _internal_set_flags(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000400U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.HistogramDataPoint.flags)
}
inline ::uint32_t HistogramDataPoint::_internal_flags() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.flags_;
}
inline void HistogramDataPoint::_internal_set_flags(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = value;
}

// optional double min = 11;
inline bool HistogramDataPoint::has_min() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000100U);
  return value;
}
inline void HistogramDataPoint::clear_min() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.min_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000100U);
}
inline double HistogramDataPoint::min() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.HistogramDataPoint.min)
  return _internal_min();
}
inline void HistogramDataPoint::set_min(double value) {
  _internal_set_min(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000100U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.HistogramDataPoint.min)
}
inline double HistogramDataPoint::_internal_min() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.min_;
}
inline void HistogramDataPoint::_internal_set_min(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.min_ = value;
}

// optional double max = 12;
inline bool HistogramDataPoint::has_max() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000200U);
  return value;
}
inline void HistogramDataPoint::clear_max() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.max_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000200U);
}
inline double HistogramDataPoint::max() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.HistogramDataPoint.max)
  return _internal_max();
}
inline void HistogramDataPoint::set_max(double value) {
  _internal_set_max(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000200U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.HistogramDataPoint.max)
}
inline double HistogramDataPoint::_internal_max() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.max_;
}
inline void HistogramDataPoint::_internal_set_max(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.max_ = value;
}

// -------------------------------------------------------------------

// ExponentialHistogramDataPoint_Buckets

// sint32 offset = 1;
inline void ExponentialHistogramDataPoint_Buckets::clear_offset() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.offset_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::int32_t ExponentialHistogramDataPoint_Buckets::offset() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.Buckets.offset)
  return _internal_offset();
}
inline void ExponentialHistogramDataPoint_Buckets::set_offset(::int32_t value) {
  _internal_set_offset(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.Buckets.offset)
}
inline ::int32_t ExponentialHistogramDataPoint_Buckets::_internal_offset() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.offset_;
}
inline void ExponentialHistogramDataPoint_Buckets::_internal_set_offset(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.offset_ = value;
}

// repeated uint64 bucket_counts = 2;
inline int ExponentialHistogramDataPoint_Buckets::_internal_bucket_counts_size() const {
  return _internal_bucket_counts().size();
}
inline int ExponentialHistogramDataPoint_Buckets::bucket_counts_size() const {
  return _internal_bucket_counts_size();
}
inline void ExponentialHistogramDataPoint_Buckets::clear_bucket_counts() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.bucket_counts_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline ::uint64_t ExponentialHistogramDataPoint_Buckets::bucket_counts(int index) const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.Buckets.bucket_counts)
  return _internal_bucket_counts().Get(index);
}
inline void ExponentialHistogramDataPoint_Buckets::set_bucket_counts(int index, ::uint64_t value) {
  _internal_mutable_bucket_counts()->Set(index, value);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.Buckets.bucket_counts)
}
inline void ExponentialHistogramDataPoint_Buckets::add_bucket_counts(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_bucket_counts()
      ->InternalAddWithArena<const ::google::protobuf::MessageLite*>(
          internal_visibility(), this, value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.Buckets.bucket_counts)
}
inline const ::google::protobuf::RepeatedField<::uint64_t>& ExponentialHistogramDataPoint_Buckets::bucket_counts() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.Buckets.bucket_counts)
  return _internal_bucket_counts();
}
inline ::google::protobuf::RepeatedField<::uint64_t>* PROTOBUF_NONNULL ExponentialHistogramDataPoint_Buckets::mutable_bucket_counts()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.Buckets.bucket_counts)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_bucket_counts();
}
inline const ::google::protobuf::RepeatedField<::uint64_t>&
ExponentialHistogramDataPoint_Buckets::_internal_bucket_counts() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.bucket_counts_;
}
inline ::google::protobuf::RepeatedField<::uint64_t>* PROTOBUF_NONNULL
ExponentialHistogramDataPoint_Buckets::_internal_mutable_bucket_counts() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.bucket_counts_;
}

// -------------------------------------------------------------------

// ExponentialHistogramDataPoint

// repeated .opentelemetry.proto.common.v1.KeyValue attributes = 1;
inline int ExponentialHistogramDataPoint::_internal_attributes_size() const {
  return _internal_attributes().size();
}
inline int ExponentialHistogramDataPoint::attributes_size() const {
  return _internal_attributes_size();
}
inline const ::opentelemetry::proto::common::v1::KeyValue& ExponentialHistogramDataPoint::attributes(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.attributes)
  return _internal_attributes().Get(index);
}
inline ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL ExponentialHistogramDataPoint::mutable_attributes(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.attributes)
  return _internal_mutable_attributes()->Mutable(index);
}
inline ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL ExponentialHistogramDataPoint::add_attributes()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::common::v1::KeyValue* _add =
      _internal_mutable_attributes()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.attributes)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>& ExponentialHistogramDataPoint::attributes() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.attributes)
  return _internal_attributes();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL
ExponentialHistogramDataPoint::mutable_attributes() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.attributes)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_attributes();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>&
ExponentialHistogramDataPoint::_internal_attributes() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.attributes_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL
ExponentialHistogramDataPoint::_internal_mutable_attributes() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.attributes_;
}

// fixed64 start_time_unix_nano = 2;
inline void ExponentialHistogramDataPoint::clear_start_time_unix_nano() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.start_time_unix_nano_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::uint64_t ExponentialHistogramDataPoint::start_time_unix_nano() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.start_time_unix_nano)
  return _internal_start_time_unix_nano();
}
inline void ExponentialHistogramDataPoint::set_start_time_unix_nano(::uint64_t value) {
  _internal_set_start_time_unix_nano(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.start_time_unix_nano)
}
inline ::uint64_t ExponentialHistogramDataPoint::_internal_start_time_unix_nano() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.start_time_unix_nano_;
}
inline void ExponentialHistogramDataPoint::_internal_set_start_time_unix_nano(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.start_time_unix_nano_ = value;
}

// fixed64 time_unix_nano = 3;
inline void ExponentialHistogramDataPoint::clear_time_unix_nano() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.time_unix_nano_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::uint64_t ExponentialHistogramDataPoint::time_unix_nano() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.time_unix_nano)
  return _internal_time_unix_nano();
}
inline void ExponentialHistogramDataPoint::set_time_unix_nano(::uint64_t value) {
  _internal_set_time_unix_nano(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.time_unix_nano)
}
inline ::uint64_t ExponentialHistogramDataPoint::_internal_time_unix_nano() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.time_unix_nano_;
}
inline void ExponentialHistogramDataPoint::_internal_set_time_unix_nano(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.time_unix_nano_ = value;
}

// fixed64 count = 4;
inline void ExponentialHistogramDataPoint::clear_count() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.count_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::uint64_t ExponentialHistogramDataPoint::count() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.count)
  return _internal_count();
}
inline void ExponentialHistogramDataPoint::set_count(::uint64_t value) {
  _internal_set_count(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.count)
}
inline ::uint64_t ExponentialHistogramDataPoint::_internal_count() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.count_;
}
inline void ExponentialHistogramDataPoint::_internal_set_count(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.count_ = value;
}

// optional double sum = 5;
inline bool ExponentialHistogramDataPoint::has_sum() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000080U);
  return value;
}
inline void ExponentialHistogramDataPoint::clear_sum() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sum_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
}
inline double ExponentialHistogramDataPoint::sum() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.sum)
  return _internal_sum();
}
inline void ExponentialHistogramDataPoint::set_sum(double value) {
  _internal_set_sum(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.sum)
}
inline double ExponentialHistogramDataPoint::_internal_sum() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sum_;
}
inline void ExponentialHistogramDataPoint::_internal_set_sum(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sum_ = value;
}

// sint32 scale = 6;
inline void ExponentialHistogramDataPoint::clear_scale() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.scale_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000200U);
}
inline ::int32_t ExponentialHistogramDataPoint::scale() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.scale)
  return _internal_scale();
}
inline void ExponentialHistogramDataPoint::set_scale(::int32_t value) {
  _internal_set_scale(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000200U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.scale)
}
inline ::int32_t ExponentialHistogramDataPoint::_internal_scale() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.scale_;
}
inline void ExponentialHistogramDataPoint::_internal_set_scale(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.scale_ = value;
}

// fixed64 zero_count = 7;
inline void ExponentialHistogramDataPoint::clear_zero_count() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.zero_count_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000100U);
}
inline ::uint64_t ExponentialHistogramDataPoint::zero_count() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.zero_count)
  return _internal_zero_count();
}
inline void ExponentialHistogramDataPoint::set_zero_count(::uint64_t value) {
  _internal_set_zero_count(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000100U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.zero_count)
}
inline ::uint64_t ExponentialHistogramDataPoint::_internal_zero_count() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.zero_count_;
}
inline void ExponentialHistogramDataPoint::_internal_set_zero_count(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.zero_count_ = value;
}

// .opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.Buckets positive = 8;
inline bool ExponentialHistogramDataPoint::has_positive() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.positive_ != nullptr);
  return value;
}
inline void ExponentialHistogramDataPoint::clear_positive() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.positive_ != nullptr) _impl_.positive_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets& ExponentialHistogramDataPoint::_internal_positive() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* p = _impl_.positive_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets>(&::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets_globals_);
}
inline const ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets& ExponentialHistogramDataPoint::positive() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.positive)
  return _internal_positive();
}
inline void ExponentialHistogramDataPoint::unsafe_arena_set_allocated_positive(
    ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.positive_);
  }
  _impl_.positive_ = reinterpret_cast<::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.positive)
}
inline ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NULLABLE ExponentialHistogramDataPoint::release_positive() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* released = _impl_.positive_;
  _impl_.positive_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NULLABLE ExponentialHistogramDataPoint::unsafe_arena_release_positive() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.positive)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* temp = _impl_.positive_;
  _impl_.positive_ = nullptr;
  return temp;
}
inline ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NONNULL ExponentialHistogramDataPoint::_internal_mutable_positive() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.positive_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets>(GetArena());
    _impl_.positive_ = reinterpret_cast<::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets*>(p);
  }
  return _impl_.positive_;
}
inline ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NONNULL ExponentialHistogramDataPoint::mutable_positive()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* _msg = _internal_mutable_positive();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.positive)
  return _msg;
}
inline void ExponentialHistogramDataPoint::set_allocated_positive(::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.positive_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.positive_ = reinterpret_cast<::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets*>(value);
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.positive)
}

// .opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.Buckets negative = 9;
inline bool ExponentialHistogramDataPoint::has_negative() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  PROTOBUF_ASSUME(!value || _impl_.negative_ != nullptr);
  return value;
}
inline void ExponentialHistogramDataPoint::clear_negative() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.negative_ != nullptr) _impl_.negative_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets& ExponentialHistogramDataPoint::_internal_negative() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* p = _impl_.negative_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets>(&::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets_globals_);
}
inline const ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets& ExponentialHistogramDataPoint::negative() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.negative)
  return _internal_negative();
}
inline void ExponentialHistogramDataPoint::unsafe_arena_set_allocated_negative(
    ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.negative_);
  }
  _impl_.negative_ = reinterpret_cast<::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.negative)
}
inline ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NULLABLE ExponentialHistogramDataPoint::release_negative() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* released = _impl_.negative_;
  _impl_.negative_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NULLABLE ExponentialHistogramDataPoint::unsafe_arena_release_negative() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.negative)

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* temp = _impl_.negative_;
  _impl_.negative_ = nullptr;
  return temp;
}
inline ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NONNULL ExponentialHistogramDataPoint::_internal_mutable_negative() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.negative_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets>(GetArena());
    _impl_.negative_ = reinterpret_cast<::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets*>(p);
  }
  return _impl_.negative_;
}
inline ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NONNULL ExponentialHistogramDataPoint::mutable_negative()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* _msg = _internal_mutable_negative();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.negative)
  return _msg;
}
inline void ExponentialHistogramDataPoint::set_allocated_negative(::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.negative_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }

  _impl_.negative_ = reinterpret_cast<::opentelemetry::proto::metrics::v1::ExponentialHistogramDataPoint_Buckets*>(value);
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.negative)
}

// uint32 flags = 10;
inline void ExponentialHistogramDataPoint::clear_flags() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000400U);
}
inline ::uint32_t ExponentialHistogramDataPoint::flags() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.flags)
  return _internal_flags();
}
inline void ExponentialHistogramDataPoint::set_flags(::uint32_t value) {
  _internal_set_flags(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000400U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.flags)
}
inline ::uint32_t ExponentialHistogramDataPoint::_internal_flags() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.flags_;
}
inline void ExponentialHistogramDataPoint::_internal_set_flags(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = value;
}

// repeated .opentelemetry.proto.metrics.v1.Exemplar exemplars = 11;
inline int ExponentialHistogramDataPoint::_internal_exemplars_size() const {
  return _internal_exemplars().size();
}
inline int ExponentialHistogramDataPoint::exemplars_size() const {
  return _internal_exemplars_size();
}
inline void ExponentialHistogramDataPoint::clear_exemplars() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.exemplars_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::opentelemetry::proto::metrics::v1::Exemplar& ExponentialHistogramDataPoint::exemplars(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.exemplars)
  return _internal_exemplars().Get(index);
}
inline ::opentelemetry::proto::metrics::v1::Exemplar* PROTOBUF_NONNULL ExponentialHistogramDataPoint::mutable_exemplars(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.exemplars)
  return _internal_mutable_exemplars()->Mutable(index);
}
inline ::opentelemetry::proto::metrics::v1::Exemplar* PROTOBUF_NONNULL ExponentialHistogramDataPoint::add_exemplars()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::metrics::v1::Exemplar* _add =
      _internal_mutable_exemplars()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.exemplars)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>& ExponentialHistogramDataPoint::exemplars() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.exemplars)
  return _internal_exemplars();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>* PROTOBUF_NONNULL
ExponentialHistogramDataPoint::mutable_exemplars() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.exemplars)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_exemplars();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>&
ExponentialHistogramDataPoint::_internal_exemplars() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.exemplars_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::Exemplar>* PROTOBUF_NONNULL
ExponentialHistogramDataPoint::_internal_mutable_exemplars() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.exemplars_;
}

// optional double min = 12;
inline bool ExponentialHistogramDataPoint::has_min() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000800U);
  return value;
}
inline void ExponentialHistogramDataPoint::clear_min() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.min_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000800U);
}
inline double ExponentialHistogramDataPoint::min() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.min)
  return _internal_min();
}
inline void ExponentialHistogramDataPoint::set_min(double value) {
  _internal_set_min(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000800U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.min)
}
inline double ExponentialHistogramDataPoint::_internal_min() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.min_;
}
inline void ExponentialHistogramDataPoint::_internal_set_min(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.min_ = value;
}

// optional double max = 13;
inline bool ExponentialHistogramDataPoint::has_max() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00001000U);
  return value;
}
inline void ExponentialHistogramDataPoint::clear_max() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.max_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00001000U);
}
inline double ExponentialHistogramDataPoint::max() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.max)
  return _internal_max();
}
inline void ExponentialHistogramDataPoint::set_max(double value) {
  _internal_set_max(value);
  SetHasBit(_impl_._has_bits_[0], 0x00001000U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.max)
}
inline double ExponentialHistogramDataPoint::_internal_max() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.max_;
}
inline void ExponentialHistogramDataPoint::_internal_set_max(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.max_ = value;
}

// double zero_threshold = 14;
inline void ExponentialHistogramDataPoint::clear_zero_threshold() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.zero_threshold_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00002000U);
}
inline double ExponentialHistogramDataPoint::zero_threshold() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.zero_threshold)
  return _internal_zero_threshold();
}
inline void ExponentialHistogramDataPoint::set_zero_threshold(double value) {
  _internal_set_zero_threshold(value);
  SetHasBit(_impl_._has_bits_[0], 0x00002000U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.ExponentialHistogramDataPoint.zero_threshold)
}
inline double ExponentialHistogramDataPoint::_internal_zero_threshold() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.zero_threshold_;
}
inline void ExponentialHistogramDataPoint::_internal_set_zero_threshold(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.zero_threshold_ = value;
}

// -------------------------------------------------------------------

// SummaryDataPoint_ValueAtQuantile

// double quantile = 1;
inline void SummaryDataPoint_ValueAtQuantile::clear_quantile() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.quantile_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline double SummaryDataPoint_ValueAtQuantile::quantile() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.SummaryDataPoint.ValueAtQuantile.quantile)
  return _internal_quantile();
}
inline void SummaryDataPoint_ValueAtQuantile::set_quantile(double value) {
  _internal_set_quantile(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.SummaryDataPoint.ValueAtQuantile.quantile)
}
inline double SummaryDataPoint_ValueAtQuantile::_internal_quantile() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.quantile_;
}
inline void SummaryDataPoint_ValueAtQuantile::_internal_set_quantile(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.quantile_ = value;
}

// double value = 2;
inline void SummaryDataPoint_ValueAtQuantile::clear_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.value_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline double SummaryDataPoint_ValueAtQuantile::value() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.SummaryDataPoint.ValueAtQuantile.value)
  return _internal_value();
}
inline void SummaryDataPoint_ValueAtQuantile::set_value(double value) {
  _internal_set_value(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.SummaryDataPoint.ValueAtQuantile.value)
}
inline double SummaryDataPoint_ValueAtQuantile::_internal_value() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.value_;
}
inline void SummaryDataPoint_ValueAtQuantile::_internal_set_value(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.value_ = value;
}

// -------------------------------------------------------------------

// SummaryDataPoint

// repeated .opentelemetry.proto.common.v1.KeyValue attributes = 7;
inline int SummaryDataPoint::_internal_attributes_size() const {
  return _internal_attributes().size();
}
inline int SummaryDataPoint::attributes_size() const {
  return _internal_attributes_size();
}
inline const ::opentelemetry::proto::common::v1::KeyValue& SummaryDataPoint::attributes(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.SummaryDataPoint.attributes)
  return _internal_attributes().Get(index);
}
inline ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL SummaryDataPoint::mutable_attributes(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.SummaryDataPoint.attributes)
  return _internal_mutable_attributes()->Mutable(index);
}
inline ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL SummaryDataPoint::add_attributes()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::common::v1::KeyValue* _add =
      _internal_mutable_attributes()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.metrics.v1.SummaryDataPoint.attributes)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>& SummaryDataPoint::attributes() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.metrics.v1.SummaryDataPoint.attributes)
  return _internal_attributes();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL
SummaryDataPoint::mutable_attributes() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.metrics.v1.SummaryDataPoint.attributes)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_attributes();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>&
SummaryDataPoint::_internal_attributes() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.attributes_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL
SummaryDataPoint::_internal_mutable_attributes() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.attributes_;
}

// fixed64 start_time_unix_nano = 2;
inline void SummaryDataPoint::clear_start_time_unix_nano() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.start_time_unix_nano_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::uint64_t SummaryDataPoint::start_time_unix_nano() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.SummaryDataPoint.start_time_unix_nano)
  return _internal_start_time_unix_nano();
}
inline void SummaryDataPoint::set_start_time_unix_nano(::uint64_t value) {
  _internal_set_start_time_unix_nano(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.SummaryDataPoint.start_time_unix_nano)
}
inline ::uint64_t SummaryDataPoint::_internal_start_time_unix_nano() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.start_time_unix_nano_;
}
inline void SummaryDataPoint::_internal_set_start_time_unix_nano(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.start_time_unix_nano_ = value;
}

// fixed64 time_unix_nano = 3;
inline void SummaryDataPoint::clear_time_unix_nano() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.time_unix_nano_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::uint64_t SummaryDataPoint::time_unix_nano() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.SummaryDataPoint.time_unix_nano)
  return _internal_time_unix_nano();
}
inline void SummaryDataPoint::set_time_unix_nano(::uint64_t value) {
  _internal_set_time_unix_nano(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.SummaryDataPoint.time_unix_nano)
}
inline ::uint64_t SummaryDataPoint::_internal_time_unix_nano() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.time_unix_nano_;
}
inline void SummaryDataPoint::_internal_set_time_unix_nano(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.time_unix_nano_ = value;
}

// fixed64 count = 4;
inline void SummaryDataPoint::clear_count() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.count_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::uint64_t SummaryDataPoint::count() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.SummaryDataPoint.count)
  return _internal_count();
}
inline void SummaryDataPoint::set_count(::uint64_t value) {
  _internal_set_count(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.SummaryDataPoint.count)
}
inline ::uint64_t SummaryDataPoint::_internal_count() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.count_;
}
inline void SummaryDataPoint::_internal_set_count(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.count_ = value;
}

// double sum = 5;
inline void SummaryDataPoint::clear_sum() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sum_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline double SummaryDataPoint::sum() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.SummaryDataPoint.sum)
  return _internal_sum();
}
inline void SummaryDataPoint::set_sum(double value) {
  _internal_set_sum(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.SummaryDataPoint.sum)
}
inline double SummaryDataPoint::_internal_sum() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sum_;
}
inline void SummaryDataPoint::_internal_set_sum(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sum_ = value;
}

// repeated .opentelemetry.proto.metrics.v1.SummaryDataPoint.ValueAtQuantile quantile_values = 6;
inline int SummaryDataPoint::_internal_quantile_values_size() const {
  return _internal_quantile_values().size();
}
inline int SummaryDataPoint::quantile_values_size() const {
  return _internal_quantile_values_size();
}
inline void SummaryDataPoint::clear_quantile_values() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.quantile_values_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::opentelemetry::proto::metrics::v1::SummaryDataPoint_ValueAtQuantile& SummaryDataPoint::quantile_values(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.SummaryDataPoint.quantile_values)
  return _internal_quantile_values().Get(index);
}
inline ::opentelemetry::proto::metrics::v1::SummaryDataPoint_ValueAtQuantile* PROTOBUF_NONNULL SummaryDataPoint::mutable_quantile_values(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.SummaryDataPoint.quantile_values)
  return _internal_mutable_quantile_values()->Mutable(index);
}
inline ::opentelemetry::proto::metrics::v1::SummaryDataPoint_ValueAtQuantile* PROTOBUF_NONNULL SummaryDataPoint::add_quantile_values()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::metrics::v1::SummaryDataPoint_ValueAtQuantile* _add =
      _internal_mutable_quantile_values()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.metrics.v1.SummaryDataPoint.quantile_values)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::SummaryDataPoint_ValueAtQuantile>& SummaryDataPoint::quantile_values() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.metrics.v1.SummaryDataPoint.quantile_values)
  return _internal_quantile_values();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::SummaryDataPoint_ValueAtQuantile>* PROTOBUF_NONNULL
SummaryDataPoint::mutable_quantile_values() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.metrics.v1.SummaryDataPoint.quantile_values)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_quantile_values();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::SummaryDataPoint_ValueAtQuantile>&
SummaryDataPoint::_internal_quantile_values() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.quantile_values_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::metrics::v1::SummaryDataPoint_ValueAtQuantile>* PROTOBUF_NONNULL
SummaryDataPoint::_internal_mutable_quantile_values() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.quantile_values_;
}

// uint32 flags = 8;
inline void SummaryDataPoint::clear_flags() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::uint32_t SummaryDataPoint::flags() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.SummaryDataPoint.flags)
  return _internal_flags();
}
inline void SummaryDataPoint::set_flags(::uint32_t value) {
  _internal_set_flags(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.SummaryDataPoint.flags)
}
inline ::uint32_t SummaryDataPoint::_internal_flags() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.flags_;
}
inline void SummaryDataPoint::_internal_set_flags(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = value;
}

// -------------------------------------------------------------------

// Exemplar

// repeated .opentelemetry.proto.common.v1.KeyValue filtered_attributes = 7;
inline int Exemplar::_internal_filtered_attributes_size() const {
  return _internal_filtered_attributes().size();
}
inline int Exemplar::filtered_attributes_size() const {
  return _internal_filtered_attributes_size();
}
inline const ::opentelemetry::proto::common::v1::KeyValue& Exemplar::filtered_attributes(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.Exemplar.filtered_attributes)
  return _internal_filtered_attributes().Get(index);
}
inline ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL Exemplar::mutable_filtered_attributes(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.Exemplar.filtered_attributes)
  return _internal_mutable_filtered_attributes()->Mutable(index);
}
inline ::opentelemetry::proto::common::v1::KeyValue* PROTOBUF_NONNULL Exemplar::add_filtered_attributes()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::opentelemetry::proto::common::v1::KeyValue* _add =
      _internal_mutable_filtered_attributes()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:opentelemetry.proto.metrics.v1.Exemplar.filtered_attributes)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>& Exemplar::filtered_attributes() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:opentelemetry.proto.metrics.v1.Exemplar.filtered_attributes)
  return _internal_filtered_attributes();
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL
Exemplar::mutable_filtered_attributes() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:opentelemetry.proto.metrics.v1.Exemplar.filtered_attributes)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_filtered_attributes();
}
inline const ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>&
Exemplar::_internal_filtered_attributes() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.filtered_attributes_;
}
inline ::google::protobuf::RepeatedPtrField<::opentelemetry::proto::common::v1::KeyValue>* PROTOBUF_NONNULL
Exemplar::_internal_mutable_filtered_attributes() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.filtered_attributes_;
}

// fixed64 time_unix_nano = 2;
inline void Exemplar::clear_time_unix_nano() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.time_unix_nano_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::uint64_t Exemplar::time_unix_nano() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.Exemplar.time_unix_nano)
  return _internal_time_unix_nano();
}
inline void Exemplar::set_time_unix_nano(::uint64_t value) {
  _internal_set_time_unix_nano(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.Exemplar.time_unix_nano)
}
inline ::uint64_t Exemplar::_internal_time_unix_nano() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.time_unix_nano_;
}
inline void Exemplar::_internal_set_time_unix_nano(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.time_unix_nano_ = value;
}

// double as_double = 3;
inline bool Exemplar::has_as_double() const {
  return value_case() == kAsDouble;
}
inline void Exemplar::set_has_as_double() {
  _impl_._oneof_case_[0] = kAsDouble;
}
inline void Exemplar::clear_as_double() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value_case() == kAsDouble) {
    _impl_.value_.as_double_ = 0;
    clear_has_value();
  }
}
inline double Exemplar::as_double() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.Exemplar.as_double)
  return _internal_as_double();
}
inline void Exemplar::set_as_double(double value) {
  if (value_case() != kAsDouble) {
    clear_value();
    set_has_as_double();
  }
  _impl_.value_.as_double_ = value;
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.Exemplar.as_double)
}
inline double Exemplar::_internal_as_double() const {
  if (value_case() == kAsDouble) {
    return _impl_.value_.as_double_;
  }
  return 0;
}

// sfixed64 as_int = 6;
inline bool Exemplar::has_as_int() const {
  return value_case() == kAsInt;
}
inline void Exemplar::set_has_as_int() {
  _impl_._oneof_case_[0] = kAsInt;
}
inline void Exemplar::clear_as_int() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value_case() == kAsInt) {
    _impl_.value_.as_int_ = ::int64_t{0};
    clear_has_value();
  }
}
inline ::int64_t Exemplar::as_int() const {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.Exemplar.as_int)
  return _internal_as_int();
}
inline void Exemplar::set_as_int(::int64_t value) {
  if (value_case() != kAsInt) {
    clear_value();
    set_has_as_int();
  }
  _impl_.value_.as_int_ = value;
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.Exemplar.as_int)
}
inline ::int64_t Exemplar::_internal_as_int() const {
  if (value_case() == kAsInt) {
    return _impl_.value_.as_int_;
  }
  return ::int64_t{0};
}

// bytes span_id = 4;
inline void Exemplar::clear_span_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.span_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& Exemplar::span_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.Exemplar.span_id)
  return _internal_span_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Exemplar::set_span_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.span_id_.SetBytes(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.Exemplar.span_id)
}
inline ::std::string* PROTOBUF_NONNULL Exemplar::mutable_span_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_span_id();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.Exemplar.span_id)
  return _s;
}
inline const ::std::string& Exemplar::_internal_span_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.span_id_.Get();
}
inline void Exemplar::_internal_set_span_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.span_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Exemplar::_internal_mutable_span_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.span_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Exemplar::release_span_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.metrics.v1.Exemplar.span_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.span_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.span_id_.Set("", GetArena());
  }
  return released;
}
inline void Exemplar::set_allocated_span_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.span_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.span_id_.IsDefault()) {
    _impl_.span_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.metrics.v1.Exemplar.span_id)
}

// bytes trace_id = 5;
inline void Exemplar::clear_trace_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.trace_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::std::string& Exemplar::trace_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:opentelemetry.proto.metrics.v1.Exemplar.trace_id)
  return _internal_trace_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Exemplar::set_trace_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.trace_id_.SetBytes(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:opentelemetry.proto.metrics.v1.Exemplar.trace_id)
}
inline ::std::string* PROTOBUF_NONNULL Exemplar::mutable_trace_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_trace_id();
  // @@protoc_insertion_point(field_mutable:opentelemetry.proto.metrics.v1.Exemplar.trace_id)
  return _s;
}
inline const ::std::string& Exemplar::_internal_trace_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.trace_id_.Get();
}
inline void Exemplar::_internal_set_trace_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.trace_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Exemplar::_internal_mutable_trace_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.trace_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Exemplar::release_trace_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:opentelemetry.proto.metrics.v1.Exemplar.trace_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.trace_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.trace_id_.Set("", GetArena());
  }
  return released;
}
inline void Exemplar::set_allocated_trace_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.trace_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.trace_id_.IsDefault()) {
    _impl_.trace_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:opentelemetry.proto.metrics.v1.Exemplar.trace_id)
}

inline bool Exemplar::has_value() const {
  return value_case() != VALUE_NOT_SET;
}
inline void Exemplar::clear_has_value() {
  _impl_._oneof_case_[0] = VALUE_NOT_SET;
}
inline Exemplar::ValueCase Exemplar::value_case() const {
  return Exemplar::ValueCase(_impl_._oneof_case_[0]);
}
#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace v1
}  // namespace metrics
}  // namespace proto
}  // namespace opentelemetry


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::opentelemetry::proto::metrics::v1::AggregationTemporality> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::opentelemetry::proto::metrics::v1::AggregationTemporality>() {
  return ::opentelemetry::proto::metrics::v1::AggregationTemporality_descriptor();
}
template <>
struct is_proto_enum<::opentelemetry::proto::metrics::v1::DataPointFlags> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::opentelemetry::proto::metrics::v1::DataPointFlags>() {
  return ::opentelemetry::proto::metrics::v1::DataPointFlags_descriptor();
}

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // opentelemetry_2fproto_2fmetrics_2fv1_2fmetrics_2eproto_2epb_2eh
